@@ -7,7 +7,7 @@ else
   exit 1
 fi
 
-BRANCH="devel"
+BRANCH="master"
 
 repo="git://git.blender.org/libmv.git"
 tmp=`mktemp -d`
@@ -30,15 +30,20 @@ rm -rf $tmp
 
 chmod 664 ./third_party/glog/src/windows/*.cc ./third_party/glog/src/windows/*.h ./third_party/glog/src/windows/glog/*.h
 
-sources=`find ./libmv -type f -iname '*.cc' -or -iname '*.cpp' -or -iname '*.c' | sed -r 's/^\.\//\t\t/' | sort -d`
-headers=`find ./libmv -type f -iname '*.h' | sed -r 's/^\.\//\t\t/' | sort -d`
+sources=`find ./libmv -type f -iname '*.cc' -or -iname '*.cpp' -or -iname '*.c' | grep -v _test.cc | grep -v test_data_sets | sed -r 's/^\.\//\t\t/' | sort -d`
+headers=`find ./libmv -type f -iname '*.h' | grep -v test_data_sets | sed -r 's/^\.\//\t\t/' | sort -d`
 
-third_sources=`find ./third_party -type f -iname '*.cc' -or -iname '*.cpp' -or -iname '*.c' | grep -v glog | grep -v ceres | sed -r 's/^\.\//\t\t/' | sort -d`
-third_headers=`find ./third_party -type f -iname '*.h' | grep -v glog | grep -v ceres | sed -r 's/^\.\//\t\t/' | sort -d`
+third_sources=`find ./third_party -type f -iname '*.cc' -or -iname '*.cpp' -or -iname '*.c' | grep -v glog | grep -v gflags | grep -v ceres | sed -r 's/^\.\//\t\t/' | sort -d`
+third_headers=`find ./third_party -type f -iname '*.h' | grep -v glog | grep -v gflags | grep -v ceres | sed -r 's/^\.\//\t\t/' | sort -d`
 
 third_glog_sources=`find ./third_party -type f -iname '*.cc' -or -iname '*.cpp' -or -iname '*.c' | grep glog | grep -v windows | sed -r 's/^\.\//\t\t\t/' | sort -d`
 third_glog_headers=`find ./third_party -type f -iname '*.h' | grep glog | grep -v windows | sed -r 's/^\.\//\t\t\t/' | sort -d`
 
+third_gflags_sources=`find ./third_party -type f -iname '*.cc' -or -iname '*.cpp' -or -iname '*.c' | grep gflags | grep -v windows | sed -r 's/^\.\//\t\t/' | sort -d`
+third_gflags_headers=`find ./third_party -type f -iname '*.h' | grep gflags | grep -v windows | sed -r 's/^\.\//\t\t/' | sort -d`
+
+tests=`find ./libmv -type f -iname '*_test.cc' | sort -d | awk ' { name=gensub(".*/([A-Za-z_]+)_test.cc", "\\\\1", $1); printf("\t\tBLENDER_SRC_GTEST(\"libmv_%s\" \"%s\" \"libmv_test_dataset;extern_libmv;extern_ceres\")\n", name, $1) } '`
+
 src_dir=`find ./libmv -type f -iname '*.cc' -exec dirname {} \; -or -iname '*.cpp' -exec dirname {} \; -or -iname '*.c' -exec dirname {} \; | sed -r 's/^\.\//\t\t/' | sort -d | uniq`
 src_third_dir=`find ./third_party -type f -iname '*.cc' -exec dirname {} \; -or -iname '*.cpp' -exec dirname {} \; -or -iname '*.c' -exec dirname {} \;  | grep -v ceres | sed -r 's/^\.\//\t\t/'  | sort -d | uniq`
 src=""
@@ -118,43 +123,121 @@ set(INC
 	.
 )
 
-set(SRC
-	libmv-capi.h
-	libmv-capi_intern.h
+set(INC_SYS
 )
 
-if(WITH_LIBMV)
+set(SRC
+	libmv-capi.h
+)
+
+if(WITH_LIBMV OR WITH_GTESTS OR (WITH_CYCLES AND WITH_CYCLES_LOGGING))
+	list(APPEND INC
+		third_party/gflags
+		third_party/glog/src
+		third_party/ceres/include
+		third_party/ceres/config
+		../../intern/guardedalloc
+	)
+
+	list(APPEND INC_SYS
+		../Eigen3
+		\${PNG_INCLUDE_DIRS}
+		\${ZLIB_INCLUDE_DIRS}
+	)
+
+	if(WIN32)
+		list(APPEND INC
+			third_party/glog/src/windows
+		)
+
+		if(NOT MINGW)
+			list(APPEND INC
+				third_party/msinttypes
+			)
+		endif()
+	endif()
+
 	add_definitions(
-		-DWITH_LIBMV
 		-DWITH_LIBMV_GUARDED_ALLOC
 		-DGOOGLE_GLOG_DLL_DECL=
 		-DLIBMV_NO_FAST_DETECTOR=
 	)
+endif()
 
-	list(APPEND INC
-		third_party/ceres/include
-		../../intern/guardedalloc
-	)
-
-	set(INC_SYS
-		../Eigen3
-		\${PNG_INCLUDE_DIR}
-		\${ZLIB_INCLUDE_DIRS}
-	)
+if(WITH_LIBMV)
+	TEST_SHARED_PTR_SUPPORT()
+	if(SHARED_PTR_FOUND)
+		if(SHARED_PTR_TR1_MEMORY_HEADER)
+			add_definitions(-DCERES_TR1_MEMORY_HEADER)
+		endif()
+		if(SHARED_PTR_TR1_NAMESPACE)
+			add_definitions(-DCERES_TR1_SHARED_PTR)
+		endif()
+	else()
+		message(FATAL_ERROR "Unable to find shared_ptr.")
+	endif()
 
 	list(APPEND SRC
-		libmv-capi.cc
+		intern/autotrack.cc
+		intern/camera_intrinsics.cc
+		intern/detector.cc
+		intern/frame_accessor.cc
+		intern/homography.cc
+		intern/image.cc
+		intern/logging.cc
+		intern/reconstruction.cc
+		intern/track_region.cc
+		intern/tracks.cc
+		intern/tracksN.cc
 ${sources}
-
 ${third_sources}
 
+		intern/autotrack.h
+		intern/camera_intrinsics.h
+		intern/detector.h
+		intern/frame_accessor.h
+		intern/homography.h
+		intern/image.h
+		intern/logging.h
+		intern/reconstruction.h
+		intern/track_region.h
+		intern/tracks.h
+		intern/tracksN.h
 ${headers}
 
 ${third_headers}
 	)
 
+
+	if(WITH_GTESTS)
+		blender_add_lib(libmv_test_dataset "./libmv/multiview/test_data_sets.cc" "${INC}" "${INC_SYS}")
+
+${tests}
+	endif()
+else()
+	list(APPEND SRC
+		intern/stub.cc
+	)
+endif()
+
+blender_add_lib(extern_libmv "\${SRC}" "\${INC}" "\${INC_SYS}")
+
+if(WITH_LIBMV)
+	add_subdirectory(third_party)
+endif()
+
+# make GLog a separate target, so it can be used for gtest as well.
+if(WITH_LIBMV OR WITH_GTESTS OR (WITH_CYCLES AND WITH_CYCLES_LOGGING))
+	# We compile GLog together with GFlag so we don't worry about
+	# adding extra lib to linker.
+	set(GLOG_SRC
+${third_gflags_sources}
+
+${third_gflags_headers}
+	)
+
 	if(WIN32)
-		list(APPEND SRC
+		list(APPEND GLOG_SRC
 			third_party/glog/src/logging.cc
 			third_party/glog/src/raw_logging.cc
 			third_party/glog/src/utilities.cc
@@ -179,37 +262,15 @@ ${third_headers}
 			third_party/glog/src/windows/port.h
 			third_party/glog/src/windows/config.h
 		)
-
-		list(APPEND INC
-			third_party/glog/src/windows
-		)
-
-		if(NOT MINGW)
-			list(APPEND INC
-				third_party/msinttypes
-			)
-		endif()
 	else()
-		list(APPEND SRC
+		list(APPEND GLOG_SRC
 ${third_glog_sources}
 
 ${third_glog_headers}
 		)
-
-		list(APPEND INC
-			third_party/glog/src
-		)
 	endif()
-else()
-	list(APPEND SRC
-		libmv-capi_stub.cc
-	)
-endif()
 
-blender_add_lib(extern_libmv "\${SRC}" "\${INC}" "\${INC_SYS}")
-
-if(WITH_LIBMV)
-	add_subdirectory(third_party)
+	blender_add_lib(extern_glog "\${GLOG_SRC}" "\${INC}" "\${INC_SYS}")
 endif()
 EOF
 
@@ -228,16 +289,12 @@ Import('env')
 defs = []
 incs = '.'
 
-if env['WITH_BF_LIBMV']:
+if env['WITH_BF_LIBMV'] or (env['WITH_BF_CYCLES'] and env['WITH_BF_CYCLES_LOGGING']):
     defs.append('GOOGLE_GLOG_DLL_DECL=')
-    defs.append('WITH_LIBMV')
     defs.append('WITH_LIBMV_GUARDED_ALLOC')
     defs.append('LIBMV_NO_FAST_DETECTOR')
 
-    src = env.Glob("libmv-capi.cc")
-$src
-
-    incs += ' ../Eigen3 third_party/ceres/include ../../intern/guardedalloc'
+    incs += ' ../Eigen3 third_party/gflags third_party/glog/src third_party/ceres/include third_party/ceres/config ../../intern/guardedalloc'
     incs += ' ' + env['BF_PNG_INC']
     incs += ' ' + env['BF_ZLIB_INC']
 
@@ -245,16 +302,38 @@ $src
         incs += ' ./third_party/glog/src/windows ./third_party/glog/src/windows/glog'
         if env['OURPLATFORM'] in ('win32-vc', 'win64-vc'):
             incs += ' ./third_party/msinttypes'
-${win_src}
-        src += ['./third_party/glog/src/logging.cc', './third_party/glog/src/raw_logging.cc', './third_party/glog/src/utilities.cc', './third_party/glog/src/vlog_is_on.cc']
-        src += ['./third_party/glog/src/windows/port.cc']
     else:
-        src += env.Glob("third_party/glog/src/*.cc")
         incs += ' ./third_party/glog/src'
-else:
-    src = env.Glob("libmv-capi_stub.cc")
 
-env.BlenderLib ( libname = 'extern_libmv', sources=src, includes=Split(incs), defines=defs, libtype=['extern', 'player'], priority=[20,137] )
+if env['WITH_BF_LIBMV']:
+    if not env['WITH_SHARED_PTR_SUPPORT']:
+        print("-- Unable to find shared_ptr which is required for compilation.")
+        exit(1)
+
+    if env['SHARED_PTR_HEADER'] == 'tr1/memory':
+        defs.append('CERES_TR1_MEMORY_HEADER')
+    if env['SHARED_PTR_NAMESPACE'] == 'std::tr1':
+        defs.append('CERES_TR1_SHARED_PTR')
+
+    src = env.Glob('intern/*.cc')
+    src.remove('intern' + os.sep + 'stub.cc')
+$src
+else:
+    src = env.Glob("intern/stub.cc")
+
+src = [src for src in src if src.find('_test.cc') == -1]
+
+env.BlenderLib(libname = 'extern_libmv', sources=src, includes=Split(incs), defines=defs, libtype=['extern', 'player'], priority=[20,137])
+
+if env['WITH_BF_LIBMV'] or (env['WITH_BF_CYCLES'] and env['WITH_BF_CYCLES_LOGGING']):
+    glog_src = []
+    if env['OURPLATFORM'] in ('win32-vc', 'win32-mingw', 'linuxcross', 'win64-vc', 'win64-mingw'):
+        glog_src += ['./third_party/glog/src/logging.cc', './third_party/glog/src/raw_logging.cc', './third_party/glog/src/utilities.cc', './third_party/glog/src/vlog_is_on.cc']
+        glog_src += ['./third_party/glog/src/windows/port.cc']
+    else:
+        glog_src += env.Glob("third_party/glog/src/*.cc")
+
+    env.BlenderLib(libname = 'extern_glog', sources=glog_src, includes=Split(incs), defines=defs, libtype=['extern', 'player'], priority=[20,137])
 
 if env['WITH_BF_LIBMV']:
     SConscript(['third_party/SConscript'])
