@@ -20,6 +20,7 @@
 #include "device.h"
 #include "scene.h"
 #include "session.h"
+#include "bake.h"
 
 #include "util_vector.h"
 
@@ -51,6 +52,8 @@ public:
 	/* offline render */
 	void render();
 
+	void bake(BL::Object b_object, const string& pass_type, BL::BakePixel pixel_array, const size_t num_pixels, const int depth, float pixels[]);
+
 	void write_render_result(BL::RenderResult b_rr, BL::RenderLayer b_rlay, RenderTile& rtile);
 	void write_render_tile(RenderTile& rtile);
 
@@ -70,6 +73,7 @@ public:
 	void get_progress(float& progress, double& total_time);
 	void test_cancel();
 	void update_status_progress();
+	void update_bake_progress();
 
 	bool background;
 	Session *session;
@@ -87,6 +91,7 @@ public:
 	string b_rlay_name;
 
 	string last_status;
+	string last_error;
 	float last_progress;
 
 	int width, height;
@@ -99,7 +104,7 @@ protected:
 	void do_write_update_render_tile(RenderTile& rtile, bool do_update_only);
 
 	int builtin_image_frame(const string &builtin_name);
-	void builtin_image_info(const string &builtin_name, void *builtin_data, bool &is_float, int &width, int &height, int &channels);
+	void builtin_image_info(const string &builtin_name, void *builtin_data, bool &is_float, int &width, int &height, int &depth, int &channels);
 	bool builtin_image_pixels(const string &builtin_name, void *builtin_data, unsigned char *pixels);
 	bool builtin_image_float_pixels(const string &builtin_name, void *builtin_data, float *pixels);
 };
