@@ -1,24 +1,23 @@
 /*
- * Copyright 2011, Blender Foundation.
+ * Copyright 2011-2013 Blender Foundation
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * http://www.apache.org/licenses/LICENSE-2.0
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License
  */
 
 #include "attribute.h"
 #include "graph.h"
 #include "nodes.h"
+#include "shader.h"
 
 #include "util_algorithm.h"
 #include "util_debug.h"
@@ -37,7 +36,7 @@ ShaderInput::ShaderInput(ShaderNode *parent_, const char *name_, ShaderSocketTyp
 	value = make_float3(0, 0, 0);
 	stack_offset = SVM_STACK_INVALID;
 	default_value = NONE;
-	osl_only = false;
+	usage = USE_ALL;
 }
 
 ShaderOutput::ShaderOutput(ShaderNode *parent_, const char *name_, ShaderSocketType type_)
@@ -85,27 +84,29 @@ ShaderOutput *ShaderNode::output(const char *name)
 	return NULL;
 }
 
-ShaderInput *ShaderNode::add_input(const char *name, ShaderSocketType type, float value)
+ShaderInput *ShaderNode::add_input(const char *name, ShaderSocketType type, float value, int usage)
 {
 	ShaderInput *input = new ShaderInput(this, name, type);
 	input->value.x = value;
+	input->usage = usage;
 	inputs.push_back(input);
 	return input;
 }
 
-ShaderInput *ShaderNode::add_input(const char *name, ShaderSocketType type, float3 value)
+ShaderInput *ShaderNode::add_input(const char *name, ShaderSocketType type, float3 value, int usage)
 {
 	ShaderInput *input = new ShaderInput(this, name, type);
 	input->value = value;
+	input->usage = usage;
 	inputs.push_back(input);
 	return input;
 }
 
-ShaderInput *ShaderNode::add_input(const char *name, ShaderSocketType type, ShaderInput::DefaultValue value, bool osl_only)
+ShaderInput *ShaderNode::add_input(const char *name, ShaderSocketType type, ShaderInput::DefaultValue value, int usage)
 {
 	ShaderInput *input = add_input(name, type);
 	input->default_value = value;
-	input->osl_only = osl_only;
+	input->usage = usage;
 	return input;
 }
 
@@ -116,14 +117,20 @@ ShaderOutput *ShaderNode::add_output(const char *name, ShaderSocketType type)
 	return output;
 }
 
-void ShaderNode::attributes(AttributeRequestSet *attributes)
+void ShaderNode::attributes(Shader *shader, AttributeRequestSet *attributes)
 {
 	foreach(ShaderInput *input, inputs) {
 		if(!input->link) {
-			if(input->default_value == ShaderInput::TEXTURE_GENERATED)
-				attributes->add(ATTR_STD_GENERATED);
-			else if(input->default_value == ShaderInput::TEXTURE_UV)
-				attributes->add(ATTR_STD_UV);
+			if(input->default_value == ShaderInput::TEXTURE_GENERATED) {
+				if(shader->has_surface)
+					attributes->add(ATTR_STD_GENERATED);
+				if(shader->has_volume)
+					attributes->add(ATTR_STD_GENERATED_TRANSFORM);
+			}
+			else if(input->default_value == ShaderInput::TEXTURE_UV) {
+				if(shader->has_surface)
+					attributes->add(ATTR_STD_UV);
+			}
 		}
 	}
 }
@@ -133,6 +140,7 @@ void ShaderNode::attributes(AttributeRequestSet *attributes)
 ShaderGraph::ShaderGraph()
 {
 	finalized = false;
+	num_node_ids = 0;
 	add(new OutputNode());
 }
 
@@ -145,14 +153,14 @@ ShaderGraph::~ShaderGraph()
 ShaderNode *ShaderGraph::add(ShaderNode *node)
 {
 	assert(!finalized);
-	node->id = nodes.size();
+	node->id = num_node_ids++;
 	nodes.push_back(node);
 	return node;
 }
 
-ShaderNode *ShaderGraph::output()
+OutputNode *ShaderGraph::output()
 {
-	return nodes.front();
+	return (OutputNode*)nodes.front();
 }
 
 ShaderGraph *ShaderGraph::copy()
@@ -189,14 +197,14 @@ void ShaderGraph::connect(ShaderOutput *from, ShaderInput *to)
 		/* for closures we can't do automatic conversion */
 		if(from->type == SHADER_SOCKET_CLOSURE || to->type == SHADER_SOCKET_CLOSURE) {
 			fprintf(stderr, "Cycles shader graph connect: can only connect closure to closure "
-			        "(ShaderNode:%s, ShaderOutput:%s , type:%d -> to ShaderNode:%s, ShaderInput:%s, type:%d).\n",
-			        from->parent->name.c_str(), from->name, (int)from->type,
-			        to->parent->name.c_str(),   to->name,   (int)to->type);
+			        "(%s.%s to %s.%s).\n",
+			        from->parent->name.c_str(), from->name,
+			        to->parent->name.c_str(), to->name);
 			return;
 		}
 
 		/* add automatic conversion node in case of type mismatch */
-		ShaderNode *convert = add(new ConvertNode(from->type, to->type));
+		ShaderNode *convert = add(new ConvertNode(from->type, to->type, true));
 
 		connect(from, convert->inputs[0]);
 		connect(convert->outputs[0], to);
@@ -229,16 +237,28 @@ void ShaderGraph::finalize(bool do_bump, bool do_osl)
 	if(!finalized) {
 		clean();
 		default_inputs(do_osl);
+		refine_bump_nodes();
+
 		if(do_bump)
 			bump_from_displacement();
 
+		ShaderInput *surface_in = output()->input("Surface");
+		ShaderInput *volume_in = output()->input("Volume");
+
+		/* todo: make this work when surface and volume closures are tangled up */
+
+		if(surface_in->link)
+			transform_multi_closure(surface_in->link->parent, NULL, false);
+		if(volume_in->link)
+			transform_multi_closure(volume_in->link->parent, NULL, true);
+
 		finalized = true;
 	}
 }
 
 void ShaderGraph::find_dependencies(set<ShaderNode*>& dependencies, ShaderInput *input)
 {
-	/* find all nodes that this input dependes on directly and indirectly */
+	/* find all nodes that this input depends on directly and indirectly */
 	ShaderNode *node = (input->link)? input->link->parent: NULL;
 
 	if(node) {
@@ -296,22 +316,26 @@ void ShaderGraph::copy_nodes(set<ShaderNode*>& nodes, map<ShaderNode*, ShaderNod
 	}
 }
 
-void ShaderGraph::remove_proxy_nodes(vector<bool>& removed)
+void ShaderGraph::remove_unneeded_nodes()
 {
+	vector<bool> removed(num_node_ids, false);
+	bool any_node_removed = false;
+
+	/* find and unlink proxy nodes */
 	foreach(ShaderNode *node, nodes) {
-		if (node->special_type == SHADER_SPECIAL_TYPE_PROXY) {
+		if(node->special_type == SHADER_SPECIAL_TYPE_PROXY) {
 			ProxyNode *proxy = static_cast<ProxyNode*>(node);
 			ShaderInput *input = proxy->inputs[0];
 			ShaderOutput *output = proxy->outputs[0];
-			
+
 			/* temp. copy of the output links list.
 			 * output->links is modified when we disconnect!
 			 */
 			vector<ShaderInput*> links(output->links);
 			ShaderOutput *from = input->link;
-			
+
 			/* bypass the proxy node */
-			if (from) {
+			if(from) {
 				disconnect(input);
 				foreach(ShaderInput *to, links) {
 					disconnect(to);
@@ -320,19 +344,40 @@ void ShaderGraph::remove_proxy_nodes(vector<bool>& removed)
 			}
 			else {
 				foreach(ShaderInput *to, links) {
+					/* remove any autoconvert nodes too if they lead to
+					 * sockets with an automatically set default value */
+					ShaderNode *tonode = to->parent;
+
+					if(tonode->special_type == SHADER_SPECIAL_TYPE_AUTOCONVERT) {
+						bool all_links_removed = true;
+						vector<ShaderInput*> links = tonode->outputs[0]->links;
+
+						foreach(ShaderInput *autoin, links) {
+							if(autoin->default_value == ShaderInput::NONE)
+								all_links_removed = false;
+							else
+								disconnect(autoin);
+						}
+
+						if(all_links_removed)
+							removed[tonode->id] = true;
+					}
+
 					disconnect(to);
 					
 					/* transfer the default input value to the target socket */
 					to->set(input->value);
+					to->set(input->value_string);
 				}
 			}
 			
 			removed[proxy->id] = true;
+			any_node_removed = true;
 		}
-
-		/* remove useless mix closures nodes */
-		if(node->special_type == SHADER_SPECIAL_TYPE_MIX_CLOSURE) {
+		else if(node->special_type == SHADER_SPECIAL_TYPE_MIX_CLOSURE) {
 			MixClosureNode *mix = static_cast<MixClosureNode*>(node);
+
+			/* remove useless mix closures nodes */
 			if(mix->outputs[0]->links.size() && mix->inputs[1]->link == mix->inputs[2]->link) {
 				ShaderOutput *output = mix->inputs[1]->link;
 				vector<ShaderInput*> inputs = mix->outputs[0]->links;
@@ -343,11 +388,108 @@ void ShaderGraph::remove_proxy_nodes(vector<bool>& removed)
 
 				foreach(ShaderInput *input, inputs) {
 					disconnect(input);
-					if (output)
+					if(output)
 						connect(output, input);
 				}
+				removed[mix->id] = true;
+				any_node_removed = true;
+			}
+		
+			/* remove unused mix closure input when factor is 0.0 or 1.0 */
+			/* check for closure links and make sure factor link is disconnected */
+			if(mix->outputs[0]->links.size() && mix->inputs[1]->link && mix->inputs[2]->link && !mix->inputs[0]->link) {
+				/* factor 0.0 */
+				if(mix->inputs[0]->value.x == 0.0f) {
+					ShaderOutput *output = mix->inputs[1]->link;
+					vector<ShaderInput*> inputs = mix->outputs[0]->links;
+
+					foreach(ShaderInput *sock, mix->inputs)
+						if(sock->link)
+							disconnect(sock);
+
+					foreach(ShaderInput *input, inputs) {
+						disconnect(input);
+						if(output)
+							connect(output, input);
+					}
+					removed[mix->id] = true;
+					any_node_removed = true;
+				}
+				/* factor 1.0 */
+				else if(mix->inputs[0]->value.x == 1.0f) {
+					ShaderOutput *output = mix->inputs[2]->link;
+					vector<ShaderInput*> inputs = mix->outputs[0]->links;
+					
+					foreach(ShaderInput *sock, mix->inputs)
+						if(sock->link)
+							disconnect(sock);
+
+					foreach(ShaderInput *input, inputs) {
+						disconnect(input);
+						if(output)
+							connect(output, input);
+					}
+					removed[mix->id] = true;
+					any_node_removed = true;
+				}
 			}
 		}
+		else if(node->special_type == SHADER_SPECIAL_TYPE_MIX_RGB) {
+			MixNode *mix = static_cast<MixNode*>(node);
+
+			/* remove unused Mix RGB inputs when factor is 0.0 or 1.0 */
+			/* check for color links and make sure factor link is disconnected */
+			if(mix->outputs[0]->links.size() && mix->inputs[1]->link && mix->inputs[2]->link && !mix->inputs[0]->link) {
+				/* factor 0.0 */
+				if(mix->inputs[0]->value.x == 0.0f) {
+					ShaderOutput *output = mix->inputs[1]->link;
+					vector<ShaderInput*> inputs = mix->outputs[0]->links;
+
+					foreach(ShaderInput *sock, mix->inputs)
+						if(sock->link)
+							disconnect(sock);
+
+					foreach(ShaderInput *input, inputs) {
+						disconnect(input);
+						if(output)
+							connect(output, input);
+					}
+					removed[mix->id] = true;
+					any_node_removed = true;
+				}
+				/* factor 1.0 */
+				else if(mix->inputs[0]->value.x == 1.0f) {
+					ShaderOutput *output = mix->inputs[2]->link;
+					vector<ShaderInput*> inputs = mix->outputs[0]->links;
+
+					foreach(ShaderInput *sock, mix->inputs)
+						if(sock->link)
+							disconnect(sock);
+
+					foreach(ShaderInput *input, inputs) {
+						disconnect(input);
+						if(output)
+							connect(output, input);
+					}
+					removed[mix->id] = true;
+					any_node_removed = true;
+				}
+			}
+		}
+	}
+
+	/* remove nodes */
+	if(any_node_removed) {
+		list<ShaderNode*> newnodes;
+
+		foreach(ShaderNode *node, nodes) {
+			if(!removed[node->id])
+				newnodes.push_back(node);
+			else
+				delete node;
+		}
+
+		nodes = newnodes;
 	}
 }
 
@@ -377,28 +519,16 @@ void ShaderGraph::break_cycles(ShaderNode *node, vector<bool>& visited, vector<b
 
 void ShaderGraph::clean()
 {
+	/* remove proxy and unnecessary mix nodes */
+	remove_unneeded_nodes();
+
 	/* we do two things here: find cycles and break them, and remove unused
 	 * nodes that don't feed into the output. how cycles are broken is
 	 * undefined, they are invalid input, the important thing is to not crash */
 
-	vector<bool> removed(nodes.size(), false);
-	vector<bool> visited(nodes.size(), false);
-	vector<bool> on_stack(nodes.size(), false);
+	vector<bool> visited(num_node_ids, false);
+	vector<bool> on_stack(num_node_ids, false);
 	
-	list<ShaderNode*> newnodes;
-	
-	/* remove proxy nodes */
-	remove_proxy_nodes(removed);
-	
-	foreach(ShaderNode *node, nodes) {
-		if(!removed[node->id])
-			newnodes.push_back(node);
-		else
-			delete node;
-	}
-	nodes = newnodes;
-	newnodes.clear();
-
 	/* break cycles */
 	break_cycles(output(), visited, on_stack);
 
@@ -408,7 +538,7 @@ void ShaderGraph::clean()
 			foreach(ShaderInput *to, node->inputs) {
 				ShaderOutput *from = to->link;
 
-				if (from) {
+				if(from) {
 					to->link = NULL;
 					from->links.erase(remove(from->links.begin(), from->links.end(), to), from->links.end());
 				}
@@ -417,6 +547,8 @@ void ShaderGraph::clean()
 	}
 
 	/* remove unused nodes */
+	list<ShaderNode*> newnodes;
+
 	foreach(ShaderNode *node, nodes) {
 		if(visited[node->id])
 			newnodes.push_back(node);
@@ -437,7 +569,7 @@ void ShaderGraph::default_inputs(bool do_osl)
 
 	foreach(ShaderNode *node, nodes) {
 		foreach(ShaderInput *input, node->inputs) {
-			if(!input->link && !(input->osl_only && !do_osl)) {
+			if(!input->link && ((input->usage & ShaderInput::USE_SVM) || do_osl)) {
 				if(input->default_value == ShaderInput::TEXTURE_GENERATED) {
 					if(!texco)
 						texco = new TextureCoordinateNode();
@@ -468,6 +600,12 @@ void ShaderGraph::default_inputs(bool do_osl)
 
 					connect(geom->output("Position"), input);
 				}
+				else if(input->default_value == ShaderInput::TANGENT) {
+					if(!geom)
+						geom = new GeometryNode();
+
+					connect(geom->output("Tangent"), input);
+				}
 			}
 		}
 	}
@@ -478,6 +616,61 @@ void ShaderGraph::default_inputs(bool do_osl)
 		add(texco);
 }
 
+void ShaderGraph::refine_bump_nodes()
+{
+	/* we transverse the node graph looking for bump nodes, when we find them,
+	 * like in bump_from_displacement(), we copy the sub-graph defined from "bump"
+	 * input to the inputs "center","dx" and "dy" What is in "bump" input is moved
+	 * to "center" input. */
+
+	foreach(ShaderNode *node, nodes) {
+		if(node->name == ustring("bump") && node->input("Height")->link) {
+			ShaderInput *bump_input = node->input("Height");
+			set<ShaderNode*> nodes_bump;
+
+			/* make 2 extra copies of the subgraph defined in Bump input */
+			map<ShaderNode*, ShaderNode*> nodes_dx;
+			map<ShaderNode*, ShaderNode*> nodes_dy;
+
+			/* find dependencies for the given input */
+			find_dependencies(nodes_bump, bump_input );
+
+			copy_nodes(nodes_bump, nodes_dx);
+			copy_nodes(nodes_bump, nodes_dy);
+	
+			/* mark nodes to indicate they are use for bump computation, so
+			   that any texture coordinates are shifted by dx/dy when sampling */
+			foreach(ShaderNode *node, nodes_bump)
+				node->bump = SHADER_BUMP_CENTER;
+			foreach(NodePair& pair, nodes_dx)
+				pair.second->bump = SHADER_BUMP_DX;
+			foreach(NodePair& pair, nodes_dy)
+				pair.second->bump = SHADER_BUMP_DY;
+
+			ShaderOutput *out = bump_input->link;
+			ShaderOutput *out_dx = nodes_dx[out->parent]->output(out->name);
+			ShaderOutput *out_dy = nodes_dy[out->parent]->output(out->name);
+
+			connect(out_dx, node->input("SampleX"));
+			connect(out_dy, node->input("SampleY"));
+			
+			/* add generated nodes */
+			foreach(NodePair& pair, nodes_dx)
+				add(pair.second);
+			foreach(NodePair& pair, nodes_dy)
+				add(pair.second);
+			
+			/* connect what is connected is bump to samplecenter input*/
+			connect(out , node->input("SampleCenter"));
+
+			/* bump input is just for connectivity purpose for the graph input,
+			 * we re-connected this input to samplecenter, so lets disconnect it
+			 * from bump input */
+			disconnect(bump_input);
+		}
+	}
+}
+
 void ShaderGraph::bump_from_displacement()
 {
 	/* generate bump mapping automatically from displacement. bump mapping is
@@ -491,7 +684,7 @@ void ShaderGraph::bump_from_displacement()
 	 * different shifted coordinates.
 	 *
 	 * these 3 displacement values are then fed into the bump node, which will
-	 * modify the normal. */
+	 * output the perturbed normal. */
 
 	ShaderInput *displacement_in = output()->input("Displacement");
 
@@ -520,6 +713,12 @@ void ShaderGraph::bump_from_displacement()
 	foreach(NodePair& pair, nodes_dy)
 		pair.second->bump = SHADER_BUMP_DY;
 
+	/* add set normal node and connect the bump normal ouput to the set normal
+	 * output, so it can finally set the shader normal, note we are only doing
+	 * this for bump from displacement, this will be the only bump allowed to
+	 * overwrite the shader normal */
+	ShaderNode *set_normal = add(new SetNormalNode());
+	
 	/* add bump node and connect copied graphs to it */
 	ShaderNode *bump = add(new BumpNode());
 
@@ -531,6 +730,9 @@ void ShaderGraph::bump_from_displacement()
 	connect(out_center, bump->input("SampleCenter"));
 	connect(out_dx, bump->input("SampleX"));
 	connect(out_dy, bump->input("SampleY"));
+	
+	/* connect the bump out to the set normal in: */
+	connect(bump->output("Normal"), set_normal->input("Direction"));
 
 	/* connect bump output to normal input nodes that aren't set yet. actually
 	 * this will only set the normal input to the geometry node that we created
@@ -538,8 +740,14 @@ void ShaderGraph::bump_from_displacement()
 	foreach(ShaderNode *node, nodes)
 		foreach(ShaderInput *input, node->inputs)
 			if(!input->link && input->default_value == ShaderInput::NORMAL)
-				connect(bump->output("Normal"), input);
-	
+				connect(set_normal->output("Normal"), input);
+
+	/* for displacement bump, clear the normal input in case the above loop
+	 * connected the setnormal out to the bump normalin */
+	ShaderInput *bump_normal_in = bump->input("Normal");
+	if(bump_normal_in)
+		bump_normal_in->link = NULL;
+
 	/* finally, add the copied nodes to the graph. we can't do this earlier
 	 * because we would create dependency cycles in the above loop */
 	foreach(NodePair& pair, nodes_center)
@@ -550,5 +758,124 @@ void ShaderGraph::bump_from_displacement()
 		add(pair.second);
 }
 
+void ShaderGraph::transform_multi_closure(ShaderNode *node, ShaderOutput *weight_out, bool volume)
+{
+	/* for SVM in multi closure mode, this transforms the shader mix/add part of
+	 * the graph into nodes that feed weights into closure nodes. this is too
+	 * avoid building a closure tree and then flattening it, and instead write it
+	 * directly to an array */
+	
+	if(node->name == ustring("mix_closure") || node->name == ustring("add_closure")) {
+		ShaderInput *fin = node->input("Fac");
+		ShaderInput *cl1in = node->input("Closure1");
+		ShaderInput *cl2in = node->input("Closure2");
+		ShaderOutput *weight1_out, *weight2_out;
+
+		if(fin) {
+			/* mix closure: add node to mix closure weights */
+			ShaderNode *mix_node = add(new MixClosureWeightNode());
+			ShaderInput *fac_in = mix_node->input("Fac"); 
+			ShaderInput *weight_in = mix_node->input("Weight"); 
+
+			if(fin->link)
+				connect(fin->link, fac_in);
+			else
+				fac_in->value = fin->value;
+
+			if(weight_out)
+				connect(weight_out, weight_in);
+
+			weight1_out = mix_node->output("Weight1");
+			weight2_out = mix_node->output("Weight2");
+		}
+		else {
+			/* add closure: just pass on any weights */
+			weight1_out = weight_out;
+			weight2_out = weight_out;
+		}
+
+		if(cl1in->link)
+			transform_multi_closure(cl1in->link->parent, weight1_out, volume);
+		if(cl2in->link)
+			transform_multi_closure(cl2in->link->parent, weight2_out, volume);
+	}
+	else {
+		ShaderInput *weight_in = node->input((volume)? "VolumeMixWeight": "SurfaceMixWeight");
+
+		/* not a closure node? */
+		if(!weight_in)
+			return;
+
+		/* already has a weight connected to it? add weights */
+		if(weight_in->link || weight_in->value.x != 0.0f) {
+			ShaderNode *math_node = add(new MathNode());
+			ShaderInput *value1_in = math_node->input("Value1");
+			ShaderInput *value2_in = math_node->input("Value2");
+
+			if(weight_in->link)
+				connect(weight_in->link, value1_in);
+			else
+				value1_in->value = weight_in->value;
+
+			if(weight_out)
+				connect(weight_out, value2_in);
+			else
+				value2_in->value.x = 1.0f;
+
+			weight_out = math_node->output("Value");
+			if(weight_in->link)
+				disconnect(weight_in);
+		}
+
+		/* connected to closure mix weight */
+		if(weight_out)
+			connect(weight_out, weight_in);
+		else
+			weight_in->value.x += 1.0f;
+	}
+}
+
+void ShaderGraph::dump_graph(const char *filename)
+{
+	FILE *fd = fopen(filename, "w");
+
+	if(fd == NULL) {
+		printf("Error opening file for dumping the graph: %s\n", filename);
+		return;
+	}
+
+	fprintf(fd, "digraph shader_graph {\n");
+	fprintf(fd, "ranksep=1.5\n");
+	fprintf(fd, "splines=false\n");
+
+	foreach(ShaderNode *node, nodes) {
+		fprintf(fd, "// NODE: %p\n", node);
+		fprintf(fd,
+		        "\"%p\" [shape=record,label=\"%s\"]\n",
+		        node,
+		        node->name.c_str());
+	}
+
+	foreach(ShaderNode *node, nodes) {
+		foreach(ShaderOutput *output, node->outputs) {
+			foreach(ShaderInput *input, output->links) {
+				fprintf(fd,
+				        "// CONNECTION: %p->%p (%s:%s)\n",
+				        output,
+				        input,
+				        output->name, input->name);
+				fprintf(fd,
+				        "\"%p\":s -> \"%p\":n [label=\"%s:%s\"]\n",
+				        output->parent,
+				        input->parent,
+				        output->name, input->name);
+			}
+		}
+	}
+
+	fprintf(fd, "}\n");
+	fclose(fd);
+}
+
 CCL_NAMESPACE_END
 
