@@ -1,23 +1,23 @@
 /*
- * Copyright 2011, Blender Foundation.
+ * Copyright 2011-2013 Blender Foundation
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * http://www.apache.org/licenses/LICENSE-2.0
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License
  */
 
+#include "background.h"
 #include "device.h"
 #include "integrator.h"
+#include "film.h"
 #include "light.h"
 #include "mesh.h"
 #include "object.h"
@@ -29,7 +29,7 @@
 
 CCL_NAMESPACE_BEGIN
 
-static void dump_background_pixels(Device *device, DeviceScene *dscene, int res, vector<float3>& pixels)
+static void shade_background_pixels(Device *device, DeviceScene *dscene, int res, vector<float3>& pixels, Progress& progress)
 {
 	/* create input */
 	int width = res;
@@ -51,7 +51,7 @@ static void dump_background_pixels(Device *device, DeviceScene *dscene, int res,
 	}
 
 	/* compute on device */
-	float4 *d_output_data = d_output.resize(width*height);
+	d_output.resize(width*height);
 	memset((void*)d_output.data_pointer, 0, d_output.memory_size());
 
 	device->const_copy_to("__data", &dscene->data, sizeof(dscene->data));
@@ -66,10 +66,12 @@ static void dump_background_pixels(Device *device, DeviceScene *dscene, int res,
 	main_task.shader_eval_type = SHADER_EVAL_BACKGROUND;
 	main_task.shader_x = 0;
 	main_task.shader_w = width*height;
+	main_task.num_samples = 1;
+	main_task.get_cancel = function_bind(&Progress::get_cancel, &progress);
 
 	/* disabled splitting for now, there's an issue with multi-GPU mem_copy_from */
 	list<DeviceTask> split_tasks;
-	main_task.split_max_size(split_tasks, 128*128); 
+	main_task.split(split_tasks, 1, 128*128);
 
 	foreach(DeviceTask& task, split_tasks) {
 		device->task_add(task);
@@ -80,7 +82,7 @@ static void dump_background_pixels(Device *device, DeviceScene *dscene, int res,
 	device->mem_free(d_input);
 	device->mem_free(d_output);
 
-	d_output_data = reinterpret_cast<float4*>(d_output.data_pointer);
+	float4 *d_output_data = reinterpret_cast<float4*>(d_output.data_pointer);
 
 	pixels.resize(width*height);
 
@@ -111,12 +113,19 @@ Light::Light()
 
 	map_resolution = 512;
 
-	spot_angle = M_PI_F/4.0f;
+	spot_angle = M_PI_4_F;
 	spot_smooth = 0.0f;
 
 	cast_shadow = true;
+	use_mis = false;
+	use_diffuse = true;
+	use_glossy = true;
+	use_transmission = true;
+	use_scatter = true;
+
 	shader = 0;
 	samples = 1;
+	max_bounces = 1024;
 }
 
 void Light::tag_update(Scene *scene)
@@ -129,6 +138,7 @@ void Light::tag_update(Scene *scene)
 LightManager::LightManager()
 {
 	need_update = true;
+	use_light_visibility = false;
 }
 
 LightManager::~LightManager()
@@ -141,17 +151,26 @@ void LightManager::device_update_distribution(Device *device, DeviceScene *dscen
 
 	/* count */
 	size_t num_lights = scene->lights.size();
+	size_t num_background_lights = 0;
 	size_t num_triangles = 0;
 
 	foreach(Object *object, scene->objects) {
 		Mesh *mesh = object->mesh;
 		bool have_emission = false;
 
+		/* skip if we are not visible for BSDFs */
+		if(!(object->visibility & (PATH_RAY_DIFFUSE|PATH_RAY_GLOSSY|PATH_RAY_TRANSMIT)))
+			continue;
+
+		/* skip motion blurred deforming meshes, not supported yet */
+		if(mesh->has_motion_blur())
+			continue;
+
 		/* skip if we have no emission shaders */
 		foreach(uint sindex, mesh->used_shaders) {
 			Shader *shader = scene->shaders[sindex];
 
-			if(shader->sample_as_light && shader->has_surface_emission) {
+			if(shader->use_mis && shader->has_surface_emission) {
 				have_emission = true;
 				break;
 			}
@@ -162,14 +181,13 @@ void LightManager::device_update_distribution(Device *device, DeviceScene *dscen
 			for(size_t i = 0; i < mesh->triangles.size(); i++) {
 				Shader *shader = scene->shaders[mesh->shader[i]];
 
-				if(shader->sample_as_light && shader->has_surface_emission)
+				if(shader->use_mis && shader->has_surface_emission)
 					num_triangles++;
 			}
 		}
 	}
 
-	size_t num_distribution = num_triangles;
-	num_distribution += num_lights;
+	size_t num_distribution = num_triangles + num_lights;
 
 	/* emission area */
 	float4 *distribution = dscene->light_distribution.resize(num_distribution + 1);
@@ -183,11 +201,23 @@ void LightManager::device_update_distribution(Device *device, DeviceScene *dscen
 		Mesh *mesh = object->mesh;
 		bool have_emission = false;
 
+		/* skip if we are not visible for BSDFs */
+		if(!(object->visibility & (PATH_RAY_DIFFUSE|PATH_RAY_GLOSSY|PATH_RAY_TRANSMIT))) {
+			j++;
+			continue;
+		}
+
+		/* skip motion blurred deforming meshes, not supported yet */
+		if(mesh->has_motion_blur()) {
+			j++;
+			continue;
+		}
+
 		/* skip if we have no emission shaders */
 		foreach(uint sindex, mesh->used_shaders) {
 			Shader *shader = scene->shaders[sindex];
 
-			if(shader->sample_as_light && shader->has_surface_emission) {
+			if(shader->use_mis && shader->has_surface_emission) {
 				have_emission = true;
 				break;
 			}
@@ -198,17 +228,35 @@ void LightManager::device_update_distribution(Device *device, DeviceScene *dscen
 			bool transform_applied = mesh->transform_applied;
 			Transform tfm = object->tfm;
 			int object_id = j;
+			int shader_flag = 0;
 
 			if(transform_applied)
 				object_id = ~object_id;
 
+			if(!(object->visibility & PATH_RAY_DIFFUSE)) {
+				shader_flag |= SHADER_EXCLUDE_DIFFUSE;
+				use_light_visibility = true;
+			}
+			if(!(object->visibility & PATH_RAY_GLOSSY)) {
+				shader_flag |= SHADER_EXCLUDE_GLOSSY;
+				use_light_visibility = true;
+			}
+			if(!(object->visibility & PATH_RAY_TRANSMIT)) {
+				shader_flag |= SHADER_EXCLUDE_TRANSMIT;
+				use_light_visibility = true;
+			}
+			if(!(object->visibility & PATH_RAY_VOLUME_SCATTER)) {
+				shader_flag |= SHADER_EXCLUDE_SCATTER;
+				use_light_visibility = true;
+			}
+
 			for(size_t i = 0; i < mesh->triangles.size(); i++) {
 				Shader *shader = scene->shaders[mesh->shader[i]];
 
-				if(shader->sample_as_light && shader->has_surface_emission) {
+				if(shader->use_mis && shader->has_surface_emission) {
 					distribution[offset].x = totarea;
 					distribution[offset].y = __int_as_float(i + mesh->tri_offset);
-					distribution[offset].z = 1.0f;
+					distribution[offset].z = __int_as_float(shader_flag);
 					distribution[offset].w = __int_as_float(object_id);
 					offset++;
 
@@ -237,13 +285,21 @@ void LightManager::device_update_distribution(Device *device, DeviceScene *dscen
 
 	/* point lights */
 	float lightarea = (totarea > 0.0f)? totarea/scene->lights.size(): 1.0f;
+	bool use_lamp_mis = false;
 
 	for(int i = 0; i < scene->lights.size(); i++, offset++) {
+		Light *light = scene->lights[i];
+
 		distribution[offset].x = totarea;
 		distribution[offset].y = __int_as_float(~(int)i);
 		distribution[offset].z = 1.0f;
-		distribution[offset].w = scene->lights[i]->size;
+		distribution[offset].w = light->size;
 		totarea += lightarea;
+
+		if(light->size > 0.0f && light->use_mis)
+			use_lamp_mis = true;
+		if(light->type == LIGHT_BACKGROUND)
+			num_background_lights++;
 	}
 
 	/* normalize cumulative distribution functions */
@@ -262,15 +318,17 @@ void LightManager::device_update_distribution(Device *device, DeviceScene *dscen
 
 	/* update device */
 	KernelIntegrator *kintegrator = &dscene->data.integrator;
+	KernelFilm *kfilm = &dscene->data.film;
 	kintegrator->use_direct_light = (totarea > 0.0f);
 
 	if(kintegrator->use_direct_light) {
 		/* number of emissives */
-		kintegrator->num_distribution = (totarea > 0.0f)? num_distribution: 0;
+		kintegrator->num_distribution = num_distribution;
 
 		/* precompute pdfs */
 		kintegrator->pdf_triangles = 0.0f;
 		kintegrator->pdf_lights = 0.0f;
+		kintegrator->inv_pdf_lights = 0.0f;
 
 		/* sample one, with 0.5 probability of light or triangle */
 		kintegrator->num_all_lights = num_lights;
@@ -285,8 +343,22 @@ void LightManager::device_update_distribution(Device *device, DeviceScene *dscen
 			kintegrator->pdf_lights = 1.0f/num_lights;
 			if(trianglearea > 0.0f)
 				kintegrator->pdf_lights *= 0.5f;
+
+			kintegrator->inv_pdf_lights = 1.0f/kintegrator->pdf_lights;
 		}
 
+		kintegrator->use_lamp_mis = use_lamp_mis;
+
+		/* bit of an ugly hack to compensate for emitting triangles influencing
+		 * amount of samples we get for this pass */
+		kfilm->pass_shadow_scale = 1.0f;
+
+		if(kintegrator->pdf_triangles != 0.0f)
+			kfilm->pass_shadow_scale *= 0.5f;
+
+		if(num_background_lights < num_lights)
+			kfilm->pass_shadow_scale *= (float)(num_lights - num_background_lights)/(float)num_lights;
+
 		/* CDF */
 		device->tex_alloc("__light_distribution", dscene->light_distribution);
 	}
@@ -297,6 +369,9 @@ void LightManager::device_update_distribution(Device *device, DeviceScene *dscen
 		kintegrator->num_all_lights = 0;
 		kintegrator->pdf_triangles = 0.0f;
 		kintegrator->pdf_lights = 0.0f;
+		kintegrator->inv_pdf_lights = 0.0f;
+		kintegrator->use_lamp_mis = false;
+		kfilm->pass_shadow_scale = 1.0f;
 	}
 }
 
@@ -330,7 +405,7 @@ void LightManager::device_update_background(Device *device, DeviceScene *dscene,
 	assert(res > 0);
 
 	vector<float3> pixels;
-	dump_background_pixels(device, dscene, res, pixels);
+	shade_background_pixels(device, dscene, res, pixels, progress);
 
 	if(progress.get_cancel())
 		return;
@@ -413,56 +488,132 @@ void LightManager::device_update_points(Device *device, DeviceScene *dscene, Sce
 	for(size_t i = 0; i < scene->lights.size(); i++) {
 		Light *light = scene->lights[i];
 		float3 co = light->co;
-		float3 dir = normalize(light->dir);
 		int shader_id = scene->shader_manager->get_shader_id(scene->lights[i]->shader);
 		float samples = __int_as_float(light->samples);
+		float max_bounces = __int_as_float(light->max_bounces);
 
 		if(!light->cast_shadow)
 			shader_id &= ~SHADER_CAST_SHADOW;
 
+		if(!light->use_diffuse) {
+			shader_id |= SHADER_EXCLUDE_DIFFUSE;
+			use_light_visibility = true;
+		}
+		if(!light->use_glossy) {
+			shader_id |= SHADER_EXCLUDE_GLOSSY;
+			use_light_visibility = true;
+		}
+		if(!light->use_transmission) {
+			shader_id |= SHADER_EXCLUDE_TRANSMIT;
+			use_light_visibility = true;
+		}
+		if(!light->use_scatter) {
+			shader_id |= SHADER_EXCLUDE_SCATTER;
+			use_light_visibility = true;
+		}
+
 		if(light->type == LIGHT_POINT) {
 			shader_id &= ~SHADER_AREA_LIGHT;
 
+			float radius = light->size;
+			float invarea = (radius > 0.0f)? 1.0f/(M_PI_F*radius*radius): 1.0f;
+
+			if(light->use_mis && radius > 0.0f)
+				shader_id |= SHADER_USE_MIS;
+
 			light_data[i*LIGHT_SIZE + 0] = make_float4(__int_as_float(light->type), co.x, co.y, co.z);
-			light_data[i*LIGHT_SIZE + 1] = make_float4(__int_as_float(shader_id), light->size, 0.0f, 0.0f);
+			light_data[i*LIGHT_SIZE + 1] = make_float4(__int_as_float(shader_id), radius, invarea, 0.0f);
 			light_data[i*LIGHT_SIZE + 2] = make_float4(0.0f, 0.0f, 0.0f, 0.0f);
 			light_data[i*LIGHT_SIZE + 3] = make_float4(samples, 0.0f, 0.0f, 0.0f);
+			light_data[i*LIGHT_SIZE + 4] = make_float4(max_bounces, 0.0f, 0.0f, 0.0f);
 		}
 		else if(light->type == LIGHT_DISTANT) {
 			shader_id &= ~SHADER_AREA_LIGHT;
 
+			float radius = light->size;
+			float angle = atanf(radius);
+			float cosangle = cosf(angle);
+			float area = M_PI_F*radius*radius;
+			float invarea = (area > 0.0f)? 1.0f/area: 1.0f;
+			float3 dir = light->dir;
+
+			dir = safe_normalize(dir);
+
+			if(light->use_mis && area > 0.0f)
+				shader_id |= SHADER_USE_MIS;
+
 			light_data[i*LIGHT_SIZE + 0] = make_float4(__int_as_float(light->type), dir.x, dir.y, dir.z);
-			light_data[i*LIGHT_SIZE + 1] = make_float4(__int_as_float(shader_id), light->size, 0.0f, 0.0f);
+			light_data[i*LIGHT_SIZE + 1] = make_float4(__int_as_float(shader_id), radius, cosangle, invarea);
 			light_data[i*LIGHT_SIZE + 2] = make_float4(0.0f, 0.0f, 0.0f, 0.0f);
 			light_data[i*LIGHT_SIZE + 3] = make_float4(samples, 0.0f, 0.0f, 0.0f);
+			light_data[i*LIGHT_SIZE + 4] = make_float4(max_bounces, 0.0f, 0.0f, 0.0f);
 		}
 		else if(light->type == LIGHT_BACKGROUND) {
+			uint visibility = scene->background->visibility;
+
 			shader_id &= ~SHADER_AREA_LIGHT;
+			shader_id |= SHADER_USE_MIS;
+
+			if(!(visibility & PATH_RAY_DIFFUSE)) {
+				shader_id |= SHADER_EXCLUDE_DIFFUSE;
+				use_light_visibility = true;
+			}
+			if(!(visibility & PATH_RAY_GLOSSY)) {
+				shader_id |= SHADER_EXCLUDE_GLOSSY;
+				use_light_visibility = true;
+			}
+			if(!(visibility & PATH_RAY_TRANSMIT)) {
+				shader_id |= SHADER_EXCLUDE_TRANSMIT;
+				use_light_visibility = true;
+			}
+			if(!(visibility & PATH_RAY_VOLUME_SCATTER)) {
+				shader_id |= SHADER_EXCLUDE_SCATTER;
+				use_light_visibility = true;
+			}
 
 			light_data[i*LIGHT_SIZE + 0] = make_float4(__int_as_float(light->type), 0.0f, 0.0f, 0.0f);
 			light_data[i*LIGHT_SIZE + 1] = make_float4(__int_as_float(shader_id), 0.0f, 0.0f, 0.0f);
 			light_data[i*LIGHT_SIZE + 2] = make_float4(0.0f, 0.0f, 0.0f, 0.0f);
 			light_data[i*LIGHT_SIZE + 3] = make_float4(samples, 0.0f, 0.0f, 0.0f);
+			light_data[i*LIGHT_SIZE + 4] = make_float4(max_bounces, 0.0f, 0.0f, 0.0f);
 		}
 		else if(light->type == LIGHT_AREA) {
 			float3 axisu = light->axisu*(light->sizeu*light->size);
 			float3 axisv = light->axisv*(light->sizev*light->size);
+			float area = len(axisu)*len(axisv);
+			float invarea = (area > 0.0f)? 1.0f/area: 1.0f;
+			float3 dir = light->dir;
+			
+			dir = safe_normalize(dir);
+
+			if(light->use_mis && area > 0.0f)
+				shader_id |= SHADER_USE_MIS;
 
 			light_data[i*LIGHT_SIZE + 0] = make_float4(__int_as_float(light->type), co.x, co.y, co.z);
 			light_data[i*LIGHT_SIZE + 1] = make_float4(__int_as_float(shader_id), axisu.x, axisu.y, axisu.z);
-			light_data[i*LIGHT_SIZE + 2] = make_float4(0.0f, axisv.x, axisv.y, axisv.z);
+			light_data[i*LIGHT_SIZE + 2] = make_float4(invarea, axisv.x, axisv.y, axisv.z);
 			light_data[i*LIGHT_SIZE + 3] = make_float4(samples, dir.x, dir.y, dir.z);
+			light_data[i*LIGHT_SIZE + 4] = make_float4(max_bounces, 0.0f, 0.0f, 0.0f);
 		}
 		else if(light->type == LIGHT_SPOT) {
 			shader_id &= ~SHADER_AREA_LIGHT;
 
+			float radius = light->size;
+			float invarea = (radius > 0.0f)? 1.0f/(M_PI_F*radius*radius): 1.0f;
 			float spot_angle = cosf(light->spot_angle*0.5f);
 			float spot_smooth = (1.0f - spot_angle)*light->spot_smooth;
+			float3 dir = light->dir;
+			
+			dir = safe_normalize(dir);
+
+			if(light->use_mis && radius > 0.0f)
+				shader_id |= SHADER_USE_MIS;
 
 			light_data[i*LIGHT_SIZE + 0] = make_float4(__int_as_float(light->type), co.x, co.y, co.z);
-			light_data[i*LIGHT_SIZE + 1] = make_float4(__int_as_float(shader_id), light->size, dir.x, dir.y);
-			light_data[i*LIGHT_SIZE + 2] = make_float4(dir.z, spot_angle, spot_smooth, 0.0f);
+			light_data[i*LIGHT_SIZE + 1] = make_float4(__int_as_float(shader_id), radius, invarea, spot_angle);
+			light_data[i*LIGHT_SIZE + 2] = make_float4(spot_smooth, dir.x, dir.y, dir.z);
 			light_data[i*LIGHT_SIZE + 3] = make_float4(samples, 0.0f, 0.0f, 0.0f);
+			light_data[i*LIGHT_SIZE + 4] = make_float4(max_bounces, 0.0f, 0.0f, 0.0f);
 		}
 	}
 	
@@ -476,6 +627,8 @@ void LightManager::device_update(Device *device, DeviceScene *dscene, Scene *sce
 
 	device_free(device, dscene);
 
+	use_light_visibility = false;
+
 	device_update_points(device, dscene, scene);
 	if(progress.get_cancel()) return;
 
@@ -485,6 +638,11 @@ void LightManager::device_update(Device *device, DeviceScene *dscene, Scene *sce
 	device_update_background(device, dscene, scene, progress);
 	if(progress.get_cancel()) return;
 
+	if(use_light_visibility != scene->film->use_light_visibility) {
+		scene->film->use_light_visibility = use_light_visibility;
+		scene->film->tag_update(scene);
+	}
+
 	need_update = false;
 }
 
