@@ -1,19 +1,17 @@
 /*
- * Copyright 2011, Blender Foundation.
+ * Copyright 2011-2013 Blender Foundation
  *
- * This program is free software; you can redistribute it and/or
- * modify it under the terms of the GNU General Public License
- * as published by the Free Software Foundation; either version 2
- * of the License, or (at your option) any later version.
+ * Licensed under the Apache License, Version 2.0 (the "License");
+ * you may not use this file except in compliance with the License.
+ * You may obtain a copy of the License at
  *
- * This program is distributed in the hope that it will be useful,
- * but WITHOUT ANY WARRANTY; without even the implied warranty of
- * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
- * GNU General Public License for more details.
+ * http://www.apache.org/licenses/LICENSE-2.0
  *
- * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software Foundation,
- * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ * Unless required by applicable law or agreed to in writing, software
+ * distributed under the License is distributed on an "AS IS" BASIS,
+ * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
+ * See the License for the specific language governing permissions and
+ * limitations under the License
  */
 
 #include <string.h>
@@ -22,8 +20,10 @@
 #include "buffers.h"
 #include "camera.h"
 #include "device.h"
+#include "integrator.h"
 #include "scene.h"
 #include "session.h"
+#include "bake.h"
 
 #include "util_foreach.h"
 #include "util_function.h"
@@ -34,24 +34,30 @@
 
 CCL_NAMESPACE_BEGIN
 
+/* Note about  preserve_tile_device option for tile manager:
+ * progressive refine and viewport rendering does requires tiles to
+ * always be allocated for the same device
+ */
 Session::Session(const SessionParams& params_)
 : params(params_),
   tile_manager(params.progressive, params.samples, params.tile_size, params.start_resolution,
-  	(params.background)? 1: max(params.device.multi_devices.size(), 1))
+       params.background == false || params.progressive_refine, params.background, params.tile_order,
+       max(params.device.multi_devices.size(), 1)),
+  stats()
 {
 	device_use_gl = ((params.device.type != DEVICE_CPU) && !params.background);
 
 	TaskScheduler::init(params.threads);
 
-	device = Device::create(params.device, params.background, params.threads);
+	device = Device::create(params.device, stats, params.background);
 
-	if(params.background) {
+	if(params.background && params.output_path.empty()) {
 		buffers = NULL;
 		display = NULL;
 	}
 	else {
 		buffers = new RenderBuffers(device);
-		display = new DisplayBuffer(device);
+		display = new DisplayBuffer(device, params.display_buffer_linear);
 	}
 
 	session_thread = NULL;
@@ -61,6 +67,7 @@ Session::Session(const SessionParams& params_)
 	reset_time = 0.0;
 	preview_time = 0.0;
 	paused_time = 0.0;
+	last_update_time = 0.0;
 
 	delayed_reset.do_reset = false;
 	delayed_reset.samples = 0;
@@ -75,6 +82,7 @@ Session::Session(const SessionParams& params_)
 Session::~Session()
 {
 	if(session_thread) {
+		/* wait for session thread to end */
 		progress.set_cancel("Exiting");
 
 		gpu_need_tonemap = false;
@@ -89,13 +97,22 @@ Session::~Session()
 		wait();
 	}
 
-	if(display && params.output_path != "") {
-		tonemap();
+	if(!params.output_path.empty()) {
+		/* tonemap and write out image if requested */
+		delete display;
+
+		display = new DisplayBuffer(device, false);
+		display->reset(device, buffers->params);
+		tonemap(params.samples);
 
 		progress.set_status("Writing Image", params.output_path);
 		display->write(device, params.output_path);
 	}
 
+	/* clean up */
+	foreach(RenderBuffers *buffers, tile_buffers)
+		delete buffers;
+
 	delete buffers;
 	delete display;
 	delete scene;
@@ -123,7 +140,9 @@ bool Session::ready_to_reset()
 
 void Session::reset_gpu(BufferParams& buffer_params, int samples)
 {
-	/* block for buffer acces and reset immediately. we can't do this
+	thread_scoped_lock pause_lock(pause_mutex);
+
+	/* block for buffer access and reset immediately. we can't do this
 	 * in the thread, because we need to allocate an OpenGL buffer, and
 	 * that only works in the main thread */
 	thread_scoped_lock display_lock(display_mutex);
@@ -140,7 +159,7 @@ void Session::reset_gpu(BufferParams& buffer_params, int samples)
 	pause_cond.notify_all();
 }
 
-bool Session::draw_gpu(BufferParams& buffer_params)
+bool Session::draw_gpu(BufferParams& buffer_params, DeviceDrawParams& draw_params)
 {
 	/* block for buffer access */
 	thread_scoped_lock display_lock(display_mutex);
@@ -154,12 +173,12 @@ bool Session::draw_gpu(BufferParams& buffer_params)
 			 * only access GL buffers from the main thread */
 			if(gpu_need_tonemap) {
 				thread_scoped_lock buffers_lock(buffers_mutex);
-				tonemap();
+				tonemap(tile_manager.state.sample);
 				gpu_need_tonemap = false;
 				gpu_need_tonemap_cond.notify_all();
 			}
 
-			display->draw(device);
+			display->draw(device, draw_params);
 
 			if(display_outdated && (time_dt() - reset_time) > params.text_timeout)
 				return false;
@@ -173,9 +192,12 @@ bool Session::draw_gpu(BufferParams& buffer_params)
 
 void Session::run_gpu()
 {
+	bool tiles_written = false;
+
 	start_time = time_dt();
 	reset_time = time_dt();
 	paused_time = 0.0;
+	last_update_time = time_dt();
 
 	if(!params.background)
 		progress.set_start_time(start_time + paused_time);
@@ -196,7 +218,12 @@ void Session::run_gpu()
 			 * wait for pause condition notify to wake up again */
 			thread_scoped_lock pause_lock(pause_mutex);
 
-			if(pause || no_tiles) {
+			if(!pause && !tile_manager.done()) {
+				/* reset could have happened after no_tiles was set, before this lock.
+				 * in this case we shall not wait for pause condition
+				 */
+			}
+			else if(pause || no_tiles) {
 				update_status_time(pause, no_tiles);
 
 				while(1) {
@@ -223,8 +250,8 @@ void Session::run_gpu()
 			/* update scene */
 			update_scene();
 
-			if(device->error_message() != "")
-				progress.set_cancel(device->error_message());
+			if(!device->error_message().empty())
+				progress.set_error(device->error_message());
 
 			if(progress.get_cancel())
 				break;
@@ -244,7 +271,7 @@ void Session::run_gpu()
 
 			device->task_wait();
 
-			if(device->error_message() != "")
+			if(!device->error_message().empty())
 				progress.set_cancel(device->error_message());
 
 			/* update status and timing */
@@ -264,13 +291,18 @@ void Session::run_gpu()
 				}
 			}
 
-			if(device->error_message() != "")
-				progress.set_cancel(device->error_message());
+			if(!device->error_message().empty())
+				progress.set_error(device->error_message());
+
+			tiles_written = update_progressive_refine(progress.get_cancel());
 
 			if(progress.get_cancel())
 				break;
 		}
 	}
+
+	if(!tiles_written)
+		update_progressive_refine(true);
 }
 
 /* CPU Session */
@@ -278,6 +310,7 @@ void Session::run_gpu()
 void Session::reset_cpu(BufferParams& buffer_params, int samples)
 {
 	thread_scoped_lock reset_lock(delayed_reset.mutex);
+	thread_scoped_lock pause_lock(pause_mutex);
 
 	display_outdated = true;
 	reset_time = time_dt();
@@ -290,7 +323,7 @@ void Session::reset_cpu(BufferParams& buffer_params, int samples)
 	pause_cond.notify_all();
 }
 
-bool Session::draw_cpu(BufferParams& buffer_params)
+bool Session::draw_cpu(BufferParams& buffer_params, DeviceDrawParams& draw_params)
 {
 	thread_scoped_lock display_lock(display_mutex);
 
@@ -299,7 +332,7 @@ bool Session::draw_cpu(BufferParams& buffer_params)
 		/* then verify the buffers have the expected size, so we don't
 		 * draw previous results in a resized window */
 		if(!buffer_params.modified(display->params)) {
-			display->draw(device);
+			display->draw(device, draw_params);
 
 			if(display_outdated && (time_dt() - reset_time) > params.text_timeout)
 				return false;
@@ -313,14 +346,18 @@ bool Session::draw_cpu(BufferParams& buffer_params)
 
 bool Session::acquire_tile(Device *tile_device, RenderTile& rtile)
 {
-	if(progress.get_cancel())
-		return false;
+	if(progress.get_cancel()) {
+		if(params.progressive_refine == false) {
+			/* for progressive refine current sample should be finished for all tiles */
+			return false;
+		}
+	}
 
 	thread_scoped_lock tile_lock(tile_mutex);
 
 	/* get next tile from manager */
 	Tile tile;
-	int device_num = (params.background)? 0: device->device_number(tile_device);
+	int device_num = device->device_number(tile_device);
 
 	if(!tile_manager.next_tile(tile, device_num))
 		return false;
@@ -336,14 +373,13 @@ bool Session::acquire_tile(Device *tile_device, RenderTile& rtile)
 
 	tile_lock.unlock();
 
-	/* in case of a permant buffer, return it, otherwise we will allocate
+	/* in case of a permanent buffer, return it, otherwise we will allocate
 	 * a new temporary buffer */
-	if(!write_render_tile_cb) {
+	if(!(params.background && params.output_path.empty())) {
 		tile_manager.state.buffer.get_offset_stride(rtile.offset, rtile.stride);
 
 		rtile.buffer = buffers->buffer.device_pointer;
 		rtile.rng_state = buffers->rng_state.device_pointer;
-		rtile.rgba = display->rgba.device_pointer;
 		rtile.buffers = buffers;
 
 		device->map_tile(tile_device, rtile);
@@ -360,15 +396,41 @@ bool Session::acquire_tile(Device *tile_device, RenderTile& rtile)
 
 	buffer_params.get_offset_stride(rtile.offset, rtile.stride);
 
+	RenderBuffers *tilebuffers;
+
 	/* allocate buffers */
-	RenderBuffers *tilebuffers = new RenderBuffers(tile_device);
-	tilebuffers->reset(tile_device, buffer_params);
+	if(params.progressive_refine) {
+		tile_lock.lock();
+
+		if(tile_buffers.size() == 0)
+			tile_buffers.resize(tile_manager.state.num_tiles, NULL);
+
+		tilebuffers = tile_buffers[tile.index];
+		if(tilebuffers == NULL) {
+			tilebuffers = new RenderBuffers(tile_device);
+			tile_buffers[tile.index] = tilebuffers;
+
+			tilebuffers->reset(tile_device, buffer_params);
+		}
+
+		tile_lock.unlock();
+	}
+	else {
+		tilebuffers = new RenderBuffers(tile_device);
+
+		tilebuffers->reset(tile_device, buffer_params);
+	}
 
 	rtile.buffer = tilebuffers->buffer.device_pointer;
 	rtile.rng_state = tilebuffers->rng_state.device_pointer;
-	rtile.rgba = 0;
 	rtile.buffers = tilebuffers;
 
+	/* this will tag tile as IN PROGRESS in blender-side render pipeline,
+	 * which is needed to highlight currently rendering tile before first
+	 * sample was processed for it
+	 */
+	update_tile_sample(rtile);
+
 	return true;
 }
 
@@ -377,9 +439,11 @@ void Session::update_tile_sample(RenderTile& rtile)
 	thread_scoped_lock tile_lock(tile_mutex);
 
 	if(update_render_tile_cb) {
-		/* todo: optimize this by making it thread safe and removing lock */
+		if(params.progressive_refine == false) {
+			/* todo: optimize this by making it thread safe and removing lock */
 
-		update_render_tile_cb(rtile);
+			update_render_tile_cb(rtile);
+		}
 	}
 
 	update_status_time();
@@ -390,10 +454,12 @@ void Session::release_tile(RenderTile& rtile)
 	thread_scoped_lock tile_lock(tile_mutex);
 
 	if(write_render_tile_cb) {
-		/* todo: optimize this by making it thread safe and removing lock */
-		write_render_tile_cb(rtile);
+		if(params.progressive_refine == false) {
+			/* todo: optimize this by making it thread safe and removing lock */
+			write_render_tile_cb(rtile);
 
-		delete rtile.buffers;
+			delete rtile.buffers;
+		}
 	}
 
 	update_status_time();
@@ -401,6 +467,10 @@ void Session::release_tile(RenderTile& rtile)
 
 void Session::run_cpu()
 {
+	bool tiles_written = false;
+
+	last_update_time = time_dt();
+
 	{
 		/* reset once to start */
 		thread_scoped_lock reset_lock(delayed_reset.mutex);
@@ -428,7 +498,16 @@ void Session::run_cpu()
 			 * wait for pause condition notify to wake up again */
 			thread_scoped_lock pause_lock(pause_mutex);
 
-			if(pause || no_tiles) {
+			if(!pause && delayed_reset.do_reset) {
+				/* reset once to start */
+				thread_scoped_lock reset_lock(delayed_reset.mutex);
+				thread_scoped_lock buffers_lock(buffers_mutex);
+				thread_scoped_lock display_lock(display_mutex);
+
+				reset_(delayed_reset.params, delayed_reset.samples);
+				delayed_reset.do_reset = false;
+			}
+			else if(pause || no_tiles) {
 				update_status_time(pause, no_tiles);
 
 				while(1) {
@@ -460,8 +539,8 @@ void Session::run_cpu()
 			/* update scene */
 			update_scene();
 
-			if(device->error_message() != "")
-				progress.set_cancel(device->error_message());
+			if(!device->error_message().empty())
+				progress.set_error(device->error_message());
 
 			if(progress.get_cancel())
 				break;
@@ -478,8 +557,8 @@ void Session::run_cpu()
 			if(!params.background)
 				need_tonemap = true;
 
-			if(device->error_message() != "")
-				progress.set_cancel(device->error_message());
+			if(!device->error_message().empty())
+				progress.set_error(device->error_message());
 		}
 
 		device->task_wait();
@@ -496,29 +575,36 @@ void Session::run_cpu()
 			}
 			else if(need_tonemap) {
 				/* tonemap only if we do not reset, we don't we don't
-				 * wan't to show the result of an incomplete sample*/
-				tonemap();
+				 * want to show the result of an incomplete sample */
+				tonemap(tile_manager.state.sample);
 			}
 
-			if(device->error_message() != "")
-				progress.set_cancel(device->error_message());
+			if(!device->error_message().empty())
+				progress.set_error(device->error_message());
+
+			tiles_written = update_progressive_refine(progress.get_cancel());
 		}
 
 		progress.set_update();
 	}
+
+	if(!tiles_written)
+		update_progressive_refine(true);
 }
 
-void Session::run()
+void Session::load_kernels()
 {
-	/* load kernels */
+	thread_scoped_lock scene_lock(scene->mutex);
+
 	if(!kernels_loaded) {
 		progress.set_status("Loading render kernels (may take a few minutes the first time)");
 
 		if(!device->load_kernels(params.experimental)) {
 			string message = device->error_message();
-			if(message == "")
+			if(message.empty())
 				message = "Failed loading render kernel, see console for errors";
 
+			progress.set_error(message);
 			progress.set_status("Error", message);
 			progress.set_update();
 			return;
@@ -526,6 +612,12 @@ void Session::run()
 
 		kernels_loaded = true;
 	}
+}
+
+void Session::run()
+{
+	/* load kernels */
+	load_kernels();
 
 	/* session thread loop */
 	progress.set_status("Waiting for render to start");
@@ -548,12 +640,12 @@ void Session::run()
 		progress.set_update();
 }
 
-bool Session::draw(BufferParams& buffer_params)
+bool Session::draw(BufferParams& buffer_params, DeviceDrawParams &draw_params)
 {
 	if(device_use_gl)
-		return draw_gpu(buffer_params);
+		return draw_gpu(buffer_params, draw_params);
 	else
-		return draw_cpu(buffer_params);
+		return draw_cpu(buffer_params, draw_params);
 }
 
 void Session::reset_(BufferParams& buffer_params, int samples)
@@ -582,6 +674,15 @@ void Session::reset(BufferParams& buffer_params, int samples)
 		reset_gpu(buffer_params, samples);
 	else
 		reset_cpu(buffer_params, samples);
+
+	if(params.progressive_refine) {
+		thread_scoped_lock buffers_lock(buffers_mutex);
+
+		foreach(RenderBuffers *buffers, tile_buffers)
+			delete buffers;
+
+		tile_buffers.clear();
+	}
 }
 
 void Session::set_samples(int samples)
@@ -632,13 +733,31 @@ void Session::update_scene()
 	Camera *cam = scene->camera;
 	int width = tile_manager.state.buffer.full_width;
 	int height = tile_manager.state.buffer.full_height;
+	int resolution = tile_manager.state.resolution_divider;
 
 	if(width != cam->width || height != cam->height) {
 		cam->width = width;
 		cam->height = height;
+		cam->resolution = resolution;
 		cam->tag_update();
 	}
 
+	/* number of samples is needed by multi jittered
+	 * sampling pattern and by baking */
+	Integrator *integrator = scene->integrator;
+	BakeManager *bake_manager = scene->bake_manager;
+
+	if(integrator->sampling_pattern == SAMPLING_PATTERN_CMJ ||
+	   bake_manager->get_baking())
+	{
+		int aa_samples = tile_manager.num_samples;
+
+		if(aa_samples != integrator->aa_samples) {
+			integrator->aa_samples = aa_samples;
+			integrator->tag_update(scene);
+		}
+	}
+
 	/* update scene */
 	if(scene->need_update()) {
 		progress.set_status("Updating Scene");
@@ -657,10 +776,13 @@ void Session::update_status_time(bool show_pause, bool show_done)
 	string status, substatus;
 
 	if(!params.progressive) {
+		bool is_gpu = params.device.type == DEVICE_CUDA || params.device.type == DEVICE_OPENCL;
+		bool is_multidevice = params.device.multi_devices.size() > 1;
+		bool is_cpu = params.device.type == DEVICE_CPU;
+
 		substatus = string_printf("Path Tracing Tile %d/%d", tile, num_tiles);
 
-		if(params.device.type == DEVICE_CUDA || params.device.type == DEVICE_OPENCL ||
-			(params.device.type == DEVICE_CPU && num_tiles == 1)) {
+		if((is_gpu && !is_multidevice) || (is_cpu && num_tiles == 1)) {
 			/* when rendering on GPU multithreading happens within single tile, as in
 			 * tiles are handling sequentially and in this case we could display
 			 * currently rendering sample number
@@ -668,7 +790,7 @@ void Session::update_status_time(bool show_pause, bool show_done)
 			 * also display the info on CPU, when using 1 tile only
 			 */
 
-			int sample = progress.get_sample(), num_samples = tile_manager.state.num_samples;
+			int sample = progress.get_sample(), num_samples = tile_manager.num_samples;
 
 			if(tile > 1) {
 				/* sample counter is global for all tiles, subtract samples
@@ -681,17 +803,21 @@ void Session::update_status_time(bool show_pause, bool show_done)
 			substatus += string_printf(", Sample %d/%d", sample, num_samples);
 		}
 	}
-	else if(params.samples == INT_MAX)
+	else if(tile_manager.num_samples == USHRT_MAX)
 		substatus = string_printf("Path Tracing Sample %d", sample+1);
 	else
-		substatus = string_printf("Path Tracing Sample %d/%d", sample+1, params.samples);
+		substatus = string_printf("Path Tracing Sample %d/%d", sample+1, tile_manager.num_samples);
 	
-	if(show_pause)
+	if(show_pause) {
 		status = "Paused";
-	else if(show_done)
+	}
+	else if(show_done) {
 		status = "Done";
-	else
-		status = "Rendering";
+	}
+	else {
+		status = substatus;
+		substatus.clear();
+	}
 
 	progress.set_status(status, substatus);
 
@@ -699,7 +825,7 @@ void Session::update_status_time(bool show_pause, bool show_done)
 	if(preview_time == 0.0 && resolution == 1)
 		preview_time = time_dt();
 	
-	double tile_time = (tile == 0)? 0.0: (time_dt() - preview_time - paused_time)/(sample);
+	double tile_time = (tile == 0 || sample == 0)? 0.0: (time_dt() - preview_time - paused_time) / sample;
 
 	/* negative can happen when we pause a bit before rendering, can discard that */
 	if(preview_time < 0.0) preview_time = 0.0;
@@ -722,23 +848,25 @@ void Session::path_trace()
 	task.get_cancel = function_bind(&Progress::get_cancel, &this->progress);
 	task.update_tile_sample = function_bind(&Session::update_tile_sample, this, _1);
 	task.update_progress_sample = function_bind(&Session::update_progress_sample, this);
+	task.need_finish_queue = params.progressive_refine;
+	task.integrator_branched = scene->integrator->method == Integrator::BRANCHED_PATH;
 
 	device->task_add(task);
 }
 
-void Session::tonemap()
+void Session::tonemap(int sample)
 {
 	/* add tonemap task */
-	DeviceTask task(DeviceTask::TONEMAP);
+	DeviceTask task(DeviceTask::FILM_CONVERT);
 
 	task.x = tile_manager.state.buffer.full_x;
 	task.y = tile_manager.state.buffer.full_y;
 	task.w = tile_manager.state.buffer.width;
 	task.h = tile_manager.state.buffer.height;
-	task.rgba = display->rgba.device_pointer;
+	task.rgba_byte = display->rgba_byte.device_pointer;
+	task.rgba_half = display->rgba_half.device_pointer;
 	task.buffer = buffers->buffer.device_pointer;
-	task.sample = tile_manager.state.sample;
-	task.resolution = tile_manager.state.resolution_divider;
+	task.sample = sample;
 	tile_manager.state.buffer.get_offset_stride(task.offset, task.stride);
 
 	if(task.w > 0 && task.h > 0) {
@@ -752,5 +880,49 @@ void Session::tonemap()
 	display_outdated = false;
 }
 
-CCL_NAMESPACE_END
+bool Session::update_progressive_refine(bool cancel)
+{
+	int sample = tile_manager.state.sample + 1;
+	bool write = sample == tile_manager.num_samples || cancel;
 
+	double current_time = time_dt();
+
+	if (current_time - last_update_time < 1.0) {
+		/* if last sample was processed, we need to write buffers anyway  */
+		if (!write)
+			return false;
+	}
+
+	if(params.progressive_refine) {
+		foreach(RenderBuffers *buffers, tile_buffers) {
+			RenderTile rtile;
+			rtile.buffers = buffers;
+			rtile.sample = sample;
+
+			if(write)
+				write_render_tile_cb(rtile);
+			else
+				update_render_tile_cb(rtile);
+		}
+	}
+
+	last_update_time = current_time;
+
+	return write;
+}
+
+void Session::device_free()
+{
+	scene->device_free();
+
+	foreach(RenderBuffers *buffers, tile_buffers)
+		delete buffers;
+
+	tile_buffers.clear();
+
+	/* used from background render only, so no need to
+	 * re-create render/display buffers here
+	 */
+}
+
+CCL_NAMESPACE_END
