@@ -107,9 +107,6 @@ public:
 		if(datasize > 0) {
 			data = (T*)malloc_aligned(sizeof(T)*datasize, alignment);
 			memcpy(data, &from[0], datasize*sizeof(T));
-			free_aligned(data);
-			data = (T*)malloc_aligned(sizeof(T)*datasize, alignment);
-			memcpy(data, &from[0], datasize*sizeof(T));
 		}
 
 		return *this;
@@ -127,8 +124,10 @@ public:
 		}
 		else if(newsize != datasize) {
 			T *newdata = (T*)malloc_aligned(sizeof(T)*newsize, alignment);
-			memcpy(newdata, data, ((datasize < newsize)? datasize: newsize)*sizeof(T));
-			free_aligned(data);
+			if(data) {
+				memcpy(newdata, data, ((datasize < newsize)? datasize: newsize)*sizeof(T));
+				free_aligned(data);
+			}
 
 			data = newdata;
 			datasize = newsize;
