@@ -123,7 +123,14 @@ void GHOST_GetMainDisplayDimensions(GHOST_SystemHandle systemhandle,
 	system->getMainDisplayDimensions(*width, *height);
 }
 
+void GHOST_GetAllDisplayDimensions(GHOST_SystemHandle systemhandle,
+                                    GHOST_TUns32 *width,
+                                    GHOST_TUns32 *height)
+{
+	GHOST_ISystem *system = (GHOST_ISystem *) systemhandle;
 
+	system->getAllDisplayDimensions(*width, *height);
+}
 
 GHOST_WindowHandle GHOST_CreateWindow(GHOST_SystemHandle systemhandle,
                                       const char *title,
@@ -137,15 +144,10 @@ GHOST_WindowHandle GHOST_CreateWindow(GHOST_SystemHandle systemhandle,
                                       const GHOST_TUns16 numOfAASamples)
 {
 	GHOST_ISystem *system = (GHOST_ISystem *) systemhandle;
-	bool bstereoVisual;
-
-	if (stereoVisual)
-		bstereoVisual = true;
-	else
-		bstereoVisual = false;
 
 	return (GHOST_WindowHandle) system->createWindow(title, left, top, width, height,
-	                                                 state, type, bstereoVisual, numOfAASamples);
+	                                                 state, type, stereoVisual != 0, false,
+	                                                 numOfAASamples);
 }
 
 GHOST_TUserDataPtr GHOST_GetWindowUserData(GHOST_WindowHandle windowhandle)
@@ -355,7 +357,7 @@ GHOST_TSuccess GHOST_SetCursorPosition(GHOST_SystemHandle systemhandle,
 
 GHOST_TSuccess GHOST_SetCursorGrab(GHOST_WindowHandle windowhandle,
                                    GHOST_TGrabCursorMode mode,
-                                   int bounds[4], int mouse_ungrab_xy[2])
+                                   int bounds[4], const int mouse_ungrab_xy[2])
 {
 	GHOST_IWindow *window = (GHOST_IWindow *) windowhandle;
 	GHOST_Rect bounds_rect, bounds_win;
@@ -683,7 +685,27 @@ GHOST_TSuccess GHOST_SwapWindowBuffers(GHOST_WindowHandle windowhandle)
 	return window->swapBuffers();
 }
 
+GHOST_TSuccess GHOST_SetSwapInterval(GHOST_WindowHandle windowhandle, int interval)
+{
+	GHOST_IWindow *window = (GHOST_IWindow *) windowhandle;
 
+	return window->setSwapInterval(interval);
+}
+
+GHOST_TSuccess GHOST_GetSwapInterval(GHOST_WindowHandle windowhandle, int* intervalOut)
+{
+	GHOST_IWindow *window = (GHOST_IWindow *) windowhandle;
+
+	return window->getSwapInterval(*intervalOut);
+}
+
+
+GHOST_TUns16 GHOST_GetNumOfAASamples(GHOST_WindowHandle windowhandle)
+{
+	GHOST_IWindow *window = (GHOST_IWindow *) windowhandle;
+
+	return window->getNumOfAASamples();
+}
 
 GHOST_TSuccess GHOST_ActivateWindowDrawingContext(GHOST_WindowHandle windowhandle)
 {
@@ -878,3 +900,36 @@ int GHOST_confirmQuit(GHOST_WindowHandle windowhandle)
 	GHOST_ISystem *system = GHOST_ISystem::getSystem();
 	return system->confirmQuit((GHOST_IWindow *) windowhandle);
 }
+
+int GHOST_UseNativePixels(void)
+{
+	GHOST_ISystem *system = GHOST_ISystem::getSystem();
+	return system->useNativePixel();
+}
+
+float GHOST_GetNativePixelSize(GHOST_WindowHandle windowhandle)
+{
+	GHOST_IWindow *window = (GHOST_IWindow *) windowhandle;
+	if (window)
+		return window->getNativePixelSize();
+	return 1.0f;
+}
+
+#ifdef WITH_INPUT_IME
+
+void GHOST_BeginIME(GHOST_WindowHandle windowhandle,
+                    GHOST_TInt32 x, GHOST_TInt32 y,
+                    GHOST_TInt32 w, GHOST_TInt32 h,
+                    int complete)
+{
+	GHOST_IWindow *window = (GHOST_IWindow *) windowhandle;
+	window->beginIME(x, y, w, h, complete);
+}
+
+void GHOST_EndIME(GHOST_WindowHandle windowhandle)
+{
+	GHOST_IWindow *window = (GHOST_IWindow *) windowhandle;
+	window->endIME();
+}
+
+#endif  /* WITH_INPUT_IME */
