@@ -46,14 +46,8 @@
 
 #include <shlobj.h>
 #include <tlhelp32.h>
-
-// win64 doesn't define GWL_USERDATA
-#ifdef WIN32
-#  ifndef GWL_USERDATA
-#    define GWL_USERDATA GWLP_USERDATA
-#    define GWL_WNDPROC GWLP_WNDPROC
-#  endif
-#endif
+#include <Psapi.h>
+#include <windowsx.h>
 
 #include "utfconv.h"
 
@@ -122,6 +116,17 @@
 #define VK_MEDIA_PLAY_PAUSE 0xB3
 #endif // VK_MEDIA_PLAY_PAUSE
 
+/* Workaround for some laptop touchpads, some of which seems to
+ * have driver issues which makes it so window function receives
+ * the message, but PeekMessage doesn't pick those messages for
+ * some reason.
+ *
+ * We send a dummy WM_USER message to force PeekMessage to receive
+ * something, making it so blender's window manager sees the new
+ * messages coming in.
+ */
+#define BROKEN_PEEK_TOUCHPAD
+
 static void initRawInput()
 {
 #ifdef WITH_INPUT_NDOF
@@ -209,52 +214,53 @@ GHOST_TUns8 GHOST_SystemWin32::getNumDisplays() const
 }
 
 
-void GHOST_SystemWin32::getMainDisplayDimensions(GHOST_TUns32& width, GHOST_TUns32& height) const
+void GHOST_SystemWin32::getMainDisplayDimensions(GHOST_TUns32 &width, GHOST_TUns32 &height) const
 {
 	width = ::GetSystemMetrics(SM_CXSCREEN);
 	height = ::GetSystemMetrics(SM_CYSCREEN);
 }
 
+void GHOST_SystemWin32::getAllDisplayDimensions(GHOST_TUns32 &width, GHOST_TUns32 &height) const
+{
+	width = ::GetSystemMetrics(SM_CXVIRTUALSCREEN);
+	height = ::GetSystemMetrics(SM_CYVIRTUALSCREEN);
+}
 
 GHOST_IWindow *GHOST_SystemWin32::createWindow(
-		const STR_String& title,
-		GHOST_TInt32 left, GHOST_TInt32 top, GHOST_TUns32 width, GHOST_TUns32 height,
-		GHOST_TWindowState state, GHOST_TDrawingContextType type,
-		bool stereoVisual, const GHOST_TUns16 numOfAASamples, const GHOST_TEmbedderWindowID parentWindow)
+        const STR_String &title,
+        GHOST_TInt32 left, GHOST_TInt32 top,
+        GHOST_TUns32 width, GHOST_TUns32 height,
+        GHOST_TWindowState state, GHOST_TDrawingContextType type,
+        bool wantStereoVisual,
+        const bool exclusive,
+        const GHOST_TUns16 wantNumOfAASamples,
+        const GHOST_TEmbedderWindowID parentWindow)
 {
-	GHOST_Window *window = 0;
-	window = new GHOST_WindowWin32(this, title, left, top, width, height, state, type, stereoVisual, numOfAASamples, parentWindow);
-	if (window) {
-		if (window->getValid()) {
-			// Store the pointer to the window
-//			if (state != GHOST_kWindowStateFullScreen) {
-			m_windowManager->addWindow(window);
-			m_windowManager->setActiveWindow(window);
-//			}
-		}
-		else {
+	GHOST_Window *window =
+		new GHOST_WindowWin32(
+		        this,
+		        title,
+		        left,
+		        top,
+		        width,
+		        height,
+		        state,
+		        type,
+		        wantStereoVisual,
+		        wantNumOfAASamples,
+		        parentWindow);
 
-			// Invalid parent window hwnd
-			if (((GHOST_WindowWin32 *)window)->getNextWindow() == NULL) {
-				delete window;
-				window = 0;
-				return window;
-			}
-
-			// An invalid window could be one that was used to test for AA
-			window = ((GHOST_WindowWin32 *)window)->getNextWindow();
-			
-			// If another window is found, let the wm know about that one, but not the old one
-			if (window->getValid()) {
-				m_windowManager->addWindow(window);
-			}
-			else {
-				delete window;
-				window = 0;
-			}
-
-		}
+	if (window->getValid()) {
+		// Store the pointer to the window
+		m_windowManager->addWindow(window);
+		m_windowManager->setActiveWindow(window);
 	}
+	else {
+		GHOST_PRINT("GHOST_SystemWin32::createWindow(): window invalid\n");
+		delete window;
+		window = 0;
+	}
+
 	return window;
 }
 
@@ -291,6 +297,9 @@ bool GHOST_SystemWin32::processEvents(bool waitForEvent)
 
 		// Process all the events waiting for us
 		while (::PeekMessageW(&msg, 0, 0, 0, PM_REMOVE) != 0) {
+			// TranslateMessage doesn't alter the message, and doesn't change our raw keyboard data.
+			// Needed for MapVirtualKey or if we ever need to get chars from wm_ime_char or similar.
+			::TranslateMessage(&msg);
 			::DispatchMessageW(&msg);
 			anyProcessed = true;
 		}
@@ -300,7 +309,7 @@ bool GHOST_SystemWin32::processEvents(bool waitForEvent)
 }
 
 
-GHOST_TSuccess GHOST_SystemWin32::getCursorPosition(GHOST_TInt32& x, GHOST_TInt32& y) const
+GHOST_TSuccess GHOST_SystemWin32::getCursorPosition(GHOST_TInt32 &x, GHOST_TInt32 &y) const
 {
 	POINT point;
 	if (::GetCursorPos(&point)) {
@@ -314,13 +323,13 @@ GHOST_TSuccess GHOST_SystemWin32::getCursorPosition(GHOST_TInt32& x, GHOST_TInt3
 
 GHOST_TSuccess GHOST_SystemWin32::setCursorPosition(GHOST_TInt32 x, GHOST_TInt32 y)
 {
-	if (!GetActiveWindow())
+	if (!::GetActiveWindow())
 		return GHOST_kFailure;
 	return ::SetCursorPos(x, y) == TRUE ? GHOST_kSuccess : GHOST_kFailure;
 }
 
 
-GHOST_TSuccess GHOST_SystemWin32::getModifierKeys(GHOST_ModifierKeys& keys) const
+GHOST_TSuccess GHOST_SystemWin32::getModifierKeys(GHOST_ModifierKeys &keys) const
 {
 	bool down = HIBYTE(::GetKeyState(VK_LSHIFT)) != 0;
 	keys.set(GHOST_kModifierKeyLeftShift, down);
@@ -347,7 +356,7 @@ GHOST_TSuccess GHOST_SystemWin32::getModifierKeys(GHOST_ModifierKeys& keys) cons
 }
 
 
-GHOST_TSuccess GHOST_SystemWin32::getButtons(GHOST_Buttons& buttons) const
+GHOST_TSuccess GHOST_SystemWin32::getButtons(GHOST_Buttons &buttons) const
 {
 	/* Check for swapped buttons (left-handed mouse buttons)
 	 * GetAsyncKeyState() will give back the state of the physical mouse buttons.
@@ -392,7 +401,7 @@ GHOST_TSuccess GHOST_SystemWin32::init()
 	}
 
 	if (success) {
-		WNDCLASSW wc;
+		WNDCLASSW wc = {0};
 		wc.style = CS_HREDRAW | CS_VREDRAW;
 		wc.lpfnWndProc = s_wndProc;
 		wc.cbClsExtra = 0;
@@ -404,7 +413,7 @@ GHOST_TSuccess GHOST_SystemWin32::init()
 			::LoadIcon(NULL, IDI_APPLICATION);
 		}
 		wc.hCursor = ::LoadCursor(0, IDC_ARROW);
-		wc.hbrBackground = (HBRUSH) ::GetStockObject(BLACK_BRUSH);
+		wc.hbrBackground = 0;
 		wc.lpszMenuName = 0;
 		wc.lpszClassName = L"GHOST_WindowClass";
 
@@ -423,17 +432,10 @@ GHOST_TSuccess GHOST_SystemWin32::exit()
 	return GHOST_System::exit();
 }
 
-GHOST_TKey GHOST_SystemWin32::hardKey(GHOST_IWindow *window, RAWINPUT const& raw, int *keyDown, char *vk)
+GHOST_TKey GHOST_SystemWin32::hardKey(GHOST_IWindow *window, RAWINPUT const &raw, int *keyDown, char *vk)
 {
-	GHOST_TKey key = GHOST_kKeyUnknown;
-
-
-	if (!keyDown)
-		return GHOST_kKeyUnknown;
-
-
 	GHOST_SystemWin32 *system = (GHOST_SystemWin32 *)getSystem();
-
+	GHOST_TKey key = GHOST_kKeyUnknown;
 	GHOST_ModifierKeys modifiers;
 	system->retrieveModifierKeys(modifiers);
 
@@ -445,8 +447,7 @@ GHOST_TKey GHOST_SystemWin32::hardKey(GHOST_IWindow *window, RAWINPUT const& raw
 	key = this->convertKey(window, raw.data.keyboard.VKey, raw.data.keyboard.MakeCode, (raw.data.keyboard.Flags & (RI_KEY_E1 | RI_KEY_E0)));
 	
 	// extra handling of modifier keys: don't send repeats out from GHOST
-	if (key >= GHOST_kKeyLeftShift && key <= GHOST_kKeyRightAlt)
-	{
+	if (key >= GHOST_kKeyLeftShift && key <= GHOST_kKeyRightAlt) {
 		bool changed = false;
 		GHOST_TModifierKeyMask modifier;
 		switch (key) {
@@ -454,43 +455,43 @@ GHOST_TKey GHOST_SystemWin32::hardKey(GHOST_IWindow *window, RAWINPUT const& raw
 			{
 				changed = (modifiers.get(GHOST_kModifierKeyLeftShift) != (bool)*keyDown);
 				modifier = GHOST_kModifierKeyLeftShift;
+				break;
 			}
-			break;
 			case GHOST_kKeyRightShift:
 			{
 				changed = (modifiers.get(GHOST_kModifierKeyRightShift) != (bool)*keyDown);
 				modifier = GHOST_kModifierKeyRightShift;
+				break;
 			}
-			break;
 			case GHOST_kKeyLeftControl:
 			{
 				changed = (modifiers.get(GHOST_kModifierKeyLeftControl) != (bool)*keyDown);
 				modifier = GHOST_kModifierKeyLeftControl;
+				break;
 			}
-			break;
 			case GHOST_kKeyRightControl:
 			{
 				changed = (modifiers.get(GHOST_kModifierKeyRightControl) != (bool)*keyDown);
 				modifier = GHOST_kModifierKeyRightControl;
+				break;
 			}
-			break;
 			case GHOST_kKeyLeftAlt:
 			{
 				changed = (modifiers.get(GHOST_kModifierKeyLeftAlt) != (bool)*keyDown);
 				modifier = GHOST_kModifierKeyLeftAlt;
+				break;
 			}
-			break;
 			case GHOST_kKeyRightAlt:
 			{
 				changed = (modifiers.get(GHOST_kModifierKeyRightAlt) != (bool)*keyDown);
 				modifier = GHOST_kModifierKeyRightAlt;
+				break;
 			}
-			break;
-			default: break;
+			default:
+				break;
 		}
 		
-		if (changed)
-		{
+		if (changed) {
 			modifiers.set(modifier, (bool)*keyDown);
 			system->storeModifierKeys(modifiers);
 		}
@@ -507,6 +508,7 @@ GHOST_TKey GHOST_SystemWin32::hardKey(GHOST_IWindow *window, RAWINPUT const& raw
 
 //! note: this function can be extended to include other exotic cases as they arise.
 // This function was added in response to bug [#25715]
+// This is going to be a long list [T42426]
 GHOST_TKey GHOST_SystemWin32::processSpecialKey(GHOST_IWindow *window, short vKey, short scanCode) const
 {
 	GHOST_TKey key = GHOST_kKeyUnknown;
@@ -514,6 +516,10 @@ GHOST_TKey GHOST_SystemWin32::processSpecialKey(GHOST_IWindow *window, short vKe
 		case LANG_FRENCH:
 			if (vKey == VK_OEM_8) key = GHOST_kKeyF13;  // oem key; used purely for shortcuts .
 			break;
+		case LANG_ENGLISH:
+			if (SUBLANGID(m_langId) == SUBLANG_ENGLISH_UK && vKey == VK_OEM_8) // "`¬"
+				key = GHOST_kKeyAccentGrave;
+			break;
 	}
 
 	return key;
@@ -599,7 +605,19 @@ GHOST_TKey GHOST_SystemWin32::convertKey(GHOST_IWindow *window, short vKey, shor
 			case VK_GR_LESS:        key = GHOST_kKeyGrLess;         break;
 
 			case VK_SHIFT:
-				key = (scanCode == 0x36) ? GHOST_kKeyRightShift : GHOST_kKeyLeftShift;
+				/* Check single shift presses */
+				if (scanCode == 0x36) {
+					key = GHOST_kKeyRightShift;
+				}
+				else if (scanCode == 0x2a) {
+					key = GHOST_kKeyLeftShift;
+				}
+				else {
+					/* Must be a combination SHIFT (Left or Right) + a Key
+					 * Ignore this as the next message will contain
+					 * the desired "Key" */
+					key = GHOST_kKeyUnknown;
+				}
 				break;
 			case VK_CONTROL:
 				key = (extend) ? GHOST_kKeyRightControl : GHOST_kKeyLeftControl;
@@ -630,7 +648,10 @@ GHOST_TKey GHOST_SystemWin32::convertKey(GHOST_IWindow *window, short vKey, shor
 	return key;
 }
 
-GHOST_EventButton *GHOST_SystemWin32::processButtonEvent(GHOST_TEventType type, GHOST_IWindow *window, GHOST_TButtonMask mask)
+GHOST_EventButton *GHOST_SystemWin32::processButtonEvent(
+        GHOST_TEventType type,
+        GHOST_IWindow *window,
+        GHOST_TButtonMask mask)
 {
 	return new GHOST_EventButton(getSystem()->getMilliSeconds(), type, window, mask);
 }
@@ -707,7 +728,7 @@ GHOST_EventWheel *GHOST_SystemWin32::processWheelEvent(GHOST_IWindow *window, WP
 }
 
 
-GHOST_EventKey *GHOST_SystemWin32::processKeyEvent(GHOST_IWindow *window, RAWINPUT const& raw)
+GHOST_EventKey *GHOST_SystemWin32::processKeyEvent(GHOST_IWindow *window, RAWINPUT const &raw)
 {
 	int keyDown = 0;
 	char vk;
@@ -724,13 +745,17 @@ GHOST_EventKey *GHOST_SystemWin32::processKeyEvent(GHOST_IWindow *window, RAWINP
 		int r;
 		GetKeyboardState((PBYTE)state);
 
-		if ((r = ToUnicodeEx(vk, 0, state, utf16, 2, 0, system->m_keylayout))) {
-			if ((r > 0 && r < 3)) {
-				utf16[r] = 0;
-				conv_utf_16_to_8(utf16, utf8_char, 6);
-			}
-			else if (r == -1) {
-				utf8_char[0] = '\0';
+		// don't call ToUnicodeEx on dead keys as it clears the buffer and so won't allow diacritical composition.
+		if (MapVirtualKeyW(vk, 2) != 0) {
+			// todo: ToUnicodeEx can respond with up to 4 utf16 chars (only 2 here). Could be up to 24 utf8 bytes.
+			if ((r = ToUnicodeEx(vk, raw.data.keyboard.MakeCode, state, utf16, 2, 0, system->m_keylayout))) {
+				if ((r > 0 && r < 3)) {
+					utf16[r] = 0;
+					conv_utf_16_to_8(utf16, utf8_char, 6);
+				}
+				else if (r == -1) {
+					utf8_char[0] = '\0';
+				}
 			}
 		}
 
@@ -742,8 +767,6 @@ GHOST_EventKey *GHOST_SystemWin32::processKeyEvent(GHOST_IWindow *window, RAWINP
 			ascii = utf8_char[0] & 0x80 ? '?' : utf8_char[0];
 		}
 
-		if (0x80 & state[VK_MENU]) utf8_char[0] = '\0';
-
 		event = new GHOST_EventKey(system->getMilliSeconds(), keyDown ? GHOST_kEventKeyDown : GHOST_kEventKeyUp, window, key, ascii, utf8_char);
 		
 #ifdef GHOST_DEBUG
@@ -763,16 +786,27 @@ GHOST_Event *GHOST_SystemWin32::processWindowEvent(GHOST_TEventType type, GHOST_
 
 	if (type == GHOST_kEventWindowActivate) {
 		system->getWindowManager()->setActiveWindow(window);
+		((GHOST_WindowWin32 *)window)->bringTabletContextToFront();
 	}
 
 	return new GHOST_Event(system->getMilliSeconds(), type, window);
 }
 
-GHOST_TSuccess GHOST_SystemWin32::pushDragDropEvent(GHOST_TEventType eventType, 
-		GHOST_TDragnDropTypes draggedObjectType,
-		GHOST_IWindow *window,
-		int mouseX, int mouseY,
-		void *data)
+#ifdef WITH_INPUT_IME
+GHOST_Event *GHOST_SystemWin32::processImeEvent(GHOST_TEventType type, GHOST_IWindow *window, GHOST_TEventImeData *data)
+{
+	GHOST_System *system = (GHOST_System *)getSystem();
+	return new GHOST_EventIME(system->getMilliSeconds(), type, window, data);
+}
+#endif
+
+
+GHOST_TSuccess GHOST_SystemWin32::pushDragDropEvent(
+        GHOST_TEventType eventType,
+        GHOST_TDragnDropTypes draggedObjectType,
+        GHOST_IWindow *window,
+        int mouseX, int mouseY,
+        void *data)
 {
 	GHOST_SystemWin32 *system = ((GHOST_SystemWin32 *)getSystem());
 	return system->pushEvent(new GHOST_EventDragnDrop(system->getMilliSeconds(),
@@ -789,7 +823,7 @@ void GHOST_SystemWin32::processMinMaxInfo(MINMAXINFO *minmax)
 }
 
 #ifdef WITH_INPUT_NDOF
-bool GHOST_SystemWin32::processNDOF(RAWINPUT const& raw)
+bool GHOST_SystemWin32::processNDOF(RAWINPUT const &raw)
 {
 	bool eventSent = false;
 	GHOST_TUns64 now = getMilliSeconds();
@@ -835,18 +869,17 @@ bool GHOST_SystemWin32::processNDOF(RAWINPUT const& raw)
 #endif
 
 	BYTE packetType = data[0];
-	switch (packetType)
-	{
+	switch (packetType) {
 		case 1: // translation
 		{
-			short *axis = (short *)(data + 1);
+			const short *axis = (short *)(data + 1);
 			// massage into blender view coords (same goes for rotation)
-			short t[3] = {axis[0], -axis[2], axis[1]};
+			const short t[3] = {axis[0], -axis[2], axis[1]};
 			m_ndofManager->updateTranslation(t, now);
 
-			if (raw.data.hid.dwSizeHid == 13)
-			{ // this report also includes rotation
-				short r[3] = {-axis[3], axis[5], -axis[4]};
+			if (raw.data.hid.dwSizeHid == 13) {
+				// this report also includes rotation
+				const short r[3] = {-axis[3], axis[5], -axis[4]};
 				m_ndofManager->updateRotation(r, now);
 
 				// I've never gotten one of these, has anyone else?
@@ -856,8 +889,8 @@ bool GHOST_SystemWin32::processNDOF(RAWINPUT const& raw)
 		}
 		case 2: // rotation
 		{
-			short *axis = (short *)(data + 1);
-			short r[3] = {-axis[0], axis[2], -axis[1]};
+			const short *axis = (short *)(data + 1);
+			const short r[3] = {-axis[0], axis[2], -axis[1]};
 			m_ndofManager->updateRotation(r, now);
 			break;
 		}
@@ -880,16 +913,22 @@ LRESULT WINAPI GHOST_SystemWin32::s_wndProc(HWND hwnd, UINT msg, WPARAM wParam,
 
 	LRESULT lResult = 0;
 	GHOST_SystemWin32 *system = ((GHOST_SystemWin32 *)getSystem());
+	GHOST_EventManager *eventManager = system->getEventManager();
 	GHOST_ASSERT(system, "GHOST_SystemWin32::s_wndProc(): system not initialized");
 
 	if (hwnd) {
-		GHOST_WindowWin32 *window = (GHOST_WindowWin32 *)::GetWindowLong(hwnd, GWL_USERDATA);
+		GHOST_WindowWin32 *window = (GHOST_WindowWin32 *)::GetWindowLongPtr(hwnd, GWLP_USERDATA);
 		if (window) {
 			switch (msg) {
 				// we need to check if new key layout has AltGr
 				case WM_INPUTLANGCHANGE:
+				{
 					system->handleKeyboardChange();
+#ifdef WITH_INPUT_IME
+					window->getImeInput()->SetInputLanguage();
+#endif
 					break;
+				}
 				////////////////////////////////////////////////////////////////////////
 				// Keyboard events, processed
 				////////////////////////////////////////////////////////////////////////
@@ -907,8 +946,7 @@ LRESULT WINAPI GHOST_SystemWin32::s_wndProc(HWND hwnd, UINT msg, WPARAM wParam,
 
 					GetRawInputData((HRAWINPUT)lParam, RID_INPUT, raw_ptr, &rawSize, sizeof(RAWINPUTHEADER));
 
-					switch (raw.header.dwType)
-					{
+					switch (raw.header.dwType) {
 						case RIM_TYPEKEYBOARD:
 							event = processKeyEvent(window, raw);
 							if (!event) {
@@ -926,6 +964,56 @@ LRESULT WINAPI GHOST_SystemWin32::s_wndProc(HWND hwnd, UINT msg, WPARAM wParam,
 					}
 					break;
 				}
+#ifdef WITH_INPUT_IME
+				////////////////////////////////////////////////////////////////////////
+				// IME events, processed, read more in GHOST_IME.h
+				////////////////////////////////////////////////////////////////////////
+				case WM_IME_SETCONTEXT:
+				{
+					window->getImeInput()->SetInputLanguage();
+					window->getImeInput()->CreateImeWindow(window->getHWND());
+					window->getImeInput()->CleanupComposition(window->getHWND());
+					window->getImeInput()->CheckFirst(window->getHWND());
+					break;
+				}
+				case WM_IME_STARTCOMPOSITION:
+				{
+					eventHandled = true;
+					/* remove input event before start comp event, avoid redundant input */
+					eventManager->removeTypeEvents(GHOST_kEventKeyDown, window);
+					window->getImeInput()->CreateImeWindow(window->getHWND());
+					window->getImeInput()->ResetComposition(window->getHWND());
+					event = processImeEvent(
+					        GHOST_kEventImeCompositionStart,
+					        window,
+					        &window->getImeInput()->eventImeData);
+					break;
+				}
+				case WM_IME_COMPOSITION:
+				{
+					eventHandled = true;
+					window->getImeInput()->UpdateImeWindow(window->getHWND());
+					window->getImeInput()->UpdateInfo(window->getHWND());
+					event = processImeEvent(
+					        GHOST_kEventImeComposition,
+					        window,
+					        &window->getImeInput()->eventImeData);
+					break;
+				}
+				case WM_IME_ENDCOMPOSITION:
+				{
+					eventHandled = true;
+					/* remove input event after end comp event, avoid redundant input */
+					eventManager->removeTypeEvents(GHOST_kEventKeyDown, window);
+					window->getImeInput()->ResetComposition(window->getHWND());
+					window->getImeInput()->DestroyImeWindow(window->getHWND());
+					event = processImeEvent(
+					        GHOST_kEventImeCompositionEnd,
+					        window,
+					        &window->getImeInput()->eventImeData);
+					break;
+				}
+#endif /* WITH_INPUT_IME */
 				////////////////////////////////////////////////////////////////////////
 				// Keyboard events, ignored
 				////////////////////////////////////////////////////////////////////////
@@ -968,7 +1056,9 @@ LRESULT WINAPI GHOST_SystemWin32::s_wndProc(HWND hwnd, UINT msg, WPARAM wParam,
 					 * maximize, minimize  or close the window are triggered. Also it is sent when ALT 
 					 * button is press for menu. To prevent this we must return preventing DefWindowProc.
 					 */
-					if (wParam == SC_KEYMENU) return 0;
+					if (wParam == SC_KEYMENU) {
+						eventHandled = true;
+					}
 					break;
 				////////////////////////////////////////////////////////////////////////
 				// Tablet events, processed
@@ -1029,6 +1119,7 @@ LRESULT WINAPI GHOST_SystemWin32::s_wndProc(HWND hwnd, UINT msg, WPARAM wParam,
 					event = processCursorEvent(GHOST_kEventCursorMove, window);
 					break;
 				case WM_MOUSEWHEEL:
+				{
 					/* The WM_MOUSEWHEEL message is sent to the focus window 
 					 * when the mouse wheel is rotated. The DefWindowProc 
 					 * function propagates the message to the window's parent.
@@ -1036,8 +1127,24 @@ LRESULT WINAPI GHOST_SystemWin32::s_wndProc(HWND hwnd, UINT msg, WPARAM wParam,
 					 * since DefWindowProc propagates it up the parent chain 
 					 * until it finds a window that processes it.
 					 */
-					event = processWheelEvent(window, wParam, lParam);
+
+					/* Get the winow under the mouse and send event to it's queue. */
+					POINT mouse_pos = {GET_X_LPARAM(lParam), GET_Y_LPARAM(lParam)};
+					HWND mouse_hwnd = ChildWindowFromPoint(HWND_DESKTOP, mouse_pos);
+					GHOST_WindowWin32 *mouse_window = (GHOST_WindowWin32 *)::GetWindowLongPtr(mouse_hwnd, GWLP_USERDATA);
+					if (mouse_window != NULL) {
+						event = processWheelEvent(mouse_window, wParam, lParam);
+					}
+					else {
+						/* Happens when wmouse is not over of any of blender windows. */
+						event = processWheelEvent(window, wParam, lParam);
+					}
+
+#ifdef BROKEN_PEEK_TOUCHPAD
+					PostMessage(hwnd, WM_USER, 0, 0);
+#endif
 					break;
+				}
 				case WM_SETCURSOR:
 					/* The WM_SETCURSOR message is sent to a window if the mouse causes the cursor
 					 * to move within a window and mouse input is not captured.
@@ -1094,9 +1201,25 @@ LRESULT WINAPI GHOST_SystemWin32::s_wndProc(HWND hwnd, UINT msg, WPARAM wParam,
 					event = processWindowEvent(LOWORD(wParam) ? GHOST_kEventWindowActivate : GHOST_kEventWindowDeactivate, window);
 					/* WARNING: Let DefWindowProc handle WM_ACTIVATE, otherwise WM_MOUSEWHEEL
 					 * will not be dispatched to OUR active window if we minimize one of OUR windows. */
+					if (LOWORD(wParam) == WA_INACTIVE)
+						window->lostMouseCapture();
+
 					lResult = ::DefWindowProc(hwnd, msg, wParam, lParam);
 					break;
 				}
+				case WM_ENTERSIZEMOVE:
+					/* The WM_ENTERSIZEMOVE message is sent one time to a window after it enters the moving 
+					 * or sizing modal loop. The window enters the moving or sizing modal loop when the user 
+					 * clicks the window's title bar or sizing border, or when the window passes the 
+					 * WM_SYSCOMMAND message to the DefWindowProc function and the wParam parameter of the 
+					 * message specifies the SC_MOVE or SC_SIZE value. The operation is complete when 
+					 * DefWindowProc returns. 
+					 */
+					window->m_inLiveResize = 1;
+					break;
+				case WM_EXITSIZEMOVE:
+					window->m_inLiveResize = 0;
+					break;
 				case WM_PAINT:
 					/* An application sends the WM_PAINT message when the system or another application 
 					 * makes a request to paint a portion of an application's window. The message is sent
@@ -1104,8 +1227,13 @@ LRESULT WINAPI GHOST_SystemWin32::s_wndProc(HWND hwnd, UINT msg, WPARAM wParam,
 					 * function when the application obtains a WM_PAINT message by using the GetMessage or 
 					 * PeekMessage function. 
 					 */
-					event = processWindowEvent(GHOST_kEventWindowUpdate, window);
-					::ValidateRect(hwnd, NULL);
+					if (!window->m_inLiveResize) {
+						event = processWindowEvent(GHOST_kEventWindowUpdate, window);
+						::ValidateRect(hwnd, NULL);
+					}
+					else {
+						eventHandled = true;
+					}
 					break;
 				case WM_GETMINMAXINFO:
 					/* The WM_GETMINMAXINFO message is sent to a window when the size or 
@@ -1116,6 +1244,7 @@ LRESULT WINAPI GHOST_SystemWin32::s_wndProc(HWND hwnd, UINT msg, WPARAM wParam,
 					processMinMaxInfo((MINMAXINFO *) lParam);
 					/* Let DefWindowProc handle it. */
 					break;
+				case WM_SIZING:
 				case WM_SIZE:
 					/* The WM_SIZE message is sent to a window after its size has changed.
 					 * The WM_SIZE and WM_MOVE messages are not sent if an application handles the 
@@ -1123,23 +1252,38 @@ LRESULT WINAPI GHOST_SystemWin32::s_wndProc(HWND hwnd, UINT msg, WPARAM wParam,
 					 * to perform any move or size change processing during the WM_WINDOWPOSCHANGED 
 					 * message without calling DefWindowProc.
 					 */
-					event = processWindowEvent(GHOST_kEventWindowSize, window);
+					/* we get first WM_SIZE before we fully init. So, do not dispatch before we continiously resizng */
+					if (window->m_inLiveResize) {
+						system->pushEvent(processWindowEvent(GHOST_kEventWindowSize, window));
+						system->dispatchEvents();
+					}
+					else {
+						event = processWindowEvent(GHOST_kEventWindowSize, window);
+					}
 					break;
 				case WM_CAPTURECHANGED:
 					window->lostMouseCapture();
 					break;
 				case WM_MOVING:
-				/* The WM_MOVING message is sent to a window that the user is moving. By processing
-				 * this message, an application can monitor the size and position of the drag rectangle
-				 * and, if needed, change its size or position.
-				 */
+					/* The WM_MOVING message is sent to a window that the user is moving. By processing
+					 * this message, an application can monitor the size and position of the drag rectangle
+					 * and, if needed, change its size or position.
+					 */
 				case WM_MOVE:
 					/* The WM_SIZE and WM_MOVE messages are not sent if an application handles the 
 					 * WM_WINDOWPOSCHANGED message without calling DefWindowProc. It is more efficient
 					 * to perform any move or size change processing during the WM_WINDOWPOSCHANGED 
 					 * message without calling DefWindowProc. 
 					 */
-					event = processWindowEvent(GHOST_kEventWindowMove, window);
+					/* see WM_SIZE comment*/
+					if (window->m_inLiveResize) {
+						system->pushEvent(processWindowEvent(GHOST_kEventWindowMove, window));
+						system->dispatchEvents();
+					}
+					else {
+						event = processWindowEvent(GHOST_kEventWindowMove, window);
+					}
+
 					break;
 				////////////////////////////////////////////////////////////////////////
 				// Window events, ignored
@@ -1180,8 +1324,8 @@ LRESULT WINAPI GHOST_SystemWin32::s_wndProc(HWND hwnd, UINT msg, WPARAM wParam,
 				case WM_KILLFOCUS:
 					/* The WM_KILLFOCUS message is sent to a window immediately before it loses the keyboard focus. 
 					 * We want to prevent this if a window is still active and it loses focus to nowhere*/
-					if (!wParam && hwnd == GetActiveWindow())
-						SetFocus(hwnd);
+					if (!wParam && hwnd == ::GetActiveWindow())
+						::SetFocus(hwnd);
 				case WM_SHOWWINDOW:
 				/* The WM_SHOWWINDOW message is sent to a window when the window is about to be hidden or shown. */
 				case WM_WINDOWPOSCHANGING:
@@ -1191,14 +1335,6 @@ LRESULT WINAPI GHOST_SystemWin32::s_wndProc(HWND hwnd, UINT msg, WPARAM wParam,
 				 */
 				case WM_SETFOCUS:
 				/* The WM_SETFOCUS message is sent to a window after it has gained the keyboard focus. */
-				case WM_ENTERSIZEMOVE:
-					/* The WM_ENTERSIZEMOVE message is sent one time to a window after it enters the moving 
-					 * or sizing modal loop. The window enters the moving or sizing modal loop when the user 
-					 * clicks the window's title bar or sizing border, or when the window passes the 
-					 * WM_SYSCOMMAND message to the DefWindowProc function and the wParam parameter of the 
-					 * message specifies the SC_MOVE or SC_SIZE value. The operation is complete when 
-					 * DefWindowProc returns. 
-					 */
 					break;
 				////////////////////////////////////////////////////////////////////////
 				// Other events
@@ -1224,6 +1360,7 @@ LRESULT WINAPI GHOST_SystemWin32::s_wndProc(HWND hwnd, UINT msg, WPARAM wParam,
 					 * In GHOST, we let DefWindowProc call the timer callback.
 					 */
 					break;
+
 			}
 		}
 		else {
@@ -1338,54 +1475,94 @@ void GHOST_SystemWin32::putClipboard(GHOST_TInt8 *buffer, bool selection) const
 	}
 }
 
+static DWORD GetParentProcessID(void)
+{
+	HANDLE snapshot;
+	PROCESSENTRY32 pe32 = {0};
+	DWORD ppid = 0, pid = GetCurrentProcessId();
+	snapshot = CreateToolhelp32Snapshot( TH32CS_SNAPPROCESS, 0 );
+	if (snapshot == INVALID_HANDLE_VALUE) {
+		return -1;
+	}
+	pe32.dwSize = sizeof( pe32 );
+	if (!Process32First(snapshot, &pe32)) {
+		CloseHandle(snapshot);
+		return -1;
+	}
+	do {
+		if (pe32.th32ProcessID == pid) {
+			ppid = pe32.th32ParentProcessID;
+			break;
+		}
+	} while (Process32Next(snapshot, &pe32));
+	CloseHandle(snapshot);
+	return ppid;
+}
+
+static bool getProcessName(int pid, char *buffer, int max_len)
+{
+	bool result = false;
+	HANDLE handle = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ,
+	                            FALSE, pid);
+	if (handle) {
+		GetModuleFileNameEx(handle, 0, buffer, max_len);
+		result = true;
+	}
+	CloseHandle(handle);
+	return result;
+}
+
+static bool isStartedFromCommandPrompt()
+{
+	HWND hwnd = GetConsoleWindow();
+	
+	if (hwnd) {
+		DWORD pid = (DWORD)-1;
+		DWORD ppid = GetParentProcessID();
+		char parent_name[MAX_PATH];
+		bool start_from_launcher = false;
+
+		GetWindowThreadProcessId(hwnd, &pid);
+		if (getProcessName(ppid, parent_name, sizeof(parent_name))) {
+			char *filename = strrchr(parent_name, '\\');
+			if (filename != NULL) {
+				start_from_launcher = strstr(filename, "blender.exe") != NULL;
+			}
+		}
+
+		/* When we're starting from a wrapper we need to compare with parent process ID. */
+		if (pid == (start_from_launcher ? ppid : GetCurrentProcessId()))
+			return true;
+	}
+
+	return false;
+}
+
 int GHOST_SystemWin32::toggleConsole(int action)
 {
 	switch (action)
 	{
-		case 3: //hide if no console
+		case 3: // startup: hide if not started from command prompt
 		{
-			DWORD sp = GetCurrentProcessId();
-			HANDLE ptree = CreateToolhelp32Snapshot(TH32CS_SNAPPROCESS, 0);
-			PROCESSENTRY32 e = {0}; e.dwSize = sizeof(PROCESSENTRY32);
-
-			if (Process32First(ptree, &e)) {
-				do {     //Searches for Blender's PROCESSENTRY32
-					if (e.th32ProcessID == sp) {
-						sp = e.th32ParentProcessID;
-						Process32First(ptree, &e);
-						do {             //Got parent id, searches for its PROCESSENTRY32
-							if (e.th32ProcessID == sp) {
-								if (strcmp("explorer.exe", e.szExeFile) == 0)
-								{             //If explorer, hide cmd
-									ShowWindow(GetConsoleWindow(), SW_HIDE);
-									m_consoleStatus = 0;
-								}
-								break;
-							}
-
-						} while (Process32Next(ptree, &e));
-						break;
-					}
-				} while (Process32Next(ptree, &e));
+			if (isStartedFromCommandPrompt()) {
+				ShowWindow(GetConsoleWindow(), SW_HIDE);
+				m_consoleStatus = 0;
 			}
-
-			CloseHandle(ptree);
 			break;
 		}
-		case 0: //hide
+		case 0: // hide
 			ShowWindow(GetConsoleWindow(), SW_HIDE);
 			m_consoleStatus = 0;
 			break;
-		case 1: //show
+		case 1: // show
 			ShowWindow(GetConsoleWindow(), SW_SHOW);
 			m_consoleStatus = 1;
 			break;
-		case 2: //toggle
+		case 2: // toggle
 			ShowWindow(GetConsoleWindow(), m_consoleStatus ? SW_HIDE : SW_SHOW);
 			m_consoleStatus = !m_consoleStatus;
 			break;
-
-	};
+	}
 
 
 	return m_consoleStatus;
@@ -1393,6 +1570,6 @@ int GHOST_SystemWin32::toggleConsole(int action)
 
 int GHOST_SystemWin32::confirmQuit(GHOST_IWindow *window) const
 {
-	return (MessageBox(window ? ((GHOST_WindowWin32 *)window)->getHWND() : 0, "Some changes have not been saved.\nDo you really want to quit ?",
+	return (MessageBox(window ? ((GHOST_WindowWin32 *)window)->getHWND() : 0, "Some changes have not been saved.\nDo you really want to quit?",
 	                   "Exit Blender", MB_OKCANCEL | MB_ICONWARNING | MB_TOPMOST) == IDOK);
 }
