@@ -37,17 +37,15 @@
 #error WIN32 only!
 #endif // WIN32
 
+#ifndef __MINGW64__
 #define _WIN32_WINNT 0x501 // require Windows XP or newer
+#endif
 #define WIN32_LEAN_AND_MEAN
 #include <windows.h>
 #include <ole2.h> // for drag-n-drop
 
 #include "GHOST_System.h"
 
-#if defined(__CYGWIN32__)
-#   define __int64 long long
-#endif
-
 class GHOST_EventButton;
 class GHOST_EventCursor;
 class GHOST_EventKey;
@@ -100,7 +98,13 @@ public:
 	 * \return The dimension of the main display.
 	 */
 	virtual void getMainDisplayDimensions(GHOST_TUns32& width, GHOST_TUns32& height) const;
-	
+
+	/**
+	 * Returns the dimensions of all displays on this system.
+	 * \return The dimension of the main display.
+	 */
+	virtual void getAllDisplayDimensions(GHOST_TUns32& width, GHOST_TUns32& height) const;
+
 	/**
 	 * Create a new window.
 	 * The new window is added to the list of windows managed. 
@@ -122,6 +126,7 @@ public:
 	    GHOST_TInt32 left, GHOST_TInt32 top, GHOST_TUns32 width, GHOST_TUns32 height,
 	    GHOST_TWindowState state, GHOST_TDrawingContextType type,
 	    const bool stereoVisual = false,
+		const bool exclusive = false,
 	    const GHOST_TUns16 numOfAASamples = 0,
 	    const GHOST_TEmbedderWindowID parentWindow = 0);
 
@@ -210,7 +215,7 @@ public:
 protected:
 	/**
 	 * Initializes the system.
-	 * For now, it justs registers the window class (WNDCLASS).
+	 * For now, it just registers the window class (WNDCLASS).
 	 * \return A success value.
 	 */
 	virtual GHOST_TSuccess init();
@@ -226,7 +231,7 @@ protected:
 	 * \param window->	The window for this handling
 	 * \param vKey		The virtual key from hardKey
 	 * \param ScanCode	The ScanCode of pressed key (simular to PS/2 Set 1)
-	 * \param extend	Flag if key is not primerly (left or right)
+	 * \param extend	Flag if key is not primly (left or right)
 	 * \return The GHOST key (GHOST_kKeyUnknown if no match).
 	 */
 	virtual GHOST_TKey convertKey(GHOST_IWindow *window, short vKey, short ScanCode, short extend) const;
@@ -301,6 +306,15 @@ protected:
 	 */
 	static GHOST_Event *processWindowEvent(GHOST_TEventType type, GHOST_IWindow *window);
 
+	/**
+	 * Creates a IME event.
+	 * \param type		The type of event to create.
+	 * \param window		The window receiving the event (the active window).
+	 * \param data		IME data.
+	 * \return The event created.
+	 */
+	static GHOST_Event *processImeEvent(GHOST_TEventType type, GHOST_IWindow *window, GHOST_TEventImeData *data);
+
 	/**
 	 * Handles minimum window size.
 	 * \param minmax	The MINMAXINFO structure.
