@@ -29,131 +29,66 @@
  *  \ingroup GHOST
  */
 
-#include <string.h>
+#define _USE_MATH_DEFINES
+
 #include "GHOST_WindowWin32.h"
 #include "GHOST_SystemWin32.h"
 #include "GHOST_DropTargetWin32.h"
+#include "GHOST_ContextNone.h"
 #include "utfconv.h"
 #include "utf_winfunc.h"
 
-// Need glew for some defines
-#include <GL/glew.h>
-#include <GL/wglew.h>
+#if defined(WITH_GL_EGL)
+#  include "GHOST_ContextEGL.h"
+#else
+#  include "GHOST_ContextWGL.h"
+#endif
+
+
 #include <math.h>
+#include <string.h>
+#include <assert.h>
 
-// MSVC6 still doesn't define M_PI
-#ifndef M_PI
-#  define M_PI 3.1415926536
-#endif
 
-// Some more multisample defines
-#define WGL_SAMPLE_BUFFERS_ARB  0x2041
-#define WGL_SAMPLES_ARB         0x2042
-
-// win64 doesn't define GWL_USERDATA
-#ifdef WIN32  /* why? - probably should remove */
-#  ifndef GWL_USERDATA
-#    define GWL_USERDATA GWLP_USERDATA
-#    define GWL_WNDPROC GWLP_WNDPROC
-#  endif
-#endif
 
 const wchar_t *GHOST_WindowWin32::s_windowClassName = L"GHOST_WindowClass";
 const int GHOST_WindowWin32::s_maxTitleLength = 128;
-HGLRC GHOST_WindowWin32::s_firsthGLRc = NULL;
-HDC GHOST_WindowWin32::s_firstHDC = NULL;
 
-static int WeightPixelFormat(PIXELFORMATDESCRIPTOR& pfd);
-static int EnumPixelFormats(HDC hdc);
 
-/*
- * Color and depth bit values are not to be trusted.
- * For instance, on TNT2:
- * When the screen color depth is set to 16 bit, we get 5 color bits
- * and 16 depth bits.
- * When the screen color depth is set to 32 bit, we get 8 color bits
- * and 24 depth bits.
- * Just to be safe, we request high waulity settings.
- */
-static PIXELFORMATDESCRIPTOR sPreferredFormat = {
-	sizeof(PIXELFORMATDESCRIPTOR),  /* size */
-	1,                              /* version */
-	PFD_SUPPORT_OPENGL |
-	PFD_DRAW_TO_WINDOW |
-	PFD_SWAP_COPY |                 /* support swap copy */
-	PFD_DOUBLEBUFFER,               /* support double-buffering */
-	PFD_TYPE_RGBA,                  /* color type */
-	32,                             /* prefered color depth */
-	0, 0, 0, 0, 0, 0,               /* color bits (ignored) */
-	0,                              /* no alpha buffer */
-	0,                              /* alpha bits (ignored) */
-	0,                              /* no accumulation buffer */
-	0, 0, 0, 0,                     /* accum bits (ignored) */
-	32,                             /* depth buffer */
-	0,                              /* no stencil buffer */
-	0,                              /* no auxiliary buffers */
-	PFD_MAIN_PLANE,                 /* main layer */
-	0,                              /* reserved */
-	0, 0, 0                         /* no layer, visible, damage masks */
-};
 
-/* Intel videocards don't work fine with multiple contexts and
- * have to share the same context for all windows.
- * But if we just share context for all windows it could work incorrect
- * with multiple videocards configuration. Suppose, that Intel videocards
- * can't be in multiple-devices configuration. */
-static int is_crappy_intel_card(void)
-{
-	int crappy = 0;
-	const char *vendor = (const char *)glGetString(GL_VENDOR);
 
-	if (strstr(vendor, "Intel"))
-		crappy = 1;
-
-	return crappy;
+/* force NVidia Optimus to used dedicated graphics */
+extern "C" {
+	__declspec(dllexport) DWORD NvOptimusEnablement = 0x00000001;
 }
 
 GHOST_WindowWin32::GHOST_WindowWin32(
-    GHOST_SystemWin32 *system,
-    const STR_String& title,
-    GHOST_TInt32 left,
-    GHOST_TInt32 top,
-    GHOST_TUns32 width,
-    GHOST_TUns32 height,
-    GHOST_TWindowState state,
-    GHOST_TDrawingContextType type,
-    const bool stereoVisual,
-    const GHOST_TUns16 numOfAASamples,
-    GHOST_TEmbedderWindowID parentwindowhwnd,
-    GHOST_TSuccess msEnabled,
-    int msPixelFormat)
-	:
-	GHOST_Window(width, height, state, GHOST_kDrawingContextTypeNone,
-	             stereoVisual, numOfAASamples),
-	m_system(system),
-	m_hDC(0),
-	m_hGlRc(0),
-	m_hasMouseCaptured(false),
-	m_hasGrabMouse(false),
-	m_nPressedButtons(0),
-	m_customCursor(0),
-	m_wintab(NULL),
-	m_tabletData(NULL),
-	m_tablet(0),
-	m_maxPressure(0),
-	m_multisample(numOfAASamples),
-	m_multisampleEnabled(msEnabled),
-	m_msPixelFormat(msPixelFormat),
-	//For recreation
-	m_title(title),
-	m_left(left),
-	m_top(top),
-	m_width(width),
-	m_height(height),
-	m_normal_state(GHOST_kWindowStateNormal),
-	m_stereo(stereoVisual),
-	m_nextWindow(NULL),
-	m_parentWindowHwnd(parentwindowhwnd)
+        GHOST_SystemWin32 *system,
+        const STR_String &title,
+        GHOST_TInt32 left,
+        GHOST_TInt32 top,
+        GHOST_TUns32 width,
+        GHOST_TUns32 height,
+        GHOST_TWindowState state,
+        GHOST_TDrawingContextType type,
+        bool wantStereoVisual,
+        GHOST_TUns16 wantNumOfAASamples,
+        GHOST_TEmbedderWindowID parentwindowhwnd)
+    : GHOST_Window(width, height, state,
+                   wantStereoVisual, false, wantNumOfAASamples),
+      m_inLiveResize(false),
+      m_system(system),
+      m_hDC(0),
+      m_hasMouseCaptured(false),
+      m_hasGrabMouse(false),
+      m_nPressedButtons(0),
+      m_customCursor(0),
+      m_wintab(NULL),
+      m_tabletData(NULL),
+      m_tablet(0),
+      m_maxPressure(0),
+      m_normal_state(GHOST_kWindowStateNormal),
+      m_parentWindowHwnd(parentwindowhwnd)
 {
 	OSVERSIONINFOEX versionInfo;
 	bool hasMinVersionForTaskbar = false;
@@ -165,13 +100,17 @@ GHOST_WindowWin32::GHOST_WindowWin32(
 	if (!GetVersionEx((OSVERSIONINFO *)&versionInfo)) {
 		versionInfo.dwOSVersionInfoSize = sizeof(OSVERSIONINFO);
 		if (GetVersionEx((OSVERSIONINFO *)&versionInfo)) {
-			if ((versionInfo.dwMajorVersion == 6 && versionInfo.dwMinorVersion >= 1) || versionInfo.dwMajorVersion >= 7) {
+			if ((versionInfo.dwMajorVersion == 6 && versionInfo.dwMinorVersion >= 1) ||
+			    (versionInfo.dwMajorVersion >= 7))
+			{
 				hasMinVersionForTaskbar = true;
 			}
 		}
 	}
 	else {
-		if ((versionInfo.dwMajorVersion == 6 && versionInfo.dwMinorVersion >= 1) || versionInfo.dwMajorVersion >= 7) {
+		if ((versionInfo.dwMajorVersion == 6 && versionInfo.dwMinorVersion >= 1) ||
+		    (versionInfo.dwMajorVersion >= 7))
+		{
 			hasMinVersionForTaskbar = true;
 		}
 	}
@@ -181,8 +120,21 @@ GHOST_WindowWin32::GHOST_WindowWin32(
 		MONITORINFO monitor;
 		GHOST_TUns32 tw, th; 
 
-		width += GetSystemMetrics(SM_CXSIZEFRAME) * 2;
-		height += GetSystemMetrics(SM_CYSIZEFRAME) * 2 + GetSystemMetrics(SM_CYCAPTION);
+#ifndef _MSC_VER
+		int cxsizeframe = GetSystemMetrics(SM_CXSIZEFRAME);
+		int cysizeframe = GetSystemMetrics(SM_CYSIZEFRAME);
+#else
+		// MSVC 2012+ returns bogus values from GetSystemMetrics, bug in Windows
+		// http://connect.microsoft.com/VisualStudio/feedback/details/753224/regression-getsystemmetrics-delivers-different-values
+		RECT cxrect = {0, 0, 0, 0};
+		AdjustWindowRectEx(&cxrect, WS_POPUP | WS_CLIPSIBLINGS | WS_CLIPCHILDREN | WS_THICKFRAME | WS_DLGFRAME, FALSE, 0);
+
+		int cxsizeframe = abs(cxrect.bottom);
+		int cysizeframe = abs(cxrect.left);
+#endif
+
+		width += cxsizeframe * 2;
+		height += cysizeframe * 2 + GetSystemMetrics(SM_CYCAPTION);
 
 		rect.left = left;
 		rect.right = left + width;
@@ -198,8 +150,7 @@ GHOST_WindowWin32::GHOST_WindowWin32(
 		th = monitor.rcWork.bottom - monitor.rcWork.top;
 		tw = monitor.rcWork.right - monitor.rcWork.left;
 
-		if (tw < width)
-		{
+		if (tw < width) {
 			width = tw;
 			left = monitor.rcWork.left;
 		}
@@ -208,8 +159,7 @@ GHOST_WindowWin32::GHOST_WindowWin32(
 		else if (left < monitor.rcWork.left)
 			left = monitor.rcWork.left;
 
-		if (th < height)
-		{
+		if (th < height) {
 			height = th;
 			top = monitor.rcWork.top;
 		}
@@ -219,8 +169,7 @@ GHOST_WindowWin32::GHOST_WindowWin32(
 			top = monitor.rcWork.top;
 
 		int wintype = WS_OVERLAPPEDWINDOW;
-		if (m_parentWindowHwnd != 0)
-		{
+		if (m_parentWindowHwnd != 0) {
 			wintype = WS_CHILD;
 			GetWindowRect((HWND)m_parentWindowHwnd, &rect);
 			left = 0;
@@ -269,41 +218,37 @@ GHOST_WindowWin32::GHOST_WindowWin32(
 		}
 
 		// Store a pointer to this class in the window structure
-		::SetWindowLongPtr(m_hWnd, GWL_USERDATA, (LONG_PTR) this);
+		::SetWindowLongPtr(m_hWnd, GWLP_USERDATA, (LONG_PTR) this);
 
 		// Store the device context
 		m_hDC = ::GetDC(m_hWnd);
 
-		if (!s_firstHDC) {
-			s_firstHDC = m_hDC;
-		}
+		GHOST_TSuccess success = setDrawingContextType(type);
 
-		// Show the window
-		int nCmdShow;
-		switch (state) {
-			case GHOST_kWindowStateMaximized:
-				nCmdShow = SW_SHOWMAXIMIZED;
-				break;
-			case GHOST_kWindowStateMinimized:
-				nCmdShow = SW_SHOWMINIMIZED;
-				break;
-			case GHOST_kWindowStateNormal:
-			default:
-				nCmdShow = SW_SHOWNORMAL;
-				break;
-		}
-		GHOST_TSuccess success;
-		success = setDrawingContextType(type);
+		if (success) {
+			// Show the window
+			int nCmdShow;
+			switch (state) {
+				case GHOST_kWindowStateMaximized:
+					nCmdShow = SW_SHOWMAXIMIZED;
+					break;
+				case GHOST_kWindowStateMinimized:
+					nCmdShow = SW_SHOWMINIMIZED;
+					break;
+				case GHOST_kWindowStateNormal:
+				default:
+					nCmdShow = SW_SHOWNORMAL;
+					break;
+			}
 
-		if (success)
-		{
 			::ShowWindow(m_hWnd, nCmdShow);
 			// Force an initial paint of the window
 			::UpdateWindow(m_hWnd);
 		}
 		else {
 			//invalidate the window
-			m_hWnd = 0;
+			::DestroyWindow(m_hWnd);
+			m_hWnd = NULL;
 		}
 	}
 
@@ -326,7 +271,8 @@ GHOST_WindowWin32::GHOST_WindowWin32(
 		if (fpWTInfo && fpWTInfo(0, 0, NULL)) {
 			// Now init the tablet
 			LOGCONTEXT lc;
-			AXIS TabletX, TabletY, Pressure, Orientation[3]; /* The maximum tablet size, pressure and orientation (tilt) */
+			/* The maximum tablet size, pressure and orientation (tilt) */
+			AXIS TabletX, TabletY, Pressure, Orientation[3];
 
 			// Open a Wintab context
 
@@ -382,11 +328,10 @@ GHOST_WindowWin32::GHOST_WindowWin32(
 
 GHOST_WindowWin32::~GHOST_WindowWin32()
 {
-	if (m_Bar)
-	{
+	if (m_Bar) {
 		m_Bar->SetProgressState(m_hWnd, TBPF_NOPROGRESS);
 		m_Bar->Release();
-	};
+	}
 
 	if (m_wintab) {
 		GHOST_WIN32_WTClose fpWTClose = (GHOST_WIN32_WTClose) ::GetProcAddress(m_wintab, "WTClose");
@@ -398,19 +343,16 @@ GHOST_WindowWin32::~GHOST_WindowWin32()
 			m_tabletData = NULL;
 		}
 	}
+
 	if (m_customCursor) {
 		DestroyCursor(m_customCursor);
 		m_customCursor = NULL;
 	}
 
-	::wglMakeCurrent(NULL, NULL);
-	m_multisampleEnabled = GHOST_kFailure;
-	m_multisample = 0;
-	setDrawingContextType(GHOST_kDrawingContextTypeNone);
-	if (m_hDC && m_hDC != s_firstHDC) {
+	if (m_hWnd != NULL && m_hDC != NULL && releaseNativeHandles()) {
 		::ReleaseDC(m_hWnd, m_hDC);
-		m_hDC = 0;
 	}
+
 	if (m_hWnd) {
 		if (m_dropTarget) {
 			// Disable DragDrop
@@ -424,14 +366,9 @@ GHOST_WindowWin32::~GHOST_WindowWin32()
 	}
 }
 
-GHOST_Window *GHOST_WindowWin32::getNextWindow()
-{
-	return m_nextWindow;
-}
-
 bool GHOST_WindowWin32::getValid() const
 {
-	return m_hWnd != 0;
+	return GHOST_Window::getValid() && m_hWnd != 0 && m_hDC != 0;
 }
 
 HWND GHOST_WindowWin32::getHWND() const
@@ -439,7 +376,7 @@ HWND GHOST_WindowWin32::getHWND() const
 	return m_hWnd;
 }
 
-void GHOST_WindowWin32::setTitle(const STR_String& title)
+void GHOST_WindowWin32::setTitle(const STR_String &title)
 {
 	wchar_t *title_16 = alloc_utf16_from_8((char *)(const char *)title, 0);
 	::SetWindowTextW(m_hWnd, (wchar_t *)title_16);
@@ -447,7 +384,7 @@ void GHOST_WindowWin32::setTitle(const STR_String& title)
 }
 
 
-void GHOST_WindowWin32::getTitle(STR_String& title) const
+void GHOST_WindowWin32::getTitle(STR_String &title) const
 {
 	char buf[s_maxTitleLength]; /*CHANGE + never used yet*/
 	::GetWindowText(m_hWnd, buf, s_maxTitleLength);
@@ -456,7 +393,7 @@ void GHOST_WindowWin32::getTitle(STR_String& title) const
 }
 
 
-void GHOST_WindowWin32::getWindowBounds(GHOST_Rect& bounds) const
+void GHOST_WindowWin32::getWindowBounds(GHOST_Rect &bounds) const
 {
 	RECT rect;
 	::GetWindowRect(m_hWnd, &rect);
@@ -467,41 +404,25 @@ void GHOST_WindowWin32::getWindowBounds(GHOST_Rect& bounds) const
 }
 
 
-void GHOST_WindowWin32::getClientBounds(GHOST_Rect& bounds) const
+void GHOST_WindowWin32::getClientBounds(GHOST_Rect &bounds) const
 {
 	RECT rect;
-	GHOST_TWindowState state = this->getState();
-	LONG_PTR result = ::GetWindowLongPtr(m_hWnd, GWL_STYLE);
-	int sm_cysizeframe = GetSystemMetrics(SM_CYSIZEFRAME);
-	::GetWindowRect(m_hWnd, &rect);
+	POINT coord;
+	::GetClientRect(m_hWnd, &rect);
 
-	if ((result & (WS_POPUP | WS_MAXIMIZE)) != (WS_POPUP | WS_MAXIMIZE)) {
-		if (state == GHOST_kWindowStateMaximized) {
-			// in maximized state we don't have borders on the window
-			bounds.m_b = rect.bottom - GetSystemMetrics(SM_CYCAPTION) - sm_cysizeframe * 2;
-			bounds.m_l = rect.left + sm_cysizeframe;
-			bounds.m_r = rect.right - sm_cysizeframe;
-			bounds.m_t = rect.top;
-		}
-		else if (state == GHOST_kWindowStateEmbedded) {
-			bounds.m_b = rect.bottom;
-			bounds.m_l = rect.left;
-			bounds.m_r = rect.right;
-			bounds.m_t = rect.top;
-		}
-		else {
-			bounds.m_b = rect.bottom - GetSystemMetrics(SM_CYCAPTION) - sm_cysizeframe * 2;
-			bounds.m_l = rect.left;
-			bounds.m_r = rect.right - sm_cysizeframe * 2;
-			bounds.m_t = rect.top;
-		}
-	}
-	else {
-		bounds.m_b = rect.bottom;
-		bounds.m_l = rect.left;
-		bounds.m_r = rect.right;
-		bounds.m_t = rect.top;
-	}
+	coord.x = rect.left;
+	coord.y = rect.top;
+	::ClientToScreen(m_hWnd, &coord);
+
+	bounds.m_l = coord.x;
+	bounds.m_t = coord.y;
+
+	coord.x = rect.right;
+	coord.y = rect.bottom;
+	::ClientToScreen(m_hWnd, &coord);
+
+	bounds.m_r = coord.x;
+	bounds.m_b = coord.y;
 }
 
 
@@ -574,6 +495,7 @@ GHOST_TWindowState GHOST_WindowWin32::getState() const
 		state = GHOST_kWindowStateEmbedded;
 		return state;
 	}
+
 	if (::IsIconic(m_hWnd)) {
 		state = GHOST_kWindowStateMinimized;
 	}
@@ -591,18 +513,22 @@ GHOST_TWindowState GHOST_WindowWin32::getState() const
 }
 
 
-void GHOST_WindowWin32::screenToClient(GHOST_TInt32 inX, GHOST_TInt32 inY, GHOST_TInt32& outX, GHOST_TInt32& outY) const
+void GHOST_WindowWin32::screenToClient(
+        GHOST_TInt32 inX, GHOST_TInt32 inY,
+        GHOST_TInt32 &outX, GHOST_TInt32 &outY) const
 {
-	POINT point = { inX, inY };
+	POINT point = {inX, inY};
 	::ScreenToClient(m_hWnd, &point);
 	outX = point.x;
 	outY = point.y;
 }
 
 
-void GHOST_WindowWin32::clientToScreen(GHOST_TInt32 inX, GHOST_TInt32 inY, GHOST_TInt32& outX, GHOST_TInt32& outY) const
+void GHOST_WindowWin32::clientToScreen(
+        GHOST_TInt32 inX, GHOST_TInt32 inY,
+        GHOST_TInt32 &outX, GHOST_TInt32 &outY) const
 {
-	POINT point = { inX, inY };
+	POINT point = {inX, inY};
 	::ClientToScreen(m_hWnd, &point);
 	outX = point.x;
 	outY = point.y;
@@ -618,13 +544,14 @@ GHOST_TSuccess GHOST_WindowWin32::setState(GHOST_TWindowState state)
 
 	if (state == GHOST_kWindowStateNormal)
 		state = m_normal_state;
+
 	switch (state) {
 		case GHOST_kWindowStateMinimized:
 			wp.showCmd = SW_SHOWMINIMIZED;
 			break;
 		case GHOST_kWindowStateMaximized:
 			wp.showCmd = SW_SHOWMAXIMIZED;
-			SetWindowLongPtr(m_hWnd, GWL_STYLE, WS_OVERLAPPEDWINDOW);
+			::SetWindowLongPtr(m_hWnd, GWL_STYLE, WS_OVERLAPPEDWINDOW);
 			break;
 		case GHOST_kWindowStateFullScreen:
 			if (curstate != state && curstate != GHOST_kWindowStateMinimized)
@@ -632,55 +559,45 @@ GHOST_TSuccess GHOST_WindowWin32::setState(GHOST_TWindowState state)
 			wp.showCmd = SW_SHOWMAXIMIZED;
 			wp.ptMaxPosition.x = 0;
 			wp.ptMaxPosition.y = 0;
-			SetWindowLongPtr(m_hWnd, GWL_STYLE, WS_POPUP | WS_MAXIMIZE);
+			::SetWindowLongPtr(m_hWnd, GWL_STYLE, WS_POPUP | WS_MAXIMIZE);
 			break;
 		case GHOST_kWindowStateEmbedded:
-			SetWindowLongPtr(m_hWnd, GWL_STYLE, WS_CHILD);
+			::SetWindowLongPtr(m_hWnd, GWL_STYLE, WS_CHILD);
 			break;
 		case GHOST_kWindowStateNormal:
 		default:
 			wp.showCmd = SW_SHOWNORMAL;
-			SetWindowLongPtr(m_hWnd, GWL_STYLE, WS_OVERLAPPEDWINDOW);
+			::SetWindowLongPtr(m_hWnd, GWL_STYLE, WS_OVERLAPPEDWINDOW);
 			break;
 	}
-	SetWindowPos(m_hWnd, 0, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED); /*Clears window cache for SetWindowLongPtr */
+	/* Clears window cache for SetWindowLongPtr */
+	::SetWindowPos(m_hWnd, 0, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE | SWP_NOZORDER | SWP_FRAMECHANGED);
+
 	return ::SetWindowPlacement(m_hWnd, &wp) == TRUE ? GHOST_kSuccess : GHOST_kFailure;
 }
 
 
 GHOST_TSuccess GHOST_WindowWin32::setOrder(GHOST_TWindowOrder order)
 {
-	HWND hWndInsertAfter = order == GHOST_kWindowOrderTop ? HWND_TOP : HWND_BOTTOM;
-	return ::SetWindowPos(m_hWnd, hWndInsertAfter, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE) == TRUE ? GHOST_kSuccess : GHOST_kFailure;
-}
+	HWND hWndInsertAfter, hWndToRaise;
 
-
-GHOST_TSuccess GHOST_WindowWin32::swapBuffers()
-{
-	HDC hDC = m_hDC;
-
-	if (is_crappy_intel_card())
-		hDC = ::wglGetCurrentDC();
-
-	return ::SwapBuffers(hDC) == TRUE ? GHOST_kSuccess : GHOST_kFailure;
-}
-
-
-GHOST_TSuccess GHOST_WindowWin32::activateDrawingContext()
-{
-	GHOST_TSuccess success;
-	if (m_drawingContextType == GHOST_kDrawingContextTypeOpenGL) {
-		if (m_hDC && m_hGlRc) {
-			success = ::wglMakeCurrent(m_hDC, m_hGlRc) == TRUE ? GHOST_kSuccess : GHOST_kFailure;
-		}
-		else {
-			success = GHOST_kFailure;
-		}
+	if (order == GHOST_kWindowOrderBottom) {
+		hWndInsertAfter = HWND_BOTTOM;
+		hWndToRaise = ::GetWindow(m_hWnd, GW_HWNDNEXT); /* the window to raise */
 	}
 	else {
-		success = GHOST_kSuccess;
+		hWndInsertAfter = HWND_TOP;
+		hWndToRaise = NULL;
 	}
-	return success;
+
+	if (::SetWindowPos(m_hWnd, hWndInsertAfter, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE) == FALSE) {
+		return GHOST_kFailure;
+	}
+
+	if (hWndToRaise && ::SetWindowPos(hWndToRaise, HWND_TOP, 0, 0, 0, 0, SWP_NOMOVE | SWP_NOSIZE) == FALSE) {
+		return GHOST_kFailure;
+	}
+	return GHOST_kSuccess;
 }
 
 
@@ -696,283 +613,119 @@ GHOST_TSuccess GHOST_WindowWin32::invalidate()
 	return success;
 }
 
-GHOST_TSuccess GHOST_WindowWin32::initMultisample(PIXELFORMATDESCRIPTOR pfd)
+
+GHOST_Context *GHOST_WindowWin32::newDrawingContext(GHOST_TDrawingContextType type)
 {
-	int pixelFormat;
-	bool success = FALSE;
-	UINT numFormats;
-	HDC hDC = GetDC(getHWND());
-	float fAttributes[] = {0, 0};
-	UINT nMaxFormats = 1;
+	if (type == GHOST_kDrawingContextTypeOpenGL) {
+#if !defined(WITH_GL_EGL)
 
-	// The attributes to look for
-	int iAttributes[] = {
-		WGL_DRAW_TO_WINDOW_ARB, GL_TRUE,
-		WGL_SUPPORT_OPENGL_ARB, GL_TRUE,
-		WGL_ACCELERATION_ARB, WGL_FULL_ACCELERATION_ARB,
-		WGL_COLOR_BITS_ARB, pfd.cColorBits,
-		WGL_DEPTH_BITS_ARB, pfd.cDepthBits,
-		WGL_STENCIL_BITS_ARB, pfd.cStencilBits,
-		WGL_DOUBLE_BUFFER_ARB, GL_TRUE,
-		WGL_SAMPLE_BUFFERS_ARB, GL_TRUE,
-		WGL_SAMPLES_ARB, m_multisample,
-		0, 0
-	};
+#if defined(WITH_GL_PROFILE_CORE)
+		GHOST_Context *context = new GHOST_ContextWGL(
+		        m_wantStereoVisual,
+		        m_wantNumOfAASamples,
+		        m_hWnd,
+		        m_hDC,
+		        WGL_CONTEXT_OPENGL_CORE_PROFILE_BIT,
+		        3, 2,
+		        GHOST_OPENGL_WGL_CONTEXT_FLAGS,
+		        GHOST_OPENGL_WGL_RESET_NOTIFICATION_STRATEGY);
+#elif defined(WITH_GL_PROFILE_ES20)
+		GHOST_Context *context = new GHOST_ContextWGL(
+		        m_wantStereoVisual,
+		        m_wantNumOfAASamples,
+		        m_hWnd,
+		        m_hDC,
+		        WGL_CONTEXT_ES2_PROFILE_BIT_EXT,
+		        2, 0,
+		        GHOST_OPENGL_WGL_CONTEXT_FLAGS,
+		        GHOST_OPENGL_WGL_RESET_NOTIFICATION_STRATEGY);
+#elif defined(WITH_GL_PROFILE_COMPAT)
+		GHOST_Context *context = new GHOST_ContextWGL(
+		        m_wantStereoVisual,
+		        m_wantNumOfAASamples,
+		        m_hWnd,
+		        m_hDC,
+		        0, // profile bit
+		        0, 0,
+		        GHOST_OPENGL_WGL_CONTEXT_FLAGS,
+		        GHOST_OPENGL_WGL_RESET_NOTIFICATION_STRATEGY);
+#else
+#  error
+#endif
 
-	// Get the function
-	PFNWGLCHOOSEPIXELFORMATARBPROC wglChoosePixelFormatARB = (PFNWGLCHOOSEPIXELFORMATARBPROC)wglGetProcAddress("wglChoosePixelFormatARB");	
+#else
 
-	if (!wglChoosePixelFormatARB)
-	{
-		m_multisampleEnabled = GHOST_kFailure;
-		return GHOST_kFailure;
+#if defined(WITH_GL_PROFILE_CORE)
+		GHOST_Context *context = new GHOST_ContextEGL(
+		        m_wantStereoVisual,
+		        m_wantNumOfAASamples,
+		        m_hWnd,
+		        m_hDC,
+		        EGL_CONTEXT_OPENGL_CORE_PROFILE_BIT,
+		        3, 2,
+		        GHOST_OPENGL_EGL_CONTEXT_FLAGS,
+		        GHOST_OPENGL_EGL_RESET_NOTIFICATION_STRATEGY,
+		        EGL_OPENGL_API);
+#elif defined(WITH_GL_PROFILE_ES20)
+		GHOST_Context *context = new GHOST_ContextEGL(
+		        m_wantStereoVisual,
+		        m_wantNumOfAASamples,
+		        m_hWnd,
+		        m_hDC,
+		        0, // profile bit
+		        2, 0,
+		        GHOST_OPENGL_EGL_CONTEXT_FLAGS,
+		        GHOST_OPENGL_EGL_RESET_NOTIFICATION_STRATEGY,
+		        EGL_OPENGL_ES_API);
+#elif defined(WITH_GL_PROFILE_COMPAT)
+		GHOST_Context *context = new GHOST_ContextEGL(
+		        m_wantStereoVisual,
+		        m_wantNumOfAASamples,
+		        m_hWnd,
+		        m_hDC,
+		        0, // profile bit
+		        0, 0,
+		        GHOST_OPENGL_EGL_CONTEXT_FLAGS,
+		        GHOST_OPENGL_EGL_RESET_NOTIFICATION_STRATEGY,
+		        EGL_OPENGL_API);
+#else
+#  error
+#endif
+
+#endif
+		if (context->initializeDrawingContext())
+			return context;
+		else
+			delete context;
 	}
 
-	// iAttributes[17] is the initial multisample. If not valid try to use the closest valid value under it.
-	while (iAttributes[17] > 0) {
-		// See if the format is valid
-		success = wglChoosePixelFormatARB(hDC, iAttributes, fAttributes, nMaxFormats, &pixelFormat, &numFormats);
-		GHOST_PRINTF("WGL_SAMPLES_ARB = %i --> success = %i, %i formats\n", iAttributes[17], success, numFormats);
-
-		if (success && numFormats >= 1 && m_multisampleEnabled == GHOST_kFailure) {
-			GHOST_PRINTF("valid pixel format with %i multisamples\n", iAttributes[17]);
-			m_multisampleEnabled = GHOST_kSuccess;
-			m_msPixelFormat = pixelFormat;
-		}
-		iAttributes[17] -= 1;
-		success = GHOST_kFailure;
-	}
-	if (m_multisampleEnabled == GHOST_kSuccess) {
-		return GHOST_kSuccess;
-	}
-	GHOST_PRINT("no available pixel format\n");
-	return GHOST_kFailure;
-}
-
-GHOST_TSuccess GHOST_WindowWin32::installDrawingContext(GHOST_TDrawingContextType type)
-{
-	GHOST_TSuccess success;
-	switch (type) {
-		case GHOST_kDrawingContextTypeOpenGL:
-		{
-			// If this window has multisample enabled, use the supplied format
-			if (m_multisampleEnabled)
-			{
-				if (SetPixelFormat(m_hDC, m_msPixelFormat, &sPreferredFormat) == FALSE)
-				{
-					success = GHOST_kFailure;
-					break;
-				}
-
-				// Create the context
-				m_hGlRc = ::wglCreateContext(m_hDC);
-				if (m_hGlRc) {
-					if (::wglMakeCurrent(m_hDC, m_hGlRc) == TRUE) {
-						if (s_firsthGLRc) {
-							if (is_crappy_intel_card()) {
-								if (::wglMakeCurrent(NULL, NULL) == TRUE) {
-									::wglDeleteContext(m_hGlRc);
-									m_hGlRc = s_firsthGLRc;
-								}
-								else {
-									::wglDeleteContext(m_hGlRc);
-									m_hGlRc = NULL;
-								}
-							}
-							else {
-								::wglCopyContext(s_firsthGLRc, m_hGlRc, GL_ALL_ATTRIB_BITS);
-								::wglShareLists(s_firsthGLRc, m_hGlRc);
-							}
-						}
-						else {
-							s_firsthGLRc = m_hGlRc;
-						}
-
-						if (m_hGlRc) {
-							success = ::wglMakeCurrent(m_hDC, m_hGlRc) == TRUE ? GHOST_kSuccess : GHOST_kFailure;
-						}
-						else {
-							success = GHOST_kFailure;
-						}
-					}
-					else {
-						success = GHOST_kFailure;
-					}
-				}
-				else {
-					success = GHOST_kFailure;
-				}
-
-				if (success == GHOST_kFailure) {
-					printf("Failed to get a context....\n");
-				}
-			}
-			else {
-				if (m_stereoVisual)
-					sPreferredFormat.dwFlags |= PFD_STEREO;
-
-				// Attempt to match device context pixel format to the preferred format
-				int iPixelFormat = EnumPixelFormats(m_hDC);
-				if (iPixelFormat == 0) {
-					success = GHOST_kFailure;
-					break;
-				}
-				if (::SetPixelFormat(m_hDC, iPixelFormat, &sPreferredFormat) == FALSE) {
-					success = GHOST_kFailure;
-					break;
-				}
-				// For debugging only: retrieve the pixel format chosen
-				PIXELFORMATDESCRIPTOR preferredFormat;
-				::DescribePixelFormat(m_hDC, iPixelFormat, sizeof(PIXELFORMATDESCRIPTOR), &preferredFormat);
-
-				// Create the context
-				m_hGlRc = ::wglCreateContext(m_hDC);
-				if (m_hGlRc) {
-					if (::wglMakeCurrent(m_hDC, m_hGlRc) == TRUE) {
-						if (s_firsthGLRc) {
-							if (is_crappy_intel_card()) {
-								if (::wglMakeCurrent(NULL, NULL) == TRUE) {
-									::wglDeleteContext(m_hGlRc);
-									m_hGlRc = s_firsthGLRc;
-								}
-								else {
-									::wglDeleteContext(m_hGlRc);
-									m_hGlRc = NULL;
-								}
-							}
-							else {
-								::wglShareLists(s_firsthGLRc, m_hGlRc);
-							}
-						}
-						else {
-							s_firsthGLRc = m_hGlRc;
-						}
-
-						if (m_hGlRc) {
-							success = ::wglMakeCurrent(m_hDC, m_hGlRc) == TRUE ? GHOST_kSuccess : GHOST_kFailure;
-						}
-						else {
-							success = GHOST_kFailure;
-						}
-					}
-					else {
-						success = GHOST_kFailure;
-					}
-				}
-				else {
-					success = GHOST_kFailure;
-				}
-					
-				if (success == GHOST_kFailure) {
-					printf("Failed to get a context....\n");
-				}
-
-				// Attempt to enable multisample
-				if (m_multisample && WGL_ARB_multisample && !m_multisampleEnabled)
-				{
-					success = initMultisample(preferredFormat);
-
-					if (success)
-					{
-
-						// Make sure we don't screw up the context
-						if (m_hGlRc == s_firsthGLRc)
-							s_firsthGLRc = NULL;
-						m_drawingContextType = GHOST_kDrawingContextTypeOpenGL;
-						removeDrawingContext();
-
-						// Create a new window
-						GHOST_TWindowState new_state = getState();
-
-						m_nextWindow = new GHOST_WindowWin32((GHOST_SystemWin32 *)GHOST_ISystem::getSystem(),
-						                                     m_title,
-						                                     m_left,
-						                                     m_top,
-						                                     m_width,
-						                                     m_height,
-						                                     new_state,
-						                                     type,
-						                                     m_stereo,
-						                                     m_multisample,
-						                                     m_parentWindowHwnd,
-						                                     m_multisampleEnabled,
-						                                     m_msPixelFormat);
-
-						// Return failure so we can trash this window.
-						success = GHOST_kFailure;
-						break;
-					}
-					else {
-						m_multisampleEnabled = GHOST_kSuccess;
-						printf("Multisample failed to initialized\n");
-						success = GHOST_kSuccess;
-					}
-				}
-			}
-
-		}
-		break;
-
-		case GHOST_kDrawingContextTypeNone:
-			success = GHOST_kSuccess;
-			break;
-
-		default:
-			success = GHOST_kFailure;
-	}
-	return success;
-}
-
-GHOST_TSuccess GHOST_WindowWin32::removeDrawingContext()
-{
-	GHOST_TSuccess success;
-	switch (m_drawingContextType) {
-		case GHOST_kDrawingContextTypeOpenGL:
-			// we shouldn't remove the drawing context if it's the first OpenGL context
-			// If we do, we get corrupted drawing. See #19997
-			if (m_hGlRc && m_hGlRc != s_firsthGLRc) {
-				success = ::wglDeleteContext(m_hGlRc) == TRUE ? GHOST_kSuccess : GHOST_kFailure;
-				m_hGlRc = 0;
-			}
-			else {
-				success = GHOST_kFailure;
-			}
-			break;
-		case GHOST_kDrawingContextTypeNone:
-			success = GHOST_kSuccess;
-			break;
-		default:
-			success = GHOST_kFailure;
-	}
-	return success;
+	return NULL;
 }
 
 void GHOST_WindowWin32::lostMouseCapture()
 {
-	if (m_hasMouseCaptured)
-	{   m_hasGrabMouse = false;
+	if (m_hasMouseCaptured) {
+		m_hasGrabMouse = false;
 		m_nPressedButtons = 0;
-		m_hasMouseCaptured = false; };
+		m_hasMouseCaptured = false;
+	}
 }
 
 void GHOST_WindowWin32::registerMouseClickEvent(int press)
 {
 
-	switch (press)
-	{
+	switch (press) {
 		case 0: m_nPressedButtons++;    break;
 		case 1: if (m_nPressedButtons) m_nPressedButtons--; break;
 		case 2: m_hasGrabMouse = true;    break;
 		case 3: m_hasGrabMouse = false;   break;
 	}
 
-	if (!m_nPressedButtons && !m_hasGrabMouse && m_hasMouseCaptured)
-	{
+	if (!m_nPressedButtons && !m_hasGrabMouse && m_hasMouseCaptured) {
 		::ReleaseCapture();
 		m_hasMouseCaptured = false;
 	}
-	else if ((m_nPressedButtons || m_hasGrabMouse) && !m_hasMouseCaptured)
-	{
+	else if ((m_nPressedButtons || m_hasGrabMouse) && !m_hasMouseCaptured) {
 		::SetCapture(m_hWnd);
 		m_hasMouseCaptured = true;
 
@@ -1089,7 +842,7 @@ GHOST_TSuccess GHOST_WindowWin32::setWindowCursorShape(GHOST_TStandardCursor cur
 
 void GHOST_WindowWin32::processWin32TabletInitEvent()
 {
-	if (m_wintab) {
+	if (m_wintab && m_tabletData) {
 		GHOST_WIN32_WTInfo fpWTInfo = (GHOST_WIN32_WTInfo) ::GetProcAddress(m_wintab, "WTInfoA");
 
 		// let's see if we can initialize tablet here
@@ -1135,10 +888,12 @@ void GHOST_WindowWin32::processWin32TabletEvent(WPARAM wParam, LPARAM lParam)
 							break;
 						case 1:
 						case 4:
+						case 7:
 							m_tabletData->Active = GHOST_kTabletModeStylus; /* stylus */
 							break;
 						case 2:
 						case 5:
+						case 8:
 							m_tabletData->Active = GHOST_kTabletModeEraser; /* eraser */
 							break;
 					}
@@ -1192,6 +947,16 @@ void GHOST_WindowWin32::processWin32TabletEvent(WPARAM wParam, LPARAM lParam)
 	}
 }
 
+void GHOST_WindowWin32::bringTabletContextToFront()
+{
+	if (m_wintab) {
+		GHOST_WIN32_WTOverlap fpWTOverlap = (GHOST_WIN32_WTOverlap) ::GetProcAddress(m_wintab, "WTOverlap");
+		if (fpWTOverlap) {
+			fpWTOverlap(m_tablet, TRUE);
+		}
+	}
+}
+
 /** Reverse the bits in a GHOST_TUns8 */
 static GHOST_TUns8 uns8ReverseBits(GHOST_TUns8 ch)
 {
@@ -1212,17 +977,19 @@ static GHOST_TUns16 uns16ReverseBits(GHOST_TUns16 shrt)
 	return shrt;
 }
 #endif
-GHOST_TSuccess GHOST_WindowWin32::setWindowCustomCursorShape(GHOST_TUns8 bitmap[16][2],
-                                                             GHOST_TUns8 mask[16][2],
-                                                             int hotX, int hotY)
+GHOST_TSuccess GHOST_WindowWin32::setWindowCustomCursorShape(
+        GHOST_TUns8 bitmap[16][2],
+        GHOST_TUns8 mask[16][2],
+        int hotX, int hotY)
 {
 	return setWindowCustomCursorShape((GHOST_TUns8 *)bitmap, (GHOST_TUns8 *)mask,
 	                                  16, 16, hotX, hotY, 0, 1);
 }
 
-GHOST_TSuccess GHOST_WindowWin32::setWindowCustomCursorShape(GHOST_TUns8 *bitmap,
-                                                             GHOST_TUns8 *mask, int sizeX, int sizeY, int hotX, int hotY,
-                                                             int fg_color, int bg_color)
+GHOST_TSuccess GHOST_WindowWin32::setWindowCustomCursorShape(
+        GHOST_TUns8 *bitmap,
+        GHOST_TUns8 *mask, int sizeX, int sizeY, int hotX, int hotY,
+        int fg_color, int bg_color)
 {
 	GHOST_TUns32 andData[32];
 	GHOST_TUns32 xorData[32];
@@ -1283,90 +1050,16 @@ GHOST_TSuccess GHOST_WindowWin32::endProgressBar()
 	return GHOST_kFailure;
 }
 
-/* Ron Fosner's code for weighting pixel formats and forcing software.
- * See http://www.opengl.org/resources/faq/technical/weight.cpp */
 
-static int WeightPixelFormat(PIXELFORMATDESCRIPTOR& pfd)
+#ifdef WITH_INPUT_IME
+void GHOST_WindowWin32::beginIME(GHOST_TInt32 x, GHOST_TInt32 y, GHOST_TInt32 w, GHOST_TInt32 h, int completed)
 {
-	int weight = 0;
-
-	/* assume desktop color depth is 32 bits per pixel */
-
-	/* cull unusable pixel formats */
-	/* if no formats can be found, can we determine why it was rejected? */
-	if (!(pfd.dwFlags & PFD_SUPPORT_OPENGL) ||
-	    !(pfd.dwFlags & PFD_DRAW_TO_WINDOW) ||
-	    !(pfd.dwFlags & PFD_DOUBLEBUFFER) || /* Blender _needs_ this */
-	    (pfd.cDepthBits <= 8) ||
-	    !(pfd.iPixelType == PFD_TYPE_RGBA))
-	{
-		return 0;
-	}
-
-	weight = 1;  /* it's usable */
-
-	/* the bigger the depth buffer the better */
-	/* give no weight to a 16-bit depth buffer, because those are crap */
-	weight += pfd.cDepthBits - 16;
-
-	weight += pfd.cColorBits - 8;
-
-	/* want swap copy capability -- it matters a lot */
-	if (pfd.dwFlags & PFD_SWAP_COPY) weight += 16;
-
-	/* but if it's a generic (not accelerated) view, it's really bad */
-	if (pfd.dwFlags & PFD_GENERIC_FORMAT) weight /= 10;
-
-	return weight;
-}
-
-/* A modification of Ron Fosner's replacement for ChoosePixelFormat */
-/* returns 0 on error, else returns the pixel format number to be used */
-static int EnumPixelFormats(HDC hdc)
-{
-	int iPixelFormat;
-	int i, n, w, weight = 0;
-	PIXELFORMATDESCRIPTOR pfd;
-
-	/* we need a device context to do anything */
-	if (!hdc) return 0;
-
-	iPixelFormat = 1; /* careful! PFD numbers are 1 based, not zero based */
-
-	/* obtain detailed information about
-	 * the device context's first pixel format */
-	n = 1 + ::DescribePixelFormat(hdc, iPixelFormat,
-	                              sizeof(PIXELFORMATDESCRIPTOR), &pfd);
-
-	/* choose a pixel format using the useless Windows function in case
-	 * we come up empty handed */
-	iPixelFormat = ::ChoosePixelFormat(hdc, &sPreferredFormat);
-
-	if (!iPixelFormat) return 0;  /* couldn't find one to use */
-
-	for (i = 1; i <= n; i++) { /* not the idiom, but it's right */
-		::DescribePixelFormat(hdc, i, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
-		w = WeightPixelFormat(pfd);
-		// be strict on stereo
-		if (!((sPreferredFormat.dwFlags ^ pfd.dwFlags) & PFD_STEREO)) {
-			if (w > weight) {
-				weight = w;
-				iPixelFormat = i;
-			}
-		}
-	}
-	if (weight == 0) {
-		// we could find the correct stereo setting, just find any suitable format
-		for (i = 1; i <= n; i++) { /* not the idiom, but it's right */
-			::DescribePixelFormat(hdc, i, sizeof(PIXELFORMATDESCRIPTOR), &pfd);
-			w = WeightPixelFormat(pfd);
-			if (w > weight) {
-				weight = w;
-				iPixelFormat = i;
-			}
-		}
-	}
-	return iPixelFormat;
+	this->getImeInput()->BeginIME(this->getHWND(),	GHOST_Rect(x, y - h , x, y), (bool)completed);
 }
 
 
+void GHOST_WindowWin32::endIME()
+{
+	this->getImeInput()->EndIME(this->getHWND());
+}
+#endif /* WITH_INPUT_IME */
