@@ -39,11 +39,9 @@
 
 #include "GHOST_Window.h"
 #include "GHOST_TaskbarWin32.h"
-
-#define _WIN32_WINNT 0x501 // require Windows XP or newer
-#define WIN32_LEAN_AND_MEAN
-#include <windows.h>
-
+#ifdef WITH_INPUT_IME
+#  include "GHOST_ImeWin32.h"
+#endif
 
 #include <wintab.h>
 #define PACKETDATA  (PK_BUTTONS | PK_NORMAL_PRESSURE | PK_ORIENTATION | PK_CURSOR)
@@ -58,6 +56,7 @@ typedef UINT (API * GHOST_WIN32_WTInfo)(UINT, UINT, LPVOID);
 typedef HCTX (API * GHOST_WIN32_WTOpen)(HWND, LPLOGCONTEXTA, BOOL);
 typedef BOOL (API * GHOST_WIN32_WTClose)(HCTX);
 typedef BOOL (API * GHOST_WIN32_WTPacket)(HCTX, UINT, LPVOID);
+typedef BOOL (API * GHOST_WIN32_WTOverlap)(HCTX, BOOL);
 
 /**
  * GHOST window on M$ Windows OSs.
@@ -77,8 +76,9 @@ public:
 	 * \param height	The height the window.
 	 * \param state		The state the window is initially opened with.
 	 * \param type		The type of drawing context installed in this window.
-	 * \param stereoVisual	Stereo visual for quad buffered stereo.
-	 * \param numOfAASamples	Number of samples used for AA (zero if no AA)
+	 * \param wantStereoVisual   Stereo visual for quad buffered stereo.
+	 * \param wantNumOfAASamples Number of samples used for AA (zero if no AA)
+	 * \param parentWindowHwnd
 	 */
 	GHOST_WindowWin32(
 	    GHOST_SystemWin32 *system,
@@ -89,11 +89,9 @@ public:
 	    GHOST_TUns32 height,
 	    GHOST_TWindowState state,
 	    GHOST_TDrawingContextType type = GHOST_kDrawingContextTypeNone,
-	    const bool stereoVisual = false,
-	    const GHOST_TUns16 numOfAASamples = 0,
-	    GHOST_TEmbedderWindowID parentWindowHwnd = 0,
-	    GHOST_TSuccess msEnabled = GHOST_kFailure,
-	    int msPixelFormat = 0
+	    bool wantStereoVisual = false,
+	    GHOST_TUns16 wantNumOfAASamples = 0,
+	    GHOST_TEmbedderWindowID parentWindowHwnd = 0
 	    );
 
 	/**
@@ -102,13 +100,6 @@ public:
 	 */
 	virtual ~GHOST_WindowWin32();
 
-	/**
-	 * Returns the window to replace this one if it's getting replaced
-	 * \return The window replacing this one.
-	 */
-
-	GHOST_Window *getNextWindow();
-
 	/**
 	 * Returns indication as to whether the window is valid.
 	 * \return The validity of the window.
@@ -143,7 +134,7 @@ public:
 	/**
 	 * Returns the client rectangle dimensions.
 	 * The left and top members of the rectangle are always zero.
-	 * \param bounds The bounding rectangle of the cleient area of the window.
+	 * \param bounds The bounding rectangle of the client area of the window.
 	 */
 	virtual void getClientBounds(GHOST_Rect& bounds) const;
 
@@ -205,16 +196,6 @@ public:
 	virtual GHOST_TSuccess setOrder(GHOST_TWindowOrder order);
 
 	/**
-	 * Swaps front and back buffers of a window.
-	 * \return Indication of success.
-	 */
-	virtual GHOST_TSuccess swapBuffers();
-
-	/**
-	 * Activates the drawing context of this window.
-	 * \return Indication of success.
-	 */
-	virtual GHOST_TSuccess activateDrawingContext();
 
 	/**
 	 * Invalidates the contents of this window.
@@ -232,13 +213,6 @@ public:
 	 */
 	virtual GHOST_TSuccess endProgressBar();
 	
-	/**
-	 * Returns the name of the window class.
-	 * \return The name of the window class.
-	 */
-	static const wchar_t *getWindowClassName() {
-		return s_windowClassName;
-	}
 
 	/**
 	 * Register a mouse click event (should be called 
@@ -273,22 +247,33 @@ public:
 
 	void processWin32TabletInitEvent();
 	void processWin32TabletEvent(WPARAM wParam, LPARAM lParam);
+	void bringTabletContextToFront();
 
-protected:
-	GHOST_TSuccess initMultisample(PIXELFORMATDESCRIPTOR pfd);
+	GHOST_TSuccess beginFullScreen() const {return GHOST_kFailure;}
+
+	GHOST_TSuccess endFullScreen() const {return GHOST_kFailure;}
+
+	/** if the window currently resizing */
+	bool m_inLiveResize;
+
+#ifdef WITH_INPUT_IME
+	GHOST_ImeWin32 *getImeInput() {return &m_imeImput;}
+
+	virtual void beginIME(
+	        GHOST_TInt32 x, GHOST_TInt32 y,
+	        GHOST_TInt32 w, GHOST_TInt32 h,
+	        int completed);
+
+	virtual void endIME();
+#endif /* WITH_INPUT_IME */
+
+private:
 
 	/**
-	 * Tries to install a rendering context in this window.
-	 * \param type	The type of rendering context installed.
+	 * \param type	The type of rendering context create.
 	 * \return Indication of success.
 	 */
-	virtual GHOST_TSuccess installDrawingContext(GHOST_TDrawingContextType type);
-
-	/**
-	 * Removes the current drawing context.
-	 * \return Indication of success.
-	 */
-	virtual GHOST_TSuccess removeDrawingContext();
+	virtual GHOST_Context *newDrawingContext(GHOST_TDrawingContextType type);
 
 	/**
 	 * Sets the cursor visibility on the window using
@@ -336,16 +321,11 @@ protected:
 	HWND m_hWnd;
 	/** Device context handle. */
 	HDC m_hDC;
-	/** OpenGL rendering context. */
-	HGLRC m_hGlRc;
-	/** The first created OpenGL context (for sharing display lists) */
-	static HGLRC s_firsthGLRc;
-	/** The first created device context handle. */
-	static HDC s_firstHDC;
+
 	/** Flag for if window has captured the mouse */
 	bool m_hasMouseCaptured;
 	/** Flag if an operator grabs the mouse with WM_cursor_grab_enable/ungrab() 
-	 * Multiple grabs must be realesed with a single ungrab*/
+	 * Multiple grabs must be released with a single ungrab */
 	bool m_hasGrabMouse;
 	/** Count of number of pressed buttons */
 	int m_nPressedButtons;
@@ -369,29 +349,15 @@ protected:
 	LONG m_maxPressure;
 	LONG m_maxAzimuth, m_maxAltitude;
 
-	/** Preferred number of samples */
-	GHOST_TUns16 m_multisample;
-
-	/** Check if multisample is supported */
-	GHOST_TSuccess m_multisampleEnabled;
-
-	/** The pixelFormat to use for multisample */
-	int m_msPixelFormat;
-
-	/** We need to following to recreate the window */
-	const STR_String& m_title;
-	GHOST_TInt32 m_left;
-	GHOST_TInt32 m_top;
-	GHOST_TUns32 m_width;
-	GHOST_TUns32 m_height;
 	GHOST_TWindowState m_normal_state;
-	bool m_stereo;
-
-	/** The GHOST_System passes this to wm if this window is being replaced */
-	GHOST_Window *m_nextWindow;
 
 	/** Hwnd to parent window */
 	GHOST_TEmbedderWindowID m_parentWindowHwnd;
+
+#ifdef WITH_INPUT_IME
+	/** Handle input method editors event */
+	GHOST_ImeWin32 m_imeImput;
+#endif
 };
 
 #endif // __GHOST_WINDOWWIN32_H__
