@@ -48,6 +48,7 @@ struct RenderData;
 struct SceneRenderLayer;
 struct Scene;
 struct Text;
+struct UnitSettings;
 struct Main;
 
 #define SCE_COPY_NEW        0
@@ -84,6 +85,7 @@ typedef struct SceneBaseIter {
 	struct ListBase *duplilist;
 	struct DupliObject *dupob;
 	float omat[4][4];
+	struct Object *dupli_refob;
 	int phase;
 } SceneBaseIter;
 
@@ -108,7 +110,7 @@ char *BKE_scene_find_marker_name(struct Scene *scene, int frame);
 char *BKE_scene_find_last_marker_name(struct Scene *scene, int frame);
 
 /* checks for cycle, returns 1 if it's all OK */
-int BKE_scene_validate_setscene(struct Main *bmain, struct Scene *sce);
+bool BKE_scene_validate_setscene(struct Main *bmain, struct Scene *sce);
 
 float BKE_scene_frame_get(struct Scene *scene);
 float BKE_scene_frame_get_from_ctime(struct Scene *scene, const float frame);
@@ -130,6 +132,9 @@ float get_render_aosss_error(struct RenderData *r, float error);
 
 bool BKE_scene_use_new_shading_nodes(struct Scene *scene);
 
+bool BKE_scene_uses_blender_internal(struct Scene *scene);
+bool BKE_scene_uses_blender_game(struct Scene *scene);
+
 void BKE_scene_disable_color_management(struct Scene *scene);
 bool BKE_scene_check_color_management_enabled(const struct Scene *scene);
 bool BKE_scene_check_rigidbody_active(const struct Scene *scene);
@@ -137,6 +142,8 @@ bool BKE_scene_check_rigidbody_active(const struct Scene *scene);
 int BKE_scene_num_threads(const struct Scene *scene);
 int BKE_render_num_threads(const struct RenderData *r);
 
+double BKE_scene_unit_scale(const struct UnitSettings *unit, const int unit_type, double value);
+
 #ifdef __cplusplus
 }
 #endif
