@@ -43,23 +43,24 @@
 #include "DNA_object_types.h"
 
 #include "BLI_blenlib.h"
-#include "BLI_bpath.h"
 #include "BLI_math.h"
 #include "BLI_utildefines.h"
 #include "BLI_ghash.h"
 
-#include "BKE_animsys.h"
+#include "BLF_translation.h"
+
 #include "BKE_action.h"
 #include "BKE_anim.h"
+#include "BKE_animsys.h"
 #include "BKE_constraint.h"
-#include "BKE_global.h"
+#include "BKE_deform.h"
 #include "BKE_fcurve.h"
+#include "BKE_global.h"
+#include "BKE_idprop.h"
 #include "BKE_library.h"
 #include "BKE_main.h"
 #include "BKE_object.h"
 
-#include "BKE_idprop.h"
-
 #include "BIK_api.h"
 
 #include "RNA_access.h"
@@ -79,11 +80,11 @@
 
 /* ***************** Library data level operations on action ************** */
 
-bAction *add_empty_action(const char name[])
+bAction *add_empty_action(Main *bmain, const char name[])
 {
 	bAction *act;
 	
-	act = BKE_libblock_alloc(&G.main->action, ID_AC, name);
+	act = BKE_libblock_alloc(bmain, ID_AC, name);
 	
 	return act;
 }	
@@ -95,8 +96,8 @@ typedef struct tMakeLocalActionContext {
 	bAction *act;       /* original action */
 	bAction *act_new;   /* new action */
 	
-	int is_lib;         /* some action users were libraries */
-	int is_local;       /* some action users were not libraries */
+	bool is_lib;        /* some action users were libraries */
+	bool is_local;      /* some action users were not libraries */
 } tMakeLocalActionContext;
 
 /* helper function for BKE_action_make_local() - local/lib init step */
@@ -105,8 +106,8 @@ static void make_localact_init_cb(ID *id, AnimData *adt, void *mlac_ptr)
 	tMakeLocalActionContext *mlac = (tMakeLocalActionContext *)mlac_ptr;
 	
 	if (adt->action == mlac->act) {
-		if (id->lib) mlac->is_lib = TRUE;
-		else mlac->is_local = TRUE;
+		if (id->lib) mlac->is_lib = true;
+		else mlac->is_local = true;
 	}
 }
 
@@ -128,7 +129,7 @@ static void make_localact_apply_cb(ID *id, AnimData *adt, void *mlac_ptr)
 // does copy_fcurve...
 void BKE_action_make_local(bAction *act)
 {
-	tMakeLocalActionContext mlac = {act, NULL, FALSE, FALSE};
+	tMakeLocalActionContext mlac = {act, NULL, false, false};
 	Main *bmain = G.main;
 	
 	if (act->id.lib == NULL)
@@ -142,7 +143,7 @@ void BKE_action_make_local(bAction *act)
 	
 	BKE_animdata_main_cb(bmain, make_localact_init_cb, &mlac);
 	
-	if (mlac.is_local && mlac.is_lib == FALSE) {
+	if (mlac.is_local && mlac.is_lib == false) {
 		id_clear_lib_data(bmain, &act->id);
 	}
 	else if (mlac.is_local && mlac.is_lib) {
@@ -192,7 +193,7 @@ bAction *BKE_action_copy(bAction *src)
 	BLI_duplicatelist(&dst->markers, &src->markers);
 	
 	/* copy F-Curves, fixing up the links as we go */
-	dst->curves.first = dst->curves.last = NULL;
+	BLI_listbase_clear(&dst->curves);
 	
 	for (sfcu = src->curves.first; sfcu; sfcu = sfcu->next) {
 		/* duplicate F-Curve */
@@ -224,7 +225,7 @@ bActionGroup *get_active_actiongroup(bAction *act)
 {
 	bActionGroup *agrp = NULL;
 	
-	if (act && act->groups.first) {	
+	if (act && act->groups.first) {
 		for (agrp = act->groups.first; agrp; agrp = agrp->next) {
 			if (agrp->flag & AGRP_ACTIVE)
 				break;
@@ -297,11 +298,11 @@ bActionGroup *action_groups_add_new(bAction *act, const char name[])
 	
 	/* make it selected, with default name */
 	agrp->flag = AGRP_SELECTED;
-	BLI_strncpy(agrp->name, name[0] ? name : "Group", sizeof(agrp->name));
+	BLI_strncpy(agrp->name, name[0] ? name : DATA_("Group"), sizeof(agrp->name));
 	
 	/* add to action, and validate */
 	BLI_addtail(&act->groups, agrp);
-	BLI_uniquename(&act->groups, agrp, "Group", '.', offsetof(bActionGroup, name), sizeof(agrp->name));	
+	BLI_uniquename(&act->groups, agrp, DATA_("Group"), '.', offsetof(bActionGroup, name), sizeof(agrp->name));
 	
 	/* return the new group */
 	return agrp;
@@ -314,11 +315,11 @@ bActionGroup *action_groups_add_new(bAction *act, const char name[])
 void action_groups_add_channel(bAction *act, bActionGroup *agrp, FCurve *fcurve)
 {	
 	/* sanity checks */
-	if (ELEM3(NULL, act, agrp, fcurve))
+	if (ELEM(NULL, act, agrp, fcurve))
 		return;
 	
 	/* if no channels anywhere, just add to two lists at the same time */
-	if (act->curves.first == NULL) {
+	if (BLI_listbase_is_empty(&act->curves)) {
 		fcurve->next = fcurve->prev = NULL;
 		
 		agrp->channels.first = agrp->channels.last = fcurve;
@@ -380,7 +381,7 @@ void action_groups_add_channel(bAction *act, bActionGroup *agrp, FCurve *fcurve)
 void action_groups_remove_channel(bAction *act, FCurve *fcu)
 {
 	/* sanity checks */
-	if (ELEM(NULL, act, fcu))	
+	if (ELEM(NULL, act, fcu))
 		return;
 	
 	/* check if any group used this directly */
@@ -389,8 +390,7 @@ void action_groups_remove_channel(bAction *act, FCurve *fcu)
 		
 		if (agrp->channels.first == agrp->channels.last) {
 			if (agrp->channels.first == fcu) {
-				agrp->channels.first = NULL;
-				agrp->channels.last = NULL;
+				BLI_listbase_clear(&agrp->channels);
 			}
 		}
 		else if (agrp->channels.first == fcu) {
@@ -417,7 +417,7 @@ void action_groups_remove_channel(bAction *act, FCurve *fcu)
 bActionGroup *BKE_action_group_find_name(bAction *act, const char name[])
 {
 	/* sanity checks */
-	if (ELEM3(NULL, act, act->groups.first, name) || (name[0] == 0))
+	if (ELEM(NULL, act, act->groups.first, name) || (name[0] == 0))
 		return NULL;
 		
 	/* do string comparisons */
@@ -440,6 +440,10 @@ void action_groups_clear_tempflags(bAction *act)
 
 /* *************** Pose channels *************** */
 
+/**
+ * Return a pointer to the pose channel of the given name
+ * from this pose.
+ */
 bPoseChannel *BKE_pose_channel_find_name(const bPose *pose, const char *name)
 {
 	if (ELEM(NULL, pose, name) || (name[0] == '\0'))
@@ -451,8 +455,14 @@ bPoseChannel *BKE_pose_channel_find_name(const bPose *pose, const char *name)
 	return BLI_findstring(&((bPose *)pose)->chanbase, name, offsetof(bPoseChannel, name));
 }
 
-/* Use with care, not on Armature poses but for temporal ones */
-/* (currently used for action constraints and in rebuild_pose) */
+/**
+ * Looks to see if the channel with the given name
+ * already exists in this pose - if not a new one is
+ * allocated and initialized.
+ *
+ * \note Use with care, not on Armature poses but for temporal ones.
+ * \note (currently used for action constraints and in rebuild_pose).
+ */
 bPoseChannel *BKE_pose_channel_verify(bPose *pose, const char *name)
 {
 	bPoseChannel *chan;
@@ -489,13 +499,34 @@ bPoseChannel *BKE_pose_channel_verify(bPose *pose, const char *name)
 	return chan;
 }
 
-/* Find the active posechannel for an object (we can't just use pose, as layer info is in armature) */
+#ifndef NDEBUG
+bool BKE_pose_channels_is_valid(const bPose *pose)
+{
+	if (pose->chanhash) {
+		bPoseChannel *pchan;
+		for (pchan = pose->chanbase.first; pchan; pchan = pchan->next) {
+			if (BLI_ghash_lookup(pose->chanhash, pchan->name) != pchan) {
+				return false;
+			}
+		}
+	}
+
+	return true;
+}
+
+#endif
+
+/**
+ * Find the active posechannel for an object (we can't just use pose, as layer info is in armature)
+ *
+ * \note: Object, not bPose is used here, as we need layer info from Armature)
+ */
 bPoseChannel *BKE_pose_channel_active(Object *ob)
 {
 	bArmature *arm = (ob) ? ob->data : NULL;
 	bPoseChannel *pchan;
 
-	if (ELEM3(NULL, ob, ob->pose, arm)) {
+	if (ELEM(NULL, ob, ob->pose, arm)) {
 		return NULL;
 	}
 
@@ -508,11 +539,27 @@ bPoseChannel *BKE_pose_channel_active(Object *ob)
 	return NULL;
 }
 
+/**
+ * \see #ED_armature_bone_get_mirrored (edit-mode, matching function)
+ */
+bPoseChannel *BKE_pose_channel_get_mirrored(const bPose *pose, const char *name)
+{
+	char name_flip[MAXBONENAME];
+
+	BKE_deform_flip_side_name(name_flip, name, false);
+
+	if (!STREQ(name_flip, name)) {
+		return BKE_pose_channel_find_name(pose, name_flip);
+	}
+
+	return NULL;
+}
+
 const char *BKE_pose_ikparam_get_name(bPose *pose)
 {
 	if (pose) {
 		switch (pose->iksolver) {
-			case IKSOLVER_LEGACY:
+			case IKSOLVER_STANDARD:
 				return NULL;
 			case IKSOLVER_ITASC:
 				return "bItasc";
@@ -520,8 +567,14 @@ const char *BKE_pose_ikparam_get_name(bPose *pose)
 	}
 	return NULL;
 }
-/* dst should be freed already, makes entire duplicate */
-void BKE_pose_copy_data(bPose **dst, bPose *src, int copycon)
+
+/**
+ * Allocate a new pose on the heap, and copy the src pose and it's channels
+ * into the new pose. *dst is set to the newly allocated structure, and assumed to be NULL.
+ *
+ * \param dst  Should be freed already, makes entire duplicate.
+ */
+void BKE_pose_copy_data(bPose **dst, bPose *src, const bool copy_constraints)
 {
 	bPose *outPose;
 	bPoseChannel *pchan;
@@ -535,16 +588,25 @@ void BKE_pose_copy_data(bPose **dst, bPose *src, int copycon)
 	outPose = MEM_callocN(sizeof(bPose), "pose");
 	
 	BLI_duplicatelist(&outPose->chanbase, &src->chanbase);
-	
+
 	outPose->iksolver = src->iksolver;
 	outPose->ikdata = NULL;
 	outPose->ikparam = MEM_dupallocN(src->ikparam);
 	outPose->avs = src->avs;
 	
 	for (pchan = outPose->chanbase.first; pchan; pchan = pchan->next) {
-		/* TODO: rename this argument... */
-		if (copycon) {
-			copy_constraints(&listb, &pchan->constraints, TRUE);  // copy_constraints NULLs listb
+
+		if (pchan->custom) {
+			id_us_plus(&pchan->custom->id);
+		}
+
+		/* warning, O(n2) here, but it's a rarely used feature. */
+		if (pchan->custom_tx) {
+			pchan->custom_tx = BKE_pose_channel_find_name(outPose, pchan->custom_tx->name);
+		}
+
+		if (copy_constraints) {
+			BKE_constraints_copy(&listb, &pchan->constraints, true);  // BKE_constraints_copy NULLs listb
 			pchan->constraints = listb;
 			pchan->mpath = NULL; /* motion paths should not get copied yet... */
 		}
@@ -555,8 +617,9 @@ void BKE_pose_copy_data(bPose **dst, bPose *src, int copycon)
 	}
 
 	/* for now, duplicate Bone Groups too when doing this */
-	if (copycon)
+	if (copy_constraints) {
 		BLI_duplicatelist(&outPose->agroups, &src->agroups);
+	}
 	
 	*dst = outPose;
 }
@@ -587,18 +650,53 @@ void BKE_pose_ikparam_init(bPose *pose)
 			BKE_pose_itasc_init(itasc);
 			pose->ikparam = itasc;
 			break;
-		case IKSOLVER_LEGACY:
+		case IKSOLVER_STANDARD:
 		default:
 			pose->ikparam = NULL;
 			break;
 	}
 }
 
+
+/* only for real IK, not for auto-IK */
+static bool pose_channel_in_IK_chain(Object *ob, bPoseChannel *pchan, int level)
+{
+	bConstraint *con;
+	Bone *bone;
+	
+	/* No need to check if constraint is active (has influence),
+	 * since all constraints with CONSTRAINT_IK_AUTO are active */
+	for (con = pchan->constraints.first; con; con = con->next) {
+		if (con->type == CONSTRAINT_TYPE_KINEMATIC) {
+			bKinematicConstraint *data = con->data;
+			if ((data->rootbone == 0) || (data->rootbone > level)) {
+				if ((data->flag & CONSTRAINT_IK_AUTO) == 0)
+					return true;
+			}
+		}
+	}
+	for (bone = pchan->bone->childbase.first; bone; bone = bone->next) {
+		pchan = BKE_pose_channel_find_name(ob->pose, bone->name);
+		if (pchan && pose_channel_in_IK_chain(ob, pchan, level + 1))
+			return true;
+	}
+	return false;
+}
+
+bool BKE_pose_channel_in_IK_chain(Object *ob, bPoseChannel *pchan)
+{
+	return pose_channel_in_IK_chain(ob, pchan, 0);
+}
+
+/**
+ * Removes the hash for quick lookup of channels, must
+ * be done when adding/removing channels.
+ */
 void BKE_pose_channels_hash_make(bPose *pose) 
 {
 	if (!pose->chanhash) {
 		bPoseChannel *pchan;
-
+		
 		pose->chanhash = BLI_ghash_str_new("make_pose_chan gh");
 		for (pchan = pose->chanbase.first; pchan; pchan = pchan->next)
 			BLI_ghash_insert(pose->chanhash, pchan->name, pchan);
@@ -613,16 +711,25 @@ void BKE_pose_channels_hash_free(bPose *pose)
 	}
 }
 
-
-void BKE_pose_channel_free(bPoseChannel *pchan)
+/**
+ * Deallocates a pose channel.
+ * Does not free the pose channel itself.
+ */
+void BKE_pose_channel_free_ex(bPoseChannel *pchan, bool do_id_user)
 {
+	if (pchan->custom) {
+		if (do_id_user) {
+			id_us_min(&pchan->custom->id);
+		}
+		pchan->custom = NULL;
+	}
 
 	if (pchan->mpath) {
 		animviz_free_motionpath(pchan->mpath);
 		pchan->mpath = NULL;
 	}
 
-	free_constraints(&pchan->constraints);
+	BKE_constraints_free(&pchan->constraints);
 	
 	if (pchan->prop) {
 		IDP_FreeProperty(pchan->prop);
@@ -630,13 +737,22 @@ void BKE_pose_channel_free(bPoseChannel *pchan)
 	}
 }
 
-void BKE_pose_channels_free(bPose *pose) 
+void BKE_pose_channel_free(bPoseChannel *pchan)
+{
+	BKE_pose_channel_free_ex(pchan, true);
+}
+
+/**
+ * Removes and deallocates all channels from a pose.
+ * Does not free the pose itself.
+ */
+void BKE_pose_channels_free_ex(bPose *pose, bool do_id_user)
 {
 	bPoseChannel *pchan;
 	
 	if (pose->chanbase.first) {
 		for (pchan = pose->chanbase.first; pchan; pchan = pchan->next)
-			BKE_pose_channel_free(pchan);
+			BKE_pose_channel_free_ex(pchan, do_id_user);
 		
 		BLI_freelistN(&pose->chanbase);
 	}
@@ -644,11 +760,19 @@ void BKE_pose_channels_free(bPose *pose)
 	BKE_pose_channels_hash_free(pose);
 }
 
-void BKE_pose_free(bPose *pose)
+void BKE_pose_channels_free(bPose *pose)
+{
+	BKE_pose_channels_free_ex(pose, true);
+}
+
+/**
+ * Removes and deallocates all data from a pose, and also frees the pose.
+ */
+void BKE_pose_free_ex(bPose *pose, bool do_id_user)
 {
 	if (pose) {
 		/* free pose-channels */
-		BKE_pose_channels_free(pose);
+		BKE_pose_channels_free_ex(pose, do_id_user);
 		
 		/* free pose-groups */
 		if (pose->agroups.first)
@@ -666,6 +790,11 @@ void BKE_pose_free(bPose *pose)
 	}
 }
 
+void BKE_pose_free(bPose *pose)
+{
+	BKE_pose_free_ex(pose, true);
+}
+
 static void copy_pose_channel_data(bPoseChannel *pchan, const bPoseChannel *chan)
 {
 	bConstraint *pcon, *con;
@@ -688,9 +817,13 @@ static void copy_pose_channel_data(bPoseChannel *pchan, const bPoseChannel *chan
 	}
 }
 
-/* makes copies of internal data, unlike copy_pose_channel_data which only
- * copies the pose state.
- * hint: use when copying bones in editmode (on returned value from BKE_pose_channel_verify) */
+/**
+ * Copy the internal members of each pose channel including constraints
+ * and ID-Props, used when duplicating bones in editmode.
+ * (unlike copy_pose_channel_data which only).
+ *
+ * \note use when copying bones in editmode (on returned value from #BKE_pose_channel_verify)
+ */
 void BKE_pose_channel_copy_data(bPoseChannel *pchan, const bPoseChannel *pchan_from)
 {
 	/* copy transform locks */
@@ -712,7 +845,7 @@ void BKE_pose_channel_copy_data(bPoseChannel *pchan, const bPoseChannel *pchan_f
 	pchan->iklinweight = pchan_from->iklinweight;
 
 	/* constraints */
-	copy_constraints(&pchan->constraints, &pchan_from->constraints, TRUE);
+	BKE_constraints_copy(&pchan->constraints, &pchan_from->constraints, true);
 
 	/* id-properties */
 	if (pchan->prop) {
@@ -727,6 +860,9 @@ void BKE_pose_channel_copy_data(bPoseChannel *pchan, const bPoseChannel *pchan_f
 
 	/* custom shape */
 	pchan->custom = pchan_from->custom;
+	if (pchan->custom) {
+		id_us_plus(&pchan->custom->id);
+	}
 }
 
 
@@ -816,63 +952,75 @@ void framechange_poses_clear_unkeyed(void)
 
 /* ************************** Bone Groups ************************** */
 
-/* Adds a new bone-group */
-void BKE_pose_add_group(Object *ob)
+/* Adds a new bone-group (name may be NULL) */
+bActionGroup *BKE_pose_add_group(bPose *pose, const char *name)
 {
-	bPose *pose = (ob) ? ob->pose : NULL;
 	bActionGroup *grp;
 	
-	if (ELEM(NULL, ob, ob->pose))
-		return;
+	if (!name) {
+		name = DATA_("Group");
+	}
 	
 	grp = MEM_callocN(sizeof(bActionGroup), "PoseGroup");
-	BLI_strncpy(grp->name, "Group", sizeof(grp->name));
+	BLI_strncpy(grp->name, name, sizeof(grp->name));
 	BLI_addtail(&pose->agroups, grp);
-	BLI_uniquename(&pose->agroups, grp, "Group", '.', offsetof(bActionGroup, name), sizeof(grp->name));
+	BLI_uniquename(&pose->agroups, grp, name, '.', offsetof(bActionGroup, name), sizeof(grp->name));
 	
-	pose->active_group = BLI_countlist(&pose->agroups);
+	pose->active_group = BLI_listbase_count(&pose->agroups);
+	
+	return grp;
 }
 
-/* Remove the active bone-group */
-void BKE_pose_remove_group(Object *ob)
+/* Remove the given bone-group (expects 'virtual' index (+1 one, used by active_group etc.))
+ * index might be invalid ( < 1), in which case it will be find from grp. */
+void BKE_pose_remove_group(bPose *pose, bActionGroup *grp, const int index)
 {
-	bPose *pose = (ob) ? ob->pose : NULL;
-	bActionGroup *grp = NULL;
 	bPoseChannel *pchan;
+	int idx = index;
 	
-	/* sanity checks */
-	if (ELEM(NULL, ob, pose))
-		return;
-	if (pose->active_group <= 0)
-		return;
+	if (idx < 1) {
+		idx = BLI_findindex(&pose->agroups, grp) + 1;
+	}
 	
-	/* get group to remove */
-	grp = BLI_findlink(&pose->agroups, pose->active_group - 1);
-	if (grp) {
-		/* adjust group references (the trouble of using indices!):
-		 *	- firstly, make sure nothing references it 
-		 *	- also, make sure that those after this item get corrected
-		 */
-		for (pchan = pose->chanbase.first; pchan; pchan = pchan->next) {
-			if (pchan->agrp_index == pose->active_group)
-				pchan->agrp_index = 0;
-			else if (pchan->agrp_index > pose->active_group)
-				pchan->agrp_index--;
-		}
-		
-		/* now, remove it from the pose */
-		BLI_freelinkN(&pose->agroups, grp);
+	BLI_assert(idx > 0);
+	
+	/* adjust group references (the trouble of using indices!):
+	 *  - firstly, make sure nothing references it
+	 *  - also, make sure that those after this item get corrected
+	 */
+	for (pchan = pose->chanbase.first; pchan; pchan = pchan->next) {
+		if (pchan->agrp_index == idx)
+			pchan->agrp_index = 0;
+		else if (pchan->agrp_index > idx)
+			pchan->agrp_index--;
+	}
+
+	/* now, remove it from the pose */
+	BLI_freelinkN(&pose->agroups, grp);
+	if (pose->active_group >= idx) {
 		pose->active_group--;
-		if (pose->active_group < 0 || pose->agroups.first == NULL) {
+		if (pose->active_group < 0 || BLI_listbase_is_empty(&pose->agroups)) {
 			pose->active_group = 0;
 		}
 	}
 }
 
+/* Remove the indexed bone-group (expects 'virtual' index (+1 one, used by active_group etc.)) */
+void BKE_pose_remove_group_index(bPose *pose, const int index)
+{
+	bActionGroup *grp = NULL;
+	
+	/* get group to remove */
+	grp = BLI_findlink(&pose->agroups, index - 1);
+	if (grp) {
+		BKE_pose_remove_group(pose, grp, index);
+	}
+}
+
 /* ************** F-Curve Utilities for Actions ****************** */
 
 /* Check if the given action has any keyframes */
-short action_has_motion(const bAction *act)
+bool action_has_motion(const bAction *act)
 {
 	FCurve *fcu;
 	
@@ -880,12 +1028,12 @@ short action_has_motion(const bAction *act)
 	if (act) {
 		for (fcu = act->curves.first; fcu; fcu = fcu->next) {
 			if (fcu->totvert)
-				return 1;
+				return true;
 		}
 	}
 	
 	/* nothing found */
-	return 0;
+	return false;
 }
 
 /* Calculate the extents of given action */
@@ -903,11 +1051,11 @@ void calc_action_range(const bAction *act, float *start, float *end, short incl_
 				
 				/* get extents for this curve */
 				/* TODO: allow enabling/disabling this? */
-				calc_fcurve_range(fcu, &nmin, &nmax, FALSE, TRUE);
+				calc_fcurve_range(fcu, &nmin, &nmax, false, true);
 				
 				/* compare to the running tally */
-				min = MIN2(min, nmin);
-				max = MAX2(max, nmax);
+				min = min_ff(min, nmin);
+				max = max_ff(max, nmax);
 				
 				foundvert = 1;
 			}
@@ -925,14 +1073,13 @@ void calc_action_range(const bAction *act, float *start, float *end, short incl_
 						FMod_Limits *fmd = (FMod_Limits *)fcm->data;
 						
 						if (fmd->flag & FCM_LIMIT_XMIN) {
-							min = MIN2(min, fmd->rect.xmin);
+							min = min_ff(min, fmd->rect.xmin);
 						}
 						if (fmd->flag & FCM_LIMIT_XMAX) {
-							max = MAX2(max, fmd->rect.xmax);
+							max = max_ff(max, fmd->rect.xmax);
 						}
+						break;
 					}
-					break;
-						
 					case FMODIFIER_TYPE_CYCLES: /* Cycles F-Modifier */
 					{
 						FMod_Cycles *fmd = (FMod_Cycles *)fcm->data;
@@ -941,9 +1088,8 @@ void calc_action_range(const bAction *act, float *start, float *end, short incl_
 							min = MINAFRAMEF;
 						if (fmd->after_mode != FCM_EXTRAPOLATE_NONE)
 							max = MAXFRAMEF;
+						break;
 					}
-					break;
-						
 					/* TODO: function modifier may need some special limits */
 						
 					default: /* all other standard modifiers are on the infinite range... */
@@ -955,7 +1101,7 @@ void calc_action_range(const bAction *act, float *start, float *end, short incl_
 				foundmod = 1;
 			}
 		}
-	}	
+	}
 	
 	if (foundvert || foundmod) {
 		if (min == max) max += 1.0f;
@@ -995,7 +1141,7 @@ short action_get_item_transforms(bAction *act, Object *ob, bPoseChannel *pchan,
 	 *	- we cannot use the groups, since they may not be grouped in that way...
 	 */
 	for (fcu = act->curves.first; fcu; fcu = fcu->next) {
-		char *bPtr = NULL, *pPtr = NULL;
+		const char *bPtr = NULL, *pPtr = NULL;
 		
 		/* if enough flags have been found, we can stop checking unless we're also getting the curves */
 		if ((flags == ACT_TRANS_ALL) && (curves == NULL))
@@ -1119,18 +1265,18 @@ void BKE_pose_rest(bPose *pose)
 }
 
 /* both poses should be in sync */
-void BKE_pose_copy_result(bPose *to, bPose *from)
+bool BKE_pose_copy_result(bPose *to, bPose *from)
 {
 	bPoseChannel *pchanto, *pchanfrom;
 	
 	if (to == NULL || from == NULL) {
-		printf("pose result copy error to:%p from:%p\n", (void *)to, (void *)from); /* debug temp */
-		return;
+		printf("Pose copy error, pose to:%p from:%p\n", (void *)to, (void *)from); /* debug temp */
+		return false;
 	}
 
 	if (to == from) {
 		printf("BKE_pose_copy_result source and target are the same\n");
-		return;
+		return false;
 	}
 
 
@@ -1154,6 +1300,7 @@ void BKE_pose_copy_result(bPose *to, bPose *from)
 			pchanto->protectflag = pchanfrom->protectflag;
 		}
 	}
+	return true;
 }
 
 /* For the calculation of the effects of an Action at the given frame on an object 
