@@ -15,9 +15,6 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
- * All rights reserved.
- *
  * Contributor(s): Blender Foundation (2008).
  *
  * ***** END GPL LICENSE BLOCK *****
@@ -27,8 +24,8 @@
  *  \ingroup bke
  */
 
-
 #include <string.h>
+#include <stdlib.h>
 #include <stddef.h>
 
 #include "MEM_guardedalloc.h"
@@ -39,19 +36,24 @@
 #include "DNA_view3d_types.h"
 #include "DNA_windowmanager_types.h"
 #include "DNA_object_types.h"
-
-#include "RNA_access.h"
+#include "DNA_linestyle_types.h"
+#include "DNA_gpencil_types.h"
 
 #include "BLI_listbase.h"
 #include "BLI_string.h"
+#include "BLI_threads.h"
 #include "BLI_utildefines.h"
 
+#include "BLF_translation.h"
+
 #include "BKE_context.h"
 #include "BKE_main.h"
 #include "BKE_screen.h"
 
+#include "RNA_access.h"
+
 #ifdef WITH_PYTHON
-#include "BPY_extern.h"
+#  include "BPY_extern.h"
 #endif
 
 /* struct */
@@ -82,9 +84,11 @@ struct bContext {
 	} data;
 	
 	/* data evaluation */
+#if 0
 	struct {
 		int render;
 	} eval;
+#endif
 };
 
 /* context */
@@ -196,8 +200,7 @@ void CTX_store_free_list(ListBase *contexts)
 {
 	bContextStore *ctx;
 
-	while ((ctx = contexts->first)) {
-		BLI_remlink(contexts, ctx);
+	while ((ctx = BLI_pophead(contexts))) {
 		CTX_store_free(ctx);
 	}
 }
@@ -231,21 +234,37 @@ struct bContextDataResult {
 	short type; /* 0: normal, 1: seq */
 };
 
-static void *ctx_wm_python_context_get(const bContext *C, const char *member, void *fall_through)
+static void *ctx_wm_python_context_get(
+        const bContext *C,
+        const char *member, const StructRNA *member_type,
+        void *fall_through)
 {
 #ifdef WITH_PYTHON
-	bContextDataResult result;
-
-	if (C && CTX_py_dict_get(C)) {
+	if (UNLIKELY(C && CTX_py_dict_get(C))) {
+		bContextDataResult result;
 		memset(&result, 0, sizeof(bContextDataResult));
 		BPY_context_member_get((bContext *)C, member, &result);
-		if (result.ptr.data)
-			return result.ptr.data;
+
+		if (result.ptr.data) {
+			if (RNA_struct_is_a(result.ptr.type, member_type)) {
+				return result.ptr.data;
+			}
+			else {
+				printf("PyContext '%s' is a '%s', expected a '%s'\n",
+				       member,
+				       RNA_struct_identifier(result.ptr.type),
+				       RNA_struct_identifier(member_type));
+			}
+		}
 	}
 #else
-	(void)C, (void)member;
+	UNUSED_VARS(C, member, member_type);
 #endif
 
+	/* don't allow UI context access from non-main threads */
+	if (!BLI_thread_is_main())
+		return NULL;
+
 	return fall_through;
 }
 
@@ -254,19 +273,24 @@ static int ctx_data_get(bContext *C, const char *member, bContextDataResult *res
 	bScreen *sc;
 	ScrArea *sa;
 	ARegion *ar;
-	int done = FALSE, recursion = C->data.recursion;
+	int done = 0, recursion = C->data.recursion;
 	int ret = 0;
 
 	memset(result, 0, sizeof(bContextDataResult));
 #ifdef WITH_PYTHON
 	if (CTX_py_dict_get(C)) {
-		return BPY_context_member_get(C, member, result);
-//		if (BPY_context_member_get(C, member, result))
-//			return 1;
+		if (BPY_context_member_get(C, member, result)) {
+			return 1;
+		}
 	}
 #endif
+
+	/* don't allow UI context access from non-main threads */
+	if (!BLI_thread_is_main())
+		return done;
+
 	/* we check recursion to ensure that we do not get infinite
-	 * loops requesting data from ourselfs in a context callback */
+	 * loops requesting data from ourselves in a context callback */
 
 	/* Ok, this looks evil...
 	 * if (ret) done = -(-ret | -done);
@@ -282,7 +306,7 @@ static int ctx_data_get(bContext *C, const char *member, bContextDataResult *res
 		entry = BLI_rfindstring(&C->wm.store->entries, member, offsetof(bContextStoreEntry, name));
 		if (entry) {
 			result->ptr = entry->ptr;
-			done = TRUE;
+			done = 1;
 		}
 	}
 	if (done != 1 && recursion < 2 && (ar = CTX_wm_region(C))) {
@@ -318,10 +342,13 @@ static void *ctx_data_pointer_get(const bContext *C, const char *member)
 {
 	bContextDataResult result;
 
-	if (C && ctx_data_get((bContext *)C, member, &result) == 1)
+	if (C && ctx_data_get((bContext *)C, member, &result) == 1) {
+		BLI_assert(result.type == CTX_DATA_TYPE_POINTER);
 		return result.ptr.data;
-
-	return NULL;
+	}
+	else {
+		return NULL;
+	}
 }
 
 static int ctx_data_pointer_verify(const bContext *C, const char *member, void **pointer)
@@ -334,6 +361,7 @@ static int ctx_data_pointer_verify(const bContext *C, const char *member, void *
 		return 1;
 	}
 	else if (ctx_data_get((bContext *)C, member, &result) == 1) {
+		BLI_assert(result.type == CTX_DATA_TYPE_POINTER);
 		*pointer = result.ptr.data;
 		return 1;
 	}
@@ -348,12 +376,12 @@ static int ctx_data_collection_get(const bContext *C, const char *member, ListBa
 	bContextDataResult result;
 
 	if (ctx_data_get((bContext *)C, member, &result) == 1) {
+		BLI_assert(result.type == CTX_DATA_TYPE_COLLECTION);
 		*list = result.list;
 		return 1;
 	}
 
-	list->first = NULL;
-	list->last = NULL;
+	BLI_listbase_clear(list);
 
 	return 0;
 }
@@ -362,10 +390,13 @@ PointerRNA CTX_data_pointer_get(const bContext *C, const char *member)
 {
 	bContextDataResult result;
 
-	if (ctx_data_get((bContext *)C, member, &result) == 1)
+	if (ctx_data_get((bContext *)C, member, &result) == 1) {
+		BLI_assert(result.type == CTX_DATA_TYPE_POINTER);
 		return result.ptr;
-	else
+	}
+	else {
 		return PointerRNA_NULL;
+	}
 }
 
 PointerRNA CTX_data_pointer_get_type(const bContext *C, const char *member, StructRNA *type)
@@ -390,6 +421,7 @@ ListBase CTX_data_collection_get(const bContext *C, const char *member)
 	bContextDataResult result;
 
 	if (ctx_data_get((bContext *)C, member, &result) == 1) {
+		BLI_assert(result.type == CTX_DATA_TYPE_COLLECTION);
 		return result.list;
 	}
 	else {
@@ -418,11 +450,11 @@ int CTX_data_get(const bContext *C, const char *member, PointerRNA *r_ptr, ListB
 	return ret;
 }
 
-static void data_dir_add(ListBase *lb, const char *member)
+static void data_dir_add(ListBase *lb, const char *member, const bool use_all)
 {
 	LinkData *link;
 	
-	if (strcmp(member, "scene") == 0) /* exception */
+	if ((use_all == false) && strcmp(member, "scene") == 0) /* exception */
 		return;
 
 	if (BLI_findstring(lb, member, offsetof(LinkData, data)))
@@ -433,7 +465,13 @@ static void data_dir_add(ListBase *lb, const char *member)
 	BLI_addtail(lb, link);
 }
 
-ListBase CTX_data_dir_get(const bContext *C)
+/**
+ * \param C Context
+ * \param use_store Use 'C->wm.store'
+ * \param use_rna Use Include the properties from 'RNA_Context'
+ * \param use_all Don't skip values (currently only "scene")
+ */
+ListBase CTX_data_dir_get_ex(const bContext *C, const bool use_store, const bool use_rna, const bool use_all)
 {
 	bContextDataResult result;
 	ListBase lb;
@@ -444,11 +482,33 @@ ListBase CTX_data_dir_get(const bContext *C)
 
 	memset(&lb, 0, sizeof(lb));
 
-	if (C->wm.store) {
+	if (use_rna) {
+		char name[256], *nameptr;
+		int namelen;
+
+		PropertyRNA *iterprop;
+		PointerRNA ctx_ptr;
+		RNA_pointer_create(NULL, &RNA_Context, (void *)C, &ctx_ptr);
+
+		iterprop = RNA_struct_iterator_property(ctx_ptr.type);
+
+		RNA_PROP_BEGIN (&ctx_ptr, itemptr, iterprop)
+		{
+			nameptr = RNA_struct_name_get_alloc(&itemptr, name, sizeof(name), &namelen);
+			data_dir_add(&lb, name, use_all);
+			if (nameptr) {
+				if (name != nameptr) {
+					MEM_freeN(nameptr);
+				}
+			}
+		}
+		RNA_PROP_END;
+	}
+	if (use_store && C->wm.store) {
 		bContextStoreEntry *entry;
 
 		for (entry = C->wm.store->entries.first; entry; entry = entry->next)
-			data_dir_add(&lb, entry->name);
+			data_dir_add(&lb, entry->name, use_all);
 	}
 	if ((ar = CTX_wm_region(C)) && ar->type && ar->type->context) {
 		memset(&result, 0, sizeof(result));
@@ -456,7 +516,7 @@ ListBase CTX_data_dir_get(const bContext *C)
 
 		if (result.dir)
 			for (a = 0; result.dir[a]; a++)
-				data_dir_add(&lb, result.dir[a]);
+				data_dir_add(&lb, result.dir[a], use_all);
 	}
 	if ((sa = CTX_wm_area(C)) && sa->type && sa->type->context) {
 		memset(&result, 0, sizeof(result));
@@ -464,7 +524,7 @@ ListBase CTX_data_dir_get(const bContext *C)
 
 		if (result.dir)
 			for (a = 0; result.dir[a]; a++)
-				data_dir_add(&lb, result.dir[a]);
+				data_dir_add(&lb, result.dir[a], use_all);
 	}
 	if ((sc = CTX_wm_screen(C)) && sc->context) {
 		bContextDataCallback cb = sc->context;
@@ -473,18 +533,23 @@ ListBase CTX_data_dir_get(const bContext *C)
 
 		if (result.dir)
 			for (a = 0; result.dir[a]; a++)
-				data_dir_add(&lb, result.dir[a]);
+				data_dir_add(&lb, result.dir[a], use_all);
 	}
 
 	return lb;
 }
 
-int CTX_data_equals(const char *member, const char *str)
+ListBase CTX_data_dir_get(const bContext *C)
+{
+	return CTX_data_dir_get_ex(C, true, false, false);
+}
+
+bool CTX_data_equals(const char *member, const char *str)
 {
 	return (strcmp(member, str) == 0);
 }
 
-int CTX_data_dir(const char *member)
+bool CTX_data_dir(const char *member)
 {
 	return member[0] == '\0';
 }
@@ -524,7 +589,7 @@ int ctx_data_list_count(const bContext *C, int (*func)(const bContext *, ListBas
 	ListBase list;
 
 	if (func(C, &list)) {
-		int tot = BLI_countlist(&list);
+		int tot = BLI_listbase_count(&list);
 		BLI_freelistN(&list);
 		return tot;
 	}
@@ -558,17 +623,17 @@ wmWindowManager *CTX_wm_manager(const bContext *C)
 
 wmWindow *CTX_wm_window(const bContext *C)
 {
-	return ctx_wm_python_context_get(C, "window", C->wm.window);
+	return ctx_wm_python_context_get(C, "window", &RNA_Window, C->wm.window);
 }
 
 bScreen *CTX_wm_screen(const bContext *C)
 {
-	return ctx_wm_python_context_get(C, "screen", C->wm.screen);
+	return ctx_wm_python_context_get(C, "screen", &RNA_Screen, C->wm.screen);
 }
 
 ScrArea *CTX_wm_area(const bContext *C)
 {
-	return ctx_wm_python_context_get(C, "area", C->wm.area);
+	return ctx_wm_python_context_get(C, "area", &RNA_Area, C->wm.area);
 }
 
 SpaceLink *CTX_wm_space_data(const bContext *C)
@@ -579,7 +644,7 @@ SpaceLink *CTX_wm_space_data(const bContext *C)
 
 ARegion *CTX_wm_region(const bContext *C)
 {
-	return ctx_wm_python_context_get(C, "region", C->wm.region);
+	return ctx_wm_python_context_get(C, "region", &RNA_Region, C->wm.region);
 }
 
 void *CTX_wm_region_data(const bContext *C)
@@ -799,7 +864,7 @@ void CTX_wm_operator_poll_msg_set(bContext *C, const char *msg)
 
 const char *CTX_wm_operator_poll_msg_get(bContext *C)
 {
-	return C->wm.operator_poll_msg;
+	return IFACE_(C->wm.operator_poll_msg);
 }
 
 /* data context */
@@ -1027,3 +1092,33 @@ int CTX_data_visible_pose_bones(const bContext *C, ListBase *list)
 	return ctx_data_collection_get(C, "visible_pose_bones", list);
 }
 
+bGPdata *CTX_data_gpencil_data(const bContext *C)
+{
+	return ctx_data_pointer_get(C, "gpencil_data");
+}
+
+bGPDlayer *CTX_data_active_gpencil_layer(const bContext *C)
+{
+	return ctx_data_pointer_get(C, "active_gpencil_layer");
+}
+
+bGPDframe *CTX_data_active_gpencil_frame(const bContext *C)
+{
+	return ctx_data_pointer_get(C, "active_gpencil_frame");
+}
+
+int CTX_data_visible_gpencil_layers(const bContext *C, ListBase *list)
+{
+	return ctx_data_collection_get(C, "visible_gpencil_layers", list);
+}
+
+int CTX_data_editable_gpencil_layers(const bContext *C, ListBase *list)
+{
+	return ctx_data_collection_get(C, "editable_gpencil_layers", list);
+}
+
+int CTX_data_editable_gpencil_strokes(const bContext *C, ListBase *list)
+{
+	return ctx_data_collection_get(C, "editable_gpencil_strokes", list);
+}
+
