@@ -34,26 +34,44 @@
 #include <math.h>
 #include <ctype.h>
 #include <stdlib.h>
+#include <stddef.h>
 
 #include "MEM_guardedalloc.h"
 
 #include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
 
-#include "BKE_deform.h"
-
 #include "BLI_listbase.h"
 #include "BLI_math.h"
 #include "BLI_path_util.h"
 #include "BLI_string.h"
 #include "BLI_utildefines.h"
 
+#include "BLF_translation.h"
+
+#include "BKE_deform.h"  /* own include */
+
+bDeformGroup *BKE_defgroup_new(Object *ob, const char *name)
+{
+	bDeformGroup *defgroup;
+
+	BLI_assert(OB_TYPE_SUPPORT_VGROUP(ob->type));
+
+	defgroup = MEM_callocN(sizeof(bDeformGroup), __func__);
+
+	BLI_strncpy(defgroup->name, name, sizeof(defgroup->name));
+
+	BLI_addtail(&ob->defbase, defgroup);
+	defgroup_unique_name(defgroup, ob);
+
+	return defgroup;
+}
 
 void defgroup_copy_list(ListBase *outbase, ListBase *inbase)
 {
 	bDeformGroup *defgroup, *defgroupn;
 
-	outbase->first = outbase->last = NULL;
+	BLI_listbase_clear(outbase);
 
 	for (defgroup = inbase->first; defgroup; defgroup = defgroup->next) {
 		defgroupn = defgroup_duplicate(defgroup);
@@ -78,7 +96,21 @@ bDeformGroup *defgroup_duplicate(bDeformGroup *ingroup)
 	return outgroup;
 }
 
-/* copy & overwrite weights */
+/* overwrite weights filtered by vgroup_subset
+ * - do nothing if neither are set.
+ * - add destination weight if needed
+ */
+void defvert_copy_subset(MDeformVert *dvert_dst, const MDeformVert *dvert_src,
+                         const bool *vgroup_subset, const int vgroup_tot)
+{
+	int defgroup;
+	for (defgroup = 0; defgroup < vgroup_tot; defgroup++) {
+		if (vgroup_subset[defgroup]) {
+			defvert_copy_index(dvert_dst, dvert_src, defgroup);
+		}
+	}
+}
+
 void defvert_copy(MDeformVert *dvert_dst, const MDeformVert *dvert_src)
 {
 	if (dvert_dst->totweight == dvert_src->totweight) {
@@ -126,7 +158,7 @@ void defvert_copy_index(MDeformVert *dvert_dst, const MDeformVert *dvert_src, co
 /* only sync over matching weights, don't add or remove groups
  * warning, loop within loop.
  */
-void defvert_sync(MDeformVert *dvert_dst, const MDeformVert *dvert_src, int use_verify)
+void defvert_sync(MDeformVert *dvert_dst, const MDeformVert *dvert_src, const bool use_verify)
 {
 	if (dvert_src->totweight && dvert_dst->totweight) {
 		int i;
@@ -145,7 +177,7 @@ void defvert_sync(MDeformVert *dvert_dst, const MDeformVert *dvert_src, int use_
 
 /* be sure all flip_map values are valid */
 void defvert_sync_mapped(MDeformVert *dvert_dst, const MDeformVert *dvert_src,
-                         const int *flip_map, const int flip_map_len, const int use_verify)
+                         const int *flip_map, const int flip_map_len, const bool use_verify)
 {
 	if (dvert_src->totweight && dvert_dst->totweight) {
 		int i;
@@ -179,9 +211,49 @@ void defvert_remap(MDeformVert *dvert, int *map, const int map_len)
 	}
 }
 
+/**
+ * Same as #defvert_normalize but takes a bool array.
+ */
+void defvert_normalize_subset(MDeformVert *dvert,
+                              const bool *vgroup_subset, const int vgroup_tot)
+{
+	if (dvert->totweight == 0) {
+		/* nothing */
+	}
+	else if (dvert->totweight == 1) {
+		MDeformWeight *dw = dvert->dw;
+		if ((dw->def_nr < vgroup_tot) && vgroup_subset[dw->def_nr]) {
+			dw->weight = 1.0f;
+		}
+	}
+	else {
+		MDeformWeight *dw;
+		unsigned int i;
+		float tot_weight = 0.0f;
+
+		for (i = dvert->totweight, dw = dvert->dw; i != 0; i--, dw++) {
+			if ((dw->def_nr < vgroup_tot) && vgroup_subset[dw->def_nr]) {
+				tot_weight += dw->weight;
+			}
+		}
+
+		if (tot_weight > 0.0f) {
+			float scalar = 1.0f / tot_weight;
+			for (i = dvert->totweight, dw = dvert->dw; i != 0; i--, dw++) {
+				if ((dw->def_nr < vgroup_tot) && vgroup_subset[dw->def_nr]) {
+					dw->weight *= scalar;
+
+					/* in case of division errors with very low weights */
+					CLAMP(dw->weight, 0.0f, 1.0f);
+				}
+			}
+		}
+	}
+}
+
 void defvert_normalize(MDeformVert *dvert)
 {
-	if (dvert->totweight <= 0) {
+	if (dvert->totweight == 0) {
 		/* nothing */
 	}
 	else if (dvert->totweight == 1) {
@@ -208,14 +280,20 @@ void defvert_normalize(MDeformVert *dvert)
 	}
 }
 
-void defvert_normalize_lock_single(MDeformVert *dvert, const int def_nr_lock)
+/* Same as defvert_normalize() if the locked vgroup is not a member of the subset */
+void defvert_normalize_lock_single(MDeformVert *dvert,
+                                   const bool *vgroup_subset, const int vgroup_tot,
+                                   const int def_nr_lock)
 {
-	if (dvert->totweight <= 0) {
+	if (dvert->totweight == 0) {
 		/* nothing */
 	}
 	else if (dvert->totweight == 1) {
-		if (def_nr_lock != 0) {
-			dvert->dw[0].weight = 1.0f;
+		MDeformWeight *dw = dvert->dw;
+		if ((dw->def_nr < vgroup_tot) && vgroup_subset[dw->def_nr]) {
+			if (def_nr_lock != 0) {
+				dw->weight = 1.0f;
+			}
 		}
 	}
 	else {
@@ -226,13 +304,15 @@ void defvert_normalize_lock_single(MDeformVert *dvert, const int def_nr_lock)
 		float lock_iweight = 1.0f;
 
 		for (i = dvert->totweight, dw = dvert->dw; i != 0; i--, dw++) {
-			if (dw->def_nr != def_nr_lock) {
-				tot_weight += dw->weight;
-			}
-			else {
-				dw_lock = dw;
-				lock_iweight = (1.0f - dw_lock->weight);
-				CLAMP(lock_iweight, 0.0f, 1.0f);
+			if ((dw->def_nr < vgroup_tot) && vgroup_subset[dw->def_nr]) {
+				if (dw->def_nr != def_nr_lock) {
+					tot_weight += dw->weight;
+				}
+				else {
+					dw_lock = dw;
+					lock_iweight = (1.0f - dw_lock->weight);
+					CLAMP(lock_iweight, 0.0f, 1.0f);
+				}
 			}
 		}
 
@@ -241,25 +321,34 @@ void defvert_normalize_lock_single(MDeformVert *dvert, const int def_nr_lock)
 
 			float scalar = (1.0f / tot_weight) * lock_iweight;
 			for (i = dvert->totweight, dw = dvert->dw; i != 0; i--, dw++) {
-				if (dw != dw_lock) {
-					dw->weight *= scalar;
+				if ((dw->def_nr < vgroup_tot) && vgroup_subset[dw->def_nr]) {
+					if (dw != dw_lock) {
+						dw->weight *= scalar;
 
-					/* in case of division errors with very low weights */
-					CLAMP(dw->weight, 0.0f, 1.0f);
+						/* in case of division errors with very low weights */
+						CLAMP(dw->weight, 0.0f, 1.0f);
+					}
 				}
 			}
 		}
 	}
 }
 
-void defvert_normalize_lock_map(MDeformVert *dvert, const char *lock_flags, const int defbase_tot)
+/* Same as defvert_normalize() if no locked vgroup is a member of the subset */
+void defvert_normalize_lock_map(
+        MDeformVert *dvert,
+        const bool *vgroup_subset, const int vgroup_tot,
+        const bool *lock_flags, const int defbase_tot)
 {
-	if (dvert->totweight <= 0) {
+	if (dvert->totweight == 0) {
 		/* nothing */
 	}
 	else if (dvert->totweight == 1) {
-		if (LIKELY(defbase_tot >= 1) && lock_flags[0]) {
-			dvert->dw[0].weight = 1.0f;
+		MDeformWeight *dw = dvert->dw;
+		if ((dw->def_nr < vgroup_tot) && vgroup_subset[dw->def_nr]) {
+			if (LIKELY(defbase_tot >= 1) && lock_flags[0]) {
+				dw->weight = 1.0f;
+			}
 		}
 	}
 	else {
@@ -269,27 +358,31 @@ void defvert_normalize_lock_map(MDeformVert *dvert, const char *lock_flags, cons
 		float lock_iweight = 0.0f;
 
 		for (i = dvert->totweight, dw = dvert->dw; i != 0; i--, dw++) {
-			if ((dw->def_nr < defbase_tot) && (lock_flags[dw->def_nr] == FALSE)) {
-				tot_weight += dw->weight;
-			}
-			else {
-				/* invert after */
-				lock_iweight += dw->weight;
+			if ((dw->def_nr < vgroup_tot) && vgroup_subset[dw->def_nr]) {
+				if ((dw->def_nr < defbase_tot) && (lock_flags[dw->def_nr] == false)) {
+					tot_weight += dw->weight;
+				}
+				else {
+					/* invert after */
+					lock_iweight += dw->weight;
+				}
 			}
 		}
 
-		lock_iweight = maxf(0.0f, 1.0f - lock_iweight);
+		lock_iweight = max_ff(0.0f, 1.0f - lock_iweight);
 
 		if (tot_weight > 0.0f) {
 			/* paranoid, should be 1.0 but in case of float error clamp anyway */
 
 			float scalar = (1.0f / tot_weight) * lock_iweight;
 			for (i = dvert->totweight, dw = dvert->dw; i != 0; i--, dw++) {
-				if ((dw->def_nr < defbase_tot) && (lock_flags[dw->def_nr] == FALSE)) {
-					dw->weight *= scalar;
+				if ((dw->def_nr < vgroup_tot) && vgroup_subset[dw->def_nr]) {
+					if ((dw->def_nr < defbase_tot) && (lock_flags[dw->def_nr] == false)) {
+						dw->weight *= scalar;
 
-					/* in case of division errors with very low weights */
-					CLAMP(dw->weight, 0.0f, 1.0f);
+						/* in case of division errors with very low weights */
+						CLAMP(dw->weight, 0.0f, 1.0f);
+					}
 				}
 			}
 		}
@@ -337,50 +430,25 @@ void defvert_flip_merged(MDeformVert *dvert, const int *flip_map, const int flip
 
 bDeformGroup *defgroup_find_name(Object *ob, const char *name)
 {
-	/* return a pointer to the deform group with this name
-	 * or return NULL otherwise.
-	 */
-	bDeformGroup *curdef;
-
-	for (curdef = ob->defbase.first; curdef; curdef = curdef->next) {
-		if (!strcmp(curdef->name, name)) {
-			return curdef;
-		}
-	}
-	return NULL;
+	return BLI_findstring(&ob->defbase, name, offsetof(bDeformGroup, name));
 }
 
 int defgroup_name_index(Object *ob, const char *name)
 {
-	/* Return the location of the named deform group within the list of
-	 * deform groups. This function is a combination of BLI_findlink and
-	 * defgroup_find_name. The other two could be called instead, but that
-	 * require looping over the vertexgroups twice.
-	 */
-	bDeformGroup *curdef;
-	int def_nr;
-
-	if (name && name[0] != '\0') {
-		for (curdef = ob->defbase.first, def_nr = 0; curdef; curdef = curdef->next, def_nr++) {
-			if (!strcmp(curdef->name, name))
-				return def_nr;
-		}
-	}
-
-	return -1;
+	return (name) ? BLI_findstringindex(&ob->defbase, name, offsetof(bDeformGroup, name)) : -1;
 }
 
 /* note, must be freed */
-int *defgroup_flip_map(Object *ob, int *flip_map_len, int use_default)
+int *defgroup_flip_map(Object *ob, int *flip_map_len, const bool use_default)
 {
-	int defbase_tot = *flip_map_len = BLI_countlist(&ob->defbase);
+	int defbase_tot = *flip_map_len = BLI_listbase_count(&ob->defbase);
 
 	if (defbase_tot == 0) {
 		return NULL;
 	}
 	else {
 		bDeformGroup *dg;
-		char name[sizeof(dg->name)];
+		char name_flip[sizeof(dg->name)];
 		int i, flip_num, *map = MEM_mallocN(defbase_tot * sizeof(int), __func__);
 
 		for (i = 0; i < defbase_tot; i++) {
@@ -394,9 +462,10 @@ int *defgroup_flip_map(Object *ob, int *flip_map_len, int use_default)
 				if (use_default)
 					map[i] = i;
 
-				flip_side_name(name, dg->name, FALSE);
-				if (strcmp(name, dg->name)) {
-					flip_num = defgroup_name_index(ob, name);
+				BKE_deform_flip_side_name(name_flip, dg->name, false);
+
+				if (!STREQ(name_flip, dg->name)) {
+					flip_num = defgroup_name_index(ob, name_flip);
 					if (flip_num >= 0) {
 						map[i] = flip_num;
 						map[flip_num] = i; /* save an extra lookup */
@@ -409,16 +478,16 @@ int *defgroup_flip_map(Object *ob, int *flip_map_len, int use_default)
 }
 
 /* note, must be freed */
-int *defgroup_flip_map_single(Object *ob, int *flip_map_len, int use_default, int defgroup)
+int *defgroup_flip_map_single(Object *ob, int *flip_map_len, const bool use_default, int defgroup)
 {
-	int defbase_tot = *flip_map_len = BLI_countlist(&ob->defbase);
+	int defbase_tot = *flip_map_len = BLI_listbase_count(&ob->defbase);
 
 	if (defbase_tot == 0) {
 		return NULL;
 	}
 	else {
 		bDeformGroup *dg;
-		char name[sizeof(dg->name)];
+		char name_flip[sizeof(dg->name)];
 		int i, flip_num, *map = MEM_mallocN(defbase_tot * sizeof(int), __func__);
 
 		for (i = 0; i < defbase_tot; i++) {
@@ -427,11 +496,11 @@ int *defgroup_flip_map_single(Object *ob, int *flip_map_len, int use_default, in
 
 		dg = BLI_findlink(&ob->defbase, defgroup);
 
-		flip_side_name(name, dg->name, FALSE);
-		if (strcmp(name, dg->name)) {
-			flip_num = defgroup_name_index(ob, name);
+		BKE_deform_flip_side_name(name_flip, dg->name, false);
+		if (!STREQ(name_flip, dg->name)) {
+			flip_num = defgroup_name_index(ob, name_flip);
 
-			if (flip_num >= 0) {
+			if (flip_num != -1) {
 				map[defgroup] = flip_num;
 				map[flip_num] = defgroup;
 			}
@@ -441,38 +510,39 @@ int *defgroup_flip_map_single(Object *ob, int *flip_map_len, int use_default, in
 	}
 }
 
-int defgroup_flip_index(Object *ob, int index, int use_default)
+int defgroup_flip_index(Object *ob, int index, const bool use_default)
 {
 	bDeformGroup *dg = BLI_findlink(&ob->defbase, index);
 	int flip_index = -1;
 
 	if (dg) {
-		char name[sizeof(dg->name)];
-		flip_side_name(name, dg->name, 0);
+		char name_flip[sizeof(dg->name)];
+		BKE_deform_flip_side_name(name_flip, dg->name, false);
 
-		if (strcmp(name, dg->name))
-			flip_index = defgroup_name_index(ob, name);
+		if (!STREQ(name_flip, dg->name)) {
+			flip_index = defgroup_name_index(ob, name_flip);
+		}
 	}
 
 	return (flip_index == -1 && use_default) ? index : flip_index;
 }
 
-static int defgroup_find_name_dupe(const char *name, bDeformGroup *dg, Object *ob)
+static bool defgroup_find_name_dupe(const char *name, bDeformGroup *dg, Object *ob)
 {
 	bDeformGroup *curdef;
 
 	for (curdef = ob->defbase.first; curdef; curdef = curdef->next) {
 		if (dg != curdef) {
 			if (!strcmp(curdef->name, name)) {
-				return 1;
+				return true;
 			}
 		}
 	}
 
-	return 0;
+	return false;
 }
 
-static int defgroup_unique_check(void *arg, const char *name)
+static bool defgroup_unique_check(void *arg, const char *name)
 {
 	struct {Object *ob; void *dg; } *data = arg;
 	return defgroup_find_name_dupe(name, data->dg, data->ob);
@@ -484,12 +554,12 @@ void defgroup_unique_name(bDeformGroup *dg, Object *ob)
 	data.ob = ob;
 	data.dg = dg;
 
-	BLI_uniquename_cb(defgroup_unique_check, &data, "Group", '.', dg->name, sizeof(dg->name));
+	BLI_uniquename_cb(defgroup_unique_check, &data, DATA_("Group"), '.', dg->name, sizeof(dg->name));
 }
 
-static int is_char_sep(const char c)
+static bool is_char_sep(const char c)
 {
-	return ELEM4(c, '.', ' ', '-', '_');
+	return ELEM(c, '.', ' ', '-', '_');
 }
 
 /* based on BLI_split_dirfile() / os.path.splitext(), "a.b.c" -> ("a.b", ".c") */
@@ -501,7 +571,7 @@ void BKE_deform_split_suffix(const char string[MAX_VGROUP_NAME], char body[MAX_V
 
 	body[0] = suf[0] = '\0';
 
-	for (i = len - 1; i > 1; i--) {
+	for (i = len; i > 0; i--) {
 		if (is_char_sep(string[i])) {
 			BLI_strncpy(body, string, i + 1);
 			BLI_strncpy(suf, string + i,  (len + 1) - i);
@@ -509,7 +579,7 @@ void BKE_deform_split_suffix(const char string[MAX_VGROUP_NAME], char body[MAX_V
 		}
 	}
 
-	BLI_strncpy(body, string, len);
+	memcpy(body, string, len + 1);
 }
 
 /* "a.b.c" -> ("a.", "b.c") */
@@ -535,7 +605,8 @@ void BKE_deform_split_prefix(const char string[MAX_VGROUP_NAME], char pre[MAX_VG
 /* finds the best possible flipped name. For renaming; check for unique names afterwards */
 /* if strip_number: removes number extensions
  * note: don't use sizeof() for 'name' or 'from_name' */
-void flip_side_name(char name[MAX_VGROUP_NAME], const char from_name[MAX_VGROUP_NAME], int strip_number)
+void BKE_deform_flip_side_name(char name[MAX_VGROUP_NAME], const char from_name[MAX_VGROUP_NAME],
+                               const bool strip_number)
 {
 	int     len;
 	char    prefix[MAX_VGROUP_NAME]  = "";   /* The part before the facing */
@@ -543,6 +614,7 @@ void flip_side_name(char name[MAX_VGROUP_NAME], const char from_name[MAX_VGROUP_
 	char    replace[MAX_VGROUP_NAME] = "";   /* The replacement string */
 	char    number[MAX_VGROUP_NAME]  = "";   /* The number extension string */
 	char    *index = NULL;
+	bool is_set = false;
 
 	/* always copy the name, since this can be called with an uninitialized string */
 	BLI_strncpy(name, from_name, MAX_VGROUP_NAME);
@@ -557,7 +629,7 @@ void flip_side_name(char name[MAX_VGROUP_NAME], const char from_name[MAX_VGROUP_
 	if (isdigit(name[len - 1])) {
 		index = strrchr(name, '.'); // last occurrence
 		if (index && isdigit(index[1])) { // doesnt handle case bone.1abc2 correct..., whatever!
-			if (strip_number == 0) {
+			if (strip_number == false) {
 				BLI_strncpy(number, index, sizeof(number));
 			}
 			*index = 0;
@@ -569,6 +641,7 @@ void flip_side_name(char name[MAX_VGROUP_NAME], const char from_name[MAX_VGROUP_
 
 	/* first case; separator . - _ with extensions r R l L  */
 	if (is_char_sep(name[len - 2])) {
+		is_set = true;
 		switch (name[len - 1]) {
 			case 'l':
 				prefix[len - 1] = 0;
@@ -586,10 +659,14 @@ void flip_side_name(char name[MAX_VGROUP_NAME], const char from_name[MAX_VGROUP_
 				prefix[len - 1] = 0;
 				strcpy(replace, "L");
 				break;
+			default:
+				is_set = false;
 		}
 	}
+
 	/* case; beginning with r R l L, with separator after it */
-	else if (is_char_sep(name[1])) {
+	if (!is_set && is_char_sep(name[1])) {
+		is_set = true;
 		switch (name[0]) {
 			case 'l':
 				strcpy(replace, "r");
@@ -611,40 +688,43 @@ void flip_side_name(char name[MAX_VGROUP_NAME], const char from_name[MAX_VGROUP_
 				BLI_strncpy(suffix, name + 1, sizeof(suffix));
 				prefix[0] = 0;
 				break;
+			default:
+				is_set = false;
 		}
 	}
-	else if (len > 5) {
+
+	if (!is_set && len > 5) {
 		/* hrms, why test for a separator? lets do the rule 'ultimate left or right' */
-		index = BLI_strcasestr(prefix, "right");
-		if (index == prefix || index == prefix + len - 5) {
-			if (index[0] == 'r')
+		if (((index = BLI_strcasestr(prefix, "right")) == prefix) ||
+		    (index == prefix + len - 5))
+		{
+			is_set = true;
+			if (index[0] == 'r') {
 				strcpy(replace, "left");
+			}
 			else {
-				if (index[1] == 'I')
-					strcpy(replace, "LEFT");
-				else
-					strcpy(replace, "Left");
+				strcpy(replace, (index[1] == 'I') ? "LEFT" : "Left");
 			}
 			*index = 0;
 			BLI_strncpy(suffix, index + 5, sizeof(suffix));
 		}
-		else {
-			index = BLI_strcasestr(prefix, "left");
-			if (index == prefix || index == prefix + len - 4) {
-				if (index[0] == 'l')
-					strcpy(replace, "right");
-				else {
-					if (index[1] == 'E')
-						strcpy(replace, "RIGHT");
-					else
-						strcpy(replace, "Right");
-				}
-				*index = 0;
-				BLI_strncpy(suffix, index + 4, sizeof(suffix));
+		else if (((index = BLI_strcasestr(prefix, "left")) == prefix) ||
+		         (index == prefix + len - 4))
+		{
+			is_set = true;
+			if (index[0] == 'l') {
+				strcpy(replace, "right");
 			}
+			else {
+				strcpy(replace, (index[1] == 'E') ? "RIGHT" : "Right");
+			}
+			*index = 0;
+			BLI_strncpy(suffix, index + 4, sizeof(suffix));
 		}
 	}
 
+	(void)is_set;  /* quiet warning */
+
 	BLI_snprintf(name, MAX_VGROUP_NAME, "%s%s%s%s", prefix, replace, suffix, number);
 }
 
@@ -699,7 +779,7 @@ MDeformWeight *defvert_verify_index(MDeformVert *dvert, const int defgroup)
 	if (dw_new)
 		return dw_new;
 
-	dw_new = MEM_callocN(sizeof(MDeformWeight) * (dvert->totweight + 1), "deformWeight");
+	dw_new = MEM_mallocN(sizeof(MDeformWeight) * (dvert->totweight + 1), "deformWeight");
 	if (dvert->dw) {
 		memcpy(dw_new, dvert->dw, sizeof(MDeformWeight) * dvert->totweight);
 		MEM_freeN(dvert->dw);
@@ -790,3 +870,93 @@ void defvert_clear(MDeformVert *dvert)
 
 	dvert->totweight = 0;
 }
+
+/**
+ * \return The first group index shared by both deform verts
+ * or -1 if none are found.
+ */
+int defvert_find_shared(const MDeformVert *dvert_a, const MDeformVert *dvert_b)
+{
+	if (dvert_a->totweight && dvert_b->totweight) {
+		MDeformWeight *dw = dvert_a->dw;
+		unsigned int i;
+
+		for (i = dvert_a->totweight; i != 0; i--, dw++) {
+			if (dw->weight > 0.0f && defvert_find_weight(dvert_b, dw->def_nr) > 0.0f) {
+				return dw->def_nr;
+			}
+		}
+	}
+
+	return -1;
+}
+
+/**
+ * return true if has no weights
+ */
+bool defvert_is_weight_zero(const struct MDeformVert *dvert, const int defgroup_tot)
+{
+	MDeformWeight *dw = dvert->dw;
+	unsigned int i;
+	for (i = dvert->totweight; i != 0; i--, dw++) {
+		if (dw->weight != 0.0f) {
+			/* check the group is in-range, happens on rare situations */
+			if (LIKELY(dw->def_nr < defgroup_tot)) {
+				return false;
+			}
+		}
+	}
+	return true;
+}
+
+/* -------------------------------------------------------------------- */
+/* Defvert Array functions */
+
+void BKE_defvert_array_copy(MDeformVert *dst, const MDeformVert *src, int copycount)
+{
+	/* Assumes dst is already set up */
+	int i;
+
+	if (!src || !dst)
+		return;
+
+	memcpy(dst, src, copycount * sizeof(MDeformVert));
+
+	for (i = 0; i < copycount; i++) {
+		if (src[i].dw) {
+			dst[i].dw = MEM_mallocN(sizeof(MDeformWeight) * src[i].totweight, "copy_deformWeight");
+			memcpy(dst[i].dw, src[i].dw, sizeof(MDeformWeight) * src[i].totweight);
+		}
+	}
+
+}
+
+void BKE_defvert_array_free_elems(MDeformVert *dvert, int totvert)
+{
+	/* Instead of freeing the verts directly,
+	 * call this function to delete any special
+	 * vert data */
+	int i;
+
+	if (!dvert)
+		return;
+
+	/* Free any special data from the verts */
+	for (i = 0; i < totvert; i++) {
+		if (dvert[i].dw) MEM_freeN(dvert[i].dw);
+	}
+}
+
+void BKE_defvert_array_free(MDeformVert *dvert, int totvert)
+{
+	/* Instead of freeing the verts directly,
+	 * call this function to delete any special
+	 * vert data */
+	if (!dvert)
+		return;
+
+	/* Free any special data from the verts */
+	BKE_defvert_array_free_elems(dvert, totvert);
+
+	MEM_freeN(dvert);
+}
