@@ -27,7 +27,6 @@
  *  \ingroup bke
  */
 
- 
 #include <stdio.h>
 #include <stdlib.h>
 #include <stddef.h>
@@ -36,16 +35,25 @@
 #include "BLI_utildefines.h"
 #include "BLI_string.h"
 #include "BLI_listbase.h"
+#include "BLI_math.h"
 
 #include "BKE_idprop.h"
 #include "BKE_library.h"
 
 #include "MEM_guardedalloc.h"
 
+#include "BLI_strict_flags.h"
+
 /* IDPropertyTemplate is a union in DNA_ID.h */
 
+/**
+ * if the new is 'IDP_ARRAY_REALLOC_LIMIT' items less,
+ * than #IDProperty.totallen, reallocate anyway.
+ */
+#define IDP_ARRAY_REALLOC_LIMIT 200
+
 /*local size table.*/
-static char idp_size_table[] = {
+static size_t idp_size_table[] = {
 	1, /*strings*/
 	sizeof(int),
 	sizeof(float),
@@ -57,13 +65,21 @@ static char idp_size_table[] = {
 	sizeof(double)
 };
 
-/* ------------Property Array Type ----------- */
-#define GETPROP(prop, i) (((IDProperty *)(prop)->data.pointer) + (i))
+
+/* -------------------------------------------------------------------- */
+/* Array Functions */
+
+/** \name IDP Array API
+ * \{ */
+
+#define GETPROP(prop, i) &(IDP_IDPArray(prop)[i])
 
 /* --------- property array type -------------*/
 
-/* note: as a start to move away from the stupid IDP_New function, this type
- * has it's own allocation function.*/
+/**
+ * \note as a start to move away from the stupid IDP_New function, this type
+ * has it's own allocation function.
+ */
 IDProperty *IDP_NewIDPArray(const char *name)
 {
 	IDProperty *prop = MEM_callocN(sizeof(IDProperty), "IDProperty prop array");
@@ -74,12 +90,15 @@ IDProperty *IDP_NewIDPArray(const char *name)
 	return prop;
 }
 
-IDProperty *IDP_CopyIDPArray(IDProperty *array)
+IDProperty *IDP_CopyIDPArray(const IDProperty *array)
 {
 	/* don't use MEM_dupallocN because this may be part of an array */
-	IDProperty *narray = MEM_mallocN(sizeof(IDProperty), "IDP_CopyIDPArray"), *tmp;
+	IDProperty *narray, *tmp;
 	int i;
 
+	BLI_assert(array->type == IDP_IDPARRAY);
+
+	narray = MEM_mallocN(sizeof(IDProperty), __func__);
 	*narray = *array;
 
 	narray->data.pointer = MEM_dupallocN(array->data.pointer);
@@ -102,6 +121,8 @@ void IDP_FreeIDPArray(IDProperty *prop)
 {
 	int i;
 	
+	BLI_assert(prop->type == IDP_IDPARRAY);
+
 	for (i = 0; i < prop->len; i++)
 		IDP_FreeProperty(GETPROP(prop, i));
 
@@ -112,7 +133,11 @@ void IDP_FreeIDPArray(IDProperty *prop)
 /*shallow copies item*/
 void IDP_SetIndexArray(IDProperty *prop, int index, IDProperty *item)
 {
-	IDProperty *old = GETPROP(prop, index);
+	IDProperty *old;
+
+	BLI_assert(prop->type == IDP_IDPARRAY);
+
+	old = GETPROP(prop, index);
 	if (index >= prop->len || index < 0) return;
 	if (item != old) IDP_FreeProperty(old);
 	
@@ -121,30 +146,49 @@ void IDP_SetIndexArray(IDProperty *prop, int index, IDProperty *item)
 
 IDProperty *IDP_GetIndexArray(IDProperty *prop, int index)
 {
+	BLI_assert(prop->type == IDP_IDPARRAY);
+
 	return GETPROP(prop, index);
 }
 
 void IDP_AppendArray(IDProperty *prop, IDProperty *item)
 {
+	BLI_assert(prop->type == IDP_IDPARRAY);
+
 	IDP_ResizeIDPArray(prop, prop->len + 1);
 	IDP_SetIndexArray(prop, prop->len - 1, item);
 }
 
 void IDP_ResizeIDPArray(IDProperty *prop, int newlen)
 {
-	void *newarr;
-	int newsize = newlen;
+	int newsize;
 
-	/*first check if the array buffer size has room*/
-	/*if newlen is 200 chars less then totallen, reallocate anyway*/
-	if (newlen <= prop->totallen && prop->totallen - newlen < 200) {
+	BLI_assert(prop->type == IDP_IDPARRAY);
+
+	/* first check if the array buffer size has room */
+	if (newlen <= prop->totallen) {
+		if (newlen < prop->len && prop->totallen - newlen < IDP_ARRAY_REALLOC_LIMIT) {
+			int i;
+
+			for (i = newlen; i < prop->len; i++)
+				IDP_FreeProperty(GETPROP(prop, i));
+
+			prop->len = newlen;
+			return;
+		}
+		else if (newlen >= prop->len) {
+			prop->len = newlen;
+			return;
+		}
+	}
+
+	/* free trailing items */
+	if (newlen < prop->len) {
+		/* newlen is smaller */
 		int i;
-
-		for (i = newlen; i < prop->len; i++)
+		for (i = newlen; i < prop->len; i++) {
 			IDP_FreeProperty(GETPROP(prop, i));
-
-		prop->len = newlen;
-		return;
+		}
 	}
 
 	/* - Note: This code comes from python, here's the corresponding comment. - */
@@ -155,25 +199,9 @@ void IDP_ResizeIDPArray(IDProperty *prop, int newlen)
 	 * system realloc().
 	 * The growth pattern is:  0, 4, 8, 16, 25, 35, 46, 58, 72, 88, ...
 	 */
+	newsize = newlen;
 	newsize = (newsize >> 3) + (newsize < 9 ? 3 : 6) + newsize;
-
-	newarr = MEM_callocN(sizeof(IDProperty) * newsize, "idproperty array resized");
-	if (newlen >= prop->len) {
-		/* newlen is bigger */
-		memcpy(newarr, prop->data.pointer, prop->len * sizeof(IDProperty));
-	}
-	else {
-		int i;
-		/* newlen is smaller */
-		for (i = newlen; i < prop->len; i++) {
-			IDP_FreeProperty(GETPROP(prop, i));
-		}
-		memcpy(newarr, prop->data.pointer, newlen * sizeof(IDProperty));
-	}
-
-	if (prop->data.pointer)
-		MEM_freeN(prop->data.pointer);
-	prop->data.pointer = newarr;
+	prop->data.pointer = MEM_recallocN(prop->data.pointer, sizeof(IDProperty) * (size_t)newsize);
 	prop->len = newlen;
 	prop->totallen = newsize;
 }
@@ -210,12 +238,11 @@ static void idp_resize_group_array(IDProperty *prop, int newlen, void *newarr)
 /*this function works for strings too!*/
 void IDP_ResizeArray(IDProperty *prop, int newlen)
 {
-	void *newarr;
-	int newsize = newlen;
+	int newsize;
+	const bool is_grow = newlen >= prop->len;
 
-	/*first check if the array buffer size has room*/
-	/*if newlen is 200 chars less then totallen, reallocate anyway*/
-	if (newlen <= prop->totallen && prop->totallen - newlen < 200) {
+	/* first check if the array buffer size has room */
+	if (newlen <= prop->totallen && prop->totallen - newlen < IDP_ARRAY_REALLOC_LIMIT) {
 		idp_resize_group_array(prop, newlen, prop->data.pointer);
 		prop->len = newlen;
 		return;
@@ -229,22 +256,18 @@ void IDP_ResizeArray(IDProperty *prop, int newlen)
 	 * system realloc().
 	 * The growth pattern is:  0, 4, 8, 16, 25, 35, 46, 58, 72, 88, ...
 	 */
+	newsize = newlen;
 	newsize = (newsize >> 3) + (newsize < 9 ? 3 : 6) + newsize;
 
-	newarr = MEM_callocN(idp_size_table[(int)prop->subtype] * newsize, "idproperty array resized");
-	if (newlen >= prop->len) {
-		/* newlen is bigger */
-		memcpy(newarr, prop->data.pointer, prop->len * idp_size_table[(int)prop->subtype]);
-		idp_resize_group_array(prop, newlen, newarr);
-	}
-	else {
-		/* newlen is smaller */
-		idp_resize_group_array(prop, newlen, newarr);
-		memcpy(newarr, prop->data.pointer, newlen * idp_size_table[(int)prop->subtype]);
-	}
+	if (is_grow == false)
+		idp_resize_group_array(prop, newlen, prop->data.pointer);
+
+	prop->data.pointer = MEM_recallocN(
+	        prop->data.pointer, idp_size_table[(int)prop->subtype] * (size_t)newsize);
+
+	if (is_grow == true)
+		idp_resize_group_array(prop, newlen, prop->data.pointer);
 
-	MEM_freeN(prop->data.pointer);
-	prop->data.pointer = newarr;
 	prop->len = newlen;
 	prop->totallen = newsize;
 }
@@ -258,7 +281,7 @@ void IDP_FreeArray(IDProperty *prop)
 }
 
 
-static IDProperty *idp_generic_copy(IDProperty *prop)
+static IDProperty *idp_generic_copy(const IDProperty *prop)
 {
 	IDProperty *newp = MEM_callocN(sizeof(IDProperty), "IDProperty array dup");
 
@@ -271,7 +294,7 @@ static IDProperty *idp_generic_copy(IDProperty *prop)
 	return newp;
 }
 
-static IDProperty *IDP_CopyArray(IDProperty *prop)
+static IDProperty *IDP_CopyArray(const IDProperty *prop)
 {
 	IDProperty *newp = idp_generic_copy(prop);
 
@@ -292,28 +315,42 @@ static IDProperty *IDP_CopyArray(IDProperty *prop)
 
 	return newp;
 }
+/** \} */
 
-/* ---------- String Type ------------ */
+
+/* -------------------------------------------------------------------- */
+/* String Functions */
+
+/** \name IDProperty String API
+ * \{ */
+
+/**
+ *
+ * \param st  The string to assign.
+ * \param name  The property name.
+ * \param maxlen  The size of the new string (including the \0 terminator).
+ * \return The new string property.
+ */
 IDProperty *IDP_NewString(const char *st, const char *name, int maxlen)
 {
 	IDProperty *prop = MEM_callocN(sizeof(IDProperty), "IDProperty string");
 
 	if (st == NULL) {
-		prop->data.pointer = MEM_callocN(DEFAULT_ALLOC_FOR_NULL_STRINGS, "id property string 1");
+		prop->data.pointer = MEM_mallocN(DEFAULT_ALLOC_FOR_NULL_STRINGS, "id property string 1");
+		*IDP_String(prop) = '\0';
 		prop->totallen = DEFAULT_ALLOC_FOR_NULL_STRINGS;
-		prop->len = 1; /*NULL string, has len of 1 to account for null byte.*/
+		prop->len = 1;  /* NULL string, has len of 1 to account for null byte. */
 	}
 	else {
-		int stlen = strlen(st);
+		/* include null terminator '\0' */
+		int stlen = (int)strlen(st) + 1;
 
 		if (maxlen > 0 && maxlen < stlen)
 			stlen = maxlen;
 
-		stlen++; /* null terminator '\0' */
-
-		prop->data.pointer = MEM_callocN(stlen, "id property string 2");
+		prop->data.pointer = MEM_mallocN((size_t)stlen, "id property string 2");
 		prop->len = prop->totallen = stlen;
-		BLI_strncpy(prop->data.pointer, st, stlen);
+		BLI_strncpy(prop->data.pointer, st, (size_t)stlen);
 	}
 
 	prop->type = IDP_STRING;
@@ -322,11 +359,15 @@ IDProperty *IDP_NewString(const char *st, const char *name, int maxlen)
 	return prop;
 }
 
-static IDProperty *IDP_CopyString(IDProperty *prop)
+static IDProperty *IDP_CopyString(const IDProperty *prop)
 {
-	IDProperty *newp = idp_generic_copy(prop);
+	IDProperty *newp;
 
-	if (prop->data.pointer) newp->data.pointer = MEM_dupallocN(prop->data.pointer);
+	BLI_assert(prop->type == IDP_STRING);
+	newp = idp_generic_copy(prop);
+
+	if (prop->data.pointer)
+		newp->data.pointer = MEM_dupallocN(prop->data.pointer);
 	newp->len = prop->len;
 	newp->subtype = prop->subtype;
 	newp->totallen = prop->totallen;
@@ -337,19 +378,21 @@ static IDProperty *IDP_CopyString(IDProperty *prop)
 
 void IDP_AssignString(IDProperty *prop, const char *st, int maxlen)
 {
-	int stlen = strlen(st);
+	int stlen;
 
+	BLI_assert(prop->type == IDP_STRING);
+	stlen = (int)strlen(st);
 	if (maxlen > 0 && maxlen < stlen)
 		stlen = maxlen;
 
 	if (prop->subtype == IDP_STRING_SUB_BYTE) {
 		IDP_ResizeArray(prop, stlen);
-		memcpy(prop->data.pointer, st, stlen);
+		memcpy(prop->data.pointer, st, (size_t)stlen);
 	}
 	else {
-		stlen++; /* make room for null byte */
+		stlen++;
 		IDP_ResizeArray(prop, stlen);
-		BLI_strncpy(prop->data.pointer, st, stlen);
+		BLI_strncpy(prop->data.pointer, st, (size_t)stlen);
 	}
 }
 
@@ -357,7 +400,9 @@ void IDP_ConcatStringC(IDProperty *prop, const char *st)
 {
 	int newlen;
 
-	newlen = prop->len + strlen(st);
+	BLI_assert(prop->type == IDP_STRING);
+
+	newlen = prop->len + (int)strlen(st);
 	/* we have to remember that prop->len includes the null byte for strings.
 	 * so there's no need to add +1 to the resize function.*/
 	IDP_ResizeArray(prop, newlen);
@@ -368,6 +413,8 @@ void IDP_ConcatString(IDProperty *str1, IDProperty *append)
 {
 	int newlen;
 
+	BLI_assert(append->type == IDP_STRING);
+
 	/* since ->len for strings includes the NULL byte, we have to subtract one or
 	 * we'll get an extra null byte after each concatenation operation.*/
 	newlen = str1->len + append->len - 1;
@@ -377,13 +424,19 @@ void IDP_ConcatString(IDProperty *str1, IDProperty *append)
 
 void IDP_FreeString(IDProperty *prop)
 {
+	BLI_assert(prop->type == IDP_STRING);
+
 	if (prop->data.pointer)
 		MEM_freeN(prop->data.pointer);
 }
+/** \} */
 
 
-/*-------- ID Type, not in use yet -------*/
+/* -------------------------------------------------------------------- */
+/* ID Type (not in use yet) */
 
+/** \name IDProperty ID API (unused)
+ * \{ */
 void IDP_LinkID(IDProperty *prop, ID *id)
 {
 	if (prop->data.pointer) ((ID *)prop->data.pointer)->us--;
@@ -395,15 +448,26 @@ void IDP_UnlinkID(IDProperty *prop)
 {
 	((ID *)prop->data.pointer)->us--;
 }
+/** \} */
 
-/*-------- Group Functions -------*/
 
-/*checks if a property with the same name as prop exists, and if so replaces it.*/
-static IDProperty *IDP_CopyGroup(IDProperty *prop)
+/* -------------------------------------------------------------------- */
+/* Group Functions */
+
+/** \name IDProperty Group API
+ * \{ */
+
+/**
+ * Checks if a property with the same name as prop exists, and if so replaces it.
+ */
+static IDProperty *IDP_CopyGroup(const IDProperty *prop)
 {
-	IDProperty *newp = idp_generic_copy(prop), *link;
-	newp->len = prop->len;
+	IDProperty *newp, *link;
 	
+	BLI_assert(prop->type == IDP_GROUP);
+	newp = idp_generic_copy(prop);
+	newp->len = prop->len;
+
 	for (link = prop->data.group.first; link; link = link->next) {
 		BLI_addtail(&newp->data.group, IDP_CopyProperty(link));
 	}
@@ -413,9 +477,13 @@ static IDProperty *IDP_CopyGroup(IDProperty *prop)
 
 /* use for syncing proxies.
  * When values name and types match, copy the values, else ignore */
-void IDP_SyncGroupValues(IDProperty *dest, IDProperty *src)
+void IDP_SyncGroupValues(IDProperty *dest, const IDProperty *src)
 {
 	IDProperty *other, *prop;
+
+	BLI_assert(dest->type == IDP_GROUP);
+	BLI_assert(src->type == IDP_GROUP);
+
 	for (prop = src->data.group.first; prop; prop = prop->next) {
 		other = BLI_findstring(&dest->data.group, prop->name, offsetof(IDProperty, name));
 		if (other && prop->type == other->type) {
@@ -438,24 +506,57 @@ void IDP_SyncGroupValues(IDProperty *dest, IDProperty *src)
 
 					IDP_FreeProperty(tmp);
 					MEM_freeN(tmp);
+					break;
 				}
 			}
 		}
 	}
 }
 
-/*
- * replaces all properties with the same name in a destination group from a source group.
+void IDP_SyncGroupTypes(IDProperty *dst, const IDProperty *src, const bool do_arraylen)
+{
+	IDProperty *prop_dst, *prop_dst_next;
+	const IDProperty *prop_src;
+
+	for (prop_dst = dst->data.group.first; prop_dst; prop_dst = prop_dst_next) {
+		prop_dst_next = prop_dst->next;
+		if ((prop_src = IDP_GetPropertyFromGroup((IDProperty *)src, prop_dst->name))) {
+			/* check of we should replace? */
+			if ((prop_dst->type != prop_src->type || prop_dst->subtype != prop_src->subtype) ||
+			    (do_arraylen && ELEM(prop_dst->type, IDP_ARRAY, IDP_IDPARRAY) && (prop_src->len != prop_dst->len)))
+			{
+				IDP_FreeFromGroup(dst, prop_dst);
+				prop_dst = IDP_CopyProperty(prop_src);
+
+				dst->len++;
+				BLI_insertlinkbefore(&dst->data.group, prop_dst_next, prop_dst);
+			}
+			else if (prop_dst->type == IDP_GROUP) {
+				IDP_SyncGroupTypes(prop_dst, prop_src, do_arraylen);
+			}
+		}
+		else {
+			IDP_FreeFromGroup(dst, prop_dst);
+		}
+	}
+}
+
+/**
+ * Replaces all properties with the same name in a destination group from a source group.
  */
-void IDP_ReplaceGroupInGroup(IDProperty *dest, IDProperty *src)
+void IDP_ReplaceGroupInGroup(IDProperty *dest, const IDProperty *src)
 {
 	IDProperty *loop, *prop;
+
+	BLI_assert(dest->type == IDP_GROUP);
+	BLI_assert(src->type == IDP_GROUP);
+
 	for (prop = src->data.group.first; prop; prop = prop->next) {
 		for (loop = dest->data.group.first; loop; loop = loop->next) {
-			if (strcmp(loop->name, prop->name) == 0) {
+			if (STREQ(loop->name, prop->name)) {
 				IDProperty *copy = IDP_CopyProperty(prop);
 
-				BLI_insertlink(&dest->data.group, loop, copy);
+				BLI_insertlinkafter(&dest->data.group, loop, copy);
 
 				BLI_remlink(&dest->data.group, loop);
 				IDP_FreeProperty(loop);
@@ -472,19 +573,23 @@ void IDP_ReplaceGroupInGroup(IDProperty *dest, IDProperty *src)
 		}
 	}
 }
-/*
- * replaces a property with the same name in a group, or adds 
- * it if the properly doesn't exist.
+
+/**
+ * Checks if a property with the same name as prop exists, and if so replaces it.
+ * Use this to preserve order!
  */
-void IDP_ReplaceInGroup(IDProperty *group, IDProperty *prop)
+void IDP_ReplaceInGroup_ex(IDProperty *group, IDProperty *prop, IDProperty *prop_exist)
 {
-	IDProperty *loop;
-	if ((loop = IDP_GetPropertyFromGroup(group, prop->name))) {
-		BLI_insertlink(&group->data.group, loop, prop);
+	BLI_assert(group->type == IDP_GROUP);
+
+	BLI_assert(prop_exist == IDP_GetPropertyFromGroup(group, prop->name));
+
+	if ((prop_exist = IDP_GetPropertyFromGroup(group, prop->name))) {
+		BLI_insertlinkafter(&group->data.group, prop_exist, prop);
 		
-		BLI_remlink(&group->data.group, loop);
-		IDP_FreeProperty(loop);
-		MEM_freeN(loop);
+		BLI_remlink(&group->data.group, prop_exist);
+		IDP_FreeProperty(prop_exist);
+		MEM_freeN(prop_exist);
 	}
 	else {
 		group->len++;
@@ -492,78 +597,123 @@ void IDP_ReplaceInGroup(IDProperty *group, IDProperty *prop)
 	}
 }
 
-/* returns 0 if an id property with the same name exists and it failed,
- * or 1 if it succeeded in adding to the group.*/
-int IDP_AddToGroup(IDProperty *group, IDProperty *prop)
+void IDP_ReplaceInGroup(IDProperty *group, IDProperty *prop)
 {
+	IDProperty *prop_exist = IDP_GetPropertyFromGroup(group, prop->name);
+
+	IDP_ReplaceInGroup_ex(group, prop, prop_exist);
+}
+
+/**
+ * If a property is missing in \a dest, add it.
+ */
+void IDP_MergeGroup(IDProperty *dest, const IDProperty *src, const bool do_overwrite)
+{
+	IDProperty *prop;
+
+	BLI_assert(dest->type == IDP_GROUP);
+	BLI_assert(src->type == IDP_GROUP);
+
+	if (do_overwrite) {
+		for (prop = src->data.group.first; prop; prop = prop->next) {
+			IDProperty *copy = IDP_CopyProperty(prop);
+			IDP_ReplaceInGroup(dest, copy);
+		}
+	}
+	else {
+		for (prop = src->data.group.first; prop; prop = prop->next) {
+			if (IDP_GetPropertyFromGroup(dest, prop->name) == NULL) {
+				IDProperty *copy = IDP_CopyProperty(prop);
+				dest->len++;
+				BLI_addtail(&dest->data.group, copy);
+			}
+		}
+	}
+}
+
+/**
+ * This function has a sanity check to make sure ID properties with the same name don't
+ * get added to the group.
+ *
+ * The sanity check just means the property is not added to the group if another property
+ * exists with the same name; the client code using ID properties then needs to detect this
+ * (the function that adds new properties to groups, IDP_AddToGroup, returns false if a property can't
+ * be added to the group, and true if it can) and free the property.
+ *
+ * Currently the code to free ID properties is designed to leave the actual struct
+ * you pass it un-freed, this is needed for how the system works.  This means
+ * to free an ID property, you first call IDP_FreeProperty then MEM_freeN the
+ * struct.  In the future this will just be IDP_FreeProperty and the code will
+ * be reorganized to work properly.
+ */
+bool IDP_AddToGroup(IDProperty *group, IDProperty *prop)
+{
+	BLI_assert(group->type == IDP_GROUP);
+
 	if (IDP_GetPropertyFromGroup(group, prop->name) == NULL) {
 		group->len++;
 		BLI_addtail(&group->data.group, prop);
-		return 1;
+		return true;
 	}
 
-	return 0;
+	return false;
 }
 
-int IDP_InsertToGroup(IDProperty *group, IDProperty *previous, IDProperty *pnew)
+/**
+ * This is the same as IDP_AddToGroup, only you pass an item
+ * in the group list to be inserted after.
+ */
+bool IDP_InsertToGroup(IDProperty *group, IDProperty *previous, IDProperty *pnew)
 {
+	BLI_assert(group->type == IDP_GROUP);
+
 	if (IDP_GetPropertyFromGroup(group, pnew->name) == NULL) {
 		group->len++;
-		BLI_insertlink(&group->data.group, previous, pnew);
-		return 1;
+		BLI_insertlinkafter(&group->data.group, previous, pnew);
+		return true;
 	}
 
-	return 0;
+	return false;
 }
 
-void IDP_RemFromGroup(IDProperty *group, IDProperty *prop)
+/**
+ * \note this does not free the property!!
+ *
+ * To free the property, you have to do:
+ * IDP_FreeProperty(prop); //free all subdata
+ * MEM_freeN(prop); //free property struct itself
+ */
+void IDP_RemoveFromGroup(IDProperty *group, IDProperty *prop)
 {
+	BLI_assert(group->type == IDP_GROUP);
+
 	group->len--;
 	BLI_remlink(&group->data.group, prop);
 }
 
-IDProperty *IDP_GetPropertyFromGroup(IDProperty *prop, const char *name)
+/**
+ * Removes the property from the group and frees it.
+ */
+void IDP_FreeFromGroup(IDProperty *group, IDProperty *prop)
 {
-	return (IDProperty *)BLI_findstring(&prop->data.group, name, offsetof(IDProperty, name));
+	IDP_RemoveFromGroup(group, prop);
+	IDP_FreeProperty(prop);
+	MEM_freeN(prop);
 }
 
+IDProperty *IDP_GetPropertyFromGroup(IDProperty *prop, const char *name)
+{
+	BLI_assert(prop->type == IDP_GROUP);
+
+	return (IDProperty *)BLI_findstring(&prop->data.group, name, offsetof(IDProperty, name));
+}
+/** same as above but ensure type match */
 IDProperty *IDP_GetPropertyTypeFromGroup(IDProperty *prop, const char *name, const char type)
 {
 	IDProperty *idprop = IDP_GetPropertyFromGroup(prop, name);
 	return (idprop && idprop->type == type) ? idprop : NULL;
 }
 
-typedef struct IDPIter {
-	void *next;
-	IDProperty *parent;
-} IDPIter;
-
-void *IDP_GetGroupIterator(IDProperty *prop)
-{
-	IDPIter *iter = MEM_callocN(sizeof(IDPIter), "IDPIter");
-	iter->next = prop->data.group.first;
-	iter->parent = prop;
-	return (void *) iter;
-}
-
-IDProperty *IDP_GroupIterNext(void *vself)
-{
-	IDPIter *self = (IDPIter *) vself;
-	Link *next = (Link *) self->next;
-	if (self->next == NULL) {
-		MEM_freeN(self);
-		return NULL;
-	}
-
-	self->next = next->next;
-	return (void *) next;
-}
-
-void IDP_FreeIterBeforeEnd(void *vself)
-{
-	MEM_freeN(vself);
-}
-
 /* Ok, the way things work, Groups free the ID Property structs of their children.
  * This is because all ID Property freeing functions free only direct data (not the ID Property
  * struct itself), but for Groups the child properties *are* considered
@@ -571,15 +721,22 @@ void IDP_FreeIterBeforeEnd(void *vself)
 static void IDP_FreeGroup(IDProperty *prop)
 {
 	IDProperty *loop;
+
+	BLI_assert(prop->type == IDP_GROUP);
 	for (loop = prop->data.group.first; loop; loop = loop->next) {
 		IDP_FreeProperty(loop);
 	}
 	BLI_freelistN(&prop->data.group);
 }
+/** \} */
 
 
-/*-------- Main Functions --------*/
-IDProperty *IDP_CopyProperty(IDProperty *prop)
+/* -------------------------------------------------------------------- */
+/* Main Functions */
+
+/** \name IDProperty Main API
+ * \{ */
+IDProperty *IDP_CopyProperty(const IDProperty *prop)
 {
 	switch (prop->type) {
 		case IDP_GROUP: return IDP_CopyGroup(prop);
@@ -590,7 +747,12 @@ IDProperty *IDP_CopyProperty(IDProperty *prop)
 	}
 }
 
-IDProperty *IDP_GetProperties(ID *id, int create_if_needed)
+/**
+ * Get the Group property that contains the id properties for ID id.  Set create_if_needed
+ * to create the Group property and attach it to id if it doesn't exist; otherwise
+ * the function will return NULL if there's no Group property attached to the ID.
+ */
+IDProperty *IDP_GetProperties(ID *id, const bool create_if_needed)
 {
 	if (id->properties) {
 		return id->properties;
@@ -608,62 +770,119 @@ IDProperty *IDP_GetProperties(ID *id, int create_if_needed)
 	}
 }
 
-int IDP_EqualsProperties(IDProperty *prop1, IDProperty *prop2)
+/**
+ * \param is_strict When false treat missing items as a match */
+bool IDP_EqualsProperties_ex(IDProperty *prop1, IDProperty *prop2, const bool is_strict)
 {
 	if (prop1 == NULL && prop2 == NULL)
-		return 1;
+		return true;
 	else if (prop1 == NULL || prop2 == NULL)
-		return 0;
+		return is_strict ? false : true;
 	else if (prop1->type != prop2->type)
-		return 0;
+		return false;
 
-	if (prop1->type == IDP_INT)
-		return (IDP_Int(prop1) == IDP_Int(prop2));
-	else if (prop1->type == IDP_FLOAT)
-		return (IDP_Float(prop1) == IDP_Float(prop2));
-	else if (prop1->type == IDP_DOUBLE)
-		return (IDP_Double(prop1) == IDP_Double(prop2));
-	else if (prop1->type == IDP_STRING)
-		return ((prop1->len == prop2->len) && strncmp(IDP_String(prop1), IDP_String(prop2), prop1->len) == 0);
-	else if (prop1->type == IDP_ARRAY) {
-		if (prop1->len == prop2->len && prop1->subtype == prop2->subtype)
-			return memcmp(IDP_Array(prop1), IDP_Array(prop2), idp_size_table[(int)prop1->subtype] * prop1->len);
-		else
-			return 0;
-	}
-	else if (prop1->type == IDP_GROUP) {
-		IDProperty *link1, *link2;
-
-		if (BLI_countlist(&prop1->data.group) != BLI_countlist(&prop2->data.group))
-			return 0;
-
-		for (link1 = prop1->data.group.first; link1; link1 = link1->next) {
-			link2 = IDP_GetPropertyFromGroup(prop2, link1->name);
-
-			if (!IDP_EqualsProperties(link1, link2))
-				return 0;
+	switch (prop1->type) {
+		case IDP_INT:
+			return (IDP_Int(prop1) == IDP_Int(prop2));
+		case IDP_FLOAT:
+#if !defined(NDEBUG) && defined(WITH_PYTHON)
+			{
+				float p1 = IDP_Float(prop1);
+				float p2 = IDP_Float(prop2);
+				if ((p1 != p2) && ((fabsf(p1 - p2) / max_ff(p1, p2)) < 0.001f)) {
+					printf("WARNING: Comparing two float properties that have nearly the same value (%f vs. %f)\n", p1, p2);
+					printf("    p1: ");
+					IDP_spit(prop1);
+					printf("    p2: ");
+					IDP_spit(prop2);
+				}
+			}
+#endif
+			return (IDP_Float(prop1) == IDP_Float(prop2));
+		case IDP_DOUBLE:
+			return (IDP_Double(prop1) == IDP_Double(prop2));
+		case IDP_STRING:
+		{
+			return (((prop1->len == prop2->len) &&
+			         strncmp(IDP_String(prop1), IDP_String(prop2), (size_t)prop1->len) == 0));
 		}
+		case IDP_ARRAY:
+			if (prop1->len == prop2->len && prop1->subtype == prop2->subtype) {
+				return (memcmp(IDP_Array(prop1),
+				               IDP_Array(prop2),
+				               idp_size_table[(int)prop1->subtype] * (size_t)prop1->len) == 0);
+			}
+			return false;
+		case IDP_GROUP:
+		{
+			IDProperty *link1, *link2;
 
-		return 1;
-	}
-	else if (prop1->type == IDP_IDPARRAY) {
-		IDProperty *array1 = IDP_IDPArray(prop1);
-		IDProperty *array2 = IDP_IDPArray(prop2);
-		int i;
+			if (is_strict && prop1->len != prop2->len)
+				return false;
 
-		if (prop1->len != prop2->len)
-			return 0;
-		
-		for (i = 0; i < prop1->len; i++)
-			if (!IDP_EqualsProperties(&array1[i], &array2[i]))
-				return 0;
+			for (link1 = prop1->data.group.first; link1; link1 = link1->next) {
+				link2 = IDP_GetPropertyFromGroup(prop2, link1->name);
+
+				if (!IDP_EqualsProperties_ex(link1, link2, is_strict))
+					return false;
+			}
+
+			return true;
+		}
+		case IDP_IDPARRAY:
+		{
+			IDProperty *array1 = IDP_IDPArray(prop1);
+			IDProperty *array2 = IDP_IDPArray(prop2);
+			int i;
+
+			if (prop1->len != prop2->len)
+				return false;
+
+			for (i = 0; i < prop1->len; i++)
+				if (!IDP_EqualsProperties(&array1[i], &array2[i]))
+					return false;
+			return true;
+		}
+		default:
+			/* should never get here */
+			BLI_assert(0);
+			break;
 	}
-	
-	return 1;
+
+	return true;
 }
 
-/* 'val' is never NULL, don't check */
-IDProperty *IDP_New(const int type, const IDPropertyTemplate *val, const char *name)
+bool IDP_EqualsProperties(IDProperty *prop1, IDProperty *prop2)
+{
+	return IDP_EqualsProperties_ex(prop1, prop2, true);
+}
+
+/**
+ * Allocate a new ID.
+ *
+ * This function takes three arguments: the ID property type, a union which defines
+ * it's initial value, and a name.
+ *
+ * The union is simple to use; see the top of this header file for its definition.
+ * An example of using this function:
+ *
+ *     IDPropertyTemplate val;
+ *     IDProperty *group, *idgroup, *color;
+ *     group = IDP_New(IDP_GROUP, val, "group1"); //groups don't need a template.
+ *
+ *     val.array.len = 4
+ *     val.array.type = IDP_FLOAT;
+ *     color = IDP_New(IDP_ARRAY, val, "color1");
+ *
+ *     idgroup = IDP_GetProperties(some_id, 1);
+ *     IDP_AddToGroup(idgroup, color);
+ *     IDP_AddToGroup(idgroup, group);
+ *
+ * Note that you MUST either attach the id property to an id property group with
+ * IDP_AddToGroup or MEM_freeN the property, doing anything else might result in
+ * a memory leak.
+ */
+IDProperty *IDP_New(const char type, const IDPropertyTemplate *val, const char *name)
 {
 	IDProperty *prop = NULL;
 
@@ -690,14 +909,14 @@ IDProperty *IDP_New(const int type, const IDPropertyTemplate *val, const char *n
 			{
 				prop = MEM_callocN(sizeof(IDProperty), "IDProperty array");
 				prop->subtype = val->array.type;
-				if (val->array.len)
-					prop->data.pointer = MEM_callocN(idp_size_table[val->array.type] * val->array.len, "id property array");
+				if (val->array.len) {
+					prop->data.pointer = MEM_callocN(
+					        idp_size_table[val->array.type] * (size_t)val->array.len, "id property array");
+				}
 				prop->len = prop->totallen = val->array.len;
 				break;
 			}
-			else {
-				return NULL;
-			}
+			return NULL;
 		}
 		case IDP_STRING:
 		{
@@ -707,28 +926,30 @@ IDProperty *IDP_New(const int type, const IDPropertyTemplate *val, const char *n
 			if (val->string.subtype == IDP_STRING_SUB_BYTE) {
 				/* note, intentionally not null terminated */
 				if (st == NULL) {
-					prop->data.pointer = MEM_callocN(DEFAULT_ALLOC_FOR_NULL_STRINGS, "id property string 1");
+					prop->data.pointer = MEM_mallocN(DEFAULT_ALLOC_FOR_NULL_STRINGS, "id property string 1");
+					*IDP_String(prop) = '\0';
 					prop->totallen = DEFAULT_ALLOC_FOR_NULL_STRINGS;
 					prop->len = 0;
 				}
 				else {
-					prop->data.pointer = MEM_mallocN(val->string.len, "id property string 2");
+					prop->data.pointer = MEM_mallocN((size_t)val->string.len, "id property string 2");
 					prop->len = prop->totallen = val->string.len;
-					memcpy(prop->data.pointer, st, val->string.len);
+					memcpy(prop->data.pointer, st, (size_t)val->string.len);
 				}
 				prop->subtype = IDP_STRING_SUB_BYTE;
 			}
 			else {
 				if (st == NULL) {
-					prop->data.pointer = MEM_callocN(DEFAULT_ALLOC_FOR_NULL_STRINGS, "id property string 1");
+					prop->data.pointer = MEM_mallocN(DEFAULT_ALLOC_FOR_NULL_STRINGS, "id property string 1");
+					*IDP_String(prop) = '\0';
 					prop->totallen = DEFAULT_ALLOC_FOR_NULL_STRINGS;
 					prop->len = 1; /*NULL string, has len of 1 to account for null byte.*/
 				}
 				else {
-					int stlen = strlen(st) + 1;
-					prop->data.pointer = MEM_mallocN(stlen, "id property string 3");
+					int stlen = (int)strlen(st) + 1;
+					prop->data.pointer = MEM_mallocN((size_t)stlen, "id property string 3");
 					prop->len = prop->totallen = stlen;
-					memcpy(prop->data.pointer, st, stlen);
+					memcpy(prop->data.pointer, st, (size_t)stlen);
 				}
 				prop->subtype = IDP_STRING_SUB_UTF8;
 			}
@@ -753,9 +974,11 @@ IDProperty *IDP_New(const int type, const IDPropertyTemplate *val, const char *n
 	return prop;
 }
 
-/* NOTE: this will free all child properties including list arrays and groups!
+/**
+ * \note this will free all child properties of list arrays and groups!
  * Also, note that this does NOT unlink anything!  Plus it doesn't free
- * the actual IDProperty struct either.*/
+ * the actual struct IDProperty struct either.
+ */
 void IDP_FreeProperty(IDProperty *prop)
 {
 	switch (prop->type) {
@@ -774,12 +997,25 @@ void IDP_FreeProperty(IDProperty *prop)
 	}
 }
 
-/* Unlinks any IDProperty<->ID linkage that might be going on.
- * note: currently unused.*/
+void IDP_ClearProperty(IDProperty *prop)
+{
+	IDP_FreeProperty(prop);
+	prop->data.pointer = NULL;
+	prop->len = prop->totallen = 0;
+}
+
+/**
+ * Unlinks any struct IDProperty<->ID linkage that might be going on.
+ *
+ * \note currently unused
+ */
 void IDP_UnlinkProperty(IDProperty *prop)
 {
 	switch (prop->type) {
 		case IDP_ID:
 			IDP_UnlinkID(prop);
+			break;
 	}
 }
+
+/** \} */
