@@ -37,7 +37,6 @@
 #include "DNA_object_force.h"
 #include "DNA_meshdata_types.h"
 
-#include "BLI_threads.h"
 #include "BLI_math.h"
 #include "BLI_linklist.h"
 #include "BLI_utildefines.h"
@@ -48,6 +47,10 @@
 #include "BKE_global.h"
 
 
+#ifdef __GNUC__
+#  pragma GCC diagnostic ignored "-Wtype-limits"
+#endif
+
 #ifdef _OPENMP
 #  define CLOTH_OPENMP_LIMIT 512
 #endif
@@ -141,14 +144,6 @@ DO_INLINE void mul_fvector_S(float to[3], float from[3], float scalar)
 	to[1] = from[1] * scalar;
 	to[2] = from[2] * scalar;
 }
-/* simple cross product */
-/* STATUS: verified */
-DO_INLINE void cross_fvector(float to[3], float vectorA[3], float vectorB[3])
-{
-	to[0] = vectorA[1] * vectorB[2] - vectorA[2] * vectorB[1];
-	to[1] = vectorA[2] * vectorB[0] - vectorA[0] * vectorB[2];
-	to[2] = vectorA[0] * vectorB[1] - vectorA[1] * vectorB[0];
-}
 /* simple v^T * v product ("outer product") */
 /* STATUS: HAS TO BE verified (*should* work) */
 DO_INLINE void mul_fvectorT_fvector(float to[3][3], float vectorA[3], float vectorB[3])
@@ -168,7 +163,7 @@ DO_INLINE void mul_fvectorT_fvectorS(float to[3][3], float vectorA[3], float vec
 	mul_fvector_S(to[2], to[2], aS);
 }
 
-
+#if 0
 /* printf vector[3] on console: for debug output */
 static void print_fvector(float m3[3])
 {
@@ -186,6 +181,8 @@ DO_INLINE void print_lfvector(float (*fLongVector)[3], unsigned int verts)
 		print_fvector(fLongVector[i]);
 	}
 }
+#endif
+
 /* create long vector */
 DO_INLINE lfVector *create_lfvector(unsigned int verts)
 {
@@ -332,6 +329,7 @@ DO_INLINE void initdiag_fmatrixS(float to[3][3], float aS)
 	to[2][2] = aS;
 }
 
+#if 0
 /* calculate determinant of 3x3 matrix */
 DO_INLINE float det_fmatrix(float m[3][3])
 {
@@ -366,6 +364,7 @@ DO_INLINE void inverse_fmatrix(float to[3][3], float from[3][3])
 	}
 
 }
+#endif
 
 /* 3x3 matrix multiplied by a scalar */
 /* STATUS: verified */
@@ -393,14 +392,6 @@ DO_INLINE void mul_fmatrix_fvector(float *to, float matrix[3][3], float from[3])
 	to[1] = dot_v3v3(matrix[1], from);
 	to[2] = dot_v3v3(matrix[2], from);
 }
-/* 3x3 matrix multiplied by a 3x3 matrix */
-/* STATUS: verified */
-DO_INLINE void mul_fmatrix_fmatrix(float to[3][3], float matrixA[3][3], float matrixB[3][3])
-{
-	mul_fvector_fmatrix(to[0], matrixA[0], matrixB);
-	mul_fvector_fmatrix(to[1], matrixA[1], matrixB);
-	mul_fvector_fmatrix(to[2], matrixA[2], matrixB);
-}
 /* 3x3 matrix addition with 3x3 matrix */
 DO_INLINE void add_fmatrix_fmatrix(float to[3][3], float matrixA[3][3], float matrixB[3][3])
 {
@@ -408,27 +399,6 @@ DO_INLINE void add_fmatrix_fmatrix(float to[3][3], float matrixA[3][3], float ma
 	VECADD(to[1], matrixA[1], matrixB[1]);
 	VECADD(to[2], matrixA[2], matrixB[2]);
 }
-/* 3x3 matrix add-addition with 3x3 matrix */
-DO_INLINE void addadd_fmatrix_fmatrix(float to[3][3], float matrixA[3][3], float matrixB[3][3])
-{
-	VECADDADD(to[0], matrixA[0], matrixB[0]);
-	VECADDADD(to[1], matrixA[1], matrixB[1]);
-	VECADDADD(to[2], matrixA[2], matrixB[2]);
-}
-/* 3x3 matrix sub-addition with 3x3 matrix */
-DO_INLINE void addsub_fmatrixS_fmatrixS(float to[3][3], float matrixA[3][3], float aS, float matrixB[3][3], float bS)
-{
-	VECADDSUBSS(to[0], matrixA[0], aS, matrixB[0], bS);
-	VECADDSUBSS(to[1], matrixA[1], aS, matrixB[1], bS);
-	VECADDSUBSS(to[2], matrixA[2], aS, matrixB[2], bS);
-}
-/* A -= B + C (3x3 matrix sub-addition with 3x3 matrix) */
-DO_INLINE void subadd_fmatrix_fmatrix(float to[3][3], float matrixA[3][3], float matrixB[3][3])
-{
-	VECSUBADD(to[0], matrixA[0], matrixB[0]);
-	VECSUBADD(to[1], matrixA[1], matrixB[1]);
-	VECSUBADD(to[2], matrixA[2], matrixB[2]);
-}
 /* A -= B*x + C*y (3x3 matrix sub-addition with 3x3 matrix) */
 DO_INLINE void subadd_fmatrixS_fmatrixS(float to[3][3], float matrixA[3][3], float aS, float matrixB[3][3], float bS)
 {
@@ -443,44 +413,9 @@ DO_INLINE void sub_fmatrix_fmatrix(float to[3][3], float matrixA[3][3], float ma
 	sub_v3_v3v3(to[1], matrixA[1], matrixB[1]);
 	sub_v3_v3v3(to[2], matrixA[2], matrixB[2]);
 }
-/* A += B - C (3x3 matrix add-subtraction with 3x3 matrix) */
-DO_INLINE void addsub_fmatrix_fmatrix(float to[3][3], float matrixA[3][3], float matrixB[3][3])
-{
-	VECADDSUB(to[0], matrixA[0], matrixB[0]);
-	VECADDSUB(to[1], matrixA[1], matrixB[1]);
-	VECADDSUB(to[2], matrixA[2], matrixB[2]);
-}
 /////////////////////////////////////////////////////////////////
 // special functions
 /////////////////////////////////////////////////////////////////
-/* a vector multiplied and added to/by a 3x3 matrix */
-DO_INLINE void muladd_fvector_fmatrix(float to[3], float from[3], float matrix[3][3])
-{
-	to[0] += matrix[0][0]*from[0] + matrix[1][0]*from[1] + matrix[2][0]*from[2];
-	to[1] += matrix[0][1]*from[0] + matrix[1][1]*from[1] + matrix[2][1]*from[2];
-	to[2] += matrix[0][2]*from[0] + matrix[1][2]*from[1] + matrix[2][2]*from[2];
-}
-/* 3x3 matrix multiplied and added  to/by a 3x3 matrix  and added to another 3x3 matrix */
-DO_INLINE void muladd_fmatrix_fmatrix(float to[3][3], float matrixA[3][3], float matrixB[3][3])
-{
-	muladd_fvector_fmatrix(to[0], matrixA[0], matrixB);
-	muladd_fvector_fmatrix(to[1], matrixA[1], matrixB);
-	muladd_fvector_fmatrix(to[2], matrixA[2], matrixB);
-}
-/* a vector multiplied and sub'd to/by a 3x3 matrix */
-DO_INLINE void mulsub_fvector_fmatrix(float to[3], float from[3], float matrix[3][3])
-{
-	to[0] -= matrix[0][0]*from[0] + matrix[1][0]*from[1] + matrix[2][0]*from[2];
-	to[1] -= matrix[0][1]*from[0] + matrix[1][1]*from[1] + matrix[2][1]*from[2];
-	to[2] -= matrix[0][2]*from[0] + matrix[1][2]*from[1] + matrix[2][2]*from[2];
-}
-/* 3x3 matrix multiplied and sub'd  to/by a 3x3 matrix  and added to another 3x3 matrix */
-DO_INLINE void mulsub_fmatrix_fmatrix(float to[3][3], float matrixA[3][3], float matrixB[3][3])
-{
-	mulsub_fvector_fmatrix(to[0], matrixA[0], matrixB);
-	mulsub_fvector_fmatrix(to[1], matrixA[1], matrixB);
-	mulsub_fvector_fmatrix(to[2], matrixA[2], matrixB);
-}
 /* 3x3 matrix multiplied+added by a vector */
 /* STATUS: verified */
 DO_INLINE void muladd_fmatrix_fvector(float to[3], float matrix[3][3], float from[3])
@@ -489,13 +424,6 @@ DO_INLINE void muladd_fmatrix_fvector(float to[3], float matrix[3][3], float fro
 	to[1] += dot_v3v3(matrix[1], from);
 	to[2] += dot_v3v3(matrix[2], from);
 }
-/* 3x3 matrix multiplied+sub'ed by a vector */
-DO_INLINE void mulsub_fmatrix_fvector(float to[3], float matrix[3][3], float from[3])
-{
-	to[0] -= dot_v3v3(matrix[0], from);
-	to[1] -= dot_v3v3(matrix[1], from);
-	to[2] -= dot_v3v3(matrix[2], from);
-}
 /////////////////////////////////////////////////////////////////
 
 ///////////////////////////
@@ -564,15 +492,6 @@ DO_INLINE void initdiag_bfmatrix(fmatrix3x3 *matrix, float m3[3][3])
 	}
 }
 
-/* multiply big matrix with scalar*/
-DO_INLINE void mul_bfmatrix_S(fmatrix3x3 *matrix, float scalar)
-{
-	unsigned int i = 0;
-	for (i = 0; i < matrix[0].vcount+matrix[0].scount; i++) {
-		mul_fmatrix_S(matrix[i].m, scalar);
-	}
-}
-
 /* SPARSE SYMMETRIC multiply big matrix with long vector*/
 /* STATUS: verified */
 DO_INLINE void mul_bfmatrix_lfvector( float (*to)[3], fmatrix3x3 *from, lfVector *fLongVector)
@@ -590,7 +509,7 @@ DO_INLINE void mul_bfmatrix_lfvector( float (*to)[3], fmatrix3x3 *from, lfVector
 			for (i = from[0].vcount; i < from[0].vcount+from[0].scount; i++) {
 				muladd_fmatrix_fvector(to[from[i].c], from[i].m, fLongVector[from[i].r]);
 			}
-		}	
+		}
 #pragma omp section
 		{
 			for (i = 0; i < from[0].vcount+from[0].scount; i++) {
@@ -605,83 +524,6 @@ DO_INLINE void mul_bfmatrix_lfvector( float (*to)[3], fmatrix3x3 *from, lfVector
 	
 }
 
-/* SPARSE SYMMETRIC multiply big matrix with long vector (for diagonal preconditioner) */
-/* STATUS: verified */
-DO_INLINE void mul_prevfmatrix_lfvector( float (*to)[3], fmatrix3x3 *from, lfVector *fLongVector)
-{
-	unsigned int i = 0;
-	
-	for (i = 0; i < from[0].vcount; i++) {
-		mul_fmatrix_fvector(to[from[i].r], from[i].m, fLongVector[from[i].c]);
-	}
-}
-
-/* SPARSE SYMMETRIC add big matrix with big matrix: A = B + C*/
-DO_INLINE void add_bfmatrix_bfmatrix( fmatrix3x3 *to, fmatrix3x3 *from,  fmatrix3x3 *matrix)
-{
-	unsigned int i = 0;
-
-	/* process diagonal elements */
-	for (i = 0; i < matrix[0].vcount+matrix[0].scount; i++) {
-		add_fmatrix_fmatrix(to[i].m, from[i].m, matrix[i].m);	
-	}
-
-}
-/* SPARSE SYMMETRIC add big matrix with big matrix: A += B + C */
-DO_INLINE void addadd_bfmatrix_bfmatrix( fmatrix3x3 *to, fmatrix3x3 *from,  fmatrix3x3 *matrix)
-{
-	unsigned int i = 0;
-
-	/* process diagonal elements */
-	for (i = 0; i < matrix[0].vcount+matrix[0].scount; i++) {
-		addadd_fmatrix_fmatrix(to[i].m, from[i].m, matrix[i].m);	
-	}
-
-}
-/* SPARSE SYMMETRIC subadd big matrix with big matrix: A -= B + C */
-DO_INLINE void subadd_bfmatrix_bfmatrix( fmatrix3x3 *to, fmatrix3x3 *from,  fmatrix3x3 *matrix)
-{
-	unsigned int i = 0;
-
-	/* process diagonal elements */
-	for (i = 0; i < matrix[0].vcount+matrix[0].scount; i++) {
-		subadd_fmatrix_fmatrix(to[i].m, from[i].m, matrix[i].m);	
-	}
-
-}
-/*  A = B - C (SPARSE SYMMETRIC sub big matrix with big matrix) */
-DO_INLINE void sub_bfmatrix_bfmatrix( fmatrix3x3 *to, fmatrix3x3 *from,  fmatrix3x3 *matrix)
-{
-	unsigned int i = 0;
-
-	/* process diagonal elements */
-	for (i = 0; i < matrix[0].vcount+matrix[0].scount; i++) {
-		sub_fmatrix_fmatrix(to[i].m, from[i].m, matrix[i].m);	
-	}
-
-}
-/* SPARSE SYMMETRIC sub big matrix with big matrix S (special constraint matrix with limited entries) */
-DO_INLINE void sub_bfmatrix_Smatrix( fmatrix3x3 *to, fmatrix3x3 *from,  fmatrix3x3 *matrix)
-{
-	unsigned int i = 0;
-
-	/* process diagonal elements */
-	for (i = 0; i < matrix[0].vcount; i++) {
-		sub_fmatrix_fmatrix(to[matrix[i].c].m, from[matrix[i].c].m, matrix[i].m);	
-	}
-
-}
-/* A += B - C (SPARSE SYMMETRIC addsub big matrix with big matrix) */
-DO_INLINE void addsub_bfmatrix_bfmatrix( fmatrix3x3 *to, fmatrix3x3 *from,  fmatrix3x3 *matrix)
-{
-	unsigned int i = 0;
-
-	/* process diagonal elements */
-	for (i = 0; i < matrix[0].vcount+matrix[0].scount; i++) {
-		addsub_fmatrix_fmatrix(to[i].m, from[i].m, matrix[i].m);	
-	}
-
-}
 /* SPARSE SYMMETRIC sub big matrix with big matrix*/
 /* A -= B * float + C * float --> for big matrix */
 /* VERIFIED */
@@ -691,7 +533,7 @@ DO_INLINE void subadd_bfmatrixS_bfmatrixS( fmatrix3x3 *to, fmatrix3x3 *from, flo
 
 	/* process diagonal elements */
 	for (i = 0; i < matrix[0].vcount+matrix[0].scount; i++) {
-		subadd_fmatrixS_fmatrixS(to[i].m, from[i].m, aS, matrix[i].m, bS);	
+		subadd_fmatrixS_fmatrixS(to[i].m, from[i].m, aS, matrix[i].m, bS);
 	}
 
 }
@@ -711,7 +553,7 @@ static void update_matrixS(ClothVertex *verts, int numverts, fmatrix3x3 *S)
 	int i = 0;
 
 	/* Clear matrix from old vertex constraints */
-	for(i = 0; i < S[0].vcount; i++)
+	for (i = 0; i < S[0].vcount; i++)
 		S[i].c = S[i].r = 0;
 
 	/* Set new vertex constraints */
@@ -740,7 +582,7 @@ int implicit_init(Object *UNUSED(ob), ClothModifierData *clmd)
 		printf("implicit_init\n");
 
 	// init memory guard
-	// MEMORY_BASE.first = MEMORY_BASE.last = NULL;
+	// BLI_listbase_clear(&MEMORY_BASE);
 
 	cloth = (Cloth *)clmd->clothObject;
 	verts = cloth->verts;
@@ -749,7 +591,7 @@ int implicit_init(Object *UNUSED(ob), ClothModifierData *clmd)
 	id = (Implicit_Data *)MEM_callocN(sizeof(Implicit_Data), "implicit vecmat");
 	cloth->implicit = id;
 
-	/* process diagonal elements */		
+	/* process diagonal elements */
 	id->A = create_bfmatrix(cloth->numverts, cloth->numsprings);
 	id->dFdV = create_bfmatrix(cloth->numverts, cloth->numsprings);
 	id->dFdX = create_bfmatrix(cloth->numverts, cloth->numsprings);
@@ -770,11 +612,10 @@ int implicit_init(Object *UNUSED(ob), ClothModifierData *clmd)
 	id->z = create_lfvector(cloth->numverts);
 	
 	id->S[0].vcount = 0;
+	update_matrixS(verts, cloth->numverts, id->S);
 
 	for (i = 0; i < cloth->numverts; i++) {
 		id->A[i].r = id->A[i].c = id->dFdV[i].r = id->dFdV[i].c = id->dFdX[i].r = id->dFdX[i].c = id->P[i].c = id->P[i].r = id->Pinv[i].c = id->Pinv[i].r = id->bigI[i].c = id->bigI[i].r = id->M[i].r = id->M[i].c = i;
-
-		update_matrixS(verts, cloth->numverts, id->S);
 		
 		initdiag_fmatrixS(id->M[i].m, verts[i].mass);
 	}
@@ -844,41 +685,44 @@ int	implicit_free(ClothModifierData *clmd)
 
 DO_INLINE float fb(float length, float L)
 {
-	float x = length/L;
-	return (-11.541f*pow(x, 4)+34.193f*pow(x, 3)-39.083f*pow(x, 2)+23.116f*x-9.713f);
+	float x = length / L;
+	float xx = x * x;
+	float xxx = xx * x;
+	float xxxx = xxx * x;
+	return (-11.541f * xxxx + 34.193f * xxx - 39.083f * xx + 23.116f * x - 9.713f);
 }
 
 DO_INLINE float fbderiv(float length, float L)
 {
 	float x = length/L;
-
-	return (-46.164f*pow(x, 3)+102.579f*pow(x, 2)-78.166f*x+23.116f);
+	float xx = x * x;
+	float xxx = xx * x;
+	return (-46.164f * xxx + 102.579f * xx - 78.166f * x + 23.116f);
 }
 
 DO_INLINE float fbstar(float length, float L, float kb, float cb)
 {
-	float tempfb = kb * fb(length, L);
-
-	float fbstar = cb * (length - L);
+	float tempfb_fl = kb * fb(length, L);
+	float fbstar_fl = cb * (length - L);
 	
-	if (tempfb < fbstar)
-		return fbstar;
+	if (tempfb_fl < fbstar_fl)
+		return fbstar_fl;
 	else
-		return tempfb;		
+		return tempfb_fl;
 }
 
 // function to calculae bending spring force (taken from Choi & Co)
 DO_INLINE float fbstar_jacobi(float length, float L, float kb, float cb)
 {
-	float tempfb = kb * fb(length, L);
-	float fbstar = cb * (length - L);
+	float tempfb_fl = kb * fb(length, L);
+	float fbstar_fl = cb * (length - L);
 
-	if (tempfb < fbstar) {
+	if (tempfb_fl < fbstar_fl) {
 		return cb;
 	}
 	else {
-		return kb * fbderiv(length, L);	
-	}	
+		return kb * fbderiv(length, L);
+	}
 }
 
 DO_INLINE void filter(lfVector *V, fmatrix3x3 *S)
@@ -918,7 +762,7 @@ static int  cg_filtered(lfVector *ldV, fmatrix3x3 *lA, lfVector *lB, lfVector *z
 	cp_lfvector(d, r, numverts);
 
 	s = dot_lfvector(r, r, numverts);
-	starget = s * sqrt(conjgrad_epsilon);
+	starget = s * sqrtf(conjgrad_epsilon);
 
 	while (s>starget && conjgrad_loopcount < conjgrad_looplimit) {
 		// Mul(q, A, d); // q = A*d;
@@ -955,6 +799,7 @@ static int  cg_filtered(lfVector *ldV, fmatrix3x3 *lA, lfVector *lB, lfVector *z
 	return conjgrad_loopcount<conjgrad_looplimit;  // true means we reached desired accuracy in given time - ie stable
 }
 
+#if 0
 // block diagonalizer
 DO_INLINE void BuildPPinv(fmatrix3x3 *lA, fmatrix3x3 *P, fmatrix3x3 *Pinv)
 {
@@ -969,7 +814,6 @@ DO_INLINE void BuildPPinv(fmatrix3x3 *lA, fmatrix3x3 *P, fmatrix3x3 *Pinv)
 		
 	}
 }
-#if 0
 /*
 // version 1.3
 static int cg_filtered_pre(lfVector *dv, fmatrix3x3 *lA, lfVector *lB, lfVector *z, fmatrix3x3 *S, fmatrix3x3 *P, fmatrix3x3 *Pinv)
@@ -1143,30 +987,6 @@ static int cg_filtered_pre(lfVector *dv, fmatrix3x3 *lA, lfVector *lB, lfVector
 }
 #endif
 
-// outer product is NOT cross product!!!
-DO_INLINE void dfdx_spring_type1(float to[3][3], float extent[3], float length, float L, float dot, float k)
-{
-	// dir is unit length direction, rest is spring's restlength, k is spring constant.
-	// return  (outerprod(dir, dir)*k + (I - outerprod(dir, dir))*(k - ((k*L)/length)));
-	float temp[3][3];
-	float temp1 = k*(1.0 - (L/length));	
-	
-	mul_fvectorT_fvectorS(temp, extent, extent, 1.0 / dot);
-	sub_fmatrix_fmatrix(to, I, temp);
-	mul_fmatrix_S(to, temp1);
-	
-	mul_fvectorT_fvectorS(temp, extent, extent, k/ dot);
-	add_fmatrix_fmatrix(to, to, temp);
-	
-	/*
-	mul_fvectorT_fvector(temp, dir, dir);
-	sub_fmatrix_fmatrix(to, I, temp);
-	mul_fmatrix_S(to, k* (1.0f-(L/length)));
-	mul_fmatrix_S(temp, k);
-	add_fmatrix_fmatrix(to, temp, to);
-	*/
-}
-
 DO_INLINE void dfdx_spring_type2(float to[3][3], float dir[3], float length, float L, float k, float cb)
 {
 	// return  outerprod(dir, dir)*fbstar_jacobi(length, L, k, cb);
@@ -1192,17 +1012,6 @@ DO_INLINE void dfdx_spring(float to[3][3],  float dir[3], float length, float L,
 	mul_fmatrix_S(to, -k);
 }
 
-// unused atm
-DO_INLINE void dfdx_damp(float to[3][3],  float dir[3], float length, const float vel[3], float rest, float damping)
-{
-	// inner spring damping   vel is the relative velocity  of the endpoints.  
-	// 	return (I-outerprod(dir, dir)) * (-damping * -(dot(dir, vel)/Max(length, rest)));
-	mul_fvectorT_fvector(to, dir, dir);
-	sub_fmatrix_fmatrix(to, I, to);
-	mul_fmatrix_S(to,  (-damping * -(dot_v3v3(dir, vel)/MAX2(length, rest))));
-
-}
-
 DO_INLINE void cloth_calc_spring_force(ClothModifierData *clmd, ClothSpring *s, lfVector *UNUSED(lF), lfVector *X, lfVector *V, fmatrix3x3 *UNUSED(dFdV), fmatrix3x3 *UNUSED(dFdX), float time)
 {
 	Cloth *cloth = clmd->clothObject;
@@ -1219,7 +1028,7 @@ DO_INLINE void cloth_calc_spring_force(ClothModifierData *clmd, ClothSpring *s,
 	float stretch_force[3] = {0, 0, 0};
 	float bending_force[3] = {0, 0, 0};
 	float damping_force[3] = {0, 0, 0};
-	float nulldfdx[3][3]={ {0, 0, 0}, {0, 0, 0}, {0, 0, 0}};
+	float nulldfdx[3][3] = {{0, 0, 0}, {0, 0, 0}, {0, 0, 0}};
 	
 	float scaling = 0.0;
 
@@ -1233,7 +1042,7 @@ DO_INLINE void cloth_calc_spring_force(ClothModifierData *clmd, ClothSpring *s,
 	sub_v3_v3v3(extent, X[s->kl], X[s->ij]);
 	sub_v3_v3v3(vel, V[s->kl], V[s->ij]);
 	dot = dot_v3v3(extent, extent);
-	length = sqrt(dot);
+	length = sqrtf(dot);
 	
 	s->flags &= ~CLOTH_SPRING_FLAG_NEEDED;
 	
@@ -1247,7 +1056,7 @@ DO_INLINE void cloth_calc_spring_force(ClothModifierData *clmd, ClothSpring *s,
 		s->flags |= CSPRING_FLAG_DEACTIVATE;
 		return;
 	}
-	} 
+	}
 		*/
 		mul_fvector_S(dir, extent, 1.0f/length);
 	}
@@ -1256,18 +1065,28 @@ DO_INLINE void cloth_calc_spring_force(ClothModifierData *clmd, ClothSpring *s,
 	}
 	
 	// calculate force of structural + shear springs
-	if ((s->type & CLOTH_SPRING_TYPE_STRUCTURAL) || (s->type & CLOTH_SPRING_TYPE_SHEAR)) {
+	if ((s->type & CLOTH_SPRING_TYPE_STRUCTURAL) || (s->type & CLOTH_SPRING_TYPE_SHEAR) || (s->type & CLOTH_SPRING_TYPE_SEWING) ) {
 		if (length > L || no_compress) {
 			s->flags |= CLOTH_SPRING_FLAG_NEEDED;
 			
 			k = clmd->sim_parms->structural;
 
-			scaling = k + s->stiffness * ABS(clmd->sim_parms->max_struct-k);
+			scaling = k + s->stiffness * fabsf(clmd->sim_parms->max_struct - k);
 
 			k = scaling / (clmd->sim_parms->avg_spring_len + FLT_EPSILON);
 
 			// TODO: verify, half verified (couldn't see error)
-			mul_fvector_S(stretch_force, dir, k*(length-L));
+			if (s->type & CLOTH_SPRING_TYPE_SEWING) {
+				// sewing springs usually have a large distance at first so clamp the force so we don't get tunnelling through colission objects
+				float force = k*(length-L);
+				if (force > clmd->sim_parms->max_sewing) {
+					force = clmd->sim_parms->max_sewing;
+				}
+				mul_fvector_S(stretch_force, dir, force);
+			}
+			else {
+				mul_fvector_S(stretch_force, dir, k * (length - L));
+			}
 
 			VECADD(s->f, s->f, stretch_force);
 
@@ -1302,13 +1121,13 @@ DO_INLINE void cloth_calc_spring_force(ClothModifierData *clmd, ClothSpring *s,
 		
 		k = clmd->sim_parms->goalspring;
 		
-		scaling = k + s->stiffness * ABS(clmd->sim_parms->max_struct-k);
+		scaling = k + s->stiffness * fabsf(clmd->sim_parms->max_struct - k);
 			
 		k = verts [s->ij].goal * scaling / (clmd->sim_parms->avg_spring_len + FLT_EPSILON);
 		
 		VECADDS(s->f, s->f, extent, -k);
 		
-		mul_fvector_S(damping_force, dir, clmd->sim_parms->goalfrict * 0.01 * dot_v3v3(vel, dir));
+		mul_fvector_S(damping_force, dir, clmd->sim_parms->goalfrict * 0.01f * dot_v3v3(vel, dir));
 		VECADD(s->f, s->f, damping_force);
 		
 		// HERE IS THE PROBLEM!!!!
@@ -1319,10 +1138,10 @@ DO_INLINE void cloth_calc_spring_force(ClothModifierData *clmd, ClothSpring *s,
 		if (length < L) {
 			s->flags |= CLOTH_SPRING_FLAG_NEEDED;
 			
-			k = clmd->sim_parms->bending;	
+			k = clmd->sim_parms->bending;
 			
-			scaling = k + s->stiffness * ABS(clmd->sim_parms->max_bend-k);			
-			cb = k = scaling / (20.0*(clmd->sim_parms->avg_spring_len + FLT_EPSILON));
+			scaling = k + s->stiffness * fabsf(clmd->sim_parms->max_bend - k);
+			cb = k = scaling / (20.0f * (clmd->sim_parms->avg_spring_len + FLT_EPSILON));
 
 			mul_fvector_S(bending_force, dir, fbstar(length, L, k, cb));
 			VECADD(s->f, s->f, bending_force);
@@ -1338,7 +1157,7 @@ DO_INLINE void cloth_apply_spring_force(ClothModifierData *UNUSED(clmd), ClothSp
 		if (!(s->type & CLOTH_SPRING_TYPE_BENDING)) {
 			sub_fmatrix_fmatrix(dFdV[s->ij].m, dFdV[s->ij].m, s->dfdv);
 			sub_fmatrix_fmatrix(dFdV[s->kl].m, dFdV[s->kl].m, s->dfdv);
-			add_fmatrix_fmatrix(dFdV[s->matrix_index].m, dFdV[s->matrix_index].m, s->dfdv);	
+			add_fmatrix_fmatrix(dFdV[s->matrix_index].m, dFdV[s->matrix_index].m, s->dfdv);
 		}
 
 		VECADD(lF[s->ij], lF[s->ij], s->f);
@@ -1349,7 +1168,7 @@ DO_INLINE void cloth_apply_spring_force(ClothModifierData *UNUSED(clmd), ClothSp
 		sub_fmatrix_fmatrix(dFdX[s->kl].m, dFdX[s->kl].m, s->dfdx);
 		sub_fmatrix_fmatrix(dFdX[s->ij].m, dFdX[s->ij].m, s->dfdx);
 		add_fmatrix_fmatrix(dFdX[s->matrix_index].m, dFdX[s->matrix_index].m, s->dfdx);
-	}	
+	}
 }
 
 
@@ -1357,15 +1176,15 @@ static void CalcFloat( float *v1, float *v2, float *v3, float *n)
 {
 	float n1[3], n2[3];
 
-	n1[0]= v1[0]-v2[0];
-	n2[0]= v2[0]-v3[0];
-	n1[1]= v1[1]-v2[1];
-	n2[1]= v2[1]-v3[1];
-	n1[2]= v1[2]-v2[2];
-	n2[2]= v2[2]-v3[2];
-	n[0]= n1[1]*n2[2]-n1[2]*n2[1];
-	n[1]= n1[2]*n2[0]-n1[0]*n2[2];
-	n[2]= n1[0]*n2[1]-n1[1]*n2[0];
+	n1[0] = v1[0]-v2[0];
+	n2[0] = v2[0]-v3[0];
+	n1[1] = v1[1]-v2[1];
+	n2[1] = v2[1]-v3[1];
+	n1[2] = v1[2]-v2[2];
+	n2[2] = v2[2]-v3[2];
+	n[0] = n1[1]*n2[2]-n1[2]*n2[1];
+	n[1] = n1[2]*n2[0]-n1[0]*n2[2];
+	n[2] = n1[0]*n2[1]-n1[1]*n2[0];
 }
 
 static void CalcFloat4( float *v1, float *v2, float *v3, float *v4, float *n)
@@ -1373,20 +1192,20 @@ static void CalcFloat4( float *v1, float *v2, float *v3, float *v4, float *n)
 	/* real cross! */
 	float n1[3], n2[3];
 
-	n1[0]= v1[0]-v3[0];
-	n1[1]= v1[1]-v3[1];
-	n1[2]= v1[2]-v3[2];
+	n1[0] = v1[0]-v3[0];
+	n1[1] = v1[1]-v3[1];
+	n1[2] = v1[2]-v3[2];
 
-	n2[0]= v2[0]-v4[0];
-	n2[1]= v2[1]-v4[1];
-	n2[2]= v2[2]-v4[2];
+	n2[0] = v2[0]-v4[0];
+	n2[1] = v2[1]-v4[1];
+	n2[2] = v2[2]-v4[2];
 
-	n[0]= n1[1]*n2[2]-n1[2]*n2[1];
-	n[1]= n1[2]*n2[0]-n1[0]*n2[2];
-	n[2]= n1[0]*n2[1]-n1[1]*n2[0];
+	n[0] = n1[1]*n2[2]-n1[2]*n2[1];
+	n[1] = n1[2]*n2[0]-n1[0]*n2[2];
+	n[2] = n1[0]*n2[1]-n1[1]*n2[0];
 }
 
-static float calculateVertexWindForce(float wind[3], float vertexnormal[3])  
+static float calculateVertexWindForce(const float wind[3], const float vertexnormal[3])
 {
 	return dot_v3v3(wind, vertexnormal);
 }
@@ -1419,7 +1238,7 @@ static void hair_velocity_smoothing(ClothModifierData *clmd, lfVector *lF, lfVec
 	float smoothfac = 2.0f * clmd->sim_parms->velocity_smooth;
 	float collfac = 2.0f * clmd->sim_parms->collider_friction;
 	unsigned int	v = 0;
-	unsigned int	i = 0;
+	int	            i = 0;
 	int				j = 0;
 	int				k = 0;
 
@@ -1480,7 +1299,7 @@ static void hair_velocity_smoothing(ClothModifierData *clmd, lfVector *lF, lfVec
 						colg[i][j][k].velocity[0] += vel[0];
 						colg[i][j][k].velocity[1] += vel[1];
 						colg[i][j][k].velocity[2] += vel[2];
-						colg[i][j][k].density += 1.0;
+						colg[i][j][k].density += 1.0f;
 					}
 				}
 			}
@@ -1514,7 +1333,7 @@ static void hair_velocity_smoothing(ClothModifierData *clmd, lfVector *lF, lfVec
 		i = HAIR_GRID_INDEX(lX[v], gmin, gmax, 0);
 		j = HAIR_GRID_INDEX(lX[v], gmin, gmax, 1);
 		k = HAIR_GRID_INDEX(lX[v], gmin, gmax, 2);
-		if (i < 0 || j < 0 || k < 0 || i > 10 || j >= 10 || k >= 10)
+		if (i < 0 || j < 0 || k < 0 || i >= 10 || j >= 10 || k >= 10)
 			continue;
 
 		lF[v][0] += smoothfac * (grid[i][j][k].velocity[0] - lV[v][0]);
@@ -1545,7 +1364,7 @@ static void cloth_calc_force(ClothModifierData *clmd, float UNUSED(frame), lfVec
 	lfVector *winvec;
 	EffectedPoint epoint;
 
-	tm2[0][0]= tm2[1][1]= tm2[2][2]= -spring_air;
+	tm2[0][0] = tm2[1][1] = tm2[2][2] = -spring_air;
 	
 	/* global acceleration (gravitation) */
 	if (clmd->scene->physics_settings.flag & PHYS_GLOBAL_GRAVITY) {
@@ -1589,11 +1408,11 @@ static void cloth_calc_force(ClothModifierData *clmd, float UNUSED(frame), lfVec
 		}
 		
 		for (i = 0; i < cloth->numfaces; i++) {
-			float trinormal[3]={0, 0, 0}; // normalized triangle normal
-			float triunnormal[3]={0, 0, 0}; // not-normalized-triangle normal
-			float tmp[3]={0, 0, 0};
+			float trinormal[3] = {0, 0, 0}; // normalized triangle normal
+			float triunnormal[3] = {0, 0, 0}; // not-normalized-triangle normal
+			float tmp[3] = {0, 0, 0};
 			float factor = (mfaces[i].v4) ? 0.25 : 1.0 / 3.0;
-			factor *= 0.02;
+			factor *= 0.02f;
 			
 			// calculate face normal
 			if (mfaces[i].v4)
@@ -1629,9 +1448,9 @@ static void cloth_calc_force(ClothModifierData *clmd, float UNUSED(frame), lfVec
 		/* Hair has only edges */
 		if (cloth->numfaces == 0) {
 			ClothSpring *spring;
-			float edgevec[3]={0, 0, 0}; //edge vector
-			float edgeunnormal[3]={0, 0, 0}; // not-normalized-edge normal
-			float tmp[3]={0, 0, 0};
+			float edgevec[3] = {0, 0, 0}; //edge vector
+			float edgeunnormal[3] = {0, 0, 0}; // not-normalized-edge normal
+			float tmp[3] = {0, 0, 0};
 			float factor = 0.01;
 
 			search = cloth->springs;
@@ -1663,7 +1482,7 @@ static void cloth_calc_force(ClothModifierData *clmd, float UNUSED(frame), lfVec
 	while (search) {
 		// only handle active springs
 		ClothSpring *spring = search->link;
-		if( !(spring->flags & CLOTH_SPRING_FLAG_DEACTIVATE))
+		if (!(spring->flags & CLOTH_SPRING_FLAG_DEACTIVATE))
 			cloth_calc_spring_force(clmd, search->link, lF, lX, lV, dFdV, dFdX, time);
 
 		search = search->next;
@@ -1717,7 +1536,7 @@ static void simulate_implicit_euler(lfVector *Vnew, lfVector *UNUSED(lX), lfVect
  * (edge distance constraints) in a lagrangian solver.  then add forces to help
  * guide the implicit solver to that state.  this function is called after
  * collisions*/
-static int UNUSED_FUNCTION(cloth_calc_helper_forces)(Object *UNUSED(ob), ClothModifierData * clmd, float (*initial_cos)[3], float UNUSED(step), float dt)
+static int UNUSED_FUNCTION(cloth_calc_helper_forces)(Object *UNUSED(ob), ClothModifierData *clmd, float (*initial_cos)[3], float UNUSED(step), float dt)
 {
 	Cloth *cloth= clmd->clothObject;
 	float (*cos)[3] = MEM_callocN(sizeof(float)*3*cloth->numverts, "cos cloth_calc_helper_forces");
@@ -1731,8 +1550,8 @@ static int UNUSED_FUNCTION(cloth_calc_helper_forces)(Object *UNUSED(ob), ClothMo
 	for (i=0; i<cloth->numverts; i++, cv++) {
 		copy_v3_v3(cos[i], cv->tx);
 		
-		if (cv->goal == 1.0f || len_v3v3(initial_cos[i], cv->tx) != 0.0) {
-			masses[i] = 1e+10;	
+		if (cv->goal == 1.0f || len_squared_v3v3(initial_cos[i], cv->tx) != 0.0f) {
+			masses[i] = 1e+10;
 		}
 		else {
 			masses[i] = cv->mass;
@@ -1759,18 +1578,18 @@ static int UNUSED_FUNCTION(cloth_calc_helper_forces)(Object *UNUSED(ob), ClothMo
 			normalize_v3(vec);
 			
 			c = (len - spring->restlen);
-			if (c == 0.0)
+			if (c == 0.0f)
 				continue;
 			
-			l = c / ((1.0/masses[v1]) + (1.0/masses[v2]));
+			l = c / ((1.0f / masses[v1]) + (1.0f / masses[v2]));
 			
-			mul_v3_fl(vec, -(1.0/masses[v1])*l);
+			mul_v3_fl(vec, -(1.0f / masses[v1]) * l);
 			add_v3_v3(cos[v1], vec);
 	
 			sub_v3_v3v3(vec, cos[v2], cos[v1]);
 			normalize_v3(vec);
 			
-			mul_v3_fl(vec, -(1.0/masses[v2])*l);
+			mul_v3_fl(vec, -(1.0f / masses[v2]) * l);
 			add_v3_v3(cos[v2], vec);
 		}
 	}
@@ -1791,6 +1610,7 @@ static int UNUSED_FUNCTION(cloth_calc_helper_forces)(Object *UNUSED(ob), ClothMo
 	
 	return 1;
 }
+
 int implicit_solver(Object *ob, float frame, ClothModifierData *clmd, ListBase *effectors)
 {
 	unsigned int i=0;
@@ -1815,7 +1635,7 @@ int implicit_solver(Object *ob, float frame, ClothModifierData *clmd, ListBase *
 				sub_v3_v3v3(id->V[i], verts[i].xconst, verts[i].xold);
 				// mul_v3_fl(id->V[i], clmd->sim_parms->stepsPerFrame);
 			}
-		}	
+		}
 	}
 	
 	while (step < tf) {
@@ -1840,7 +1660,7 @@ int implicit_solver(Object *ob, float frame, ClothModifierData *clmd, ListBase *
 					mul_fvector_S(tvect, tvect, step+dt);
 					VECADD(tvect, tvect, verts[i].xold);
 					copy_v3_v3(id->Xnew[i], tvect);
-				}	
+				}
 			}
 			
 			copy_v3_v3(verts[i].txold, id->X[i]);
@@ -1875,10 +1695,8 @@ int implicit_solver(Object *ob, float frame, ClothModifierData *clmd, ListBase *
 			//if (do_extra_solve)
 			//	cloth_calc_helper_forces(ob, clmd, initial_cos, step/clmd->sim_parms->timescale, dt/clmd->sim_parms->timescale);
 			
-			for (i = 0; i < numverts; i++) {
-
-				if (do_extra_solve) {
-					
+			if (do_extra_solve) {
+				for (i = 0; i < numverts; i++) {				
 					if ((clmd->sim_parms->flags & CLOTH_SIMSETTINGS_FLAG_GOAL) && (verts [i].flags & CLOTH_VERT_FLAG_PINNED))
 						continue;
 
@@ -1898,7 +1716,7 @@ int implicit_solver(Object *ob, float frame, ClothModifierData *clmd, ListBase *
 				cp_lfvector(id->V, id->Vnew, numverts);
 
 				// calculate 
-				cloth_calc_force(clmd, frame, id->F, id->X, id->V, id->dFdV, id->dFdX, effectors, step+dt, id->M);	
+				cloth_calc_force(clmd, frame, id->F, id->X, id->V, id->dFdV, id->dFdX, effectors, step+dt, id->M);
 				
 				simulate_implicit_euler(id->Vnew, id->X, id->V, id->F, id->dFdV, id->dFdX, dt / 2.0f, id->A, id->B, id->dV, id->S, id->z, id->olddV, id->P, id->Pinv, id->M, id->bigI);
 			}
@@ -1947,6 +1765,6 @@ void implicit_set_positions(ClothModifierData *clmd)
 		copy_v3_v3(id->V[i], verts[i].v);
 	}
 	if (G.debug_value > 0)
-		printf("implicit_set_positions\n");	
+		printf("implicit_set_positions\n");
 }
 
