@@ -60,11 +60,11 @@
 #include "DNA_world_types.h"
 #include "DNA_object_types.h"
 
-#include "BLI_math.h" /* windows needs for M_PI */
 #include "BLI_blenlib.h"
 #include "BLI_dynstr.h"
 #include "BLI_utildefines.h"
 
+#include "BLF_translation.h"
 
 #include "BKE_ipo.h"
 #include "BKE_animsys.h"
@@ -77,6 +77,10 @@
 
 #include "MEM_guardedalloc.h"
 
+#ifdef WIN32
+#  include "BLI_math_base.h"  /* M_PI */
+#endif
+
 /* *************************************************** */
 /* Old-Data Freeing Tools */
 
@@ -569,7 +573,7 @@ static const char *material_adrcodes_to_paths(int adrcode, int *array_index)
 			return mtex_adrcodes_to_paths(adrcode, array_index);
 	}
 	
-	return NULL;	
+	return NULL;
 }
 
 /* Camera Types */
@@ -711,23 +715,12 @@ static const char *world_adrcodes_to_paths(int adrcode, int *array_index)
 			return "mist.start";
 		case WO_MISTHI:
 			return "mist.height";
-
-		case WO_STAR_R:
-		case WO_STAR_G:
-		case WO_STAR_B:
-			printf("WARNING: WO_STAR_R/G/B deprecated\n");
-			return NULL;
-		
-		case WO_STARDIST:
-			return "stars.min_distance";
-		case WO_STARSIZE:
-			return "stars.size";
 		
 		default: /* for now, we assume that the others were MTex channels */
 			return mtex_adrcodes_to_paths(adrcode, array_index);
 	}
 		
-	return NULL;	
+	return NULL;
 }
 
 /* Particle Types */
@@ -873,7 +866,7 @@ static char *get_rna_access(int blocktype, int adrcode, char actname[], char con
 			propname = "eval_time";
 			break;
 		
-		/* XXX problematic blocktypes */		
+		/* XXX problematic blocktypes */
 		case ID_SEQ: /* sequencer strip */
 			//SEQ_FAC1:
 			switch (adrcode) {
@@ -981,21 +974,21 @@ static char *get_rna_access(int blocktype, int adrcode, char actname[], char con
 static short adrcode_to_dtar_transchan(short adrcode)
 {
 	switch (adrcode) {
-		case OB_LOC_X:	
+		case OB_LOC_X:
 			return DTAR_TRANSCHAN_LOCX;
 		case OB_LOC_Y:
 			return DTAR_TRANSCHAN_LOCY;
 		case OB_LOC_Z:
 			return DTAR_TRANSCHAN_LOCZ;
 		
-		case OB_ROT_X:	
+		case OB_ROT_X:
 			return DTAR_TRANSCHAN_ROTX;
 		case OB_ROT_Y:
 			return DTAR_TRANSCHAN_ROTY;
 		case OB_ROT_Z:
 			return DTAR_TRANSCHAN_ROTZ;
 		
-		case OB_SIZE_X:	
+		case OB_SIZE_X:
 			return DTAR_TRANSCHAN_SCALEX;
 		case OB_SIZE_Y:
 			return DTAR_TRANSCHAN_SCALEX;
@@ -1117,7 +1110,8 @@ static void fcurve_add_to_list(ListBase *groups, ListBase *list, FCurve *fcu, ch
 			BLI_strncpy(agrp->name, grpname, sizeof(agrp->name));
 
 			BLI_addtail(&tmp_act.groups, agrp);
-			BLI_uniquename(&tmp_act.groups, agrp, "Group", '.', offsetof(bActionGroup, name), sizeof(agrp->name));
+			BLI_uniquename(&tmp_act.groups, agrp, DATA_("Group"), '.', offsetof(bActionGroup, name),
+			               sizeof(agrp->name));
 		}
 		
 		/* add F-Curve to group */
@@ -1172,9 +1166,8 @@ static void icu_to_fcurves(ID *id, ListBase *groups, ListBase *list, IpoCurve *i
 		{
 			/* just copy, as the new defines match the old ones... */
 			fcu->extend = icu->extrap;
+			break;
 		}
-		break;
-			
 		case IPO_CYCL: /* cyclic extrapolation */
 		case IPO_CYCLX: /* cyclic extrapolation + offset */
 		{
@@ -1189,8 +1182,8 @@ static void icu_to_fcurves(ID *id, ListBase *groups, ListBase *list, IpoCurve *i
 				data->before_mode = data->after_mode = FCM_EXTRAPOLATE_CYCLIC_OFFSET;
 			else
 				data->before_mode = data->after_mode = FCM_EXTRAPOLATE_CYCLIC;
+			break;
 		}
-		break;
 	}
 	
 	/* -------- */
@@ -1315,8 +1308,8 @@ static void icu_to_fcurves(ID *id, ListBase *groups, ListBase *list, IpoCurve *i
 				 *	- they were degrees/10 
 				 *	- we need radians for RNA to do the right thing
 				 */
-				if ( ((icu->blocktype == ID_OB) && ELEM3(icu->adrcode, OB_ROT_X, OB_ROT_Y, OB_ROT_Z)) ||
-				     ((icu->blocktype == ID_PO) && ELEM3(icu->adrcode, AC_EUL_X, AC_EUL_Y, AC_EUL_Z)) )
+				if ( ((icu->blocktype == ID_OB) && ELEM(icu->adrcode, OB_ROT_X, OB_ROT_Y, OB_ROT_Z)) ||
+				     ((icu->blocktype == ID_PO) && ELEM(icu->adrcode, AC_EUL_X, AC_EUL_Y, AC_EUL_Z)) )
 				{
 					const float fac = (float)M_PI / 18.0f; //10.0f * M_PI/180.0f;
 					
@@ -1348,7 +1341,7 @@ static void icu_to_fcurves(ID *id, ListBase *groups, ListBase *list, IpoCurve *i
 					DriverVar *dvar = fcu->driver->variables.first;
 					DriverTarget *dtar = &dvar->targets[0];
 					
-					if (ELEM3(dtar->transChan, DTAR_TRANSCHAN_ROTX, DTAR_TRANSCHAN_ROTY, DTAR_TRANSCHAN_ROTZ)) {
+					if (ELEM(dtar->transChan, DTAR_TRANSCHAN_ROTX, DTAR_TRANSCHAN_ROTY, DTAR_TRANSCHAN_ROTZ)) {
 						const float fac = (float)M_PI / 18.0f;
 						
 						dst->vec[0][0] *= fac;
@@ -1359,8 +1352,8 @@ static void icu_to_fcurves(ID *id, ListBase *groups, ListBase *list, IpoCurve *i
 				
 				/* correct values for sequencer curves, that were not locked to frame */
 				if (seq && (seq->flag & SEQ_IPO_FRAME_LOCKED) == 0) {
-					double mul = (seq->enddisp - seq->startdisp) / 100.0f;
-					double offset = seq->startdisp;
+					const float mul = (seq->enddisp - seq->startdisp) / 100.0f;
+					const float offset = seq->startdisp;
 					
 					dst->vec[0][0] *= mul;
 					dst->vec[0][0] += offset;
@@ -1395,7 +1388,7 @@ static void ipo_to_animato(ID *id, Ipo *ipo, char actname[], char constname[], S
 	IpoCurve *icu;
 	
 	/* sanity check */
-	if (ELEM3(NULL, ipo, anim, drivers))
+	if (ELEM(NULL, ipo, anim, drivers))
 		return;
 		
 	if (G.debug & G_DEBUG) printf("ipo_to_animato\n");
@@ -1465,7 +1458,7 @@ static void action_to_animato(ID *id, bAction *act, ListBase *groups, ListBase *
 	bConstraintChannel *conchan, *conchann;
 	
 	/* only continue if there are Action Channels (indicating unconverted data) */
-	if (act->chanbase.first == NULL)
+	if (BLI_listbase_is_empty(&act->chanbase))
 		return;
 		
 	/* get rid of all Action Groups */
@@ -1530,7 +1523,7 @@ static void ipo_to_animdata(ID *id, Ipo *ipo, char actname[], char constname[],
 	if (G.debug & G_DEBUG) {
 		printf("ipo to animdata - ID:%s, IPO:%s, actname:%s constname:%s seqname:%s  curves:%d\n",
 		       id->name + 2, ipo->id.name + 2, (actname) ? actname : "<None>", (constname) ? constname : "<None>", (seq) ? (seq->name + 2) : "<None>",
-		       BLI_countlist(&ipo->curve));
+		       BLI_listbase_count(&ipo->curve));
 	}
 	
 	/* Convert curves to animato system (separated into separate lists of F-Curves for animation and drivers),
@@ -1548,7 +1541,7 @@ static void ipo_to_animdata(ID *id, Ipo *ipo, char actname[], char constname[],
 			
 			BLI_snprintf(nameBuf, sizeof(nameBuf), "CDA:%s", ipo->id.name + 2);
 			
-			adt->action = add_empty_action(nameBuf);
+			adt->action = add_empty_action(G.main, nameBuf);
 			if (G.debug & G_DEBUG) printf("\t\tadded new action - '%s'\n", nameBuf);
 		}
 		
@@ -1645,7 +1638,7 @@ static void nlastrips_to_animdata(ID *id, ListBase *strips)
 				/* by default, we now always extrapolate, while in the past this was optional */
 				if ((as->flag & ACTSTRIP_HOLDLASTFRAME) == 0)
 					strip->extendmode = NLASTRIP_EXTEND_NOTHING;
-			}	
+			}
 			
 			/* try to add this strip to the current NLA-Track (i.e. the 'last' one on the stack atm) */
 			if (BKE_nlatrack_add_strip(nlt, strip) == 0) {
@@ -2093,7 +2086,7 @@ void do_versions_ipos_to_animato(Main *main)
 			bAction *new_act;
 			
 			/* add a new action for this, and convert all data into that action */
-			new_act = add_empty_action(id->name+2);
+			new_act = add_empty_action(main, id->name + 2);
 			ipo_to_animato(NULL, ipo, NULL, NULL, NULL, NULL, &new_act->curves, &drivers);
 			new_act->idroot = ipo->blocktype;
 		}
