@@ -40,6 +40,8 @@
 #include "BLI_math_vector.h"
 #include "BLI_utildefines.h"
 
+#include "BLF_translation.h"
+
 #include "DNA_anim_types.h"
 #include "DNA_key_types.h"
 #include "DNA_lattice_types.h"
@@ -56,10 +58,7 @@
 #include "BKE_key.h"
 #include "BKE_lattice.h"
 #include "BKE_library.h"
-#include "BKE_tessmesh.h"
-#include "BKE_main.h"
-#include "BKE_object.h"
-#include "BKE_deform.h"
+#include "BKE_editmesh.h"
 #include "BKE_scene.h"
 
 
@@ -83,50 +82,31 @@ void BKE_key_free(Key *key)
 	KeyBlock *kb;
 	
 	BKE_free_animdata((ID *)key);
-	
-	while ( (kb = key->block.first) ) {
-		
-		if (kb->data) MEM_freeN(kb->data);
-		
-		BLI_remlink(&key->block, kb);
+
+	while ((kb = BLI_pophead(&key->block))) {
+		if (kb->data)
+			MEM_freeN(kb->data);
 		MEM_freeN(kb);
 	}
-	
 }
 
 void BKE_key_free_nolib(Key *key)
 {
 	KeyBlock *kb;
-	
-	while ( (kb = key->block.first) ) {
-		
-		if (kb->data) MEM_freeN(kb->data);
-		
-		BLI_remlink(&key->block, kb);
+
+	while ((kb = BLI_pophead(&key->block))) {
+		if (kb->data)
+			MEM_freeN(kb->data);
 		MEM_freeN(kb);
 	}
-	
 }
 
-/* GS reads the memory pointed at in a specific ordering. There are,
- * however two definitions for it. I have jotted them down here, both,
- * but I think the first one is actually used. The thing is that
- * big-endian systems might read this the wrong way round. OTOH, we
- * constructed the IDs that are read out with this macro explicitly as
- * well. I expect we'll sort it out soon... */
-
-/* from blendef: */
-#define GS(a)   (*((short *)(a)))
-
-/* from misc_util: flip the bytes from x  */
-/*  #define GS(x) (((unsigned char *)(x))[0] << 8 | ((unsigned char *)(x))[1]) */
-
 Key *BKE_key_add(ID *id)    /* common function */
 {
 	Key *key;
 	char *el;
 	
-	key = BKE_libblock_alloc(&G.main->key, ID_KE, "Key");
+	key = BKE_libblock_alloc(G.main, ID_KE, "Key");
 	
 	key->type = KEY_NORMAL;
 	key->from = id;
@@ -201,7 +181,8 @@ Key *BKE_key_copy_nolib(Key *key)
 	Key *keyn;
 	KeyBlock *kbn, *kb;
 	
-	if (key == 0) return 0;
+	if (key == NULL)
+		return NULL;
 	
 	keyn = MEM_dupallocN(key);
 
@@ -259,7 +240,7 @@ void BKE_key_sort(Key *key)
 		/* find the right location and insert before */
 		for (kb2 = key->block.first; kb2; kb2 = kb2->next) {
 			if (kb2->pos > kb->pos) {
-				BLI_insertlink(&key->block, kb2->prev, kb);
+				BLI_insertlinkafter(&key->block, kb2->prev, kb);
 				break;
 			}
 		}
@@ -300,6 +281,16 @@ void key_curve_position_weights(float t, float data[4], int type)
 		data[2] = -0.5f        * t3  + 0.5f * t2   + 0.5f * t    + 0.16666666f;
 		data[3] =  0.16666666f * t3;
 	}
+	else if (type == KEY_CATMULL_ROM) {
+		t2 = t * t;
+		t3 = t2 * t;
+		fc = 0.5f;
+
+		data[0] = -fc          * t3  + 2.0f * fc          * t2 - fc * t;
+		data[1] =  (2.0f - fc) * t3  + (fc - 3.0f)        * t2 + 1.0f;
+		data[2] =  (fc - 2.0f) * t3  + (3.0f - 2.0f * fc) * t2 + fc * t;
+		data[3] =  fc          * t3  - fc * t2;
+	}
 }
 
 /* first derivative */
@@ -330,6 +321,15 @@ void key_curve_tangent_weights(float t, float data[4], int type)
 		data[2] = -1.5f * t2  + t         + 0.5f;
 		data[3] =  0.5f * t2;
 	}
+	else if (type == KEY_CATMULL_ROM) {
+		t2 = t * t;
+		fc = 0.5f;
+
+		data[0] = -3.0f * fc          * t2  + 4.0f * fc * t                  - fc;
+		data[1] =  3.0f * (2.0f - fc) * t2  + 2.0f * (fc - 3.0f) * t;
+		data[2] =  3.0f * (fc - 2.0f) * t2  + 2.0f * (3.0f - 2.0f * fc) * t  + fc;
+		data[3] =  3.0f * fc          * t2  - 2.0f * fc * t;
+	}
 }
 
 /* second derivative */
@@ -357,6 +357,14 @@ void key_curve_normal_weights(float t, float data[4], int type)
 		data[2] = -3.0f * t  + 1.0f;
 		data[3] =  1.0f * t;
 	}
+	else if (type == KEY_CATMULL_ROM) {
+		fc = 0.5f;
+
+		data[0] = -6.0f * fc          * t  + 4.0f * fc;
+		data[1] =  6.0f * (2.0f - fc) * t  + 2.0f * (fc - 3.0f);
+		data[2] =  6.0f * (fc - 2.0f) * t  + 2.0f * (3.0f - 2.0f * fc);
+		data[3] =  6.0f * fc          * t  - 2.0f * fc;
+	}
 }
 
 static int setkeys(float fac, ListBase *lb, KeyBlock *k[], float t[4], int cycl)
@@ -389,7 +397,7 @@ static int setkeys(float fac, ListBase *lb, KeyBlock *k[], float t[4], int cycl)
 			if (k1->next == NULL) k[0] = k1;
 			k1 = k1->next;
 		}
-		/* k1= k[1]; */ /* UNUSED */
+		/* k1 = k[1]; */ /* UNUSED */
 		t[0] = k[0]->pos;
 		t[1] += dpos;
 		t[2] = k[2]->pos + dpos;
@@ -418,9 +426,13 @@ static int setkeys(float fac, ListBase *lb, KeyBlock *k[], float t[4], int cycl)
 				k1 = firstkey;
 				ofs += dpos;
 			}
-			else if (t[2] == t[3]) break;
+			else if (t[2] == t[3]) {
+				break;
+			}
+		}
+		else {
+			k1 = k1->next;
 		}
-		else k1 = k1->next;
 
 		t[0] = t[1];
 		k[0] = k[1];
@@ -513,7 +525,7 @@ static char *key_block_get_data(Key *key, KeyBlock *actkb, KeyBlock *kb, char **
 
 			if (me->edit_btmesh && me->edit_btmesh->bm->totvert == kb->totelem) {
 				a = 0;
-				co = MEM_callocN(sizeof(float) * 3 * me->edit_btmesh->bm->totvert, "key_block_get_data");
+				co = MEM_mallocN(sizeof(float) * 3 * me->edit_btmesh->bm->totvert, "key_block_get_data");
 
 				BM_ITER_MESH (eve, &iter, me->edit_btmesh->bm, BM_VERTS_OF_MESH) {
 					copy_v3_v3(co[a], eve->co);
@@ -532,10 +544,10 @@ static char *key_block_get_data(Key *key, KeyBlock *actkb, KeyBlock *kb, char **
 
 
 /* currently only the first value of 'ofs' may be set. */
-static short key_pointer_size(const Key *key, const int mode, int *poinsize, int *ofs)
+static bool key_pointer_size(const Key *key, const int mode, int *poinsize, int *ofs)
 {
 	if (key->from == NULL) {
-		return FALSE;
+		return false;
 	}
 
 	switch (GS(key->from->name)) {
@@ -560,10 +572,10 @@ static short key_pointer_size(const Key *key, const int mode, int *poinsize, int
 			break;
 		default:
 			BLI_assert(!"invalid 'key->from' ID type");
-			return FALSE;
+			return false;
 	}
 
-	return TRUE;
+	return true;
 }
 
 static void cp_key(const int start, int end, const int tot, char *poin, Key *key, KeyBlock *actkb, KeyBlock *kb, float *weights, const int mode)
@@ -608,8 +620,10 @@ static void cp_key(const int start, int end, const int tot, char *poin, Key *key
 				k1 += a * key->elemsize;
 			}
 		}
-		else k1 += start * key->elemsize;
-	}	
+		else {
+			k1 += start * key->elemsize;
+		}
+	}
 	
 	if (mode == KEY_MODE_BEZTRIPLE) {
 		elemstr[0] = 1;
@@ -691,8 +705,8 @@ static void cp_cu_key(Curve *cu, Key *key, KeyBlock *actkb, KeyBlock *kb, const
 		if (nu->bp) {
 			step = nu->pntsu * nu->pntsv;
 
-			a1 = maxi(a, start);
-			a2 = mini(a + step, end);
+			a1 = max_ii(a, start);
+			a2 = min_ii(a + step, end);
 
 			if (a1 < a2) cp_key(a1, a2, tot, out, key, actkb, kb, NULL, KEY_MODE_BPOINT);
 		}
@@ -700,8 +714,8 @@ static void cp_cu_key(Curve *cu, Key *key, KeyBlock *actkb, KeyBlock *kb, const
 			step = 3 * nu->pntsu;
 
 			/* exception because keys prefer to work with complete blocks */
-			a1 = maxi(a, start);
-			a2 = mini(a + step, end);
+			a1 = max_ii(a, start);
+			a2 = min_ii(a + step, end);
 
 			if (a1 < a2) cp_key(a1, a2, tot, out, key, actkb, kb, NULL, KEY_MODE_BEZTRIPLE);
 		}
@@ -711,12 +725,13 @@ static void cp_cu_key(Curve *cu, Key *key, KeyBlock *actkb, KeyBlock *kb, const
 	}
 }
 
-void BKE_key_evaluate_relative(const int start, int end, const int tot, char *basispoin, Key *key, KeyBlock *actkb, const int mode)
+void BKE_key_evaluate_relative(const int start, int end, const int tot, char *basispoin, Key *key, KeyBlock *actkb,
+                               float **per_keyblock_weights, const int mode)
 {
 	KeyBlock *kb;
 	int *ofsp, ofs[3], elemsize, b;
 	char *cp, *poin, *reffrom, *from, elemstr[8];
-	int poinsize;
+	int poinsize, keyblock_index;
 
 	/* currently always 0, in future key_pointer_size may assign */
 	ofs[1] = 0;
@@ -740,14 +755,14 @@ void BKE_key_evaluate_relative(const int start, int end, const int tot, char *ba
 	
 	/* step 2: do it */
 	
-	for (kb = key->block.first; kb; kb = kb->next) {
+	for (kb = key->block.first, keyblock_index = 0; kb; kb = kb->next, keyblock_index++) {
 		if (kb != key->refkey) {
 			float icuval = kb->curval;
 			
 			/* only with value, and no difference allowed */
 			if (!(kb->flag & KEYBLOCK_MUTE) && icuval != 0.0f && kb->totelem == tot) {
 				KeyBlock *refb;
-				float weight, *weights = kb->weights;
+				float weight, *weights = per_keyblock_weights ? per_keyblock_weights[keyblock_index] : NULL;
 				char *freefrom = NULL, *freereffrom = NULL;
 
 				/* reference now can be any block */
@@ -841,7 +856,9 @@ static void do_key(const int start, int end, const int tot, char *poin, Key *key
 		if (k[0]->totelem) {
 			k1d = k[0]->totelem / (float)tot;
 		}
-		else flagdo -= 1;
+		else {
+			flagdo -= 1;
+		}
 	}
 	if (tot != k[1]->totelem) {
 		k2tot = 0.0;
@@ -849,7 +866,9 @@ static void do_key(const int start, int end, const int tot, char *poin, Key *key
 		if (k[0]->totelem) {
 			k2d = k[1]->totelem / (float)tot;
 		}
-		else flagdo -= 2;
+		else {
+			flagdo -= 2;
+		}
 	}
 	if (tot != k[2]->totelem) {
 		k3tot = 0.0;
@@ -857,7 +876,9 @@ static void do_key(const int start, int end, const int tot, char *poin, Key *key
 		if (k[0]->totelem) {
 			k3d = k[2]->totelem / (float)tot;
 		}
-		else flagdo -= 4;
+		else {
+			flagdo -= 4;
+		}
 	}
 	if (tot != k[3]->totelem) {
 		k4tot = 0.0;
@@ -865,7 +886,9 @@ static void do_key(const int start, int end, const int tot, char *poin, Key *key
 		if (k[0]->totelem) {
 			k4d = k[3]->totelem / (float)tot;
 		}
-		else flagdo -= 8;
+		else {
+			flagdo -= 8;
+		}
 	}
 
 	/* this exception needed for slurphing */
@@ -882,7 +905,9 @@ static void do_key(const int start, int end, const int tot, char *poin, Key *key
 					k1 += a * key->elemsize;
 				}
 			}
-			else k1 += start * key->elemsize;
+			else {
+				k1 += start * key->elemsize;
+			}
 		}
 		if (flagdo & 2) {
 			if (flagflo & 2) {
@@ -893,7 +918,9 @@ static void do_key(const int start, int end, const int tot, char *poin, Key *key
 					k2 += a * key->elemsize;
 				}
 			}
-			else k2 += start * key->elemsize;
+			else {
+				k2 += start * key->elemsize;
+			}
 		}
 		if (flagdo & 4) {
 			if (flagflo & 4) {
@@ -904,7 +931,9 @@ static void do_key(const int start, int end, const int tot, char *poin, Key *key
 					k3 += a * key->elemsize;
 				}
 			}
-			else k3 += start * key->elemsize;
+			else {
+				k3 += start * key->elemsize;
+			}
 		}
 		if (flagdo & 8) {
 			if (flagflo & 8) {
@@ -915,7 +944,9 @@ static void do_key(const int start, int end, const int tot, char *poin, Key *key
 					k4 += a * key->elemsize;
 				}
 			}
-			else k4 += start * key->elemsize;
+			else {
+				k4 += start * key->elemsize;
+			}
 		}
 
 	}
@@ -981,7 +1012,9 @@ static void do_key(const int start, int end, const int tot, char *poin, Key *key
 					k2 += elemsize;
 				}
 			}
-			else k2 += elemsize;
+			else {
+				k2 += elemsize;
+			}
 		}
 		if (flagdo & 4) {
 			if (flagflo & 4) {
@@ -991,7 +1024,9 @@ static void do_key(const int start, int end, const int tot, char *poin, Key *key
 					k3 += elemsize;
 				}
 			}
-			else k3 += elemsize;
+			else {
+				k3 += elemsize;
+			}
 		}
 		if (flagdo & 8) {
 			if (flagflo & 8) {
@@ -1001,7 +1036,9 @@ static void do_key(const int start, int end, const int tot, char *poin, Key *key
 					k4 += elemsize;
 				}
 			}
-			else k4 += elemsize;
+			else {
+				k4 += elemsize;
+			}
 		}
 		
 		if (mode == KEY_MODE_BEZTRIPLE) a += 2;
@@ -1013,7 +1050,7 @@ static void do_key(const int start, int end, const int tot, char *poin, Key *key
 	if (freek4) MEM_freeN(freek4);
 }
 
-static float *get_weights_array(Object *ob, char *vgroup)
+static float *get_weights_array(Object *ob, char *vgroup, WeightsArrayCache *cache)
 {
 	MDeformVert *dvert = NULL;
 	BMEditMesh *em = NULL;
@@ -1043,19 +1080,31 @@ static float *get_weights_array(Object *ob, char *vgroup)
 	
 	/* find the group (weak loop-in-loop) */
 	defgrp_index = defgroup_name_index(ob, vgroup);
-	if (defgrp_index >= 0) {
+	if (defgrp_index != -1) {
 		float *weights;
 		int i;
-		
-		weights = MEM_callocN(totvert * sizeof(float), "weights");
+
+		if (cache) {
+			if (cache->defgroup_weights == NULL) {
+				int num_defgroup = BLI_listbase_count(&ob->defbase);
+				cache->defgroup_weights =
+				    MEM_callocN(sizeof(*cache->defgroup_weights) * num_defgroup,
+				                "cached defgroup weights");
+				cache->num_defgroup_weights = num_defgroup;
+			}
+
+			if (cache->defgroup_weights[defgrp_index]) {
+				return cache->defgroup_weights[defgrp_index];
+			}
+		}
+
+		weights = MEM_mallocN(totvert * sizeof(float), "weights");
 
 		if (em) {
+			const int cd_dvert_offset = CustomData_get_offset(&em->bm->vdata, CD_MDEFORMVERT);
 			BM_ITER_MESH_INDEX (eve, &iter, em->bm, BM_VERTS_OF_MESH, i) {
-				dvert = CustomData_bmesh_get(&em->bm->vdata, eve->head.data, CD_MDEFORMVERT);
-
-				if (dvert) {
-					weights[i] = defvert_find_weight(dvert, defgrp_index);
-				}
+				dvert = BM_ELEM_CD_GET_VOID_P(eve, cd_dvert_offset);
+				weights[i] = defvert_find_weight(dvert, defgrp_index);
 			}
 		}
 		else {
@@ -1064,11 +1113,61 @@ static float *get_weights_array(Object *ob, char *vgroup)
 			}
 		}
 
+		if (cache) {
+			cache->defgroup_weights[defgrp_index] = weights;
+		}
+
 		return weights;
 	}
 	return NULL;
 }
 
+float **BKE_keyblock_get_per_block_weights(Object *ob, Key *key, WeightsArrayCache *cache)
+{
+	KeyBlock *keyblock;
+	float **per_keyblock_weights;
+	int keyblock_index;
+
+	per_keyblock_weights =
+		MEM_mallocN(sizeof(*per_keyblock_weights) * key->totkey,
+		            "per keyblock weights");
+
+	for (keyblock = key->block.first, keyblock_index = 0;
+	     keyblock;
+	     keyblock = keyblock->next, keyblock_index++)
+	{
+		per_keyblock_weights[keyblock_index] = get_weights_array(ob, keyblock->vgroup, cache);
+	}
+
+	return per_keyblock_weights;
+}
+
+void BKE_keyblock_free_per_block_weights(Key *key, float **per_keyblock_weights, WeightsArrayCache *cache)
+{
+	int a;
+
+	if (cache) {
+		if (cache->num_defgroup_weights) {
+			for (a = 0; a < cache->num_defgroup_weights; a++) {
+				if (cache->defgroup_weights[a]) {
+					MEM_freeN(cache->defgroup_weights[a]);
+				}
+			}
+			MEM_freeN(cache->defgroup_weights);
+		}
+		cache->defgroup_weights = NULL;
+	}
+	else {
+		for (a = 0; a < key->totkey; a++) {
+			if (per_keyblock_weights[a]) {
+				MEM_freeN(per_keyblock_weights[a]);
+			}
+		}
+	}
+
+	MEM_freeN(per_keyblock_weights);
+}
+
 static void do_mesh_key(Scene *scene, Object *ob, Key *key, char *out, const int tot)
 {
 	KeyBlock *k[4], *actkb = BKE_keyblock_from_object(ob);
@@ -1078,7 +1177,7 @@ static void do_mesh_key(Scene *scene, Object *ob, Key *key, char *out, const int
 	if (key->slurph && key->type != KEY_RELATIVE) {
 		const float ctime_scaled = key->ctime / 100.0f;
 		float delta = (float)key->slurph / tot;
-		float cfra = (float)scene->r.cfra;
+		float cfra = BKE_scene_frame_get(scene);
 		int step, a;
 
 		if (tot > 100 && slurph_opt) {
@@ -1101,17 +1200,11 @@ static void do_mesh_key(Scene *scene, Object *ob, Key *key, char *out, const int
 	}
 	else {
 		if (key->type == KEY_RELATIVE) {
-			KeyBlock *kb;
-			for (kb = key->block.first; kb; kb = kb->next) {
-				kb->weights = get_weights_array(ob, kb->vgroup);
-			}
-
-			BKE_key_evaluate_relative(0, tot, tot, (char *)out, key, actkb, KEY_MODE_DUMMY);
-			
-			for (kb = key->block.first; kb; kb = kb->next) {
-				if (kb->weights) MEM_freeN(kb->weights);
-				kb->weights = NULL;
-			}
+			WeightsArrayCache cache = {0, NULL};
+			float **per_keyblock_weights;
+			per_keyblock_weights = BKE_keyblock_get_per_block_weights(ob, key, &cache);
+			BKE_key_evaluate_relative(0, tot, tot, (char *)out, key, actkb, per_keyblock_weights, KEY_MODE_DUMMY);
+			BKE_keyblock_free_per_block_weights(key, per_keyblock_weights, &cache);
 		}
 		else {
 			const float ctime_scaled = key->ctime / 100.0f;
@@ -1154,11 +1247,11 @@ static void do_rel_cu_key(Curve *cu, Key *key, KeyBlock *actkb, char *out, const
 	for (a = 0, nu = cu->nurb.first; nu; nu = nu->next, a += step) {
 		if (nu->bp) {
 			step = nu->pntsu * nu->pntsv;
-			BKE_key_evaluate_relative(a, a + step, tot, out, key, actkb, KEY_MODE_BPOINT);
+			BKE_key_evaluate_relative(a, a + step, tot, out, key, actkb, NULL, KEY_MODE_BPOINT);
 		}
 		else if (nu->bezt) {
 			step = 3 * nu->pntsu;
-			BKE_key_evaluate_relative(a, a + step, tot, out, key, actkb, KEY_MODE_BEZTRIPLE);
+			BKE_key_evaluate_relative(a, a + step, tot, out, key, actkb, NULL, KEY_MODE_BEZTRIPLE);
 		}
 		else {
 			step = 0;
@@ -1176,7 +1269,7 @@ static void do_curve_key(Scene *scene, Object *ob, Key *key, char *out, const in
 	if (key->slurph && key->type != KEY_RELATIVE) {
 		const float ctime_scaled = key->ctime / 100.0f;
 		float delta = (float)key->slurph / tot;
-		float cfra = (float)scene->r.cfra;
+		float cfra = BKE_scene_frame_get(scene);
 		Nurb *nu;
 		int i = 0, remain = 0;
 		int step, a;
@@ -1217,7 +1310,7 @@ static void do_curve_key(Scene *scene, Object *ob, Key *key, char *out, const in
 					remain = step;
 				}
 
-				count = mini(remain, estep);
+				count = min_ii(remain, estep);
 				if (mode == KEY_MODE_BEZTRIPLE) {
 					count += 3 - count % 3;
 				}
@@ -1258,7 +1351,7 @@ static void do_latt_key(Scene *scene, Object *ob, Key *key, char *out, const int
 	if (key->slurph && key->type != KEY_RELATIVE) {
 		const float ctime_scaled = key->ctime / 100.0f;
 		float delta = (float)key->slurph / tot;
-		float cfra = (float)scene->r.cfra;
+		float cfra = BKE_scene_frame_get(scene);
 		int a;
 
 		for (a = 0; a < tot; a++, cfra += delta) {
@@ -1268,21 +1361,14 @@ static void do_latt_key(Scene *scene, Object *ob, Key *key, char *out, const int
 				do_key(a, a + 1, tot, out, key, actkb, k, t, KEY_MODE_DUMMY);
 			else
 				cp_key(a, a + 1, tot, out, key, actkb, k[2], NULL, KEY_MODE_DUMMY);
-		}		
+		}
 	}
 	else {
 		if (key->type == KEY_RELATIVE) {
-			KeyBlock *kb;
-			
-			for (kb = key->block.first; kb; kb = kb->next)
-				kb->weights = get_weights_array(ob, kb->vgroup);
-			
-			BKE_key_evaluate_relative(0, tot, tot, out, key, actkb, KEY_MODE_DUMMY);
-			
-			for (kb = key->block.first; kb; kb = kb->next) {
-				if (kb->weights) MEM_freeN(kb->weights);
-				kb->weights = NULL;
-			}
+			float **per_keyblock_weights;
+			per_keyblock_weights = BKE_keyblock_get_per_block_weights(ob, key, NULL);
+			BKE_key_evaluate_relative(0, tot, tot, (char *)out, key, actkb, per_keyblock_weights, KEY_MODE_DUMMY);
+			BKE_keyblock_free_per_block_weights(key, per_keyblock_weights, NULL);
 		}
 		else {
 			const float ctime_scaled = key->ctime / 100.0f;
@@ -1300,14 +1386,15 @@ static void do_latt_key(Scene *scene, Object *ob, Key *key, char *out, const int
 }
 
 /* returns key coordinates (+ tilt) when key applied, NULL otherwise */
-float *do_ob_key(Scene *scene, Object *ob)
+float *BKE_key_evaluate_object_ex(Scene *scene, Object *ob, int *r_totelem,
+                                  float *arr, size_t arr_size)
 {
 	Key *key = BKE_key_from_object(ob);
 	KeyBlock *actkb = BKE_keyblock_from_object(ob);
 	char *out;
 	int tot = 0, size = 0;
-	
-	if (key == NULL || key->block.first == NULL)
+
+	if (key == NULL || BLI_listbase_is_empty(&key->block))
 		return NULL;
 
 	/* compute size of output array */
@@ -1344,7 +1431,16 @@ float *do_ob_key(Scene *scene, Object *ob)
 		return NULL;
 	
 	/* allocate array */
-	out = MEM_callocN(size, "do_ob_key out");
+	if (arr == NULL) {
+		out = MEM_callocN(size, "BKE_key_evaluate_object out");
+	}
+	else {
+		if (arr_size != size) {
+			return NULL;
+		}
+
+		out = (char *)arr;
+	}
 
 	/* prevent python from screwing this up? anyhoo, the from pointer could be dropped */
 	key->from = (ID *)ob->data;
@@ -1362,7 +1458,7 @@ float *do_ob_key(Scene *scene, Object *ob)
 		}
 		
 		if (OB_TYPE_SUPPORT_VGROUP(ob->type)) {
-			float *weights = get_weights_array(ob, kb->vgroup);
+			float *weights = get_weights_array(ob, kb->vgroup, NULL);
 
 			cp_key(0, tot, tot, out, key, actkb, kb, weights, 0);
 
@@ -1372,10 +1468,6 @@ float *do_ob_key(Scene *scene, Object *ob)
 			cp_cu_key(ob->data, key, actkb, kb, 0, tot, out, tot);
 	}
 	else {
-		/* do shapekey local drivers */
-		float ctime = (float)scene->r.cfra; // XXX this needs to be checked
-
-		BKE_animsys_evaluate_animdata(scene, &key->id, key->adt, ctime, ADT_RECALC_DRIVERS);
 		
 		if (ob->type == OB_MESH) do_mesh_key(scene, ob, key, out, tot);
 		else if (ob->type == OB_LATTICE) do_latt_key(scene, ob, key, out, tot);
@@ -1383,9 +1475,17 @@ float *do_ob_key(Scene *scene, Object *ob)
 		else if (ob->type == OB_SURF) do_curve_key(scene, ob, key, out, tot);
 	}
 	
+	if (r_totelem) {
+		*r_totelem = tot;
+	}
 	return (float *)out;
 }
 
+float *BKE_key_evaluate_object(Scene *scene, Object *ob, int *r_totelem)
+{
+	return BKE_key_evaluate_object_ex(scene, ob, r_totelem, NULL, 0);
+}
+
 Key *BKE_key_from_object(Object *ob)
 {
 	if (ob == NULL) return NULL;
@@ -1418,16 +1518,18 @@ KeyBlock *BKE_keyblock_add(Key *key, const char *name)
 	BLI_addtail(&key->block, kb);
 	kb->type = KEY_CARDINAL;
 	
-	tot = BLI_countlist(&key->block);
+	tot = BLI_listbase_count(&key->block);
 	if (name) {
 		BLI_strncpy(kb->name, name, sizeof(kb->name));
 	}
 	else {
-		if (tot == 1) BLI_strncpy(kb->name, "Basis", sizeof(kb->name));
-		else BLI_snprintf(kb->name, sizeof(kb->name), "Key %d", tot - 1);
+		if (tot == 1)
+			BLI_strncpy(kb->name, DATA_("Basis"), sizeof(kb->name));
+		else
+			BLI_snprintf(kb->name, sizeof(kb->name), DATA_("Key %d"), tot - 1);
 	}
 
-	BLI_uniquename(&key->block, kb, "Key", '.', offsetof(KeyBlock, name), sizeof(kb->name));
+	BLI_uniquename(&key->block, kb, DATA_("Key"), '.', offsetof(KeyBlock, name), sizeof(kb->name));
 
 	kb->uid = key->uidgen++;
 
@@ -1453,12 +1555,26 @@ KeyBlock *BKE_keyblock_add(Key *key, const char *name)
  * \param name Optional name for the new keyblock.
  * \param do_force always use ctime even for relative keys.
  */
-KeyBlock *BKE_keyblock_add_ctime(Key *key, const char *name, const short do_force)
+KeyBlock *BKE_keyblock_add_ctime(Key *key, const char *name, const bool do_force)
 {
 	KeyBlock *kb = BKE_keyblock_add(key, name);
+	const float cpos = key->ctime / 100.0f;
 
+	/* In case of absolute keys, there is no point in adding more than one key with the same pos.
+	 * Hence only set new keybloc pos to current time if none previous one already use it.
+	 * Now at least people just adding absolute keys without touching to ctime
+	 * won't have to systematically use retiming func (and have ordering issues, too). See T39897.
+	 */
+	if (!do_force && (key->type != KEY_RELATIVE)) {
+		KeyBlock *it_kb;
+		for (it_kb = key->block.first; it_kb; it_kb = it_kb->next) {
+			if (it_kb->pos == cpos) {
+				return kb;
+			}
+		}
+	}
 	if (do_force || (key->type != KEY_RELATIVE)) {
-		kb->pos = key->ctime / 100.0f;
+		kb->pos = cpos;
 		BKE_key_sort(key);
 	}
 
@@ -1553,46 +1669,58 @@ char *BKE_keyblock_curval_rnapath_get(Key *key, KeyBlock *kb)
 /* conversion functions */
 
 /************************* Lattice ************************/
-void BKE_key_convert_from_lattice(Lattice *lt, KeyBlock *kb)
+void BKE_keyblock_update_from_lattice(Lattice *lt, KeyBlock *kb)
 {
 	BPoint *bp;
-	float *fp;
+	float (*fp)[3];
 	int a, tot;
 
+	BLI_assert(kb->totelem == lt->pntsu * lt->pntsv * lt->pntsw);
+
+	tot = kb->totelem;
+	if (tot == 0) return;
+
+	bp = lt->def;
+	fp = kb->data;
+	for (a = 0; a < kb->totelem; a++, fp++, bp++) {
+		copy_v3_v3(*fp, bp->vec);
+	}
+}
+
+void BKE_keyblock_convert_from_lattice(Lattice *lt, KeyBlock *kb)
+{
+	int tot;
+
 	tot = lt->pntsu * lt->pntsv * lt->pntsw;
 	if (tot == 0) return;
 
-	if (kb->data) MEM_freeN(kb->data);
+	MEM_SAFE_FREE(kb->data);
 
-	kb->data = MEM_callocN(lt->key->elemsize * tot, "kb->data");
+	kb->data = MEM_mallocN(lt->key->elemsize * tot, __func__);
 	kb->totelem = tot;
 
-	bp = lt->def;
-	fp = kb->data;
-	for (a = 0; a < kb->totelem; a++, fp += 3, bp++) {
-		copy_v3_v3(fp, bp->vec);
-	}
+	BKE_keyblock_update_from_lattice(lt, kb);
 }
 
-void BKE_key_convert_to_lattice(KeyBlock *kb, Lattice *lt)
+void BKE_keyblock_convert_to_lattice(KeyBlock *kb, Lattice *lt)
 {
 	BPoint *bp;
-	float *fp;
+	const float (*fp)[3];
 	int a, tot;
 
 	bp = lt->def;
 	fp = kb->data;
 
 	tot = lt->pntsu * lt->pntsv * lt->pntsw;
-	tot = mini(kb->totelem, tot);
+	tot = min_ii(kb->totelem, tot);
 
-	for (a = 0; a < tot; a++, fp += 3, bp++) {
-		copy_v3_v3(bp->vec, fp);
+	for (a = 0; a < tot; a++, fp++, bp++) {
+		copy_v3_v3(bp->vec, *fp);
 	}
 }
 
 /************************* Curve ************************/
-void BKE_key_convert_from_curve(Curve *cu, KeyBlock *kb, ListBase *nurb)
+void BKE_keyblock_update_from_curve(Curve *UNUSED(cu), KeyBlock *kb, ListBase *nurb)
 {
 	Nurb *nu;
 	BezTriple *bezt;
@@ -1601,219 +1729,198 @@ void BKE_key_convert_from_curve(Curve *cu, KeyBlock *kb, ListBase *nurb)
 	int a, tot;
 
 	/* count */
-	tot = BKE_nurbList_verts_count(nurb);
+	BLI_assert(BKE_nurbList_verts_count(nurb) == kb->totelem);
+
+	tot = kb->totelem;
 	if (tot == 0) return;
 
-	if (kb->data) MEM_freeN(kb->data);
-
-	kb->data = MEM_callocN(cu->key->elemsize * tot, "kb->data");
-	kb->totelem = tot;
-
-	nu = nurb->first;
 	fp = kb->data;
-	while (nu) {
-
+	for (nu = nurb->first; nu; nu = nu->next) {
 		if (nu->bezt) {
-			bezt = nu->bezt;
-			a = nu->pntsu;
-			while (a--) {
-				copy_v3_v3(fp, bezt->vec[0]);
-				fp += 3;
-				copy_v3_v3(fp, bezt->vec[1]);
-				fp += 3;
-				copy_v3_v3(fp, bezt->vec[2]);
-				fp += 3;
+			for (a = nu->pntsu, bezt = nu->bezt; a; a--, bezt++) {
+				int i;
+
+				for (i = 0; i < 3; i++, fp += 3) {
+					copy_v3_v3(fp, bezt->vec[i]);
+				}
 				fp[0] = bezt->alfa;
 				fp += 3; /* alphas */
-				bezt++;
 			}
 		}
 		else {
-			bp = nu->bp;
-			a = nu->pntsu * nu->pntsv;
-			while (a--) {
+			
+			;
+			for (a = nu->pntsu * nu->pntsv, bp = nu->bp; a; a--, fp += 4, bp++) {
 				copy_v3_v3(fp, bp->vec);
 				fp[3] = bp->alfa;
-
-				fp += 4;
-				bp++;
 			}
 		}
-		nu = nu->next;
 	}
 }
 
-void BKE_key_convert_to_curve(KeyBlock *kb, Curve *UNUSED(cu), ListBase *nurb)
+void BKE_keyblock_convert_from_curve(Curve *cu, KeyBlock *kb, ListBase *nurb)
+{
+	int tot;
+
+	/* count */
+	tot = BKE_nurbList_verts_count(nurb);
+	if (tot == 0) return;
+
+	MEM_SAFE_FREE(kb->data);
+
+	kb->data = MEM_mallocN(cu->key->elemsize * tot, __func__);
+	kb->totelem = tot;
+
+	BKE_keyblock_update_from_curve(cu, kb, nurb);
+}
+
+void BKE_keyblock_convert_to_curve(KeyBlock *kb, Curve *UNUSED(cu), ListBase *nurb)
 {
 	Nurb *nu;
 	BezTriple *bezt;
 	BPoint *bp;
-	float *fp;
+	const float *fp;
 	int a, tot;
 
-	nu = nurb->first;
-	fp = kb->data;
-
 	tot = BKE_nurbList_verts_count(nurb);
+	tot = min_ii(kb->totelem, tot);
 
-	tot = mini(kb->totelem, tot);
-
-	while (nu && tot > 0) {
-
+	fp = kb->data;
+	for (nu = nurb->first; nu && tot > 0; nu = nu->next) {
 		if (nu->bezt) {
-			bezt = nu->bezt;
-			a = nu->pntsu;
-			while (a-- && tot > 0) {
-				copy_v3_v3(bezt->vec[0], fp);
-				fp += 3;
-				copy_v3_v3(bezt->vec[1], fp);
-				fp += 3;
-				copy_v3_v3(bezt->vec[2], fp);
-				fp += 3;
+			for (a = nu->pntsu, bezt = nu->bezt; a && tot > 0; a--, tot -= 3, bezt++) {
+				int i;
+
+				for (i = 0; i < 3; i++, fp += 3) {
+					copy_v3_v3(bezt->vec[i], fp);
+				}
 				bezt->alfa = fp[0];
 				fp += 3; /* alphas */
-
-				tot -= 3;
-				bezt++;
 			}
 		}
 		else {
-			bp = nu->bp;
-			a = nu->pntsu * nu->pntsv;
-			while (a-- && tot > 0) {
+			for (a = nu->pntsu * nu->pntsv, bp = nu->bp; a && tot; a--, tot--, fp += 4, bp++) {
 				copy_v3_v3(bp->vec, fp);
 				bp->alfa = fp[3];
-
-				fp += 4;
-				tot--;
-				bp++;
 			}
 		}
-		nu = nu->next;
 	}
 }
 
 /************************* Mesh ************************/
-void BKE_key_convert_from_mesh(Mesh *me, KeyBlock *kb)
+void BKE_keyblock_update_from_mesh(Mesh *me, KeyBlock *kb)
 {
 	MVert *mvert;
-	float *fp;
-	int a;
+	float (*fp)[3];
+	int a, tot;
 
-	if (me->totvert == 0) return;
+	BLI_assert(me->totvert == kb->totelem);
 
-	if (kb->data) MEM_freeN(kb->data);
-
-	kb->data = MEM_callocN(me->key->elemsize * me->totvert, "kb->data");
-	kb->totelem = me->totvert;
+	tot = me->totvert;
+	if (tot == 0) return;
 
 	mvert = me->mvert;
 	fp = kb->data;
-	for (a = 0; a < kb->totelem; a++, fp += 3, mvert++) {
-		copy_v3_v3(fp, mvert->co);
-
+	for (a = 0; a < tot; a++, fp++, mvert++) {
+		copy_v3_v3(*fp, mvert->co);
 	}
 }
 
-void BKE_key_convert_to_mesh(KeyBlock *kb, Mesh *me)
+void BKE_keyblock_convert_from_mesh(Mesh *me, KeyBlock *kb)
+{
+	int tot = me->totvert;
+
+	if (me->totvert == 0) return;
+
+	MEM_SAFE_FREE(kb->data);
+
+	kb->data = MEM_mallocN(me->key->elemsize * tot, __func__);
+	kb->totelem = tot;
+
+	BKE_keyblock_update_from_mesh(me, kb);
+}
+
+void BKE_keyblock_convert_to_mesh(KeyBlock *kb, Mesh *me)
 {
 	MVert *mvert;
-	float *fp;
+	const float (*fp)[3];
 	int a, tot;
 
 	mvert = me->mvert;
 	fp = kb->data;
 
-	tot = mini(kb->totelem, me->totvert);
+	tot = min_ii(kb->totelem, me->totvert);
 
-	for (a = 0; a < tot; a++, fp += 3, mvert++) {
-		copy_v3_v3(mvert->co, fp);
+	for (a = 0; a < tot; a++, fp++, mvert++) {
+		copy_v3_v3(mvert->co, *fp);
 	}
 }
 
-/************************* vert coords ************************/
-float (*BKE_key_convert_to_vertcos(Object * ob, KeyBlock * kb))[3]
+/************************* raw coords ************************/
+void BKE_keyblock_update_from_vertcos(Object *ob, KeyBlock *kb, float (*vertCos)[3])
 {
-	float (*vertCos)[3], *co;
+	float (*co)[3] = vertCos;
 	float *fp = kb->data;
-	int tot = 0, a;
+	int tot, a;
 
-	/* Count of vertex coords in array */
-	if (ob->type == OB_MESH) {
-		Mesh *me = (Mesh *)ob->data;
-		tot = me->totvert;
-	}
-	else if (ob->type == OB_LATTICE) {
-		Lattice *lt = (Lattice *)ob->data;
-		tot = lt->pntsu * lt->pntsv * lt->pntsw;
+#ifndef NDEBUG
+	if (ob->type == OB_LATTICE) {
+		Lattice *lt = ob->data;
+		BLI_assert((lt->pntsu * lt->pntsv * lt->pntsw) == kb->totelem);
 	}
 	else if (ELEM(ob->type, OB_CURVE, OB_SURF)) {
-		Curve *cu = (Curve *)ob->data;
-		tot = BKE_nurbList_verts_count(&cu->nurb);
+		Curve *cu = ob->data;
+		BLI_assert(BKE_nurbList_verts_count(&cu->nurb) == kb->totelem);
 	}
+	else if (ob->type == OB_MESH) {
+		Mesh *me = ob->data;
+		BLI_assert(me->totvert == kb->totelem);
+	}
+	else {
+		BLI_assert(0 == kb->totelem);
+	}
+#endif
 
-	if (tot == 0) return NULL;
-
-	vertCos = MEM_callocN(tot * sizeof(*vertCos), "BKE_key_convert_to_vertcos vertCos");
-
-	/* Copy coords to array */
-	co = (float *)vertCos;
+	tot = kb->totelem;
+	if (tot == 0) return;
 
+	/* Copy coords to keyblock */
 	if (ELEM(ob->type, OB_MESH, OB_LATTICE)) {
-		for (a = 0; a < tot; a++, fp += 3, co += 3) {
-			copy_v3_v3(co, fp);
+		for (a = 0; a < tot; a++, fp += 3, co++) {
+			copy_v3_v3(fp, *co);
 		}
 	}
 	else if (ELEM(ob->type, OB_CURVE, OB_SURF)) {
 		Curve *cu = (Curve *)ob->data;
-		Nurb *nu = cu->nurb.first;
+		Nurb *nu;
 		BezTriple *bezt;
 		BPoint *bp;
 
-		while (nu) {
+		for (nu = cu->nurb.first; nu; nu = nu->next) {
 			if (nu->bezt) {
-				int i;
-				bezt = nu->bezt;
-				a = nu->pntsu;
+				for (a = nu->pntsu, bezt = nu->bezt; a; a--, bezt++) {
+					int i;
 
-				while (a--) {
-					for (i = 0; i < 3; i++) {
-						copy_v3_v3(co, fp);
-						fp += 3; co += 3;
+					for (i = 0; i < 3; i++, fp += 3, co++) {
+						copy_v3_v3(fp, *co);
 					}
-
 					fp += 3; /* skip alphas */
-
-					bezt++;
 				}
 			}
 			else {
-				bp = nu->bp;
-				a = nu->pntsu * nu->pntsv;
-
-				while (a--) {
-					copy_v3_v3(co, fp);
-
-					fp += 4;
-					co += 3;
-
-					bp++;
+				for (a = nu->pntsu * nu->pntsv, bp = nu->bp; a; a--, bp++, fp += 4, co++) {
+					copy_v3_v3(fp, *co);
 				}
 			}
-
-			nu = nu->next;
 		}
 	}
-
-	return vertCos;
 }
 
-void BKE_key_convert_from_vertcos(Object *ob, KeyBlock *kb, float (*vertCos)[3])
+void BKE_keyblock_convert_from_vertcos(Object *ob, KeyBlock *kb, float (*vertCos)[3])
 {
-	float *co = (float *)vertCos, *fp;
-	int tot = 0, a, elemsize;
+	int tot = 0, elemsize;
 
-	if (kb->data) MEM_freeN(kb->data);
+	MEM_SAFE_FREE(kb->data);
 
 	/* Count of vertex coords in array */
 	if (ob->type == OB_MESH) {
@@ -1832,110 +1939,212 @@ void BKE_key_convert_from_vertcos(Object *ob, KeyBlock *kb, float (*vertCos)[3])
 		tot = BKE_nurbList_verts_count(&cu->nurb);
 	}
 
-	if (tot == 0) {
-		kb->data = NULL;
-		return;
-	}
+	if (tot == 0) return;
 
-	fp = kb->data = MEM_callocN(tot * elemsize, "BKE_key_convert_to_vertcos vertCos");
+	kb->data = MEM_mallocN(tot * elemsize, __func__);
 
 	/* Copy coords to keyblock */
+	BKE_keyblock_update_from_vertcos(ob, kb, vertCos);
+}
 
+float (*BKE_keyblock_convert_to_vertcos(Object *ob, KeyBlock *kb))[3]
+{
+	float (*vertCos)[3], (*co)[3];
+	const float *fp = kb->data;
+	int tot = 0, a;
+
+	/* Count of vertex coords in array */
+	if (ob->type == OB_MESH) {
+		Mesh *me = (Mesh *)ob->data;
+		tot = me->totvert;
+	}
+	else if (ob->type == OB_LATTICE) {
+		Lattice *lt = (Lattice *)ob->data;
+		tot = lt->pntsu * lt->pntsv * lt->pntsw;
+	}
+	else if (ELEM(ob->type, OB_CURVE, OB_SURF)) {
+		Curve *cu = (Curve *)ob->data;
+		tot = BKE_nurbList_verts_count(&cu->nurb);
+	}
+
+	if (tot == 0) return NULL;
+
+	co = vertCos = MEM_mallocN(tot * sizeof(*vertCos), __func__);
+
+	/* Copy coords to array */
 	if (ELEM(ob->type, OB_MESH, OB_LATTICE)) {
-		for (a = 0; a < tot; a++, fp += 3, co += 3) {
-			copy_v3_v3(fp, co);
+		for (a = 0; a < tot; a++, fp += 3, co++) {
+			copy_v3_v3(*co, fp);
 		}
 	}
 	else if (ELEM(ob->type, OB_CURVE, OB_SURF)) {
 		Curve *cu = (Curve *)ob->data;
-		Nurb *nu = cu->nurb.first;
+		Nurb *nu;
 		BezTriple *bezt;
 		BPoint *bp;
 
-		while (nu) {
+		for (nu = cu->nurb.first; nu; nu = nu->next) {
 			if (nu->bezt) {
-				int i;
-				bezt = nu->bezt;
-				a = nu->pntsu;
+				for (a = nu->pntsu, bezt = nu->bezt; a; a--, bezt++) {
+					int i;
 
-				while (a--) {
-					for (i = 0; i < 3; i++) {
-						copy_v3_v3(fp, co);
-						fp += 3; co += 3;
+					for (i = 0; i < 3; i++, fp += 3, co++) {
+						copy_v3_v3(*co, fp);
 					}
-
 					fp += 3; /* skip alphas */
-
-					bezt++;
 				}
 			}
 			else {
-				bp = nu->bp;
-				a = nu->pntsu * nu->pntsv;
-
-				while (a--) {
-					copy_v3_v3(fp, co);
-
-					fp += 4;
-					co += 3;
-
-					bp++;
+				for (a = nu->pntsu * nu->pntsv, bp = nu->bp; a; a--, bp++, fp += 4, co++) {
+					copy_v3_v3(*co, fp);
 				}
 			}
-
-			nu = nu->next;
 		}
 	}
+
+	return vertCos;
 }
 
-void BKE_key_convert_from_offset(Object *ob, KeyBlock *kb, float (*ofs)[3])
+/************************* raw coord offsets ************************/
+void BKE_keyblock_update_from_offset(Object *ob, KeyBlock *kb, float (*ofs)[3])
 {
 	int a;
-	float *co = (float *)ofs, *fp = kb->data;
+	float *fp = kb->data;
 
 	if (ELEM(ob->type, OB_MESH, OB_LATTICE)) {
-		for (a = 0; a < kb->totelem; a++, fp += 3, co += 3) {
-			add_v3_v3(fp, co);
+		for (a = 0; a < kb->totelem; a++, fp += 3, ofs++) {
+			add_v3_v3(fp, *ofs);
 		}
 	}
 	else if (ELEM(ob->type, OB_CURVE, OB_SURF)) {
 		Curve *cu = (Curve *)ob->data;
-		Nurb *nu = cu->nurb.first;
+		Nurb *nu;
 		BezTriple *bezt;
 		BPoint *bp;
 
-		while (nu) {
+		for (nu = cu->nurb.first; nu; nu = nu->next) {
 			if (nu->bezt) {
-				int i;
-				bezt = nu->bezt;
-				a = nu->pntsu;
+				for (a = nu->pntsu, bezt = nu->bezt; a; a--, bezt++) {
+					int i;
 
-				while (a--) {
-					for (i = 0; i < 3; i++) {
-						add_v3_v3(fp, co);
-						fp += 3; co += 3;
+					for (i = 0; i < 3; i++, fp += 3, ofs++) {
+						add_v3_v3(fp, *ofs);
 					}
-
 					fp += 3; /* skip alphas */
-
-					bezt++;
 				}
 			}
 			else {
-				bp = nu->bp;
-				a = nu->pntsu * nu->pntsv;
-
-				while (a--) {
-					add_v3_v3(fp, co);
-
-					fp += 4;
-					co += 3;
-
-					bp++;
+				for (a = nu->pntsu * nu->pntsv, bp = nu->bp; a; a--, bp++, fp += 4, ofs++) {
+					add_v3_v3(fp, *ofs);
 				}
 			}
-
-			nu = nu->next;
 		}
 	}
 }
+
+/* ==========================================================*/
+
+/** Move shape key from org_index to new_index. Safe, clamps index to valid range, updates reference keys,
+ * the object's active shape index, the 'frame' value in case of absolute keys, etc.
+ * Note indices are expected in real values (not 'fake' shapenr +1 ones).
+ *
+ * \param org_index if < 0, current object's active shape will be used as skey to move.
+ * \return true if something was done, else false.
+ */
+bool BKE_keyblock_move(Object *ob, int org_index, int new_index)
+{
+	Key *key = BKE_key_from_object(ob);
+	KeyBlock *kb;
+	const int act_index = ob->shapenr - 1;
+	const int totkey = key->totkey;
+	int i;
+	bool rev, in_range = false;
+
+	if (org_index < 0) {
+		org_index = act_index;
+	}
+
+	CLAMP(new_index, 0, key->totkey - 1);
+	CLAMP(org_index, 0, key->totkey - 1);
+
+	if (new_index == org_index) {
+		return false;
+	}
+
+	rev = ((new_index - org_index) < 0) ? true : false;
+
+	/* We swap 'org' element with its previous/next neighbor (depending on direction of the move) repeatedly,
+	 * until we reach final position.
+	 * This allows us to only loop on the list once! */
+	for (kb = (rev ? key->block.last : key->block.first), i = (rev ? totkey - 1 : 0);
+	     kb;
+	     kb = (rev ? kb->prev : kb->next), rev ? i-- : i++)
+	{
+		if (i == org_index) {
+			in_range = true;  /* Start list items swapping... */
+		}
+		else if (i == new_index) {
+			in_range = false;  /* End list items swapping. */
+		}
+
+		if (in_range) {
+			KeyBlock *other_kb = rev ? kb->prev : kb->next;
+
+			/* Swap with previous/next list item. */
+			BLI_listbase_swaplinks(&key->block, kb, other_kb);
+
+			/* Swap absolute positions. */
+			SWAP(float, kb->pos, other_kb->pos);
+
+			kb = other_kb;
+		}
+
+		/* Adjust relative indices, this has to be done on the whole list! */
+		if (kb->relative == org_index) {
+			kb->relative = new_index;
+		}
+		else if (kb->relative < org_index && kb->relative >= new_index) {
+			/* remove after, insert before this index */
+			kb->relative++;
+		}
+		else if (kb->relative > org_index && kb->relative <= new_index) {
+			/* remove before, insert after this index */
+			kb->relative--;
+		}
+	}
+
+	/* Need to update active shape number if it's affected, same principle as for relative indices above. */
+	if (org_index == act_index) {
+		ob->shapenr = new_index + 1;
+	}
+	else if (act_index < org_index && act_index >= new_index) {
+		ob->shapenr++;
+	}
+	else if (act_index > org_index && act_index <= new_index) {
+		ob->shapenr--;
+	}
+
+	/* First key is always refkey, matches interface and BKE_key_sort */
+	key->refkey = key->block.first;
+
+	return true;
+}
+
+/**
+ * Check if given keyblock (as index) is used as basis by others in given key.
+ */
+bool BKE_keyblock_is_basis(Key *key, const int index)
+{
+	KeyBlock *kb;
+	int i;
+
+	if (key->type == KEY_RELATIVE) {
+		for (i = 0, kb = key->block.first; kb; i++, kb = kb->next) {
+			if ((i != index) && (kb->relative == index)) {
+				return true;
+			}
+		}
+	}
+
+	return false;
+}
