@@ -36,10 +36,10 @@
 
 #include "MEM_guardedalloc.h"
 
-#include "BLI_blenlib.h"
-#include "BLI_bpath.h"
-#include "BLI_math.h"
 #include "BLI_utildefines.h"
+#include "BLI_listbase.h"
+#include "BLI_bitmap.h"
+#include "BLI_math.h"
 
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
@@ -52,31 +52,98 @@
 #include "BKE_animsys.h"
 #include "BKE_anim.h"
 #include "BKE_cdderivedmesh.h"
+#include "BKE_curve.h"
 #include "BKE_displist.h"
 #include "BKE_global.h"
 #include "BKE_key.h"
 #include "BKE_lattice.h"
 #include "BKE_library.h"
 #include "BKE_main.h"
-#include "BKE_mesh.h"
 #include "BKE_modifier.h"
 
 #include "BKE_deform.h"
 
+/* Workaround for cyclic depenndnecy with curves.
+ * In such case curve_cache might not be ready yet,
+ */
+#define CYCLIC_DEPENDENCY_WORKAROUND
 
-void calc_lat_fudu(int flag, int res, float *fu, float *du)
+int BKE_lattice_index_from_uvw(Lattice *lt,
+                               const int u, const int v, const int w)
+{
+	const int totu = lt->pntsu;
+	const int totv = lt->pntsv;
+
+	return (w * (totu * totv) + (v * totu) + u);
+}
+
+void BKE_lattice_index_to_uvw(Lattice *lt, const int index,
+                              int *r_u, int *r_v, int *r_w)
+{
+	const int totu = lt->pntsu;
+	const int totv = lt->pntsv;
+
+	*r_u = (index % totu);
+	*r_v = (index / totu) % totv;
+	*r_w = (index / (totu * totv));
+}
+
+int BKE_lattice_index_flip(Lattice *lt, const int index,
+                           const bool flip_u, const bool flip_v, const bool flip_w)
+{
+	int u, v, w;
+
+	BKE_lattice_index_to_uvw(lt, index, &u, &v, &w);
+
+	if (flip_u) {
+		u = (lt->pntsu - 1) - u;
+	}
+
+	if (flip_v) {
+		v = (lt->pntsv - 1) - v;
+	}
+
+	if (flip_w) {
+		w = (lt->pntsw - 1) - w;
+	}
+
+	return BKE_lattice_index_from_uvw(lt, u, v, w);
+}
+
+void BKE_lattice_bitmap_from_flag(Lattice *lt, BLI_bitmap *bitmap, const short flag,
+                                  const bool clear, const bool respecthide)
+{
+	const unsigned int tot = lt->pntsu * lt->pntsv * lt->pntsw;
+	unsigned int i;
+	BPoint *bp;
+
+	bp = lt->def;
+	for (i = 0; i < tot; i++, bp++) {
+		if ((bp->f1 & flag) && (!respecthide || !bp->hide)) {
+			BLI_BITMAP_ENABLE(bitmap, i);
+		}
+		else {
+			if (clear) {
+				BLI_BITMAP_DISABLE(bitmap, i);
+			}
+		}
+	}
+
+}
+
+void calc_lat_fudu(int flag, int res, float *r_fu, float *r_du)
 {
 	if (res == 1) {
-		*fu = 0.0;
-		*du = 0.0;
+		*r_fu = 0.0;
+		*r_du = 0.0;
 	}
 	else if (flag & LT_GRID) {
-		*fu = -0.5f * (res - 1);
-		*du = 1.0f;
+		*r_fu = -0.5f * (res - 1);
+		*r_du = 1.0f;
 	}
 	else {
-		*fu = -1.0f;
-		*du = 2.0f / (res - 1);
+		*r_fu = -1.0f;
+		*r_du = 2.0f / (res - 1);
 	}
 }
 
@@ -89,7 +156,7 @@ void BKE_lattice_resize(Lattice *lt, int uNew, int vNew, int wNew, Object *ltOb)
 	
 	/* vertex weight groups are just freed all for now */
 	if (lt->dvert) {
-		free_dverts(lt->dvert, lt->pntsu * lt->pntsv * lt->pntsw);
+		BKE_defvert_array_free(lt->dvert, lt->pntsu * lt->pntsv * lt->pntsw);
 		lt->dvert = NULL;
 	}
 	
@@ -146,7 +213,7 @@ void BKE_lattice_resize(Lattice *lt, int uNew, int vNew, int wNew, Object *ltOb)
 		lt->typeu = lt->typev = lt->typew = KEY_LINEAR;
 
 		/* prevent using deformed locations */
-		BKE_displist_free(&ltOb->disp);
+		BKE_displist_free(&ltOb->curve_cache->disp);
 
 		copy_m4_m4(mat, ltOb->obmat);
 		unit_m4(ltOb->obmat);
@@ -169,6 +236,7 @@ void BKE_lattice_resize(Lattice *lt, int uNew, int vNew, int wNew, Object *ltOb)
 	lt->pntsv = vNew;
 	lt->pntsw = wNew;
 
+	lt->actbp = LT_ACTBP_NONE;
 	MEM_freeN(lt->def);
 	lt->def = MEM_callocN(lt->pntsu * lt->pntsv * lt->pntsw * sizeof(BPoint), "lattice bp");
 	
@@ -181,11 +249,11 @@ void BKE_lattice_resize(Lattice *lt, int uNew, int vNew, int wNew, Object *ltOb)
 	MEM_freeN(vertexCos);
 }
 
-Lattice *BKE_lattice_add(const char *name)
+Lattice *BKE_lattice_add(Main *bmain, const char *name)
 {
 	Lattice *lt;
 	
-	lt = BKE_libblock_alloc(&G.main->latt, ID_LT, name);
+	lt = BKE_libblock_alloc(bmain, ID_LT, name);
 	
 	lt->flag = LT_GRID;
 	
@@ -193,6 +261,7 @@ Lattice *BKE_lattice_add(const char *name)
 	
 	lt->def = MEM_callocN(sizeof(BPoint), "lattvert"); /* temporary */
 	BKE_lattice_resize(lt, 2, 2, 2, NULL);  /* creates a uniform lattice */
+	lt->actbp = LT_ACTBP_NONE;
 		
 	return lt;
 }
@@ -210,7 +279,7 @@ Lattice *BKE_lattice_copy(Lattice *lt)
 	if (lt->dvert) {
 		int tot = lt->pntsu * lt->pntsv * lt->pntsw;
 		ltn->dvert = MEM_mallocN(sizeof(MDeformVert) * tot, "Lattice MDeformVert");
-		copy_dverts(ltn->dvert, lt->dvert, tot);
+		BKE_defvert_array_copy(ltn->dvert, lt->dvert, tot);
 	}
 
 	ltn->editlatt = NULL;
@@ -221,12 +290,12 @@ Lattice *BKE_lattice_copy(Lattice *lt)
 void BKE_lattice_free(Lattice *lt)
 {
 	if (lt->def) MEM_freeN(lt->def);
-	if (lt->dvert) free_dverts(lt->dvert, lt->pntsu * lt->pntsv * lt->pntsw);
+	if (lt->dvert) BKE_defvert_array_free(lt->dvert, lt->pntsu * lt->pntsv * lt->pntsw);
 	if (lt->editlatt) {
 		Lattice *editlt = lt->editlatt->latt;
 
 		if (editlt->def) MEM_freeN(editlt->def);
-		if (editlt->dvert) free_dverts(editlt->dvert, lt->pntsu * lt->pntsv * lt->pntsw);
+		if (editlt->dvert) BKE_defvert_array_free(editlt->dvert, lt->pntsu * lt->pntsv * lt->pntsw);
 
 		MEM_freeN(editlt);
 		MEM_freeN(lt->editlatt);
@@ -244,7 +313,7 @@ void BKE_lattice_make_local(Lattice *lt)
 {
 	Main *bmain = G.main;
 	Object *ob;
-	int is_local = FALSE, is_lib = FALSE;
+	bool is_local = false, is_lib = false;
 
 	/* - only lib users: do nothing
 	 * - only local users: set flag
@@ -257,14 +326,14 @@ void BKE_lattice_make_local(Lattice *lt)
 		return;
 	}
 	
-	for (ob = bmain->object.first; ob && ELEM(FALSE, is_lib, is_local); ob = ob->id.next) {
+	for (ob = bmain->object.first; ob && ELEM(false, is_lib, is_local); ob = ob->id.next) {
 		if (ob->data == lt) {
-			if (ob->id.lib) is_lib = TRUE;
-			else is_local = TRUE;
+			if (ob->id.lib) is_lib = true;
+			else is_local = true;
 		}
 	}
 	
-	if (is_local && is_lib == FALSE) {
+	if (is_local && is_lib == false) {
 		id_clear_lib_data(bmain, &lt->id);
 	}
 	else if (is_local && is_lib) {
@@ -286,37 +355,46 @@ void BKE_lattice_make_local(Lattice *lt)
 	}
 }
 
-void init_latt_deform(Object *oblatt, Object *ob)
+typedef struct LatticeDeformData {
+	Object *object;
+	float *latticedata;
+	float latmat[4][4];
+} LatticeDeformData;
+
+LatticeDeformData *init_latt_deform(Object *oblatt, Object *ob)
 {
 	/* we make an array with all differences */
 	Lattice *lt = oblatt->data;
 	BPoint *bp;
-	DispList *dl = BKE_displist_find(&oblatt->disp, DL_VERTS);
-	float *co = dl ? dl->verts : NULL;
+	DispList *dl = oblatt->curve_cache ? BKE_displist_find(&oblatt->curve_cache->disp, DL_VERTS) : NULL;
+	const float *co = dl ? dl->verts : NULL;
 	float *fp, imat[4][4];
 	float fu, fv, fw;
 	int u, v, w;
+	float *latticedata;
+	float latmat[4][4];
+	LatticeDeformData *lattice_deform_data;
 
 	if (lt->editlatt) lt = lt->editlatt->latt;
 	bp = lt->def;
 	
-	fp = lt->latticedata = MEM_mallocN(sizeof(float) * 3 * lt->pntsu * lt->pntsv * lt->pntsw, "latticedata");
+	fp = latticedata = MEM_mallocN(sizeof(float) * 3 * lt->pntsu * lt->pntsv * lt->pntsw, "latticedata");
 	
 	/* for example with a particle system: (ob == NULL) */
 	if (ob == NULL) {
 		/* in deformspace, calc matrix  */
-		invert_m4_m4(lt->latmat, oblatt->obmat);
+		invert_m4_m4(latmat, oblatt->obmat);
 	
 		/* back: put in deform array */
-		invert_m4_m4(imat, lt->latmat);
+		invert_m4_m4(imat, latmat);
 	}
 	else {
 		/* in deformspace, calc matrix */
 		invert_m4_m4(imat, oblatt->obmat);
-		mult_m4_m4m4(lt->latmat, imat, ob->obmat);
+		mul_m4_m4m4(latmat, imat, ob->obmat);
 	
 		/* back: put in deform array */
-		invert_m4_m4(imat, lt->latmat);
+		invert_m4_m4(imat, latmat);
 	}
 	
 	for (w = 0, fw = lt->fw; w < lt->pntsw; w++, fw += lt->dw) {
@@ -337,10 +415,18 @@ void init_latt_deform(Object *oblatt, Object *ob)
 			}
 		}
 	}
+
+	lattice_deform_data = MEM_mallocN(sizeof(LatticeDeformData), "Lattice Deform Data");
+	lattice_deform_data->latticedata = latticedata;
+	lattice_deform_data->object = oblatt;
+	copy_m4_m4(lattice_deform_data->latmat, latmat);
+
+	return lattice_deform_data;
 }
 
-void calc_latt_deform(Object *ob, float co[3], float weight)
+void calc_latt_deform(LatticeDeformData *lattice_deform_data, float co[3], float weight)
 {
+	Object *ob = lattice_deform_data->object;
 	Lattice *lt = ob->data;
 	float u, v, w, tu[4], tv[4], tw[4];
 	float vec[3];
@@ -348,21 +434,21 @@ void calc_latt_deform(Object *ob, float co[3], float weight)
 	int ui, vi, wi, uu, vv, ww;
 
 	/* vgroup influence */
-	int defgroup_nr = -1;
+	int defgrp_index = -1;
 	float co_prev[3], weight_blend = 0.0f;
 	MDeformVert *dvert = BKE_lattice_deform_verts_get(ob);
 
 
 	if (lt->editlatt) lt = lt->editlatt->latt;
-	if (lt->latticedata == NULL) return;
+	if (lattice_deform_data->latticedata == NULL) return;
 
 	if (lt->vgroup[0] && dvert) {
-		defgroup_nr = defgroup_name_index(ob, lt->vgroup);
+		defgrp_index = defgroup_name_index(ob, lt->vgroup);
 		copy_v3_v3(co_prev, co);
 	}
 
 	/* co is in local coords, treat with latmat */
-	mul_v3_m4v3(vec, lt->latmat, co);
+	mul_v3_m4v3(vec, lattice_deform_data->latmat, co);
 
 	/* u v w coords */
 
@@ -405,9 +491,11 @@ void calc_latt_deform(Object *ob, float co[3], float weight)
 		if (w != 0.0f) {
 			if (ww > 0) {
 				if (ww < lt->pntsw) idx_w = ww * lt->pntsu * lt->pntsv;
-				else idx_w = (lt->pntsw - 1) * lt->pntsu * lt->pntsv;
+				else                idx_w = (lt->pntsw - 1) * lt->pntsu * lt->pntsv;
+			}
+			else {
+				idx_w = 0;
 			}
-			else idx_w = 0;
 
 			for (vv = vi - 1; vv <= vi + 2; vv++) {
 				v = w * tv[vv - vi + 1];
@@ -415,9 +503,11 @@ void calc_latt_deform(Object *ob, float co[3], float weight)
 				if (v != 0.0f) {
 					if (vv > 0) {
 						if (vv < lt->pntsv) idx_v = idx_w + vv * lt->pntsu;
-						else idx_v = idx_w + (lt->pntsv - 1) * lt->pntsu;
+						else                idx_v = idx_w + (lt->pntsv - 1) * lt->pntsu;
+					}
+					else {
+						idx_v = idx_w;
 					}
-					else idx_v = idx_w;
 
 					for (uu = ui - 1; uu <= ui + 2; uu++) {
 						u = weight * v * tu[uu - ui + 1];
@@ -425,14 +515,16 @@ void calc_latt_deform(Object *ob, float co[3], float weight)
 						if (u != 0.0f) {
 							if (uu > 0) {
 								if (uu < lt->pntsu) idx_u = idx_v + uu;
-								else idx_u = idx_v + (lt->pntsu - 1);
+								else                idx_u = idx_v + (lt->pntsu - 1);
+							}
+							else {
+								idx_u = idx_v;
 							}
-							else idx_u = idx_v;
 
-							madd_v3_v3fl(co, &lt->latticedata[idx_u * 3], u);
+							madd_v3_v3fl(co, &lattice_deform_data->latticedata[idx_u * 3], u);
 
-							if (defgroup_nr != -1)
-								weight_blend += (u * defvert_find_weight(dvert + idx_u, defgroup_nr));
+							if (defgrp_index != -1)
+								weight_blend += (u * defvert_find_weight(dvert + idx_u, defgrp_index));
 						}
 					}
 				}
@@ -440,20 +532,17 @@ void calc_latt_deform(Object *ob, float co[3], float weight)
 		}
 	}
 
-	if (defgroup_nr != -1)
+	if (defgrp_index != -1)
 		interp_v3_v3v3(co, co_prev, co, weight_blend);
 
 }
 
-void end_latt_deform(Object *ob)
+void end_latt_deform(LatticeDeformData *lattice_deform_data)
 {
-	Lattice *lt = ob->data;
-	
-	if (lt->editlatt) lt = lt->editlatt->latt;
-	
-	if (lt->latticedata)
-		MEM_freeN(lt->latticedata);
-	lt->latticedata = NULL;
+	if (lattice_deform_data->latticedata)
+		MEM_freeN(lattice_deform_data->latticedata);
+
+	MEM_freeN(lattice_deform_data);
 }
 
 /* calculations is in local space of deformed object
@@ -468,7 +557,7 @@ typedef struct {
 static void init_curve_deform(Object *par, Object *ob, CurveDeform *cd)
 {
 	invert_m4_m4(ob->imat, ob->obmat);
-	mult_m4_m4m4(cd->objectspace, ob->imat, par->obmat);
+	mul_m4_m4m4(cd->objectspace, ob->imat, par->obmat);
 	invert_m4_m4(cd->curvespace, cd->objectspace);
 	copy_m3_m4(cd->objectspace3, cd->objectspace);
 	cd->no_rot_axis = 0;
@@ -478,28 +567,29 @@ static void init_curve_deform(Object *par, Object *ob, CurveDeform *cd)
  *
  * returns OK: 1/0
  */
-static int where_on_path_deform(Object *ob, float ctime, float vec[4], float dir[3], float quat[4], float *radius)
+static bool where_on_path_deform(Object *ob, float ctime, float vec[4], float dir[3], float quat[4], float *radius)
 {
-	Curve *cu = ob->data;
 	BevList *bl;
 	float ctime1;
 	int cycl = 0;
 	
 	/* test for cyclic */
-	bl = cu->bev.first;
-	if (!bl->nr) return 0;
+	bl = ob->curve_cache->bev.first;
+	if (!bl->nr) return false;
 	if (bl->poly > -1) cycl = 1;
 
 	if (cycl == 0) {
 		ctime1 = CLAMPIS(ctime, 0.0f, 1.0f);
 	}
-	else ctime1 = ctime;
+	else {
+		ctime1 = ctime;
+	}
 	
 	/* vec needs 4 items */
 	if (where_on_path(ob, ctime1, vec, dir, quat, radius, NULL)) {
 		
 		if (cycl == 0) {
-			Path *path = cu->path;
+			Path *path = ob->curve_cache->path;
 			float dvec[3];
 			
 			if (ctime < 0.0f) {
@@ -518,9 +608,9 @@ static int where_on_path_deform(Object *ob, float ctime, float vec[4], float dir
 				/* weight - not used but could be added */
 			}
 		}
-		return 1;
+		return true;
 	}
-	return 0;
+	return false;
 }
 
 /* for each point, rotate & translate to curve */
@@ -528,34 +618,46 @@ static int where_on_path_deform(Object *ob, float ctime, float vec[4], float dir
 /* co: local coord, result local too */
 /* returns quaternion for rotation, using cd->no_rot_axis */
 /* axis is using another define!!! */
-static int calc_curve_deform(Scene *scene, Object *par, float co[3],
-                             const short axis, CurveDeform *cd, float quat_r[4])
+static bool calc_curve_deform(Scene *scene, Object *par, float co[3],
+                              const short axis, CurveDeform *cd, float r_quat[4])
 {
 	Curve *cu = par->data;
 	float fac, loc[4], dir[3], new_quat[4], radius;
 	short index;
-	const int is_neg_axis = (axis > 2);
+	const bool is_neg_axis = (axis > 2);
 
-	/* to be sure, mostly after file load */
-	if (cu->path == NULL) {
-		BKE_displist_make_curveTypes(scene, par, 0);
-		if (cu->path == NULL) return 0;  // happens on append...
+	/* to be sure, mostly after file load, also cyclic dependencies */
+#ifdef CYCLIC_DEPENDENCY_WORKAROUND
+	if (par->curve_cache == NULL) {
+		BKE_displist_make_curveTypes(scene, par, false);
 	}
-	
+#endif
+
+	if (par->curve_cache->path == NULL) {
+		return false;  /* happens on append, cyclic dependencies and empty curves */
+	}
+
 	/* options */
 	if (is_neg_axis) {
 		index = axis - 3;
 		if (cu->flag & CU_STRETCH)
 			fac = (-co[index] - cd->dmax[index]) / (cd->dmax[index] - cd->dmin[index]);
 		else
-			fac = -(co[index] - cd->dmax[index]) / (cu->path->totdist);
+			fac = -(co[index] - cd->dmax[index]) / (par->curve_cache->path->totdist);
 	}
 	else {
 		index = axis;
-		if (cu->flag & CU_STRETCH)
+		if (cu->flag & CU_STRETCH) {
 			fac = (co[index] - cd->dmin[index]) / (cd->dmax[index] - cd->dmin[index]);
-		else
-			fac = +(co[index] - cd->dmin[index]) / (cu->path->totdist);
+		}
+		else {
+			if (LIKELY(par->curve_cache->path->totdist > FLT_EPSILON)) {
+				fac = +(co[index] - cd->dmin[index]) / (par->curve_cache->path->totdist);
+			}
+			else {
+				fac = 0.0f;
+			}
+		}
 	}
 	
 	if (where_on_path_deform(par, fac, loc, dir, new_quat, &radius)) {  /* returns OK */
@@ -611,35 +713,33 @@ static int calc_curve_deform(Scene *scene, Object *par, float co[3],
 		/* translation */
 		add_v3_v3v3(co, cent, loc);
 
-		if (quat_r)
-			copy_qt_qt(quat_r, quat);
+		if (r_quat)
+			copy_qt_qt(r_quat, quat);
 
-		return 1;
+		return true;
 	}
-	return 0;
+	return false;
 }
 
-void curve_deform_verts(Scene *scene, Object *cuOb, Object *target,
-                        DerivedMesh *dm, float (*vertexCos)[3],
+void curve_deform_verts(Scene *scene, Object *cuOb, Object *target, DerivedMesh *dm, float (*vertexCos)[3],
                         int numVerts, const char *vgroup, short defaxis)
 {
 	Curve *cu;
-	int a, flag;
+	int a;
 	CurveDeform cd;
-	int use_vgroups;
-	const int is_neg_axis = (defaxis > 2);
+	MDeformVert *dvert = NULL;
+	int defgrp_index = -1;
+	const bool is_neg_axis = (defaxis > 2);
 
 	if (cuOb->type != OB_CURVE)
 		return;
 
 	cu = cuOb->data;
-	flag = cu->flag;
-	cu->flag |= (CU_PATH | CU_FOLLOW); // needed for path & bevlist
 
 	init_curve_deform(cuOb, target, &cd);
 
 	/* dummy bounds, keep if CU_DEFORM_BOUNDS_OFF is set */
-	if (is_neg_axis == FALSE) {
+	if (is_neg_axis == false) {
 		cd.dmin[0] = cd.dmin[1] = cd.dmin[2] = 0.0f;
 		cd.dmax[0] = cd.dmax[1] = cd.dmax[2] = 1.0f;
 	}
@@ -649,75 +749,63 @@ void curve_deform_verts(Scene *scene, Object *cuOb, Object *target,
 		cd.dmax[0] = cd.dmax[1] = cd.dmax[2] =  0.0f;
 	}
 	
-	/* check whether to use vertex groups (only possible if target is a Mesh)
-	 * we want either a Mesh with no derived data, or derived data with
-	 * deformverts
+	/* Check whether to use vertex groups (only possible if target is a Mesh or Lattice).
+	 * We want either a Mesh/Lattice with no derived data, or derived data with deformverts.
 	 */
-	if (target && target->type == OB_MESH) {
-		/* if there's derived data without deformverts, don't use vgroups */
-		if (dm) {
-			use_vgroups = (dm->getVertData(dm, 0, CD_MDEFORMVERT) != NULL);
-		}
-		else {
-			Mesh *me = target->data;
-			use_vgroups = (me->dvert != NULL);
-		}
-	}
-	else {
-		use_vgroups = FALSE;
-	}
-	
-	if (vgroup && vgroup[0] && use_vgroups) {
-		Mesh *me = target->data;
-		int index = defgroup_name_index(target, vgroup);
+	if (vgroup && vgroup[0] && ELEM(target->type, OB_MESH, OB_LATTICE)) {
+		defgrp_index = defgroup_name_index(target, vgroup);
 
-		if (index != -1 && (me->dvert || dm)) {
-			MDeformVert *dvert = me->dvert;
-			float vec[3];
-			float weight;
-	
-
-			if (cu->flag & CU_DEFORM_BOUNDS_OFF) {
-				dvert = me->dvert;
-				for (a = 0; a < numVerts; a++, dvert++) {
-					if (dm) dvert = dm->getVertData(dm, a, CD_MDEFORMVERT);
-					weight = defvert_find_weight(dvert, index);
-	
-					if (weight > 0.0f) {
-						mul_m4_v3(cd.curvespace, vertexCos[a]);
-						copy_v3_v3(vec, vertexCos[a]);
-						calc_curve_deform(scene, cuOb, vec, defaxis, &cd, NULL);
-						interp_v3_v3v3(vertexCos[a], vertexCos[a], vec, weight);
-						mul_m4_v3(cd.objectspace, vertexCos[a]);
-					}
-				}
+		if (defgrp_index != -1) {
+			/* if there's derived data without deformverts, don't use vgroups */
+			if (dm) {
+				dvert = dm->getVertDataArray(dm, CD_MDEFORMVERT);
+			}
+			else if (target->type == OB_LATTICE) {
+				dvert = ((Lattice *)target->data)->dvert;
 			}
 			else {
-				/* set mesh min/max bounds */
-				INIT_MINMAX(cd.dmin, cd.dmax);
-	
-				for (a = 0; a < numVerts; a++, dvert++) {
-					if (dm) dvert = dm->getVertData(dm, a, CD_MDEFORMVERT);
-					
-					if (defvert_find_weight(dvert, index) > 0.0f) {
-						mul_m4_v3(cd.curvespace, vertexCos[a]);
-						minmax_v3v3_v3(cd.dmin, cd.dmax, vertexCos[a]);
-					}
+				dvert = ((Mesh *)target->data)->dvert;
+			}
+		}
+	}
+
+	if (dvert) {
+		MDeformVert *dvert_iter;
+		float vec[3];
+
+		if (cu->flag & CU_DEFORM_BOUNDS_OFF) {
+			for (a = 0, dvert_iter = dvert; a < numVerts; a++, dvert_iter++) {
+				const float weight = defvert_find_weight(dvert_iter, defgrp_index);
+
+				if (weight > 0.0f) {
+					mul_m4_v3(cd.curvespace, vertexCos[a]);
+					copy_v3_v3(vec, vertexCos[a]);
+					calc_curve_deform(scene, cuOb, vec, defaxis, &cd, NULL);
+					interp_v3_v3v3(vertexCos[a], vertexCos[a], vec, weight);
+					mul_m4_v3(cd.objectspace, vertexCos[a]);
 				}
-	
-				dvert = me->dvert;
-				for (a = 0; a < numVerts; a++, dvert++) {
-					if (dm) dvert = dm->getVertData(dm, a, CD_MDEFORMVERT);
-					
-					weight = defvert_find_weight(dvert, index);
-	
-					if (weight > 0.0f) {
-						/* already in 'cd.curvespace', prev for loop */
-						copy_v3_v3(vec, vertexCos[a]);
-						calc_curve_deform(scene, cuOb, vec, defaxis, &cd, NULL);
-						interp_v3_v3v3(vertexCos[a], vertexCos[a], vec, weight);
-						mul_m4_v3(cd.objectspace, vertexCos[a]);
-					}
+			}
+		}
+		else {
+			/* set mesh min/max bounds */
+			INIT_MINMAX(cd.dmin, cd.dmax);
+
+			for (a = 0, dvert_iter = dvert; a < numVerts; a++, dvert_iter++) {
+				if (defvert_find_weight(dvert_iter, defgrp_index) > 0.0f) {
+					mul_m4_v3(cd.curvespace, vertexCos[a]);
+					minmax_v3v3_v3(cd.dmin, cd.dmax, vertexCos[a]);
+				}
+			}
+
+			for (a = 0, dvert_iter = dvert; a < numVerts; a++, dvert_iter++) {
+				const float weight = defvert_find_weight(dvert_iter, defgrp_index);
+
+				if (weight > 0.0f) {
+					/* already in 'cd.curvespace', prev for loop */
+					copy_v3_v3(vec, vertexCos[a]);
+					calc_curve_deform(scene, cuOb, vec, defaxis, &cd, NULL);
+					interp_v3_v3v3(vertexCos[a], vertexCos[a], vec, weight);
+					mul_m4_v3(cd.objectspace, vertexCos[a]);
 				}
 			}
 		}
@@ -746,14 +834,13 @@ void curve_deform_verts(Scene *scene, Object *cuOb, Object *target,
 			}
 		}
 	}
-	cu->flag = flag;
 }
 
 /* input vec and orco = local coord in armature space */
 /* orco is original not-animated or deformed reference point */
 /* result written in vec and mat */
 void curve_deform_vector(Scene *scene, Object *cuOb, Object *target,
-                         float orco[3], float vec[3], float mat[][3], int no_rot_axis)
+                         float orco[3], float vec[3], float mat[3][3], int no_rot_axis)
 {
 	CurveDeform cd;
 	float quat[4];
@@ -787,13 +874,14 @@ void curve_deform_vector(Scene *scene, Object *cuOb, Object *target,
 void lattice_deform_verts(Object *laOb, Object *target, DerivedMesh *dm,
                           float (*vertexCos)[3], int numVerts, const char *vgroup, float fac)
 {
+	LatticeDeformData *lattice_deform_data;
 	int a;
-	int use_vgroups;
+	bool use_vgroups;
 
 	if (laOb->type != OB_LATTICE)
 		return;
 
-	init_latt_deform(laOb, target);
+	lattice_deform_data = init_latt_deform(laOb, target);
 
 	/* check whether to use vertex groups (only possible if target is a Mesh)
 	 * we want either a Mesh with no derived data, or derived data with
@@ -802,7 +890,7 @@ void lattice_deform_verts(Object *laOb, Object *target, DerivedMesh *dm,
 	if (target && target->type == OB_MESH) {
 		/* if there's derived data without deformverts, don't use vgroups */
 		if (dm) {
-			use_vgroups = (dm->getVertData(dm, 0, CD_MDEFORMVERT) != NULL);
+			use_vgroups = (dm->getVertDataArray(dm, CD_MDEFORMVERT) != NULL);
 		}
 		else {
 			Mesh *me = target->data;
@@ -810,36 +898,36 @@ void lattice_deform_verts(Object *laOb, Object *target, DerivedMesh *dm,
 		}
 	}
 	else {
-		use_vgroups = FALSE;
+		use_vgroups = false;
 	}
 	
 	if (vgroup && vgroup[0] && use_vgroups) {
 		Mesh *me = target->data;
-		int index = defgroup_name_index(target, vgroup);
+		const int defgrp_index = defgroup_name_index(target, vgroup);
 		float weight;
 
-		if (index >= 0 && (me->dvert || dm)) {
+		if (defgrp_index >= 0 && (me->dvert || dm)) {
 			MDeformVert *dvert = me->dvert;
 			
 			for (a = 0; a < numVerts; a++, dvert++) {
 				if (dm) dvert = dm->getVertData(dm, a, CD_MDEFORMVERT);
 
-				weight = defvert_find_weight(dvert, index);
+				weight = defvert_find_weight(dvert, defgrp_index);
 
 				if (weight > 0.0f)
-					calc_latt_deform(laOb, vertexCos[a], weight * fac);
+					calc_latt_deform(lattice_deform_data, vertexCos[a], weight * fac);
 			}
 		}
 	}
 	else {
 		for (a = 0; a < numVerts; a++) {
-			calc_latt_deform(laOb, vertexCos[a], fac);
+			calc_latt_deform(lattice_deform_data, vertexCos[a], fac);
 		}
 	}
-	end_latt_deform(laOb);
+	end_latt_deform(lattice_deform_data);
 }
 
-int object_deform_mball(Object *ob, ListBase *dispbase)
+bool object_deform_mball(Object *ob, ListBase *dispbase)
 {
 	if (ob->parent && ob->parent->type == OB_LATTICE && ob->partype == PARSKEL) {
 		DispList *dl;
@@ -849,16 +937,16 @@ int object_deform_mball(Object *ob, ListBase *dispbase)
 			                     (float(*)[3])dl->verts, dl->nr, NULL, 1.0f);
 		}
 
-		return 1;
+		return true;
 	}
 	else {
-		return 0;
+		return false;
 	}
 }
 
 static BPoint *latt_bp(Lattice *lt, int u, int v, int w)
 {
-	return &lt->def[LT_INDEX(lt, u, v, w)];
+	return &lt->def[BKE_lattice_index_from_uvw(lt, u, v, w)];
 }
 
 void outside_lattice(Lattice *lt)
@@ -913,7 +1001,7 @@ void outside_lattice(Lattice *lt)
 						bp->vec[1] += (1.0f - fac1) * bp1->vec[1] + fac1 * bp2->vec[1];
 						bp->vec[2] += (1.0f - fac1) * bp1->vec[2] + fac1 * bp2->vec[2];
 						
-						mul_v3_fl(bp->vec, 0.3333333f);
+						mul_v3_fl(bp->vec, 1.0f / 3.0f);
 						
 					}
 				}
@@ -932,14 +1020,14 @@ void outside_lattice(Lattice *lt)
 	}
 }
 
-float (*BKE_lattice_vertexcos_get(struct Object *ob, int *numVerts_r))[3]
+float (*BKE_lattice_vertexcos_get(struct Object *ob, int *r_numVerts))[3]
 {
 	Lattice *lt = ob->data;
 	int i, numVerts;
 	float (*vertexCos)[3];
 
 	if (lt->editlatt) lt = lt->editlatt->latt;
-	numVerts = *numVerts_r = lt->pntsu * lt->pntsv * lt->pntsw;
+	numVerts = *r_numVerts = lt->pntsu * lt->pntsv * lt->pntsw;
 	
 	vertexCos = MEM_mallocN(sizeof(*vertexCos) * numVerts, "lt_vcos");
 	
@@ -963,11 +1051,17 @@ void BKE_lattice_vertexcos_apply(struct Object *ob, float (*vertexCos)[3])
 void BKE_lattice_modifiers_calc(Scene *scene, Object *ob)
 {
 	Lattice *lt = ob->data;
-	ModifierData *md = modifiers_getVirtualModifierList(ob);
+	VirtualModifierData virtualModifierData;
+	ModifierData *md = modifiers_getVirtualModifierList(ob, &virtualModifierData);
 	float (*vertexCos)[3] = NULL;
 	int numVerts, editmode = (lt->editlatt != NULL);
 
-	BKE_displist_free(&ob->disp);
+	if (ob->curve_cache) {
+		BKE_displist_free(&ob->curve_cache->disp);
+	}
+	else {
+		ob->curve_cache = MEM_callocN(sizeof(CurveCache), "CurveCache for lattice");
+	}
 
 	for (; md; md = md->next) {
 		ModifierTypeInfo *mti = modifierType_getInfo(md->type);
@@ -993,7 +1087,7 @@ void BKE_lattice_modifiers_calc(Scene *scene, Object *ob)
 		dl->nr = numVerts;
 		dl->verts = (float *) vertexCos;
 		
-		BLI_addtail(&ob->disp, dl);
+		BLI_addtail(&ob->curve_cache->disp, dl);
 	}
 }
 
@@ -1004,3 +1098,106 @@ struct MDeformVert *BKE_lattice_deform_verts_get(struct Object *oblatt)
 	if (lt->editlatt) lt = lt->editlatt->latt;
 	return lt->dvert;
 }
+
+struct BPoint *BKE_lattice_active_point_get(Lattice *lt)
+{
+	BLI_assert(GS(lt->id.name) == ID_LT);
+
+	if (lt->editlatt) {
+		lt = lt->editlatt->latt;
+	}
+
+	BLI_assert(lt->actbp < lt->pntsu * lt->pntsv * lt->pntsw);
+
+	if ((lt->actbp != LT_ACTBP_NONE) && (lt->actbp < lt->pntsu * lt->pntsv * lt->pntsw)) {
+		return &lt->def[lt->actbp];
+	}
+	else {
+		return NULL;
+	}
+}
+
+void BKE_lattice_center_median(Lattice *lt, float cent[3])
+{
+	int i, numVerts;
+
+	if (lt->editlatt) lt = lt->editlatt->latt;
+	numVerts = lt->pntsu * lt->pntsv * lt->pntsw;
+
+	zero_v3(cent);
+
+	for (i = 0; i < numVerts; i++)
+		add_v3_v3(cent, lt->def[i].vec);
+
+	mul_v3_fl(cent, 1.0f / (float)numVerts);
+}
+
+void BKE_lattice_minmax(Lattice *lt, float min[3], float max[3])
+{
+	int i, numVerts;
+
+	if (lt->editlatt) lt = lt->editlatt->latt;
+	numVerts = lt->pntsu * lt->pntsv * lt->pntsw;
+
+	for (i = 0; i < numVerts; i++)
+		minmax_v3v3_v3(min, max, lt->def[i].vec);
+}
+
+void BKE_lattice_center_bounds(Lattice *lt, float cent[3])
+{
+	float min[3], max[3];
+
+	INIT_MINMAX(min, max);
+
+	BKE_lattice_minmax(lt, min, max);
+	mid_v3_v3v3(cent, min, max);
+}
+
+void BKE_lattice_transform(Lattice *lt, float mat[4][4], bool do_keys)
+{
+	BPoint *bp = lt->def;
+	int i = lt->pntsu * lt->pntsv * lt->pntsw;
+
+	while (i--) {
+		mul_m4_v3(mat, bp->vec);
+		bp++;
+	}
+
+	if (do_keys && lt->key) {
+		KeyBlock *kb;
+
+		for (kb = lt->key->block.first; kb; kb = kb->next) {
+			float *fp = kb->data;
+			for (i = kb->totelem; i--; fp += 3) {
+				mul_m4_v3(mat, fp);
+			}
+		}
+	}
+}
+
+void BKE_lattice_translate(Lattice *lt, float offset[3], bool do_keys)
+{
+	int i, numVerts;
+
+	numVerts = lt->pntsu * lt->pntsv * lt->pntsw;
+
+	if (lt->def)
+		for (i = 0; i < numVerts; i++)
+			add_v3_v3(lt->def[i].vec, offset);
+
+	if (lt->editlatt)
+		for (i = 0; i < numVerts; i++)
+			add_v3_v3(lt->editlatt->latt->def[i].vec, offset);
+
+	if (do_keys && lt->key) {
+		KeyBlock *kb;
+
+		for (kb = lt->key->block.first; kb; kb = kb->next) {
+			float *fp = kb->data;
+			for (i = kb->totelem; i--; fp += 3) {
+				add_v3_v3(fp, offset);
+			}
+		}
+	}
+}
+
