@@ -35,11 +35,14 @@
 #include "MEM_guardedalloc.h"
 
 #include "BLI_utildefines.h"
+#include "BLI_ghash.h"
 #include "BLI_path_util.h"
 #include "BLI_string.h"
 #include "BLI_listbase.h"
 #include "BLI_math.h"
 
+#include "BLF_translation.h"
+
 #include "DNA_mask_types.h"
 #include "DNA_node_types.h"
 #include "DNA_screen_types.h"
@@ -47,7 +50,6 @@
 #include "DNA_sequence_types.h"
 
 #include "BKE_curve.h"
-#include "BKE_global.h"
 #include "BKE_library.h"
 #include "BKE_main.h"
 #include "BKE_mask.h"
@@ -57,6 +59,11 @@
 #include "BKE_movieclip.h"
 #include "BKE_image.h"
 
+static struct {
+	ListBase splines;
+	struct GHash *id_hash;
+} mask_clipboard = {{NULL}};
+
 static MaskSplinePoint *mask_spline_point_next(MaskSpline *spline, MaskSplinePoint *points_array, MaskSplinePoint *point)
 {
 	if (point == &points_array[spline->tot_point - 1]) {
@@ -102,23 +109,6 @@ BezTriple *BKE_mask_spline_point_next_bezt(MaskSpline *spline, MaskSplinePoint *
 	}
 }
 
-#if 0
-static BezTriple *mask_spline_point_prev_bezt(MaskSpline *spline, MaskSplinePoint *points_array, MaskSplinePoint *point)
-{
-	if (point == points_array) {
-		if (spline->flag & MASK_SPLINE_CYCLIC) {
-			return &(points_array[0].bezt);
-		}
-		else {
-			return NULL;
-		}
-	}
-	else {
-		return &((point - 1))->bezt;
-	}
-}
-#endif
-
 MaskSplinePoint *BKE_mask_spline_point_array(MaskSpline *spline)
 {
 	return spline->points_deform ? spline->points_deform : spline->points;
@@ -157,6 +147,7 @@ MaskLayer *BKE_mask_layer_new(Mask *mask, const char *name)
 
 	masklay->blend = MASK_BLEND_MERGE_ADD;
 	masklay->alpha = 1.0f;
+	masklay->flag = MASK_LAYERFLAG_FILL_DISCRETE | MASK_LAYERFLAG_FILL_OVERLAP;
 
 	return masklay;
 }
@@ -185,7 +176,8 @@ void BKE_mask_layer_remove(Mask *mask, MaskLayer *masklay)
 
 void BKE_mask_layer_unique_name(Mask *mask, MaskLayer *masklay)
 {
-	BLI_uniquename(&mask->masklayers, masklay, "MaskLayer", '.', offsetof(MaskLayer, name), sizeof(masklay->name));
+	BLI_uniquename(&mask->masklayers, masklay, DATA_("MaskLayer"), '.', offsetof(MaskLayer, name),
+	               sizeof(masklay->name));
 }
 
 MaskLayer *BKE_mask_layer_copy(MaskLayer *masklay)
@@ -266,6 +258,17 @@ MaskSpline *BKE_mask_spline_add(MaskLayer *masklay)
 	return spline;
 }
 
+bool BKE_mask_spline_remove(MaskLayer *mask_layer, MaskSpline *spline)
+{
+	if (BLI_remlink_safe(&mask_layer->splines, spline) == false) {
+		return false;
+	}
+
+	BKE_mask_spline_free(spline);
+
+	return true;
+}
+
 void BKE_mask_point_direction_switch(MaskSplinePoint *point)
 {
 	const int tot_uw = point->tot_uw;
@@ -353,8 +356,8 @@ void BKE_mask_spline_direction_switch(MaskLayer *masklay, MaskSpline *spline)
 float BKE_mask_spline_project_co(MaskSpline *spline, MaskSplinePoint *point,
                                  float start_u, const float co[2], const eMaskSign sign)
 {
-	const float proj_eps         = 1e-3;
-	const float proj_eps_squared = proj_eps * proj_eps;
+	const float proj_eps    = 1e-3;
+	const float proj_eps_sq = proj_eps * proj_eps;
 	const int N = 1000;
 	float u = -1.0f, du = 1.0f / N, u1 = start_u, u2 = start_u;
 	float ang = -1.0f;
@@ -376,10 +379,10 @@ float BKE_mask_spline_project_co(MaskSpline *spline, MaskSplinePoint *point,
 			    ((sign == MASK_PROJ_POS) && (dot_v2v2(v1, n1) >= 0.0f)))
 			{
 
-				if (len_squared_v2(v1) > proj_eps_squared) {
+				if (len_squared_v2(v1) > proj_eps_sq) {
 					ang1 = angle_v2v2(v1, n1);
-					if (ang1 > M_PI / 2.0f)
-						ang1 = M_PI  - ang1;
+					if (ang1 > (float)M_PI / 2.0f)
+						ang1 = (float)M_PI - ang1;
 
 					if (ang < 0.0f || ang1 < ang) {
 						ang = ang1;
@@ -403,10 +406,10 @@ float BKE_mask_spline_project_co(MaskSpline *spline, MaskSplinePoint *point,
 			    ((sign == MASK_PROJ_POS) && (dot_v2v2(v2, n2) >= 0.0f)))
 			{
 
-				if (len_squared_v2(v2) > proj_eps_squared) {
+				if (len_squared_v2(v2) > proj_eps_sq) {
 					ang2 = angle_v2v2(v2, n2);
-					if (ang2 > M_PI / 2.0f)
-						ang2 = M_PI  - ang2;
+					if (ang2 > (float)M_PI / 2.0f)
+						ang2 = (float)M_PI - ang2;
 
 					if (ang2 < ang) {
 						ang = ang2;
@@ -429,58 +432,87 @@ float BKE_mask_spline_project_co(MaskSpline *spline, MaskSplinePoint *point,
 
 /* point */
 
-int BKE_mask_point_has_handle(MaskSplinePoint *point)
+eMaskhandleMode BKE_mask_point_handles_mode_get(MaskSplinePoint *point)
 {
 	BezTriple *bezt = &point->bezt;
 
-	return bezt->h1 == HD_ALIGN;
+	if (bezt->h1 == bezt->h2 && bezt->h1 == HD_ALIGN) {
+		return MASK_HANDLE_MODE_STICK;
+	}
+
+	return MASK_HANDLE_MODE_INDIVIDUAL_HANDLES;
 }
 
-void BKE_mask_point_handle(MaskSplinePoint *point, float handle[2])
+void BKE_mask_point_handle(MaskSplinePoint *point, eMaskWhichHandle which_handle, float handle[2])
 {
-	float vec[2];
+	BezTriple *bezt = &point->bezt;
 
-	sub_v2_v2v2(vec, point->bezt.vec[0], point->bezt.vec[1]);
+	if (which_handle == MASK_WHICH_HANDLE_STICK) {
+		float vec[2];
 
-	handle[0] = (point->bezt.vec[1][0] + vec[1]);
-	handle[1] = (point->bezt.vec[1][1] - vec[0]);
+		sub_v2_v2v2(vec, bezt->vec[0], bezt->vec[1]);
+
+		handle[0] = (bezt->vec[1][0] + vec[1]);
+		handle[1] = (bezt->vec[1][1] - vec[0]);
+	}
+	else if (which_handle == MASK_WHICH_HANDLE_LEFT) {
+		copy_v2_v2(handle, bezt->vec[0]);
+	}
+	else if (which_handle == MASK_WHICH_HANDLE_RIGHT) {
+		copy_v2_v2(handle, bezt->vec[2]);
+	}
+	else {
+		BLI_assert(!"Unknown handle passed to BKE_mask_point_handle");
+	}
 }
 
-void BKE_mask_point_set_handle(MaskSplinePoint *point, float loc[2], int keep_direction,
+void BKE_mask_point_set_handle(MaskSplinePoint *point, eMaskWhichHandle which_handle,
+                               float loc[2], bool keep_direction,
                                float orig_handle[2], float orig_vec[3][3])
 {
 	BezTriple *bezt = &point->bezt;
-	float v1[2], v2[2], vec[2];
 
-	if (keep_direction) {
-		sub_v2_v2v2(v1, loc, orig_vec[1]);
-		sub_v2_v2v2(v2, orig_handle, orig_vec[1]);
+	if (which_handle == MASK_WHICH_HANDLE_STICK) {
+		float v1[2], v2[2], vec[2];
+		if (keep_direction) {
+			sub_v2_v2v2(v1, loc, orig_vec[1]);
+			sub_v2_v2v2(v2, orig_handle, orig_vec[1]);
 
-		project_v2_v2v2(vec, v1, v2);
+			project_v2_v2v2(vec, v1, v2);
 
-		if (dot_v2v2(v2, vec) > 0) {
-			float len = len_v2(vec);
+			if (dot_v2v2(v2, vec) > 0) {
+				float len = len_v2(vec);
 
-			sub_v2_v2v2(v1, orig_vec[0], orig_vec[1]);
+				sub_v2_v2v2(v1, orig_vec[0], orig_vec[1]);
 
-			mul_v2_fl(v1, len / len_v2(v1));
+				mul_v2_fl(v1, len / len_v2(v1));
 
-			add_v2_v2v2(bezt->vec[0], bezt->vec[1], v1);
-			sub_v2_v2v2(bezt->vec[2], bezt->vec[1], v1);
+				add_v2_v2v2(bezt->vec[0], bezt->vec[1], v1);
+				sub_v2_v2v2(bezt->vec[2], bezt->vec[1], v1);
+			}
+			else {
+				copy_v3_v3(bezt->vec[0], bezt->vec[1]);
+				copy_v3_v3(bezt->vec[2], bezt->vec[1]);
+			}
 		}
 		else {
-			copy_v3_v3(bezt->vec[0], bezt->vec[1]);
-			copy_v3_v3(bezt->vec[2], bezt->vec[1]);
+			sub_v2_v2v2(v1, loc, bezt->vec[1]);
+
+			v2[0] = -v1[1];
+			v2[1] =  v1[0];
+
+			add_v2_v2v2(bezt->vec[0], bezt->vec[1], v2);
+			sub_v2_v2v2(bezt->vec[2], bezt->vec[1], v2);
 		}
 	}
+	else if (which_handle == MASK_WHICH_HANDLE_LEFT) {
+		copy_v2_v2(bezt->vec[0], loc);
+	}
+	else if (which_handle == MASK_WHICH_HANDLE_RIGHT) {
+		copy_v2_v2(bezt->vec[2], loc);
+	}
 	else {
-		sub_v2_v2v2(v1, loc, bezt->vec[1]);
-
-		v2[0] = -v1[1];
-		v2[1] =  v1[0];
-
-		add_v2_v2v2(bezt->vec[0], bezt->vec[1], v2);
-		sub_v2_v2v2(bezt->vec[2], bezt->vec[1], v2);
+		BLI_assert(!"unknown handle passed to BKE_mask_point_set_handle");
 	}
 }
 
@@ -508,36 +540,68 @@ void BKE_mask_point_segment_co(MaskSpline *spline, MaskSplinePoint *point, float
 	interp_v2_v2v2(co, r0, r1, u);
 }
 
+BLI_INLINE void orthogonal_direction_get(float vec[2], float result[2])
+{
+	result[0] = -vec[1];
+	result[1] = vec[0];
+	normalize_v2(result);
+}
+
+/* TODO(sergey): This function will re-calculate loads of stuff again and again
+ *               when differentiating feather points. This might be easily cached
+ *               in the callee function for this case.
+ */
 void BKE_mask_point_normal(MaskSpline *spline, MaskSplinePoint *point, float u, float n[2])
 {
-	MaskSplinePoint *points_array = BKE_mask_spline_point_array_from_point(spline, point);
 
-	BezTriple *bezt = &point->bezt, *bezt_next;
-	float q0[2], q1[2], q2[2], r0[2], r1[2], vec[2];
+	MaskSplinePoint *point_prev, *point_next;
 
-	bezt_next = BKE_mask_spline_point_next_bezt(spline, points_array, point);
+	/* TODO(sergey): This actually depends on a resolution. */
+	const float du = 0.05f;
 
-	if (!bezt_next) {
-		BKE_mask_point_handle(point, vec);
+	BKE_mask_get_handle_point_adjacent(spline, point, &point_prev, &point_next);
 
-		sub_v2_v2v2(n, vec, bezt->vec[1]);
-		normalize_v2(n);
-		return;
+	if (u - du < 0.0f && point_prev == NULL) {
+		float co[2], dir[2];
+		BKE_mask_point_segment_co(spline, point, u + du, co);
+		sub_v2_v2v2(dir, co, point->bezt.vec[1]);
+		orthogonal_direction_get(dir, n);
 	}
+	else if (u + du > 1.0f && point_next == NULL) {
+		float co[2], dir[2];
+		BKE_mask_point_segment_co(spline, point, u - du, co);
+		sub_v2_v2v2(dir, point->bezt.vec[1], co);
+		orthogonal_direction_get(dir, n);
+	}
+	else {
+		float prev_co[2], next_co[2], co[2];
+		float dir1[2], dir2[2], dir[2];
 
-	interp_v2_v2v2(q0, bezt->vec[1], bezt->vec[2], u);
-	interp_v2_v2v2(q1, bezt->vec[2], bezt_next->vec[0], u);
-	interp_v2_v2v2(q2, bezt_next->vec[0], bezt_next->vec[1], u);
+		if (u - du < 0.0f) {
+			BKE_mask_point_segment_co(spline, point_prev, 1.0f + (u - du), prev_co);
+		}
+		else {
+			BKE_mask_point_segment_co(spline, point, u - du, prev_co);
+		}
 
-	interp_v2_v2v2(r0, q0, q1, u);
-	interp_v2_v2v2(r1, q1, q2, u);
+		BKE_mask_point_segment_co(spline, point, u, co);
 
-	sub_v2_v2v2(vec, r1, r0);
+		if (u + du > 1.0f) {
+			BKE_mask_point_segment_co(spline, point_next, u + du - 1.0f, next_co);
+		}
+		else {
+			BKE_mask_point_segment_co(spline, point, u + du, next_co);
+		}
 
-	n[0] = -vec[1];
-	n[1] =  vec[0];
+		sub_v2_v2v2(dir1, co, prev_co);
+		sub_v2_v2v2(dir2, next_co, co);
 
-	normalize_v2(n);
+		normalize_v2(dir1);
+		normalize_v2(dir2);
+		add_v2_v2v2(dir, dir1, dir2);
+
+		orthogonal_direction_get(dir, n);
+	}
 }
 
 static float mask_point_interp_weight(BezTriple *bezt, BezTriple *bezt_next, const float u)
@@ -555,7 +619,7 @@ float BKE_mask_point_weight_scalar(MaskSpline *spline, MaskSplinePoint *point, c
 	if (!bezt_next) {
 		return bezt->weight;
 	}
-	else if (u <= 0.0) {
+	else if (u <= 0.0f) {
 		return bezt->weight;
 	}
 	else if (u >= 1.0f) {
@@ -576,7 +640,7 @@ float BKE_mask_point_weight(MaskSpline *spline, MaskSplinePoint *point, const fl
 	if (!bezt_next) {
 		return bezt->weight;
 	}
-	else if (u <= 0.0) {
+	else if (u <= 0.0f) {
 		return bezt->weight;
 	}
 	else if (u >= 1.0f) {
@@ -586,7 +650,7 @@ float BKE_mask_point_weight(MaskSpline *spline, MaskSplinePoint *point, const fl
 		float cur_u = 0.0f, cur_w = 0.0f, next_u = 0.0f, next_w = 0.0f, fac; /* Quite warnings */
 		int i;
 
-		for (i = 0; i < point->tot_uw + 1; i++) {
+		for (i = 0; i <= point->tot_uw; i++) {
 
 			if (i == 0) {
 				cur_u = 0.0f;
@@ -666,7 +730,7 @@ void BKE_mask_point_add_uw(MaskSplinePoint *point, float u, float w)
 	BKE_mask_point_sort_uw(point, &point->uw[point->tot_uw - 1]);
 }
 
-void BKE_mask_point_select_set(MaskSplinePoint *point, const short do_select)
+void BKE_mask_point_select_set(MaskSplinePoint *point, const bool do_select)
 {
 	int i;
 
@@ -687,29 +751,53 @@ void BKE_mask_point_select_set(MaskSplinePoint *point, const short do_select)
 	}
 }
 
-void BKE_mask_point_select_set_handle(MaskSplinePoint *point, const short do_select)
+void BKE_mask_point_select_set_handle(MaskSplinePoint *point, const eMaskWhichHandle which_handle, const bool do_select)
 {
 	if (do_select) {
-		MASKPOINT_SEL_HANDLE(point);
+		if (ELEM(which_handle, MASK_WHICH_HANDLE_STICK, MASK_WHICH_HANDLE_BOTH)) {
+			point->bezt.f1 |= SELECT;
+			point->bezt.f3 |= SELECT;
+		}
+		else if (which_handle == MASK_WHICH_HANDLE_LEFT) {
+			point->bezt.f1 |= SELECT;
+		}
+		else if (which_handle == MASK_WHICH_HANDLE_RIGHT) {
+			point->bezt.f3 |= SELECT;
+		}
+		else {
+			BLI_assert(!"Wrong which_handle passed to BKE_mask_point_select_set_handle");
+		}
 	}
 	else {
-		MASKPOINT_DESEL_HANDLE(point);
+		if (ELEM(which_handle, MASK_WHICH_HANDLE_STICK, MASK_WHICH_HANDLE_BOTH)) {
+			point->bezt.f1 &= ~SELECT;
+			point->bezt.f3 &= ~SELECT;
+		}
+		else if (which_handle == MASK_WHICH_HANDLE_LEFT) {
+			point->bezt.f1 &= ~SELECT;
+		}
+		else if (which_handle == MASK_WHICH_HANDLE_RIGHT) {
+			point->bezt.f3 &= ~SELECT;
+		}
+		else {
+			BLI_assert(!"Wrong which_handle passed to BKE_mask_point_select_set_handle");
+		}
 	}
 }
 
 /* only mask block itself */
-static Mask *mask_alloc(const char *name)
+static Mask *mask_alloc(Main *bmain, const char *name)
 {
 	Mask *mask;
 
-	mask = BKE_libblock_alloc(&G.main->mask, ID_MSK, name);
+	mask = BKE_libblock_alloc(bmain, ID_MSK, name);
 
 	mask->id.flag |= LIB_FAKEUSER;
 
 	return mask;
 }
 
-Mask *BKE_mask_new(const char *name)
+Mask *BKE_mask_new(Main *bmain, const char *name)
 {
 	Mask *mask;
 	char mask_name[MAX_ID_NAME - 2];
@@ -719,7 +807,7 @@ Mask *BKE_mask_new(const char *name)
 	else
 		strcpy(mask_name, "Mask");
 
-	mask = mask_alloc(mask_name);
+	mask = mask_alloc(bmain, mask_name);
 
 	/* arbitrary defaults */
 	mask->sfra = 1;
@@ -728,6 +816,7 @@ Mask *BKE_mask_new(const char *name)
 	return mask;
 }
 
+/* TODO(sergey): Use generic BKE_libblock_copy_nolib() instead. */
 Mask *BKE_mask_copy_nolib(Mask *mask)
 {
 	Mask *mask_new;
@@ -737,8 +826,7 @@ Mask *BKE_mask_copy_nolib(Mask *mask)
 	/*take care here! - we may want to copy anim data  */
 	mask_new->adt = NULL;
 
-	mask_new->masklayers.first = NULL;
-	mask_new->masklayers.last = NULL;
+	BLI_listbase_clear(&mask_new->masklayers);
 
 	BKE_mask_layer_copy_list(&mask_new->masklayers, &mask->masklayers);
 
@@ -757,8 +845,7 @@ Mask *BKE_mask_copy(Mask *mask)
 
 	mask_new = BKE_libblock_copy(&mask->id);
 
-	mask_new->masklayers.first = NULL;
-	mask_new->masklayers.last = NULL;
+	BLI_listbase_clear(&mask_new->masklayers);
 
 	BKE_mask_layer_copy_list(&mask_new->masklayers, &mask->masklayers);
 
@@ -801,6 +888,19 @@ void BKE_mask_spline_free(MaskSpline *spline)
 	MEM_freeN(spline);
 }
 
+void BKE_mask_spline_free_list(ListBase *splines)
+{
+	MaskSpline *spline = splines->first;
+	while (spline) {
+		MaskSpline *next_spline = spline->next;
+
+		BLI_remlink(splines, spline);
+		BKE_mask_spline_free(spline);
+
+		spline = next_spline;
+	}
+}
+
 static MaskSplinePoint *mask_spline_points_copy(MaskSplinePoint *points, int tot_point)
 {
 	MaskSplinePoint *npoints;
@@ -877,18 +977,8 @@ void BKE_mask_layer_free_shapes(MaskLayer *masklay)
 
 void BKE_mask_layer_free(MaskLayer *masklay)
 {
-	MaskSpline *spline;
-
 	/* free splines */
-	spline = masklay->splines.first;
-	while (spline) {
-		MaskSpline *next_spline = spline->next;
-
-		BLI_remlink(&masklay->splines, spline);
-		BKE_mask_spline_free(spline);
-
-		spline = next_spline;
-	}
+	BKE_mask_spline_free_list(&masklay->splines);
 
 	/* free animation data */
 	BKE_mask_layer_free_shapes(masklay);
@@ -964,10 +1054,9 @@ void BKE_mask_free(Main *bmain, Mask *mask)
 		}
 	}
 
-	{
-		bNodeTreeType *treetype = ntreeGetType(NTREE_COMPOSIT);
-		treetype->foreach_nodetree(bmain, (void *)mask, &BKE_node_tree_unlink_id_cb);
-	}
+	FOREACH_NODETREE(bmain, ntree, id) {
+		BKE_node_tree_unlink_id((ID *)mask, ntree);
+	} FOREACH_NODETREE_END
 
 	/* free mask data */
 	BKE_mask_layer_free_list(&mask->masklayers);
@@ -988,6 +1077,7 @@ void BKE_mask_coord_from_frame(float r_co[2], const float co[2], const float fra
 		r_co[1] = ((co[1] - 0.5f) * (frame_size[1] / frame_size[0])) + 0.5f;
 	}
 }
+
 void BKE_mask_coord_from_movieclip(MovieClip *clip, MovieClipUser *user, float r_co[2], const float co[2])
 {
 	float aspx, aspy;
@@ -1001,6 +1091,7 @@ void BKE_mask_coord_from_movieclip(MovieClip *clip, MovieClipUser *user, float r
 
 	BKE_mask_coord_from_frame(r_co, co, frame_size);
 }
+
 void BKE_mask_coord_from_image(Image *image, ImageUser *iuser, float r_co[2], const float co[2])
 {
 	float aspx, aspy;
@@ -1030,6 +1121,7 @@ void BKE_mask_coord_to_frame(float r_co[2], const float co[2], const float frame
 		r_co[1] = ((co[1] - 0.5f) / (frame_size[1] / frame_size[0])) + 0.5f;
 	}
 }
+
 void BKE_mask_coord_to_movieclip(MovieClip *clip, MovieClipUser *user, float r_co[2], const float co[2])
 {
 	float aspx, aspy;
@@ -1039,10 +1131,11 @@ void BKE_mask_coord_to_movieclip(MovieClip *clip, MovieClipUser *user, float r_c
 	BKE_movieclip_get_size_fl(clip, user, frame_size);
 	BKE_movieclip_get_aspect(clip, &aspx, &aspy);
 
-	frame_size[1] /= (aspy / aspx);
+	frame_size[1] *= (aspy / aspx);
 
 	BKE_mask_coord_to_frame(r_co, co, frame_size);
 }
+
 void BKE_mask_coord_to_image(Image *image, ImageUser *iuser, float r_co[2], const float co[2])
 {
 	float aspx, aspy;
@@ -1052,15 +1145,20 @@ void BKE_mask_coord_to_image(Image *image, ImageUser *iuser, float r_co[2], cons
 	BKE_image_get_size_fl(image, iuser, frame_size);
 	BKE_image_get_aspect(image, &aspx, &aspy);
 
-	frame_size[1] /= (aspy / aspx);
+	frame_size[1] *= (aspy / aspx);
 
 	BKE_mask_coord_to_frame(r_co, co, frame_size);
 }
 
-static int BKE_mask_evaluate_parent(MaskParent *parent, float ctime, float r_co[2])
+void BKE_mask_point_parent_matrix_get(MaskSplinePoint *point, float ctime, float parent_matrix[3][3])
 {
-	if (!parent)
-		return FALSE;
+	MaskParent *parent = &point->parent;
+
+	unit_m3(parent_matrix);
+
+	if (!parent) {
+		return;
+	}
 
 	if (parent->id_type == ID_MC) {
 		if (parent->id) {
@@ -1069,38 +1167,67 @@ static int BKE_mask_evaluate_parent(MaskParent *parent, float ctime, float r_co[
 			MovieTrackingObject *ob = BKE_tracking_object_get_named(tracking, parent->parent);
 
 			if (ob) {
-				MovieTrackingTrack *track = BKE_tracking_track_get_named(tracking, ob, parent->sub_parent);
-				float clip_framenr = BKE_movieclip_remap_scene_to_clip_frame(clip, ctime);
-
 				MovieClipUser user = {0};
-				user.framenr = ctime;
+				float clip_framenr = BKE_movieclip_remap_scene_to_clip_frame(clip, ctime);
+				BKE_movieclip_user_set_frame(&user, ctime);
 
-				if (track) {
-					float marker_pos_ofs[2];
-					BKE_tracking_marker_get_subframe_position(track, clip_framenr, marker_pos_ofs);
-					BKE_mask_coord_from_movieclip(clip, &user, r_co, marker_pos_ofs);
+				if (parent->type == MASK_PARENT_POINT_TRACK) {
+					MovieTrackingTrack *track = BKE_tracking_track_get_named(tracking, ob, parent->sub_parent);
 
-					return TRUE;
+					if (track) {
+						float marker_position[2], parent_co[2];
+						BKE_tracking_marker_get_subframe_position(track, clip_framenr, marker_position);
+						BKE_mask_coord_from_movieclip(clip, &user, parent_co, marker_position);
+						sub_v2_v2v2(parent_matrix[2], parent_co, parent->parent_orig);
+					}
+				}
+				else /* if (parent->type == MASK_PARENT_PLANE_TRACK) */ {
+					MovieTrackingPlaneTrack *plane_track = BKE_tracking_plane_track_get_named(tracking, ob, parent->sub_parent);
+
+					if (plane_track) {
+						float corners[4][2];
+						float aspx, aspy;
+						float frame_size[2], H[3][3], mask_from_clip_matrix[3][3], mask_to_clip_matrix[3][3];
+
+						BKE_tracking_plane_marker_get_subframe_corners(plane_track, ctime, corners);
+						BKE_tracking_homography_between_two_quads(parent->parent_corners_orig, corners, H);
+
+						unit_m3(mask_from_clip_matrix);
+
+						BKE_movieclip_get_size_fl(clip, &user, frame_size);
+						BKE_movieclip_get_aspect(clip, &aspx, &aspy);
+
+						frame_size[1] *= (aspy / aspx);
+						if (frame_size[0] == frame_size[1]) {
+							/* pass */
+						}
+						else if (frame_size[0] < frame_size[1]) {
+							mask_from_clip_matrix[0][0] = frame_size[1] / frame_size[0];
+							mask_from_clip_matrix[2][0] = -0.5f * (frame_size[1] / frame_size[0]) + 0.5f;
+						}
+						else { /* (frame_size[0] > frame_size[1]) */
+							mask_from_clip_matrix[1][1] = frame_size[1] / frame_size[0];
+							mask_from_clip_matrix[2][1] = -0.5f * (frame_size[1] / frame_size[0]) + 0.5f;
+						}
+
+						invert_m3_m3(mask_to_clip_matrix, mask_from_clip_matrix);
+						mul_m3_series(parent_matrix, mask_from_clip_matrix, H, mask_to_clip_matrix);
+					}
 				}
 			}
 		}
 	}
-
-	return FALSE;
 }
 
-/* could make external but for now its only used internally */
-static int mask_evaluate_parent_delta(MaskParent *parent, float ctime, float r_delta[2])
+static void mask_evaluate_apply_point_parent(MaskSplinePoint *point, float ctime)
 {
-	float parent_co[2];
+	float parent_matrix[3][3];
 
-	if (BKE_mask_evaluate_parent(parent, ctime, parent_co)) {
-		sub_v2_v2v2(r_delta, parent_co, parent->parent_orig);
-		return TRUE;
-	}
-	else {
-		return FALSE;
-	}
+	BKE_mask_point_parent_matrix_get(point, ctime, parent_matrix);
+
+	mul_m3_v2(parent_matrix, point->bezt.vec[0]);
+	mul_m3_v2(parent_matrix, point->bezt.vec[1]);
+	mul_m3_v2(parent_matrix, point->bezt.vec[2]);
 }
 
 static void mask_calc_point_handle(MaskSplinePoint *point, MaskSplinePoint *point_prev, MaskSplinePoint *point_next)
@@ -1126,7 +1253,7 @@ static void mask_calc_point_handle(MaskSplinePoint *point, MaskSplinePoint *poin
 	else if (handle_type == HD_AUTO) {
 		BKE_nurb_handle_calc(bezt, bezt_prev, bezt_next, 0);
 	}
-	else if (handle_type == HD_ALIGN) {
+	else if (handle_type == HD_ALIGN || handle_type == HD_ALIGN_DOUBLESIDE) {
 		float v1[3], v2[3];
 		float vec[3], h[3];
 
@@ -1134,7 +1261,7 @@ static void mask_calc_point_handle(MaskSplinePoint *point, MaskSplinePoint *poin
 		sub_v3_v3v3(v2, bezt->vec[2], bezt->vec[1]);
 		add_v3_v3v3(vec, v1, v2);
 
-		if (len_v3(vec) > 1e-3) {
+		if (len_squared_v3(vec) > (1e-3f * 1e-3f)) {
 			h[0] = vec[1];
 			h[1] = -vec[0];
 			h[2] = 0.0f;
@@ -1159,7 +1286,7 @@ void BKE_mask_get_handle_point_adjacent(MaskSpline *spline, MaskSplinePoint *poi
 	*r_point_next = mask_spline_point_next(spline, points_array, point);
 }
 
-/* calculates the tanget of a point by its previous and next
+/* calculates the tangent of a point by its previous and next
  * (ignoring handles - as if its a poly line) */
 void BKE_mask_calc_tangent_polyline(MaskSpline *spline, MaskSplinePoint *point, float t[2])
 {
@@ -1254,7 +1381,7 @@ void BKE_mask_calc_handle_adjacent_interp(MaskSpline *spline, MaskSplinePoint *p
  * Useful for giving sane defaults.
  */
 void BKE_mask_calc_handle_point_auto(MaskSpline *spline, MaskSplinePoint *point,
-                                     const short do_recalc_length)
+                                     const bool do_recalc_length)
 {
 	MaskSplinePoint *point_prev, *point_next;
 	const char h_back[2] = {point->bezt.h1, point->bezt.h2};
@@ -1273,7 +1400,7 @@ void BKE_mask_calc_handle_point_auto(MaskSpline *spline, MaskSplinePoint *point,
 	point->bezt.h2 = h_back[1];
 
 	/* preserve length by applying it back */
-	if (do_recalc_length == FALSE) {
+	if (do_recalc_length == false) {
 		dist_ensure_v2_v2fl(point->bezt.vec[0], point->bezt.vec[1], length_average);
 		dist_ensure_v2_v2fl(point->bezt.vec[2], point->bezt.vec[1], length_average);
 	}
@@ -1290,62 +1417,6 @@ void BKE_mask_layer_calc_handles(MaskLayer *masklay)
 	}
 }
 
-void BKE_mask_layer_calc_handles_deform(MaskLayer *masklay)
-{
-	MaskSpline *spline;
-	for (spline = masklay->splines.first; spline; spline = spline->next) {
-		int i;
-		for (i = 0; i < spline->tot_point; i++) {
-			BKE_mask_calc_handle_point(spline, &spline->points_deform[i]);
-		}
-	}
-}
-
-void BKE_mask_calc_handles(Mask *mask)
-{
-	MaskLayer *masklay;
-	for (masklay = mask->masklayers.first; masklay; masklay = masklay->next) {
-		BKE_mask_layer_calc_handles(masklay);
-	}
-}
-
-void BKE_mask_update_deform(Mask *mask)
-{
-	MaskLayer *masklay;
-
-	for (masklay = mask->masklayers.first; masklay; masklay = masklay->next) {
-		MaskSpline *spline;
-
-		for (spline = masklay->splines.first; spline; spline = spline->next) {
-			int i;
-
-			for (i = 0; i < spline->tot_point; i++) {
-				const int i_prev = (i - 1) % spline->tot_point;
-				const int i_next = (i + 1) % spline->tot_point;
-
-				BezTriple *bezt_prev = &spline->points[i_prev].bezt;
-				BezTriple *bezt      = &spline->points[i].bezt;
-				BezTriple *bezt_next = &spline->points[i_next].bezt;
-
-				BezTriple *bezt_def_prev = &spline->points_deform[i_prev].bezt;
-				BezTriple *bezt_def      = &spline->points_deform[i].bezt;
-				BezTriple *bezt_def_next = &spline->points_deform[i_next].bezt;
-
-				float w_src[4];
-				int j;
-
-				for (j = 0; j <= 2; j += 2) { /* (0, 2) */
-					printf("--- %d %d, %d, %d\n", i, j, i_prev, i_next);
-					barycentric_weights_v2(bezt_prev->vec[1], bezt->vec[1], bezt_next->vec[1],
-					                       bezt->vec[j], w_src);
-					interp_v3_v3v3v3(bezt_def->vec[j],
-					                 bezt_def_prev->vec[1], bezt_def->vec[1], bezt_def_next->vec[1], w_src);
-				}
-			}
-		}
-	}
-}
-
 void BKE_mask_spline_ensure_deform(MaskSpline *spline)
 {
 	int allocated_points = (MEM_allocN_len(spline->points_deform) / sizeof(*spline->points_deform));
@@ -1372,7 +1443,7 @@ void BKE_mask_spline_ensure_deform(MaskSpline *spline)
 	}
 }
 
-void BKE_mask_layer_evaluate(MaskLayer *masklay, const float ctime, const int do_newframe)
+void BKE_mask_layer_evaluate(MaskLayer *masklay, const float ctime, const bool do_newframe)
 {
 	/* animation if available */
 	if (do_newframe) {
@@ -1385,7 +1456,7 @@ void BKE_mask_layer_evaluate(MaskLayer *masklay, const float ctime, const int do
 		{
 			if (found == 1) {
 #if 0
-				printf("%s: exact %d %d (%d)\n", __func__, (int)ctime, BLI_countlist(&masklay->splines_shapes),
+				printf("%s: exact %d %d (%d)\n", __func__, (int)ctime, BLI_listbase_count(&masklay->splines_shapes),
 				       masklay_shape_a->frame);
 #endif
 
@@ -1394,7 +1465,7 @@ void BKE_mask_layer_evaluate(MaskLayer *masklay, const float ctime, const int do
 			else if (found == 2) {
 				float w = masklay_shape_b->frame - masklay_shape_a->frame;
 #if 0
-				printf("%s: tween %d %d (%d %d)\n", __func__, (int)ctime, BLI_countlist(&masklay->splines_shapes),
+				printf("%s: tween %d %d (%d %d)\n", __func__, (int)ctime, BLI_listbase_count(&masklay->splines_shapes),
 				       masklay_shape_a->frame, masklay_shape_b->frame);
 #endif
 				BKE_mask_layer_shape_to_mask_interp(masklay, masklay_shape_a, masklay_shape_b,
@@ -1416,36 +1487,31 @@ void BKE_mask_layer_evaluate(MaskLayer *masklay, const float ctime, const int do
 
 		for (spline = masklay->splines.first; spline; spline = spline->next) {
 			int i;
-			int has_auto = FALSE;
+			bool need_handle_recalc = false;
 
 			BKE_mask_spline_ensure_deform(spline);
 
 			for (i = 0; i < spline->tot_point; i++) {
 				MaskSplinePoint *point = &spline->points[i];
 				MaskSplinePoint *point_deform = &spline->points_deform[i];
-				float delta[2];
 
 				BKE_mask_point_free(point_deform);
 
 				*point_deform = *point;
 				point_deform->uw = point->uw ? MEM_dupallocN(point->uw) : NULL;
 
-				if (mask_evaluate_parent_delta(&point->parent, ctime, delta)) {
-					add_v2_v2(point_deform->bezt.vec[0], delta);
-					add_v2_v2(point_deform->bezt.vec[1], delta);
-					add_v2_v2(point_deform->bezt.vec[2], delta);
-				}
+				mask_evaluate_apply_point_parent(point_deform, ctime);
 
-				if (point->bezt.h1 == HD_AUTO) {
-					has_auto = TRUE;
+				if (ELEM(point->bezt.h1, HD_AUTO, HD_VECT)) {
+					need_handle_recalc = true;
 				}
 			}
 
-			/* if the spline has auto handles, these need to be recalculated after deformation */
-			if (has_auto) {
+			/* if the spline has auto or vector handles, these need to be recalculated after deformation */
+			if (need_handle_recalc) {
 				for (i = 0; i < spline->tot_point; i++) {
 					MaskSplinePoint *point_deform = &spline->points_deform[i];
-					if (point_deform->bezt.h1 == HD_AUTO) {
+					if (ELEM(point_deform->bezt.h1, HD_AUTO, HD_VECT)) {
 						BKE_mask_calc_handle_point(spline, point_deform);
 					}
 				}
@@ -1455,7 +1521,7 @@ void BKE_mask_layer_evaluate(MaskLayer *masklay, const float ctime, const int do
 	}
 }
 
-void BKE_mask_evaluate(Mask *mask, const float ctime, const int do_newframe)
+void BKE_mask_evaluate(Mask *mask, const float ctime, const bool do_newframe)
 {
 	MaskLayer *masklay;
 
@@ -1496,10 +1562,10 @@ void BKE_mask_update_display(Mask *mask, float ctime)
 	}
 #endif
 
-	BKE_mask_evaluate(mask, ctime, FALSE);
+	BKE_mask_evaluate(mask, ctime, false);
 }
 
-void BKE_mask_evaluate_all_masks(Main *bmain, float ctime, const int do_newframe)
+void BKE_mask_evaluate_all_masks(Main *bmain, float ctime, const bool do_newframe)
 {
 	Mask *mask;
 
@@ -1508,13 +1574,14 @@ void BKE_mask_evaluate_all_masks(Main *bmain, float ctime, const int do_newframe
 	}
 }
 
-void BKE_mask_update_scene(Main *bmain, Scene *scene, const int do_newframe)
+void BKE_mask_update_scene(Main *bmain, Scene *scene)
 {
 	Mask *mask;
 
 	for (mask = bmain->mask.first; mask; mask = mask->id.next) {
-		if (mask->id.flag & LIB_ID_RECALC) {
-			BKE_mask_evaluate_all_masks(bmain, CFRA, do_newframe);
+		if (mask->id.flag & (LIB_ID_RECALC | LIB_ID_RECALC_DATA)) {
+			bool do_new_frame = (mask->id.flag & LIB_ID_RECALC_DATA) != 0;
+			BKE_mask_evaluate_all_masks(bmain, CFRA, do_new_frame);
 		}
 	}
 }
@@ -1618,8 +1685,8 @@ void BKE_mask_layer_shape_to_mask_interp(MaskLayer *masklay,
 {
 	int tot = BKE_mask_layer_shape_totvert(masklay);
 	if (masklay_shape_a->tot_vert == tot && masklay_shape_b->tot_vert == tot) {
-		float *fp_a = masklay_shape_a->data;
-		float *fp_b = masklay_shape_b->data;
+		const float *fp_a = masklay_shape_a->data;
+		const float *fp_b = masklay_shape_b->data;
 		const float ifac = 1.0f - fac;
 
 		MaskSpline *spline;
@@ -1705,7 +1772,7 @@ int BKE_mask_layer_shape_find_frame_range(MaskLayer *masklay, const float frame,
 	}
 }
 
-MaskLayerShape *BKE_mask_layer_shape_varify_frame(MaskLayer *masklay, const int frame)
+MaskLayerShape *BKE_mask_layer_shape_verify_frame(MaskLayer *masklay, const int frame)
 {
 	MaskLayerShape *masklay_shape;
 
@@ -1753,10 +1820,10 @@ void BKE_mask_layer_shape_unlink(MaskLayer *masklay, MaskLayerShape *masklay_sha
 	BKE_mask_layer_shape_free(masklay_shape);
 }
 
-static int mask_layer_shape_sort_cb(void *masklay_shape_a_ptr, void *masklay_shape_b_ptr)
+static int mask_layer_shape_sort_cb(const void *masklay_shape_a_ptr, const void *masklay_shape_b_ptr)
 {
-	MaskLayerShape *masklay_shape_a = (MaskLayerShape *)masklay_shape_a_ptr;
-	MaskLayerShape *masklay_shape_b = (MaskLayerShape *)masklay_shape_b_ptr;
+	const MaskLayerShape *masklay_shape_a = masklay_shape_a_ptr;
+	const MaskLayerShape *masklay_shape_b = masklay_shape_b_ptr;
 
 	if      (masklay_shape_a->frame < masklay_shape_b->frame)  return -1;
 	else if (masklay_shape_a->frame > masklay_shape_b->frame)  return  1;
@@ -1765,11 +1832,11 @@ static int mask_layer_shape_sort_cb(void *masklay_shape_a_ptr, void *masklay_sha
 
 void BKE_mask_layer_shape_sort(MaskLayer *masklay)
 {
-	BLI_sortlist(&masklay->splines_shapes, mask_layer_shape_sort_cb);
+	BLI_listbase_sort(&masklay->splines_shapes, mask_layer_shape_sort_cb);
 }
 
-int BKE_mask_layer_shape_spline_from_index(MaskLayer *masklay, int index,
-                                           MaskSpline **r_masklay_shape, int *r_index)
+bool BKE_mask_layer_shape_spline_from_index(MaskLayer *masklay, int index,
+                                            MaskSpline **r_masklay_shape, int *r_index)
 {
 	MaskSpline *spline;
 
@@ -1777,12 +1844,12 @@ int BKE_mask_layer_shape_spline_from_index(MaskLayer *masklay, int index,
 		if (index < spline->tot_point) {
 			*r_masklay_shape = spline;
 			*r_index = index;
-			return TRUE;
+			return true;
 		}
 		index -= spline->tot_point;
 	}
 
-	return FALSE;
+	return false;
 }
 
 int BKE_mask_layer_shape_spline_to_index(MaskLayer *masklay, MaskSpline *spline)
@@ -1805,7 +1872,7 @@ static void interp_weights_uv_v2_calc(float r_uv[2], const float pt[2], const fl
 	float pt_on_line[2];
 	r_uv[0] = closest_to_line_v2(pt_on_line, pt, pt_a, pt_b);
 	r_uv[1] = (len_v2v2(pt_on_line, pt) / len_v2v2(pt_a, pt_b)) *
-	          ((line_point_side_v2(pt_a, pt_b, pt) < 0.0f) ? -1.0 : 1.0);  /* this line only sets the sign */
+	          ((line_point_side_v2(pt_a, pt_b, pt) < 0.0f) ? -1.0f : 1.0f);  /* this line only sets the sign */
 }
 
 
@@ -1822,9 +1889,9 @@ static void interp_weights_uv_v2_apply(const float uv[2], float r_pt[2], const f
 	r_pt[1] +=  dvec[0] * uv[1];
 }
 
-/* when a now points added - resize all shapekey array  */
+/* when a new points added - resize all shapekey array  */
 void BKE_mask_layer_shape_changed_add(MaskLayer *masklay, int index,
-                                      int do_init, int do_init_interpolate)
+                                      bool do_init, bool do_init_interpolate)
 {
 	MaskLayerShape *masklay_shape;
 
@@ -1955,5 +2022,97 @@ void BKE_mask_layer_shape_changed_remove(MaskLayer *masklay, int index, int coun
 
 int BKE_mask_get_duration(Mask *mask)
 {
-	return maxi(1, mask->efra - mask->sfra);
+	return max_ii(1, mask->efra - mask->sfra);
+}
+
+/*********************** clipboard *************************/
+
+static void mask_clipboard_free_ex(bool final_free)
+{
+	BKE_mask_spline_free_list(&mask_clipboard.splines);
+	BLI_listbase_clear(&mask_clipboard.splines);
+	if (mask_clipboard.id_hash) {
+		if (final_free) {
+			BLI_ghash_free(mask_clipboard.id_hash, NULL, MEM_freeN);
+		}
+		else {
+			BLI_ghash_clear(mask_clipboard.id_hash, NULL, MEM_freeN);
+		}
+	}
+}
+
+/* Free the clipboard. */
+void BKE_mask_clipboard_free(void)
+{
+	mask_clipboard_free_ex(true);
+}
+
+/* Copy selected visible splines from the given layer to clipboard. */
+void BKE_mask_clipboard_copy_from_layer(MaskLayer *mask_layer)
+{
+	MaskSpline *spline;
+
+	/* Nothing to do if selection if disabled for the given layer. */
+	if (mask_layer->restrictflag & MASK_RESTRICT_SELECT) {
+		return;
+	}
+
+	mask_clipboard_free_ex(false);
+	if (mask_clipboard.id_hash == NULL) {
+		mask_clipboard.id_hash = BLI_ghash_ptr_new("mask clipboard ID hash");
+	}
+
+	for (spline = mask_layer->splines.first; spline; spline = spline->next) {
+		if (spline->flag & SELECT) {
+			MaskSpline *spline_new = BKE_mask_spline_copy(spline);
+			int i;
+			for (i = 0; i < spline_new->tot_point; i++) {
+				MaskSplinePoint *point = &spline_new->points[i];
+				if (point->parent.id) {
+					if (!BLI_ghash_lookup(mask_clipboard.id_hash, point->parent.id)) {
+						int len = strlen(point->parent.id->name);
+						char *name_copy = MEM_mallocN(len + 1, "mask clipboard ID name");
+						strcpy(name_copy, point->parent.id->name);
+						BLI_ghash_insert(mask_clipboard.id_hash,
+						                 point->parent.id,
+						                 name_copy);
+					}
+				}
+			}
+
+			BLI_addtail(&mask_clipboard.splines, spline_new);
+		}
+	}
+}
+
+/* Check clipboard is empty. */
+bool BKE_mask_clipboard_is_empty(void)
+{
+	return BLI_listbase_is_empty(&mask_clipboard.splines);
+}
+
+/* Paste the contents of clipboard to given mask layer */
+void BKE_mask_clipboard_paste_to_layer(Main *bmain, MaskLayer *mask_layer)
+{
+	MaskSpline *spline;
+
+	for (spline = mask_clipboard.splines.first; spline; spline = spline->next) {
+		MaskSpline *spline_new = BKE_mask_spline_copy(spline);
+		int i;
+
+		for (i = 0; i < spline_new->tot_point; i++) {
+			MaskSplinePoint *point = &spline_new->points[i];
+			if (point->parent.id) {
+				const char *id_name = BLI_ghash_lookup(mask_clipboard.id_hash, point->parent.id);
+				ListBase *listbase;
+
+				BLI_assert(id_name != NULL);
+
+				listbase = which_libbase(bmain, GS(id_name));
+				point->parent.id = BLI_findstring(listbase, id_name + 2, offsetof(ID, name) + 2);
+			}
+		}
+
+		BLI_addtail(&mask_layer->splines, spline_new);
+	}
 }
