@@ -38,6 +38,7 @@
 
 #include "DNA_anim_types.h"
 #include "DNA_curve_types.h"
+#include "DNA_group_types.h"
 #include "DNA_material_types.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
@@ -51,7 +52,6 @@
 #include "BLI_math.h"		
 #include "BLI_listbase.h"		
 #include "BLI_utildefines.h"
-#include "BLI_bpath.h"
 #include "BLI_string.h"
 
 #include "BKE_animsys.h"
@@ -63,6 +63,7 @@
 #include "BKE_main.h"
 #include "BKE_material.h"
 #include "BKE_mesh.h"
+#include "BKE_scene.h"
 #include "BKE_node.h"
 #include "BKE_curve.h"
 
@@ -80,11 +81,11 @@ void init_def_material(void)
 /* not material itself */
 void BKE_material_free(Material *ma)
 {
-	BKE_material_free_ex(ma, TRUE);
+	BKE_material_free_ex(ma, true);
 }
 
 /* not material itself */
-void BKE_material_free_ex(Material *ma, int do_id_user)
+void BKE_material_free_ex(Material *ma, bool do_id_user)
 {
 	MTex *mtex;
 	int a;
@@ -111,8 +112,11 @@ void BKE_material_free_ex(Material *ma, int do_id_user)
 		MEM_freeN(ma->nodetree);
 	}
 
+	if (ma->texpaintslot)
+		MEM_freeN(ma->texpaintslot);
+
 	if (ma->gpumaterial.first)
-		GPU_material_free(ma);
+		GPU_material_free(&ma->gpumaterial);
 }
 
 void init_material(Material *ma)
@@ -153,6 +157,7 @@ void init_material(Material *ma)
 	ma->tx_limit = 0.0;
 	ma->tx_falloff = 1.0;
 	ma->shad_alpha = 1.0f;
+	ma->vcol_alpha = 0;
 	
 	ma->gloss_mir = ma->gloss_tra = 1.0;
 	ma->samp_gloss_mir = ma->samp_gloss_tra = 18;
@@ -202,19 +207,20 @@ void init_material(Material *ma)
 	ma->game.face_orientation = 0;
 	
 	ma->mode = MA_TRACEBLE | MA_SHADBUF | MA_SHADOW | MA_RAYBIAS | MA_TANGENT_STR | MA_ZTRANSP;
+	ma->mode2 = MA_CASTSHADOW;
 	ma->shade_flag = MA_APPROX_OCCLUSION;
 	ma->preview = NULL;
 }
 
-Material *BKE_material_add(const char *name)
+Material *BKE_material_add(Main *bmain, const char *name)
 {
 	Material *ma;
 
-	ma = BKE_libblock_alloc(&G.main->mat, ID_MA, name);
+	ma = BKE_libblock_alloc(bmain, ID_MA, name);
 	
 	init_material(ma);
 	
-	return ma;	
+	return ma;
 }
 
 /* XXX keep synced with next function */
@@ -244,7 +250,7 @@ Material *BKE_material_copy(Material *ma)
 		man->nodetree = ntreeCopyTree(ma->nodetree);
 	}
 
-	man->gpumaterial.first = man->gpumaterial.last = NULL;
+	BLI_listbase_clear(&man->gpumaterial);
 	
 	return man;
 }
@@ -255,8 +261,7 @@ Material *localize_material(Material *ma)
 	Material *man;
 	int a;
 	
-	man = BKE_libblock_copy(&ma->id);
-	BLI_remlink(&G.main->mat, man);
+	man = BKE_libblock_copy_nolib(&ma->id, false);
 
 	/* no increment for texture ID users, in previewrender.c it prevents decrement */
 	for (a = 0; a < MAX_MTEX; a++) {
@@ -268,13 +273,14 @@ Material *localize_material(Material *ma)
 	
 	if (ma->ramp_col) man->ramp_col = MEM_dupallocN(ma->ramp_col);
 	if (ma->ramp_spec) man->ramp_spec = MEM_dupallocN(ma->ramp_spec);
-	
+
+	man->texpaintslot = NULL;
 	man->preview = NULL;
 	
 	if (ma->nodetree)
 		man->nodetree = ntreeLocalize(ma->nodetree);
 	
-	man->gpumaterial.first = man->gpumaterial.last = NULL;
+	BLI_listbase_clear(&man->gpumaterial);
 	
 	return man;
 }
@@ -294,7 +300,8 @@ void BKE_material_make_local(Material *ma)
 	Mesh *me;
 	Curve *cu;
 	MetaBall *mb;
-	int a, is_local = FALSE, is_lib = FALSE;
+	int a;
+	bool is_local = false, is_lib = false;
 
 	/* - only lib users: do nothing
 	 * - only local users: set flag
@@ -318,8 +325,8 @@ void BKE_material_make_local(Material *ma)
 		if (ob->mat) {
 			for (a = 0; a < ob->totcol; a++) {
 				if (ob->mat[a] == ma) {
-					if (ob->id.lib) is_lib = TRUE;
-					else is_local = TRUE;
+					if (ob->id.lib) is_lib = true;
+					else is_local = true;
 				}
 			}
 		}
@@ -331,8 +338,8 @@ void BKE_material_make_local(Material *ma)
 		if (me->mat) {
 			for (a = 0; a < me->totcol; a++) {
 				if (me->mat[a] == ma) {
-					if (me->id.lib) is_lib = TRUE;
-					else is_local = TRUE;
+					if (me->id.lib) is_lib = true;
+					else is_local = true;
 				}
 			}
 		}
@@ -344,8 +351,8 @@ void BKE_material_make_local(Material *ma)
 		if (cu->mat) {
 			for (a = 0; a < cu->totcol; a++) {
 				if (cu->mat[a] == ma) {
-					if (cu->id.lib) is_lib = TRUE;
-					else is_local = TRUE;
+					if (cu->id.lib) is_lib = true;
+					else is_local = true;
 				}
 			}
 		}
@@ -357,8 +364,8 @@ void BKE_material_make_local(Material *ma)
 		if (mb->mat) {
 			for (a = 0; a < mb->totcol; a++) {
 				if (mb->mat[a] == ma) {
-					if (mb->id.lib) is_lib = TRUE;
-					else is_local = TRUE;
+					if (mb->id.lib) is_lib = true;
+					else is_local = true;
 				}
 			}
 		}
@@ -366,7 +373,7 @@ void BKE_material_make_local(Material *ma)
 	}
 
 	/* Only local users. */
-	if (is_local && is_lib == FALSE) {
+	if (is_local && is_lib == false) {
 		id_clear_lib_data(bmain, &ma->id);
 		extern_local_material(ma);
 	}
@@ -465,7 +472,7 @@ Material ***give_matarar(Object *ob)
 		me = ob->data;
 		return &(me->mat);
 	}
-	else if (ELEM3(ob->type, OB_CURVE, OB_FONT, OB_SURF)) {
+	else if (ELEM(ob->type, OB_CURVE, OB_FONT, OB_SURF)) {
 		cu = ob->data;
 		return &(cu->mat);
 	}
@@ -486,7 +493,7 @@ short *give_totcolp(Object *ob)
 		me = ob->data;
 		return &(me->totcol);
 	}
-	else if (ELEM3(ob->type, OB_CURVE, OB_FONT, OB_SURF)) {
+	else if (ELEM(ob->type, OB_CURVE, OB_FONT, OB_SURF)) {
 		cu = ob->data;
 		return &(cu->totcol);
 	}
@@ -506,13 +513,10 @@ Material ***give_matarar_id(ID *id)
 	switch (GS(id->name)) {
 		case ID_ME:
 			return &(((Mesh *)id)->mat);
-			break;
 		case ID_CU:
 			return &(((Curve *)id)->mat);
-			break;
 		case ID_MB:
 			return &(((MetaBall *)id)->mat);
-			break;
 	}
 	return NULL;
 }
@@ -525,28 +529,25 @@ short *give_totcolp_id(ID *id)
 	switch (GS(id->name)) {
 		case ID_ME:
 			return &(((Mesh *)id)->totcol);
-			break;
 		case ID_CU:
 			return &(((Curve *)id)->totcol);
-			break;
 		case ID_MB:
 			return &(((MetaBall *)id)->totcol);
-			break;
 	}
 	return NULL;
 }
 
-static void data_delete_material_index_id(ID *id, short index)
+static void material_data_index_remove_id(ID *id, short index)
 {
 	/* ensure we don't try get materials from non-obdata */
 	BLI_assert(OB_DATA_SUPPORT_ID(GS(id->name)));
 
 	switch (GS(id->name)) {
 		case ID_ME:
-			BKE_mesh_delete_material_index((Mesh *)id, index);
+			BKE_mesh_material_index_remove((Mesh *)id, index);
 			break;
 		case ID_CU:
-			BKE_curve_delete_material_index((Curve *)id, index);
+			BKE_curve_material_index_remove((Curve *)id, index);
 			break;
 		case ID_MB:
 			/* meta-elems don't have materials atm */
@@ -554,7 +555,53 @@ static void data_delete_material_index_id(ID *id, short index)
 	}
 }
 
-void material_append_id(ID *id, Material *ma)
+static void material_data_index_clear_id(ID *id)
+{
+	/* ensure we don't try get materials from non-obdata */
+	BLI_assert(OB_DATA_SUPPORT_ID(GS(id->name)));
+
+	switch (GS(id->name)) {
+		case ID_ME:
+			BKE_mesh_material_index_clear((Mesh *)id);
+			break;
+		case ID_CU:
+			BKE_curve_material_index_clear((Curve *)id);
+			break;
+		case ID_MB:
+			/* meta-elems don't have materials atm */
+			break;
+	}
+}
+
+void BKE_material_resize_id(struct ID *id, short totcol, bool do_id_user)
+{
+	Material ***matar = give_matarar_id(id);
+	short *totcolp = give_totcolp_id(id);
+
+	if (matar == NULL) {
+		return;
+	}
+
+	if (do_id_user && totcol < (*totcolp)) {
+		short i;
+		for (i = totcol; i < (*totcolp); i++) {
+			id_us_min((ID *)(*matar)[i]);
+		}
+	}
+
+	if (totcol == 0) {
+		if (*totcolp) {
+			MEM_freeN(*matar);
+			*matar = NULL;
+		}
+	}
+	else {
+		*matar = MEM_recallocN(*matar, sizeof(void *) * totcol);
+	}
+	*totcolp = totcol;
+}
+
+void BKE_material_append_id(ID *id, Material *ma)
 {
 	Material ***matar;
 	if ((matar = give_matarar_id(id))) {
@@ -567,11 +614,11 @@ void material_append_id(ID *id, Material *ma)
 		(*matar)[(*totcol)++] = ma;
 
 		id_us_plus((ID *)ma);
-		test_object_materials(id);
+		test_object_materials(G.main, id);
 	}
 }
 
-Material *material_pop_id(ID *id, int index_i, int remove_material_slot)
+Material *BKE_material_pop_id(ID *id, int index_i, bool update_data)
 {
 	short index = (short)index_i;
 	Material *ret = NULL;
@@ -582,57 +629,68 @@ Material *material_pop_id(ID *id, int index_i, int remove_material_slot)
 			ret = (*matar)[index];
 			id_us_min((ID *)ret);
 
-			if (remove_material_slot) {
-				if (*totcol <= 1) {
-					*totcol = 0;
-					MEM_freeN(*matar);
-					*matar = NULL;
-				}
-				else {
-					Material **mat;
-					if (index + 1 != (*totcol))
-						memmove((*matar) + index, (*matar) + (index + 1), sizeof(void *) * ((*totcol) - (index + 1)));
+			if (*totcol <= 1) {
+				*totcol = 0;
+				MEM_freeN(*matar);
+				*matar = NULL;
+			}
+			else {
+				if (index + 1 != (*totcol))
+					memmove((*matar) + index, (*matar) + (index + 1), sizeof(void *) * ((*totcol) - (index + 1)));
 
-					(*totcol)--;
-					
-					mat = MEM_callocN(sizeof(void *) * (*totcol), "newmatar");
-					memcpy(mat, *matar, sizeof(void *) * (*totcol));
-					MEM_freeN(*matar);
-
-					*matar = mat;
-					test_object_materials(id);
-				}
-
-				/* decrease mat_nr index */
-				data_delete_material_index_id(id, index);
+				(*totcol)--;
+				*matar = MEM_reallocN(*matar, sizeof(void *) * (*totcol));
+				test_object_materials(G.main, id);
 			}
 
-			/* don't remove material slot, only clear it*/
-			else
-				(*matar)[index] = NULL;
+			if (update_data) {
+				/* decrease mat_nr index */
+				material_data_index_remove_id(id, index);
+			}
 		}
 	}
 	
 	return ret;
 }
 
+void BKE_material_clear_id(struct ID *id, bool update_data)
+{
+	Material ***matar;
+	if ((matar = give_matarar_id(id))) {
+		short *totcol = give_totcolp_id(id);
+		*totcol = 0;
+		if (*matar) {
+			MEM_freeN(*matar);
+			*matar = NULL;
+		}
+
+		if (update_data) {
+			/* decrease mat_nr index */
+			material_data_index_clear_id(id);
+		}
+	}
+}
+
 Material *give_current_material(Object *ob, short act)
 {
 	Material ***matarar, *ma;
-	short *totcolp;
-	
+	const short *totcolp;
+
 	if (ob == NULL) return NULL;
 	
 	/* if object cannot have material, (totcolp == NULL) */
 	totcolp = give_totcolp(ob);
 	if (totcolp == NULL || ob->totcol == 0) return NULL;
-	
-	if (act < 0) {
-		printf("no!\n");
+
+	/* return NULL for invalid 'act', can happen for mesh face indices */
+	if (act > ob->totcol)
+		return NULL;
+	else if (act <= 0) {
+		if (act < 0) {
+			printf("Negative material index!\n");
+		}
+		return NULL;
 	}
-	
-	if (act > ob->totcol) act = ob->totcol;
-	else if (act <= 0) act = 1;
 
 	if (ob->matbits && ob->matbits[act - 1]) {    /* in object */
 		ma = ob->mat[act - 1];
@@ -678,24 +736,18 @@ Material *give_node_material(Material *ma)
 	return NULL;
 }
 
-/* GS reads the memory pointed at in a specific ordering. There are,
- * however two definitions for it. I have jotted them down here, both,
- * but I think the first one is actually used. The thing is that
- * big-endian systems might read this the wrong way round. OTOH, we
- * constructed the IDs that are read out with this macro explicitly as
- * well. I expect we'll sort it out soon... */
-
-/* from blendef: */
-#define GS(a)   (*((short *)(a)))
-
-/* from misc_util: flip the bytes from x  */
-/*  #define GS(x) (((unsigned char *)(x))[0] << 8 | ((unsigned char *)(x))[1]) */
-
-void resize_object_material(Object *ob, const short totcol)
+void BKE_material_resize_object(Object *ob, const short totcol, bool do_id_user)
 {
 	Material **newmatar;
 	char *newmatbits;
 
+	if (do_id_user && totcol < ob->totcol) {
+		short i;
+		for (i = totcol; i < ob->totcol; i++) {
+			id_us_min((ID *)ob->mat[i]);
+		}
+	}
+
 	if (totcol == 0) {
 		if (ob->totcol) {
 			MEM_freeN(ob->mat);
@@ -716,26 +768,30 @@ void resize_object_material(Object *ob, const short totcol)
 		ob->mat = newmatar;
 		ob->matbits = newmatbits;
 	}
+	/* XXX, why not realloc on shrink? - campbell */
+
 	ob->totcol = totcol;
 	if (ob->totcol && ob->actcol == 0) ob->actcol = 1;
 	if (ob->actcol > ob->totcol) ob->actcol = ob->totcol;
 }
 
-void test_object_materials(ID *id)
+void test_object_materials(Main *bmain, ID *id)
 {
 	/* make the ob mat-array same size as 'ob->data' mat-array */
 	Object *ob;
-	short *totcol;
+	const short *totcol;
 
 	if (id == NULL || (totcol = give_totcolp_id(id)) == NULL) {
 		return;
 	}
 
-	for (ob = G.main->object.first; ob; ob = ob->id.next) {
+	BKE_main_lock(bmain);
+	for (ob = bmain->object.first; ob; ob = ob->id.next) {
 		if (ob->data == id) {
-			resize_object_material(ob, *totcol);
+			BKE_material_resize_object(ob, *totcol, false);
 		}
 	}
+	BKE_main_unlock(bmain);
 }
 
 void assign_material_id(ID *id, Material *ma, short act)
@@ -777,13 +833,12 @@ void assign_material_id(ID *id, Material *ma, short act)
 	if (ma)
 		id_us_plus((ID *)ma);
 
-	test_object_materials(id);
+	test_object_materials(G.main, id);
 }
 
 void assign_material(Object *ob, Material *ma, short act, int assign_type)
 {
 	Material *mao, **matar, ***matarar;
-	char *matbits;
 	short *totcolp;
 	char bit = 0;
 
@@ -835,16 +890,8 @@ void assign_material(Object *ob, Material *ma, short act, int assign_type)
 
 	if (act > ob->totcol) {
 		/* Need more space in the material arrays */
-		matar = MEM_callocN(sizeof(void *) * act, "matarray2");
-		matbits = MEM_callocN(sizeof(char) * act, "matbits1");
-		if (ob->totcol) {
-			memcpy(matar, ob->mat, sizeof(void *) * ob->totcol);
-			memcpy(matbits, ob->matbits, sizeof(char) * (*totcolp));
-			MEM_freeN(ob->mat);
-			MEM_freeN(ob->matbits);
-		}
-		ob->mat = matar;
-		ob->matbits = matbits;
+		ob->mat = MEM_recallocN_id(ob->mat, sizeof(void *) * act, "matarray2");
+		ob->matbits = MEM_recallocN_id(ob->matbits, sizeof(char) * act, "matbits1");
 		ob->totcol = act;
 	}
 	
@@ -864,7 +911,7 @@ void assign_material(Object *ob, Material *ma, short act, int assign_type)
 
 	if (ma)
 		id_us_plus((ID *)ma);
-	test_object_materials(ob->data);
+	test_object_materials(G.main, ob->data);
 }
 
 /* XXX - this calls many more update calls per object then are needed, could be optimized */
@@ -903,17 +950,17 @@ short find_material_index(Object *ob, Material *ma)
 			break;
 	if (a < *totcolp)
 		return a + 1;
-	return 0;	   
+	return 0;
 }
 
-int object_add_material_slot(Object *ob)
+bool object_add_material_slot(Object *ob)
 {
-	if (ob == NULL) return FALSE;
-	if (ob->totcol >= MAXMAT) return FALSE;
+	if (ob == NULL) return false;
+	if (ob->totcol >= MAXMAT) return false;
 	
 	assign_material(ob, NULL, ob->totcol + 1, BKE_MAT_ASSIGN_USERPREF);
 	ob->actcol = ob->totcol;
-	return TRUE;
+	return true;
 }
 
 static void do_init_render_material(Material *ma, int r_mode, float *amb)
@@ -969,17 +1016,18 @@ static void do_init_render_material(Material *ma, int r_mode, float *amb)
 		ma->ambr = ma->amb * amb[0];
 		ma->ambg = ma->amb * amb[1];
 		ma->ambb = ma->amb * amb[2];
-	}	
-	/* will become or-ed result of all node modes */
-	ma->mode_l = ma->mode;
-	ma->mode_l &= ~MA_SHLESS;
+	}
 
-	if (ma->strand_surfnor > 0.0f)
-		ma->mode_l |= MA_STR_SURFDIFF;
+	/* local group override */
+	if ((ma->shade_flag & MA_GROUP_LOCAL) && ma->id.lib && ma->group && ma->group->id.lib) {
+		Group *group;
 
-	/* parses the geom+tex nodes */
-	if (ma->nodetree && ma->use_nodes)
-		ntreeShaderGetTexcoMode(ma->nodetree, r_mode, &ma->texco, &ma->mode_l);
+		for (group = G.main->group.first; group; group = group->id.next) {
+			if (!group->id.lib && strcmp(group->id.name, ma->group->id.name) == 0) {
+				ma->group = group;
+			}
+		}
+	}
 }
 
 static void init_render_nodetree(bNodeTree *ntree, Material *basemat, int r_mode, float *amb)
@@ -993,8 +1041,16 @@ static void init_render_nodetree(bNodeTree *ntree, Material *basemat, int r_mode
 				if (ma != basemat) {
 					do_init_render_material(ma, r_mode, amb);
 					basemat->texco |= ma->texco;
-					basemat->mode_l |= ma->mode_l & ~(MA_TRANSP | MA_ZTRANSP | MA_RAYTRANSP);
 				}
+
+				basemat->mode_l |= ma->mode & ~(MA_MODE_PIPELINE | MA_SHLESS);
+				basemat->mode2_l |= ma->mode2 & ~MA_MODE2_PIPELINE;
+				/* basemat only considered shadeless if all node materials are too */
+				if (!(ma->mode & MA_SHLESS))
+					basemat->mode_l &= ~MA_SHLESS;
+
+				if (ma->strand_surfnor > 0.0f)
+					basemat->mode_l |= MA_STR_SURFDIFF;
 			}
 			else if (node->type == NODE_GROUP)
 				init_render_nodetree((bNodeTree *)node->id, basemat, r_mode, amb);
@@ -1008,10 +1064,26 @@ void init_render_material(Material *mat, int r_mode, float *amb)
 	do_init_render_material(mat, r_mode, amb);
 	
 	if (mat->nodetree && mat->use_nodes) {
+		/* mode_l will take the pipeline options from the main material, and the or-ed
+		 * result of non-pipeline options from the nodes. shadeless is an exception,
+		 * mode_l will have it set when all node materials are shadeless. */
+		mat->mode_l = (mat->mode & MA_MODE_PIPELINE) | MA_SHLESS;
+		mat->mode2_l = mat->mode2 & MA_MODE2_PIPELINE;
+
+		/* parses the geom+tex nodes */
+		ntreeShaderGetTexcoMode(mat->nodetree, r_mode, &mat->texco, &mat->mode_l);
+
 		init_render_nodetree(mat->nodetree, mat, r_mode, amb);
 		
 		if (!mat->nodetree->execdata)
-			mat->nodetree->execdata = ntreeShaderBeginExecTree(mat->nodetree, 1);
+			mat->nodetree->execdata = ntreeShaderBeginExecTree(mat->nodetree);
+	}
+	else {
+		mat->mode_l = mat->mode;
+		mat->mode2_l = mat->mode2;
+
+		if (mat->strand_surfnor > 0.0f)
+			mat->mode_l |= MA_STR_SURFDIFF;
 	}
 }
 
@@ -1037,7 +1109,7 @@ void init_render_materials(Main *bmain, int r_mode, float *amb)
 			init_render_material(ma, r_mode, amb);
 	}
 	
-	do_init_render_material(&defmaterial, r_mode, amb);
+	init_render_material(&defmaterial, r_mode, amb);
 }
 
 /* only needed for nodes now */
@@ -1045,7 +1117,7 @@ void end_render_material(Material *mat)
 {
 	if (mat && mat->nodetree && mat->use_nodes) {
 		if (mat->nodetree->execdata)
-			ntreeShaderEndExecTree(mat->nodetree->execdata, 1);
+			ntreeShaderEndExecTree(mat->nodetree->execdata);
 	}
 }
 
@@ -1057,7 +1129,7 @@ void end_render_materials(Main *bmain)
 			end_render_material(ma);
 }
 
-static int material_in_nodetree(bNodeTree *ntree, Material *mat)
+static bool material_in_nodetree(bNodeTree *ntree, Material *mat)
 {
 	bNode *node;
 
@@ -1065,28 +1137,28 @@ static int material_in_nodetree(bNodeTree *ntree, Material *mat)
 		if (node->id) {
 			if (GS(node->id->name) == ID_MA) {
 				if (node->id == (ID *)mat) {
-					return 1;
+					return true;
 				}
 			}
 			else if (node->type == NODE_GROUP) {
 				if (material_in_nodetree((bNodeTree *)node->id, mat)) {
-					return 1;
+					return true;
 				}
 			}
 		}
 	}
 
-	return 0;
+	return false;
 }
 
-int material_in_material(Material *parmat, Material *mat)
+bool material_in_material(Material *parmat, Material *mat)
 {
 	if (parmat == mat)
-		return 1;
+		return true;
 	else if (parmat->nodetree && parmat->use_nodes)
 		return material_in_nodetree(parmat->nodetree, mat);
 	else
-		return 0;
+		return false;
 }
 
 
@@ -1131,8 +1203,8 @@ void material_drivers_update(Scene *scene, Material *ma, float ctime)
 	 */
 	if (ma->id.flag & LIB_DOIT)
 		return;
-	else
-		ma->id.flag |= LIB_DOIT;
+
+	ma->id.flag |= LIB_DOIT;
 	
 	/* material itself */
 	if (ma->adt && ma->adt->drivers.first) {
@@ -1143,58 +1215,11 @@ void material_drivers_update(Scene *scene, Material *ma, float ctime)
 	if (ma->nodetree) {
 		material_node_drivers_update(scene, ma->nodetree, ctime);
 	}
+
+	ma->id.flag &= ~LIB_DOIT;
 }
-	
-/* ****************** */
-#if 0 /* UNUSED */
-static char colname_array[125][20]= {
-"Black", "DarkRed", "HalfRed", "Red", "Red",
-"DarkGreen", "DarkOlive", "Brown", "Chocolate", "OrangeRed",
-"HalfGreen", "GreenOlive", "DryOlive", "Goldenrod", "DarkOrange",
-"LightGreen", "Chartreuse", "YellowGreen", "Yellow", "Gold",
-"Green", "LawnGreen", "GreenYellow", "LightOlive", "Yellow",
-"DarkBlue", "DarkPurple", "HotPink", "VioletPink", "RedPink",
-"SlateGray", "DarkGray", "PalePurple", "IndianRed", "Tomato",
-"SeaGreen", "PaleGreen", "GreenKhaki", "LightBrown", "LightSalmon",
-"SpringGreen", "PaleGreen", "MediumOlive", "YellowBrown", "LightGold",
-"LightGreen", "LightGreen", "LightGreen", "GreenYellow", "PaleYellow",
-"HalfBlue", "DarkSky", "HalfMagenta", "VioletRed", "DeepPink",
-"SteelBlue", "SkyBlue", "Orchid", "LightHotPink", "HotPink",
-"SeaGreen", "SlateGray", "MediumGray", "Burlywood", "LightPink",
-"SpringGreen", "Aquamarine", "PaleGreen", "Khaki", "PaleOrange",
-"SpringGreen", "SeaGreen", "PaleGreen", "PaleWhite", "YellowWhite",
-"LightBlue", "Purple", "MediumOrchid", "Magenta", "Magenta",
-"RoyalBlue", "SlateBlue", "MediumOrchid", "Orchid", "Magenta",
-"DeepSkyBlue", "LightSteelBlue", "LightSkyBlue", "Violet", "LightPink",
-"Cyan", "DarkTurquoise", "SkyBlue", "Gray", "Snow",
-"Mint", "Mint", "Aquamarine", "MintCream", "Ivory",
-"Blue", "Blue", "DarkMagenta", "DarkOrchid", "Magenta",
-"SkyBlue", "RoyalBlue", "LightSlateBlue", "MediumOrchid", "Magenta",
-"DodgerBlue", "SteelBlue", "MediumPurple", "PalePurple", "Plum",
-"DeepSkyBlue", "PaleBlue", "LightSkyBlue", "PalePurple", "Thistle",
-"Cyan", "ColdBlue", "PaleTurquoise", "GhostWhite", "White"
-};
 
-void automatname(Material *ma)
-{
-	int nr, r, g, b;
-	float ref;
-	
-	if (ma == NULL) return;
-	if (ma->mode & MA_SHLESS) ref = 1.0;
-	else ref = ma->ref;
-
-	r = (int)(4.99f * (ref * ma->r));
-	g = (int)(4.99f * (ref * ma->g));
-	b = (int)(4.99f * (ref * ma->b));
-	nr = r + 5 * g + 25 * b;
-	if (nr > 124) nr = 124;
-	new_id(&G.main->mat, (ID *)ma, colname_array[nr]);
-	
-}
-#endif
-
-int object_remove_material_slot(Object *ob)
+bool object_remove_material_slot(Object *ob)
 {
 	Material *mao, ***matarar;
 	Object *obt;
@@ -1202,14 +1227,14 @@ int object_remove_material_slot(Object *ob)
 	short a, actcol;
 	
 	if (ob == NULL || ob->totcol == 0) {
-		return FALSE;
+		return false;
 	}
 
 	/* this should never happen and used to crash */
 	if (ob->actcol <= 0) {
 		printf("%s: invalid material index %d, report a bug!\n", __func__, ob->actcol);
 		BLI_assert(0);
-		return FALSE;
+		return false;
 	}
 
 	/* take a mesh/curve/mball as starting point, remove 1 index,
@@ -1221,8 +1246,15 @@ int object_remove_material_slot(Object *ob)
 	totcolp = give_totcolp(ob);
 	matarar = give_matarar(ob);
 
-	if (*matarar == NULL) return FALSE;
+	if (ELEM(NULL, matarar, *matarar)) {
+		return false;
+	}
 
+	/* can happen on face selection in editmode */
+	if (ob->actcol > ob->totcol) {
+		ob->actcol = ob->totcol;
+	}
+	
 	/* we delete the actcol */
 	mao = (*matarar)[ob->actcol - 1];
 	if (mao) mao->id.us--;
@@ -1264,12 +1296,132 @@ int object_remove_material_slot(Object *ob)
 	}
 
 	/* check indices from mesh */
-	if (ELEM4(ob->type, OB_MESH, OB_CURVE, OB_SURF, OB_FONT)) {
-		data_delete_material_index_id((ID *)ob->data, actcol - 1);
-		BKE_displist_free(&ob->disp);
+	if (ELEM(ob->type, OB_MESH, OB_CURVE, OB_SURF, OB_FONT)) {
+		material_data_index_remove_id((ID *)ob->data, actcol - 1);
+		if (ob->curve_cache) {
+			BKE_displist_free(&ob->curve_cache->disp);
+		}
 	}
 
-	return TRUE;
+	return true;
+}
+
+static bool get_mtex_slot_valid_texpaint(struct MTex *mtex)
+{
+	return (mtex && (mtex->texco == TEXCO_UV) &&
+	        mtex->tex && (mtex->tex->type == TEX_IMAGE) &&
+	        mtex->tex->ima);
+}
+
+void BKE_texpaint_slot_refresh_cache(Scene *scene, Material *ma)
+{
+	MTex **mtex;
+	short count = 0;
+	short index = 0, i;
+
+	bool use_nodes = BKE_scene_use_new_shading_nodes(scene);
+	bool is_bi = BKE_scene_uses_blender_internal(scene) || BKE_scene_uses_blender_game(scene);
+	
+	if (!ma)
+		return;
+
+	if (ma->texpaintslot) {
+		MEM_freeN(ma->texpaintslot);
+		ma->tot_slots = 0;
+		ma->texpaintslot = NULL;
+	}
+
+	if (scene->toolsettings->imapaint.mode == IMAGEPAINT_MODE_IMAGE) {
+		ma->paint_active_slot = 0;
+		ma->paint_clone_slot = 0;
+		return;
+	}
+	
+	if (use_nodes || ma->use_nodes) {
+		bNode *node, *active_node;
+
+		if (!(ma->nodetree)) {
+			ma->paint_active_slot = 0;
+			ma->paint_clone_slot = 0;
+			return;
+		}
+
+		for (node = ma->nodetree->nodes.first; node; node = node->next) {
+			if (node->typeinfo->nclass == NODE_CLASS_TEXTURE && node->typeinfo->type == SH_NODE_TEX_IMAGE && node->id)
+				count++;
+		}
+
+		if (count == 0) {
+			ma->paint_active_slot = 0;
+			ma->paint_clone_slot = 0;
+			return;
+		}
+		ma->texpaintslot = MEM_callocN(sizeof(*ma->texpaintslot) * count, "texpaint_slots");
+
+		active_node = nodeGetActiveTexture(ma->nodetree);
+
+		for (node = ma->nodetree->nodes.first; node; node = node->next) {
+			if (node->typeinfo->nclass == NODE_CLASS_TEXTURE && node->typeinfo->type == SH_NODE_TEX_IMAGE && node->id) {
+				if (active_node == node)
+					ma->paint_active_slot = index;
+				ma->texpaintslot[index++].ima = (Image *)node->id;
+			}
+		}
+	}
+	else if (is_bi) {
+		for (mtex = ma->mtex, i = 0; i < MAX_MTEX; i++, mtex++) {
+			if (get_mtex_slot_valid_texpaint(*mtex)) {
+				count++;
+			}
+		}
+
+		if (count == 0) {
+			ma->paint_active_slot = 0;
+			ma->paint_clone_slot = 0;
+			return;
+		}
+
+		ma->texpaintslot = MEM_callocN(sizeof(*ma->texpaintslot) * count, "texpaint_slots");
+
+		for (mtex = ma->mtex, i = 0; i < MAX_MTEX; i++, mtex++) {
+			if (get_mtex_slot_valid_texpaint(*mtex)) {
+				ma->texpaintslot[index].ima = (*mtex)->tex->ima;
+				ma->texpaintslot[index].uvname = (*mtex)->uvname;
+				ma->texpaintslot[index].index = i;
+				
+				index++;
+			}
+		}
+	}
+	else {
+		ma->paint_active_slot = 0;
+		ma->paint_clone_slot = 0;
+		return;
+	}	
+
+
+	ma->tot_slots = count;
+	
+	
+	if (ma->paint_active_slot >= count) {
+		ma->paint_active_slot = count - 1;
+	}
+
+	if (ma->paint_clone_slot >= count) {
+		ma->paint_clone_slot = count - 1;
+	}
+
+	return;
+}
+
+void BKE_texpaint_slots_refresh_object(Scene *scene, struct Object *ob)
+{
+	int i;
+
+	for (i = 1; i < ob->totcol + 1; i++) {
+		Material *ma = give_current_material(ob, i);
+		BKE_texpaint_slot_refresh_cache(scene, ma);
+	}
 }
 
 
@@ -1332,12 +1484,9 @@ void ramp_blend(int type, float r_col[3], const float fac, const float col[3])
 			r_col[2] = facm * (r_col[2]) + fac * fabsf(r_col[2] - col[2]);
 			break;
 		case MA_RAMP_DARK:
-			tmp = col[0] + ((1 - col[0]) * facm);
-			if (tmp < r_col[0]) r_col[0] = tmp;
-			tmp = col[1] + ((1 - col[1]) * facm);
-			if (tmp < r_col[1]) r_col[1] = tmp;
-			tmp = col[2] + ((1 - col[2]) * facm);
-			if (tmp < r_col[2]) r_col[2] = tmp;
+			r_col[0] = min_ff(r_col[0], col[0]) * fac + r_col[0] * facm;
+			r_col[1] = min_ff(r_col[1], col[1]) * fac + r_col[1] * facm;
+			r_col[2] = min_ff(r_col[2], col[2]) * fac + r_col[2] * facm;
 			break;
 		case MA_RAMP_LIGHT:
 			tmp = fac * col[0];
@@ -1421,8 +1570,8 @@ void ramp_blend(int type, float r_col[3], const float fac, const float col[3])
 				r_col[1] = facm * (r_col[1]) + fac * tmpg;
 				r_col[2] = facm * (r_col[2]) + fac * tmpb;
 			}
+			break;
 		}
-		break;
 		case MA_RAMP_SAT:
 		{
 			float rH, rS, rV;
@@ -1432,8 +1581,8 @@ void ramp_blend(int type, float r_col[3], const float fac, const float col[3])
 				rgb_to_hsv(col[0], col[1], col[2], &colH, &colS, &colV);
 				hsv_to_rgb(rH, (facm * rS + fac * colS), rV, r_col + 0, r_col + 1, r_col + 2);
 			}
+			break;
 		}
-		break;
 		case MA_RAMP_VAL:
 		{
 			float rH, rS, rV;
@@ -1441,8 +1590,8 @@ void ramp_blend(int type, float r_col[3], const float fac, const float col[3])
 			rgb_to_hsv(r_col[0], r_col[1], r_col[2], &rH, &rS, &rV);
 			rgb_to_hsv(col[0], col[1], col[2], &colH, &colS, &colV);
 			hsv_to_rgb(rH, rS, (facm * rV + fac * colV), r_col + 0, r_col + 1, r_col + 2);
+			break;
 		}
-		break;
 		case MA_RAMP_COLOR:
 		{
 			float rH, rS, rV;
@@ -1456,8 +1605,8 @@ void ramp_blend(int type, float r_col[3], const float fac, const float col[3])
 				r_col[1] = facm * (r_col[1]) + fac * tmpg;
 				r_col[2] = facm * (r_col[2]) + fac * tmpb;
 			}
+			break;
 		}
-		break;
 		case MA_RAMP_SOFT:
 		{
 			float scr, scg, scb;
@@ -1470,8 +1619,8 @@ void ramp_blend(int type, float r_col[3], const float fac, const float col[3])
 			r_col[0] = facm * (r_col[0]) + fac * (((1.0f - r_col[0]) * col[0] * (r_col[0])) + (r_col[0] * scr));
 			r_col[1] = facm * (r_col[1]) + fac * (((1.0f - r_col[1]) * col[1] * (r_col[1])) + (r_col[1] * scg));
 			r_col[2] = facm * (r_col[2]) + fac * (((1.0f - r_col[2]) * col[2] * (r_col[2])) + (r_col[2] * scb));
+			break;
 		}
-		break;
 		case MA_RAMP_LINEAR:
 			if (col[0] > 0.5f)
 				r_col[0] = r_col[0] + fac * (2.0f * (col[0] - 0.5f));
@@ -1521,7 +1670,7 @@ void free_matcopybuf(void)
 	matcopybuf.ramp_spec = NULL;
 
 	if (matcopybuf.nodetree) {
-		ntreeFreeTree_ex(matcopybuf.nodetree, FALSE);
+		ntreeFreeTree_ex(matcopybuf.nodetree, false);
 		MEM_freeN(matcopybuf.nodetree);
 		matcopybuf.nodetree = NULL;
 	}
@@ -1547,9 +1696,9 @@ void copy_matcopybuf(Material *ma)
 			matcopybuf.mtex[a] = MEM_dupallocN(mtex);
 		}
 	}
-	matcopybuf.nodetree = ntreeCopyTree_ex(ma->nodetree, FALSE);
+	matcopybuf.nodetree = ntreeCopyTree_ex(ma->nodetree, G.main, false);
 	matcopybuf.preview = NULL;
-	matcopybuf.gpumaterial.first = matcopybuf.gpumaterial.last = NULL;
+	BLI_listbase_clear(&matcopybuf.gpumaterial);
 	matcopied = 1;
 }
 
@@ -1575,7 +1724,7 @@ void paste_matcopybuf(Material *ma)
 		MEM_freeN(ma->nodetree);
 	}
 
-	GPU_material_free(ma);
+	GPU_material_free(&ma->gpumaterial);
 
 	id = (ma->id);
 	memcpy(ma, &matcopybuf, sizeof(Material));
@@ -1588,11 +1737,19 @@ void paste_matcopybuf(Material *ma)
 		mtex = ma->mtex[a];
 		if (mtex) {
 			ma->mtex[a] = MEM_dupallocN(mtex);
-			if (mtex->tex) id_us_plus((ID *)mtex->tex);
+			if (mtex->tex) {
+				/* first check this is in main (we may have loaded another file) [#35500] */
+				if (BLI_findindex(&G.main->tex, mtex->tex) != -1) {
+					id_us_plus((ID *)mtex->tex);
+				}
+				else {
+					ma->mtex[a]->tex = NULL;
+				}
+			}
 		}
 	}
 
-	ma->nodetree = ntreeCopyTree_ex(matcopybuf.nodetree, FALSE);
+	ma->nodetree = ntreeCopyTree_ex(matcopybuf.nodetree, G.main, false);
 }
 
 
@@ -1603,7 +1760,7 @@ static int encode_tfaceflag(MTFace *tf, int convertall)
 	/* calculate the flag */
 	int flag = tf->mode;
 
-	/* options that change the material offline render */	
+	/* options that change the material offline render */
 	if (!convertall) {
 		flag &= ~TF_OBCOL;
 	}
@@ -1627,7 +1784,7 @@ static int encode_tfaceflag(MTFace *tf, int convertall)
 /* set the material options based in the tface flag */
 static void decode_tfaceflag(Material *ma, int flag, int convertall)
 {
-	int alphablend;	
+	int alphablend;
 	GameSettings *game = &ma->game;
 
 	/* flag is shifted in 1 to make 0 != no flag yet (see encode_tfaceflag) */
@@ -1733,7 +1890,7 @@ static short mesh_getmaterialnumber(Mesh *me, Material *ma)
 /* append material */
 static short mesh_addmaterial(Mesh *me, Material *ma)
 {
-	material_append_id(&me->id, NULL);
+	BKE_material_append_id(&me->id, NULL);
 	me->mat[me->totcol - 1] = ma;
 
 	id_us_plus(&ma->id);
@@ -1772,7 +1929,7 @@ static short convert_tfacenomaterial(Main *main, Mesh *me, MTFace *tf, int flag)
 	}
 	/* create a new material */
 	else {
-		ma = BKE_material_add(idname + 2);
+		ma = BKE_material_add(main, idname + 2);
 
 		if (ma) {
 			printf("TexFace Convert: Material \"%s\" created.\n", idname + 2);
@@ -1786,13 +1943,15 @@ static short convert_tfacenomaterial(Main *main, Mesh *me, MTFace *tf, int flag)
 			 * for now store the flag into the material and change light/tex/collision
 			 * store the flag as a negative number */
 			ma->game.flag = -flag;
-			id_us_min((ID *)ma);	
+			id_us_min((ID *)ma);
+		}
+		else {
+			printf("Error: Unable to create Material \"%s\" for Mesh \"%s\".", idname + 2, me->id.name + 2);
 		}
-		else printf("Error: Unable to create Material \"%s\" for Mesh \"%s\".", idname + 2, me->id.name + 2);
 	}
 
 	/* set as converted, no need to go bad to this face */
-	tf->mode |= TF_CONVERTED;	
+	tf->mode |= TF_CONVERTED;
 	return mat_nr;
 }
 
@@ -1844,8 +2003,8 @@ static void convert_tfacematerial(Main *main, Material *ma)
 				mat_new = BKE_material_copy(ma);
 				if (mat_new) {
 					/* rename the material*/
-					strcpy(mat_new->id.name, idname);
-					id_us_min((ID *)mat_new);	
+					BLI_strncpy(mat_new->id.name, idname, sizeof(mat_new->id.name));
+					id_us_min((ID *)mat_new);
 
 					mat_nr = mesh_addmaterial(me, mat_new);
 					decode_tfaceflag(mat_new, flag, 1);
@@ -1868,15 +2027,21 @@ static void convert_tfacematerial(Main *main, Material *ma)
 			mf->mat_nr = mat_nr;
 		}
 		/* remove material from mesh */
-		for (a = 0; a < me->totcol; )
-			if (me->mat[a] == ma) material_pop_id(&me->id, a, 1); else a++;
+		for (a = 0; a < me->totcol; ) {
+			if (me->mat[a] == ma) {
+				BKE_material_pop_id(&me->id, a, true);
+			}
+			else {
+				a++;
+			}
+		}
 	}
 }
 
 
 #define MAT_BGE_DISPUTED -99999
 
-int do_version_tface(Main *main, int fileload)
+int do_version_tface(Main *main)
 {
 	Mesh *me;
 	Material *ma;
@@ -1886,6 +2051,9 @@ int do_version_tface(Main *main, int fileload)
 	int a;
 	int flag;
 	int index;
+	
+	/* Operator in help menu has been removed for 2.7x */
+	int fileload = 1;
 
 	/* sometimes mesh has no materials but will need a new one. In those
 	 * cases we need to ignore the mf->mat_nr and only look at the face
@@ -2029,11 +2197,13 @@ int do_version_tface(Main *main, int fileload)
 		if (ma->game.flag == MAT_BGE_DISPUTED) {
 			ma->game.flag = 0;
 			if (fileload) {
-				printf("Warning: material \"%s\" skipped - to convert old game texface to material go to the Help menu.\n", ma->id.name + 2);
+				printf("Warning: material \"%s\" skipped.\n", ma->id.name + 2);
 				nowarning = 0;
 			}
-			else
-				convert_tfacematerial(main, ma);			continue;	
+			else {
+				convert_tfacematerial(main, ma);
+			}
+			continue;
 		}
 	
 		/* no conflicts in this material - 90% of cases
