@@ -36,7 +36,7 @@
  *  \ingroup bke
  */
 
-
+#include <stdlib.h>
 #include <stddef.h>
 #include <string.h>
 #include <stdarg.h>
@@ -47,7 +47,6 @@
 
 #include "DNA_armature_types.h"
 #include "DNA_object_types.h"
-#include "DNA_meshdata_types.h"
 
 #include "BLI_utildefines.h"
 #include "BLI_path_util.h"
@@ -55,9 +54,12 @@
 #include "BLI_linklist.h"
 #include "BLI_string.h"
 
-#include "BKE_cloth.h"
+#include "BLF_translation.h"
+
+#include "BKE_appdir.h"
 #include "BKE_key.h"
 #include "BKE_multires.h"
+#include "BKE_DerivedMesh.h"
 
 /* may move these, only for modifier_path_relbase */
 #include "BKE_global.h" /* ugh, G.main->name only */
@@ -66,19 +68,44 @@
 
 #include "MOD_modifiertypes.h"
 
+static ModifierTypeInfo *modifier_types[NUM_MODIFIER_TYPES] = {NULL};
+static VirtualModifierData virtualModifierCommonData;
+
+void BKE_modifier_init(void)
+{
+	ModifierData *md;
+
+	/* Initialize modifier types */
+	modifier_type_init(modifier_types); /* MOD_utils.c */
+
+	/* Initialize global cmmon storage used for virtual modifier list */
+	md = modifier_new(eModifierType_Armature);
+	virtualModifierCommonData.amd = *((ArmatureModifierData *) md);
+	modifier_free(md);
+
+	md = modifier_new(eModifierType_Curve);
+	virtualModifierCommonData.cmd = *((CurveModifierData *) md);
+	modifier_free(md);
+
+	md = modifier_new(eModifierType_Lattice);
+	virtualModifierCommonData.lmd = *((LatticeModifierData *) md);
+	modifier_free(md);
+
+	md = modifier_new(eModifierType_ShapeKey);
+	virtualModifierCommonData.smd = *((ShapeKeyModifierData *) md);
+	modifier_free(md);
+
+	virtualModifierCommonData.amd.modifier.mode |= eModifierMode_Virtual;
+	virtualModifierCommonData.cmd.modifier.mode |= eModifierMode_Virtual;
+	virtualModifierCommonData.lmd.modifier.mode |= eModifierMode_Virtual;
+	virtualModifierCommonData.smd.modifier.mode |= eModifierMode_Virtual;
+}
+
 ModifierTypeInfo *modifierType_getInfo(ModifierType type)
 {
-	static ModifierTypeInfo *types[NUM_MODIFIER_TYPES] = {NULL};
-	static int types_init = 1;
-
-	if (types_init) {
-		modifier_type_init(types); /* MOD_utils.c */
-		types_init = 0;
-	}
-
 	/* type unsigned, no need to check < 0 */
-	if (type < NUM_MODIFIER_TYPES && types[type]->name[0] != '\0') {
-		return types[type];
+	if (type < NUM_MODIFIER_TYPES && modifier_types[type]->name[0] != '\0') {
+		return modifier_types[type];
 	}
 	else {
 		return NULL;
@@ -93,7 +120,7 @@ ModifierData *modifier_new(int type)
 	ModifierData *md = MEM_callocN(mti->structSize, mti->structName);
 	
 	/* note, this name must be made unique later */
-	BLI_strncpy(md->name, mti->name, sizeof(md->name));
+	BLI_strncpy(md->name, DATA_(mti->name), sizeof(md->name));
 
 	md->type = type;
 	md->mode = eModifierMode_Realtime | eModifierMode_Render | eModifierMode_Expanded;
@@ -120,19 +147,19 @@ void modifier_unique_name(ListBase *modifiers, ModifierData *md)
 {
 	if (modifiers && md) {
 		ModifierTypeInfo *mti = modifierType_getInfo(md->type);
-		
-		BLI_uniquename(modifiers, md, mti->name, '.', offsetof(ModifierData, name), sizeof(md->name));
+
+		BLI_uniquename(modifiers, md, DATA_(mti->name), '.', offsetof(ModifierData, name), sizeof(md->name));
 	}
 }
 
-int modifier_dependsOnTime(ModifierData *md) 
+bool modifier_dependsOnTime(ModifierData *md)
 {
 	ModifierTypeInfo *mti = modifierType_getInfo(md->type);
 
 	return mti->dependsOnTime && mti->dependsOnTime(md);
 }
 
-int modifier_supportsMapping(ModifierData *md)
+bool modifier_supportsMapping(ModifierData *md)
 {
 	ModifierTypeInfo *mti = modifierType_getInfo(md->type);
 
@@ -140,17 +167,20 @@ int modifier_supportsMapping(ModifierData *md)
 	        (mti->flags & eModifierTypeFlag_SupportsMapping));
 }
 
-int modifier_isPreview(ModifierData *md)
+bool modifier_isPreview(ModifierData *md)
 {
 	ModifierTypeInfo *mti = modifierType_getInfo(md->type);
 
-	if (!(mti->flags & eModifierTypeFlag_UsesPreview))
-		return FALSE;
+	/* Constructive modifiers are highly likely to also modify data like vgroups or vcol! */
+	if (!((mti->flags & eModifierTypeFlag_UsesPreview) || (mti->type == eModifierTypeType_Constructive))) {
+		return false;
+	}
 
-	if (md->mode & eModifierMode_Realtime)
-		return TRUE;
+	if (md->mode & eModifierMode_Realtime) {
+		return true;
+	}
 
-	return FALSE;
+	return false;
 }
 
 ModifierData *modifiers_findByType(Object *ob, ModifierType type)
@@ -225,6 +255,19 @@ void modifiers_foreachTexLink(Object *ob, TexWalkFunc walk, void *userData)
 	}
 }
 
+/* callback's can use this
+ * to avoid copying every member.
+ */
+void modifier_copyData_generic(const ModifierData *md_src, ModifierData *md_dst)
+{
+	ModifierTypeInfo *mti = modifierType_getInfo(md_src->type);
+	const size_t data_size = sizeof(ModifierData);
+	const char *md_src_data = ((char *)md_src) + data_size;
+	char       *md_dst_data = ((char *)md_dst) + data_size;
+	BLI_assert(data_size <= (size_t)mti->structSize);
+	memcpy(md_dst_data, md_src_data, (size_t)mti->structSize - data_size);
+}
+
 void modifier_copyData(ModifierData *md, ModifierData *target)
 {
 	ModifierTypeInfo *mti = modifierType_getInfo(md->type);
@@ -235,7 +278,19 @@ void modifier_copyData(ModifierData *md, ModifierData *target)
 		mti->copyData(md, target);
 }
 
-int modifier_couldBeCage(struct Scene *scene, ModifierData *md)
+
+bool modifier_supportsCage(struct Scene *scene, ModifierData *md)
+{
+	ModifierTypeInfo *mti = modifierType_getInfo(md->type);
+
+	md->scene = scene;
+
+	return ((!mti->isDisabled || !mti->isDisabled(md, 0)) &&
+	        (mti->flags & eModifierTypeFlag_SupportsEditmode) &&
+	        modifier_supportsMapping(md));
+}
+
+bool modifier_couldBeCage(struct Scene *scene, ModifierData *md)
 {
 	ModifierTypeInfo *mti = modifierType_getInfo(md->type);
 
@@ -247,24 +302,25 @@ int modifier_couldBeCage(struct Scene *scene, ModifierData *md)
 	        modifier_supportsMapping(md));
 }
 
-int modifier_sameTopology(ModifierData *md)
+bool modifier_isSameTopology(ModifierData *md)
 {
 	ModifierTypeInfo *mti = modifierType_getInfo(md->type);
 	return ELEM(mti->type, eModifierTypeType_OnlyDeform, eModifierTypeType_NonGeometrical);
 }
 
-int modifier_nonGeometrical(ModifierData *md)
+bool modifier_isNonGeometrical(ModifierData *md)
 {
 	ModifierTypeInfo *mti = modifierType_getInfo(md->type);
 	return (mti->type == eModifierTypeType_NonGeometrical);
 }
 
-void modifier_setError(ModifierData *md, const char *format, ...)
+void modifier_setError(ModifierData *md, const char *_format, ...)
 {
 	char buffer[512];
 	va_list ap;
+	const char *format = TIP_(_format);
 
-	va_start(ap, format);
+	va_start(ap, _format);
 	vsnprintf(buffer, sizeof(buffer), format, ap);
 	va_end(ap);
 	buffer[sizeof(buffer) - 1] = '\0';
@@ -283,32 +339,39 @@ void modifier_setError(ModifierData *md, const char *format, ...)
  * then is NULL) 
  * also used for some mesh tools to give warnings
  */
-int modifiers_getCageIndex(struct Scene *scene, Object *ob, int *lastPossibleCageIndex_r, int virtual_)
+int modifiers_getCageIndex(struct Scene *scene, Object *ob, int *r_lastPossibleCageIndex, bool is_virtual)
 {
-	ModifierData *md = (virtual_) ? modifiers_getVirtualModifierList(ob) : ob->modifiers.first;
+	VirtualModifierData virtualModifierData;
+	ModifierData *md = (is_virtual) ? modifiers_getVirtualModifierList(ob, &virtualModifierData) : ob->modifiers.first;
 	int i, cageIndex = -1;
 
-	if (lastPossibleCageIndex_r) {
+	if (r_lastPossibleCageIndex) {
 		/* ensure the value is initialized */
-		*lastPossibleCageIndex_r = -1;
+		*r_lastPossibleCageIndex = -1;
 	}
 
 	/* Find the last modifier acting on the cage. */
 	for (i = 0; md; i++, md = md->next) {
 		ModifierTypeInfo *mti = modifierType_getInfo(md->type);
+		bool supports_mapping;
 
 		md->scene = scene;
 
-		if (!(md->mode & eModifierMode_Realtime)) continue;
-		if (!(md->mode & eModifierMode_Editmode)) continue;
 		if (mti->isDisabled && mti->isDisabled(md, 0)) continue;
 		if (!(mti->flags & eModifierTypeFlag_SupportsEditmode)) continue;
 		if (md->mode & eModifierMode_DisableTemporary) continue;
 
-		if (!modifier_supportsMapping(md))
+		supports_mapping = modifier_supportsMapping(md);
+		if (r_lastPossibleCageIndex && supports_mapping) {
+			*r_lastPossibleCageIndex = i;
+		}
+
+		if (!(md->mode & eModifierMode_Realtime)) continue;
+		if (!(md->mode & eModifierMode_Editmode)) continue;
+
+		if (!supports_mapping)
 			break;
 
-		if (lastPossibleCageIndex_r) *lastPossibleCageIndex_r = i;
 		if (md->mode & eModifierMode_OnCage)
 			cageIndex = i;
 	}
@@ -317,50 +380,51 @@ int modifiers_getCageIndex(struct Scene *scene, Object *ob, int *lastPossibleCag
 }
 
 
-int modifiers_isSoftbodyEnabled(Object *ob)
+bool modifiers_isSoftbodyEnabled(Object *ob)
 {
 	ModifierData *md = modifiers_findByType(ob, eModifierType_Softbody);
 
 	return (md && md->mode & (eModifierMode_Realtime | eModifierMode_Render));
 }
 
-int modifiers_isClothEnabled(Object *ob)
+bool modifiers_isClothEnabled(Object *ob)
 {
 	ModifierData *md = modifiers_findByType(ob, eModifierType_Cloth);
 
 	return (md && md->mode & (eModifierMode_Realtime | eModifierMode_Render));
 }
 
-int modifiers_isModifierEnabled(Object *ob, int modifierType)
+bool modifiers_isModifierEnabled(Object *ob, int modifierType)
 {
 	ModifierData *md = modifiers_findByType(ob, modifierType);
 
 	return (md && md->mode & (eModifierMode_Realtime | eModifierMode_Render));
 }
 
-int modifiers_isParticleEnabled(Object *ob)
+bool modifiers_isParticleEnabled(Object *ob)
 {
 	ModifierData *md = modifiers_findByType(ob, eModifierType_ParticleSystem);
 
 	return (md && md->mode & (eModifierMode_Realtime | eModifierMode_Render));
 }
 
-int modifier_isEnabled(struct Scene *scene, ModifierData *md, int required_mode)
+bool modifier_isEnabled(struct Scene *scene, ModifierData *md, int required_mode)
 {
 	ModifierTypeInfo *mti = modifierType_getInfo(md->type);
 
 	md->scene = scene;
 
-	if ((md->mode & required_mode) != required_mode) return 0;
-	if (mti->isDisabled && mti->isDisabled(md, required_mode == eModifierMode_Render)) return 0;
-	if (md->mode & eModifierMode_DisableTemporary) return 0;
-	if (required_mode & eModifierMode_Editmode)
-		if (!(mti->flags & eModifierTypeFlag_SupportsEditmode)) return 0;
+	if ((md->mode & required_mode) != required_mode) return false;
+	if (mti->isDisabled && mti->isDisabled(md, required_mode == eModifierMode_Render)) return false;
+	if (md->mode & eModifierMode_DisableTemporary) return false;
+	if ((required_mode & eModifierMode_Editmode) && !(mti->flags & eModifierTypeFlag_SupportsEditmode)) return false;
 	
-	return 1;
+	return true;
 }
 
-CDMaskLink *modifiers_calcDataMasks(struct Scene *scene, Object *ob, ModifierData *md, CustomDataMask dataMask, int required_mode)
+CDMaskLink *modifiers_calcDataMasks(struct Scene *scene, Object *ob, ModifierData *md,
+                                    CustomDataMask dataMask, int required_mode,
+                                    ModifierData *previewmd, CustomDataMask previewmask)
 {
 	CDMaskLink *dataMasks = NULL;
 	CDMaskLink *curr, *prev;
@@ -371,10 +435,15 @@ CDMaskLink *modifiers_calcDataMasks(struct Scene *scene, Object *ob, ModifierDat
 
 		curr = MEM_callocN(sizeof(CDMaskLink), "CDMaskLink");
 		
-		if (modifier_isEnabled(scene, md, required_mode))
+		if (modifier_isEnabled(scene, md, required_mode)) {
 			if (mti->requiredDataMask)
 				curr->mask = mti->requiredDataMask(ob, md);
 
+			if (previewmd == md) {
+				curr->mask |= previewmask;
+			}
+		}
+
 		/* prepend new datamask */
 		curr->next = dataMasks;
 		dataMasks = curr;
@@ -410,7 +479,7 @@ ModifierData *modifiers_getLastPreview(struct Scene *scene, ModifierData *md, in
 {
 	ModifierData *tmp_md = NULL;
 
-	if (required_mode != eModifierMode_Realtime)
+	if ((required_mode & ~eModifierMode_Editmode) != eModifierMode_Realtime)
 		return tmp_md;
 
 	/* Find the latest modifier in stack generating preview. */
@@ -421,86 +490,58 @@ ModifierData *modifiers_getLastPreview(struct Scene *scene, ModifierData *md, in
 	return tmp_md;
 }
 
-/* NOTE: these aren't used anymore */
-ModifierData *modifiers_getVirtualModifierList(Object *ob)
+/* NOTE: This is to support old files from before Blender supported modifiers,
+ * in some cases versioning code updates these so for new files this will
+ * return an empty list. */
+ModifierData *modifiers_getVirtualModifierList(Object *ob, VirtualModifierData *virtualModifierData)
 {
-	/* Kinda hacky, but should be fine since we are never
-	 * re-entrant and avoid free hassles.
-	 */
-	static ArmatureModifierData amd;
-	static CurveModifierData cmd;
-	static LatticeModifierData lmd;
-	static ShapeKeyModifierData smd;
-	static int init = 1;
 	ModifierData *md;
 
-	if (init) {
-		md = modifier_new(eModifierType_Armature);
-		amd = *((ArmatureModifierData *) md);
-		modifier_free(md);
-
-		md = modifier_new(eModifierType_Curve);
-		cmd = *((CurveModifierData *) md);
-		modifier_free(md);
-
-		md = modifier_new(eModifierType_Lattice);
-		lmd = *((LatticeModifierData *) md);
-		modifier_free(md);
-
-		md = modifier_new(eModifierType_ShapeKey);
-		smd = *((ShapeKeyModifierData *) md);
-		modifier_free(md);
-
-		amd.modifier.mode |= eModifierMode_Virtual;
-		cmd.modifier.mode |= eModifierMode_Virtual;
-		lmd.modifier.mode |= eModifierMode_Virtual;
-		smd.modifier.mode |= eModifierMode_Virtual;
-
-		init = 0;
-	}
-
 	md = ob->modifiers.first;
 
+	*virtualModifierData = virtualModifierCommonData;
+
 	if (ob->parent) {
 		if (ob->parent->type == OB_ARMATURE && ob->partype == PARSKEL) {
-			amd.object = ob->parent;
-			amd.modifier.next = md;
-			amd.deformflag = ((bArmature *)(ob->parent->data))->deformflag;
-			md = &amd.modifier;
+			virtualModifierData->amd.object = ob->parent;
+			virtualModifierData->amd.modifier.next = md;
+			virtualModifierData->amd.deformflag = ((bArmature *)(ob->parent->data))->deformflag;
+			md = &virtualModifierData->amd.modifier;
 		}
 		else if (ob->parent->type == OB_CURVE && ob->partype == PARSKEL) {
-			cmd.object = ob->parent;
-			cmd.defaxis = ob->trackflag + 1;
-			cmd.modifier.next = md;
-			md = &cmd.modifier;
+			virtualModifierData->cmd.object = ob->parent;
+			virtualModifierData->cmd.defaxis = ob->trackflag + 1;
+			virtualModifierData->cmd.modifier.next = md;
+			md = &virtualModifierData->cmd.modifier;
 		}
 		else if (ob->parent->type == OB_LATTICE && ob->partype == PARSKEL) {
-			lmd.object = ob->parent;
-			lmd.modifier.next = md;
-			md = &lmd.modifier;
+			virtualModifierData->lmd.object = ob->parent;
+			virtualModifierData->lmd.modifier.next = md;
+			md = &virtualModifierData->lmd.modifier;
 		}
 	}
 
 	/* shape key modifier, not yet for curves */
 	if (ELEM(ob->type, OB_MESH, OB_LATTICE) && BKE_key_from_object(ob)) {
 		if (ob->type == OB_MESH && (ob->shapeflag & OB_SHAPE_EDIT_MODE))
-			smd.modifier.mode |= eModifierMode_Editmode | eModifierMode_OnCage;
+			virtualModifierData->smd.modifier.mode |= eModifierMode_Editmode | eModifierMode_OnCage;
 		else
-			smd.modifier.mode &= ~eModifierMode_Editmode | eModifierMode_OnCage;
+			virtualModifierData->smd.modifier.mode &= ~eModifierMode_Editmode | eModifierMode_OnCage;
 
-		smd.modifier.next = md;
-		md = &smd.modifier;
+		virtualModifierData->smd.modifier.next = md;
+		md = &virtualModifierData->smd.modifier;
 	}
 
 	return md;
 }
-/* Takes an object and returns its first selected armature, else just its
- * armature
+
+/* Takes an object and returns its first selected armature, else just its armature
  * This should work for multiple armatures per object
  */
 Object *modifiers_isDeformedByArmature(Object *ob)
 {
-	ModifierData *md = modifiers_getVirtualModifierList(ob);
+	VirtualModifierData virtualModifierData;
+	ModifierData *md = modifiers_getVirtualModifierList(ob, &virtualModifierData);
 	ArmatureModifierData *amd = NULL;
 	
 	/* return the first selected armature, this lets us use multiple armatures */
@@ -518,13 +559,13 @@ Object *modifiers_isDeformedByArmature(Object *ob)
 	return NULL;
 }
 
-/* Takes an object and returns its first selected lattice, else just its
- * lattice
- * This should work for multiple lattics per object
+/* Takes an object and returns its first selected lattice, else just its lattice
+ * This should work for multiple lattices per object
  */
 Object *modifiers_isDeformedByLattice(Object *ob)
 {
-	ModifierData *md = modifiers_getVirtualModifierList(ob);
+	VirtualModifierData virtualModifierData;
+	ModifierData *md = modifiers_getVirtualModifierList(ob, &virtualModifierData);
 	LatticeModifierData *lmd = NULL;
 	
 	/* return the first selected lattice, this lets us use multiple lattices */
@@ -542,70 +583,84 @@ Object *modifiers_isDeformedByLattice(Object *ob)
 	return NULL;
 }
 
-
-
-int modifiers_usesArmature(Object *ob, bArmature *arm)
+/* Takes an object and returns its first selected curve, else just its curve
+ * This should work for multiple curves per object
+ */
+Object *modifiers_isDeformedByCurve(Object *ob)
 {
-	ModifierData *md = modifiers_getVirtualModifierList(ob);
+	VirtualModifierData virtualModifierData;
+	ModifierData *md = modifiers_getVirtualModifierList(ob, &virtualModifierData);
+	CurveModifierData *cmd = NULL;
+	
+	/* return the first selected curve, this lets us use multiple curves */
+	for (; md; md = md->next) {
+		if (md->type == eModifierType_Curve) {
+			cmd = (CurveModifierData *) md;
+			if (cmd->object && (cmd->object->flag & SELECT))
+				return cmd->object;
+		}
+	}
+	
+	if (cmd) /* if were still here then return the last curve */
+		return cmd->object;
+	
+	return NULL;
+}
+
+bool modifiers_usesArmature(Object *ob, bArmature *arm)
+{
+	VirtualModifierData virtualModifierData;
+	ModifierData *md = modifiers_getVirtualModifierList(ob, &virtualModifierData);
 
 	for (; md; md = md->next) {
 		if (md->type == eModifierType_Armature) {
 			ArmatureModifierData *amd = (ArmatureModifierData *) md;
 			if (amd->object && amd->object->data == arm)
-				return 1;
+				return true;
 		}
 	}
 
-	return 0;
+	return false;
 }
 
-int modifier_isCorrectableDeformed(ModifierData *md)
+bool modifier_isCorrectableDeformed(ModifierData *md)
 {
-	if (md->type == eModifierType_Armature)
-		return 1;
-	if (md->type == eModifierType_ShapeKey)
-		return 1;
-	
-	return 0;
+	ModifierTypeInfo *mti = modifierType_getInfo(md->type);
+	return (mti->deformMatricesEM != NULL);
 }
 
-int modifiers_isCorrectableDeformed(Object *ob)
+bool modifiers_isCorrectableDeformed(struct Scene *scene, Object *ob)
 {
-	ModifierData *md = modifiers_getVirtualModifierList(ob);
+	VirtualModifierData virtualModifierData;
+	ModifierData *md = modifiers_getVirtualModifierList(ob, &virtualModifierData);
+	int required_mode = eModifierMode_Realtime;
+
+	if (ob->mode == OB_MODE_EDIT)
+		required_mode |= eModifierMode_Editmode;
 	
 	for (; md; md = md->next) {
-		if (ob->mode == OB_MODE_EDIT && (md->mode & eModifierMode_Editmode) == 0) {
+		if (!modifier_isEnabled(scene, md, required_mode)) {
 			/* pass */
 		}
 		else if (modifier_isCorrectableDeformed(md)) {
-			return 1;
+			return true;
 		}
 	}
-	return 0;
+	return false;
 }
 
 /* Check whether the given object has a modifier in its stack that uses WEIGHT_MCOL CD layer
  * to preview something... Used by DynamicPaint and WeightVG currently. */
-int modifiers_isPreview(Object *ob)
+bool modifiers_isPreview(Object *ob)
 {
 	ModifierData *md = ob->modifiers.first;
 
 	for (; md; md = md->next) {
 		if (modifier_isPreview(md))
-			return TRUE;
+			return true;
 	}
 
-	return FALSE;
-}
-
-int modifiers_indexInObject(Object *ob, ModifierData *md_seek)
-{
-	int i = 0;
-	ModifierData *md;
-	
-	for (md = ob->modifiers.first; (md && md_seek != md); md = md->next, i++) ;
-	if (!md) return -1;  /* modifier isn't in the object */
-	return i;
+	return false;
 }
 
 void modifier_freeTemporaryData(ModifierData *md)
@@ -658,7 +713,7 @@ const char *modifier_path_relbase(Object *ob)
 	else {
 		/* last resort, better then using "" which resolves to the current
 		 * working directory */
-		return BLI_temporary_dir();
+		return BKE_tempdir_session();
 	}
 }
 
@@ -668,6 +723,68 @@ void modifier_path_init(char *path, int path_maxlen, const char *name)
 	/* elubie: changed this to default to the same dir as the render output
 	 * to prevent saving to C:\ on Windows */
 	BLI_join_dirfile(path, path_maxlen,
-	                 G.relbase_valid ? "//" : BLI_temporary_dir(),
+	                 G.relbase_valid ? "//" : BKE_tempdir_session(),
 	                 name);
 }
+
+
+/* wrapper around ModifierTypeInfo.applyModifier that ensures valid normals */
+
+struct DerivedMesh *modwrap_applyModifier(
+        ModifierData *md, Object *ob,
+        struct DerivedMesh *dm,
+        ModifierApplyFlag flag)
+{
+	ModifierTypeInfo *mti = modifierType_getInfo(md->type);
+	BLI_assert(CustomData_has_layer(&dm->polyData, CD_NORMAL) == false);
+
+	if (mti->dependsOnNormals && mti->dependsOnNormals(md)) {
+		DM_ensure_normals(dm);
+	}
+	return mti->applyModifier(md, ob, dm, flag);
+}
+
+struct DerivedMesh *modwrap_applyModifierEM(
+        ModifierData *md, Object *ob,
+        struct BMEditMesh *em,
+        DerivedMesh *dm,
+        ModifierApplyFlag flag)
+{
+	ModifierTypeInfo *mti = modifierType_getInfo(md->type);
+	BLI_assert(CustomData_has_layer(&dm->polyData, CD_NORMAL) == false);
+
+	if (mti->dependsOnNormals && mti->dependsOnNormals(md)) {
+		DM_ensure_normals(dm);
+	}
+	return mti->applyModifierEM(md, ob, em, dm, flag);
+}
+
+void modwrap_deformVerts(
+        ModifierData *md, Object *ob,
+        DerivedMesh *dm,
+        float (*vertexCos)[3], int numVerts,
+        ModifierApplyFlag flag)
+{
+	ModifierTypeInfo *mti = modifierType_getInfo(md->type);
+	BLI_assert(!dm || CustomData_has_layer(&dm->polyData, CD_NORMAL) == false);
+
+	if (dm && mti->dependsOnNormals && mti->dependsOnNormals(md)) {
+		DM_ensure_normals(dm);
+	}
+	mti->deformVerts(md, ob, dm, vertexCos, numVerts, flag);
+}
+
+void modwrap_deformVertsEM(
+        ModifierData *md, Object *ob,
+        struct BMEditMesh *em, DerivedMesh *dm,
+        float (*vertexCos)[3], int numVerts)
+{
+	ModifierTypeInfo *mti = modifierType_getInfo(md->type);
+	BLI_assert(!dm || CustomData_has_layer(&dm->polyData, CD_NORMAL) == false);
+
+	if (dm && mti->dependsOnNormals && mti->dependsOnNormals(md)) {
+		DM_ensure_normals(dm);
+	}
+	mti->deformVertsEM(md, ob, em, dm, vertexCos, numVerts);
+}
+/* end modifier callback wrappers */
