@@ -34,18 +34,13 @@
 #include <fcntl.h>
 
 #ifndef WIN32
-#include <unistd.h>
+#  include <unistd.h>
 #else
-#include <io.h>
+#  include <io.h>
 #endif
 
 #include <time.h>
 
-#ifdef _WIN32
-#define open _open
-#define close _close
-#endif
-
 #include "MEM_guardedalloc.h"
 
 #include "DNA_constraint_types.h"
@@ -62,11 +57,9 @@
 #include "BLI_blenlib.h"
 #include "BLI_ghash.h"
 #include "BLI_math.h"
-#include "BLI_mempool.h"
 #include "BLI_threads.h"
 
 #include "BKE_animsys.h"
-#include "BKE_constraint.h"
 #include "BKE_colortools.h"
 #include "BKE_library.h"
 #include "BKE_global.h"
@@ -76,11 +69,14 @@
 #include "BKE_image.h"  /* openanim */
 #include "BKE_tracking.h"
 
-#include "IMB_colormanagement.h"
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"
 #include "IMB_moviecache.h"
 
+#ifdef WITH_OPENEXR
+#  include "intern/openexr/openexr_multi.h"
+#endif
+
 /*********************** movieclip buffer loaders *************************/
 
 static int sequence_guess_offset(const char *full_name, int head_len, unsigned short numlen)
@@ -170,7 +166,7 @@ static void get_sequence_fname(MovieClip *clip, int framenr, char *name)
 }
 
 /* supposed to work with sequences only */
-static void get_proxy_fname(MovieClip *clip, int proxy_render_size, int undistorted, int framenr, char *name)
+static void get_proxy_fname(MovieClip *clip, int proxy_render_size, bool undistorted, int framenr, char *name)
 {
 	int size = rendersize_to_number(proxy_render_size);
 	char dir[FILE_MAX], clipdir[FILE_MAX], clipfile[FILE_MAX];
@@ -200,7 +196,8 @@ static ImBuf *movieclip_load_sequence_file(MovieClip *clip, MovieClipUser *user,
 {
 	struct ImBuf *ibuf;
 	char name[FILE_MAX];
-	int loadflag, use_proxy = FALSE;
+	int loadflag;
+	bool use_proxy = false;
 	char *colorspace;
 
 	use_proxy = (flag & MCLIP_USE_PROXY) && user->render_size != MCLIP_PROXY_RENDER_SIZE_FULL;
@@ -208,19 +205,37 @@ static ImBuf *movieclip_load_sequence_file(MovieClip *clip, MovieClipUser *user,
 		int undistort = user->render_flag & MCLIP_PROXY_RENDER_UNDISTORT;
 		get_proxy_fname(clip, user->render_size, undistort, framenr, name);
 
-		/* proxies were built using default color space settings */
-		colorspace = NULL;
+		/* Well, this is a bit weird, but proxies for movie sources
+		 * are built in the same exact color space as the input,
+		 *
+		 * But image sequences are built in the display space.
+		 */
+		if (clip->source == MCLIP_SRC_MOVIE) {
+			colorspace = clip->colorspace_settings.name;
+		}
+		else {
+			colorspace = NULL;
+		}
 	}
 	else {
 		get_sequence_fname(clip, framenr, name);
 		colorspace = clip->colorspace_settings.name;
 	}
 
-	loadflag = IB_rect | IB_multilayer;
+	loadflag = IB_rect | IB_multilayer | IB_alphamode_detect;
 
 	/* read ibuf */
 	ibuf = IMB_loadiffname(name, loadflag, colorspace);
 
+#ifdef WITH_OPENEXR
+	if (ibuf) {
+		if (ibuf->ftype == OPENEXR && ibuf->userdata) {
+			IMB_exr_close(ibuf->userdata);
+			ibuf->userdata = NULL;
+		}
+	}
+#endif
+
 	return ibuf;
 }
 
@@ -255,17 +270,7 @@ static ImBuf *movieclip_load_movie_file(MovieClip *clip, MovieClipUser *user, in
 	movieclip_open_anim_file(clip);
 
 	if (clip->anim) {
-		int dur;
-		int fra;
-
-		dur = IMB_anim_get_duration(clip->anim, tc);
-		fra = framenr - clip->start_frame + clip->frame_offset;
-
-		if (fra < 0)
-			fra = 0;
-
-		if (fra > (dur - 1))
-			fra = dur - 1;
+		int fra = framenr - clip->start_frame + clip->frame_offset;
 
 		ibuf = IMB_anim_absolute(clip->anim, fra, tc, proxy);
 	}
@@ -283,7 +288,6 @@ static void movieclip_calc_length(MovieClip *clip)
 		}
 	}
 	else if (clip->source == MCLIP_SRC_SEQUENCE) {
-		int framenr = 1;
 		unsigned short numlen;
 		char name[FILE_MAX], head[FILE_MAX], tail[FILE_MAX];
 
@@ -291,18 +295,17 @@ static void movieclip_calc_length(MovieClip *clip)
 
 		if (numlen == 0) {
 			/* there's no number group in file name, assume it's single framed sequence */
-			clip->len = framenr + 1;
+			clip->len = 1;
 		}
 		else {
+			clip->len = 0;
 			for (;;) {
-				get_sequence_fname(clip, framenr, name);
+				get_sequence_fname(clip, clip->len + clip->start_frame, name);
 
-				if (!BLI_exists(name)) {
-					clip->len = framenr;
+				if (BLI_exists(name))
+					clip->len++;
+				else
 					break;
-				}
-
-				framenr++;
 			}
 		}
 	}
@@ -322,8 +325,10 @@ typedef struct MovieClipCache {
 
 		/* cache for undistorted shot */
 		float principal[2];
-		float k1, k2, k3;
-		short undistortion_used;
+		float polynomial_k1, polynomial_k2, polynomial_k3;
+		float division_k1, division_k2;
+		short distortion_model;
+		bool undistortion_used;
 
 		int proxy;
 		short render_flag;
@@ -341,6 +346,8 @@ typedef struct MovieClipCache {
 		int proxy, filter;
 		short render_flag;
 	} stabilized;
+
+	int sequence_offset;
 } MovieClipCache;
 
 typedef struct MovieClipImBufCacheKey {
@@ -353,6 +360,32 @@ typedef struct MovieClipCachePriorityData {
 	int framenr;
 } MovieClipCachePriorityData;
 
+static int user_frame_to_cache_frame(MovieClip *clip, int framenr)
+{
+	int index;
+
+	index = framenr - clip->start_frame + clip->frame_offset;
+
+	if (clip->source == MCLIP_SRC_SEQUENCE) {
+		if (clip->cache->sequence_offset == -1) {
+			unsigned short numlen;
+			char head[FILE_MAX], tail[FILE_MAX];
+
+			BLI_stringdec(clip->name, head, tail, &numlen);
+
+			/* see comment in get_sequence_fname */
+			clip->cache->sequence_offset = sequence_guess_offset(clip->name, strlen(head), numlen);
+		}
+
+		index += clip->cache->sequence_offset;
+	}
+
+	if (index < 0)
+		return framenr - index;
+
+	return framenr;
+}
+
 static void moviecache_keydata(void *userkey, int *framenr, int *proxy, int *render_flags)
 {
 	MovieClipImBufCacheKey *key = (MovieClipImBufCacheKey *)userkey;
@@ -370,27 +403,14 @@ static unsigned int moviecache_hashhash(const void *keyv)
 	return rval;
 }
 
-static int moviecache_hashcmp(const void *av, const void *bv)
+static bool moviecache_hashcmp(const void *av, const void *bv)
 {
 	const MovieClipImBufCacheKey *a = (MovieClipImBufCacheKey *)av;
 	const MovieClipImBufCacheKey *b = (MovieClipImBufCacheKey *)bv;
 
-	if (a->framenr < b->framenr)
-		return -1;
-	else if (a->framenr > b->framenr)
-		return 1;
-
-	if (a->proxy < b->proxy)
-		return -1;
-	else if (a->proxy > b->proxy)
-		return 1;
-
-	if (a->render_flag < b->render_flag)
-		return -1;
-	else if (a->render_flag > b->render_flag)
-		return 1;
-
-	return 0;
+	return ((a->framenr != b->framenr) ||
+	        (a->proxy != b->proxy) ||
+	        (a->render_flag != b->render_flag));
 }
 
 static void *moviecache_getprioritydata(void *key_v)
@@ -398,7 +418,7 @@ static void *moviecache_getprioritydata(void *key_v)
 	MovieClipImBufCacheKey *key = (MovieClipImBufCacheKey *) key_v;
 	MovieClipCachePriorityData *priority_data;
 
-	priority_data = MEM_callocN(sizeof(priority_data), "movie cache clip priority data");
+	priority_data = MEM_callocN(sizeof(*priority_data), "movie cache clip priority data");
 	priority_data->framenr = key->framenr;
 
 	return priority_data;
@@ -424,7 +444,7 @@ static ImBuf *get_imbuf_cache(MovieClip *clip, MovieClipUser *user, int flag)
 	if (clip->cache) {
 		MovieClipImBufCacheKey key;
 
-		key.framenr = user->framenr;
+		key.framenr = user_frame_to_cache_frame(clip, user->framenr);
 
 		if (flag & MCLIP_USE_PROXY) {
 			key.proxy = rendersize_to_proxy(user, flag);
@@ -441,7 +461,29 @@ static ImBuf *get_imbuf_cache(MovieClip *clip, MovieClipUser *user, int flag)
 	return NULL;
 }
 
-static void put_imbuf_cache(MovieClip *clip, MovieClipUser *user, ImBuf *ibuf, int flag)
+static bool has_imbuf_cache(MovieClip *clip, MovieClipUser *user, int flag)
+{
+	if (clip->cache) {
+		MovieClipImBufCacheKey key;
+
+		key.framenr = user_frame_to_cache_frame(clip, user->framenr);
+
+		if (flag & MCLIP_USE_PROXY) {
+			key.proxy = rendersize_to_proxy(user, flag);
+			key.render_flag = user->render_flag;
+		}
+		else {
+			key.proxy = IMB_PROXY_NONE;
+			key.render_flag = 0;
+		}
+
+		return IMB_moviecache_has_frame(clip->cache->moviecache, &key);
+	}
+
+	return false;
+}
+
+static bool put_imbuf_cache(MovieClip *clip, MovieClipUser *user, ImBuf *ibuf, int flag, bool destructive)
 {
 	MovieClipImBufCacheKey key;
 
@@ -460,9 +502,10 @@ static void put_imbuf_cache(MovieClip *clip, MovieClipUser *user, ImBuf *ibuf, i
 		                                     moviecache_prioritydeleter);
 
 		clip->cache->moviecache = moviecache;
+		clip->cache->sequence_offset = -1;
 	}
 
-	key.framenr = user->framenr;
+	key.framenr = user_frame_to_cache_frame(clip, user->framenr);
 
 	if (flag & MCLIP_USE_PROXY) {
 		key.proxy = rendersize_to_proxy(user, flag);
@@ -473,17 +516,32 @@ static void put_imbuf_cache(MovieClip *clip, MovieClipUser *user, ImBuf *ibuf, i
 		key.render_flag = 0;
 	}
 
-	IMB_moviecache_put(clip->cache->moviecache, &key, ibuf);
+	if (destructive) {
+		IMB_moviecache_put(clip->cache->moviecache, &key, ibuf);
+		return true;
+	}
+	else {
+		return IMB_moviecache_put_if_possible(clip->cache->moviecache, &key, ibuf);
+	}
+}
+
+static bool moviecache_check_free_proxy(ImBuf *UNUSED(ibuf),
+                                        void *userkey,
+                                        void *UNUSED(userdata))
+{
+	MovieClipImBufCacheKey *key = (MovieClipImBufCacheKey *)userkey;
+
+	return !(key->proxy == IMB_PROXY_NONE && key->render_flag == 0);
 }
 
 /*********************** common functions *************************/
 
 /* only image block itself */
-static MovieClip *movieclip_alloc(const char *name)
+static MovieClip *movieclip_alloc(Main *bmain, const char *name)
 {
 	MovieClip *clip;
 
-	clip = BKE_libblock_alloc(&G.main->movieclip, ID_MC, name);
+	clip = BKE_libblock_alloc(bmain, ID_MC, name);
 
 	clip->aspx = clip->aspy = 1.0f;
 
@@ -520,11 +578,29 @@ static void movieclip_load_get_szie(MovieClip *clip)
 	}
 }
 
+static void detect_clip_source(MovieClip *clip)
+{
+	ImBuf *ibuf;
+	char name[FILE_MAX];
+
+	BLI_strncpy(name, clip->name, sizeof(name));
+	BLI_path_abs(name, G.main->name);
+
+	ibuf = IMB_testiffname(name, IB_rect | IB_multilayer);
+	if (ibuf) {
+		clip->source = MCLIP_SRC_SEQUENCE;
+		IMB_freeImBuf(ibuf);
+	}
+	else {
+		clip->source = MCLIP_SRC_MOVIE;
+	}
+}
+
 /* checks if image was already loaded, then returns same image
  * otherwise creates new.
  * does not load ibuf itself
  * pass on optional frame for #name images */
-MovieClip *BKE_movieclip_file_add(const char *name)
+MovieClip *BKE_movieclip_file_add(Main *bmain, const char *name)
 {
 	MovieClip *clip;
 	int file, len;
@@ -532,7 +608,7 @@ MovieClip *BKE_movieclip_file_add(const char *name)
 	char str[FILE_MAX], strtest[FILE_MAX];
 
 	BLI_strncpy(str, name, sizeof(str));
-	BLI_path_abs(str, G.main->name);
+	BLI_path_abs(str, bmain->name);
 
 	/* exists? */
 	file = BLI_open(str, O_BINARY | O_RDONLY, 0);
@@ -541,7 +617,7 @@ MovieClip *BKE_movieclip_file_add(const char *name)
 	close(file);
 
 	/* ** first search an identical clip ** */
-	for (clip = G.main->movieclip.first; clip; clip = clip->id.next) {
+	for (clip = bmain->movieclip.first; clip; clip = clip->id.next) {
 		BLI_strncpy(strtest, clip->name, sizeof(clip->name));
 		BLI_path_abs(strtest, G.main->name);
 
@@ -562,13 +638,10 @@ MovieClip *BKE_movieclip_file_add(const char *name)
 		len--;
 	libname = name + len;
 
-	clip = movieclip_alloc(libname);
+	clip = movieclip_alloc(bmain, libname);
 	BLI_strncpy(clip->name, name, sizeof(clip->name));
 
-	if (BLI_testextensie_array(name, imb_ext_movie))
-		clip->source = MCLIP_SRC_MOVIE;
-	else
-		clip->source = MCLIP_SRC_SEQUENCE;
+	detect_clip_source(clip);
 
 	movieclip_load_get_szie(clip);
 	if (clip->lastsize[0]) {
@@ -616,11 +689,6 @@ static ImBuf *get_undistorted_ibuf(MovieClip *clip, struct MovieDistortion *dist
 	else
 		undistibuf = BKE_tracking_undistort_frame(&clip->tracking, ibuf, ibuf->x, ibuf->y, 0.0f);
 
-	if (undistibuf->userflags & IB_RECT_INVALID) {
-		ibuf->userflags &= ~IB_RECT_INVALID;
-		IMB_rect_from_float(undistibuf);
-	}
-
 	IMB_scaleImBuf(undistibuf, ibuf->x, ibuf->y);
 
 	return undistibuf;
@@ -646,19 +714,29 @@ static int need_postprocessed_frame(MovieClipUser *user, int postprocess_flag)
 	return result;
 }
 
-static int check_undistortion_cache_flags(MovieClip *clip)
+static bool check_undistortion_cache_flags(MovieClip *clip)
 {
 	MovieClipCache *cache = clip->cache;
 	MovieTrackingCamera *camera = &clip->tracking.camera;
 
 	/* check for distortion model changes */
-	if (!equals_v2v2(camera->principal, cache->postprocessed.principal))
-		return FALSE;
+	if (!equals_v2v2(camera->principal, cache->postprocessed.principal)) {
+		return false;
+	}
 
-	if (!equals_v3v3(&camera->k1, &cache->postprocessed.k1))
-		return FALSE;
+	if (camera->distortion_model != cache->postprocessed.distortion_model) {
+		return false;
+	}
 
-	return TRUE;
+	if (!equals_v3v3(&camera->k1, &cache->postprocessed.polynomial_k1)) {
+		return false;
+	}
+
+	if (!equals_v2v2(&camera->division_k1, &cache->postprocessed.division_k1)) {
+		return false;
+	}
+
+	return true;
 }
 
 static ImBuf *get_postprocessed_cached_frame(MovieClip *clip, MovieClipUser *user, int flag, int postprocess_flag)
@@ -700,12 +778,35 @@ static ImBuf *get_postprocessed_cached_frame(MovieClip *clip, MovieClipUser *use
 	return cache->postprocessed.ibuf;
 }
 
-static ImBuf *put_postprocessed_frame_to_cache(MovieClip *clip, MovieClipUser *user, ImBuf *ibuf,
+static ImBuf *postprocess_frame(MovieClip *clip, MovieClipUser *user, ImBuf *ibuf, int postprocess_flag)
+{
+	ImBuf *postproc_ibuf = NULL;
+
+	if (need_undistortion_postprocess(user)) {
+		postproc_ibuf = get_undistorted_ibuf(clip, NULL, ibuf);
+	}
+	else {
+		postproc_ibuf = IMB_dupImBuf(ibuf);
+	}
+
+	if (postprocess_flag) {
+		bool disable_red   = (postprocess_flag & MOVIECLIP_DISABLE_RED) != 0,
+			 disable_green = (postprocess_flag & MOVIECLIP_DISABLE_GREEN) != 0,
+			 disable_blue  = (postprocess_flag & MOVIECLIP_DISABLE_BLUE) != 0,
+			 grayscale     = (postprocess_flag & MOVIECLIP_PREVIEW_GRAYSCALE) != 0;
+
+		if (disable_red || disable_green || disable_blue || grayscale)
+			BKE_tracking_disable_channels(postproc_ibuf, disable_red, disable_green, disable_blue, 1);
+	}
+
+	return postproc_ibuf;
+}
+
+static void put_postprocessed_frame_to_cache(MovieClip *clip, MovieClipUser *user, ImBuf *ibuf,
                                                int flag, int postprocess_flag)
 {
 	MovieClipCache *cache = clip->cache;
 	MovieTrackingCamera *camera = &clip->tracking.camera;
-	ImBuf *postproc_ibuf = NULL;
 
 	cache->postprocessed.framenr = user->framenr;
 	cache->postprocessed.flag = postprocess_flag;
@@ -720,43 +821,30 @@ static ImBuf *put_postprocessed_frame_to_cache(MovieClip *clip, MovieClipUser *u
 	}
 
 	if (need_undistortion_postprocess(user)) {
+		cache->postprocessed.distortion_model = camera->distortion_model;
 		copy_v2_v2(cache->postprocessed.principal, camera->principal);
-		copy_v3_v3(&cache->postprocessed.k1, &camera->k1);
-		cache->postprocessed.undistortion_used = TRUE;
-		postproc_ibuf = get_undistorted_ibuf(clip, NULL, ibuf);
+		copy_v3_v3(&cache->postprocessed.polynomial_k1, &camera->k1);
+		copy_v2_v2(&cache->postprocessed.division_k1, &camera->division_k1);
+		cache->postprocessed.undistortion_used = true;
 	}
 	else {
-		cache->postprocessed.undistortion_used = FALSE;
+		cache->postprocessed.undistortion_used = false;
 	}
 
-	if (postprocess_flag) {
-		int disable_red   = postprocess_flag & MOVIECLIP_DISABLE_RED,
-		    disable_green = postprocess_flag & MOVIECLIP_DISABLE_GREEN,
-		    disable_blue  = postprocess_flag & MOVIECLIP_DISABLE_BLUE,
-		    grayscale     = postprocess_flag & MOVIECLIP_PREVIEW_GRAYSCALE;
-
-		if (!postproc_ibuf)
-			postproc_ibuf = IMB_dupImBuf(ibuf);
-
-		if (disable_red || disable_green || disable_blue || grayscale)
-			BKE_tracking_disable_channels(postproc_ibuf, disable_red, disable_green, disable_blue, 1);
-	}
-
-	IMB_refImBuf(postproc_ibuf);
+	IMB_refImBuf(ibuf);
 
 	if (cache->postprocessed.ibuf)
 		IMB_freeImBuf(cache->postprocessed.ibuf);
 
-	cache->postprocessed.ibuf = postproc_ibuf;
-
-	return postproc_ibuf;
+	cache->postprocessed.ibuf = ibuf;
 }
 
 static ImBuf *movieclip_get_postprocessed_ibuf(MovieClip *clip, MovieClipUser *user, int flag,
                                                int postprocess_flag, int cache_flag)
 {
 	ImBuf *ibuf = NULL;
-	int framenr = user->framenr, need_postprocess = FALSE;
+	int framenr = user->framenr;
+	bool need_postprocess = false;
 
 	/* cache isn't threadsafe itself and also loading of movies
 	 * can't happen from concurent threads that's why we use lock here */
@@ -767,14 +855,14 @@ static ImBuf *movieclip_get_postprocessed_ibuf(MovieClip *clip, MovieClipUser *u
 		ibuf = get_postprocessed_cached_frame(clip, user, flag, postprocess_flag);
 
 		if (!ibuf)
-			need_postprocess = TRUE;
+			need_postprocess = true;
 	}
 
 	if (!ibuf)
 		ibuf = get_imbuf_cache(clip, user, flag);
 
 	if (!ibuf) {
-		int use_sequence = FALSE;
+		bool use_sequence = false;
 
 		/* undistorted proxies for movies should be read as image sequence */
 		use_sequence = (user->render_flag & MCLIP_PROXY_RENDER_UNDISTORT) &&
@@ -788,18 +876,21 @@ static ImBuf *movieclip_get_postprocessed_ibuf(MovieClip *clip, MovieClipUser *u
 		}
 
 		if (ibuf && (cache_flag & MOVIECLIP_CACHE_SKIP) == 0)
-			put_imbuf_cache(clip, user, ibuf, flag);
+			put_imbuf_cache(clip, user, ibuf, flag, true);
 	}
 
 	if (ibuf) {
 		clip->lastframe = framenr;
 		real_ibuf_size(clip, user, ibuf, &clip->lastsize[0], &clip->lastsize[1]);
 
-		/* postprocess frame and put to cache */
+		/* postprocess frame and put to cache if needed*/
 		if (need_postprocess) {
 			ImBuf *tmpibuf = ibuf;
-			ibuf = put_postprocessed_frame_to_cache(clip, user, tmpibuf, flag, postprocess_flag);
+			ibuf = postprocess_frame(clip, user, tmpibuf, postprocess_flag);
 			IMB_freeImBuf(tmpibuf);
+			if (ibuf && (cache_flag & MOVIECLIP_CACHE_SKIP) == 0) {
+				put_postprocessed_frame_to_cache(clip, user, ibuf, flag, postprocess_flag);
+			}
 		}
 	}
 
@@ -890,6 +981,7 @@ static ImBuf *put_stabilized_frame_to_cache(MovieClip *clip, MovieClipUser *user
 
 	copy_v2_v2(cache->stabilized.loc, tloc);
 
+	cache->stabilized.reference_ibuf = ibuf;
 	cache->stabilized.scale = tscale;
 	cache->stabilized.angle = tangle;
 	cache->stabilized.framenr = framenr;
@@ -967,16 +1059,16 @@ ImBuf *BKE_movieclip_get_stable_ibuf(MovieClip *clip, MovieClipUser *user, float
 
 }
 
-int BKE_movieclip_has_frame(MovieClip *clip, MovieClipUser *user)
+bool BKE_movieclip_has_frame(MovieClip *clip, MovieClipUser *user)
 {
 	ImBuf *ibuf = BKE_movieclip_get_ibuf(clip, user);
 
 	if (ibuf) {
 		IMB_freeImBuf(ibuf);
-		return TRUE;
+		return true;
 	}
 
-	return FALSE;
+	return false;
 }
 
 void BKE_movieclip_get_size(MovieClip *clip, MovieClipUser *user, int *width, int *height)
@@ -1079,23 +1171,46 @@ static void free_buffers(MovieClip *clip)
 	BKE_free_animdata((ID *) clip);
 }
 
+void BKE_movieclip_clear_cache(MovieClip *clip)
+{
+	free_buffers(clip);
+}
+
+void BKE_movieclip_clear_proxy_cache(MovieClip *clip)
+{
+	if (clip->cache && clip->cache->moviecache) {
+		IMB_moviecache_cleanup(clip->cache->moviecache,
+		                       moviecache_check_free_proxy,
+		                       NULL);
+	}
+}
+
 void BKE_movieclip_reload(MovieClip *clip)
 {
 	/* clear cache */
 	free_buffers(clip);
 
-	clip->tracking.stabilization.ok = FALSE;
+	clip->tracking.stabilization.ok = false;
 
 	/* update clip source */
-	if (BLI_testextensie_array(clip->name, imb_ext_movie))
-		clip->source = MCLIP_SRC_MOVIE;
-	else
-		clip->source = MCLIP_SRC_SEQUENCE;
+	detect_clip_source(clip);
 
 	clip->lastsize[0] = clip->lastsize[1] = 0;
 	movieclip_load_get_szie(clip);
 
 	movieclip_calc_length(clip);
+
+	/* same as for image update -- don't use notifiers because they are not 100% sure to succeeded
+	 * (node trees which are not currently visible wouldn't be refreshed)
+	 */
+	{
+		Scene *scene;
+		for (scene = G.main->scene.first; scene; scene = scene->id.next) {
+			if (scene->nodetree) {
+				nodeUpdateID(scene->nodetree, &clip->id);
+			}
+		}
+	}
 }
 
 void BKE_movieclip_update_scopes(MovieClip *clip, MovieClipUser *user, MovieClipScopes *scopes)
@@ -1115,7 +1230,7 @@ void BKE_movieclip_update_scopes(MovieClip *clip, MovieClipUser *user, MovieClip
 
 	scopes->marker = NULL;
 	scopes->track = NULL;
-	scopes->track_locked = TRUE;
+	scopes->track_locked = true;
 
 	if (clip) {
 		MovieTrackingTrack *act_track = BKE_tracking_track_get_active(&clip->tracking);
@@ -1129,15 +1244,14 @@ void BKE_movieclip_update_scopes(MovieClip *clip, MovieClipUser *user, MovieClip
 			scopes->track = track;
 
 			if (marker->flag & MARKER_DISABLED) {
-				scopes->track_disabled = TRUE;
+				scopes->track_disabled = true;
 			}
 			else {
 				ImBuf *ibuf = BKE_movieclip_get_ibuf(clip, user);
 
-				scopes->track_disabled = FALSE;
+				scopes->track_disabled = false;
 
 				if (ibuf && (ibuf->rect || ibuf->rect_float)) {
-					ImBuf *search_ibuf;
 					MovieTrackingMarker undist_marker = *marker;
 
 					if (user->render_flag & MCLIP_PROXY_RENDER_UNDISTORT) {
@@ -1155,20 +1269,14 @@ void BKE_movieclip_update_scopes(MovieClip *clip, MovieClipUser *user, MovieClip
 						undist_marker.pos[1] /= height * aspy;
 					}
 
-					search_ibuf = BKE_tracking_get_search_imbuf(ibuf, track, &undist_marker, TRUE, TRUE);
-
-					if (!search_ibuf->rect_float) {
-						/* sampling happens in float buffer */
-						IMB_float_from_rect(search_ibuf);
-					}
+					scopes->track_search = BKE_tracking_get_search_imbuf(ibuf, track, &undist_marker, true, true);
 
 					scopes->undist_marker = undist_marker;
-					scopes->track_search = search_ibuf;
 
 					scopes->frame_width = ibuf->x;
 					scopes->frame_height = ibuf->y;
 
-					scopes->use_track_mask = track->flag & TRACK_PREVIEW_ALPHA;
+					scopes->use_track_mask = (track->flag & TRACK_PREVIEW_ALPHA) != 0;
 				}
 
 				IMB_freeImBuf(ibuf);
@@ -1177,7 +1285,7 @@ void BKE_movieclip_update_scopes(MovieClip *clip, MovieClipUser *user, MovieClip
 			if ((track->flag & TRACK_LOCKED) == 0) {
 				float pat_min[2], pat_max[2];
 
-				scopes->track_locked = FALSE;
+				scopes->track_locked = false;
 
 				/* XXX: would work fine with non-transformed patterns, but would likely fail
 				 *      with transformed patterns, but that would be easier to debug when
@@ -1191,10 +1299,10 @@ void BKE_movieclip_update_scopes(MovieClip *clip, MovieClipUser *user, MovieClip
 	}
 
 	scopes->framenr = user->framenr;
-	scopes->ok = TRUE;
+	scopes->ok = true;
 }
 
-static void movieclip_build_proxy_ibuf(MovieClip *clip, ImBuf *ibuf, int cfra, int proxy_render_size, int undistorted)
+static void movieclip_build_proxy_ibuf(MovieClip *clip, ImBuf *ibuf, int cfra, int proxy_render_size, bool undistorted, bool threaded)
 {
 	char name[FILE_MAX];
 	int quality, rectx, recty;
@@ -1208,7 +1316,10 @@ static void movieclip_build_proxy_ibuf(MovieClip *clip, ImBuf *ibuf, int cfra, i
 
 	scaleibuf = IMB_dupImBuf(ibuf);
 
-	IMB_scaleImBuf(scaleibuf, (short)rectx, (short)recty);
+	if (threaded)
+		IMB_scaleImBuf_threaded(scaleibuf, (short)rectx, (short)recty);
+	else
+		IMB_scaleImBuf(scaleibuf, (short)rectx, (short)recty);
 
 	quality = clip->proxy.quality;
 	scaleibuf->ftype = JPG | quality;
@@ -1217,6 +1328,10 @@ static void movieclip_build_proxy_ibuf(MovieClip *clip, ImBuf *ibuf, int cfra, i
 	if (scaleibuf->planes == 32)
 		scaleibuf->planes = 24;
 
+	/* TODO: currently the most weak part of multithreaded proxies,
+	 *       could be solved in a way that thread only prepares memory
+	 *       buffer and write to disk happens separately
+	 */
 	BLI_lock_thread(LOCK_MOVIECLIP);
 
 	BLI_make_existing_file(name);
@@ -1228,12 +1343,18 @@ static void movieclip_build_proxy_ibuf(MovieClip *clip, ImBuf *ibuf, int cfra, i
 	IMB_freeImBuf(scaleibuf);
 }
 
+/* note: currently used by proxy job for movies, threading happens within single frame
+ * (meaning scaling shall be threaded)
+ */
 void BKE_movieclip_build_proxy_frame(MovieClip *clip, int clip_flag, struct MovieDistortion *distortion,
-                                     int cfra, int *build_sizes, int build_count, int undistorted)
+                                     int cfra, int *build_sizes, int build_count, bool undistorted)
 {
 	ImBuf *ibuf;
 	MovieClipUser user;
 
+	if (!build_count)
+		return;
+
 	user.framenr = cfra;
 	user.render_flag = 0;
 	user.render_size = MCLIP_PROXY_RENDER_SIZE_FULL;
@@ -1248,7 +1369,7 @@ void BKE_movieclip_build_proxy_frame(MovieClip *clip, int clip_flag, struct Movi
 			tmpibuf = get_undistorted_ibuf(clip, distortion, ibuf);
 
 		for (i = 0; i < build_count; i++)
-			movieclip_build_proxy_ibuf(clip, tmpibuf, cfra, build_sizes[i], undistorted);
+			movieclip_build_proxy_ibuf(clip, tmpibuf, cfra, build_sizes[i], undistorted, true);
 
 		IMB_freeImBuf(ibuf);
 
@@ -1257,6 +1378,30 @@ void BKE_movieclip_build_proxy_frame(MovieClip *clip, int clip_flag, struct Movi
 	}
 }
 
+/* note: currently used by proxy job for sequences, threading happens within sequence
+ * (different threads handles different frames, no threading within frame is needed)
+ */
+void BKE_movieclip_build_proxy_frame_for_ibuf(MovieClip *clip, ImBuf *ibuf, struct MovieDistortion *distortion,
+                                              int cfra, int *build_sizes, int build_count, bool undistorted)
+{
+	if (!build_count)
+		return;
+
+	if (ibuf) {
+		ImBuf *tmpibuf = ibuf;
+		int i;
+
+		if (undistorted)
+			tmpibuf = get_undistorted_ibuf(clip, distortion, ibuf);
+
+		for (i = 0; i < build_count; i++)
+			movieclip_build_proxy_ibuf(clip, tmpibuf, cfra, build_sizes[i], undistorted, false);
+
+		if (tmpibuf != ibuf)
+			IMB_freeImBuf(tmpibuf);
+	}
+}
+
 void BKE_movieclip_free(MovieClip *clip)
 {
 	free_buffers(clip);
@@ -1303,21 +1448,19 @@ void BKE_movieclip_unlink(Main *bmain, MovieClip *clip)
 		bConstraint *con;
 
 		for (con = ob->constraints.first; con; con = con->next) {
-			bConstraintTypeInfo *cti = constraint_get_typeinfo(con);
-
-			if (cti->type == CONSTRAINT_TYPE_FOLLOWTRACK) {
+			if (con->type == CONSTRAINT_TYPE_FOLLOWTRACK) {
 				bFollowTrackConstraint *data = (bFollowTrackConstraint *) con->data;
 
 				if (data->clip == clip)
 					data->clip = NULL;
 			}
-			else if (cti->type == CONSTRAINT_TYPE_CAMERASOLVER) {
+			else if (con->type == CONSTRAINT_TYPE_CAMERASOLVER) {
 				bCameraSolverConstraint *data = (bCameraSolverConstraint *) con->data;
 
 				if (data->clip == clip)
 					data->clip = NULL;
 			}
-			else if (cti->type == CONSTRAINT_TYPE_OBJECTSOLVER) {
+			else if (con->type == CONSTRAINT_TYPE_OBJECTSOLVER) {
 				bObjectSolverConstraint *data = (bObjectSolverConstraint *) con->data;
 
 				if (data->clip == clip)
@@ -1326,10 +1469,9 @@ void BKE_movieclip_unlink(Main *bmain, MovieClip *clip)
 		}
 	}
 
-	{
-		bNodeTreeType *treetype = ntreeGetType(NTREE_COMPOSIT);
-		treetype->foreach_nodetree(bmain, (void *)clip, &BKE_node_tree_unlink_id_cb);
-	}
+	FOREACH_NODETREE(bmain, ntree, id) {
+		BKE_node_tree_unlink_id((ID *)clip, ntree);
+	} FOREACH_NODETREE_END
 
 	clip->id.us = 0;
 }
@@ -1343,3 +1485,59 @@ float BKE_movieclip_remap_clip_to_scene_frame(MovieClip *clip, float framenr)
 {
 	return framenr + (float) clip->start_frame - 1.0f;
 }
+
+void BKE_movieclip_filename_for_frame(MovieClip *clip, MovieClipUser *user, char *name)
+{
+	if (clip->source == MCLIP_SRC_SEQUENCE) {
+		int use_proxy;
+
+		use_proxy = (clip->flag & MCLIP_USE_PROXY) && user->render_size != MCLIP_PROXY_RENDER_SIZE_FULL;
+
+		if (use_proxy) {
+			int undistort = user->render_flag & MCLIP_PROXY_RENDER_UNDISTORT;
+			get_proxy_fname(clip, user->render_size, undistort, user->framenr, name);
+		}
+		else {
+			get_sequence_fname(clip, user->framenr, name);
+		}
+	}
+	else {
+		BLI_strncpy(name, clip->name, FILE_MAX);
+		BLI_path_abs(name, ID_BLEND_PATH(G.main, &clip->id));
+	}
+}
+
+ImBuf *BKE_movieclip_anim_ibuf_for_frame(MovieClip *clip, MovieClipUser *user)
+{
+	ImBuf *ibuf = NULL;
+
+	if (clip->source == MCLIP_SRC_MOVIE) {
+		BLI_lock_thread(LOCK_MOVIECLIP);
+		ibuf = movieclip_load_movie_file(clip, user, user->framenr, clip->flag);
+		BLI_unlock_thread(LOCK_MOVIECLIP);
+	}
+
+	return ibuf;
+}
+
+bool BKE_movieclip_has_cached_frame(MovieClip *clip, MovieClipUser *user)
+{
+	bool has_frame = false;
+
+	BLI_lock_thread(LOCK_MOVIECLIP);
+	has_frame = has_imbuf_cache(clip, user, clip->flag);
+	BLI_unlock_thread(LOCK_MOVIECLIP);
+
+	return has_frame;
+}
+
+bool BKE_movieclip_put_frame_if_possible(MovieClip *clip, MovieClipUser *user, ImBuf *ibuf)
+{
+	bool result;
+
+	BLI_lock_thread(LOCK_MOVIECLIP);
+	result = put_imbuf_cache(clip, user, ibuf, clip->flag, false);
+	BLI_unlock_thread(LOCK_MOVIECLIP);
+
+	return result;
+}
