@@ -45,6 +45,8 @@
 #include "BLI_string.h"
 #include "BLI_ghash.h"
 
+#include "BLF_translation.h"
+
 #include "DNA_anim_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_sound_types.h"
@@ -150,13 +152,17 @@ void free_nladata(ListBase *tracks)
 	}
 	
 	/* clear the list's pointers to be safe */
-	tracks->first = tracks->last = NULL;
+	BLI_listbase_clear(tracks);
 }
 
 /* Copying ------------------------------------------- */
 
-/* Copy NLA strip */
-NlaStrip *copy_nlastrip(NlaStrip *strip)
+/**
+ * Copy NLA strip
+ *
+ * \param use_same_action When true, the existing action is used (instead of being duplicated)
+ */
+NlaStrip *copy_nlastrip(NlaStrip *strip, const bool use_same_action)
 {
 	NlaStrip *strip_d;
 	NlaStrip *cs, *cs_d;
@@ -169,19 +175,27 @@ NlaStrip *copy_nlastrip(NlaStrip *strip)
 	strip_d = MEM_dupallocN(strip);
 	strip_d->next = strip_d->prev = NULL;
 	
-	/* increase user-count of action */
-	if (strip_d->act)
-		id_us_plus(&strip_d->act->id);
+	/* handle action */
+	if (strip_d->act) {
+		if (use_same_action) {
+			/* increase user-count of action */
+			id_us_plus(&strip_d->act->id);
+		}
+		else {
+			/* use a copy of the action instead (user count shouldn't have changed yet) */
+			strip_d->act = BKE_action_copy(strip_d->act);
+		}
+	}
 		
 	/* copy F-Curves and modifiers */
 	copy_fcurves(&strip_d->fcurves, &strip->fcurves);
 	copy_fmodifiers(&strip_d->modifiers, &strip->modifiers);
 	
 	/* make a copy of all the child-strips, one at a time */
-	strip_d->strips.first = strip_d->strips.last = NULL;
+	BLI_listbase_clear(&strip_d->strips);
 	
 	for (cs = strip->strips.first; cs; cs = cs->next) {
-		cs_d = copy_nlastrip(cs);
+		cs_d = copy_nlastrip(cs, use_same_action);
 		BLI_addtail(&strip_d->strips, cs_d);
 	}
 	
@@ -190,7 +204,7 @@ NlaStrip *copy_nlastrip(NlaStrip *strip)
 }
 
 /* Copy NLA Track */
-NlaTrack *copy_nlatrack(NlaTrack *nlt)
+NlaTrack *copy_nlatrack(NlaTrack *nlt, const bool use_same_actions)
 {
 	NlaStrip *strip, *strip_d;
 	NlaTrack *nlt_d;
@@ -204,10 +218,10 @@ NlaTrack *copy_nlatrack(NlaTrack *nlt)
 	nlt_d->next = nlt_d->prev = NULL;
 	
 	/* make a copy of all the strips, one at a time */
-	nlt_d->strips.first = nlt_d->strips.last = NULL;
+	BLI_listbase_clear(&nlt_d->strips);
 	
 	for (strip = nlt->strips.first; strip; strip = strip->next) {
-		strip_d = copy_nlastrip(strip);
+		strip_d = copy_nlastrip(strip, use_same_actions);
 		BLI_addtail(&nlt_d->strips, strip_d);
 	}
 	
@@ -225,12 +239,13 @@ void copy_nladata(ListBase *dst, ListBase *src)
 		return;
 		
 	/* clear out the destination list first for precautions... */
-	dst->first = dst->last = NULL;
+	BLI_listbase_clear(dst);
 		
 	/* copy each NLA-track, one at a time */
 	for (nlt = src->first; nlt; nlt = nlt->next) {
 		/* make a copy, and add the copy to the destination list */
-		nlt_d = copy_nlatrack(nlt);
+		// XXX: we need to fix this sometime
+		nlt_d = copy_nlatrack(nlt, true);
 		BLI_addtail(dst, nlt_d);
 	}
 }
@@ -253,7 +268,7 @@ NlaTrack *add_nlatrack(AnimData *adt, NlaTrack *prev)
 	
 	/* set settings requiring the track to not be part of the stack yet */
 	nlt->flag = NLATRACK_SELECTED;
-	nlt->index = BLI_countlist(&adt->nla_tracks);
+	nlt->index = BLI_listbase_count(&adt->nla_tracks);
 	
 	/* add track to stack, and make it the active one */
 	if (prev)
@@ -264,7 +279,7 @@ NlaTrack *add_nlatrack(AnimData *adt, NlaTrack *prev)
 	
 	/* must have unique name, but we need to seed this */
 	strcpy(nlt->name, "NlaTrack");
-	BLI_uniquename(&adt->nla_tracks, nlt, "NlaTrack", '.', offsetof(NlaTrack, name), sizeof(nlt->name));
+	BLI_uniquename(&adt->nla_tracks, nlt, DATA_("NlaTrack"), '.', offsetof(NlaTrack, name), sizeof(nlt->name));
 	
 	/* return the new track */
 	return nlt;
@@ -415,7 +430,7 @@ static float nlastrip_get_frame_actionclip(NlaStrip *strip, float cframe, short
 			return (strip->end + (strip->actstart * scale - cframe)) / scale;
 		}
 		else { /* if (mode == NLATIME_CONVERT_EVAL) */
-			if (IS_EQF(cframe, strip->end) && IS_EQF(strip->repeat, ((int)strip->repeat))) {
+			if (IS_EQF((float)cframe, strip->end) && IS_EQF(strip->repeat, floorf(strip->repeat))) {
 				/* this case prevents the motion snapping back to the first frame at the end of the strip 
 				 * by catching the case where repeats is a whole number, which means that the end of the strip
 				 * could also be interpreted as the end of the start of a repeat
@@ -423,7 +438,7 @@ static float nlastrip_get_frame_actionclip(NlaStrip *strip, float cframe, short
 				return strip->actstart;
 			}
 			else {
-				/* - the 'fmod(..., actlength*scale)' is needed to get the repeats working
+				/* - the 'fmod(..., actlength * scale)' is needed to get the repeats working
 				 * - the '/ scale' is needed to ensure that scaling influences the timing within the repeat
 				 */
 				return strip->actend - fmodf(cframe - strip->start, actlength * scale) / scale;
@@ -438,7 +453,7 @@ static float nlastrip_get_frame_actionclip(NlaStrip *strip, float cframe, short
 			return strip->actstart + (cframe - strip->start) / scale;
 		}
 		else { /* if (mode == NLATIME_CONVERT_EVAL) */
-			if (IS_EQF(cframe, strip->end) && IS_EQF(strip->repeat, ((int)strip->repeat))) {
+			if (IS_EQF(cframe, strip->end) && IS_EQF(strip->repeat, floorf(strip->repeat))) {
 				/* this case prevents the motion snapping back to the first frame at the end of the strip 
 				 * by catching the case where repeats is a whole number, which means that the end of the strip
 				 * could also be interpreted as the end of the start of a repeat
@@ -446,7 +461,7 @@ static float nlastrip_get_frame_actionclip(NlaStrip *strip, float cframe, short
 				return strip->actend;
 			}
 			else {
-				/* - the 'fmod(..., actlength*scale)' is needed to get the repeats working
+				/* - the 'fmod(..., actlength * scale)' is needed to get the repeats working
 				 * - the '/ scale' is needed to ensure that scaling influences the timing within the repeat
 				 */
 				return strip->actstart + fmodf(cframe - strip->start, actlength * scale) / scale;
@@ -496,12 +511,12 @@ float nlastrip_get_frame(NlaStrip *strip, float cframe, short mode)
 		case NLASTRIP_TYPE_CLIP: /* action-clip (default) */
 		default:
 			return nlastrip_get_frame_actionclip(strip, cframe, mode);
-	}	
+	}
 }
 
 
 /* Non clipped mapping for strip-time <-> global time
- *	mode = eNlaTime_ConvertModesp[] -> NLATIME_CONVERT_*
+ *	mode = eNlaTime_ConvertModes -> NLATIME_CONVERT_*
  *
  * Public API method - perform this mapping using the given AnimData block
  * and perform any necessary sanity checks on the value
@@ -546,13 +561,13 @@ float BKE_nla_tweakedit_remap(AnimData *adt, float cframe, short mode)
 /* (these functions are used for NLA-Tracks and also for nested/meta-strips) */
 
 /* Check if there is any space in the given list to add the given strip */
-short BKE_nlastrips_has_space(ListBase *strips, float start, float end)
+bool BKE_nlastrips_has_space(ListBase *strips, float start, float end)
 {
 	NlaStrip *strip;
 	
 	/* sanity checks */
 	if ((strips == NULL) || IS_EQF(start, end))
-		return 0;
+		return false;
 	if (start > end) {
 		puts("BKE_nlastrips_has_space() error... start and end arguments swapped");
 		SWAP(float, start, end);
@@ -564,17 +579,17 @@ short BKE_nlastrips_has_space(ListBase *strips, float start, float end)
 		 * we've gone past the window we need to check for, so things are fine
 		 */
 		if (strip->start >= end)
-			return 1;
+			return true;
 		
 		/* if the end of the strip is greater than either of the boundaries, the range
 		 * must fall within the extents of the strip
 		 */
 		if ((strip->end > start) || (strip->end > end))
-			return 0;
+			return false;
 	}
 	
 	/* if we are still here, we haven't encountered any overlapping strips */
-	return 1;
+	return true;
 }
 
 /* Rearrange the strips in the track so that they are always in order 
@@ -624,18 +639,18 @@ void BKE_nlastrips_sort_strips(ListBase *strips)
 /* Add the given NLA-Strip to the given list of strips, assuming that it 
  * isn't currently a member of another list
  */
-short BKE_nlastrips_add_strip(ListBase *strips, NlaStrip *strip)
+bool BKE_nlastrips_add_strip(ListBase *strips, NlaStrip *strip)
 {
 	NlaStrip *ns;
-	short not_added = 1;
+	bool not_added = true;
 	
 	/* sanity checks */
 	if (ELEM(NULL, strips, strip))
-		return 0;
+		return false;
 		
 	/* check if any space to add */
 	if (BKE_nlastrips_has_space(strips, strip->start, strip->end) == 0)
-		return 0;
+		return false;
 	
 	/* find the right place to add the strip to the nominated track */
 	for (ns = strips->first; ns; ns = ns->next) {
@@ -652,7 +667,7 @@ short BKE_nlastrips_add_strip(ListBase *strips, NlaStrip *strip)
 	}
 	
 	/* added... */
-	return 1;
+	return true;
 }
 
 
@@ -662,7 +677,7 @@ short BKE_nlastrips_add_strip(ListBase *strips, NlaStrip *strip)
  * contained within 'Meta-Strips' which act as strips which contain strips.
  *	temp: are the meta-strips to be created 'temporary' ones used for transforms?
  */
-void BKE_nlastrips_make_metas(ListBase *strips, short temp)
+void BKE_nlastrips_make_metas(ListBase *strips, bool is_temp)
 {
 	NlaStrip *mstrip = NULL;
 	NlaStrip *strip, *stripn;
@@ -687,7 +702,7 @@ void BKE_nlastrips_make_metas(ListBase *strips, short temp)
 				mstrip->flag = NLASTRIP_FLAG_SELECT;
 				
 				/* set temp flag if appropriate (i.e. for transform-type editing) */
-				if (temp)
+				if (is_temp)
 					mstrip->flag |= NLASTRIP_FLAG_TEMP_META;
 					
 				/* set default repeat/scale values to prevent warnings */
@@ -739,7 +754,7 @@ void BKE_nlastrips_clear_metastrip(ListBase *strips, NlaStrip *strip)
  *	sel: only consider selected meta-strips, otherwise all meta-strips are removed
  *	onlyTemp: only remove the 'temporary' meta-strips used for transforms
  */
-void BKE_nlastrips_clear_metas(ListBase *strips, short onlySel, short onlyTemp)
+void BKE_nlastrips_clear_metas(ListBase *strips, bool only_sel, bool only_temp)
 {
 	NlaStrip *strip, *stripn;
 	
@@ -754,8 +769,8 @@ void BKE_nlastrips_clear_metas(ListBase *strips, short onlySel, short onlyTemp)
 		/* check if strip is a meta-strip */
 		if (strip->type == NLASTRIP_TYPE_META) {
 			/* if check if selection and 'temporary-only' considerations are met */
-			if ((onlySel == 0) || (strip->flag & NLASTRIP_FLAG_SELECT)) {
-				if ((!onlyTemp) || (strip->flag & NLASTRIP_FLAG_TEMP_META)) {
+			if ((!only_sel) || (strip->flag & NLASTRIP_FLAG_SELECT)) {
+				if ((!only_temp) || (strip->flag & NLASTRIP_FLAG_TEMP_META)) {
 					BKE_nlastrips_clear_metastrip(strips, strip);
 				}
 			}
@@ -766,15 +781,15 @@ void BKE_nlastrips_clear_metas(ListBase *strips, short onlySel, short onlyTemp)
 /* Add the given NLA-Strip to the given Meta-Strip, assuming that the
  * strip isn't attached to any list of strips
  */
-short BKE_nlameta_add_strip(NlaStrip *mstrip, NlaStrip *strip)
+bool BKE_nlameta_add_strip(NlaStrip *mstrip, NlaStrip *strip)
 {
 	/* sanity checks */
 	if (ELEM(NULL, mstrip, strip))
-		return 0;
+		return false;
 		
 	/* firstly, check if the meta-strip has space for this */
 	if (BKE_nlastrips_has_space(&mstrip->strips, strip->start, strip->end) == 0)
-		return 0;
+		return false;
 		
 	/* check if this would need to be added to the ends of the meta,
 	 * and subsequently, if the neighboring strips allow us enough room
@@ -788,10 +803,10 @@ short BKE_nlameta_add_strip(NlaStrip *mstrip, NlaStrip *strip)
 			BLI_addhead(&mstrip->strips, strip);
 			mstrip->start = strip->start;
 			
-			return 1;
+			return true;
 		}
 		else /* failed... no room before */
-			return 0;
+			return false;
 	}
 	else if (strip->end > mstrip->end) {
 		/* check if strip to the right (if it exists) starts before the 
@@ -802,10 +817,10 @@ short BKE_nlameta_add_strip(NlaStrip *mstrip, NlaStrip *strip)
 			BLI_addtail(&mstrip->strips, strip);
 			mstrip->end = strip->end;
 			
-			return 1;
+			return true;
 		}
 		else /* failed... no room after */
-			return 0;
+			return false;
 	}
 	else {
 		/* just try to add to the meta-strip (no dimension changes needed) */
@@ -965,7 +980,7 @@ void BKE_nlatrack_set_active(ListBase *tracks, NlaTrack *nlt_a)
 }
 
 /* Check if there is any space in the given track to add a strip of the given length */
-short BKE_nlatrack_has_space(NlaTrack *nlt, float start, float end)
+bool BKE_nlatrack_has_space(NlaTrack *nlt, float start, float end)
 {
 	/* sanity checks 
 	 *  - track must exist
@@ -973,7 +988,7 @@ short BKE_nlatrack_has_space(NlaTrack *nlt, float start, float end)
 	 *  - bounds cannot be equal (0-length is nasty)
 	 */
 	if ((nlt == NULL) || (nlt->flag & NLATRACK_PROTECTED) || IS_EQF(start, end))
-		return 0;
+		return false;
 	
 	if (start > end) {
 		puts("BKE_nlatrack_has_space() error... start and end arguments swapped");
@@ -1000,11 +1015,11 @@ void BKE_nlatrack_sort_strips(NlaTrack *nlt)
 /* Add the given NLA-Strip to the given NLA-Track, assuming that it 
  * isn't currently attached to another one 
  */
-short BKE_nlatrack_add_strip(NlaTrack *nlt, NlaStrip *strip)
+bool BKE_nlatrack_add_strip(NlaTrack *nlt, NlaStrip *strip)
 {
 	/* sanity checks */
 	if (ELEM(NULL, nlt, strip))
-		return 0;
+		return false;
 		
 	/* try to add the strip to the track using a more generic function */
 	return BKE_nlastrips_add_strip(&nlt->strips, strip);
@@ -1013,7 +1028,7 @@ short BKE_nlatrack_add_strip(NlaTrack *nlt, NlaStrip *strip)
 /* Get the extents of the given NLA-Track including gaps between strips,
  * returning whether this succeeded or not
  */
-short BKE_nlatrack_get_bounds(NlaTrack *nlt, float bounds[2])
+bool BKE_nlatrack_get_bounds(NlaTrack *nlt, float bounds[2])
 {
 	NlaStrip *strip;
 	
@@ -1021,11 +1036,11 @@ short BKE_nlatrack_get_bounds(NlaTrack *nlt, float bounds[2])
 	if (bounds)
 		bounds[0] = bounds[1] = 0.0f;
 	else
-		return 0;
+		return false;
 	
 	/* sanity checks */
 	if (ELEM(NULL, nlt, nlt->strips.first))
-		return 0;
+		return false;
 		
 	/* lower bound is first strip's start frame */
 	strip = nlt->strips.first;
@@ -1036,7 +1051,7 @@ short BKE_nlatrack_get_bounds(NlaTrack *nlt, float bounds[2])
 	bounds[1] = strip->end;
 	
 	/* done */
-	return 1;
+	return true;
 }
 
 /* NLA Strips -------------------------------------- */
@@ -1083,14 +1098,14 @@ void BKE_nlastrip_set_active(AnimData *adt, NlaStrip *strip)
 
 
 /* Does the given NLA-strip fall within the given bounds (times)? */
-short BKE_nlastrip_within_bounds(NlaStrip *strip, float min, float max)
+bool BKE_nlastrip_within_bounds(NlaStrip *strip, float min, float max)
 {
 	const float stripLen = (strip) ? strip->end - strip->start : 0.0f;
 	const float boundsLen = fabsf(max - min);
 	
 	/* sanity checks */
 	if ((strip == NULL) || IS_EQF(stripLen, 0.0f) || IS_EQF(boundsLen, 0.0f))
-		return 0;
+		return false;
 	
 	/* only ok if at least part of the strip is within the bounding window
 	 *	- first 2 cases cover when the strip length is less than the bounding area
@@ -1100,17 +1115,61 @@ short BKE_nlastrip_within_bounds(NlaStrip *strip, float min, float max)
 	    !(IN_RANGE(strip->start, min, max) ||
 	      IN_RANGE(strip->end, min, max)))
 	{
-		return 0;
+		return false;
 	}
 	if ((stripLen > boundsLen) &&
 	    !(IN_RANGE(min, strip->start, strip->end) ||
 	      IN_RANGE(max, strip->start, strip->end)) )
 	{
-		return 0;
+		return false;
 	}
 	
 	/* should be ok! */
-	return 1;
+	return true;
+}
+
+
+/* Ensure that strip doesn't overlap those around it after resizing by offsetting those which follow */
+static void nlastrip_fix_resize_overlaps(NlaStrip *strip)
+{
+	/* next strips - do this first, since we're often just getting longer */
+	if (strip->next) {
+		NlaStrip *nls = strip->next;
+		float offset = 0.0f;
+		
+		if (strip->end > nls->start) {
+			/* NOTE: need to ensure we don't have a fractional frame offset, even if that leaves a gap,
+			 * otherwise it will be very hard to get rid of later
+			 */
+			offset = ceilf(strip->end - nls->start);
+			
+			/* apply to times of all strips in this direction */
+			for (; nls; nls = nls->next) {
+				nls->start += offset;
+				nls->end   += offset;
+			}
+		}
+	}
+	
+	/* previous strips - same routine as before */
+	/* NOTE: when strip bounds are recalculated, this is not considered! */
+	if (strip->prev) {
+		NlaStrip *nls = strip->prev;
+		float offset = 0.0f;
+		
+		if (strip->start < nls->end) {
+			/* NOTE: need to ensure we don't have a fractional frame offset, even if that leaves a gap,
+			 * otherwise it will be very hard to get rid of later
+			 */
+			offset = ceilf(nls->end - strip->start);
+			
+			/* apply to times of all strips in this direction */
+			for (; nls; nls = nls->prev) {
+				nls->start -= offset;
+				nls->end   -= offset;
+			}
+		}
+	}
 }
 
 /* Recalculate the start and end frames for the current strip, after changing
@@ -1136,22 +1195,25 @@ void BKE_nlastrip_recalculate_bounds(NlaStrip *strip)
 	/* adjust endpoint of strip in response to this */
 	if (IS_EQF(mapping, 0.0f) == 0)
 		strip->end = (actlen * mapping) + strip->start;
+	
+	/* make sure we don't overlap our neighbours */
+	nlastrip_fix_resize_overlaps(strip);
 }
 
 /* Is the given NLA-strip the first one to occur for the given AnimData block */
 // TODO: make this an api method if necesary, but need to add prefix first
-static short nlastrip_is_first(AnimData *adt, NlaStrip *strip)
+static bool nlastrip_is_first(AnimData *adt, NlaStrip *strip)
 {
 	NlaTrack *nlt;
 	NlaStrip *ns;
 	
 	/* sanity checks */
 	if (ELEM(NULL, adt, strip))
-		return 0;
+		return false;
 		
 	/* check if strip has any strips before it */
 	if (strip->prev)
-		return 0;
+		return false;
 		
 	/* check other tracks to see if they have a strip that's earlier */
 	/* TODO: or should we check that the strip's track is also the first? */
@@ -1160,52 +1222,52 @@ static short nlastrip_is_first(AnimData *adt, NlaStrip *strip)
 		ns = nlt->strips.first;
 		if (ns) {
 			if (ns->start < strip->start)
-				return 0;
+				return false;
 		}
-	}	
+	}
 	
 	/* should be first now */
-	return 1;
+	return true;
 }
 
 /* Animated Strips ------------------------------------------- */
 
 /* Check if the given NLA-Track has any strips with own F-Curves */
-short BKE_nlatrack_has_animated_strips(NlaTrack *nlt)
+bool BKE_nlatrack_has_animated_strips(NlaTrack *nlt)
 {
 	NlaStrip *strip;
 	
 	/* sanity checks */
 	if (ELEM(NULL, nlt, nlt->strips.first))
-		return 0;
+		return false;
 		
 	/* check each strip for F-Curves only (don't care about whether the flags are set) */
 	for (strip = nlt->strips.first; strip; strip = strip->next) {
 		if (strip->fcurves.first)
-			return 1;
+			return true;
 	}
 	
 	/* none found */
-	return 0;
+	return false;
 }
 
 /* Check if given NLA-Tracks have any strips with own F-Curves */
-short BKE_nlatracks_have_animated_strips(ListBase *tracks)
+bool BKE_nlatracks_have_animated_strips(ListBase *tracks)
 {
 	NlaTrack *nlt;
 	
 	/* sanity checks */
 	if (ELEM(NULL, tracks, tracks->first))
-		return 0;
+		return false;
 		
 	/* check each track, stopping on the first hit */
 	for (nlt = tracks->first; nlt; nlt = nlt->next) {
 		if (BKE_nlatrack_has_animated_strips(nlt))
-			return 1;
+			return true;
 	}
 	
 	/* none found */
-	return 0;
+	return false;
 }
 
 /* Validate the NLA-Strips 'control' F-Curves based on the flags set*/
@@ -1262,7 +1324,7 @@ void BKE_nlastrip_validate_fcurves(NlaStrip *strip)
 
 /* Sanity Validation ------------------------------------ */
 
-static int nla_editbone_name_check(void *arg, const char *name)
+static bool nla_editbone_name_check(void *arg, const char *name)
 {
 	return BLI_ghash_haskey((GHash *)arg, (void *)name);
 }
@@ -1301,7 +1363,7 @@ void BKE_nlastrip_validate_name(AnimData *adt, NlaStrip *strip)
 	}
 	
 	/* build a hash-table of all the strips in the tracks 
-	 *	- this is easier than iterating over all the tracks+strips hierarchy everytime
+	 *	- this is easier than iterating over all the tracks+strips hierarchy every time
 	 *	  (and probably faster)
 	 */
 	gh = BLI_ghash_str_new("nlastrip_validate_name gh");
@@ -1320,7 +1382,7 @@ void BKE_nlastrip_validate_name(AnimData *adt, NlaStrip *strip)
 	/* if the hash-table has a match for this name, try other names... 
 	 *	- in an extreme case, it might not be able to find a name, but then everything else in Blender would fail too :)
 	 */
-	BLI_uniquename_cb(nla_editbone_name_check, (void *)gh, "NlaStrip", '.', strip->name, sizeof(strip->name));
+	BLI_uniquename_cb(nla_editbone_name_check, (void *)gh, DATA_("NlaStrip"), '.', strip->name, sizeof(strip->name));
 
 	/* free the hash... */
 	BLI_ghash_free(gh, NULL, NULL);
@@ -1447,9 +1509,9 @@ void BKE_nla_validate_state(AnimData *adt)
 			if (strip->extendmode != NLASTRIP_EXTEND_NOTHING) {
 				/* 1) First strip must be set to extend hold, otherwise, stuff before acts dodgy
 				 * 2) Only overwrite extend mode if *not* changing it will most probably result in 
-				 * occlusion problems, which will occur iff
-				 *	- blendmode = REPLACE
-				 *	- all channels the same (this is fiddly to test, so is currently assumed)
+				 * occlusion problems, which will occur if...
+				 * - blendmode = REPLACE
+				 * - all channels the same (this is fiddly to test, so is currently assumed)
 				 *
 				 * Should fix problems such as [#29869]
 				 */
@@ -1491,7 +1553,7 @@ void BKE_nla_action_pushdown(AnimData *adt)
 	/* add a new NLA strip to the track, which references the active action */
 	strip = add_nlastrip_to_stack(adt, adt->action);
 	
-	/* do other necessary work on strip */	
+	/* do other necessary work on strip */
 	if (strip) {
 		/* clear reference to action now that we've pushed it onto the stack */
 		id_us_min(&adt->action->id);
@@ -1518,20 +1580,20 @@ void BKE_nla_action_pushdown(AnimData *adt)
 /* Find the active strip + track combo, and set them up as the tweaking track,
  * and return if successful or not.
  */
-short BKE_nla_tweakmode_enter(AnimData *adt)
+bool BKE_nla_tweakmode_enter(AnimData *adt)
 {
 	NlaTrack *nlt, *activeTrack = NULL;
 	NlaStrip *strip, *activeStrip = NULL;
 	
 	/* verify that data is valid */
 	if (ELEM(NULL, adt, adt->nla_tracks.first))
-		return 0;
+		return false;
 		
 	/* if block is already in tweakmode, just leave, but we should report 
 	 * that this block is in tweakmode (as our returncode)
 	 */
 	if (adt->flag & ADT_NLA_EDIT_ON)
-		return 1;
+		return true;
 		
 	/* go over the tracks, finding the active one, and its active strip
 	 *  - if we cannot find both, then there's nothing to do
@@ -1545,7 +1607,7 @@ short BKE_nla_tweakmode_enter(AnimData *adt)
 			/* now try to find active strip */
 			activeStrip = BKE_nlastrip_find_active(nlt);
 			break;
-		}	
+		}
 	}
 	
 	/* There are situations where we may have multiple strips selected and we want to enter tweakmode on all 
@@ -1563,7 +1625,7 @@ short BKE_nla_tweakmode_enter(AnimData *adt)
 				activeStrip = BKE_nlastrip_find_active(nlt);
 				break;
 			}
-		}	
+		}
 	}
 	if ((activeTrack) && (activeStrip == NULL)) {
 		/* no active strip in active or last selected track; compromise for first selected (assuming only single)... */
@@ -1575,12 +1637,12 @@ short BKE_nla_tweakmode_enter(AnimData *adt)
 		}
 	}
 	
-	if (ELEM3(NULL, activeTrack, activeStrip, activeStrip->act)) {
+	if (ELEM(NULL, activeTrack, activeStrip, activeStrip->act)) {
 		if (G.debug & G_DEBUG) {
 			printf("NLA tweakmode enter - neither active requirement found\n");
 			printf("\tactiveTrack = %p, activeStrip = %p\n", (void *)activeTrack, (void *)activeStrip);
 		}
-		return 0;
+		return false;
 	}
 		
 	/* go over all the tracks up to the active one, tagging each strip that uses the same 
@@ -1615,7 +1677,7 @@ short BKE_nla_tweakmode_enter(AnimData *adt)
 	adt->flag |= ADT_NLA_EDIT_ON;
 	
 	/* done! */
-	return 1;
+	return true;
 }
 
 /* Exit tweakmode for this AnimData block */
@@ -1632,7 +1694,22 @@ void BKE_nla_tweakmode_exit(AnimData *adt)
 	if ((adt->flag & ADT_NLA_EDIT_ON) == 0)
 		return;
 		
-	/* TODO: need to sync the user-strip with the new state of the action! */
+	/* sync the length of the user-strip with the new state of the action
+	 * but only if the user has explicitly asked for this to happen
+	 * (see [#34645] for things to be careful about)
+	 */
+	if ((adt->actstrip) && (adt->actstrip->flag & NLASTRIP_FLAG_SYNC_LENGTH)) {
+		strip = adt->actstrip;
+		
+		/* must be action-clip only (transitions don't have scale) */
+		if ((strip->type == NLASTRIP_TYPE_CLIP) && (strip->act)) {
+			/* recalculate the length of the action */
+			calc_action_range(strip->act, &strip->actstart, &strip->actend, 0);
+			
+			/* adjust the strip extents in response to this */
+			BKE_nlastrip_recalculate_bounds(strip);
+		}
+	}
 
 	/* for all Tracks, clear the 'disabled' flag
 	 * for all Strips, clear the 'tweak-user' flag
@@ -1660,14 +1737,14 @@ void BKE_nla_tweakmode_exit(AnimData *adt)
 
 /* Baking Tools ------------------------------------------- */
 
-static void UNUSED_FUNCTION(BKE_nla_bake) (Scene * scene, ID *UNUSED(id), AnimData * adt, int UNUSED(flag))
+static void UNUSED_FUNCTION(BKE_nla_bake) (Scene *scene, ID *UNUSED(id), AnimData *adt, int UNUSED(flag))
 {
 
 	/* verify that data is valid 
 	 *	1) Scene and AnimData must be provided 
 	 *	2) there must be tracks to merge...
 	 */
-	if (ELEM3(NULL, scene, adt, adt->nla_tracks.first))
+	if (ELEM(NULL, scene, adt, adt->nla_tracks.first))
 		return;
 	
 	/* if animdata currently has an action, 'push down' this onto the stack first */
