@@ -36,6 +36,7 @@
 
 #include "BLI_blenlib.h"
 #include "BLI_math.h"
+#include "BLI_threads.h"
 
 #include "DNA_anim_types.h"
 #include "DNA_object_types.h"
@@ -53,19 +54,13 @@
 #include "BKE_global.h"
 #include "BKE_main.h"
 #include "BKE_sound.h"
-#include "BKE_context.h"
 #include "BKE_library.h"
 #include "BKE_packedFile.h"
-#include "BKE_animsys.h"
 #include "BKE_sequencer.h"
 #include "BKE_scene.h"
 
-// evil quiet NaN definition
-static const int NAN_INT = 0x7FC00000;
-#define NAN_FLT *((float *)(&NAN_INT))
-
 #ifdef WITH_AUDASPACE
-// evil global ;-)
+/* evil global ;-) */
 static int sound_cfra;
 #endif
 
@@ -74,7 +69,7 @@ bSound *sound_new_file(struct Main *bmain, const char *filename)
 	bSound *sound = NULL;
 
 	char str[FILE_MAX];
-	char *path;
+	const char *path;
 
 	size_t len;
 
@@ -88,14 +83,14 @@ bSound *sound_new_file(struct Main *bmain, const char *filename)
 	while (len > 0 && filename[len - 1] != '/' && filename[len - 1] != '\\')
 		len--;
 
-	sound = BKE_libblock_alloc(&bmain->sound, ID_SO, filename + len);
+	sound = BKE_libblock_alloc(bmain, ID_SO, filename + len);
 	BLI_strncpy(sound->name, filename, FILE_MAX);
-// XXX unused currently	sound->type = SOUND_TYPE_FILE;
+	/* sound->type = SOUND_TYPE_FILE; */ /* XXX unused currently */
 
 	sound_load(bmain, sound);
 
 	if (!sound->playback_handle) {
-		BKE_libblock_free(&bmain->sound, sound);
+		BKE_libblock_free(bmain, sound);
 		sound = NULL;
 	}
 
@@ -122,7 +117,13 @@ void BKE_sound_free(bSound *sound)
 	}
 
 	sound_free_waveform(sound);
-#endif // WITH_AUDASPACE
+	
+	if (sound->mutex) {
+		BLI_mutex_free(sound->mutex);
+		sound->mutex = NULL;
+	}
+	
+#endif  /* WITH_AUDASPACE */
 }
 
 #ifdef WITH_AUDASPACE
@@ -172,7 +173,7 @@ void sound_force_device(int device)
 void sound_init_once(void)
 {
 	AUD_initOnce();
-	atexit(sound_exit);
+	atexit(sound_exit_once);
 }
 
 void sound_init(struct Main *bmain)
@@ -221,7 +222,13 @@ void sound_exit(void)
 	AUD_exit();
 }
 
-// XXX unused currently
+void sound_exit_once(void)
+{
+	AUD_exit();
+	AUD_exitOnce();
+}
+
+/* XXX unused currently */
 #if 0
 bSound *sound_new_buffer(struct Main *bmain, bSound *source)
 {
@@ -231,7 +238,7 @@ bSound *sound_new_buffer(struct Main *bmain, bSound *source)
 	strcpy(name, "buf_");
 	strcpy(name + 4, source->id.name);
 
-	sound = BKE_libblock_alloc(&bmain->sound, ID_SO, name);
+	sound = BKE_libblock_alloc(bmain, ID_SO, name);
 
 	sound->child_sound = source;
 	sound->type = SOUND_TYPE_BUFFER;
@@ -240,7 +247,7 @@ bSound *sound_new_buffer(struct Main *bmain, bSound *source)
 
 	if (!sound->playback_handle)
 	{
-		BKE_libblock_free(&bmain->sound, sound);
+		BKE_libblock_free(bmain, sound);
 		sound = NULL;
 	}
 
@@ -255,7 +262,7 @@ bSound *sound_new_limiter(struct Main *bmain, bSound *source, float start, float
 	strcpy(name, "lim_");
 	strcpy(name + 4, source->id.name);
 
-	sound = BKE_libblock_alloc(&bmain->sound, ID_SO, name);
+	sound = BKE_libblock_alloc(bmain, ID_SO, name);
 
 	sound->child_sound = source;
 	sound->start = start;
@@ -266,7 +273,7 @@ bSound *sound_new_limiter(struct Main *bmain, bSound *source, float start, float
 
 	if (!sound->playback_handle)
 	{
-		BKE_libblock_free(&bmain->sound, sound);
+		BKE_libblock_free(bmain, sound);
 		sound = NULL;
 	}
 
@@ -279,7 +286,7 @@ void sound_delete(struct Main *bmain, bSound *sound)
 	if (sound) {
 		BKE_sound_free(sound);
 
-		BKE_libblock_free(&bmain->sound, sound);
+		BKE_libblock_free(bmain, sound);
 	}
 }
 
@@ -296,12 +303,6 @@ void sound_cache(bSound *sound)
 		sound->playback_handle = sound->handle;
 }
 
-void sound_cache_notifying(struct Main *main, bSound *sound)
-{
-	sound_cache(sound);
-	sound_update_sequencer(main, sound);
-}
-
 void sound_delete_cache(bSound *sound)
 {
 	sound->flags &= ~SOUND_FLAGS_CACHING;
@@ -328,7 +329,7 @@ void sound_load(struct Main *bmain, bSound *sound)
 
 		sound_free_waveform(sound);
 
-// XXX unused currently
+/* XXX unused currently */
 #if 0
 		switch (sound->type)
 		{
@@ -351,19 +352,19 @@ void sound_load(struct Main *bmain, bSound *sound)
 			else
 				sound->handle = AUD_load(fullpath);
 		}
-// XXX unused currently
+/* XXX unused currently */
 #if 0
-		break;
+			break;
+		}
+		case SOUND_TYPE_BUFFER:
+			if (sound->child_sound && sound->child_sound->handle)
+				sound->handle = AUD_bufferSound(sound->child_sound->handle);
+			break;
+		case SOUND_TYPE_LIMITER:
+			if (sound->child_sound && sound->child_sound->handle)
+				sound->handle = AUD_limitSound(sound->child_sound, sound->start, sound->end);
+			break;
 	}
-	case SOUND_TYPE_BUFFER:
-		if (sound->child_sound && sound->child_sound->handle)
-			sound->handle = AUD_bufferSound(sound->child_sound->handle);
-		break;
-	case SOUND_TYPE_LIMITER:
-		if (sound->child_sound && sound->child_sound->handle)
-			sound->handle = AUD_limitSound(sound->child_sound, sound->start, sound->end);
-		break;
-}
 #endif
 		if (sound->flags & SOUND_FLAGS_MONO) {
 			void *handle = AUD_monoSound(sound->handle);
@@ -425,6 +426,8 @@ void sound_update_fps(struct Scene *scene)
 {
 	if (scene->sound_scene)
 		AUD_setSequencerFPS(scene->sound_scene, FPS);
+
+	BKE_sequencer_refresh_sound_length(scene);
 }
 
 void sound_update_scene_listener(struct Scene *scene)
@@ -433,10 +436,14 @@ void sound_update_scene_listener(struct Scene *scene)
 	                        scene->audio.doppler_factor, scene->audio.distance_model);
 }
 
-void *sound_scene_add_scene_sound(struct Scene *scene, struct Sequence *sequence, int startframe, int endframe, int frameskip)
+void *sound_scene_add_scene_sound(struct Scene *scene, struct Sequence *sequence,
+                                  int startframe, int endframe, int frameskip)
 {
-	if (scene != sequence->scene)
-		return AUD_addSequence(scene->sound_scene, sequence->scene->sound_scene, startframe / FPS, endframe / FPS, frameskip / FPS);
+	if (scene != sequence->scene) {
+		const double fps = FPS;
+		return AUD_addSequence(scene->sound_scene, sequence->scene->sound_scene,
+		                       startframe / fps, endframe / fps, frameskip / fps);
+	}
 	return NULL;
 }
 
@@ -449,7 +456,9 @@ void *sound_scene_add_scene_sound_defaults(struct Scene *scene, struct Sequence
 
 void *sound_add_scene_sound(struct Scene *scene, struct Sequence *sequence, int startframe, int endframe, int frameskip)
 {
-	void *handle = AUD_addSequence(scene->sound_scene, sequence->sound->playback_handle, startframe / FPS, endframe / FPS, frameskip / FPS);
+	const double fps = FPS;
+	void *handle = AUD_addSequence(scene->sound_scene, sequence->sound->playback_handle,
+	                               startframe / fps, endframe / fps, frameskip / fps);
 	AUD_muteSequence(handle, (sequence->flag & SEQ_MUTE) != 0);
 	AUD_setSequenceAnimData(handle, AUD_AP_VOLUME, CFRA, &sequence->volume, 0);
 	AUD_setSequenceAnimData(handle, AUD_AP_PITCH, CFRA, &sequence->pitch, 0);
@@ -476,7 +485,8 @@ void sound_mute_scene_sound(void *handle, char mute)
 
 void sound_move_scene_sound(struct Scene *scene, void *handle, int startframe, int endframe, int frameskip)
 {
-	AUD_moveSequence(handle, startframe / FPS, endframe / FPS, frameskip / FPS);
+	const double fps = FPS;
+	AUD_moveSequence(handle, startframe / fps, endframe / fps, frameskip / fps);
 }
 
 void sound_move_scene_sound_defaults(struct Scene *scene, struct Sequence *sequence)
@@ -500,7 +510,8 @@ void sound_set_cfra(int cfra)
 
 void sound_set_scene_volume(struct Scene *scene, float volume)
 {
-	AUD_setSequencerAnimData(scene->sound_scene, AUD_AP_VOLUME, CFRA, &volume, (scene->audio.flag & AUDIO_VOLUME_ANIMATED) != 0);
+	AUD_setSequencerAnimData(scene->sound_scene, AUD_AP_VOLUME, CFRA, &volume,
+	                         (scene->audio.flag & AUDIO_VOLUME_ANIMATED) != 0);
 }
 
 void sound_set_scene_sound_volume(void *handle, float volume, char animated)
@@ -541,20 +552,23 @@ static void sound_start_play_scene(struct Scene *scene)
 void sound_play_scene(struct Scene *scene)
 {
 	AUD_Status status;
+	const float cur_time = (float)((double)CFRA / FPS);
+
 	AUD_lock();
 
 	status = scene->sound_scene_handle ? AUD_getStatus(scene->sound_scene_handle) : AUD_STATUS_INVALID;
 
-	if (status == AUD_STATUS_INVALID)
+	if (status == AUD_STATUS_INVALID) {
 		sound_start_play_scene(scene);
 
-	if (!scene->sound_scene_handle) {
-		AUD_unlock();
-		return;
+		if (!scene->sound_scene_handle) {
+			AUD_unlock();
+			return;
+		}
 	}
 
 	if (status != AUD_STATUS_PLAYING) {
-		AUD_seek(scene->sound_scene_handle, CFRA / FPS);
+		AUD_seek(scene->sound_scene_handle, cur_time);
 		AUD_resume(scene->sound_scene_handle);
 	}
 
@@ -580,6 +594,9 @@ void sound_seek_scene(struct Main *bmain, struct Scene *scene)
 	bScreen *screen;
 	int animation_playing;
 
+	const float one_frame = (float)(1.0 / FPS);
+	const float cur_time = (float)((double)CFRA / FPS);
+
 	AUD_lock();
 
 	status = scene->sound_scene_handle ? AUD_getStatus(scene->sound_scene_handle) : AUD_STATUS_INVALID;
@@ -604,26 +621,31 @@ void sound_seek_scene(struct Main *bmain, struct Scene *scene)
 
 	if (scene->audio.flag & AUDIO_SCRUB && !animation_playing) {
 		if (scene->audio.flag & AUDIO_SYNC) {
-			AUD_seek(scene->sound_scene_handle, CFRA / FPS);
-			AUD_seekSequencer(scene->sound_scene_handle, CFRA / FPS);
+			AUD_seek(scene->sound_scene_handle, cur_time);
+			AUD_seekSequencer(scene->sound_scene_handle, cur_time);
 		}
-		else
-			AUD_seek(scene->sound_scene_handle, CFRA / FPS);
-		AUD_resume(scene->sound_scene_handle);
-		if (scene->sound_scrub_handle && AUD_getStatus(scene->sound_scrub_handle) != AUD_STATUS_INVALID)
-			AUD_seek(scene->sound_scrub_handle, 0);
 		else {
-			if (scene->sound_scrub_handle)
+			AUD_seek(scene->sound_scene_handle, cur_time);
+		}
+		AUD_resume(scene->sound_scene_handle);
+		if (scene->sound_scrub_handle && AUD_getStatus(scene->sound_scrub_handle) != AUD_STATUS_INVALID) {
+			AUD_seek(scene->sound_scrub_handle, 0);
+		}
+		else {
+			if (scene->sound_scrub_handle) {
 				AUD_stop(scene->sound_scrub_handle);
-			scene->sound_scrub_handle = AUD_pauseAfter(scene->sound_scene_handle, 1 / FPS);
+			}
+			scene->sound_scrub_handle = AUD_pauseAfter(scene->sound_scene_handle, one_frame);
 		}
 	}
 	else {
-		if (scene->audio.flag & AUDIO_SYNC)
-			AUD_seekSequencer(scene->sound_scene_handle, CFRA / FPS);
+		if (scene->audio.flag & AUDIO_SYNC) {
+			AUD_seekSequencer(scene->sound_scene_handle, cur_time);
+		}
 		else {
-			if (status == AUD_STATUS_PLAYING)
-				AUD_seek(scene->sound_scene_handle, CFRA / FPS);
+			if (status == AUD_STATUS_PLAYING) {
+				AUD_seek(scene->sound_scene_handle, cur_time);
+			}
 		}
 	}
 
@@ -659,25 +681,43 @@ void sound_free_waveform(bSound *sound)
 	sound->waveform = NULL;
 }
 
-void sound_read_waveform(bSound *sound)
+void sound_read_waveform(bSound *sound, bool locked, short *stop)
 {
 	AUD_SoundInfo info;
-
+	SoundWaveform *waveform = NULL;
+	
 	info = AUD_getInfo(sound->playback_handle);
-
+	
 	if (info.length > 0) {
-		SoundWaveform *waveform = MEM_mallocN(sizeof(SoundWaveform), "SoundWaveform");
 		int length = info.length * SOUND_WAVE_SAMPLES_PER_SECOND;
-
+		
+		waveform = MEM_mallocN(sizeof(SoundWaveform), "SoundWaveform");
 		waveform->data = MEM_mallocN(length * sizeof(float) * 3, "SoundWaveform.samples");
-		waveform->length = AUD_readSound(sound->playback_handle, waveform->data, length, SOUND_WAVE_SAMPLES_PER_SECOND);
-
+		waveform->length = AUD_readSound(sound->playback_handle, waveform->data, length, SOUND_WAVE_SAMPLES_PER_SECOND, stop);
+		
+		if (*stop) {
+			MEM_freeN(waveform->data);
+			MEM_freeN(waveform);
+			if (locked)
+				BLI_mutex_lock(sound->mutex);
+			sound->flags &= ~SOUND_FLAGS_WAVEFORM_LOADING;
+			if (locked)
+				BLI_mutex_unlock(sound->mutex);
+			return;
+		}
+		
 		sound_free_waveform(sound);
-		sound->waveform = waveform;
 	}
+	
+	if (locked)
+		BLI_mutex_lock(sound->mutex);
+	sound->waveform = waveform;
+	sound->flags &= ~SOUND_FLAGS_WAVEFORM_LOADING;
+	if (locked)
+		BLI_mutex_unlock(sound->mutex);
 }
 
-void sound_update_scene(struct Scene *scene)
+void sound_update_scene(Main *bmain, struct Scene *scene)
 {
 	Object *ob;
 	Base *base;
@@ -690,47 +730,54 @@ void sound_update_scene(struct Scene *scene)
 	void *handle;
 	float quat[4];
 
-	for (SETLOOPER(scene, sce_it, base)) {
-		ob = base->object;
-		if (ob->type == OB_SPEAKER) {
-			if (ob->adt) {
-				for (track = ob->adt->nla_tracks.first; track; track = track->next) {
-					for (strip = track->strips.first; strip; strip = strip->next) {
-						if (strip->type == NLASTRIP_TYPE_SOUND) {
-							speaker = (Speaker *)ob->data;
+	/* cheap test to skip looping over all objects (no speakers is a common case) */
+	if (!BLI_listbase_is_empty(&bmain->speaker)) {
+		for (SETLOOPER(scene, sce_it, base)) {
+			ob = base->object;
+			if ((ob->type != OB_SPEAKER) || !ob->adt) {
+				continue;
+			}
+			for (track = ob->adt->nla_tracks.first; track; track = track->next) {
+				for (strip = track->strips.first; strip; strip = strip->next) {
+					if (strip->type != NLASTRIP_TYPE_SOUND) {
+						continue;
+					}
+					speaker = (Speaker *)ob->data;
 
-							if (AUD_removeSet(scene->speaker_handles, strip->speaker_handle)) {
-								if (speaker->sound)
-									AUD_moveSequence(strip->speaker_handle, strip->start / FPS, -1, 0);
-								else {
-									AUD_removeSequence(scene->sound_scene, strip->speaker_handle);
-									strip->speaker_handle = NULL;
-								}
-							}
-							else {
-								if (speaker->sound) {
-									strip->speaker_handle = AUD_addSequence(scene->sound_scene, speaker->sound->playback_handle, strip->start / FPS, -1, 0);
-									AUD_setRelativeSequence(strip->speaker_handle, 0);
-								}
-							}
-
-							if (strip->speaker_handle) {
-								AUD_addSet(new_set, strip->speaker_handle);
-								AUD_updateSequenceData(strip->speaker_handle, speaker->volume_max,
-								                       speaker->volume_min, speaker->distance_max,
-								                       speaker->distance_reference, speaker->attenuation,
-								                       speaker->cone_angle_outer, speaker->cone_angle_inner,
-								                       speaker->cone_volume_outer);
-
-								mat4_to_quat(quat, ob->obmat);
-								AUD_setSequenceAnimData(strip->speaker_handle, AUD_AP_LOCATION, CFRA, ob->obmat[3], 1);
-								AUD_setSequenceAnimData(strip->speaker_handle, AUD_AP_ORIENTATION, CFRA, quat, 1);
-								AUD_setSequenceAnimData(strip->speaker_handle, AUD_AP_VOLUME, CFRA, &speaker->volume, 1);
-								AUD_setSequenceAnimData(strip->speaker_handle, AUD_AP_PITCH, CFRA, &speaker->pitch, 1);
-								AUD_updateSequenceSound(strip->speaker_handle, speaker->sound->playback_handle);
-								AUD_muteSequence(strip->speaker_handle, ((strip->flag & NLASTRIP_FLAG_MUTED) != 0) || ((speaker->flag & SPK_MUTED) != 0));
-							}
+					if (AUD_removeSet(scene->speaker_handles, strip->speaker_handle)) {
+						if (speaker->sound) {
+							AUD_moveSequence(strip->speaker_handle, (double)strip->start / FPS, FLT_MAX, 0);
 						}
+						else {
+							AUD_removeSequence(scene->sound_scene, strip->speaker_handle);
+							strip->speaker_handle = NULL;
+						}
+					}
+					else {
+						if (speaker->sound) {
+							strip->speaker_handle = AUD_addSequence(scene->sound_scene,
+							                                        speaker->sound->playback_handle,
+							                                        (double)strip->start / FPS, FLT_MAX, 0);
+							AUD_setRelativeSequence(strip->speaker_handle, 0);
+						}
+					}
+
+					if (strip->speaker_handle) {
+						const bool mute = ((strip->flag & NLASTRIP_FLAG_MUTED) || (speaker->flag & SPK_MUTED));
+						AUD_addSet(new_set, strip->speaker_handle);
+						AUD_updateSequenceData(strip->speaker_handle, speaker->volume_max,
+						                       speaker->volume_min, speaker->distance_max,
+						                       speaker->distance_reference, speaker->attenuation,
+						                       speaker->cone_angle_outer, speaker->cone_angle_inner,
+						                       speaker->cone_volume_outer);
+
+						mat4_to_quat(quat, ob->obmat);
+						AUD_setSequenceAnimData(strip->speaker_handle, AUD_AP_LOCATION, CFRA, ob->obmat[3], 1);
+						AUD_setSequenceAnimData(strip->speaker_handle, AUD_AP_ORIENTATION, CFRA, quat, 1);
+						AUD_setSequenceAnimData(strip->speaker_handle, AUD_AP_VOLUME, CFRA, &speaker->volume, 1);
+						AUD_setSequenceAnimData(strip->speaker_handle, AUD_AP_PITCH, CFRA, &speaker->pitch, 1);
+						AUD_updateSequenceSound(strip->speaker_handle, speaker->sound->playback_handle);
+						AUD_muteSequence(strip->speaker_handle, mute);
 					}
 				}
 			}
@@ -753,7 +800,7 @@ void sound_update_scene(struct Scene *scene)
 
 void *sound_get_factory(void *sound)
 {
-	return ((bSound *) sound)->playback_handle;
+	return ((bSound *)sound)->playback_handle;
 }
 
 /* stupid wrapper because AUD_C-API.h includes Python.h which makesrna doesn't like */
@@ -764,45 +811,56 @@ float sound_get_length(bSound *sound)
 	return info.length;
 }
 
-#else // WITH_AUDASPACE
+bool sound_is_jack_supported(void)
+{
+	return (bool)AUD_isJackSupported();
+}
+
+#else  /* WITH_AUDASPACE */
 
 #include "BLI_utildefines.h"
 
-int sound_define_from_str(const char *UNUSED(str)) { return -1;}
+int sound_define_from_str(const char *UNUSED(str)) { return -1; }
 void sound_force_device(int UNUSED(device)) {}
 void sound_init_once(void) {}
 void sound_init(struct Main *UNUSED(bmain)) {}
 void sound_exit(void) {}
-void sound_cache(struct bSound *UNUSED(sound)) { }
+void sound_exit_once(void) {}
+void sound_cache(struct bSound *UNUSED(sound)) {}
 void sound_delete_cache(struct bSound *UNUSED(sound)) {}
 void sound_load(struct Main *UNUSED(bmain), struct bSound *UNUSED(sound)) {}
 void sound_create_scene(struct Scene *UNUSED(scene)) {}
 void sound_destroy_scene(struct Scene *UNUSED(scene)) {}
 void sound_mute_scene(struct Scene *UNUSED(scene), int UNUSED(muted)) {}
-void *sound_scene_add_scene_sound(struct Scene *UNUSED(scene), struct Sequence *UNUSED(sequence), int UNUSED(startframe), int UNUSED(endframe), int UNUSED(frameskip)) { return NULL; }
-void *sound_scene_add_scene_sound_defaults(struct Scene *UNUSED(scene), struct Sequence *UNUSED(sequence)) { return NULL; }
-void *sound_add_scene_sound(struct Scene *UNUSED(scene), struct Sequence *UNUSED(sequence), int UNUSED(startframe), int UNUSED(endframe), int UNUSED(frameskip)) { return NULL; }
+void *sound_scene_add_scene_sound(struct Scene *UNUSED(scene), struct Sequence *UNUSED(sequence),
+                                  int UNUSED(startframe), int UNUSED(endframe), int UNUSED(frameskip)) { return NULL; }
+void *sound_scene_add_scene_sound_defaults(struct Scene *UNUSED(scene),
+                                           struct Sequence *UNUSED(sequence)) { return NULL; }
+void *sound_add_scene_sound(struct Scene *UNUSED(scene), struct Sequence *UNUSED(sequence), int UNUSED(startframe),
+                            int UNUSED(endframe), int UNUSED(frameskip)) { return NULL; }
 void *sound_add_scene_sound_defaults(struct Scene *UNUSED(scene), struct Sequence *UNUSED(sequence)) { return NULL; }
 void sound_remove_scene_sound(struct Scene *UNUSED(scene), void *UNUSED(handle)) {}
 void sound_mute_scene_sound(void *UNUSED(handle), char UNUSED(mute)) {}
-void sound_move_scene_sound(struct Scene *UNUSED(scene), void *UNUSED(handle), int UNUSED(startframe), int UNUSED(endframe), int UNUSED(frameskip)) {}
+void sound_move_scene_sound(struct Scene *UNUSED(scene), void *UNUSED(handle), int UNUSED(startframe),
+                            int UNUSED(endframe), int UNUSED(frameskip)) {}
 void sound_move_scene_sound_defaults(struct Scene *UNUSED(scene), struct Sequence *UNUSED(sequence)) {}
 void sound_play_scene(struct Scene *UNUSED(scene)) {}
 void sound_stop_scene(struct Scene *UNUSED(scene)) {}
 void sound_seek_scene(struct Main *UNUSED(bmain), struct Scene *UNUSED(scene)) {}
 float sound_sync_scene(struct Scene *UNUSED(scene)) { return NAN_FLT; }
 int sound_scene_playing(struct Scene *UNUSED(scene)) { return -1; }
-void sound_read_waveform(struct bSound *sound) { (void)sound; }
-void sound_init_main(struct Main *bmain) { (void)bmain; }
-void sound_set_cfra(int cfra) { (void)cfra; }
-void sound_update_sequencer(struct Main *main, struct bSound *sound) { (void)main; (void)sound; }
-void sound_update_scene(struct Scene *scene) { (void)scene; }
-void sound_update_scene_sound(void *handle, struct bSound *sound) { (void)handle; (void)sound; }
-void sound_update_scene_listener(struct Scene *scene) { (void)scene; }
-void sound_update_fps(struct Scene *scene) { (void)scene; }
-void sound_set_scene_sound_volume(void *handle, float volume, char animated) { (void)handle; (void)volume; (void)animated; }
-void sound_set_scene_sound_pan(void *handle, float pan, char animated) { (void)handle; (void)pan; (void)animated; }
-void sound_set_scene_volume(struct Scene *scene, float volume) { (void)scene; (void)volume; }
-void sound_set_scene_sound_pitch(void *handle, float pitch, char animated) { (void)handle; (void)pitch; (void)animated; }
-float sound_get_length(struct bSound *sound) { (void)sound; return 0; }
-#endif // WITH_AUDASPACE
+void sound_read_waveform(struct bSound *sound, bool locked, short *stop) { UNUSED_VARS(sound, locked, stop); }
+void sound_init_main(struct Main *UNUSED(bmain)) {}
+void sound_set_cfra(int UNUSED(cfra)) {}
+void sound_update_sequencer(struct Main *UNUSED(main), struct bSound *UNUSED(sound)) {}
+void sound_update_scene(struct Main *UNUSED(bmain), struct Scene *UNUSED(scene)) {}
+void sound_update_scene_sound(void *UNUSED(handle), struct bSound *UNUSED(sound)) {}
+void sound_update_scene_listener(struct Scene *UNUSED(scene)) {}
+void sound_update_fps(struct Scene *UNUSED(scene)) {}
+void sound_set_scene_sound_volume(void *UNUSED(handle), float UNUSED(volume), char UNUSED(animated)) {}
+void sound_set_scene_sound_pan(void *UNUSED(handle), float UNUSED(pan), char UNUSED(animated)) {}
+void sound_set_scene_volume(struct Scene *UNUSED(scene), float UNUSED(volume)) {}
+void sound_set_scene_sound_pitch(void *UNUSED(handle), float UNUSED(pitch), char UNUSED(animated)) {}
+float sound_get_length(struct bSound *UNUSED(sound)) { return 0; }
+bool sound_is_jack_supported(void) { return false; }
+#endif  /* WITH_AUDASPACE */
