@@ -37,12 +37,10 @@
 
 #include "MEM_guardedalloc.h"
 
-#include "BLI_blenlib.h"
-#include "BLI_dynlib.h"
 #include "BLI_math.h"
 #include "BLI_kdopbvh.h"
 #include "BLI_utildefines.h"
-#include "BLI_bpath.h"
+#include "BLI_math_color.h"
 
 #include "DNA_key_types.h"
 #include "DNA_object_types.h"
@@ -53,12 +51,12 @@
 #include "DNA_node_types.h"
 #include "DNA_color_types.h"
 #include "DNA_particle_types.h"
+#include "DNA_linestyle_types.h"
 
 #include "IMB_imbuf.h"
 
 #include "BKE_global.h"
 #include "BKE_main.h"
-#include "BKE_ocean.h"
 
 #include "BKE_library.h"
 #include "BKE_image.h"
@@ -69,19 +67,22 @@
 #include "BKE_node.h"
 #include "BKE_animsys.h"
 #include "BKE_colortools.h"
+#include "BKE_scene.h"
+
+#include "RE_shader_ext.h"
 
 /* ****************** Mapping ******************* */
 
-TexMapping *add_tex_mapping(void)
+TexMapping *add_tex_mapping(int type)
 {
 	TexMapping *texmap = MEM_callocN(sizeof(TexMapping), "TexMapping");
 	
-	default_tex_mapping(texmap);
+	default_tex_mapping(texmap, type);
 	
 	return texmap;
 }
 
-void default_tex_mapping(TexMapping *texmap)
+void default_tex_mapping(TexMapping *texmap, int type)
 {
 	memset(texmap, 0, sizeof(TexMapping));
 
@@ -93,11 +94,12 @@ void default_tex_mapping(TexMapping *texmap)
 	texmap->projy = PROJ_Y;
 	texmap->projz = PROJ_Z;
 	texmap->mapping = MTEX_FLAT;
+	texmap->type = type;
 }
 
 void init_tex_mapping(TexMapping *texmap)
 {
-	float smat[3][3], rmat[3][3], mat[3][3], proj[3][3];
+	float smat[4][4], rmat[4][4], tmat[4][4], proj[4][4], size[3];
 
 	if (texmap->projx == PROJ_X && texmap->projy == PROJ_Y && texmap->projz == PROJ_Z &&
 	    is_zero_v3(texmap->loc) && is_zero_v3(texmap->rot) && is_one_v3(texmap->size))
@@ -108,7 +110,8 @@ void init_tex_mapping(TexMapping *texmap)
 	}
 	else {
 		/* axis projection */
-		zero_m3(proj);
+		zero_m4(proj);
+		proj[3][3] = 1.0f;
 
 		if (texmap->projx != PROJ_N)
 			proj[texmap->projx - 1][0] = 1.0f;
@@ -118,19 +121,50 @@ void init_tex_mapping(TexMapping *texmap)
 			proj[texmap->projz - 1][2] = 1.0f;
 
 		/* scale */
-		size_to_mat3(smat, texmap->size);
+		copy_v3_v3(size, texmap->size);
+
+		if (ELEM(texmap->type, TEXMAP_TYPE_TEXTURE, TEXMAP_TYPE_NORMAL)) {
+			/* keep matrix invertible */
+			if (fabsf(size[0]) < 1e-5f)
+				size[0] = signf(size[0]) * 1e-5f;
+			if (fabsf(size[1]) < 1e-5f)
+				size[1] = signf(size[1]) * 1e-5f;
+			if (fabsf(size[2]) < 1e-5f)
+				size[2] = signf(size[2]) * 1e-5f;
+		}
 		
+		size_to_mat4(smat, texmap->size);
+
 		/* rotation */
-		/* TexMapping rotation are now in radians. */
-		eul_to_mat3(rmat, texmap->rot);
-		
-		/* compose it all */
-		mul_m3_m3m3(mat, rmat, smat);
-		mul_m3_m3m3(mat, proj, mat);
-		
+		eul_to_mat4(rmat, texmap->rot);
+
 		/* translation */
-		copy_m4_m3(texmap->mat, mat);
-		copy_v3_v3(texmap->mat[3], texmap->loc);
+		unit_m4(tmat);
+		copy_v3_v3(tmat[3], texmap->loc);
+
+		if (texmap->type == TEXMAP_TYPE_TEXTURE) {
+			/* to transform a texture, the inverse transform needs
+			 * to be applied to the texture coordinate */
+			mul_m4_series(texmap->mat, tmat, rmat, smat);
+			invert_m4(texmap->mat);
+		}
+		else if (texmap->type == TEXMAP_TYPE_POINT) {
+			/* forward transform */
+			mul_m4_series(texmap->mat, tmat, rmat, smat);
+		}
+		else if (texmap->type == TEXMAP_TYPE_VECTOR) {
+			/* no translation for vectors */
+			mul_m4_m4m4(texmap->mat, rmat, smat);
+		}
+		else if (texmap->type == TEXMAP_TYPE_NORMAL) {
+			/* no translation for normals, and inverse transpose */
+			mul_m4_m4m4(texmap->mat, rmat, smat);
+			invert_m4(texmap->mat);
+			transpose_m4(texmap->mat);
+		}
+
+		/* projection last */
+		mul_m4_m4m4(texmap->mat, texmap->mat, proj);
 
 		texmap->flag &= ~TEXMAP_UNIT_MATRIX;
 	}
@@ -149,7 +183,7 @@ void default_color_mapping(ColorMapping *colormap)
 {
 	memset(colormap, 0, sizeof(ColorMapping));
 
-	init_colorband(&colormap->coba, 1);
+	init_colorband(&colormap->coba, true);
 
 	colormap->bright = 1.0;
 	colormap->contrast = 1.0;
@@ -164,7 +198,7 @@ void default_color_mapping(ColorMapping *colormap)
 
 /* ****************** COLORBAND ******************* */
 
-void init_colorband(ColorBand *coba, int rangetype)
+void init_colorband(ColorBand *coba, bool rangetype)
 {
 	int a;
 	
@@ -203,10 +237,10 @@ void init_colorband(ColorBand *coba, int rangetype)
 	}
 	
 	coba->tot = 2;
-	
+	coba->color_mode = COLBAND_BLEND_RGB;
 }
 
-ColorBand *add_colorband(int rangetype)
+ColorBand *add_colorband(bool rangetype)
 {
 	ColorBand *coba;
 	
@@ -218,104 +252,209 @@ ColorBand *add_colorband(int rangetype)
 
 /* ------------------------------------------------------------------------- */
 
-int do_colorband(const ColorBand *coba, float in, float out[4])
+static float colorband_hue_interp(
+        const int ipotype_hue,
+        const float mfac, const float fac,
+        float h1, float h2)
+{
+	float h_interp;
+	int mode = 0;
+
+#define HUE_INTERP(h_a, h_b) ((mfac * (h_a)) + (fac * (h_b)))
+#define HUE_MOD(h) (((h) < 1.0f) ? (h) : (h) - 1.0f)
+
+	h1 = HUE_MOD(h1);
+	h2 = HUE_MOD(h2);
+
+	BLI_assert(h1 >= 0.0f && h1 < 1.0f);
+	BLI_assert(h2 >= 0.0f && h2 < 1.0f);
+
+	switch (ipotype_hue) {
+		case COLBAND_HUE_NEAR:
+		{
+			if      ((h1 < h2) && (h2 - h1) > +0.5f) mode = 1;
+			else if ((h1 > h2) && (h2 - h1) < -0.5f) mode = 2;
+			else                                     mode = 0;
+			break;
+		}
+		case COLBAND_HUE_FAR:
+		{
+			if      ((h1 < h2) && (h2 - h1) < +0.5f) mode = 1;
+			else if ((h1 > h2) && (h2 - h1) > -0.5f) mode = 2;
+			else                                     mode = 0;
+			break;
+		}
+		case COLBAND_HUE_CCW:
+		{
+			if (h1 > h2) mode = 2;
+			else         mode = 0;
+			break;
+		}
+		case COLBAND_HUE_CW:
+		{
+			if (h1 < h2) mode = 1;
+			else         mode = 0;
+			break;
+		}
+	}
+
+	switch (mode) {
+		case 0:
+			h_interp = HUE_INTERP(h1, h2);
+			break;
+		case 1:
+			h_interp = HUE_INTERP(h1 + 1.0f, h2);
+			h_interp = HUE_MOD(h_interp);
+			break;
+		case 2:
+			h_interp = HUE_INTERP(h1, h2 + 1.0f);
+			h_interp = HUE_MOD(h_interp);
+			break;
+	}
+
+	BLI_assert(h_interp >= 0.0f && h_interp < 1.0f);
+
+#undef HUE_INTERP
+#undef HUE_MOD
+
+	return h_interp;
+}
+
+bool do_colorband(const ColorBand *coba, float in, float out[4])
 {
 	const CBData *cbd1, *cbd2, *cbd0, *cbd3;
-	float fac, mfac, t[4];
+	float fac;
+	int ipotype;
 	int a;
 	
-	if (coba == NULL || coba->tot == 0) return 0;
+	if (coba == NULL || coba->tot == 0) return false;
 	
 	cbd1 = coba->data;
+
+	ipotype = (coba->color_mode == COLBAND_BLEND_RGB) ? coba->ipotype : COLBAND_INTERP_LINEAR;
+
 	if (coba->tot == 1) {
 		out[0] = cbd1->r;
 		out[1] = cbd1->g;
 		out[2] = cbd1->b;
 		out[3] = cbd1->a;
 	}
+	else if ((in <= cbd1->pos) && ELEM(ipotype, COLBAND_INTERP_LINEAR, COLBAND_INTERP_EASE)) {
+		out[0] = cbd1->r;
+		out[1] = cbd1->g;
+		out[2] = cbd1->b;
+		out[3] = cbd1->a;
+	}
 	else {
-		if (in <= cbd1->pos && coba->ipotype < 2) {
+		CBData left, right;
+
+		/* we're looking for first pos > in */
+		for (a = 0; a < coba->tot; a++, cbd1++) if (cbd1->pos > in) break;
+
+		if (a == coba->tot) {
+			cbd2 = cbd1 - 1;
+			right = *cbd2;
+			right.pos = 1.0f;
+			cbd1 = &right;
+		}
+		else if (a == 0) {
+			left = *cbd1;
+			left.pos = 0.0f;
+			cbd2 = &left;
+		}
+		else {
+			cbd2 = cbd1 - 1;
+		}
+
+		if ((in >= cbd1->pos) && ELEM(ipotype, COLBAND_INTERP_LINEAR, COLBAND_INTERP_EASE)) {
 			out[0] = cbd1->r;
 			out[1] = cbd1->g;
 			out[2] = cbd1->b;
 			out[3] = cbd1->a;
 		}
 		else {
-			CBData left, right;
-			
-			/* we're looking for first pos > in */
-			for (a = 0; a < coba->tot; a++, cbd1++) if (cbd1->pos > in) break;
-				
-			if (a == coba->tot) {
-				cbd2 = cbd1 - 1;
-				right = *cbd2;
-				right.pos = 1.0f;
-				cbd1 = &right;
-			}
-			else if (a == 0) {
-				left = *cbd1;
-				left.pos = 0.0f;
-				cbd2 = &left;
-			}
-			else cbd2 = cbd1 - 1;
-			
-			if (in >= cbd1->pos && coba->ipotype < 2) {
-				out[0] = cbd1->r;
-				out[1] = cbd1->g;
-				out[2] = cbd1->b;
-				out[3] = cbd1->a;
+
+			if (cbd2->pos != cbd1->pos) {
+				fac = (in - cbd1->pos) / (cbd2->pos - cbd1->pos);
 			}
 			else {
-		
-				if (cbd2->pos != cbd1->pos)
-					fac = (in - cbd1->pos) / (cbd2->pos - cbd1->pos);
-				else {
-					/* was setting to 0.0 in 2.56 & previous, but this
-					 * is incorrect for the last element, see [#26732] */
-					fac = (a != coba->tot) ? 0.0f : 1.0f;
-				}
-				
-				if (coba->ipotype == 4) {
-					/* constant */
-					out[0] = cbd2->r;
-					out[1] = cbd2->g;
-					out[2] = cbd2->b;
-					out[3] = cbd2->a;
-					return 1;
-				}
-				
-				if (coba->ipotype >= 2) {
-					/* ipo from right to left: 3 2 1 0 */
-					
-					if (a >= coba->tot - 1) cbd0 = cbd1;
-					else cbd0 = cbd1 + 1;
-					if (a < 2) cbd3 = cbd2;
-					else cbd3 = cbd2 - 1;
-					
-					CLAMP(fac, 0.0f, 1.0f);
-					
-					if (coba->ipotype == 3)
-						key_curve_position_weights(fac, t, KEY_CARDINAL);
-					else
-						key_curve_position_weights(fac, t, KEY_BSPLINE);
+				/* was setting to 0.0 in 2.56 & previous, but this
+				 * is incorrect for the last element, see [#26732] */
+				fac = (a != coba->tot) ? 0.0f : 1.0f;
+			}
 
-					out[0] = t[3] * cbd3->r + t[2] * cbd2->r + t[1] * cbd1->r + t[0] * cbd0->r;
-					out[1] = t[3] * cbd3->g + t[2] * cbd2->g + t[1] * cbd1->g + t[0] * cbd0->g;
-					out[2] = t[3] * cbd3->b + t[2] * cbd2->b + t[1] * cbd1->b + t[0] * cbd0->b;
-					out[3] = t[3] * cbd3->a + t[2] * cbd2->a + t[1] * cbd1->a + t[0] * cbd0->a;
-					CLAMP(out[0], 0.0f, 1.0f);
-					CLAMP(out[1], 0.0f, 1.0f);
-					CLAMP(out[2], 0.0f, 1.0f);
-					CLAMP(out[3], 0.0f, 1.0f);
+			if (ipotype == COLBAND_INTERP_CONSTANT) {
+				/* constant */
+				out[0] = cbd2->r;
+				out[1] = cbd2->g;
+				out[2] = cbd2->b;
+				out[3] = cbd2->a;
+			}
+			else if (ipotype >= COLBAND_INTERP_B_SPLINE) {
+				/* ipo from right to left: 3 2 1 0 */
+				float t[4];
+
+				if (a >= coba->tot - 1) cbd0 = cbd1;
+				else cbd0 = cbd1 + 1;
+				if (a < 2) cbd3 = cbd2;
+				else cbd3 = cbd2 - 1;
+
+				CLAMP(fac, 0.0f, 1.0f);
+
+				if (ipotype == COLBAND_INTERP_CARDINAL) {
+					key_curve_position_weights(fac, t, KEY_CARDINAL);
 				}
 				else {
-				
-					if (coba->ipotype == 1) { /* EASE */
-						mfac = fac * fac;
-						fac = 3.0f * mfac - 2.0f * mfac * fac;
-					}
-					mfac = 1.0f - fac;
-					
+					key_curve_position_weights(fac, t, KEY_BSPLINE);
+				}
+
+				out[0] = t[3] * cbd3->r + t[2] * cbd2->r + t[1] * cbd1->r + t[0] * cbd0->r;
+				out[1] = t[3] * cbd3->g + t[2] * cbd2->g + t[1] * cbd1->g + t[0] * cbd0->g;
+				out[2] = t[3] * cbd3->b + t[2] * cbd2->b + t[1] * cbd1->b + t[0] * cbd0->b;
+				out[3] = t[3] * cbd3->a + t[2] * cbd2->a + t[1] * cbd1->a + t[0] * cbd0->a;
+				CLAMP(out[0], 0.0f, 1.0f);
+				CLAMP(out[1], 0.0f, 1.0f);
+				CLAMP(out[2], 0.0f, 1.0f);
+				CLAMP(out[3], 0.0f, 1.0f);
+			}
+			else {
+				float mfac;
+
+				if (ipotype == COLBAND_INTERP_EASE) {
+					mfac = fac * fac;
+					fac = 3.0f * mfac - 2.0f * mfac * fac;
+				}
+
+				mfac = 1.0f - fac;
+
+				if (UNLIKELY(coba->color_mode == COLBAND_BLEND_HSV)) {
+					float col1[3], col2[3];
+
+					rgb_to_hsv_v(&cbd1->r, col1);
+					rgb_to_hsv_v(&cbd2->r, col2);
+
+					out[0] = colorband_hue_interp(coba->ipotype_hue, mfac, fac, col1[0], col2[0]);
+					out[1] = mfac * col1[1] + fac * col2[1];
+					out[2] = mfac * col1[2] + fac * col2[2];
+					out[3] = mfac * cbd1->a + fac * cbd2->a;
+
+					hsv_to_rgb_v(out, out);
+				}
+				else if (UNLIKELY(coba->color_mode == COLBAND_BLEND_HSL)) {
+					float col1[3], col2[3];
+
+					rgb_to_hsl_v(&cbd1->r, col1);
+					rgb_to_hsl_v(&cbd2->r, col2);
+
+					out[0] = colorband_hue_interp(coba->ipotype_hue, mfac, fac, col1[0], col2[0]);
+					out[1] = mfac * col1[1] + fac * col2[1];
+					out[2] = mfac * col1[2] + fac * col2[2];
+					out[3] = mfac * cbd1->a + fac * cbd2->a;
+
+					hsl_to_rgb_v(out, out);
+				}
+				else {
+					/* COLBAND_BLEND_RGB */
 					out[0] = mfac * cbd1->r + fac * cbd2->r;
 					out[1] = mfac * cbd1->g + fac * cbd2->g;
 					out[2] = mfac * cbd1->b + fac * cbd2->b;
@@ -324,7 +463,7 @@ int do_colorband(const ColorBand *coba, float in, float out[4])
 			}
 		}
 	}
-	return 1;   /* OK */
+	return true;   /* OK */
 }
 
 void colorband_table_RGBA(ColorBand *coba, float **array, int *size)
@@ -372,19 +511,18 @@ CBData *colorband_element_add(struct ColorBand *coba, float position)
 	if (coba->tot == MAXCOLORBAND) {
 		return NULL;
 	}
-	else if (coba->tot > 0) {
+	else {
 		CBData *xnew;
-		float col[4];
-
-		do_colorband(coba, position, col);
 
 		xnew = &coba->data[coba->tot];
 		xnew->pos = position;
 
-		xnew->r = col[0];
-		xnew->g = col[1];
-		xnew->b = col[2];
-		xnew->a = col[3];
+		if (coba->tot != 0) {
+			do_colorband(coba, position, &xnew->r);
+		}
+		else {
+			zero_v4(&xnew->r);
+		}
 	}
 
 	coba->tot++;
@@ -438,7 +576,8 @@ void BKE_texture_free(Tex *tex)
 
 void default_tex(Tex *tex)
 {
-	tex->type = TEX_CLOUDS;
+	tex->type = TEX_IMAGE;
+	tex->ima = NULL;
 	tex->stype = 0;
 	tex->flag = TEX_CHECKER_ODD;
 	tex->imaflag = TEX_INTERPOL | TEX_MIPMAP | TEX_USEALPHA;
@@ -541,12 +680,11 @@ void tex_set_type(Tex *tex, int type)
 
 /* ------------------------------------------------------------------------- */
 
-Tex *add_texture(const char *name)
+Tex *add_texture(Main *bmain, const char *name)
 {
-	Main *bmain = G.main;
 	Tex *tex;
 
-	tex = BKE_libblock_alloc(&bmain->tex, ID_TE, name);
+	tex = BKE_libblock_alloc(bmain, ID_TE, name);
 	
 	default_tex(tex);
 	
@@ -557,7 +695,7 @@ Tex *add_texture(const char *name)
 
 void default_mtex(MTex *mtex)
 {
-	mtex->texco = TEXCO_ORCO;
+	mtex->texco = TEXCO_UV;
 	mtex->mapto = MAP_COL;
 	mtex->object = NULL;
 	mtex->projx = PROJ_X;
@@ -571,7 +709,7 @@ void default_mtex(MTex *mtex)
 	mtex->size[1] = 1.0;
 	mtex->size[2] = 1.0;
 	mtex->tex = NULL;
-	mtex->texflag = MTEX_3TAP_BUMP | MTEX_BUMP_OBJECTSPACE;
+	mtex->texflag = MTEX_3TAP_BUMP | MTEX_BUMP_OBJECTSPACE | MTEX_MAPTO_BOUNDS;
 	mtex->colormodel = 0;
 	mtex->r = 1.0;
 	mtex->g = 0.0;
@@ -616,6 +754,7 @@ void default_mtex(MTex *mtex)
 	mtex->gravityfac = 1.0f;
 	mtex->fieldfac = 1.0f;
 	mtex->normapspace = MTEX_NSPACE_TANGENT;
+	mtex->brush_map_mode = MTEX_MAP_MODE_TILED;
 }
 
 
@@ -646,7 +785,7 @@ MTex *add_mtex_id(ID *id, int slot)
 	
 	if (slot == -1) {
 		/* find first free */
-		int i;		
+		int i;
 		for (i = 0; i < MAX_MTEX; i++) {
 			if (!mtex_ar[i]) {
 				slot = i;
@@ -669,6 +808,10 @@ MTex *add_mtex_id(ID *id, int slot)
 		MEM_freeN(mtex_ar[slot]);
 		mtex_ar[slot] = NULL;
 	}
+	else if (GS(id->name) == ID_MA) {
+		/* Reset this slot's ON/OFF toggle, for materials, when slot was empty. */
+		((Material *)id)->septex &= ~(1 << slot);
+	}
 
 	mtex_ar[slot] = add_mtex();
 
@@ -694,7 +837,7 @@ Tex *BKE_texture_copy(Tex *tex)
 
 	if (tex->nodetree) {
 		if (tex->nodetree->execdata) {
-			ntreeTexEndExecTree(tex->nodetree->execdata, 1);
+			ntreeTexEndExecTree(tex->nodetree->execdata);
 		}
 		texn->nodetree = ntreeCopyTree(tex->nodetree);
 	}
@@ -707,8 +850,7 @@ Tex *localize_texture(Tex *tex)
 {
 	Tex *texn;
 	
-	texn = BKE_libblock_copy(&tex->id);
-	BLI_remlink(&G.main->tex, texn);
+	texn = BKE_libblock_copy_nolib(&tex->id, false);
 	
 	/* image texture: BKE_texture_free also doesn't decrease */
 	
@@ -752,7 +894,9 @@ void BKE_texture_make_local(Tex *tex)
 	Lamp *la;
 	Brush *br;
 	ParticleSettings *pa;
-	int a, is_local = FALSE, is_lib = FALSE;
+	FreestyleLineStyle *ls;
+	int a;
+	bool is_local = false, is_lib = false;
 
 	/* - only lib users: do nothing
 	 * - only local users: set flag
@@ -771,8 +915,8 @@ void BKE_texture_make_local(Tex *tex)
 	while (ma) {
 		for (a = 0; a < MAX_MTEX; a++) {
 			if (ma->mtex[a] && ma->mtex[a]->tex == tex) {
-				if (ma->id.lib) is_lib = TRUE;
-				else is_local = TRUE;
+				if (ma->id.lib) is_lib = true;
+				else is_local = true;
 			}
 		}
 		ma = ma->id.next;
@@ -781,8 +925,8 @@ void BKE_texture_make_local(Tex *tex)
 	while (la) {
 		for (a = 0; a < MAX_MTEX; a++) {
 			if (la->mtex[a] && la->mtex[a]->tex == tex) {
-				if (la->id.lib) is_lib = TRUE;
-				else is_local = TRUE;
+				if (la->id.lib) is_lib = true;
+				else is_local = true;
 			}
 		}
 		la = la->id.next;
@@ -791,8 +935,8 @@ void BKE_texture_make_local(Tex *tex)
 	while (wrld) {
 		for (a = 0; a < MAX_MTEX; a++) {
 			if (wrld->mtex[a] && wrld->mtex[a]->tex == tex) {
-				if (wrld->id.lib) is_lib = TRUE;
-				else is_local = TRUE;
+				if (wrld->id.lib) is_lib = true;
+				else is_local = true;
 			}
 		}
 		wrld = wrld->id.next;
@@ -800,8 +944,12 @@ void BKE_texture_make_local(Tex *tex)
 	br = bmain->brush.first;
 	while (br) {
 		if (br->mtex.tex == tex) {
-			if (br->id.lib) is_lib = TRUE;
-			else is_local = TRUE;
+			if (br->id.lib) is_lib = true;
+			else is_local = true;
+		}
+		if (br->mask_mtex.tex == tex) {
+			if (br->id.lib) is_lib = true;
+			else is_local = true;
 		}
 		br = br->id.next;
 	}
@@ -809,14 +957,24 @@ void BKE_texture_make_local(Tex *tex)
 	while (pa) {
 		for (a = 0; a < MAX_MTEX; a++) {
 			if (pa->mtex[a] && pa->mtex[a]->tex == tex) {
-				if (pa->id.lib) is_lib = TRUE;
-				else is_local = TRUE;
+				if (pa->id.lib) is_lib = true;
+				else is_local = true;
 			}
 		}
 		pa = pa->id.next;
 	}
+	ls = bmain->linestyle.first;
+	while (ls) {
+		for (a = 0; a < MAX_MTEX; a++) {
+			if (ls->mtex[a] && ls->mtex[a]->tex == tex) {
+				if (ls->id.lib) is_lib = true;
+				else is_local = true;
+			}
+		}
+		ls = ls->id.next;
+	}
 	
-	if (is_local && is_lib == FALSE) {
+	if (is_local && is_lib == false) {
 		id_clear_lib_data(bmain, &tex->id);
 		extern_local_texture(tex);
 	}
@@ -876,6 +1034,13 @@ void BKE_texture_make_local(Tex *tex)
 					tex->id.us--;
 				}
 			}
+			if (br->mask_mtex.tex == tex) {
+				if (br->id.lib == NULL) {
+					br->mask_mtex.tex = tex_new;
+					tex_new->id.us++;
+					tex->id.us--;
+				}
+			}
 			br = br->id.next;
 		}
 		pa = bmain->particle.first;
@@ -891,42 +1056,21 @@ void BKE_texture_make_local(Tex *tex)
 			}
 			pa = pa->id.next;
 		}
-	}
-}
-
-/* ------------------------------------------------------------------------- */
-#if 0 /* UNUSED */
-void autotexname(Tex *tex)
-{
-	Main *bmain = G.main;
-	char texstr[20][15] = {"None", "Clouds", "Wood", "Marble", "Magic", "Blend",
-		                   "Stucci", "Noise", "Image", "EnvMap", "Musgrave",
-		                   "Voronoi", "DistNoise", "Point Density", "Voxel Data", "Ocean", "", "", ""};
-	Image *ima;
-	char di[FILE_MAXDIR], fi[FILE_MAXFILE];
-	
-	if (tex) {
-		if (tex->use_nodes) {
-			new_id(&bmain->tex, (ID *)tex, "Noddy");
-		}
-		else
-		if (tex->type == TEX_IMAGE) {
-			ima = tex->ima;
-			if (ima) {
-				BLI_strncpy(di, ima->name, sizeof(di));
-				BLI_splitdirstring(di, fi);
-				strcpy(di, "I.");
-				strcat(di, fi);
-				new_id(&bmain->tex, (ID *)tex, di);
+		ls = bmain->linestyle.first;
+		while (ls) {
+			for (a = 0; a < MAX_MTEX; a++) {
+				if (ls->mtex[a] && ls->mtex[a]->tex == tex) {
+					if (ls->id.lib == NULL) {
+						ls->mtex[a]->tex = tex_new;
+						tex_new->id.us++;
+						tex->id.us--;
+					}
+				}
 			}
-			else new_id(&bmain->tex, (ID *)tex, texstr[tex->type]);
+			ls = ls->id.next;
 		}
-		else new_id(&bmain->tex, (ID *)tex, texstr[tex->type]);
 	}
 }
-#endif
-
-/* ------------------------------------------------------------------------- */
 
 Tex *give_current_object_texture(Object *ob)
 {
@@ -986,6 +1130,41 @@ void set_current_lamp_texture(Lamp *la, Tex *newtex)
 	}
 }
 
+Tex *give_current_linestyle_texture(FreestyleLineStyle *linestyle)
+{
+	MTex *mtex = NULL;
+	Tex *tex = NULL;
+
+	if (linestyle) {
+		mtex = linestyle->mtex[(int)(linestyle->texact)];
+		if (mtex) tex = mtex->tex;
+	}
+
+	return tex;
+}
+
+void set_current_linestyle_texture(FreestyleLineStyle *linestyle, Tex *newtex)
+{
+	int act = linestyle->texact;
+
+	if (linestyle->mtex[act] && linestyle->mtex[act]->tex)
+		id_us_min(&linestyle->mtex[act]->tex->id);
+
+	if (newtex) {
+		if (!linestyle->mtex[act]) {
+			linestyle->mtex[act] = add_mtex();
+			linestyle->mtex[act]->texco = TEXCO_STROKE;
+		}
+
+		linestyle->mtex[act]->tex = newtex;
+		id_us_plus(&newtex->id);
+	}
+	else if (linestyle->mtex[act]) {
+		MEM_freeN(linestyle->mtex[act]);
+		linestyle->mtex[act] = NULL;
+	}
+}
+
 bNode *give_current_material_texture_node(Material *ma)
 {
 	if (ma && ma->use_nodes && ma->nodetree)
@@ -1020,7 +1199,7 @@ Tex *give_current_material_texture(Material *ma)
 	return tex;
 }
 
-int give_active_mtex(ID *id, MTex ***mtex_ar, short *act)
+bool give_active_mtex(ID *id, MTex ***mtex_ar, short *act)
 {
 	switch (GS(id->name)) {
 		case ID_MA:
@@ -1035,6 +1214,10 @@ int give_active_mtex(ID *id, MTex ***mtex_ar, short *act)
 			*mtex_ar =       ((Lamp *)id)->mtex;
 			if (act) *act =  (((Lamp *)id)->texact);
 			break;
+		case ID_LS:
+			*mtex_ar =       ((FreestyleLineStyle *)id)->mtex;
+			if (act) *act =  (((FreestyleLineStyle *)id)->texact);
+			break;
 		case ID_PA:
 			*mtex_ar =       ((ParticleSettings *)id)->mtex;
 			if (act) *act =  (((ParticleSettings *)id)->texact);
@@ -1042,10 +1225,10 @@ int give_active_mtex(ID *id, MTex ***mtex_ar, short *act)
 		default:
 			*mtex_ar = NULL;
 			if (act) *act =  0;
-			return FALSE;
+			return false;
 	}
 
-	return TRUE;
+	return true;
 }
 
 void set_active_mtex(ID *id, short act)
@@ -1063,6 +1246,9 @@ void set_active_mtex(ID *id, short act)
 		case ID_LA:
 			((Lamp *)id)->texact = act;
 			break;
+		case ID_LS:
+			((FreestyleLineStyle *)id)->texact = act;
+			break;
 		case ID_PA:
 			((ParticleSettings *)id)->texact = act;
 			break;
@@ -1094,8 +1280,11 @@ void set_current_material_texture(Material *ma, Tex *newtex)
 		id_us_min(&tex->id);
 
 		if (newtex) {
-			if (!ma->mtex[act])
+			if (!ma->mtex[act]) {
 				ma->mtex[act] = add_mtex();
+				/* Reset this slot's ON/OFF toggle, for materials, when slot was empty. */
+				ma->septex &= ~(1 << act);
+			}
 			
 			ma->mtex[act]->tex = newtex;
 			id_us_plus(&newtex->id);
@@ -1107,7 +1296,7 @@ void set_current_material_texture(Material *ma, Tex *newtex)
 	}
 }
 
-int has_current_material_texture(Material *ma)
+bool has_current_material_texture(Material *ma)
 {
 	bNode *node;
 
@@ -1115,7 +1304,7 @@ int has_current_material_texture(Material *ma)
 		node = nodeGetActiveID(ma->nodetree, ID_TE);
 
 		if (node)
-			return 1;
+			return true;
 	}
 
 	return (ma != NULL);
@@ -1282,7 +1471,7 @@ PointDensity *BKE_add_pointdensity(void)
 	pd->noise_depth = 1;
 	pd->noise_fac = 1.0f;
 	pd->noise_influence = TEX_PD_NOISE_STATIC;
-	pd->coba = add_colorband(1);
+	pd->coba = add_colorband(true);
 	pd->speed_scale = 1.0f;
 	pd->totpoints = 0;
 	pd->object = NULL;
@@ -1293,7 +1482,7 @@ PointDensity *BKE_add_pointdensity(void)
 	pd->falloff_curve->preset = CURVE_PRESET_LINE;
 	pd->falloff_curve->cm->flag &= ~CUMA_EXTEND_EXTRAPOLATE;
 	curvemap_reset(pd->falloff_curve->cm, &pd->falloff_curve->clipr, pd->falloff_curve->preset, CURVEMAP_SLOPE_POSITIVE);
-	curvemapping_changed(pd->falloff_curve, FALSE);
+	curvemapping_changed(pd->falloff_curve, false);
 
 	return pd;
 } 
@@ -1406,22 +1595,44 @@ void BKE_free_oceantex(struct OceanTex *ot)
 
 
 /* ------------------------------------------------------------------------- */
-int BKE_texture_dependsOnTime(const struct Tex *texture)
+bool BKE_texture_dependsOnTime(const struct Tex *texture)
 {
-	if (texture->ima &&
-	         ELEM(texture->ima->source, IMA_SRC_SEQUENCE, IMA_SRC_MOVIE))
-	{
-		return 1;
+	if (texture->ima && BKE_image_is_animated(texture->ima)) {
+		return true;
 	}
 	else if (texture->adt) {
 		/* assume anything in adt means the texture is animated */
-		return 1;
+		return true;
 	}
 	else if (texture->type == TEX_NOISE) {
 		/* noise always varies with time */
-		return 1;
+		return true;
 	}
-	return 0;
+	return false;
 }
 
 /* ------------------------------------------------------------------------- */
+
+void BKE_texture_get_value(Scene *scene, Tex *texture, float *tex_co, TexResult *texres, bool use_color_management)
+{
+	int result_type;
+	bool do_color_manage = false;
+
+	if (scene && use_color_management) {
+		do_color_manage = BKE_scene_check_color_management_enabled(scene);
+	}
+
+	/* no node textures for now */
+	result_type = multitex_ext_safe(texture, tex_co, texres, NULL, do_color_manage);
+
+	/* if the texture gave an RGB value, we assume it didn't give a valid
+	 * intensity, since this is in the context of modifiers don't use perceptual color conversion.
+	 * if the texture didn't give an RGB value, copy the intensity across
+	 */
+	if (result_type & TEX_RGB) {
+		texres->tin = (1.0f / 3.0f) * (texres->tr + texres->tg + texres->tb);
+	}
+	else {
+		copy_v3_fl(&texres->tr, texres->tin);
+	}
+}
