@@ -38,20 +38,20 @@
 #include "DNA_scene_types.h"
 #include "DNA_texture_types.h"
 
-#include "BLI_listbase.h"
 #include "BLI_utildefines.h"
-#include "BLI_bpath.h"
+#include "BLI_listbase.h"
 
 #include "BKE_animsys.h"
 #include "BKE_global.h"
 #include "BKE_icons.h"
 #include "BKE_library.h"
-#include "BKE_library.h"
 #include "BKE_main.h"
 #include "BKE_node.h"
 #include "BKE_world.h"
 
-void BKE_world_free_ex(World *wrld, int do_id_user)
+#include "GPU_material.h"
+
+void BKE_world_free_ex(World *wrld, bool do_id_user)
 {
 	MTex *mtex;
 	int a;
@@ -71,21 +71,23 @@ void BKE_world_free_ex(World *wrld, int do_id_user)
 		MEM_freeN(wrld->nodetree);
 	}
 
+	if (wrld->gpumaterial.first)
+		GPU_material_free(&wrld->gpumaterial);
+	
 	BKE_icon_delete((struct ID *)wrld);
 	wrld->id.icon_id = 0;
 }
 
 void BKE_world_free(World *wrld)
 {
-	BKE_world_free_ex(wrld, TRUE);
+	BKE_world_free_ex(wrld, true);
 }
 
-World *add_world(const char *name)
+World *add_world(Main *bmain, const char *name)
 {
-	Main *bmain = G.main;
 	World *wrld;
 
-	wrld = BKE_libblock_alloc(&bmain->world, ID_WO, name);
+	wrld = BKE_libblock_alloc(bmain, ID_WO, name);
 	
 	wrld->horr = 0.05f;
 	wrld->horg = 0.05f;
@@ -94,8 +96,6 @@ World *add_world(const char *name)
 	wrld->zeng = 0.01f;
 	wrld->zenb = 0.01f;
 	wrld->skytype = 0;
-	wrld->stardist = 15.0f;
-	wrld->starsize = 2.0f;
 
 	wrld->exp = 0.0f;
 	wrld->exposure = wrld->range = 1.0f;
@@ -107,7 +107,7 @@ World *add_world(const char *name)
 	wrld->ao_indirect_energy = 1.0f;
 	wrld->ao_indirect_bounces = 1;
 	wrld->aobias = 0.05f;
-	wrld->ao_samp_method = WO_AOSAMP_HAMMERSLEY;	
+	wrld->ao_samp_method = WO_AOSAMP_HAMMERSLEY;
 	wrld->ao_approx_error = 0.25f;
 	
 	wrld->preview = NULL;
@@ -139,6 +139,8 @@ World *BKE_world_copy(World *wrld)
 	if (wrld->preview)
 		wrldn->preview = BKE_previewimg_copy(wrld->preview);
 
+	BLI_listbase_clear(&wrldn->gpumaterial);
+	
 	return wrldn;
 }
 
@@ -147,8 +149,7 @@ World *localize_world(World *wrld)
 	World *wrldn;
 	int a;
 	
-	wrldn = BKE_libblock_copy(&wrld->id);
-	BLI_remlink(&G.main->world, wrldn);
+	wrldn = BKE_libblock_copy_nolib(&wrld->id, false);
 	
 	for (a = 0; a < MAX_MTEX; a++) {
 		if (wrld->mtex[a]) {
@@ -164,6 +165,8 @@ World *localize_world(World *wrld)
 	
 	wrldn->preview = NULL;
 	
+	BLI_listbase_clear(&wrldn->gpumaterial);
+	
 	return wrldn;
 }
 
@@ -171,7 +174,7 @@ void BKE_world_make_local(World *wrld)
 {
 	Main *bmain = G.main;
 	Scene *sce;
-	int is_local = FALSE, is_lib = FALSE;
+	bool is_local = false, is_lib = false;
 
 	/* - only lib users: do nothing
 	 * - only local users: set flag
@@ -184,14 +187,14 @@ void BKE_world_make_local(World *wrld)
 		return;
 	}
 	
-	for (sce = bmain->scene.first; sce && ELEM(FALSE, is_lib, is_local); sce = sce->id.next) {
+	for (sce = bmain->scene.first; sce && ELEM(false, is_lib, is_local); sce = sce->id.next) {
 		if (sce->world == wrld) {
-			if (sce->id.lib) is_lib = TRUE;
-			else is_local = TRUE;
+			if (sce->id.lib) is_lib = true;
+			else is_local = true;
 		}
 	}
 
-	if (is_local && is_lib == FALSE) {
+	if (is_local && is_lib == false) {
 		id_clear_lib_data(bmain, &wrld->id);
 	}
 	else if (is_local && is_lib) {
