@@ -30,13 +30,12 @@
  */
 
 
-#include "zlib.h"
-
 #include <limits.h>
 
 #ifndef WIN32
 #  include <unistd.h> // for read close
 #else
+#  include <zlib.h>  /* odd include order-issue */
 #  include <io.h> // for open close read
 #  include "winsock2.h"
 #  include "BLI_winstuff.h"
@@ -77,14 +76,12 @@
 #include "BLI_utildefines.h"
 #include "BLI_blenlib.h"
 #include "BLI_math.h"
-#include "BLI_edgehash.h"
 
 #include "BKE_armature.h"
 #include "BKE_colortools.h"
 #include "BKE_constraint.h"
 #include "BKE_deform.h"
 #include "BKE_fcurve.h"
-#include "BKE_global.h" // for G
 #include "BKE_image.h"
 #include "BKE_lattice.h"
 #include "BKE_main.h" // for Main
@@ -96,14 +93,9 @@
 #include "BKE_scene.h"
 #include "BKE_sequencer.h"
 
-#include "IMB_imbuf.h"  // for proxy / timecode versioning stuff
-
 #include "NOD_socket.h"
 
 #include "BLO_readfile.h"
-#include "BLO_undofile.h"
-
-#include "RE_engine.h"
 
 #include "readfile.h"
 
@@ -289,11 +281,10 @@ static void ntree_version_245(FileData *fd, Library *lib, bNodeTree *ntree)
 				iuser = node->storage;
 				if (iuser->flag & IMA_OLD_PREMUL) {
 					iuser->flag &= ~IMA_OLD_PREMUL;
-					iuser->flag |= IMA_DO_PREMUL;
 				}
 				if (iuser->flag & IMA_DO_PREMUL) {
 					image->flag &= ~IMA_OLD_PREMUL;
-					image->flag |= IMA_DO_PREMUL;
+					image->alpha_mode = IMA_ALPHA_STRAIGHT;
 				}
 			}
 		}
@@ -355,7 +346,7 @@ static void alphasort_version_246(FileData *fd, Library *lib, Mesh *me)
 				ma = NULL;
 
 			for (b = 0; ma && b < MAX_MTEX; b++)
-				if (ma->mtex && ma->mtex[b] && ma->mtex[b]->mapto & MAP_ALPHA)
+				if (ma->mtex[b] && ma->mtex[b]->mapto & MAP_ALPHA)
 					texalpha = 1;
 		}
 		else {
@@ -454,7 +445,7 @@ static void customdata_version_242(Mesh *me)
 		}
 	}
 
-	mesh_update_customdata_pointers(me, TRUE);
+	BKE_mesh_update_customdata_pointers(me, true);
 }
 
 /*only copy render texface layer from active*/
@@ -476,7 +467,7 @@ static void do_version_ntree_242_2(bNodeTree *ntree)
 
 	if (ntree->type == NTREE_COMPOSIT) {
 		for (node = ntree->nodes.first; node; node = node->next) {
-			if (ELEM3(node->type, CMP_NODE_IMAGE, CMP_NODE_VIEWER, CMP_NODE_SPLITVIEWER)) {
+			if (ELEM(node->type, CMP_NODE_IMAGE, CMP_NODE_VIEWER, CMP_NODE_SPLITVIEWER)) {
 				/* only image had storage */
 				if (node->storage) {
 					NodeImageAnim *nia = node->storage;
@@ -519,11 +510,8 @@ static void do_version_free_effects_245(ListBase *lb)
 {
 	Effect *eff;
 
-	eff = lb->first;
-	while (eff) {
-		BLI_remlink(lb, eff);
+	while ((eff = BLI_pophead(lb))) {
 		do_version_free_effect_245(eff);
-		eff = lb->first;
 	}
 }
 
@@ -545,7 +533,7 @@ void blo_do_version_old_trackto_to_constraints(Object *ob)
 {
 	/* create new trackto constraint from the relationship */
 	if (ob->track) {
-		bConstraint *con = add_ob_constraint(ob, "AutoTrack", CONSTRAINT_TYPE_TRACKTO);
+		bConstraint *con = BKE_constraint_add_for_object(ob, "AutoTrack", CONSTRAINT_TYPE_TRACKTO);
 		bTrackToConstraint *data = con->data;
 
 		/* copy tracking settings from the object */
@@ -1237,7 +1225,7 @@ void blo_do_versions_pre250(FileData *fd, Library *lib, Main *main)
 					TFace *tf = &((TFace*) me->tface)[i];
 
 					for (j = 0; j < 4; j++) {
-						char *col = (char*) &tf->col[j];
+						char *col = (char *) &tf->col[j];
 
 						col[0] = 255;
 					}
@@ -1548,7 +1536,9 @@ void blo_do_versions_pre250(FileData *fd, Library *lib, Main *main)
 						else if (sbuts->mainb == BUTS_EDIT) {
 							sbuts->mainb = CONTEXT_EDITING;
 						}
-						else sbuts->mainb = CONTEXT_SCENE;
+						else {
+							sbuts->mainb = CONTEXT_SCENE;
+						}
 					}
 				}
 			}
@@ -1840,7 +1830,7 @@ void blo_do_versions_pre250(FileData *fd, Library *lib, Main *main)
 				SEQ_BEGIN (sce->ed, seq)
 				{
 					if (seq->type == SEQ_TYPE_IMAGE || seq->type == SEQ_TYPE_MOVIE)
-						seq->flag |= SEQ_MAKE_PREMUL;
+						seq->alpha_mode = SEQ_ALPHA_STRAIGHT;
 				}
 				SEQ_END
 			}
@@ -1996,15 +1986,7 @@ void blo_do_versions_pre250(FileData *fd, Library *lib, Main *main)
 		while (sce) {
 			if (sce->toolsettings == NULL) {
 				sce->toolsettings = MEM_callocN(sizeof(struct ToolSettings), "Tool Settings Struct");
-				sce->toolsettings->cornertype =0;
-				sce->toolsettings->degr = 90;
-				sce->toolsettings->step = 9;
-				sce->toolsettings->turn = 1;
-				sce->toolsettings->extr_offs = 1;
 				sce->toolsettings->doublimit = 0.001f;
-				sce->toolsettings->segments = 32;
-				sce->toolsettings->rings = 32;
-				sce->toolsettings->vertices = 32;
 			}
 			sce = sce->id.next;
 		}
@@ -2091,7 +2073,7 @@ void blo_do_versions_pre250(FileData *fd, Library *lib, Main *main)
 
 		for (me = main->mesh.first; me; me = me->id.next) {
 			if (!me->medge) {
-				BKE_mesh_make_edges(me, 1);	/* 1 = use mface->edcode */
+				BKE_mesh_calc_edges_legacy(me, true);  /* true = use mface->edcode */
 			}
 			else {
 				BKE_mesh_strip_loose_faces(me);
@@ -2151,9 +2133,6 @@ void blo_do_versions_pre250(FileData *fd, Library *lib, Main *main)
 		}
 
 		for (; sce; sce = sce->id.next) {
-			/* make 'innervert' the default subdivide type, for backwards compat */
-			sce->toolsettings->cornertype = 1;
-
 			if (sce->r.scemode & R_PASSEPARTOUT) {
 				set_passepartout = 1;
 				sce->r.scemode &= ~R_PASSEPARTOUT;
@@ -2223,8 +2202,9 @@ void blo_do_versions_pre250(FileData *fd, Library *lib, Main *main)
 				sce->r.yparts = 4;
 
 			/* adds default layer */
-			if (sce->r.layers.first == NULL)
+			if (BLI_listbase_is_empty(&sce->r.layers)) {
 				BKE_scene_add_render_layer(sce, NULL);
+			}
 			else {
 				SceneRenderLayer *srl;
 				/* new layer flag for sky, was default for solid */
@@ -2240,17 +2220,13 @@ void blo_do_versions_pre250(FileData *fd, Library *lib, Main *main)
 				ntree_version_241(sce->nodetree);
 
 			/* uv calculation options moved to toolsettings */
-			if (sce->toolsettings->uvcalc_radius == 0.0f) {
-				sce->toolsettings->uvcalc_radius = 1.0f;
-				sce->toolsettings->uvcalc_cubesize = 1.0f;
-				sce->toolsettings->uvcalc_mapdir = 1;
-				sce->toolsettings->uvcalc_mapalign = 1;
+			if (sce->toolsettings->unwrapper == 0) {
 				sce->toolsettings->uvcalc_flag = UVCALC_FILLHOLES;
 				sce->toolsettings->unwrapper = 1;
 			}
 
 			if (sce->r.mode & R_PANORAMA) {
-				/* all these checks to ensure saved files with svn version keep working... */
+				/* all these checks to ensure saved files between released versions keep working... */
 				if (sce->r.xsch < sce->r.ysch) {
 					Object *obc = blo_do_versions_newlibadr(fd, lib, sce->camera);
 					if (obc && obc->type == OB_CAMERA) {
@@ -2305,7 +2281,7 @@ void blo_do_versions_pre250(FileData *fd, Library *lib, Main *main)
 			Image *ima;
 			for (ima = main->image.first; ima; ima = ima->id.next)
 				if (strcmp(ima->name, "Compositor") == 0) {
-					strcpy(ima->id.name+2, "Viewer Node");
+					strcpy(ima->id.name + 2, "Viewer Node");
 					strcpy(ima->name, "Viewer Node");
 				}
 		}
@@ -2345,8 +2321,6 @@ void blo_do_versions_pre250(FileData *fd, Library *lib, Main *main)
 		for (sce = main->scene.first; sce; sce = sce->id.next) {
 			if (sce->toolsettings->select_thresh == 0.0f)
 				sce->toolsettings->select_thresh = 0.01f;
-			if (sce->toolsettings->clean_thresh == 0.0f)
-				sce->toolsettings->clean_thresh = 0.1f;
 
 			if (sce->r.threads == 0) {
 				if (sce->r.mode & R_THREADS)
@@ -2494,11 +2468,11 @@ void blo_do_versions_pre250(FileData *fd, Library *lib, Main *main)
 				ima->gen_x = 256; ima->gen_y = 256;
 				ima->gen_type = 1;
 
-				if (0 == strncmp(ima->id.name+2, "Viewer Node", sizeof(ima->id.name) - 2)) {
+				if (0 == strncmp(ima->id.name + 2, "Viewer Node", sizeof(ima->id.name) - 2)) {
 					ima->source = IMA_SRC_VIEWER;
 					ima->type = IMA_TYPE_COMPOSITE;
 				}
-				if (0 == strncmp(ima->id.name+2, "Render Result", sizeof(ima->id.name) - 2)) {
+				if (0 == strncmp(ima->id.name + 2, "Render Result", sizeof(ima->id.name) - 2)) {
 					ima->source = IMA_SRC_VIEWER;
 					ima->type = IMA_TYPE_R_RESULT;
 				}
@@ -2551,19 +2525,11 @@ void blo_do_versions_pre250(FileData *fd, Library *lib, Main *main)
 		if (main->subversionfile < 4) {
 			for (sce = main->scene.first; sce; sce = sce->id.next) {
 				sce->r.bake_mode = 1;	/* prevent to include render stuff here */
-				sce->r.bake_filter = 2;
+				sce->r.bake_filter = 16;
 				sce->r.bake_osa = 5;
 				sce->r.bake_flag = R_BAKE_CLEAR;
 			}
 		}
-
-		if (main->subversionfile < 5) {
-			for (sce = main->scene.first; sce; sce = sce->id.next) {
-				/* improved triangle to quad conversion settings */
-				if (sce->toolsettings->jointrilimit == 0.0f)
-					sce->toolsettings->jointrilimit = 0.8f;
-			}
-		}
 	}
 
 	if (main->versionfile <= 243) {
@@ -2808,8 +2774,10 @@ void blo_do_versions_pre250(FileData *fd, Library *lib, Main *main)
 			for (md = ob->modifiers.first; md; md = md->next) {
 				if (md->type == eModifierType_Cloth) {
 					ClothModifierData *clmd = (ClothModifierData*) md;
-					if (!clmd->point_cache)
+					if (!clmd->point_cache) {
 						clmd->point_cache = BKE_ptcache_add(&clmd->ptcaches);
+						clmd->point_cache->step = 1;
+					}
 				}
 			}
 		}
@@ -2901,20 +2869,19 @@ void blo_do_versions_pre250(FileData *fd, Library *lib, Main *main)
 		for (ima = main->image.first; ima; ima = ima->id.next) {
 			if (ima->flag & IMA_OLD_PREMUL) {
 				ima->flag &= ~IMA_OLD_PREMUL;
-				ima->flag |= IMA_DO_PREMUL;
+				ima->alpha_mode = IMA_ALPHA_STRAIGHT;
 			}
 		}
 
 		for (tex = main->tex.first; tex; tex = tex->id.next) {
 			if (tex->iuser.flag & IMA_OLD_PREMUL) {
 				tex->iuser.flag &= ~IMA_OLD_PREMUL;
-				tex->iuser.flag |= IMA_DO_PREMUL;
 			}
 
 			ima = blo_do_versions_newlibadr(fd, lib, tex->ima);
 			if (ima && (tex->iuser.flag & IMA_DO_PREMUL)) {
 				ima->flag &= ~IMA_OLD_PREMUL;
-				ima->flag |= IMA_DO_PREMUL;
+				ima->alpha_mode = IMA_ALPHA_STRAIGHT;
 			}
 		}
 	}
@@ -3107,12 +3074,12 @@ void blo_do_versions_pre250(FileData *fd, Library *lib, Main *main)
 				part->id.flag |= (ob->id.flag & LIB_NEED_LINK);
 
 				psys->totpart = 0;
-				psys->flag = PSYS_ENABLED|PSYS_CURRENT;
+				psys->flag = PSYS_CURRENT;
 
 				BLI_addtail(&ob->particlesystem, psys);
 
 				md = modifier_new(eModifierType_ParticleSystem);
-				BLI_snprintf(md->name, sizeof(md->name), "ParticleSystem %i", BLI_countlist(&ob->particlesystem));
+				BLI_snprintf(md->name, sizeof(md->name), "ParticleSystem %i", BLI_listbase_count(&ob->particlesystem));
 				psmd = (ParticleSystemModifierData*) md;
 				psmd->psys = psys;
 				BLI_addtail(&ob->modifiers, md);
@@ -3597,8 +3564,6 @@ void blo_do_versions_pre250(FileData *fd, Library *lib, Main *main)
 		Object *ob;
 		World *wrld;
 		for (ob = main->object.first; ob; ob = ob->id.next) {
-			/* pad3 is used for m_contactProcessingThreshold */
-			ob->m_contactProcessingThreshold = 1.0f;
 			if (ob->parent) {
 				/* check if top parent has compound shape set and if yes, set this object
 				 * to compound shaper as well (was the behavior before, now it's optional) */
