@@ -33,15 +33,14 @@
 
 #include "MEM_guardedalloc.h"
 
-#include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 
+#include "BLI_alloca.h"
+#include "BLI_math.h"
+
 #include "BKE_customdata.h"
 #include "BKE_multires.h"
 
-#include "BLI_array.h"
-#include "BLI_math.h"
-
 #include "bmesh.h"
 #include "intern/bmesh_private.h"
 
@@ -55,7 +54,7 @@ static void bm_data_interp_from_elem(CustomData *data_layer, BMElem *ele1, BMEle
 				/* do nothing */
 			}
 			else {
-				CustomData_bmesh_free_block(data_layer, &ele_dst->head.data);
+				CustomData_bmesh_free_block_data(data_layer, ele_dst->head.data);
 				CustomData_bmesh_copy_data(data_layer, data_layer, ele1->head.data, &ele_dst->head.data);
 			}
 		}
@@ -64,7 +63,7 @@ static void bm_data_interp_from_elem(CustomData *data_layer, BMElem *ele1, BMEle
 				/* do nothing */
 			}
 			else {
-				CustomData_bmesh_free_block(data_layer, &ele_dst->head.data);
+				CustomData_bmesh_free_block_data(data_layer, ele_dst->head.data);
 				CustomData_bmesh_copy_data(data_layer, data_layer, ele2->head.data, &ele_dst->head.data);
 			}
 		}
@@ -126,7 +125,7 @@ void BM_data_interp_face_vert_edge(BMesh *bm, BMVert *v1, BMVert *UNUSED(v2), BM
 {
 	void *src[2];
 	float w[2];
-	BMLoop *v1loop = NULL, *vloop = NULL, *v2loop = NULL;
+	BMLoop *l_v1 = NULL, *l_v = NULL, *l_v2 = NULL;
 	BMLoop *l_iter = NULL;
 
 	if (!e1->l) {
@@ -139,23 +138,23 @@ void BM_data_interp_face_vert_edge(BMesh *bm, BMVert *v1, BMVert *UNUSED(v2), BM
 	l_iter = e1->l;
 	do {
 		if (l_iter->v == v1) {
-			v1loop = l_iter;
-			vloop = v1loop->next;
-			v2loop = vloop->next;
+			l_v1 = l_iter;
+			l_v = l_v1->next;
+			l_v2 = l_v->next;
 		}
 		else if (l_iter->v == v) {
-			v1loop = l_iter->next;
-			vloop = l_iter;
-			v2loop = l_iter->prev;
+			l_v1 = l_iter->next;
+			l_v = l_iter;
+			l_v2 = l_iter->prev;
 		}
 		
-		if (!v1loop || !v2loop)
+		if (!l_v1 || !l_v2)
 			return;
 		
-		src[0] = v1loop->head.data;
-		src[1] = v2loop->head.data;
+		src[0] = l_v1->head.data;
+		src[1] = l_v2->head.data;
 
-		CustomData_bmesh_interp(&bm->ldata, src, w, NULL, 2, vloop->head.data);
+		CustomData_bmesh_interp(&bm->ldata, src, w, NULL, 2, l_v->head.data);
 	} while ((l_iter = l_iter->radial_next) != e1->l);
 }
 
@@ -167,39 +166,57 @@ void BM_data_interp_face_vert_edge(BMesh *bm, BMVert *v1, BMVert *UNUSED(v2), BM
  *
  * \note Only handles loop customdata. multires is handled.
  */
-void BM_face_interp_from_face(BMesh *bm, BMFace *target, BMFace *source)
+void BM_face_interp_from_face_ex(BMesh *bm, BMFace *target, BMFace *source, const bool do_vertex,
+                                 void **blocks_l, void **blocks_v, float (*cos_2d)[2], float axis_mat[3][3])
 {
 	BMLoop *l_iter;
 	BMLoop *l_first;
 
-	void **blocks = NULL;
-	float (*cos)[3] = NULL, *w = NULL;
-	BLI_array_fixedstack_declare(cos,     BM_NGON_STACK_SIZE, source->len, __func__);
-	BLI_array_fixedstack_declare(w,       BM_NGON_STACK_SIZE, source->len, __func__);
-	BLI_array_fixedstack_declare(blocks,  BM_NGON_STACK_SIZE, source->len, __func__);
+	float *w = BLI_array_alloca(w, source->len);
+	float co[2];
 	int i;
-	
-	BM_elem_attrs_copy(bm, bm, source, target);
+
+	if (source != target)
+		BM_elem_attrs_copy(bm, bm, source, target);
+
+	/* interpolate */
+	i = 0;
+	l_iter = l_first = BM_FACE_FIRST_LOOP(target);
+	do {
+		mul_v2_m3v3(co, axis_mat, l_iter->v->co);
+		interp_weights_poly_v2(w, cos_2d, source->len, co);
+		CustomData_bmesh_interp(&bm->ldata, blocks_l, w, NULL, source->len, l_iter->head.data);
+		if (do_vertex) {
+			CustomData_bmesh_interp(&bm->vdata, blocks_v, w, NULL, source->len, l_iter->v->head.data);
+		}
+	} while (i++, (l_iter = l_iter->next) != l_first);
+}
+
+void BM_face_interp_from_face(BMesh *bm, BMFace *target, BMFace *source, const bool do_vertex)
+{
+	BMLoop *l_iter;
+	BMLoop *l_first;
+
+	void **blocks_l    = BLI_array_alloca(blocks_l, source->len);
+	void **blocks_v    = do_vertex ? BLI_array_alloca(blocks_v, source->len) : NULL;
+	float (*cos_2d)[2] = BLI_array_alloca(cos_2d, source->len);
+	float axis_mat[3][3];  /* use normal to transform into 2d xy coords */
+	int i;
+
+	/* convert the 3d coords into 2d for projection */
+	BLI_assert(BM_face_is_normal_valid(source));
+	axis_dominant_v3_to_m3(axis_mat, source->no);
 
 	i = 0;
 	l_iter = l_first = BM_FACE_FIRST_LOOP(source);
 	do {
-		copy_v3_v3(cos[i], l_iter->v->co);
-		blocks[i] = l_iter->head.data;
-		i++;
-	} while ((l_iter = l_iter->next) != l_first);
+		mul_v2_m3v3(cos_2d[i], axis_mat, l_iter->v->co);
+		blocks_l[i] = l_iter->head.data;
+		if (do_vertex) blocks_v[i] = l_iter->v->head.data;
+	} while (i++, (l_iter = l_iter->next) != l_first);
 
-	i = 0;
-	l_iter = l_first = BM_FACE_FIRST_LOOP(target);
-	do {
-		interp_weights_poly_v3(w, cos, source->len, l_iter->v->co);
-		CustomData_bmesh_interp(&bm->ldata, blocks, w, NULL, source->len, l_iter->head.data);
-		i++;
-	} while ((l_iter = l_iter->next) != l_first);
-
-	BLI_array_fixedstack_free(cos);
-	BLI_array_fixedstack_free(w);
-	BLI_array_fixedstack_free(blocks);
+	BM_face_interp_from_face_ex(bm, target, source, do_vertex,
+	                            blocks_l, blocks_v, cos_2d, axis_mat);
 }
 
 /**
@@ -224,10 +241,8 @@ static int compute_mdisp_quad(BMLoop *l, float v1[3], float v2[3], float v3[3],
 	/* computer center */
 	BM_face_calc_center_mean(l->f, cent);
 
-	add_v3_v3v3(p, l->prev->v->co, l->v->co);
-	mul_v3_fl(p, 0.5);
-	add_v3_v3v3(n, l->next->v->co, l->v->co);
-	mul_v3_fl(n, 0.5);
+	mid_v3_v3v3(p, l->prev->v->co, l->v->co);
+	mid_v3_v3v3(n, l->next->v->co, l->v->co);
 	
 	copy_v3_v3(v1, cent);
 	copy_v3_v3(v2, p);
@@ -241,24 +256,24 @@ static int compute_mdisp_quad(BMLoop *l, float v1[3], float v2[3], float v3[3],
 }
 
 /* funnily enough, I think this is identical to face_to_crn_interp, heh */
-static float quad_coord(float aa[3], float bb[3], float cc[3], float dd[3], int a1, int a2)
+static float quad_coord(const float aa[3], const float bb[3], const float cc[3], const float dd[3], int a1, int a2)
 {
 	float x, y, z, f1;
+	float div;
 	
 	x = aa[a1] * cc[a2] - cc[a1] * aa[a2];
 	y = aa[a1] * dd[a2] + bb[a1] * cc[a2] - cc[a1] * bb[a2] - dd[a1] * aa[a2];
 	z = bb[a1] * dd[a2] - dd[a1] * bb[a2];
-	
-	if (fabsf(2.0f * (x - y + z)) > FLT_EPSILON * 10.0f) {
-		float f2;
 
-		f1 = ( sqrtf(y * y - 4.0f * x * z) - y + 2.0f * z) / (2.0f * (x - y + z));
-		f2 = (-sqrtf(y * y - 4.0f * x * z) - y + 2.0f * z) / (2.0f * (x - y + z));
+	div = 2.0f * (x - y + z);
 
-		f1 = fabsf(f1);
-		f2 = fabsf(f2);
-		f1 = minf(f1, f2);
-		CLAMP(f1, 0.0f, 1.0f + FLT_EPSILON);
+	if (fabsf(div) > FLT_EPSILON * 10.0f) {
+		const float f_tmp = sqrtf(y * y - 4.0f * x * z);
+
+		f1 = min_ff(fabsf(( f_tmp - y + 2.0f * z) / div),
+		            fabsf((-f_tmp - y + 2.0f * z) / div));
+
+		CLAMP_MAX(f1, 1.0f + FLT_EPSILON);
 	}
 	else {
 		f1 = -z / (y - 2 * z);
@@ -279,8 +294,9 @@ static float quad_coord(float aa[3], float bb[3], float cc[3], float dd[3], int
 	return f1;
 }
 
-static int quad_co(float *x, float *y, float v1[3], float v2[3], float v3[3], float v4[3],
-                   float p[3], float n[3])
+static int quad_co(float *x, float *y,
+                   const float v1[3], const float v2[3], const float v3[3], const float v4[3],
+                   const float p[3], const float n[3])
 {
 	float projverts[5][3], n2[3];
 	float dprojverts[4][3], origin[3] = {0.0f, 0.0f, 0.0f};
@@ -301,12 +317,7 @@ static int quad_co(float *x, float *y, float v1[3], float v2[3], float v3[3], fl
 
 	/* rotate */
 	poly_rotate_plane(n, projverts, 5);
-	
-	/* flatten */
-	for (i = 0; i < 5; i++) {
-		projverts[i][2] = 0.0f;
-	}
-	
+
 	/* subtract origin */
 	for (i = 0; i < 4; i++) {
 		sub_v3_v3(projverts[i], projverts[4]);
@@ -339,15 +350,15 @@ static void mdisp_axis_from_quad(float v1[3], float v2[3], float UNUSED(v3[3]),
 
 /* tl is loop to project onto, l is loop whose internal displacement, co, is being
  * projected.  x and y are location in loop's mdisps grid of point co. */
-static int mdisp_in_mdispquad(BMLoop *l, BMLoop *tl, float p[3], float *x, float *y,
-                              int res, float axis_x[3], float axis_y[3])
+static bool mdisp_in_mdispquad(BMLoop *l, BMLoop *tl, float p[3], float *x, float *y,
+                               int res, float axis_x[3], float axis_y[3])
 {
 	float v1[3], v2[3], c[3], v3[3], v4[3], e1[3], e2[3];
 	float eps = FLT_EPSILON * 4000;
 	
-	if (len_v3(l->v->no) == 0.0f)
+	if (is_zero_v3(l->v->no))
 		BM_vert_normal_update_all(l->v);
-	if (len_v3(tl->v->no) == 0.0f)
+	if (is_zero_v3(tl->v->no))
 		BM_vert_normal_update_all(tl->v);
 
 	compute_mdisp_quad(tl, v1, v2, v3, v4, e1, e2);
@@ -373,8 +384,8 @@ static int mdisp_in_mdispquad(BMLoop *l, BMLoop *tl, float p[3], float *x, float
 	return 1;
 }
 
-static float bm_loop_flip_equotion(float mat[2][2], float b[2], float target_axis_x[3], float target_axis_y[3],
-                                   float coord[3], int i, int j)
+static float bm_loop_flip_equotion(float mat[2][2], float b[2], const float target_axis_x[3], const float target_axis_y[3],
+                                   const float coord[3], int i, int j)
 {
 	mat[0][0] = target_axis_x[i];
 	mat[0][1] = target_axis_y[i];
@@ -386,8 +397,8 @@ static float bm_loop_flip_equotion(float mat[2][2], float b[2], float target_axi
 	return mat[0][0] * mat[1][1] - mat[0][1] * mat[1][0];
 }
 
-static void bm_loop_flip_disp(float source_axis_x[3], float source_axis_y[3],
-                                 float target_axis_x[3], float target_axis_y[3], float disp[3])
+static void bm_loop_flip_disp(const float source_axis_x[3], const float source_axis_y[3],
+                              const float target_axis_x[3], const float target_axis_y[3], float disp[3])
 {
 	float vx[3], vy[3], coord[3];
 	float n[3], vec[3];
@@ -414,34 +425,32 @@ static void bm_loop_flip_disp(float source_axis_x[3], float source_axis_y[3],
 	disp[1] = (mat[0][0] * b[1] - b[0] * mat[1][0]) / d;
 }
 
-static void bm_loop_interp_mdisps(BMesh *bm, BMLoop *target, BMFace *source)
+static void bm_loop_interp_mdisps(BMesh *bm, BMLoop *l_dst, BMFace *f_src)
 {
-	MDisps *mdisps;
-	BMLoop *l_iter;
-	BMLoop *l_first;
-	float x, y, d, v1[3], v2[3], v3[3], v4[3] = {0.0f, 0.0f, 0.0f}, e1[3], e2[3];
-	int ix, iy, res;
+	const int cd_loop_mdisp_offset = CustomData_get_offset(&bm->ldata, CD_MDISPS);
+	MDisps *md_dst;
+	float d, v1[3], v2[3], v3[3], v4[3] = {0.0f, 0.0f, 0.0f}, e1[3], e2[3];
+	int ix, res;
 	float axis_x[3], axis_y[3];
+
+	if (cd_loop_mdisp_offset == -1)
+		return;
 	
 	/* ignore 2-edged faces */
-	if (UNLIKELY(target->f->len < 3))
+	if (UNLIKELY(l_dst->f->len < 3))
 		return;
+
+	md_dst = BM_ELEM_CD_GET_VOID_P(l_dst, cd_loop_mdisp_offset);
+	compute_mdisp_quad(l_dst, v1, v2, v3, v4, e1, e2);
 	
-	if (!CustomData_has_layer(&bm->ldata, CD_MDISPS))
-		return;
-	
-	mdisps = CustomData_bmesh_get(&bm->ldata, target->head.data, CD_MDISPS);
-	compute_mdisp_quad(target, v1, v2, v3, v4, e1, e2);
-	
-	/* if no disps data allocate a new grid, the size of the first grid in source. */
-	if (!mdisps->totdisp) {
-		MDisps *md2 = CustomData_bmesh_get(&bm->ldata, BM_FACE_FIRST_LOOP(source)->head.data, CD_MDISPS);
+	/* if no disps data allocate a new grid, the size of the first grid in f_src. */
+	if (!md_dst->totdisp) {
+		MDisps *md_src = BM_ELEM_CD_GET_VOID_P(BM_FACE_FIRST_LOOP(f_src), cd_loop_mdisp_offset);
 		
-		mdisps->totdisp = md2->totdisp;
-		mdisps->level = md2->level;
-		if (mdisps->totdisp) {
-			mdisps->disps = MEM_callocN(sizeof(float) * 3 * mdisps->totdisp,
-			                            "mdisp->disps in bmesh_loop_intern_mdisps");
+		md_dst->totdisp = md_src->totdisp;
+		md_dst->level = md_src->level;
+		if (md_dst->totdisp) {
+			md_dst->disps = MEM_callocN(sizeof(float) * 3 * md_dst->totdisp, __func__);
 		}
 		else {
 			return;
@@ -450,12 +459,17 @@ static void bm_loop_interp_mdisps(BMesh *bm, BMLoop *target, BMFace *source)
 	
 	mdisp_axis_from_quad(v1, v2, v3, v4, axis_x, axis_y);
 
-	res = (int)sqrt(mdisps->totdisp);
+	res = (int)sqrt(md_dst->totdisp);
 	d = 1.0f / (float)(res - 1);
-	for (x = 0.0f, ix = 0; ix < res; x += d, ix++) {
+#pragma omp parallel for if (res > 3)
+	for (ix = 0; ix < res; ix++) {
+		float x = d * ix, y;
+		int iy;
 		for (y = 0.0f, iy = 0; iy < res; y += d, iy++) {
+			BMLoop *l_iter;
+			BMLoop *l_first;
 			float co1[3], co2[3], co[3];
-			
+
 			copy_v3_v3(co1, e1);
 			
 			mul_v3_fl(co1, y);
@@ -469,18 +483,17 @@ static void bm_loop_interp_mdisps(BMesh *bm, BMLoop *target, BMFace *source)
 			mul_v3_fl(co, x);
 			add_v3_v3(co, co1);
 			
-			l_iter = l_first = BM_FACE_FIRST_LOOP(source);
+			l_iter = l_first = BM_FACE_FIRST_LOOP(f_src);
 			do {
 				float x2, y2;
-				MDisps *md1, *md2;
+				MDisps *md_src;
 				float src_axis_x[3], src_axis_y[3];
 
-				md1 = CustomData_bmesh_get(&bm->ldata, target->head.data, CD_MDISPS);
-				md2 = CustomData_bmesh_get(&bm->ldata, l_iter->head.data, CD_MDISPS);
+				md_src = BM_ELEM_CD_GET_VOID_P(l_iter, cd_loop_mdisp_offset);
 				
-				if (mdisp_in_mdispquad(target, l_iter, co, &x2, &y2, res, src_axis_x, src_axis_y)) {
-					old_mdisps_bilinear(md1->disps[iy * res + ix], md2->disps, res, (float)x2, (float)y2);
-					bm_loop_flip_disp(src_axis_x, src_axis_y, axis_x, axis_y, md1->disps[iy * res + ix]);
+				if (mdisp_in_mdispquad(l_dst, l_iter, co, &x2, &y2, res, src_axis_x, src_axis_y)) {
+					old_mdisps_bilinear(md_dst->disps[iy * res + ix], md_src->disps, res, (float)x2, (float)y2);
+					bm_loop_flip_disp(src_axis_x, src_axis_y, axis_x, axis_y, md_dst->disps[iy * res + ix]);
 
 					break;
 				}
@@ -490,21 +503,22 @@ static void bm_loop_interp_mdisps(BMesh *bm, BMLoop *target, BMFace *source)
 }
 
 /**
- * smoothes boundaries between multires grids,
+ * smooths boundaries between multires grids,
  * including some borders in adjacent faces
  */
 void BM_face_multires_bounds_smooth(BMesh *bm, BMFace *f)
 {
+	const int cd_loop_mdisp_offset = CustomData_get_offset(&bm->ldata, CD_MDISPS);
 	BMLoop *l;
 	BMIter liter;
 	
-	if (!CustomData_has_layer(&bm->ldata, CD_MDISPS))
+	if (cd_loop_mdisp_offset == -1)
 		return;
 	
 	BM_ITER_ELEM (l, &liter, f, BM_LOOPS_OF_FACE) {
-		MDisps *mdp = CustomData_bmesh_get(&bm->ldata, l->prev->head.data, CD_MDISPS);
-		MDisps *mdl = CustomData_bmesh_get(&bm->ldata, l->head.data, CD_MDISPS);
-		MDisps *mdn = CustomData_bmesh_get(&bm->ldata, l->next->head.data, CD_MDISPS);
+		MDisps *mdp = BM_ELEM_CD_GET_VOID_P(l->prev, cd_loop_mdisp_offset);
+		MDisps *mdl = BM_ELEM_CD_GET_VOID_P(l, cd_loop_mdisp_offset);
+		MDisps *mdn = BM_ELEM_CD_GET_VOID_P(l->next, cd_loop_mdisp_offset);
 		float co1[3];
 		int sides;
 		int y;
@@ -525,8 +539,7 @@ void BM_face_multires_bounds_smooth(BMesh *bm, BMFace *f)
 
 		sides = (int)sqrt(mdp->totdisp);
 		for (y = 0; y < sides; y++) {
-			add_v3_v3v3(co1, mdn->disps[y * sides], mdl->disps[y]);
-			mul_v3_fl(co1, 0.5);
+			mid_v3_v3v3(co1, mdn->disps[y * sides], mdl->disps[y]);
 
 			copy_v3_v3(mdn->disps[y * sides], co1);
 			copy_v3_v3(mdl->disps[y], co1);
@@ -534,7 +547,7 @@ void BM_face_multires_bounds_smooth(BMesh *bm, BMFace *f)
 	}
 	
 	BM_ITER_ELEM (l, &liter, f, BM_LOOPS_OF_FACE) {
-		MDisps *mdl1 = CustomData_bmesh_get(&bm->ldata, l->head.data, CD_MDISPS);
+		MDisps *mdl1 = BM_ELEM_CD_GET_VOID_P(l, cd_loop_mdisp_offset);
 		MDisps *mdl2;
 		float co1[3], co2[3], co[3];
 		int sides;
@@ -558,9 +571,9 @@ void BM_face_multires_bounds_smooth(BMesh *bm, BMFace *f)
 			continue;
 
 		if (l->radial_next->v == l->v)
-			mdl2 = CustomData_bmesh_get(&bm->ldata, l->radial_next->head.data, CD_MDISPS);
+			mdl2 = BM_ELEM_CD_GET_VOID_P(l->radial_next, cd_loop_mdisp_offset);
 		else
-			mdl2 = CustomData_bmesh_get(&bm->ldata, l->radial_next->next->head.data, CD_MDISPS);
+			mdl2 = BM_ELEM_CD_GET_VOID_P(l->radial_next->next, cd_loop_mdisp_offset);
 
 		sides = (int)sqrt(mdl1->totdisp);
 		for (y = 0; y < sides; y++) {
@@ -608,79 +621,44 @@ void BM_loop_interp_multires(BMesh *bm, BMLoop *target, BMFace *source)
  * if do_vertex is true, target's vert data will also get interpolated.
  */
 void BM_loop_interp_from_face(BMesh *bm, BMLoop *target, BMFace *source,
-                              int do_vertex, int do_multires)
+                              const bool do_vertex, const bool do_multires)
 {
 	BMLoop *l_iter;
 	BMLoop *l_first;
-	void **blocks = NULL;
-	void **vblocks = NULL;
-	float (*cos)[3] = NULL, co[3], *w = NULL;
-	float cent[3] = {0.0f, 0.0f, 0.0f};
-	BLI_array_fixedstack_declare(cos,      BM_NGON_STACK_SIZE, source->len, __func__);
-	BLI_array_fixedstack_declare(w,        BM_NGON_STACK_SIZE, source->len, __func__);
-	BLI_array_fixedstack_declare(blocks,   BM_NGON_STACK_SIZE, source->len, __func__);
-	BLI_array_fixedstack_declare(vblocks,  BM_NGON_STACK_SIZE, do_vertex ? source->len : 0, __func__);
-	int i, ax, ay;
+	void **vblocks  = do_vertex ? BLI_array_alloca(vblocks, source->len) : NULL;
+	void **blocks   = BLI_array_alloca(blocks, source->len);
+	float (*cos_2d)[2] = BLI_array_alloca(cos_2d, source->len);
+	float *w        = BLI_array_alloca(w, source->len);
+	float axis_mat[3][3];  /* use normal to transform into 2d xy coords */
+	float co[2];
+	int i;
 
-	BM_elem_attrs_copy(bm, bm, source, target->f);
+	/* convert the 3d coords into 2d for projection */
+	BLI_assert(BM_face_is_normal_valid(source));
+	axis_dominant_v3_to_m3(axis_mat, source->no);
 
 	i = 0;
 	l_iter = l_first = BM_FACE_FIRST_LOOP(source);
 	do {
-		copy_v3_v3(cos[i], l_iter->v->co);
-		add_v3_v3(cent, cos[i]);
-
-		w[i] = 0.0f;
+		mul_v2_m3v3(cos_2d[i], axis_mat, l_iter->v->co);
 		blocks[i] = l_iter->head.data;
 
 		if (do_vertex) {
 			vblocks[i] = l_iter->v->head.data;
 		}
-		i++;
-
-	} while ((l_iter = l_iter->next) != l_first);
-
-	/* find best projection of face XY, XZ or YZ: barycentric weights of
-	 * the 2d projected coords are the same and faster to compute */
-
-	axis_dominant_v3(&ax, &ay, source->no);
-
-	/* scale source face coordinates a bit, so points sitting directly on an
-	 * edge will work. */
-	mul_v3_fl(cent, 1.0f / (float)source->len);
-	for (i = 0; i < source->len; i++) {
-		float vec[3], tmp[3];
-		sub_v3_v3v3(vec, cent, cos[i]);
-		mul_v3_fl(vec, 0.001f);
-		add_v3_v3(cos[i], vec);
-
-		copy_v3_v3(tmp, cos[i]);
-		cos[i][0] = tmp[ax];
-		cos[i][1] = tmp[ay];
-		cos[i][2] = 0.0f;
-	}
+	} while (i++, (l_iter = l_iter->next) != l_first);
 
+	mul_v2_m3v3(co, axis_mat, target->v->co);
 
 	/* interpolate */
-	co[0] = target->v->co[ax];
-	co[1] = target->v->co[ay];
-	co[2] = 0.0f;
-
-	interp_weights_poly_v3(w, cos, source->len, co);
+	interp_weights_poly_v2(w, cos_2d, source->len, co);
 	CustomData_bmesh_interp(&bm->ldata, blocks, w, NULL, source->len, target->head.data);
 	if (do_vertex) {
 		CustomData_bmesh_interp(&bm->vdata, vblocks, w, NULL, source->len, target->v->head.data);
-		BLI_array_fixedstack_free(vblocks);
 	}
 
-	BLI_array_fixedstack_free(cos);
-	BLI_array_fixedstack_free(w);
-	BLI_array_fixedstack_free(blocks);
-
 	if (do_multires) {
-		if (CustomData_has_layer(&bm->ldata, CD_MDISPS)) {
-			bm_loop_interp_mdisps(bm, target, source);
-		}
+		bm_loop_interp_mdisps(bm, target, source);
 	}
 }
 
@@ -689,42 +667,29 @@ void BM_vert_interp_from_face(BMesh *bm, BMVert *v, BMFace *source)
 {
 	BMLoop *l_iter;
 	BMLoop *l_first;
-	void **blocks = NULL;
-	float (*cos)[3] = NULL, *w = NULL;
-	float cent[3] = {0.0f, 0.0f, 0.0f};
-	BLI_array_fixedstack_declare(cos,      BM_NGON_STACK_SIZE, source->len, __func__);
-	BLI_array_fixedstack_declare(w,        BM_NGON_STACK_SIZE, source->len, __func__);
-	BLI_array_fixedstack_declare(blocks,   BM_NGON_STACK_SIZE, source->len, __func__);
+	void **blocks   = BLI_array_alloca(blocks, source->len);
+	float (*cos_2d)[2] = BLI_array_alloca(cos_2d, source->len);
+	float *w        = BLI_array_alloca(w,      source->len);
+	float axis_mat[3][3];  /* use normal to transform into 2d xy coords */
+	float co[2];
 	int i;
 
+	/* convert the 3d coords into 2d for projection */
+	BLI_assert(BM_face_is_normal_valid(source));
+	axis_dominant_v3_to_m3(axis_mat, source->no);
+
 	i = 0;
 	l_iter = l_first = BM_FACE_FIRST_LOOP(source);
 	do {
-		copy_v3_v3(cos[i], l_iter->v->co);
-		add_v3_v3(cent, cos[i]);
-
-		w[i] = 0.0f;
+		mul_v2_m3v3(cos_2d[i], axis_mat, l_iter->v->co);
 		blocks[i] = l_iter->v->head.data;
-		i++;
-	} while ((l_iter = l_iter->next) != l_first);
+	} while (i++, (l_iter = l_iter->next) != l_first);
 
-	/* scale source face coordinates a bit, so points sitting directly on an
-	 * edge will work. */
-	mul_v3_fl(cent, 1.0f / (float)source->len);
-	for (i = 0; i < source->len; i++) {
-		float vec[3];
-		sub_v3_v3v3(vec, cent, cos[i]);
-		mul_v3_fl(vec, 0.01f);
-		add_v3_v3(cos[i], vec);
-	}
+	mul_v2_m3v3(co, axis_mat, v->co);
 
 	/* interpolate */
-	interp_weights_poly_v3(w, cos, source->len, v->co);
+	interp_weights_poly_v2(w, cos_2d, source->len, co);
 	CustomData_bmesh_interp(&bm->vdata, blocks, w, NULL, source->len, v->head.data);
-
-	BLI_array_fixedstack_free(cos);
-	BLI_array_fixedstack_free(w);
-	BLI_array_fixedstack_free(blocks);
 }
 
 static void update_data_blocks(BMesh *bm, CustomData *olddata, CustomData *data)
@@ -806,7 +771,7 @@ void BM_data_layer_add(BMesh *bm, CustomData *data, int type)
 	CustomData olddata;
 
 	olddata = *data;
-	olddata.layers = (olddata.layers) ? MEM_dupallocN(olddata.layers): NULL;
+	olddata.layers = (olddata.layers) ? MEM_dupallocN(olddata.layers) : NULL;
 
 	/* the pool is now owned by olddata and must not be shared */
 	data->pool = NULL;
@@ -822,7 +787,7 @@ void BM_data_layer_add_named(BMesh *bm, CustomData *data, int type, const char *
 	CustomData olddata;
 
 	olddata = *data;
-	olddata.layers = (olddata.layers) ? MEM_dupallocN(olddata.layers): NULL;
+	olddata.layers = (olddata.layers) ? MEM_dupallocN(olddata.layers) : NULL;
 
 	/* the pool is now owned by olddata and must not be shared */
 	data->pool = NULL;
@@ -836,14 +801,17 @@ void BM_data_layer_add_named(BMesh *bm, CustomData *data, int type, const char *
 void BM_data_layer_free(BMesh *bm, CustomData *data, int type)
 {
 	CustomData olddata;
+	bool has_layer;
 
 	olddata = *data;
-	olddata.layers = (olddata.layers) ? MEM_dupallocN(olddata.layers): NULL;
+	olddata.layers = (olddata.layers) ? MEM_dupallocN(olddata.layers) : NULL;
 
 	/* the pool is now owned by olddata and must not be shared */
 	data->pool = NULL;
 
-	CustomData_free_layer_active(data, type, 0);
+	has_layer = CustomData_free_layer_active(data, type, 0);
+	/* assert because its expensive to realloc - better not do if layer isnt present */
+	BLI_assert(has_layer != false);
 
 	update_data_blocks(bm, &olddata, data);
 	if (olddata.layers) MEM_freeN(olddata.layers);
@@ -852,14 +820,17 @@ void BM_data_layer_free(BMesh *bm, CustomData *data, int type)
 void BM_data_layer_free_n(BMesh *bm, CustomData *data, int type, int n)
 {
 	CustomData olddata;
+	bool has_layer;
 
 	olddata = *data;
-	olddata.layers = (olddata.layers) ? MEM_dupallocN(olddata.layers): NULL;
+	olddata.layers = (olddata.layers) ? MEM_dupallocN(olddata.layers) : NULL;
 
 	/* the pool is now owned by olddata and must not be shared */
 	data->pool = NULL;
 
-	CustomData_free_layer(data, type, 0, CustomData_get_layer_index_n(data, type, n));
+	has_layer = CustomData_free_layer(data, type, 0, CustomData_get_layer_index_n(data, type, n));
+	/* assert because its expensive to realloc - better not do if layer isnt present */
+	BLI_assert(has_layer != false);
 	
 	update_data_blocks(bm, &olddata, data);
 	if (olddata.layers) MEM_freeN(olddata.layers);
@@ -913,7 +884,7 @@ void BM_data_layer_copy(BMesh *bm, CustomData *data, int type, int src_n, int ds
 
 float BM_elem_float_data_get(CustomData *cd, void *element, int type)
 {
-	float *f = CustomData_bmesh_get(cd, ((BMHeader *)element)->data, type);
+	const float *f = CustomData_bmesh_get(cd, ((BMHeader *)element)->data, type);
 	return f ? *f : 0.0f;
 }
 
