@@ -44,8 +44,6 @@
 
 static void recount_totsels(BMesh *bm)
 {
-	BMIter iter;
-	BMElem *ele;
 	const char iter_types[3] = {BM_VERTS_OF_MESH,
 	                            BM_EDGES_OF_MESH,
 	                            BM_FACES_OF_MESH};
@@ -58,12 +56,90 @@ static void recount_totsels(BMesh *bm)
 	tots[1] = &bm->totedgesel;
 	tots[2] = &bm->totfacesel;
 
+#pragma omp parallel for schedule(static) if (bm->totvert + bm->totedge + bm->totface >= BM_OMP_LIMIT)
 	for (i = 0; i < 3; i++) {
-		ele = BM_iter_new(&iter, bm, iter_types[i], NULL);
-		for ( ; ele; ele = BM_iter_step(&iter)) {
-			if (BM_elem_flag_test(ele, BM_ELEM_SELECT)) *tots[i] += 1;
+		BMIter iter;
+		BMElem *ele;
+		int count = 0;
+
+		BM_ITER_MESH (ele, &iter, bm, iter_types[i]) {
+			if (BM_elem_flag_test(ele, BM_ELEM_SELECT)) count += 1;
+		}
+		*tots[i] = count;
+	}
+}
+
+/**
+ * \brief Select Mode Clean
+ *
+ * Remove isolated selected elements when in a mode doesn't support them.
+ * eg: in edge-mode a selected vertex must be connected to a selected edge.
+ *
+ * \note this could be made apart of #BM_mesh_select_mode_flush_ex
+ */
+void BM_mesh_select_mode_clean_ex(BMesh *bm, const short selectmode)
+{
+	if (selectmode & SCE_SELECT_VERTEX) {
+		/* pass */
+	}
+	else if (selectmode & SCE_SELECT_EDGE) {
+		BMIter iter;
+
+		if (bm->totvertsel) {
+			BMVert *v;
+			BM_ITER_MESH (v, &iter, bm, BM_VERTS_OF_MESH) {
+				BM_elem_flag_disable(v, BM_ELEM_SELECT);
+			}
+			bm->totvertsel = 0;
+		}
+
+		if (bm->totedgesel) {
+			BMEdge *e;
+			BM_ITER_MESH (e, &iter, bm, BM_EDGES_OF_MESH) {
+				if (BM_elem_flag_test(e, BM_ELEM_SELECT)) {
+					BM_vert_select_set(bm, e->v1, true);
+					BM_vert_select_set(bm, e->v2, true);
+				}
+			}
 		}
 	}
+	else if (selectmode & SCE_SELECT_FACE) {
+		BMIter iter;
+
+		if (bm->totvertsel) {
+			BMVert *v;
+			BM_ITER_MESH (v, &iter, bm, BM_VERTS_OF_MESH) {
+				BM_elem_flag_disable(v, BM_ELEM_SELECT);
+			}
+			bm->totvertsel = 0;
+		}
+
+		if (bm->totedgesel) {
+			BMEdge *e;
+			BM_ITER_MESH (e, &iter, bm, BM_EDGES_OF_MESH) {
+				BM_elem_flag_disable(e, BM_ELEM_SELECT);
+			}
+			bm->totedgesel = 0;
+		}
+
+		if (bm->totfacesel) {
+			BMFace *f;
+			BM_ITER_MESH (f, &iter, bm, BM_FACES_OF_MESH) {
+				if (BM_elem_flag_test(f, BM_ELEM_SELECT)) {
+					BMLoop *l_iter, *l_first;
+					l_iter = l_first = BM_FACE_FIRST_LOOP(f);
+					do {
+						BM_edge_select_set(bm, l_iter->e, true);
+					} while ((l_iter = l_iter->next) != l_first);
+				}
+			}
+		}
+	}
+}
+
+void BM_mesh_select_mode_clean(BMesh *bm)
+{
+	BM_mesh_select_mode_clean_ex(bm, bm->selectmode);
 }
 
 /**
@@ -83,52 +159,61 @@ void BM_mesh_select_mode_flush_ex(BMesh *bm, const short selectmode)
 	BMIter eiter;
 	BMIter fiter;
 
-	int ok;
-
 	if (selectmode & SCE_SELECT_VERTEX) {
-		BM_ITER_MESH (e, &eiter, bm, BM_EDGES_OF_MESH) {
-			if (BM_elem_flag_test(e->v1, BM_ELEM_SELECT) &&
-			    BM_elem_flag_test(e->v2, BM_ELEM_SELECT) &&
-			    !BM_elem_flag_test(e, BM_ELEM_HIDDEN))
+		/* both loops only set edge/face flags and read off verts */
+#pragma omp parallel sections if (bm->totedge + bm->totface >= BM_OMP_LIMIT)
+		{
+#pragma omp section
 			{
-				BM_elem_flag_enable(e, BM_ELEM_SELECT);
-			}
-			else {
-				BM_elem_flag_disable(e, BM_ELEM_SELECT);
-			}
-		}
-		BM_ITER_MESH (f, &fiter, bm, BM_FACES_OF_MESH) {
-			ok = TRUE;
-			if (!BM_elem_flag_test(f, BM_ELEM_HIDDEN)) {
-				l_iter = l_first = BM_FACE_FIRST_LOOP(f);
-				do {
-					if (!BM_elem_flag_test(l_iter->v, BM_ELEM_SELECT)) {
-						ok = FALSE;
-						break;
+				BM_ITER_MESH (e, &eiter, bm, BM_EDGES_OF_MESH) {
+					if (BM_elem_flag_test(e->v1, BM_ELEM_SELECT) &&
+					    BM_elem_flag_test(e->v2, BM_ELEM_SELECT) &&
+					    !BM_elem_flag_test(e, BM_ELEM_HIDDEN))
+					{
+						BM_elem_flag_enable(e, BM_ELEM_SELECT);
 					}
-				} while ((l_iter = l_iter->next) != l_first);
-			}
-			else {
-				ok = FALSE;
+					else {
+						BM_elem_flag_disable(e, BM_ELEM_SELECT);
+					}
+				}
 			}
+#pragma omp section
+			{
+				BM_ITER_MESH (f, &fiter, bm, BM_FACES_OF_MESH) {
+					bool ok = true;
+					if (!BM_elem_flag_test(f, BM_ELEM_HIDDEN)) {
+						l_iter = l_first = BM_FACE_FIRST_LOOP(f);
+						do {
+							if (!BM_elem_flag_test(l_iter->v, BM_ELEM_SELECT)) {
+								ok = false;
+								break;
+							}
+						} while ((l_iter = l_iter->next) != l_first);
+					}
+					else {
+						ok = false;
+					}
 
-			BM_elem_flag_set(f, BM_ELEM_SELECT, ok);
+					BM_elem_flag_set(f, BM_ELEM_SELECT, ok);
+				}
+			}
 		}
+		/* end sections */
 	}
 	else if (selectmode & SCE_SELECT_EDGE) {
 		BM_ITER_MESH (f, &fiter, bm, BM_FACES_OF_MESH) {
-			ok = TRUE;
+			bool ok = true;
 			if (!BM_elem_flag_test(f, BM_ELEM_HIDDEN)) {
 				l_iter = l_first = BM_FACE_FIRST_LOOP(f);
 				do {
 					if (!BM_elem_flag_test(l_iter->e, BM_ELEM_SELECT)) {
-						ok = FALSE;
+						ok = false;
 						break;
 					}
 				} while ((l_iter = l_iter->next) != l_first);
 			}
 			else {
-				ok = FALSE;
+				ok = false;
 			}
 
 			BM_elem_flag_set(f, BM_ELEM_SELECT, ok);
@@ -151,42 +236,28 @@ void BM_mesh_select_mode_flush(BMesh *bm)
  */
 void BM_mesh_deselect_flush(BMesh *bm)
 {
-	BMEdge *e;
-	BMLoop *l_iter;
-	BMLoop *l_first;
-	BMFace *f;
-
 	BMIter eiter;
-	BMIter fiter;
-
-	int ok;
+	BMEdge *e;
 
 	BM_ITER_MESH (e, &eiter, bm, BM_EDGES_OF_MESH) {
-		if (!(BM_elem_flag_test(e->v1, BM_ELEM_SELECT) &&
-		      BM_elem_flag_test(e->v2, BM_ELEM_SELECT) &&
-		      !BM_elem_flag_test(e, BM_ELEM_HIDDEN)))
-		{
-			BM_elem_flag_disable(e, BM_ELEM_SELECT);
-		}
-	}
-
-	BM_ITER_MESH (f, &fiter, bm, BM_FACES_OF_MESH) {
-		ok = TRUE;
-		if (!BM_elem_flag_test(f, BM_ELEM_HIDDEN)) {
-			l_iter = l_first = BM_FACE_FIRST_LOOP(f);
-			do {
-				if (!BM_elem_flag_test(l_iter->v, BM_ELEM_SELECT)) {
-					ok = FALSE;
-					break;
+		if (!BM_elem_flag_test(e, BM_ELEM_HIDDEN)) {
+			if (BM_elem_flag_test(e, BM_ELEM_SELECT)) {
+				if (!BM_elem_flag_test(e->v1, BM_ELEM_SELECT) ||
+				    !BM_elem_flag_test(e->v2, BM_ELEM_SELECT))
+				{
+					BM_elem_flag_disable(e, BM_ELEM_SELECT);
 				}
-			} while ((l_iter = l_iter->next) != l_first);
-		}
-		else {
-			ok = FALSE;
-		}
+			}
 
-		if (ok == FALSE) {
-			BM_elem_flag_disable(f, BM_ELEM_SELECT);
+			if (e->l && !BM_elem_flag_test(e, BM_ELEM_SELECT)) {
+				BMLoop *l_iter;
+				BMLoop *l_first;
+
+				l_iter = l_first = e->l;
+				do {
+					BM_elem_flag_disable(l_iter->f, BM_ELEM_SELECT);
+				} while ((l_iter = l_iter->radial_next) != l_first);
+			}
 		}
 	}
 
@@ -210,34 +281,44 @@ void BM_mesh_select_flush(BMesh *bm)
 	BMIter eiter;
 	BMIter fiter;
 
-	int ok;
+	bool ok;
 
-	BM_ITER_MESH (e, &eiter, bm, BM_EDGES_OF_MESH) {
-		if (BM_elem_flag_test(e->v1, BM_ELEM_SELECT) &&
-		    BM_elem_flag_test(e->v2, BM_ELEM_SELECT) &&
-		    !BM_elem_flag_test(e, BM_ELEM_HIDDEN))
+	/* we can use 2 sections here because the second loop isnt checking edge selection */
+#pragma omp parallel sections if (bm->totedge + bm->totface >= BM_OMP_LIMIT)
+	{
+#pragma omp section
 		{
-			BM_elem_flag_enable(e, BM_ELEM_SELECT);
-		}
-	}
-
-	BM_ITER_MESH (f, &fiter, bm, BM_FACES_OF_MESH) {
-		ok = TRUE;
-		if (!BM_elem_flag_test(f, BM_ELEM_HIDDEN)) {
-			l_iter = l_first = BM_FACE_FIRST_LOOP(f);
-			do {
-				if (!BM_elem_flag_test(l_iter->v, BM_ELEM_SELECT)) {
-					ok = FALSE;
-					break;
+			BM_ITER_MESH (e, &eiter, bm, BM_EDGES_OF_MESH) {
+				if (BM_elem_flag_test(e->v1, BM_ELEM_SELECT) &&
+				    BM_elem_flag_test(e->v2, BM_ELEM_SELECT) &&
+				    !BM_elem_flag_test(e, BM_ELEM_HIDDEN))
+				{
+					BM_elem_flag_enable(e, BM_ELEM_SELECT);
 				}
-			} while ((l_iter = l_iter->next) != l_first);
-		}
-		else {
-			ok = FALSE;
+			}
 		}
 
-		if (ok) {
-			BM_elem_flag_enable(f, BM_ELEM_SELECT);
+#pragma omp section
+		{
+			BM_ITER_MESH (f, &fiter, bm, BM_FACES_OF_MESH) {
+				ok = true;
+				if (!BM_elem_flag_test(f, BM_ELEM_HIDDEN)) {
+					l_iter = l_first = BM_FACE_FIRST_LOOP(f);
+					do {
+						if (!BM_elem_flag_test(l_iter->v, BM_ELEM_SELECT)) {
+							ok = false;
+							break;
+						}
+					} while ((l_iter = l_iter->next) != l_first);
+				}
+				else {
+					ok = false;
+				}
+
+				if (ok) {
+					BM_elem_flag_enable(f, BM_ELEM_SELECT);
+				}
+			}
 		}
 	}
 
@@ -250,7 +331,7 @@ void BM_mesh_select_flush(BMesh *bm)
  * Changes selection state of a single vertex
  * in a mesh
  */
-void BM_vert_select_set(BMesh *bm, BMVert *v, int select)
+void BM_vert_select_set(BMesh *bm, BMVert *v, const bool select)
 {
 	BLI_assert(v->head.htype == BM_VERT);
 
@@ -277,7 +358,7 @@ void BM_vert_select_set(BMesh *bm, BMVert *v, int select)
  *
  * Changes selection state of a single edge in a mesh.
  */
-void BM_edge_select_set(BMesh *bm, BMEdge *e, int select)
+void BM_edge_select_set(BMesh *bm, BMEdge *e, const bool select)
 {
 	BLI_assert(e->head.htype == BM_EDGE);
 
@@ -289,25 +370,22 @@ void BM_edge_select_set(BMesh *bm, BMEdge *e, int select)
 		if (!BM_elem_flag_test(e, BM_ELEM_SELECT)) bm->totedgesel += 1;
 
 		BM_elem_flag_enable(e, BM_ELEM_SELECT);
-		BM_vert_select_set(bm, e->v1, TRUE);
-		BM_vert_select_set(bm, e->v2, TRUE);
+		BM_vert_select_set(bm, e->v1, true);
+		BM_vert_select_set(bm, e->v2, true);
 	}
 	else {
 		if (BM_elem_flag_test(e, BM_ELEM_SELECT)) bm->totedgesel -= 1;
 		BM_elem_flag_disable(e, BM_ELEM_SELECT);
 
-		if (bm->selectmode == SCE_SELECT_EDGE ||
-		    bm->selectmode == SCE_SELECT_FACE ||
-		    bm->selectmode == (SCE_SELECT_EDGE | SCE_SELECT_FACE))
-		{
-
+		if ((bm->selectmode & SCE_SELECT_VERTEX) == 0) {
 			BMIter iter;
 			BMVert *verts[2] = {e->v1, e->v2};
 			BMEdge *e2;
 			int i;
 
+			/* check if the vert is used by a selected edge */
 			for (i = 0; i < 2; i++) {
-				int deselect = 1;
+				bool deselect = true;
 
 				for (e2 = BM_iter_new(&iter, bm, BM_EDGES_OF_VERT, verts[i]); e2; e2 = BM_iter_step(&iter)) {
 					if (e2 == e) {
@@ -315,19 +393,19 @@ void BM_edge_select_set(BMesh *bm, BMEdge *e, int select)
 					}
 
 					if (BM_elem_flag_test(e2, BM_ELEM_SELECT)) {
-						deselect = 0;
+						deselect = false;
 						break;
 					}
 				}
 
 				if (deselect) {
-					BM_vert_select_set(bm, verts[i], FALSE);
+					BM_vert_select_set(bm, verts[i], false);
 				}
 			}
 		}
 		else {
-			BM_vert_select_set(bm, e->v1, FALSE);
-			BM_vert_select_set(bm, e->v2, FALSE);
+			BM_vert_select_set(bm, e->v1, false);
+			BM_vert_select_set(bm, e->v2, false);
 		}
 
 	}
@@ -339,7 +417,7 @@ void BM_edge_select_set(BMesh *bm, BMEdge *e, int select)
  * Changes selection state of a single
  * face in a mesh.
  */
-void BM_face_select_set(BMesh *bm, BMFace *f, int select)
+void BM_face_select_set(BMesh *bm, BMFace *f, const bool select)
 {
 	BMLoop *l_iter;
 	BMLoop *l_first;
@@ -358,8 +436,8 @@ void BM_face_select_set(BMesh *bm, BMFace *f, int select)
 		BM_elem_flag_enable(f, BM_ELEM_SELECT);
 		l_iter = l_first = BM_FACE_FIRST_LOOP(f);
 		do {
-			BM_vert_select_set(bm, l_iter->v, TRUE);
-			BM_edge_select_set(bm, l_iter->e, TRUE);
+			BM_vert_select_set(bm, l_iter->v, true);
+			BM_edge_select_set(bm, l_iter->e, true);
 		} while ((l_iter = l_iter->next) != l_first);
 	}
 	else {
@@ -379,7 +457,7 @@ void BM_face_select_set(BMesh *bm, BMFace *f, int select)
 			}
 
 			if (!f2) {
-				BM_edge_select_set(bm, l->e, FALSE);
+				BM_edge_select_set(bm, l->e, false);
 			}
 		}
 
@@ -393,7 +471,7 @@ void BM_face_select_set(BMesh *bm, BMFace *f, int select)
 			}
 
 			if (!e) {
-				BM_vert_select_set(bm, l->v, FALSE);
+				BM_vert_select_set(bm, l->v, false);
 			}
 		}
 	}
@@ -434,7 +512,7 @@ void BM_mesh_select_mode_set(BMesh *bm, int selectmode)
 
 		BM_ITER_MESH (ele, &iter, bm, BM_EDGES_OF_MESH) {
 			if (BM_elem_flag_test(ele, BM_ELEM_SELECT)) {
-				BM_edge_select_set(bm, (BMEdge *)ele, TRUE);
+				BM_edge_select_set(bm, (BMEdge *)ele, true);
 			}
 		}
 		BM_mesh_select_mode_flush(bm);
@@ -448,7 +526,7 @@ void BM_mesh_select_mode_set(BMesh *bm, int selectmode)
 #endif
 		BM_ITER_MESH (ele, &iter, bm, BM_FACES_OF_MESH) {
 			if (BM_elem_flag_test(ele, BM_ELEM_SELECT)) {
-				BM_face_select_set(bm, (BMFace *)ele, TRUE);
+				BM_face_select_set(bm, (BMFace *)ele, true);
 			}
 		}
 		BM_mesh_select_mode_flush(bm);
@@ -459,28 +537,28 @@ void BM_mesh_select_mode_set(BMesh *bm, int selectmode)
  * counts number of elements with flag enabled/disabled
  */
 static int bm_mesh_flag_count(BMesh *bm, const char htype, const char hflag,
-                              const short respecthide, const short test_for_enabled)
+                              const bool respecthide, const bool test_for_enabled)
 {
 	BMElem *ele;
 	BMIter iter;
 	int tot = 0;
 
-	BLI_assert(ELEM(test_for_enabled, TRUE, FALSE));
+	BLI_assert((htype & ~BM_ALL_NOLOOP) == 0);
 
 	if (htype & BM_VERT) {
-		for (ele = BM_iter_new(&iter, bm, BM_VERTS_OF_MESH, NULL); ele; ele = BM_iter_step(&iter)) {
+		BM_ITER_MESH (ele, &iter, bm, BM_VERTS_OF_MESH) {
 			if (respecthide && BM_elem_flag_test(ele, BM_ELEM_HIDDEN)) continue;
 			if (BM_elem_flag_test_bool(ele, hflag) == test_for_enabled) tot++;
 		}
 	}
 	if (htype & BM_EDGE) {
-		for (ele = BM_iter_new(&iter, bm, BM_EDGES_OF_MESH, NULL); ele; ele = BM_iter_step(&iter)) {
+		BM_ITER_MESH (ele, &iter, bm, BM_EDGES_OF_MESH) {
 			if (respecthide && BM_elem_flag_test(ele, BM_ELEM_HIDDEN)) continue;
 			if (BM_elem_flag_test_bool(ele, hflag) == test_for_enabled) tot++;
 		}
 	}
 	if (htype & BM_FACE) {
-		for (ele = BM_iter_new(&iter, bm, BM_FACES_OF_MESH, NULL); ele; ele = BM_iter_step(&iter)) {
+		BM_ITER_MESH (ele, &iter, bm, BM_FACES_OF_MESH) {
 			if (respecthide && BM_elem_flag_test(ele, BM_ELEM_HIDDEN)) continue;
 			if (BM_elem_flag_test_bool(ele, hflag) == test_for_enabled) tot++;
 		}
@@ -489,21 +567,21 @@ static int bm_mesh_flag_count(BMesh *bm, const char htype, const char hflag,
 	return tot;
 }
 
-int BM_mesh_elem_hflag_count_enabled(BMesh *bm, const char htype, const char hflag, int respecthide)
+int BM_mesh_elem_hflag_count_enabled(BMesh *bm, const char htype, const char hflag, const bool respecthide)
 {
-	return bm_mesh_flag_count(bm, htype, hflag, respecthide, TRUE);
+	return bm_mesh_flag_count(bm, htype, hflag, respecthide, true);
 }
 
-int BM_mesh_elem_hflag_count_disabled(BMesh *bm, const char htype, const char hflag, int respecthide)
+int BM_mesh_elem_hflag_count_disabled(BMesh *bm, const char htype, const char hflag, const bool respecthide)
 {
-	return bm_mesh_flag_count(bm, htype, hflag, respecthide, FALSE);
+	return bm_mesh_flag_count(bm, htype, hflag, respecthide, false);
 }
 
 /**
  * \note use BM_elem_flag_test(ele, BM_ELEM_SELECT) to test selection
  * \note by design, this will not touch the editselection history stuff
  */
-void BM_elem_select_set(BMesh *bm, BMElem *ele, int select)
+void BM_elem_select_set(BMesh *bm, BMElem *ele, const bool select)
 {
 	switch (ele->head.htype) {
 		case BM_VERT:
@@ -522,17 +600,17 @@ void BM_elem_select_set(BMesh *bm, BMElem *ele, int select)
 }
 
 /* this replaces the active flag used in uv/face mode */
-void BM_active_face_set(BMesh *bm, BMFace *efa)
+void BM_mesh_active_face_set(BMesh *bm, BMFace *efa)
 {
 	bm->act_face = efa;
 }
 
-BMFace *BM_active_face_get(BMesh *bm, int sloppy, int selected)
+BMFace *BM_mesh_active_face_get(BMesh *bm, const bool is_sloppy, const bool is_selected)
 {
-	if (bm->act_face && (!selected || BM_elem_flag_test(bm->act_face, BM_ELEM_SELECT))) {
+	if (bm->act_face && (!is_selected || BM_elem_flag_test(bm->act_face, BM_ELEM_SELECT))) {
 		return bm->act_face;
 	}
-	else if (sloppy) {
+	else if (is_sloppy) {
 		BMIter iter;
 		BMFace *f = NULL;
 		BMEditSelection *ese;
@@ -546,7 +624,7 @@ BMFace *BM_active_face_get(BMesh *bm, int sloppy, int selected)
 				if (BM_elem_flag_test(f, BM_ELEM_HIDDEN)) {
 					f = NULL;
 				}
-				else if (selected && !BM_elem_flag_test(f, BM_ELEM_SELECT)) {
+				else if (is_selected && !BM_elem_flag_test(f, BM_ELEM_SELECT)) {
 					f = NULL;
 				}
 				else {
@@ -567,6 +645,45 @@ BMFace *BM_active_face_get(BMesh *bm, int sloppy, int selected)
 	return NULL;
 }
 
+BMEdge *BM_mesh_active_edge_get(BMesh *bm)
+{
+	if (bm->selected.last) {
+		BMEditSelection *ese = bm->selected.last;
+
+		if (ese && ese->htype == BM_EDGE) {
+			return (BMEdge *)ese->ele;
+		}
+	}
+
+	return NULL;
+}
+
+BMVert *BM_mesh_active_vert_get(BMesh *bm)
+{
+	if (bm->selected.last) {
+		BMEditSelection *ese = bm->selected.last;
+
+		if (ese && ese->htype == BM_VERT) {
+			return (BMVert *)ese->ele;
+		}
+	}
+
+	return NULL;
+}
+
+BMElem *BM_mesh_active_elem_get(BMesh *bm)
+{
+	if (bm->selected.last) {
+		BMEditSelection *ese = bm->selected.last;
+
+		if (ese) {
+			return ese->ele;
+		}
+	}
+
+	return NULL;
+}
+
 /**
  * Generic way to get data from an EditSelection type
  * These functions were written to be used by the Modifier widget
@@ -584,12 +701,11 @@ void BM_editselection_center(BMEditSelection *ese, float r_center[3])
 	}
 	else if (ese->htype == BM_EDGE) {
 		BMEdge *eed = (BMEdge *)ese->ele;
-		add_v3_v3v3(r_center, eed->v1->co, eed->v2->co);
-		mul_v3_fl(r_center, 0.5);
+		mid_v3_v3v3(r_center, eed->v1->co, eed->v2->co);
 	}
 	else if (ese->htype == BM_FACE) {
 		BMFace *efa = (BMFace *)ese->ele;
-		BM_face_calc_center_bounds(efa, r_center);
+		BM_face_calc_center_mean(efa, r_center);
 	}
 }
 
@@ -645,94 +761,65 @@ void BM_editselection_plane(BMEditSelection *ese, float r_plane[3])
 			else                        vec[2] = 1.0f;
 			cross_v3_v3v3(r_plane, eve->no, vec);
 		}
+		normalize_v3(r_plane);
 	}
 	else if (ese->htype == BM_EDGE) {
 		BMEdge *eed = (BMEdge *)ese->ele;
 
-		/* the plane is simple, it runs along the edge
-		 * however selecting different edges can swap the direction of the y axis.
-		 * this makes it less likely for the y axis of the manipulator
-		 * (running along the edge).. to flip less often.
-		 * at least its more predictable */
-		if (eed->v2->co[1] > eed->v1->co[1]) {  /* check which to do first */
-			sub_v3_v3v3(r_plane, eed->v2->co, eed->v1->co);
+		if (BM_edge_is_boundary(eed)) {
+			sub_v3_v3v3(r_plane, eed->l->v->co, eed->l->next->v->co);
 		}
 		else {
-			sub_v3_v3v3(r_plane, eed->v1->co, eed->v2->co);
+			/* the plane is simple, it runs along the edge
+			 * however selecting different edges can swap the direction of the y axis.
+			 * this makes it less likely for the y axis of the manipulator
+			 * (running along the edge).. to flip less often.
+			 * at least its more predictable */
+			if (eed->v2->co[1] > eed->v1->co[1]) {  /* check which to do first */
+				sub_v3_v3v3(r_plane, eed->v2->co, eed->v1->co);
+			}
+			else {
+				sub_v3_v3v3(r_plane, eed->v1->co, eed->v2->co);
+			}
 		}
-		
+
+		normalize_v3(r_plane);
 	}
 	else if (ese->htype == BM_FACE) {
 		BMFace *efa = (BMFace *)ese->ele;
-		float vec[3] = {0.0f, 0.0f, 0.0f};
-		
-		/* for now, use face normal */
-
-		/* make a fake plane thats at rightangles to the normal
-		 * we cant make a crossvec from a vec thats the same as the vec
-		 * unlikely but possible, so make sure if the normal is (0, 0, 1)
-		 * that vec isn't the same or in the same direction even. */
-		if (UNLIKELY(efa->len < 3)) {
-			/* crappy fallback method */
-			if      (efa->no[0] < 0.5f)	vec[0] = 1.0f;
-			else if (efa->no[1] < 0.5f)	vec[1] = 1.0f;
-			else                        vec[2] = 1.0f;
-			cross_v3_v3v3(r_plane, efa->no, vec);
-		}
-		else {
-			BMVert *verts[4] = {NULL};
-
-			BM_iter_as_array(NULL, BM_VERTS_OF_FACE, efa, (void **)verts, 4);
-
-			if (efa->len == 4) {
-				float vecA[3], vecB[3];
-				sub_v3_v3v3(vecA, verts[3]->co, verts[2]->co);
-				sub_v3_v3v3(vecB, verts[0]->co, verts[1]->co);
-				add_v3_v3v3(r_plane, vecA, vecB);
-
-				sub_v3_v3v3(vecA, verts[0]->co, verts[3]->co);
-				sub_v3_v3v3(vecB, verts[1]->co, verts[2]->co);
-				add_v3_v3v3(vec, vecA, vecB);
-				/* use the biggest edge length */
-				if (dot_v3v3(r_plane, r_plane) < dot_v3v3(vec, vec)) {
-					copy_v3_v3(r_plane, vec);
-				}
-			}
-			else {
-				BMLoop *l_long  = BM_face_find_longest_loop(efa);
-
-				sub_v3_v3v3(r_plane, l_long->v->co, l_long->next->v->co);
-			}
-
-		}
+		BM_face_calc_plane(efa, r_plane);
 	}
-	normalize_v3(r_plane);
 }
 
+static BMEditSelection *bm_select_history_create(BMHeader *ele)
+{
+	BMEditSelection *ese = (BMEditSelection *) MEM_callocN(sizeof(BMEditSelection), "BMEdit Selection");
+	ese->htype = ele->htype;
+	ese->ele = (BMElem *)ele;
+	return ese;
+}
 
 /* --- macro wrapped funcs --- */
-int _bm_select_history_check(BMesh *bm, const BMHeader *ele)
+bool _bm_select_history_check(BMesh *bm, const BMHeader *ele)
 {
 	return (BLI_findptr(&bm->selected, ele, offsetof(BMEditSelection, ele)) != NULL);
 }
 
-int _bm_select_history_remove(BMesh *bm, BMHeader *ele)
+bool _bm_select_history_remove(BMesh *bm, BMHeader *ele)
 {
 	BMEditSelection *ese = BLI_findptr(&bm->selected, ele, offsetof(BMEditSelection, ele));
 	if (ese) {
 		BLI_freelinkN(&bm->selected, ese);
-		return TRUE;
+		return true;
 	}
 	else {
-		return FALSE;
+		return false;
 	}
 }
 
 void _bm_select_history_store_notest(BMesh *bm, BMHeader *ele)
 {
-	BMEditSelection *ese = (BMEditSelection *) MEM_callocN(sizeof(BMEditSelection), "BMEdit Selection");
-	ese->htype = ele->htype;
-	ese->ele = (BMElem *)ele;
+	BMEditSelection *ese = bm_select_history_create(ele);
 	BLI_addtail(&(bm->selected), ese);
 }
 
@@ -742,36 +829,48 @@ void _bm_select_history_store(BMesh *bm, BMHeader *ele)
 		BM_select_history_store_notest(bm, (BMElem *)ele);
 	}
 }
+
+
+void _bm_select_history_store_after_notest(BMesh *bm, BMEditSelection *ese_ref, BMHeader *ele)
+{
+	BMEditSelection *ese = bm_select_history_create(ele);
+	BLI_insertlinkafter(&(bm->selected), ese_ref, ese);
+}
+
+void _bm_select_history_store_after(BMesh *bm, BMEditSelection *ese_ref, BMHeader *ele)
+{
+	if (!BM_select_history_check(bm, (BMElem *)ele)) {
+		BM_select_history_store_after_notest(bm, ese_ref, (BMElem *)ele);
+	}
+}
 /* --- end macro wrapped funcs --- */
 
 
 void BM_select_history_clear(BMesh *bm)
 {
 	BLI_freelistN(&bm->selected);
-	bm->selected.first = bm->selected.last = NULL;
 }
 
 
 void BM_select_history_validate(BMesh *bm)
 {
-	BMEditSelection *ese, *nextese;
+	BMEditSelection *ese, *ese_next;
 
-	ese = bm->selected.first;
-
-	while (ese) {
-		nextese = ese->next;
+	for (ese = bm->selected.first; ese; ese = ese_next) {
+		ese_next = ese->next;
 		if (!BM_elem_flag_test(ese->ele, BM_ELEM_SELECT)) {
 			BLI_freelinkN(&(bm->selected), ese);
 		}
-		ese = nextese;
 	}
 }
 
-/* utility function */
-int BM_select_history_active_get(BMesh *bm, BMEditSelection *ese)
+/**
+ * Get the active mesh element (with active-face fallback).
+ */
+bool BM_select_history_active_get(BMesh *bm, BMEditSelection *ese)
 {
 	BMEditSelection *ese_last = bm->selected.last;
-	BMFace *efa = BM_active_face_get(bm, FALSE, FALSE);
+	BMFace *efa = BM_mesh_active_face_get(bm, false, false);
 
 	ese->next = ese->prev = NULL;
 
@@ -790,20 +889,42 @@ int BM_select_history_active_get(BMesh *bm, BMEditSelection *ese)
 			ese->htype = ese_last->htype;
 		}
 	}
-	else if (efa) { /* no */
+	else if (efa) {
+		/* no edit-selection, fallback to active face */
 		ese->ele   = (BMElem *)efa;
 		ese->htype = BM_FACE;
 	}
 	else {
 		ese->ele = NULL;
-		return FALSE;
+		return false;
 	}
 
-	return TRUE;
+	return true;
+}
+
+/**
+ * Return a map from BMVert/Edge/Face -> BMEditSelection
+ */
+GHash *BM_select_history_map_create(BMesh *bm)
+{
+	BMEditSelection *ese;
+	GHash *map;
+
+	if (BLI_listbase_is_empty(&bm->selected)) {
+		return NULL;
+	}
+
+	map = BLI_ghash_ptr_new(__func__);
+
+	for (ese = bm->selected.first; ese; ese = ese->next) {
+		BLI_ghash_insert(map, ese->ele, ese);
+	}
+
+	return map;
 }
 
 void BM_mesh_elem_hflag_disable_test(BMesh *bm, const char htype, const char hflag,
-                                     int respecthide, const char hflag_test)
+                                     const bool respecthide, const bool overwrite, const char hflag_test)
 {
 	const char iter_types[3] = {BM_VERTS_OF_MESH,
 	                            BM_EDGES_OF_MESH,
@@ -811,46 +932,62 @@ void BM_mesh_elem_hflag_disable_test(BMesh *bm, const char htype, const char hfl
 
 	const char flag_types[3] = {BM_VERT, BM_EDGE, BM_FACE};
 
-	BMIter iter;
-	BMElem *ele;
+	const char hflag_nosel = hflag & ~BM_ELEM_SELECT;
+
 	int i;
 
+	BLI_assert((htype & ~BM_ALL_NOLOOP) == 0);
+
 	if (hflag & BM_ELEM_SELECT) {
 		BM_select_history_clear(bm);
 	}
 
 	if ((htype == (BM_VERT | BM_EDGE | BM_FACE)) &&
 	    (hflag == BM_ELEM_SELECT) &&
-	    (respecthide == FALSE) &&
+	    (respecthide == false) &&
 	    (hflag_test == 0))
 	{
 		/* fast path for deselect all, avoid topology loops
 		 * since we know all will be de-selected anyway. */
+
+#pragma omp parallel for schedule(static) if (bm->totvert + bm->totedge + bm->totface >= BM_OMP_LIMIT)
 		for (i = 0; i < 3; i++) {
+			BMIter iter;
+			BMElem *ele;
+
 			ele = BM_iter_new(&iter, bm, iter_types[i], NULL);
 			for ( ; ele; ele = BM_iter_step(&iter)) {
 				BM_elem_flag_disable(ele, BM_ELEM_SELECT);
 			}
 		}
+
 		bm->totvertsel = bm->totedgesel = bm->totfacesel = 0;
 	}
 	else {
 		for (i = 0; i < 3; i++) {
+			BMIter iter;
+			BMElem *ele;
+
 			if (htype & flag_types[i]) {
 				ele = BM_iter_new(&iter, bm, iter_types[i], NULL);
 				for ( ; ele; ele = BM_iter_step(&iter)) {
 
-					if (respecthide && BM_elem_flag_test(ele, BM_ELEM_HIDDEN)) {
-						continue;
+					if (UNLIKELY(respecthide && BM_elem_flag_test(ele, BM_ELEM_HIDDEN))) {
+						/* pass */
 					}
-					if (hflag_test && !BM_elem_flag_test(ele, hflag_test)) {
-						continue;
+					else if (!hflag_test || BM_elem_flag_test(ele, hflag_test)) {
+						if (hflag & BM_ELEM_SELECT) {
+							BM_elem_select_set(bm, ele, false);
+						}
+						BM_elem_flag_disable(ele, hflag);
 					}
-
-					if (hflag & BM_ELEM_SELECT) {
-						BM_elem_select_set(bm, ele, FALSE);
+					else if (overwrite) {
+						/* no match! */
+						if (hflag & BM_ELEM_SELECT) {
+							BM_elem_select_set(bm, ele, true);
+						}
+						BM_elem_flag_enable(ele, hflag_nosel);
 					}
-					BM_elem_flag_disable(ele, hflag);
 				}
 			}
 		}
@@ -858,7 +995,7 @@ void BM_mesh_elem_hflag_disable_test(BMesh *bm, const char htype, const char hfl
 }
 
 void BM_mesh_elem_hflag_enable_test(BMesh *bm, const char htype, const char hflag,
-                                    int respecthide, const char hflag_test)
+                                    const bool respecthide, const bool overwrite, const char hflag_test)
 {
 	const char iter_types[3] = {BM_VERTS_OF_MESH,
 	                            BM_EDGES_OF_MESH,
@@ -875,6 +1012,8 @@ void BM_mesh_elem_hflag_enable_test(BMesh *bm, const char htype, const char hfla
 	BMElem *ele;
 	int i;
 
+	BLI_assert((htype & ~BM_ALL_NOLOOP) == 0);
+
 	if (hflag & BM_ELEM_SELECT) {
 		BM_select_history_clear(bm);
 	}
@@ -888,34 +1027,40 @@ void BM_mesh_elem_hflag_enable_test(BMesh *bm, const char htype, const char hfla
 			ele = BM_iter_new(&iter, bm, iter_types[i], NULL);
 			for ( ; ele; ele = BM_iter_step(&iter)) {
 
-				if (respecthide && BM_elem_flag_test(ele, BM_ELEM_HIDDEN)) {
-					continue;
+				if (UNLIKELY(respecthide && BM_elem_flag_test(ele, BM_ELEM_HIDDEN))) {
+					/* pass */
 				}
-				if (hflag_test && !BM_elem_flag_test(ele, hflag_test)) {
-					continue;
+				else if (!hflag_test || BM_elem_flag_test(ele, hflag_test)) {
+					/* match! */
+					if (hflag & BM_ELEM_SELECT) {
+						BM_elem_select_set(bm, ele, true);
+					}
+					BM_elem_flag_enable(ele, hflag_nosel);
 				}
-
-				if (hflag & BM_ELEM_SELECT) {
-					BM_elem_select_set(bm, ele, TRUE);
+				else if (overwrite) {
+					/* no match! */
+					if (hflag & BM_ELEM_SELECT) {
+						BM_elem_select_set(bm, ele, false);
+					}
+					BM_elem_flag_disable(ele, hflag);
 				}
-				BM_elem_flag_enable(ele, hflag_nosel);
 			}
 		}
 	}
 }
 
 void BM_mesh_elem_hflag_disable_all(BMesh *bm, const char htype, const char hflag,
-                                    int respecthide)
+                                    const bool respecthide)
 {
 	/* call with 0 hflag_test */
-	BM_mesh_elem_hflag_disable_test(bm, htype, hflag, respecthide, 0);
+	BM_mesh_elem_hflag_disable_test(bm, htype, hflag, respecthide, false, 0);
 }
 
 void BM_mesh_elem_hflag_enable_all(BMesh *bm, const char htype, const char hflag,
-                                   int respecthide)
+                                   const bool respecthide)
 {
 	/* call with 0 hflag_test */
-	BM_mesh_elem_hflag_enable_test(bm, htype, hflag, respecthide, 0);
+	BM_mesh_elem_hflag_enable_test(bm, htype, hflag, respecthide, false, 0);
 }
 
 /***************** Mesh Hiding stuff *********** */
@@ -924,7 +1069,7 @@ static void vert_flush_hide_set(BMVert *v)
 {
 	BMIter iter;
 	BMEdge *e;
-	int hide = TRUE;
+	bool hide = true;
 
 	BM_ITER_ELEM (e, &iter, v, BM_EDGES_OF_VERT) {
 		hide = hide && BM_elem_flag_test(e, BM_ELEM_HIDDEN);
@@ -937,7 +1082,7 @@ static void edge_flush_hide(BMEdge *e)
 {
 	BMIter iter;
 	BMFace *f;
-	int hide = TRUE;
+	bool hide = true;
 
 	BM_ITER_ELEM (f, &iter, e, BM_FACES_OF_EDGE) {
 		hide = hide && BM_elem_flag_test(f, BM_ELEM_HIDDEN);
@@ -946,13 +1091,15 @@ static void edge_flush_hide(BMEdge *e)
 	BM_elem_flag_set(e, BM_ELEM_HIDDEN, hide);
 }
 
-void BM_vert_hide_set(BMVert *v, int hide)
+void BM_vert_hide_set(BMVert *v, const bool hide)
 {
 	/* vert hiding: vert + surrounding edges and faces */
 	BMIter iter, fiter;
 	BMEdge *e;
 	BMFace *f;
 
+	BLI_assert(v->head.htype == BM_VERT);
+
 	BM_elem_flag_set(v, BM_ELEM_HIDDEN, hide);
 
 	BM_ITER_ELEM (e, &iter, v, BM_EDGES_OF_VERT) {
@@ -964,12 +1111,14 @@ void BM_vert_hide_set(BMVert *v, int hide)
 	}
 }
 
-void BM_edge_hide_set(BMEdge *e, int hide)
+void BM_edge_hide_set(BMEdge *e, const bool hide)
 {
 	BMIter iter;
 	BMFace *f;
 	/* BMVert *v; */
 
+	BLI_assert(e->head.htype == BM_EDGE);
+
 	/* edge hiding: faces around the edge */
 	BM_ITER_ELEM (f, &iter, e, BM_FACES_OF_EDGE) {
 		BM_elem_flag_set(f, BM_ELEM_HIDDEN, hide);
@@ -982,11 +1131,13 @@ void BM_edge_hide_set(BMEdge *e, int hide)
 	vert_flush_hide_set(e->v2);
 }
 
-void BM_face_hide_set(BMFace *f, int hide)
+void BM_face_hide_set(BMFace *f, const bool hide)
 {
 	BMIter iter;
 	BMLoop *l;
 
+	BLI_assert(f->head.htype == BM_FACE);
+
 	BM_elem_flag_set(f, BM_ELEM_HIDDEN, hide);
 
 	BM_ITER_ELEM (l, &iter, f, BM_LOOPS_OF_FACE) {
@@ -998,21 +1149,21 @@ void BM_face_hide_set(BMFace *f, int hide)
 	}
 }
 
-void _bm_elem_hide_set(BMesh *bm, BMHeader *head, int hide)
+void _bm_elem_hide_set(BMesh *bm, BMHeader *head, const bool hide)
 {
 	/* Follow convention of always deselecting before
 	 * hiding an element */
 	switch (head->htype) {
 		case BM_VERT:
-			if (hide) BM_vert_select_set(bm, (BMVert *)head, FALSE);
+			if (hide) BM_vert_select_set(bm, (BMVert *)head, false);
 			BM_vert_hide_set((BMVert *)head, hide);
 			break;
 		case BM_EDGE:
-			if (hide) BM_edge_select_set(bm, (BMEdge *)head, FALSE);
+			if (hide) BM_edge_select_set(bm, (BMEdge *)head, false);
 			BM_edge_hide_set((BMEdge *)head, hide);
 			break;
 		case BM_FACE:
-			if (hide) BM_face_select_set(bm, (BMFace *)head, FALSE);
+			if (hide) BM_face_select_set(bm, (BMFace *)head, false);
 			BM_face_hide_set((BMFace *)head, hide);
 			break;
 		default:
