@@ -22,6 +22,8 @@
 
 /** \file blender/bmesh/operators/bmo_subdivide.c
  *  \ingroup bmesh
+ *
+ * Edge based subdivision with various subdivision patterns.
  */
 
 #include "MEM_guardedalloc.h"
@@ -30,17 +32,78 @@
 #include "BLI_rand.h"
 #include "BLI_array.h"
 #include "BLI_noise.h"
+#include "BLI_stack.h"
 
 #include "BKE_customdata.h"
 
-#include "DNA_object_types.h"
 
 #include "bmesh.h"
 #include "intern/bmesh_private.h"
+#include "intern/bmesh_operators_private.h"
 
-#include "intern/bmesh_operators_private.h" /* own include */
+typedef struct SubDParams {
+	int numcuts;
+	float smooth;
+	int   smooth_falloff;
+	float fractal;
+	float along_normal;
+	//int beauty;
+	bool use_smooth;
+	bool use_smooth_even;
+	bool use_sphere;
+	bool use_fractal;
+	int seed;
+	BMOperator *op;
+	BMOpSlot *slot_edge_percents;  /* BMO_slot_get(params->op->slots_in, "edge_percents"); */
+	BMOpSlot *slot_custom_patterns;  /* BMO_slot_get(params->op->slots_in, "custom_patterns"); */
+	float fractal_ofs[3];
 
-#include "bmo_subdivide.h" /* own include */
+	/* rumtime storage for shape key */
+	struct {
+		int cd_vert_shape_offset;
+		int cd_vert_shape_offset_tmp;
+		int totlayer;
+
+		/* shapekey holding displaced vertex coordinates for current geometry */
+		int tmpkey;
+	} shape_info;
+
+} SubDParams;
+
+static void bmo_subd_init_shape_info(BMesh *bm, SubDParams *params)
+{
+	const int skey = CustomData_number_of_layers(&bm->vdata, CD_SHAPEKEY) - 1;
+	params->shape_info.tmpkey = skey;
+	params->shape_info.cd_vert_shape_offset = CustomData_get_offset(&bm->vdata, CD_SHAPEKEY);
+	params->shape_info.cd_vert_shape_offset_tmp = CustomData_get_n_offset(&bm->vdata, CD_SHAPEKEY, skey);
+	params->shape_info.totlayer = CustomData_number_of_layers(&bm->vdata, CD_SHAPEKEY);
+
+}
+
+typedef void (*subd_pattern_fill_fp)(BMesh *bm, BMFace *face, BMVert **verts,
+                                     const SubDParams *params);
+
+/*
+ * note: this is a pattern-based edge subdivider.
+ * it tries to match a pattern to edge selections on faces,
+ * then executes functions to cut them.
+ */
+typedef struct SubDPattern {
+	int seledges[20]; /* selected edges mask, for splitting */
+
+	/* verts starts at the first new vert cut, not the first vert in the face */
+	subd_pattern_fill_fp connectexec;
+	int len; /* total number of verts, before any subdivision */
+} SubDPattern;
+
+/* generic subdivision rules:
+ *
+ * - two selected edges in a face should make a link
+ *   between them.
+ *
+ * - one edge should do, what? make pretty topology, or just
+ *   split the edge only?
+ */
 
 /* flags for all elements share a common bitfield space */
 #define SUBD_SPLIT	1
@@ -54,7 +117,7 @@
 #define ELE_SPLIT	16
 
 /* see bug [#32665], 0.00005 means a we get face splits at a little under 1.0 degrees */
-#define FLT_FACE_SPLIT_EPSILON 0.00005
+#define FLT_FACE_SPLIT_EPSILON 0.00005f
 
 /*
  * NOTE: beauty has been renamed to flag!
@@ -71,46 +134,43 @@
 
 /* connects face with smallest len, which I think should always be correct for
  * edge subdivision */
-static BMEdge *connect_smallest_face(BMesh *bm, BMVert *v1, BMVert *v2, BMFace **r_nf)
+static BMEdge *connect_smallest_face(BMesh *bm, BMVert *v_a, BMVert *v_b, BMFace **r_f_new)
 {
-	BMIter iter, iter2;
-	BMVert *v;
-	BMLoop *nl;
-	BMFace *face, *curf = NULL;
+	BMLoop *l_a, *l_b;
+	BMFace *f;
 
 	/* this isn't the best thing in the world.  it doesn't handle cases where there's
 	 * multiple faces yet.  that might require a convexity test to figure out which
-	 * face is "best" and who knows what for non-manifold conditions. */
-	for (face = BM_iter_new(&iter, bm, BM_FACES_OF_VERT, v1); face; face = BM_iter_step(&iter)) {
-		for (v = BM_iter_new(&iter2, bm, BM_VERTS_OF_FACE, face); v; v = BM_iter_step(&iter2)) {
-			if (v == v2) {
-				if (!curf || face->len < curf->len) curf = face;
-			}
-		}
-	}
+	 * face is "best" and who knows what for non-manifold conditions.
+	 *
+	 * note: we allow adjacent here, since theres no chance this happens.
+	 */
+	f = BM_vert_pair_share_face_by_len(v_a, v_b, &l_a, &l_b, true);
 
-	if (curf) {
-		face = BM_face_split(bm, curf, v1, v2, &nl, NULL, FALSE);
+
+	if (f) {
+		BMFace *f_new;
+		BMLoop *l_new;
+
+		BLI_assert(!BM_loop_is_adjacent(l_a, l_b));
+
+		f_new = BM_face_split(bm, f, l_a, l_b, &l_new, NULL, false);
 		
-		if (r_nf) *r_nf = face;
-		return nl ? nl->e : NULL;
+		if (r_f_new) *r_f_new = f_new;
+		return l_new ? l_new->e : NULL;
 	}
 
 	return NULL;
 }
 /* calculates offset for co, based on fractal, sphere or smooth settings  */
-static void alter_co(BMesh *bm, BMVert *v, BMEdge *UNUSED(origed), const SubDParams *params, float perc,
+static void alter_co(BMVert *v, BMEdge *UNUSED(origed), const SubDParams *params, float perc,
                      BMVert *vsta, BMVert *vend)
 {
-	float tvec[3], prev_co[3], fac;
-	float *co = NULL;
-	int i, totlayer = CustomData_number_of_layers(&bm->vdata, CD_SHAPEKEY);
-	
-	BM_vert_normal_update_all(v);
+	float tvec[3], fac;
+	float *co = BM_ELEM_CD_GET_VOID_P(v, params->shape_info.cd_vert_shape_offset_tmp);
+	int i;
 
-	co = CustomData_bmesh_get_n(&bm->vdata, v->head.data, CD_SHAPEKEY, params->origkey);
 	copy_v3_v3(co, v->co);
-	copy_v3_v3(prev_co, co);
 
 	if (UNLIKELY(params->use_sphere)) { /* subdivide sphere */
 		normalize_v3(co);
@@ -118,7 +178,7 @@ static void alter_co(BMesh *bm, BMVert *v, BMEdge *UNUSED(origed), const SubDPar
 	}
 	else if (params->use_smooth) {
 		/* we calculate an offset vector vec1[], to be added to *co */
-		float len, nor[3], nor1[3], nor2[3], smooth = params->smooth;
+		float len, nor[3], nor1[3], nor2[3], val;
 
 		sub_v3_v3v3(nor, vsta->co, vend->co);
 		len = 0.5f * normalize_v3(nor);
@@ -135,23 +195,28 @@ static void alter_co(BMesh *bm, BMVert *v, BMEdge *UNUSED(origed), const SubDPar
 		madd_v3_v3fl(tvec, nor2, fac);
 
 		/* falloff for multi subdivide */
-		smooth *= sqrtf(fabsf(1.0f - 2.0f * fabsf(0.5f - perc)));
+		val = fabsf(1.0f - 2.0f * fabsf(0.5f - perc));
+		val = bmesh_subd_falloff_calc(params->smooth_falloff, val);
 
-		mul_v3_fl(tvec, smooth * len);
+		if (params->use_smooth_even) {
+			val *= BM_vert_calc_shell_factor(v);
+		}
+
+		mul_v3_fl(tvec, params->smooth * val * len);
 
 		add_v3_v3(co, tvec);
 	}
 
 	if (params->use_fractal) {
-		float len = len_v3v3(vsta->co, vend->co);
-		float normal[3] = {0.0f, 0.0f, 0.0f}, co2[3], base1[3], base2[3];
+		const float len = len_v3v3(vsta->co, vend->co);
+		float normal[3], co2[3], base1[3], base2[3];
 
 		fac = params->fractal * len;
 
 		mid_v3_v3v3(normal, vsta->no, vend->no);
 		ortho_basis_v3v3_v3(base1, base2, normal);
 
-		add_v3_v3v3(co2, v->co, params->off);
+		add_v3_v3v3(co2, v->co, params->fractal_ofs);
 		mul_v3_fl(co2, 10.0f);
 
 		tvec[0] = fac * (BLI_gTurbulence(1.0, co2[0], co2[1], co2[2], 15, 0, 2) - 0.5f);
@@ -165,19 +230,21 @@ static void alter_co(BMesh *bm, BMVert *v, BMEdge *UNUSED(origed), const SubDPar
 	}
 
 	/* apply the new difference to the rest of the shape keys,
-	 * note that this doent take rotations into account, we _could_ support
+	 * note that this doesn't take rotations into account, we _could_ support
 	 * this by getting the normals and coords for each shape key and
 	 * re-calculate the smooth value for each but this is quite involved.
 	 * for now its ok to simply apply the difference IMHO - campbell */
-	sub_v3_v3v3(tvec, prev_co, co);
+	sub_v3_v3v3(tvec, v->co, co);
 
-	for (i = 0; i < totlayer; i++) {
-		if (params->origkey != i) {
-			co = CustomData_bmesh_get_n(&bm->vdata, v->head.data, CD_SHAPEKEY, i);
-			sub_v3_v3(co, tvec);
+	if (params->shape_info.totlayer > 1) {
+		/* skip the last layer since its the temp */
+		i = params->shape_info.totlayer - 1;
+		co = BM_ELEM_CD_GET_VOID_P(v, params->shape_info.cd_vert_shape_offset);
+		while (i--) {
+			BLI_assert(co != BM_ELEM_CD_GET_VOID_P(v, params->shape_info.cd_vert_shape_offset_tmp));
+			sub_v3_v3(co += 3, tvec);
 		}
 	}
-
 }
 
 /* assumes in the edge is the correct interpolated vertices already */
@@ -195,7 +262,7 @@ static BMVert *bm_subdivide_edge_addvert(BMesh *bm, BMEdge *edge, BMEdge *oedge,
 	BMO_elem_flag_enable(bm, ev, ELE_INNER);
 
 	/* offset for smooth or sphere or fractal */
-	alter_co(bm, ev, oedge, params, percent2, vsta, vend);
+	alter_co(ev, oedge, params, percent2, vsta, vend);
 
 #if 0 //BMESH_TODO
 	/* clip if needed by mirror modifier */
@@ -225,8 +292,9 @@ static BMVert *subdivideedgenum(BMesh *bm, BMEdge *edge, BMEdge *oedge,
 	BMVert *ev;
 	float percent, percent2 = 0.0f;
 
-	if (BMO_elem_flag_test(bm, edge, EDGE_PERCENT) && totpoint == 1)
-		percent = BMO_slot_map_float_get(bm, params->op, "edgepercents", edge);
+	if (BMO_elem_flag_test(bm, edge, EDGE_PERCENT) && totpoint == 1) {
+		percent = BMO_slot_map_float_get(params->slot_edge_percents, edge);
+	}
 	else {
 		percent = 1.0f / (float)(totpoint + 1 - curpoint);
 		percent2 = (float)(curpoint + 1) / (float)(totpoint + 1);
@@ -241,31 +309,31 @@ static BMVert *subdivideedgenum(BMesh *bm, BMEdge *edge, BMEdge *oedge,
 static void bm_subdivide_multicut(BMesh *bm, BMEdge *edge, const SubDParams *params,
                                   BMVert *vsta, BMVert *vend)
 {
-	BMEdge *eed = edge, *newe, temp = *edge;
-	BMVert *v, ov1 = *edge->v1, ov2 = *edge->v2, *v1 = edge->v1, *v2 = edge->v2;
+	BMEdge *eed = edge, *e_new, e_tmp = *edge;
+	BMVert *v, v1_tmp = *edge->v1, v2_tmp = *edge->v2, *v1 = edge->v1, *v2 = edge->v2;
 	int i, numcuts = params->numcuts;
 
-	temp.v1 = &ov1;
-	temp.v2 = &ov2;
+	e_tmp.v1 = &v1_tmp;
+	e_tmp.v2 = &v2_tmp;
 	
 	for (i = 0; i < numcuts; i++) {
-		v = subdivideedgenum(bm, eed, &temp, i, params->numcuts, params, &newe, vsta, vend);
+		v = subdivideedgenum(bm, eed, &e_tmp, i, params->numcuts, params, &e_new, vsta, vend);
 
 		BMO_elem_flag_enable(bm, v, SUBD_SPLIT);
 		BMO_elem_flag_enable(bm, eed, SUBD_SPLIT);
-		BMO_elem_flag_enable(bm, newe, SUBD_SPLIT);
+		BMO_elem_flag_enable(bm, e_new, SUBD_SPLIT);
 
 		BMO_elem_flag_enable(bm, v, ELE_SPLIT);
 		BMO_elem_flag_enable(bm, eed, ELE_SPLIT);
-		BMO_elem_flag_enable(bm, newe, SUBD_SPLIT);
+		BMO_elem_flag_enable(bm, e_new, SUBD_SPLIT);
 
 		BM_CHECK_ELEMENT(v);
 		if (v->e) BM_CHECK_ELEMENT(v->e);
 		if (v->e && v->e->l) BM_CHECK_ELEMENT(v->e->l->f);
 	}
 	
-	alter_co(bm, v1, &temp, params, 0, &ov1, &ov2);
-	alter_co(bm, v2, &temp, params, 1.0, &ov1, &ov2);
+	alter_co(v1, &e_tmp, params, 0, &v1_tmp, &v2_tmp);
+	alter_co(v2, &e_tmp, params, 1.0, &v1_tmp, &v2_tmp);
 }
 
 /* note: the patterns are rotated as necessary to
@@ -285,7 +353,7 @@ static void bm_subdivide_multicut(BMesh *bm, BMEdge *edge, const SubDParams *par
 static void quad_1edge_split(BMesh *bm, BMFace *UNUSED(face),
                              BMVert **verts, const SubDParams *params)
 {
-	BMFace *nf;
+	BMFace *f_new;
 	int i, add, numcuts = params->numcuts;
 
 	/* if it's odd, the middle face is a quad, otherwise it's a triangle */
@@ -295,16 +363,16 @@ static void quad_1edge_split(BMesh *bm, BMFace *UNUSED(face),
 			if (i == numcuts / 2) {
 				add -= 1;
 			}
-			connect_smallest_face(bm, verts[i], verts[numcuts + add], &nf);
+			connect_smallest_face(bm, verts[i], verts[numcuts + add], &f_new);
 		}
 	}
 	else {
 		add = 2;
 		for (i = 0; i < numcuts; i++) {
-			connect_smallest_face(bm, verts[i], verts[numcuts + add], &nf);
+			connect_smallest_face(bm, verts[i], verts[numcuts + add], &f_new);
 			if (i == numcuts / 2) {
 				add -= 1;
-				connect_smallest_face(bm, verts[i], verts[numcuts + add], &nf);
+				connect_smallest_face(bm, verts[i], verts[numcuts + add], &f_new);
 			}
 		}
 
@@ -331,13 +399,13 @@ static const SubDPattern quad_1edge = {
 static void quad_2edge_split_path(BMesh *bm, BMFace *UNUSED(face), BMVert **verts,
                                   const SubDParams *params)
 {
-	BMFace *nf;
+	BMFace *f_new;
 	int i, numcuts = params->numcuts;
 	
 	for (i = 0; i < numcuts; i++) {
-		connect_smallest_face(bm, verts[i], verts[numcuts + (numcuts - i)], &nf);
+		connect_smallest_face(bm, verts[i], verts[numcuts + (numcuts - i)], &f_new);
 	}
-	connect_smallest_face(bm, verts[numcuts * 2 + 3], verts[numcuts * 2 + 1], &nf);
+	connect_smallest_face(bm, verts[numcuts * 2 + 3], verts[numcuts * 2 + 1], &f_new);
 }
 
 static const SubDPattern quad_2edge_path = {
@@ -359,27 +427,27 @@ static const SubDPattern quad_2edge_path = {
 static void quad_2edge_split_innervert(BMesh *bm, BMFace *UNUSED(face), BMVert **verts,
                                        const SubDParams *params)
 {
-	BMFace *nf;
-	BMVert *v, *lastv;
-	BMEdge *e, *ne, olde;
+	BMFace *f_new;
+	BMVert *v, *v_last;
+	BMEdge *e, *e_new, e_tmp;
 	int i, numcuts = params->numcuts;
 	
-	lastv = verts[numcuts];
+	v_last = verts[numcuts];
 
 	for (i = numcuts - 1; i >= 0; i--) {
-		e = connect_smallest_face(bm, verts[i], verts[numcuts + (numcuts - i)], &nf);
+		e = connect_smallest_face(bm, verts[i], verts[numcuts + (numcuts - i)], &f_new);
 
-		olde = *e;
-		v = bm_subdivide_edge_addvert(bm, e, &olde, params, 0.5f, 0.5f, &ne, e->v1, e->v2);
+		e_tmp = *e;
+		v = bm_subdivide_edge_addvert(bm, e, &e_tmp, params, 0.5f, 0.5f, &e_new, e->v1, e->v2);
 
 		if (i != numcuts - 1) {
-			connect_smallest_face(bm, lastv, v, &nf);
+			connect_smallest_face(bm, v_last, v, &f_new);
 		}
 
-		lastv = v;
+		v_last = v;
 	}
 
-	connect_smallest_face(bm, lastv, verts[numcuts * 2 + 2], &nf);
+	connect_smallest_face(bm, v_last, verts[numcuts * 2 + 2], &f_new);
 }
 
 static const SubDPattern quad_2edge_innervert = {
@@ -401,15 +469,15 @@ static const SubDPattern quad_2edge_innervert = {
 static void quad_2edge_split_fan(BMesh *bm, BMFace *UNUSED(face), BMVert **verts,
                                  const SubDParams *params)
 {
-	BMFace *nf;
+	BMFace *f_new;
 	/* BMVert *v; */               /* UNUSED */
-	/* BMVert *lastv = verts[2]; */ /* UNUSED */
-	/* BMEdge *e, *ne; */          /* UNUSED */
+	/* BMVert *v_last = verts[2]; */ /* UNUSED */
+	/* BMEdge *e, *e_new; */          /* UNUSED */
 	int i, numcuts = params->numcuts;
 
 	for (i = 0; i < numcuts; i++) {
-		connect_smallest_face(bm, verts[i], verts[numcuts * 2 + 2], &nf);
-		connect_smallest_face(bm, verts[numcuts + (numcuts - i)], verts[numcuts * 2 + 2], &nf);
+		connect_smallest_face(bm, verts[i], verts[numcuts * 2 + 2], &f_new);
+		connect_smallest_face(bm, verts[numcuts + (numcuts - i)], verts[numcuts * 2 + 2], &f_new);
 	}
 }
 
@@ -434,21 +502,21 @@ static const SubDPattern quad_2edge_fan = {
 static void quad_3edge_split(BMesh *bm, BMFace *UNUSED(face), BMVert **verts,
                              const SubDParams *params)
 {
-	BMFace *nf;
+	BMFace *f_new;
 	int i, add = 0, numcuts = params->numcuts;
 	
 	for (i = 0; i < numcuts; i++) {
 		if (i == numcuts / 2) {
 			if (numcuts % 2 != 0) {
-				connect_smallest_face(bm, verts[numcuts - i - 1 + add], verts[i + numcuts + 1], &nf);
+				connect_smallest_face(bm, verts[numcuts - i - 1 + add], verts[i + numcuts + 1], &f_new);
 			}
 			add = numcuts * 2 + 2;
 		}
-		connect_smallest_face(bm, verts[numcuts - i - 1 + add], verts[i + numcuts + 1], &nf);
+		connect_smallest_face(bm, verts[numcuts - i - 1 + add], verts[i + numcuts + 1], &f_new);
 	}
 
 	for (i = 0; i < numcuts / 2 + 1; i++) {
-		connect_smallest_face(bm, verts[i], verts[(numcuts - i) + numcuts * 2 + 1], &nf);
+		connect_smallest_face(bm, verts[i], verts[(numcuts - i) + numcuts * 2 + 1], &f_new);
 	}
 }
 
@@ -473,9 +541,9 @@ static const SubDPattern quad_3edge = {
 static void quad_4edge_subdivide(BMesh *bm, BMFace *UNUSED(face), BMVert **verts,
                                  const SubDParams *params)
 {
-	BMFace *nf;
+	BMFace *f_new;
 	BMVert *v, *v1, *v2;
-	BMEdge *e, *ne, temp;
+	BMEdge *e, *e_new, e_tmp;
 	BMVert **lines;
 	int numcuts = params->numcuts;
 	int i, j, a, b, s = numcuts + 2 /* , totv = numcuts * 4 + 4 */;
@@ -500,39 +568,39 @@ static void quad_4edge_subdivide(BMesh *bm, BMFace *UNUSED(face), BMVert **verts
 		a = i;
 		b = numcuts + 1 + numcuts + 1 + (numcuts - i - 1);
 		
-		e = connect_smallest_face(bm, verts[a], verts[b], &nf);
+		e = connect_smallest_face(bm, verts[a], verts[b], &f_new);
 		if (!e)
 			continue;
 
 		BMO_elem_flag_enable(bm, e, ELE_INNER);
-		BMO_elem_flag_enable(bm, nf, ELE_INNER);
+		BMO_elem_flag_enable(bm, f_new, ELE_INNER);
 
 		
 		v1 = lines[(i + 1) * s] = verts[a];
 		v2 = lines[(i + 1) * s + s - 1] = verts[b];
 		
-		temp = *e;
+		e_tmp = *e;
 		for (a = 0; a < numcuts; a++) {
-			v = subdivideedgenum(bm, e, &temp, a, numcuts, params, &ne,
+			v = subdivideedgenum(bm, e, &e_tmp, a, numcuts, params, &e_new,
 			                     v1, v2);
 
 			BMESH_ASSERT(v != NULL);
 
-			BMO_elem_flag_enable(bm, ne, ELE_INNER);
+			BMO_elem_flag_enable(bm, e_new, ELE_INNER);
 			lines[(i + 1) * s + a + 1] = v;
 		}
 	}
 
 	for (i = 1; i < numcuts + 2; i++) {
-		for (j = 1; j < numcuts + 1; j++) {
+		for (j = 1; j <= numcuts; j++) {
 			a = i * s + j;
 			b = (i - 1) * s + j;
-			e = connect_smallest_face(bm, lines[a], lines[b], &nf);
+			e = connect_smallest_face(bm, lines[a], lines[b], &f_new);
 			if (!e)
 				continue;
 
 			BMO_elem_flag_enable(bm, e, ELE_INNER);
-			BMO_elem_flag_enable(bm, nf, ELE_INNER);
+			BMO_elem_flag_enable(bm, f_new, ELE_INNER);
 		}
 	}
 
@@ -554,11 +622,11 @@ static void quad_4edge_subdivide(BMesh *bm, BMFace *UNUSED(face), BMVert **verts
 static void tri_1edge_split(BMesh *bm, BMFace *UNUSED(face), BMVert **verts,
                             const SubDParams *params)
 {
-	BMFace *nf;
+	BMFace *f_new;
 	int i, numcuts = params->numcuts;
 	
 	for (i = 0; i < numcuts; i++) {
-		connect_smallest_face(bm, verts[i], verts[numcuts + 1], &nf);
+		connect_smallest_face(bm, verts[i], verts[numcuts + 1], &f_new);
 	}
 }
 
@@ -583,9 +651,9 @@ static const SubDPattern tri_1edge = {
 static void tri_3edge_subdivide(BMesh *bm, BMFace *UNUSED(face), BMVert **verts,
                                 const SubDParams *params)
 {
-	BMFace *nf;
-	BMEdge *e, *ne, temp;
-	BMVert ***lines, *v, ov1, ov2;
+	BMFace *f_new;
+	BMEdge *e, *e_new, e_tmp;
+	BMVert ***lines, *v, v1_tmp, v2_tmp;
 	void *stackarr[1];
 	int i, j, a, b, numcuts = params->numcuts;
 	
@@ -606,26 +674,26 @@ static void tri_3edge_subdivide(BMesh *bm, BMFace *UNUSED(face), BMVert **verts,
 		lines[i + 1] = MEM_callocN(sizeof(void *) * (2 + i), "triangle vert table row");
 		a = numcuts * 2 + 2 + i;
 		b = numcuts + numcuts - i;
-		e = connect_smallest_face(bm, verts[a], verts[b], &nf);
+		e = connect_smallest_face(bm, verts[a], verts[b], &f_new);
 		if (!e) goto cleanup;
 
 		BMO_elem_flag_enable(bm, e, ELE_INNER);
-		BMO_elem_flag_enable(bm, nf, ELE_INNER);
+		BMO_elem_flag_enable(bm, f_new, ELE_INNER);
 
 		lines[i + 1][0] = verts[a];
 		lines[i + 1][i + 1] = verts[b];
 		
-		temp = *e;
-		ov1 = *verts[a];
-		ov2 = *verts[b];
-		temp.v1 = &ov1;
-		temp.v2 = &ov2;
+		e_tmp = *e;
+		v1_tmp = *verts[a];
+		v2_tmp = *verts[b];
+		e_tmp.v1 = &v1_tmp;
+		e_tmp.v2 = &v2_tmp;
 		for (j = 0; j < i; j++) {
-			v = subdivideedgenum(bm, e, &temp, j, i, params, &ne,
+			v = subdivideedgenum(bm, e, &e_tmp, j, i, params, &e_new,
 			                     verts[a], verts[b]);
 			lines[i + 1][j + 1] = v;
 
-			BMO_elem_flag_enable(bm, ne, ELE_INNER);
+			BMO_elem_flag_enable(bm, e_new, ELE_INNER);
 		}
 	}
 	
@@ -641,17 +709,17 @@ static void tri_3edge_subdivide(BMesh *bm, BMFace *UNUSED(face), BMVert **verts,
 	 *      s    s
 	 * </pre>
 	 */
-	for (i = 1; i < numcuts + 1; i++) {
+	for (i = 1; i <= numcuts; i++) {
 		for (j = 0; j < i; j++) {
-			e = connect_smallest_face(bm, lines[i][j], lines[i + 1][j + 1], &nf);
+			e = connect_smallest_face(bm, lines[i][j], lines[i + 1][j + 1], &f_new);
 
 			BMO_elem_flag_enable(bm, e, ELE_INNER);
-			BMO_elem_flag_enable(bm, nf, ELE_INNER);
+			BMO_elem_flag_enable(bm, f_new, ELE_INNER);
 
-			e = connect_smallest_face(bm, lines[i][j + 1], lines[i + 1][j + 1], &nf);
+			e = connect_smallest_face(bm, lines[i][j + 1], lines[i + 1][j + 1], &f_new);
 
 			BMO_elem_flag_enable(bm, e, ELE_INNER);
-			BMO_elem_flag_enable(bm, nf, ELE_INNER);
+			BMO_elem_flag_enable(bm, f_new, ELE_INNER);
 		}
 	}
 
@@ -685,7 +753,7 @@ static const SubDPattern *patterns[] = {
 	NULL,
 };
 
-#define PATTERNS_TOT  (sizeof(patterns) / sizeof(void *))
+#define PATTERNS_TOT  ARRAY_SIZE(patterns)
 
 typedef struct SubDFaceData {
 	BMVert *start;
@@ -699,8 +767,7 @@ void bmo_subdivide_edges_exec(BMesh *bm, BMOperator *op)
 	BMOpSlot *einput;
 	const SubDPattern *pat;
 	SubDParams params;
-	SubDFaceData *facedata = NULL;
-	BLI_array_declare(facedata);
+	BLI_Stack *facedata;
 	BMIter viter, fiter, liter;
 	BMVert *v, **verts = NULL;
 	BMEdge *edge;
@@ -710,43 +777,43 @@ void bmo_subdivide_edges_exec(BMesh *bm, BMOperator *op)
 	BLI_array_declare(loops_split);
 	BMLoop **loops = NULL;
 	BLI_array_declare(loops);
-	BMLoop *nl, *l;
+	BMLoop *l_new, *l;
 	BMFace *face;
 	BLI_array_declare(verts);
 	float smooth, fractal, along_normal;
-	int use_sphere, cornertype, use_singleedge, use_gridfill;
-	int skey, seed, i, j, matched, a, b, numcuts, totesel;
+	bool use_sphere, use_single_edge, use_grid_fill, use_only_quads;
+	int cornertype, seed, i, j, a, b, numcuts, totesel, smooth_falloff;
 	
-	BMO_slot_buffer_flag_enable(bm, op, "edges", BM_EDGE, SUBD_SPLIT);
+	BMO_slot_buffer_flag_enable(bm, op->slots_in, "edges", BM_EDGE, SUBD_SPLIT);
 	
-	numcuts = BMO_slot_int_get(op, "numcuts");
-	seed = BMO_slot_int_get(op, "seed");
-	smooth = BMO_slot_float_get(op, "smooth");
-	fractal = BMO_slot_float_get(op, "fractal");
-	along_normal = BMO_slot_float_get(op, "along_normal");
-	cornertype = BMO_slot_int_get(op, "quadcornertype");
+	numcuts = BMO_slot_int_get(op->slots_in, "cuts");
+	seed = BMO_slot_int_get(op->slots_in, "seed");
+	smooth = BMO_slot_float_get(op->slots_in, "smooth");
+	smooth_falloff = BMO_slot_int_get(op->slots_in, "smooth_falloff");
+	fractal = BMO_slot_float_get(op->slots_in, "fractal");
+	along_normal = BMO_slot_float_get(op->slots_in, "along_normal");
+	cornertype = BMO_slot_int_get(op->slots_in, "quad_corner_type");
+
+	use_single_edge = BMO_slot_bool_get(op->slots_in, "use_single_edge");
+	use_grid_fill = BMO_slot_bool_get(op->slots_in, "use_grid_fill");
+	use_only_quads = BMO_slot_bool_get(op->slots_in, "use_only_quads");
+	use_sphere = BMO_slot_bool_get(op->slots_in, "use_sphere");
 
-	use_singleedge = BMO_slot_bool_get(op, "use_singleedge");
-	use_gridfill   = BMO_slot_bool_get(op, "use_gridfill");
-	use_sphere     = BMO_slot_bool_get(op, "use_sphere");
-	
-	BLI_srandom(seed);
-	
 	patterns[1] = NULL;
 	/* straight cut is patterns[1] == NULL */
 	switch (cornertype) {
-		case SUBD_PATH:
+		case SUBD_CORNER_PATH:
 			patterns[1] = &quad_2edge_path;
 			break;
-		case SUBD_INNERVERT:
+		case SUBD_CORNER_INNERVERT:
 			patterns[1] = &quad_2edge_innervert;
 			break;
-		case SUBD_FAN:
+		case SUBD_CORNER_FAN:
 			patterns[1] = &quad_2edge_fan;
 			break;
 	}
 	
-	if (use_singleedge) {
+	if (use_single_edge) {
 		patterns[0] = &quad_1edge;
 		patterns[2] = &tri_1edge;
 	}
@@ -755,7 +822,7 @@ void bmo_subdivide_edges_exec(BMesh *bm, BMOperator *op)
 		patterns[2] = NULL;
 	}
 
-	if (use_gridfill) {
+	if (use_grid_fill) {
 		patterns[3] = &quad_4edge;
 		patterns[5] = &tri_3edge;
 	}
@@ -766,40 +833,58 @@ void bmo_subdivide_edges_exec(BMesh *bm, BMOperator *op)
 	
 	/* add a temporary shapekey layer to store displacements on current geometry */
 	BM_data_layer_add(bm, &bm->vdata, CD_SHAPEKEY);
-	skey = CustomData_number_of_layers(&bm->vdata, CD_SHAPEKEY) - 1;
+
+	bmo_subd_init_shape_info(bm, &params);
 	
 	BM_ITER_MESH (v, &viter, bm, BM_VERTS_OF_MESH) {
-		float *co = CustomData_bmesh_get_n(&bm->vdata, v->head.data, CD_SHAPEKEY, skey);
+		float *co = BM_ELEM_CD_GET_VOID_P(v, params.shape_info.cd_vert_shape_offset_tmp);
 		copy_v3_v3(co, v->co);
 	}
 
 	/* first go through and tag edges */
-	BMO_slot_buffer_from_enabled_flag(bm, op, "edges", BM_EDGE, SUBD_SPLIT);
+	BMO_slot_buffer_from_enabled_flag(bm, op, op->slots_in, "edges", BM_EDGE, SUBD_SPLIT);
 
 	params.numcuts = numcuts;
 	params.op = op;
+	params.slot_edge_percents   = BMO_slot_get(op->slots_in, "edge_percents");
+	params.slot_custom_patterns = BMO_slot_get(op->slots_in, "custom_patterns");
 	params.smooth = smooth;
+	params.smooth_falloff = smooth_falloff;
 	params.seed = seed;
 	params.fractal = fractal;
 	params.along_normal = along_normal;
 	params.use_smooth  = (smooth  != 0.0f);
+	params.use_smooth_even = BMO_slot_bool_get(op->slots_in, "use_smooth_even");
 	params.use_fractal = (fractal != 0.0f);
 	params.use_sphere  = use_sphere;
-	params.origkey = skey;
-	params.off[0] = (float)BLI_drand() * 200.0f;
-	params.off[1] = (float)BLI_drand() * 200.0f;
-	params.off[2] = (float)BLI_drand() * 200.0f;
+
+	if (params.use_fractal) {
+		RNG *rng = BLI_rng_new_srandom(seed);
+
+		params.fractal_ofs[0] = BLI_rng_get_float(rng) * 200.0f;
+		params.fractal_ofs[1] = BLI_rng_get_float(rng) * 200.0f;
+		params.fractal_ofs[2] = BLI_rng_get_float(rng) * 200.0f;
+
+		BLI_rng_free(rng);
+	}
 	
-	BMO_slot_map_to_flag(bm, op, "custompatterns",
+	BMO_slot_map_to_flag(bm, op->slots_in, "custom_patterns",
 	                     BM_FACE, FACE_CUSTOMFILL);
 
-	BMO_slot_map_to_flag(bm, op, "edgepercents",
+	BMO_slot_map_to_flag(bm, op->slots_in, "edge_percents",
 	                     BM_EDGE, EDGE_PERCENT);
 
 
+	facedata = BLI_stack_new(sizeof(SubDFaceData), __func__);
+
 	BM_ITER_MESH (face, &fiter, bm, BM_FACES_OF_MESH) {
 		BMEdge *e1 = NULL, *e2 = NULL;
 		float vec1[3], vec2[3];
+		bool matched = false;
+
+		/* skip non-quads if requested */
+		if (use_only_quads && face->len != 4)
+			continue;
 
 		/* figure out which pattern to use */
 
@@ -809,12 +894,10 @@ void bmo_subdivide_edges_exec(BMesh *bm, BMOperator *op)
 		BLI_array_grow_items(edges, face->len);
 		BLI_array_grow_items(verts, face->len);
 
-		matched = 0;
-
 		totesel = 0;
-		BM_ITER_ELEM_INDEX (nl, &liter, face, BM_LOOPS_OF_FACE, i) {
-			edges[i] = nl->e;
-			verts[i] = nl->v;
+		BM_ITER_ELEM_INDEX (l_new, &liter, face, BM_LOOPS_OF_FACE, i) {
+			edges[i] = l_new->e;
+			verts[i] = l_new->v;
 
 			if (BMO_elem_flag_test(bm, edges[i], SUBD_SPLIT)) {
 				if (!e1) e1 = edges[i];
@@ -825,7 +908,7 @@ void bmo_subdivide_edges_exec(BMesh *bm, BMOperator *op)
 		}
 
 		/* make sure the two edges have a valid angle to each other */
-		if (totesel == 2 && BM_edge_share_vert_count(e1, e2)) {
+		if (totesel == 2 && BM_edge_share_vert_check(e1, e2)) {
 			sub_v3_v3v3(vec1, e1->v2->co, e1->v1->co);
 			sub_v3_v3v3(vec2, e2->v2->co, e2->v1->co);
 			normalize_v3(vec1);
@@ -837,8 +920,7 @@ void bmo_subdivide_edges_exec(BMesh *bm, BMOperator *op)
 		}
 
 		if (BMO_elem_flag_test(bm, face, FACE_CUSTOMFILL)) {
-			pat = BMO_slot_map_data_get(bm, op,
-			                            "custompatterns", face);
+			pat = *BMO_slot_map_data_get(params.slot_custom_patterns, face);
 			for (i = 0; i < pat->len; i++) {
 				matched = 1;
 				for (j = 0; j < pat->len; j++) {
@@ -849,12 +931,13 @@ void bmo_subdivide_edges_exec(BMesh *bm, BMOperator *op)
 					}
 				}
 				if (matched) {
-					BLI_array_grow_one(facedata);
-					b = BLI_array_count(facedata) - 1;
-					facedata[b].pat = pat;
-					facedata[b].start = verts[i];
-					facedata[b].face = face;
-					facedata[b].totedgesel = totesel;
+					SubDFaceData *fd;
+
+					fd = BLI_stack_push_r(facedata);
+					fd->pat = pat;
+					fd->start = verts[i];
+					fd->face = face;
+					fd->totedgesel = totesel;
 					BMO_elem_flag_enable(bm, face, SUBD_SPLIT);
 					break;
 				}
@@ -885,15 +968,15 @@ void bmo_subdivide_edges_exec(BMesh *bm, BMOperator *op)
 					}
 				}
 				if (matched) {
-					BLI_array_grow_one(facedata);
-					j = BLI_array_count(facedata) - 1;
+					SubDFaceData *fd;
 
 					BMO_elem_flag_enable(bm, face, SUBD_SPLIT);
 
-					facedata[j].pat = pat;
-					facedata[j].start = verts[a];
-					facedata[j].face = face;
-					facedata[j].totedgesel = totesel;
+					fd = BLI_stack_push_r(facedata);
+					fd->pat = pat;
+					fd->start = verts[a];
+					fd->face = face;
+					fd->totedgesel = totesel;
 					break;
 				}
 			}
@@ -901,39 +984,44 @@ void bmo_subdivide_edges_exec(BMesh *bm, BMOperator *op)
 		}
 		
 		if (!matched && totesel) {
-			BLI_array_grow_one(facedata);
-			j = BLI_array_count(facedata) - 1;
+			SubDFaceData *fd;
 			
 			BMO_elem_flag_enable(bm, face, SUBD_SPLIT);
-			facedata[j].totedgesel = totesel;
-			facedata[j].face = face;
+
+			/* must initialize all members here */
+			fd = BLI_stack_push_r(facedata);
+			fd->start = NULL;
+			fd->pat = NULL;
+			fd->totedgesel = totesel;
+			fd->face = face;
 		}
 	}
 
-	einput = BMO_slot_get(op, "edges");
+	einput = BMO_slot_get(op->slots_in, "edges");
 
 	/* go through and split edges */
 	for (i = 0; i < einput->len; i++) {
-		edge = ((BMEdge **)einput->data.p)[i];
+		edge = einput->data.buf[i];
 		bm_subdivide_multicut(bm, edge, &params, edge->v1, edge->v2);
 	}
 
 	/* copy original-geometry displacements to current coordinates */
 	BM_ITER_MESH (v, &viter, bm, BM_VERTS_OF_MESH) {
-		float *co = CustomData_bmesh_get_n(&bm->vdata, v->head.data, CD_SHAPEKEY, skey);
+		const float *co = BM_ELEM_CD_GET_VOID_P(v, params.shape_info.cd_vert_shape_offset_tmp);
 		copy_v3_v3(v->co, co);
 	}
 
-	i = 0;
-	for (i = 0; i < BLI_array_count(facedata); i++) {
-		face = facedata[i].face;
+	for (; !BLI_stack_is_empty(facedata); BLI_stack_discard(facedata)) {
+		SubDFaceData *fd = BLI_stack_peek(facedata);
+
+		face = fd->face;
 
 		/* figure out which pattern to use */
 		BLI_array_empty(verts);
 
-		pat = facedata[i].pat;
+		pat = fd->pat;
 
-		if (!pat && facedata[i].totedgesel == 2) {
+		if (!pat && fd->totedgesel == 2) {
 			int vlen;
 			
 			/* ok, no pattern.  we still may be able to do something */
@@ -976,13 +1064,13 @@ void bmo_subdivide_edges_exec(BMesh *bm, BMOperator *op)
 
 			BLI_array_grow_items(loops_split, numcuts);
 			for (j = 0; j < numcuts; j++) {
-				int ok = TRUE;
+				bool ok = true;
 
 				/* Check for special case: [#32500]
-				 * This edge pair could be used by more then one face,
+				 * This edge pair could be used by more than one face,
 				 * in this case it used to (2.63), split both faces along the same verts
 				 * while it could be calculated which face should do the split,
-				 * its ambigious, so in this case we're better off to skip them as exceptional cases
+				 * it's ambiguous, so in this case we're better off to skip them as exceptional cases
 				 * and not try to be clever guessing which face to cut up.
 				 *
 				 * To avoid this case we need to check:
@@ -999,7 +1087,7 @@ void bmo_subdivide_edges_exec(BMesh *bm, BMOperator *op)
 								BLI_assert(other_loop->prev->v != loops[a]->v);
 								BLI_assert(other_loop->next->v != loops[a]->v);
 
-								ok = FALSE;
+								ok = false;
 								break;
 							}
 						}
@@ -1007,7 +1095,7 @@ void bmo_subdivide_edges_exec(BMesh *bm, BMOperator *op)
 				}
 
 
-				if (ok == TRUE) {
+				if (ok == true) {
 					loops_split[j][0] = loops[a];
 					loops_split[j][1] = loops[b];
 				}
@@ -1025,14 +1113,16 @@ void bmo_subdivide_edges_exec(BMesh *bm, BMOperator *op)
 			 * - concave corner of an ngon.
 			 * - 2 edges being used in 2+ ngons.
 			 */
-//			BM_face_legal_splits(bm, face, loops_split, BLI_array_count(loops_split));
+//			BM_face_splits_check_legal(bm, face, loops_split, BLI_array_count(loops_split));
 
 			for (j = 0; j < BLI_array_count(loops_split); j++) {
 				if (loops_split[j][0]) {
-					BLI_assert(BM_edge_exists(loops_split[j][0]->v, loops_split[j][1]->v) == FALSE);
-
-					/* BMFace *nf = */ /* UNUSED */
-					BM_face_split(bm, face, loops_split[j][0]->v, loops_split[j][1]->v, &nl, NULL, FALSE);
+					BMFace *f_new;
+					BLI_assert(BM_edge_exists(loops_split[j][0]->v, loops_split[j][1]->v) == NULL);
+					f_new = BM_face_split(bm, face, loops_split[j][0], loops_split[j][1], &l_new, NULL, false);
+					if (f_new) {
+						BMO_elem_flag_enable(bm, l_new->e, ELE_INNER);
+					}
 				}
 			}
 
@@ -1043,8 +1133,8 @@ void bmo_subdivide_edges_exec(BMesh *bm, BMOperator *op)
 		}
 
 		a = 0;
-		BM_ITER_ELEM_INDEX (nl, &liter, face, BM_LOOPS_OF_FACE, j) {
-			if (nl->v == facedata[i].start) {
+		BM_ITER_ELEM_INDEX (l_new, &liter, face, BM_LOOPS_OF_FACE, j) {
+			if (l_new->v == fd->start) {
 				a = j + 1;
 				break;
 			}
@@ -1052,9 +1142,9 @@ void bmo_subdivide_edges_exec(BMesh *bm, BMOperator *op)
 
 		BLI_array_grow_items(verts, face->len);
 
-		BM_ITER_ELEM_INDEX (nl, &liter, face, BM_LOOPS_OF_FACE, j) {
+		BM_ITER_ELEM_INDEX (l_new, &liter, face, BM_LOOPS_OF_FACE, j) {
 			b = (j - a + face->len) % face->len;
-			verts[b] = nl->v;
+			verts[b] = l_new->v;
 		}
 
 		BM_CHECK_ELEMENT(face);
@@ -1063,89 +1153,73 @@ void bmo_subdivide_edges_exec(BMesh *bm, BMOperator *op)
 
 	/* copy original-geometry displacements to current coordinates */
 	BM_ITER_MESH (v, &viter, bm, BM_VERTS_OF_MESH) {
-		float *co = CustomData_bmesh_get_n(&bm->vdata, v->head.data, CD_SHAPEKEY, skey);
+		const float *co = BM_ELEM_CD_GET_VOID_P(v, params.shape_info.cd_vert_shape_offset_tmp);
 		copy_v3_v3(v->co, co);
 	}
 
-	BM_data_layer_free_n(bm, &bm->vdata, CD_SHAPEKEY, skey);
+	BM_data_layer_free_n(bm, &bm->vdata, CD_SHAPEKEY, params.shape_info.tmpkey);
 	
-	if (facedata) BLI_array_free(facedata);
+	BLI_stack_free(facedata);
 	if (edges) BLI_array_free(edges);
 	if (verts) BLI_array_free(verts);
 	BLI_array_free(loops_split);
 	BLI_array_free(loops);
 
-	BMO_slot_buffer_from_enabled_flag(bm, op, "outinner", BM_ALL, ELE_INNER);
-	BMO_slot_buffer_from_enabled_flag(bm, op, "outsplit", BM_ALL, ELE_SPLIT);
+	BMO_slot_buffer_from_enabled_flag(bm, op, op->slots_out, "geom_inner.out", BM_ALL_NOLOOP, ELE_INNER);
+	BMO_slot_buffer_from_enabled_flag(bm, op, op->slots_out, "geom_split.out", BM_ALL_NOLOOP, ELE_SPLIT);
 	
-	BMO_slot_buffer_from_enabled_flag(bm, op, "geomout", BM_ALL, ELE_INNER | ELE_SPLIT | SUBD_SPLIT);
+	BMO_slot_buffer_from_enabled_flag(bm, op, op->slots_out, "geom.out", BM_ALL_NOLOOP, ELE_INNER | ELE_SPLIT | SUBD_SPLIT);
 }
 
 /* editmesh-emulating function */
 void BM_mesh_esubdivide(BMesh *bm, const char edge_hflag,
-                        float smooth, float fractal, float along_normal,
-                        int numcuts,
-                        int seltype, int cornertype,
-                        const short use_singleedge, const short use_gridfill,
-                        int seed)
+                        const float smooth, const short smooth_falloff, const bool use_smooth_even,
+                        const float fractal, const float along_normal,
+                        const int numcuts,
+                        const int seltype, const int cornertype,
+                        const short use_single_edge, const short use_grid_fill,
+                        const short use_only_quads,
+                        const int seed)
 {
 	BMOperator op;
 	
 	/* use_sphere isnt exposed here since its only used for new primitives */
 	BMO_op_initf(bm, &op, BMO_FLAG_DEFAULTS,
 	             "subdivide_edges edges=%he "
-	             "smooth=%f fractal=%f along_normal=%f "
-	             "numcuts=%i "
-	             "quadcornertype=%i "
-	             "use_singleedge=%b use_gridfill=%b "
+	             "smooth=%f smooth_falloff=%i use_smooth_even=%b "
+	             "fractal=%f along_normal=%f "
+	             "cuts=%i "
+	             "quad_corner_type=%i "
+	             "use_single_edge=%b use_grid_fill=%b "
+	             "use_only_quads=%b "
 	             "seed=%i",
 	             edge_hflag,
-	             smooth, fractal, along_normal,
+	             smooth, smooth_falloff, use_smooth_even,
+	             fractal, along_normal,
 	             numcuts,
 	             cornertype,
-	             use_singleedge, use_gridfill,
+	             use_single_edge, use_grid_fill,
+	             use_only_quads,
 	             seed);
 	
 	BMO_op_exec(bm, &op);
 	
-	if (seltype == SUBDIV_SELECT_INNER) {
-		BMOIter iter;
-		BMElem *ele;
-
-		for (ele = BMO_iter_new(&iter, bm, &op, "outinner", BM_EDGE | BM_VERT); ele; ele = BMO_iter_step(&iter)) {
-			BM_elem_select_set(bm, ele, TRUE);
-		}
-	}
-	else if (seltype == SUBDIV_SELECT_LOOPCUT) {
-		BMOIter iter;
-		BMElem *ele;
-		
-		/* deselect input */
-		BM_mesh_elem_hflag_disable_all(bm, BM_VERT | BM_EDGE | BM_FACE, BM_ELEM_SELECT, FALSE);
-
-		for (ele = BMO_iter_new(&iter, bm, &op, "outinner", BM_EDGE | BM_VERT); ele; ele = BMO_iter_step(&iter)) {
-			BM_elem_select_set(bm, ele, TRUE);
-
-			if (ele->head.htype == BM_VERT) {
-				BMEdge *e;
-				BMIter eiter;
-
-				BM_ITER_ELEM (e, &eiter, ele, BM_EDGES_OF_VERT) {
-					if (!BM_elem_flag_test(e, BM_ELEM_SELECT) &&
-					     BM_elem_flag_test(e->v1, BM_ELEM_SELECT) &&
-					     BM_elem_flag_test(e->v2, BM_ELEM_SELECT))
-					{
-						BM_edge_select_set(bm, e, TRUE);
-					}
-					else if (BM_elem_flag_test(e, BM_ELEM_SELECT) &&
-					         (!BM_elem_flag_test(e->v1, BM_ELEM_SELECT) ||
-					          !BM_elem_flag_test(e->v2, BM_ELEM_SELECT)))
-					{
-						BM_edge_select_set(bm, e, FALSE);
-					}
-				}
-			}
-		}
+	switch (seltype) {
+		case SUBDIV_SELECT_NONE:
+			break;
+		case SUBDIV_SELECT_ORIG:
+			/* set the newly created data to be selected */
+			BMO_slot_buffer_hflag_enable(bm, op.slots_out, "geom_inner.out", BM_ALL_NOLOOP, BM_ELEM_SELECT, true);
+			BM_mesh_select_flush(bm);
+			break;
+		case SUBDIV_SELECT_INNER:
+			BMO_slot_buffer_hflag_enable(bm, op.slots_out, "geom_inner.out", BM_EDGE | BM_VERT, BM_ELEM_SELECT, true);
+			break;
+		case SUBDIV_SELECT_LOOPCUT:
+			/* deselect input */
+			BM_mesh_elem_hflag_disable_all(bm, BM_VERT | BM_EDGE | BM_FACE, BM_ELEM_SELECT, false);
+			BMO_slot_buffer_hflag_enable(bm, op.slots_out, "geom_inner.out", BM_EDGE, BM_ELEM_SELECT, true);
+			break;
 	}
 
 	BMO_op_finish(bm, &op);
@@ -1156,22 +1230,21 @@ void bmo_bisect_edges_exec(BMesh *bm, BMOperator *op)
 	BMOIter siter;
 	BMEdge *e;
 	SubDParams params = {0};
-	int skey;
 	
-	params.numcuts = BMO_slot_int_get(op, "numcuts");
+	params.numcuts = BMO_slot_int_get(op->slots_in, "cuts");
 	params.op = op;
+	params.slot_edge_percents = BMO_slot_get(op->slots_in, "edge_percents");
 	
 	BM_data_layer_add(bm, &bm->vdata, CD_SHAPEKEY);
-	skey = CustomData_number_of_layers(&bm->vdata, CD_SHAPEKEY) - 1;
-	
-	params.origkey = skey;
+
+	bmo_subd_init_shape_info(bm, &params);
 
 	/* go through and split edges */
-	BMO_ITER (e, &siter, bm, op, "edges", BM_EDGE) {
+	BMO_ITER (e, &siter, op->slots_in, "edges", BM_EDGE) {
 		bm_subdivide_multicut(bm, e, &params, e->v1, e->v2);
 	}
 
-	BMO_slot_buffer_from_enabled_flag(bm, op, "outsplit", BM_ALL, ELE_SPLIT);
+	BMO_slot_buffer_from_enabled_flag(bm, op, op->slots_out, "geom_split.out", BM_ALL_NOLOOP, ELE_SPLIT);
 
-	BM_data_layer_free_n(bm, &bm->vdata, CD_SHAPEKEY, skey);
+	BM_data_layer_free_n(bm, &bm->vdata, CD_SHAPEKEY, params.shape_info.tmpkey);
 }
