@@ -37,6 +37,7 @@
 #include "BKE_object.h"
 #include "BKE_armature.h"
 #include "BLI_string.h"
+#include "BLI_listbase.h"
 #include "ED_armature.h"
 
 #include "ArmatureImporter.h"
@@ -49,8 +50,37 @@ static const char *bc_get_joint_name(T *node)
 	return id.size() ? id.c_str() : node->getOriginalId().c_str();
 }
 
-ArmatureImporter::ArmatureImporter(UnitConverter *conv, MeshImporterBase *mesh, AnimationImporterBase *anim, Scene *sce) :
-	TransformReader(conv), scene(sce), empty(NULL), mesh_importer(mesh), anim_importer(anim) {
+static EditBone *get_edit_bone(bArmature * armature, char *name) {
+	EditBone  *eBone;
+
+	eBone = (EditBone *)armature->edbo->first;
+	for (eBone; eBone; eBone = eBone->next) {
+		if (!strcmp(name, eBone->name))
+			return eBone;
+	}
+
+	return NULL;
+
+}
+
+
+
+ArmatureImporter::ArmatureImporter(UnitConverter *conv, MeshImporterBase *mesh, Scene *sce, const ImportSettings *import_settings) :
+    import_settings(import_settings),
+	unit_converter(conv),
+	TransformReader(conv), 
+	scene(sce), 
+	empty(NULL), 
+	mesh_importer(mesh) {
+}
+
+void ArmatureImporter::clear_extended_boneset()
+{
+	for (std::map<std::string, BoneExtended *>::iterator it = extended_bones.begin(); it != extended_bones.end(); ++it) {
+		if (it->second != NULL)
+			delete it->second;
+	}
+	extended_bones.clear();
 }
 
 ArmatureImporter::~ArmatureImporter()
@@ -60,6 +90,7 @@ ArmatureImporter::~ArmatureImporter()
 	for (it = skin_by_data_uid.begin(); it != skin_by_data_uid.end(); it++) {
 		it->second.free();
 	}
+	clear_extended_boneset();
 }
 
 #if 0
@@ -78,241 +109,218 @@ JointData *ArmatureImporter::get_joint_data(COLLADAFW::Node *node);
 	return &joint_index_to_joint_info_map[joint_index];
 }
 #endif
-void ArmatureImporter::create_unskinned_bone(COLLADAFW::Node *node, EditBone *parent, int totchild,
-                                             float parent_mat[][4], Object *ob_arm)
+
+int ArmatureImporter::create_bone(SkinInfo *skin, COLLADAFW::Node *node, EditBone *parent, int totchild,
+                                   float parent_mat[4][4], bArmature *arm)
 {
+	float mat[4][4];
+	float joint_inv_bind_mat[4][4];
+	int chain_length = 0;
+
+	//Checking if bone is already made.
 	std::vector<COLLADAFW::Node *>::iterator it;
 	it = std::find(finished_joints.begin(), finished_joints.end(), node);
-	if (it != finished_joints.end()) return;
+	if (it != finished_joints.end()) return chain_length;
 
-	float mat[4][4];
-	float obmat[4][4];
-
-	// object-space
-	get_node_mat(obmat, node, NULL, NULL);
-
-	EditBone *bone = ED_armature_edit_bone_add((bArmature *)ob_arm->data, (char *)bc_get_joint_name(node));
-	totbone++;
+	// JointData* jd = get_joint_data(node);
 	
+	// TODO rename from Node "name" attrs later
+	EditBone *bone = ED_armature_edit_bone_add(arm, (char *)bc_get_joint_name(node));
+	totbone++;
+
+	if (skin && skin->get_joint_inv_bind_matrix(joint_inv_bind_mat, node)) {
+		// get original world-space matrix
+		invert_m4_m4(mat, joint_inv_bind_mat);
+
+		// And make local to armature
+		Object *ob_arm = skin->BKE_armature_from_object();
+		if (ob_arm) {
+			float invmat[4][4];
+			invert_m4_m4(invmat, ob_arm->obmat);
+			mul_m4_m4m4(mat, invmat, mat);
+		}
+	}
+	// create a bone even if there's no joint data for it (i.e. it has no influence)
+	else {
+		float obmat[4][4];
+		// bone-space
+		get_node_mat(obmat, node, NULL, NULL);
+
+		// get world-space
+		if (parent) {
+			mul_m4_m4m4(mat, parent_mat, obmat);
+		}
+		else {
+			copy_m4_m4(mat, obmat);
+		}
+	}
+
 	if (parent) bone->parent = parent;
 
-	float angle = 0;
+	float loc[3], size[3], rot[3][3]; 
+	float angle;
+	float vec[3] = {0.0f, 0.5f, 0.0f};
+	mat4_to_loc_rot_size(loc, rot, size, mat);
+	//copy_m3_m4(bonemat,mat);
+	mat3_to_vec_roll(rot, vec, &angle);
 
-	// get world-space
-	if (parent) {
-		mult_m4_m4m4(mat, parent_mat, obmat);
-
-	}
-	else {
-		copy_m4_m4(mat, obmat);
-
-	}
-	float loc[3], size[3], rot[3][3];
-	mat4_to_loc_rot_size(loc, rot, size, obmat);
-	mat3_to_vec_roll(rot, NULL, &angle);
 	bone->roll = angle;
 	// set head
 	copy_v3_v3(bone->head, mat[3]);
 
 	// set tail, don't set it to head because 0-length bones are not allowed
-	float vec[3] = {0.0f, 0.5f, 0.0f};
 	add_v3_v3v3(bone->tail, bone->head, vec);
 
-	// set parent tail
-	if (parent && totchild == 1) {
-		copy_v3_v3(parent->tail, bone->head);
-		
-		// not setting BONE_CONNECTED because this would lock child bone location with respect to parent
-		// bone->flag |= BONE_CONNECTED;
-	
-		// XXX increase this to prevent "very" small bones?
-		const float epsilon = 0.000001f;
+	/* find smallest bone length in armature (used later for leaf bone length) */
+	if (parent) {
 
-		// derive leaf bone length
-		float length = len_v3v3(parent->head, parent->tail);
-		if ((length < leaf_bone_length || totbone == 0) && length > epsilon) {
+		/* guess reasonable leaf bone length */
+		float length = len_v3v3(parent->head, bone->head);
+		if ((length < leaf_bone_length || totbone == 0) && length > MINIMUM_BONE_LENGTH) {
 			leaf_bone_length = length;
 		}
-
-		// treat zero-sized bone like a leaf bone
-		if (length <= epsilon) {
-			add_leaf_bone(parent_mat, parent, node);
-		}
-
 	}
 
 	COLLADAFW::NodePointerArray& children = node->getChildNodes();
+
+	BoneExtended &be = add_bone_extended(bone, node);
+	be.set_leaf_bone(true);
+
 	for (unsigned int i = 0; i < children.getCount(); i++) {
-		create_unskinned_bone(children[i], bone, children.getCount(), mat, ob_arm);
+		int cl = create_bone(skin, children[i], bone, children.getCount(), mat, arm);
+		if (cl > chain_length)
+			chain_length = cl;
 	}
 
-	// in second case it's not a leaf bone, but we handle it the same way
-	if (!children.getCount() || children.getCount() > 1) {
-		add_leaf_bone(mat, bone, node);
-	}
 
+	bone->length = len_v3v3(bone->head, bone->tail);
+	joint_by_uid[node->getUniqueId()] = node;
 	finished_joints.push_back(node);
 
+	be.set_chain_length(chain_length + 1);
+
+	return chain_length + 1;
 }
 
-void ArmatureImporter::create_bone(SkinInfo& skin, COLLADAFW::Node *node, EditBone *parent, int totchild,
-                                   float parent_mat[][4], bArmature *arm)
+/**
+  * Collada only knows Joints, hence bones at the end of a bone chain
+  * don't have a defined length. This function guesses reasonable
+  * tail locations for the affected bones (nodes which don't have any connected child)
+  * Hint: The extended_bones set gets populated in ArmatureImporter::create_bone
+**/
+void ArmatureImporter::fix_leaf_bones(bArmature *armature, Bone *bone)
 {
-	//Checking if bone is already made.
-	std::vector<COLLADAFW::Node *>::iterator it;
-	it = std::find(finished_joints.begin(), finished_joints.end(), node);
-	if (it != finished_joints.end()) return;
+	/* armature has no bones */
+	if (bone == NULL)
+		return;
 
-	float joint_inv_bind_mat[4][4];
+	BoneExtended *be = extended_bones[bone->name];
+	if (be != NULL && be->is_leaf_bone() ) {
+		/* Collada only knows Joints, Here we guess a reasonable leaf bone length */
+		float leaf_length = (leaf_bone_length == FLT_MAX) ? 1.0 : leaf_bone_length;
 
-	// JointData* jd = get_joint_data(node);
-
-	float mat[4][4];
-
-	// TODO rename from Node "name" attrs later
-	EditBone *bone = ED_armature_edit_bone_add(arm, (char *)bc_get_joint_name(node));
-	totbone++;
-
-	if (skin.get_joint_inv_bind_matrix(joint_inv_bind_mat, node)) {
-		// get original world-space matrix
-		invert_m4_m4(mat, joint_inv_bind_mat);
-	}
-	// create a bone even if there's no joint data for it (i.e. it has no influence)
-	else {
-		float obmat[4][4];
-
-		// object-space
-		get_node_mat(obmat, node, NULL, NULL);
-
-		// get world-space
-		if (parent)
-			mult_m4_m4m4(mat, parent_mat, obmat);
-		else
-			copy_m4_m4(mat, obmat);
-
-		float loc[3], size[3], rot[3][3], angle;
-		mat4_to_loc_rot_size(loc, rot, size, obmat);
-		mat3_to_vec_roll(rot, NULL, &angle);
-		bone->roll = angle;
-	}
-
-	
-	if (parent) bone->parent = parent;
-
-	// set head
-	copy_v3_v3(bone->head, mat[3]);
-
-	// set tail, don't set it to head because 0-length bones are not allowed
-	float vec[3] = {0.0f, 0.5f, 0.0f};
-	add_v3_v3v3(bone->tail, bone->head, vec);
-
-	// set parent tail
-	if (parent && totchild == 1) {
-		copy_v3_v3(parent->tail, bone->head);
-
-		// not setting BONE_CONNECTED because this would lock child bone location with respect to parent
-		// bone->flag |= BONE_CONNECTED;
-
-		// XXX increase this to prevent "very" small bones?
-		const float epsilon = 0.000001f;
-
-		// derive leaf bone length
-		float length = len_v3v3(parent->head, parent->tail);
-		if ((length < leaf_bone_length || totbone == 0) && length > epsilon) {
-			leaf_bone_length = length;
-		}
-
-		// treat zero-sized bone like a leaf bone
-		if (length <= epsilon) {
-			add_leaf_bone(parent_mat, parent, node);
-		}
-
-		/*
-#if 0
-		// and which row in mat is bone direction
+		EditBone *ebone = get_edit_bone(armature, bone->name);
 		float vec[3];
-		sub_v3_v3v3(vec, parent->tail, parent->head);
-#ifdef COLLADA_DEBUG
-		print_v3("tail - head", vec);
-		print_m4("matrix", parent_mat);
-#endif
-		for (int i = 0; i < 3; i++) {
-#ifdef COLLADA_DEBUG
-			char *axis_names[] = {"X", "Y", "Z"};
-			printf("%s-axis length is %f\n", axis_names[i], len_v3(parent_mat[i]));
-#endif
-			float angle = angle_v2v2(vec, parent_mat[i]);
-			if (angle < min_angle) {
-#ifdef COLLADA_DEBUG
-				print_v3("picking", parent_mat[i]);
-				printf("^ %s axis of %s's matrix\n", axis_names[i], get_dae_name(node));
-#endif
-				bone_direction_row = i;
-				min_angle = angle;
+
+		if (this->import_settings->fix_orientation) {
+			if (ebone->parent != NULL) {
+				EditBone *parent = ebone->parent;
+				sub_v3_v3v3(vec, ebone->head, parent->tail);
+				if (len_squared_v3(vec) < MINIMUM_BONE_LENGTH)
+				{
+					sub_v3_v3v3(vec, parent->tail, parent->head);
+				}
+			}
+			else {
+				vec[2] = 0.1f;
+				sub_v3_v3v3(vec, ebone->tail, ebone->head);
 			}
 		}
-#endif
-		*/
+		else {
+			sub_v3_v3v3(vec, ebone->tail, ebone->head);
+		}
+
+		normalize_v3_v3(vec, vec);
+		mul_v3_fl(vec, leaf_length);
+		add_v3_v3v3(ebone->tail, ebone->head, vec);
 	}
 
-	COLLADAFW::NodePointerArray& children = node->getChildNodes();
-	for (unsigned int i = 0; i < children.getCount(); i++) {
-		create_bone(skin, children[i], bone, children.getCount(), mat, arm);
+	for (Bone *child = (Bone *)bone->childbase.first; child; child = child->next) {
+		fix_leaf_bones(armature, child);
 	}
 
-	// in second case it's not a leaf bone, but we handle it the same way
-	if (!children.getCount() || children.getCount() > 1) {
-		add_leaf_bone(mat, bone, node);
-	}
-
-	finished_joints.push_back(node);
 }
 
-void ArmatureImporter::add_leaf_bone(float mat[][4], EditBone *bone,  COLLADAFW::Node *node)
+
+void ArmatureImporter::connect_bone_chains(bArmature *armature, Bone *parentbone, int clip)
 {
-	LeafBone leaf;
+	BoneExtended *dominant_child = NULL;
+	int maxlen = 0;
+	Bone *child = (Bone *)parentbone->childbase.first;
+	if (child && (import_settings->find_chains || child->next==NULL) )
+	{
+		for (child; child; child = child->next) {
+			BoneExtended *be = extended_bones[child->name];
+			if (be != NULL) {
+				if (be->get_chain_length() <= clip) {
+					if (be->get_chain_length() > maxlen) {
+						dominant_child = be;
+						maxlen = be->get_chain_length();
+					}
+					else if (be->get_chain_length() == maxlen) {
+						dominant_child = NULL;
+					}
+				}
+			}
+		}
+	}
 
-	leaf.bone = bone;
-	copy_m4_m4(leaf.mat, mat);
-	BLI_strncpy(leaf.name, bone->name, sizeof(leaf.name));
-	
-	TagsMap::iterator etit;
-	ExtraTags *et = 0;
-	etit = uid_tags_map.find(node->getUniqueId().toAscii());
-	if (etit != uid_tags_map.end()) {
-		et = etit->second;
-		//else return;
+	BoneExtended *pbe = extended_bones[parentbone->name];
+	if (dominant_child != NULL) {
+		/* Found a valid chain. Now connect current bone with that chain.*/
+		EditBone *pebone = get_edit_bone(armature, parentbone->name);
+		EditBone *cebone = get_edit_bone(armature, dominant_child->get_name());
+		if (pebone && !(cebone->flag & BONE_CONNECTED)) {
 
-		float x, y, z;
-		et->setData("tip_x", &x);
-		et->setData("tip_y", &y);
-		et->setData("tip_z", &z);
-		float vec[3] = {x, y, z};
-		copy_v3_v3(leaf.bone->tail, leaf.bone->head);
-		add_v3_v3v3(leaf.bone->tail, leaf.bone->head, vec);
+			float vec[3];
+			sub_v3_v3v3(vec, cebone->head, pebone->head);
+
+			/*
+			 * It is possible that the child's head is located on the parents head.
+			 * When this happens, then moving the parent's tail to the child's head
+			 * would result in a zero sized bone and Blender would  silently remove the bone.
+			 * So we move the tail only when the resulting bone has a minimum length:
+			 */
+
+			if (len_squared_v3(vec) > MINIMUM_BONE_LENGTH)
+			{
+				pebone->tail[0] = cebone->head[0];
+				pebone->tail[1] = cebone->head[1];
+				pebone->tail[2] = cebone->head[2];
+
+				if (pbe && pbe->get_chain_length() >= this->import_settings->min_chain_length) {
+					cebone->flag |= BONE_CONNECTED;
+					printf("Connecting chain: parent %s --> %s (child)\n", pebone->name, cebone->name);
+					pbe->set_leaf_bone(false);
+				}
+			}
+		}
+		for (Bone *child = (Bone *)parentbone->childbase.first; child; child = child->next) {
+			ArmatureImporter::connect_bone_chains(armature, child, UNLIMITED_CHAIN_MAX);
+		}
+	}
+	else if (maxlen>1 && maxlen > this->import_settings->min_chain_length) {
+		/* Try again with smaller chain length */
+		ArmatureImporter::connect_bone_chains(armature, parentbone, maxlen - 1);
 	}
 	else {
-		leaf_bones.push_back(leaf);
+		/* can't connect this Bone. Proceed with children ... */
+		if (pbe) pbe->set_leaf_bone(true);
+		for (Bone *child = (Bone *)parentbone->childbase.first; child; child = child->next) {
+			ArmatureImporter::connect_bone_chains(armature, child, UNLIMITED_CHAIN_MAX);
+		}
 	}
-}
 
-void ArmatureImporter::fix_leaf_bones( )
-{
-	// just setting tail for leaf bones here
-
-	std::vector<LeafBone>::iterator it;
-	for (it = leaf_bones.begin(); it != leaf_bones.end(); it++) {
-		LeafBone& leaf = *it;
-
-		// pointing up
-		float vec[3] = {0.0f, 0.0f, 0.1f};
-		
-		// if parent: take parent length and direction
-		if (leaf.bone->parent) sub_v3_v3v3(vec, leaf.bone->parent->tail, leaf.bone->parent->head);
-
-		copy_v3_v3(leaf.bone->tail, leaf.bone->head);
-		add_v3_v3v3(leaf.bone->tail, leaf.bone->head, vec);
-	}
 }
 
 #if 0
@@ -410,46 +418,50 @@ ArmatureJoints& ArmatureImporter::get_armature_joints(Object *ob_arm)
 void ArmatureImporter::create_armature_bones( )
 {
 	std::vector<COLLADAFW::Node *>::iterator ri;
+
+	leaf_bone_length = FLT_MAX;
 	//if there is an armature created for root_joint next root_joint
 	for (ri = root_joints.begin(); ri != root_joints.end(); ri++) {
 		if (get_armature_for_joint(*ri) != NULL) continue;
 		
-		//add armature object for current joint
-		//Object *ob_arm = bc_add_object(scene, OB_ARMATURE, NULL);
-
 		Object *ob_arm = joint_parent_map[(*ri)->getUniqueId()];
-
 		if (!ob_arm)
 			continue;
 
-		//ob_arm->type = OB_ARMATURE;
-		ED_armature_to_edit(ob_arm);
+		bArmature * armature = (bArmature *)ob_arm->data;
+		if (!armature)
+			continue;
 
-		// min_angle = 360.0f;		// minimum angle between bone head-tail and a row of bone matrix
+		char * bone_name = (char *)bc_get_joint_name(*ri);
+		Bone *bone = BKE_armature_find_bone_name(armature, bone_name);
+		if (bone) {
+			fprintf(stderr, "Reuse of child bone [%s] as root bone in same Armature is not supported.\n", bone_name);
+			continue;
+		}
 
-		// create unskinned bones
-		/*
-		 * TODO:
-		 * check if bones have already been created for a given joint
-		 */
-		leaf_bone_length = FLT_MAX;
-		create_unskinned_bone(*ri, NULL, (*ri)->getChildNodes().getCount(), NULL, ob_arm);
+		clear_extended_boneset();
 
-		fix_leaf_bones();
+		ED_armature_to_edit(armature);
+
+		create_bone(NULL, *ri , NULL, (*ri)->getChildNodes().getCount(), NULL, armature);
+
+		/* exit armature edit mode to populate the Armature object */
+		ED_armature_from_edit(armature);
+		ED_armature_edit_free(armature);
+
+		/* and step back to edit mode to fix the leaf nodes */
+		ED_armature_to_edit(armature);
+
+		connect_bone_chains(armature, (Bone *)armature->bonebase.first, UNLIMITED_CHAIN_MAX);
+		fix_leaf_bones(armature, (Bone *)armature->bonebase.first);
 
 		// exit armature edit mode
-	
 		unskinned_armature_map[(*ri)->getUniqueId()] = ob_arm;
 
-		ED_armature_from_edit(ob_arm);
-
-		set_pose(ob_arm, *ri, NULL, NULL);
-
-		ED_armature_edit_free(ob_arm);
+		ED_armature_from_edit(armature);
+		ED_armature_edit_free(armature);
 		DAG_id_tag_update(&ob_arm->id, OB_RECALC_OB | OB_RECALC_DATA);
 	}
-
-	
 }
 
 void ArmatureImporter::create_armature_bones(SkinInfo& skin)
@@ -521,19 +533,33 @@ void ArmatureImporter::create_armature_bones(SkinInfo& skin)
 			break;
 	}
 
-	if (shared)
+	if (!shared && this->joint_parent_map.size() > 0) {
+		// All armatures have been created while creating the Node tree. 
+		// The Collada exporter currently does not create a 
+		// strict relationship between geometries and armatures
+		// So when we reimport a Blender collada file, then we have
+		// to guess what is meant.
+		// XXX This is not safe when we have more than one armatures
+		// in the import.
+		shared = this->joint_parent_map.begin()->second;
+	}
+
+	if (shared) {
 		ob_arm = skin.set_armature(shared);
-	else
+	}
+	else {
 		ob_arm = skin.create_armature(scene);  //once for every armature
+	}
 
 	// enter armature edit mode
-	ED_armature_to_edit(ob_arm);
+	bArmature * armature = (bArmature *)ob_arm->data;
+	ED_armature_to_edit(armature);
+
+	clear_extended_boneset();
 
-	leaf_bones.clear();
 	totbone = 0;
 	// bone_direction_row = 1; // TODO: don't default to Y but use asset and based on it decide on default row
 	leaf_bone_length = FLT_MAX;
-	// min_angle = 360.0f;		// minimum angle between bone head-tail and a row of bone matrix
 
 	// create bones
 	/*
@@ -549,38 +575,35 @@ void ArmatureImporter::create_armature_bones(SkinInfo& skin)
 
 		// since root_joints may contain joints for multiple controllers, we need to filter
 		if (skin.uses_joint_or_descendant(*ri)) {
-			create_bone(skin, *ri, NULL, (*ri)->getChildNodes().getCount(), NULL, (bArmature *)ob_arm->data);
+			create_bone(&skin, *ri, NULL, (*ri)->getChildNodes().getCount(), NULL, armature);
 
 			if (joint_parent_map.find((*ri)->getUniqueId()) != joint_parent_map.end() && !skin.get_parent())
 				skin.set_parent(joint_parent_map[(*ri)->getUniqueId()]);
 		}
 	}
 
-	fix_leaf_bones();
+	/* exit armature edit mode to populate the Armature object */
+	ED_armature_from_edit(armature);
+	ED_armature_edit_free(armature);
+
+	/* and step back to edit mode to fix the leaf nodes */
+	ED_armature_to_edit(armature);
+
+	connect_bone_chains(armature, (Bone *)armature->bonebase.first, UNLIMITED_CHAIN_MAX);
+	fix_leaf_bones(armature, (Bone *)armature->bonebase.first);
 
 	// exit armature edit mode
-	ED_armature_from_edit(ob_arm);
-	ED_armature_edit_free(ob_arm);
+	ED_armature_from_edit(armature);
+	ED_armature_edit_free(armature);
 	DAG_id_tag_update(&ob_arm->id, OB_RECALC_OB | OB_RECALC_DATA);
-
-	// set_leaf_bone_shapes(ob_arm);
-	// set_euler_rotmode();
 }
 
-
-// root - if this joint is the top joint in hierarchy, if a joint
-// is a child of a node (not joint), root should be true since
-// this is where we build armature bones from
-
-void ArmatureImporter::set_pose(Object *ob_arm,  COLLADAFW::Node *root_node, const char *parentname, float parent_mat[][4])
+void ArmatureImporter::set_pose(Object *ob_arm,  COLLADAFW::Node *root_node, const char *parentname, float parent_mat[4][4])
 { 
 	char *bone_name = (char *) bc_get_joint_name(root_node);
 	float mat[4][4];
 	float obmat[4][4];
 
-	float ax[3];
-	float angle = 0.0f;
-	
 	// object-space
 	get_node_mat(obmat, root_node, NULL, NULL);
 
@@ -590,21 +613,24 @@ void ArmatureImporter::set_pose(Object *ob_arm,  COLLADAFW::Node *root_node, con
 
 	// get world-space
 	if (parentname) {
-		mult_m4_m4m4(mat, parent_mat, obmat);
+		mul_m4_m4m4(mat, parent_mat, obmat);
 		bPoseChannel *parchan = BKE_pose_channel_find_name(ob_arm->pose, parentname);
 
-		mult_m4_m4m4(pchan->pose_mat, parchan->pose_mat, mat);
+		mul_m4_m4m4(pchan->pose_mat, parchan->pose_mat, mat);
 
 	}
 	else {
+		
 		copy_m4_m4(mat, obmat);
 		float invObmat[4][4];
 		invert_m4_m4(invObmat, ob_arm->obmat);
-		mult_m4_m4m4(pchan->pose_mat, invObmat, mat);
+		mul_m4_m4m4(pchan->pose_mat, invObmat, mat);
+		
 	}
 
-	mat4_to_axis_angle(ax, &angle, mat);
-	pchan->bone->roll = angle;
+	//float angle = 0.0f;
+	///*mat4_to_axis_angle(ax, &angle, mat);
+	//pchan->bone->roll = angle;*/
 
 
 	COLLADAFW::NodePointerArray& children = root_node->getChildNodes();
@@ -614,16 +640,16 @@ void ArmatureImporter::set_pose(Object *ob_arm,  COLLADAFW::Node *root_node, con
 
 }
 
-void ArmatureImporter::add_joint(COLLADAFW::Node *node, bool root, Object *parent, Scene *sce)
+/**
+  * root - if this joint is the top joint in hierarchy, if a joint
+  * is a child of a node (not joint), root should be true since
+  * this is where we build armature bones from
+  **/
+void ArmatureImporter::add_root_joint(COLLADAFW::Node *node, Object *parent)
 {
-	joint_by_uid[node->getUniqueId()] = node;
-	if (root) {
-		root_joints.push_back(node);
-
-		if (parent) {
-					
-			joint_parent_map[node->getUniqueId()] = parent;
-		}
+	root_joints.push_back(node);
+	if (parent) {
+		joint_parent_map[node->getUniqueId()] = parent;
 	}
 }
 
@@ -729,13 +755,12 @@ bool ArmatureImporter::write_skin_controller_data(const COLLADAFW::SkinControlle
 bool ArmatureImporter::write_controller(const COLLADAFW::Controller *controller)
 {
 	// - create and store armature object
-
-	const COLLADAFW::UniqueId& skin_id = controller->getUniqueId();
+	const COLLADAFW::UniqueId& con_id = controller->getUniqueId();
 
 	if (controller->getControllerType() == COLLADAFW::Controller::CONTROLLER_TYPE_SKIN) {
 		COLLADAFW::SkinController *co = (COLLADAFW::SkinController *)controller;
 		// to be able to find geom id by controller id
-		geom_uid_by_controller_uid[skin_id] = co->getSource();
+		geom_uid_by_controller_uid[con_id] = co->getSource();
 
 		const COLLADAFW::UniqueId& data_uid = co->getSkinControllerData();
 		if (skin_by_data_uid.find(data_uid) == skin_by_data_uid.end()) {
@@ -746,14 +771,71 @@ bool ArmatureImporter::write_controller(const COLLADAFW::Controller *controller)
 		skin_by_data_uid[data_uid].set_controller(co);
 	}
 	// morph controller
-	else {
-		// shape keys? :)
-		fprintf(stderr, "Morph controller is not supported yet.\n");
+	else if (controller->getControllerType() == COLLADAFW::Controller::CONTROLLER_TYPE_MORPH) {
+		COLLADAFW::MorphController *co = (COLLADAFW::MorphController *)controller;
+		// to be able to find geom id by controller id
+		geom_uid_by_controller_uid[con_id] = co->getSource();
+		//Shape keys are applied in DocumentImporter->finish()
+		morph_controllers.push_back(co);
 	}
 
 	return true;
 }
 
+void ArmatureImporter::make_shape_keys()
+{
+	std::vector<COLLADAFW::MorphController *>::iterator mc;
+	float weight;
+
+	for (mc = morph_controllers.begin(); mc != morph_controllers.end(); mc++) {
+		//Controller data
+		COLLADAFW::UniqueIdArray& morphTargetIds = (*mc)->getMorphTargets();
+		COLLADAFW::FloatOrDoubleArray& morphWeights = (*mc)->getMorphWeights();
+
+		//Prereq: all the geometries must be imported and mesh objects must be made
+		Object *source_ob = this->mesh_importer->get_object_by_geom_uid((*mc)->getSource());
+		
+		if (source_ob) {
+
+			Mesh *source_me = (Mesh *)source_ob->data;
+			//insert key to source mesh
+			Key *key = source_me->key = BKE_key_add((ID *)source_me);
+			key->type = KEY_RELATIVE;
+			KeyBlock *kb;
+			
+			//insert basis key
+			kb = BKE_keyblock_add_ctime(key, "Basis", false);
+			BKE_keyblock_convert_from_mesh(source_me, kb);
+
+			//insert other shape keys
+			for (int i = 0 ; i < morphTargetIds.getCount() ; i++ ) {
+				//better to have a separate map of morph objects,
+				//This'll do for now since only mesh morphing is imported
+
+				Mesh *me = this->mesh_importer->get_mesh_by_geom_uid(morphTargetIds[i]);
+				
+				if (me) {
+					me->key = key;
+					std::string morph_name = *this->mesh_importer->get_geometry_name(me->id.name);
+
+					kb = BKE_keyblock_add_ctime(key, morph_name.c_str(), false);
+					BKE_keyblock_convert_from_mesh(me, kb);
+					
+					//apply weights
+					weight =  morphWeights.getFloatValues()->getData()[i];
+					kb->curval = weight;
+				}
+				else {
+					fprintf(stderr, "Morph target geometry not found.\n");
+				}
+			}
+		}
+		else {
+			fprintf(stderr, "Morph target object not found.\n");
+		}
+	}
+}
+
 
 COLLADAFW::UniqueId *ArmatureImporter::get_geometry_uid(const COLLADAFW::UniqueId& controller_uid)
 {
@@ -792,7 +874,7 @@ void ArmatureImporter::get_rna_path_for_joint(COLLADAFW::Node *node, char *joint
 }
 
 // gives a world-space mat
-bool ArmatureImporter::get_joint_bind_mat(float m[][4], COLLADAFW::Node *joint)
+bool ArmatureImporter::get_joint_bind_mat(float m[4][4], COLLADAFW::Node *joint)
 {
 	std::map<COLLADAFW::UniqueId, SkinInfo>::iterator it;
 	bool found = false;
@@ -806,3 +888,71 @@ bool ArmatureImporter::get_joint_bind_mat(float m[][4], COLLADAFW::Node *joint)
 
 	return found;
 }
+
+
+/**
+  * BoneExtended is a helper class needed for the Bone chain finder
+  * See ArmatureImporter::fix_leaf_bones()
+  * and ArmatureImporter::connect_bone_chains()
+  **/
+
+BoneExtended::BoneExtended(EditBone *aBone) 
+{
+	this->set_name(aBone->name);
+	this->chain_length = 0;
+	this->is_leaf = false;
+}
+
+char *BoneExtended::get_name() 
+{
+	return name;
+}
+
+void BoneExtended::set_name(char *aName) 
+{
+	BLI_strncpy(name, aName, MAXBONENAME);
+}
+
+int BoneExtended::get_chain_length() 
+{
+	return chain_length;
+}
+
+void BoneExtended::set_chain_length(const int aLength) 
+{
+	chain_length = aLength;
+}
+
+
+void BoneExtended::set_leaf_bone(bool state)
+{
+	is_leaf = state;
+}
+
+bool BoneExtended::is_leaf_bone()
+{
+	return is_leaf;
+}
+
+BoneExtended &ArmatureImporter::add_bone_extended(EditBone *bone, COLLADAFW::Node *node)
+{
+
+	TagsMap::iterator etit;
+	ExtraTags *et = 0;
+	etit = uid_tags_map.find(node->getUniqueId().toAscii());
+	if (etit != uid_tags_map.end()) {
+		float x, y, z;
+
+		et = etit->second;
+		et->setData("tip_x", &x);
+		et->setData("tip_y", &y);
+		et->setData("tip_z", &z);
+		float vec[3] = { x, y, z };
+		copy_v3_v3(bone->tail, bone->head);
+		add_v3_v3v3(bone->tail, bone->head, vec);
+	}
+
+	BoneExtended *be = new BoneExtended(bone);
+	extended_bones[bone->name] = be;
+	return *be;
+}
