@@ -54,6 +54,7 @@ extern "C" {
 #include "BLI_math.h"
 #include "BLI_string.h"
 #include "BLI_utildefines.h"
+#include "BLI_fileops.h"
 
 #include "BKE_camera.h"
 #include "BKE_main.h"
@@ -76,6 +77,9 @@ extern "C" {
 
 #include "MEM_guardedalloc.h"
 
+#include "WM_api.h"
+#include "WM_types.h"
+
 }
 
 #include "ExtraHandler.h"
@@ -96,11 +100,11 @@ extern "C" {
 // creates empties for each imported bone on layer 2, for debugging
 // #define ARMATURE_TEST
 
-DocumentImporter::DocumentImporter(bContext *C, const char *filename) :
+DocumentImporter::DocumentImporter(bContext *C, const ImportSettings *import_settings) :
+	import_settings(import_settings),
 	mImportStage(General),
-	mFilename(filename),
 	mContext(C),
-	armature_importer(&unit_converter, &mesh_importer, &anim_importer, CTX_data_scene(C)),
+	armature_importer(&unit_converter, &mesh_importer, CTX_data_scene(C), import_settings),
 	mesh_importer(&unit_converter, &armature_importer, CTX_data_scene(C)),
 	anim_importer(&unit_converter, &armature_importer, CTX_data_scene(C))
 {
@@ -128,7 +132,9 @@ bool DocumentImporter::import()
 	// deselect all to select new objects
 	BKE_scene_base_deselect_all(CTX_data_scene(mContext));
 
-	if (!root.loadDocument(mFilename)) {
+	std::string mFilename = std::string(this->import_settings->filepath);
+	const std::string encodedFilename = bc_url_encode(mFilename);
+	if (!root.loadDocument(encodedFilename)) {
 		fprintf(stderr, "COLLADAFW::Root::loadDocument() returned false on 1st pass\n");
 		return false;
 	}
@@ -143,7 +149,7 @@ bool DocumentImporter::import()
 	COLLADASaxFWL::Loader loader2;
 	COLLADAFW::Root root2(&loader2, this);
 	
-	if (!root2.loadDocument(mFilename)) {
+	if (!root2.loadDocument(encodedFilename)) {
 		fprintf(stderr, "COLLADAFW::Root::loadDocument() returned false on 2nd pass\n");
 		return false;
 	}
@@ -151,7 +157,8 @@ bool DocumentImporter::import()
 	
 	delete ehandler;
 
-	mesh_importer.bmeshConversion();
+	//XXX No longer needed (geometries are now created as bmesh)
+	//mesh_importer.bmeshConversion();
 
 	return true;
 }
@@ -174,46 +181,66 @@ void DocumentImporter::finish()
 {
 	if (mImportStage != General)
 		return;
-		
+
+	Main *bmain = CTX_data_main(mContext);
+	// TODO: create a new scene except the selected <visual_scene> - use current blender scene for it
+	Scene *sce = CTX_data_scene(mContext);
+	unit_converter.calculate_scale(*sce);
+
+	std::vector<Object *> *objects_to_scale = new std::vector<Object *>();
+
 	/** TODO Break up and put into 2-pass parsing of DAE */
 	std::vector<const COLLADAFW::VisualScene *>::iterator it;
 	for (it = vscenes.begin(); it != vscenes.end(); it++) {
 		PointerRNA sceneptr, unit_settings;
 		PropertyRNA *system, *scale;
-		// TODO: create a new scene except the selected <visual_scene> - use current blender scene for it
-		Scene *sce = CTX_data_scene(mContext);
 		
 		// for scene unit settings: system, scale_length
+
 		RNA_id_pointer_create(&sce->id, &sceneptr);
 		unit_settings = RNA_pointer_get(&sceneptr, "unit_settings");
 		system = RNA_struct_find_property(&unit_settings, "system");
 		scale = RNA_struct_find_property(&unit_settings, "scale_length");
-		
-		switch (unit_converter.isMetricSystem()) {
-			case UnitConverter::Metric:
-				RNA_property_enum_set(&unit_settings, system, USER_UNIT_METRIC);
-				break;
-			case UnitConverter::Imperial:
-				RNA_property_enum_set(&unit_settings, system, USER_UNIT_IMPERIAL);
-				break;
-			default:
-				RNA_property_enum_set(&unit_settings, system, USER_UNIT_NONE);
-				break;
-		}
-		RNA_property_float_set(&unit_settings, scale, unit_converter.getLinearMeter());
-		
-		const COLLADAFW::NodePointerArray& roots = (*it)->getRootNodes();
 
-		for (unsigned int i = 0; i < roots.getCount(); i++) {
-			write_node(roots[i], NULL, sce, NULL, false);
+		if (this->import_settings->import_units) {
+			
+			switch (unit_converter.isMetricSystem()) {
+				case UnitConverter::Metric:
+					RNA_property_enum_set(&unit_settings, system, USER_UNIT_METRIC);
+					break;
+				case UnitConverter::Imperial:
+					RNA_property_enum_set(&unit_settings, system, USER_UNIT_IMPERIAL);
+					break;
+				default:
+					RNA_property_enum_set(&unit_settings, system, USER_UNIT_NONE);
+					break;
+			}
+			float unit_factor = unit_converter.getLinearMeter();
+			RNA_property_float_set(&unit_settings, scale, unit_factor);
+			fprintf(stdout, "Collada: Adjusting Blender units to Importset units: %f.\n", unit_factor);
+
 		}
+
+		// Write nodes to scene
+		const COLLADAFW::NodePointerArray& roots = (*it)->getRootNodes();
+		for (unsigned int i = 0; i < roots.getCount(); i++) {
+			std::vector<Object *> *objects_done = write_node(roots[i], NULL, sce, NULL, false);
+			objects_to_scale->insert(objects_to_scale->end(), objects_done->begin(), objects_done->end());
+		}
+
+		// update scene
+		DAG_relations_tag_update(bmain);
+		WM_event_add_notifier(mContext, NC_OBJECT | ND_TRANSFORM, NULL);
+
 	}
 
 
-	mesh_importer.optimize_material_assignments();
+	mesh_importer.optimize_material_assignements();
 
 	armature_importer.set_tags_map(this->uid_tags_map);
 	armature_importer.make_armatures(mContext);
+	armature_importer.make_shape_keys();
+	DAG_relations_tag_update(bmain);
 
 #if 0
 	armature_importer.fix_animation();
@@ -222,8 +249,9 @@ void DocumentImporter::finish()
 	for (std::vector<const COLLADAFW::VisualScene *>::iterator it = vscenes.begin(); it != vscenes.end(); it++) {
 		const COLLADAFW::NodePointerArray& roots = (*it)->getRootNodes();
 
-		for (unsigned int i = 0; i < roots.getCount(); i++)
+		for (unsigned int i = 0; i < roots.getCount(); i++) {
 			translate_anim_recursive(roots[i], NULL, NULL);
+		}
 	}
 
 	if (libnode_ob.size()) {
@@ -238,7 +266,7 @@ void DocumentImporter::finish()
 			Base *base = BKE_scene_base_find(sce, ob);
 			if (base) {
 				BLI_remlink(&sce->base, base);
-				BKE_libblock_free_us(&G.main->object, base->object);
+				BKE_libblock_free_us(G.main, base->object);
 				if (sce->basact == base)
 					sce->basact = NULL;
 				MEM_freeN(base);
@@ -246,9 +274,10 @@ void DocumentImporter::finish()
 		}
 		libnode_ob.clear();
 
-		DAG_scene_sort(CTX_data_main(mContext), sce);
-		DAG_ids_flush_update(CTX_data_main(mContext), 0);
+		DAG_relations_tag_update(bmain);
 	}
+	
+	bc_match_scale(objects_to_scale, unit_converter, !this->import_settings->import_units);
 }
 
 
@@ -256,7 +285,7 @@ void DocumentImporter::translate_anim_recursive(COLLADAFW::Node *node, COLLADAFW
 {
 
 	// The split in #29246, rootmap must point at actual root when
-	// calculating bones in apply_curves_as_matrix.
+	// calculating bones in apply_curves_as_matrix. - actual root is the root node.
 	// This has to do with inverse bind poses being world space
 	// (the sources for skinned bones' restposes) and the way
 	// non-skinning nodes have their "restpose" recursively calculated.
@@ -265,7 +294,7 @@ void DocumentImporter::translate_anim_recursive(COLLADAFW::Node *node, COLLADAFW
 	if (par) { // && par->getType() == COLLADAFW::Node::JOINT) {
 		// par is root if there's no corresp. key in root_map
 		if (root_map.find(par->getUniqueId()) == root_map.end())
-			root_map[node->getUniqueId()] = par;
+			root_map[node->getUniqueId()] = node;
 		else
 			root_map[node->getUniqueId()] = root_map[par->getUniqueId()];
 	}
@@ -282,14 +311,49 @@ void DocumentImporter::translate_anim_recursive(COLLADAFW::Node *node, COLLADAFW
 #endif
 	unsigned int i;
 
-	//for (i = 0; i < 4; i++)
-	//    ob =
-	anim_importer.translate_Animations(node, root_map, object_map, FW_object_map);
-
-	COLLADAFW::NodePointerArray &children = node->getChildNodes();
-	for (i = 0; i < children.getCount(); i++) {
-		translate_anim_recursive(children[i], node, NULL);
+	if (node->getType() == COLLADAFW::Node::JOINT && par == NULL) {
+		// For Skeletons without root node we have to simulate the
+		// root node here and recursively enter the same function
+		// XXX: maybe this can be made more elegant.
+		translate_anim_recursive(node, node, parob);
 	}
+	else {
+		anim_importer.translate_Animations(node, root_map, object_map, FW_object_map);
+		COLLADAFW::NodePointerArray &children = node->getChildNodes();
+		for (i = 0; i < children.getCount(); i++) {
+			translate_anim_recursive(children[i], node, NULL);
+		}
+	}
+}
+
+/**
+ * If the imported file was made with Blender, return the Blender version used,
+ * otherwise return an empty std::string
+ */
+std::string DocumentImporter::get_import_version(const COLLADAFW::FileInfo *asset)
+{
+	const char AUTORING_TOOL[] = "authoring_tool";
+	const std::string BLENDER("Blender ");
+	const COLLADAFW::FileInfo::ValuePairPointerArray &valuePairs = asset->getValuePairArray();
+	for ( size_t i = 0, count = valuePairs.getCount(); i < count; ++i)
+	{
+		const COLLADAFW::FileInfo::ValuePair* valuePair = valuePairs[i];
+		const COLLADAFW::String& key = valuePair->first;
+		const COLLADAFW::String& value = valuePair->second;
+		if ( key == AUTORING_TOOL )
+		{
+			if (value.compare(0, BLENDER.length(), BLENDER) == 0)
+			{
+				// Was made with Blender, now get version string
+				std::string v            = value.substr(BLENDER.length());
+				std::string::size_type n = v.find(" ");
+				if (n > 0) {
+					return v.substr(0,n);
+				}
+			}
+		}
+	}
+	return "";
 }
 
 /** When this method is called, the writer must write the global document asset.
@@ -297,7 +361,8 @@ void DocumentImporter::translate_anim_recursive(COLLADAFW::Node *node, COLLADAFW
 bool DocumentImporter::writeGlobalAsset(const COLLADAFW::FileInfo *asset)
 {
 	unit_converter.read_asset(asset);
-
+	import_from_version = get_import_version(asset);
+	anim_importer.set_import_from_version(import_from_version);
 	return true;
 }
 
@@ -311,7 +376,7 @@ bool DocumentImporter::writeScene(const COLLADAFW::Scene *scene)
 Object *DocumentImporter::create_camera_object(COLLADAFW::InstanceCamera *camera, Scene *sce)
 {
 	const COLLADAFW::UniqueId& cam_uid = camera->getInstanciatedObjectId();
-	if (uid_camera_map.find(cam_uid) == uid_camera_map.end()) {	
+	if (uid_camera_map.find(cam_uid) == uid_camera_map.end()) {
 		// fprintf(stderr, "Couldn't find camera by UID.\n");
 		return NULL;
 	}
@@ -322,14 +387,14 @@ Object *DocumentImporter::create_camera_object(COLLADAFW::InstanceCamera *camera
 	ob->data = cam;
 	old_cam->id.us--;
 	if (old_cam->id.us == 0)
-		BKE_libblock_free(&G.main->camera, old_cam);
+		BKE_libblock_free(G.main, old_cam);
 	return ob;
 }
 
 Object *DocumentImporter::create_lamp_object(COLLADAFW::InstanceLight *lamp, Scene *sce)
 {
 	const COLLADAFW::UniqueId& lamp_uid = lamp->getInstanciatedObjectId();
-	if (uid_lamp_map.find(lamp_uid) == uid_lamp_map.end()) {	
+	if (uid_lamp_map.find(lamp_uid) == uid_lamp_map.end()) {
 		fprintf(stderr, "Couldn't find lamp by UID.\n");
 		return NULL;
 	}
@@ -340,7 +405,7 @@ Object *DocumentImporter::create_lamp_object(COLLADAFW::InstanceLight *lamp, Sce
 	ob->data = la;
 	old_lamp->id.us--;
 	if (old_lamp->id.us == 0)
-		BKE_libblock_free(&G.main->lamp, old_lamp);
+		BKE_libblock_free(G.main, old_lamp);
 	return ob;
 }
 
@@ -349,7 +414,7 @@ Object *DocumentImporter::create_instance_node(Object *source_ob, COLLADAFW::Nod
 	fprintf(stderr, "create <instance_node> under node id=%s from node id=%s\n", instance_node ? instance_node->getOriginalId().c_str() : NULL, source_node ? source_node->getOriginalId().c_str() : NULL);
 
 	Object *obn = BKE_object_copy(source_ob);
-	obn->recalc |= OB_RECALC_OB | OB_RECALC_DATA | OB_RECALC_TIME;
+	DAG_id_tag_update(&obn->id, OB_RECALC_OB | OB_RECALC_DATA | OB_RECALC_TIME);
 	BKE_scene_base_add(sce, obn);
 
 	if (instance_node) {
@@ -368,7 +433,7 @@ Object *DocumentImporter::create_instance_node(Object *source_ob, COLLADAFW::Nod
 				}
 			}
 			// calc new matrix and apply
-			mult_m4_m4m4(obn->obmat, obn->obmat, mat);
+			mul_m4_m4m4(obn->obmat, obn->obmat, mat);
 			BKE_object_apply_mat4(obn, obn->obmat, 0, 0);
 		}
 	}
@@ -376,8 +441,7 @@ Object *DocumentImporter::create_instance_node(Object *source_ob, COLLADAFW::Nod
 		anim_importer.read_node_transform(source_node, obn);
 	}
 
-	DAG_scene_sort(CTX_data_main(mContext), sce);
-	DAG_ids_flush_update(CTX_data_main(mContext), 0);
+	/*DAG_relations_tag_update(CTX_data_main(mContext));*/
 
 	COLLADAFW::NodePointerArray &children = source_node->getChildNodes();
 	if (children.getCount()) {
@@ -406,23 +470,65 @@ Object *DocumentImporter::create_instance_node(Object *source_ob, COLLADAFW::Nod
 	return obn;
 }
 
-void DocumentImporter::write_node(COLLADAFW::Node *node, COLLADAFW::Node *parent_node, Scene *sce, Object *par, bool is_library_node)
+// to create constraints off node <extra> tags. Assumes only constraint data in
+// current <extra> with blender profile.
+void DocumentImporter::create_constraints(ExtraTags *et, Object *ob)
+{
+	if (et && et->isProfile("blender")) {
+		std::string name;
+		short* type = 0;
+		et->setData("type", type);
+		BKE_constraint_add_for_object(ob, "Test_con", *type);
+		
+	}
+}
+
+void DocumentImporter::report_unknown_reference(const COLLADAFW::Node &node, const std::string object_type)
+{
+	std::string id = node.getOriginalId();
+	std::string name = node.getName();
+	fprintf(stderr,
+		"error: node id=\"%s\", name=\"%s\" refers to an undefined %s.\n",
+		id.c_str(),
+		name.c_str(),
+		object_type.c_str());
+}
+
+std::vector<Object *> *DocumentImporter::write_node(COLLADAFW::Node *node, COLLADAFW::Node *parent_node, Scene *sce, Object *par, bool is_library_node)
 {
 	Object *ob = NULL;
 	bool is_joint = node->getType() == COLLADAFW::Node::JOINT;
 	bool read_transform = true;
+	std::string id   = node->getOriginalId();
+	std::string name = node->getName();
 
 	std::vector<Object *> *objects_done = new std::vector<Object *>();
+	std::vector<Object *> *root_objects = new std::vector<Object *>();
+
+	fprintf(stderr,
+			"Writing node id='%s', name='%s'\n",
+			id.c_str(),
+			name.c_str());
 
 	if (is_joint) {
-		if (par) {
-			Object *empty = par;
-			par = bc_add_object(sce, OB_ARMATURE, NULL);
-			bc_set_parent(par, empty->parent, mContext);
-			//remove empty : todo
-			object_map.insert(std::make_pair<COLLADAFW::UniqueId, Object *>(parent_node->getUniqueId(), par));
+		if (parent_node == NULL) {
+			// A Joint on root level is a skeleton without root node.
+			// Here we add the armature "on the fly":
+			par = bc_add_object(sce, OB_ARMATURE, std::string("Armature").c_str());
+			objects_done->push_back(par);
+			root_objects->push_back(par);
+			object_map.insert(std::pair<COLLADAFW::UniqueId, Object *>(node->getUniqueId(), par));
+			node_map[node->getUniqueId()] = node;
+		}
+		if (parent_node == NULL || parent_node->getType() != COLLADAFW::Node::JOINT) {
+			armature_importer.add_root_joint(node, par);
+		}
+
+		if (parent_node == NULL) {
+			// for skeletons without root node all has been done above.
+			// Skeletons with root node are handled further down.
+			return root_objects;
 		}
-		armature_importer.add_joint(node, parent_node == NULL || parent_node->getType() != COLLADAFW::Node::JOINT, par, sce);
 	}
 	else {
 		COLLADAFW::InstanceGeometryPointerArray &geom = node->getInstanceGeometries();
@@ -442,29 +548,55 @@ void DocumentImporter::write_node(COLLADAFW::Node *node, COLLADAFW::Node *parent
 		while (geom_done < geom.getCount()) {
 			ob = mesh_importer.create_mesh_object(node, geom[geom_done], false, uid_material_map,
 			                                      material_texture_mapping_map);
-			objects_done->push_back(ob);
+			if (ob == NULL) {
+				report_unknown_reference(*node, "instance_mesh");
+			}
+			else {
+				objects_done->push_back(ob);
+				if (parent_node == NULL) {
+					root_objects->push_back(ob);
+				}
+			}
 			++geom_done;
 		}
 		while (camera_done < camera.getCount()) {
 			ob = create_camera_object(camera[camera_done], sce);
 			if (ob == NULL) {
-				std::string id = node->getOriginalId();
-				std::string name = node->getName();
-				fprintf(stderr, "<node id=\"%s\", name=\"%s\" >...contains a reference to an unknown instance_camera.\n", id.c_str(), name.c_str());
+				report_unknown_reference(*node, "instance_camera");
 			}
-			else
+			else {
 				objects_done->push_back(ob);
+				if (parent_node == NULL) {
+					root_objects->push_back(ob);
+				}
+			}
 			++camera_done;
 		}
 		while (lamp_done < lamp.getCount()) {
 			ob = create_lamp_object(lamp[lamp_done], sce);
-			objects_done->push_back(ob);
+			if (ob == NULL) {
+				report_unknown_reference(*node, "instance_lamp");
+			}
+			else {
+				objects_done->push_back(ob);
+				if (parent_node == NULL) {
+					root_objects->push_back(ob);
+				}
+			}
 			++lamp_done;
 		}
 		while (controller_done < controller.getCount()) {
 			COLLADAFW::InstanceGeometry *geom = (COLLADAFW::InstanceGeometry *)controller[controller_done];
 			ob = mesh_importer.create_mesh_object(node, geom, true, uid_material_map, material_texture_mapping_map);
-			objects_done->push_back(ob);
+			if (ob == NULL) {
+				report_unknown_reference(*node, "instance_controller");
+			}
+			else {
+				objects_done->push_back(ob);
+				if (parent_node == NULL) {
+					root_objects->push_back(ob);
+				}
+			}
 			++controller_done;
 		}
 		// XXX instance_node is not supported yet
@@ -480,34 +612,51 @@ void DocumentImporter::write_node(COLLADAFW::Node *node, COLLADAFW::Node *parent
 					Object *source_ob = (Object *)it2->second;
 					COLLADAFW::Node *source_node = node_map[node_id];
 					ob = create_instance_node(source_ob, source_node, node, sce, is_library_node);
+					objects_done->push_back(ob);
+					if (parent_node == NULL) {
+						root_objects->push_back(ob);
+					}
 				}
 			}
-			if (ob != NULL) objects_done->push_back(ob);
 			++inst_done;
 
 			read_transform = false;
 		}
+
 		// if node is empty - create empty object
 		// XXX empty node may not mean it is empty object, not sure about this
 		if ( (geom_done + camera_done + lamp_done + controller_done + inst_done) < 1) {
-			ob = bc_add_object(sce, OB_EMPTY, NULL);
+			//Check if Object is armature, by checking if immediate child is a JOINT node.
+			if (is_armature(node)) {
+				ob = bc_add_object(sce, OB_ARMATURE, name.c_str());
+			}
+			else {
+				ob = bc_add_object(sce, OB_EMPTY, NULL);
+			}
 			objects_done->push_back(ob);
+			if (parent_node == NULL) {
+				root_objects->push_back(ob);
+			}
 		}
 		
 		// XXX: if there're multiple instances, only one is stored
 
-		if (!ob) return;
+		if (!ob) return root_objects;
+
 		for (std::vector<Object *>::iterator it = objects_done->begin(); it != objects_done->end(); ++it) {
 			ob = *it;
 			std::string nodename = node->getName().size() ? node->getName() : node->getOriginalId();
 			rename_id(&ob->id, (char *)nodename.c_str());
-			object_map.insert(std::make_pair<COLLADAFW::UniqueId, Object *>(node->getUniqueId(), ob));
+			object_map.insert(std::pair<COLLADAFW::UniqueId, Object *>(node->getUniqueId(), ob));
 			node_map[node->getUniqueId()] = node;
 
 			if (is_library_node)
 				libnode_ob.push_back(ob);
 		}
 
+
+		//create_constraints(et,ob);
+
 	}
 
 	for (std::vector<Object *>::iterator it = objects_done->begin(); it != objects_done->end(); ++it) {
@@ -517,20 +666,35 @@ void DocumentImporter::write_node(COLLADAFW::Node *node, COLLADAFW::Node *parent
 			anim_importer.read_node_transform(node, ob);  // overwrites location set earlier
 
 		if (!is_joint) {
-			// if par was given make this object child of the previous
-			if (par && ob)
-				bc_set_parent(ob, par, mContext);
+			if (par && ob) {
+				ob->parent = par;
+				ob->partype = PAROBJECT;
+				ob->parsubstr[0] = 0;
+
+				//bc_set_parent(ob, par, mContext, false);
+			}
 		}
 	}
+
 	// if node has child nodes write them
 	COLLADAFW::NodePointerArray &child_nodes = node->getChildNodes();
-	for (unsigned int i = 0; i < child_nodes.getCount(); i++) {	
+
+	if (objects_done->size() > 0) {
+		ob = *objects_done->begin();
+	}
+	else {
+		ob = NULL;
+	}
+
+	for (unsigned int i = 0; i < child_nodes.getCount(); i++) {
 		write_node(child_nodes[i], node, sce, ob, is_library_node);
 	}
+
+	return root_objects;
 }
 
 /** When this method is called, the writer must write the entire visual scene.
- * \return The writer should return true, if writing succeeded, false otherwise.*/
+ *  Return The writer should return true, if writing succeeded, false otherwise. */
 bool DocumentImporter::writeVisualScene(const COLLADAFW::VisualScene *visualScene)
 {
 	if (mImportStage != General)
@@ -552,8 +716,8 @@ bool DocumentImporter::writeVisualScene(const COLLADAFW::VisualScene *visualScen
 }
 
 /** When this method is called, the writer must handle all nodes contained in the 
-* library nodes.
-* \return The writer should return true, if writing succeeded, false otherwise.*/
+ * library nodes.
+ * \return The writer should return true, if writing succeeded, false otherwise.*/
 bool DocumentImporter::writeLibraryNodes(const COLLADAFW::LibraryNodes *libraryNodes)
 {
 	if (mImportStage != General)
@@ -588,7 +752,7 @@ bool DocumentImporter::writeMaterial(const COLLADAFW::Material *cmat)
 		return true;
 		
 	const std::string& str_mat_id = cmat->getName().size() ? cmat->getName() : cmat->getOriginalId();
-	Material *ma = BKE_material_add((char *)str_mat_id.c_str());
+	Material *ma = BKE_material_add(G.main, (char *)str_mat_id.c_str());
 	
 	this->uid_effect_map[cmat->getInstantiatedEffect()] = ma;
 	this->uid_material_map[cmat->getUniqueId()] = ma;
@@ -612,9 +776,8 @@ MTex *DocumentImporter::create_texture(COLLADAFW::EffectCommon *ef, COLLADAFW::T
 	
 	ma->mtex[i] = add_mtex();
 	ma->mtex[i]->texco = TEXCO_UV;
-	ma->mtex[i]->tex = add_texture("Texture");
+	ma->mtex[i]->tex = add_texture(G.main, "Texture");
 	ma->mtex[i]->tex->type = TEX_IMAGE;
-	ma->mtex[i]->tex->imaflag &= ~TEX_USEALPHA;
 	ma->mtex[i]->tex->ima = uid_image_map[ima_uid];
 	
 	texindex_texarray_map[ctex.getTextureMapId()].push_back(ma->mtex[i]);
@@ -658,10 +821,14 @@ void DocumentImporter::write_profile_COMMON(COLLADAFW::EffectCommon *ef, Materia
 	// DIFFUSE
 	// color
 	if (ef->getDiffuse().isColor()) {
+		/* too high intensity can create artefacts (fireflies)
+		   So here we take care that intensity is set to 0.8 wherever possible
+		*/
 		col = ef->getDiffuse().getColor();
-		ma->r = col.getRed();
-		ma->g = col.getGreen();
-		ma->b = col.getBlue();
+		ma->ref = max_ffff(col.getRed(), col.getGreen(), col.getBlue(), 0.8);
+		ma->r = col.getRed()   / ma->ref;
+		ma->g = col.getGreen() / ma->ref;
+		ma->b = col.getBlue()  / ma->ref;
 	}
 	// texture
 	else if (ef->getDiffuse().isTexture()) {
@@ -745,7 +912,6 @@ void DocumentImporter::write_profile_COMMON(COLLADAFW::EffectCommon *ef, Materia
 		mtex = create_texture(ef, ctex, ma, i, texindex_texarray_map);
 		if (mtex != NULL) {
 			mtex->mapto = MAP_ALPHA;
-			mtex->tex->imaflag |= TEX_USEALPHA;
 			i++;
 			ma->spectra = ma->alpha = 0;
 			ma->mode |= MA_ZTRANSP | MA_TRANSP;
@@ -818,15 +984,22 @@ bool DocumentImporter::writeCamera(const COLLADAFW::Camera *camera)
 	Camera *cam = NULL;
 	std::string cam_id, cam_name;
 	
+	ExtraTags *et=getExtraTags(camera->getUniqueId());
 	cam_id = camera->getOriginalId();
 	cam_name = camera->getName();
-	if (cam_name.size()) cam = (Camera *)BKE_camera_add((char *)cam_name.c_str());
-	else cam = (Camera *)BKE_camera_add((char *)cam_id.c_str());
+	if (cam_name.size()) cam = (Camera *)BKE_camera_add(G.main, (char *)cam_name.c_str());
+	else cam = (Camera *)BKE_camera_add(G.main, (char *)cam_id.c_str());
 	
 	if (!cam) {
 		fprintf(stderr, "Cannot create camera.\n");
 		return true;
 	}
+
+	if (et && et->isProfile("blender")) {
+		et->setData("shiftx",&(cam->shiftx));
+		et->setData("shifty",&(cam->shifty));
+		et->setData("YF_dofdist",&(cam->YF_dofdist));
+	}
 	cam->clipsta = camera->getNearClippingPlane().getValue();
 	cam->clipend = camera->getFarClippingPlane().getValue();
 	
@@ -933,17 +1106,29 @@ bool DocumentImporter::writeImage(const COLLADAFW::Image *image)
 	if (mImportStage != General)
 		return true;
 		
-	// XXX maybe it is necessary to check if the path is absolute or relative
-	const std::string& filepath = image->getImageURI().toNativePath();
-	const char *filename = (const char *)mFilename.c_str();
+	const std::string& imagepath = image->getImageURI().toNativePath();
+
 	char dir[FILE_MAX];
-	char full_path[FILE_MAX];
-	
-	BLI_split_dir_part(filename, dir, sizeof(dir));
-	BLI_join_dirfile(full_path, sizeof(full_path), dir, filepath.c_str());
-	Image *ima = BKE_image_load_exists(full_path);
+	char absolute_path[FILE_MAX];
+	const char *workpath;
+
+	BLI_split_dir_part(this->import_settings->filepath, dir, sizeof(dir));
+	BLI_join_dirfile(absolute_path, sizeof(absolute_path), dir, imagepath.c_str());
+	if (BLI_exists(absolute_path)) {
+		workpath = absolute_path;
+	} 
+	else {
+		// Maybe imagepath was already absolute ?
+		if (!BLI_exists(imagepath.c_str())) {
+			fprintf(stderr, "Image not found: %s.\n", imagepath.c_str() );
+			return true;
+		}
+		workpath = imagepath.c_str();
+	}
+
+	Image *ima = BKE_image_load_exists(workpath);
 	if (!ima) {
-		fprintf(stderr, "Cannot create image.\n");
+		fprintf(stderr, "Cannot create image: %s\n", workpath);
 		return true;
 	}
 	this->uid_image_map[image->getUniqueId()] = ima;
@@ -961,16 +1146,17 @@ bool DocumentImporter::writeLight(const COLLADAFW::Light *light)
 	Lamp *lamp = NULL;
 	std::string la_id, la_name;
 
-	TagsMap::iterator etit;
+	ExtraTags *et = getExtraTags(light->getUniqueId());
+	/*TagsMap::iterator etit;
 	ExtraTags *et = 0;
 	etit = uid_tags_map.find(light->getUniqueId().toAscii());
 	if (etit != uid_tags_map.end())
-		et = etit->second;
+		et = etit->second;*/
 
 	la_id = light->getOriginalId();
 	la_name = light->getName();
-	if (la_name.size()) lamp = (Lamp *)BKE_lamp_add((char *)la_name.c_str());
-	else lamp = (Lamp *)BKE_lamp_add((char *)la_id.c_str());
+	if (la_name.size()) lamp = (Lamp *)BKE_lamp_add(G.main, (char *)la_name.c_str());
+	else lamp = (Lamp *)BKE_lamp_add(G.main, (char *)la_id.c_str());
 
 	if (!lamp) {
 		fprintf(stderr, "Cannot create lamp.\n");
@@ -992,6 +1178,7 @@ bool DocumentImporter::writeLight(const COLLADAFW::Light *light)
 		et->setData("energy", &(lamp->energy));
 		et->setData("dist", &(lamp->dist));
 		et->setData("spotsize", &(lamp->spotsize));
+		lamp->spotsize = DEG2RADF(lamp->spotsize);
 		et->setData("spotblend", &(lamp->spotblend));
 		et->setData("halo_intensity", &(lamp->haint));
 		et->setData("att1", &(lamp->att1));
@@ -999,7 +1186,6 @@ bool DocumentImporter::writeLight(const COLLADAFW::Light *light)
 		et->setData("falloff_type", &(lamp->falloff_type));
 		et->setData("clipsta", &(lamp->clipsta));
 		et->setData("clipend", &(lamp->clipend));
-		et->setData("shadspotsize", &(lamp->shadspotsize));
 		et->setData("bias", &(lamp->bias));
 		et->setData("soft", &(lamp->soft));
 		et->setData("compressthresh", &(lamp->compressthresh));
@@ -1089,7 +1275,7 @@ bool DocumentImporter::writeLight(const COLLADAFW::Light *light)
 					lamp->falloff_type = LA_FALLOFF_INVSQUARE;
 				if (IS_EQ(att2, 0.0f) && att1 > 0)
 					lamp->falloff_type = LA_FALLOFF_INVLINEAR;
-				lamp->spotsize = light->getFallOffAngle().getValue();
+				lamp->spotsize = DEG2RADF(light->getFallOffAngle().getValue());
 				lamp->spotblend = light->getFallOffExponent().getValue();
 			}
 			break;
@@ -1185,3 +1371,18 @@ bool DocumentImporter::addExtraTags(const COLLADAFW::UniqueId &uid, ExtraTags *e
 	return true;
 }
 
+bool DocumentImporter::is_armature(COLLADAFW::Node *node)
+{
+	COLLADAFW::NodePointerArray &child_nodes = node->getChildNodes();
+	for (unsigned int i = 0; i < child_nodes.getCount(); i++) {
+		if (child_nodes[i]->getType() == COLLADAFW::Node::JOINT) {
+			return true;
+		}
+		else {
+			continue;
+		}
+	}
+
+	//no child is JOINT
+	return false;
+}
