@@ -36,7 +36,6 @@
 #include "MEM_guardedalloc.h"
 
 #include "BLI_blenlib.h"
-#include "BLI_math.h"
 #include "BLI_utildefines.h"
 
 #include "DNA_anim_types.h"
@@ -50,7 +49,6 @@
 #include "BKE_global.h"
 
 #include "RNA_access.h"
-#include "RNA_enum_types.h"
 
 #include "ED_anim_api.h"
 #include "ED_keyframing.h"
@@ -73,7 +71,7 @@
  * Not recommended to be used many times successively. For that
  * there is delete_fcurve_keys(). 
  */
-void delete_fcurve_key(FCurve *fcu, int index, short do_recalc)
+void delete_fcurve_key(FCurve *fcu, int index, bool do_recalc)
 {
 	/* sanity check */
 	if (fcu == NULL) 
@@ -104,12 +102,13 @@ void delete_fcurve_key(FCurve *fcu, int index, short do_recalc)
 }
 
 /* Delete selected keyframes in given F-Curve */
-void delete_fcurve_keys(FCurve *fcu)
+bool delete_fcurve_keys(FCurve *fcu)
 {
 	int i;
+	bool changed = false;
 	
 	if (fcu->bezt == NULL) /* ignore baked curves */
-		return;
+		return false;
 
 	/* Delete selected BezTriples */
 	for (i = 0; i < fcu->totvert; i++) {
@@ -117,12 +116,15 @@ void delete_fcurve_keys(FCurve *fcu)
 			memmove(&fcu->bezt[i], &fcu->bezt[i + 1], sizeof(BezTriple) * (fcu->totvert - i - 1));
 			fcu->totvert--;
 			i--;
+			changed = true;
 		}
 	}
 	
 	/* Free the array of BezTriples if there are not keyframes */
 	if (fcu->totvert == 0)
 		clear_fcurve_keys(fcu);
+
+	return changed;
 }
 
 
@@ -188,7 +190,7 @@ void clean_fcurve(FCurve *fcu, float thresh)
 	
 	/* make a copy of the old BezTriples, and clear F-Curve */
 	old_bezts = fcu->bezt;
-	totCount = fcu->totvert;	
+	totCount = fcu->totvert;
 	fcu->bezt = NULL;
 	fcu->totvert = 0;
 	
@@ -256,7 +258,7 @@ void clean_fcurve(FCurve *fcu, float thresh)
 					insert_vert_fcurve(fcu, cur[0], cur[1], 0);
 				}
 			}
-			else {	
+			else {
 				/* add if value doesn't equal that of previous */
 				if (IS_EQT(cur[1], prev[1], thresh) == 0) {
 					/* add new keyframe */
@@ -355,7 +357,7 @@ void smooth_fcurve(FCurve *fcu)
 		/* round 2: apply new values */
 		tsb = tarray;
 		for (i = 0; i < totSel; i++, tsb++) {
-			/* don't touch end points, as their values were't touched above */
+			/* don't touch end points, as their values weren't touched above */
 			if (ELEM(i, 0, (totSel - 1)) == 0) {
 				/* y2 takes the average of the 2 points */
 				*tsb->h2 = tsb->y2;
@@ -405,8 +407,8 @@ void sample_fcurve(FCurve *fcu)
 				 * keyframes while sampling will affect the outcome...
 				 *	- only start sampling+adding from index=1, so that we don't overwrite original keyframe
 				 */
-				range = (int)(ceil(end->vec[1][0] - start->vec[1][0]) );
-				sfra = (int)(floor(start->vec[1][0]) );
+				range = (int)(ceil(end->vec[1][0] - start->vec[1][0]));
+				sfra = (int)(floor(start->vec[1][0]));
 				
 				if (range) {
 					value_cache = MEM_callocN(sizeof(TempFrameValCache) * range, "IcuFrameValCache");
@@ -501,7 +503,7 @@ void free_anim_copybuf(void)
 	}
 	
 	/* restore initial state */
-	animcopybuf.first = animcopybuf.last = NULL;
+	BLI_listbase_clear(&animcopybuf);
 	animcopy_firstframe = 999999999.0f;
 	animcopy_lastframe = -999999999.0f;
 }
@@ -541,7 +543,8 @@ short copy_animedit_keys(bAnimContext *ac, ListBase *anim_data)
 		BLI_addtail(&animcopybuf, aci);
 		
 		/* add selected keyframes to buffer */
-		// TODO: currently, we resize array everytime we add a new vert - this works ok as long as it is assumed only a few keys are copied
+		/* TODO: currently, we resize array every time we add a new vert -
+		 * this works ok as long as it is assumed only a few keys are copied */
 		for (i = 0, bezt = fcu->bezt; i < fcu->totvert; i++, bezt++) {
 			if (BEZSELECTED(bezt)) {
 				/* add to buffer */
@@ -627,9 +630,8 @@ static tAnimCopybufItem *pastebuf_match_path_property(FCurve *fcu, const short f
 				PropertyRNA *prop;
 				
 				RNA_id_pointer_create(aci->id, &id_ptr);
-				RNA_path_resolve(&id_ptr, aci->rna_path, &rptr, &prop);
 				
-				if (prop) {
+				if (RNA_path_resolve_property(&id_ptr, aci->rna_path, &rptr, &prop)) {
 					const char *identifier = RNA_property_identifier(prop);
 					int len_id = strlen(identifier);
 					int len_path = strlen(fcu->rna_path);
@@ -721,7 +723,7 @@ static void paste_animedit_keys_fcurve(FCurve *fcu, tAnimCopybufItem *aci, float
 		}
 	}
 	
-	/* just start pasting, with the the first keyframe on the current frame, and so on */
+	/* just start pasting, with the first keyframe on the current frame, and so on */
 	for (i = 0, bezt = aci->bezt; i < aci->totvert; i++, bezt++) {
 		/* temporarily apply offset to src beztriple while copying */
 		bezt->vec[0][0] += offset;
@@ -729,9 +731,9 @@ static void paste_animedit_keys_fcurve(FCurve *fcu, tAnimCopybufItem *aci, float
 		bezt->vec[2][0] += offset;
 		
 		/* insert the keyframe
-		 * NOTE: no special flags here for now
+		 * NOTE: we do not want to inherit handles from existing keyframes in this case!
 		 */
-		insert_bezt_fcurve(fcu, bezt, 0); 
+		insert_bezt_fcurve(fcu, bezt, INSERTKEY_OVERWRITE_FULL);
 		
 		/* un-apply offset from src beztriple after copying */
 		bezt->vec[0][0] -= offset;
@@ -760,8 +762,10 @@ EnumPropertyItem keyframe_paste_merge_items[] = {
 	{0, NULL, 0, NULL, NULL}};
 
 
-/* This function pastes data from the keyframes copy/paste buffer 
- * > return status code is whether the method FAILED to do anything
+/**
+ * This function pastes data from the keyframes copy/paste buffer
+ *
+ * \return Status code is whether the method FAILED to do anything
  */
 short paste_animedit_keys(bAnimContext *ac, ListBase *anim_data,
                           const eKeyPasteOffset offset_mode, const eKeyMergeMode merge_mode)
@@ -770,19 +774,19 @@ short paste_animedit_keys(bAnimContext *ac, ListBase *anim_data,
 	
 	const Scene *scene = (ac->scene);
 	
-	const short from_single = (animcopybuf.first == animcopybuf.last);
-	const short to_simple = (anim_data->first == anim_data->last);
+	const bool from_single = BLI_listbase_is_single(&animcopybuf);
+	const bool to_simple = BLI_listbase_is_single(anim_data);
 	
 	float offset = 0.0f;
 	int pass;
 
 	/* check if buffer is empty */
-	if (animcopybuf.first == NULL) {
+	if (BLI_listbase_is_empty(&animcopybuf)) {
 		BKE_report(ac->reports, RPT_ERROR, "No animation data in buffer to paste");
 		return -1;
 	}
 
-	if (anim_data->first == NULL) {
+	if (BLI_listbase_is_empty(anim_data)) {
 		BKE_report(ac->reports, RPT_ERROR, "No selected F-Curves to paste into");
 		return -1;
 	}
@@ -854,6 +858,8 @@ short paste_animedit_keys(bAnimContext *ac, ListBase *anim_data,
 					totmatch++;
 					paste_animedit_keys_fcurve(fcu, aci, offset, merge_mode);
 				}
+
+				ale->update |= ANIM_UPDATE_DEFAULT;
 			}
 			
 			/* don't continue if some fcurves were pasted */
@@ -862,6 +868,8 @@ short paste_animedit_keys(bAnimContext *ac, ListBase *anim_data,
 		}
 	}
 	
+	ANIM_animdata_update(ac, anim_data);
+
 	return 0;
 }
 
