@@ -39,8 +39,6 @@
 #include "MEM_guardedalloc.h"
 
 #include "BLI_blenlib.h"
-#include "BLI_math.h"
-#include "BLI_dynstr.h"
 #include "BLI_utildefines.h"
 
 #include "DNA_anim_types.h"
@@ -129,7 +127,7 @@ static int add_default_keyingset_exec(bContext *C, wmOperator *UNUSED(op))
 	/* call the API func, and set the active keyingset index */
 	BKE_keyingset_add(&scene->keyingsets, NULL, NULL, flag, keyingflag);
 	
-	scene->active_keyingset = BLI_countlist(&scene->keyingsets);
+	scene->active_keyingset = BLI_listbase_count(&scene->keyingsets);
 	
 	/* send notifiers */
 	WM_event_add_notifier(C, NC_SCENE | ND_KEYINGSET, NULL);
@@ -161,11 +159,11 @@ static int remove_active_keyingset_exec(bContext *C, wmOperator *op)
 	 *	- return error if it doesn't exist
 	 */
 	if (scene->active_keyingset == 0) {
-		BKE_report(op->reports, RPT_ERROR, "No active Keying Set to remove");
+		BKE_report(op->reports, RPT_ERROR, "No active keying set to remove");
 		return OPERATOR_CANCELLED;
 	}
 	else if (scene->active_keyingset < 0) {
-		BKE_report(op->reports, RPT_ERROR, "Cannot remove built in Keying Set");
+		BKE_report(op->reports, RPT_ERROR, "Cannot remove built in keying set");
 		return OPERATOR_CANCELLED;
 	}
 	else
@@ -209,7 +207,7 @@ static int add_empty_ks_path_exec(bContext *C, wmOperator *op)
 	 *	- return error if it doesn't exist
 	 */
 	if (scene->active_keyingset == 0) {
-		BKE_report(op->reports, RPT_ERROR, "No active Keying Set to add empty path to");
+		BKE_report(op->reports, RPT_ERROR, "No active keying set to add empty path to");
 		return OPERATOR_CANCELLED;
 	}
 	else
@@ -218,7 +216,7 @@ static int add_empty_ks_path_exec(bContext *C, wmOperator *op)
 	/* don't use the API method for this, since that checks on values... */
 	ksp = MEM_callocN(sizeof(KS_Path), "KeyingSetPath Empty");
 	BLI_addtail(&ks->paths, ksp);
-	ks->active_path = BLI_countlist(&ks->paths);
+	ks->active_path = BLI_listbase_count(&ks->paths);
 	
 	ksp->groupmode = KSP_GROUP_KSNAME; // XXX?
 	ksp->idtype = ID_OB;
@@ -258,12 +256,12 @@ static int remove_active_ks_path_exec(bContext *C, wmOperator *op)
 			ks->active_path--;
 		}
 		else {
-			BKE_report(op->reports, RPT_ERROR, "No active Keying Set Path to remove");
+			BKE_report(op->reports, RPT_ERROR, "No active keying set path to remove");
 			return OPERATOR_CANCELLED;
 		}
 	}
 	else {
-		BKE_report(op->reports, RPT_ERROR, "No active Keying Set to remove a path from");
+		BKE_report(op->reports, RPT_ERROR, "No active keying set to remove a path from");
 		return OPERATOR_CANCELLED;
 	}
 	
@@ -289,7 +287,6 @@ void ANIM_OT_keying_set_path_remove(wmOperatorType *ot)
 
 static int add_keyingset_button_exec(bContext *C, wmOperator *op)
 {
-	Main *bmain = CTX_data_main(C);
 	Scene *scene = CTX_data_scene(C);
 	KeyingSet *ks = NULL;
 	PropertyRNA *prop = NULL;
@@ -297,7 +294,7 @@ static int add_keyingset_button_exec(bContext *C, wmOperator *op)
 	char *path = NULL;
 	short success = 0;
 	int index = 0, pflag = 0;
-	int all = RNA_boolean_get(op->ptr, "all");
+	const bool all = RNA_boolean_get(op->ptr, "all");
 	
 	/* verify the Keying Set to use:
 	 *	- use the active one for now (more control over this can be added later)
@@ -319,17 +316,18 @@ static int add_keyingset_button_exec(bContext *C, wmOperator *op)
 		/* call the API func, and set the active keyingset index */
 		ks = BKE_keyingset_add(&scene->keyingsets, "ButtonKeyingSet", "Button Keying Set", flag, keyingflag);
 		
-		scene->active_keyingset = BLI_countlist(&scene->keyingsets);
+		scene->active_keyingset = BLI_listbase_count(&scene->keyingsets);
 	}
 	else if (scene->active_keyingset < 0) {
-		BKE_report(op->reports, RPT_ERROR, "Cannot add property to built in Keying Set");
+		BKE_report(op->reports, RPT_ERROR, "Cannot add property to built in keying set");
 		return OPERATOR_CANCELLED;
 	}
-	else
+	else {
 		ks = BLI_findlink(&scene->keyingsets, scene->active_keyingset - 1);
+	}
 	
 	/* try to add to keyingset using property retrieved from UI */
-	uiContextActiveProperty(C, &ptr, &prop, &index);
+	UI_context_active_but_prop_get(C, &ptr, &prop, &index);
 	
 	/* check if property is able to be added */
 	if (ptr.id.data && ptr.data && prop && RNA_property_animateable(&ptr, prop)) {
@@ -349,7 +347,7 @@ static int add_keyingset_button_exec(bContext *C, wmOperator *op)
 				
 			/* add path to this setting */
 			BKE_keyingset_add_path(ks, ptr.id.data, NULL, path, index, pflag, KSP_GROUP_KSNAME);
-			ks->active_path = BLI_countlist(&ks->paths);
+			ks->active_path = BLI_listbase_count(&ks->paths);
 			success = 1;
 			
 			/* free the temp path created */
@@ -359,10 +357,10 @@ static int add_keyingset_button_exec(bContext *C, wmOperator *op)
 	
 	if (success) {
 		/* send updates */
-		DAG_ids_flush_update(bmain, 0);
-		
-		/* for now, only send ND_KEYS for KeyingSets */
 		WM_event_add_notifier(C, NC_SCENE | ND_KEYINGSET, NULL);
+		
+		/* show notification/report header, so that users notice that something changed */
+		BKE_reportf(op->reports, RPT_INFO, "Property added to Keying Set: '%s'", ks->name);
 	}
 	
 	return (success) ? OPERATOR_FINISHED : OPERATOR_CANCELLED;
@@ -377,7 +375,7 @@ void ANIM_OT_keyingset_button_add(wmOperatorType *ot)
 	
 	/* callbacks */
 	ot->exec = add_keyingset_button_exec; 
-	//op->poll= ???
+	//op->poll = ???
 	
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -390,7 +388,6 @@ void ANIM_OT_keyingset_button_add(wmOperatorType *ot)
 
 static int remove_keyingset_button_exec(bContext *C, wmOperator *op)
 {
-	Main *bmain = CTX_data_main(C);
 	Scene *scene = CTX_data_scene(C);
 	KeyingSet *ks = NULL;
 	PropertyRNA *prop = NULL;
@@ -404,18 +401,19 @@ static int remove_keyingset_button_exec(bContext *C, wmOperator *op)
 	 *	- return error if it doesn't exist
 	 */
 	if (scene->active_keyingset == 0) {
-		BKE_report(op->reports, RPT_ERROR, "No active Keying Set to remove property from");
+		BKE_report(op->reports, RPT_ERROR, "No active keying set to remove property from");
 		return OPERATOR_CANCELLED;
 	}
 	else if (scene->active_keyingset < 0) {
-		BKE_report(op->reports, RPT_ERROR, "Cannot remove property from built in Keying Set");
+		BKE_report(op->reports, RPT_ERROR, "Cannot remove property from built in keying set");
 		return OPERATOR_CANCELLED;
 	}
-	else
+	else {
 		ks = BLI_findlink(&scene->keyingsets, scene->active_keyingset - 1);
+	}
 	
 	/* try to add to keyingset using property retrieved from UI */
-	uiContextActiveProperty(C, &ptr, &prop, &index);
+	UI_context_active_but_prop_get(C, &ptr, &prop, &index);
 
 	if (ptr.id.data && ptr.data && prop) {
 		path = RNA_path_from_ID_to_property(&ptr, prop);
@@ -425,7 +423,7 @@ static int remove_keyingset_button_exec(bContext *C, wmOperator *op)
 			
 			/* try to find a path matching this description */
 			ksp = BKE_keyingset_find_path(ks, ptr.id.data, ks->name, path, index, KSP_GROUP_KSNAME);
-
+			
 			if (ksp) {
 				BKE_keyingset_free_path(ks, ksp);
 				success = 1;
@@ -439,10 +437,10 @@ static int remove_keyingset_button_exec(bContext *C, wmOperator *op)
 	
 	if (success) {
 		/* send updates */
-		DAG_ids_flush_update(bmain, 0);
-		
-		/* for now, only send ND_KEYS for KeyingSets */
 		WM_event_add_notifier(C, NC_SCENE | ND_KEYINGSET, NULL);
+		
+		/* show warning */
+		BKE_report(op->reports, RPT_INFO, "Property removed from Keying Set");
 	}
 	
 	return (success) ? OPERATOR_FINISHED : OPERATOR_CANCELLED;
@@ -457,7 +455,7 @@ void ANIM_OT_keyingset_button_remove(wmOperatorType *ot)
 	
 	/* callbacks */
 	ot->exec = remove_keyingset_button_exec; 
-	//op->poll= ???
+	//op->poll = ???
 	
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -468,18 +466,18 @@ void ANIM_OT_keyingset_button_remove(wmOperatorType *ot)
 /* Change Active KeyingSet Operator ------------------------ */
 /* This operator checks if a menu should be shown for choosing the KeyingSet to make the active one */
 
-static int keyingset_active_menu_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int keyingset_active_menu_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	uiPopupMenu *pup;
 	uiLayout *layout;
 	
 	/* call the menu, which will call this operator again, hence the canceled */
-	pup = uiPupMenuBegin(C, op->type->name, ICON_NONE);
-	layout = uiPupMenuLayout(pup);
+	pup = UI_popup_menu_begin(C, op->type->name, ICON_NONE);
+	layout = UI_popup_menu_layout(pup);
 	uiItemsEnumO(layout, "ANIM_OT_keying_set_active_set", "type");
-	uiPupMenuEnd(C, pup);
+	UI_popup_menu_end(C, pup);
 	
-	return OPERATOR_CANCELLED;
+	return OPERATOR_INTERFACE;
 }
 
 static int keyingset_active_menu_exec(bContext *C, wmOperator *op)
@@ -641,6 +639,16 @@ void ANIM_keyingset_infos_exit(void)
 	BKE_keyingsets_free(&builtin_keyingsets);
 }
 
+/* Check if the ID appears in the paths specified by the KeyingSet */
+bool ANIM_keyingset_find_id(KeyingSet *ks, ID *id)
+{
+	/* sanity checks */
+	if (ELEM(NULL, ks, id))
+		return false;
+
+	return BLI_findptr(&ks->paths, id, offsetof(KS_Path, id)) != NULL;
+}
+
 /* ******************************************* */
 /* KEYING SETS API (for UI) */
 
@@ -712,7 +720,7 @@ KeyingSet *ANIM_get_keyingset_for_autokeying(Scene *scene, const char *tranformK
 /* Menu of All Keying Sets ----------------------------- */
 
 /* Dynamically populate an enum of Keying Sets */
-EnumPropertyItem *ANIM_keying_sets_enum_itemf(bContext *C, PointerRNA *UNUSED(ptr), PropertyRNA *UNUSED(prop), int *free)
+EnumPropertyItem *ANIM_keying_sets_enum_itemf(bContext *C, PointerRNA *UNUSED(ptr), PropertyRNA *UNUSED(prop), bool *r_free)
 {
 	Scene *scene = CTX_data_scene(C);
 	KeyingSet *ks;
@@ -772,7 +780,7 @@ EnumPropertyItem *ANIM_keying_sets_enum_itemf(bContext *C, PointerRNA *UNUSED(pt
 	}
 
 	RNA_enum_item_end(&item, &totitem);
-	*free = 1;
+	*r_free = true;
 
 	return item;
 }
@@ -783,7 +791,7 @@ EnumPropertyItem *ANIM_keying_sets_enum_itemf(bContext *C, PointerRNA *UNUSED(pt
 /* Polling API ----------------------------------------------- */
 
 /* Check if KeyingSet can be used in the current context */
-short ANIM_keyingset_context_ok_poll(bContext *C, KeyingSet *ks)
+bool ANIM_keyingset_context_ok_poll(bContext *C, KeyingSet *ks)
 {
 	if ((ks->flag & KEYINGSET_ABSOLUTE) == 0) {
 		KeyingSetInfo *ksi = ANIM_keyingset_info_find_name(ks->typeinfo);
@@ -797,7 +805,7 @@ short ANIM_keyingset_context_ok_poll(bContext *C, KeyingSet *ks)
 		return (ksi->poll(ksi, C));
 	}
 	
-	return 1;
+	return true;
 }
 
 /* Special 'Overrides' Iterator for Relative KeyingSets ------ */
@@ -879,7 +887,7 @@ short ANIM_validate_keyingset(bContext *C, ListBase *dsources, KeyingSet *ks)
 		if (ksi == NULL)
 			return MODIFYKEY_MISSING_TYPEINFO;
 		/* TODO: check for missing callbacks! */
-
+		
 		/* check if it can be used in the current context */
 		if (ksi->poll(ksi, C)) {
 			/* if a list of data sources are provided, run a special iterator over them,
@@ -891,11 +899,13 @@ short ANIM_validate_keyingset(bContext *C, ListBase *dsources, KeyingSet *ks)
 				ksi->iter(ksi, C, ks);
 				
 			/* if we don't have any paths now, then this still qualifies as invalid context */
-			if (ks->paths.first == NULL)
+			// FIXME: we need some error conditions (to be retrieved from the iterator why this failed!)
+			if (BLI_listbase_is_empty(&ks->paths))
 				return MODIFYKEY_INVALID_CONTEXT;
 		}
 		else {
 			/* poll callback tells us that KeyingSet is useless in current context */
+			// FIXME: the poll callback needs to give us more info why
 			return MODIFYKEY_INVALID_CONTEXT;
 		}
 	}
@@ -914,7 +924,7 @@ int ANIM_apply_keyingset(bContext *C, ListBase *dsources, bAction *act, KeyingSe
 	ReportList *reports = CTX_wm_reports(C);
 	KS_Path *ksp;
 	int kflag = 0, success = 0;
-	char *groupname = NULL;
+	const char *groupname = NULL;
 	
 	/* sanity checks */
 	if (ks == NULL)
@@ -925,7 +935,7 @@ int ANIM_apply_keyingset(bContext *C, ListBase *dsources, bAction *act, KeyingSe
 		/* use KeyingSet's flags as base */
 		kflag = ks->keyingflag;
 		
-		/* suppliment with info from the context */
+		/* supplement with info from the context */
 		kflag |= ANIM_get_keyframing_flags(scene, 1);
 	}
 	else if (mode == MODIFYKEY_MODE_DELETE)
@@ -947,7 +957,7 @@ int ANIM_apply_keyingset(bContext *C, ListBase *dsources, bAction *act, KeyingSe
 		/* skip path if no ID pointer is specified */
 		if (ksp->id == NULL) {
 			BKE_reportf(reports, RPT_WARNING,
-			            "Skipping path in Keying Set, as it has no ID (KS = '%s', Path = '%s'[%d])",
+			            "Skipping path in keying set, as it has no ID (KS = '%s', path = '%s[%d]')",
 			            ks->name, ksp->rna_path, ksp->array_index);
 			continue;
 		}
@@ -975,7 +985,7 @@ int ANIM_apply_keyingset(bContext *C, ListBase *dsources, bAction *act, KeyingSe
 			PropertyRNA *prop;
 			
 			RNA_id_pointer_create(ksp->id, &id_ptr);
-			if (RNA_path_resolve(&id_ptr, ksp->rna_path, &ptr, &prop) && prop)
+			if (RNA_path_resolve_property(&id_ptr, ksp->rna_path, &ptr, &prop))
 				arraylen = RNA_property_array_length(&ptr, prop);
 		}
 		
@@ -1000,13 +1010,14 @@ int ANIM_apply_keyingset(bContext *C, ListBase *dsources, bAction *act, KeyingSe
 			{
 				Object *ob = (Object *)ksp->id;
 				
-				ob->recalc |= OB_RECALC_OB | OB_RECALC_DATA | OB_RECALC_TIME; // XXX: only object transforms only?
+				// XXX: only object transforms?
+				DAG_id_tag_update(&ob->id, OB_RECALC_OB | OB_RECALC_DATA | OB_RECALC_TIME);
+				break;
 			}
-			break;
 		}
 		
 		/* send notifiers for updates (this doesn't require context to work!) */
-		WM_main_add_notifier(NC_ANIMATION | ND_KEYFRAME | NA_EDITED, NULL);
+		WM_main_add_notifier(NC_ANIMATION | ND_KEYFRAME | NA_ADDED, NULL);
 	}
 	
 	/* return the number of channels successfully affected */
