@@ -22,30 +22,21 @@
  *  \ingroup edarmature
  */
 
-#include <string.h>
-#include <math.h>
-#include <float.h>
-
 #include "MEM_guardedalloc.h"
 
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_armature_types.h"
 
-#include "RNA_define.h"
-#include "RNA_access.h"
-
 #include "BLI_blenlib.h"
 #include "BLI_math.h"
-#include "BLI_utildefines.h"
-#include "BLI_graph.h"
-#include "BLI_ghash.h"
-
-#include "BLF_translation.h"
 
 #include "BKE_context.h"
 #include "BKE_sketch.h"
 
+#include "RNA_define.h"
+#include "RNA_access.h"
+
 #include "ED_view3d.h"
 #include "ED_screen.h"
 
@@ -60,7 +51,7 @@
 #include "WM_api.h"
 #include "WM_types.h"
 
-#include "reeb.h"
+#include "GPU_select.h"
 
 typedef int (*GestureDetectFct)(bContext *, SK_Gesture *, SK_Sketch *);
 typedef void (*GestureApplyFct)(bContext *, SK_Gesture *, SK_Sketch *);
@@ -174,25 +165,26 @@ void BIF_makeListTemplates(const bContext *C)
 	}
 }
 
+#if 0  /* UNUSED */
 const char *BIF_listTemplates(const bContext *UNUSED(C))
 {
 	GHashIterator ghi;
 	const char *menu_header = IFACE_("Template %t|None %x0|");
 	char *p;
+	const size_t template_size = (BLI_ghash_size(TEMPLATES_HASH) * 32 + 30);
 
 	if (TEMPLATES_MENU != NULL) {
 		MEM_freeN(TEMPLATES_MENU);
 	}
 
-	TEMPLATES_MENU = MEM_callocN(sizeof(char) * (BLI_ghash_size(TEMPLATES_HASH) * 32 + 30), "skeleton template menu");
+	TEMPLATES_MENU = MEM_callocN(sizeof(char) * template_size, "skeleton template menu");
 
 	p = TEMPLATES_MENU;
-
-	p += sprintf(TEMPLATES_MENU, "%s", menu_header);
+	p += BLI_strncpy_rlen(p, menu_header, template_size);
 
 	BLI_ghashIterator_init(&ghi, TEMPLATES_HASH);
 
-	while (!BLI_ghashIterator_isDone(&ghi)) {
+	while (!BLI_ghashIterator_done(&ghi)) {
 		Object *ob = BLI_ghashIterator_getValue(&ghi);
 		int key = GET_INT_FROM_POINTER(BLI_ghashIterator_getKey(&ghi));
 
@@ -203,6 +195,7 @@ const char *BIF_listTemplates(const bContext *UNUSED(C))
 
 	return TEMPLATES_MENU;
 }
+#endif
 
 int   BIF_currentTemplate(const bContext *C)
 {
@@ -212,7 +205,7 @@ int   BIF_currentTemplate(const bContext *C)
 		GHashIterator ghi;
 		BLI_ghashIterator_init(&ghi, TEMPLATES_HASH);
 
-		while (!BLI_ghashIterator_isDone(&ghi)) {
+		while (!BLI_ghashIterator_done(&ghi)) {
 			Object *ob = BLI_ghashIterator_getValue(&ghi);
 			int key = GET_INT_FROM_POINTER(BLI_ghashIterator_getKey(&ghi));
 
@@ -361,7 +354,7 @@ static void sk_autoname(bContext *C, ReebArc *arc)
 	}
 }
 
-static ReebNode *sk_pointToNode(SK_Point *pt, float imat[][4], float tmat[][3])
+static ReebNode *sk_pointToNode(SK_Point *pt, float imat[4][4], float tmat[3][3])
 {
 	ReebNode *node;
 
@@ -375,7 +368,7 @@ static ReebNode *sk_pointToNode(SK_Point *pt, float imat[][4], float tmat[][3])
 	return node;
 }
 
-static ReebArc *sk_strokeToArc(SK_Stroke *stk, float imat[][4], float tmat[][3])
+static ReebArc *sk_strokeToArc(SK_Stroke *stk, float imat[4][4], float tmat[3][3])
 {
 	ReebArc *arc;
 	int i;
@@ -408,9 +401,7 @@ static void sk_retargetStroke(bContext *C, SK_Stroke *stk)
 	RigGraph *rg;
 
 	invert_m4_m4(imat, obedit->obmat);
-
-	copy_m3_m4(tmat, obedit->obmat);
-	transpose_m3(tmat);
+	transpose_m3_m4(tmat, obedit->obmat);
 
 	arc = sk_strokeToArc(stk, imat, tmat);
 
@@ -440,7 +431,7 @@ static void sk_cancelStroke(SK_Sketch *sketch)
 
 static float sk_clampPointSize(SK_Point *pt, float size)
 {
-	return MAX2(size * pt->size, size / 2);
+	return max_ff(size * pt->size, size / 2);
 }
 
 static void sk_drawPoint(GLUquadric *quad, SK_Point *pt, float size)
@@ -500,7 +491,7 @@ static void sk_drawStroke(SK_Stroke *stk, int id, float color[3], int start, int
 	gluQuadricNormals(quad, GLU_SMOOTH);
 
 	if (id != -1) {
-		glLoadName(id);
+		GPU_select_load_id(id);
 
 		for (i = 0; i < stk->nb_points; i++) {
 			glPushMatrix();
@@ -635,7 +626,7 @@ static void sk_drawStrokeSubdivision(ToolSettings *toolsettings, SK_Stroke *stk)
 	}
 }
 
-static SK_Point *sk_snapPointStroke(bContext *C, SK_Stroke *stk, int mval[2], int *dist, int *index, int all_pts)
+static SK_Point *sk_snapPointStroke(bContext *C, SK_Stroke *stk, int mval[2], float *r_dist_px, int *index, int all_pts)
 {
 	ARegion *ar = CTX_wm_region(C);
 	SK_Point *pt = NULL;
@@ -650,8 +641,8 @@ static SK_Point *sk_snapPointStroke(bContext *C, SK_Stroke *stk, int mval[2], in
 
 				pdist = ABS(pval[0] - mval[0]) + ABS(pval[1] - mval[1]);
 
-				if (pdist < *dist) {
-					*dist = pdist;
+				if (pdist < *r_dist_px) {
+					*r_dist_px = pdist;
 					pt = stk->points + i;
 
 					if (index != NULL) {
@@ -738,13 +729,13 @@ static void sk_updateOverdraw(bContext *C, SK_Sketch *sketch, SK_Stroke *stk, SK
 	if (sketch->over.target == NULL) {
 		SK_Stroke *target;
 		int closest_index = -1;
-		int dist = SNAP_MIN_DISTANCE * 2;
+		float dist_px = SNAP_MIN_DISTANCE * 2;
 
 		for (target = sketch->strokes.first; target; target = target->next) {
 			if (target != stk) {
 				int index;
 
-				SK_Point *spt = sk_snapPointStroke(C, target, dd->mval, &dist, &index, 1);
+				SK_Point *spt = sk_snapPointStroke(C, target, dd->mval, &dist_px, &index, 1);
 
 				if (spt != NULL) {
 					sketch->over.target = target;
@@ -773,10 +764,10 @@ static void sk_updateOverdraw(bContext *C, SK_Sketch *sketch, SK_Stroke *stk, SK
 	}
 	else if (sketch->over.target != NULL) {
 		SK_Point *closest_pt = NULL;
-		int dist = SNAP_MIN_DISTANCE * 2;
+		float dist_px = SNAP_MIN_DISTANCE * 2;
 		int index;
 
-		closest_pt = sk_snapPointStroke(C, sketch->over.target, dd->mval, &dist, &index, 1);
+		closest_pt = sk_snapPointStroke(C, sketch->over.target, dd->mval, &dist_px, &index, 1);
 
 		if (closest_pt != NULL) {
 			if (sk_lastStrokePoint(stk)->type == PT_EXACT) {
@@ -913,7 +904,7 @@ static void sk_interpolateDepth(bContext *C, SK_Stroke *stk, int start, int end,
 		float pval[2] = {0, 0};
 
 		ED_view3d_project_float_global(ar, stk->points[i].p, pval, V3D_PROJ_TEST_NOP);
-		ED_view3d_win_to_ray(ar, v3d, pval, ray_start, ray_normal);
+		ED_view3d_win_to_ray(ar, v3d, pval, ray_start, ray_normal, false);
 
 		mul_v3_fl(ray_normal, distance * progress / length);
 		add_v3_v3(stk->points[i].p, ray_normal);
@@ -931,17 +922,17 @@ static void sk_projectDrawPoint(bContext *C, float vec[3], SK_Stroke *stk, SK_Dr
 	float fp[3] = {0, 0, 0};
 	float dvec[3];
 	float mval_f[2];
+	float zfac;
 
 	if (last != NULL) {
 		copy_v3_v3(fp, last->p);
 	}
 
-	initgrabz(ar->regiondata, fp[0], fp[1], fp[2]);
+	zfac = ED_view3d_calc_zfac(ar->regiondata, fp, NULL);
 
-	/* method taken from editview.c - mouse_cursor() */
 	if (ED_view3d_project_short_global(ar, fp, cval, V3D_PROJ_TEST_NOP) == V3D_PROJ_RET_OK) {
 		VECSUB2D(mval_f, cval, dd->mval);
-		ED_view3d_win_to_delta(ar, mval_f, dvec);
+		ED_view3d_win_to_delta(ar, mval_f, dvec, zfac);
 		sub_v3_v3v3(vec, fp, dvec);
 	}
 	else {
@@ -987,7 +978,7 @@ static int sk_getStrokeSnapPoint(bContext *C, SK_Point *pt, SK_Sketch *sketch, S
 		float mvalf[2];
 
 		BLI_freelistN(&sketch->depth_peels);
-		sketch->depth_peels.first = sketch->depth_peels.last = NULL;
+		BLI_listbase_clear(&sketch->depth_peels);
 
 		mvalf[0] = dd->mval[0];
 		mvalf[1] = dd->mval[1];
@@ -1072,17 +1063,17 @@ static int sk_getStrokeSnapPoint(bContext *C, SK_Point *pt, SK_Sketch *sketch, S
 		float no[3];
 		float mval[2];
 		int found = 0;
-		int dist = SNAP_MIN_DISTANCE; // Use a user defined value here
+		float dist_px = SNAP_MIN_DISTANCE; // Use a user defined value here
 
 		/* snap to strokes */
 		// if (ts->snap_mode == SCE_SNAP_MODE_VERTEX) /* snap all the time to strokes */
 		for (snap_stk = sketch->strokes.first; snap_stk; snap_stk = snap_stk->next) {
 			SK_Point *spt = NULL;
 			if (snap_stk == stk) {
-				spt = sk_snapPointStroke(C, snap_stk, dd->mval, &dist, NULL, 0);
+				spt = sk_snapPointStroke(C, snap_stk, dd->mval, &dist_px, NULL, 0);
 			}
 			else {
-				spt = sk_snapPointStroke(C, snap_stk, dd->mval, &dist, NULL, 1);
+				spt = sk_snapPointStroke(C, snap_stk, dd->mval, &dist_px, NULL, 1);
 			}
 
 			if (spt != NULL) {
@@ -1095,7 +1086,7 @@ static int sk_getStrokeSnapPoint(bContext *C, SK_Point *pt, SK_Sketch *sketch, S
 		mval[1] = dd->mval[1];
 
 		/* try to snap to closer object */
-		found = snapObjectsContext(C, mval, &dist, vec, no, SNAP_NOT_SELECTED);
+		found = snapObjectsContext(C, mval, &dist_px, vec, no, SNAP_NOT_SELECTED);
 		if (found == 1) {
 			pt->type = dd->type;
 			pt->mode = PT_SNAP;
@@ -1155,7 +1146,7 @@ static int sk_addStrokeSnapPoint(bContext *C, SK_Sketch *sketch, SK_Stroke *stk,
 	return point_added;
 }
 
-static void sk_addStrokePoint(bContext *C, SK_Sketch *sketch, SK_Stroke *stk, SK_DrawData *dd, short snap)
+static void sk_addStrokePoint(bContext *C, SK_Sketch *sketch, SK_Stroke *stk, SK_DrawData *dd, const bool snap)
 {
 	ToolSettings *ts = CTX_data_tool_settings(C);
 	int point_added = 0;
@@ -1173,7 +1164,7 @@ static void sk_addStrokePoint(bContext *C, SK_Sketch *sketch, SK_Stroke *stk, SK
 	}
 }
 
-static void sk_getStrokePoint(bContext *C, SK_Point *pt, SK_Sketch *sketch, SK_Stroke *stk, SK_DrawData *dd, short snap)
+static void sk_getStrokePoint(bContext *C, SK_Point *pt, SK_Sketch *sketch, SK_Stroke *stk, SK_DrawData *dd, const bool snap)
 {
 	int point_added = 0;
 
@@ -1363,9 +1354,7 @@ static void sk_convertStroke(bContext *C, SK_Stroke *stk)
 	head = NULL;
 
 	invert_m4_m4(invmat, obedit->obmat);
-
-	copy_m3_m4(tmat, obedit->obmat);
-	transpose_m3(tmat);
+	transpose_m3_m4(tmat, obedit->obmat);
 
 	for (i = 0; i < stk->nb_points; i++) {
 		SK_Point *pt = stk->points + i;
@@ -1497,9 +1486,9 @@ static int sk_getSelfIntersections(bContext *C, ListBase *list, SK_Stroke *gestu
 	return added;
 }
 
-static int cmpIntersections(void *i1, void *i2)
+static int cmpIntersections(const void *i1, const void *i2)
 {
-	SK_Intersection *isect1 = i1, *isect2 = i2;
+	const SK_Intersection *isect1 = i1, *isect2 = i2;
 
 	if (isect1->stroke == isect2->stroke) {
 		if (isect1->before < isect2->before) {
@@ -1565,7 +1554,7 @@ static int sk_getIntersections(bContext *C, ListBase *list, SK_Sketch *sketch, S
 
 					mval[0] = vi[0];
 					mval[1] = vi[1];
-					ED_view3d_win_to_segment_clip(ar, v3d, mval, ray_start, ray_end);
+					ED_view3d_win_to_segment(ar, v3d, mval, ray_start, ray_end, true);
 
 					isect_line_line_v3(stk->points[s_i].p,
 					                   stk->points[s_i + 1].p,
@@ -1584,7 +1573,7 @@ static int sk_getIntersections(bContext *C, ListBase *list, SK_Sketch *sketch, S
 		added = MAX2(s_added, added);
 	}
 
-	BLI_sortlist(list, cmpIntersections);
+	BLI_listbase_sort(list, cmpIntersections);
 
 	return added;
 }
@@ -1703,7 +1692,7 @@ int sk_detectCommandGesture(bContext *UNUSED(C), SK_Gesture *gest, SK_Sketch *UN
 	if (gest->nb_segments > 2 && gest->nb_intersections == 2 && gest->nb_self_intersections == 1) {
 		SK_Intersection *isect, *self_isect;
 
-		/* get the the last intersection of the first pair */
+		/* get the last intersection of the first pair */
 		for (isect = gest->intersections.first; isect; isect = isect->next) {
 			if (isect->stroke == isect->next->stroke) {
 				isect = isect->next;
@@ -1923,8 +1912,8 @@ void sk_applyConvertGesture(bContext *C, SK_Gesture *UNUSED(gest), SK_Sketch *sk
 
 static void sk_initGesture(bContext *C, SK_Gesture *gest, SK_Sketch *sketch)
 {
-	gest->intersections.first = gest->intersections.last = NULL;
-	gest->self_intersections.first = gest->self_intersections.last = NULL;
+	BLI_listbase_clear(&gest->intersections);
+	BLI_listbase_clear(&gest->self_intersections);
 
 	gest->segments = sk_createStroke();
 	gest->stk = sketch->gesture;
@@ -1976,7 +1965,7 @@ static int sk_selectStroke(bContext *C, SK_Sketch *sketch, const int mval[2], in
 	rect.ymin = mval[1] - 5;
 	rect.ymax = mval[1] + 5;
 
-	hits = view3d_opengl_select(&vc, buffer, MAXPICKBUF, &rect);
+	hits = view3d_opengl_select(&vc, buffer, MAXPICKBUF, &rect, true);
 
 	if (hits > 0) {
 		int besthitresult = -1;
@@ -2039,7 +2028,7 @@ static void sk_drawSketch(Scene *scene, View3D *UNUSED(v3d), SK_Sketch *sketch,
 			sk_drawStroke(stk, id, NULL, -1, -1);
 		}
 
-		glLoadName(-1);
+		GPU_select_load_id(-1);
 	}
 	else {
 		float selected_rgb[3] = {1, 0, 0};
@@ -2101,18 +2090,17 @@ static void sk_drawSketch(Scene *scene, View3D *UNUSED(v3d), SK_Sketch *sketch,
 	}
 
 #if 0
-	if (sketch->depth_peels.first != NULL)
-	{
+	if (BLI_listbase_is_empty(&sketch->depth_peels) == false) {
 		float colors[8][3] = {
-								{1, 0, 0},
-								{0, 1, 0},
-								{0, 0, 1},
-								{1, 1, 0},
-								{1, 0, 1},
-								{0, 1, 1},
-								{1, 1, 1},
-								{0, 0, 0}
-							};
+			{1, 0, 0},
+			{0, 1, 0},
+			{0, 0, 1},
+			{1, 1, 0},
+			{1, 0, 1},
+			{0, 1, 1},
+			{1, 1, 1},
+			{0, 0, 0}
+		};
 		DepthPeel *p;
 		GLUquadric *quad = gluNewQuadric();
 		gluQuadricNormals(quad, GLU_SMOOTH);
@@ -2188,7 +2176,7 @@ static void sk_start_draw_gesture(SK_Sketch *sketch)
 	sketch->gesture = sk_createStroke();
 }
 
-static int sk_draw_stroke(bContext *C, SK_Sketch *sketch, SK_Stroke *stk, SK_DrawData *dd, short snap)
+static int sk_draw_stroke(bContext *C, SK_Sketch *sketch, SK_Stroke *stk, SK_DrawData *dd, bool snap)
 {
 	if (sk_stroke_filtermval(dd)) {
 		sk_addStrokePoint(C, sketch, stk, dd, snap);
@@ -2240,7 +2228,7 @@ void BDR_drawSketch(const bContext *C)
 	}
 }
 
-static int sketch_delete(bContext *C, wmOperator *UNUSED(op), wmEvent *UNUSED(event))
+static int sketch_delete(bContext *C, wmOperator *UNUSED(op), const wmEvent *UNUSED(event))
 {
 	SK_Sketch *sketch = contextSketch(C, 0);
 	if (sketch) {
@@ -2337,7 +2325,7 @@ SK_Sketch *viewcontextSketch(ViewContext *vc, int create)
 	return sketch;
 }
 
-static int sketch_convert(bContext *C, wmOperator *UNUSED(op), wmEvent *UNUSED(event))
+static int sketch_convert(bContext *C, wmOperator *UNUSED(op), const wmEvent *UNUSED(event))
 {
 	SK_Sketch *sketch = contextSketch(C, 0);
 	if (sketch != NULL) {
@@ -2347,7 +2335,7 @@ static int sketch_convert(bContext *C, wmOperator *UNUSED(op), wmEvent *UNUSED(e
 	return OPERATOR_FINISHED;
 }
 
-static int sketch_cancel(bContext *C, wmOperator *UNUSED(op), wmEvent *UNUSED(event))
+static int sketch_cancel_invoke(bContext *C, wmOperator *UNUSED(op), const wmEvent *UNUSED(event))
 {
 	SK_Sketch *sketch = contextSketch(C, 0);
 	if (sketch != NULL) {
@@ -2358,7 +2346,7 @@ static int sketch_cancel(bContext *C, wmOperator *UNUSED(op), wmEvent *UNUSED(ev
 	return OPERATOR_PASS_THROUGH;
 }
 
-static int sketch_finish(bContext *C, wmOperator *UNUSED(op), wmEvent *UNUSED(event))
+static int sketch_finish(bContext *C, wmOperator *UNUSED(op), const wmEvent *UNUSED(event))
 {
 	SK_Sketch *sketch = contextSketch(C, 0);
 	if (sketch != NULL) {
@@ -2370,7 +2358,7 @@ static int sketch_finish(bContext *C, wmOperator *UNUSED(op), wmEvent *UNUSED(ev
 	return OPERATOR_PASS_THROUGH;
 }
 
-static int sketch_select(bContext *C, wmOperator *UNUSED(op), wmEvent *event)
+static int sketch_select(bContext *C, wmOperator *UNUSED(op), const wmEvent *event)
 {
 	SK_Sketch *sketch = contextSketch(C, 0);
 	if (sketch) {
@@ -2382,21 +2370,20 @@ static int sketch_select(bContext *C, wmOperator *UNUSED(op), wmEvent *event)
 	return OPERATOR_FINISHED;
 }
 
-static int sketch_draw_stroke_cancel(bContext *C, wmOperator *op)
+static void sketch_draw_stroke_cancel(bContext *C, wmOperator *op)
 {
 	SK_Sketch *sketch = contextSketch(C, 1); /* create just to be sure */
 	sk_cancelStroke(sketch);
 	MEM_freeN(op->customdata);
-	return OPERATOR_CANCELLED;
 }
 
-static int sketch_draw_stroke(bContext *C, wmOperator *op, wmEvent *event)
+static int sketch_draw_stroke(bContext *C, wmOperator *op, const wmEvent *event)
 {
-	short snap = RNA_boolean_get(op->ptr, "snap");
+	const bool snap = RNA_boolean_get(op->ptr, "snap");
 	SK_DrawData *dd;
 	SK_Sketch *sketch = contextSketch(C, 1);
 
-	op->customdata = dd = MEM_callocN(sizeof("SK_DrawData"), "SketchDrawData");
+	op->customdata = dd = MEM_callocN(sizeof(SK_DrawData), "SketchDrawData");
 	sk_initDrawData(dd, event->mval);
 
 	sk_start_draw_stroke(sketch);
@@ -2408,22 +2395,21 @@ static int sketch_draw_stroke(bContext *C, wmOperator *op, wmEvent *event)
 	return OPERATOR_RUNNING_MODAL;
 }
 
-static int sketch_draw_gesture_cancel(bContext *C, wmOperator *op)
+static void sketch_draw_gesture_cancel(bContext *C, wmOperator *op)
 {
 	SK_Sketch *sketch = contextSketch(C, 1); /* create just to be sure */
 	sk_cancelStroke(sketch);
 	MEM_freeN(op->customdata);
-	return OPERATOR_CANCELLED;
 }
 
-static int sketch_draw_gesture(bContext *C, wmOperator *op, wmEvent *event)
+static int sketch_draw_gesture(bContext *C, wmOperator *op, const wmEvent *event)
 {
-	short snap = RNA_boolean_get(op->ptr, "snap");
+	const bool snap = RNA_boolean_get(op->ptr, "snap");
 	SK_DrawData *dd;
 	SK_Sketch *sketch = contextSketch(C, 1); /* create just to be sure */
 	sk_cancelStroke(sketch);
 
-	op->customdata = dd = MEM_callocN(sizeof("SK_DrawData"), "SketchDrawData");
+	op->customdata = dd = MEM_callocN(sizeof(SK_DrawData), "SketchDrawData");
 	sk_initDrawData(dd, event->mval);
 
 	sk_start_draw_gesture(sketch);
@@ -2434,9 +2420,9 @@ static int sketch_draw_gesture(bContext *C, wmOperator *op, wmEvent *event)
 	return OPERATOR_RUNNING_MODAL;
 }
 
-static int sketch_draw_modal(bContext *C, wmOperator *op, wmEvent *event, short gesture, SK_Stroke *stk)
+static int sketch_draw_modal(bContext *C, wmOperator *op, const wmEvent *event, short gesture, SK_Stroke *stk)
 {
-	short snap = RNA_boolean_get(op->ptr, "snap");
+	bool snap = RNA_boolean_get(op->ptr, "snap");
 	SK_DrawData *dd = op->customdata;
 	SK_Sketch *sketch = contextSketch(C, 1); /* create just to be sure */
 	int retval = OPERATOR_RUNNING_MODAL;
@@ -2444,7 +2430,7 @@ static int sketch_draw_modal(bContext *C, wmOperator *op, wmEvent *event, short
 	switch (event->type) {
 		case LEFTCTRLKEY:
 		case RIGHTCTRLKEY:
-			snap = event->ctrl;
+			snap = event->ctrl != 0;
 			RNA_boolean_set(op->ptr, "snap", snap);
 			break;
 		case MOUSEMOVE:
@@ -2492,21 +2478,21 @@ static int sketch_draw_modal(bContext *C, wmOperator *op, wmEvent *event, short
 	return retval;
 }
 
-static int sketch_draw_stroke_modal(bContext *C, wmOperator *op, wmEvent *event)
+static int sketch_draw_stroke_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	SK_Sketch *sketch = contextSketch(C, 1); /* create just to be sure */
 	return sketch_draw_modal(C, op, event, 0, sketch->active_stroke);
 }
 
-static int sketch_draw_gesture_modal(bContext *C, wmOperator *op, wmEvent *event)
+static int sketch_draw_gesture_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	SK_Sketch *sketch = contextSketch(C, 1); /* create just to be sure */
 	return sketch_draw_modal(C, op, event, 1, sketch->gesture);
 }
 
-static int sketch_draw_preview(bContext *C, wmOperator *op, wmEvent *event)
+static int sketch_draw_preview(bContext *C, wmOperator *op, const wmEvent *event)
 {
-	short snap = RNA_boolean_get(op->ptr, "snap");
+	const bool snap = RNA_boolean_get(op->ptr, "snap");
 	SK_Sketch *sketch = contextSketch(C, 0);
 
 	if (sketch) {
@@ -2630,7 +2616,7 @@ void SKETCH_OT_cancel_stroke(wmOperatorType *ot)
 	ot->description = "Cancel the current sketch stroke";
 
 	/* api callbacks */
-	ot->invoke = sketch_cancel;
+	ot->invoke = sketch_cancel_invoke;
 
 	ot->poll = ED_operator_sketch_mode_active_stroke;
 
