@@ -48,6 +48,9 @@ typedef enum {
 #undef DEF_ICON
 #undef DEF_VICO
 
+/* use to denote intentionally unset theme color */
+#define TH_UNDEFINED -1
+
 enum {
 	TH_REDALERT,
 
@@ -58,16 +61,21 @@ enum {
 	TH_TEXT,
 	TH_TEXT_HI,
 	TH_TITLE,
+	TH_TAB_ACTIVE,
+	TH_TAB_INACTIVE,
+	TH_TAB_BACK,
+	TH_TAB_OUTLINE,
 	
 	TH_HEADER,
 	TH_HEADERDESEL,
 	TH_HEADER_TEXT,
 	TH_HEADER_TEXT_HI,
 	
-	/* float panels */
-	TH_PANEL,
-	TH_PANEL_TEXT,
-	TH_PANEL_TEXT_HI,
+	/* panels */
+	TH_PANEL_HEADER,
+	TH_PANEL_BACK,
+	TH_PANEL_SHOW_HEADER,
+	TH_PANEL_SHOW_BACK,
 	
 	TH_BUTBACK,
 	TH_BUTBACK_TEXT,
@@ -79,6 +87,8 @@ enum {
 
 	TH_GRID,
 	TH_WIRE,
+	TH_WIRE_INNER,
+	TH_WIRE_EDIT,
 	TH_SELECT,
 	TH_ACTIVE,
 	TH_GROUP,
@@ -86,6 +96,7 @@ enum {
 	TH_TRANSFORM,
 	TH_VERTEX,
 	TH_VERTEX_SELECT,
+	TH_VERTEX_UNREFERENCED,
 	TH_VERTEX_SIZE,
 	TH_OUTLINE_WIDTH,
 	TH_EDGE,
@@ -96,6 +107,7 @@ enum {
 	TH_FACE_SELECT,
 	TH_NORMAL,
 	TH_VNORMAL,
+	TH_LNORMAL,
 	TH_FACE_DOT,
 	TH_FACEDOT_SIZE,
 	TH_CFRAME,
@@ -117,13 +129,16 @@ enum {
 	TH_HANDLE_SEL_AUTOCLAMP,
 
 	TH_ACTIVE_SPLINE,
-	TH_LASTSEL_POINT,
+	TH_ACTIVE_VERT, /* equivalent of TH_EDITMESH_ACTIVE for splines */
 
 	TH_SYNTAX_B,
 	TH_SYNTAX_V,
+	TH_SYNTAX_R,
 	TH_SYNTAX_C,
 	TH_SYNTAX_L,
+	TH_SYNTAX_D,
 	TH_SYNTAX_N,
+	TH_SYNTAX_S,
 	
 	TH_BONE_SOLID,
 	TH_BONE_POSE,
@@ -132,23 +147,47 @@ enum {
 	TH_STRIP,
 	TH_STRIP_SELECT,
 	
+	TH_KEYTYPE_KEYFRAME, /* KEYTYPES */
+	TH_KEYTYPE_KEYFRAME_SELECT,
+	TH_KEYTYPE_EXTREME,
+	TH_KEYTYPE_EXTREME_SELECT,
+	TH_KEYTYPE_BREAKDOWN,
+	TH_KEYTYPE_BREAKDOWN_SELECT,
+	TH_KEYTYPE_JITTER,
+	TH_KEYTYPE_JITTER_SELECT,
+	
+	TH_KEYBORDER,
+	TH_KEYBORDER_SELECT,
+	
 	TH_LAMP,
 	TH_SPEAKER,
 	TH_CAMERA,
 	TH_EMPTY,
 	
 	TH_NODE,
-	TH_NODE_IN_OUT,
-	TH_NODE_OPERATOR,
+	TH_NODE_INPUT,
+	TH_NODE_OUTPUT,
+	TH_NODE_COLOR,
+	TH_NODE_FILTER,
+	TH_NODE_VECTOR,
+	TH_NODE_TEXTURE,
+	TH_NODE_PATTERN,
+	TH_NODE_SCRIPT,
+	TH_NODE_LAYOUT,
+	TH_NODE_SHADER,
+	TH_NODE_INTERFACE,
 	TH_NODE_CONVERTOR,
 	TH_NODE_GROUP,
 	TH_NODE_FRAME,
+	TH_NODE_MATTE,
+	TH_NODE_DISTORT,
 	
 	TH_CONSOLE_OUTPUT,
 	TH_CONSOLE_INPUT,
 	TH_CONSOLE_INFO,
 	TH_CONSOLE_ERROR,
 	TH_CONSOLE_CURSOR,
+	TH_CONSOLE_SELECT,
 	
 	TH_SEQ_MOVIE,
 	TH_SEQ_MOVIECLIP,
@@ -176,6 +215,7 @@ enum {
 	TH_EDGE_CREASE,
 
 	TH_DRAWEXTRA_EDGELEN,
+	TH_DRAWEXTRA_EDGEANG,
 	TH_DRAWEXTRA_FACEAREA,
 	TH_DRAWEXTRA_FACEANG,
 
@@ -199,6 +239,15 @@ enum {
 	TH_STITCH_PREVIEW_UNSTITCHABLE,
 	TH_STITCH_PREVIEW_ACTIVE,
 
+	TH_PAINT_CURVE_HANDLE,
+	TH_PAINT_CURVE_PIVOT,
+
+	TH_UV_SHADOW,
+	TH_UV_OTHERS,
+
+	TH_FREESTYLE_EDGE_MARK,
+	TH_FREESTYLE_FACE_MARK,
+
 	TH_MATCH,           /* highlight color for search matches */
 	TH_SELECT_HIGHLIGHT, /* highlight color for selected outliner item */
 
@@ -215,7 +264,29 @@ enum {
 	TH_NLA_META,
 	TH_NLA_META_SEL,
 	TH_NLA_SOUND,
-	TH_NLA_SOUND_SEL
+	TH_NLA_SOUND_SEL,
+	
+	TH_WIDGET_EMBOSS,
+
+	TH_AXIS_X,		/* X/Y/Z Axis */
+	TH_AXIS_Y,
+	TH_AXIS_Z,
+
+	TH_LOW_GRAD,
+	TH_HIGH_GRAD,
+	TH_SHOW_BACK_GRAD,
+
+	TH_INFO_SELECTED,
+	TH_INFO_SELECTED_TEXT,
+	TH_INFO_ERROR,
+	TH_INFO_ERROR_TEXT,
+	TH_INFO_WARNING,
+	TH_INFO_WARNING_TEXT,
+	TH_INFO_INFO,
+	TH_INFO_INFO_TEXT,
+	TH_INFO_DEBUG,
+	TH_INFO_DEBUG_TEXT,
+	TH_VIEW_OVERLAY,
 };
 /* XXX WARNING: previous is saved in file, so do not change order! */
 
@@ -250,6 +321,7 @@ int     UI_GetThemeValue(int colorid);
 
 // get three color values, scaled to 0.0-1.0 range
 void    UI_GetThemeColor3fv(int colorid, float col[3]);
+void    UI_GetThemeColorBlend3ubv(int colorid1, int colorid2, float fac, unsigned char col[3]);
 // get the color, range 0.0-1.0, complete with shading offset
 void    UI_GetThemeColorShade3fv(int colorid, int offset, float col[3]);
 void    UI_GetThemeColorShade3ubv(int colorid, int offset, unsigned char col[3]);
@@ -282,9 +354,12 @@ void    UI_SetTheme(int spacetype, int regionid);
 // get current theme
 struct bTheme *UI_GetTheme(void);
 
+// return shadow width outside menus and popups */
+int UI_ThemeMenuShadowWidth(void);
+
 /* only for buttons in theme editor! */
 const unsigned char *UI_ThemeGetColorPtr(struct bTheme *btheme, int spacetype, int colorid);
 
 void UI_make_axis_color(const unsigned char *src_col, unsigned char *dst_col, const char axis);
 
-#endif /*  UI_ICONS_H */
+#endif  /* __UI_RESOURCES_H__ */
