@@ -27,20 +27,15 @@
  *  \ingroup edinterface
  */
 
-#include <stdio.h>
-#include <math.h>
 #include <string.h>
 
 #include "MEM_guardedalloc.h"
 
-#include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_text_types.h" /* for UI_OT_reports_to_text */
 
 #include "BLI_blenlib.h"
 #include "BLI_math_color.h"
-#include "BLI_math_vector.h"
-#include "BLI_utildefines.h"
 
 #include "BLF_api.h"
 #include "BLF_translation.h"
@@ -54,303 +49,26 @@
 #include "RNA_access.h"
 #include "RNA_define.h"
 
-#include "BIF_gl.h"
-
 #include "UI_interface.h"
 
-#include "IMB_colormanagement.h"
-
 #include "interface_intern.h"
 
 #include "WM_api.h"
 #include "WM_types.h"
 
+#include "ED_paint.h"
+
 /* only for UI_OT_editsource */
 #include "ED_screen.h"
 #include "BKE_main.h"
 #include "BLI_ghash.h"
 
-#include "ED_image.h"  /* for HDR color sampling */
-#include "ED_node.h"   /* for HDR color sampling */
-#include "ED_clip.h"   /* for HDR color sampling */
-
-/* ********************************************************** */
-
-typedef struct Eyedropper {
-	struct ColorManagedDisplay *display;
-
-	PointerRNA ptr;
-	PropertyRNA *prop;
-	int index;
-
-	int   accum_start; /* has mouse been presed */
-	float accum_col[3];
-	int   accum_tot;
-} Eyedropper;
-
-static int eyedropper_init(bContext *C, wmOperator *op)
-{
-	Scene *scene = CTX_data_scene(C);
-	Eyedropper *eye;
-	
-	op->customdata = eye = MEM_callocN(sizeof(Eyedropper), "Eyedropper");
-	
-	uiContextActiveProperty(C, &eye->ptr, &eye->prop, &eye->index);
-
-	if ((eye->ptr.data == NULL) ||
-	    (eye->prop == NULL) ||
-	    (RNA_property_editable(&eye->ptr, eye->prop) == FALSE) ||
-	    (RNA_property_array_length(&eye->ptr, eye->prop) < 3) ||
-	    (RNA_property_type(eye->prop) != PROP_FLOAT))
-	{
-		return FALSE;
-	}
-
-	if (RNA_property_subtype(eye->prop) == PROP_COLOR) {
-		const char *display_device;
-
-		display_device = scene->display_settings.display_device;
-		eye->display = IMB_colormanagement_display_get_named(display_device);
-	}
-
-	return TRUE;
-}
-
-static void eyedropper_exit(bContext *C, wmOperator *op)
-{
-	WM_cursor_restore(CTX_wm_window(C));
-	
-	if (op->customdata)
-		MEM_freeN(op->customdata);
-	op->customdata = NULL;
-}
-
-static int eyedropper_cancel(bContext *C, wmOperator *op)
-{
-	eyedropper_exit(C, op);
-	return OPERATOR_CANCELLED;
-}
-
-/* *** eyedropper_color_ helper functions *** */
-
-/**
- * \brief get the color from the screen.
- *
- * Special check for image or nodes where we MAY have HDR pixels which don't display.
- */
-static void eyedropper_color_sample_fl(bContext *C, Eyedropper *UNUSED(eye), int mx, int my, float r_col[3])
-{
-
-	/* we could use some clever */
-	wmWindow *win = CTX_wm_window(C);
-	ScrArea *sa;
-	for (sa = win->screen->areabase.first; sa; sa = sa->next) {
-		if (BLI_rcti_isect_pt(&sa->totrct, mx, my)) {
-			if (sa->spacetype == SPACE_IMAGE) {
-				ARegion *ar = BKE_area_find_region_type(sa, RGN_TYPE_WINDOW);
-				if (BLI_rcti_isect_pt(&ar->winrct, mx, my)) {
-					SpaceImage *sima = sa->spacedata.first;
-					int mval[2] = {mx - ar->winrct.xmin,
-					               my - ar->winrct.ymin};
-
-					if (ED_space_image_color_sample(sima, ar, mval, r_col)) {
-						return;
-					}
-				}
-			}
-			else if (sa->spacetype == SPACE_NODE) {
-				ARegion *ar = BKE_area_find_region_type(sa, RGN_TYPE_WINDOW);
-				if (BLI_rcti_isect_pt(&ar->winrct, mx, my)) {
-					SpaceNode *snode = sa->spacedata.first;
-					int mval[2] = {mx - ar->winrct.xmin,
-					               my - ar->winrct.ymin};
-
-					if (ED_space_node_color_sample(snode, ar, mval, r_col)) {
-						return;
-					}
-				}
-			}
-			else if (sa->spacetype == SPACE_CLIP) {
-				ARegion *ar = BKE_area_find_region_type(sa, RGN_TYPE_WINDOW);
-				if (BLI_rcti_isect_pt(&ar->winrct, mx, my)) {
-					SpaceClip *sc = sa->spacedata.first;
-					int mval[2] = {mx - ar->winrct.xmin,
-					               my - ar->winrct.ymin};
-
-					if (ED_space_clip_color_sample(sc, ar, mval, r_col)) {
-						return;
-					}
-				}
-			}
-		}
-	}
-
-	/* fallback to simple opengl picker */
-	glReadBuffer(GL_FRONT);
-	glReadPixels(mx, my, 1, 1, GL_RGB, GL_FLOAT, r_col);
-	glReadBuffer(GL_BACK);
-}
-
-/* sets the sample color RGB, maintaining A */
-static void eyedropper_color_set(bContext *C, Eyedropper *eye, const float col[3])
-{
-	float col_conv[4];
-
-	/* to maintain alpha */
-	RNA_property_float_get_array(&eye->ptr, eye->prop, col_conv);
-
-	/* convert from display space to linear rgb space */
-	if (eye->display) {
-		copy_v3_v3(col_conv, col);
-		IMB_colormanagement_display_to_scene_linear_v3(col_conv, eye->display);
-	}
-	else {
-		copy_v3_v3(col_conv, col);
-	}
-
-	RNA_property_float_set_array(&eye->ptr, eye->prop, col_conv);
-
-	RNA_property_update(C, &eye->ptr, eye->prop);
-}
-
-/* set sample from accumulated values */
-static void eyedropper_color_set_accum(bContext *C, Eyedropper *eye)
-{
-	float col[3];
-	mul_v3_v3fl(col, eye->accum_col, 1.0f / (float)eye->accum_tot);
-	eyedropper_color_set(C, eye, col);
-}
-
-/* single point sample & set */
-static void eyedropper_color_sample(bContext *C, Eyedropper *eye, int mx, int my)
-{
-	float col[3];
-	eyedropper_color_sample_fl(C, eye, mx, my, col);
-	eyedropper_color_set(C, eye, col);
-}
-
-static void eyedropper_color_sample_accum(bContext *C, Eyedropper *eye, int mx, int my)
-{
-	float col[3];
-	eyedropper_color_sample_fl(C, eye, mx, my, col);
-	/* delay linear conversion */
-	add_v3_v3(eye->accum_col, col);
-	eye->accum_tot++;
-}
-
-/* main modal status check */
-static int eyedropper_modal(bContext *C, wmOperator *op, wmEvent *event)
-{
-	Eyedropper *eye = (Eyedropper *)op->customdata;
-	
-	switch (event->type) {
-		case ESCKEY:
-		case RIGHTMOUSE:
-			return eyedropper_cancel(C, op);
-		case LEFTMOUSE:
-			if (event->val == KM_RELEASE) {
-				if (eye->accum_tot == 0) {
-					eyedropper_color_sample(C, eye, event->x, event->y);
-				}
-				else {
-					eyedropper_color_set_accum(C, eye);
-				}
-				eyedropper_exit(C, op);
-				return OPERATOR_FINISHED;
-			}
-			else if (event->val == KM_PRESS) {
-				/* enable accum and make first sample */
-				eye->accum_start = TRUE;
-				eyedropper_color_sample_accum(C, eye, event->x, event->y);
-			}
-			break;
-		case MOUSEMOVE:
-			if (eye->accum_start) {
-				/* button is pressed so keep sampling */
-				eyedropper_color_sample_accum(C, eye, event->x, event->y);
-				eyedropper_color_set_accum(C, eye);
-			}
-			break;
-		case SPACEKEY:
-			if (event->val == KM_RELEASE) {
-				eye->accum_tot = 0;
-				zero_v3(eye->accum_col);
-				eyedropper_color_sample_accum(C, eye, event->x, event->y);
-				eyedropper_color_set_accum(C, eye);
-			}
-			break;
-	}
-	
-	return OPERATOR_RUNNING_MODAL;
-}
-
-/* Modal Operator init */
-static int eyedropper_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
-{
-	/* init */
-	if (eyedropper_init(C, op)) {
-		WM_cursor_modal(CTX_wm_window(C), BC_EYEDROPPER_CURSOR);
-
-		/* add temp handler */
-		WM_event_add_modal_handler(C, op);
-		
-		return OPERATOR_RUNNING_MODAL;
-	}
-	else {
-		eyedropper_exit(C, op);
-		return OPERATOR_CANCELLED;
-	}
-}
-
-/* Repeat operator */
-static int eyedropper_exec(bContext *C, wmOperator *op)
-{
-	/* init */
-	if (eyedropper_init(C, op)) {
-		
-		/* do something */
-		
-		/* cleanup */
-		eyedropper_exit(C, op);
-		
-		return OPERATOR_FINISHED;
-	}
-	else {
-		return OPERATOR_CANCELLED;
-	}
-}
-
-static int eyedropper_poll(bContext *C)
-{
-	if (!CTX_wm_window(C)) return 0;
-	else return 1;
-}
-
-static void UI_OT_eyedropper(wmOperatorType *ot)
-{
-	/* identifiers */
-	ot->name = "Eyedropper";
-	ot->idname = "UI_OT_eyedropper";
-	ot->description = "Sample a color from the Blender Window to store in a property";
-	
-	/* api callbacks */
-	ot->invoke = eyedropper_invoke;
-	ot->modal = eyedropper_modal;
-	ot->cancel = eyedropper_cancel;
-	ot->exec = eyedropper_exec;
-	ot->poll = eyedropper_poll;
-	
-	/* flags */
-	ot->flag = OPTYPE_BLOCKING;
-	
-	/* properties */
-}
-
 /* Reset Default Theme ------------------------ */
 
 static int reset_default_theme_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	ui_theme_init_default();
+	ui_style_init_default();
 	WM_event_add_notifier(C, NC_WINDOW, NULL);
 	
 	return OPERATOR_FINISHED;
@@ -379,7 +97,7 @@ static int copy_data_path_button_poll(bContext *C)
 	char *path;
 	int index;
 
-	uiContextActiveProperty(C, &ptr, &prop, &index);
+	UI_context_active_but_prop_get(C, &ptr, &prop, &index);
 
 	if (ptr.id.data && ptr.data && prop) {
 		path = RNA_path_from_ID_to_property(&ptr, prop);
@@ -401,13 +119,13 @@ static int copy_data_path_button_exec(bContext *C, wmOperator *UNUSED(op))
 	int index;
 
 	/* try to create driver using property retrieved from UI */
-	uiContextActiveProperty(C, &ptr, &prop, &index);
+	UI_context_active_but_prop_get(C, &ptr, &prop, &index);
 
 	if (ptr.id.data && ptr.data && prop) {
 		path = RNA_path_from_ID_to_property(&ptr, prop);
 		
 		if (path) {
-			WM_clipboard_text_set(path, FALSE);
+			WM_clipboard_text_set(path, false);
 			MEM_freeN(path);
 			return OPERATOR_FINISHED;
 		}
@@ -433,13 +151,35 @@ static void UI_OT_copy_data_path_button(wmOperatorType *ot)
 
 /* Reset to Default Values Button Operator ------------------------ */
 
+static int operator_button_property_finish(bContext *C, PointerRNA *ptr, PropertyRNA *prop)
+{
+	ID *id = ptr->id.data;
+
+	/* perform updates required for this property */
+	RNA_property_update(C, ptr, prop);
+
+	/* as if we pressed the button */
+	UI_context_active_but_prop_handle(C);
+
+	/* Since we don't want to undo _all_ edits to settings, eg window
+	 * edits on the screen or on operator settings.
+	 * it might be better to move undo's inline - campbell */
+	if (id && ID_CHECK_UNDO(id)) {
+		/* do nothing, go ahead with undo */
+		return OPERATOR_FINISHED;
+	}
+	else {
+		return OPERATOR_CANCELLED;
+	}
+}
+
 static int reset_default_button_poll(bContext *C)
 {
 	PointerRNA ptr;
 	PropertyRNA *prop;
 	int index;
 
-	uiContextActiveProperty(C, &ptr, &prop, &index);
+	UI_context_active_but_prop_get(C, &ptr, &prop, &index);
 	
 	return (ptr.data && prop && RNA_property_editable(&ptr, prop));
 }
@@ -448,40 +188,19 @@ static int reset_default_button_exec(bContext *C, wmOperator *op)
 {
 	PointerRNA ptr;
 	PropertyRNA *prop;
-	int success = 0;
-	int index, all = RNA_boolean_get(op->ptr, "all");
+	int index;
+	const bool all = RNA_boolean_get(op->ptr, "all");
 
 	/* try to reset the nominated setting to its default value */
-	uiContextActiveProperty(C, &ptr, &prop, &index);
+	UI_context_active_but_prop_get(C, &ptr, &prop, &index);
 	
 	/* if there is a valid property that is editable... */
 	if (ptr.data && prop && RNA_property_editable(&ptr, prop)) {
-		if (RNA_property_reset(&ptr, prop, (all) ? -1 : index)) {
-			/* perform updates required for this property */
-			RNA_property_update(C, &ptr, prop);
-
-			/* as if we pressed the button */
-			uiContextActivePropertyHandle(C);
-
-			success = 1;
-		}
+		if (RNA_property_reset(&ptr, prop, (all) ? -1 : index))
+			return operator_button_property_finish(C, &ptr, prop);
 	}
 
-	/* Since we don't want to undo _all_ edits to settings, eg window
-	 * edits on the screen or on operator settings.
-	 * it might be better to move undo's inline - campbell */
-	if (success) {
-		ID *id = ptr.id.data;
-		if (id && ID_CHECK_UNDO(id)) {
-			/* do nothing, go ahead with undo */
-		}
-		else {
-			return OPERATOR_CANCELLED;
-		}
-	}
-	/* end hack */
-
-	return (success) ? OPERATOR_FINISHED : OPERATOR_CANCELLED;
+	return OPERATOR_CANCELLED;
 }
 
 static void UI_OT_reset_default_button(wmOperatorType *ot)
@@ -502,130 +221,173 @@ static void UI_OT_reset_default_button(wmOperatorType *ot)
 	RNA_def_boolean(ot->srna, "all", 1, "All", "Reset to default values all elements of the array");
 }
 
-/* Copy To Selected Operator ------------------------ */
+/* Unset Property Button Operator ------------------------ */
 
-static int copy_to_selected_list(bContext *C, PointerRNA *ptr, ListBase *lb, int *use_path)
+static int unset_property_button_exec(bContext *C, wmOperator *UNUSED(op))
 {
-	*use_path = FALSE;
+	PointerRNA ptr;
+	PropertyRNA *prop;
+	int index;
 
-	if (RNA_struct_is_a(ptr->type, &RNA_EditBone))
-		*lb = CTX_data_collection_get(C, "selected_editable_bones");
-	else if (RNA_struct_is_a(ptr->type, &RNA_PoseBone))
-		*lb = CTX_data_collection_get(C, "selected_pose_bones");
-	else if (RNA_struct_is_a(ptr->type, &RNA_Sequence))
-		*lb = CTX_data_collection_get(C, "selected_editable_sequences");
-	else {
-		ID *id = ptr->id.data;
+	/* try to unset the nominated property */
+	UI_context_active_but_prop_get(C, &ptr, &prop, &index);
 
-		if (id && GS(id->name) == ID_OB) {
-			*lb = CTX_data_collection_get(C, "selected_editable_objects");
-			*use_path = TRUE;
-		}
-		else
-			return 0;
+	/* if there is a valid property that is editable... */
+	if (ptr.data && prop && RNA_property_editable(&ptr, prop) &&
+	    /* RNA_property_is_idprop(prop) && */
+	    RNA_property_is_set(&ptr, prop))
+	{
+		RNA_property_unset(&ptr, prop);
+		return operator_button_property_finish(C, &ptr, prop);
 	}
-	
-	return 1;
+
+	return OPERATOR_CANCELLED;
 }
 
-static int copy_to_selected_button_poll(bContext *C)
+static void UI_OT_unset_property_button(wmOperatorType *ot)
+{
+	/* identifiers */
+	ot->name = "Unset property";
+	ot->idname = "UI_OT_unset_property_button";
+	ot->description = "Clear the property and use default or generated value in operators";
+
+	/* callbacks */
+	ot->poll = ED_operator_regionactive;
+	ot->exec = unset_property_button_exec;
+
+	/* flags */
+	ot->flag = OPTYPE_UNDO;
+}
+
+/* Copy To Selected Operator ------------------------ */
+
+static bool copy_to_selected_list(
+        bContext *C, PointerRNA *ptr, PropertyRNA *prop,
+        ListBase *r_lb, bool *r_use_path_from_id, char **r_path)
+{
+	*r_use_path_from_id = false;
+	*r_path = NULL;
+
+	if (RNA_struct_is_a(ptr->type, &RNA_EditBone)) {
+		*r_lb = CTX_data_collection_get(C, "selected_editable_bones");
+	}
+	else if (RNA_struct_is_a(ptr->type, &RNA_PoseBone)) {
+		*r_lb = CTX_data_collection_get(C, "selected_pose_bones");
+	}
+	else if (RNA_struct_is_a(ptr->type, &RNA_Sequence)) {
+		*r_lb = CTX_data_collection_get(C, "selected_editable_sequences");
+	}
+	else if (ptr->id.data) {
+		ID *id = ptr->id.data;
+
+		if (GS(id->name) == ID_OB) {
+			*r_lb = CTX_data_collection_get(C, "selected_editable_objects");
+			*r_use_path_from_id = true;
+			*r_path = RNA_path_from_ID_to_property(ptr, prop);
+		}
+		else if (GS(id->name) == ID_SCE) {
+			/* Sequencer's ID is scene :/ */
+			/* Try to recursively find an RNA_Sequence ancestor, to handle situations like T41062... */
+			if ((*r_path = RNA_path_resolve_from_type_to_property(ptr, prop, &RNA_Sequence)) != NULL) {
+				*r_lb = CTX_data_collection_get(C, "selected_editable_sequences");
+			}
+		}
+		return (*r_path != NULL);
+	}
+	else {
+		return false;
+	}
+
+	return true;
+}
+
+/**
+ * called from both exec & poll
+ *
+ * \note: normally we wouldn't call a loop from within a poll function,
+ * However this is a special case, and for regular poll calls, getting
+ * the context from the button will fail early.
+ */
+static bool copy_to_selected_button(bContext *C, bool all, bool poll)
 {
 	PointerRNA ptr, lptr, idptr;
 	PropertyRNA *prop, *lprop;
-	int index, success = 0;
+	bool success = false;
+	int index;
 
-	uiContextActiveProperty(C, &ptr, &prop, &index);
+	/* try to reset the nominated setting to its default value */
+	UI_context_active_but_prop_get(C, &ptr, &prop, &index);
 
+	/* if there is a valid property that is editable... */
 	if (ptr.data && prop) {
 		char *path = NULL;
-		int use_path;
+		bool use_path_from_id;
 		CollectionPointerLink *link;
 		ListBase lb;
 
-		if (!copy_to_selected_list(C, &ptr, &lb, &use_path))
+		if (!copy_to_selected_list(C, &ptr, prop, &lb, &use_path_from_id, &path))
 			return success;
 
-		if (!use_path || (path = RNA_path_from_ID_to_property(&ptr, prop))) {
-			for (link = lb.first; link; link = link->next) {
-				if (link->ptr.data != ptr.data) {
-					if (use_path) {
-						lprop = NULL;
-						RNA_id_pointer_create(link->ptr.id.data, &idptr);
-						RNA_path_resolve(&idptr, path, &lptr, &lprop);
-					}
-					else {
-						lptr = link->ptr;
-						lprop = prop;
-					}
+		for (link = lb.first; link; link = link->next) {
+			if (link->ptr.data != ptr.data) {
+				if (use_path_from_id) {
+					/* Path relative to ID. */
+					lprop = NULL;
+					RNA_id_pointer_create(link->ptr.id.data, &idptr);
+					RNA_path_resolve_property(&idptr, path, &lptr, &lprop);
+				}
+				else if (path) {
+					/* Path relative to elements from list. */
+					lprop = NULL;
+					RNA_path_resolve_property(&link->ptr, path, &lptr, &lprop);
+				}
+				else {
+					lptr = link->ptr;
+					lprop = prop;
+				}
 
-					if (lprop == prop) {
-						if (RNA_property_editable(&lptr, prop))
-							success = 1;
+				if (lptr.data == ptr.data) {
+					/* lptr might not be the same as link->ptr! */
+					continue;
+				}
+
+				if (lprop == prop) {
+					if (RNA_property_editable(&lptr, lprop)) {
+						if (poll) {
+							success = true;
+							break;
+						}
+						else {
+							if (RNA_property_copy(&lptr, &ptr, prop, (all) ? -1 : index)) {
+								RNA_property_update(C, &lptr, prop);
+								success = true;
+							}
+						}
 					}
 				}
 			}
-
-			if (path)
-				MEM_freeN(path);
 		}
 
+		MEM_SAFE_FREE(path);
 		BLI_freelistN(&lb);
 	}
 
 	return success;
 }
 
+static int copy_to_selected_button_poll(bContext *C)
+{
+	return copy_to_selected_button(C, false, true);
+}
+
 static int copy_to_selected_button_exec(bContext *C, wmOperator *op)
 {
-	PointerRNA ptr, lptr, idptr;
-	PropertyRNA *prop, *lprop;
-	int success = 0;
-	int index, all = RNA_boolean_get(op->ptr, "all");
+	bool success;
 
-	/* try to reset the nominated setting to its default value */
-	uiContextActiveProperty(C, &ptr, &prop, &index);
-	
-	/* if there is a valid property that is editable... */
-	if (ptr.data && prop) {
-		char *path = NULL;
-		int use_path;
-		CollectionPointerLink *link;
-		ListBase lb;
+	const bool all = RNA_boolean_get(op->ptr, "all");
 
-		if (!copy_to_selected_list(C, &ptr, &lb, &use_path))
-			return success;
+	success = copy_to_selected_button(C, all, false);
 
-		if (!use_path || (path = RNA_path_from_ID_to_property(&ptr, prop))) {
-			for (link = lb.first; link; link = link->next) {
-				if (link->ptr.data != ptr.data) {
-					if (use_path) {
-						lprop = NULL;
-						RNA_id_pointer_create(link->ptr.id.data, &idptr);
-						RNA_path_resolve(&idptr, path, &lptr, &lprop);
-					}
-					else {
-						lptr = link->ptr;
-						lprop = prop;
-					}
-
-					if (lprop == prop) {
-						if (RNA_property_editable(&lptr, lprop)) {
-							if (RNA_property_copy(&lptr, &ptr, prop, (all) ? -1 : index)) {
-								RNA_property_update(C, &lptr, prop);
-								success = 1;
-							}
-						}
-					}
-				}
-			}
-
-			if (path)
-				MEM_freeN(path);
-		}
-
-		BLI_freelistN(&lb);
-	}
-	
 	return (success) ? OPERATOR_FINISHED : OPERATOR_CANCELLED;
 }
 
@@ -661,11 +423,12 @@ static int reports_to_text_poll(bContext *C)
 static int reports_to_text_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	ReportList *reports = CTX_wm_reports(C);
+	Main *bmain = CTX_data_main(C);
 	Text *txt;
 	char *str;
 	
 	/* create new text-block to write to */
-	txt = BKE_text_add("Recent Reports");
+	txt = BKE_text_add(bmain, "Recent Reports");
 	
 	/* convert entire list to a display string, and add this to the text-block
 	 *	- if commandline debug option enabled, show debug reports too
@@ -702,12 +465,12 @@ static void UI_OT_reports_to_textblock(wmOperatorType *ot)
 /* EditSource Utility funcs and operator,
  * note, this includes utility functions and button matching checks */
 
-struct uiEditSourceStore {
+typedef struct uiEditSourceStore {
 	uiBut but_orig;
 	GHash *hash;
 } uiEditSourceStore;
 
-struct uiEditSourceButStore {
+typedef struct uiEditSourceButStore {
 	char py_dbg_fn[FILE_MAX];
 	int py_dbg_ln;
 } uiEditSourceButStore;
@@ -715,7 +478,7 @@ struct uiEditSourceButStore {
 /* should only ever be set while the edit source operator is running */
 static struct uiEditSourceStore *ui_editsource_info = NULL;
 
-int  UI_editsource_enable_check(void)
+bool UI_editsource_enable_check(void)
 {
 	return (ui_editsource_info != NULL);
 }
@@ -732,12 +495,12 @@ static void ui_editsource_active_but_set(uiBut *but)
 
 static void ui_editsource_active_but_clear(void)
 {
-	BLI_ghash_free(ui_editsource_info->hash, NULL, (GHashValFreeFP)MEM_freeN);
+	BLI_ghash_free(ui_editsource_info->hash, NULL, MEM_freeN);
 	MEM_freeN(ui_editsource_info);
 	ui_editsource_info = NULL;
 }
 
-static int ui_editsource_uibut_match(uiBut *but_a, uiBut *but_b)
+static bool ui_editsource_uibut_match(uiBut *but_a, uiBut *but_b)
 {
 #if 0
 	printf("matching buttons: '%s' == '%s'\n",
@@ -754,10 +517,10 @@ static int ui_editsource_uibut_match(uiBut *but_a, uiBut *but_b)
 	    (but_a->unit_type == but_b->unit_type) &&
 	    (strncmp(but_a->drawstr, but_b->drawstr, UI_MAX_DRAW_STR) == 0))
 	{
-		return TRUE;
+		return true;
 	}
 	else {
-		return FALSE;
+		return false;
 	}
 }
 
@@ -803,12 +566,11 @@ static int editsource_text_edit(bContext *C, wmOperator *op,
 	}
 
 	if (text == NULL) {
-		text = BKE_text_load(filepath, bmain->name);
+		text = BKE_text_load(bmain, filepath, bmain->name);
 	}
 
 	if (text == NULL) {
-		BKE_reportf(op->reports, RPT_WARNING,
-		            "file: '%s' can't be opened", filepath);
+		BKE_reportf(op->reports, RPT_WARNING, "File '%s' cannot be opened", filepath);
 		return OPERATOR_CANCELLED;
 	}
 	else {
@@ -820,11 +582,10 @@ static int editsource_text_edit(bContext *C, wmOperator *op,
 			st->text = text;
 		}
 		else {
-			BKE_reportf(op->reports, RPT_INFO,
-			            "See '%s' in the text editor", text->id.name + 2);
+			BKE_reportf(op->reports, RPT_INFO, "See '%s' in the text editor", text->id.name + 2);
 		}
 
-		txt_move_toline(text, line - 1, FALSE);
+		txt_move_toline(text, line - 1, false);
 		WM_event_add_notifier(C, NC_TEXT | ND_CURSOR, text);
 	}
 
@@ -833,7 +594,7 @@ static int editsource_text_edit(bContext *C, wmOperator *op,
 
 static int editsource_exec(bContext *C, wmOperator *op)
 {
-	uiBut *but = uiContextActiveButton(C);
+	uiBut *but = UI_context_active_but_get(C);
 
 	if (but) {
 		GHashIterator ghi;
@@ -843,7 +604,7 @@ static int editsource_exec(bContext *C, wmOperator *op)
 		int ret;
 
 		/* needed else the active button does not get tested */
-		uiFreeActiveButtons(C, CTX_wm_screen(C));
+		UI_screen_free_active_but(C, CTX_wm_screen(C));
 
 		// printf("%s: begin\n", __func__);
 
@@ -854,11 +615,11 @@ static int editsource_exec(bContext *C, wmOperator *op)
 		ED_region_do_draw(C, ar);
 
 		for (BLI_ghashIterator_init(&ghi, ui_editsource_info->hash);
-		     !BLI_ghashIterator_isDone(&ghi);
+		     BLI_ghashIterator_done(&ghi) == false;
 		     BLI_ghashIterator_step(&ghi))
 		{
-			uiBut *but = BLI_ghashIterator_getKey(&ghi);
-			if (but && ui_editsource_uibut_match(&ui_editsource_info->but_orig, but)) {
+			uiBut *but_key = BLI_ghashIterator_getKey(&ghi);
+			if (but_key && ui_editsource_uibut_match(&ui_editsource_info->but_orig, but_key)) {
 				but_store = BLI_ghashIterator_getValue(&ghi);
 				break;
 			}
@@ -872,14 +633,12 @@ static int editsource_exec(bContext *C, wmOperator *op)
 				                           but_store->py_dbg_ln);
 			}
 			else {
-				BKE_report(op->reports, RPT_ERROR,
-				           "Active button isn't from a script, cant edit source.");
+				BKE_report(op->reports, RPT_ERROR, "Active button is not from a script, cannot edit source");
 				ret = OPERATOR_CANCELLED;
 			}
 		}
 		else {
-			BKE_report(op->reports, RPT_ERROR,
-			           "Active button match can't be found.");
+			BKE_report(op->reports, RPT_ERROR, "Active button match cannot be found");
 			ret = OPERATOR_CANCELLED;
 		}
 
@@ -919,13 +678,13 @@ static void edittranslation_find_po_file(const char *root, const char *uilng, ch
 	/* First, full lang code. */
 	BLI_snprintf(tstr, sizeof(tstr), "%s.po", uilng);
 	BLI_join_dirfile(path, maxlen, root, uilng);
-	BLI_join_dirfile(path, maxlen, path, tstr);
+	BLI_path_append(path, maxlen, tstr);
 	if (BLI_is_file(path))
 		return;
 
 	/* Now try without the second iso code part (_ES in es_ES). */
 	{
-		char *tc = NULL;
+		const char *tc = NULL;
 		size_t szt = 0;
 		tstr[0] = '\0';
 
@@ -943,7 +702,7 @@ static void edittranslation_find_po_file(const char *root, const char *uilng, ch
 
 			BLI_join_dirfile(path, maxlen, root, tstr);
 			strcat(tstr, ".po");
-			BLI_join_dirfile(path, maxlen, path, tstr);
+			BLI_path_append(path, maxlen, tstr);
 			if (BLI_is_file(path))
 				return;
 		}
@@ -955,16 +714,16 @@ static void edittranslation_find_po_file(const char *root, const char *uilng, ch
 
 static int edittranslation_exec(bContext *C, wmOperator *op)
 {
-	uiBut *but = uiContextActiveButton(C);
+	uiBut *but = UI_context_active_but_get(C);
 	int ret = OPERATOR_CANCELLED;
 
 	if (but) {
+		wmOperatorType *ot;
 		PointerRNA ptr;
 		char popath[FILE_MAX];
 		const char *root = U.i18ndir;
 		const char *uilng = BLF_lang_get();
 
-		const int bufs_nbr = 10;
 		uiStringInfo but_label = {BUT_GET_LABEL, NULL};
 		uiStringInfo rna_label = {BUT_GET_RNA_LABEL, NULL};
 		uiStringInfo enum_label = {BUT_GET_RNAENUM_LABEL, NULL};
@@ -977,27 +736,28 @@ static int edittranslation_exec(bContext *C, wmOperator *op)
 		uiStringInfo rna_ctxt = {BUT_GET_RNA_LABEL_CONTEXT, NULL};
 
 		if (!BLI_is_dir(root)) {
-			BKE_report(op->reports, RPT_ERROR, "Please set your User Preferences' \"Translation Branches "
-			                                   "Directory\" path to a valid directory.");
+			BKE_report(op->reports, RPT_ERROR, "Please set your User Preferences' 'Translation Branches "
+			                                   "Directory' path to a valid directory");
 			return OPERATOR_CANCELLED;
 		}
-		if (!WM_operatortype_find(EDTSRC_I18N_OP_NAME, 0)) {
-			BKE_reportf(op->reports, RPT_ERROR, "Could not find operator \"%s\"! Please enable ui_translate addon "
-			                                    "in the User Preferences.", EDTSRC_I18N_OP_NAME);
+		ot = WM_operatortype_find(EDTSRC_I18N_OP_NAME, 0);
+		if (ot == NULL) {
+			BKE_reportf(op->reports, RPT_ERROR, "Could not find operator '%s'! Please enable ui_translate addon "
+			                                    "in the User Preferences", EDTSRC_I18N_OP_NAME);
 			return OPERATOR_CANCELLED;
 		}
 		/* Try to find a valid po file for current language... */
 		edittranslation_find_po_file(root, uilng, popath, FILE_MAX);
 /*		printf("po path: %s\n", popath);*/
 		if (popath[0] == '\0') {
-			BKE_reportf(op->reports, RPT_ERROR, "No valid po found for language '%s' under %s.", uilng, root);
+			BKE_reportf(op->reports, RPT_ERROR, "No valid po found for language '%s' under %s", uilng, root);
 			return OPERATOR_CANCELLED;
 		}
 
-		uiButGetStrInfo(C, but, bufs_nbr, &but_label, &rna_label, &enum_label, &but_tip, &rna_tip, &enum_tip,
-		                &rna_struct, &rna_prop, &rna_enum, &rna_ctxt);
+		UI_but_string_info_get(C, but, &but_label, &rna_label, &enum_label, &but_tip, &rna_tip, &enum_tip,
+		                &rna_struct, &rna_prop, &rna_enum, &rna_ctxt, NULL);
 
-		WM_operator_properties_create(&ptr, EDTSRC_I18N_OP_NAME);
+		WM_operator_properties_create_ptr(&ptr, ot);
 		RNA_string_set(&ptr, "lang", uilng);
 		RNA_string_set(&ptr, "po_file", popath);
 		RNA_string_set(&ptr, "but_label", but_label.strinfo);
@@ -1010,7 +770,7 @@ static int edittranslation_exec(bContext *C, wmOperator *op)
 		RNA_string_set(&ptr, "rna_prop", rna_prop.strinfo);
 		RNA_string_set(&ptr, "rna_enum", rna_enum.strinfo);
 		RNA_string_set(&ptr, "rna_ctxt", rna_ctxt.strinfo);
-		ret = WM_operator_name_call(C, EDTSRC_I18N_OP_NAME, WM_OP_INVOKE_DEFAULT, &ptr);
+		ret = WM_operator_name_call_ptr(C, ot, WM_OP_INVOKE_DEFAULT, &ptr);
 
 		/* Clean up */
 		if (but_label.strinfo)
@@ -1075,22 +835,118 @@ static void UI_OT_reloadtranslation(wmOperatorType *ot)
 	ot->exec = reloadtranslation_exec;
 }
 
+int UI_drop_color_poll(struct bContext *C, wmDrag *drag, const wmEvent *UNUSED(event))
+{
+	/* should only return true for regions that include buttons, for now
+	 * return true always */
+	if (drag->type == WM_DRAG_COLOR) {
+		SpaceImage *sima = CTX_wm_space_image(C);
+		ARegion *ar = CTX_wm_region(C);
+
+		if (UI_but_active_drop_color(C))
+			return 1;
+
+		if (sima && (sima->mode == SI_MODE_PAINT) &&
+		    sima->image && (ar && ar->regiontype == RGN_TYPE_WINDOW))
+		{
+			return 1;
+		}
+	}
+
+	return 0;
+}
+
+void UI_drop_color_copy(wmDrag *drag, wmDropBox *drop)
+{
+	uiDragColorHandle *drag_info = (uiDragColorHandle *)drag->poin;
+
+	RNA_float_set_array(drop->ptr, "color", drag_info->color);
+	RNA_boolean_set(drop->ptr, "gamma", drag_info->gamma_corrected);
+}
+
+static int drop_color_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
+{
+	ARegion *ar = CTX_wm_region(C);
+	uiBut *but = NULL;
+	float color[4];
+	bool gamma;
+
+	RNA_float_get_array(op->ptr, "color", color);
+	gamma = RNA_boolean_get(op->ptr, "gamma");
+
+	/* find button under mouse, check if it has RNA color property and
+	 * if it does copy the data */
+	but = ui_but_find_active_in_region(ar);
+
+	if (but && but->type == UI_BTYPE_COLOR && but->rnaprop) {
+		const int color_len = RNA_property_array_length(&but->rnapoin, but->rnaprop);
+		BLI_assert(color_len <= 4);
+
+		/* keep alpha channel as-is */
+		if (color_len == 4) {
+			color[3] = RNA_property_float_get_index(&but->rnapoin, but->rnaprop, 3);
+		}
+
+		if (RNA_property_subtype(but->rnaprop) == PROP_COLOR_GAMMA) {
+			if (!gamma)
+				ui_block_cm_to_display_space_v3(but->block, color);
+			RNA_property_float_set_array(&but->rnapoin, but->rnaprop, color);
+			RNA_property_update(C, &but->rnapoin, but->rnaprop);
+		}
+		else if (RNA_property_subtype(but->rnaprop) == PROP_COLOR) {
+			if (gamma)
+				ui_block_cm_to_scene_linear_v3(but->block, color);
+			RNA_property_float_set_array(&but->rnapoin, but->rnaprop, color);
+			RNA_property_update(C, &but->rnapoin, but->rnaprop);
+		}
+	}
+	else {
+		if (gamma) {
+			srgb_to_linearrgb_v3_v3(color, color);
+		}
+
+		ED_imapaint_bucket_fill(C, color, op);
+	}
+
+	ED_region_tag_redraw(ar);
+
+	return OPERATOR_FINISHED;
+}
+
+
+static void UI_OT_drop_color(wmOperatorType *ot)
+{
+	ot->name = "Drop Color";
+	ot->idname = "UI_OT_drop_color";
+	ot->description = "Drop colors to buttons";
+
+	ot->invoke = drop_color_invoke;
+
+	RNA_def_float_color(ot->srna, "color", 3, NULL, 0.0, FLT_MAX, "Color", "Source color", 0.0, 1.0);
+	RNA_def_boolean(ot->srna, "gamma", 0, "Gamma Corrected", "The source color is gamma corrected ");
+}
+
+
+
 /* ********************************************************* */
 /* Registration */
 
-void UI_buttons_operatortypes(void)
+void ED_button_operatortypes(void)
 {
-	WM_operatortype_append(UI_OT_eyedropper);
 	WM_operatortype_append(UI_OT_reset_default_theme);
 	WM_operatortype_append(UI_OT_copy_data_path_button);
 	WM_operatortype_append(UI_OT_reset_default_button);
+	WM_operatortype_append(UI_OT_unset_property_button);
 	WM_operatortype_append(UI_OT_copy_to_selected_button);
 	WM_operatortype_append(UI_OT_reports_to_textblock);  /* XXX: temp? */
-
+	WM_operatortype_append(UI_OT_drop_color);
 #ifdef WITH_PYTHON
 	WM_operatortype_append(UI_OT_editsource);
 	WM_operatortype_append(UI_OT_edittranslation_init);
 #endif
 	WM_operatortype_append(UI_OT_reloadtranslation);
-}
 
+	/* external */
+	WM_operatortype_append(UI_OT_eyedropper_color);
+	WM_operatortype_append(UI_OT_eyedropper_id);
+}
