@@ -38,7 +38,6 @@
 #include "DNA_screen_types.h"
 #include "DNA_userdef_types.h"
 
-#include "BLI_math.h"
 #include "BLI_listbase.h"
 #include "BLI_rect.h"
 #include "BLI_string.h"
@@ -48,7 +47,9 @@
 
 
 #include "BLF_api.h"
-#include "BLF_translation.h"
+#ifdef WITH_INTERNATIONAL
+#  include "BLF_translation.h"
+#endif
 
 #include "UI_interface.h"
 
@@ -56,6 +57,9 @@
 
 #include "interface_intern.h"
 
+#ifdef WIN32
+#  include "BLI_math_base.h" /* M_PI */
+#endif
 
 /* style + theme + layout-engine = UI */
 
@@ -143,30 +147,30 @@ static uiFont *uifont_to_blfont(int id)
 /* *************** draw ************************ */
 
 
-void uiStyleFontDrawExt(uiFontStyle *fs, rcti *rect, const char *str,
-                        float *r_xofs, float *r_yofs)
+void UI_fontstyle_draw_ex(uiFontStyle *fs, const rcti *rect, const char *str,
+                        size_t len, float *r_xofs, float *r_yofs)
 {
 	float height;
 	int xofs = 0, yofs;
 	
-	uiStyleFontSet(fs);
+	UI_fontstyle_set(fs);
 
 	height = BLF_ascender(fs->uifont_id);
 	yofs = ceil(0.5f * (BLI_rcti_size_y(rect) - height));
 
 	if (fs->align == UI_STYLE_TEXT_CENTER) {
-		xofs = floor(0.5f * (BLI_rcti_size_x(rect) - BLF_width(fs->uifont_id, str)));
+		xofs = floor(0.5f * (BLI_rcti_size_x(rect) - BLF_width(fs->uifont_id, str, len)));
 		/* don't center text if it chops off the start of the text, 2 gives some margin */
 		if (xofs < 2) {
 			xofs = 2;
 		}
 	}
 	else if (fs->align == UI_STYLE_TEXT_RIGHT) {
-		xofs = BLI_rcti_size_x(rect) - BLF_width(fs->uifont_id, str) - 1;
+		xofs = BLI_rcti_size_x(rect) - BLF_width(fs->uifont_id, str, len) - 0.1f * U.widget_unit;
 	}
 	
 	/* clip is very strict, so we give it some space */
-	BLF_clipping(fs->uifont_id, rect->xmin - 1, rect->ymin - 4, rect->xmax + 1, rect->ymax + 4);
+	BLF_clipping(fs->uifont_id, rect->xmin - 2, rect->ymin - 4, rect->xmax + 1, rect->ymax + 4);
 	BLF_enable(fs->uifont_id, BLF_CLIPPING);
 	BLF_position(fs->uifont_id, rect->xmin + xofs, rect->ymin + yofs, 0.0f);
 
@@ -179,7 +183,7 @@ void uiStyleFontDrawExt(uiFontStyle *fs, rcti *rect, const char *str,
 	if (fs->kerning == 1)
 		BLF_enable(fs->uifont_id, BLF_KERNING_DEFAULT);
 
-	BLF_draw(fs->uifont_id, str, BLF_DRAW_STR_DUMMY_MAX);
+	BLF_draw(fs->uifont_id, str, len);
 	BLF_disable(fs->uifont_id, BLF_CLIPPING);
 	if (fs->shadow)
 		BLF_disable(fs->uifont_id, BLF_SHADOW);
@@ -190,22 +194,22 @@ void uiStyleFontDrawExt(uiFontStyle *fs, rcti *rect, const char *str,
 	*r_yofs = yofs;
 }
 
-void uiStyleFontDraw(uiFontStyle *fs, rcti *rect, const char *str)
+void UI_fontstyle_draw(uiFontStyle *fs, const rcti *rect, const char *str)
 {
 	float xofs, yofs;
-	uiStyleFontDrawExt(fs, rect, str,
-	                   &xofs, &yofs);
+	UI_fontstyle_draw_ex(fs, rect, str,
+	                   BLF_DRAW_STR_DUMMY_MAX, &xofs, &yofs);
 }
 
 /* drawn same as above, but at 90 degree angle */
-void uiStyleFontDrawRotated(uiFontStyle *fs, rcti *rect, const char *str)
+void UI_fontstyle_draw_rotated(uiFontStyle *fs, const rcti *rect, const char *str)
 {
 	float height;
 	int xofs, yofs;
 	float angle;
 	rcti txtrect;
 
-	uiStyleFontSet(fs);
+	UI_fontstyle_set(fs);
 
 	height = BLF_ascender(fs->uifont_id);
 	/* becomes x-offset when rotated */
@@ -215,8 +219,8 @@ void uiStyleFontDrawRotated(uiFontStyle *fs, rcti *rect, const char *str)
 
 	/* rotate counter-clockwise for now (assumes left-to-right language)*/
 	xofs += height;
-	yofs = BLF_width(fs->uifont_id, str) + 5;
-	angle = 90.0f;
+	yofs = BLF_width(fs->uifont_id, str, BLF_DRAW_STR_DUMMY_MAX) + 5;
+	angle = (float)M_PI / 2.0f;
 
 	/* translate rect to vertical */
 	txtrect.xmin = rect->xmin - BLI_rcti_size_y(rect);
@@ -253,7 +257,7 @@ void uiStyleFontDrawRotated(uiFontStyle *fs, rcti *rect, const char *str)
 
 /* ************** helpers ************************ */
 /* XXX: read a style configure */
-uiStyle *UI_GetStyle(void)
+uiStyle *UI_style_get(void)
 {
 	uiStyle *style = NULL;
 	/* offset is two struct uiStyle pointers */
@@ -261,18 +265,44 @@ uiStyle *UI_GetStyle(void)
 	return (style != NULL) ? style : U.uistyles.first;
 }
 
-/* temporarily, does widget font */
-int UI_GetStringWidth(const char *str)
+/* for drawing, scaled with DPI setting */
+uiStyle *UI_style_get_dpi(void)
 {
-	uiStyle *style = UI_GetStyle();
+	uiStyle *style = UI_style_get();
+	static uiStyle _style;
+	
+	_style = *style;
+	
+	_style.paneltitle.shadx = (short)(UI_DPI_FAC * _style.paneltitle.shadx);
+	_style.paneltitle.shady = (short)(UI_DPI_FAC * _style.paneltitle.shady);
+	_style.grouplabel.shadx = (short)(UI_DPI_FAC * _style.grouplabel.shadx);
+	_style.grouplabel.shady = (short)(UI_DPI_FAC * _style.grouplabel.shady);
+	_style.widgetlabel.shadx = (short)(UI_DPI_FAC * _style.widgetlabel.shadx);
+	_style.widgetlabel.shady = (short)(UI_DPI_FAC * _style.widgetlabel.shady);
+	
+	_style.columnspace = (short)(UI_DPI_FAC * _style.columnspace);
+	_style.templatespace = (short)(UI_DPI_FAC * _style.templatespace);
+	_style.boxspace = (short)(UI_DPI_FAC * _style.boxspace);
+	_style.buttonspacex = (short)(UI_DPI_FAC * _style.buttonspacex);
+	_style.buttonspacey = (short)(UI_DPI_FAC * _style.buttonspacey);
+	_style.panelspace = (short)(UI_DPI_FAC * _style.panelspace);
+	_style.panelouter = (short)(UI_DPI_FAC * _style.panelouter);
+	
+	return &_style;
+}
+
+/* temporarily, does widget font */
+int UI_fontstyle_string_width(const char *str)
+{
+	uiStyle *style = UI_style_get();
 	uiFontStyle *fstyle = &style->widget;
 	int width;
 	
 	if (fstyle->kerning == 1) /* for BLF_width */
 		BLF_enable(fstyle->uifont_id, BLF_KERNING_DEFAULT);
 	
-	uiStyleFontSet(fstyle);
-	width = BLF_width(fstyle->uifont_id, str);
+	UI_fontstyle_set(fstyle);
+	width = BLF_width(fstyle->uifont_id, str, BLF_DRAW_STR_DUMMY_MAX);
 	
 	if (fstyle->kerning == 1)
 		BLF_disable(fstyle->uifont_id, BLF_KERNING_DEFAULT);
@@ -281,14 +311,14 @@ int UI_GetStringWidth(const char *str)
 }
 
 /* temporarily, does widget font */
-void UI_DrawString(float x, float y, const char *str)
+void UI_draw_string(float x, float y, const char *str)
 {
-	uiStyle *style = UI_GetStyle();
+	uiStyle *style = UI_style_get();
 	
 	if (style->widget.kerning == 1)
 		BLF_enable(style->widget.uifont_id, BLF_KERNING_DEFAULT);
 
-	uiStyleFontSet(&style->widget);
+	UI_fontstyle_set(&style->widget);
 	BLF_position(style->widget.uifont_id, x, y, 0.0f);
 	BLF_draw(style->widget.uifont_id, str, BLF_DRAW_STR_DUMMY_MAX);
 
@@ -302,19 +332,33 @@ void UI_DrawString(float x, float y, const char *str)
 /* reading without uifont will create one */
 void uiStyleInit(void)
 {
-	uiFont *font = U.uifonts.first;
+	uiFont *font;
 	uiStyle *style = U.uistyles.first;
+	int monofont_size = datatoc_bmonofont_ttf_size;
+	unsigned char *monofont_ttf = (unsigned char *)datatoc_bmonofont_ttf;
 	
 	/* recover from uninitialized dpi */
 	if (U.dpi == 0)
 		U.dpi = 72;
-	CLAMP(U.dpi, 48, 128);
+	CLAMP(U.dpi, 48, 144);
 	
+	for (font = U.uifonts.first; font; font = font->next) {
+		BLF_unload_id(font->blf_id);
+	}
+
+	font = U.uifonts.first;
+
 	/* default builtin */
 	if (font == NULL) {
 		font = MEM_callocN(sizeof(uiFont), "ui font");
 		BLI_addtail(&U.uifonts, font);
-		
+	}
+
+	if (U.font_path_ui[0]) {
+		BLI_strncpy(font->filename, U.font_path_ui, sizeof(font->filename));
+		font->uifont_id = UIFONT_CUSTOM1;
+	}
+	else {
 		BLI_strncpy(font->filename, "default", sizeof(font->filename));
 		font->uifont_id = UIFONT_DEFAULT;
 	}
@@ -348,11 +392,15 @@ void uiStyleInit(void)
 #else
 			font->blf_id = BLF_load_mem("default", (unsigned char *)datatoc_bfont_ttf, datatoc_bfont_ttf_size);
 #endif
-		}		
+		}
 		else {
 			font->blf_id = BLF_load(font->filename);
-			if (font->blf_id == -1)
+			if (font->blf_id == -1) {
 				font->blf_id = BLF_load_mem("default", (unsigned char *)datatoc_bfont_ttf, datatoc_bfont_ttf_size);
+			}
+			else {
+				BLF_default_set(font->blf_id);
+			}
 		}
 
 		if (font->blf_id == -1) {
@@ -364,9 +412,9 @@ void uiStyleInit(void)
 			 * Yes, this build the glyph cache and create
 			 * the texture.
 			 */
-			BLF_size(font->blf_id, 11, U.dpi);
-			BLF_size(font->blf_id, 12, U.dpi);
-			BLF_size(font->blf_id, 14, U.dpi);
+			BLF_size(font->blf_id, 11 * U.pixelsize, U.dpi);
+			BLF_size(font->blf_id, 12 * U.pixelsize, U.dpi);
+			BLF_size(font->blf_id, 14 * U.pixelsize, U.dpi);
 		}
 	}
 	
@@ -374,23 +422,41 @@ void uiStyleInit(void)
 		ui_style_new(&U.uistyles, "Default Style", UIFONT_DEFAULT);
 	}
 	
+#ifdef WITH_INTERNATIONAL
+	/* use unicode font for text editor and interactive console */
+	if (U.transopts & USER_DOTRANSLATE) {
+		monofont_ttf = BLF_get_unifont_mono(&monofont_size);
+
+		if (!monofont_ttf) {
+			/* fall back if not found */
+			monofont_size = datatoc_bmonofont_ttf_size;
+			monofont_ttf = (unsigned char *)datatoc_bmonofont_ttf;
+		}
+	}
+
+	/* reload */
+	BLF_unload("monospace");
+	blf_mono_font = -1;
+	blf_mono_font_render = -1;
+#endif
+
 	/* XXX, this should be moved into a style, but for now best only load the monospaced font once. */
 	if (blf_mono_font == -1)
-		blf_mono_font = BLF_load_mem_unique("monospace", (unsigned char *)datatoc_bmonofont_ttf, datatoc_bmonofont_ttf_size);
+		blf_mono_font = BLF_load_mem_unique("monospace", monofont_ttf, monofont_size);
 
-	BLF_size(blf_mono_font, 12, 72);
+	BLF_size(blf_mono_font, 12 * U.pixelsize, 72);
 	
 	/* second for rendering else we get threading problems */
 	if (blf_mono_font_render == -1)
-		blf_mono_font_render = BLF_load_mem_unique("monospace", (unsigned char *)datatoc_bmonofont_ttf, datatoc_bmonofont_ttf_size);
+		blf_mono_font_render = BLF_load_mem_unique("monospace", monofont_ttf, monofont_size);
 
-	BLF_size(blf_mono_font_render, 12, 72);
+	BLF_size(blf_mono_font_render, 12 * U.pixelsize, 72);
 }
 
-void uiStyleFontSet(uiFontStyle *fs)
+void UI_fontstyle_set(uiFontStyle *fs)
 {
 	uiFont *font = uifont_to_blfont(fs->uifont_id);
 	
-	BLF_size(font->blf_id, fs->points, U.dpi);
+	BLF_size(font->blf_id, fs->points * U.pixelsize, U.dpi);
 }
 
