@@ -27,37 +27,27 @@
 
 /** \file blender/editors/mesh/meshtools.c
  *  \ingroup edmesh
+ *
+ * meshtools.c: no editmode (violated already :), mirror & join),
+ * tools operating on meshes
  */
 
-
-/*
- * meshtools.c: no editmode (violated already :), tools operating on meshes
- */
-
-#include <stddef.h>
-#include <stdlib.h>
-#include <math.h>
-#include <float.h>
-
 #include "MEM_guardedalloc.h"
 
 #include "DNA_mesh_types.h"
-#include "DNA_view3d_types.h"
 #include "DNA_key_types.h"
 #include "DNA_material_types.h"
-#include "DNA_meshdata_types.h"
 #include "DNA_modifier_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
+#include "DNA_screen_types.h"
+#include "DNA_view3d_types.h"
 
 #include "BLI_math.h"
 #include "BLI_blenlib.h"
-#include "BLI_utildefines.h"
-#include "BLI_ghash.h"
-#include "BLI_rand.h" /* for randome face sorting */
-#include "BLI_threads.h"
 
 
+#include "BLI_kdtree.h"
 #include "BKE_context.h"
 #include "BKE_depsgraph.h"
 #include "BKE_deform.h"
@@ -68,11 +58,9 @@
 #include "BKE_mesh.h"
 #include "BKE_material.h"
 #include "BKE_report.h"
-#include "BKE_tessmesh.h"
+#include "BKE_editmesh.h"
 #include "BKE_multires.h"
 
-#include "BLO_sys_types.h" // for intptr_t support
-
 #include "ED_mesh.h"
 #include "ED_object.h"
 #include "ED_view3d.h"
@@ -80,10 +68,6 @@
 #include "WM_api.h"
 #include "WM_types.h"
 
-/* own include */
-#include "mesh_intern.h"
-#include "uvedit_intern.h"
-
 /* * ********************** no editmode!!! *********** */
 
 /*********************** JOIN ***************************/
@@ -105,15 +89,16 @@ int join_mesh_exec(bContext *C, wmOperator *op)
 	Key *key, *nkey = NULL;
 	KeyBlock *kb, *okb, *kbn;
 	float imat[4][4], cmat[4][4], *fp1, *fp2;
-	int a, b, totcol, totmat = 0, totedge = 0, totvert = 0, ok = 0;
+	int a, b, totcol, totmat = 0, totedge = 0, totvert = 0;
 	int totloop = 0, totpoly = 0, vertofs, *matmap = NULL;
 	int i, j, index, haskey = 0, edgeofs, loopofs, polyofs;
+	bool ok = false;
 	bDeformGroup *dg, *odg;
 	MDeformVert *dvert;
 	CustomData vdata, edata, fdata, ldata, pdata;
 
 	if (scene->obedit) {
-		BKE_report(op->reports, RPT_WARNING, "Cant join while in editmode");
+		BKE_report(op->reports, RPT_WARNING, "Cannot join while in edit mode");
 		return OPERATOR_CANCELLED;
 	}
 	
@@ -136,7 +121,7 @@ int join_mesh_exec(bContext *C, wmOperator *op)
 			totmat += base->object->totcol;
 			
 			if (base->object == ob)
-				ok = 1;
+				ok = true;
 			
 			/* check for shapekeys */
 			if (me->key)
@@ -146,7 +131,7 @@ int join_mesh_exec(bContext *C, wmOperator *op)
 	CTX_DATA_END;
 	
 	/* that way the active object is always selected */ 
-	if (ok == 0) {
+	if (ok == false) {
 		BKE_report(op->reports, RPT_WARNING, "Active object is not a selected mesh");
 		return OPERATOR_CANCELLED;
 	}
@@ -161,10 +146,13 @@ int join_mesh_exec(bContext *C, wmOperator *op)
 	}
 	
 	if (totvert > MESH_MAX_VERTS) {
-		BKE_reportf(op->reports, RPT_WARNING, "Joining results in %d vertices, limit is " STRINGIFY(MESH_MAX_VERTS), totvert);
-		return OPERATOR_CANCELLED;		
+		BKE_reportf(op->reports, RPT_WARNING, "Joining results in %d vertices, limit is %ld", totvert, MESH_MAX_VERTS);
+		return OPERATOR_CANCELLED;
 	}
 
+	/* remove tessface to ensure we don't hold references to invalid faces */
+	BKE_mesh_tessface_clear(me);
+
 	/* new material indices and material array */
 	matar = MEM_callocN(sizeof(void *) * totmat, "join_mesh matar");
 	if (totmat) matmap = MEM_callocN(sizeof(int) * totmat, "join_mesh matmap");
@@ -191,7 +179,6 @@ int join_mesh_exec(bContext *C, wmOperator *op)
 			if (kb->data) MEM_freeN(kb->data);
 			kb->data = MEM_callocN(sizeof(float) * 3 * totvert, "join_shapekey");
 			kb->totelem = totvert;
-			kb->weights = NULL;
 		}
 	}
 	else if (haskey) {
@@ -264,7 +251,7 @@ int join_mesh_exec(bContext *C, wmOperator *op)
 
 							/* adjust settings to fit (allocate a new data-array) */
 							kbn->data = MEM_callocN(sizeof(float) * 3 * totvert, "joined_shapekey");
-							kbn->totelem = totvert;	
+							kbn->totelem = totvert;
 		
 							/* XXX 2.5 Animato */
 #if 0
@@ -296,12 +283,12 @@ int join_mesh_exec(bContext *C, wmOperator *op)
 
 
 	/* setup new data for destination mesh */
-	memset(&vdata, 0, sizeof(vdata));
-	memset(&edata, 0, sizeof(edata));
-	memset(&fdata, 0, sizeof(fdata));
-	memset(&ldata, 0, sizeof(ldata));
-	memset(&pdata, 0, sizeof(pdata));
-	
+	CustomData_reset(&vdata);
+	CustomData_reset(&edata);
+	CustomData_reset(&fdata);
+	CustomData_reset(&ldata);
+	CustomData_reset(&pdata);
+
 	mvert = CustomData_add_layer(&vdata, CD_MVERT, CD_CALLOC, NULL, totvert);
 	medge = CustomData_add_layer(&edata, CD_MEDGE, CD_CALLOC, NULL, totedge);
 	mloop = CustomData_add_layer(&ldata, CD_MLOOP, CD_CALLOC, NULL, totloop);
@@ -322,9 +309,13 @@ int join_mesh_exec(bContext *C, wmOperator *op)
 			me = base->object->data;
 			
 			if (me->totvert) {
+
+				/* merge customdata flag */
+				((Mesh *)ob->data)->cd_flag |= me->cd_flag;
+
 				/* standard data */
 				CustomData_merge(&me->vdata, &vdata, CD_MASK_MESH, CD_DEFAULT, totvert);
-				CustomData_copy_data(&me->vdata, &vdata, 0, vertofs, me->totvert);
+				CustomData_copy_data_named(&me->vdata, &vdata, 0, vertofs, me->totvert);
 				
 				/* vertex groups */
 				dvert = CustomData_get(&vdata, vertofs, CD_MDEFORMVERT);
@@ -351,7 +342,7 @@ int join_mesh_exec(bContext *C, wmOperator *op)
 				/* if this is the object we're merging into, no need to do anything */
 				if (base->object != ob) {
 					/* watch this: switch matmul order really goes wrong */
-					mult_m4_m4m4(cmat, imat, base->object->obmat);
+					mul_m4_m4m4(cmat, imat, base->object->obmat);
 					
 					/* transform vertex coordinates into new space */
 					for (a = 0, mv = mvert; a < me->totvert; a++, mv++) {
@@ -425,7 +416,7 @@ int join_mesh_exec(bContext *C, wmOperator *op)
 			
 			if (me->totedge) {
 				CustomData_merge(&me->edata, &edata, CD_MASK_MESH, CD_DEFAULT, totedge);
-				CustomData_copy_data(&me->edata, &edata, 0, edgeofs, me->totedge);
+				CustomData_copy_data_named(&me->edata, &edata, 0, edgeofs, me->totedge);
 				
 				for (a = 0; a < me->totedge; a++, medge++) {
 					medge->v1 += vertofs;
@@ -439,15 +430,15 @@ int join_mesh_exec(bContext *C, wmOperator *op)
 
 					multiresModifier_prepare_join(scene, base->object, ob);
 
-					if ((mmd = get_multires_modifier(scene, base->object, TRUE))) {
-						ED_object_iter_other(bmain, base->object, TRUE,
+					if ((mmd = get_multires_modifier(scene, base->object, true))) {
+						ED_object_iter_other(bmain, base->object, true,
 						                     ED_object_multires_update_totlevels_cb,
 						                     &mmd->totlvl);
 					}
 				}
 				
 				CustomData_merge(&me->ldata, &ldata, CD_MASK_MESH, CD_DEFAULT, totloop);
-				CustomData_copy_data(&me->ldata, &ldata, 0, loopofs, me->totloop);
+				CustomData_copy_data_named(&me->ldata, &ldata, 0, loopofs, me->totloop);
 				
 				for (a = 0; a < me->totloop; a++, mloop++) {
 					mloop->v += vertofs;
@@ -471,7 +462,7 @@ int join_mesh_exec(bContext *C, wmOperator *op)
 				}
 
 				CustomData_merge(&me->pdata, &pdata, CD_MASK_MESH, CD_DEFAULT, totpoly);
-				CustomData_copy_data(&me->pdata, &pdata, 0, polyofs, me->totpoly);
+				CustomData_copy_data_named(&me->pdata, &pdata, 0, polyofs, me->totpoly);
 				
 				for (a = 0; a < me->totpoly; a++, mpoly++) {
 					mpoly->loopstart += loopofs;
@@ -513,7 +504,11 @@ int join_mesh_exec(bContext *C, wmOperator *op)
 	me->ldata = ldata;
 	me->pdata = pdata;
 
-	mesh_update_customdata_pointers(me, TRUE); /* BMESH_TODO, check if this arg can be failse, non urgent - campbell */
+	/* tessface data removed above, no need to update */
+	BKE_mesh_update_customdata_pointers(me, false);
+
+	/* update normals in case objects with non-uniform scale are joined */
+	BKE_mesh_calc_normals(me);
 	
 	/* old material array */
 	for (a = 1; a <= ob->totcol; a++) {
@@ -543,7 +538,7 @@ int join_mesh_exec(bContext *C, wmOperator *op)
 	if (matmap) MEM_freeN(matmap);
 	
 	/* other mesh users */
-	test_object_materials((ID *)me);
+	test_object_materials(bmain, (ID *)me);
 	
 	/* free temp copy of destination shapekeys (if applicable) */
 	if (nkey) {
@@ -568,14 +563,14 @@ int join_mesh_exec(bContext *C, wmOperator *op)
 	}
 
 
-	DAG_scene_sort(bmain, scene);   // removed objects, need to rebuild dag before editmode call
+	DAG_relations_tag_update(bmain);   // removed objects, need to rebuild dag
 
 #if 0
-	ED_object_enter_editmode(C, EM_WAITCURSOR);
-	ED_object_exit_editmode(C, EM_FREEDATA | EM_WAITCURSOR | EM_DO_UNDO);
+	ED_object_editmode_enter(C, EM_WAITCURSOR);
+	ED_object_editmode_exit(C, EM_FREEDATA | EM_WAITCURSOR | EM_DO_UNDO);
 #else
 	/* toggle editmode using lower level functions so this can be called from python */
-	EDBM_mesh_make(scene->toolsettings, scene, ob);
+	EDBM_mesh_make(scene->toolsettings, ob);
 	EDBM_mesh_load(ob);
 	EDBM_mesh_free(me->edit_btmesh);
 	MEM_freeN(me->edit_btmesh);
@@ -601,7 +596,7 @@ int join_mesh_shapes_exec(bContext *C, wmOperator *op)
 	DerivedMesh *dm = NULL;
 	Key *key = me->key;
 	KeyBlock *kb;
-	int ok = 0, nonequal_verts = 0;
+	bool ok = false, nonequal_verts = false;
 	
 	CTX_DATA_BEGIN (C, Base *, base, selected_editable_bases)
 	{
@@ -611,7 +606,7 @@ int join_mesh_shapes_exec(bContext *C, wmOperator *op)
 			selme = (Mesh *)base->object->data;
 			
 			if (selme->totvert == me->totvert)
-				ok++;
+				ok = true;
 			else
 				nonequal_verts = 1;
 		}
@@ -632,7 +627,7 @@ int join_mesh_shapes_exec(bContext *C, wmOperator *op)
 
 		/* first key added, so it was the basis. initialize it with the existing mesh */
 		kb = BKE_keyblock_add(key, NULL);
-		BKE_key_convert_from_mesh(me, kb);
+		BKE_keyblock_convert_from_mesh(me, kb);
 	}
 	
 	/* now ready to add new keys from selected meshes */
@@ -663,264 +658,129 @@ int join_mesh_shapes_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-/* ********************* MESH VERTEX OCTREE LOOKUP ************* */
+/* -------------------------------------------------------------------- */
+/* Mesh Mirror (Spatial) */
 
-/* important note; this is unfinished, needs better API for editmode, and custom threshold */
+/** \name Mesh Spatial Mirror API
+ * \{ */
 
-#define MOC_RES         8
-#define MOC_NODE_RES    8
-#define MOC_THRESH      0.00002f
+#define KD_THRESH      0.00002f
 
-typedef struct MocNode {
-	struct MocNode *next;
-	intptr_t index[MOC_NODE_RES];
-} MocNode;
-
-static int mesh_octree_get_base_offs(const float co[3], const float offs[3], const float div[3])
-{
-	int vx, vy, vz;
-	
-	vx = floor((co[0] - offs[0]) / div[0]);
-	vy = floor((co[1] - offs[1]) / div[1]);
-	vz = floor((co[2] - offs[2]) / div[2]);
-
-	CLAMP(vx, 0, MOC_RES - 1);
-	CLAMP(vy, 0, MOC_RES - 1);
-	CLAMP(vz, 0, MOC_RES - 1);
-
-	return (vx * MOC_RES * MOC_RES) + vy * MOC_RES + vz;
-}
-
-static void mesh_octree_add_node(MocNode **bt, intptr_t index)
-{
-	if (*bt == NULL) {
-		*bt = MEM_callocN(sizeof(MocNode), "MocNode");
-		(*bt)->index[0] = index;
-	}
-	else {
-		int a;
-		for (a = 0; a < MOC_NODE_RES; a++) {
-			if ((*bt)->index[a] == index)
-				return;
-			else if ((*bt)->index[a] == 0) {
-				(*bt)->index[a] = index;
-				return;
-			}
-		}
-		mesh_octree_add_node(&(*bt)->next, index);
-	}
-}
-
-static void mesh_octree_free_node(MocNode **bt)
-{
-	if ( (*bt)->next) {
-		mesh_octree_free_node(&(*bt)->next);
-	}
-	MEM_freeN(*bt);
-}
-
-
-/* temporal define, just to make nicer code below */
-#define MOC_INDEX(vx, vy, vz)  (((vx) * MOC_RES * MOC_RES) + (vy) * MOC_RES + (vz))
-
-static void mesh_octree_add_nodes(MocNode **basetable, const float co[3], const float offs[3],
-                                  const float div[3], intptr_t index)
-{
-	float fx, fy, fz;
-	int vx, vy, vz;
-	
-	if ((finite(co[0]) == FALSE) ||
-	    (finite(co[1]) == FALSE) ||
-	    (finite(co[2]) == FALSE))
-	{
-		return;
-	}
-	
-	fx = (co[0] - offs[0]) / div[0];
-	fy = (co[1] - offs[1]) / div[1];
-	fz = (co[2] - offs[2]) / div[2];
-	CLAMP(fx, 0.0f, MOC_RES - MOC_THRESH);
-	CLAMP(fy, 0.0f, MOC_RES - MOC_THRESH);
-	CLAMP(fz, 0.0f, MOC_RES - MOC_THRESH);
-
-	vx = (int)floorf(fx);
-	vy = (int)floorf(fy);
-	vz = (int)floorf(fz);
-
-	mesh_octree_add_node(basetable + MOC_INDEX(vx, vy, vz), index);
-
-	if (vx > 0)
-		if (fx - ((float)vx) - MOC_THRESH < 0.0f)
-			mesh_octree_add_node(basetable + MOC_INDEX(vx - 1, vy, vz), index);
-	if (vx < MOC_RES - 2)
-		if (fx - ((float)vx) + MOC_THRESH > 1.0f)
-			mesh_octree_add_node(basetable + MOC_INDEX(vx + 1, vy, vz), index);
-
-	if (vy > 0)
-		if (fy - ((float)vy) - MOC_THRESH < 0.0f)
-			mesh_octree_add_node(basetable + MOC_INDEX(vx, vy - 1, vz), index);
-	if (vy < MOC_RES - 2)
-		if (fy - ((float)vy) + MOC_THRESH > 1.0f)
-			mesh_octree_add_node(basetable + MOC_INDEX(vx, vy + 1, vz), index);
-
-	if (vz > 0)
-		if (fz - ((float)vz) - MOC_THRESH < 0.0f)
-			mesh_octree_add_node(basetable + MOC_INDEX(vx, vy, vz - 1), index);
-	if (vz < MOC_RES - 2)
-		if (fz - ((float)vz) + MOC_THRESH > 1.0f)
-			mesh_octree_add_node(basetable + MOC_INDEX(vx, vy, vz + 1), index);
-
-}
-
-static intptr_t mesh_octree_find_index(MocNode **bt, MVert *mvert, const float co[3])
-{
-	float *vec;
-	int a;
-	
-	if (*bt == NULL)
-		return -1;
-	
-	for (a = 0; a < MOC_NODE_RES; a++) {
-		if ((*bt)->index[a]) {
-			/* does mesh verts and editmode, code looks potential dangerous, octree should really be filled OK! */
-			if (mvert) {
-				vec = (mvert + (*bt)->index[a] - 1)->co;
-				if (compare_v3v3(vec, co, MOC_THRESH))
-					return (*bt)->index[a] - 1;
-			}
-			else {
-				BMVert *eve = (BMVert *)((*bt)->index[a]);
-				if (compare_v3v3(eve->co, co, MOC_THRESH))
-					return (*bt)->index[a];
-			}
-		}
-		else return -1;
-	}
-	if ( (*bt)->next)
-		return mesh_octree_find_index(&(*bt)->next, mvert, co);
-	
-	return -1;
-}
-
-static struct {
-	MocNode **table;
-	float offs[3], div[3];
-} MeshOctree = {NULL, {0, 0, 0}, {0, 0, 0}};
+static struct { void *tree; } MirrKdStore = {NULL};
 
 /* mode is 's' start, or 'e' end, or 'u' use */
 /* if end, ob can be NULL */
-intptr_t mesh_octree_table(Object *ob, BMEditMesh *em, const float co[3], char mode)
+int ED_mesh_mirror_spatial_table(Object *ob, BMEditMesh *em, const float co[3], char mode)
 {
-	MocNode **bt;
-	
 	if (mode == 'u') {        /* use table */
-		if (MeshOctree.table == NULL)
-			mesh_octree_table(ob, em, NULL, 's');
+		if (MirrKdStore.tree == NULL)
+			ED_mesh_mirror_spatial_table(ob, em, NULL, 's');
 
-		if (MeshOctree.table) {
-			Mesh *me = ob->data;
-			bt = MeshOctree.table + mesh_octree_get_base_offs(co, MeshOctree.offs, MeshOctree.div);
-			if (em)
-				return mesh_octree_find_index(bt, NULL, co);
-			else
-				return mesh_octree_find_index(bt, me->mvert, co);
+		if (MirrKdStore.tree) {
+			KDTreeNearest nearest;
+
+			int i;
+
+			i = BLI_kdtree_find_nearest(MirrKdStore.tree, co, &nearest);
+
+			if (i != -1) {
+				if (nearest.dist < KD_THRESH) {
+					return i;
+				}
+			}
 		}
 		return -1;
 	}
 	else if (mode == 's') {   /* start table */
 		Mesh *me = ob->data;
-		float min[3], max[3];
+		int totvert;
 
-		/* we compute own bounding box and don't reuse ob->bb because
-		 * we are using the undeformed coordinates*/
-		INIT_MINMAX(min, max);
+		if (MirrKdStore.tree) /* happens when entering this call without ending it */
+			ED_mesh_mirror_spatial_table(ob, em, co, 'e');
 
 		if (em && me->edit_btmesh == em) {
-			BMIter iter;
-			BMVert *eve;
-			
-			BM_ITER_MESH (eve, &iter, em->bm, BM_VERTS_OF_MESH) {
-				minmax_v3v3_v3(min, max, eve->co);
-			}
+			totvert = em->bm->totvert;
 		}
-		else {		
-			MVert *mvert;
-			int a;
-			
-			for (a = 0, mvert = me->mvert; a < me->totvert; a++, mvert++)
-				minmax_v3v3_v3(min, max, mvert->co);
+		else {
+			totvert = me->totvert;
 		}
-		
-		/* for quick unit coordinate calculus */
-		copy_v3_v3(MeshOctree.offs, min);
-		/* we offset it 1 threshold unit extra */
-		add_v3_fl(MeshOctree.offs, -MOC_THRESH);
-		
-		sub_v3_v3v3(MeshOctree.div, max, min);
-		/* and divide with 2 threshold unit more extra (try 8x8 unit grid on paint) */
-		add_v3_fl(MeshOctree.div, 2.0f * MOC_THRESH);
 
-		mul_v3_fl(MeshOctree.div, 1.0f / MOC_RES);
-		if (MeshOctree.div[0] == 0.0f) MeshOctree.div[0] = 1.0f;
-		if (MeshOctree.div[1] == 0.0f) MeshOctree.div[1] = 1.0f;
-		if (MeshOctree.div[2] == 0.0f) MeshOctree.div[2] = 1.0f;
-			
-		if (MeshOctree.table) /* happens when entering this call without ending it */
-			mesh_octree_table(ob, em, co, 'e');
-		
-		MeshOctree.table = MEM_callocN(MOC_RES * MOC_RES * MOC_RES * sizeof(void *), "sym table");
-		
+		MirrKdStore.tree = BLI_kdtree_new(totvert);
+
 		if (em && me->edit_btmesh == em) {
+
 			BMVert *eve;
 			BMIter iter;
+			int i;
 
-			BM_ITER_MESH (eve, &iter, em->bm, BM_VERTS_OF_MESH) {
-				mesh_octree_add_nodes(MeshOctree.table, eve->co, MeshOctree.offs, MeshOctree.div, (intptr_t)(eve));
+			/* this needs to be valid for index lookups later (callers need) */
+			BM_mesh_elem_table_ensure(em->bm, BM_VERT);
+
+			BM_ITER_MESH_INDEX (eve, &iter, em->bm, BM_VERTS_OF_MESH, i) {
+				BLI_kdtree_insert(MirrKdStore.tree, i, eve->co);
 			}
 		}
-		else {		
+		else {
 			MVert *mvert;
-			int a;
+			int i;
 			
-			for (a = 0, mvert = me->mvert; a < me->totvert; a++, mvert++)
-				mesh_octree_add_nodes(MeshOctree.table, mvert->co, MeshOctree.offs, MeshOctree.div, a + 1);
+			for (i = 0, mvert = me->mvert; i < me->totvert; i++, mvert++) {
+				BLI_kdtree_insert(MirrKdStore.tree, i, mvert->co);
+			}
 		}
+
+		BLI_kdtree_balance(MirrKdStore.tree);
 	}
 	else if (mode == 'e') { /* end table */
-		if (MeshOctree.table) {
-			int a;
-			
-			for (a = 0, bt = MeshOctree.table; a < MOC_RES * MOC_RES * MOC_RES; a++, bt++) {
-				if (*bt) mesh_octree_free_node(bt);
-			}
-			MEM_freeN(MeshOctree.table);
-			MeshOctree.table = NULL;
+		if (MirrKdStore.tree) {
+			BLI_kdtree_free(MirrKdStore.tree);
+			MirrKdStore.tree = NULL;
 		}
 	}
+	else {
+		BLI_assert(0);
+	}
+
 	return 0;
 }
 
+/** \} */
+
+
+/* -------------------------------------------------------------------- */
+/* Mesh Mirror (Topology) */
+
+/** \name Mesh Topology Mirror API
+ * \{ */
+
 static MirrTopoStore_t mesh_topo_store = {NULL, -1. - 1, -1};
 
 /* mode is 's' start, or 'e' end, or 'u' use */
 /* if end, ob can be NULL */
 /* note, is supposed return -1 on error, which callers are currently checking for, but is not used so far */
-int mesh_mirrtopo_table(Object *ob, char mode)
+int ED_mesh_mirror_topo_table(Object *ob, char mode)
 {
 	if (mode == 'u') {        /* use table */
 		if (ED_mesh_mirrtopo_recalc_check(ob->data, ob->mode, &mesh_topo_store)) {
-			mesh_mirrtopo_table(ob, 's');
+			ED_mesh_mirror_topo_table(ob, 's');
 		}
 	}
 	else if (mode == 's') { /* start table */
-		ED_mesh_mirrtopo_init(ob->data, ob->mode, &mesh_topo_store, FALSE);
+		ED_mesh_mirrtopo_init(ob->data, ob->mode, &mesh_topo_store, false);
 	}
 	else if (mode == 'e') { /* end table */
 		ED_mesh_mirrtopo_free(&mesh_topo_store);
 	}
+	else {
+		BLI_assert(0);
+	}
+
 	return 0;
 }
 
+/** \} */
+
+
 static int mesh_get_x_mirror_vert_spatial(Object *ob, int index)
 {
 	Mesh *me = ob->data;
@@ -932,37 +792,36 @@ static int mesh_get_x_mirror_vert_spatial(Object *ob, int index)
 	vec[1] = mvert->co[1];
 	vec[2] = mvert->co[2];
 	
-	return mesh_octree_table(ob, NULL, vec, 'u');
+	return ED_mesh_mirror_spatial_table(ob, NULL, vec, 'u');
 }
 
 static int mesh_get_x_mirror_vert_topo(Object *ob, int index)
 {
-	if (mesh_mirrtopo_table(ob, 'u') == -1)
+	if (ED_mesh_mirror_topo_table(ob, 'u') == -1)
 		return -1;
 
 	return mesh_topo_store.index_lookup[index];
 }
 
-int mesh_get_x_mirror_vert(Object *ob, int index)
+int mesh_get_x_mirror_vert(Object *ob, int index, const bool use_topology)
 {
-	if (((Mesh *)ob->data)->editflag & ME_EDIT_MIRROR_TOPO) {
+	if (use_topology) {
 		return mesh_get_x_mirror_vert_topo(ob, index);
 	}
 	else {
 		return mesh_get_x_mirror_vert_spatial(ob, index);
 	}
-	return 0;
 }
 
 static BMVert *editbmesh_get_x_mirror_vert_spatial(Object *ob, BMEditMesh *em, const float co[3])
 {
 	float vec[3];
-	intptr_t poinval;
+	int i;
 	
 	/* ignore nan verts */
-	if ((finite(co[0]) == FALSE) ||
-	    (finite(co[1]) == FALSE) ||
-	    (finite(co[2]) == FALSE))
+	if ((finite(co[0]) == false) ||
+	    (finite(co[1]) == false) ||
+	    (finite(co[2]) == false))
 	{
 		return NULL;
 	}
@@ -971,16 +830,17 @@ static BMVert *editbmesh_get_x_mirror_vert_spatial(Object *ob, BMEditMesh *em, c
 	vec[1] = co[1];
 	vec[2] = co[2];
 	
-	poinval = mesh_octree_table(ob, em, vec, 'u');
-	if (poinval != -1)
-		return (BMVert *)(poinval);
+	i = ED_mesh_mirror_spatial_table(ob, em, vec, 'u');
+	if (i != -1) {
+		return BM_vert_at_index(em->bm, i);
+	}
 	return NULL;
 }
 
 static BMVert *editbmesh_get_x_mirror_vert_topo(Object *ob, struct BMEditMesh *em, BMVert *eve, int index)
 {
 	intptr_t poinval;
-	if (mesh_mirrtopo_table(ob, 'u') == -1)
+	if (ED_mesh_mirror_topo_table(ob, 'u') == -1)
 		return NULL;
 
 	if (index == -1) {
@@ -1006,9 +866,9 @@ static BMVert *editbmesh_get_x_mirror_vert_topo(Object *ob, struct BMEditMesh *e
 	return NULL;
 }	
 
-BMVert *editbmesh_get_x_mirror_vert(Object *ob, struct BMEditMesh *em, BMVert *eve, const float co[3], int index)
+BMVert *editbmesh_get_x_mirror_vert(Object *ob, struct BMEditMesh *em, BMVert *eve, const float co[3], int index, const bool use_topology)
 {
-	if (((Mesh *)ob->data)->editflag & ME_EDIT_MIRROR_TOPO) {
+	if (use_topology) {
 		return editbmesh_get_x_mirror_vert_topo(ob, em, eve, index);
 	}
 	else {
@@ -1016,6 +876,31 @@ BMVert *editbmesh_get_x_mirror_vert(Object *ob, struct BMEditMesh *em, BMVert *e
 	}
 }
 
+/**
+ * Wrapper for objectmode/editmode.
+ *
+ * call #BM_mesh_elem_table_ensure first for editmesh.
+ */
+int ED_mesh_mirror_get_vert(Object *ob, int index)
+{
+	Mesh *me = ob->data;
+	BMEditMesh *em = me->edit_btmesh;
+	bool use_topology = (me->editflag & ME_EDIT_MIRROR_TOPO) != 0;
+	int index_mirr;
+
+	if (em) {
+		BMVert *eve, *eve_mirr;
+		eve = BM_vert_at_index(em->bm, index);
+		eve_mirr = editbmesh_get_x_mirror_vert(ob, em, eve, eve->co, index, use_topology);
+		index_mirr = eve_mirr ? BM_elem_index_get(eve_mirr) : -1;
+	}
+	else {
+		 index_mirr = mesh_get_x_mirror_vert(ob, index, use_topology);
+	}
+
+	return index_mirr;
+}
+
 #if 0
 
 static float *editmesh_get_mirror_uv(BMEditMesh *em, int axis, float *uv, float *mirrCent, float *face_cent)
@@ -1051,7 +936,7 @@ static float *editmesh_get_mirror_uv(BMEditMesh *em, int axis, float *uv, float
 		BMFace *efa;
 		
 		BM_ITER_MESH (efa, &iter, em->bm, BM_FACES_OF_MESH) {
-			uv_poly_center(em, efa, cent);
+			uv_poly_center(efa, cent, cd_loop_uv_offset);
 			
 			if ( (fabsf(cent[0] - cent_vec[0]) < 0.001f) && (fabsf(cent[1] - cent_vec[1]) < 0.001f) ) {
 				BMIter liter;
@@ -1076,7 +961,7 @@ static float *editmesh_get_mirror_uv(BMEditMesh *em, int axis, float *uv, float
 static unsigned int mirror_facehash(const void *ptr)
 {
 	const MFace *mf = ptr;
-	int v0, v1;
+	unsigned int v0, v1;
 
 	if (mf->v4) {
 		v0 = MIN4(mf->v1, mf->v2, mf->v3, mf->v4);
@@ -1114,7 +999,7 @@ static int mirror_facerotation(MFace *a, MFace *b)
 	return -1;
 }
 
-static int mirror_facecmp(const void *a, const void *b)
+static bool mirror_facecmp(const void *a, const void *b)
 {
 	return (mirror_facerotation((MFace *)a, (MFace *)b) == -1);
 }
@@ -1126,20 +1011,21 @@ int *mesh_get_x_mirror_faces(Object *ob, BMEditMesh *em)
 	MVert *mv, *mvert = me->mvert;
 	MFace mirrormf, *mf, *hashmf, *mface = me->mface;
 	GHash *fhash;
+	const bool use_topology = (me->editflag & ME_EDIT_MIRROR_TOPO) != 0;
 	int *mirrorverts, *mirrorfaces;
 	int a;
 
 	mirrorverts = MEM_callocN(sizeof(int) * me->totvert, "MirrorVerts");
 	mirrorfaces = MEM_callocN(sizeof(int) * 2 * me->totface, "MirrorFaces");
 
-	mesh_octree_table(ob, em, NULL, 's');
+	ED_mesh_mirror_spatial_table(ob, em, NULL, 's');
 
 	for (a = 0, mv = mvert; a < me->totvert; a++, mv++)
-		mirrorverts[a] = mesh_get_x_mirror_vert(ob, a);
+		mirrorverts[a] = mesh_get_x_mirror_vert(ob, a, use_topology);
 
-	mesh_octree_table(ob, em, NULL, 'e');
+	ED_mesh_mirror_spatial_table(ob, em, NULL, 'e');
 
-	fhash = BLI_ghash_new(mirror_facehash, mirror_facecmp, "mirror_facehash gh");
+	fhash = BLI_ghash_new_ex(mirror_facehash, mirror_facecmp, "mirror_facehash gh", me->totface);
 	for (a = 0, mf = mface; a < me->totface; a++, mf++)
 		BLI_ghash_insert(fhash, mf, mf);
 
@@ -1177,14 +1063,17 @@ int *mesh_get_x_mirror_faces(Object *ob, BMEditMesh *em)
  * Face selection in object mode,
  * currently only weight-paint and vertex-paint use this.
  *
- * \return boolean TRUE == Found
+ * \return boolean true == Found
  */
-int ED_mesh_pick_face(bContext *C, Mesh *me, const int mval[2], unsigned int *index, int size)
+bool ED_mesh_pick_face(bContext *C, Object *ob, const int mval[2], unsigned int *index, int size)
 {
 	ViewContext vc;
+	Mesh *me = ob->data;
+
+	BLI_assert(me && GS(me->id.name) == ID_ME);
 
 	if (!me || me->totpoly == 0)
-		return 0;
+		return false;
 
 	view3d_set_viewcontext(C, &vc);
 
@@ -1192,7 +1081,7 @@ int ED_mesh_pick_face(bContext *C, Mesh *me, const int mval[2], unsigned int *in
 		/* sample rect to increase chances of selecting, so that when clicking
 		 * on an edge in the backbuf, we can still select a face */
 
-		int dummy_dist;
+		float dummy_dist;
 		*index = view3d_sample_backbuf_rect(&vc, mval, size, 1, me->totpoly + 1, &dummy_dist, 0, NULL, NULL);
 	}
 	else {
@@ -1200,95 +1089,262 @@ int ED_mesh_pick_face(bContext *C, Mesh *me, const int mval[2], unsigned int *in
 		*index = view3d_sample_backbuf(&vc, mval[0], mval[1]);
 	}
 
-	if ((*index) <= 0 || (*index) > (unsigned int)me->totpoly)
-		return 0;
+	if ((*index) == 0 || (*index) > (unsigned int)me->totpoly)
+		return false;
 
 	(*index)--;
 
-	return 1;
+	return true;
+}
+static void ed_mesh_pick_face_vert__mpoly_find(
+        /* context */
+        struct ARegion *ar, const float mval[2],
+        /* mesh data */
+        DerivedMesh *dm, MPoly *mp, MLoop *mloop,
+        /* return values */
+        float *r_len_best, int *r_v_idx_best)
+{
+	const MLoop *ml;
+	int j = mp->totloop;
+	for (ml = &mloop[mp->loopstart]; j--; ml++) {
+		float co[3], sco[2], len;
+		const int v_idx = ml->v;
+		dm->getVertCo(dm, v_idx, co);
+		if (ED_view3d_project_float_object(ar, co, sco, V3D_PROJ_TEST_NOP) == V3D_PROJ_RET_OK) {
+			len = len_manhattan_v2v2(mval, sco);
+			if (len < *r_len_best) {
+				*r_len_best = len;
+				*r_v_idx_best = v_idx;
+			}
+		}
+	}
 }
 /**
  * Use when the back buffer stores face index values. but we want a vert.
  * This gets the face then finds the closest vertex to mval.
  */
-int ED_mesh_pick_face_vert(bContext *C, Mesh *me, Object *ob, const int mval[2], unsigned int *index, int size)
+bool ED_mesh_pick_face_vert(bContext *C, Object *ob, const int mval[2], unsigned int *index, int size)
 {
 	unsigned int poly_index;
+	Mesh *me = ob->data;
 
-	if (ED_mesh_pick_face(C, me, mval, &poly_index, size)) {
+	BLI_assert(me && GS(me->id.name) == ID_ME);
+
+	if (ED_mesh_pick_face(C, ob, mval, &poly_index, size)) {
 		Scene *scene = CTX_data_scene(C);
 		struct ARegion *ar = CTX_wm_region(C);
 
 		/* derived mesh to find deformed locations */
-		DerivedMesh *dm = mesh_get_derived_final(scene, ob, CD_MASK_BAREMESH);
-		int v_idx_best = -1;
+		DerivedMesh *dm = mesh_get_derived_final(scene, ob, CD_MASK_BAREMESH | CD_MASK_ORIGINDEX);
 
-		if (dm->getVertCo) {
-			/* find the vert closest to 'mval' */
-			const float mval_f[2] = {(float)mval[0],
-			                         (float)mval[1]};
-			MPoly *mp = &me->mpoly[poly_index];
-			int fidx;
-			float len_best = FLT_MAX;
+		int v_idx_best = ORIGINDEX_NONE;
 
-			fidx = mp->totloop - 1;
-			do {
-				float co[3], sco[2], len;
-				const int v_idx = me->mloop[mp->loopstart + fidx].v;
-				dm->getVertCo(dm, v_idx, co);
-				mul_m4_v3(ob->obmat, co);
-				if (ED_view3d_project_float_global(ar, co, sco, V3D_PROJ_TEST_NOP) == V3D_PROJ_RET_OK) {
-					len = len_squared_v2v2(mval_f, sco);
-					if (len < len_best) {
-						len_best = len;
-						v_idx_best = v_idx;
-					}
+		/* find the vert closest to 'mval' */
+		const float mval_f[2] = {UNPACK2(mval)};
+		float len_best = FLT_MAX;
+
+		MPoly *dm_mpoly;
+		MLoop *dm_mloop;
+		unsigned int dm_mpoly_tot;
+		const int *index_mp_to_orig;
+
+		dm_mpoly = dm->getPolyArray(dm);
+		dm_mloop = dm->getLoopArray(dm);
+
+		dm_mpoly_tot = dm->getNumPolys(dm);
+
+		index_mp_to_orig = dm->getPolyDataArray(dm, CD_ORIGINDEX);
+
+		/* tag all verts using this face */
+		if (index_mp_to_orig) {
+			unsigned int i;
+
+			for (i = 0; i < dm_mpoly_tot; i++) {
+				if (index_mp_to_orig[i] == poly_index) {
+					ed_mesh_pick_face_vert__mpoly_find(
+					        ar, mval_f,
+					        dm, &dm_mpoly[i], dm_mloop,
+					        &len_best, &v_idx_best);
 				}
-			} while (fidx--);
+			}
+		}
+		else {
+			if (poly_index < dm_mpoly_tot) {
+				ed_mesh_pick_face_vert__mpoly_find(
+				        ar, mval_f,
+				        dm, &dm_mpoly[poly_index], dm_mloop,
+				        &len_best, &v_idx_best);
+			}
+		}
+
+		/* map 'dm -> me' index if possible */
+		if (v_idx_best != ORIGINDEX_NONE) {
+			const int *index_mv_to_orig;
+
+			index_mv_to_orig = dm->getVertDataArray(dm, CD_ORIGINDEX);
+			if (index_mv_to_orig) {
+				v_idx_best = index_mv_to_orig[v_idx_best];
+			}
 		}
 
 		dm->release(dm);
 
-		if (v_idx_best != -1) {
+		if ((v_idx_best != ORIGINDEX_NONE) && (v_idx_best < me->totvert)) {
 			*index = v_idx_best;
-			return 1;
+			return true;
 		}
 	}
 
-	return 0;
+	return false;
 }
 
 /**
  * Vertex selection in object mode,
  * currently only weight paint uses this.
  *
- * \return boolean TRUE == Found
+ * \return boolean true == Found
  */
-int ED_mesh_pick_vert(bContext *C, Mesh *me, const int mval[2], unsigned int *index, int size)
+typedef struct VertPickData {
+	const MVert *mvert;
+	const float *mval_f;  /* [2] */
+	ARegion *ar;
+
+	/* runtime */
+	float len_best;
+	int v_idx_best;
+} VertPickData;
+
+static void ed_mesh_pick_vert__mapFunc(void *userData, int index, const float co[3],
+                                       const float UNUSED(no_f[3]), const short UNUSED(no_s[3]))
+{
+	VertPickData *data = userData;
+	if ((data->mvert[index].flag & ME_HIDE) == 0) {
+		float sco[2];
+
+		if (ED_view3d_project_float_object(data->ar, co, sco, V3D_PROJ_TEST_CLIP_DEFAULT) == V3D_PROJ_RET_OK) {
+			const float len = len_manhattan_v2v2(data->mval_f, sco);
+			if (len < data->len_best) {
+				data->len_best = len;
+				data->v_idx_best = index;
+			}
+		}
+	}
+}
+bool ED_mesh_pick_vert(bContext *C, Object *ob, const int mval[2], unsigned int *index, int size, bool use_zbuf)
 {
 	ViewContext vc;
+	Mesh *me = ob->data;
+
+	BLI_assert(me && GS(me->id.name) == ID_ME);
 
 	if (!me || me->totvert == 0)
-		return 0;
+		return false;
 
 	view3d_set_viewcontext(C, &vc);
 
-	if (size > 0) {
-		/* sample rect to increase chances of selecting, so that when clicking
-		 * on an face in the backbuf, we can still select a vert */
+	if (use_zbuf) {
+		if (size > 0) {
+			/* sample rect to increase chances of selecting, so that when clicking
+			 * on an face in the backbuf, we can still select a vert */
 
-		int dummy_dist;
-		*index = view3d_sample_backbuf_rect(&vc, mval, size, 1, me->totvert + 1, &dummy_dist, 0, NULL, NULL);
+			float dummy_dist;
+			*index = view3d_sample_backbuf_rect(&vc, mval, size, 1, me->totvert + 1, &dummy_dist, 0, NULL, NULL);
+		}
+		else {
+			/* sample only on the exact position */
+			*index = view3d_sample_backbuf(&vc, mval[0], mval[1]);
+		}
+
+		if ((*index) == 0 || (*index) > (unsigned int)me->totvert)
+			return false;
+
+		(*index)--;
 	}
 	else {
-		/* sample only on the exact position */
-		*index = view3d_sample_backbuf(&vc, mval[0], mval[1]);
+		/* derived mesh to find deformed locations */
+		DerivedMesh *dm = mesh_get_derived_final(vc.scene, ob, CD_MASK_BAREMESH);
+		ARegion *ar = vc.ar;
+		RegionView3D *rv3d = ar->regiondata;
+
+		/* find the vert closest to 'mval' */
+		const float mval_f[2] = {(float)mval[0],
+		                         (float)mval[1]};
+
+		VertPickData data = {NULL};
+
+		ED_view3d_init_mats_rv3d(ob, rv3d);
+
+		if (dm == NULL) {
+			return false;
+		}
+
+		/* setup data */
+		data.mvert = me->mvert;
+		data.ar = ar;
+		data.mval_f = mval_f;
+		data.len_best = FLT_MAX;
+		data.v_idx_best = -1;
+
+		dm->foreachMappedVert(dm, ed_mesh_pick_vert__mapFunc, &data, DM_FOREACH_NOP);
+
+		dm->release(dm);
+
+		if (data.v_idx_best == -1) {
+			return false;
+		}
+
+		*index = data.v_idx_best;
 	}
 
-	if ((*index) <= 0 || (*index) > (unsigned int)me->totvert)
-		return 0;
+	return true;
+}
 
-	(*index)--;
 
-	return 1;
+MDeformVert *ED_mesh_active_dvert_get_em(Object *ob, BMVert **r_eve)
+{
+	if (ob->mode & OB_MODE_EDIT && ob->type == OB_MESH && ob->defbase.first) {
+		Mesh *me = ob->data;
+		BMesh *bm = me->edit_btmesh->bm;
+		const int cd_dvert_offset = CustomData_get_offset(&bm->vdata, CD_MDEFORMVERT);
+
+		if (cd_dvert_offset != -1) {
+			BMVert *eve = BM_mesh_active_vert_get(bm);
+
+			if (eve) {
+				if (r_eve) *r_eve = eve;
+				return BM_ELEM_CD_GET_VOID_P(eve, cd_dvert_offset);
+			}
+		}
+	}
+
+	if (r_eve) *r_eve = NULL;
+	return NULL;
+}
+
+MDeformVert *ED_mesh_active_dvert_get_ob(Object *ob, int *r_index)
+{
+	Mesh *me = ob->data;
+	int index = BKE_mesh_mselect_active_get(me, ME_VSEL);
+	if (r_index) *r_index = index;
+	if (index == -1 || me->dvert == NULL) {
+		return NULL;
+	}
+	else {
+		return me->dvert + index;
+	}
+}
+
+MDeformVert *ED_mesh_active_dvert_get_only(Object *ob)
+{
+	if (ob->type == OB_MESH) {
+		if (ob->mode & OB_MODE_EDIT) {
+			return ED_mesh_active_dvert_get_em(ob, NULL);
+		}
+		else {
+			return ED_mesh_active_dvert_get_ob(ob, NULL);
+		}
+	}
+	else {
+		return NULL;
+	}
 }
