@@ -63,14 +63,17 @@
 #include "BKE_lattice.h"
 #include "BKE_main.h"
 #include "BKE_mesh.h"
+#include "BKE_mesh_mapping.h"
 #include "BKE_modifier.h"
 #include "BKE_multires.h"
 #include "BKE_report.h"
 #include "BKE_object.h"
+#include "BKE_object_deform.h"
 #include "BKE_ocean.h"
+#include "BKE_paint.h"
 #include "BKE_particle.h"
 #include "BKE_softbody.h"
-#include "BKE_tessmesh.h"
+#include "BKE_editmesh.h"
 
 #include "RNA_access.h"
 #include "RNA_define.h"
@@ -79,7 +82,6 @@
 #include "ED_armature.h"
 #include "ED_object.h"
 #include "ED_screen.h"
-#include "ED_sculpt.h"
 #include "ED_mesh.h"
 
 #include "WM_api.h"
@@ -87,7 +89,6 @@
 
 #include "object_intern.h"
 
-static void modifier_skin_customdata_ensure(struct Object *ob);
 static void modifier_skin_customdata_delete(struct Object *ob);
 
 /******************************** API ****************************/
@@ -98,14 +99,14 @@ ModifierData *ED_object_modifier_add(ReportList *reports, Main *bmain, Scene *sc
 	ModifierTypeInfo *mti = modifierType_getInfo(type);
 	
 	/* only geometry objects should be able to get modifiers [#25291] */
-	if (!ELEM5(ob->type, OB_MESH, OB_CURVE, OB_SURF, OB_FONT, OB_LATTICE)) {
-		BKE_reportf(reports, RPT_WARNING, "Modifiers cannot be added to Object '%s'", ob->id.name + 2);
+	if (!ELEM(ob->type, OB_MESH, OB_CURVE, OB_SURF, OB_FONT, OB_LATTICE)) {
+		BKE_reportf(reports, RPT_WARNING, "Modifiers cannot be added to object '%s'", ob->id.name + 2);
 		return NULL;
 	}
 	
 	if (mti->flags & eModifierTypeFlag_Single) {
 		if (modifiers_findByType(ob, type)) {
-			BKE_report(reports, RPT_WARNING, "Only one modifier of this type allowed");
+			BKE_report(reports, RPT_WARNING, "Only one modifier of this type is allowed");
 			return NULL;
 		}
 	}
@@ -151,22 +152,22 @@ ModifierData *ED_object_modifier_add(ReportList *reports, Main *bmain, Scene *sc
 				ob->pd = object_add_collision_fields(0);
 			
 			ob->pd->deflect = 1;
-			DAG_scene_sort(bmain, scene);
+			DAG_relations_tag_update(bmain);
 		}
 		else if (type == eModifierType_Surface)
-			DAG_scene_sort(bmain, scene);
+			DAG_relations_tag_update(bmain);
 		else if (type == eModifierType_Multires) {
 			/* set totlvl from existing MDISPS layer if object already had it */
 			multiresModifier_set_levels_from_disps((MultiresModifierData *)new_md, ob);
 
 			if (ob->mode & OB_MODE_SCULPT) {
 				/* ensure that grid paint mask layer is created */
-				ED_sculpt_mask_layers_ensure(ob, (MultiresModifierData *)new_md);
+				BKE_sculpt_mask_layers_ensure(ob, (MultiresModifierData *)new_md);
 			}
 		}
 		else if (type == eModifierType_Skin) {
 			/* ensure skin-node customdata exists */
-			modifier_skin_customdata_ensure(ob);
+			BKE_mesh_ensure_skin_customdata(ob->data);
 		}
 	}
 
@@ -175,32 +176,32 @@ ModifierData *ED_object_modifier_add(ReportList *reports, Main *bmain, Scene *sc
 	return new_md;
 }
 
-/* Return TRUE if the object has a modifier of type 'type' other than
- * the modifier pointed to be 'exclude', otherwise returns FALSE. */
-static int object_has_modifier(const Object *ob, const ModifierData *exclude,
-                               ModifierType type)
+/* Return true if the object has a modifier of type 'type' other than
+ * the modifier pointed to be 'exclude', otherwise returns false. */
+static bool object_has_modifier(const Object *ob, const ModifierData *exclude,
+                                ModifierType type)
 {
 	ModifierData *md;
 
 	for (md = ob->modifiers.first; md; md = md->next) {
 		if ((md != exclude) && (md->type == type))
-			return TRUE;
+			return true;
 	}
 
-	return FALSE;
+	return false;
 }
 
 /* If the object data of 'orig_ob' has other users, run 'callback' on
  * each of them.
  *
- * If include_orig is TRUE, the callback will run on 'orig_ob' too.
+ * If include_orig is true, the callback will run on 'orig_ob' too.
  * 
- * If the callback ever returns TRUE, iteration will stop and the
- * function value will be TRUE. Otherwise the function returns FALSE.
+ * If the callback ever returns true, iteration will stop and the
+ * function value will be true. Otherwise the function returns false.
  */
-int ED_object_iter_other(Main *bmain, Object *orig_ob, int include_orig,
-                         int (*callback)(Object *ob, void *callback_data),
-                         void *callback_data)
+bool ED_object_iter_other(Main *bmain, Object *orig_ob, const bool include_orig,
+                          bool (*callback)(Object *ob, void *callback_data),
+                          void *callback_data)
 {
 	ID *ob_data_id = orig_ob->data;
 	int users = ob_data_id->us;
@@ -220,7 +221,7 @@ int ED_object_iter_other(Main *bmain, Object *orig_ob, int include_orig,
 			    (ob->data == orig_ob->data))
 			{
 				if (callback(ob, callback_data))
-					return TRUE;
+					return true;
 
 				totfound++;
 			}
@@ -230,10 +231,10 @@ int ED_object_iter_other(Main *bmain, Object *orig_ob, int include_orig,
 		return callback(orig_ob, callback_data);
 	}
 
-	return FALSE;
+	return false;
 }
 
-static int object_has_modifier_cb(Object *ob, void *data)
+static bool object_has_modifier_cb(Object *ob, void *data)
 {
 	ModifierType type = *((ModifierType *)data);
 
@@ -243,7 +244,7 @@ static int object_has_modifier_cb(Object *ob, void *data)
 /* Use with ED_object_iter_other(). Sets the total number of levels
  * for any multires modifiers on the object to the int pointed to by
  * callback_data. */
-int ED_object_multires_update_totlevels_cb(Object *ob, void *totlevel_v)
+bool ED_object_multires_update_totlevels_cb(Object *ob, void *totlevel_v)
 {
 	ModifierData *md;
 	int totlevel = *((int *)totlevel_v);
@@ -254,21 +255,21 @@ int ED_object_multires_update_totlevels_cb(Object *ob, void *totlevel_v)
 			DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
 		}
 	}
-	return FALSE;
+	return false;
 }
 
-/* Return TRUE if no modifier of type 'type' other than 'exclude' */
-static int object_modifier_safe_to_delete(Main *bmain, Object *ob,
-                                          ModifierData *exclude,
-                                          ModifierType type)
+/* Return true if no modifier of type 'type' other than 'exclude' */
+static bool object_modifier_safe_to_delete(Main *bmain, Object *ob,
+                                           ModifierData *exclude,
+                                           ModifierType type)
 {
 	return (!object_has_modifier(ob, exclude, type) &&
-	        !ED_object_iter_other(bmain, ob, FALSE,
+	        !ED_object_iter_other(bmain, ob, false,
 	                              object_has_modifier_cb, &type));
 }
 
-static int object_modifier_remove(Main *bmain, Object *ob, ModifierData *md,
-                                  int *sort_depsgraph)
+static bool object_modifier_remove(Main *bmain, Object *ob, ModifierData *md,
+                                   bool *r_sort_depsgraph)
 {
 	/* It seems on rapid delete it is possible to
 	 * get called twice on same modifier, so make
@@ -296,16 +297,10 @@ static int object_modifier_remove(Main *bmain, Object *ob, ModifierData *md,
 		if (ob->pd)
 			ob->pd->deflect = 0;
 
-		*sort_depsgraph = 1;
+		*r_sort_depsgraph = true;
 	}
 	else if (md->type == eModifierType_Surface) {
-		if (ob->pd && ob->pd->shape == PFIELD_SHAPE_SURFACE)
-			ob->pd->shape = PFIELD_SHAPE_PLANE;
-
-		*sort_depsgraph = 1;
-	}
-	else if (md->type == eModifierType_Smoke) {
-		ob->dt = OB_TEXTURE;
+		*r_sort_depsgraph = true;
 	}
 	else if (md->type == eModifierType_Multires) {
 		/* Delete MDisps layer if not used by another multires modifier */
@@ -319,7 +314,7 @@ static int object_modifier_remove(Main *bmain, Object *ob, ModifierData *md,
 	}
 
 	if (ELEM(md->type, eModifierType_Softbody, eModifierType_Cloth) &&
-	    ob->particlesystem.first == NULL)
+	    BLI_listbase_is_empty(&ob->particlesystem))
 	{
 		ob->mode &= ~OB_MODE_PARTICLE_EDIT;
 	}
@@ -330,10 +325,10 @@ static int object_modifier_remove(Main *bmain, Object *ob, ModifierData *md,
 	return 1;
 }
 
-int ED_object_modifier_remove(ReportList *reports, Main *bmain, Scene *scene, Object *ob, ModifierData *md)
+bool ED_object_modifier_remove(ReportList *reports, Main *bmain, Object *ob, ModifierData *md)
 {
-	int sort_depsgraph = 0;
-	int ok;
+	bool sort_depsgraph = false;
+	bool ok;
 
 	ok = object_modifier_remove(bmain, ob, md, &sort_depsgraph);
 
@@ -343,18 +338,15 @@ int ED_object_modifier_remove(ReportList *reports, Main *bmain, Scene *scene, Ob
 	}
 
 	DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
-
-	/* sorting has to be done after the update so that dynamic systems can react properly */
-	if (sort_depsgraph)
-		DAG_scene_sort(bmain, scene);
+	DAG_relations_tag_update(bmain);
 
 	return 1;
 }
 
-void ED_object_modifier_clear(Main *bmain, Scene *scene, Object *ob)
+void ED_object_modifier_clear(Main *bmain, Object *ob)
 {
 	ModifierData *md = ob->modifiers.first;
-	int sort_depsgraph = 0;
+	bool sort_depsgraph = false;
 
 	if (!md)
 		return;
@@ -370,10 +362,7 @@ void ED_object_modifier_clear(Main *bmain, Scene *scene, Object *ob)
 	}
 
 	DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
-
-	/* sorting has to be done after the update so that dynamic systems can react properly */
-	if (sort_depsgraph)
-		DAG_scene_sort(bmain, scene);
+	DAG_relations_tag_update(bmain);
 }
 
 int ED_object_modifier_move_up(ReportList *reports, Object *ob, ModifierData *md)
@@ -391,7 +380,7 @@ int ED_object_modifier_move_up(ReportList *reports, Object *ob, ModifierData *md
 		}
 
 		BLI_remlink(&ob->modifiers, md);
-		BLI_insertlink(&ob->modifiers, md->prev->prev, md);
+		BLI_insertlinkbefore(&ob->modifiers, md->prev, md);
 	}
 
 	return 1;
@@ -412,7 +401,7 @@ int ED_object_modifier_move_down(ReportList *reports, Object *ob, ModifierData *
 		}
 
 		BLI_remlink(&ob->modifiers, md);
-		BLI_insertlink(&ob->modifiers, md->next, md);
+		BLI_insertlinkafter(&ob->modifiers, md->next, md);
 	}
 
 	return 1;
@@ -470,7 +459,7 @@ int ED_object_modifier_convert(ReportList *UNUSED(reports), Main *bmain, Scene *
 	if (totvert == 0) return 0;
 
 	/* add new mesh */
-	obn = BKE_object_add(scene, OB_MESH);
+	obn = BKE_object_add(bmain, scene, OB_MESH);
 	me = obn->data;
 	
 	me->totvert = totvert;
@@ -522,7 +511,7 @@ int ED_object_modifier_convert(ReportList *UNUSED(reports), Main *bmain, Scene *
 		}
 	}
 
-	DAG_scene_sort(bmain, scene);
+	DAG_relations_tag_update(bmain);
 
 	return 1;
 }
@@ -555,8 +544,8 @@ static int modifier_apply_shape(ReportList *reports, Scene *scene, Object *ob, M
 		Key *key = me->key;
 		KeyBlock *kb;
 		
-		if (!modifier_sameTopology(md) || mti->type == eModifierTypeType_NonGeometrical) {
-			BKE_report(reports, RPT_ERROR, "Only deforming modifiers can be applied to Shapes");
+		if (!modifier_isSameTopology(md) || mti->type == eModifierTypeType_NonGeometrical) {
+			BKE_report(reports, RPT_ERROR, "Only deforming modifiers can be applied to shapes");
 			return 0;
 		}
 		
@@ -572,7 +561,7 @@ static int modifier_apply_shape(ReportList *reports, Scene *scene, Object *ob, M
 			/* if that was the first key block added, then it was the basis.
 			 * Initialize it with the mesh, and add another for the modifier */
 			kb = BKE_keyblock_add(key, NULL);
-			BKE_key_convert_from_mesh(me, kb);
+			BKE_keyblock_convert_from_mesh(me, kb);
 		}
 
 		kb = BKE_keyblock_add(key, md->name);
@@ -604,7 +593,7 @@ static int modifier_apply_obdata(ReportList *reports, Scene *scene, Object *ob,
 		MultiresModifierData *mmd = find_multires_modifier_before(scene, md);
 
 		if (me->key && mti->type != eModifierTypeType_NonGeometrical) {
-			BKE_report(reports, RPT_ERROR, "Modifier cannot be applied to Mesh with Shape Keys");
+			BKE_report(reports, RPT_ERROR, "Modifier cannot be applied to a mesh with shape keys");
 			return 0;
 		}
 
@@ -625,7 +614,7 @@ static int modifier_apply_obdata(ReportList *reports, Scene *scene, Object *ob,
 				return 0;
 			}
 
-			DM_to_mesh(dm, me, ob);
+			DM_to_mesh(dm, me, ob, CD_MASK_MESH);
 
 			dm->release(dm);
 
@@ -646,9 +635,9 @@ static int modifier_apply_obdata(ReportList *reports, Scene *scene, Object *ob,
 		cu = ob->data;
 		BKE_report(reports, RPT_INFO, "Applied modifier only changed CV points, not tessellated/bevel vertices");
 
-		vertexCos = BKE_curve_vertexCos_get(cu, &cu->nurb, &numVerts);
+		vertexCos = BKE_curve_nurbs_vertexCos_get(&cu->nurb, &numVerts);
 		mti->deformVerts(md, ob, NULL, vertexCos, numVerts, 0);
-		BK_curve_vertexCos_apply(cu, &cu->nurb, vertexCos);
+		BK_curve_nurbs_vertexCos_apply(&cu->nurb, vertexCos);
 
 		MEM_freeN(vertexCos);
 
@@ -681,13 +670,20 @@ int ED_object_modifier_apply(ReportList *reports, Scene *scene, Object *ob, Modi
 	int prev_mode;
 
 	if (scene->obedit) {
-		BKE_report(reports, RPT_ERROR, "Modifiers cannot be applied in editmode");
+		BKE_report(reports, RPT_ERROR, "Modifiers cannot be applied in edit mode");
 		return 0;
 	}
 	else if (((ID *) ob->data)->us > 1) {
 		BKE_report(reports, RPT_ERROR, "Modifiers cannot be applied to multi-user data");
 		return 0;
 	}
+	else if ((ob->mode & OB_MODE_SCULPT) &&
+	         (find_multires_modifier_before(scene, md)) &&
+	         (modifier_isSameTopology(md) == false))
+	{
+		BKE_report(reports, RPT_ERROR, "Constructive modifier cannot be applied to multi-res data in sculpt mode");
+		return 0;
+	}
 
 	if (md != ob->modifiers.first)
 		BKE_report(reports, RPT_INFO, "Applied modifier was not first, result may not be as expected");
@@ -721,7 +717,7 @@ int ED_object_modifier_copy(ReportList *UNUSED(reports), Object *ob, ModifierDat
 	
 	nmd = modifier_new(md->type);
 	modifier_copyData(md, nmd);
-	BLI_insertlink(&ob->modifiers, md, nmd);
+	BLI_insertlinkafter(&ob->modifiers, md, nmd);
 	modifier_unique_name(&ob->modifiers, nmd);
 
 	return 1;
@@ -744,7 +740,7 @@ static int modifier_add_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static EnumPropertyItem *modifier_add_itemf(bContext *C, PointerRNA *UNUSED(ptr), PropertyRNA *UNUSED(prop), int *free)
+static EnumPropertyItem *modifier_add_itemf(bContext *C, PointerRNA *UNUSED(ptr), PropertyRNA *UNUSED(prop), bool *r_free)
 {	
 	Object *ob = ED_object_active_context(C);
 	EnumPropertyItem *item = NULL, *md_item, *group_item = NULL;
@@ -782,7 +778,7 @@ static EnumPropertyItem *modifier_add_itemf(bContext *C, PointerRNA *UNUSED(ptr)
 	}
 
 	RNA_enum_item_end(&item, &totitem);
-	*free = 1;
+	*r_free = true;
 
 	return item;
 }
@@ -831,7 +827,7 @@ static int edit_modifier_poll(bContext *C)
 
 static void edit_modifier_properties(wmOperatorType *ot)
 {
-	RNA_def_string(ot->srna, "modifier", "", MAX_NAME, "Modifier", "Name of the modifier to edit");
+	RNA_def_string(ot->srna, "modifier", NULL, MAX_NAME, "Modifier", "Name of the modifier to edit");
 }
 
 static int edit_modifier_invoke_properties(bContext *C, wmOperator *op)
@@ -839,18 +835,18 @@ static int edit_modifier_invoke_properties(bContext *C, wmOperator *op)
 	ModifierData *md;
 	
 	if (RNA_struct_property_is_set(op->ptr, "modifier")) {
-		return TRUE;
+		return true;
 	}
 	else {
 		PointerRNA ptr = CTX_data_pointer_get_type(C, "modifier", &RNA_Modifier);
 		if (ptr.data) {
 			md = ptr.data;
 			RNA_string_set(op->ptr, "modifier", md->name);
-			return TRUE;
+			return true;
 		}
 	}
 
-	return FALSE;
+	return false;
 }
 
 static ModifierData *edit_modifier_property_get(wmOperator *op, Object *ob, int type)
@@ -875,9 +871,9 @@ static int modifier_remove_exec(bContext *C, wmOperator *op)
 	Scene *scene = CTX_data_scene(C);
 	Object *ob = ED_object_active_context(C);
 	ModifierData *md = edit_modifier_property_get(op, ob, 0);
-	int mode_orig = ob ? ob->mode : 0;
+	int mode_orig = ob->mode;
 	
-	if (!ob || !md || !ED_object_modifier_remove(op->reports, bmain, scene, ob, md))
+	if (!md || !ED_object_modifier_remove(op->reports, bmain, ob, md))
 		return OPERATOR_CANCELLED;
 
 	WM_event_add_notifier(C, NC_OBJECT | ND_MODIFIER, ob);
@@ -891,7 +887,7 @@ static int modifier_remove_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int modifier_remove_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int modifier_remove_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	if (edit_modifier_invoke_properties(C, op))
 		return modifier_remove_exec(C, op);
@@ -921,7 +917,7 @@ static int modifier_move_up_exec(bContext *C, wmOperator *op)
 	Object *ob = ED_object_active_context(C);
 	ModifierData *md = edit_modifier_property_get(op, ob, 0);
 
-	if (!ob || !md || !ED_object_modifier_move_up(op->reports, ob, md))
+	if (!md || !ED_object_modifier_move_up(op->reports, ob, md))
 		return OPERATOR_CANCELLED;
 
 	DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
@@ -930,7 +926,7 @@ static int modifier_move_up_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int modifier_move_up_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int modifier_move_up_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	if (edit_modifier_invoke_properties(C, op))
 		return modifier_move_up_exec(C, op);
@@ -960,7 +956,7 @@ static int modifier_move_down_exec(bContext *C, wmOperator *op)
 	Object *ob = ED_object_active_context(C);
 	ModifierData *md = edit_modifier_property_get(op, ob, 0);
 
-	if (!ob || !md || !ED_object_modifier_move_down(op->reports, ob, md))
+	if (!md || !ED_object_modifier_move_down(op->reports, ob, md))
 		return OPERATOR_CANCELLED;
 
 	DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
@@ -969,7 +965,7 @@ static int modifier_move_down_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int modifier_move_down_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int modifier_move_down_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	if (edit_modifier_invoke_properties(C, op))
 		return modifier_move_down_exec(C, op);
@@ -1000,8 +996,8 @@ static int modifier_apply_exec(bContext *C, wmOperator *op)
 	Object *ob = ED_object_active_context(C);
 	ModifierData *md = edit_modifier_property_get(op, ob, 0);
 	int apply_as = RNA_enum_get(op->ptr, "apply_as");
-	
-	if (!ob || !md || !ED_object_modifier_apply(op->reports, scene, ob, md, apply_as)) {
+
+	if (!md || !ED_object_modifier_apply(op->reports, scene, ob, md, apply_as)) {
 		return OPERATOR_CANCELLED;
 	}
 
@@ -1011,7 +1007,7 @@ static int modifier_apply_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int modifier_apply_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int modifier_apply_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	if (edit_modifier_invoke_properties(C, op))
 		return modifier_apply_exec(C, op);
@@ -1051,7 +1047,7 @@ static int modifier_convert_exec(bContext *C, wmOperator *op)
 	Object *ob = ED_object_active_context(C);
 	ModifierData *md = edit_modifier_property_get(op, ob, 0);
 	
-	if (!ob || !md || !ED_object_modifier_convert(op->reports, bmain, scene, ob, md))
+	if (!md || !ED_object_modifier_convert(op->reports, bmain, scene, ob, md))
 		return OPERATOR_CANCELLED;
 
 	DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
@@ -1060,7 +1056,7 @@ static int modifier_convert_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int modifier_convert_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int modifier_convert_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	if (edit_modifier_invoke_properties(C, op))
 		return modifier_convert_exec(C, op);
@@ -1090,7 +1086,7 @@ static int modifier_copy_exec(bContext *C, wmOperator *op)
 	Object *ob = ED_object_active_context(C);
 	ModifierData *md = edit_modifier_property_get(op, ob, 0);
 
-	if (!ob || !md || !ED_object_modifier_copy(op->reports, ob, md))
+	if (!md || !ED_object_modifier_copy(op->reports, ob, md))
 		return OPERATOR_CANCELLED;
 
 	DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
@@ -1099,7 +1095,7 @@ static int modifier_copy_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int modifier_copy_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int modifier_copy_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	if (edit_modifier_invoke_properties(C, op))
 		return modifier_copy_exec(C, op);
@@ -1139,7 +1135,7 @@ static int multires_higher_levels_delete_exec(bContext *C, wmOperator *op)
 	
 	multiresModifier_del_levels(mmd, ob, 1);
 
-	ED_object_iter_other(CTX_data_main(C), ob, TRUE,
+	ED_object_iter_other(CTX_data_main(C), ob, true,
 	                     ED_object_multires_update_totlevels_cb,
 	                     &mmd->totlvl);
 	
@@ -1148,7 +1144,7 @@ static int multires_higher_levels_delete_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int multires_higher_levels_delete_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int multires_higher_levels_delete_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	if (edit_modifier_invoke_properties(C, op))
 		return multires_higher_levels_delete_exec(C, op);
@@ -1183,17 +1179,22 @@ static int multires_subdivide_exec(bContext *C, wmOperator *op)
 	
 	multiresModifier_subdivide(mmd, ob, 0, mmd->simple);
 
-	ED_object_iter_other(CTX_data_main(C), ob, TRUE,
+	ED_object_iter_other(CTX_data_main(C), ob, true,
 	                     ED_object_multires_update_totlevels_cb,
 	                     &mmd->totlvl);
 
 	DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
 	WM_event_add_notifier(C, NC_OBJECT | ND_MODIFIER, ob);
+
+	if (ob->mode & OB_MODE_SCULPT) {
+		/* ensure that grid paint mask layer is created */
+		BKE_sculpt_mask_layers_ensure(ob, mmd);
+	}
 	
 	return OPERATOR_FINISHED;
 }
 
-static int multires_subdivide_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int multires_subdivide_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	if (edit_modifier_invoke_properties(C, op))
 		return multires_subdivide_exec(C, op);
@@ -1242,7 +1243,7 @@ static int multires_reshape_exec(bContext *C, wmOperator *op)
 	CTX_DATA_END;
 
 	if (!secondob) {
-		BKE_report(op->reports, RPT_ERROR, "Second selected mesh object require to copy shape from");
+		BKE_report(op->reports, RPT_ERROR, "Second selected mesh object required to copy shape from");
 		return OPERATOR_CANCELLED;
 	}
 
@@ -1257,7 +1258,7 @@ static int multires_reshape_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int multires_reshape_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int multires_reshape_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	if (edit_modifier_invoke_properties(C, op))
 		return multires_reshape_exec(C, op);
@@ -1289,7 +1290,7 @@ static int multires_external_save_exec(bContext *C, wmOperator *op)
 	Object *ob = ED_object_active_context(C);
 	Mesh *me = (ob) ? ob->data : op->customdata;
 	char path[FILE_MAX];
-	int relative = RNA_boolean_get(op->ptr, "relative_path");
+	const bool relative = RNA_boolean_get(op->ptr, "relative_path");
 
 	if (!me)
 		return OPERATOR_CANCELLED;
@@ -1308,7 +1309,7 @@ static int multires_external_save_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int multires_external_save_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int multires_external_save_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	Object *ob = ED_object_active_context(C);
 	MultiresModifierData *mmd;
@@ -1404,7 +1405,7 @@ static int multires_base_apply_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int multires_base_apply_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int multires_base_apply_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	if (edit_modifier_invoke_properties(C, op))
 		return multires_base_apply_exec(C, op);
@@ -1431,39 +1432,6 @@ void OBJECT_OT_multires_base_apply(wmOperatorType *ot)
 
 /************************** skin modifier ***********************/
 
-static void modifier_skin_customdata_ensure(Object *ob)
-{
-	Mesh *me = ob->data;
-	BMesh *bm = me->edit_btmesh ? me->edit_btmesh->bm : NULL;
-	MVertSkin *vs;
-
-	if (bm && !CustomData_has_layer(&bm->vdata, CD_MVERT_SKIN)) {
-		BMVert *v;
-		BMIter iter;
-
-		BM_data_layer_add(bm, &bm->vdata, CD_MVERT_SKIN);
-		
-		/* Mark an arbitrary vertex as root */
-		BM_ITER_MESH (v, &iter, bm, BM_VERTS_OF_MESH) {
-			vs = CustomData_bmesh_get(&bm->vdata, v->head.data,
-			                          CD_MVERT_SKIN);
-			vs->flag |= MVERT_SKIN_ROOT;
-			break;
-		}
-	}
-	else if (!CustomData_has_layer(&me->vdata, CD_MVERT_SKIN)) {
-		vs = CustomData_add_layer(&me->vdata,
-		                          CD_MVERT_SKIN,
-		                          CD_DEFAULT,
-		                          NULL,
-		                          me->totvert);
-
-		/* Mark an arbitrary vertex as root */
-		if (vs)
-			vs->flag |= MVERT_SKIN_ROOT;
-	}
-}
-
 static void modifier_skin_customdata_delete(Object *ob)
 {
 	Mesh *me = ob->data;
@@ -1487,7 +1455,7 @@ static int skin_edit_poll(bContext *C)
 	        edit_modifier_poll_generic(C, &RNA_SkinModifier, (1 << OB_MESH)));
 }
 
-static void skin_root_clear(BMesh *bm, BMVert *bm_vert, GHash *visited)
+static void skin_root_clear(BMesh *bm, BMVert *bm_vert, GSet *visited)
 {
 	BMEdge *bm_edge;
 	BMIter bm_iter;
@@ -1495,14 +1463,14 @@ static void skin_root_clear(BMesh *bm, BMVert *bm_vert, GHash *visited)
 	BM_ITER_ELEM (bm_edge, &bm_iter, bm_vert, BM_EDGES_OF_VERT) {
 		BMVert *v2 = BM_edge_other_vert(bm_edge, bm_vert);
 
-		if (!BLI_ghash_lookup(visited, v2)) {
+		if (!BLI_gset_haskey(visited, v2)) {
 			MVertSkin *vs = CustomData_bmesh_get(&bm->vdata,
 			                                     v2->head.data,
 			                                     CD_MVERT_SKIN);
 
 			/* clear vertex root flag and add to visited set */
 			vs->flag &= ~MVERT_SKIN_ROOT;
-			BLI_ghash_insert(visited, v2, v2);
+			BLI_gset_insert(visited, v2);
 
 			skin_root_clear(bm, v2, visited);
 		}
@@ -1512,19 +1480,19 @@ static void skin_root_clear(BMesh *bm, BMVert *bm_vert, GHash *visited)
 static int skin_root_mark_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	Object *ob = CTX_data_edit_object(C);
-	BMEditMesh *em = BMEdit_FromObject(ob);
+	BMEditMesh *em = BKE_editmesh_from_object(ob);
 	BMesh *bm = em->bm;
 	BMVert *bm_vert;
 	BMIter bm_iter;
-	GHash *visited;
+	GSet *visited;
 
-	visited = BLI_ghash_ptr_new("skin_root_mark_exec visited");
+	visited = BLI_gset_ptr_new(__func__);
 
-	modifier_skin_customdata_ensure(ob);
+	BKE_mesh_ensure_skin_customdata(ob->data);
 
 	BM_ITER_MESH (bm_vert, &bm_iter, bm, BM_VERTS_OF_MESH) {
-		if (!BLI_ghash_lookup(visited, bm_vert) &&
-		    bm_vert->head.hflag & BM_ELEM_SELECT)
+		if (!BLI_gset_haskey(visited, bm_vert) &&
+		    BM_elem_flag_test(bm_vert, BM_ELEM_SELECT))
 		{
 			MVertSkin *vs = CustomData_bmesh_get(&bm->vdata,
 			                                     bm_vert->head.data,
@@ -1532,14 +1500,14 @@ static int skin_root_mark_exec(bContext *C, wmOperator *UNUSED(op))
 
 			/* mark vertex as root and add to visited set */
 			vs->flag |= MVERT_SKIN_ROOT;
-			BLI_ghash_insert(visited, bm_vert, bm_vert);
+			BLI_gset_insert(visited, bm_vert);
 
 			/* clear root flag from all connected vertices (recursively) */
 			skin_root_clear(bm, bm_vert, visited);
 		}
 	}
 
-	BLI_ghash_free(visited, NULL, NULL);
+	BLI_gset_free(visited, NULL);
 
 	DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
 	WM_event_add_notifier(C, NC_OBJECT | ND_MODIFIER, ob);
@@ -1568,7 +1536,7 @@ typedef enum {
 static int skin_loose_mark_clear_exec(bContext *C, wmOperator *op)
 {
 	Object *ob = CTX_data_edit_object(C);
-	BMEditMesh *em = BMEdit_FromObject(ob);
+	BMEditMesh *em = BKE_editmesh_from_object(ob);
 	BMesh *bm = em->bm;
 	BMVert *bm_vert;
 	BMIter bm_iter;
@@ -1579,7 +1547,7 @@ static int skin_loose_mark_clear_exec(bContext *C, wmOperator *op)
 	}
 
 	BM_ITER_MESH (bm_vert, &bm_iter, bm, BM_VERTS_OF_MESH) {
-		if (bm_vert->head.hflag & BM_ELEM_SELECT) {
+		if (BM_elem_flag_test(bm_vert, BM_ELEM_SELECT)) {
 			MVertSkin *vs = CustomData_bmesh_get(&bm->vdata,
 			                                     bm_vert->head.data,
 			                                     CD_MVERT_SKIN);
@@ -1626,7 +1594,7 @@ void OBJECT_OT_skin_loose_mark_clear(wmOperatorType *ot)
 static int skin_radii_equalize_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	Object *ob = CTX_data_edit_object(C);
-	BMEditMesh *em = BMEdit_FromObject(ob);
+	BMEditMesh *em = BKE_editmesh_from_object(ob);
 	BMesh *bm = em->bm;
 	BMVert *bm_vert;
 	BMIter bm_iter;
@@ -1636,7 +1604,7 @@ static int skin_radii_equalize_exec(bContext *C, wmOperator *UNUSED(op))
 	}
 
 	BM_ITER_MESH (bm_vert, &bm_iter, bm, BM_VERTS_OF_MESH) {
-		if (bm_vert->head.hflag & BM_ELEM_SELECT) {
+		if (BM_elem_flag_test(bm_vert, BM_ELEM_SELECT)) {
 			MVertSkin *vs = CustomData_bmesh_get(&bm->vdata,
 			                                     bm_vert->head.data,
 			                                     CD_MVERT_SKIN);
@@ -1668,7 +1636,7 @@ void OBJECT_OT_skin_radii_equalize(wmOperatorType *ot)
 static void skin_armature_bone_create(Object *skin_ob,
                                       MVert *mvert, MEdge *medge,
                                       bArmature *arm,
-                                      BLI_bitmap edges_visited,
+                                      BLI_bitmap *edges_visited,
                                       const MeshElemMap *emap,
                                       EditBone *parent_bone,
                                       int parent_v)
@@ -1683,9 +1651,9 @@ static void skin_armature_bone_create(Object *skin_ob,
 		int v;
 
 		/* ignore edge if already visited */
-		if (BLI_BITMAP_GET(edges_visited, endx))
+		if (BLI_BITMAP_TEST(edges_visited, endx))
 			continue;
-		BLI_BITMAP_SET(edges_visited, endx);
+		BLI_BITMAP_ENABLE(edges_visited, endx);
 
 		v = (e->v1 == parent_v ? e->v2 : e->v1);
 
@@ -1700,7 +1668,7 @@ static void skin_armature_bone_create(Object *skin_ob,
 		BLI_snprintf(bone->name, sizeof(bone->name), "Bone.%.2d", endx);
 
 		/* add bDeformGroup */
-		if ((dg = ED_vgroup_add_name(skin_ob, bone->name))) {
+		if ((dg = BKE_object_defgroup_add_name(skin_ob, bone->name))) {
 			ED_vgroup_vert_add(skin_ob, dg, parent_v, 1, WEIGHT_REPLACE);
 			ED_vgroup_vert_add(skin_ob, dg, v, 1, WEIGHT_REPLACE);
 		}
@@ -1715,10 +1683,9 @@ static void skin_armature_bone_create(Object *skin_ob,
 	}
 }
 
-static Object *modifier_skin_armature_create(struct Scene *scene,
-                                             Object *skin_ob)
+static Object *modifier_skin_armature_create(Main *bmain, Scene *scene, Object *skin_ob)
 {
-	BLI_bitmap edges_visited;
+	BLI_bitmap *edges_visited;
 	DerivedMesh *deform_dm;
 	MVert *mvert;
 	Mesh *me = skin_ob->data;
@@ -1739,7 +1706,7 @@ static Object *modifier_skin_armature_create(struct Scene *scene,
 	                     NULL,
 	                     me->totvert);
 	
-	arm_ob = BKE_object_add(scene, OB_ARMATURE);
+	arm_ob = BKE_object_add(bmain, scene, OB_ARMATURE);
 	BKE_object_transform_copy(arm_ob, skin_ob);
 	arm = arm_ob->data;
 	arm->layer = 1;
@@ -1748,7 +1715,7 @@ static Object *modifier_skin_armature_create(struct Scene *scene,
 	arm->edbo = MEM_callocN(sizeof(ListBase), "edbo armature");
 
 	mvert_skin = CustomData_get_layer(&me->vdata, CD_MVERT_SKIN);
-	create_vert_edge_map(&emap, &emap_mem,
+	BKE_mesh_vert_edge_map_create(&emap, &emap_mem,
 	                     me->medge, me->totvert, me->totedge);
 
 	edges_visited = BLI_BITMAP_NEW(me->totedge, "edge_visited");
@@ -1788,8 +1755,8 @@ static Object *modifier_skin_armature_create(struct Scene *scene,
 	MEM_freeN(emap);
 	MEM_freeN(emap_mem);
 
-	ED_armature_from_edit(arm_ob);
-	ED_armature_edit_free(arm_ob);
+	ED_armature_from_edit(arm);
+	ED_armature_edit_free(arm);
 
 	return arm_ob;
 }
@@ -1809,7 +1776,7 @@ static int skin_armature_create_exec(bContext *C, wmOperator *op)
 	}
 
 	/* create new armature */
-	arm_ob = modifier_skin_armature_create(scene, ob);
+	arm_ob = modifier_skin_armature_create(bmain, scene, ob);
 
 	/* add a modifier to connect the new armature to the mesh */
 	arm_md = (ArmatureModifierData *)modifier_new(eModifierType_Armature);
@@ -1819,7 +1786,7 @@ static int skin_armature_create_exec(bContext *C, wmOperator *op)
 
 		arm_md->object = arm_ob;
 		arm_md->deformflag = ARM_DEF_VGROUP | ARM_DEF_QUATERNION;
-		DAG_scene_sort(bmain, scene);
+		DAG_relations_tag_update(bmain);
 		DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
 	}
 
@@ -1828,7 +1795,7 @@ static int skin_armature_create_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int skin_armature_create_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int skin_armature_create_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	if (edit_modifier_invoke_properties(C, op))
 		return skin_armature_create_exec(C, op);
@@ -1855,7 +1822,7 @@ void OBJECT_OT_skin_armature_create(wmOperatorType *ot)
 
 static int meshdeform_poll(bContext *C)
 {
-	return edit_modifier_poll_generic(C, &RNA_MeshDeformModifier, (1 << OB_MESH));
+	return edit_modifier_poll_generic(C, &RNA_MeshDeformModifier, 0);
 }
 
 static int meshdeform_bind_exec(bContext *C, wmOperator *op)
@@ -1908,9 +1875,9 @@ static int meshdeform_bind_exec(bContext *C, wmOperator *op)
 			BKE_lattice_modifiers_calc(scene, ob);
 		}
 		else if (ob->type == OB_MBALL) {
-			BKE_displist_make_mball(scene, ob);
+			BKE_displist_make_mball(CTX_data_main(C)->eval_ctx, scene, ob);
 		}
-		else if (ELEM3(ob->type, OB_CURVE, OB_SURF, OB_FONT)) {
+		else if (ELEM(ob->type, OB_CURVE, OB_SURF, OB_FONT)) {
 			BKE_displist_make_curveTypes(scene, ob, 0);
 		}
 
@@ -1921,7 +1888,7 @@ static int meshdeform_bind_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int meshdeform_bind_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int meshdeform_bind_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	if (edit_modifier_invoke_properties(C, op))
 		return meshdeform_bind_exec(C, op);
@@ -1969,7 +1936,7 @@ static int explode_refresh_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int explode_refresh_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int explode_refresh_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	if (edit_modifier_invoke_properties(C, op))
 		return explode_refresh_exec(C, op);
@@ -2008,7 +1975,7 @@ static void init_ocean_modifier_bake(struct Ocean *oc, struct OceanModifierData
 	
 	if (!omd || !oc) return; 
 	
-	do_heightfield = TRUE;
+	do_heightfield = true;
 	do_chop = (omd->chop_amount > 0);
 	do_normals = (omd->flag & MOD_OCEAN_GENERATE_NORMALS);
 	do_jacobian = (omd->flag & MOD_OCEAN_GENERATE_FOAM);
@@ -2057,7 +2024,7 @@ static void oceanbake_update(void *customdata, float progress, int *cancel)
 	if (oceanbake_breakjob(oj))
 		*cancel = 1;
 	
-	*(oj->do_update) = TRUE;
+	*(oj->do_update) = true;
 	*(oj->progress) = progress;
 }
 
@@ -2069,11 +2036,11 @@ static void oceanbake_startjob(void *customdata, short *stop, short *do_update,
 	oj->do_update = do_update;
 	oj->progress = progress;
 	
-	G.is_break = FALSE;   /* XXX shared with render - replace with job 'stop' switch */
+	G.is_break = false;   /* XXX shared with render - replace with job 'stop' switch */
 	
 	BKE_bake_ocean(oj->ocean, oj->och, oceanbake_update, (void *)oj);
 	
-	*do_update = TRUE;
+	*do_update = true;
 	*stop = 0;
 }
 
@@ -2087,7 +2054,7 @@ static void oceanbake_endjob(void *customdata)
 	}
 	
 	oj->omd->oceancache = oj->och;
-	oj->omd->cached = TRUE;
+	oj->omd->cached = true;
 }
 
 static int ocean_bake_exec(bContext *C, wmOperator *op)
@@ -2098,7 +2065,7 @@ static int ocean_bake_exec(bContext *C, wmOperator *op)
 	OceanCache *och;
 	struct Ocean *ocean;
 	int f, cfra, i = 0;
-	int free = RNA_boolean_get(op->ptr, "free");
+	const bool free = RNA_boolean_get(op->ptr, "free");
 	
 	wmJob *wm_job;
 	OceanBakeJob *oj;
@@ -2154,7 +2121,7 @@ static int ocean_bake_exec(bContext *C, wmOperator *op)
 	BKE_bake_ocean(ocean, och);
 	
 	omd->oceancache = och;
-	omd->cached = TRUE;
+	omd->cached = true;
 	
 	scene->r.cfra = cfra;
 	
@@ -2185,7 +2152,7 @@ static int ocean_bake_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int ocean_bake_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int ocean_bake_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	if (edit_modifier_invoke_properties(C, op))
 		return ocean_bake_exec(C, op);
@@ -2208,6 +2175,55 @@ void OBJECT_OT_ocean_bake(wmOperatorType *ot)
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO | OPTYPE_INTERNAL;
 	edit_modifier_properties(ot);
 	
-	RNA_def_boolean(ot->srna, "free", FALSE, "Free", "Free the bake, rather than generating it");
+	RNA_def_boolean(ot->srna, "free", false, "Free", "Free the bake, rather than generating it");
 }
 
+/************************ LaplacianDeform bind operator *********************/
+
+static int laplaciandeform_poll(bContext *C)
+{
+	return edit_modifier_poll_generic(C, &RNA_LaplacianDeformModifier, 0);
+}
+
+static int laplaciandeform_bind_exec(bContext *C, wmOperator *op)
+{
+	Object *ob = ED_object_active_context(C);
+	LaplacianDeformModifierData *lmd = (LaplacianDeformModifierData *)edit_modifier_property_get(op, ob, eModifierType_LaplacianDeform);
+
+	if (!lmd)
+		return OPERATOR_CANCELLED;
+	if (lmd->flag & MOD_LAPLACIANDEFORM_BIND) {
+		lmd->flag &= ~MOD_LAPLACIANDEFORM_BIND;
+	}
+	else {
+		lmd->flag |= MOD_LAPLACIANDEFORM_BIND;
+	}
+	DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
+	WM_event_add_notifier(C, NC_OBJECT | ND_MODIFIER, ob);
+	return OPERATOR_FINISHED;
+}
+
+static int laplaciandeform_bind_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
+{
+	if (edit_modifier_invoke_properties(C, op))
+		return laplaciandeform_bind_exec(C, op);
+	else 
+		return OPERATOR_CANCELLED;
+}
+
+void OBJECT_OT_laplaciandeform_bind(wmOperatorType *ot)
+{
+	/* identifiers */
+	ot->name = "Laplacian Deform Bind";
+	ot->description = "Bind mesh to system in laplacian deform modifier";
+	ot->idname = "OBJECT_OT_laplaciandeform_bind";
+	
+	/* api callbacks */
+	ot->poll = laplaciandeform_poll;
+	ot->invoke = laplaciandeform_bind_invoke;
+	ot->exec = laplaciandeform_bind_exec;
+	
+	/* flags */
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO | OPTYPE_INTERNAL;
+	edit_modifier_properties(ot);
+}
