@@ -57,9 +57,10 @@
 #include "BKE_library.h"
 #include "BKE_main.h"
 #include "BKE_object.h"
+#include "BKE_lattice.h"
 #include "BKE_curve.h"
 
-#include "BLO_sys_types.h" // for intptr_t support
+#include "BLI_sys_types.h" // for intptr_t support
 
 #include "ED_object.h"
 #include "ED_mesh.h"
@@ -74,7 +75,7 @@
 
 /*********************** add shape key ***********************/
 
-static void ED_object_shape_key_add(bContext *C, Scene *scene, Object *ob, int from_mix)
+static void ED_object_shape_key_add(bContext *C, Scene *scene, Object *ob, const bool from_mix)
 {
 	KeyBlock *kb;
 	if ((kb = BKE_object_insert_shape_key(scene, ob, NULL, from_mix))) {
@@ -88,17 +89,34 @@ static void ED_object_shape_key_add(bContext *C, Scene *scene, Object *ob, int f
 
 /*********************** remove shape key ***********************/
 
-static int ED_object_shape_key_remove(bContext *C, Object *ob)
+static bool ED_object_shape_key_remove_all(Main *bmain, Object *ob)
 {
-	Main *bmain = CTX_data_main(C);
-	KeyBlock *kb, *rkb;
 	Key *key;
-	//IpoCurve *icu;
 
 	key = BKE_key_from_object(ob);
 	if (key == NULL)
-		return 0;
-	
+		return false;
+
+	switch (GS(key->from->name)) {
+		case ID_ME: ((Mesh *)key->from)->key    = NULL; break;
+		case ID_CU: ((Curve *)key->from)->key   = NULL; break;
+		case ID_LT: ((Lattice *)key->from)->key = NULL; break;
+	}
+
+	BKE_libblock_free_us(bmain, key);
+
+	return true;
+}
+
+static bool ED_object_shape_key_remove(Main *bmain, Object *ob)
+{
+	KeyBlock *kb, *rkb;
+	Key *key;
+
+	key = BKE_key_from_object(ob);
+	if (key == NULL)
+		return false;
+
 	kb = BLI_findlink(&key->block, ob->shapenr - 1);
 
 	if (kb) {
@@ -115,14 +133,14 @@ static int ED_object_shape_key_remove(bContext *C, Object *ob)
 				/* apply new basis key on original data */
 				switch (ob->type) {
 					case OB_MESH:
-						BKE_key_convert_to_mesh(key->refkey, ob->data);
+						BKE_keyblock_convert_to_mesh(key->refkey, ob->data);
 						break;
 					case OB_CURVE:
 					case OB_SURF:
-						BKE_key_convert_to_curve(key->refkey, ob->data, BKE_curve_nurbs_get(ob->data));
+						BKE_keyblock_convert_to_curve(key->refkey, ob->data, BKE_curve_nurbs_get(ob->data));
 						break;
 					case OB_LATTICE:
-						BKE_key_convert_to_lattice(key->refkey, ob->data);
+						BKE_keyblock_convert_to_lattice(key->refkey, ob->data);
 						break;
 				}
 			}
@@ -137,23 +155,26 @@ static int ED_object_shape_key_remove(bContext *C, Object *ob)
 	}
 	
 	if (key->totkey == 0) {
-		if (GS(key->from->name) == ID_ME) ((Mesh *)key->from)->key = NULL;
-		else if (GS(key->from->name) == ID_CU) ((Curve *)key->from)->key = NULL;
-		else if (GS(key->from->name) == ID_LT) ((Lattice *)key->from)->key = NULL;
+		switch (GS(key->from->name)) {
+			case ID_ME: ((Mesh *)key->from)->key    = NULL; break;
+			case ID_CU: ((Curve *)key->from)->key   = NULL; break;
+			case ID_LT: ((Lattice *)key->from)->key = NULL; break;
+		}
 
-		BKE_libblock_free_us(&(bmain->key), key);
+		BKE_libblock_free_us(bmain, key);
 	}
-	
-	DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
-	WM_event_add_notifier(C, NC_OBJECT | ND_DRAW, ob);
 
-	return 1;
+	return true;
 }
 
-static int object_shape_key_mirror(bContext *C, Object *ob)
+static bool object_shape_key_mirror(bContext *C, Object *ob,
+                                    int *r_totmirr, int *r_totfail, bool use_topology)
 {
 	KeyBlock *kb;
 	Key *key;
+	int totmirr = 0, totfail = 0;
+
+	*r_totmirr = *r_totfail = 0;
 
 	key = BKE_key_from_object(ob);
 	if (key == NULL)
@@ -162,24 +183,25 @@ static int object_shape_key_mirror(bContext *C, Object *ob)
 	kb = BLI_findlink(&key->block, ob->shapenr - 1);
 
 	if (kb) {
-		int i1, i2;
-		float *fp1, *fp2;
-		float tvec[3];
 		char *tag_elem = MEM_callocN(sizeof(char) * kb->totelem, "shape_key_mirror");
 
 
 		if (ob->type == OB_MESH) {
 			Mesh *me = ob->data;
 			MVert *mv;
+			int i1, i2;
+			float *fp1, *fp2;
+			float tvec[3];
 
-			mesh_octree_table(ob, NULL, NULL, 's');
+			ED_mesh_mirror_spatial_table(ob, NULL, NULL, 's');
 
 			for (i1 = 0, mv = me->mvert; i1 < me->totvert; i1++, mv++) {
-				i2 = mesh_get_x_mirror_vert(ob, i1);
+				i2 = mesh_get_x_mirror_vert(ob, i1, use_topology);
 				if (i2 == i1) {
 					fp1 = ((float *)kb->data) + i1 * 3;
 					fp1[0] = -fp1[0];
 					tag_elem[i1] = 1;
+					totmirr++;
 				}
 				else if (i2 != -1) {
 					if (tag_elem[i1] == 0 && tag_elem[i2] == 0) {
@@ -193,12 +215,16 @@ static int object_shape_key_mirror(bContext *C, Object *ob)
 						/* flip x axis */
 						fp1[0] = -fp1[0];
 						fp2[0] = -fp2[0];
+						totmirr++;
 					}
 					tag_elem[i1] = tag_elem[i2] = 1;
 				}
+				else {
+					totfail++;
+				}
 			}
 
-			mesh_octree_table(ob, NULL, NULL, 'e');
+			ED_mesh_mirror_spatial_table(ob, NULL, NULL, 'e');
 		}
 		else if (ob->type == OB_LATTICE) {
 			Lattice *lt = ob->data;
@@ -211,7 +237,7 @@ static int object_shape_key_mirror(bContext *C, Object *ob)
 			/* currently editmode isn't supported by mesh so
 			 * ignore here for now too */
 
-			/* if (lt->editlatt) lt= lt->editlatt->latt; */
+			/* if (lt->editlatt) lt = lt->editlatt->latt; */
 
 			for (w = 0; w < lt->pntsw; w++) {
 				for (v = 0; v < lt->pntsv; v++) {
@@ -219,13 +245,14 @@ static int object_shape_key_mirror(bContext *C, Object *ob)
 						int u_inv = (lt->pntsu - 1) - u;
 						float tvec[3];
 						if (u == u_inv) {
-							i1 = LT_INDEX(lt, u, v, w);
+							i1 = BKE_lattice_index_from_uvw(lt, u, v, w);
 							fp1 = ((float *)kb->data) + i1 * 3;
 							fp1[0] = -fp1[0];
+							totmirr++;
 						}
 						else {
-							i1 = LT_INDEX(lt, u, v, w);
-							i2 = LT_INDEX(lt, u_inv, v, w);
+							i1 = BKE_lattice_index_from_uvw(lt, u, v, w);
+							i2 = BKE_lattice_index_from_uvw(lt, u_inv, v, w);
 
 							fp1 = ((float *)kb->data) + i1 * 3;
 							fp2 = ((float *)kb->data) + i2 * 3;
@@ -235,6 +262,7 @@ static int object_shape_key_mirror(bContext *C, Object *ob)
 							copy_v3_v3(fp2, tvec);
 							fp1[0] = -fp1[0];
 							fp2[0] = -fp2[0];
+							totmirr++;
 						}
 					}
 				}
@@ -244,6 +272,9 @@ static int object_shape_key_mirror(bContext *C, Object *ob)
 		MEM_freeN(tag_elem);
 	}
 	
+	*r_totmirr = totmirr;
+	*r_totfail = totfail;
+
 	DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
 	WM_event_add_notifier(C, NC_OBJECT | ND_DRAW, ob);
 
@@ -259,6 +290,27 @@ static int shape_key_mode_poll(bContext *C)
 	return (ob && !ob->id.lib && data && !data->lib && ob->mode != OB_MODE_EDIT);
 }
 
+static int shape_key_mode_exists_poll(bContext *C)
+{
+	Object *ob = ED_object_context(C);
+	ID *data = (ob) ? ob->data : NULL;
+
+	/* same as shape_key_mode_poll */
+	return (ob && !ob->id.lib && data && !data->lib && ob->mode != OB_MODE_EDIT) &&
+	       /* check a keyblock exists */
+	       (BKE_keyblock_from_object(ob) != NULL);
+}
+
+static int shape_key_move_poll(bContext *C)
+{
+	/* Same as shape_key_mode_exists_poll above, but ensure we have at least two shapes! */
+	Object *ob = ED_object_context(C);
+	ID *data = (ob) ? ob->data : NULL;
+	Key *key = BKE_key_from_object(ob);
+
+	return (ob && !ob->id.lib && data && !data->lib && ob->mode != OB_MODE_EDIT && key && key->totkey > 1);
+}
+
 static int shape_key_poll(bContext *C)
 {
 	Object *ob = ED_object_context(C);
@@ -270,7 +322,7 @@ static int shape_key_add_exec(bContext *C, wmOperator *op)
 {
 	Scene *scene = CTX_data_scene(C);
 	Object *ob = ED_object_context(C);
-	int from_mix = RNA_boolean_get(op->ptr, "from_mix");
+	const bool from_mix = RNA_boolean_get(op->ptr, "from_mix");
 
 	ED_object_shape_key_add(C, scene, ob, from_mix);
 
@@ -292,17 +344,31 @@ void OBJECT_OT_shape_key_add(wmOperatorType *ot)
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 
 	/* properties */
-	RNA_def_boolean(ot->srna, "from_mix", 1, "From Mix", "Create the new shape key from the existing mix of keys");
+	RNA_def_boolean(ot->srna, "from_mix", true, "From Mix", "Create the new shape key from the existing mix of keys");
 }
 
-static int shape_key_remove_exec(bContext *C, wmOperator *UNUSED(op))
+static int shape_key_remove_exec(bContext *C, wmOperator *op)
 {
+	Main *bmain = CTX_data_main(C);
 	Object *ob = ED_object_context(C);
+	bool changed = false;
 
-	if (!ED_object_shape_key_remove(C, ob))
+	if (RNA_boolean_get(op->ptr, "all")) {
+		changed = ED_object_shape_key_remove_all(bmain, ob);
+	}
+	else {
+		changed = ED_object_shape_key_remove(bmain, ob);
+	}
+
+	if (changed) {
+		DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
+		WM_event_add_notifier(C, NC_OBJECT | ND_DRAW, ob);
+
+		return OPERATOR_FINISHED;
+	}
+	else {
 		return OPERATOR_CANCELLED;
-	
-	return OPERATOR_FINISHED;
+	}
 }
 
 void OBJECT_OT_shape_key_remove(wmOperatorType *ot)
@@ -313,11 +379,14 @@ void OBJECT_OT_shape_key_remove(wmOperatorType *ot)
 	ot->description = "Remove shape key from the object";
 	
 	/* api callbacks */
-	ot->poll = shape_key_mode_poll;
+	ot->poll = shape_key_mode_exists_poll;
 	ot->exec = shape_key_remove_exec;
 
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
+
+	/* properties */
+	RNA_def_boolean(ot->srna, "all", 0, "All", "Remove all shape keys");
 }
 
 static int shape_key_clear_exec(bContext *C, wmOperator *UNUSED(op))
@@ -388,13 +457,17 @@ void OBJECT_OT_shape_key_retime(wmOperatorType *ot)
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 }
 
-static int shape_key_mirror_exec(bContext *C, wmOperator *UNUSED(op))
+static int shape_key_mirror_exec(bContext *C, wmOperator *op)
 {
 	Object *ob = ED_object_context(C);
+	int totmirr = 0, totfail = 0;
+	bool use_topology = RNA_boolean_get(op->ptr, "use_topology");
 
-	if (!object_shape_key_mirror(C, ob))
+	if (!object_shape_key_mirror(C, ob, &totmirr, &totfail, use_topology))
 		return OPERATOR_CANCELLED;
 
+	ED_mesh_report_mirror(op, totmirr, totfail);
+
 	return OPERATOR_FINISHED;
 }
 
@@ -411,51 +484,47 @@ void OBJECT_OT_shape_key_mirror(wmOperatorType *ot)
 
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
+
+	/* properties */
+	RNA_def_boolean(ot->srna, "use_topology", 0, "Topology Mirror",
+	                "Use topology based mirroring (for when both sides of mesh have matching, unique topology)");
 }
 
 
+enum {
+	KB_MOVE_TOP = -2,
+	KB_MOVE_UP = -1,
+	KB_MOVE_DOWN = 1,
+	KB_MOVE_BOTTOM = 2,
+};
+
 static int shape_key_move_exec(bContext *C, wmOperator *op)
 {
 	Object *ob = ED_object_context(C);
 
-	int type = RNA_enum_get(op->ptr, "type");
 	Key *key = BKE_key_from_object(ob);
+	const int type = RNA_enum_get(op->ptr, "type");
+	const int totkey = key->totkey;
+	const int act_index = ob->shapenr - 1;
+	int new_index;
 
-	if (key) {
-		KeyBlock *kb, *kb_other;
-		int shapenr_act = ob->shapenr - 1;
-		int shapenr_swap = shapenr_act + type;
-		kb = BLI_findlink(&key->block, shapenr_act);
+	switch (type) {
+		case KB_MOVE_TOP:
+			/* Replace the ref key only if we're at the top already (only for relative keys) */
+			new_index = (ELEM(act_index, 0, 1) || key->type == KEY_NORMAL) ? 0 : 1;
+			break;
+		case KB_MOVE_BOTTOM:
+			new_index = totkey - 1;
+			break;
+		case KB_MOVE_UP:
+		case KB_MOVE_DOWN:
+		default:
+			new_index = (totkey + act_index + type) % totkey;
+			break;
+	}
 
-		if ((type == -1 && kb->prev == NULL) || (type == 1 && kb->next == NULL)) {
-			return OPERATOR_CANCELLED;
-		}
-
-		for (kb_other = key->block.first; kb_other; kb_other = kb_other->next) {
-			if (kb_other->relative == shapenr_act) {
-				kb_other->relative += type;
-			}
-			else if (kb_other->relative == shapenr_swap) {
-				kb_other->relative -= type;
-			}
-		}
-
-		if (type == -1) {
-			/* move back */
-			kb_other = kb->prev;
-			BLI_remlink(&key->block, kb);
-			BLI_insertlinkbefore(&key->block, kb_other, kb);
-			ob->shapenr--;
-		}
-		else {
-			/* move next */
-			kb_other = kb->next;
-			BLI_remlink(&key->block, kb);
-			BLI_insertlinkafter(&key->block, kb_other, kb);
-			ob->shapenr++;
-		}
-
-		SWAP(float, kb_other->pos, kb->pos); /* for absolute shape keys */
+	if (!BKE_keyblock_move(ob, act_index, new_index)) {
+		return OPERATOR_CANCELLED;
 	}
 
 	DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
@@ -467,9 +536,11 @@ static int shape_key_move_exec(bContext *C, wmOperator *op)
 void OBJECT_OT_shape_key_move(wmOperatorType *ot)
 {
 	static EnumPropertyItem slot_move[] = {
-		{-1, "UP", 0, "Up", ""},
-		{1, "DOWN", 0, "Down", ""},
-		{0, NULL, 0, NULL, NULL}
+		{KB_MOVE_TOP, "TOP", 0, "Top", "Top of the list"},
+		{KB_MOVE_UP, "UP", 0, "Up", ""},
+		{KB_MOVE_DOWN, "DOWN", 0, "Down", ""},
+		{KB_MOVE_BOTTOM, "BOTTOM", 0, "Bottom", "Bottom of the list"},
+		{ 0, NULL, 0, NULL, NULL }
 	};
 
 	/* identifiers */
@@ -478,7 +549,7 @@ void OBJECT_OT_shape_key_move(wmOperatorType *ot)
 	ot->description = "Move the active shape key up/down in the list";
 
 	/* api callbacks */
-	ot->poll = shape_key_mode_poll;
+	ot->poll = shape_key_move_poll;
 	ot->exec = shape_key_move_exec;
 
 	/* flags */
