@@ -31,19 +31,16 @@
 #include <string.h>
 #include <stddef.h>
 
-#include <GL/glew.h>
-
 #include "MEM_guardedalloc.h"
 
 #include "BLI_math.h"
+#include "BLI_math_color_blend.h"
 #include "BLI_blenlib.h"
-#include "BLI_dlrbTree.h"
 #include "BLI_utildefines.h"
 #include "BLI_jitter.h"
 
 #include "DNA_scene_types.h"
 #include "DNA_object_types.h"
-#include "DNA_world_types.h"
 
 #include "BKE_context.h"
 #include "BKE_global.h"
@@ -59,7 +56,7 @@
 
 #include "ED_screen.h"
 #include "ED_view3d.h"
-#include "ED_image.h"
+#include "ED_gpencil.h"
 
 #include "RE_pipeline.h"
 #include "IMB_imbuf_types.h"
@@ -69,12 +66,9 @@
 #include "RNA_access.h"
 #include "RNA_define.h"
 
-#include "BIF_gl.h"
-#include "BIF_glutil.h"
-
 #include "GPU_extensions.h"
+#include "GPU_glew.h"
 
-#include "wm_window.h"
 
 #include "render_intern.h"
 
@@ -145,7 +139,9 @@ static void screen_opengl_render_apply(OGLRender *oglrender)
 
 	if (oglrender->is_sequencer) {
 		SeqRenderData context;
-		int chanshown = oglrender->sseq ? oglrender->sseq->chanshown : 0;
+		SpaceSeq *sseq = oglrender->sseq;
+		int chanshown = sseq ? sseq->chanshown : 0;
+		struct bGPdata *gpd = (sseq && (sseq->flag & SEQ_SHOW_GPENCIL)) ? sseq->gpd : NULL;
 
 		context = BKE_sequencer_new_render_data(oglrender->bmain->eval_ctx, oglrender->bmain,
 		                                        scene, oglrender->sizex, oglrender->sizey, 100.0f);
@@ -177,6 +173,35 @@ static void screen_opengl_render_apply(OGLRender *oglrender)
 
 			IMB_freeImBuf(linear_ibuf);
 		}
+
+		if (gpd) {
+			int i;
+			unsigned char *gp_rect;
+
+			GPU_offscreen_bind(oglrender->ofs);
+
+			glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
+			glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
+
+			wmOrtho2(0, sizex, 0, sizey);
+			glTranslatef(sizex / 2, sizey / 2, 0.0f);
+
+			G.f |= G_RENDER_OGL;
+			ED_gpencil_draw_ex(scene, gpd, sizex, sizey, scene->r.cfra, SPACE_SEQ);
+			G.f &= ~G_RENDER_OGL;
+
+			gp_rect = MEM_mallocN(sizex * sizey * sizeof(unsigned char) * 4, "offscreen rect");
+			GPU_offscreen_read_pixels(oglrender->ofs, GL_UNSIGNED_BYTE, gp_rect);
+
+			for (i = 0; i < sizex * sizey * 4; i += 4) {
+				float  col_src[4];
+				rgba_uchar_to_float(col_src, &gp_rect[i]);
+				blend_color_mix_float(&rr->rectf[i], &rr->rectf[i], col_src);
+			}
+			GPU_offscreen_unbind(oglrender->ofs);
+
+			MEM_freeN(gp_rect);
+		}
 	}
 	else if (view_context) {
 		ED_view3d_draw_offscreen_init(scene, v3d);
@@ -212,7 +237,7 @@ static void screen_opengl_render_apply(OGLRender *oglrender)
 			int *accum_buffer = MEM_mallocN(sizex * sizey * sizeof(int) * 4, "accum1");
 			int i, j;
 
-			BLI_jitter_init(jit_ofs[0], scene->r.osa);
+			BLI_jitter_init(jit_ofs, scene->r.osa);
 
 			/* first sample buffer, also initializes 'rv3d->persmat' */
 			ED_view3d_draw_offscreen(scene, v3d, ar, sizex, sizey, NULL, winmat, draw_bgpic, draw_sky);
@@ -247,7 +272,7 @@ static void screen_opengl_render_apply(OGLRender *oglrender)
 		/* shouldnt suddenly give errors mid-render but possible */
 		char err_out[256] = "unknown";
 		ImBuf *ibuf_view = ED_view3d_draw_offscreen_imbuf_simple(scene, scene->camera, oglrender->sizex, oglrender->sizey,
-		                                                         IB_rect, OB_SOLID, FALSE, TRUE,
+		                                                         IB_rect, OB_SOLID, false, true, true,
 		                                                         (draw_sky) ? R_ADDSKY : R_ALPHAPREMUL, err_out);
 		camera = scene->camera;
 
@@ -308,8 +333,9 @@ static void screen_opengl_render_apply(OGLRender *oglrender)
 				IMB_color_to_bw(ibuf);
 			}
 
-			BKE_makepicstring(name, scene->r.pic, oglrender->bmain->name, scene->r.cfra, &scene->r.im_format, scene->r.scemode & R_EXTENSION, FALSE);
-			ok = BKE_imbuf_write_as(ibuf, name, &scene->r.im_format, TRUE); /* no need to stamp here */
+			BKE_makepicstring(name, scene->r.pic, oglrender->bmain->name, scene->r.cfra,
+			                  &scene->r.im_format, (scene->r.scemode & R_EXTENSION) != 0, false);
+			ok = BKE_imbuf_write_as(ibuf, name, &scene->r.im_format, true); /* no need to stamp here */
 			if (ok) printf("OpenGL Render written to '%s'\n", name);
 			else printf("OpenGL Render failed to write '%s'\n", name);
 		}
@@ -345,20 +371,24 @@ static bool screen_opengl_render_init(bContext *C, wmOperator *op)
 		return false;
 	}
 
-	/* ensure we have a 3d view */
-
-	if (!ED_view3d_context_activate(C)) {
-		RNA_boolean_set(op->ptr, "view_context", FALSE);
-		is_view_context = false;
-	}
-
 	/* only one render job at a time */
 	if (WM_jobs_test(wm, scene, WM_JOB_TYPE_RENDER))
 		return false;
-	
-	if (!is_view_context && scene->camera == NULL) {
-		BKE_report(op->reports, RPT_ERROR, "Scene has no camera");
-		return false;
+
+	if (is_sequencer) {
+		is_view_context = false;
+	}
+	else {
+		/* ensure we have a 3d view */
+		if (!ED_view3d_context_activate(C)) {
+			RNA_boolean_set(op->ptr, "view_context", false);
+			is_view_context = false;
+		}
+
+		if (!is_view_context && scene->camera == NULL) {
+			BKE_report(op->reports, RPT_ERROR, "Scene has no camera");
+			return false;
+		}
 	}
 
 	if (!is_animation && is_write_still && BKE_imtype_is_movie(scene->r.im_format.imtype)) {
@@ -511,8 +541,8 @@ static bool screen_opengl_render_anim_step(bContext *C, wmOperator *op)
 	ImBuf *ibuf, *ibuf_save = NULL;
 	void *lock;
 	char name[FILE_MAX];
-	int ok = 0;
-	const short view_context = (oglrender->v3d != NULL);
+	bool ok = false;
+	const bool view_context = (oglrender->v3d != NULL);
 	Object *camera = NULL;
 	bool is_movie;
 
@@ -532,7 +562,8 @@ static bool screen_opengl_render_anim_step(bContext *C, wmOperator *op)
 	is_movie = BKE_imtype_is_movie(scene->r.im_format.imtype);
 
 	if (!is_movie) {
-		BKE_makepicstring(name, scene->r.pic, oglrender->bmain->name, scene->r.cfra, &scene->r.im_format, scene->r.scemode & R_EXTENSION, TRUE);
+		BKE_makepicstring(name, scene->r.pic, oglrender->bmain->name, scene->r.cfra,
+		                  &scene->r.im_format, (scene->r.scemode & R_EXTENSION) != 0, true);
 
 		if ((scene->r.mode & R_NO_OVERWRITE) && BLI_exists(name)) {
 			BKE_reportf(op->reports, RPT_INFO, "Skipping existing frame \"%s\"", name);
@@ -569,7 +600,7 @@ static bool screen_opengl_render_anim_step(bContext *C, wmOperator *op)
 	ibuf = BKE_image_acquire_ibuf(oglrender->ima, &oglrender->iuser, &lock);
 
 	if (ibuf) {
-		int needs_free = FALSE;
+		bool needs_free = false;
 
 		ibuf_save = ibuf;
 
@@ -577,7 +608,7 @@ static bool screen_opengl_render_anim_step(bContext *C, wmOperator *op)
 			ibuf_save = IMB_colormanagement_imbuf_for_write(ibuf, true, true, &scene->view_settings,
 			                                                &scene->display_settings, &scene->r.im_format);
 
-			needs_free = TRUE;
+			needs_free = true;
 		}
 
 		/* color -> grayscale */
