@@ -43,13 +43,12 @@
 #endif   
 #include "MEM_guardedalloc.h"
 
-#include "BLO_readfile.h" 
-
 #include "BLI_math.h"
 #include "BLI_blenlib.h"
-#include "BLI_threads.h"
 #include "BLI_utildefines.h"
 
+#include "BLO_readfile.h"
+
 #include "DNA_world_types.h"
 #include "DNA_camera_types.h"
 #include "DNA_material_types.h"
@@ -57,15 +56,14 @@
 #include "DNA_object_types.h"
 #include "DNA_lamp_types.h"
 #include "DNA_space_types.h"
-#include "DNA_view3d_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_brush_types.h"
 #include "DNA_screen_types.h"
 
+#include "BKE_appdir.h"
 #include "BKE_brush.h"
 #include "BKE_context.h"
 #include "BKE_colortools.h"
-#include "BKE_depsgraph.h"
 #include "BKE_global.h"
 #include "BKE_idprop.h"
 #include "BKE_image.h"
@@ -75,39 +73,35 @@
 #include "BKE_main.h"
 #include "BKE_material.h"
 #include "BKE_node.h"
-#include "BKE_object.h"
 #include "BKE_scene.h"
 #include "BKE_texture.h"
 #include "BKE_world.h"
 
 #include "IMB_imbuf.h"
 #include "IMB_imbuf_types.h"
-#include "IMB_colormanagement.h"
+
 
 #include "BIF_gl.h"
 #include "BIF_glutil.h"
 
-#include "PIL_time.h"
 
 #include "RE_pipeline.h"
-
+#include "RE_engine.h"
 
 #include "WM_api.h"
 #include "WM_types.h"
 
+#include "ED_datafiles.h"
 #include "ED_render.h"
-#include "ED_view3d.h"
 
-#include "UI_interface.h"
 
-#include "render_intern.h"
 
 ImBuf *get_brush_icon(Brush *brush)
 {
 	static const int flags = IB_rect | IB_multilayer | IB_metadata;
 
 	char path[FILE_MAX];
-	char *folder;
+	const char *folder;
 
 	if (!(brush->icon_imbuf)) {
 		if (brush->flag & BRUSH_CUSTOM_ICON) {
@@ -123,7 +117,7 @@ ImBuf *get_brush_icon(Brush *brush)
 
 				// otherwise lets try to find it in other directories
 				if (!(brush->icon_imbuf)) {
-					folder = BLI_get_folder(BLENDER_DATAFILES, "brushicons");
+					folder = BKE_appdir_folder_id(BLENDER_DATAFILES, "brushicons");
 
 					BLI_make_file_string(G.main->name, path, folder, brush->icon_filepath);
 
@@ -166,7 +160,8 @@ typedef struct ShaderPreview {
 	int sizex, sizey;
 	unsigned int *pr_rect;
 	int pr_method;
-	
+
+	Main *pr_main;
 } ShaderPreview;
 
 typedef struct IconPreviewSize {
@@ -184,31 +179,50 @@ typedef struct IconPreview {
 
 /* *************************** Preview for buttons *********************** */
 
-static Main *pr_main = NULL;
+static Main *G_pr_main = NULL;
+static Main *G_pr_main_cycles = NULL;
+
+#ifndef WITH_HEADLESS
+static Main *load_main_from_memory(const void *blend, int blend_size)
+{
+	const int fileflags = G.fileflags;
+	Main *bmain = NULL;
+	BlendFileData *bfd;
+
+	G.fileflags |= G_FILE_NO_UI;
+	bfd = BLO_read_from_memory(blend, blend_size, NULL);
+	if (bfd) {
+		bmain = bfd->main;
+
+		MEM_freeN(bfd);
+	}
+	G.fileflags = fileflags;
+
+	return bmain;
+}
+#endif
 
 void ED_preview_init_dbase(void)
 {
 #ifndef WITH_HEADLESS
-	BlendFileData *bfd;
-	extern int datatoc_preview_blend_size;
-	extern char datatoc_preview_blend[];
-	const int fileflags = G.fileflags;
-	
-	G.fileflags |= G_FILE_NO_UI;
-	bfd = BLO_read_from_memory(datatoc_preview_blend, datatoc_preview_blend_size, NULL);
-	if (bfd) {
-		pr_main = bfd->main;
-		
-		MEM_freeN(bfd);
-	}
-	G.fileflags = fileflags;
+	G_pr_main = load_main_from_memory(datatoc_preview_blend, datatoc_preview_blend_size);
+	G_pr_main_cycles = load_main_from_memory(datatoc_preview_cycles_blend, datatoc_preview_cycles_blend_size);
 #endif
 }
 
+static bool check_engine_supports_textures(Scene *scene)
+{
+	RenderEngineType *type = RE_engines_find(scene->r.engine);
+	return type->flag & RE_USE_TEXTURE_PREVIEW;
+}
+
 void ED_preview_free_dbase(void)
 {
-	if (pr_main)
-		free_main(pr_main);
+	if (G_pr_main)
+		BKE_main_free(G_pr_main);
+
+	if (G_pr_main_cycles)
+		BKE_main_free(G_pr_main_cycles);
 }
 
 static int preview_mat_has_sss(Material *mat, bNodeTree *ntree)
@@ -237,20 +251,27 @@ static int preview_mat_has_sss(Material *mat, bNodeTree *ntree)
 	return 0;
 }
 
+static Scene *preview_get_scene(Main *pr_main)
+{
+	if (pr_main == NULL) return NULL;
+	
+	return pr_main->scene.first;
+}
+
+
 /* call this with a pointer to initialize preview scene */
 /* call this with NULL to restore assigned ID pointers in preview scene */
 static Scene *preview_prepare_scene(Scene *scene, ID *id, int id_type, ShaderPreview *sp)
 {
 	Scene *sce;
 	Base *base;
+	Main *pr_main = sp->pr_main;
 	
-	if (pr_main == NULL) return NULL;
-	
-	sce = pr_main->scene.first;
+	sce = preview_get_scene(pr_main);
 	if (sce) {
 		
 		/* this flag tells render to not execute depsgraph or ipos etc */
-		sce->r.scemode |= R_PREVIEWBUTS;
+		sce->r.scemode |= R_BUTS_PREVIEW;
 		/* set world always back, is used now */
 		sce->world = pr_main->world.first;
 		/* now: exposure copy */
@@ -266,23 +287,12 @@ static Scene *preview_prepare_scene(Scene *scene, ID *id, int id_type, ShaderPre
 		BKE_color_managed_view_settings_copy(&sce->view_settings, &scene->view_settings);
 		
 		/* prevent overhead for small renders and icons (32) */
-		if (id && sp->sizex < 40)
-			sce->r.xparts = sce->r.yparts = 1;
-		else
-			sce->r.xparts = sce->r.yparts = 4;
-		
-		/* exception: don't apply render part of display transform for texture previews or icons */
-		if ((id && sp->pr_method == PR_ICON_RENDER) || id_type == ID_TE) {
-			ColorManagedDisplaySettings *display_settings = &sce->display_settings;
-			ColorManagedViewSettings *view_settings = &sce->view_settings;
-
-			const char *default_view_name = IMB_colormanagement_view_get_default_name(display_settings->display_device);
-
-			view_settings->exposure = 0.0f;
-			view_settings->gamma = 1.0f;
-			view_settings->flag &= ~COLORMANAGE_VIEW_USE_CURVES;
-
-			BLI_strncpy(view_settings->view_transform, default_view_name, sizeof(view_settings->view_transform));
+		if (id && sp->sizex < 40) {
+			sce->r.tilex = sce->r.tiley = 64;
+		}
+		else {
+			sce->r.tilex = sce->r.xsch / 4;
+			sce->r.tiley = sce->r.ysch / 4;
 		}
 		
 		if ((id && sp->pr_method == PR_ICON_RENDER) && id_type != ID_WO)
@@ -291,7 +301,17 @@ static Scene *preview_prepare_scene(Scene *scene, ID *id, int id_type, ShaderPre
 			sce->r.alphamode = R_ADDSKY;
 
 		sce->r.cfra = scene->r.cfra;
-		BLI_strncpy(sce->r.engine, scene->r.engine, sizeof(sce->r.engine));
+
+		if (id_type == ID_TE && !check_engine_supports_textures(scene)) {
+			/* Force blender internal for texture icons and nodes render,
+			 * seems commonly used render engines does not support
+			 * such kind of rendering.
+			 */
+			BLI_strncpy(sce->r.engine, RE_engine_id_BLENDER_RENDER, sizeof(sce->r.engine));
+		}
+		else {
+			BLI_strncpy(sce->r.engine, scene->r.engine, sizeof(sce->r.engine));
+		}
 		
 		if (id_type == ID_MA) {
 			Material *mat = NULL, *origmat = (Material *)id;
@@ -302,65 +322,76 @@ static Scene *preview_prepare_scene(Scene *scene, ID *id, int id_type, ShaderPre
 				sp->matcopy = mat;
 				BLI_addtail(&pr_main->mat, mat);
 				
-				init_render_material(mat, 0, NULL);     /* call that retrieves mode_l */
-				end_render_material(mat);
-				
-				/* un-useful option */
-				if (sp->pr_method == PR_ICON_RENDER)
-					mat->shade_flag &= ~MA_OBCOLOR;
+				if (!BKE_scene_use_new_shading_nodes(scene)) {
+					init_render_material(mat, 0, NULL);     /* call that retrieves mode_l */
+					end_render_material(mat);
+					
+					/* un-useful option */
+					if (sp->pr_method == PR_ICON_RENDER)
+						mat->shade_flag &= ~MA_OBCOLOR;
 
-				/* turn on raytracing if needed */
-				if (mat->mode_l & MA_RAYMIRROR)
-					sce->r.mode |= R_RAYTRACE;
-				if (mat->material_type == MA_TYPE_VOLUME)
-					sce->r.mode |= R_RAYTRACE;
-				if ((mat->mode_l & MA_RAYTRANSP) && (mat->mode_l & MA_TRANSP))
-					sce->r.mode |= R_RAYTRACE;
-				if (preview_mat_has_sss(mat, NULL))
-					sce->r.mode |= R_SSS;
-				
-				/* turn off fake shadows if needed */
-				/* this only works in a specific case where the preview.blend contains
-				 * an object starting with 'c' which has a material linked to it (not the obdata)
-				 * and that material has a fake shadow texture in the active texture slot */
-				for (base = sce->base.first; base; base = base->next) {
-					if (base->object->id.name[2] == 'c') {
-						Material *shadmat = give_current_material(base->object, base->object->actcol);
-						if (shadmat) {
-							if (mat->mode & MA_SHADBUF) shadmat->septex = 0;
-							else shadmat->septex |= 1;
+					/* turn on raytracing if needed */
+					if (mat->mode_l & MA_RAYMIRROR)
+						sce->r.mode |= R_RAYTRACE;
+					if (mat->material_type == MA_TYPE_VOLUME)
+						sce->r.mode |= R_RAYTRACE;
+					if ((mat->mode_l & MA_RAYTRANSP) && (mat->mode_l & MA_TRANSP))
+						sce->r.mode |= R_RAYTRACE;
+					if (preview_mat_has_sss(mat, NULL))
+						sce->r.mode |= R_SSS;
+					
+					/* turn off fake shadows if needed */
+					/* this only works in a specific case where the preview.blend contains
+					 * an object starting with 'c' which has a material linked to it (not the obdata)
+					 * and that material has a fake shadow texture in the active texture slot */
+					for (base = sce->base.first; base; base = base->next) {
+						if (base->object->id.name[2] == 'c') {
+							Material *shadmat = give_current_material(base->object, base->object->actcol);
+							if (shadmat) {
+								if (mat->mode2 & MA_CASTSHADOW) shadmat->septex = 0;
+								else shadmat->septex |= 1;
+							}
 						}
 					}
-				}
-				
-				/* turn off bounce lights for volume, 
-				 * doesn't make much visual difference and slows it down too */
-				if (mat->material_type == MA_TYPE_VOLUME) {
+					
+					/* turn off bounce lights for volume, 
+					 * doesn't make much visual difference and slows it down too */
 					for (base = sce->base.first; base; base = base->next) {
 						if (base->object->type == OB_LAMP) {
 							/* if doesn't match 'Lamp.002' --> main key light */
 							if (strcmp(base->object->id.name + 2, "Lamp.002") != 0) {
-								base->object->restrictflag |= OB_RESTRICT_RENDER;
+								if (mat->material_type == MA_TYPE_VOLUME)
+									base->object->restrictflag |= OB_RESTRICT_RENDER;
+								else
+									base->object->restrictflag &= ~OB_RESTRICT_RENDER;
 							}
 						}
 					}
 				}
-
+				else {
+					/* use current scene world to light sphere */
+					if (mat->pr_type == MA_SPHERE_A)
+						sce->world = scene->world;
+				}
 				
 				if (sp->pr_method == PR_ICON_RENDER) {
 					if (mat->material_type == MA_TYPE_HALO) {
 						sce->lay = 1 << MA_FLAT;
-					} 
+					}
 					else {
 						sce->lay = 1 << MA_SPHERE_A;
+
+						/* same as above, use current scene world to light sphere */
+						if (BKE_scene_use_new_shading_nodes(scene))
+							sce->world = scene->world;
 					}
 				}
 				else {
 					sce->lay = 1 << mat->pr_type;
 					if (mat->nodetree && sp->pr_method == PR_NODE_RENDER) {
 						/* two previews, they get copied by wmJob */
-						ntreeInitPreview(mat->nodetree, sp->sizex, sp->sizey);
-						ntreeInitPreview(origmat->nodetree, sp->sizex, sp->sizey);
+						BKE_node_preview_init_tree(mat->nodetree, sp->sizex, sp->sizey, true);
+						BKE_node_preview_init_tree(origmat->nodetree, sp->sizex, sp->sizey, true);
 					}
 				}
 			}
@@ -377,7 +408,7 @@ static Scene *preview_prepare_scene(Scene *scene, ID *id, int id_type, ShaderPre
 					if (OB_TYPE_SUPPORT_MATERIAL(base->object->type)) {
 						/* don't use assign_material, it changed mat->id.us, which shows in the UI */
 						Material ***matar = give_matarar(base->object);
-						int actcol = MAX2(base->object->actcol - 1, 0);
+						int actcol = max_ii(base->object->actcol - 1, 0);
 
 						if (matar && actcol < base->object->totcol)
 							(*matar)[actcol] = mat;
@@ -395,7 +426,7 @@ static Scene *preview_prepare_scene(Scene *scene, ID *id, int id_type, ShaderPre
 				tex = localize_texture(origtex);
 				sp->texcopy = tex;
 				BLI_addtail(&pr_main->tex, tex);
-			}			
+			}
 			sce->lay = 1 << MA_TEXTURE;
 			
 			for (base = sce->base.first; base; base = base->next) {
@@ -406,15 +437,16 @@ static Scene *preview_prepare_scene(Scene *scene, ID *id, int id_type, ShaderPre
 						
 						if (tex && sp->slot)
 							mat->mtex[0]->which_output = sp->slot->which_output;
-						
+
+						mat->mtex[0]->mapto &= ~MAP_ALPHA;
+						mat->alpha = 1.0f;
+
 						/* show alpha in this case */
 						if (tex == NULL || (tex->flag & TEX_PRV_ALPHA)) {
-							mat->mtex[0]->mapto |= MAP_ALPHA;
-							mat->alpha = 0.0f;
-						}
-						else {
-							mat->mtex[0]->mapto &= ~MAP_ALPHA;
-							mat->alpha = 1.0f;
+							if (!(tex && tex->type == TEX_IMAGE && (tex->imaflag & (TEX_USEALPHA | TEX_CALCALPHA)) == 0)) {
+								mat->mtex[0]->mapto |= MAP_ALPHA;
+								mat->alpha = 0.0f;
+							}
 						}
 					}
 				}
@@ -422,8 +454,8 @@ static Scene *preview_prepare_scene(Scene *scene, ID *id, int id_type, ShaderPre
 
 			if (tex && tex->nodetree && sp->pr_method == PR_NODE_RENDER) {
 				/* two previews, they get copied by wmJob */
-				ntreeInitPreview(origtex->nodetree, sp->sizex, sp->sizey);
-				ntreeInitPreview(tex->nodetree, sp->sizex, sp->sizey);
+				BKE_node_preview_init_tree(origtex->nodetree, sp->sizex, sp->sizey, true);
+				BKE_node_preview_init_tree(tex->nodetree, sp->sizex, sp->sizey, true);
 			}
 		}
 		else if (id_type == ID_LA) {
@@ -435,19 +467,21 @@ static Scene *preview_prepare_scene(Scene *scene, ID *id, int id_type, ShaderPre
 				sp->lampcopy = la;
 				BLI_addtail(&pr_main->lamp, la);
 			}
-			
-			if (la && la->type == LA_SUN && (la->sun_effect_type & LA_SUN_EFFECT_SKY)) {
-				sce->lay = 1 << MA_ATMOS;
-				sce->world = scene->world;
-				sce->camera = (Object *)BLI_findstring(&pr_main->object, "CameraAtmo", offsetof(ID, name) + 2);
+
+			sce->lay = 1 << MA_LAMP;
+
+			if (!BKE_scene_use_new_shading_nodes(scene)) {
+				if (la && la->type == LA_SUN && (la->sun_effect_type & LA_SUN_EFFECT_SKY)) {
+					sce->lay = 1 << MA_ATMOS;
+					sce->world = scene->world;
+					sce->camera = (Object *)BLI_findstring(&pr_main->object, "CameraAtmo", offsetof(ID, name) + 2);
+				}
+				else {
+					sce->world = NULL;
+					sce->camera = (Object *)BLI_findstring(&pr_main->object, "Camera", offsetof(ID, name) + 2);
+				}
 			}
-			else {
-				sce->lay = 1 << MA_LAMP;
-				sce->world = NULL;
-				sce->camera = (Object *)BLI_findstring(&pr_main->object, "Camera", offsetof(ID, name) + 2);
-			}
-			sce->r.mode &= ~R_SHADOW;
-			
+				
 			for (base = sce->base.first; base; base = base->next) {
 				if (base->object->id.name[2] == 'p') {
 					if (base->object->type == OB_LAMP)
@@ -457,8 +491,8 @@ static Scene *preview_prepare_scene(Scene *scene, ID *id, int id_type, ShaderPre
 
 			if (la && la->nodetree && sp->pr_method == PR_NODE_RENDER) {
 				/* two previews, they get copied by wmJob */
-				ntreeInitPreview(origla->nodetree, sp->sizex, sp->sizey);
-				ntreeInitPreview(la->nodetree, sp->sizex, sp->sizey);
+				BKE_node_preview_init_tree(origla->nodetree, sp->sizex, sp->sizey, true);
+				BKE_node_preview_init_tree(la->nodetree, sp->sizex, sp->sizey, true);
 			}
 		}
 		else if (id_type == ID_WO) {
@@ -475,8 +509,8 @@ static Scene *preview_prepare_scene(Scene *scene, ID *id, int id_type, ShaderPre
 
 			if (wrld && wrld->nodetree && sp->pr_method == PR_NODE_RENDER) {
 				/* two previews, they get copied by wmJob */
-				ntreeInitPreview(wrld->nodetree, sp->sizex, sp->sizey);
-				ntreeInitPreview(origwrld->nodetree, sp->sizex, sp->sizey);
+				BKE_node_preview_init_tree(wrld->nodetree, sp->sizex, sp->sizey, true);
+				BKE_node_preview_init_tree(origwrld->nodetree, sp->sizex, sp->sizey, true);
 			}
 		}
 		
@@ -487,24 +521,16 @@ static Scene *preview_prepare_scene(Scene *scene, ID *id, int id_type, ShaderPre
 }
 
 /* new UI convention: draw is in pixel space already. */
-/* uses ROUNDBOX button in block to get the rect */
-static int ed_preview_draw_rect(ScrArea *sa, Scene *sce, ID *id, int split, int first, rcti *rect, rcti *newrect)
+/* uses UI_BTYPE_ROUNDBOX button in block to get the rect */
+static bool ed_preview_draw_rect(ScrArea *sa, int split, int first, rcti *rect, rcti *newrect)
 {
 	Render *re;
 	RenderResult rres;
 	char name[32];
-	int do_gamma_correct = FALSE, do_predivide = FALSE;
 	int offx = 0;
 	int newx = BLI_rcti_size_x(rect);
 	int newy = BLI_rcti_size_y(rect);
-
-	if (id && GS(id->name) != ID_TE) {
-		/* exception: don't color manage texture previews - show the raw values */
-		if (sce) {
-			do_gamma_correct = TRUE;
-			do_predivide = sce->r.color_mgt_flag & R_COLOR_MANAGEMENT_PREDIVIDE;
-		}
-	}
+	bool ok = false;
 
 	if (!split || first) sprintf(name, "Preview %p", (void *)sa);
 	else sprintf(name, "SecondPreview %p", (void *)sa);
@@ -520,6 +546,7 @@ static int ed_preview_draw_rect(ScrArea *sa, Scene *sce, ID *id, int split, int
 		}
 	}
 
+	/* test if something rendered ok */
 	re = RE_GetRender(name);
 	RE_AcquireResultImage(re, &rres);
 
@@ -527,56 +554,40 @@ static int ed_preview_draw_rect(ScrArea *sa, Scene *sce, ID *id, int split, int
 		
 		if (ABS(rres.rectx - newx) < 2 && ABS(rres.recty - newy) < 2) {
 
-			newrect->xmax = MAX2(newrect->xmax, rect->xmin + rres.rectx + offx);
-			newrect->ymax = MAX2(newrect->ymax, rect->ymin + rres.recty);
+			newrect->xmax = max_ii(newrect->xmax, rect->xmin + rres.rectx + offx);
+			newrect->ymax = max_ii(newrect->ymax, rect->ymin + rres.recty);
 
 			if (rres.rectx && rres.recty) {
-				/* temporary conversion to byte for drawing */
+				unsigned char *rect_byte = MEM_mallocN(rres.rectx * rres.recty * sizeof(int), "ed_preview_draw_rect");
 				float fx = rect->xmin + offx;
 				float fy = rect->ymin;
-				int dither = 0;
-				unsigned char *rect_byte;
-
-				rect_byte = MEM_mallocN(rres.rectx * rres.recty * sizeof(int), "ed_preview_draw_rect");
-
-				if (do_gamma_correct) {
-					IMB_display_buffer_transform_apply(rect_byte, rres.rectf, rres.rectx, rres.recty, 4,
-					                                   &sce->view_settings, &sce->display_settings, do_predivide);
-
-				}
-				else {
-					/* OCIO_TODO: currently seems an exception for textures (came fro mlegacish time),
-					 *            but is it indeed expected behavior, or textures should be
-					 *            color managed as well?
-					 */
-					IMB_buffer_byte_from_float(rect_byte, rres.rectf,
-					                           4, dither, IB_PROFILE_SRGB, IB_PROFILE_LINEAR_RGB, do_predivide,
-					                           rres.rectx, rres.recty, rres.rectx, rres.rectx);
-				}
-
+				if (re)
+					RE_AcquiredResultGet32(re, &rres, (unsigned int *)rect_byte);
+				
 				glaDrawPixelsSafe(fx, fy, rres.rectx, rres.recty, rres.rectx, GL_RGBA, GL_UNSIGNED_BYTE, rect_byte);
-
+				
 				MEM_freeN(rect_byte);
+				
+				ok = 1;
 			}
-
-			RE_ReleaseResultImage(re);
-			return 1;
 		}
 	}
 
 	RE_ReleaseResultImage(re);
-	return 0;
+
+	return ok;
 }
 
 void ED_preview_draw(const bContext *C, void *idp, void *parentp, void *slotp, rcti *rect)
 {
 	if (idp) {
+		wmWindowManager *wm = CTX_wm_manager(C);
 		ScrArea *sa = CTX_wm_area(C);
-		Scene *sce = CTX_data_scene(C);
 		ID *id = (ID *)idp;
 		ID *parent = (ID *)parentp;
 		MTex *slot = (MTex *)slotp;
 		SpaceButs *sbuts = sa->spacedata.first;
+		ShaderPreview *sp = WM_jobs_customdata(wm, sa);
 		rcti newrect;
 		int ok;
 		int newx = BLI_rcti_size_x(rect);
@@ -588,35 +599,36 @@ void ED_preview_draw(const bContext *C, void *idp, void *parentp, void *slotp, r
 		newrect.ymax = rect->ymin;
 
 		if (parent) {
-			ok = ed_preview_draw_rect(sa, sce, id, 1, 1, rect, &newrect);
-			ok &= ed_preview_draw_rect(sa, sce, parent, 1, 0, rect, &newrect);
+			ok = ed_preview_draw_rect(sa, 1, 1, rect, &newrect);
+			ok &= ed_preview_draw_rect(sa, 1, 0, rect, &newrect);
 		}
 		else
-			ok = ed_preview_draw_rect(sa, sce, id, 0, 0, rect, &newrect);
+			ok = ed_preview_draw_rect(sa, 0, 0, rect, &newrect);
 
 		if (ok)
 			*rect = newrect;
 
-		/* check for spacetype... */
-		if (sbuts->spacetype == SPACE_BUTS && sbuts->preview) {
+		/* start a new preview render job if signalled through sbuts->preview,
+		 * if no render result was found and no preview render job is running,
+		 * or if the job is running and the size of preview changed */
+		if ((sbuts->spacetype == SPACE_BUTS && sbuts->preview) ||
+		    (!ok && !WM_jobs_test(wm, sa, WM_JOB_TYPE_RENDER_PREVIEW)) ||
+		    (sp && (ABS(sp->sizex - newx) >= 2 || ABS(sp->sizey - newy) > 2)))
+		{
 			sbuts->preview = 0;
-			ok = 0;
-		}
-	
-		if (ok == 0) {
 			ED_preview_shader_job(C, sa, id, parent, slot, newx, newy, PR_BUTS_RENDER);
 		}
-	}	
+	}
 }
 
 /* **************************** new shader preview system ****************** */
 
 /* inside thread, called by renderer, sets job update value */
-static void shader_preview_draw(void *spv, RenderResult *UNUSED(rr), volatile struct rcti *UNUSED(rect))
+static void shader_preview_update(void *spv, RenderResult *UNUSED(rr), volatile struct rcti *UNUSED(rect))
 {
 	ShaderPreview *sp = spv;
 	
-	*(sp->do_update) = TRUE;
+	*(sp->do_update) = true;
 }
 
 /* called by renderer, checks job value */
@@ -658,7 +670,7 @@ static void shader_preview_updatejob(void *spv)
 				if (sp->lampcopy && la->nodetree && sp->lampcopy->nodetree)
 					ntreeLocalSync(sp->lampcopy->nodetree, la->nodetree);
 			}
-		}		
+		}
 	}
 }
 
@@ -670,9 +682,27 @@ static void shader_preview_render(ShaderPreview *sp, ID *id, int split, int firs
 	short idtype = GS(id->name);
 	char name[32];
 	int sizex;
+	Main *pr_main = sp->pr_main;
+	
+	/* in case of split preview, use border render */
+	if (split) {
+		if (first) sizex = sp->sizex / 2;
+		else sizex = sp->sizex - sp->sizex / 2;
+	}
+	else {
+		sizex = sp->sizex;
+	}
+
+	/* we have to set preview variables first */
+	sce = preview_get_scene(pr_main);
+	if (sce) {
+		sce->r.xsch = sizex;
+		sce->r.ysch = sp->sizey;
+		sce->r.size = 100;
+	}
 	
 	/* get the stuff from the builtin preview dbase */
-	sce = preview_prepare_scene(sp->scene, id, idtype, sp); // XXX sizex
+	sce = preview_prepare_scene(sp->scene, id, idtype, sp);
 	if (sce == NULL) return;
 	
 	if (!split || first) sprintf(name, "Preview %p", sp->owner);
@@ -700,21 +730,10 @@ static void shader_preview_render(ShaderPreview *sp, ID *id, int split, int firs
 		sce->r.mode |= R_OSA;
 	}
 
-	/* in case of split preview, use border render */
-	if (split) {
-		if (first) sizex = sp->sizex / 2;
-		else sizex = sp->sizex - sp->sizex / 2;
-	}
-	else sizex = sp->sizex;
-
-	/* allocates or re-uses render result */
-	sce->r.xsch = sizex;
-	sce->r.ysch = sp->sizey;
-	sce->r.size = 100;
 
 	/* callbacs are cleared on GetRender() */
 	if (ELEM(sp->pr_method, PR_BUTS_RENDER, PR_NODE_RENDER)) {
-		RE_display_draw_cb(re, sp, shader_preview_draw);
+		RE_display_update_cb(re, sp, shader_preview_update);
 	}
 	/* set this for all previews, default is react to G.is_break still */
 	RE_test_break_cb(re, sp, shader_preview_break);
@@ -736,12 +755,6 @@ static void shader_preview_render(ShaderPreview *sp, ID *id, int split, int firs
 		if (sp->pr_rect)
 			RE_ResultGet32(re, sp->pr_rect);
 	}
-	else {
-		/* validate owner */
-		//if (ri->rect == NULL)
-		//	ri->rect= MEM_mallocN(sizeof(int)*ri->pr_rectx*ri->pr_recty, "BIF_previewrender");
-		//RE_ResultGet32(re, ri->rect);
-	}
 
 	/* unassign the pointers, reset vars */
 	preview_prepare_scene(sp->scene, NULL, GS(id->name), sp);
@@ -770,12 +783,13 @@ static void shader_preview_startjob(void *customdata, short *stop, short *do_upd
 	else
 		shader_preview_render(sp, sp->id, 0, 0);
 
-	*do_update = TRUE;
+	*do_update = true;
 }
 
 static void shader_preview_free(void *customdata)
 {
 	ShaderPreview *sp = customdata;
+	Main *pr_main = sp->pr_main;
 	
 	if (sp->matcopy) {
 		struct IDProperty *properties;
@@ -786,9 +800,9 @@ static void shader_preview_free(void *customdata)
 		/* get rid of copied material */
 		BLI_remlink(&pr_main->mat, sp->matcopy);
 		
-		BKE_material_free_ex(sp->matcopy, FALSE);
+		BKE_material_free_ex(sp->matcopy, false);
 
-		properties = IDP_GetProperties((ID *)sp->matcopy, FALSE);
+		properties = IDP_GetProperties((ID *)sp->matcopy, false);
 		if (properties) {
 			IDP_FreeProperty(properties);
 			MEM_freeN(properties);
@@ -804,7 +818,7 @@ static void shader_preview_free(void *customdata)
 		BLI_remlink(&pr_main->tex, sp->texcopy);
 		BKE_texture_free(sp->texcopy);
 		
-		properties = IDP_GetProperties((ID *)sp->texcopy, FALSE);
+		properties = IDP_GetProperties((ID *)sp->texcopy, false);
 		if (properties) {
 			IDP_FreeProperty(properties);
 			MEM_freeN(properties);
@@ -818,9 +832,9 @@ static void shader_preview_free(void *customdata)
 		
 		/* get rid of copied world */
 		BLI_remlink(&pr_main->world, sp->worldcopy);
-		BKE_world_free_ex(sp->worldcopy, FALSE);
+		BKE_world_free_ex(sp->worldcopy, true); /* [#32865] - we need to unlink the texture copies, unlike for materials */
 		
-		properties = IDP_GetProperties((ID *)sp->worldcopy, FALSE);
+		properties = IDP_GetProperties((ID *)sp->worldcopy, false);
 		if (properties) {
 			IDP_FreeProperty(properties);
 			MEM_freeN(properties);
@@ -836,7 +850,7 @@ static void shader_preview_free(void *customdata)
 		BLI_remlink(&pr_main->lamp, sp->lampcopy);
 		BKE_lamp_free(sp->lampcopy);
 		
-		properties = IDP_GetProperties((ID *)sp->lampcopy, FALSE);
+		properties = IDP_GetProperties((ID *)sp->lampcopy, false);
 		if (properties) {
 			IDP_FreeProperty(properties);
 			MEM_freeN(properties);
@@ -870,7 +884,7 @@ static void icon_copy_rect(ImBuf *ibuf, unsigned int w, unsigned int h, unsigned
 		scaledx = (float)w;
 		scaledy =  ( (float)ima->y / (float)ima->x) * (float)w;
 	}
-	else {			
+	else {
 		scaledx =  ( (float)ima->x / (float)ima->y) * (float)h;
 		scaledy = (float)h;
 	}
@@ -913,7 +927,7 @@ static void icon_preview_startjob(void *customdata, short *stop, short *do_updat
 	ShaderPreview *sp = customdata;
 	ID *id = sp->id;
 	short idtype = GS(id->name);
-
+	
 	if (idtype == ID_IM) {
 		Image *ima = (Image *)id;
 		ImBuf *ibuf = NULL;
@@ -930,27 +944,31 @@ static void icon_preview_startjob(void *customdata, short *stop, short *do_updat
 		/* elubie: this needs to be changed: here image is always loaded if not
 		 * already there. Very expensive for large images. Need to find a way to 
 		 * only get existing ibuf */
-		ibuf = BKE_image_get_ibuf(ima, &iuser);
-		if (ibuf == NULL || ibuf->rect == NULL)
+		ibuf = BKE_image_acquire_ibuf(ima, &iuser, NULL);
+		if (ibuf == NULL || ibuf->rect == NULL) {
+			BKE_image_release_ibuf(ima, ibuf, NULL);
 			return;
+		}
 		
 		icon_copy_rect(ibuf, sp->sizex, sp->sizey, sp->pr_rect);
 
-		*do_update = TRUE;
+		*do_update = true;
+
+		BKE_image_release_ibuf(ima, ibuf, NULL);
 	}
 	else if (idtype == ID_BR) {
 		Brush *br = (Brush *)id;
 
 		br->icon_imbuf = get_brush_icon(br);
 
-		memset(sp->pr_rect, 0x888888, sp->sizex * sp->sizey * sizeof(unsigned int));
+		memset(sp->pr_rect, 0x88, sp->sizex * sp->sizey * sizeof(unsigned int));
 
 		if (!(br->icon_imbuf) || !(br->icon_imbuf->rect))
 			return;
 
 		icon_copy_rect(br->icon_imbuf, sp->sizex, sp->sizey, sp->pr_rect);
 
-		*do_update = TRUE;
+		*do_update = true;
 	}
 	else {
 		/* re-use shader job */
@@ -1010,6 +1028,7 @@ static void icon_preview_startjob_all_sizes(void *customdata, short *stop, short
 {
 	IconPreview *ip = (IconPreview *)customdata;
 	IconPreviewSize *cur_size = ip->sizes.first;
+	const bool use_new_shading = BKE_scene_use_new_shading_nodes(ip->scene);
 
 	while (cur_size) {
 		ShaderPreview *sp = MEM_callocN(sizeof(ShaderPreview), "Icon ShaderPreview");
@@ -1023,6 +1042,20 @@ static void icon_preview_startjob_all_sizes(void *customdata, short *stop, short
 		sp->pr_rect = cur_size->rect;
 		sp->id = ip->id;
 
+		if (use_new_shading) {
+			/* texture icon rendering is hardcoded to use BI,
+			 * so don't even think of using cycle's bmain for
+			 * texture icons
+			 */
+			if (GS(ip->id->name) != ID_TE)
+				sp->pr_main = G_pr_main_cycles;
+			else
+				sp->pr_main = G_pr_main;
+		}
+		else {
+			sp->pr_main = G_pr_main;
+		}
+
 		common_preview_startjob(sp, stop, do_update, progress);
 		shader_preview_free(sp);
 
@@ -1034,8 +1067,27 @@ static void icon_preview_endjob(void *customdata)
 {
 	IconPreview *ip = customdata;
 
-	if (ip->id && GS(ip->id->name) == ID_BR)
-		WM_main_add_notifier(NC_BRUSH | NA_EDITED, ip->id);
+	if (ip->id) {
+
+		if (GS(ip->id->name) == ID_BR)
+			WM_main_add_notifier(NC_BRUSH | NA_EDITED, ip->id);
+#if 0		
+		if (GS(ip->id->name) == ID_MA) {
+			Material *ma = (Material *)ip->id;
+			PreviewImage *prv_img = ma->preview;
+			int i;
+
+			/* signal to gpu texture */
+			for (i = 0; i < NUM_ICON_SIZES; ++i) {
+				if (prv_img->gputexture[i]) {
+					GPU_texture_free(prv_img->gputexture[i]);
+					prv_img->gputexture[i] = NULL;
+					WM_main_add_notifier(NC_MATERIAL|ND_SHADING_DRAW, ip->id);
+				}
+			}
+		}
+#endif
+	}
 }
 
 static void icon_preview_free(void *customdata)
@@ -1071,7 +1123,7 @@ void ED_preview_icon_job(const bContext *C, void *owner, ID *id, unsigned int *r
 
 	/* setup job */
 	WM_jobs_customdata_set(wm_job, ip, icon_preview_free);
-	WM_jobs_timer(wm_job, 0.25, NC_MATERIAL, NC_MATERIAL);
+	WM_jobs_timer(wm_job, 0.1, NC_MATERIAL, NC_MATERIAL);
 	WM_jobs_callbacks(wm_job, icon_preview_startjob_all_sizes, NULL, NULL, icon_preview_endjob);
 
 	WM_jobs_start(CTX_wm_manager(C), wm_job);
@@ -1082,13 +1134,21 @@ void ED_preview_shader_job(const bContext *C, void *owner, ID *id, ID *parent, M
 	Object *ob = CTX_data_active_object(C);
 	wmJob *wm_job;
 	ShaderPreview *sp;
+	Scene *scene = CTX_data_scene(C);
+	short id_type = GS(id->name);
+	bool use_new_shading = BKE_scene_use_new_shading_nodes(scene);
+
+	/* Only texture node preview is supported with Cycles. */
+	if (use_new_shading && method == PR_NODE_RENDER && id_type != ID_TE) {
+		return;
+	}
 
 	wm_job = WM_jobs_get(CTX_wm_manager(C), CTX_wm_window(C), owner, "Shader Preview",
 	                    WM_JOB_EXCL_RENDER, WM_JOB_TYPE_RENDER_PREVIEW);
 	sp = MEM_callocN(sizeof(ShaderPreview), "shader preview");
 
 	/* customdata for preview thread */
-	sp->scene = CTX_data_scene(C);
+	sp->scene = scene;
 	sp->owner = owner;
 	sp->sizex = sizex;
 	sp->sizey = sizey;
@@ -1096,6 +1156,16 @@ void ED_preview_shader_job(const bContext *C, void *owner, ID *id, ID *parent, M
 	sp->id = id;
 	sp->parent = parent;
 	sp->slot = slot;
+
+	/* hardcoded preview .blend for cycles/internal, this should be solved
+	 * once with custom preview .blend path for external engines */
+	if ((method != PR_NODE_RENDER) && id_type != ID_TE && use_new_shading) {
+		sp->pr_main = G_pr_main_cycles;
+	}
+	else {
+		sp->pr_main = G_pr_main;
+	}
+
 	if (ob && ob->totcol) copy_v4_v4(sp->col, ob->col);
 	else sp->col[0] = sp->col[1] = sp->col[2] = sp->col[3] = 1.0f;
 	
@@ -1107,10 +1177,11 @@ void ED_preview_shader_job(const bContext *C, void *owner, ID *id, ID *parent, M
 	WM_jobs_start(CTX_wm_manager(C), wm_job);
 }
 
-void ED_preview_kill_jobs(const struct bContext *C)
+void ED_preview_kill_jobs(wmWindowManager *wm, Main *bmain)
 {
-	wmWindowManager *wm = CTX_wm_manager(C);
 	if (wm)
 		WM_jobs_kill(wm, NULL, common_preview_startjob);
+
+	ED_viewport_render_kill_jobs(wm, bmain, false);
 }
 
