@@ -29,20 +29,14 @@
 #include <string.h>
 #include <stddef.h>
 
-#include "MEM_guardedalloc.h"
-
-#include "BLI_blenlib.h"
 #include "BLI_utildefines.h"
 
 #include "DNA_scene_types.h"
+#include "DNA_userdef_types.h"
 
-#include "BKE_blender.h"
 #include "BKE_context.h"
 #include "BKE_image.h"
 #include "BKE_global.h"
-#include "BKE_main.h"
-#include "BKE_node.h"
-#include "BKE_report.h"
 #include "BKE_screen.h"
 
 #include "WM_api.h"
@@ -86,7 +80,7 @@ static ScrArea *biggest_non_image_area(bContext *C)
 	return big;
 }
 
-static ScrArea *find_area_showing_r_result(bContext *C, wmWindow **win)
+static ScrArea *find_area_showing_r_result(bContext *C, Scene *scene, wmWindow **win)
 {
 	wmWindowManager *wm = CTX_wm_manager(C);
 	ScrArea *sa = NULL;
@@ -94,15 +88,17 @@ static ScrArea *find_area_showing_r_result(bContext *C, wmWindow **win)
 
 	/* find an imagewindow showing render result */
 	for (*win = wm->windows.first; *win; *win = (*win)->next) {
-		for (sa = (*win)->screen->areabase.first; sa; sa = sa->next) {
-			if (sa->spacetype == SPACE_IMAGE) {
-				sima = sa->spacedata.first;
-				if (sima->image && sima->image->type == IMA_TYPE_R_RESULT)
-					break;
+		if ((*win)->screen->scene == scene) {
+			for (sa = (*win)->screen->areabase.first; sa; sa = sa->next) {
+				if (sa->spacetype == SPACE_IMAGE) {
+					sima = sa->spacedata.first;
+					if (sima->image && sima->image->type == IMA_TYPE_R_RESULT)
+						break;
+				}
 			}
+			if (sa)
+				break;
 		}
-		if (sa)
-			break;
 	}
 	
 	return sa;
@@ -129,16 +125,16 @@ static ScrArea *find_area_image_empty(bContext *C)
 /********************** open image editor for render *************************/
 
 /* new window uses x,y to set position */
-void render_view_open(bContext *C, int mx, int my)
+ScrArea *render_view_open(bContext *C, int mx, int my)
 {
 	wmWindow *win = CTX_wm_window(C);
 	Scene *scene = CTX_data_scene(C);
 	ScrArea *sa = NULL;
 	SpaceImage *sima;
-	int area_was_image = 0;
+	bool area_was_image = false;
 
 	if (scene->r.displaymode == R_OUTPUT_NONE)
-		return;
+		return NULL;
 	
 	if (scene->r.displaymode == R_OUTPUT_WINDOW) {
 		rcti rect;
@@ -151,9 +147,10 @@ void render_view_open(bContext *C, int mx, int my)
 		if (sizex < 320) sizex = 320;
 		if (sizey < 256) sizey = 256;
 
-		/* XXX some magic to calculate postition */
-		rect.xmin = mx + win->posx - sizex / 2;
-		rect.ymin = my + win->posy - sizey / 2;
+		/* some magic to calculate postition */
+		/* pixelsize: mouse coords are in U.pixelsize units :/ */
+		rect.xmin = (mx / U.pixelsize) + win->posx - sizex / 2;
+		rect.ymin = (my / U.pixelsize) + win->posy - sizey / 2;
 		rect.xmax = rect.xmin + sizex;
 		rect.ymax = rect.ymin + sizey;
 
@@ -163,15 +160,16 @@ void render_view_open(bContext *C, int mx, int my)
 		sa = CTX_wm_area(C);
 	}
 	else if (scene->r.displaymode == R_OUTPUT_SCREEN) {
-		if (CTX_wm_area(C) && CTX_wm_area(C)->spacetype == SPACE_IMAGE)
-			area_was_image = 1;
+		sa = CTX_wm_area(C);
+		if (sa && sa->spacetype == SPACE_IMAGE)
+			area_was_image = true;
 
 		/* this function returns with changed context */
-		sa = ED_screen_full_newspace(C, CTX_wm_area(C), SPACE_IMAGE);
+		sa = ED_screen_full_newspace(C, sa, SPACE_IMAGE);
 	}
 
 	if (!sa) {
-		sa = find_area_showing_r_result(C, &win);
+		sa = find_area_showing_r_result(C, scene, &win);
 		if (sa == NULL)
 			sa = find_area_image_empty(C);
 		
@@ -220,9 +218,10 @@ void render_view_open(bContext *C, int mx, int my)
 		else {
 			/* Leave it alone so the image editor will just go back from
 			 * full screen to the original tiled setup */
-			;
 		}
 	}
+
+	return sa;
 }
 
 /*************************** cancel render viewer **********************/
@@ -234,8 +233,8 @@ static int render_view_cancel_exec(bContext *C, wmOperator *UNUSED(op))
 	SpaceImage *sima = sa->spacedata.first;
 
 	/* test if we have a temp screen in front */
-	if (CTX_wm_window(C)->screen->temp) {
-		wm_window_lower(CTX_wm_window(C));
+	if (win->screen->temp) {
+		wm_window_lower(win);
 		return OPERATOR_FINISHED;
 	}
 	/* determine if render already shows */
@@ -253,7 +252,7 @@ static int render_view_cancel_exec(bContext *C, wmOperator *UNUSED(op))
 	}
 	else if (sima->flag & SI_FULLWINDOW) {
 		sima->flag &= ~SI_FULLWINDOW;
-		ED_screen_full_toggle(C, win, sa);
+		ED_screen_state_toggle(C, win, sa, SCREENMAXIMIZED);
 		return OPERATOR_FINISHED;
 	}
 
@@ -274,7 +273,7 @@ void RENDER_OT_view_cancel(struct wmOperatorType *ot)
 
 /************************* show render viewer *****************/
 
-static int render_view_show_invoke(bContext *C, wmOperator *UNUSED(op), wmEvent *event)
+static int render_view_show_invoke(bContext *C, wmOperator *UNUSED(op), const wmEvent *event)
 {
 	wmWindow *wincur = CTX_wm_window(C);
 	
@@ -282,13 +281,16 @@ static int render_view_show_invoke(bContext *C, wmOperator *UNUSED(op), wmEvent
 	if (wincur->screen->temp) {
 		wm_window_lower(wincur);
 	}
-	else { 
+	else {
 		wmWindow *win, *winshow;
-		ScrArea *sa = find_area_showing_r_result(C, &winshow);
+		ScrArea *sa = find_area_showing_r_result(C, CTX_data_scene(C), &winshow);
 		
-		/* is there another window showing result? */
+		/* is there another window on current scene showing result? */
 		for (win = CTX_wm_manager(C)->windows.first; win; win = win->next) {
-			if (win->screen->temp || (win == winshow && winshow != wincur)) {
+			bScreen *sc = win->screen;
+			if ((sc->temp && ((ScrArea *)sc->areabase.first)->spacetype == SPACE_IMAGE) ||
+			    (win == winshow && winshow != wincur))
+			{
 				wm_window_raise(win);
 				return OPERATOR_FINISHED;
 			}
@@ -297,7 +299,7 @@ static int render_view_show_invoke(bContext *C, wmOperator *UNUSED(op), wmEvent
 		/* determine if render already shows */
 		if (sa) {
 			/* but don't close it when rendering */
-			if (G.is_rendering == FALSE) {
+			if (G.is_rendering == false) {
 				SpaceImage *sima = sa->spacedata.first;
 
 				if (sima->flag & SI_PREVSPACE) {
