@@ -36,6 +36,7 @@
 #include "DNA_scene_types.h"
 #include "DNA_userdef_types.h"
 
+#include "BLI_math.h"
 #include "BLI_blenlib.h"
 #include "BLI_utildefines.h"
 
@@ -54,11 +55,9 @@
 #include "WM_api.h"
 #include "WM_types.h"
 
-#include "ED_image.h"
 #include "ED_object.h"
 #include "ED_screen.h"
 #include "ED_screen_types.h"
-#include "ED_fileselect.h"
 #include "ED_clip.h"
 #include "ED_render.h"
 
@@ -193,7 +192,9 @@ void removenotused_scrverts(bScreen *sc)
 			BLI_remlink(&sc->vertbase, sv);
 			MEM_freeN(sv);
 		}
-		else sv->flag = 0;
+		else {
+			sv->flag = 0;
+		}
 		sv = svn;
 	}
 }
@@ -249,36 +250,48 @@ void removenotused_scredges(bScreen *sc)
 			BLI_remlink(&sc->edgebase, se);
 			MEM_freeN(se);
 		}
-		else se->flag = 0;
+		else {
+			se->flag = 0;
+		}
 		se = sen;
 	}
 }
 
-int scredge_is_horizontal(ScrEdge *se)
+bool scredge_is_horizontal(ScrEdge *se)
 {
 	return (se->v1->vec.y == se->v2->vec.y);
 }
 
-ScrEdge *screen_find_active_scredge(bScreen *sc, int mx, int my)
+/* need win size to make sure not to include edges along screen edge */
+ScrEdge *screen_find_active_scredge(bScreen *sc,
+                                    const int winsize_x, const int winsize_y,
+                                    const int mx, const int my)
 {
 	ScrEdge *se;
+	int safety = U.widget_unit / 10;
+	
+	if (safety < 2) safety = 2;
 	
 	for (se = sc->edgebase.first; se; se = se->next) {
 		if (scredge_is_horizontal(se)) {
-			short min, max;
-			min = MIN2(se->v1->vec.x, se->v2->vec.x);
-			max = MAX2(se->v1->vec.x, se->v2->vec.x);
-			
-			if (abs(my - se->v1->vec.y) <= 2 && mx >= min && mx <= max)
-				return se;
-		} 
+			if (se->v1->vec.y > 0 && se->v1->vec.y < winsize_y - 1) {
+				short min, max;
+				min = MIN2(se->v1->vec.x, se->v2->vec.x);
+				max = MAX2(se->v1->vec.x, se->v2->vec.x);
+				
+				if (abs(my - se->v1->vec.y) <= safety && mx >= min && mx <= max)
+					return se;
+			}
+		}
 		else {
-			short min, max;
-			min = MIN2(se->v1->vec.y, se->v2->vec.y);
-			max = MAX2(se->v1->vec.y, se->v2->vec.y);
-			
-			if (abs(mx - se->v1->vec.x) <= 2 && my >= min && my <= max)
-				return se;
+			if (se->v1->vec.x > 0 && se->v1->vec.x < winsize_x - 1) {
+				short min, max;
+				min = MIN2(se->v1->vec.y, se->v2->vec.y);
+				max = MAX2(se->v1->vec.y, se->v2->vec.y);
+				
+				if (abs(mx - se->v1->vec.x) <= safety && my >= min && my <= max)
+					return se;
+			}
 		}
 	}
 	
@@ -319,10 +332,12 @@ static void screen_delarea(bContext *C, bScreen *sc, ScrArea *sa)
 static short testsplitpoint(ScrArea *sa, char dir, float fac)
 {
 	short x, y;
+	const short area_min_x = AREAMINX;
+	const short area_min_y = ED_area_headersize();
 	
 	// area big enough?
-	if (dir == 'v' && (sa->v4->vec.x - sa->v1->vec.x <= 2 * AREAMINX)) return 0;
-	if (dir == 'h' && (sa->v2->vec.y - sa->v1->vec.y <= 2 * AREAMINY)) return 0;
+	if (dir == 'v' && (sa->v4->vec.x - sa->v1->vec.x <= 2 * area_min_x)) return 0;
+	if (dir == 'h' && (sa->v2->vec.y - sa->v1->vec.y <= 2 * area_min_y)) return 0;
 	
 	// to be sure
 	CLAMP(fac, 0.0f, 1.0f);
@@ -330,10 +345,10 @@ static short testsplitpoint(ScrArea *sa, char dir, float fac)
 	if (dir == 'h') {
 		y = sa->v1->vec.y + fac * (sa->v2->vec.y - sa->v1->vec.y);
 		
-		if (y - sa->v1->vec.y < AREAMINY)
-			y = sa->v1->vec.y + AREAMINY;
-		else if (sa->v2->vec.y - y < AREAMINY)
-			y = sa->v2->vec.y - AREAMINY;
+		if (y - sa->v1->vec.y < area_min_y)
+			y = sa->v1->vec.y + area_min_y;
+		else if (sa->v2->vec.y - y < area_min_y)
+			y = sa->v2->vec.y - area_min_y;
 		else y -= (y % AREAGRID);
 		
 		return y;
@@ -341,10 +356,10 @@ static short testsplitpoint(ScrArea *sa, char dir, float fac)
 	else {
 		x = sa->v1->vec.x + fac * (sa->v4->vec.x - sa->v1->vec.x);
 		
-		if (x - sa->v1->vec.x < AREAMINX)
-			x = sa->v1->vec.x + AREAMINX;
-		else if (sa->v4->vec.x - x < AREAMINX)
-			x = sa->v4->vec.x - AREAMINX;
+		if (x - sa->v1->vec.x < area_min_x)
+			x = sa->v1->vec.x + area_min_x;
+		else if (sa->v4->vec.x - x < area_min_x)
+			x = sa->v4->vec.x - area_min_x;
 		else x -= (x % AREAGRID);
 		
 		return x;
@@ -362,6 +377,10 @@ ScrArea *area_split(bScreen *sc, ScrArea *sa, char dir, float fac, int merge)
 	split = testsplitpoint(sa, dir, fac);
 	if (split == 0) return NULL;
 	
+	/* note regarding (fac > 0.5f) checks below.
+	 * normally it shouldn't matter which is used since the copy should match the original
+	 * however with viewport rendering and python console this isn't the case. - campbell */
+
 	if (dir == 'h') {
 		/* new vertices */
 		sv1 = screen_addvert(sc, sa->v1->vec.x, split);
@@ -374,13 +393,24 @@ ScrArea *area_split(bScreen *sc, ScrArea *sa, char dir, float fac, int merge)
 		screen_addedge(sc, sv2, sa->v4);
 		screen_addedge(sc, sv1, sv2);
 		
-		/* new areas: top */
-		newa = screen_addarea(sc, sv1, sa->v2, sa->v3, sv2, sa->headertype, sa->spacetype);
-		area_copy_data(newa, sa, 0);
-		
-		/* area below */
-		sa->v2 = sv1;
-		sa->v3 = sv2;
+		if (fac > 0.5f) {
+			/* new areas: top */
+			newa = screen_addarea(sc, sv1, sa->v2, sa->v3, sv2, sa->headertype, sa->spacetype);
+
+			/* area below */
+			sa->v2 = sv1;
+			sa->v3 = sv2;
+		}
+		else {
+			/* new areas: bottom */
+			newa = screen_addarea(sc, sa->v1, sv1, sv2, sa->v4, sa->headertype, sa->spacetype);
+
+			/* area above */
+			sa->v1 = sv1;
+			sa->v4 = sv2;
+		}
+
+		ED_area_data_copy(newa, sa, true);
 		
 	}
 	else {
@@ -395,13 +425,24 @@ ScrArea *area_split(bScreen *sc, ScrArea *sa, char dir, float fac, int merge)
 		screen_addedge(sc, sv2, sa->v3);
 		screen_addedge(sc, sv1, sv2);
 		
-		/* new areas: left */
-		newa = screen_addarea(sc, sa->v1, sa->v2, sv2, sv1, sa->headertype, sa->spacetype);
-		area_copy_data(newa, sa, 0);
-		
-		/* area right */
-		sa->v1 = sv1;
-		sa->v2 = sv2;
+		if (fac > 0.5f) {
+			/* new areas: right */
+			newa = screen_addarea(sc, sv1, sv2, sa->v3, sa->v4, sa->headertype, sa->spacetype);
+
+			/* area left */
+			sa->v3 = sv2;
+			sa->v4 = sv1;
+		}
+		else {
+			/* new areas: left */
+			newa = screen_addarea(sc, sa->v1, sa->v2, sv2, sv1, sa->headertype, sa->spacetype);
+
+			/* area right */
+			sa->v1 = sv1;
+			sa->v2 = sv2;
+		}
+
+		ED_area_data_copy(newa, sa, true);
 	}
 	
 	/* remove double vertices en edges */
@@ -417,19 +458,22 @@ ScrArea *area_split(bScreen *sc, ScrArea *sa, char dir, float fac, int merge)
 /* uses window size */
 bScreen *ED_screen_add(wmWindow *win, Scene *scene, const char *name)
 {
+	const int winsize_x = WM_window_pixels_x(win);
+	const int winsize_y = WM_window_pixels_y(win);
+
 	bScreen *sc;
 	ScrVert *sv1, *sv2, *sv3, *sv4;
 	
-	sc = BKE_libblock_alloc(&G.main->screen, ID_SCR, name);
+	sc = BKE_libblock_alloc(G.main, ID_SCR, name);
 	sc->scene = scene;
-	sc->do_refresh = TRUE;
+	sc->do_refresh = true;
 	sc->redraws_flag = TIME_ALL_3D_WIN | TIME_ALL_ANIM_WIN;
 	sc->winid = win->winid;
 
 	sv1 = screen_addvert(sc, 0, 0);
-	sv2 = screen_addvert(sc, 0, win->sizey - 1);
-	sv3 = screen_addvert(sc, win->sizex - 1, win->sizey - 1);
-	sv4 = screen_addvert(sc, win->sizex - 1, 0);
+	sv2 = screen_addvert(sc, 0, winsize_y - 1);
+	sv3 = screen_addvert(sc, winsize_x - 1, winsize_y - 1);
+	sv4 = screen_addvert(sc, winsize_x - 1, 0);
 	
 	screen_addedge(sc, sv1, sv2);
 	screen_addedge(sc, sv2, sv3);
@@ -454,7 +498,7 @@ static void screen_copy(bScreen *to, bScreen *from)
 	BLI_duplicatelist(&to->vertbase, &from->vertbase);
 	BLI_duplicatelist(&to->edgebase, &from->edgebase);
 	BLI_duplicatelist(&to->areabase, &from->areabase);
-	to->regionbase.first = to->regionbase.last = NULL;
+	BLI_listbase_clear(&to->regionbase);
 	
 	s2 = to->vertbase.first;
 	for (s1 = from->vertbase.first; s1; s1 = s1->next, s2 = s2->next) {
@@ -474,12 +518,12 @@ static void screen_copy(bScreen *to, bScreen *from)
 		sa->v3 = sa->v3->newv;
 		sa->v4 = sa->v4->newv;
 
-		sa->spacedata.first = sa->spacedata.last = NULL;
-		sa->regionbase.first = sa->regionbase.last = NULL;
-		sa->actionzones.first = sa->actionzones.last = NULL;
-		sa->handlers.first = sa->handlers.last = NULL;
+		BLI_listbase_clear(&sa->spacedata);
+		BLI_listbase_clear(&sa->regionbase);
+		BLI_listbase_clear(&sa->actionzones);
+		BLI_listbase_clear(&sa->handlers);
 		
-		area_copy_data(sa, saf, 0);
+		ED_area_data_copy(sa, saf, true);
 	}
 	
 	/* put at zero (needed?) */
@@ -534,7 +578,7 @@ int screen_area_join(bContext *C, bScreen *scr, ScrArea *sa1, ScrArea *sa2)
 	dir = area_getorientation(sa1, sa2);
 	/*printf("dir is : %i\n", dir);*/
 	
-	if (dir < 0) {
+	if (dir == -1) {
 		if (sa1) sa1->flag &= ~AREA_FLAG_DRAWJOINFROM;
 		if (sa2) sa2->flag &= ~AREA_FLAG_DRAWJOINTO;
 		return 0;
@@ -619,22 +663,25 @@ void select_connected_scredge(bScreen *sc, ScrEdge *edge)
 }
 
 /* test if screen vertices should be scaled */
-static void screen_test_scale(bScreen *sc, int winsizex, int winsizey)
+static void screen_test_scale(bScreen *sc, int winsize_x, int winsize_y)
 {
+	/* clamp Y size of header sized areas when expanding windows
+	 * avoids annoying empty space around file menu */
+#define USE_HEADER_SIZE_CLAMP
+
+	const int headery_init = ED_area_headersize();
 	ScrVert *sv = NULL;
 	ScrArea *sa;
-	int sizex, sizey;
+	int winsize_x_prev, winsize_y_prev;
 	float facx, facy, tempf, min[2], max[2];
 	
 	/* calculate size */
-	min[0] = min[1] = 10000.0f;
+	min[0] = min[1] = 20000.0f;
 	max[0] = max[1] = 0.0f;
 	
 	for (sv = sc->vertbase.first; sv; sv = sv->next) {
-		min[0] = MIN2(min[0], sv->vec.x);
-		min[1] = MIN2(min[1], sv->vec.y);
-		max[0] = MAX2(max[0], sv->vec.x);
-		max[1] = MAX2(max[1], sv->vec.y);
+		const float fv[2] = {(float)sv->vec.x, (float)sv->vec.y};
+		minmax_v2v2_v2(min, max, fv);
 	}
 	
 	/* always make 0.0 left under */
@@ -643,14 +690,41 @@ static void screen_test_scale(bScreen *sc, int winsizex, int winsizey)
 		sv->vec.y -= min[1];
 	}
 	
-	sizex = max[0] - min[0];
-	sizey = max[1] - min[1];
-	
-	if (sizex != winsizex || sizey != winsizey) {
-		facx = winsizex;
-		facx /= (float)sizex;
-		facy = winsizey;
-		facy /= (float)sizey;
+	winsize_x_prev = (max[0] - min[0]) + 1;
+	winsize_y_prev = (max[1] - min[1]) + 1;
+
+
+#ifdef USE_HEADER_SIZE_CLAMP
+#define TEMP_BOTTOM 1
+#define TEMP_TOP 2
+
+	/* if the window's Y axis grows, clamp header sized areas */
+	if (winsize_y_prev < winsize_y) {  /* growing? */
+		const int headery_margin_max = headery_init + 4;
+		for (sa = sc->areabase.first; sa; sa = sa->next) {
+			ARegion *ar = BKE_area_find_region_type(sa, RGN_TYPE_HEADER);
+			sa->temp = 0;
+
+			if (ar && !(ar->flag & RGN_FLAG_HIDDEN)) {
+				if (sa->v2->vec.y == winsize_y_prev - 1) {
+					if ((sa->v2->vec.y - sa->v1->vec.y) < headery_margin_max) {
+						sa->temp = TEMP_TOP;
+					}
+				}
+				else if (sa->v1->vec.y == 0) {
+					if ((sa->v2->vec.y - sa->v1->vec.y) < headery_margin_max) {
+						sa->temp = TEMP_BOTTOM;
+					}
+				}
+			}
+		}
+	}
+#endif
+
+
+	if (winsize_x_prev != winsize_x || winsize_y_prev != winsize_y) {
+		facx = ((float)winsize_x - 1) / ((float)winsize_x_prev - 1);
+		facy = ((float)winsize_y - 1) / ((float)winsize_y_prev - 1);
 		
 		/* make sure it fits! */
 		for (sv = sc->vertbase.first; sv; sv = sv->next) {
@@ -658,28 +732,82 @@ static void screen_test_scale(bScreen *sc, int winsizex, int winsizey)
 			 * need some way to store these as floats internally and re-apply from there. */
 			tempf = ((float)sv->vec.x) * facx;
 			sv->vec.x = (short)(tempf + 0.5f);
-			sv->vec.x += AREAGRID - 1;
-			sv->vec.x -=  (sv->vec.x % AREAGRID);
+			//sv->vec.x += AREAGRID - 1;
+			//sv->vec.x -=  (sv->vec.x % AREAGRID);
 
-			CLAMP(sv->vec.x, 0, winsizex);
+			CLAMP(sv->vec.x, 0, winsize_x - 1);
 			
 			tempf = ((float)sv->vec.y) * facy;
 			sv->vec.y = (short)(tempf + 0.5f);
-			sv->vec.y += AREAGRID - 1;
-			sv->vec.y -=  (sv->vec.y % AREAGRID);
+			//sv->vec.y += AREAGRID - 1;
+			//sv->vec.y -=  (sv->vec.y % AREAGRID);
 
-			CLAMP(sv->vec.y, 0, winsizey);
+			CLAMP(sv->vec.y, 0, winsize_y - 1);
 		}
 	}
-	
+
+
+#ifdef USE_HEADER_SIZE_CLAMP
+	if (winsize_y_prev < winsize_y) {  /* growing? */
+		for (sa = sc->areabase.first; sa; sa = sa->next) {
+			ScrEdge *se = NULL;
+
+			if (sa->temp == 0)
+				continue;
+
+			if (sa->v1 == sa->v2)
+				continue;
+
+			/* adjust headery if verts are along the edge of window */
+			if (sa->temp == TEMP_TOP) {
+				/* lower edge */
+				const int yval = sa->v2->vec.y - headery_init;
+				se = screen_findedge(sc, sa->v4, sa->v1);
+				select_connected_scredge(sc, se);
+				for (sv = sc->vertbase.first; sv; sv = sv->next) {
+					if (sv != sa->v2 && sv != sa->v3) {
+						if (sv->flag) {
+							sv->vec.y = yval;
+						}
+					}
+				}
+			}
+			else {
+				/* upper edge */
+				const int yval = sa->v1->vec.y + headery_init;
+				se = screen_findedge(sc, sa->v2, sa->v3);
+				select_connected_scredge(sc, se);
+				for (sv = sc->vertbase.first; sv; sv = sv->next) {
+					if (sv != sa->v1 && sv != sa->v4) {
+						if (sv->flag) {
+							sv->vec.y = yval;
+						}
+					}
+				}
+			}
+		}
+	}
+
+#undef USE_HEADER_SIZE_CLAMP
+#undef TEMP_BOTTOM
+#undef TEMP_TOP
+#endif
+
+
 	/* test for collapsed areas. This could happen in some blender version... */
 	/* ton: removed option now, it needs Context... */
 	
 	/* make each window at least ED_area_headersize() high */
 	for (sa = sc->areabase.first; sa; sa = sa->next) {
-		int headery = ED_area_headersize() + 1;
+		int headery = headery_init;
 		
-		if (sa->v1->vec.y + headery > sa->v2->vec.y) {
+		/* adjust headery if verts are along the edge of window */
+		if (sa->v1->vec.y > 0)
+			headery += U.pixelsize;
+		if (sa->v2->vec.y < winsize_y - 1)
+			headery += U.pixelsize;
+		
+		if (sa->v2->vec.y - sa->v1->vec.y + 1 < headery) {
 			/* lower edge */
 			ScrEdge *se = screen_findedge(sc, sa->v4, sa->v1);
 			if (se && sa->v1 != sa->v2) {
@@ -688,14 +816,14 @@ static void screen_test_scale(bScreen *sc, int winsizex, int winsizey)
 				select_connected_scredge(sc, se);
 				
 				/* all selected vertices get the right offset */
-				yval = sa->v2->vec.y - headery;
-				sv = sc->vertbase.first;
-				while (sv) {
+				yval = sa->v2->vec.y - headery + 1;
+				for (sv = sc->vertbase.first; sv; sv = sv->next) {
 					/* if is a collapsed area */
 					if (sv != sa->v2 && sv != sa->v3) {
-						if (sv->flag) sv->vec.y = yval;
+						if (sv->flag) {
+							sv->vec.y = yval;
+						}
 					}
-					sv = sv->next;
 				}
 			}
 		}
@@ -875,13 +1003,13 @@ static void scrarea_draw_shape_light(ScrArea *sa, char UNUSED(dir))
 	glBlendFunc(GL_DST_COLOR, GL_SRC_ALPHA);
 	glEnable(GL_BLEND);
 	/* value 181 was hardly computed: 181~105 */
-	glColor4ub(255, 255, 255, 50);		
+	glColor4ub(255, 255, 255, 50);
 	/* draw_join_shape(sa, dir); */
 	glRecti(sa->v1->vec.x, sa->v1->vec.y, sa->v3->vec.x, sa->v3->vec.y);
 	glDisable(GL_BLEND);
 }
 
-static void drawscredge_area_draw(int sizex, int sizey, short x1, short y1, short x2, short y2, short a) 
+static void drawscredge_area_draw(int sizex, int sizey, int x1, int y1, int x2, int y2, int a)
 {
 	/* right border area */
 	if (x2 < sizex - 1)
@@ -908,18 +1036,18 @@ static void drawscredge_area(ScrArea *sa, int sizex, int sizey, int center)
 	short y1 = sa->v1->vec.y;
 	short x2 = sa->v3->vec.x;
 	short y2 = sa->v3->vec.y;
-	short a, rt;
-	
-	rt = 0; // CLAMPIS(G.debug_value, 0, 16);
 	
 	if (center == 0) {
-		cpack(0x505050);
-		for (a = -rt; a <= rt; a++)
-			if (a != 0)
-				drawscredge_area_draw(sizex, sizey, x1, y1, x2, y2, a);
+		if (U.pixelsize > 1.0f) {
+		
+			glColor3ub(0x50, 0x50, 0x50);
+			glLineWidth((2.0f * U.pixelsize) - 1);
+			drawscredge_area_draw(sizex, sizey, x1, y1, x2, y2, 0);
+			glLineWidth(1.0f);
+		}
 	}
 	else {
-		cpack(0x0);
+		glColor3ub(0, 0, 0);
 		drawscredge_area_draw(sizex, sizey, x1, y1, x2, y2, 0);
 	}
 }
@@ -930,7 +1058,7 @@ bScreen *ED_screen_duplicate(wmWindow *win, bScreen *sc)
 {
 	bScreen *newsc;
 	
-	if (sc->full != SCREENNORMAL) return NULL;  /* XXX handle this case! */
+	if (sc->state != SCREENNORMAL) return NULL;  /* XXX handle this case! */
 	
 	/* make new empty screen: */
 	newsc = ED_screen_add(win, sc->scene, sc->id.name + 2);
@@ -969,20 +1097,18 @@ void ED_screen_do_listen(bContext *C, wmNotifier *note)
 	switch (note->category) {
 		case NC_WM:
 			if (note->data == ND_FILEREAD)
-				win->screen->do_draw = TRUE;
+				win->screen->do_draw = true;
 			break;
 		case NC_WINDOW:
-			win->screen->do_draw = TRUE;
+			win->screen->do_draw = true;
 			break;
 		case NC_SCREEN:
-			if (note->data == ND_SUBWINACTIVE)
-				uiFreeActiveButtons(C, win->screen);
 			if (note->action == NA_EDITED)
-				win->screen->do_draw = win->screen->do_refresh = TRUE;
+				win->screen->do_draw = win->screen->do_refresh = true;
 			break;
 		case NC_SCENE:
 			if (note->data == ND_MODE)
-				region_cursor_set(win, note->swinid, TRUE);
+				region_cursor_set(win, note->swinid, true);
 			break;
 	}
 }
@@ -990,6 +1116,9 @@ void ED_screen_do_listen(bContext *C, wmNotifier *note)
 /* only for edge lines between areas, and the blended join arrows */
 void ED_screen_draw(wmWindow *win)
 {
+	const int winsize_x = WM_window_pixels_x(win);
+	const int winsize_y = WM_window_pixels_y(win);
+
 	ScrArea *sa;
 	ScrArea *sa1 = NULL;
 	ScrArea *sa2 = NULL;
@@ -1003,15 +1132,15 @@ void ED_screen_draw(wmWindow *win)
 		if (sa->flag & AREA_FLAG_DRAWJOINFROM) sa1 = sa;
 		if (sa->flag & AREA_FLAG_DRAWJOINTO) sa2 = sa;
 		if (sa->flag & (AREA_FLAG_DRAWSPLIT_H | AREA_FLAG_DRAWSPLIT_V)) sa3 = sa;
-		drawscredge_area(sa, win->sizex, win->sizey, 0);
+		drawscredge_area(sa, winsize_x, winsize_y, 0);
 	}
 	for (sa = win->screen->areabase.first; sa; sa = sa->next)
-		drawscredge_area(sa, win->sizex, win->sizey, 1);
+		drawscredge_area(sa, winsize_x, winsize_y, 1);
 	
 	/* blended join arrow */
 	if (sa1 && sa2) {
 		dir = area_getorientation(sa1, sa2);
-		if (dir >= 0) {
+		if (dir != -1) {
 			switch (dir) {
 				case 0: /* W */
 					dir = 'r';
@@ -1054,7 +1183,7 @@ void ED_screen_draw(wmWindow *win)
 		glDisable(GL_BLEND);
 	}
 	
-	win->screen->do_draw = FALSE;
+	win->screen->do_draw = false;
 }
 
 /* helper call for below, dpi changes headers */
@@ -1066,7 +1195,7 @@ static void screen_refresh_headersizes(void)
 	for (st = lb->first; st; st = st->next) {
 		ARegionType *art = BKE_regiontype_from_id(st, RGN_TYPE_HEADER);
 		if (art) art->prefsizey = ED_area_headersize();
-	}		
+	}
 }
 
 /* make this screen usable */
@@ -1075,24 +1204,26 @@ void ED_screen_refresh(wmWindowManager *wm, wmWindow *win)
 {	
 	/* exception for bg mode, we only need the screen context */
 	if (!G.background) {
+		const int winsize_x = WM_window_pixels_x(win);
+		const int winsize_y = WM_window_pixels_y(win);
 		ScrArea *sa;
 		rcti winrct;
 	
 		winrct.xmin = 0;
-		winrct.xmax = win->sizex - 1;
+		winrct.xmax = winsize_x - 1;
 		winrct.ymin = 0;
-		winrct.ymax = win->sizey - 1;
+		winrct.ymax = winsize_y - 1;
 		
-		screen_test_scale(win->screen, win->sizex, win->sizey);
+		/* header size depends on DPI, let's verify */
+		screen_refresh_headersizes();
+		
+		screen_test_scale(win->screen, winsize_x, winsize_y);
 		
 		if (win->screen->mainwin == 0)
 			win->screen->mainwin = wm_subwindow_open(win, &winrct);
 		else
 			wm_subwindow_position(win, win->screen->mainwin, &winrct);
 		
-		/* header size depends on DPI, let's verify */
-		screen_refresh_headersizes();
-		
 		for (sa = win->screen->areabase.first; sa; sa = sa->next) {
 			/* set spacetype and region callbacks, calls init() */
 			/* sets subwindows for regions, adds handlers */
@@ -1101,13 +1232,13 @@ void ED_screen_refresh(wmWindowManager *wm, wmWindow *win)
 	
 		/* wake up animtimer */
 		if (win->screen->animtimer)
-			WM_event_timer_sleep(wm, win, win->screen->animtimer, 0);
+			WM_event_timer_sleep(wm, win, win->screen->animtimer, false);
 	}
 
 	if (G.debug & G_DEBUG_EVENTS) {
 		printf("%s: set screen\n", __func__);
 	}
-	win->screen->do_refresh = FALSE;
+	win->screen->do_refresh = false;
 
 	win->screen->context = ed_screen_context;
 }
@@ -1131,8 +1262,12 @@ void ED_screens_initialize(wmWindowManager *wm)
 
 void ED_region_exit(bContext *C, ARegion *ar)
 {
+	wmWindowManager *wm = CTX_wm_manager(C);
 	ARegion *prevar = CTX_wm_region(C);
 
+	if (ar->type && ar->type->exit)
+		ar->type->exit(wm, ar);
+
 	CTX_wm_region_set(C, ar);
 	WM_event_remove_handlers(C, &ar->handlers);
 	if (ar->swinid)
@@ -1143,23 +1278,20 @@ void ED_region_exit(bContext *C, ARegion *ar)
 		MEM_freeN(ar->headerstr);
 	ar->headerstr = NULL;
 	
+	if (ar->regiontimer)
+		WM_event_remove_timer(CTX_wm_manager(C), CTX_wm_window(C), ar->regiontimer);
+
 	CTX_wm_region_set(C, prevar);
 }
 
 void ED_area_exit(bContext *C, ScrArea *sa)
 {
+	wmWindowManager *wm = CTX_wm_manager(C);
 	ScrArea *prevsa = CTX_wm_area(C);
 	ARegion *ar;
 
-	if (sa->spacetype == SPACE_FILE) {
-		SpaceLink *sl = sa->spacedata.first;
-		if (sl && sl->spacetype == SPACE_FILE) {
-			ED_fileselect_exit(C, (SpaceFile *)sl);
-		}
-	}
-	else if (sa->spacetype == SPACE_VIEW3D) {
-		ED_render_engine_area_exit(sa);
-	}
+	if (sa->type && sa->type->exit)
+		sa->type->exit(wm, sa);
 
 	CTX_wm_area_set(C, sa);
 	for (ar = sa->regionbase.first; ar; ar = ar->next)
@@ -1212,6 +1344,9 @@ void ED_screen_exit(bContext *C, wmWindow *window, bScreen *screen)
 /* case when on area-edge or in azones, or outside window */
 static void screen_cursor_set(wmWindow *win, wmEvent *event)
 {
+	const int winsize_x = WM_window_pixels_x(win);
+	const int winsize_y = WM_window_pixels_y(win);
+
 	AZone *az = NULL;
 	ScrArea *sa;
 	
@@ -1230,7 +1365,7 @@ static void screen_cursor_set(wmWindow *win, wmEvent *event)
 		}
 	}
 	else {
-		ScrEdge *actedge = screen_find_active_scredge(win->screen, event->x, event->y);
+		ScrEdge *actedge = screen_find_active_scredge(win->screen, winsize_x, winsize_y, event->x, event->y);
 		
 		if (actedge) {
 			if (scredge_is_horizontal(actedge))
@@ -1263,9 +1398,12 @@ void ED_screen_set_subwinactive(bContext *C, wmEvent *event)
 						break;
 		}
 		if (sa) {
+			/* make overlap active when mouse over */
 			for (ar = sa->regionbase.first; ar; ar = ar->next) {
-				if (BLI_rcti_isect_pt_v(&ar->winrct, &event->x))
+				if (BLI_rcti_isect_pt_v(&ar->winrct, &event->x)) {
 					scr->subwinactive = ar->swinid;
+					break;
+				}
 			}
 		}
 		else
@@ -1275,11 +1413,11 @@ void ED_screen_set_subwinactive(bContext *C, wmEvent *event)
 		if (oldswin != scr->subwinactive) {
 
 			for (sa = scr->areabase.first; sa; sa = sa->next) {
-				int do_draw = FALSE;
+				bool do_draw = false;
 				
 				for (ar = sa->regionbase.first; ar; ar = ar->next)
 					if (ar->swinid == oldswin || ar->swinid == scr->subwinactive)
-						do_draw = TRUE;
+						do_draw = true;
 				
 				if (do_draw) {
 					for (ar = sa->regionbase.first; ar; ar = ar->next)
@@ -1294,12 +1432,17 @@ void ED_screen_set_subwinactive(bContext *C, wmEvent *event)
 			screen_cursor_set(win, event);
 		}
 		else {
+			/* notifier invokes freeing the buttons... causing a bit too much redraws */
 			if (oldswin != scr->subwinactive) {
-				region_cursor_set(win, scr->subwinactive, TRUE);
-				WM_event_add_notifier(C, NC_SCREEN | ND_SUBWINACTIVE, scr);
+				region_cursor_set(win, scr->subwinactive, true);
+
+				/* this used to be a notifier, but needs to be done immediate
+				 * because it can undo setting the right button as active due
+				 * to delayed notifier handling */
+				UI_screen_free_active_but(C, win->screen);
 			}
 			else
-				region_cursor_set(win, scr->subwinactive, FALSE);
+				region_cursor_set(win, scr->subwinactive, false);
 		}
 	}
 }
@@ -1320,7 +1463,7 @@ int ED_screen_area_active(const bContext *C)
 		for (ar = sa->regionbase.first; ar; ar = ar->next)
 			if (ar->swinid == sc->subwinactive)
 				return 1;
-	}	
+	}
 	return 0;
 }
 
@@ -1328,25 +1471,23 @@ int ED_screen_area_active(const bContext *C)
 /* Do NOT call in area/region queues! */
 void ED_screen_set(bContext *C, bScreen *sc)
 {
+	Main *bmain = CTX_data_main(C);
 	wmWindowManager *wm = CTX_wm_manager(C);
 	wmWindow *win = CTX_wm_window(C);
 	bScreen *oldscreen = CTX_wm_screen(C);
 	ID *id;
 	
 	/* validate screen, it's called with notifier reference */
-	for (id = CTX_data_main(C)->screen.first; id; id = id->next)
+	for (id = bmain->screen.first; id; id = id->next)
 		if (sc == (bScreen *)id)
 			break;
 	if (id == NULL)
 		return;
 	
-	/* check for valid winid */
-	if (sc->winid != 0 && sc->winid != win->winid)
-		return;
-	
-	if (sc->full) {             /* find associated full */
+
+	if (sc->state == SCREENFULL) {             /* find associated full */
 		bScreen *sc1;
-		for (sc1 = CTX_data_main(C)->screen.first; sc1; sc1 = sc1->id.next) {
+		for (sc1 = bmain->screen.first; sc1; sc1 = sc1->id.next) {
 			ScrArea *sa = sc1->areabase.first;
 			if (sa->full == sc) {
 				sc = sc1;
@@ -1354,10 +1495,15 @@ void ED_screen_set(bContext *C, bScreen *sc)
 			}
 		}
 	}
+
+	/* check for valid winid */
+	if (sc->winid != 0 && sc->winid != win->winid)
+		return;
 	
 	if (oldscreen != sc) {
 		wmTimer *wt = oldscreen->animtimer;
 		ScrArea *sa;
+		Scene *oldscene = oldscreen->scene;
 
 		/* remove handlers referencing areas in old screen */
 		for (sa = oldscreen->areabase.first; sa; sa = sa->next) {
@@ -1366,12 +1512,24 @@ void ED_screen_set(bContext *C, bScreen *sc)
 
 		/* we put timer to sleep, so screen_exit has to think there's no timer */
 		oldscreen->animtimer = NULL;
-		if (wt)
-			WM_event_timer_sleep(wm, win, wt, 1);
-		
+		if (wt) {
+			WM_event_timer_sleep(wm, win, wt, true);
+		}
+
 		ED_screen_exit(C, win, oldscreen);
-		oldscreen->animtimer = wt;
-		
+
+		/* Same scene, "transfer" playback to new screen. */
+		if (wt) {
+			if (oldscene == sc->scene) {
+				sc->animtimer = wt;
+			}
+			/* Else, stop playback. */
+			else {
+				oldscreen->animtimer = wt;
+				ED_screen_animation_play(C, 0, 0);
+			}
+		}
+
 		win->screen = sc;
 		CTX_wm_window_set(C, win);  // stores C->wm.screen... hrmf
 		
@@ -1384,6 +1542,26 @@ void ED_screen_set(bContext *C, bScreen *sc)
 		
 		/* makes button hilites work */
 		WM_event_add_mousemove(C);
+
+		/* Needed to make sure all the derivedMeshes are
+		 * up-to-date before viewport starts acquiring this.
+		 *
+		 * This is needed in cases when, for example, boolean
+		 * modifier uses operant from invisible layer.
+		 * Without this trick boolean wouldn't apply correct.
+		 *
+		 * Quite the same happens when setting screen's scene,
+		 * so perhaps this is in fact correct thing to do.
+		 */
+		if (oldscene != sc->scene) {
+			BKE_scene_set_background(bmain, sc->scene);
+		}
+
+		/* Always do visible update since it's possible new screen will
+		 * have different layers visible in 3D viewpots. This is possible
+		 * because of view3d.lock_camera_and_layers option.
+		 */
+		DAG_on_visible_update(bmain, false);
 	}
 }
 
@@ -1408,7 +1586,7 @@ void ED_screen_delete(bContext *C, bScreen *sc)
 	int delete = 1;
 	
 	/* don't allow deleting temp fullscreens for now */
-	if (sc->full == SCREENFULL) {
+	if (ELEM(sc->state, SCREENMAXIMIZED, SCREENFULL)) {
 		return;
 	}
 	
@@ -1433,7 +1611,37 @@ void ED_screen_delete(bContext *C, bScreen *sc)
 	ED_screen_set(C, newsc);
 
 	if (delete && win->screen != sc)
-		BKE_libblock_free(&bmain->screen, sc);
+		BKE_libblock_free(bmain, sc);
+}
+
+static void ed_screen_set_3dview_camera(Scene *scene, bScreen *sc, ScrArea *sa, View3D *v3d)
+{
+	/* fix any cameras that are used in the 3d view but not in the scene */
+	BKE_screen_view3d_sync(v3d, scene);
+
+	if (!v3d->camera || !BKE_scene_base_find(scene, v3d->camera)) {
+		v3d->camera = BKE_scene_camera_find(sc->scene);
+		// XXX if (sc == curscreen) handle_view3d_lock();
+		if (!v3d->camera) {
+			ARegion *ar;
+			ListBase *regionbase;
+			
+			/* regionbase is in different place depending if space is active */
+			if (v3d == sa->spacedata.first)
+				regionbase = &sa->regionbase;
+			else
+				regionbase = &v3d->regionbase;
+				
+			for (ar = regionbase->first; ar; ar = ar->next) {
+				if (ar->regiontype == RGN_TYPE_WINDOW) {
+					RegionView3D *rv3d = ar->regiondata;
+					if (rv3d->persp == RV3D_CAMOB) {
+						rv3d->persp = RV3D_PERSP;
+					}
+				}
+			}
+		}
+	}
 }
 
 /* only call outside of area/region loops */
@@ -1446,17 +1654,17 @@ void ED_screen_set_scene(bContext *C, bScreen *screen, Scene *scene)
 		return;
 	
 	if (ed_screen_used(CTX_wm_manager(C), screen))
-		ED_object_exit_editmode(C, EM_FREEDATA | EM_DO_UNDO);
+		ED_object_editmode_exit(C, EM_FREEDATA | EM_DO_UNDO);
 
-	for (sc = CTX_data_main(C)->screen.first; sc; sc = sc->id.next) {
+	for (sc = bmain->screen.first; sc; sc = sc->id.next) {
 		if ((U.flag & USER_SCENEGLOBAL) || sc == screen) {
 			
 			if (scene != sc->scene) {
 				/* all areas endlocalview */
-				// XXX	ScrArea *sa= sc->areabase.first;
+				// XXX	ScrArea *sa = sc->areabase.first;
 				//	while (sa) {
 				//		endlocalview(sa);
-				//		sa= sa->next;
+				//		sa = sa->next;
 				//	}
 				sc->scene = scene;
 			}
@@ -1467,7 +1675,7 @@ void ED_screen_set_scene(bContext *C, bScreen *screen, Scene *scene)
 	//  copy_view3d_lock(0);	/* space.c */
 	
 	/* are there cameras in the views that are not in the scene? */
-	for (sc = CTX_data_main(C)->screen.first; sc; sc = sc->id.next) {
+	for (sc = bmain->screen.first; sc; sc = sc->id.next) {
 		if ((U.flag & USER_SCENEGLOBAL) || sc == screen) {
 			ScrArea *sa = sc->areabase.first;
 			while (sa) {
@@ -1475,24 +1683,8 @@ void ED_screen_set_scene(bContext *C, bScreen *screen, Scene *scene)
 				while (sl) {
 					if (sl->spacetype == SPACE_VIEW3D) {
 						View3D *v3d = (View3D *) sl;
+						ed_screen_set_3dview_camera(scene, sc, sa, v3d);
 
-						BKE_screen_view3d_sync(v3d, scene);
-
-						if (!v3d->camera || !BKE_scene_base_find(scene, v3d->camera)) {
-							v3d->camera = BKE_scene_camera_find(sc->scene);
-							// XXX if (sc == curscreen) handle_view3d_lock();
-							if (!v3d->camera) {
-								ARegion *ar;
-								for (ar = v3d->regionbase.first; ar; ar = ar->next) {
-									if (ar->regiontype == RGN_TYPE_WINDOW) {
-										RegionView3D *rv3d = ar->regiondata;
-
-										if (rv3d->persp == RV3D_CAMOB)
-											rv3d->persp = RV3D_PERSP;
-									}
-								}
-							}
-						}
 					}
 					sl = sl->next;
 				}
@@ -1503,7 +1695,7 @@ void ED_screen_set_scene(bContext *C, bScreen *screen, Scene *scene)
 	
 	CTX_data_scene_set(C, scene);
 	BKE_scene_set_background(bmain, scene);
-	DAG_on_visible_update(bmain, FALSE);
+	DAG_on_visible_update(bmain, false);
 	
 	ED_render_engine_changed(bmain);
 	ED_update_for_newframe(bmain, scene, 1);
@@ -1538,11 +1730,11 @@ ScrArea *ED_screen_full_newspace(bContext *C, ScrArea *sa, int type)
 	ScrArea *newsa = NULL;
 
 	if (!sa || sa->full == NULL) {
-		newsa = ED_screen_full_toggle(C, win, sa);
+		newsa = ED_screen_state_toggle(C, win, sa, SCREENMAXIMIZED);
 	}
 	
 	if (!newsa) {
-		if (sa->full) {
+		if (sa->full && (screen->state == SCREENMAXIMIZED)) {
 			/* if this has been called from the temporary info header generated in
 			 * temp fullscreen layouts, find the correct fullscreen area to change
 			 * to create a new space inside */
@@ -1568,7 +1760,7 @@ void ED_screen_full_prevspace(bContext *C, ScrArea *sa)
 	ED_area_prevspace(C, sa);
 	
 	if (sa->full)
-		ED_screen_full_toggle(C, win, sa);
+		ED_screen_state_toggle(C, win, sa, SCREENMAXIMIZED);
 }
 
 /* restore a screen / area back to default operation, after temp fullscreen modes */
@@ -1576,6 +1768,8 @@ void ED_screen_full_restore(bContext *C, ScrArea *sa)
 {
 	wmWindow *win = CTX_wm_window(C);
 	SpaceLink *sl = sa->spacedata.first;
+	bScreen *screen = CTX_wm_screen(C);
+	short state = (screen ? screen->state : SCREENMAXIMIZED);
 	
 	/* if fullscreen area has a secondary space (such as a file browser or fullscreen render 
 	 * overlaid on top of a existing setup) then return to the previous space */
@@ -1583,39 +1777,33 @@ void ED_screen_full_restore(bContext *C, ScrArea *sa)
 	if (sl->next) {
 		/* specific checks for space types */
 
-		int sima_restore = 0;
-
 		/* Special check added for non-render image window (back from fullscreen through "Back to Previous" button) */
 		if (sl->spacetype == SPACE_IMAGE) {
 			SpaceImage *sima = sa->spacedata.first;
-			if (!(sima->flag & SI_PREVSPACE) && !(sima->flag & SI_FULLWINDOW))
-				sima_restore = 1;
-		}
 
-		if (sl->spacetype == SPACE_IMAGE && !sima_restore) {
-			SpaceImage *sima = sa->spacedata.first;
-			if (sima->flag & SI_PREVSPACE)
+			if (sima->flag & (SI_PREVSPACE | SI_FULLWINDOW)) {
 				sima->flag &= ~SI_PREVSPACE;
-			if (sima->flag & SI_FULLWINDOW) {
 				sima->flag &= ~SI_FULLWINDOW;
 				ED_screen_full_prevspace(C, sa);
 			}
+			else
+				ED_screen_state_toggle(C, win, sa, state);
 		}
 		else if (sl->spacetype == SPACE_FILE) {
 			ED_screen_full_prevspace(C, sa);
 		}
 		else {
-			ED_screen_full_toggle(C, win, sa);
+			ED_screen_state_toggle(C, win, sa, state);
 		}
 	}
 	/* otherwise just tile the area again */
 	else {
-		ED_screen_full_toggle(C, win, sa);
+		ED_screen_state_toggle(C, win, sa, state);
 	}
 }
 
-/* this function toggles: if area is full then the parent will be restored */
-ScrArea *ED_screen_full_toggle(bContext *C, wmWindow *win, ScrArea *sa)
+/* this function toggles: if area is maximized/full then the parent will be restored */
+ScrArea *ED_screen_state_toggle(bContext *C, wmWindow *win, ScrArea *sa, const short state)
 {
 	bScreen *sc, *oldscreen;
 	ARegion *ar;
@@ -1625,24 +1813,21 @@ ScrArea *ED_screen_full_toggle(bContext *C, wmWindow *win, ScrArea *sa)
 		 * switching screens with tooltip open because region and tooltip
 		 * are no longer in the same screen */
 		for (ar = sa->regionbase.first; ar; ar = ar->next)
-			uiFreeBlocks(C, &ar->uiblocks);
-		
+			UI_blocklist_free(C, &ar->uiblocks);
+
 		/* prevent hanging header prints */
 		ED_area_headerprint(sa, NULL);
 	}
 
 	if (sa && sa->full) {
+		/* restoring back to SCREENNORMAL */
 		ScrArea *old;
-		/*short fulltype;*/ /*UNUSED*/
 
 		sc = sa->full;       /* the old screen to restore */
 		oldscreen = win->screen; /* the one disappearing */
 
-		/*fulltype = sc->full;*/
-		sc->full = 0;
+		sc->state = SCREENNORMAL;
 
-		/* removed: SCREENAUTOPLAY exception here */
-	
 		/* find old area */
 		for (old = sc->areabase.first; old; old = old->next)
 			if (old->full) break;
@@ -1652,7 +1837,14 @@ ScrArea *ED_screen_full_toggle(bContext *C, wmWindow *win, ScrArea *sa)
 			return NULL;
 		}
 
-		area_copy_data(old, sa, 1); /*  1 = swap spacelist */
+		if (state == SCREENFULL) {
+			/* restore the old side panels/header visibility */
+			for (ar = sa->regionbase.first; ar; ar = ar->next) {
+				ar->flag = ar->flagfullscreen;
+			}
+		}
+
+		ED_area_data_swap(old, sa);
 		if (sa->flag & AREA_TEMP_INFO) sa->flag &= ~AREA_TEMP_INFO;
 		old->full = NULL;
 
@@ -1663,48 +1855,70 @@ ScrArea *ED_screen_full_toggle(bContext *C, wmWindow *win, ScrArea *sa)
 		ED_screen_set(C, sc);
 
 		BKE_screen_free(oldscreen);
-		BKE_libblock_free(&CTX_data_main(C)->screen, oldscreen);
+		BKE_libblock_free(CTX_data_main(C), oldscreen);
 
 	}
 	else {
+		/* change from SCREENNORMAL to new state */
 		ScrArea *newa;
 		char newname[MAX_ID_NAME - 2];
 
 		oldscreen = win->screen;
 
-		/* nothing wrong with having only 1 area, as far as I can see...
-		 * is there only 1 area? */
-#if 0
-		if (oldscreen->areabase.first == oldscreen->areabase.last)
-			return NULL;
-#endif
-
-		oldscreen->full = SCREENFULL;
-		BLI_snprintf(newname, sizeof(newname), "%s-%s", oldscreen->id.name + 2, "full");
+		oldscreen->state = state;
+		BLI_snprintf(newname, sizeof(newname), "%s-%s", oldscreen->id.name + 2, "nonnormal");
 		sc = ED_screen_add(win, oldscreen->scene, newname);
-		sc->full = SCREENFULL; // XXX
+		sc->state = state;
 
 		/* timer */
 		sc->animtimer = oldscreen->animtimer;
 		oldscreen->animtimer = NULL;
 
-		/* returns the top small area */
-		newa = area_split(sc, (ScrArea *)sc->areabase.first, 'h', 0.99f, 1);
-		ED_area_newspace(C, newa, SPACE_INFO);
-
 		/* use random area when we have no active one, e.g. when the
 		 * mouse is outside of the window and we open a file browser */
 		if (!sa)
 			sa = oldscreen->areabase.first;
 
-		/* copy area */
-		newa = newa->prev;
-		area_copy_data(newa, sa, 1);  /* 1 = swap spacelist */
-		sa->flag |= AREA_TEMP_INFO;
+		if (state == SCREENMAXIMIZED) {
+			/* returns the top small area */
+			newa = area_split(sc, (ScrArea *)sc->areabase.first, 'h', 0.99f, 1);
+			ED_area_newspace(C, newa, SPACE_INFO);
 
-		sa->full = oldscreen;
-		newa->full = oldscreen;
-		newa->next->full = oldscreen; // XXX
+			/* copy area */
+			newa = newa->prev;
+			ED_area_data_swap(newa, sa);
+			sa->flag |= AREA_TEMP_INFO;
+
+			sa->full = oldscreen;
+			newa->full = oldscreen;
+			newa->next->full = oldscreen; // XXX
+		}
+		else if (state == SCREENFULL) {
+			newa = (ScrArea *)sc->areabase.first;
+
+			/* copy area */
+			ED_area_data_swap(newa, sa);
+			newa->flag = sa->flag; /* mostly for AREA_FLAG_WASFULLSCREEN */
+
+			/* temporarily hide the side panels/header */
+			for (ar = newa->regionbase.first; ar; ar = ar->next) {
+				ar->flagfullscreen = ar->flag;
+
+				if (ELEM(ar->regiontype,
+				         RGN_TYPE_UI,
+				         RGN_TYPE_HEADER,
+				         RGN_TYPE_TOOLS))
+				{
+					ar->flag |= RGN_FLAG_HIDDEN;
+				}
+			}
+
+			sa->full = oldscreen;
+			newa->full = oldscreen;
+		}
+		else {
+			BLI_assert(false);
+		}
 
 		ED_screen_set(C, sc);
 	}
@@ -1733,7 +1947,7 @@ void ED_refresh_viewport_fps(bContext *C)
 		fpsi->redrawtime = fpsi->lredrawtime;
 		fpsi->lredrawtime = animtimer->ltime;
 	}
-	else {	
+	else {
 		/* playback stopped or shouldn't be running */
 		if (scene->fps_info)
 			MEM_freeN(scene->fps_info);
@@ -1854,10 +2068,6 @@ void ED_update_for_newframe(Main *bmain, Scene *scene, int UNUSED(mute))
 
 	//extern void audiostream_scrub(unsigned int frame);	/* seqaudio.c */
 	
-	/* update animated image textures for gpu, etc,
-	 * call before BKE_scene_update_for_newframe so modifiers with textures don't lag 1 frame */
-	ED_image_update_frame(bmain, scene->r.cfra);
-
 	ED_clip_update_frame(bmain, scene->r.cfra);
 
 	/* get layers from all windows */
@@ -1865,7 +2075,7 @@ void ED_update_for_newframe(Main *bmain, Scene *scene, int UNUSED(mute))
 		layers |= BKE_screen_visible_layers(window->screen, scene);
 
 	/* this function applies the changes too */
-	BKE_scene_update_for_newframe(bmain, scene, layers); /* BKE_scene.h */
+	BKE_scene_update_for_newframe(bmain->eval_ctx, bmain, scene, layers);
 	
 	//if ( (CFRA>1) && (!mute) && (scene->r.audio.flag & AUDIO_SCRUB)) 
 	//	audiostream_scrub( CFRA );
