@@ -35,6 +35,7 @@
 
 #include "BLI_blenlib.h"
 #include "BLI_utildefines.h"
+#include "BLI_math.h"
 
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"
@@ -52,6 +53,7 @@
 #include "BKE_writeavi.h"
 
 #include "BIF_gl.h"
+#include "BIF_glutil.h"
 
 #include "RNA_access.h"
 #include "RNA_define.h"
@@ -63,7 +65,6 @@
 
 #include "PIL_time.h"
 
-#include "ED_screen_types.h"
 
 #include "screen_intern.h"
 
@@ -75,6 +76,18 @@ typedef struct ScreenshotData {
 	ImageFormatData im_format;
 } ScreenshotData;
 
+static void screenshot_read_pixels(int x, int y, int w, int h, unsigned char *rect)
+{
+	int i;
+
+	glReadPixels(x, y, w, h, GL_RGBA, GL_UNSIGNED_BYTE, rect);
+	glFinish();
+
+	/* clear alpha, it is not set to a meaningful value in opengl */
+	for (i = 0, rect += 3; i < w * h; i++, rect += 4)
+		*rect = 255;
+}
+
 /* get shot from frontbuffer */
 static unsigned int *screenshot(bContext *C, int *dumpsx, int *dumpsy)
 {
@@ -84,15 +97,14 @@ static unsigned int *screenshot(bContext *C, int *dumpsx, int *dumpsy)
 
 	x = 0;
 	y = 0;
-	*dumpsx = win->sizex;
-	*dumpsy = win->sizey;
+	*dumpsx = WM_window_pixels_x(win);
+	*dumpsy = WM_window_pixels_y(win);
 
 	if (*dumpsx && *dumpsy) {
 		
 		dumprect = MEM_mallocN(sizeof(int) * (*dumpsx) * (*dumpsy), "dumprect");
 		glReadBuffer(GL_FRONT);
-		glReadPixels(x, y, *dumpsx, *dumpsy, GL_RGBA, GL_UNSIGNED_BYTE, dumprect);
-		glFinish();
+		screenshot_read_pixels(x, y, *dumpsx, *dumpsy, (unsigned char *)dumprect);
 		glReadBuffer(GL_BACK);
 	}
 
@@ -125,11 +137,11 @@ static int screenshot_data_create(bContext *C, wmOperator *op)
 
 		op->customdata = scd;
 
-		return TRUE;
+		return true;
 	}
 	else {
 		op->customdata = NULL;
-		return FALSE;
+		return false;
 	}
 }
 
@@ -202,7 +214,7 @@ static int screenshot_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int screenshot_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int screenshot_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	if (screenshot_data_create(C, op)) {
 		if (RNA_struct_property_is_set(op->ptr, "filepath"))
@@ -214,27 +226,26 @@ static int screenshot_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event)
 		WM_event_add_fileselect(C, op);
 	
 		return OPERATOR_RUNNING_MODAL;
-	}	
+	}
 	return OPERATOR_CANCELLED;
 }
 
-static int screenshot_check(bContext *UNUSED(C), wmOperator *op)
+static bool screenshot_check(bContext *UNUSED(C), wmOperator *op)
 {
 	ScreenshotData *scd = op->customdata;
-	return WM_operator_filesel_ensure_ext_imtype(op, scd->im_format.imtype);
+	return WM_operator_filesel_ensure_ext_imtype(op, &scd->im_format);
 }
 
-static int screenshot_cancel(bContext *UNUSED(C), wmOperator *op)
+static void screenshot_cancel(bContext *UNUSED(C), wmOperator *op)
 {
 	screenshot_data_free(op);
-	return OPERATOR_CANCELLED;
 }
 
-static int screenshot_draw_check_prop(PointerRNA *UNUSED(ptr), PropertyRNA *prop)
+static bool screenshot_draw_check_prop(PointerRNA *UNUSED(ptr), PropertyRNA *prop)
 {
 	const char *prop_id = RNA_property_identifier(prop);
 
-	return !(strcmp(prop_id, "filepath") == 0);
+	return !(STREQ(prop_id, "filepath"));
 }
 
 static void screenshot_draw(bContext *UNUSED(C), wmOperator *op)
@@ -245,13 +256,20 @@ static void screenshot_draw(bContext *UNUSED(C), wmOperator *op)
 
 	/* image template */
 	RNA_pointer_create(NULL, &RNA_ImageFormatSettings, &scd->im_format, &ptr);
-	uiTemplateImageSettings(layout, &ptr, FALSE);
+	uiTemplateImageSettings(layout, &ptr, false);
 
 	/* main draw call */
 	RNA_pointer_create(NULL, op->type->srna, op->properties, &ptr);
 	uiDefAutoButsRNA(layout, &ptr, screenshot_draw_check_prop, '\0');
 }
 
+static int screenshot_poll(bContext *C)
+{
+	if (G.background)
+		return false;
+
+	return WM_operator_winactive(C);
+}
 
 void SCREEN_OT_screenshot(wmOperatorType *ot)
 {
@@ -264,13 +282,14 @@ void SCREEN_OT_screenshot(wmOperatorType *ot)
 	ot->exec = screenshot_exec;
 	ot->cancel = screenshot_cancel;
 	ot->ui = screenshot_draw;
-	ot->poll = WM_operator_winactive;
+	ot->poll = screenshot_poll;
 	
 	ot->flag = 0;
 	
 	WM_operator_properties_filesel(ot, FOLDERFILE | IMAGEFILE, FILE_SPECIAL, FILE_SAVE,
 	                               WM_FILESEL_FILEPATH, FILE_DEFAULTDISPLAY);
-	RNA_def_boolean(ot->srna, "full", 1, "Full Screen", "Screenshot the whole Blender window");
+	RNA_def_boolean(ot->srna, "full", 1, "Full Screen",
+	                "Capture the whole window (otherwise only capture the active area)");
 }
 
 /* *************** screenshot movie job ************************* */
@@ -278,10 +297,11 @@ void SCREEN_OT_screenshot(wmOperatorType *ot)
 typedef struct ScreenshotJob {
 	Main *bmain;
 	Scene *scene;
+	wmWindowManager *wm;
 	unsigned int *dumprect;
 	int x, y, dumpsx, dumpsy;
-	short *stop;
-	short *do_update;
+	const short *stop;
+	const short *do_update;
 	ReportList reports;
 } ScreenshotJob;
 
@@ -305,8 +325,7 @@ static void screenshot_updatejob(void *sjv)
 	
 	if (sj->dumprect == NULL) {
 		dumprect = MEM_mallocN(sizeof(int) * sj->dumpsx * sj->dumpsy, "dumprect");
-		glReadPixels(sj->x, sj->y, sj->dumpsx, sj->dumpsy, GL_RGBA, GL_UNSIGNED_BYTE, dumprect);
-		glFinish();
+		screenshot_read_pixels(sj->x, sj->y, sj->dumpsx, sj->dumpsy, (unsigned char *)dumprect);
 		
 		sj->dumprect = dumprect;
 	}
@@ -336,7 +355,7 @@ static void screenshot_startjob(void *sjv, short *stop, short *do_update, float
 	sj->stop = stop;
 	sj->do_update = do_update;
 	
-	*do_update = TRUE; /* wait for opengl rect */
+	*do_update = true; /* wait for opengl rect */
 	
 	while (*stop == 0) {
 		
@@ -358,14 +377,15 @@ static void screenshot_startjob(void *sjv, short *stop, short *do_update, float
 				char name[FILE_MAX];
 				int ok;
 				
-				BKE_makepicstring(name, rd.pic, sj->bmain->name, rd.cfra, rd.im_format.imtype, rd.scemode & R_EXTENSION, TRUE);
+				BKE_makepicstring(name, rd.pic, sj->bmain->name, rd.cfra,
+				                  &rd.im_format, (rd.scemode & R_EXTENSION) != 0, true);
 				
 				ibuf->rect = sj->dumprect;
 				ok = BKE_imbuf_write(ibuf, name, &rd.im_format);
 				
 				if (ok == 0) {
 					printf("Write error: cannot save %s\n", name);
-					BKE_reportf(&sj->reports, RPT_INFO, "Write error: cannot save %s\n", name);
+					BKE_reportf(&sj->reports, RPT_INFO, "Write error: cannot save %s", name);
 					break;
 				}
 				else {
@@ -374,13 +394,13 @@ static void screenshot_startjob(void *sjv, short *stop, short *do_update, float
 				}
 				
 				/* imbuf knows which rects are not part of ibuf */
-				IMB_freeImBuf(ibuf);	
+				IMB_freeImBuf(ibuf);
 			}
 			
 			MEM_freeN(sj->dumprect);
 			sj->dumprect = NULL;
 			
-			*do_update = TRUE;
+			*do_update = true;
 			
 			rd.cfra++;
 
@@ -395,20 +415,76 @@ static void screenshot_startjob(void *sjv, short *stop, short *do_update, float
 	BKE_report(&sj->reports, RPT_INFO, "Screencast job stopped");
 }
 
+/* Helper callback for drawing the cursor itself */
+static void screencast_draw_cursor(bContext *UNUSED(C), int x, int y, void *UNUSED(p_ptr))
+{
+	
+	glPushMatrix();
+	
+	glTranslatef((float)x, (float)y, 0.0f);
+	
+	
+	glEnable(GL_LINE_SMOOTH);
+	glEnable(GL_BLEND);
+	
+	glColor4ub(0, 0, 0, 32);
+	glutil_draw_filled_arc(0.0, M_PI * 2.0, 20, 40);
+	
+	glColor4ub(255, 255, 255, 128);
+	glutil_draw_lined_arc(0.0, M_PI * 2.0, 20, 40);
+	
+	glDisable(GL_BLEND);
+	glDisable(GL_LINE_SMOOTH);
+	
+	glPopMatrix();
+}
+
+/* Turn brush cursor in 3D view on/off */
+static void screencast_cursor_toggle(wmWindowManager *wm, short enable)
+{
+	static void *cursor = NULL;
+	
+	if (cursor && !enable) {
+		/* clear cursor */
+		WM_paint_cursor_end(wm, cursor);
+		cursor = NULL;
+	}
+	else if (enable) {
+		/* enable cursor */
+		cursor = WM_paint_cursor_activate(wm, NULL, screencast_draw_cursor, NULL);
+	}
+}
+
+static void screenshot_endjob(void *sjv)
+{
+	ScreenshotJob *sj = sjv;
+	
+	screencast_cursor_toggle(sj->wm, 0);
+}
+
+
 static int screencast_exec(bContext *C, wmOperator *op)
 {
+	wmWindowManager *wm = CTX_wm_manager(C);
+	wmWindow *win = CTX_wm_window(C);
 	bScreen *screen = CTX_wm_screen(C);
-	wmJob *wm_job = WM_jobs_get(CTX_wm_manager(C), CTX_wm_window(C), screen, "Screencast", 0, WM_JOB_TYPE_SCREENCAST);
-	ScreenshotJob *sj = MEM_callocN(sizeof(ScreenshotJob), "screenshot job");
+	wmJob *wm_job;
+	ScreenshotJob *sj;
 
+	/* if called again, stop the running job */
+	if (WM_jobs_test(wm, screen, WM_JOB_TYPE_SCREENCAST))
+		WM_jobs_stop(wm, screen, screenshot_startjob);
+	
+	wm_job = WM_jobs_get(wm, win, screen, "Screencast", 0, WM_JOB_TYPE_SCREENCAST);
+	sj = MEM_callocN(sizeof(ScreenshotJob), "screenshot job");
+	
 	/* setup sj */
 	if (RNA_boolean_get(op->ptr, "full")) {
-		wmWindow *win = CTX_wm_window(C);
 		sj->x = 0;
 		sj->y = 0;
-		sj->dumpsx = win->sizex;
-		sj->dumpsy = win->sizey;
-	} 
+		sj->dumpsx = WM_window_pixels_x(win);
+		sj->dumpsy = WM_window_pixels_y(win);
+	}
 	else {
 		ScrArea *curarea = CTX_wm_area(C);
 		sj->x = curarea->totrct.xmin;
@@ -418,15 +494,18 @@ static int screencast_exec(bContext *C, wmOperator *op)
 	}
 	sj->bmain = CTX_data_main(C);
 	sj->scene = CTX_data_scene(C);
-
+	sj->wm = wm;
+	
 	BKE_reports_init(&sj->reports, RPT_PRINT);
 
 	/* setup job */
 	WM_jobs_customdata_set(wm_job, sj, screenshot_freejob);
 	WM_jobs_timer(wm_job, 0.1, 0, NC_SCREEN | ND_SCREENCAST);
-	WM_jobs_callbacks(wm_job, screenshot_startjob, NULL, screenshot_updatejob, NULL);
+	WM_jobs_callbacks(wm_job, screenshot_startjob, NULL, screenshot_updatejob, screenshot_endjob);
 	
-	WM_jobs_start(CTX_wm_manager(C), wm_job);
+	WM_jobs_start(sj->wm, wm_job);
+	
+	screencast_cursor_toggle(sj->wm, 1);
 	
 	WM_event_add_notifier(C, NC_SCREEN | ND_SCREENCAST, screen);
 	
@@ -441,10 +520,11 @@ void SCREEN_OT_screencast(wmOperatorType *ot)
 	
 	ot->invoke = WM_operator_confirm;
 	ot->exec = screencast_exec;
-	ot->poll = WM_operator_winactive;
+	ot->poll = screenshot_poll;  /* shared poll */
 	
 	ot->flag = 0;
 	
 	RNA_def_property(ot->srna, "filepath", PROP_STRING, PROP_FILEPATH);
-	RNA_def_boolean(ot->srna, "full", 1, "Full Screen", "Screencast the whole Blender window");
+	RNA_def_boolean(ot->srna, "full", 1, "Full Screen",
+	                "Capture the whole window (otherwise only capture the active area)");
 }
