@@ -40,17 +40,16 @@
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_brush_types.h"
-#include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 
 #include "BKE_brush.h"
 #include "BKE_paint.h"
+#include "BKE_colortools.h"
 #include "BKE_context.h"
-#include "BKE_main.h"
 #include "BKE_depsgraph.h"
-#include "BKE_mesh.h"
+#include "BKE_mesh_mapping.h"
 #include "BKE_customdata.h"
-#include "BKE_tessmesh.h"
+#include "BKE_editmesh.h"
 
 #include "ED_screen.h"
 #include "ED_image.h"
@@ -61,11 +60,13 @@
 
 #include "RNA_access.h"
 #include "RNA_define.h"
-#include "RNA_enum_types.h"
 
 #include "paint_intern.h"
 #include "uvedit_intern.h"
 
+#include "BIF_gl.h"
+#include "BIF_glutil.h"
+
 #include "UI_view2d.h"
 
 #define MARK_BOUNDARY   1
@@ -144,6 +145,115 @@ typedef struct UvSculptData {
 	char invert;
 } UvSculptData;
 
+
+static Brush *uv_sculpt_brush(bContext *C)
+{
+	Scene *scene = CTX_data_scene(C);
+	ToolSettings *settings = scene->toolsettings;
+
+	if (!settings->uvsculpt)
+		return NULL;
+	return BKE_paint_brush(&settings->uvsculpt->paint);
+}
+
+
+static int uv_sculpt_brush_poll(bContext *C)
+{
+	BMEditMesh *em;
+	int ret;
+	Object *obedit = CTX_data_edit_object(C);
+	SpaceImage *sima = CTX_wm_space_image(C);
+	Scene *scene = CTX_data_scene(C);
+	ToolSettings *toolsettings = scene->toolsettings;
+
+	if (!uv_sculpt_brush(C) || !obedit || obedit->type != OB_MESH ||
+	    !sima || ED_space_image_show_render(sima) || (sima->mode == SI_MODE_PAINT))
+	{
+		return 0;
+	}
+
+	em = BKE_editmesh_from_object(obedit);
+	ret = EDBM_mtexpoly_check(em);
+
+	if (ret) {
+		ARegion *ar = CTX_wm_region(C);
+		if ((toolsettings->use_uv_sculpt) && ar->regiontype == RGN_TYPE_WINDOW)
+			return 1;
+	}
+
+	return 0;
+}
+
+static void brush_drawcursor_uvsculpt(bContext *C, int x, int y, void *UNUSED(customdata))
+{
+#define PX_SIZE_FADE_MAX 12.0f
+#define PX_SIZE_FADE_MIN 4.0f
+
+	Scene *scene = CTX_data_scene(C);
+	//Brush *brush = image_paint_brush(C);
+	Paint *paint = BKE_paint_get_active_from_context(C);
+	Brush *brush = BKE_paint_brush(paint);
+
+	if (paint && brush && paint->flags & PAINT_SHOW_BRUSH) {
+		const float size = (float)BKE_brush_size_get(scene, brush);
+		float alpha = 0.5f;
+
+		/* fade out the brush (cheap trick to work around brush interfering with sampling [#])*/
+		if (size < PX_SIZE_FADE_MIN) {
+			return;
+		}
+		else if (size < PX_SIZE_FADE_MAX) {
+			alpha *= (size - PX_SIZE_FADE_MIN) / (PX_SIZE_FADE_MAX - PX_SIZE_FADE_MIN);
+		}
+
+		glPushMatrix();
+
+		glTranslatef((float)x, (float)y, 0.0f);
+
+		glColor4f(brush->add_col[0], brush->add_col[1], brush->add_col[2], alpha);
+		glEnable(GL_LINE_SMOOTH);
+		glEnable(GL_BLEND);
+		glutil_draw_lined_arc(0, (float)(M_PI * 2.0), size, 40);
+		glDisable(GL_BLEND);
+		glDisable(GL_LINE_SMOOTH);
+
+		glPopMatrix();
+	}
+#undef PX_SIZE_FADE_MAX
+#undef PX_SIZE_FADE_MIN
+}
+
+
+void ED_space_image_uv_sculpt_update(wmWindowManager *wm, ToolSettings *settings)
+{
+	if (settings->use_uv_sculpt) {
+		if (!settings->uvsculpt) {
+			settings->uvsculpt = MEM_callocN(sizeof(*settings->uvsculpt), "UV Smooth paint");
+			settings->uv_sculpt_tool = UV_SCULPT_TOOL_GRAB;
+			settings->uv_sculpt_settings = UV_SCULPT_LOCK_BORDERS | UV_SCULPT_ALL_ISLANDS;
+			settings->uv_relax_method = UV_SCULPT_TOOL_RELAX_LAPLACIAN;
+			/* Uv sculpting does not include explicit brush view control yet, always enable */
+			settings->uvsculpt->paint.flags |= PAINT_SHOW_BRUSH;
+		}
+
+		BKE_paint_init(&settings->uvsculpt->paint, PAINT_CURSOR_SCULPT);
+
+		settings->uvsculpt->paint.paint_cursor = WM_paint_cursor_activate(wm, uv_sculpt_brush_poll,
+		                                                                  brush_drawcursor_uvsculpt, NULL);
+	}
+	else {
+		if (settings->uvsculpt) {
+			WM_paint_cursor_end(wm, settings->uvsculpt->paint.paint_cursor);
+			settings->uvsculpt->paint.paint_cursor = NULL;
+		}
+	}
+}
+
+int uv_sculpt_poll(bContext *C)
+{
+	return uv_sculpt_brush_poll(C);
+}
+
 /*********** Improved Laplacian Relaxation Operator ************************/
 /* original code by Raul Fernandez Hernandez "farsthary"                   *
  * adapted to uv smoothing by Antony Riakiatakis                           *
@@ -162,8 +272,8 @@ static void HC_relaxation_iteration_uv(BMEditMesh *em, UvSculptData *sculptdata,
 	Temp_UVData *tmp_uvdata;
 	float diff[2];
 	int i;
-	float radius_root = sqrt(radius);
-	Brush *brush = paint_brush(sculptdata->uvsculpt);
+	float radius_root = sqrtf(radius);
+	Brush *brush = BKE_paint_brush(sculptdata->uvsculpt);
 
 	tmp_uvdata = (Temp_UVData *)MEM_callocN(sculptdata->totalUniqueUvs * sizeof(Temp_UVData), "Temporal data");
 
@@ -205,7 +315,7 @@ static void HC_relaxation_iteration_uv(BMEditMesh *em, UvSculptData *sculptdata,
 		if ((dist = dot_v2v2(diff, diff)) <= radius) {
 			UvElement *element;
 			float strength;
-			strength = alpha * BKE_brush_curve_strength(brush, sqrt(dist), radius_root);
+			strength = alpha * BKE_brush_curve_strength(brush, sqrtf(dist), radius_root);
 
 			sculptdata->uv[i].uv[0] = (1.0f - strength) * sculptdata->uv[i].uv[0] + strength * (tmp_uvdata[i].p[0] - 0.5f * (tmp_uvdata[i].b[0] + tmp_uvdata[i].sum_b[0] / tmp_uvdata[i].ncounter));
 			sculptdata->uv[i].uv[1] = (1.0f - strength) * sculptdata->uv[i].uv[1] + strength * (tmp_uvdata[i].p[1] - 0.5f * (tmp_uvdata[i].b[1] + tmp_uvdata[i].sum_b[1] / tmp_uvdata[i].ncounter));
@@ -234,8 +344,8 @@ static void laplacian_relaxation_iteration_uv(BMEditMesh *em, UvSculptData *scul
 	Temp_UVData *tmp_uvdata;
 	float diff[2];
 	int i;
-	float radius_root = sqrt(radius);
-	Brush *brush = paint_brush(sculptdata->uvsculpt);
+	float radius_root = sqrtf(radius);
+	Brush *brush = BKE_paint_brush(sculptdata->uvsculpt);
 
 	tmp_uvdata = (Temp_UVData *)MEM_callocN(sculptdata->totalUniqueUvs * sizeof(Temp_UVData), "Temporal data");
 
@@ -269,7 +379,7 @@ static void laplacian_relaxation_iteration_uv(BMEditMesh *em, UvSculptData *scul
 		if ((dist = dot_v2v2(diff, diff)) <= radius) {
 			UvElement *element;
 			float strength;
-			strength = alpha * BKE_brush_curve_strength(brush, sqrt(dist), radius_root);
+			strength = alpha * BKE_brush_curve_strength(brush, sqrtf(dist), radius_root);
 
 			sculptdata->uv[i].uv[0] = (1.0f - strength) * sculptdata->uv[i].uv[0] + strength * tmp_uvdata[i].p[0];
 			sculptdata->uv[i].uv[1] = (1.0f - strength) * sculptdata->uv[i].uv[1] + strength * tmp_uvdata[i].p[1];
@@ -294,12 +404,12 @@ static void laplacian_relaxation_iteration_uv(BMEditMesh *em, UvSculptData *scul
 }
 
 
-static void uv_sculpt_stroke_apply(bContext *C, wmOperator *op, wmEvent *event, Object *obedit)
+static void uv_sculpt_stroke_apply(bContext *C, wmOperator *op, const wmEvent *event, Object *obedit)
 {
 	float co[2], radius, radius_root;
 	Scene *scene = CTX_data_scene(C);
 	ARegion *ar = CTX_wm_region(C);
-	BMEditMesh *em = BMEdit_FromObject(obedit);
+	BMEditMesh *em = BKE_editmesh_from_object(obedit);
 	unsigned int tool;
 	UvSculptData *sculptdata = (UvSculptData *)op->customdata;
 	SpaceImage *sima;
@@ -307,7 +417,7 @@ static void uv_sculpt_stroke_apply(bContext *C, wmOperator *op, wmEvent *event,
 	int width, height;
 	float aspectRatio;
 	float alpha, zoomx, zoomy;
-	Brush *brush = paint_brush(sculptdata->uvsculpt);
+	Brush *brush = BKE_paint_brush(sculptdata->uvsculpt);
 	ToolSettings *toolsettings = CTX_data_tool_settings(C);
 	tool = sculptdata->tool;
 	invert = sculptdata->invert ? -1 : 1;
@@ -323,7 +433,7 @@ static void uv_sculpt_stroke_apply(bContext *C, wmOperator *op, wmEvent *event,
 
 	/* We will compare squares to save some computation */
 	radius = radius * radius;
-	radius_root = sqrt(radius);
+	radius_root = sqrtf(radius);
 
 	/*
 	 * Pinch Tool
@@ -344,7 +454,7 @@ static void uv_sculpt_stroke_apply(bContext *C, wmOperator *op, wmEvent *event,
 			if ((dist = dot_v2v2(diff, diff)) <= radius) {
 				UvElement *element;
 				float strength;
-				strength = alpha * BKE_brush_curve_strength(brush, sqrt(dist), radius_root);
+				strength = alpha * BKE_brush_curve_strength(brush, sqrtf(dist), radius_root);
 				normalize_v2(diff);
 
 				sculptdata->uv[i].uv[0] -= strength * diff[0] * 0.001f;
@@ -416,7 +526,7 @@ static void uv_sculpt_stroke_exit(bContext *C, wmOperator *op)
 		WM_event_remove_timer(CTX_wm_manager(C), CTX_wm_window(C), data->timer);
 	}
 	if (data->elementMap) {
-		EDBM_uv_element_map_free(data->elementMap);
+		BM_uv_element_map_free(data->elementMap);
 	}
 	if (data->uv) {
 		MEM_freeN(data->uv);
@@ -435,9 +545,9 @@ static void uv_sculpt_stroke_exit(bContext *C, wmOperator *op)
 	op->customdata = NULL;
 }
 
-static int uv_element_offset_from_face_get(UvElementMap *map, BMFace *efa, BMLoop *l, int island_index, int doIslands)
+static int uv_element_offset_from_face_get(UvElementMap *map, BMFace *efa, BMLoop *l, int island_index, const bool doIslands)
 {
-	UvElement *element = ED_uv_element_get(map, efa, l);
+	UvElement *element = BM_uv_element_get(map, efa, l);
 	if (!element || (doIslands && element->island != island_index)) {
 		return -1;
 	}
@@ -448,11 +558,11 @@ static int uv_element_offset_from_face_get(UvElementMap *map, BMFace *efa, BMLoo
 static unsigned int uv_edge_hash(const void *key)
 {
 	UvEdge *edge = (UvEdge *)key;
-	return (BLI_ghashutil_inthash(SET_INT_IN_POINTER(edge->uv2)) +
-	        BLI_ghashutil_inthash(SET_INT_IN_POINTER(edge->uv1)));
+	return (BLI_ghashutil_uinthash(edge->uv2) +
+	        BLI_ghashutil_uinthash(edge->uv1));
 }
 
-static int uv_edge_compare(const void *a, const void *b)
+static bool uv_edge_compare(const void *a, const void *b)
 {
 	UvEdge *edge1 = (UvEdge *)a;
 	UvEdge *edge2 = (UvEdge *)b;
@@ -464,17 +574,19 @@ static int uv_edge_compare(const void *a, const void *b)
 }
 
 
-static UvSculptData *uv_sculpt_stroke_init(bContext *C, wmOperator *op, wmEvent *event)
+static UvSculptData *uv_sculpt_stroke_init(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	Scene *scene = CTX_data_scene(C);
 	Object *obedit = CTX_data_edit_object(C);
 	ToolSettings *ts = scene->toolsettings;
 	UvSculptData *data = MEM_callocN(sizeof(*data), "UV Smooth Brush Data");
-	BMEditMesh *em = BMEdit_FromObject(obedit);
+	BMEditMesh *em = BKE_editmesh_from_object(obedit);
 	BMesh *bm = em->bm;
 
 	op->customdata = data;
 
+	curvemapping_initialize(ts->uvsculpt->paint.brush->curve);
+
 	if (data) {
 		int counter = 0, i;
 		ARegion *ar = CTX_wm_region(C);
@@ -488,7 +600,7 @@ static UvSculptData *uv_sculpt_stroke_init(bContext *C, wmOperator *op, wmEvent
 		GHash *edgeHash;
 		GHashIterator *ghi;
 
-		int do_island_optimization = !(ts->uv_sculpt_settings & UV_SCULPT_ALL_ISLANDS);
+		bool do_island_optimization = !(ts->uv_sculpt_settings & UV_SCULPT_ALL_ISLANDS);
 		int island_index = 0;
 		/* Holds, for each UvElement in elementMap, a pointer to its unique uv.*/
 		int *uniqueUv;
@@ -500,18 +612,18 @@ static UvSculptData *uv_sculpt_stroke_init(bContext *C, wmOperator *op, wmEvent
 		if (do_island_optimization) {
 			/* We will need island information */
 			if (ts->uv_flag & UV_SYNC_SELECTION) {
-				data->elementMap = EDBM_uv_element_map_create(em, 0, 1);
+				data->elementMap = BM_uv_element_map_create(bm, false, true);
 			}
 			else {
-				data->elementMap = EDBM_uv_element_map_create(em, 1, 1);
+				data->elementMap = BM_uv_element_map_create(bm, true, true);
 			}
 		}
 		else {
 			if (ts->uv_flag & UV_SYNC_SELECTION) {
-				data->elementMap = EDBM_uv_element_map_create(em, 0, 0);
+				data->elementMap = BM_uv_element_map_create(bm, false, false);
 			}
 			else {
-				data->elementMap = EDBM_uv_element_map_create(em, 1, 0);
+				data->elementMap = BM_uv_element_map_create(bm, true, false);
 			}
 		}
 
@@ -530,7 +642,7 @@ static UvSculptData *uv_sculpt_stroke_init(bContext *C, wmOperator *op, wmEvent
 			Image *ima = CTX_data_edit_image(C);
 			uv_find_nearest_vert(scene, ima, em, co, NULL, &hit);
 
-			element = ED_uv_element_get(data->elementMap, hit.efa, hit.l);
+			element = BM_uv_element_get(data->elementMap, hit.efa, hit.l);
 			island_index = element->island;
 		}
 
@@ -558,7 +670,7 @@ static UvSculptData *uv_sculpt_stroke_init(bContext *C, wmOperator *op, wmEvent
 				MEM_freeN(uniqueUv);
 			}
 			if (edgeHash) {
-				MEM_freeN(edgeHash);
+				BLI_ghash_free(edgeHash, NULL, NULL);
 			}
 			uv_sculpt_stroke_exit(C, op);
 			return NULL;
@@ -599,6 +711,7 @@ static UvSculptData *uv_sculpt_stroke_init(bContext *C, wmOperator *op, wmEvent
 			BM_ITER_ELEM (l, &liter, efa, BM_LOOPS_OF_FACE) {
 				int offset1, itmp1 = uv_element_offset_from_face_get(data->elementMap, efa, l, island_index, do_island_optimization);
 				int offset2, itmp2 = uv_element_offset_from_face_get(data->elementMap, efa, l->next, island_index, do_island_optimization);
+				char *flag;
 
 				/* Skip edge if not found(unlikely) or not on valid island */
 				if (itmp1 == -1 || itmp2 == -1)
@@ -619,8 +732,8 @@ static UvSculptData *uv_sculpt_stroke_init(bContext *C, wmOperator *op, wmEvent
 					edges[counter].uv2 = offset1;
 				}
 				/* Hack! Set the value of the key to its flag. Now we can set the flag when an edge exists twice :) */
-				if (BLI_ghash_haskey(edgeHash, &edges[counter])) {
-					char *flag = BLI_ghash_lookup(edgeHash, &edges[counter]);
+				flag = BLI_ghash_lookup(edgeHash, &edges[counter]);
+				if (flag) {
 					*flag = 1;
 				}
 				else {
@@ -649,7 +762,7 @@ static UvSculptData *uv_sculpt_stroke_init(bContext *C, wmOperator *op, wmEvent
 			return NULL;
 		}
 		/* fill the edges with data */
-		for (i = 0; !BLI_ghashIterator_isDone(ghi); BLI_ghashIterator_step(ghi)) {
+		for (i = 0; !BLI_ghashIterator_done(ghi); BLI_ghashIterator_step(ghi)) {
 			data->uvedges[i++] = *((UvEdge *)BLI_ghashIterator_getKey(ghi));
 		}
 		data->totalUvEdges = BLI_ghash_size(edgeHash);
@@ -677,7 +790,7 @@ static UvSculptData *uv_sculpt_stroke_init(bContext *C, wmOperator *op, wmEvent
 			int width, height;
 			float aspectRatio;
 			float alpha, zoomx, zoomy;
-			Brush *brush = paint_brush(sculptdata->uvsculpt);
+			Brush *brush = BKE_paint_brush(sculptdata->uvsculpt);
 
 			alpha = BKE_brush_alpha_get(scene, brush);
 
@@ -689,7 +802,7 @@ static UvSculptData *uv_sculpt_stroke_init(bContext *C, wmOperator *op, wmEvent
 			aspectRatio = width / (float)height;
 			radius /= (width * zoomx);
 			radius = radius * radius;
-			radius_root = sqrt(radius);
+			radius_root = sqrtf(radius);
 
 			/* Allocate selection stack */
 			data->initial_stroke = MEM_mallocN(sizeof(*data->initial_stroke), "uv_sculpt_initial_stroke");
@@ -715,7 +828,7 @@ static UvSculptData *uv_sculpt_stroke_init(bContext *C, wmOperator *op, wmEvent
 				diff[1] /= aspectRatio;
 				if ((dist = dot_v2v2(diff, diff)) <= radius) {
 					float strength;
-					strength = alpha * BKE_brush_curve_strength(brush, sqrt(dist), radius_root);
+					strength = alpha * BKE_brush_curve_strength(brush, sqrtf(dist), radius_root);
 
 					data->initial_stroke->initialSelection[counter].uv = i;
 					data->initial_stroke->initialSelection[counter].strength = strength;
@@ -731,7 +844,7 @@ static UvSculptData *uv_sculpt_stroke_init(bContext *C, wmOperator *op, wmEvent
 	return op->customdata;
 }
 
-static int uv_sculpt_stroke_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int uv_sculpt_stroke_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	UvSculptData *data;
 	Object *obedit = CTX_data_edit_object(C);
@@ -754,7 +867,7 @@ static int uv_sculpt_stroke_invoke(bContext *C, wmOperator *op, wmEvent *event)
 }
 
 
-static int uv_sculpt_stroke_modal(bContext *C, wmOperator *op, wmEvent *event)
+static int uv_sculpt_stroke_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	UvSculptData *data = (UvSculptData *)op->customdata;
 	Object *obedit = CTX_data_edit_object(C);
