@@ -42,7 +42,6 @@
 
 #include "DNA_anim_types.h"
 #include "DNA_gpencil_types.h"
-#include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_mask_types.h"
 
@@ -52,6 +51,8 @@
 
 #include "BKE_action.h"
 #include "BKE_fcurve.h"
+#include "BKE_global.h"
+#include "BKE_main.h"
 #include "BKE_nla.h"
 #include "BKE_context.h"
 #include "BKE_report.h"
@@ -63,7 +64,6 @@
 #include "ED_keyframing.h"
 #include "ED_keyframes_edit.h"
 #include "ED_screen.h"
-#include "ED_transform.h"
 #include "ED_markers.h"
 #include "ED_mask.h"
 
@@ -85,7 +85,7 @@ static int act_new_exec(bContext *C, wmOperator *UNUSED(op))
 	PropertyRNA *prop;
 
 	/* hook into UI */
-	uiIDContextProperty(C, &ptr, &prop);
+	UI_context_active_but_prop_get_templateID(C, &ptr, &prop);
 	
 	if (prop) {
 		bAction *action = NULL, *oldact = NULL;
@@ -103,8 +103,10 @@ static int act_new_exec(bContext *C, wmOperator *UNUSED(op))
 			action = BKE_action_copy(oldact);
 		}
 		else {
+			Main *bmain = CTX_data_main(C);
+
 			/* just make a new (empty) action */
-			action = add_empty_action("Action");
+			action = add_empty_action(bmain, "Action");
 		}
 		
 		/* when creating new ID blocks, use is already 1 (fake user), 
@@ -118,7 +120,7 @@ static int act_new_exec(bContext *C, wmOperator *UNUSED(op))
 	}
 	
 	/* set notifier that keyframes have changed */
-	WM_event_add_notifier(C, NC_ANIMATION | ND_KEYFRAME | NA_EDITED, NULL);
+	WM_event_add_notifier(C, NC_ANIMATION | ND_KEYFRAME | NA_ADDED, NULL);
 	
 	return OPERATOR_FINISHED;
 }
@@ -132,7 +134,7 @@ void ACTION_OT_new(wmOperatorType *ot)
 	
 	/* api callbacks */
 	ot->exec = act_new_exec;
-	// NOTE: this is used in the NLA too...
+	/* NOTE: this is used in the NLA too... */
 	//ot->poll = ED_operator_action_active;
 	
 	/* flags */
@@ -227,15 +229,16 @@ void ACTION_OT_markers_make_local(wmOperatorType *ot)
 /* *************************** Calculate Range ************************** */
 
 /* Get the min/max keyframes*/
-static void get_keyframe_extents(bAnimContext *ac, float *min, float *max, const short onlySel)
+static bool get_keyframe_extents(bAnimContext *ac, float *min, float *max, const short onlySel)
 {
 	ListBase anim_data = {NULL, NULL};
 	bAnimListElem *ale;
 	int filter;
+	bool found = false;
 	
 	/* get data to filter, from Action or Dopesheet */
-	// XXX: what is sel doing here?!
-	//      Commented it, was breaking things (eg. the "auto preview range" tool).
+	/* XXX: what is sel doing here?!
+	 *      Commented it, was breaking things (eg. the "auto preview range" tool). */
 	filter = (ANIMFILTER_DATA_VISIBLE | ANIMFILTER_LIST_VISIBLE /*| ANIMFILTER_SEL *//*| ANIMFILTER_CURVESONLY*/ | ANIMFILTER_NODUPLIS);
 	ANIM_animdata_filter(ac, &anim_data, filter, ac->data, ac->datatype);
 	
@@ -254,8 +257,10 @@ static void get_keyframe_extents(bAnimContext *ac, float *min, float *max, const
 
 				/* find gp-frame which is less than or equal to cframe */
 				for (gpf = gpl->frames.first; gpf; gpf = gpf->next) {
-					*min = MIN2(*min, gpf->framenum);
-					*max = MAX2(*max, gpf->framenum);
+					const float framenum = (float)gpf->framenum;
+					*min = min_ff(*min, framenum);
+					*max = max_ff(*max, framenum);
+					found = true;
 				}
 			}
 			else if (ale->datatype == ALE_MASKLAY) {
@@ -267,8 +272,10 @@ static void get_keyframe_extents(bAnimContext *ac, float *min, float *max, const
 				     masklay_shape;
 				     masklay_shape = masklay_shape->next)
 				{
-					*min = MIN2(*min, masklay_shape->frame);
-					*max = MAX2(*max, masklay_shape->frame);
+					const float framenum = (float)masklay_shape->frame;
+					*min = min_ff(*min, framenum);
+					*max = max_ff(*max, framenum);
+					found = true;
 				}
 			}
 			else {
@@ -276,21 +283,23 @@ static void get_keyframe_extents(bAnimContext *ac, float *min, float *max, const
 				float tmin, tmax;
 
 				/* get range and apply necessary scaling before processing */
-				calc_fcurve_range(fcu, &tmin, &tmax, onlySel, TRUE);
+				if (calc_fcurve_range(fcu, &tmin, &tmax, onlySel, true)) {
 
-				if (adt) {
-					tmin = BKE_nla_tweakedit_remap(adt, tmin, NLATIME_CONVERT_MAP);
-					tmax = BKE_nla_tweakedit_remap(adt, tmax, NLATIME_CONVERT_MAP);
+					if (adt) {
+						tmin = BKE_nla_tweakedit_remap(adt, tmin, NLATIME_CONVERT_MAP);
+						tmax = BKE_nla_tweakedit_remap(adt, tmax, NLATIME_CONVERT_MAP);
+					}
+
+					/* try to set cur using these values, if they're more extreme than previously set values */
+					*min = min_ff(*min, tmin);
+					*max = max_ff(*max, tmax);
+					found = true;
 				}
-
-				/* try to set cur using these values, if they're more extreme than previously set values */
-				*min = MIN2(*min, tmin);
-				*max = MAX2(*max, tmax);
 			}
 		}
 		
 		/* free memory */
-		BLI_freelistN(&anim_data);
+		ANIM_animdata_freelist(&anim_data);
 	}
 	else {
 		/* set default range */
@@ -303,6 +312,8 @@ static void get_keyframe_extents(bAnimContext *ac, float *min, float *max, const
 			*max = 100;
 		}
 	}
+
+	return found;
 }
 
 /* ****************** Automatic Preview-Range Operator ****************** */
@@ -322,10 +333,10 @@ static int actkeys_previewrange_exec(bContext *C, wmOperator *UNUSED(op))
 		scene = ac.scene;
 	
 	/* set the range directly */
-	get_keyframe_extents(&ac, &min, &max, FALSE);
+	get_keyframe_extents(&ac, &min, &max, false);
 	scene->r.flag |= SCER_PRV_RANGE;
-	scene->r.psfra = (int)floor(min + 0.5f);
-	scene->r.pefra = (int)floor(max + 0.5f);
+	scene->r.psfra = iroundf(min);
+	scene->r.pefra = iroundf(max);
 	
 	/* set notifier that things have changed */
 	// XXX err... there's nothing for frame ranges yet, but this should do fine too
@@ -351,11 +362,67 @@ void ACTION_OT_previewrange_set(wmOperatorType *ot)
 
 /* ****************** View-All Operator ****************** */
 
-static int actkeys_viewall(bContext *C, const short onlySel)
+/**
+ * Find the extents of the active channel
+ *
+ * \param[out] min Bottom y-extent of channel
+ * \param[out] max Top y-extent of channel
+ * \return Success of finding a selected channel
+ */
+static bool actkeys_channels_get_selected_extents(bAnimContext *ac, float *min, float *max)
+{
+	ListBase anim_data = {NULL, NULL};
+	bAnimListElem *ale;
+	int filter;
+	
+	short found = 0; /* NOTE: not bool, since we want prioritise individual channels over expanders */
+	float y;
+	
+	/* get all items - we need to do it this way */
+	filter = (ANIMFILTER_DATA_VISIBLE | ANIMFILTER_LIST_VISIBLE | ANIMFILTER_LIST_CHANNELS);
+	ANIM_animdata_filter(ac, &anim_data, filter, ac->data, ac->datatype);
+	
+	/* loop through all channels, finding the first one that's selected */
+	y = (float)ACHANNEL_FIRST;
+	
+	for (ale = anim_data.first; ale; ale = ale->next) {
+		bAnimChannelType *acf = ANIM_channel_get_typeinfo(ale);
+		
+		/* must be selected... */
+		if (acf && acf->has_setting(ac, ale, ACHANNEL_SETTING_SELECT) && 
+		    ANIM_channel_setting_get(ac, ale, ACHANNEL_SETTING_SELECT))
+		{
+			/* update best estimate */
+			*min = (float)(y - ACHANNEL_HEIGHT_HALF);
+			*max = (float)(y + ACHANNEL_HEIGHT_HALF);
+			
+			/* is this high enough priority yet? */
+			found = acf->channel_role;
+			
+			/* only stop our search when we've found an actual channel
+			 * - datablock expanders get less priority so that we don't abort prematurely
+			 */
+			if (found == ACHANNEL_ROLE_CHANNEL) {
+				break;
+			}
+		}
+		
+		/* adjust y-position for next one */
+		y -= ACHANNEL_STEP;
+	}
+	
+	/* free all temp data */
+	ANIM_animdata_freelist(&anim_data);
+	
+	return (found != 0);
+}
+
+static int actkeys_viewall(bContext *C, const bool only_sel)
 {
 	bAnimContext ac;
 	View2D *v2d;
-	float extra;
+	float extra, min, max;
+	bool found;
 	
 	/* get editor data */
 	if (ANIM_animdata_get_context(C, &ac) == 0)
@@ -363,15 +430,38 @@ static int actkeys_viewall(bContext *C, const short onlySel)
 	v2d = &ac.ar->v2d;
 	
 	/* set the horizontal range, with an extra offset so that the extreme keys will be in view */
-	get_keyframe_extents(&ac, &v2d->cur.xmin, &v2d->cur.xmax, onlySel);
-	
+	found = get_keyframe_extents(&ac, &min, &max, only_sel);
+
+	if (only_sel && (found == false))
+		return OPERATOR_CANCELLED;
+
+	v2d->cur.xmin = min;
+	v2d->cur.xmax = max;
+
 	extra = 0.1f * BLI_rctf_size_x(&v2d->cur);
 	v2d->cur.xmin -= extra;
 	v2d->cur.xmax += extra;
 	
 	/* set vertical range */
-	v2d->cur.ymax = 0.0f;
-	v2d->cur.ymin = (float)-BLI_rcti_size_y(&v2d->mask);
+	if (only_sel == false) {
+		/* view all -> the summary channel is usually the shows everything, and resides right at the top... */
+		v2d->cur.ymax = 0.0f;
+		v2d->cur.ymin = (float)-BLI_rcti_size_y(&v2d->mask);
+	}
+	else {
+		/* locate first selected channel (or the active one), and frame those */
+		float ymin = v2d->cur.ymin;
+		float ymax = v2d->cur.ymax;
+		
+		if (actkeys_channels_get_selected_extents(&ac, &ymin, &ymax)) {
+			/* recenter the view so that this range is in the middle */
+			float ymid = (ymax - ymin) / 2.0f + ymin;
+			float x_center;
+			
+			UI_view2d_center_get(v2d, &x_center, NULL);
+			UI_view2d_center_set(v2d, x_center, ymid);
+		}
+	}
 	
 	/* do View2D syncing */
 	UI_view2d_sync(CTX_wm_screen(C), CTX_wm_area(C), v2d, V2D_LOCK_COPY);
@@ -387,13 +477,13 @@ static int actkeys_viewall(bContext *C, const short onlySel)
 static int actkeys_viewall_exec(bContext *C, wmOperator *UNUSED(op))
 {	
 	/* whole range */
-	return actkeys_viewall(C, FALSE);
+	return actkeys_viewall(C, false);
 }
 
 static int actkeys_viewsel_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	/* only selected */
-	return actkeys_viewall(C, TRUE);
+	return actkeys_viewall(C, true);
 }
  
 void ACTION_OT_view_all(wmOperatorType *ot)
@@ -448,7 +538,7 @@ static short copy_action_keys(bAnimContext *ac)
 	ok = copy_animedit_keys(ac, &anim_data);
 	
 	/* clean up */
-	BLI_freelistN(&anim_data);
+	ANIM_animdata_freelist(&anim_data);
 
 	return ok;
 }
@@ -473,9 +563,9 @@ static short paste_action_keys(bAnimContext *ac,
 	
 	/* paste keyframes */
 	ok = paste_animedit_keys(ac, &anim_data, offset_mode, merge_mode);
-	
+
 	/* clean up */
-	BLI_freelistN(&anim_data);
+	ANIM_animdata_freelist(&anim_data);
 
 	return ok;
 }
@@ -491,9 +581,9 @@ static int actkeys_copy_exec(bContext *C, wmOperator *op)
 		return OPERATOR_CANCELLED;
 
 	/* copy keyframes */
-	if (ELEM(ac.datatype, ANIMCONT_GPENCIL, ANIMCONT_MASK)) {
+	if (ac.datatype == ANIMCONT_GPENCIL) {
 		/* FIXME... */
-		BKE_report(op->reports, RPT_ERROR, "Keyframe pasting is not available for Grease Pencil mode");
+		BKE_report(op->reports, RPT_ERROR, "Keyframe pasting is not available for grease pencil mode");
 		return OPERATOR_CANCELLED;
 	}
 	else if (ac.datatype == ANIMCONT_MASK) {
@@ -502,7 +592,7 @@ static int actkeys_copy_exec(bContext *C, wmOperator *op)
 		return OPERATOR_CANCELLED;
 	}
 	else {
-		if (copy_action_keys(&ac)) {	
+		if (copy_action_keys(&ac)) {
 			BKE_report(op->reports, RPT_ERROR, "No keyframes copied to keyframes copy/paste buffer");
 			return OPERATOR_CANCELLED;
 		}
@@ -542,8 +632,8 @@ static int actkeys_paste_exec(bContext *C, wmOperator *op)
 	
 	/* paste keyframes */
 	if (ELEM(ac.datatype, ANIMCONT_GPENCIL, ANIMCONT_MASK)) {
-		// FIXME...
-		BKE_report(op->reports, RPT_ERROR, "Keyframe pasting is not available for Grease Pencil or Mask mode");
+		/* FIXME... */
+		BKE_report(op->reports, RPT_ERROR, "Keyframe pasting is not available for grease pencil or mask mode");
 		return OPERATOR_CANCELLED;
 	}
 	else {
@@ -552,10 +642,7 @@ static int actkeys_paste_exec(bContext *C, wmOperator *op)
 			return OPERATOR_CANCELLED;
 		}
 	}
-	
-	/* validate keyframes after editing */
-	ANIM_editkeyframes_refresh(&ac);
-	
+
 	/* set notifier that keyframes have changed */
 	WM_event_add_notifier(C, NC_ANIMATION | ND_KEYFRAME | NA_EDITED, NULL);
 	
@@ -588,7 +675,7 @@ void ACTION_OT_paste(wmOperatorType *ot)
 static EnumPropertyItem prop_actkeys_insertkey_types[] = {
 	{1, "ALL", 0, "All Channels", ""},
 	{2, "SEL", 0, "Only Selected Channels", ""},
-	{3, "GROUP", 0, "In Active Group", ""}, // xxx not in all cases
+	{3, "GROUP", 0, "In Active Group", ""},  /* XXX not in all cases */
 	{0, NULL, 0, NULL, NULL}
 };
 
@@ -630,9 +717,12 @@ static void insert_action_keys(bAnimContext *ac, short mode)
 			insert_keyframe(reports, ale->id, NULL, ((fcu->grp) ? (fcu->grp->name) : (NULL)), fcu->rna_path, fcu->array_index, cfra, flag);
 		else
 			insert_vert_fcurve(fcu, cfra, fcu->curval, 0);
+
+		ale->update |= ANIM_UPDATE_DEFAULT;
 	}
-	
-	BLI_freelistN(&anim_data);
+
+	ANIM_animdata_update(ac, &anim_data);
+	ANIM_animdata_freelist(&anim_data);
 }
 
 /* ------------------- */
@@ -653,12 +743,9 @@ static int actkeys_insertkey_exec(bContext *C, wmOperator *op)
 	
 	/* insert keyframes */
 	insert_action_keys(&ac, mode);
-	
-	/* validate keyframes after editing */
-	ANIM_editkeyframes_refresh(&ac);
-	
+
 	/* set notifier that keyframes have changed */
-	WM_event_add_notifier(C, NC_ANIMATION | ND_KEYFRAME | NA_EDITED, NULL);
+	WM_event_add_notifier(C, NC_ANIMATION | ND_KEYFRAME | NA_ADDED, NULL);
 	
 	return OPERATOR_FINISHED;
 }
@@ -707,10 +794,12 @@ static void duplicate_action_keys(bAnimContext *ac)
 			ED_masklayer_frames_duplicate((MaskLayer *)ale->data);
 		else
 			BLI_assert(0);
+
+		ale->update |= ANIM_UPDATE_DEFAULT;
 	}
-	
-	/* free filtered list */
-	BLI_freelistN(&anim_data);
+
+	ANIM_animdata_update(ac, &anim_data);
+	ANIM_animdata_freelist(&anim_data);
 }
 
 /* ------------------- */
@@ -725,18 +814,14 @@ static int actkeys_duplicate_exec(bContext *C, wmOperator *UNUSED(op))
 		
 	/* duplicate keyframes */
 	duplicate_action_keys(&ac);
-	
-	/* validate keyframes after editing */
-	if (!ELEM(ac.datatype, ANIMCONT_GPENCIL, ANIMCONT_MASK))
-		ANIM_editkeyframes_refresh(&ac);
-	
+
 	/* set notifier that keyframes have changed */
-	WM_event_add_notifier(C, NC_ANIMATION | ND_KEYFRAME | NA_EDITED, NULL);
+	WM_event_add_notifier(C, NC_ANIMATION | ND_KEYFRAME | NA_ADDED, NULL);
 	
 	return OPERATOR_FINISHED;
 }
 
-static int actkeys_duplicate_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int actkeys_duplicate_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	actkeys_duplicate_exec(C, op);
 	
@@ -761,42 +846,54 @@ void ACTION_OT_duplicate(wmOperatorType *ot)
 
 /* ******************** Delete Keyframes Operator ************************* */
 
-static void delete_action_keys(bAnimContext *ac)
+static bool delete_action_keys(bAnimContext *ac)
 {
 	ListBase anim_data = {NULL, NULL};
 	bAnimListElem *ale;
 	int filter;
-	
+	bool changed_final = false;
+
 	/* filter data */
 	if (ELEM(ac->datatype, ANIMCONT_GPENCIL, ANIMCONT_MASK))
 		filter = (ANIMFILTER_DATA_VISIBLE | ANIMFILTER_LIST_VISIBLE | ANIMFILTER_FOREDIT | ANIMFILTER_NODUPLIS);
 	else
 		filter = (ANIMFILTER_DATA_VISIBLE | ANIMFILTER_LIST_VISIBLE | ANIMFILTER_FOREDIT /*| ANIMFILTER_CURVESONLY*/ | ANIMFILTER_NODUPLIS);
 	ANIM_animdata_filter(ac, &anim_data, filter, ac->data, ac->datatype);
-	
+
 	/* loop through filtered data and delete selected keys */
 	for (ale = anim_data.first; ale; ale = ale->next) {
+		bool changed = false;
+
 		if (ale->type == ANIMTYPE_GPLAYER) {
-			ED_gplayer_frames_delete((bGPDlayer *)ale->data);
+			changed = ED_gplayer_frames_delete((bGPDlayer *)ale->data);
 		}
 		else if (ale->type == ANIMTYPE_MASKLAYER) {
-			ED_masklayer_frames_delete((MaskLayer *)ale->data);
+			changed = ED_masklayer_frames_delete((MaskLayer *)ale->data);
 		}
 		else {
 			FCurve *fcu = (FCurve *)ale->key_data;
 			AnimData *adt = ale->adt;
 			
 			/* delete selected keyframes only */
-			delete_fcurve_keys(fcu); 
+			changed = delete_fcurve_keys(fcu);
 			
 			/* Only delete curve too if it won't be doing anything anymore */
-			if ((fcu->totvert == 0) && (list_has_suitable_fmodifier(&fcu->modifiers, 0, FMI_TYPE_GENERATE_CURVE) == 0))
+			if ((fcu->totvert == 0) && (list_has_suitable_fmodifier(&fcu->modifiers, 0, FMI_TYPE_GENERATE_CURVE) == 0)) {
 				ANIM_fcurve_delete_from_animdata(ac, adt, fcu);
+				ale->key_data = NULL;
+			}
+		}
+
+		if (changed) {
+			ale->update |= ANIM_UPDATE_DEFAULT;
+			changed_final = true;
 		}
 	}
-	
-	/* free filtered list */
-	BLI_freelistN(&anim_data);
+
+	ANIM_animdata_update(ac, &anim_data);
+	ANIM_animdata_freelist(&anim_data);
+
+	return changed_final;
 }
 
 /* ------------------- */
@@ -810,14 +907,11 @@ static int actkeys_delete_exec(bContext *C, wmOperator *UNUSED(op))
 		return OPERATOR_CANCELLED;
 		
 	/* delete keyframes */
-	delete_action_keys(&ac);
-	
-	/* validate keyframes after editing */
-	if (!ELEM(ac.datatype, ANIMCONT_GPENCIL, ANIMCONT_MASK))
-		ANIM_editkeyframes_refresh(&ac);
+	if (!delete_action_keys(&ac))
+		return OPERATOR_CANCELLED;
 	
 	/* set notifier that keyframes have changed */
-	WM_event_add_notifier(C, NC_ANIMATION | ND_KEYFRAME | NA_EDITED, NULL);
+	WM_event_add_notifier(C, NC_ANIMATION | ND_KEYFRAME | NA_REMOVED, NULL);
 	
 	return OPERATOR_FINISHED;
 }
@@ -851,11 +945,14 @@ static void clean_action_keys(bAnimContext *ac, float thresh)
 	ANIM_animdata_filter(ac, &anim_data, filter, ac->data, ac->datatype);
 	
 	/* loop through filtered data and clean curves */
-	for (ale = anim_data.first; ale; ale = ale->next)
+	for (ale = anim_data.first; ale; ale = ale->next) {
 		clean_fcurve((FCurve *)ale->key_data, thresh);
-	
-	/* free temp data */
-	BLI_freelistN(&anim_data);
+
+		ale->update |= ANIM_UPDATE_DEFAULT;
+	}
+
+	ANIM_animdata_update(ac, &anim_data);
+	ANIM_animdata_freelist(&anim_data);
 }
 
 /* ------------------- */
@@ -868,8 +965,11 @@ static int actkeys_clean_exec(bContext *C, wmOperator *op)
 	/* get editor data */
 	if (ANIM_animdata_get_context(C, &ac) == 0)
 		return OPERATOR_CANCELLED;
-	if (ELEM(ac.datatype, ANIMCONT_GPENCIL, ANIMCONT_MASK))
+		
+	if (ELEM(ac.datatype, ANIMCONT_GPENCIL, ANIMCONT_MASK)) {
+		BKE_report(op->reports, RPT_ERROR, "Not implemented");
 		return OPERATOR_PASS_THROUGH;
+	}
 		
 	/* get cleaning threshold */
 	thresh = RNA_float_get(op->ptr, "threshold");
@@ -877,9 +977,6 @@ static int actkeys_clean_exec(bContext *C, wmOperator *op)
 	/* clean keyframes */
 	clean_action_keys(&ac, thresh);
 	
-	/* validate keyframes after editing */
-	ANIM_editkeyframes_refresh(&ac);
-	
 	/* set notifier that keyframes have changed */
 	WM_event_add_notifier(C, NC_ANIMATION | ND_KEYFRAME | NA_EDITED, NULL);
 	
@@ -919,31 +1016,34 @@ static void sample_action_keys(bAnimContext *ac)
 	ANIM_animdata_filter(ac, &anim_data, filter, ac->data, ac->datatype);
 	
 	/* loop through filtered data and add keys between selected keyframes on every frame  */
-	for (ale = anim_data.first; ale; ale = ale->next)
+	for (ale = anim_data.first; ale; ale = ale->next) {
 		sample_fcurve((FCurve *)ale->key_data);
-	
-	/* admin and redraws */
-	BLI_freelistN(&anim_data);
+
+		ale->update |= ANIM_UPDATE_DEPS;
+	}
+
+	ANIM_animdata_update(ac, &anim_data);
+	ANIM_animdata_freelist(&anim_data);
 }
 
 /* ------------------- */
 
-static int actkeys_sample_exec(bContext *C, wmOperator *UNUSED(op))
+static int actkeys_sample_exec(bContext *C, wmOperator *op)
 {
 	bAnimContext ac;
 	
 	/* get editor data */
 	if (ANIM_animdata_get_context(C, &ac) == 0)
 		return OPERATOR_CANCELLED;
-	if (ELEM(ac.datatype, ANIMCONT_GPENCIL, ANIMCONT_MASK))
+		
+	if (ELEM(ac.datatype, ANIMCONT_GPENCIL, ANIMCONT_MASK)) {
+		BKE_report(op->reports, RPT_ERROR, "Not implemented");
 		return OPERATOR_PASS_THROUGH;
+	}
 	
 	/* sample keyframes */
 	sample_action_keys(&ac);
 	
-	/* validate keyframes after editing */
-	ANIM_editkeyframes_refresh(&ac);
-	
 	/* set notifier that keyframes have changed */
 	WM_event_add_notifier(C, NC_ANIMATION | ND_KEYFRAME | NA_EDITED, NULL);
 	
@@ -976,8 +1076,8 @@ void ACTION_OT_sample(wmOperatorType *ot)
 
 /* defines for set extrapolation-type for selected keyframes tool */
 static EnumPropertyItem prop_actkeys_expo_types[] = {
-	{FCURVE_EXTRAPOLATE_CONSTANT, "CONSTANT", 0, "Constant Extrapolation", ""},
-	{FCURVE_EXTRAPOLATE_LINEAR, "LINEAR", 0, "Linear Extrapolation", ""},
+	{FCURVE_EXTRAPOLATE_CONSTANT, "CONSTANT", 0, "Constant Extrapolation", "Values on endpoint keyframes are held"},
+	{FCURVE_EXTRAPOLATE_LINEAR, "LINEAR", 0, "Linear Extrapolation", "Straight-line slope of end segments are extended past the endpoint keyframes"},
 	
 	{MAKE_CYCLIC_EXPO, "MAKE_CYCLIC", 0, "Make Cyclic (F-Modifier)", "Add Cycles F-Modifier if one doesn't exist already"},
 	{CLEAR_CYCLIC_EXPO, "CLEAR_CYCLIC", 0, "Clear Cyclic (F-Modifier)", "Remove Cycles F-Modifier if not needed anymore"},
@@ -1010,7 +1110,7 @@ static void setexpo_action_keys(bAnimContext *ac, short mode)
 			if (mode == MAKE_CYCLIC_EXPO) {
 				/* only add if one doesn't exist */
 				if (list_has_suitable_fmodifier(&fcu->modifiers, FMODIFIER_TYPE_CYCLES, -1) == 0) {
-					// TODO: add some more preset versions which set different extrapolation options?
+					/* TODO: add some more preset versions which set different extrapolation options? */
 					add_fmodifier(&fcu->modifiers, FMODIFIER_TYPE_CYCLES);
 				}
 			}
@@ -1026,10 +1126,12 @@ static void setexpo_action_keys(bAnimContext *ac, short mode)
 				}
 			}
 		}
+
+		ale->update |= ANIM_UPDATE_DEFAULT;
 	}
-	
-	/* cleanup */
-	BLI_freelistN(&anim_data);
+
+	ANIM_animdata_update(ac, &anim_data);
+	ANIM_animdata_freelist(&anim_data);
 }
 
 /* ------------------- */
@@ -1042,8 +1144,11 @@ static int actkeys_expo_exec(bContext *C, wmOperator *op)
 	/* get editor data */
 	if (ANIM_animdata_get_context(C, &ac) == 0)
 		return OPERATOR_CANCELLED;
-	if (ELEM(ac.datatype, ANIMCONT_GPENCIL, ANIMCONT_MASK))
+		
+	if (ELEM(ac.datatype, ANIMCONT_GPENCIL, ANIMCONT_MASK)) {
+		BKE_report(op->reports, RPT_ERROR, "Not implemented");
 		return OPERATOR_PASS_THROUGH;
+	}
 		
 	/* get handle setting mode */
 	mode = RNA_enum_get(op->ptr, "type");
@@ -1051,9 +1156,6 @@ static int actkeys_expo_exec(bContext *C, wmOperator *op)
 	/* set handle type */
 	setexpo_action_keys(&ac, mode);
 	
-	/* validate keyframes after editing */
-	ANIM_editkeyframes_refresh(&ac);
-	
 	/* set notifier that keyframe properties have changed */
 	WM_event_add_notifier(C, NC_ANIMATION | ND_KEYFRAME_PROP, NULL);
 	
@@ -1096,11 +1198,14 @@ static void setipo_action_keys(bAnimContext *ac, short mode)
 	/* loop through setting BezTriple interpolation
 	 * Note: we do not supply KeyframeEditData to the looper yet. Currently that's not necessary here...
 	 */
-	for (ale = anim_data.first; ale; ale = ale->next)
+	for (ale = anim_data.first; ale; ale = ale->next) {
 		ANIM_fcurve_keyframes_loop(NULL, ale->key_data, NULL, set_cb, calchandles_fcurve);
-	
-	/* cleanup */
-	BLI_freelistN(&anim_data);
+
+		ale->update |= ANIM_UPDATE_DEFAULT;
+	}
+
+	ANIM_animdata_update(ac, &anim_data);
+	ANIM_animdata_freelist(&anim_data);
 }
 
 /* ------------------- */
@@ -1113,8 +1218,11 @@ static int actkeys_ipo_exec(bContext *C, wmOperator *op)
 	/* get editor data */
 	if (ANIM_animdata_get_context(C, &ac) == 0)
 		return OPERATOR_CANCELLED;
-	if (ELEM(ac.datatype, ANIMCONT_GPENCIL, ANIMCONT_MASK))
+		
+	if (ELEM(ac.datatype, ANIMCONT_GPENCIL, ANIMCONT_MASK)) {
+		BKE_report(op->reports, RPT_ERROR, "Not implemented");
 		return OPERATOR_PASS_THROUGH;
+	}
 		
 	/* get handle setting mode */
 	mode = RNA_enum_get(op->ptr, "type");
@@ -1122,9 +1230,6 @@ static int actkeys_ipo_exec(bContext *C, wmOperator *op)
 	/* set handle type */
 	setipo_action_keys(&ac, mode);
 	
-	/* validate keyframes after editing */
-	ANIM_editkeyframes_refresh(&ac);
-	
 	/* set notifier that keyframe properties have changed */
 	WM_event_add_notifier(C, NC_ANIMATION | ND_KEYFRAME_PROP, NULL);
 	
@@ -1176,11 +1281,13 @@ static void sethandles_action_keys(bAnimContext *ac, short mode)
 		if (ANIM_fcurve_keyframes_loop(NULL, fcu, NULL, sel_cb, NULL)) {
 			/* change type of selected handles */
 			ANIM_fcurve_keyframes_loop(NULL, fcu, NULL, edit_cb, calchandles_fcurve);
+
+			ale->update |= ANIM_UPDATE_DEFAULT;
 		}
 	}
-	
-	/* cleanup */
-	BLI_freelistN(&anim_data);
+
+	ANIM_animdata_update(ac, &anim_data);
+	ANIM_animdata_freelist(&anim_data);
 }
 
 /* ------------------- */
@@ -1193,8 +1300,11 @@ static int actkeys_handletype_exec(bContext *C, wmOperator *op)
 	/* get editor data */
 	if (ANIM_animdata_get_context(C, &ac) == 0)
 		return OPERATOR_CANCELLED;
-	if (ELEM(ac.datatype, ANIMCONT_GPENCIL, ANIMCONT_MASK))
+		
+	if (ELEM(ac.datatype, ANIMCONT_GPENCIL, ANIMCONT_MASK)) {
+		BKE_report(op->reports, RPT_ERROR, "Not implemented");
 		return OPERATOR_PASS_THROUGH;
+	}
 		
 	/* get handle setting mode */
 	mode = RNA_enum_get(op->ptr, "type");
@@ -1202,9 +1312,6 @@ static int actkeys_handletype_exec(bContext *C, wmOperator *op)
 	/* set handle type */
 	sethandles_action_keys(&ac, mode);
 	
-	/* validate keyframes after editing */
-	ANIM_editkeyframes_refresh(&ac);
-	
 	/* set notifier that keyframe properties have changed */
 	WM_event_add_notifier(C, NC_ANIMATION | ND_KEYFRAME_PROP, NULL);
 	
@@ -1232,7 +1339,7 @@ void ACTION_OT_handle_type(wmOperatorType *ot)
 
 /* ******************** Set Keyframe-Type Operator *********************** */
 
-/* this function is responsible for setting interpolation mode for keyframes */
+/* this function is responsible for setting keyframe type for keyframes */
 static void setkeytype_action_keys(bAnimContext *ac, short mode) 
 {
 	ListBase anim_data = {NULL, NULL};
@@ -1247,11 +1354,37 @@ static void setkeytype_action_keys(bAnimContext *ac, short mode)
 	/* loop through setting BezTriple interpolation
 	 * Note: we do not supply KeyframeEditData to the looper yet. Currently that's not necessary here...
 	 */
-	for (ale = anim_data.first; ale; ale = ale->next)
+	for (ale = anim_data.first; ale; ale = ale->next) {
 		ANIM_fcurve_keyframes_loop(NULL, ale->key_data, NULL, set_cb, NULL);
+
+		ale->update |= ANIM_UPDATE_DEPS | ANIM_UPDATE_HANDLES;
+	}
+
+	ANIM_animdata_update(ac, &anim_data);
+	ANIM_animdata_freelist(&anim_data);
+}
+
+/* this function is responsible for setting the keyframe type for Grease Pencil frames */
+static void setkeytype_gpencil_keys(bAnimContext *ac, short mode)
+{
+	ListBase anim_data = {NULL, NULL};
+	bAnimListElem *ale;
+	int filter;
 	
-	/* cleanup */
-	BLI_freelistN(&anim_data);
+	/* filter data */
+	filter = (ANIMFILTER_DATA_VISIBLE | ANIMFILTER_LIST_VISIBLE | ANIMFILTER_FOREDIT | ANIMFILTER_NODUPLIS);
+	ANIM_animdata_filter(ac, &anim_data, filter, ac->data, ac->datatype);
+	
+	/* loop through each layer */
+	for (ale = anim_data.first; ale; ale = ale->next) {
+		if (ale->type == ANIMTYPE_GPLAYER) {
+			ED_gplayer_frames_keytype_set(ale->data, mode);
+			ale->update |= ANIM_UPDATE_DEPS;
+		}
+	}
+
+	ANIM_animdata_update(ac, &anim_data);
+	ANIM_animdata_freelist(&anim_data);
 }
 
 /* ------------------- */
@@ -1264,17 +1397,22 @@ static int actkeys_keytype_exec(bContext *C, wmOperator *op)
 	/* get editor data */
 	if (ANIM_animdata_get_context(C, &ac) == 0)
 		return OPERATOR_CANCELLED;
-	if (ELEM(ac.datatype, ANIMCONT_GPENCIL, ANIMCONT_MASK))
+		
+	if (ac.datatype == ANIMCONT_MASK) {
+		BKE_report(op->reports, RPT_ERROR, "Not implemented for Masks");
 		return OPERATOR_PASS_THROUGH;
+	}
 		
 	/* get handle setting mode */
 	mode = RNA_enum_get(op->ptr, "type");
 	
 	/* set handle type */
-	setkeytype_action_keys(&ac, mode);
-	
-	/* validate keyframes after editing */
-	ANIM_editkeyframes_refresh(&ac);
+	if (ac.datatype == ANIMCONT_GPENCIL) {
+		setkeytype_gpencil_keys(&ac, mode);
+	}
+	else {
+		setkeytype_action_keys(&ac, mode);
+	}
 	
 	/* set notifier that keyframe properties have changed */
 	WM_event_add_notifier(C, NC_ANIMATION | ND_KEYFRAME_PROP, NULL);
@@ -1306,6 +1444,15 @@ void ACTION_OT_keyframe_type(wmOperatorType *ot)
 
 /* ***************** Jump to Selected Frames Operator *********************** */
 
+static int actkeys_framejump_poll(bContext *C)
+{
+	/* prevent changes during render */
+	if (G.is_rendering)
+		return 0;
+
+	return ED_operator_action_active(C);
+}
+
 /* snap current-frame indicator to 'average time' of selected keyframe */
 static int actkeys_framejump_exec(bContext *C, wmOperator *UNUSED(op))
 {
@@ -1319,7 +1466,7 @@ static int actkeys_framejump_exec(bContext *C, wmOperator *UNUSED(op))
 	if (ANIM_animdata_get_context(C, &ac) == 0)
 		return OPERATOR_CANCELLED;
 	
-	/* init edit data */	
+	/* init edit data */
 	/* loop over action data, averaging values */
 	filter = (ANIMFILTER_DATA_VISIBLE | ANIMFILTER_LIST_VISIBLE /*| ANIMFILTER_CURVESONLY */ | ANIMFILTER_NODUPLIS);
 	ANIM_animdata_filter(&ac, &anim_data, filter, ac.data, ac.datatype);
@@ -1335,12 +1482,12 @@ static int actkeys_framejump_exec(bContext *C, wmOperator *UNUSED(op))
 			ANIM_fcurve_keyframes_loop(&ked, ale->key_data, NULL, bezt_calc_average, NULL);
 	}
 	
-	BLI_freelistN(&anim_data);
+	ANIM_animdata_freelist(&anim_data);
 	
 	/* set the new current frame value, based on the average time */
 	if (ked.i1) {
 		Scene *scene = ac.scene;
-		CFRA = (int)floor((ked.f1 / ked.i1) + 0.5f);
+		CFRA = iroundf(ked.f1 / ked.i1);
 		SUBFRA = 0.f;
 	}
 	
@@ -1353,13 +1500,13 @@ static int actkeys_framejump_exec(bContext *C, wmOperator *UNUSED(op))
 void ACTION_OT_frame_jump(wmOperatorType *ot)
 {
 	/* identifiers */
-	ot->name = "Jump to Frame";
+	ot->name = "Jump to Keyframes";
 	ot->idname = "ACTION_OT_frame_jump";
-	ot->description = "Set the current frame to the average frame of the selected keyframes";
+	ot->description = "Set the current frame to the average frame value of selected keyframes";
 	
 	/* api callbacks */
 	ot->exec = actkeys_framejump_exec;
-	ot->poll = ED_operator_action_active;
+	ot->poll = actkeys_framejump_poll;
 	
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -1369,13 +1516,13 @@ void ACTION_OT_frame_jump(wmOperatorType *ot)
 
 /* defines for snap keyframes tool */
 static EnumPropertyItem prop_actkeys_snap_types[] = {
-	{ACTKEYS_SNAP_CFRA, "CFRA", 0, "Current frame", 
+	{ACTKEYS_SNAP_CFRA, "CFRA", 0, "Current frame",
 	 "Snap selected keyframes to the current frame"},
-	{ACTKEYS_SNAP_NEAREST_FRAME, "NEAREST_FRAME", 0, "Nearest Frame", 
+	{ACTKEYS_SNAP_NEAREST_FRAME, "NEAREST_FRAME", 0, "Nearest Frame",
 	 "Snap selected keyframes to the nearest (whole) frame (use to fix accidental sub-frame offsets)"},
-	{ACTKEYS_SNAP_NEAREST_SECOND, "NEAREST_SECOND", 0, "Nearest Second", 
+	{ACTKEYS_SNAP_NEAREST_SECOND, "NEAREST_SECOND", 0, "Nearest Second",
 	 "Snap selected keyframes to the nearest second"},
-	{ACTKEYS_SNAP_NEAREST_MARKER, "NEAREST_MARKER", 0, "Nearest Marker", 
+	{ACTKEYS_SNAP_NEAREST_MARKER, "NEAREST_MARKER", 0, "Nearest Marker",
 	 "Snap selected keyframes to the nearest marker"},
 	{0, NULL, 0, NULL, NULL}
 };
@@ -1410,18 +1557,26 @@ static void snap_action_keys(bAnimContext *ac, short mode)
 	for (ale = anim_data.first; ale; ale = ale->next) {
 		AnimData *adt = ANIM_nla_mapping_get(ac, ale);
 		
-		if (adt) {
+		if (ale->type == ANIMTYPE_GPLAYER) {
+			ED_gplayer_snap_frames(ale->data, ac->scene, mode);
+		}
+		else if (ale->type == ANIMTYPE_MASKLAYER) {
+			ED_masklayer_snap_frames(ale->data, ac->scene, mode);
+		}
+		else if (adt) {
 			ANIM_nla_mapping_apply_fcurve(adt, ale->key_data, 0, 1); 
 			ANIM_fcurve_keyframes_loop(&ked, ale->key_data, NULL, edit_cb, calchandles_fcurve);
 			ANIM_nla_mapping_apply_fcurve(adt, ale->key_data, 1, 1);
 		}
-		//else if (ale->type == ACTTYPE_GPLAYER)
-		//	snap_gplayer_frames(ale->data, mode);
-		else 
+		else {
 			ANIM_fcurve_keyframes_loop(&ked, ale->key_data, NULL, edit_cb, calchandles_fcurve);
+		}
+
+		ale->update |= ANIM_UPDATE_DEFAULT;
 	}
-	
-	BLI_freelistN(&anim_data);
+
+	ANIM_animdata_update(ac, &anim_data);
+	ANIM_animdata_freelist(&anim_data);
 }
 
 /* ------------------- */
@@ -1434,20 +1589,13 @@ static int actkeys_snap_exec(bContext *C, wmOperator *op)
 	/* get editor data */
 	if (ANIM_animdata_get_context(C, &ac) == 0)
 		return OPERATOR_CANCELLED;
-		
-	// XXX...
-	if (ELEM(ac.datatype, ANIMCONT_GPENCIL, ANIMCONT_MASK))
-		return OPERATOR_PASS_THROUGH;
-		
+	
 	/* get snapping mode */
 	mode = RNA_enum_get(op->ptr, "type");
 	
 	/* snap keyframes */
 	snap_action_keys(&ac, mode);
 	
-	/* validate keyframes after editing */
-	ANIM_editkeyframes_refresh(&ac);
-	
 	/* set notifier that keyframes have changed */
 	WM_event_add_notifier(C, NC_ANIMATION | ND_KEYFRAME | NA_EDITED, NULL);
 	
@@ -1477,11 +1625,11 @@ void ACTION_OT_snap(wmOperatorType *ot)
 
 /* defines for mirror keyframes tool */
 static EnumPropertyItem prop_actkeys_mirror_types[] = {
-	{ACTKEYS_MIRROR_CFRA, "CFRA", 0, "By Times over Current frame", 
+	{ACTKEYS_MIRROR_CFRA, "CFRA", 0, "By Times over Current frame",
 	 "Flip times of selected keyframes using the current frame as the mirror line"},
-	{ACTKEYS_MIRROR_XAXIS, "XAXIS", 0, "By Values over Value=0", 
+	{ACTKEYS_MIRROR_XAXIS, "XAXIS", 0, "By Values over Value=0",
 	 "Flip values of selected keyframes (i.e. negative values become positive, and vice versa)"},
-	{ACTKEYS_MIRROR_MARKER, "MARKER", 0, "By Times over First Selected Marker", 
+	{ACTKEYS_MIRROR_MARKER, "MARKER", 0, "By Times over First Selected Marker",
 	 "Flip times of selected keyframes using the first selected marker as the reference point"},
 	{0, NULL, 0, NULL, NULL}
 };
@@ -1502,7 +1650,7 @@ static void mirror_action_keys(bAnimContext *ac, short mode)
 	ked.scene = ac->scene;
 	
 	/* for 'first selected marker' mode, need to find first selected marker first! */
-	// XXX should this be made into a helper func in the API?
+	/* XXX should this be made into a helper func in the API? */
 	if (mode == ACTKEYS_MIRROR_MARKER) {
 		TimeMarker *marker = ED_markers_get_first_selected(ac->markers);
 		
@@ -1532,9 +1680,12 @@ static void mirror_action_keys(bAnimContext *ac, short mode)
 		//	snap_gplayer_frames(ale->data, mode);
 		else 
 			ANIM_fcurve_keyframes_loop(&ked, ale->key_data, NULL, edit_cb, calchandles_fcurve);
+
+		ale->update |= ANIM_UPDATE_DEFAULT;
 	}
-	
-	BLI_freelistN(&anim_data);
+
+	ANIM_animdata_update(ac, &anim_data);
+	ANIM_animdata_freelist(&anim_data);
 }
 
 /* ------------------- */
@@ -1548,7 +1699,7 @@ static int actkeys_mirror_exec(bContext *C, wmOperator *op)
 	if (ANIM_animdata_get_context(C, &ac) == 0)
 		return OPERATOR_CANCELLED;
 		
-	// XXX...
+	/* XXX... */
 	if (ELEM(ac.datatype, ANIMCONT_GPENCIL, ANIMCONT_MASK))
 		return OPERATOR_PASS_THROUGH;
 		
@@ -1558,9 +1709,6 @@ static int actkeys_mirror_exec(bContext *C, wmOperator *op)
 	/* mirror keyframes */
 	mirror_action_keys(&ac, mode);
 	
-	/* validate keyframes after editing */
-	ANIM_editkeyframes_refresh(&ac);
-	
 	/* set notifier that keyframes have changed */
 	WM_event_add_notifier(C, NC_ANIMATION | ND_KEYFRAME | NA_EDITED, NULL);
 	
