@@ -30,22 +30,34 @@
  */
 
 #include <errno.h>
+#include <sys/types.h>
+#include <fcntl.h>
+
+#ifndef WIN32
+#  include <unistd.h>
+#else
+#  include <io.h>
+#endif
 
 #include "MEM_guardedalloc.h"
 
 #include "DNA_userdef_types.h"
 #include "DNA_scene_types.h"	/* min/max frames */
 
-#include "BLI_path_util.h"
 #include "BLI_utildefines.h"
+#include "BLI_fileops.h"
+#include "BLI_path_util.h"
 #include "BLI_math.h"
 #include "BLI_rect.h"
+#include "BLI_task.h"
+#include "BLI_string.h"
+
+#include "BLF_translation.h"
 
 #include "BKE_context.h"
 #include "BKE_global.h"
 #include "BKE_report.h"
 #include "BKE_main.h"
-#include "BKE_library.h"
 #include "BKE_movieclip.h"
 #include "BKE_sound.h"
 #include "BKE_tracking.h"
@@ -67,6 +79,8 @@
 
 #include "UI_view2d.h"
 
+#include "PIL_time.h"
+
 #include "clip_intern.h"	// own include
 
 /******************** view navigation utilities *********************/
@@ -97,10 +111,21 @@ static void sclip_zoom_set(const bContext *C, float zoom, float location[2])
 	}
 
 	if ((U.uiflag & USER_ZOOM_TO_MOUSEPOS) && location) {
+		float dx, dy;
+
 		ED_space_clip_get_size(sc, &width, &height);
 
-		sc->xof += ((location[0] - 0.5f) * width - sc->xof) * (sc->zoom - oldzoom) / sc->zoom;
-		sc->yof += ((location[1] - 0.5f) * height - sc->yof) * (sc->zoom - oldzoom) / sc->zoom;
+		dx = ((location[0] - 0.5f) * width - sc->xof) * (sc->zoom - oldzoom) / sc->zoom;
+		dy = ((location[1] - 0.5f) * height - sc->yof) * (sc->zoom - oldzoom) / sc->zoom;
+
+		if (sc->flag & SC_LOCK_SELECTION) {
+			sc->xlockof += dx;
+			sc->ylockof += dy;
+		}
+		else {
+			sc->xof += dx;
+			sc->yof += dy;
+		}
 	}
 }
 
@@ -111,7 +136,7 @@ static void sclip_zoom_set_factor(const bContext *C, float zoomfac, float locati
 	sclip_zoom_set(C, sc->zoom * zoomfac, location);
 }
 
-static void sclip_zoom_set_factor_exec(bContext *C, wmEvent *event, float factor)
+static void sclip_zoom_set_factor_exec(bContext *C, const wmEvent *event, float factor)
 {
 	ARegion *ar = CTX_wm_region(C);
 
@@ -143,21 +168,20 @@ static void open_init(bContext *C, wmOperator *op)
 	PropertyPointerRNA *pprop;
 
 	op->customdata = pprop = MEM_callocN(sizeof(PropertyPointerRNA), "OpenPropertyPointerRNA");
-	uiIDContextProperty(C, &pprop->ptr, &pprop->prop);
+	UI_context_active_but_prop_get_templateID(C, &pprop->ptr, &pprop->prop);
 }
 
-static int open_cancel(bContext *UNUSED(C), wmOperator *op)
+static void open_cancel(bContext *UNUSED(C), wmOperator *op)
 {
 	MEM_freeN(op->customdata);
 	op->customdata = NULL;
-
-	return OPERATOR_CANCELLED;
 }
 
 static int open_exec(bContext *C, wmOperator *op)
 {
 	SpaceClip *sc = CTX_wm_space_clip(C);
 	bScreen *screen = CTX_wm_screen(C);
+	Main *bmain = CTX_data_main(C);
 	PropertyPointerRNA *pprop;
 	PointerRNA idptr;
 	MovieClip *clip = NULL;
@@ -167,7 +191,7 @@ static int open_exec(bContext *C, wmOperator *op)
 		PointerRNA fileptr;
 		PropertyRNA *prop;
 		char dir_only[FILE_MAX], file_only[FILE_MAX];
-		int relative = RNA_boolean_get(op->ptr, "relative_path");
+		bool relative = RNA_boolean_get(op->ptr, "relative_path");
 
 		RNA_string_get(op->ptr, "directory", dir_only);
 		if (relative)
@@ -180,7 +204,7 @@ static int open_exec(bContext *C, wmOperator *op)
 		BLI_join_dirfile(str, sizeof(str), dir_only, file_only);
 	}
 	else {
-		BKE_reportf(op->reports, RPT_ERROR, "No files selected to be opened");
+		BKE_report(op->reports, RPT_ERROR, "No files selected to be opened");
 
 		return OPERATOR_CANCELLED;
 	}
@@ -189,14 +213,14 @@ static int open_exec(bContext *C, wmOperator *op)
 
 	errno = 0;
 
-	clip = BKE_movieclip_file_add(str);
+	clip = BKE_movieclip_file_add(bmain, str);
 
 	if (!clip) {
 		if (op->customdata)
 			MEM_freeN(op->customdata);
 
-		BKE_reportf(op->reports, RPT_ERROR, "Can't read: \"%s\", %s.", str,
-		            errno ? strerror(errno) : "Unsupported movie clip format");
+		BKE_reportf(op->reports, RPT_ERROR, "Cannot read '%s': %s", str,
+		            errno ? strerror(errno) : TIP_("unsupported movie clip format"));
 
 		return OPERATOR_CANCELLED;
 	}
@@ -227,7 +251,7 @@ static int open_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int open_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int open_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	SpaceClip *sc = CTX_wm_space_clip(C);
 	char path[FILE_MAX];
@@ -237,13 +261,13 @@ static int open_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
 		clip = ED_space_clip_get_clip(sc);
 
 	if (clip) {
-		strncpy(path, clip->name, sizeof(path));
+		BLI_strncpy(path, clip->name, sizeof(path));
 
 		BLI_path_abs(path, G.main->name);
 		BLI_parent_dir(path);
 	}
 	else {
-		strncpy(path, U.textudir, sizeof(path));
+		BLI_strncpy(path, U.textudir, sizeof(path));
 	}
 
 	if (RNA_struct_property_is_set(op->ptr, "files"))
@@ -315,13 +339,13 @@ typedef struct ViewPanData {
 	float *vec;
 } ViewPanData;
 
-static void view_pan_init(bContext *C, wmOperator *op, wmEvent *event)
+static void view_pan_init(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	SpaceClip *sc = CTX_wm_space_clip(C);
 	ViewPanData *vpd;
 
 	op->customdata = vpd = MEM_callocN(sizeof(ViewPanData), "ClipViewPanData");
-	WM_cursor_modal(CTX_wm_window(C), BC_NSEW_SCROLLCURSOR);
+	WM_cursor_modal_set(CTX_wm_window(C), BC_NSEW_SCROLLCURSOR);
 
 	vpd->x = event->x;
 	vpd->y = event->y;
@@ -339,7 +363,7 @@ static void view_pan_init(bContext *C, wmOperator *op, wmEvent *event)
 	WM_event_add_modal_handler(C, op);
 }
 
-static void view_pan_exit(bContext *C, wmOperator *op, int cancel)
+static void view_pan_exit(bContext *C, wmOperator *op, bool cancel)
 {
 	ViewPanData *vpd = op->customdata;
 
@@ -349,7 +373,7 @@ static void view_pan_exit(bContext *C, wmOperator *op, int cancel)
 		ED_region_tag_redraw(CTX_wm_region(C));
 	}
 
-	WM_cursor_restore(CTX_wm_window(C));
+	WM_cursor_modal_restore(CTX_wm_window(C));
 	MEM_freeN(op->customdata);
 }
 
@@ -374,14 +398,14 @@ static int view_pan_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int view_pan_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int view_pan_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	if (event->type == MOUSEPAN) {
 		SpaceClip *sc = CTX_wm_space_clip(C);
 		float offset[2];
 
-		offset[0] = (event->x - event->prevx) / sc->zoom;
-		offset[1] = (event->y - event->prevy) / sc->zoom;
+		offset[0] = (event->prevx - event->x) / sc->zoom;
+		offset[1] = (event->prevy - event->y) / sc->zoom;
 
 		RNA_float_set_array(op->ptr, "offset", offset);
 
@@ -396,7 +420,7 @@ static int view_pan_invoke(bContext *C, wmOperator *op, wmEvent *event)
 	}
 }
 
-static int view_pan_modal(bContext *C, wmOperator *op, wmEvent *event)
+static int view_pan_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	SpaceClip *sc = CTX_wm_space_clip(C);
 	ViewPanData *vpd = op->customdata;
@@ -430,11 +454,9 @@ static int view_pan_modal(bContext *C, wmOperator *op, wmEvent *event)
 	return OPERATOR_RUNNING_MODAL;
 }
 
-static int view_pan_cancel(bContext *C, wmOperator *op)
+static void view_pan_cancel(bContext *C, wmOperator *op)
 {
-	view_pan_exit(C, op, 1);
-
-	return OPERATOR_CANCELLED;
+	view_pan_exit(C, op, true);
 }
 
 void CLIP_OT_view_pan(wmOperatorType *ot)
@@ -466,17 +488,24 @@ typedef struct ViewZoomData {
 	float zoom;
 	int event_type;
 	float location[2];
+	wmTimer *timer;
+	double timer_lastdraw;
 } ViewZoomData;
 
-static void view_zoom_init(bContext *C, wmOperator *op, wmEvent *event)
+static void view_zoom_init(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	SpaceClip *sc = CTX_wm_space_clip(C);
 	ARegion *ar = CTX_wm_region(C);
-
 	ViewZoomData *vpd;
 
 	op->customdata = vpd = MEM_callocN(sizeof(ViewZoomData), "ClipViewZoomData");
-	WM_cursor_modal(CTX_wm_window(C), BC_NSEW_SCROLLCURSOR);
+	WM_cursor_modal_set(CTX_wm_window(C), BC_NSEW_SCROLLCURSOR);
+
+	if (U.viewzoom == USER_ZOOM_CONT) {
+		/* needs a timer to continue redrawing */
+		vpd->timer = WM_event_add_timer(CTX_wm_manager(C), CTX_wm_window(C), TIMER, 0.01f);
+		vpd->timer_lastdraw = PIL_check_seconds_timer();
+	}
 
 	vpd->x = event->x;
 	vpd->y = event->y;
@@ -488,7 +517,7 @@ static void view_zoom_init(bContext *C, wmOperator *op, wmEvent *event)
 	WM_event_add_modal_handler(C, op);
 }
 
-static void view_zoom_exit(bContext *C, wmOperator *op, int cancel)
+static void view_zoom_exit(bContext *C, wmOperator *op, bool cancel)
 {
 	SpaceClip *sc = CTX_wm_space_clip(C);
 	ViewZoomData *vpd = op->customdata;
@@ -498,7 +527,11 @@ static void view_zoom_exit(bContext *C, wmOperator *op, int cancel)
 		ED_region_tag_redraw(CTX_wm_region(C));
 	}
 
-	WM_cursor_restore(CTX_wm_window(C));
+	if (vpd->timer) {
+		WM_event_remove_timer(CTX_wm_manager(C), vpd->timer->win, vpd->timer);
+	}
+
+	WM_cursor_modal_restore(CTX_wm_window(C));
 	MEM_freeN(op->customdata);
 }
 
@@ -511,12 +544,17 @@ static int view_zoom_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int view_zoom_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int view_zoom_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
-	if (event->type == MOUSEZOOM) {
-		float factor;
+	if (event->type == MOUSEZOOM || event->type == MOUSEPAN) {
+		float delta, factor;
 
-		factor = 1.0f + (event->x - event->prevx + event->y - event->prevy) / 300.0f;
+		delta = event->prevx - event->x + event->prevy - event->y;
+
+		if (U.uiflag & USER_ZOOM_INVERT)
+			delta *= -1;
+
+		factor = 1.0f + delta / 300.0f;
 		RNA_float_set(op->ptr, "factor", factor);
 
 		sclip_zoom_set_factor_exec(C, event, factor);
@@ -530,17 +568,61 @@ static int view_zoom_invoke(bContext *C, wmOperator *op, wmEvent *event)
 	}
 }
 
-static int view_zoom_modal(bContext *C, wmOperator *op, wmEvent *event)
+static void view_zoom_apply(bContext *C,
+                            ViewZoomData *vpd,
+                            wmOperator *op,
+                            const wmEvent *event)
 {
-	ViewZoomData *vpd = op->customdata;
 	float factor;
 
+	if (U.viewzoom == USER_ZOOM_CONT) {
+		SpaceClip *sclip = CTX_wm_space_clip(C);
+		double time = PIL_check_seconds_timer();
+		float time_step = (float)(time - vpd->timer_lastdraw);
+		float fac;
+		float zfac;
+
+		if (U.uiflag & USER_ZOOM_HORIZ) {
+			fac = (float)(event->x - vpd->x);
+		}
+		else {
+			fac = (float)(event->y - vpd->y);
+		}
+
+		if (U.uiflag & USER_ZOOM_INVERT) {
+			fac = -fac;
+		}
+
+		zfac = 1.0f + ((fac / 20.0f) * time_step);
+		vpd->timer_lastdraw = time;
+		factor = (sclip->zoom * zfac) / vpd->zoom;
+	}
+	else {
+		float delta = event->x - vpd->x + event->y - vpd->y;
+
+		if (U.uiflag & USER_ZOOM_INVERT) {
+			delta *= -1;
+		}
+
+		factor = 1.0f + delta / 300.0f;
+	}
+
+	RNA_float_set(op->ptr, "factor", factor);
+	sclip_zoom_set(C, vpd->zoom * factor, vpd->location);
+	ED_region_tag_redraw(CTX_wm_region(C));
+}
+
+static int view_zoom_modal(bContext *C, wmOperator *op, const wmEvent *event)
+{
+	ViewZoomData *vpd = op->customdata;
 	switch (event->type) {
+		case TIMER:
+			if (event->customdata == vpd->timer) {
+				view_zoom_apply(C, vpd, op, event);
+			}
+			break;
 		case MOUSEMOVE:
-			factor = 1.0f + (vpd->x - event->x + vpd->y - event->y) / 300.0f;
-			RNA_float_set(op->ptr, "factor", factor);
-			sclip_zoom_set(C, vpd->zoom * factor, vpd->location);
-			ED_region_tag_redraw(CTX_wm_region(C));
+			view_zoom_apply(C, vpd, op, event);
 			break;
 		default:
 			if (event->type == vpd->event_type && event->val == KM_RELEASE) {
@@ -554,15 +636,15 @@ static int view_zoom_modal(bContext *C, wmOperator *op, wmEvent *event)
 	return OPERATOR_RUNNING_MODAL;
 }
 
-static int view_zoom_cancel(bContext *C, wmOperator *op)
+static void view_zoom_cancel(bContext *C, wmOperator *op)
 {
-	view_zoom_exit(C, op, 1);
-
-	return OPERATOR_CANCELLED;
+	view_zoom_exit(C, op, true);
 }
 
 void CLIP_OT_view_zoom(wmOperatorType *ot)
 {
+	PropertyRNA *prop;
+
 	/* identifiers */
 	ot->name = "View Zoom";
 	ot->idname = "CLIP_OT_view_zoom";
@@ -579,8 +661,9 @@ void CLIP_OT_view_zoom(wmOperatorType *ot)
 	ot->flag = OPTYPE_BLOCKING | OPTYPE_GRAB_POINTER;
 
 	/* properties */
-	RNA_def_float(ot->srna, "factor", 0.0f, 0.0f, FLT_MAX,
-	              "Factor", "Zoom factor, values higher than 1.0 zoom in, lower values zoom out", -FLT_MAX, FLT_MAX);
+	prop = RNA_def_float(ot->srna, "factor", 0.0f, -FLT_MAX, FLT_MAX, "Factor",
+	                     "Zoom factor, values higher than 1.0 zoom in, lower values zoom out", -FLT_MAX, FLT_MAX);
+	RNA_def_property_flag(prop, PROP_HIDDEN);
 }
 
 /********************** view zoom in/out operator *********************/
@@ -591,14 +674,14 @@ static int view_zoom_in_exec(bContext *C, wmOperator *op)
 
 	RNA_float_get_array(op->ptr, "location", location);
 
-	sclip_zoom_set_factor(C, 1.25f, location);
+	sclip_zoom_set_factor(C, powf(2.0f, 1.0f / 3.0f), location);
 
 	ED_region_tag_redraw(CTX_wm_region(C));
 
 	return OPERATOR_FINISHED;
 }
 
-static int view_zoom_in_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int view_zoom_in_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	SpaceClip *sc = CTX_wm_space_clip(C);
 	ARegion *ar = CTX_wm_region(C);
@@ -613,6 +696,8 @@ static int view_zoom_in_invoke(bContext *C, wmOperator *op, wmEvent *event)
 
 void CLIP_OT_view_zoom_in(wmOperatorType *ot)
 {
+	PropertyRNA *prop;
+
 	/* identifiers */
 	ot->name = "View Zoom In";
 	ot->idname = "CLIP_OT_view_zoom_in";
@@ -624,8 +709,9 @@ void CLIP_OT_view_zoom_in(wmOperatorType *ot)
 	ot->poll = ED_space_clip_view_clip_poll;
 
 	/* properties */
-	RNA_def_float_vector(ot->srna, "location", 2, NULL, -FLT_MAX, FLT_MAX, "Location",
-	                     "Cursor location in screen coordinates", -10.0f, 10.0f);
+	prop = RNA_def_float_vector(ot->srna, "location", 2, NULL, -FLT_MAX, FLT_MAX, "Location",
+	                            "Cursor location in screen coordinates", -10.0f, 10.0f);
+	RNA_def_property_flag(prop, PROP_HIDDEN);
 }
 
 static int view_zoom_out_exec(bContext *C, wmOperator *op)
@@ -634,14 +720,14 @@ static int view_zoom_out_exec(bContext *C, wmOperator *op)
 
 	RNA_float_get_array(op->ptr, "location", location);
 
-	sclip_zoom_set_factor(C, 0.8f, location);
+	sclip_zoom_set_factor(C, powf(0.5f, 1.0f / 3.0f), location);
 
 	ED_region_tag_redraw(CTX_wm_region(C));
 
 	return OPERATOR_FINISHED;
 }
 
-static int view_zoom_out_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int view_zoom_out_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	SpaceClip *sc = CTX_wm_space_clip(C);
 	ARegion *ar = CTX_wm_region(C);
@@ -656,6 +742,8 @@ static int view_zoom_out_invoke(bContext *C, wmOperator *op, wmEvent *event)
 
 void CLIP_OT_view_zoom_out(wmOperatorType *ot)
 {
+	PropertyRNA *prop;
+
 	/* identifiers */
 	ot->name = "View Zoom Out";
 	ot->idname = "CLIP_OT_view_zoom_out";
@@ -667,8 +755,9 @@ void CLIP_OT_view_zoom_out(wmOperatorType *ot)
 	ot->poll = ED_space_clip_view_clip_poll;
 
 	/* properties */
-	RNA_def_float_vector(ot->srna, "location", 2, NULL, -FLT_MAX, FLT_MAX, "Location",
-	                     "Cursor location in normalized (0.0-1.0) coordinates", -10.0f, 10.0f);
+	prop = RNA_def_float_vector(ot->srna, "location", 2, NULL, -FLT_MAX, FLT_MAX, "Location",
+	                            "Cursor location in normalized (0.0-1.0) coordinates", -10.0f, 10.0f);
+	RNA_def_property_flag(prop, PROP_HIDDEN);
 }
 
 /********************** view zoom ratio operator *********************/
@@ -700,7 +789,7 @@ void CLIP_OT_view_zoom_ratio(wmOperatorType *ot)
 	ot->poll = ED_space_clip_view_clip_poll;
 
 	/* properties */
-	RNA_def_float(ot->srna, "ratio", 0.0f, 0.0f, FLT_MAX,
+	RNA_def_float(ot->srna, "ratio", 0.0f, -FLT_MAX, FLT_MAX,
 	              "Ratio", "Zoom ratio, 1.0 is 1:1, higher is zoomed in, lower is zoomed out", -FLT_MAX, FLT_MAX);
 }
 
@@ -712,7 +801,7 @@ static int view_all_exec(bContext *C, wmOperator *op)
 	ARegion *ar;
 	int w, h, width, height;
 	float aspx, aspy;
-	int fit_view = RNA_boolean_get(op->ptr, "fit_view");
+	bool fit_view = RNA_boolean_get(op->ptr, "fit_view");
 	float zoomx, zoomy;
 
 	/* retrieve state */
@@ -735,7 +824,7 @@ static int view_all_exec(bContext *C, wmOperator *op)
 		zoomx = (float) width / (w + 2 * margin);
 		zoomy = (float) height / (h + 2 * margin);
 
-		sclip_zoom_set(C, minf(zoomx, zoomy), NULL);
+		sclip_zoom_set(C, min_ff(zoomx, zoomy), NULL);
 	}
 	else {
 		if ((w >= width || h >= height) && (width > 0 && height > 0)) {
@@ -743,7 +832,7 @@ static int view_all_exec(bContext *C, wmOperator *op)
 			zoomy = (float) height / h;
 
 			/* find the zoom value that will fit the image in the image space */
-			sclip_zoom_set(C, 1.0f / power_of_2(1.0f / minf(zoomx, zoomy)), NULL);
+			sclip_zoom_set(C, 1.0f / power_of_2(1.0f / min_ff(zoomx, zoomy)), NULL);
 		}
 		else
 			sclip_zoom_set(C, 1.0f, NULL);
@@ -751,13 +840,15 @@ static int view_all_exec(bContext *C, wmOperator *op)
 
 	sc->xof = sc->yof = 0.0f;
 
-	ED_region_tag_redraw(CTX_wm_region(C));
+	ED_region_tag_redraw(ar);
 
 	return OPERATOR_FINISHED;
 }
 
 void CLIP_OT_view_all(wmOperatorType *ot)
 {
+	PropertyRNA *prop;
+
 	/* identifiers */
 	ot->name = "View All";
 	ot->idname = "CLIP_OT_view_all";
@@ -768,7 +859,8 @@ void CLIP_OT_view_all(wmOperatorType *ot)
 	ot->poll = ED_space_clip_view_clip_poll;
 
 	/* properties */
-	RNA_def_boolean(ot->srna, "fit_view", 0, "Fit View", "Fit frame to the viewport");
+	prop = RNA_def_boolean(ot->srna, "fit_view", 0, "Fit View", "Fit frame to the viewport");
+	RNA_def_property_flag(prop, PROP_SKIP_SAVE);
 }
 
 /********************** view selected operator *********************/
@@ -782,7 +874,7 @@ static int view_selected_exec(bContext *C, wmOperator *UNUSED(op))
 	sc->ylockof = 0.0f;
 
 	ED_clip_view_selection(C, ar, 1);
-	ED_region_tag_redraw(CTX_wm_region(C));
+	ED_region_tag_redraw(ar);
 
 	return OPERATOR_FINISHED;
 }
@@ -820,7 +912,7 @@ static void change_frame_apply(bContext *C, wmOperator *op)
 	SUBFRA = 0.0f;
 
 	/* do updates */
-	sound_seek_scene(CTX_data_main(C), CTX_data_scene(C));
+	sound_seek_scene(CTX_data_main(C), scene);
 	WM_event_add_notifier(C, NC_SCENE | ND_FRAME, scene);
 }
 
@@ -831,7 +923,7 @@ static int change_frame_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int frame_from_event(bContext *C, wmEvent *event)
+static int frame_from_event(bContext *C, const wmEvent *event)
 {
 	ARegion *ar = CTX_wm_region(C);
 	Scene *scene = CTX_data_scene(C);
@@ -847,13 +939,13 @@ static int frame_from_event(bContext *C, wmEvent *event)
 
 		UI_view2d_region_to_view(&ar->v2d, event->mval[0], event->mval[1], &viewx, &viewy);
 
-		framenr = (int) floor(viewx + 0.5f);
+		framenr = iroundf(viewx);
 	}
 
 	return framenr;
 }
 
-static int change_frame_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int change_frame_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	ARegion *ar = CTX_wm_region(C);
 
@@ -872,7 +964,7 @@ static int change_frame_invoke(bContext *C, wmOperator *op, wmEvent *event)
 	return OPERATOR_RUNNING_MODAL;
 }
 
-static int change_frame_modal(bContext *C, wmOperator *op, wmEvent *event)
+static int change_frame_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	switch (event->type) {
 		case ESCKEY:
@@ -896,7 +988,7 @@ static int change_frame_modal(bContext *C, wmOperator *op, wmEvent *event)
 void CLIP_OT_change_frame(wmOperatorType *ot)
 {
 	/* identifiers */
-	ot->name = "Change frame";
+	ot->name = "Change Frame";
 	ot->idname = "CLIP_OT_change_frame";
 	ot->description = "Interactively change the current frame number";
 
@@ -919,7 +1011,8 @@ typedef struct ProxyBuildJob {
 	Scene *scene;
 	struct Main *main;
 	MovieClip *clip;
-	int clip_flag, stop;
+	int clip_flag;
+	bool stop;
 	struct IndexBuildContext *index_context;
 } ProxyJob;
 
@@ -958,15 +1051,234 @@ static int proxy_bitflag_to_array(int size_flag, int build_sizes[4], int undisto
 	return build_count;
 }
 
-/* only this runs inside thread */
-static void proxy_startjob(void *pjv, short *stop, short *do_update, float *progress)
+/* simple case for movies -- handle frame-by-frame, do threading within single frame */
+static void do_movie_proxy(void *pjv, int *UNUSED(build_sizes), int UNUSED(build_count),
+                           int *build_undistort_sizes, int build_undistort_count,
+                           short *stop, short *do_update, float *progress)
 {
 	ProxyJob *pj = pjv;
 	Scene *scene = pj->scene;
 	MovieClip *clip = pj->clip;
 	struct MovieDistortion *distortion = NULL;
-	short size_flag;
 	int cfra, sfra = SFRA, efra = EFRA;
+
+	if (pj->index_context)
+		IMB_anim_index_rebuild(pj->index_context, stop, do_update, progress);
+
+	if (!build_undistort_count) {
+		if (*stop)
+			pj->stop = 1;
+
+		return;
+	}
+	else {
+		sfra = 1;
+		efra = clip->len;
+	}
+
+	if (build_undistort_count) {
+		int threads = BLI_system_thread_count();
+		int width, height;
+
+		BKE_movieclip_get_size(clip, NULL, &width, &height);
+
+		distortion = BKE_tracking_distortion_new(&clip->tracking, width, height);
+		BKE_tracking_distortion_set_threads(distortion, threads);
+	}
+
+	for (cfra = sfra; cfra <= efra; cfra++) {
+		BKE_movieclip_build_proxy_frame(clip, pj->clip_flag, distortion, cfra,
+		                                build_undistort_sizes, build_undistort_count, 1);
+
+		if (*stop || G.is_break)
+			break;
+
+		*do_update = true;
+		*progress = ((float) cfra - sfra) / (efra - sfra);
+	}
+
+	if (distortion)
+		BKE_tracking_distortion_free(distortion);
+
+	if (*stop)
+		pj->stop = 1;
+}
+
+/* *****
+ * special case for sequences -- handle different frames in different threads,
+ * loading from disk happens in critical section, decoding frame happens from
+ * thread for maximal speed
+ */
+
+typedef struct ProxyQueue {
+	int cfra;
+	int sfra;
+	int efra;
+	SpinLock spin;
+
+	const short *stop;
+	short *do_update;
+	float *progress;
+} ProxyQueue;
+
+typedef struct ProxyThread {
+	MovieClip *clip;
+	struct MovieDistortion *distortion;
+	int *build_sizes, build_count;
+	int *build_undistort_sizes, build_undistort_count;
+} ProxyThread;
+
+static unsigned char *proxy_thread_next_frame(ProxyQueue *queue, MovieClip *clip, size_t *size_r, int *cfra_r)
+{
+	unsigned char *mem = NULL;
+
+	BLI_spin_lock(&queue->spin);
+	if (!*queue->stop && queue->cfra <= queue->efra) {
+		MovieClipUser user = {0};
+		char name[FILE_MAX];
+		size_t size;
+		int file;
+
+		user.framenr = queue->cfra;
+
+		BKE_movieclip_filename_for_frame(clip, &user, name);
+
+		file = BLI_open(name, O_BINARY | O_RDONLY, 0);
+		if (file < 0) {
+			BLI_spin_unlock(&queue->spin);
+			return NULL;
+		}
+
+		size = BLI_file_descriptor_size(file);
+		if (size < 1) {
+			close(file);
+			BLI_spin_unlock(&queue->spin);
+			return NULL;
+		}
+
+		mem = MEM_mallocN(size, "movieclip proxy memory file");
+
+		if (read(file, mem, size) != size) {
+			close(file);
+			BLI_spin_unlock(&queue->spin);
+			MEM_freeN(mem);
+			return NULL;
+		}
+
+		*size_r = size;
+		*cfra_r = queue->cfra;
+
+		queue->cfra++;
+		close(file);
+
+		*queue->do_update = 1;
+		*queue->progress = (float)(queue->cfra - queue->sfra) / (queue->efra - queue->sfra);
+	}
+	BLI_spin_unlock(&queue->spin);
+
+	return mem;
+}
+
+static void proxy_task_func(TaskPool *pool, void *task_data, int UNUSED(threadid))
+{
+	ProxyThread *data = (ProxyThread *)task_data;
+	ProxyQueue *queue = (ProxyQueue *)BLI_task_pool_userdata(pool);
+	unsigned char *mem;
+	size_t size;
+	int cfra;
+
+	while ((mem = proxy_thread_next_frame(queue, data->clip, &size, &cfra))) {
+		ImBuf *ibuf;
+
+		ibuf = IMB_ibImageFromMemory(mem, size, IB_rect | IB_multilayer | IB_alphamode_detect,
+		                             data->clip->colorspace_settings.name, "proxy frame");
+
+		BKE_movieclip_build_proxy_frame_for_ibuf(data->clip, ibuf, NULL, cfra,
+		                                         data->build_sizes, data->build_count, false);
+
+		BKE_movieclip_build_proxy_frame_for_ibuf(data->clip, ibuf, data->distortion, cfra,
+		                                         data->build_undistort_sizes, data->build_undistort_count, true);
+
+		IMB_freeImBuf(ibuf);
+
+		MEM_freeN(mem);
+	}
+}
+
+static void do_sequence_proxy(void *pjv, int *build_sizes, int build_count,
+                              int *build_undistort_sizes, int build_undistort_count,
+                              short *stop, short *do_update, float *progress)
+{
+	ProxyJob *pj = pjv;
+	MovieClip *clip = pj->clip;
+	Scene *scene = pj->scene;
+	TaskScheduler *task_scheduler = BLI_task_scheduler_get();
+	TaskPool *task_pool;
+	int sfra = SFRA, efra = EFRA;
+	ProxyThread *handles;
+	int i, tot_thread = BLI_task_scheduler_num_threads(task_scheduler);
+	int width, height;
+	ProxyQueue queue;
+
+	if (build_undistort_count) {
+		BKE_movieclip_get_size(clip, NULL, &width, &height);
+	}
+
+	BLI_spin_init(&queue.spin);
+
+	queue.cfra = sfra;
+	queue.sfra = sfra;
+	queue.efra = efra;
+	queue.stop = stop;
+	queue.do_update = do_update;
+	queue.progress = progress;
+
+	task_pool = BLI_task_pool_create(task_scheduler, &queue);
+	handles = MEM_callocN(sizeof(ProxyThread) * tot_thread,
+	                      "proxy threaded handles");
+	for (i = 0; i < tot_thread; i++) {
+		ProxyThread *handle = &handles[i];
+
+		handle->clip = clip;
+
+		handle->build_count = build_count;
+		handle->build_sizes = build_sizes;
+
+		handle->build_undistort_count = build_undistort_count;
+		handle->build_undistort_sizes = build_undistort_sizes;
+
+		if (build_undistort_count) {
+			handle->distortion = BKE_tracking_distortion_new(&clip->tracking,
+			                                                 width, height);
+		}
+
+		BLI_task_pool_push(task_pool,
+		                   proxy_task_func,
+		                   handle,
+		                   false,
+		                   TASK_PRIORITY_LOW);
+	}
+
+	BLI_task_pool_work_and_wait(task_pool);
+	BLI_task_pool_free(task_pool);
+
+	if (build_undistort_count) {
+		for (i = 0; i < tot_thread; i++) {
+			ProxyThread *handle = &handles[i];
+			BKE_tracking_distortion_free(handle->distortion);
+		}
+	}
+
+	BLI_spin_end(&queue.spin);
+	MEM_freeN(handles);
+}
+
+static void proxy_startjob(void *pjv, short *stop, short *do_update, float *progress)
+{
+	ProxyJob *pj = pjv;
+	MovieClip *clip = pj->clip;
+
+	short size_flag;
 	int build_sizes[4], build_count = 0;
 	int build_undistort_sizes[4], build_undistort_count = 0;
 
@@ -976,43 +1288,13 @@ static void proxy_startjob(void *pjv, short *stop, short *do_update, float *prog
 	build_undistort_count = proxy_bitflag_to_array(size_flag, build_undistort_sizes, 1);
 
 	if (clip->source == MCLIP_SRC_MOVIE) {
-		if (pj->index_context)
-			IMB_anim_index_rebuild(pj->index_context, stop, do_update, progress);
-
-		if (!build_undistort_count) {
-			if (*stop)
-				pj->stop = 1;
-
-			return;
-		}
-		else {
-			sfra = 1;
-			efra = IMB_anim_get_duration(clip->anim, IMB_TC_NONE);
-		}
+		do_movie_proxy(pjv, build_sizes, build_count, build_undistort_sizes,
+		               build_undistort_count, stop, do_update, progress);
 	}
-
-	if (build_undistort_count)
-		distortion = BKE_tracking_distortion_new();
-
-	for (cfra = sfra; cfra <= efra; cfra++) {
-		if (clip->source != MCLIP_SRC_MOVIE)
-			BKE_movieclip_build_proxy_frame(clip, pj->clip_flag, NULL, cfra, build_sizes, build_count, 0);
-
-		BKE_movieclip_build_proxy_frame(clip, pj->clip_flag, distortion, cfra,
-		                                build_undistort_sizes, build_undistort_count, 1);
-
-		if (*stop || G.is_break)
-			break;
-
-		*do_update = TRUE;
-		*progress = ((float) cfra - sfra) / (efra - sfra);
+	else {
+		do_sequence_proxy(pjv, build_sizes, build_count, build_undistort_sizes,
+		                 build_undistort_count, stop, do_update, progress);
 	}
-
-	if (distortion)
-		BKE_tracking_distortion_free(distortion);
-
-	if (*stop)
-		pj->stop = 1;
 }
 
 static void proxy_endjob(void *pjv)
@@ -1025,7 +1307,14 @@ static void proxy_endjob(void *pjv)
 	if (pj->index_context)
 		IMB_anim_index_rebuild_finish(pj->index_context, pj->stop);
 
-	BKE_movieclip_reload(pj->clip);
+	if (pj->clip->source == MCLIP_SRC_MOVIE) {
+		/* Timecode might have changed, so do a full reload to deal with this. */
+		BKE_movieclip_reload(pj->clip);
+	}
+	else {
+		/* For image sequences we'll preserve original cache. */
+		BKE_movieclip_clear_proxy_cache(pj->clip);
+	}
 
 	WM_main_add_notifier(NC_MOVIECLIP | ND_DISPLAY, pj->clip);
 }
@@ -1053,17 +1342,17 @@ static int clip_rebuild_proxy_exec(bContext *C, wmOperator *UNUSED(op))
 
 	if (clip->anim) {
 		pj->index_context = IMB_anim_index_rebuild_context(clip->anim, clip->proxy.build_tc_flag,
-					clip->proxy.build_size_flag, clip->proxy.quality);
+		                                                   clip->proxy.build_size_flag, clip->proxy.quality);
 	}
 
 	WM_jobs_customdata_set(wm_job, pj, proxy_freejob);
 	WM_jobs_timer(wm_job, 0.2, NC_MOVIECLIP | ND_DISPLAY, 0);
 	WM_jobs_callbacks(wm_job, proxy_startjob, NULL, NULL, proxy_endjob);
 
-	G.is_break = FALSE;
+	G.is_break = false;
 	WM_jobs_start(CTX_wm_manager(C), wm_job);
 
-	ED_area_tag_redraw(CTX_wm_area(C));
+	ED_area_tag_redraw(sa);
 
 	return OPERATOR_FINISHED;
 }
@@ -1121,39 +1410,26 @@ void CLIP_OT_mode_set(wmOperatorType *ot)
  * that explains the negative signs in the code below
  */
 
-static int clip_view_ndof_invoke(bContext *C, wmOperator *UNUSED(op), wmEvent *event)
+static int clip_view_ndof_invoke(bContext *C, wmOperator *UNUSED(op), const wmEvent *event)
 {
 	if (event->type != NDOF_MOTION)
 		return OPERATOR_CANCELLED;
 	else {
 		SpaceClip *sc = CTX_wm_space_clip(C);
 		ARegion *ar = CTX_wm_region(C);
+		float pan_vec[3];
 
-		wmNDOFMotionData *ndof = (wmNDOFMotionData *) event->customdata;
+		const wmNDOFMotionData *ndof = event->customdata;
+		const float speed = NDOF_PIXELS_PER_SECOND;
 
-		float dt = ndof->dt;
+		WM_event_ndof_pan_get(ndof, pan_vec, true);
 
-		/* tune these until it feels right */
-		const float zoom_sensitivity = 0.5f;  /* 50% per second (I think) */
-		const float pan_sensitivity = 300.0f; /* screen pixels per second */
+		mul_v2_fl(pan_vec, (speed * ndof->dt) / sc->zoom);
+		pan_vec[2] *= -ndof->dt;
 
-		float pan_x = pan_sensitivity * dt * ndof->tvec[0] / sc->zoom;
-		float pan_y = pan_sensitivity * dt * ndof->tvec[1] / sc->zoom;
-
-		/* "mouse zoom" factor = 1 + (dx + dy) / 300
-		 * what about "ndof zoom" factor? should behave like this:
-		 * at rest -> factor = 1
-		 * move forward -> factor > 1
-		 * move backward -> factor < 1
-		 */
-		float zoom_factor = 1.0f + zoom_sensitivity * dt * - ndof->tvec[2];
-
-		if (U.ndof_flag & NDOF_ZOOM_INVERT)
-			zoom_factor = -zoom_factor;
-
-		sclip_zoom_set_factor(C, zoom_factor, NULL);
-		sc->xof += pan_x;
-		sc->yof += pan_y;
+		sclip_zoom_set_factor(C, 1.0f + pan_vec[2], NULL);
+		sc->xof += pan_vec[0];
+		sc->yof += pan_vec[1];
 
 		ED_region_tag_redraw(ar);
 
@@ -1170,6 +1446,135 @@ void CLIP_OT_view_ndof(wmOperatorType *ot)
 
 	/* api callbacks */
 	ot->invoke = clip_view_ndof_invoke;
+	ot->poll = ED_space_clip_view_clip_poll;
+}
+
+/********************** Prefetch operator *********************/
+
+static int clip_prefetch_modal(bContext *C, wmOperator *UNUSED(op), const wmEvent *event)
+{
+	/* no running blender, remove handler and pass through */
+	if (0 == WM_jobs_test(CTX_wm_manager(C), CTX_wm_area(C), WM_JOB_TYPE_CLIP_PREFETCH))
+		return OPERATOR_FINISHED | OPERATOR_PASS_THROUGH;
+
+	/* running render */
+	switch (event->type) {
+		case ESCKEY:
+			return OPERATOR_RUNNING_MODAL;
+	}
+
+	return OPERATOR_PASS_THROUGH;
+}
+
+static int clip_prefetch_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(_event))
+{
+	clip_start_prefetch_job(C);
+
+	/* add modal handler for ESC */
+	WM_event_add_modal_handler(C, op);
+
+	return OPERATOR_RUNNING_MODAL;
+}
+
+void CLIP_OT_prefetch(wmOperatorType *ot)
+{
+	/* identifiers */
+	ot->name = "Prefetch Frames";
+	ot->idname = "CLIP_OT_prefetch";
+	ot->description = "Prefetch frames from disk for faster playback/tracking";
+
+	/* api callbacks */
+	ot->poll = ED_space_clip_view_clip_poll;
+	ot->invoke = clip_prefetch_invoke;
+	ot->modal = clip_prefetch_modal;
+}
+
+/********************** Set scene frames *********************/
+
+static int clip_set_scene_frames_exec(bContext *C, wmOperator *UNUSED(op))
+{
+	MovieClip *clip = CTX_data_edit_movieclip(C);
+	Scene *scene = CTX_data_scene(C);
+	int clip_length;
+
+	if (ELEM(NULL, scene, clip))
+		return OPERATOR_CANCELLED;
+
+	clip_length = BKE_movieclip_get_duration(clip);
+
+	scene->r.sfra = clip->start_frame;
+	scene->r.efra = scene->r.sfra + clip_length - 1;
+
+	scene->r.efra = max_ii(scene->r.sfra, scene->r.efra);
+
+	WM_event_add_notifier(C, NC_SCENE | ND_FRAME, scene);
+
+	return OPERATOR_FINISHED;
+}
+
+void CLIP_OT_set_scene_frames(wmOperatorType *ot)
+{
+	/* identifiers */
+	ot->name = "Set Scene Frames";
+	ot->idname = "CLIP_OT_set_scene_frames";
+	ot->description = "Set scene's start and end frame to match clip's start frame and length";
+
+	/* api callbacks */
+	ot->poll = ED_space_clip_view_clip_poll;
+	ot->exec = clip_set_scene_frames_exec;
+}
+
+/******************** set 3d cursor operator ********************/
+
+static int clip_set_2d_cursor_exec(bContext *C, wmOperator *op)
+{
+	SpaceClip *sclip = CTX_wm_space_clip(C);
+	bool show_cursor = false;
+
+	show_cursor |= sclip->mode == SC_MODE_MASKEDIT;
+	show_cursor |= sclip->around == V3D_CURSOR;
+
+	if (!show_cursor) {
+		return OPERATOR_CANCELLED;
+	}
+
+	RNA_float_get_array(op->ptr, "location", sclip->cursor);
+
+	WM_event_add_notifier(C, NC_SPACE | ND_SPACE_CLIP, NULL);
+
+	return OPERATOR_FINISHED;
+}
+
+static int clip_set_2d_cursor_invoke(bContext *C, wmOperator *op, const wmEvent *event)
+{
+	ARegion *ar = CTX_wm_region(C);
+	SpaceClip *sclip = CTX_wm_space_clip(C);
+	float location[2];
+
+	ED_clip_mouse_pos(sclip, ar, event->mval, location);
+	RNA_float_set_array(op->ptr, "location", location);
+
+	return clip_set_2d_cursor_exec(C, op);
+}
+
+void CLIP_OT_cursor_set(wmOperatorType *ot)
+{
+	/* identifiers */
+	ot->name = "Set 2D Cursor";
+	ot->description = "Set 2D cursor location";
+	ot->idname = "CLIP_OT_cursor_set";
+
+	/* api callbacks */
+	ot->exec = clip_set_2d_cursor_exec;
+	ot->invoke = clip_set_2d_cursor_invoke;
+	ot->poll = ED_space_clip_poll;
+
+	/* flags */
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
+
+	/* properties */
+	RNA_def_float_vector(ot->srna, "location", 2, NULL, -FLT_MAX, FLT_MAX, "Location",
+	                     "Cursor location in normalized clip coordinates", -10.0f, 10.0f);
 }
 
 /********************** macroses *********************/
@@ -1190,5 +1595,5 @@ void ED_operatormacros_clip(void)
 	                                  OPTYPE_UNDO | OPTYPE_REGISTER);
 	WM_operatortype_macro_define(ot, "CLIP_OT_add_marker");
 	otmacro = WM_operatortype_macro_define(ot, "TRANSFORM_OT_translate");
-	RNA_boolean_set(otmacro->ptr, "release_confirm", TRUE);
+	RNA_boolean_set(otmacro->ptr, "release_confirm", true);
 }
