@@ -31,50 +31,26 @@
 
 #include "MEM_guardedalloc.h"
 
-#include "DNA_camera_types.h"
-#include "DNA_constraint_types.h"
-#include "DNA_gpencil_types.h"
 #include "DNA_movieclip_types.h"
-#include "DNA_object_types.h"   /* SELECT */
 #include "DNA_scene_types.h"
 
 #include "BLI_utildefines.h"
 #include "BLI_math.h"
-#include "BLI_listbase.h"
 #include "BLI_rect.h"
 #include "BLI_lasso.h"
-#include "BLI_blenlib.h"
 
-#include "BKE_main.h"
 #include "BKE_context.h"
-#include "BKE_constraint.h"
-#include "BKE_movieclip.h"
 #include "BKE_tracking.h"
-#include "BKE_global.h"
-#include "BKE_depsgraph.h"
-#include "BKE_object.h"
-#include "BKE_report.h"
-#include "BKE_scene.h"
-#include "BKE_library.h"
-#include "BKE_sound.h"
 
 #include "WM_api.h"
 #include "WM_types.h"
 
 #include "ED_screen.h"
 #include "ED_clip.h"
-#include "ED_keyframing.h"
-
-#include "IMB_imbuf_types.h"
-#include "IMB_imbuf.h"
-
-#include "UI_interface.h"
 
 #include "RNA_access.h"
 #include "RNA_define.h"
 
-#include "PIL_time.h"
-
 #include "UI_view2d.h"
 
 #include "clip_intern.h"    // own include
@@ -107,7 +83,7 @@ static int mouse_on_crns(float co[2], float pos[2], float crns[4][2], float epsx
 {
 	float dist = dist_to_crns(co, pos, crns);
 
-	return dist < maxf(epsx, epsy);
+	return dist < max_ff(epsx, epsy);
 }
 
 static int track_mouse_area(const bContext *C, float co[2], MovieTrackingTrack *track)
@@ -123,13 +99,13 @@ static int track_mouse_area(const bContext *C, float co[2], MovieTrackingTrack *
 
 	BKE_tracking_marker_pattern_minmax(marker, pat_min, pat_max);
 
-	epsx = MIN4(pat_min[0] - marker->search_min[0], marker->search_max[0] - pat_max[0],
-	            fabsf(pat_min[0]), fabsf(pat_max[0])) / 2;
-	epsy = MIN4(pat_min[1] - marker->search_min[1], marker->search_max[1] - pat_max[1],
-	            fabsf(pat_min[1]), fabsf(pat_max[1])) / 2;
+	epsx = min_ffff(pat_min[0] - marker->search_min[0], marker->search_max[0] - pat_max[0],
+	                fabsf(pat_min[0]), fabsf(pat_max[0])) / 2;
+	epsy = min_ffff(pat_min[1] - marker->search_min[1], marker->search_max[1] - pat_max[1],
+	                fabsf(pat_min[1]), fabsf(pat_max[1])) / 2;
 
-	epsx = maxf(epsx, 2.0f / width);
-	epsy = maxf(epsy, 2.0f / height);
+	epsx = max_ff(epsx, 2.0f / width);
+	epsy = max_ff(epsy, 2.0f / height);
 
 	if (sc->flag & SC_SHOW_MARKER_SEARCH) {
 		if (mouse_on_rect(co, marker->pos, marker->search_min, marker->search_max, epsx, epsy))
@@ -166,25 +142,41 @@ static float dist_to_rect(float co[2], float pos[2], float min[2], float max[2])
 	d3 = dist_squared_to_line_segment_v2(p, v3, v4);
 	d4 = dist_squared_to_line_segment_v2(p, v4, v1);
 
-	return sqrtf(MIN4(d1, d2, d3, d4));
+	return sqrtf(min_ffff(d1, d2, d3, d4));
 }
 
+/* Distance to quad defined by it's corners, corners are relative to pos */
 static float dist_to_crns(float co[2], float pos[2], float crns[4][2])
 {
 	float d1, d2, d3, d4;
 	float p[2] = {co[0] - pos[0], co[1] - pos[1]};
-	float *v1 = crns[0], *v2 = crns[1];
-	float *v3 = crns[2], *v4 = crns[3];
+	const float *v1 = crns[0], *v2 = crns[1];
+	const float *v3 = crns[2], *v4 = crns[3];
 
 	d1 = dist_squared_to_line_segment_v2(p, v1, v2);
 	d2 = dist_squared_to_line_segment_v2(p, v2, v3);
 	d3 = dist_squared_to_line_segment_v2(p, v3, v4);
 	d4 = dist_squared_to_line_segment_v2(p, v4, v1);
 
-	return sqrtf(MIN4(d1, d2, d3, d4));
+	return sqrtf(min_ffff(d1, d2, d3, d4));
 }
 
-static MovieTrackingTrack *find_nearest_track(SpaceClip *sc, ListBase *tracksbase, float co[2])
+/* Same as above, but all the coordinates are absolute */
+static float dist_to_crns_abs(float co[2], float corners[4][2])
+{
+	float d1, d2, d3, d4;
+	const float *v1 = corners[0], *v2 = corners[1];
+	const float *v3 = corners[2], *v4 = corners[3];
+
+	d1 = dist_squared_to_line_segment_v2(co, v1, v2);
+	d2 = dist_squared_to_line_segment_v2(co, v2, v3);
+	d3 = dist_squared_to_line_segment_v2(co, v3, v4);
+	d4 = dist_squared_to_line_segment_v2(co, v4, v1);
+
+	return sqrtf(min_ffff(d1, d2, d3, d4));
+}
+
+static MovieTrackingTrack *find_nearest_track(SpaceClip *sc, ListBase *tracksbase, float co[2], float *distance_r)
 {
 	MovieTrackingTrack *track = NULL, *cur;
 	float mindist = 0.0f;
@@ -210,7 +202,7 @@ static MovieTrackingTrack *find_nearest_track(SpaceClip *sc, ListBase *tracksbas
 				d3 = dist_to_rect(co, marker->pos, marker->search_min, marker->search_max);
 
 			/* choose minimal distance. useful for cases of overlapped markers. */
-			dist = MIN3(d1, d2, d3);
+			dist = min_fff(d1, d2, d3);
 
 			if (track == NULL || dist < mindist) {
 				track = cur;
@@ -221,19 +213,88 @@ static MovieTrackingTrack *find_nearest_track(SpaceClip *sc, ListBase *tracksbas
 		cur = cur->next;
 	}
 
+	*distance_r = mindist;
+
 	return track;
 }
 
+static MovieTrackingPlaneTrack *find_nearest_plane_track(SpaceClip *sc, ListBase *plane_tracks_base,
+                                                         float co[2], float *distance_r)
+{
+	MovieTrackingPlaneTrack *plane_track = NULL, *current_plane_track;
+	float min_distance = 0.0f;
+	int framenr = ED_space_clip_get_clip_frame_number(sc);
+
+	for (current_plane_track = plane_tracks_base->first;
+	     current_plane_track;
+	     current_plane_track = current_plane_track->next)
+	{
+		MovieTrackingPlaneMarker *plane_marker = BKE_tracking_plane_marker_get(current_plane_track, framenr);
+
+		if ((current_plane_track->flag & TRACK_HIDDEN) == 0) {
+			float distance = dist_to_crns_abs(co, plane_marker->corners);
+			if (plane_track == NULL || distance < min_distance) {
+				plane_track = current_plane_track;
+				min_distance = distance;
+			}
+		}
+	}
+
+	*distance_r = min_distance;
+
+	return plane_track;
+}
+
+static void delect_all_tracks(ListBase *tracks_base)
+{
+	MovieTrackingTrack *track;
+	for (track = tracks_base->first;
+	     track;
+	     track = track->next)
+	{
+		BKE_tracking_track_flag_clear(track, TRACK_AREA_ALL, SELECT);
+	}
+}
+
+static void delect_all_plane_tracks(ListBase *plane_tracks_base)
+{
+	MovieTrackingPlaneTrack *plane_track;
+	for (plane_track = plane_tracks_base->first;
+	     plane_track;
+	     plane_track = plane_track->next)
+	{
+		plane_track->flag &= ~SELECT;
+	}
+}
+
 static int mouse_select(bContext *C, float co[2], int extend)
 {
 	SpaceClip *sc = CTX_wm_space_clip(C);
 	MovieClip *clip = ED_space_clip_get_clip(sc);
 	MovieTracking *tracking = &clip->tracking;
 	ListBase *tracksbase = BKE_tracking_get_active_tracks(tracking);
+	ListBase *plane_tracks_base = BKE_tracking_get_active_plane_tracks(tracking);
 	MovieTrackingTrack *act_track = BKE_tracking_track_get_active(tracking);
-	MovieTrackingTrack *track = NULL;   /* selected marker */
+	MovieTrackingTrack *track;
+	MovieTrackingPlaneTrack *plane_track;
+	float distance_to_track, distance_to_plane_track;
 
-	track = find_nearest_track(sc, tracksbase, co);
+	track = find_nearest_track(sc, tracksbase, co, &distance_to_track);
+	plane_track = find_nearest_plane_track(sc, plane_tracks_base, co, &distance_to_plane_track);
+
+	/* Between track and plane we choose closest to the mouse for selection here. */
+	if (track && plane_track) {
+		if (distance_to_track < distance_to_plane_track) {
+			plane_track = NULL;
+		}
+		else {
+			track = NULL;
+		}
+	}
+
+	if (!extend) {
+		delect_all_plane_tracks(plane_tracks_base);
+	}
 
 	if (track) {
 		int area = track_mouse_area(C, co, track);
@@ -242,10 +303,13 @@ static int mouse_select(bContext *C, float co[2], int extend)
 			area = TRACK_AREA_ALL;
 
 		if (extend && TRACK_AREA_SELECTED(track, area)) {
-			if (track == act_track)
+			if (track == act_track) {
 				BKE_tracking_track_deselect(track, area);
-			else
+			}
+			else {
 				clip->tracking.act_track = track;
+				clip->tracking.act_plane_track = NULL;
+			}
 		}
 		else {
 			if (area == TRACK_AREA_POINT)
@@ -253,8 +317,26 @@ static int mouse_select(bContext *C, float co[2], int extend)
 
 			BKE_tracking_track_select(tracksbase, track, area, extend);
 			clip->tracking.act_track = track;
+			clip->tracking.act_plane_track = NULL;
 		}
 	}
+	else if (plane_track) {
+		if (!extend) {
+			delect_all_tracks(tracksbase);
+		}
+
+		if (PLANE_TRACK_VIEW_SELECTED(plane_track)) {
+			if (extend) {
+				plane_track->flag &= ~SELECT;
+			}
+		}
+		else {
+			plane_track->flag |= SELECT;
+		}
+
+		clip->tracking.act_track = NULL;
+		clip->tracking.act_plane_track = plane_track;
+	}
 
 	if (!extend) {
 		sc->xlockof = 0.0f;
@@ -268,6 +350,17 @@ static int mouse_select(bContext *C, float co[2], int extend)
 	return OPERATOR_FINISHED;
 }
 
+static int select_poll(bContext *C)
+{
+	SpaceClip *sc = CTX_wm_space_clip(C);
+
+	if (sc) {
+		return sc->clip && sc->view == SC_VIEW_CLIP;
+	}
+
+	return false;
+}
+
 static int select_exec(bContext *C, wmOperator *op)
 {
 	float co[2];
@@ -279,19 +372,18 @@ static int select_exec(bContext *C, wmOperator *op)
 	return mouse_select(C, co, extend);
 }
 
-static int select_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int select_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	SpaceClip *sc = CTX_wm_space_clip(C);
 	ARegion *ar = CTX_wm_region(C);
 
 	float co[2];
-	int extend = RNA_boolean_get(op->ptr, "extend");
+	const bool extend = RNA_boolean_get(op->ptr, "extend");
 
 	if (!extend) {
 		MovieTrackingTrack *track = tracking_marker_check_slide(C, event, NULL, NULL, NULL);
 
 		if (track) {
-			SpaceClip *sc = CTX_wm_space_clip(C);
 			MovieClip *clip = ED_space_clip_get_clip(sc);
 
 			clip->tracking.act_track = track;
@@ -318,8 +410,7 @@ void CLIP_OT_select(wmOperatorType *ot)
 	/* api callbacks */
 	ot->exec = select_exec;
 	ot->invoke = select_invoke;
-	//ot->poll = ED_space_clip_tracking_poll; // so mask view can Ctrl+RMB markers
-	ot->poll = ED_space_clip_view_clip_poll;
+	ot->poll = select_poll;
 
 	/* flags */
 	ot->flag = OPTYPE_UNDO;
@@ -341,10 +432,13 @@ static int border_select_exec(bContext *C, wmOperator *op)
 	MovieClip *clip = ED_space_clip_get_clip(sc);
 	MovieTracking *tracking = &clip->tracking;
 	MovieTrackingTrack *track;
+	MovieTrackingPlaneTrack *plane_track;
 	ListBase *tracksbase = BKE_tracking_get_active_tracks(tracking);
+	ListBase *plane_tracks_base = BKE_tracking_get_active_plane_tracks(tracking);
 	rcti rect;
 	rctf rectf;
-	int change = FALSE, mode, extend;
+	bool changed = false;
+	int mode, extend;
 	int framenr = ED_space_clip_get_clip_frame_number(sc);
 
 	/* get rectangle from operator */
@@ -373,14 +467,41 @@ static int border_select_exec(bContext *C, wmOperator *op)
 					BKE_tracking_track_flag_clear(track, TRACK_AREA_ALL, SELECT);
 				}
 
-				change = TRUE;
+				changed = true;
 			}
 		}
 
 		track = track->next;
 	}
 
-	if (change) {
+	for (plane_track = plane_tracks_base->first;
+	     plane_track;
+	     plane_track = plane_track->next)
+	{
+		if ((plane_track->flag & PLANE_TRACK_HIDDEN) == 0) {
+			MovieTrackingPlaneMarker *plane_marker =
+				BKE_tracking_plane_marker_get(plane_track, framenr);
+			int i;
+
+			for (i = 0; i < 4; i++) {
+				if (BLI_rctf_isect_pt_v(&rectf, plane_marker->corners[i])) {
+					if (mode == GESTURE_MODAL_SELECT) {
+						plane_track->flag |= SELECT;
+					}
+					else {
+						plane_track->flag &= ~SELECT;
+					}
+				}
+				else if (!extend) {
+					plane_track->flag &= ~SELECT;
+				}
+			}
+
+			changed = true;
+		}
+	}
+
+	if (changed) {
 		BKE_tracking_dopesheet_tag_update(tracking);
 
 		WM_event_add_notifier(C, NC_GEOM | ND_SELECT, NULL);
@@ -408,12 +529,12 @@ void CLIP_OT_select_border(wmOperatorType *ot)
 	ot->flag = OPTYPE_UNDO;
 
 	/* properties */
-	WM_operator_properties_gesture_border(ot, TRUE);
+	WM_operator_properties_gesture_border(ot, true);
 }
 
 /********************** lasso select operator *********************/
 
-static int do_lasso_select_marker(bContext *C, const int mcords[][2], const short moves, short select)
+static int do_lasso_select_marker(bContext *C, const int mcords[][2], const short moves, bool select)
 {
 	SpaceClip *sc = CTX_wm_space_clip(C);
 	ARegion *ar = CTX_wm_region(C);
@@ -421,9 +542,11 @@ static int do_lasso_select_marker(bContext *C, const int mcords[][2], const shor
 	MovieClip *clip = ED_space_clip_get_clip(sc);
 	MovieTracking *tracking = &clip->tracking;
 	MovieTrackingTrack *track;
+	MovieTrackingPlaneTrack *plane_track;
 	ListBase *tracksbase = BKE_tracking_get_active_tracks(tracking);
+	ListBase *plane_tracks_base = BKE_tracking_get_active_plane_tracks(tracking);
 	rcti rect;
-	int change = FALSE;
+	bool changed = false;
 	int framenr = ED_space_clip_get_clip_frame_number(sc);
 
 	/* get rectangle from operator */
@@ -450,20 +573,51 @@ static int do_lasso_select_marker(bContext *C, const int mcords[][2], const shor
 						BKE_tracking_track_flag_clear(track, TRACK_AREA_ALL, SELECT);
 				}
 
-				change = TRUE;
+				changed = true;
 			}
 		}
 
 		track = track->next;
 	}
 
-	if (change) {
+	for (plane_track = plane_tracks_base->first;
+	     plane_track;
+	     plane_track = plane_track->next)
+	{
+		if ((plane_track->flag & PLANE_TRACK_HIDDEN) == 0) {
+			MovieTrackingPlaneMarker *plane_marker =
+				BKE_tracking_plane_marker_get(plane_track, framenr);
+			int i;
+
+			for (i = 0; i < 4; i++) {
+				float screen_co[2];
+
+				/* marker in screen coords */
+				ED_clip_point_stable_pos__reverse(sc, ar, plane_marker->corners[i], screen_co);
+
+				if (BLI_rcti_isect_pt(&rect, screen_co[0], screen_co[1]) &&
+				    BLI_lasso_is_point_inside(mcords, moves, screen_co[0], screen_co[1], V2D_IS_CLIPPED))
+				{
+					if (select) {
+						plane_track->flag |= SELECT;
+					}
+					else {
+						plane_track->flag &= ~SELECT;
+					}
+				}
+			}
+
+			changed = true;
+		}
+	}
+
+	if (changed) {
 		BKE_tracking_dopesheet_tag_update(tracking);
 
 		WM_event_add_notifier(C, NC_GEOM | ND_SELECT, NULL);
 	}
 
-	return change;
+	return changed;
 }
 
 static int clip_lasso_select_exec(bContext *C, wmOperator *op)
@@ -472,12 +626,12 @@ static int clip_lasso_select_exec(bContext *C, wmOperator *op)
 	const int (*mcords)[2] = WM_gesture_lasso_path_to_array(C, op, &mcords_tot);
 
 	if (mcords) {
-		short select;
+		bool select;
 
 		select = !RNA_boolean_get(op->ptr, "deselect");
 		do_lasso_select_marker(C, mcords, mcords_tot, select);
 
-		MEM_freeN(mcords);
+		MEM_freeN((void *)mcords);
 
 		return OPERATOR_FINISHED;
 	}
@@ -509,17 +663,22 @@ void CLIP_OT_select_lasso(wmOperatorType *ot)
 
 /********************** circle select operator *********************/
 
-static int marker_inside_ellipse(MovieTrackingMarker *marker, float offset[2], float ellipse[2])
+static int point_inside_ellipse(float point[2], float offset[2], float ellipse[2])
 {
 	/* normalized ellipse: ell[0] = scaleX, ell[1] = scaleY */
 	float x, y;
 
-	x = (marker->pos[0] - offset[0]) * ellipse[0];
-	y = (marker->pos[1] - offset[1]) * ellipse[1];
+	x = (point[0] - offset[0]) * ellipse[0];
+	y = (point[1] - offset[1]) * ellipse[1];
 
 	return x * x + y * y < 1.0f;
 }
 
+static int marker_inside_ellipse(MovieTrackingMarker *marker, float offset[2], float ellipse[2])
+{
+	return point_inside_ellipse(marker->pos, offset, ellipse);
+}
+
 static int circle_select_exec(bContext *C, wmOperator *op)
 {
 	SpaceClip *sc = CTX_wm_space_clip(C);
@@ -528,8 +687,11 @@ static int circle_select_exec(bContext *C, wmOperator *op)
 	MovieClip *clip = ED_space_clip_get_clip(sc);
 	MovieTracking *tracking = &clip->tracking;
 	MovieTrackingTrack *track;
+	MovieTrackingPlaneTrack *plane_track;
 	ListBase *tracksbase = BKE_tracking_get_active_tracks(tracking);
-	int x, y, radius, width, height, mode, change = FALSE;
+	ListBase *plane_tracks_base = BKE_tracking_get_active_plane_tracks(tracking);
+	int x, y, radius, width, height, mode;
+	bool changed = false;
 	float zoomx, zoomy, offset[2], ellipse[2];
 	int framenr = ED_space_clip_get_clip_frame_number(sc);
 
@@ -561,14 +723,38 @@ static int circle_select_exec(bContext *C, wmOperator *op)
 				else
 					BKE_tracking_track_flag_clear(track, TRACK_AREA_ALL, SELECT);
 
-				change = TRUE;
+				changed = true;
 			}
 		}
 
 		track = track->next;
 	}
 
-	if (change) {
+	for (plane_track = plane_tracks_base->first;
+	     plane_track;
+	     plane_track = plane_track->next)
+	{
+		if ((plane_track->flag & PLANE_TRACK_HIDDEN) == 0) {
+			MovieTrackingPlaneMarker *plane_marker =
+				BKE_tracking_plane_marker_get(plane_track, framenr);
+			int i;
+
+			for (i = 0; i < 4; i++) {
+				if (point_inside_ellipse(plane_marker->corners[i], offset, ellipse)) {
+					if (mode == GESTURE_MODAL_SELECT) {
+						plane_track->flag |= SELECT;
+					}
+					else {
+						plane_track->flag &= ~SELECT;
+					}
+				}
+			}
+
+			changed = true;
+		}
+	}
+
+	if (changed) {
 		BKE_tracking_dopesheet_tag_update(tracking);
 
 		WM_event_add_notifier(C, NC_GEOM | ND_SELECT, NULL);
@@ -598,7 +784,7 @@ void CLIP_OT_select_circle(wmOperatorType *ot)
 	/* properties */
 	RNA_def_int(ot->srna, "x", 0, INT_MIN, INT_MAX, "X", "", INT_MIN, INT_MAX);
 	RNA_def_int(ot->srna, "y", 0, INT_MIN, INT_MAX, "Y", "", INT_MIN, INT_MAX);
-	RNA_def_int(ot->srna, "radius", 0, INT_MIN, INT_MAX, "Radius", "", INT_MIN, INT_MAX);
+	RNA_def_int(ot->srna, "radius", 1, 1, INT_MAX, "Radius", "", 1, INT_MAX);
 	RNA_def_int(ot->srna, "gesture_mode", 0, INT_MIN, INT_MAX, "Gesture Mode", "", INT_MIN, INT_MAX);
 }
 
@@ -610,16 +796,18 @@ static int select_all_exec(bContext *C, wmOperator *op)
 	MovieClip *clip = ED_space_clip_get_clip(sc);
 	MovieTracking *tracking = &clip->tracking;
 	MovieTrackingTrack *track = NULL;   /* selected track */
+	MovieTrackingPlaneTrack *plane_track = NULL;   /* selected plane track */
 	MovieTrackingMarker *marker;
 	ListBase *tracksbase = BKE_tracking_get_active_tracks(tracking);
+	ListBase *plane_tracks_base = BKE_tracking_get_active_plane_tracks(tracking);
 	int action = RNA_enum_get(op->ptr, "action");
 	int framenr = ED_space_clip_get_clip_frame_number(sc);
-	int has_selection = FALSE;
+	bool has_selection = false;
 
 	if (action == SEL_TOGGLE) {
 		action = SEL_SELECT;
-		track = tracksbase->first;
-		while (track) {
+
+		for (track = tracksbase->first; track; track = track->next) {
 			if (TRACK_VIEW_SELECTED(sc, track)) {
 				marker = BKE_tracking_marker_get(track, framenr);
 
@@ -628,13 +816,20 @@ static int select_all_exec(bContext *C, wmOperator *op)
 					break;
 				}
 			}
+		}
 
-			track = track->next;
+		for (plane_track = plane_tracks_base->first;
+		     plane_track;
+		     plane_track = plane_track->next)
+		{
+			if (PLANE_TRACK_VIEW_SELECTED(plane_track)) {
+				action = SEL_DESELECT;
+				break;
+			}
 		}
 	}
 
-	track = tracksbase->first;
-	while (track) {
+	for (track = tracksbase->first; track; track = track->next) {
 		if ((track->flag & TRACK_HIDDEN) == 0) {
 			marker = BKE_tracking_marker_get(track, framenr);
 
@@ -660,9 +855,29 @@ static int select_all_exec(bContext *C, wmOperator *op)
 		}
 
 		if (TRACK_VIEW_SELECTED(sc, track))
-			has_selection = TRUE;
+			has_selection = true;
+	}
 
-		track = track->next;
+	for (plane_track = plane_tracks_base->first;
+	     plane_track;
+	     plane_track = plane_track->next)
+	{
+		if ((plane_track->flag & PLANE_TRACK_HIDDEN) == 0) {
+			switch (action) {
+				case SEL_SELECT:
+					plane_track->flag |= SELECT;
+					break;
+				case SEL_DESELECT:
+					plane_track->flag &= ~SELECT;
+					break;
+				case SEL_INVERT:
+					plane_track->flag ^= SELECT;
+					break;
+			}
+			if (plane_track->flag & SELECT) {
+				has_selection = true;
+			}
+		}
 	}
 
 	if (!has_selection)
@@ -707,7 +922,7 @@ static int select_groped_exec(bContext *C, wmOperator *op)
 
 	track = tracksbase->first;
 	while (track) {
-		int ok = FALSE;
+		bool ok = false;
 
 		marker = BKE_tracking_marker_get(track, framenr);
 
