@@ -34,14 +34,15 @@
 
 #include "DNA_userdef_types.h"
 
+#include "BLI_utildefines.h"
 #include "BLI_listbase.h"
 #include "BLI_string_cursor_utf8.h"
+#include "BLI_string_utf8.h"
 #include "BLI_string.h"
 #include "BLI_dynstr.h"
-#include "BLI_utildefines.h"
+#include "BLI_math.h"
 
 #include "BKE_context.h"
-#include "BKE_text.h" /* only for character utility funcs */
 
 #include "WM_api.h"
 #include "WM_types.h"
@@ -94,7 +95,7 @@ static void console_scrollback_limit(SpaceConsole *sc)
 	
 	if (U.scrollback < 32) U.scrollback = 256;  // XXX - save in user defaults
 	
-	for (tot = BLI_countlist(&sc->scrollback); tot > U.scrollback; tot--)
+	for (tot = BLI_listbase_count(&sc->scrollback); tot > U.scrollback; tot--)
 		console_scrollback_free(sc, sc->scrollback.first);
 }
 
@@ -114,7 +115,7 @@ static ConsoleLine *console_history_find(SpaceConsole *sc, const char *str, Cons
 }
 
 /* return 0 if no change made, clamps the range */
-static int console_line_cursor_set(ConsoleLine *cl, int cursor)
+static bool console_line_cursor_set(ConsoleLine *cl, int cursor)
 {
 	int cursor_new;
 
@@ -123,11 +124,11 @@ static int console_line_cursor_set(ConsoleLine *cl, int cursor)
 	else cursor_new = cursor;
 	
 	if (cursor_new == cl->cursor) {
-		return FALSE;
+		return false;
 	}
 	
 	cl->cursor = cursor_new;
-	return TRUE;
+	return true;
 }
 
 #if 0 // XXX unused 
@@ -135,7 +136,7 @@ static void console_lb_debug__internal(ListBase *lb)
 {
 	ConsoleLine *cl;
 
-	printf("%d: ", BLI_countlist(lb));
+	printf("%d: ", BLI_listbase_count(lb));
 	for (cl = lb->first; cl; cl = cl->next)
 		printf("<%s> ", cl->line);
 	printf("\n");
@@ -156,10 +157,9 @@ static ConsoleLine *console_lb_add__internal(ListBase *lb, ConsoleLine *from)
 	ConsoleLine *ci = MEM_callocN(sizeof(ConsoleLine), "ConsoleLine Add");
 	
 	if (from) {
-		ci->line = BLI_strdup(from->line);
-		ci->len = strlen(ci->line);
-		ci->len_alloc = ci->len;
-		
+		BLI_assert(strlen(from->line) == from->len);
+		ci->line = BLI_strdupn(from->line, from->len);
+		ci->len = ci->len_alloc = from->len;
 		ci->cursor = from->cursor;
 		ci->type = from->type;
 	}
@@ -173,10 +173,8 @@ static ConsoleLine *console_lb_add__internal(ListBase *lb, ConsoleLine *from)
 	return ci;
 }
 
-static ConsoleLine *console_history_add(const bContext *C, ConsoleLine *from)
+static ConsoleLine *console_history_add(SpaceConsole *sc, ConsoleLine *from)
 {
-	SpaceConsole *sc = CTX_wm_space_console(C);
-	
 	return console_lb_add__internal(&sc->history, from);
 }
 
@@ -189,7 +187,7 @@ static ConsoleLine *console_scrollback_add(const bContext *C, ConsoleLine *from)
 }
 #endif
 
-static ConsoleLine *console_lb_add_str__internal(ListBase *lb, char *str, int own)
+static ConsoleLine *console_lb_add_str__internal(ListBase *lb, char *str, bool own)
 {
 	ConsoleLine *ci = MEM_callocN(sizeof(ConsoleLine), "ConsoleLine Add");
 	if (own) ci->line = str;
@@ -200,11 +198,11 @@ static ConsoleLine *console_lb_add_str__internal(ListBase *lb, char *str, int ow
 	BLI_addtail(lb, ci);
 	return ci;
 }
-ConsoleLine *console_history_add_str(SpaceConsole *sc, char *str, int own)
+ConsoleLine *console_history_add_str(SpaceConsole *sc, char *str, bool own)
 {
 	return console_lb_add_str__internal(&sc->history, str, own);
 }
-ConsoleLine *console_scrollback_add_str(SpaceConsole *sc, char *str, int own)
+ConsoleLine *console_scrollback_add_str(SpaceConsole *sc, char *str, bool own)
 {
 	ConsoleLine *ci = console_lb_add_str__internal(&sc->scrollback, str, own);
 	console_select_offset(sc, ci->len + 1);
@@ -216,7 +214,7 @@ ConsoleLine *console_history_verify(const bContext *C)
 	SpaceConsole *sc = CTX_wm_space_console(C);
 	ConsoleLine *ci = sc->history.last;
 	if (ci == NULL)
-		ci = console_history_add(C, NULL);
+		ci = console_history_add(sc, NULL);
 	
 	return ci;
 }
@@ -226,7 +224,12 @@ static void console_line_verify_length(ConsoleLine *ci, int len)
 {
 	/* resize the buffer if needed */
 	if (len >= ci->len_alloc) {
-		int new_len = len * 2; /* new length */
+		/* new length */
+#ifndef NDEBUG
+		int new_len = len + 1;
+#else
+		int new_len = (len + 1) * 2;
+#endif
 		char *new_line = MEM_callocN(new_len, "console line");
 		memcpy(new_line, ci->line, ci->len);
 		MEM_freeN(ci->line);
@@ -277,7 +280,7 @@ static int console_move_exec(bContext *C, wmOperator *op)
 	ConsoleLine *ci = console_history_verify(C);
 	
 	int type = RNA_enum_get(op->ptr, "type");
-	int done = FALSE;
+	bool done = false;
 	int pos;
 	
 	switch (type) {
@@ -285,28 +288,28 @@ static int console_move_exec(bContext *C, wmOperator *op)
 			pos = ci->cursor;
 			BLI_str_cursor_step_utf8(ci->line, ci->len,
 			                         &pos, STRCUR_DIR_PREV,
-			                         STRCUR_JUMP_ALL);
+			                         STRCUR_JUMP_ALL, true);
 			done = console_line_cursor_set(ci, pos);
 			break;
 		case LINE_END:
 			pos = ci->cursor;
 			BLI_str_cursor_step_utf8(ci->line, ci->len,
 			                         &pos, STRCUR_DIR_NEXT,
-			                         STRCUR_JUMP_ALL);
+			                         STRCUR_JUMP_ALL, true);
 			done = console_line_cursor_set(ci, pos);
 			break;
 		case PREV_CHAR:
 			pos = ci->cursor;
 			BLI_str_cursor_step_utf8(ci->line, ci->len,
 			                         &pos, STRCUR_DIR_PREV,
-			                         STRCUR_JUMP_NONE);
+			                         STRCUR_JUMP_NONE, true);
 			done = console_line_cursor_set(ci, pos);
 			break;
 		case NEXT_CHAR:
 			pos = ci->cursor;
 			BLI_str_cursor_step_utf8(ci->line, ci->len,
 			                         &pos, STRCUR_DIR_NEXT,
-			                         STRCUR_JUMP_NONE);
+			                         STRCUR_JUMP_NONE, true);
 			done = console_line_cursor_set(ci, pos);
 			break;
 
@@ -316,14 +319,14 @@ static int console_move_exec(bContext *C, wmOperator *op)
 			pos = ci->cursor;
 			BLI_str_cursor_step_utf8(ci->line, ci->len,
 			                         &pos, STRCUR_DIR_PREV,
-			                         STRCUR_JUMP_DELIM);
+			                         STRCUR_JUMP_DELIM, true);
 			done = console_line_cursor_set(ci, pos);
 			break;
 		case NEXT_WORD:
 			pos = ci->cursor;
 			BLI_str_cursor_step_utf8(ci->line, ci->len,
 			                         &pos, STRCUR_DIR_NEXT,
-			                         STRCUR_JUMP_DELIM);
+			                         STRCUR_JUMP_DELIM, true);
 			done = console_line_cursor_set(ci, pos);
 			break;
 	}
@@ -391,19 +394,30 @@ static int console_insert_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-static int console_insert_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int console_insert_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	// if (!RNA_struct_property_is_set(op->ptr, "text")) { /* always set from keymap XXX */
 	if (!RNA_string_length(op->ptr, "text")) {
-		/* if alt/ctrl/super are pressed pass through */
-		if (event->ctrl || event->oskey) {
+		/* if alt/ctrl/super are pressed pass through except for utf8 character event
+		 * (when input method are used for utf8 inputs, the user may assign key event
+		 * including alt/ctrl/super like ctrl+m to commit utf8 string.  in such case,
+		 * the modifiers in the utf8 character event make no sense.) */
+		if ((event->ctrl || event->oskey) && !event->utf8_buf[0]) {
 			return OPERATOR_PASS_THROUGH;
 		}
 		else {
-			char str[2];
-			str[0] = event->ascii;
-			str[1] = '\0';
-
+			char str[BLI_UTF8_MAX + 1];
+			size_t len;
+			
+			if (event->utf8_buf[0]) {
+				len = BLI_str_utf8_size_safe(event->utf8_buf);
+				memcpy(str, event->utf8_buf, len);
+			}
+			else {
+				/* in theory, ghost can set value to extended ascii here */
+				len = BLI_str_utf8_from_unicode(event->ascii, str);
+			}
+			str[len] = '\0';
 			RNA_string_set(op->ptr, "text", str);
 		}
 	}
@@ -425,7 +439,7 @@ void CONSOLE_OT_insert(wmOperatorType *ot)
 	ot->poll = ED_operator_console_active;
 
 	/* properties */
-	prop = RNA_def_string(ot->srna, "text", "", 0, "Text", "Text to insert at the cursor position");
+	prop = RNA_def_string(ot->srna, "text", NULL, 0, "Text", "Text to insert at the cursor position");
 	RNA_def_property_flag(prop, PROP_SKIP_SAVE);
 }
 
@@ -446,10 +460,12 @@ static int console_indent_exec(bContext *C, wmOperator *UNUSED(op))
 
 	console_line_verify_length(ci, ci->len + len);
 
-	memmove(ci->line + len, ci->line, ci->len);
+	memmove(ci->line + len, ci->line, ci->len + 1);
 	memset(ci->line, ' ', len);
 	ci->len += len;
+	BLI_assert(ci->len >= 0);
 	console_line_cursor_set(ci, ci->cursor + len);
+	console_select_offset(sc, len);
 
 	console_textview_update_rect(sc, ar);
 	ED_area_tag_redraw(CTX_wm_area(C));
@@ -495,9 +511,10 @@ static int console_unindent_exec(bContext *C, wmOperator *UNUSED(op))
 
 	memmove(ci->line, ci->line + len, (ci->len - len) + 1);
 	ci->len -= len;
-	console_line_cursor_set(ci, ci->cursor - len);
+	BLI_assert(ci->len >= 0);
 
-	//console_select_offset(sc, -4);
+	console_line_cursor_set(ci, ci->cursor - len);
+	console_select_offset(sc, -len);
 
 	console_textview_update_rect(sc, ar);
 	ED_area_tag_redraw(CTX_wm_area(C));
@@ -536,7 +553,7 @@ static int console_delete_exec(bContext *C, wmOperator *op)
 	int stride;
 
 	const short type = RNA_enum_get(op->ptr, "type");
-	int done = FALSE;
+	bool done = false;
 	
 	if (ci->len == 0) {
 		return OPERATOR_CANCELLED;
@@ -549,12 +566,13 @@ static int console_delete_exec(bContext *C, wmOperator *op)
 				pos = ci->cursor;
 				BLI_str_cursor_step_utf8(ci->line, ci->len,
 				                         &pos, STRCUR_DIR_NEXT,
-				                         (type == DEL_NEXT_CHAR) ? STRCUR_JUMP_NONE : STRCUR_JUMP_DELIM);
+				                         (type == DEL_NEXT_CHAR) ? STRCUR_JUMP_NONE : STRCUR_JUMP_DELIM, true);
 				stride = pos - ci->cursor;
 				if (stride) {
-					memmove(ci->line + ci->cursor, ci->line + ci->cursor + stride, (ci->len - ci->cursor) + 1);
+					memmove(ci->line + ci->cursor, ci->line + ci->cursor + stride, (ci->len - (ci->cursor + stride)) + 1);
 					ci->len -= stride;
-					done = TRUE;
+					BLI_assert(ci->len >= 0);
+					done = true;
 				}
 			}
 			break;
@@ -564,13 +582,14 @@ static int console_delete_exec(bContext *C, wmOperator *op)
 				pos = ci->cursor;
 				BLI_str_cursor_step_utf8(ci->line, ci->len,
 				                         &pos, STRCUR_DIR_PREV,
-				                         (type == DEL_PREV_CHAR) ? STRCUR_JUMP_NONE : STRCUR_JUMP_DELIM);
+				                         (type == DEL_PREV_CHAR) ? STRCUR_JUMP_NONE : STRCUR_JUMP_DELIM, true);
 				stride = ci->cursor - pos;
 				if (stride) {
 					ci->cursor -= stride; /* same as above */
-					memmove(ci->line + ci->cursor, ci->line + ci->cursor + stride, (ci->len - ci->cursor) + 1);
+					memmove(ci->line + ci->cursor, ci->line + ci->cursor + stride, (ci->len - (ci->cursor + stride)) + 1);
 					ci->len -= stride;
-					done = TRUE;
+					BLI_assert(ci->len >= 0);
+					done = true;
 				}
 			}
 			break;
@@ -580,7 +599,7 @@ static int console_delete_exec(bContext *C, wmOperator *op)
 		return OPERATOR_CANCELLED;
 	}
 	else {
-		console_select_offset(sc, -1);
+		console_select_offset(sc, -stride);
 	}
 
 	console_textview_update_rect(sc, ar);
@@ -617,8 +636,9 @@ static int console_clear_line_exec(bContext *C, wmOperator *UNUSED(op))
 		return OPERATOR_CANCELLED;
 	}
 
-	console_history_add(C, ci);
-	console_history_add(C, NULL);
+	console_history_add(sc, ci);
+	console_history_add(sc, NULL);
+	console_select_offset(sc, -ci->len);
 
 	console_textview_update_rect(sc, ar);
 
@@ -647,10 +667,10 @@ static int console_clear_exec(bContext *C, wmOperator *op)
 	SpaceConsole *sc = CTX_wm_space_console(C);
 	ARegion *ar = CTX_wm_region(C);
 	
-	short scrollback = RNA_boolean_get(op->ptr, "scrollback");
-	short history = RNA_boolean_get(op->ptr, "history");
+	const bool scrollback = RNA_boolean_get(op->ptr, "scrollback");
+	const bool history = RNA_boolean_get(op->ptr, "history");
 	
-	/*ConsoleLine *ci= */ console_history_verify(C);
+	/*ConsoleLine *ci = */ console_history_verify(C);
 	
 	if (scrollback) { /* last item in mistory */
 		while (sc->scrollback.first)
@@ -660,6 +680,7 @@ static int console_clear_exec(bContext *C, wmOperator *op)
 	if (history) {
 		while (sc->history.first)
 			console_history_free(sc, sc->history.first);
+		console_history_verify(C);
 	}
 
 	console_textview_update_rect(sc, ar);
@@ -693,7 +714,7 @@ static int console_history_cycle_exec(bContext *C, wmOperator *op)
 	ARegion *ar = CTX_wm_region(C);
 
 	ConsoleLine *ci = console_history_verify(C); /* TODO - stupid, just prevents crashes when no command line */
-	short reverse = RNA_boolean_get(op->ptr, "reverse"); /* assumes down, reverse is up */
+	const bool reverse = RNA_boolean_get(op->ptr, "reverse"); /* assumes down, reverse is up */
 	int prev_len = ci->len;
 
 	/* keep a copy of the line above so when history is cycled
@@ -721,7 +742,7 @@ static int console_history_cycle_exec(bContext *C, wmOperator *op)
 		while ((cl = console_history_find(sc, ci->line, ci)))
 			console_history_free(sc, cl);
 
-		console_history_add(C, (ConsoleLine *)sc->history.last);
+		console_history_add(sc, (ConsoleLine *)sc->history.last);
 	}
 	
 	ci = sc->history.last;
@@ -761,7 +782,7 @@ static int console_history_append_exec(bContext *C, wmOperator *op)
 	ConsoleLine *ci = console_history_verify(C);
 	char *str = RNA_string_get_alloc(op->ptr, "text", NULL, 0); /* own this text in the new line, don't free */
 	int cursor = RNA_int_get(op->ptr, "current_character");
-	short rem_dupes = RNA_boolean_get(op->ptr, "remove_duplicates");
+	const bool rem_dupes = RNA_boolean_get(op->ptr, "remove_duplicates");
 	int prev_len = ci->len;
 
 	if (rem_dupes) {
@@ -803,7 +824,7 @@ void CONSOLE_OT_history_append(wmOperatorType *ot)
 	ot->poll = ED_operator_console_active;
 	
 	/* properties */
-	RNA_def_string(ot->srna, "text", "", 0, "Text", "Text to insert at the cursor position");	
+	RNA_def_string(ot->srna, "text", NULL, 0, "Text", "Text to insert at the cursor position");
 	RNA_def_int(ot->srna, "current_character", 0, 0, INT_MAX, "Cursor", "The index of the cursor", 0, 10000);
 	RNA_def_boolean(ot->srna, "remove_duplicates", 0, "Remove Duplicates", "Remove duplicate items in the history");
 }
@@ -858,7 +879,7 @@ void CONSOLE_OT_scrollback_append(wmOperatorType *ot)
 	ot->poll = ED_operator_console_active;
 	
 	/* properties */
-	RNA_def_string(ot->srna, "text", "", 0, "Text", "Text to insert at the cursor position");
+	RNA_def_string(ot->srna, "text", NULL, 0, "Text", "Text to insert at the cursor position");
 	RNA_def_enum(ot->srna, "type", console_line_type_items, CONSOLE_LINE_OUTPUT, "Type", "Console output type");
 }
 
@@ -867,7 +888,7 @@ static int console_copy_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	SpaceConsole *sc = CTX_wm_space_console(C);
 
-	DynStr *buf_dyn = BLI_dynstr_new();
+	DynStr *buf_dyn;
 	char *buf_str;
 	
 	ConsoleLine *cl;
@@ -876,14 +897,6 @@ static int console_copy_exec(bContext *C, wmOperator *UNUSED(op))
 
 	ConsoleLine cl_dummy = {NULL};
 
-#if 0
-	/* copy whole file */
-	for (cl = sc->scrollback.first; cl; cl = cl->next) {
-		BLI_dynstr_append(buf_dyn, cl->line);
-		BLI_dynstr_append(buf_dyn, "\n");
-	}
-#endif
-
 	if (sc->sel_start == sc->sel_end)
 		return OPERATOR_CANCELLED;
 
@@ -898,14 +911,15 @@ static int console_copy_exec(bContext *C, wmOperator *UNUSED(op))
 		return OPERATOR_CANCELLED;
 	}
 
+	buf_dyn = BLI_dynstr_new();
 	offset -= 1;
 	sel[0] = offset - sc->sel_end;
 	sel[1] = offset - sc->sel_start;
 
 	for (cl = sc->scrollback.first; cl; cl = cl->next) {
 		if (sel[0] <= cl->len && sel[1] >= 0) {
-			int sta = MAX2(sel[0], 0);
-			int end = MIN2(sel[1], cl->len);
+			int sta = max_ii(sel[0], 0);
+			int end = min_ii(sel[1], cl->len);
 
 			if (BLI_dynstr_get_len(buf_dyn))
 				BLI_dynstr_append(buf_dyn, "\n");
@@ -948,8 +962,9 @@ static int console_paste_exec(bContext *C, wmOperator *UNUSED(op))
 	SpaceConsole *sc = CTX_wm_space_console(C);
 	ARegion *ar = CTX_wm_region(C);
 	ConsoleLine *ci = console_history_verify(C);
+	int buf_len;
 
-	char *buf_str = WM_clipboard_text_get(0);
+	char *buf_str = WM_clipboard_text_get(false, &buf_len);
 	char *buf_step, *buf_next;
 
 	if (buf_str == NULL)
@@ -1026,7 +1041,7 @@ static void console_cursor_set_to_pos(SpaceConsole *sc, ARegion *ar, SetConsoleC
 	}
 }
 
-static void console_modal_select_apply(bContext *C, wmOperator *op, wmEvent *event)
+static void console_modal_select_apply(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	SpaceConsole *sc = CTX_wm_space_console(C);
 	ARegion *ar = CTX_wm_region(C);
@@ -1040,7 +1055,7 @@ static void console_modal_select_apply(bContext *C, wmOperator *op, wmEvent *eve
 	sel_prev[0] = sc->sel_start;
 	sel_prev[1] = sc->sel_end;
 	
-	console_cursor_set_to_pos(sc, ar, scu, mval, TRUE);
+	console_cursor_set_to_pos(sc, ar, scu, mval, true);
 
 	/* only redraw if the selection changed */
 	if (sel_prev[0] != sc->sel_start || sel_prev[1] != sc->sel_end) {
@@ -1050,7 +1065,7 @@ static void console_modal_select_apply(bContext *C, wmOperator *op, wmEvent *eve
 
 static void console_cursor_set_exit(bContext *UNUSED(C), wmOperator *op)
 {
-//	SpaceConsole *sc= CTX_wm_space_console(C);
+//	SpaceConsole *sc = CTX_wm_space_console(C);
 	SetConsoleCursor *scu = op->customdata;
 
 #if 0
@@ -1064,10 +1079,10 @@ static void console_cursor_set_exit(bContext *UNUSED(C), wmOperator *op)
 	MEM_freeN(scu);
 }
 
-static int console_modal_select_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int console_modal_select_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	SpaceConsole *sc = CTX_wm_space_console(C);
-//	ARegion *ar= CTX_wm_region(C);
+//	ARegion *ar = CTX_wm_region(C);
 	SetConsoleCursor *scu;
 
 	op->customdata = MEM_callocN(sizeof(SetConsoleCursor), "SetConsoleCursor");
@@ -1085,7 +1100,7 @@ static int console_modal_select_invoke(bContext *C, wmOperator *op, wmEvent *eve
 	return OPERATOR_RUNNING_MODAL;
 }
 
-static int console_modal_select(bContext *C, wmOperator *op, wmEvent *event)
+static int console_modal_select(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	switch (event->type) {
 		case LEFTMOUSE:
@@ -1101,10 +1116,9 @@ static int console_modal_select(bContext *C, wmOperator *op, wmEvent *event)
 	return OPERATOR_RUNNING_MODAL;
 }
 
-static int console_modal_select_cancel(bContext *C, wmOperator *op)
+static void console_modal_select_cancel(bContext *C, wmOperator *op)
 {
 	console_cursor_set_exit(C, op);
-	return OPERATOR_FINISHED;
 }
 
 void CONSOLE_OT_select_set(wmOperatorType *ot)
