@@ -34,7 +34,7 @@
 
 #include "BLI_blenlib.h"
 #include "BLI_utildefines.h"
-#include "BLI_dynstr.h"
+#include "BLI_fileops_types.h"
 
 #ifdef WIN32
 #  include "BLI_winstuff.h"
@@ -47,14 +47,12 @@
 #include "BKE_global.h"
 #include "BKE_main.h"
 
-#include "BLF_api.h"
 #include "BLF_translation.h"
 
 #include "IMB_imbuf_types.h"
- 
-#include "MEM_guardedalloc.h"
 
 #include "DNA_userdef_types.h"
+#include "DNA_windowmanager_types.h"
 
 #include "RNA_access.h"
 
@@ -68,30 +66,10 @@
 
 #include "WM_types.h"
 
-#include "fsmenu.h"
 #include "filelist.h"
 
 #include "file_intern.h"    // own include
 
-/* button events */
-enum {
-	B_FS_DIRNAME,
-	B_FS_FILENAME
-} /*eFile_ButEvents*/;
-
-
-static void do_file_buttons(bContext *C, void *UNUSED(arg), int event)
-{
-	switch (event) {
-		case B_FS_FILENAME:
-			file_filename_exec(C, NULL);
-			break;
-		case B_FS_DIRNAME:
-			file_directory_exec(C, NULL);
-			break;
-	}
-}
-
 /* Note: This function uses pixelspace (0, 0, winx, winy), not view2d. 
  * The controls are laid out as follows:
  *
@@ -136,8 +114,7 @@ void file_draw_buttons(const bContext *C, ARegion *ar)
 	
 	/* Initialize UI block. */
 	BLI_snprintf(uiblockstr, sizeof(uiblockstr), "win %p", (void *)ar);
-	block = uiBeginBlock(C, ar, uiblockstr, UI_EMBOSS);
-	uiBlockSetHandleFunc(block, do_file_buttons, NULL);
+	block = UI_block_begin(C, ar, uiblockstr, UI_EMBOSS);
 
 	/* exception to make space for collapsed region icon */
 	for (artmp = CTX_wm_area(C)->regionbase.first; artmp; artmp = artmp->next) {
@@ -149,12 +126,9 @@ void file_draw_buttons(const bContext *C, ARegion *ar)
 	}
 	
 	/* Is there enough space for the execute / cancel buttons? */
-	loadbutton = UI_GetStringWidth(sfile->params->title) + btn_margin;
-	if (loadbutton < btn_minw) {
-		loadbutton = MAX2(btn_minw, 
-		                  btn_margin + UI_GetStringWidth(params->title));
-	}
-	
+	loadbutton = UI_fontstyle_string_width(params->title) + btn_margin;
+	CLAMP_MIN(loadbutton, btn_minw);
+
 	if (available_w <= loadbutton + separator + input_minw || params->title[0] == 0) {
 		loadbutton = 0;
 	}
@@ -176,70 +150,80 @@ void file_draw_buttons(const bContext *C, ARegion *ar)
 	if (available_w > 0) {
 		int overwrite_alert = file_draw_check_exists(sfile);
 		/* callbacks for operator check functions */
-		uiBlockSetFunc(block, file_draw_check_cb, NULL, NULL);
+		UI_block_func_set(block, file_draw_check_cb, NULL, NULL);
 
-		but = uiDefButTextO(block, TEX, "FILE_OT_directory", 0, "",
-		                    min_x, line1_y, line1_w - chan_offs, btn_h,
-		                    params->dir, 0.0, (float)FILE_MAX, 0, 0,
-		                    TIP_("File path"));
-		uiButSetCompleteFunc(but, autocomplete_directory, NULL);
-		uiButSetFlag(but, UI_BUT_NO_UTF8);
+		but = uiDefBut(block, UI_BTYPE_TEXT, -1, "",
+		               min_x, line1_y, line1_w - chan_offs, btn_h,
+		               params->dir, 0.0, (float)FILE_MAX, 0, 0,
+		               TIP_("File path"));
+		UI_but_func_complete_set(but, autocomplete_directory, NULL);
+		UI_but_flag_enable(but, UI_BUT_NO_UTF8);
+		UI_but_flag_disable(but, UI_BUT_UNDO);
+		UI_but_funcN_set(but, file_directory_enter_handle, NULL, but);
+
+		/* TODO, directory editing is non-functional while a library is loaded
+		 * until this is properly supported just disable it. */
+		if (sfile->files && filelist_lib(sfile->files))
+			UI_but_flag_enable(but, UI_BUT_DISABLED);
 
 		if ((params->flag & FILE_DIRSEL_ONLY) == 0) {
-			but = uiDefBut(block, TEX, B_FS_FILENAME, "",
+			but = uiDefBut(block, UI_BTYPE_TEXT, -1, "",
 			               min_x, line2_y, line2_w - chan_offs, btn_h,
 			               params->file, 0.0, (float)FILE_MAXFILE, 0, 0,
 			               TIP_(overwrite_alert ? N_("File name, overwrite existing") : N_("File name")));
-			uiButSetCompleteFunc(but, autocomplete_file, NULL);
-			uiButSetFlag(but, UI_BUT_NO_UTF8);
-			uiButClearFlag(but, UI_BUT_UNDO); /* operator button above does this automatic */
+			UI_but_func_complete_set(but, autocomplete_file, NULL);
+			UI_but_flag_enable(but, UI_BUT_NO_UTF8);
+			UI_but_flag_disable(but, UI_BUT_UNDO);
+			/* silly workaround calling NFunc to ensure this does not get called
+			 * immediate ui_apply_but_func but only after button deactivates */
+			UI_but_funcN_set(but, file_filename_enter_handle, NULL, but);
 
 			/* check if this overrides a file and if the operator option is used */
 			if (overwrite_alert) {
-				uiButSetFlag(but, UI_BUT_REDALERT);
+				UI_but_flag_enable(but, UI_BUT_REDALERT);
 			}
 		}
 		
 		/* clear func */
-		uiBlockSetFunc(block, NULL, NULL, NULL);
+		UI_block_func_set(block, NULL, NULL, NULL);
 	}
 	
 	/* Filename number increment / decrement buttons. */
 	if (fnumbuttons && (params->flag & FILE_DIRSEL_ONLY) == 0) {
-		uiBlockBeginAlign(block);
-		but = uiDefIconButO(block, BUT, "FILE_OT_filenum", 0, ICON_ZOOMOUT,
+		UI_block_align_begin(block);
+		but = uiDefIconButO(block, UI_BTYPE_BUT, "FILE_OT_filenum", 0, ICON_ZOOMOUT,
 		                    min_x + line2_w + separator - chan_offs, line2_y,
 		                    btn_fn_w, btn_h,
 		                    TIP_("Decrement the filename number"));
-		RNA_int_set(uiButGetOperatorPtrRNA(but), "increment", -1);
+		RNA_int_set(UI_but_operator_ptr_get(but), "increment", -1);
 
-		but = uiDefIconButO(block, BUT, "FILE_OT_filenum", 0, ICON_ZOOMIN,
+		but = uiDefIconButO(block, UI_BTYPE_BUT, "FILE_OT_filenum", 0, ICON_ZOOMIN,
 		                    min_x + line2_w + separator + btn_fn_w - chan_offs, line2_y,
 		                    btn_fn_w, btn_h,
 		                    TIP_("Increment the filename number"));
-		RNA_int_set(uiButGetOperatorPtrRNA(but), "increment", 1);
-		uiBlockEndAlign(block);
+		RNA_int_set(UI_but_operator_ptr_get(but), "increment", 1);
+		UI_block_align_end(block);
 	}
 	
 	/* Execute / cancel buttons. */
 	if (loadbutton) {
 		/* params->title is already translated! */
-		uiDefButO(block, BUT, "FILE_OT_execute", WM_OP_EXEC_REGION_WIN, params->title,
+		uiDefButO(block, UI_BTYPE_BUT, "FILE_OT_execute", WM_OP_EXEC_REGION_WIN, params->title,
 		          max_x - loadbutton, line1_y, loadbutton, btn_h, "");
-		uiDefButO(block, BUT, "FILE_OT_cancel", WM_OP_EXEC_REGION_WIN, IFACE_("Cancel"),
+		uiDefButO(block, UI_BTYPE_BUT, "FILE_OT_cancel", WM_OP_EXEC_REGION_WIN, IFACE_("Cancel"),
 		          max_x - loadbutton, line2_y, loadbutton, btn_h, "");
 	}
 	
-	uiEndBlock(C, block);
-	uiDrawBlock(C, block);
+	UI_block_end(C, block);
+	UI_block_draw(C, block);
 }
 
 
 static void draw_tile(int sx, int sy, int width, int height, int colorid, int shade)
 {
 	UI_ThemeColorShade(colorid, shade);
-	uiSetRoundBox(UI_CNR_ALL);
-	uiRoundBox((float)sx, (float)(sy - height), (float)(sx + width), (float)sy, 5.0f);
+	UI_draw_roundbox_corner_set(UI_CNR_ALL);
+	UI_draw_roundbox((float)sx, (float)(sy - height), (float)(sx + width), (float)sy, 5.0f);
 }
 
 
@@ -249,6 +233,9 @@ static int get_file_icon(struct direntry *file)
 		if (strcmp(file->relname, "..") == 0) {
 			return ICON_FILE_PARENT;
 		}
+		if (file->flags & APPLICATIONBUNDLE) {
+			return ICON_UGLYPACKAGE;
+		}
 		if (file->flags & BLENDERFILE) {
 			return ICON_FILE_BLEND;
 		}
@@ -257,44 +244,48 @@ static int get_file_icon(struct direntry *file)
 	else if (file->flags & BLENDERFILE)
 		return ICON_FILE_BLEND;
 	else if (file->flags & BLENDERFILE_BACKUP)
-		return ICON_FILE_BLEND;
+		return ICON_FILE_BACKUP;
 	else if (file->flags & IMAGEFILE)
 		return ICON_FILE_IMAGE;
 	else if (file->flags & MOVIEFILE)
 		return ICON_FILE_MOVIE;
 	else if (file->flags & PYSCRIPTFILE)
 		return ICON_FILE_SCRIPT;
-	else if (file->flags & SOUNDFILE) 
+	else if (file->flags & SOUNDFILE)
 		return ICON_FILE_SOUND;
-	else if (file->flags & FTFONTFILE) 
+	else if (file->flags & FTFONTFILE)
 		return ICON_FILE_FONT;
-	else if (file->flags & BTXFILE) 
+	else if (file->flags & BTXFILE)
 		return ICON_FILE_BLANK;
-	else if (file->flags & COLLADAFILE) 
+	else if (file->flags & COLLADAFILE)
 		return ICON_FILE_BLANK;
+	else if (file->flags & TEXTFILE)
+		return ICON_FILE_TEXT;
 	else
 		return ICON_FILE_BLANK;
 }
 
-static void file_draw_icon(uiBlock *block, char *path, int sx, int sy, int icon, int width, int height)
+static void file_draw_icon(uiBlock *block, char *path, int sx, int sy, int icon, int width, int height, bool drag)
 {
 	uiBut *but;
 	int x, y;
-	/*float alpha=1.0f;*/
+	// float alpha = 1.0f;
 	
 	x = sx;
 	y = sy - height;
 	
 	/*if (icon == ICON_FILE_BLANK) alpha = 0.375f;*/
 
-	but = uiDefIconBut(block, LABEL, 0, icon, x, y, width, height, NULL, 0.0f, 0.0f, 0.0f, 0.0f, "");
-	uiButSetDragPath(but, path);
+	but = uiDefIconBut(block, UI_BTYPE_LABEL, 0, icon, x, y, width, height, NULL, 0.0f, 0.0f, 0.0f, 0.0f, "");
+
+	if (drag)
+		UI_but_drag_set_path(but, path);
 }
 
 
 static void file_draw_string(int sx, int sy, const char *string, float width, int height, short align)
 {
-	uiStyle *style = UI_GetStyle();
+	uiStyle *style = UI_style_get();
 	uiFontStyle fs = style->widgetlabel;
 	rcti rect;
 	char fname[FILE_MAXFILE];
@@ -304,13 +295,13 @@ static void file_draw_string(int sx, int sy, const char *string, float width, in
 	BLI_strncpy(fname, string, FILE_MAXFILE);
 	file_shorten_string(fname, width + 1.0f, 0);
 
-	/* no text clipping needed, uiStyleFontDraw does it but is a bit too strict (for buttons it works) */
+	/* no text clipping needed, UI_fontstyle_draw does it but is a bit too strict (for buttons it works) */
 	rect.xmin = sx;
 	rect.xmax = (int)(sx + ceil(width + 4.0f));
 	rect.ymin = sy - height;
 	rect.ymax = sy;
 	
-	uiStyleFontDraw(&fs, &rect, fname);
+	UI_fontstyle_draw(&fs, &rect, fname);
 }
 
 void file_calc_previews(const bContext *C, ARegion *ar)
@@ -319,11 +310,10 @@ void file_calc_previews(const bContext *C, ARegion *ar)
 	View2D *v2d = &ar->v2d;
 	
 	ED_fileselect_init_layout(sfile, ar);
-	/* +SCROLL_HEIGHT is bad hack to work around issue in UI_view2d_totRect_set */
-	UI_view2d_totRect_set(v2d, sfile->layout->width, sfile->layout->height + V2D_SCROLL_HEIGHT);
+	UI_view2d_totRect_set(v2d, sfile->layout->width, sfile->layout->height);
 }
 
-static void file_draw_preview(uiBlock *block, struct direntry *file, int sx, int sy, ImBuf *imb, FileLayout *layout, short dropshadow)
+static void file_draw_preview(uiBlock *block, struct direntry *file, int sx, int sy, ImBuf *imb, FileLayout *layout, bool dropshadow, bool drag)
 {
 	if (imb) {
 		uiBut *but;
@@ -334,7 +324,9 @@ static void file_draw_preview(uiBlock *block, struct direntry *file, int sx, int
 		float scale;
 		int ex, ey;
 		
-		if ( (imb->x > layout->prv_w) || (imb->y > layout->prv_h) ) {
+		if ((imb->x * UI_DPI_FAC > layout->prv_w) ||
+		    (imb->y * UI_DPI_FAC > layout->prv_h))
+		{
 			if (imb->x > imb->y) {
 				scaledx = (float)layout->prv_w;
 				scaledy =  ( (float)imb->y / (float)imb->x) * layout->prv_w;
@@ -347,10 +339,11 @@ static void file_draw_preview(uiBlock *block, struct direntry *file, int sx, int
 			}
 		}
 		else {
-			scaledx = (float)imb->x;
-			scaledy = (float)imb->y;
-			scale = 1.0;
+			scaledx = (float)imb->x * UI_DPI_FAC;
+			scaledy = (float)imb->y * UI_DPI_FAC;
+			scale = UI_DPI_FAC;
 		}
+
 		ex = (int)scaledx;
 		ey = (int)scaledy;
 		fx = ((float)layout->prv_w - (float)ex) / 2.0f;
@@ -364,13 +357,13 @@ static void file_draw_preview(uiBlock *block, struct direntry *file, int sx, int
 		
 		/* shadow */
 		if (dropshadow)
-			uiDrawBoxShadow(220, (float)xco, (float)yco, (float)(xco + ex), (float)(yco + ey));
+			UI_draw_box_shadow(220, (float)xco, (float)yco, (float)(xco + ex), (float)(yco + ey));
 
 		glEnable(GL_BLEND);
 		
 		/* the image */
 		glColor4f(1.0, 1.0, 1.0, 1.0);
-		glaDrawPixelsTexScaled((float)xco, (float)yco, imb->x, imb->y, GL_UNSIGNED_BYTE, imb->rect, scale, scale);
+		glaDrawPixelsTexScaled((float)xco, (float)yco, imb->x, imb->y, GL_RGBA, GL_UNSIGNED_BYTE, GL_NEAREST, imb->rect, scale, scale);
 		
 		/* border */
 		if (dropshadow) {
@@ -379,11 +372,12 @@ static void file_draw_preview(uiBlock *block, struct direntry *file, int sx, int
 		}
 		
 		/* dragregion */
-		but = uiDefBut(block, LABEL, 0, "", xco, yco, ex, ey, NULL, 0.0, 0.0, 0, 0, "");
-		uiButSetDragImage(but, file->path, get_file_icon(file), imb, scale);
+		if (drag) {
+			but = uiDefBut(block, UI_BTYPE_LABEL, 0, "", xco, yco, ex, ey, NULL, 0.0, 0.0, 0, 0, "");
+			UI_but_drag_set_image(but, file->path, get_file_icon(file), imb, scale);
+		}
 		
 		glDisable(GL_BLEND);
-		imb = NULL;
 	}
 }
 
@@ -392,6 +386,7 @@ static void renamebutton_cb(bContext *C, void *UNUSED(arg1), char *oldname)
 	char newname[FILE_MAX + 12];
 	char orgname[FILE_MAX + 12];
 	char filename[FILE_MAX + 12];
+	wmWindowManager *wm = CTX_wm_manager(C);
 	SpaceFile *sfile = (SpaceFile *)CTX_wm_space_data(C);
 	ARegion *ar = CTX_wm_region(C);
 
@@ -403,7 +398,7 @@ static void renamebutton_cb(bContext *C, void *UNUSED(arg1), char *oldname)
 		if (!BLI_exists(newname)) {
 			BLI_rename(orgname, newname);
 			/* to make sure we show what is on disk */
-			ED_fileselect_clear(C, sfile);
+			ED_fileselect_clear(wm, sfile);
 		}
 
 		ED_region_tag_redraw(ar);
@@ -416,11 +411,12 @@ static void draw_background(FileLayout *layout, View2D *v2d)
 	int i;
 	int sy;
 
+	UI_ThemeColorShade(TH_BACK, -7);
+
 	/* alternating flat shade background */
 	for (i = 0; (i <= layout->rows); i += 2) {
 		sy = (int)v2d->cur.ymax - i * (layout->tile_h + 2 * layout->tile_border_y) - layout->tile_border_y;
 
-		UI_ThemeColorShade(TH_BACK, -7);
 		glRectf(v2d->cur.xmin, (float)sy, v2d->cur.xmax, (float)(sy + layout->tile_h + 2 * layout->tile_border_y));
 		
 	}
@@ -428,18 +424,36 @@ static void draw_background(FileLayout *layout, View2D *v2d)
 
 static void draw_dividers(FileLayout *layout, View2D *v2d)
 {
+	const int step = (layout->tile_w + 2 * layout->tile_border_x);
+	int v1[2], v2[2];
 	int sx;
+	unsigned char col_hi[3], col_lo[3];
+
+	UI_GetThemeColorShade3ubv(TH_BACK,  30, col_hi);
+	UI_GetThemeColorShade3ubv(TH_BACK, -30, col_lo);
+
+	v1[1] = v2d->cur.ymax - layout->tile_border_y;
+	v2[1] = v2d->cur.ymin;
+
+	glBegin(GL_LINES);
 
 	/* vertical column dividers */
 	sx = (int)v2d->tot.xmin;
 	while (sx < v2d->cur.xmax) {
-		sx += (layout->tile_w + 2 * layout->tile_border_x);
-		
-		UI_ThemeColorShade(TH_BACK, 30);
-		sdrawline(sx + 1, (short)(v2d->cur.ymax - layout->tile_border_y),  sx + 1,  (short)v2d->cur.ymin);
-		UI_ThemeColorShade(TH_BACK, -30);
-		sdrawline(sx, (short)(v2d->cur.ymax - layout->tile_border_y),  sx,  (short)v2d->cur.ymin); 
+		sx += step;
+
+		glColor3ubv(col_lo);
+		v1[0] = v2[0] = sx;
+		glVertex2iv(v1);
+		glVertex2iv(v2);
+
+		glColor3ubv(col_hi);
+		v1[0] = v2[0] = sx + 1;
+		glVertex2iv(v1);
+		glVertex2iv(v2);
 	}
+
+	glEnd();
 }
 
 void file_draw_list(const bContext *C, ARegion *ar)
@@ -451,16 +465,17 @@ void file_draw_list(const bContext *C, ARegion *ar)
 	struct FileList *files = sfile->files;
 	struct direntry *file;
 	ImBuf *imb;
-	uiBlock *block = uiBeginBlock(C, ar, __func__, UI_EMBOSS);
+	uiBlock *block = UI_block_begin(C, ar, __func__, UI_EMBOSS);
 	int numfiles;
 	int numfiles_layout;
 	int sx, sy;
 	int offset;
 	int textwidth, textheight;
 	int i;
-	short is_icon;
+	bool is_icon;
 	short align;
-
+	bool do_drag;
+	int column_space = 0.6f * UI_UNIT_X;
 
 	numfiles = filelist_numfiles(files);
 	
@@ -491,10 +506,10 @@ void file_draw_list(const bContext *C, ARegion *ar)
 
 	for (i = offset; (i < numfiles) && (i < offset + numfiles_layout); i++) {
 		ED_fileselect_layout_tilepos(layout, i, &sx, &sy);
-		sx += (int)(v2d->tot.xmin + 2.0f);
+		sx += (int)(v2d->tot.xmin + 0.1f * UI_UNIT_X);
 		sy = (int)(v2d->tot.ymax - sy);
 
-		file = filelist_file(files, i);	
+		file = filelist_file(files, i);
 		
 		UI_ThemeColor4(TH_TEXT);
 
@@ -506,7 +521,10 @@ void file_draw_list(const bContext *C, ARegion *ar)
 				draw_tile(sx, sy - 1, layout->tile_w + 4, sfile->layout->tile_h + layout->tile_border_y, colorid, shade);
 			}
 		}
-		uiSetRoundBox(UI_CNR_NONE);
+		UI_draw_roundbox_corner_set(UI_CNR_NONE);
+
+		/* don't drag parent or refresh items */
+		do_drag = !(STREQ(file->relname, "..") || STREQ(file->relname, "."));
 
 		if (FILE_IMGDISPLAY == params->display) {
 			is_icon = 0;
@@ -516,22 +534,38 @@ void file_draw_list(const bContext *C, ARegion *ar)
 				is_icon = 1;
 			}
 			
-			file_draw_preview(block, file, sx, sy, imb, layout, !is_icon && (file->flags & IMAGEFILE));
+			file_draw_preview(block, file, sx, sy, imb, layout, !is_icon && (file->flags & IMAGEFILE), do_drag);
 		}
 		else {
-			file_draw_icon(block, file->path, sx, sy - (UI_UNIT_Y / 6), get_file_icon(file), ICON_DEFAULT_WIDTH_SCALE, ICON_DEFAULT_HEIGHT_SCALE);
-			sx += ICON_DEFAULT_WIDTH_SCALE + 4;
+			file_draw_icon(block, file->path, sx, sy - (UI_UNIT_Y / 6), get_file_icon(file), ICON_DEFAULT_WIDTH_SCALE, ICON_DEFAULT_HEIGHT_SCALE, do_drag);
+			sx += ICON_DEFAULT_WIDTH_SCALE + 0.2f * UI_UNIT_X;
 		}
 
 		UI_ThemeColor4(TH_TEXT);
 
 		if (file->selflag & EDITING_FILE) {
-			uiBut *but = uiDefBut(block, TEX, 1, "", sx, sy - layout->tile_h - 3,
-			                      textwidth, textheight, sfile->params->renameedit, 1.0f, (float)sizeof(sfile->params->renameedit), 0, 0, "");
-			uiButSetRenameFunc(but, renamebutton_cb, file);
-			uiButSetFlag(but, UI_BUT_NO_UTF8); /* allow non utf8 names */
-			uiButClearFlag(but, UI_BUT_UNDO);
-			if (0 == uiButActiveOnly(C, block, but)) {
+			uiBut *but;
+			short width;
+
+			if (params->display == FILE_SHORTDISPLAY) {
+				width = layout->tile_w - (ICON_DEFAULT_WIDTH_SCALE + 0.2f * UI_UNIT_X);
+			}
+			else if (params->display == FILE_LONGDISPLAY) {
+				width = layout->column_widths[COLUMN_NAME]  + layout->column_widths[COLUMN_MODE1] +
+				        layout->column_widths[COLUMN_MODE2] + layout->column_widths[COLUMN_MODE3] +
+				        (column_space * 3.5f);
+			}
+			else {
+				BLI_assert(params->display == FILE_IMGDISPLAY);
+				width = textwidth;
+			}
+
+			but = uiDefBut(block, UI_BTYPE_TEXT, 1, "", sx, sy - layout->tile_h - 0.15f * UI_UNIT_X,
+			               width, textheight, sfile->params->renameedit, 1.0f, (float)sizeof(sfile->params->renameedit), 0, 0, "");
+			UI_but_func_rename_set(but, renamebutton_cb, file);
+			UI_but_flag_enable(but, UI_BUT_NO_UTF8); /* allow non utf8 names */
+			UI_but_flag_disable(but, UI_BUT_UNDO);
+			if (false == UI_but_active_only(C, ar, block, but)) {
 				file->selflag &= ~EDITING_FILE;
 			}
 		}
@@ -542,44 +576,44 @@ void file_draw_list(const bContext *C, ARegion *ar)
 		}
 
 		if (params->display == FILE_SHORTDISPLAY) {
-			sx += (int)layout->column_widths[COLUMN_NAME] + 12;
+			sx += (int)layout->column_widths[COLUMN_NAME] + column_space;
 			if (!(file->type & S_IFDIR)) {
-				file_draw_string(sx, sy, file->size, layout->column_widths[COLUMN_SIZE], layout->tile_h, align);	
-				sx += (int)layout->column_widths[COLUMN_SIZE] + 12;
+				file_draw_string(sx, sy, file->size, layout->column_widths[COLUMN_SIZE], layout->tile_h, align);
+				sx += (int)layout->column_widths[COLUMN_SIZE] + column_space;
 			}
 		}
 		else if (params->display == FILE_LONGDISPLAY) {
-			sx += (int)layout->column_widths[COLUMN_NAME] + 12;
+			sx += (int)layout->column_widths[COLUMN_NAME] + column_space;
 
 #ifndef WIN32
 			/* rwx rwx rwx */
 			file_draw_string(sx, sy, file->mode1, layout->column_widths[COLUMN_MODE1], layout->tile_h, align); 
-			sx += layout->column_widths[COLUMN_MODE1] + 12;
+			sx += layout->column_widths[COLUMN_MODE1] + column_space;
 
 			file_draw_string(sx, sy, file->mode2, layout->column_widths[COLUMN_MODE2], layout->tile_h, align);
-			sx += layout->column_widths[COLUMN_MODE2] + 12;
+			sx += layout->column_widths[COLUMN_MODE2] + column_space;
 
 			file_draw_string(sx, sy, file->mode3, layout->column_widths[COLUMN_MODE3], layout->tile_h, align);
-			sx += layout->column_widths[COLUMN_MODE3] + 12;
+			sx += layout->column_widths[COLUMN_MODE3] + column_space;
 
 			file_draw_string(sx, sy, file->owner, layout->column_widths[COLUMN_OWNER], layout->tile_h, align);
-			sx += layout->column_widths[COLUMN_OWNER] + 12;
+			sx += layout->column_widths[COLUMN_OWNER] + column_space;
 #endif
 
 			file_draw_string(sx, sy, file->date, layout->column_widths[COLUMN_DATE], layout->tile_h, align);
-			sx += (int)layout->column_widths[COLUMN_DATE] + 12;
+			sx += (int)layout->column_widths[COLUMN_DATE] + column_space;
 
 			file_draw_string(sx, sy, file->time, layout->column_widths[COLUMN_TIME], layout->tile_h, align);
-			sx += (int)layout->column_widths[COLUMN_TIME] + 12;
+			sx += (int)layout->column_widths[COLUMN_TIME] + column_space;
 
 			if (!(file->type & S_IFDIR)) {
 				file_draw_string(sx, sy, file->size, layout->column_widths[COLUMN_SIZE], layout->tile_h, align);
-				sx += (int)layout->column_widths[COLUMN_SIZE] + 12;
+				sx += (int)layout->column_widths[COLUMN_SIZE] + column_space;
 			}
 		}
 	}
 
-	uiEndBlock(C, block);
-	uiDrawBlock(C, block);
+	UI_block_end(C, block);
+	UI_block_draw(C, block);
 
 }
