@@ -28,16 +28,19 @@
  *  \ingroup spfile
  */
 
+#include "BLI_blenlib.h"
+#include "BLI_utildefines.h"
+#include "BLI_fileops_types.h"
 
+#include "BLO_readfile.h"
+
+#include "BKE_appdir.h"
 #include "BKE_context.h"
 #include "BKE_screen.h"
 #include "BKE_global.h"
 #include "BKE_report.h"
 #include "BKE_main.h"
 
-#include "BLI_blenlib.h"
-#include "BLI_utildefines.h"
-
 #ifdef WIN32
 #  include "BLI_winstuff.h"
 #endif
@@ -45,6 +48,8 @@
 #include "ED_screen.h"
 #include "ED_fileselect.h"
 
+#include "UI_interface.h"
+
 #include "MEM_guardedalloc.h"
 
 #include "RNA_access.h"
@@ -65,18 +70,24 @@
 #include <ctype.h>
 
 /* ---------- FILE SELECTION ------------ */
-static FileSelection find_file_mouse_rect(SpaceFile *sfile, ARegion *ar, const rcti *rect)
+static FileSelection find_file_mouse_rect(SpaceFile *sfile, ARegion *ar, const rcti *rect_region)
 {
 	FileSelection sel;
-	float fxmin, fymin, fxmax, fymax;
 	
 	View2D *v2d = &ar->v2d;
 	rcti rect_view;
+	rctf rect_view_fl;
+	rctf rect_region_fl;
 
-	UI_view2d_region_to_view(v2d, rect->xmin, rect->ymin, &fxmin, &fymin);
-	UI_view2d_region_to_view(v2d, rect->xmax, rect->ymax, &fxmax, &fymax);
+	BLI_rctf_rcti_copy(&rect_region_fl, rect_region);
 
-	BLI_rcti_init(&rect_view, (int)(v2d->tot.xmin + fxmin), (int)(v2d->tot.xmin + fxmax), (int)(v2d->tot.ymax - fymin), (int)(v2d->tot.ymax - fymax));
+	UI_view2d_region_to_view_rctf(v2d, &rect_region_fl, &rect_view_fl);
+
+	BLI_rcti_init(&rect_view,
+	              (int)(v2d->tot.xmin + rect_view_fl.xmin),
+	              (int)(v2d->tot.xmin + rect_view_fl.xmax),
+	              (int)(v2d->tot.ymax - rect_view_fl.ymin),
+	              (int)(v2d->tot.ymax - rect_view_fl.ymax));
 
 	sel  = ED_fileselect_layout_offset_rect(sfile->layout, &rect_view);
 	
@@ -123,7 +134,7 @@ static void clamp_to_filelist(int numfiles, FileSelection *sel)
 	}
 }
 
-static FileSelection file_selection_get(bContext *C, const rcti *rect, short fill)
+static FileSelection file_selection_get(bContext *C, const rcti *rect, bool fill)
 {
 	ARegion *ar = CTX_wm_region(C);
 	SpaceFile *sfile = CTX_wm_space_file(C);
@@ -151,7 +162,7 @@ static FileSelection file_selection_get(bContext *C, const rcti *rect, short fil
 	return sel;
 }
 
-static FileSelect file_select_do(bContext *C, int selected_idx, short do_diropen)
+static FileSelect file_select_do(bContext *C, int selected_idx, bool do_diropen)
 {
 	FileSelect retval = FILE_SELECT_NOTHING;
 	SpaceFile *sfile = CTX_wm_space_file(C);
@@ -167,7 +178,7 @@ static FileSelect file_select_do(bContext *C, int selected_idx, short do_diropen
 		params->active_file = selected_idx;
 
 		if (S_ISDIR(file->type)) {
-			if (do_diropen == FALSE) {
+			if (do_diropen == false) {
 				params->file[0] = '\0';
 				retval = FILE_SELECT_DIR;
 			}
@@ -201,7 +212,7 @@ static FileSelect file_select_do(bContext *C, int selected_idx, short do_diropen
 }
 
 
-static FileSelect file_select(bContext *C, const rcti *rect, FileSelType select, short fill, short do_diropen)
+static FileSelect file_select(bContext *C, const rcti *rect, FileSelType select, bool fill, bool do_diropen)
 {
 	SpaceFile *sfile = CTX_wm_space_file(C);
 	FileSelect retval = FILE_SELECT_NOTHING;
@@ -228,7 +239,7 @@ static FileSelect file_select(bContext *C, const rcti *rect, FileSelType select,
 	return retval;
 }
 
-static int file_border_select_modal(bContext *C, wmOperator *op, wmEvent *event)
+static int file_border_select_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	ARegion *ar = CTX_wm_region(C);
 	SpaceFile *sfile = CTX_wm_space_file(C);
@@ -270,8 +281,8 @@ static int file_border_select_exec(bContext *C, wmOperator *op)
 	ARegion *ar = CTX_wm_region(C);
 	rcti rect;
 	FileSelect ret;
-	int extend = RNA_boolean_get(op->ptr, "extend");
-	short select = (RNA_int_get(op->ptr, "gesture_mode") == GESTURE_MODAL_SELECT);
+	const bool select = (RNA_int_get(op->ptr, "gesture_mode") == GESTURE_MODAL_SELECT);
+	const bool extend = RNA_boolean_get(op->ptr, "extend");
 
 	WM_operator_properties_border_to_rcti(op, &rect);
 
@@ -283,7 +294,7 @@ static int file_border_select_exec(bContext *C, wmOperator *op)
 
 	BLI_rcti_isect(&(ar->v2d.mask), &rect, &rect);
 
-	ret = file_select(C, &rect, select ? FILE_SEL_ADD : FILE_SEL_REMOVE, FALSE, FALSE);
+	ret = file_select(C, &rect, select ? FILE_SEL_ADD : FILE_SEL_REMOVE, false, false);
 	if (FILE_SELECT_DIR == ret) {
 		WM_event_add_notifier(C, NC_SPACE | ND_SPACE_FILE_LIST, NULL);
 	}
@@ -311,15 +322,15 @@ void FILE_OT_select_border(wmOperatorType *ot)
 	WM_operator_properties_gesture_border(ot, 1);
 }
 
-static int file_select_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int file_select_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	ARegion *ar = CTX_wm_region(C);
 	SpaceFile *sfile = CTX_wm_space_file(C);
 	FileSelect ret;
 	rcti rect;
-	int extend = RNA_boolean_get(op->ptr, "extend");
-	int fill = RNA_boolean_get(op->ptr, "fill");
-	int do_diropen = RNA_boolean_get(op->ptr, "open");
+	const bool extend = RNA_boolean_get(op->ptr, "extend");
+	const bool fill = RNA_boolean_get(op->ptr, "fill");
+	const bool do_diropen = RNA_boolean_get(op->ptr, "open");
 
 	if (ar->regiontype != RGN_TYPE_WINDOW)
 		return OPERATOR_CANCELLED;
@@ -359,11 +370,11 @@ void FILE_OT_select(wmOperatorType *ot)
 	ot->poll = ED_operator_file_active;
 
 	/* properties */
-	prop = RNA_def_boolean(ot->srna, "extend", FALSE, "Extend", "Extend selection instead of deselecting everything first");
+	prop = RNA_def_boolean(ot->srna, "extend", false, "Extend", "Extend selection instead of deselecting everything first");
 	RNA_def_property_flag(prop, PROP_SKIP_SAVE);
-	prop = RNA_def_boolean(ot->srna, "fill", FALSE, "Fill", "Select everything beginning with the last selection");
+	prop = RNA_def_boolean(ot->srna, "fill", false, "Fill", "Select everything beginning with the last selection");
 	RNA_def_property_flag(prop, PROP_SKIP_SAVE);
-	prop = RNA_def_boolean(ot->srna, "open", TRUE, "Open", "Open a directory when selecting it");
+	prop = RNA_def_boolean(ot->srna, "open", true, "Open", "Open a directory when selecting it");
 	RNA_def_property_flag(prop, PROP_SKIP_SAVE);
 }
 
@@ -374,7 +385,7 @@ static int file_select_all_exec(bContext *C, wmOperator *UNUSED(op))
 	FileSelection sel;
 	int numfiles = filelist_numfiles(sfile->files);
 	int i;
-	int is_selected = 0;
+	bool is_selected = false;
 
 	sel.first = 0; 
 	sel.last = numfiles - 1;
@@ -382,7 +393,7 @@ static int file_select_all_exec(bContext *C, wmOperator *UNUSED(op))
 	/* Is any file selected ? */
 	for (i = 0; i < numfiles; ++i) {
 		if (filelist_is_selected(sfile->files, i, CHECK_ALL)) {
-			is_selected = 1;
+			is_selected = true;
 			break;
 		}
 	}
@@ -447,7 +458,7 @@ void FILE_OT_select_bookmark(wmOperatorType *ot)
 	ot->poll = ED_operator_file_active;
 
 	/* properties */
-	prop = RNA_def_string(ot->srna, "dir", "", FILE_MAXDIR, "Dir", "");
+	prop = RNA_def_string(ot->srna, "dir", NULL, FILE_MAXDIR, "Dir", "");
 	RNA_def_property_flag(prop, PROP_SKIP_SAVE);
 }
 
@@ -462,7 +473,7 @@ static int bookmark_add_exec(bContext *C, wmOperator *UNUSED(op))
 		char name[FILE_MAX];
 	
 		fsmenu_insert_entry(fsmenu, FS_CATEGORY_BOOKMARKS, params->dir, FS_INSERT_SAVE);
-		BLI_make_file_string("/", name, BLI_get_folder_create(BLENDER_USER_CONFIG, NULL), BLENDER_BOOKMARK_FILE);
+		BLI_make_file_string("/", name, BKE_appdir_folder_id_create(BLENDER_USER_CONFIG, NULL), BLENDER_BOOKMARK_FILE);
 		fsmenu_write_file(fsmenu, name);
 	}
 
@@ -490,11 +501,11 @@ static int bookmark_delete_exec(bContext *C, wmOperator *op)
 	
 	if (RNA_struct_find_property(op->ptr, "index")) {
 		int index = RNA_int_get(op->ptr, "index");
-		if ( (index > -1) && (index < nentries)) {
+		if ((index > -1) && (index < nentries)) {
 			char name[FILE_MAX];
 			
 			fsmenu_remove_entry(fsmenu, FS_CATEGORY_BOOKMARKS, index);
-			BLI_make_file_string("/", name, BLI_get_folder_create(BLENDER_USER_CONFIG, NULL), BLENDER_BOOKMARK_FILE);
+			BLI_make_file_string("/", name, BKE_appdir_folder_id_create(BLENDER_USER_CONFIG, NULL), BLENDER_BOOKMARK_FILE);
 			fsmenu_write_file(fsmenu, name);
 			ED_area_tag_redraw(sa);
 		}
@@ -503,14 +514,14 @@ static int bookmark_delete_exec(bContext *C, wmOperator *op)
 	return OPERATOR_FINISHED;
 }
 
-void FILE_OT_delete_bookmark(wmOperatorType *ot)
+void FILE_OT_bookmark_delete(wmOperatorType *ot)
 {
 	PropertyRNA *prop;
 
 	/* identifiers */
 	ot->name = "Delete Bookmark";
 	ot->description = "Delete selected bookmark";
-	ot->idname = "FILE_OT_delete_bookmark";
+	ot->idname = "FILE_OT_bookmark_delete";
 	
 	/* api callbacks */
 	ot->exec = bookmark_delete_exec;
@@ -530,7 +541,7 @@ static int reset_recent_exec(bContext *C, wmOperator *UNUSED(op))
 	while (fsmenu_get_entry(fsmenu, FS_CATEGORY_RECENT, 0) != NULL) {
 		fsmenu_remove_entry(fsmenu, FS_CATEGORY_RECENT, 0);
 	}
-	BLI_make_file_string("/", name, BLI_get_folder_create(BLENDER_USER_CONFIG, NULL), BLENDER_BOOKMARK_FILE);
+	BLI_make_file_string("/", name, BKE_appdir_folder_id_create(BLENDER_USER_CONFIG, NULL), BLENDER_BOOKMARK_FILE);
 	fsmenu_write_file(fsmenu, name);
 	ED_area_tag_redraw(sa);
 		
@@ -554,7 +565,7 @@ int file_highlight_set(SpaceFile *sfile, ARegion *ar, int mx, int my)
 {
 	View2D *v2d = &ar->v2d;
 	FileSelectParams *params;
-	int numfiles, origfile;	
+	int numfiles, origfile;
 
 	if (sfile == NULL || sfile->files == NULL) return 0;
 
@@ -585,7 +596,7 @@ int file_highlight_set(SpaceFile *sfile, ARegion *ar, int mx, int my)
 	return (params->active_file != origfile);
 }
 
-static int file_highlight_invoke(bContext *C, wmOperator *UNUSED(op), wmEvent *event)
+static int file_highlight_invoke(bContext *C, wmOperator *UNUSED(op), const wmEvent *event)
 {
 	ARegion *ar = CTX_wm_region(C);
 	SpaceFile *sfile = CTX_wm_space_file(C);
@@ -612,12 +623,13 @@ void FILE_OT_highlight(struct wmOperatorType *ot)
 
 int file_cancel_exec(bContext *C, wmOperator *UNUSED(unused))
 {
+	wmWindowManager *wm = CTX_wm_manager(C);
 	SpaceFile *sfile = CTX_wm_space_file(C);
 	wmOperator *op = sfile->op;
 	
 	sfile->op = NULL;
 
-	WM_event_fileselect_event(C, op, EVT_FILESELECT_CANCEL);
+	WM_event_fileselect_event(wm, op, EVT_FILESELECT_CANCEL);
 	
 	return OPERATOR_FINISHED;
 }
@@ -760,7 +772,7 @@ void file_draw_check_cb(bContext *C, void *UNUSED(arg1), void *UNUSED(arg2))
 	}
 }
 
-int file_draw_check_exists(SpaceFile *sfile)
+bool file_draw_check_exists(SpaceFile *sfile)
 {
 	if (sfile->op) { /* fails on reload */
 		if (RNA_struct_find_property(sfile->op->ptr, "check_existing")) {
@@ -768,18 +780,19 @@ int file_draw_check_exists(SpaceFile *sfile)
 				char filepath[FILE_MAX];
 				BLI_join_dirfile(filepath, sizeof(filepath), sfile->params->dir, sfile->params->file);
 				if (BLI_is_file(filepath)) {
-					return TRUE;
+					return true;
 				}
 			}
 		}
 	}
 
-	return FALSE;
+	return false;
 }
 
 /* sends events now, so things get handled on windowqueue level */
 int file_exec(bContext *C, wmOperator *exec_op)
 {
+	wmWindowManager *wm = CTX_wm_manager(C);
 	SpaceFile *sfile = CTX_wm_space_file(C);
 	char filepath[FILE_MAX];
 	
@@ -809,9 +822,9 @@ int file_exec(bContext *C, wmOperator *exec_op)
 			fsmenu_insert_entry(fsmenu_get(), FS_CATEGORY_RECENT, sfile->params->dir, FS_INSERT_SAVE | FS_INSERT_FIRST);
 		}
 
-		BLI_make_file_string(G.main->name, filepath, BLI_get_folder_create(BLENDER_USER_CONFIG, NULL), BLENDER_BOOKMARK_FILE);
+		BLI_make_file_string(G.main->name, filepath, BKE_appdir_folder_id_create(BLENDER_USER_CONFIG, NULL), BLENDER_BOOKMARK_FILE);
 		fsmenu_write_file(fsmenu_get(), filepath);
-		WM_event_fileselect_event(C, op, EVT_FILESELECT_EXEC);
+		WM_event_fileselect_event(wm, op, EVT_FILESELECT_EXEC);
 
 	}
 				
@@ -843,13 +856,24 @@ int file_parent_exec(bContext *C, wmOperator *UNUSED(unused))
 	SpaceFile *sfile = CTX_wm_space_file(C);
 	
 	if (sfile->params) {
-		if (BLI_has_parent(sfile->params->dir)) {
-			BLI_parent_dir(sfile->params->dir);
+		if (BLI_parent_dir(sfile->params->dir)) {
 			BLI_cleanup_dir(G.main->name, sfile->params->dir);
-			file_change_dir(C, 0);
+			/* if not browsing in .blend file, we still want to check whether the path is a directory */
+			if (sfile->params->type == FILE_LOADLIB) {
+				char tdir[FILE_MAX], tgroup[FILE_MAX];
+				if (BLO_is_a_library(sfile->params->dir, tdir, tgroup)) {
+					file_change_dir(C, 0);
+				}
+				else {
+					file_change_dir(C, 1);
+				}
+			}
+			else {
+				file_change_dir(C, 1);
+			}
 			WM_event_add_notifier(C, NC_SPACE | ND_SPACE_FILE_LIST, NULL);
 		}
-	}		
+	}
 	
 	return OPERATOR_FINISHED;
 
@@ -871,10 +895,11 @@ void FILE_OT_parent(struct wmOperatorType *ot)
 
 static int file_refresh_exec(bContext *C, wmOperator *UNUSED(unused))
 {
+	wmWindowManager *wm = CTX_wm_manager(C);
 	SpaceFile *sfile = CTX_wm_space_file(C);
 	struct FSMenu *fsmenu = fsmenu_get();
 
-	ED_fileselect_clear(C, sfile);
+	ED_fileselect_clear(wm, sfile);
 
 	/* refresh system directory menu */
 	fsmenu_refresh_system_category(fsmenu);
@@ -942,7 +967,7 @@ int file_next_exec(bContext *C, wmOperator *UNUSED(unused))
 		folderlist_pushdir(sfile->folders_prev, sfile->params->dir);
 
 		file_change_dir(C, 1);
-	}		
+	}
 	WM_event_add_notifier(C, NC_SPACE | ND_SPACE_FILE_LIST, NULL);
 
 	return OPERATOR_FINISHED;
@@ -950,7 +975,7 @@ int file_next_exec(bContext *C, wmOperator *UNUSED(unused))
 
 
 /* only meant for timer usage */
-static int file_smoothscroll_invoke(bContext *C, wmOperator *UNUSED(op), wmEvent *event)
+static int file_smoothscroll_invoke(bContext *C, wmOperator *UNUSED(op), const wmEvent *event)
 {
 	ScrArea *sa = CTX_wm_area(C);
 	SpaceFile *sfile = CTX_wm_space_file(C);
@@ -1036,7 +1061,7 @@ static int file_smoothscroll_invoke(bContext *C, wmOperator *UNUSED(op), wmEvent
 		}
 	}
 	
-	ED_region_tag_redraw(CTX_wm_region(C));
+	ED_region_tag_redraw(ar);
 	
 	/* and restore context */
 	CTX_wm_region_set(C, oldar);
@@ -1088,6 +1113,7 @@ int file_directory_new_exec(bContext *C, wmOperator *op)
 	char path[FILE_MAX];
 	int generate_name = 1;
 
+	wmWindowManager *wm = CTX_wm_manager(C);
 	SpaceFile *sfile = CTX_wm_space_file(C);
 	
 	if (!sfile->params) {
@@ -1105,7 +1131,7 @@ int file_directory_new_exec(bContext *C, wmOperator *op)
 	if (generate_name) {
 		/* create a new, non-existing folder name */
 		if (!new_folder_path(sfile->params->dir, path, name)) {
-			BKE_report(op->reports, RPT_ERROR, "Couldn't create new folder name");
+			BKE_report(op->reports, RPT_ERROR, "Could not create new folder name");
 			return OPERATOR_CANCELLED;
 		}
 	}
@@ -1114,19 +1140,25 @@ int file_directory_new_exec(bContext *C, wmOperator *op)
 	BLI_dir_create_recursive(path);
 
 	if (!BLI_exists(path)) {
-		BKE_report(op->reports, RPT_ERROR, "Couldn't create new folder");
+		BKE_report(op->reports, RPT_ERROR, "Could not create new folder");
 		return OPERATOR_CANCELLED;
-	} 
+	}
 
 	/* now remember file to jump into editing */
 	BLI_strncpy(sfile->params->renamefile, name, FILE_MAXFILE);
 
 	/* set timer to smoothly view newly generated file */
-	sfile->smoothscroll_timer = WM_event_add_timer(CTX_wm_manager(C), CTX_wm_window(C), TIMER1, 1.0 / 1000.0);    /* max 30 frs/sec */
+	sfile->smoothscroll_timer = WM_event_add_timer(wm, CTX_wm_window(C), TIMER1, 1.0 / 1000.0);  /* max 30 frs/sec */
 	sfile->scroll_offset = 0;
 
 	/* reload dir to make sure we're seeing what's in the directory */
-	ED_fileselect_clear(C, sfile);
+	ED_fileselect_clear(wm, sfile);
+
+	if (RNA_boolean_get(op->ptr, "open")) {
+		BLI_strncpy(sfile->params->dir, path, sizeof(sfile->params->dir));
+		file_change_dir(C, 1);
+	}
+
 	WM_event_add_notifier(C, NC_SPACE | ND_SPACE_FILE_LIST, NULL);
 
 	return OPERATOR_FINISHED;
@@ -1147,9 +1179,10 @@ void FILE_OT_directory_new(struct wmOperatorType *ot)
 	ot->exec = file_directory_new_exec;
 	ot->poll = ED_operator_file_active; /* <- important, handler is on window level */
 
-	prop = RNA_def_string_dir_path(ot->srna, "directory", "", FILE_MAX, "Directory", "Name of new directory");
+	prop = RNA_def_string_dir_path(ot->srna, "directory", NULL, FILE_MAX, "Directory", "Name of new directory");
+	RNA_def_property_flag(prop, PROP_SKIP_SAVE);
+	prop = RNA_def_boolean(ot->srna, "open", false, "Open", "Open new directory");
 	RNA_def_property_flag(prop, PROP_SKIP_SAVE);
-
 }
 
 
@@ -1165,7 +1198,7 @@ static void file_expand_directory(bContext *C)
 		else if (sfile->params->dir[0] == '~') {
 			char tmpstr[sizeof(sfile->params->dir) - 1];
 			BLI_strncpy(tmpstr, sfile->params->dir + 1, sizeof(tmpstr));
-			BLI_join_dirfile(sfile->params->dir, sizeof(sfile->params->dir), BLI_getDefaultDocumentFolder(), tmpstr);
+			BLI_join_dirfile(sfile->params->dir, sizeof(sfile->params->dir), BKE_appdir_folder_default(), tmpstr);
 		}
 
 		else if (sfile->params->dir[0] == '\0')
@@ -1179,48 +1212,44 @@ static void file_expand_directory(bContext *C)
 			get_default_root(sfile->params->dir);
 		}
 		/* change "C:" --> "C:\", [#28102] */
-		else if (   (isalpha(sfile->params->dir[0]) &&
-		             (sfile->params->dir[1] == ':')) &&
-		            (sfile->params->dir[2] == '\0')
-
-		            ) {
+		else if ((isalpha(sfile->params->dir[0]) &&
+		          (sfile->params->dir[1] == ':')) &&
+		         (sfile->params->dir[2] == '\0'))
+		{
 			sfile->params->dir[2] = '\\';
 			sfile->params->dir[3] = '\0';
 		}
+		else if (BLI_path_is_unc(sfile->params->dir)) {
+			BLI_cleanup_unc(sfile->params->dir, FILE_MAX_LIBEXTRA);
+		}
 #endif
 	}
 }
 
-static int file_directory_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+#if defined(WIN32)
+static bool can_create_dir(const char *dir)
 {
-	SpaceFile *sfile = CTX_wm_space_file(C);
-
-	if (sfile->params) {
-		file_expand_directory(C);
-		
-		if (!BLI_exists(sfile->params->dir)) {
-			return WM_operator_confirm_message(C, op, "Create new directory?");
-		}
-
-		return file_directory_exec(C, op);
+	/* for UNC paths we need to check whether the parent of the new
+	 * directory is a proper directory itself and not a share or the
+	 * UNC root (server name) itself. Calling BLI_is_dir does this
+	 */
+	if (BLI_path_is_unc(dir)) {
+		char parent[PATH_MAX];
+		BLI_strncpy(parent, dir, PATH_MAX);
+		BLI_parent_dir(parent);
+		return BLI_is_dir(parent);
 	}
-
-	return OPERATOR_CANCELLED;
+	return true;
 }
+#endif
 
-
-
-int file_directory_exec(bContext *C, wmOperator *UNUSED(unused))
+void file_directory_enter_handle(bContext *C, void *UNUSED(arg_unused), void *UNUSED(arg_but))
 {
 	SpaceFile *sfile = CTX_wm_space_file(C);
 	
 	if (sfile->params) {
 		file_expand_directory(C);
 
-		if (!BLI_exists(sfile->params->dir)) {
-			BLI_dir_create_recursive(sfile->params->dir);
-		}
-
 		/* special case, user may have pasted a filepath into the directory */
 		if (BLI_is_file(sfile->params->dir)) {
 			char path[sizeof(sfile->params->dir)];
@@ -1229,22 +1258,61 @@ int file_directory_exec(bContext *C, wmOperator *UNUSED(unused))
 		}
 
 		BLI_cleanup_dir(G.main->name, sfile->params->dir);
-		BLI_add_slash(sfile->params->dir);
-		file_change_dir(C, 1);
+
+		if (BLI_exists(sfile->params->dir)) {
+			/* if directory exists, enter it immediately */
+			file_change_dir(C, 1);
+
+			/* don't do for now because it selects entire text instead of
+			 * placing cursor at the end */
+			/* UI_textbutton_activate_but(C, but); */
+		}
+#if defined(WIN32)
+		else if (!can_create_dir(sfile->params->dir)) {
+			const char *lastdir = folderlist_peeklastdir(sfile->folders_prev);
+			if (lastdir)
+				BLI_strncpy(sfile->params->dir, lastdir, sizeof(sfile->params->dir));
+		}
+#endif
+		else {
+			const char *lastdir = folderlist_peeklastdir(sfile->folders_prev);
+
+			/* if not, ask to create it and enter if confirmed */
+			wmOperatorType *ot = WM_operatortype_find("FILE_OT_directory_new", false);
+			PointerRNA ptr;
+			WM_operator_properties_create_ptr(&ptr, ot);
+			RNA_string_set(&ptr, "directory", sfile->params->dir);
+			RNA_boolean_set(&ptr, "open", true);
+
+			if (lastdir)
+				BLI_strncpy(sfile->params->dir, lastdir, sizeof(sfile->params->dir));
+
+
+			WM_operator_name_call_ptr(C, ot, WM_OP_INVOKE_DEFAULT, &ptr);
+			WM_operator_properties_free(&ptr);
+		}
 
 		WM_event_add_notifier(C, NC_SPACE | ND_SPACE_FILE_LIST, NULL);
 	}
-
-	return OPERATOR_FINISHED;
 }
 
-int file_filename_exec(bContext *C, wmOperator *UNUSED(unused))
+void file_filename_enter_handle(bContext *C, void *UNUSED(arg_unused), void *arg_but)
 {
 	SpaceFile *sfile = CTX_wm_space_file(C);
+	uiBut *but = arg_but;
 	char matched_file[FILE_MAX];
+	char filepath[sizeof(sfile->params->dir)];
+
 	if (sfile->params) {
+		int matches;
 		matched_file[0] = '\0';
-		if (file_select_match(sfile, sfile->params->file, matched_file)) {
+		filepath[0] = '\0';
+
+		file_expand_directory(C);
+
+		matches = file_select_match(sfile, sfile->params->file, matched_file);
+
+		if (matches) {
 			/* int i, numfiles = filelist_numfiles(sfile->files); */ /* XXX UNUSED */
 			sfile->params->file[0] = '\0';
 			/* replace the pattern (or filename that the user typed in, with the first selected file of the match */
@@ -1252,31 +1320,34 @@ int file_filename_exec(bContext *C, wmOperator *UNUSED(unused))
 			
 			WM_event_add_notifier(C, NC_SPACE | ND_SPACE_FILE_PARAMS, NULL);
 		}
+
+		if (matches == 1) {
+
+			BLI_join_dirfile(filepath, sizeof(sfile->params->dir), sfile->params->dir, sfile->params->file);
+
+			/* if directory, open it and empty filename field */
+			if (BLI_is_dir(filepath)) {
+				BLI_cleanup_dir(G.main->name, filepath);
+				BLI_strncpy(sfile->params->dir, filepath, sizeof(sfile->params->dir));
+				sfile->params->file[0] = '\0';
+				file_change_dir(C, 1);
+				UI_textbutton_activate_but(C, but);
+				WM_event_add_notifier(C, NC_SPACE | ND_SPACE_FILE_PARAMS, NULL);
+			}
+			else if (sfile->params->type == FILE_LOADLIB) {
+				char tdir[FILE_MAX], tgroup[FILE_MAX];
+				BLI_add_slash(filepath);
+				if (BLO_is_a_library(filepath, tdir, tgroup)) {
+					BLI_cleanup_dir(G.main->name, filepath);
+					BLI_strncpy(sfile->params->dir, filepath, sizeof(sfile->params->dir));
+					sfile->params->file[0] = '\0';
+					file_change_dir(C, 0);
+					UI_textbutton_activate_but(C, but);
+					WM_event_add_notifier(C, NC_SPACE | ND_SPACE_FILE_LIST, NULL);
+				}
+			}
+		}
 	}
-
-	return OPERATOR_FINISHED;
-}
-
-/* TODO, directory operator is non-functional while a library is loaded
- * until this is properly supported just disable it. */
-static int file_directory_poll(bContext *C)
-{
-	/* sfile->files can be NULL on file load */
-	SpaceFile *sfile = CTX_wm_space_file(C);
-	return ED_operator_file_active(C) && (sfile->files == NULL || filelist_lib(sfile->files) == NULL);
-}
-
-void FILE_OT_directory(struct wmOperatorType *ot)
-{
-	/* identifiers */
-	ot->name = "Enter Directory Name";
-	ot->description = "Enter a directory name";
-	ot->idname = "FILE_OT_directory";
-	
-	/* api callbacks */
-	ot->invoke = file_directory_invoke;
-	ot->exec = file_directory_exec;
-	ot->poll = file_directory_poll; /* <- important, handler is on window level */
 }
 
 void FILE_OT_refresh(struct wmOperatorType *ot)
@@ -1293,11 +1364,12 @@ void FILE_OT_refresh(struct wmOperatorType *ot)
 
 static int file_hidedot_exec(bContext *C, wmOperator *UNUSED(unused))
 {
+	wmWindowManager *wm = CTX_wm_manager(C);
 	SpaceFile *sfile = CTX_wm_space_file(C);
 	
 	if (sfile->params) {
 		sfile->params->flag ^= FILE_HIDE_DOT;
-		ED_fileselect_clear(C, sfile);
+		ED_fileselect_clear(wm, sfile);
 		WM_event_add_notifier(C, NC_SPACE | ND_SPACE_FILE_LIST, NULL);
 	}
 	
@@ -1427,11 +1499,20 @@ static int file_rename_poll(bContext *C)
 	SpaceFile *sfile = CTX_wm_space_file(C);
 
 	if (sfile && sfile->params) {
-		if (sfile->params->active_file < 0) { 
+		int idx = sfile->params->active_file;
+
+		if (idx >= 0) {
+			struct direntry *file = filelist_file(sfile->files, idx);
+			if (STREQ(file->relname, "..") || STREQ(file->relname, ".")) {
+				poll = 0;
+			}
+		}
+
+		if (sfile->params->active_file < 0) {
 			poll = 0;
 		}
 		else {
-			char dir[FILE_MAX], group[FILE_MAX];	
+			char dir[FILE_MAX], group[FILE_MAX];
 			if (filelist_islibrary(sfile->files, dir, group)) poll = 0;
 		}
 	}
@@ -1457,17 +1538,21 @@ static int file_delete_poll(bContext *C)
 {
 	int poll = ED_operator_file_active(C);
 	SpaceFile *sfile = CTX_wm_space_file(C);
-	struct direntry *file;
 
 	if (sfile && sfile->params) {
-		if (sfile->params->active_file < 0) { 
-			poll = 0;
+		char dir[FILE_MAX], group[FILE_MAX];
+		int numfiles = filelist_numfiles(sfile->files);
+		int i;
+		int num_selected = 0;
+
+		if (filelist_islibrary(sfile->files, dir, group)) poll = 0;
+		for (i = 0; i < numfiles; i++) {
+			if (filelist_is_selected(sfile->files, i, CHECK_FILES)) {
+				num_selected++;
+			}
 		}
-		else {
-			char dir[FILE_MAX], group[FILE_MAX];	
-			if (filelist_islibrary(sfile->files, dir, group)) poll = 0;
-			file = filelist_file(sfile->files, sfile->params->active_file);
-			if (file && S_ISDIR(file->type)) poll = 0;
+		if (num_selected <= 0) {
+			poll = 0;
 		}
 	}
 	else
@@ -1479,14 +1564,21 @@ static int file_delete_poll(bContext *C)
 int file_delete_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	char str[FILE_MAX];
+	wmWindowManager *wm = CTX_wm_manager(C);
 	SpaceFile *sfile = CTX_wm_space_file(C);
-	struct direntry *file;
+	struct direntry *file;	
+	int numfiles = filelist_numfiles(sfile->files);
+	int i;
+
+	for (i = 0; i < numfiles; i++) {
+		if (filelist_is_selected(sfile->files, i, CHECK_FILES)) {
+			file = filelist_file(sfile->files, i);
+			BLI_make_file_string(G.main->name, str, sfile->params->dir, file->relname);
+			BLI_delete(str, false, false);
+		}
+	}
 	
-	
-	file = filelist_file(sfile->files, sfile->params->active_file);
-	BLI_make_file_string(G.main->name, str, sfile->params->dir, file->relname);
-	BLI_delete(str, 0, 0);	
-	ED_fileselect_clear(C, sfile);
+	ED_fileselect_clear(wm, sfile);
 	WM_event_add_notifier(C, NC_SPACE | ND_SPACE_FILE_LIST, NULL);
 	
 	return OPERATOR_FINISHED;
@@ -1496,8 +1588,8 @@ int file_delete_exec(bContext *C, wmOperator *UNUSED(op))
 void FILE_OT_delete(struct wmOperatorType *ot)
 {
 	/* identifiers */
-	ot->name = "Delete File";
-	ot->description = "Delete selected file";
+	ot->name = "Delete Selected Files";
+	ot->description = "Delete selected files";
 	ot->idname = "FILE_OT_delete";
 	
 	/* api callbacks */
