@@ -28,13 +28,12 @@
  *  \ingroup spfile
  */
 
+#include "BLI_blenlib.h"
+#include "BLI_utildefines.h"
 
 #include "BKE_context.h"
 #include "BKE_screen.h"
 
-#include "BLI_blenlib.h"
-#include "BLI_utildefines.h"
-
 #include "BLF_translation.h"
 
 #include "DNA_screen_types.h"
@@ -58,12 +57,13 @@
 
 static void file_panel_cb(bContext *C, void *arg_entry, void *UNUSED(arg_v))
 {
+	wmOperatorType *ot = WM_operatortype_find("FILE_OT_select_bookmark", false);
 	PointerRNA ptr;
-	char *entry = (char *)arg_entry;
+	const char *entry = (char *)arg_entry;
 
-	WM_operator_properties_create(&ptr, "FILE_OT_select_bookmark");
+	WM_operator_properties_create_ptr(&ptr, ot);
 	RNA_string_set(&ptr, "dir", entry);
-	WM_operator_name_call(C, "FILE_OT_select_bookmark", WM_OP_INVOKE_REGION_WIN, &ptr);
+	WM_operator_name_call_ptr(C, ot, WM_OP_INVOKE_REGION_WIN, &ptr);
 	WM_operator_properties_free(&ptr);
 }
 
@@ -87,12 +87,12 @@ static void file_panel_category(const bContext *C, Panel *pa, FSMenuCategory cat
 	uiLayoutSetAlignment(pa->layout, UI_LAYOUT_ALIGN_LEFT);
 	block = uiLayoutGetBlock(pa->layout);
 	box = uiLayoutBox(pa->layout);
-	col = uiLayoutColumn(box, TRUE);
+	col = uiLayoutColumn(box, true);
 
 	for (i = 0; i < nentries; ++i) {
 		char dir[FILE_MAX];
 		char temp[FILE_MAX];
-		uiLayout *layout = uiLayoutRow(col, FALSE);
+		uiLayout *layout = uiLayoutRow(col, false);
 		char *entry;
 		
 		entry = fsmenu_get_entry(fsmenu, category, i);
@@ -114,15 +114,16 @@ static void file_panel_category(const bContext *C, Panel *pa, FSMenuCategory cat
 			BLI_strncpy(dir, entry, FILE_MAX);
 
 		/* create list item */
-		but = uiDefIconTextButS(block, LISTROW, 0, icon, dir, 0, 0, UI_UNIT_X * 10, UI_UNIT_Y, nr, 0, i, 0, 0, entry);
-		uiButSetFunc(but, file_panel_cb, entry, NULL);
-		uiButSetFlag(but, UI_ICON_LEFT | UI_TEXT_LEFT);
+		but = uiDefIconTextButS(block, UI_BTYPE_LISTROW, 0, icon, dir, 0, 0, UI_UNIT_X * 10, UI_UNIT_Y, nr, 0, i, 0, 0, entry);
+		UI_but_func_set(but, file_panel_cb, entry, NULL);
+		UI_but_flag_disable(but, UI_BUT_UNDO); /* skip undo on screen buttons */
+		UI_but_drawflag_enable(but, UI_BUT_ICON_LEFT | UI_BUT_TEXT_LEFT);
 
 		/* create delete button */
 		if (allow_delete && fsmenu_can_save(fsmenu, category, i)) {
-			uiBlockSetEmboss(block, UI_EMBOSSN);
-			uiItemIntO(layout, "", ICON_X, "FILE_OT_delete_bookmark", "index", i);
-			uiBlockSetEmboss(block, UI_EMBOSS);
+			UI_block_emboss_set(block, UI_EMBOSS_NONE);
+			uiItemIntO(layout, "", ICON_X, "FILE_OT_bookmark_delete", "index", i);
+			UI_block_emboss_set(block, UI_EMBOSS);
 		}
 	}
 }
@@ -135,13 +136,29 @@ static void file_panel_system(const bContext *C, Panel *pa)
 		file_panel_category(C, pa, FS_CATEGORY_SYSTEM, &sfile->systemnr, ICON_DISK_DRIVE, 0);
 }
 
+static int file_panel_system_bookmarks_poll(const bContext *C, PanelType *UNUSED(pt))
+{
+	SpaceFile *sfile = CTX_wm_space_file(C);
+	return (sfile && !(U.uiflag & USER_HIDE_SYSTEM_BOOKMARKS));
+}
+
+static void file_panel_system_bookmarks(const bContext *C, Panel *pa)
+{
+	SpaceFile *sfile = CTX_wm_space_file(C);
+
+	if (sfile && !(U.uiflag & USER_HIDE_SYSTEM_BOOKMARKS)) {
+		file_panel_category(C, pa, FS_CATEGORY_SYSTEM_BOOKMARKS, &sfile->systemnr, ICON_BOOKMARKS, 0);
+	}
+
+}
+
 static void file_panel_bookmarks(const bContext *C, Panel *pa)
 {
 	SpaceFile *sfile = CTX_wm_space_file(C);
 	uiLayout *row;
 
 	if (sfile) {
-		row = uiLayoutRow(pa->layout, FALSE);
+		row = uiLayoutRow(pa->layout, false);
 		uiItemO(row, IFACE_("Add"), ICON_ZOOMIN, "file.bookmark_add");
 		uiItemL(row, NULL, ICON_NONE);
 
@@ -149,14 +166,20 @@ static void file_panel_bookmarks(const bContext *C, Panel *pa)
 	}
 }
 
+static int file_panel_recent_poll(const bContext *C, PanelType *UNUSED(pt))
+{
+	SpaceFile *sfile = CTX_wm_space_file(C);
+	return (sfile && !(U.uiflag & USER_HIDE_RECENT));
+}
+
 static void file_panel_recent(const bContext *C, Panel *pa)
 {
 	SpaceFile *sfile = CTX_wm_space_file(C);
 	uiLayout *row;
 
 	if (sfile) {
-		if (!(U.uiflag & USER_HIDE_RECENT) ) {
-			row = uiLayoutRow(pa->layout, FALSE);
+		if (!(U.uiflag & USER_HIDE_RECENT)) {
+			row = uiLayoutRow(pa->layout, false);
 			uiItemO(row, IFACE_("Reset"), ICON_X, "file.reset_recent");
 			uiItemL(row, NULL, ICON_NONE);
 
@@ -180,12 +203,12 @@ static void file_panel_operator_header(const bContext *C, Panel *pa)
 	BLI_strncpy(pa->drawname, RNA_struct_ui_name(op->type->srna), sizeof(pa->drawname));
 }
 
-static int file_panel_check_prop(PointerRNA *UNUSED(ptr), PropertyRNA *prop)
+static bool file_panel_check_prop(PointerRNA *UNUSED(ptr), PropertyRNA *prop)
 {
 	const char *prop_id = RNA_property_identifier(prop);
-	return !(strcmp(prop_id, "filepath") == 0 ||
-	         strcmp(prop_id, "directory") == 0 ||
-	         strcmp(prop_id, "filename") == 0
+	return !(STREQ(prop_id, "filepath") ||
+	         STREQ(prop_id, "directory") ||
+	         STREQ(prop_id, "filename")
 	         );
 }
 
@@ -193,13 +216,13 @@ static void file_panel_operator(const bContext *C, Panel *pa)
 {
 	SpaceFile *sfile = CTX_wm_space_file(C);
 	wmOperator *op = sfile->op;
-	// int empty= 1, flag;
+	// int empty = 1, flag;
 	
-	uiBlockSetFunc(uiLayoutGetBlock(pa->layout), file_draw_check_cb, NULL, NULL);
+	UI_block_func_set(uiLayoutGetBlock(pa->layout), file_draw_check_cb, NULL, NULL);
 
 	uiLayoutOperatorButs(C, pa->layout, op, file_panel_check_prop, '\0', UI_LAYOUT_OP_SHOW_EMPTY);
 
-	uiBlockSetFunc(uiLayoutGetBlock(pa->layout), NULL, NULL, NULL);
+	UI_block_func_set(uiLayoutGetBlock(pa->layout), NULL, NULL, NULL);
 }
 
 void file_panels_register(ARegionType *art)
@@ -209,24 +232,37 @@ void file_panels_register(ARegionType *art)
 	pt = MEM_callocN(sizeof(PanelType), "spacetype file system directories");
 	strcpy(pt->idname, "FILE_PT_system");
 	strcpy(pt->label, N_("System"));
+	strcpy(pt->translation_context, BLF_I18NCONTEXT_DEFAULT_BPYRNA);
 	pt->draw = file_panel_system;
 	BLI_addtail(&art->paneltypes, pt);
 
+	pt = MEM_callocN(sizeof(PanelType), "spacetype file system bookmarks");
+	strcpy(pt->idname, "FILE_PT_system_bookmarks");
+	strcpy(pt->label, N_("System Bookmarks"));
+	strcpy(pt->translation_context, BLF_I18NCONTEXT_DEFAULT_BPYRNA);
+	pt->draw = file_panel_system_bookmarks;
+	pt->poll = file_panel_system_bookmarks_poll;
+	BLI_addtail(&art->paneltypes, pt);
+
 	pt = MEM_callocN(sizeof(PanelType), "spacetype file bookmarks");
 	strcpy(pt->idname, "FILE_PT_bookmarks");
 	strcpy(pt->label, N_("Bookmarks"));
+	strcpy(pt->translation_context, BLF_I18NCONTEXT_DEFAULT_BPYRNA);
 	pt->draw = file_panel_bookmarks;
 	BLI_addtail(&art->paneltypes, pt);
 
 	pt = MEM_callocN(sizeof(PanelType), "spacetype file recent directories");
 	strcpy(pt->idname, "FILE_PT_recent");
 	strcpy(pt->label, N_("Recent"));
+	strcpy(pt->translation_context, BLF_I18NCONTEXT_DEFAULT_BPYRNA);
 	pt->draw = file_panel_recent;
+	pt->poll = file_panel_recent_poll;
 	BLI_addtail(&art->paneltypes, pt);
 
 	pt = MEM_callocN(sizeof(PanelType), "spacetype file operator properties");
 	strcpy(pt->idname, "FILE_PT_operator");
 	strcpy(pt->label, N_("Operator"));
+	strcpy(pt->translation_context, BLF_I18NCONTEXT_DEFAULT_BPYRNA);
 	pt->poll = file_panel_operator_poll;
 	pt->draw_header = file_panel_operator_header;
 	pt->draw = file_panel_operator;
