@@ -46,7 +46,6 @@
 
 #include "BLI_blenlib.h"
 #include "BLI_linklist.h"
-#include "BLI_threads.h"
 #include "BLI_utildefines.h"
 #include "BLI_fileops_types.h"
 
@@ -94,8 +93,8 @@ typedef struct FileImage {
 
 typedef struct ThumbnailJob {
 	ListBase loadimages;
-	short *stop;
-	short *do_update;
+	const short *stop;
+	const short *do_update;
 	struct FileList *filelist;
 	ReportList reports;
 } ThumbnailJob;
@@ -299,21 +298,21 @@ static bool is_hidden_file(const char *filename, short hide_dot)
 
 	if (hide_dot) {
 		if (filename[0] == '.' && filename[1] != '.' && filename[1] != 0) {
-			is_hidden = 1; /* ignore .file */
+			is_hidden = true; /* ignore .file */
 		}
 		else if (((filename[0] == '.') && (filename[1] == 0))) {
-			is_hidden = 1; /* ignore . */
+			is_hidden = true; /* ignore . */
 		}
 		else {
 			int len = strlen(filename);
 			if ((len > 0) && (filename[len - 1] == '~')) {
-				is_hidden = 1;  /* ignore file~ */
+				is_hidden = true;  /* ignore file~ */
 			}
 		}
 	}
 	else {
 		if (((filename[0] == '.') && (filename[1] == 0))) {
-			is_hidden = 1; /* ignore . */
+			is_hidden = true; /* ignore . */
 		}
 	}
 	return is_hidden;
@@ -324,16 +323,16 @@ static bool is_filtered_file(struct direntry *file, const char *UNUSED(dir), uns
 	bool is_filtered = false;
 	if (filter) {
 		if (file->flags & filter) {
-			is_filtered = 1;
+			is_filtered = true;
 		}
 		else if (file->type & S_IFDIR) {
 			if (filter & FOLDERFILE) {
-				is_filtered = 1;
+				is_filtered = true;
 			}
 		}
 	}
 	else {
-		is_filtered = 1;
+		is_filtered = true;
 	}
 	return is_filtered && !is_hidden_file(file->relname, hide_dot);
 }
@@ -393,7 +392,7 @@ void filelist_init_icons(void)
 	ImBuf *bbuf;
 	ImBuf *ibuf;
 
-	BLI_assert(G.background == FALSE);
+	BLI_assert(G.background == false);
 
 #ifdef WITH_HEADLESS
 	bbuf = NULL;
@@ -421,7 +420,7 @@ void filelist_free_icons(void)
 {
 	int i;
 
-	BLI_assert(G.background == FALSE);
+	BLI_assert(G.background == false);
 
 	for (i = 0; i < SPECIAL_IMG_MAX; ++i) {
 		IMB_freeImBuf(gSpecialFileImages[i]);
@@ -624,7 +623,7 @@ ImBuf *filelist_getimage(struct FileList *filelist, int index)
 	ImBuf *ibuf = NULL;
 	int fidx = 0;
 
-	BLI_assert(G.background == FALSE);
+	BLI_assert(G.background == false);
 
 	if ((index < 0) || (index >= filelist->numfiltered)) {
 		return NULL;
@@ -641,7 +640,7 @@ ImBuf *filelist_geticon(struct FileList *filelist, int index)
 	struct direntry *file = NULL;
 	int fidx = 0;
 
-	BLI_assert(G.background == FALSE);
+	BLI_assert(G.background == false);
 
 	if ((index < 0) || (index >= filelist->numfiltered)) {
 		return NULL;
@@ -1097,7 +1096,7 @@ void filelist_from_library(struct FileList *filelist)
 	filelist->filelist[0].type |= S_IFDIR;
 		
 	for (i = 0, l = names; i < nnames; i++, l = l->next) {
-		char *blockname = l->link;
+		const char *blockname = l->link;
 
 		filelist->filelist[i + 1].relname = BLI_strdup(blockname);
 		if (idcode) {
@@ -1333,7 +1332,7 @@ static void thumbnails_startjob(void *tjv, short *stop, short *do_update, float
 				limg->flags |= MOVIEFILE_ICON;
 			}
 		}
-		*do_update = TRUE;
+		*do_update = true;
 		PIL_sleep_ms(10);
 		limg = limg->next;
 	}
@@ -1353,7 +1352,7 @@ static void thumbnails_update(void *tjv)
 					tj->filelist->filelist[limg->index].flags &= ~MOVIEFILE;
 					tj->filelist->filelist[limg->index].flags |= MOVIEFILE_ICON;
 				}
-				limg->done = TRUE;
+				limg->done = true;
 			}
 			limg = limg->next;
 		}
