@@ -37,12 +37,8 @@
 
 #include "MEM_guardedalloc.h"
 
-#include "DNA_space_types.h" /* FILE_MAX */
-
 #include "BLI_utildefines.h"
 #include "BLI_blenlib.h"
-#include "BLI_linklist.h"
-#include "BLI_dynstr.h"
 
 #ifdef WIN32
 #  include <windows.h> /* need to include windows.h so _WIN32_IE is defined  */
@@ -55,9 +51,7 @@
 #endif
 
 #ifdef __APPLE__
-/* XXX BIG WARNING: carbon.h can not be included in blender code, it conflicts with struct ID */
-#  define ID ID_
-#  include <CoreServices/CoreServices.h>
+#include <Carbon/Carbon.h>
 #endif /* __APPLE__ */
 
 #ifdef __linux__
@@ -80,9 +74,9 @@ struct _FSMenuEntry {
 
 typedef struct FSMenu {
 	FSMenuEntry *fsmenu_system;
+	FSMenuEntry *fsmenu_system_bookmarks;
 	FSMenuEntry *fsmenu_bookmarks;
 	FSMenuEntry *fsmenu_recent;
-
 } FSMenu;
 
 static FSMenu *g_fsmenu = NULL;
@@ -103,6 +97,9 @@ static FSMenuEntry *fsmenu_get_category(struct FSMenu *fsmenu, FSMenuCategory ca
 		case FS_CATEGORY_SYSTEM:
 			fsm_head = fsmenu->fsmenu_system;
 			break;
+		case FS_CATEGORY_SYSTEM_BOOKMARKS:
+			fsm_head = fsmenu->fsmenu_system_bookmarks;
+			break;
 		case FS_CATEGORY_BOOKMARKS:
 			fsm_head = fsmenu->fsmenu_bookmarks;
 			break;
@@ -119,6 +116,9 @@ static void fsmenu_set_category(struct FSMenu *fsmenu, FSMenuCategory category,
 		case FS_CATEGORY_SYSTEM:
 			fsmenu->fsmenu_system = fsm_head;
 			break;
+		case FS_CATEGORY_SYSTEM_BOOKMARKS:
+			fsmenu->fsmenu_system_bookmarks = fsm_head;
+			break;
 		case FS_CATEGORY_BOOKMARKS:
 			fsmenu->fsmenu_bookmarks = fsm_head;
 			break;
@@ -275,14 +275,14 @@ void fsmenu_write_file(struct FSMenu *fsmenu, const char *filename)
 
 void fsmenu_read_bookmarks(struct FSMenu *fsmenu, const char *filename)
 {
-	char line[256];
+	char line[FILE_MAXDIR];
 	FSMenuCategory category = FS_CATEGORY_BOOKMARKS;
 	FILE *fp;
 
 	fp = BLI_fopen(filename, "r");
 	if (!fp) return;
 
-	while (fgets(line, 256, fp) != NULL) {       /* read a line */
+	while (fgets(line, sizeof(line), fp) != NULL) {       /* read a line */
 		if (strncmp(line, "[Bookmarks]", 11) == 0) {
 			category = FS_CATEGORY_BOOKMARKS;
 		}
@@ -312,7 +312,7 @@ void fsmenu_read_bookmarks(struct FSMenu *fsmenu, const char *filename)
 
 void fsmenu_read_system(struct FSMenu *fsmenu, int read_bookmarks)
 {
-	char line[256];
+	char line[FILE_MAXDIR];
 #ifdef WIN32
 	/* Add the drive names to the listing */
 	{
@@ -336,15 +336,15 @@ void fsmenu_read_system(struct FSMenu *fsmenu, int read_bookmarks)
 		/* Adding Desktop and My Documents */
 		if (read_bookmarks) {
 			SHGetSpecialFolderPath(0, line, CSIDL_PERSONAL, 0);
-			fsmenu_insert_entry(fsmenu, FS_CATEGORY_BOOKMARKS, line, FS_INSERT_SORTED);
+			fsmenu_insert_entry(fsmenu, FS_CATEGORY_SYSTEM_BOOKMARKS, line, FS_INSERT_SORTED);
 			SHGetSpecialFolderPath(0, line, CSIDL_DESKTOPDIRECTORY, 0);
-			fsmenu_insert_entry(fsmenu, FS_CATEGORY_BOOKMARKS, line, FS_INSERT_SORTED);
+			fsmenu_insert_entry(fsmenu, FS_CATEGORY_SYSTEM_BOOKMARKS, line, FS_INSERT_SORTED);
 		}
 	}
 #else
 #ifdef __APPLE__
 	{
-#if (MAC_OS_X_VERSION_MIN_REQUIRED <= MAC_OS_X_VERSION_10_4)
+#if (MAC_OS_X_VERSION_MIN_REQUIRED <= 1050)
 		OSErr err = noErr;
 		int i;
 		const char *home;
@@ -367,94 +367,64 @@ void fsmenu_read_system(struct FSMenu *fsmenu, int read_bookmarks)
 
 		/* As 10.4 doesn't provide proper API to retrieve the favorite places,
 		 * assume they are the standard ones 
-		 * TODO : replace hardcoded paths with proper BLI_get_folder calls */
+		 * TODO : replace hardcoded paths with proper BKE_appdir_folder_id calls */
 		home = getenv("HOME");
 		if (read_bookmarks && home) {
-			BLI_snprintf(line, 256, "%s/", home);
-			fsmenu_insert_entry(fsmenu, FS_CATEGORY_BOOKMARKS, line, FS_INSERT_SORTED);
-			BLI_snprintf(line, 256, "%s/Desktop/", home);
+			BLI_snprintf(line, sizeof(line), "%s/", home);
+			fsmenu_insert_entry(fsmenu, FS_CATEGORY_SYSTEM_BOOKMARKS, line, FS_INSERT_SORTED);
+			BLI_snprintf(line, sizeof(line), "%s/Desktop/", home);
 			if (BLI_exists(line)) {
-				fsmenu_insert_entry(fsmenu, FS_CATEGORY_BOOKMARKS, line, FS_INSERT_SORTED);
+				fsmenu_insert_entry(fsmenu, FS_CATEGORY_SYSTEM_BOOKMARKS, line, FS_INSERT_SORTED);
 			}
-			BLI_snprintf(line, 256, "%s/Documents/", home);
+			BLI_snprintf(line, sizeof(line), "%s/Documents/", home);
 			if (BLI_exists(line)) {
-				fsmenu_insert_entry(fsmenu, FS_CATEGORY_BOOKMARKS, line, FS_INSERT_SORTED);
+				fsmenu_insert_entry(fsmenu, FS_CATEGORY_SYSTEM_BOOKMARKS, line, FS_INSERT_SORTED);
 			}
-			BLI_snprintf(line, 256, "%s/Pictures/", home);
+			BLI_snprintf(line, sizeof(line), "%s/Pictures/", home);
 			if (BLI_exists(line)) {
-				fsmenu_insert_entry(fsmenu, FS_CATEGORY_BOOKMARKS, line, FS_INSERT_SORTED);
+				fsmenu_insert_entry(fsmenu, FS_CATEGORY_SYSTEM_BOOKMARKS, line, FS_INSERT_SORTED);
 			}
-			BLI_snprintf(line, 256, "%s/Music/", home);
+			BLI_snprintf(line, sizeof(line), "%s/Music/", home);
 			if (BLI_exists(line)) {
-				fsmenu_insert_entry(fsmenu, FS_CATEGORY_BOOKMARKS, line, FS_INSERT_SORTED);
+				fsmenu_insert_entry(fsmenu, FS_CATEGORY_SYSTEM_BOOKMARKS, line, FS_INSERT_SORTED);
 			}
-			BLI_snprintf(line, 256, "%s/Movies/", home);
+			BLI_snprintf(line, sizeof(line), "%s/Movies/", home);
 			if (BLI_exists(line)) {
-				fsmenu_insert_entry(fsmenu, FS_CATEGORY_BOOKMARKS, line, FS_INSERT_SORTED);
+				fsmenu_insert_entry(fsmenu, FS_CATEGORY_SYSTEM_BOOKMARKS, line, FS_INSERT_SORTED);
 			}
 		}
-#else
-		/* 10.5 provides ability to retrieve Finder favorite places */
-		UInt32 seed;
-		OSErr err = noErr;
-		CFArrayRef pathesArray;
-		LSSharedFileListRef list;
-		LSSharedFileListItemRef itemRef;
-		CFIndex i, pathesCount;
+#else /* OSX 10.6+ */
+		/* Get mounted volumes better method OSX 10.6 and higher, see: */
+		/*https://developer.apple.com/library/mac/#documentation/CoreFOundation/Reference/CFURLRef/Reference/reference.html*/
+		/* we get all volumes sorted including network and do not relay on user-defined finder visibility, less confusing */
+		
 		CFURLRef cfURL = NULL;
-		CFStringRef pathString = NULL;
+		CFURLEnumeratorResult result = kCFURLEnumeratorSuccess;
+		CFURLEnumeratorRef volEnum = CFURLEnumeratorCreateForMountedVolumes(NULL, kCFURLEnumeratorSkipInvisibles, NULL);
 		
-		/* First get local mounted volumes */
-		list = LSSharedFileListCreate(NULL, kLSSharedFileListFavoriteVolumes, NULL);
-		pathesArray = LSSharedFileListCopySnapshot(list, &seed);
-		pathesCount = CFArrayGetCount(pathesArray);
-		
-		for (i = 0; i < pathesCount; i++) {
-			itemRef = (LSSharedFileListItemRef)CFArrayGetValueAtIndex(pathesArray, i);
-			
-			err = LSSharedFileListItemResolve(itemRef, 
-			                                  kLSSharedFileListNoUserInteraction |
-			                                  kLSSharedFileListDoNotMountVolumes,
-			                                  &cfURL, NULL);
-			if (err != noErr)
+		while (result != kCFURLEnumeratorEnd) {
+			unsigned char defPath[FILE_MAX];
+
+			result = CFURLEnumeratorGetNextURL(volEnum, &cfURL, NULL);
+			if (result != kCFURLEnumeratorSuccess)
 				continue;
 			
-			pathString = CFURLCopyFileSystemPath(cfURL, kCFURLPOSIXPathStyle);
-			
-			if (!CFStringGetCString(pathString, line, 256, kCFStringEncodingASCII))
-				continue;
-			fsmenu_insert_entry(fsmenu, FS_CATEGORY_SYSTEM, line, FS_INSERT_SORTED);
-			
-			CFRelease(pathString);
-			CFRelease(cfURL);
+			CFURLGetFileSystemRepresentation(cfURL, false, (UInt8 *)defPath, FILE_MAX);
+			fsmenu_insert_entry(fsmenu, FS_CATEGORY_SYSTEM, (char *)defPath, FS_INSERT_SORTED);
 		}
 		
-		CFRelease(pathesArray);
-		CFRelease(list);
-		
-		/* Then get network volumes */
-		err = noErr;
-		for (i = 1; err != nsvErr; i++) {
-			FSRef dir;
-			FSVolumeRefNum volRefNum;
-			struct GetVolParmsInfoBuffer volParmsBuffer;
-			unsigned char path[FILE_MAX];
-			
-			err = FSGetVolumeInfo(kFSInvalidVolumeRefNum, i, &volRefNum, kFSVolInfoNone, NULL, NULL, &dir);
-			if (err != noErr)
-				continue;
-			
-			err = FSGetVolumeParms(volRefNum, &volParmsBuffer, sizeof(volParmsBuffer));
-			if ((err != noErr) || (volParmsBuffer.vMServerAdr == 0)) /* Exclude local devices */
-				continue;
-			
-			
-			FSRefMakePath(&dir, path, FILE_MAX);
-			fsmenu_insert_entry(fsmenu, FS_CATEGORY_SYSTEM, (char *)path, FS_INSERT_SORTED);
-		}
+		CFRelease(volEnum);
 		
 		/* Finally get user favorite places */
 		if (read_bookmarks) {
+			UInt32 seed;
+			OSErr err = noErr;
+			CFArrayRef pathesArray;
+			LSSharedFileListRef list;
+			LSSharedFileListItemRef itemRef;
+			CFIndex i, pathesCount;
+			CFURLRef cfURL = NULL;
+			CFStringRef pathString = NULL;
 			list = LSSharedFileListCreate(NULL, kLSSharedFileListFavoriteItems, NULL);
 			pathesArray = LSSharedFileListCopySnapshot(list, &seed);
 			pathesCount = CFArrayGetCount(pathesArray);
@@ -471,9 +441,14 @@ void fsmenu_read_system(struct FSMenu *fsmenu, int read_bookmarks)
 				
 				pathString = CFURLCopyFileSystemPath(cfURL, kCFURLPOSIXPathStyle);
 				
-				if (!CFStringGetCString(pathString, line, 256, kCFStringEncodingASCII))
+				if (pathString == NULL || !CFStringGetCString(pathString, line, sizeof(line), kCFStringEncodingASCII))
 					continue;
-				fsmenu_insert_entry(fsmenu, FS_CATEGORY_BOOKMARKS, line, FS_INSERT_SORTED);
+
+				/* Exclude "all my files" as it makes no sense in blender fileselector */
+				/* Exclude "airdrop" if wlan not active as it would show "" ) */
+				if (!strstr(line, "myDocuments.cannedSearch") && (*line != '\0')) {
+					fsmenu_insert_entry(fsmenu, FS_CATEGORY_SYSTEM_BOOKMARKS, line, FS_APPEND_LAST);
+				}
 				
 				CFRelease(pathString);
 				CFRelease(cfURL);
@@ -482,7 +457,7 @@ void fsmenu_read_system(struct FSMenu *fsmenu, int read_bookmarks)
 			CFRelease(pathesArray);
 			CFRelease(list);
 		}
-#endif /* OSX 10.5+ */
+#endif /* OSX 10.6+ */
 	}
 #else
 	/* unix */
@@ -490,11 +465,11 @@ void fsmenu_read_system(struct FSMenu *fsmenu, int read_bookmarks)
 		const char *home = getenv("HOME");
 
 		if (read_bookmarks && home) {
-			BLI_snprintf(line, FILE_MAXDIR, "%s/", home);
-			fsmenu_insert_entry(fsmenu, FS_CATEGORY_BOOKMARKS, line, FS_INSERT_SORTED);
-			BLI_snprintf(line, FILE_MAXDIR, "%s/Desktop/", home);
+			BLI_snprintf(line, sizeof(line), "%s/", home);
+			fsmenu_insert_entry(fsmenu, FS_CATEGORY_SYSTEM_BOOKMARKS, line, FS_INSERT_SORTED);
+			BLI_snprintf(line, sizeof(line), "%s/Desktop/", home);
 			if (BLI_exists(line)) {
-				fsmenu_insert_entry(fsmenu, FS_CATEGORY_BOOKMARKS, line, FS_INSERT_SORTED);
+				fsmenu_insert_entry(fsmenu, FS_CATEGORY_SYSTEM_BOOKMARKS, line, FS_INSERT_SORTED);
 			}
 		}
 
@@ -518,7 +493,7 @@ void fsmenu_read_system(struct FSMenu *fsmenu, int read_bookmarks)
 
 					len = strlen(mnt->mnt_dir);
 					if (len && mnt->mnt_dir[len - 1] != '/') {
-						BLI_snprintf(line, FILE_MAXDIR, "%s/", mnt->mnt_dir);
+						BLI_snprintf(line, sizeof(line), "%s/", mnt->mnt_dir);
 						fsmenu_insert_entry(fsmenu, FS_CATEGORY_SYSTEM, line, FS_INSERT_SORTED);
 					}
 					else {
@@ -564,14 +539,23 @@ void fsmenu_refresh_system_category(struct FSMenu *fsmenu)
 	fsmenu_free_category(fsmenu, FS_CATEGORY_SYSTEM);
 	fsmenu_set_category(fsmenu, FS_CATEGORY_SYSTEM, NULL);
 
+	fsmenu_free_category(fsmenu, FS_CATEGORY_SYSTEM_BOOKMARKS);
+	fsmenu_set_category(fsmenu, FS_CATEGORY_SYSTEM_BOOKMARKS, NULL);
+
 	/* Add all entries to system category */
-	fsmenu_read_system(fsmenu, FALSE);
+	fsmenu_read_system(fsmenu, true);
 }
 
-void fsmenu_free(struct FSMenu *fsmenu)
+void fsmenu_free(void)
 {
-	fsmenu_free_category(fsmenu, FS_CATEGORY_SYSTEM);
-	fsmenu_free_category(fsmenu, FS_CATEGORY_BOOKMARKS);
-	fsmenu_free_category(fsmenu, FS_CATEGORY_RECENT);
-	MEM_freeN(fsmenu);
+	if (g_fsmenu) {
+		fsmenu_free_category(g_fsmenu, FS_CATEGORY_SYSTEM);
+		fsmenu_free_category(g_fsmenu, FS_CATEGORY_SYSTEM_BOOKMARKS);
+		fsmenu_free_category(g_fsmenu, FS_CATEGORY_BOOKMARKS);
+		fsmenu_free_category(g_fsmenu, FS_CATEGORY_RECENT);
+		MEM_freeN(g_fsmenu);
+	}
+
+	g_fsmenu = NULL;
 }
+
