@@ -28,28 +28,28 @@
  *  \ingroup spfile
  */
 
-
 #include <string.h>
 #include <stdio.h>
 
-
-#include "RNA_access.h"
-
 #include "MEM_guardedalloc.h"
 
 #include "BIF_gl.h"
 
-#include "BLO_readfile.h"
-
 #include "BLI_blenlib.h"
-#include "BLI_math.h"
-#include "BLI_rand.h"
 #include "BLI_utildefines.h"
+#include "BLI_fileops_types.h"
 
+
+#include "BKE_appdir.h"
 #include "BKE_context.h"
 #include "BKE_screen.h"
 #include "BKE_global.h"
 
+#include "RNA_access.h"
+
+#include "WM_api.h"
+#include "WM_types.h"
+
 #include "ED_space_api.h"
 #include "ED_screen.h"
 #include "ED_fileselect.h"
@@ -57,9 +57,6 @@
 #include "IMB_imbuf_types.h"
 #include "IMB_thumbs.h"
 
-#include "WM_api.h"
-#include "WM_types.h"
-
 #include "UI_resources.h"
 #include "UI_view2d.h"
 
@@ -147,17 +144,22 @@ static void file_free(SpaceLink *sl)
 
 
 /* spacetype; init callback, area size changes, screen set, etc */
-static void file_init(struct wmWindowManager *UNUSED(wm), ScrArea *sa)
+static void file_init(wmWindowManager *UNUSED(wm), ScrArea *sa)
 {
 	SpaceFile *sfile = (SpaceFile *)sa->spacedata.first;
-	//printf("file_init\n");
 
 	/* refresh system directory list */
 	fsmenu_refresh_system_category(fsmenu_get());
 
-	if (sfile->layout) sfile->layout->dirty = TRUE;
+	if (sfile->layout) sfile->layout->dirty = true;
 }
 
+static void file_exit(wmWindowManager *wm, ScrArea *sa)
+{
+	SpaceFile *sfile = (SpaceFile *)sa->spacedata.first;
+
+	ED_fileselect_exit(wm, sfile);
+}
 
 static SpaceLink *file_duplicate(SpaceLink *sl)
 {
@@ -187,6 +189,7 @@ static SpaceLink *file_duplicate(SpaceLink *sl)
 
 static void file_refresh(const bContext *C, ScrArea *UNUSED(sa))
 {
+	wmWindowManager *wm = CTX_wm_manager(C);
 	SpaceFile *sfile = CTX_wm_space_file(C);
 	FileSelectParams *params = ED_fileselect_get_params(sfile);
 
@@ -198,11 +201,11 @@ static void file_refresh(const bContext *C, ScrArea *UNUSED(sa))
 		params->active_file = -1; // added this so it opens nicer (ton)
 	}
 	filelist_hidedot(sfile->files, params->flag & FILE_HIDE_DOT);
-	filelist_setfilter(sfile->files, params->flag & FILE_FILTER ? params->filter : 0);	
+	filelist_setfilter(sfile->files, params->flag & FILE_FILTER ? params->filter : 0);
 	filelist_setfilter_types(sfile->files, params->filter_glob);
 
 	if (filelist_empty(sfile->files)) {
-		thumbnails_stop(sfile->files, C);
+		thumbnails_stop(wm, sfile->files);
 		filelist_readdir(sfile->files);
 		if (params->sort != FILE_SORT_NONE) {
 			filelist_sort(sfile->files, params->sort);
@@ -214,7 +217,7 @@ static void file_refresh(const bContext *C, ScrArea *UNUSED(sa))
 	}
 	else {
 		if (params->sort != FILE_SORT_NONE) {
-			thumbnails_stop(sfile->files, C);
+			thumbnails_stop(wm, sfile->files);
 			filelist_sort(sfile->files, params->sort);
 			if (params->display == FILE_IMGDISPLAY) {
 				thumbnails_start(sfile->files, C);
@@ -222,14 +225,14 @@ static void file_refresh(const bContext *C, ScrArea *UNUSED(sa))
 		}
 		else {
 			if (params->display == FILE_IMGDISPLAY) {
-				if (!thumbnails_running(sfile->files, C)) {
+				if (!thumbnails_running(wm, sfile->files)) {
 					thumbnails_start(sfile->files, C);
 				}
 			}
 			else {
 				/* stop any running thumbnail jobs if we're not 
 				 * displaying them - speedup for NFS */
-				thumbnails_stop(sfile->files, C);
+				thumbnails_stop(wm, sfile->files);
 			}
 			filelist_filter(sfile->files);
 		}
@@ -246,11 +249,11 @@ static void file_refresh(const bContext *C, ScrArea *UNUSED(sa))
 		BLI_strncpy(sfile->params->renameedit, sfile->params->renamefile, sizeof(sfile->params->renameedit));
 		params->renamefile[0] = '\0';
 	}
-	if (sfile->layout) sfile->layout->dirty = TRUE;
+	if (sfile->layout) sfile->layout->dirty = true;
 
 }
 
-static void file_listener(ScrArea *sa, wmNotifier *wmn)
+static void file_listener(bScreen *UNUSED(sc), ScrArea *sa, wmNotifier *wmn)
 {
 	/* SpaceFile *sfile = (SpaceFile *)sa->spacedata.first; */
 
@@ -286,7 +289,7 @@ static void file_main_area_init(wmWindowManager *wm, ARegion *ar)
 	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
 }
 
-static void file_main_area_listener(ARegion *ar, wmNotifier *wmn)
+static void file_main_area_listener(bScreen *UNUSED(sc), ScrArea *UNUSED(sa), ARegion *ar, wmNotifier *wmn)
 {
 	/* context changes */
 	switch (wmn->category) {
@@ -308,7 +311,6 @@ static void file_main_area_draw(const bContext *C, ARegion *ar)
 	/* draw entirely, view changes should be handled here */
 	SpaceFile *sfile = CTX_wm_space_file(C);
 	FileSelectParams *params = ED_fileselect_get_params(sfile);
-	FileLayout *layout = NULL;
 
 	View2D *v2d = &ar->v2d;
 	View2DScrollers *scrollers;
@@ -318,15 +320,14 @@ static void file_main_area_draw(const bContext *C, ARegion *ar)
 	if (!sfile->files || filelist_empty(sfile->files))
 		file_refresh(C, NULL);
 
-	layout = ED_fileselect_get_layout(sfile, ar);
-
 	/* clear and setup matrix */
 	UI_GetThemeColor3fv(TH_BACK, col);
 	glClearColor(col[0], col[1], col[2], 0.0);
 	glClear(GL_COLOR_BUFFER_BIT);
 	
 	/* Allow dynamically sliders to be set, saves notifiers etc. */
-	if (layout && (layout->flag == FILE_LAYOUT_VER)) {
+	
+	if (params->display == FILE_IMGDISPLAY) {
 		v2d->scroll = V2D_SCROLL_RIGHT;
 		v2d->keepofs &= ~V2D_LOCKOFS_Y;
 		v2d->keepofs |= V2D_LOCKOFS_X;
@@ -385,7 +386,7 @@ static void file_operatortypes(void)
 	WM_operatortype_append(FILE_OT_refresh);
 	WM_operatortype_append(FILE_OT_bookmark_toggle);
 	WM_operatortype_append(FILE_OT_bookmark_add);
-	WM_operatortype_append(FILE_OT_delete_bookmark);
+	WM_operatortype_append(FILE_OT_bookmark_delete);
 	WM_operatortype_append(FILE_OT_reset_recent);
 	WM_operatortype_append(FILE_OT_hidedot);
 	WM_operatortype_append(FILE_OT_filenum);
@@ -393,7 +394,6 @@ static void file_operatortypes(void)
 	WM_operatortype_append(FILE_OT_delete);
 	WM_operatortype_append(FILE_OT_rename);
 	WM_operatortype_append(FILE_OT_smoothscroll);
-	WM_operatortype_append(FILE_OT_directory);
 }
 
 /* NOTE: do not add .blend file reading on this level */
@@ -402,13 +402,13 @@ static void file_keymap(struct wmKeyConfig *keyconf)
 	wmKeyMapItem *kmi;
 	/* keys for all areas */
 	wmKeyMap *keymap = WM_keymap_find(keyconf, "File Browser", SPACE_FILE, 0);
-	WM_keymap_add_item(keymap, "FILE_OT_bookmark_toggle", NKEY, KM_PRESS, 0, 0);
+	WM_keymap_add_item(keymap, "FILE_OT_bookmark_toggle", TKEY, KM_PRESS, 0, 0);
 	WM_keymap_add_item(keymap, "FILE_OT_parent", PKEY, KM_PRESS, 0, 0);
 	WM_keymap_add_item(keymap, "FILE_OT_bookmark_add", BKEY, KM_PRESS, KM_CTRL, 0);
 	WM_keymap_add_item(keymap, "FILE_OT_hidedot", HKEY, KM_PRESS, 0, 0);
 	WM_keymap_add_item(keymap, "FILE_OT_previous", BACKSPACEKEY, KM_PRESS, 0, 0);
 	WM_keymap_add_item(keymap, "FILE_OT_next", BACKSPACEKEY, KM_PRESS, KM_SHIFT, 0);
-	WM_keymap_add_item(keymap, "FILE_OT_directory_new", IKEY, KM_PRESS, 0, 0);  /* XXX needs button */
+	WM_keymap_add_item(keymap, "FILE_OT_directory_new", IKEY, KM_PRESS, 0, 0);
 	WM_keymap_add_item(keymap, "FILE_OT_delete", XKEY, KM_PRESS, 0, 0);
 	WM_keymap_add_item(keymap, "FILE_OT_delete", DELKEY, KM_PRESS, 0, 0);
 	WM_keymap_verify_item(keymap, "FILE_OT_smoothscroll", TIMER1, KM_ANY, KM_ANY, 0);
@@ -416,26 +416,30 @@ static void file_keymap(struct wmKeyConfig *keyconf)
 	/* keys for main area */
 	keymap = WM_keymap_find(keyconf, "File Browser Main", SPACE_FILE, 0);
 	kmi = WM_keymap_add_item(keymap, "FILE_OT_execute", LEFTMOUSE, KM_DBL_CLICK, 0, 0);
-	RNA_boolean_set(kmi->ptr, "need_active", TRUE);
+	RNA_boolean_set(kmi->ptr, "need_active", true);
 
 	/* left mouse selects and opens */
 	WM_keymap_add_item(keymap, "FILE_OT_select", LEFTMOUSE, KM_CLICK, 0, 0);
 	kmi = WM_keymap_add_item(keymap, "FILE_OT_select", LEFTMOUSE, KM_CLICK, KM_SHIFT, 0);
-	RNA_boolean_set(kmi->ptr, "extend", TRUE);
-	kmi = WM_keymap_add_item(keymap, "FILE_OT_select", LEFTMOUSE, KM_CLICK, KM_ALT, 0);
-	RNA_boolean_set(kmi->ptr, "extend", TRUE);
-	RNA_boolean_set(kmi->ptr, "fill", TRUE);
+	RNA_boolean_set(kmi->ptr, "extend", true);
+	kmi = WM_keymap_add_item(keymap, "FILE_OT_select", LEFTMOUSE, KM_CLICK, KM_CTRL | KM_SHIFT, 0);
+	RNA_boolean_set(kmi->ptr, "extend", true);
+	RNA_boolean_set(kmi->ptr, "fill", true);
 
 	/* right mouse selects without opening */
 	kmi = WM_keymap_add_item(keymap, "FILE_OT_select", RIGHTMOUSE, KM_CLICK, 0, 0);
-	RNA_boolean_set(kmi->ptr, "open", FALSE);
+	RNA_boolean_set(kmi->ptr, "open", false);
 	kmi = WM_keymap_add_item(keymap, "FILE_OT_select", RIGHTMOUSE, KM_CLICK, KM_SHIFT, 0);
-	RNA_boolean_set(kmi->ptr, "extend", TRUE);
-	RNA_boolean_set(kmi->ptr, "open", FALSE);
+	RNA_boolean_set(kmi->ptr, "extend", true);
+	RNA_boolean_set(kmi->ptr, "open", false);
 	kmi = WM_keymap_add_item(keymap, "FILE_OT_select", RIGHTMOUSE, KM_CLICK, KM_ALT, 0);
-	RNA_boolean_set(kmi->ptr, "extend", TRUE);
-	RNA_boolean_set(kmi->ptr, "fill", TRUE);
-	RNA_boolean_set(kmi->ptr, "open", FALSE);
+	RNA_boolean_set(kmi->ptr, "extend", true);
+	RNA_boolean_set(kmi->ptr, "fill", true);
+	RNA_boolean_set(kmi->ptr, "open", false);
+
+	/* front and back mouse folder navigation */
+	WM_keymap_add_item(keymap, "FILE_OT_previous", BUTTON4MOUSE, KM_CLICK, 0, 0);
+	WM_keymap_add_item(keymap, "FILE_OT_next", BUTTON5MOUSE, KM_CLICK, 0, 0);
 
 	WM_keymap_add_item(keymap, "FILE_OT_select_all_toggle", AKEY, KM_PRESS, 0, 0);
 	WM_keymap_add_item(keymap, "FILE_OT_refresh", PADPERIOD, KM_PRESS, 0, 0);
@@ -478,10 +482,11 @@ static void file_channel_area_init(wmWindowManager *wm, ARegion *ar)
 {
 	wmKeyMap *keymap;
 
+	ar->v2d.scroll = V2D_SCROLL_RIGHT | V2D_SCROLL_VERTICAL_HIDE;
 	ED_region_panels_init(wm, ar);
 
 	/* own keymaps */
-	keymap = WM_keymap_find(wm->defaultconf, "File Browser", SPACE_FILE, 0);	
+	keymap = WM_keymap_find(wm->defaultconf, "File Browser", SPACE_FILE, 0);
 	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
 }
 
@@ -490,12 +495,14 @@ static void file_channel_area_draw(const bContext *C, ARegion *ar)
 	ED_region_panels(C, ar, 1, NULL, -1);
 }
 
-static void file_channel_area_listener(ARegion *UNUSED(ar), wmNotifier *wmn)
+static void file_channel_area_listener(bScreen *UNUSED(sc), ScrArea *UNUSED(sa), ARegion *UNUSED(ar), wmNotifier *UNUSED(wmn))
 {
+#if 0
 	/* context changes */
 	switch (wmn->category) {
 		
 	}
+#endif
 }
 
 /* add handlers, stuff you only do once or on area/region changes */
@@ -505,7 +512,7 @@ static void file_header_area_init(wmWindowManager *wm, ARegion *ar)
 	
 	ED_region_header_init(ar);
 	
-	keymap = WM_keymap_find(wm->defaultconf, "File Browser", SPACE_FILE, 0);	
+	keymap = WM_keymap_find(wm->defaultconf, "File Browser", SPACE_FILE, 0);
 	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
 }
 
@@ -533,7 +540,7 @@ static void file_ui_area_draw(const bContext *C, ARegion *ar)
 {
 	float col[3];
 	/* clear */
-	UI_GetThemeColor3fv(TH_PANEL, col);
+	UI_GetThemeColor3fv(TH_BACK, col);
 	glClearColor(col[0], col[1], col[2], 0.0);
 	glClear(GL_COLOR_BUFFER_BIT);
 
@@ -550,7 +557,7 @@ static void file_ui_area_draw(const bContext *C, ARegion *ar)
 	UI_view2d_view_restore(C);
 }
 
-static void file_ui_area_listener(ARegion *ar, wmNotifier *wmn)
+static void file_ui_area_listener(bScreen *UNUSED(sc), ScrArea *UNUSED(sa), ARegion *ar, wmNotifier *wmn)
 {
 	/* context changes */
 	switch (wmn->category) {
@@ -576,6 +583,7 @@ void ED_spacetype_file(void)
 	st->new = file_new;
 	st->free = file_free;
 	st->init = file_init;
+	st->exit = file_exit;
 	st->duplicate = file_duplicate;
 	st->refresh = file_refresh;
 	st->listener = file_listener;
@@ -598,7 +606,7 @@ void ED_spacetype_file(void)
 	art->keymapflag = ED_KEYMAP_UI | ED_KEYMAP_VIEW2D | ED_KEYMAP_HEADER;
 	art->init = file_header_area_init;
 	art->draw = file_header_area_draw;
-	// art->listener= file_header_area_listener;
+	// art->listener = file_header_area_listener;
 	BLI_addhead(&st->regiontypes, art);
 	
 	/* regions: ui */
@@ -628,17 +636,9 @@ void ED_spacetype_file(void)
 
 void ED_file_init(void)
 {
-	char *cfgdir = BLI_get_folder(BLENDER_USER_CONFIG, NULL);
-	
-	fsmenu_read_system(fsmenu_get(), TRUE);
+	ED_file_read_bookmarks();
 
-	if (cfgdir) {
-		char name[FILE_MAX];
-		BLI_make_file_string("/", name, cfgdir, BLENDER_BOOKMARK_FILE);
-		fsmenu_read_bookmarks(fsmenu_get(), name);
-	}
-	
-	if (G.background == FALSE) {
+	if (G.background == false) {
 		filelist_init_icons();
 	}
 
@@ -647,9 +647,25 @@ void ED_file_init(void)
 
 void ED_file_exit(void)
 {
-	fsmenu_free(fsmenu_get());
+	fsmenu_free();
 
-	if (G.background == FALSE) {
+	if (G.background == false) {
 		filelist_free_icons();
 	}
 }
+
+void ED_file_read_bookmarks(void)
+{
+	const char * const cfgdir = BKE_appdir_folder_id(BLENDER_USER_CONFIG, NULL);
+	
+	fsmenu_free();
+
+	fsmenu_read_system(fsmenu_get(), true);
+
+	if (cfgdir) {
+		char name[FILE_MAX];
+		BLI_make_file_string("/", name, cfgdir, BLENDER_BOOKMARK_FILE);
+		fsmenu_read_bookmarks(fsmenu_get(), name);
+	}
+}
+
