@@ -32,19 +32,11 @@
 #include <string.h>
 #include <float.h>
 
-#ifndef _WIN32
-#include <unistd.h>
-#else
-#include <io.h>
-#endif
-
-
 #include "BLI_blenlib.h"
 #include "BLI_math.h"
 #include "BLI_utildefines.h"
 
 #include "DNA_anim_types.h"
-#include "DNA_object_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_space_types.h"
 #include "DNA_windowmanager_types.h"
@@ -83,7 +75,7 @@ static float fcurve_display_alpha(FCurve *fcu)
 
 /* Envelope -------------- */
 
-// TODO: draw a shaded poly showing the region of influence too!!!
+/* TODO: draw a shaded poly showing the region of influence too!!! */
 static void draw_fcurve_modifier_controls_envelope(FModifier *fcm, View2D *v2d)
 {
 	FMod_Envelope *env = (FMod_Envelope *)fcm->data;
@@ -101,13 +93,13 @@ static void draw_fcurve_modifier_controls_envelope(FModifier *fcm, View2D *v2d)
 		
 	glVertex2f(v2d->cur.xmin, env->midval + env->max);
 	glVertex2f(v2d->cur.xmax, env->midval + env->max);
-	glEnd(); // GL_LINES
+	glEnd();  /* GL_LINES */
 	setlinestyle(0);
 	
 	/* set size of vertices (non-adjustable for now) */
 	glPointSize(2.0f);
 	
-	// for now, point color is fixed, and is white
+	/* for now, point color is fixed, and is white */
 	glColor3f(1.0f, 1.0f, 1.0f);
 	
 	/* we use bgl points not standard gl points, to workaround vertex 
@@ -123,7 +115,7 @@ static void draw_fcurve_modifier_controls_envelope(FModifier *fcm, View2D *v2d)
 			glVertex2f(fed->time, fed->max);
 		}
 	}
-	bglEnd(); // GL_POINTS
+	bglEnd();  /* GL_POINTS */
 	
 	glPointSize(1.0f);
 }
@@ -160,13 +152,13 @@ static void draw_fcurve_vertices_keyframes(FCurve *fcu, SpaceIpo *UNUSED(sipo),
 			}
 			else {
 				/* no check for selection here, as curve is not editable... */
-				// XXX perhaps we don't want to even draw points?   maybe add an option for that later
+				/* XXX perhaps we don't want to even draw points?   maybe add an option for that later */
 				bglVertex3fv(bezt->vec[1]);
 			}
 		}
 	}
 	
-	bglEnd(); // GL_POINTS
+	bglEnd();  /* GL_POINTS */
 }
 
 
@@ -206,7 +198,7 @@ static void draw_fcurve_handle_control(float x, float y, float xscale, float ysc
 }
 
 /* helper func - draw handle vertices only for an F-Curve (if it is not protected) */
-static void draw_fcurve_vertices_handles(FCurve *fcu, SpaceIpo *sipo, View2D *v2d, short sel, short sel_handle_only)
+static void draw_fcurve_vertices_handles(FCurve *fcu, SpaceIpo *sipo, View2D *v2d, short sel, short sel_handle_only, float units_scale)
 {
 	BezTriple *bezt = fcu->bezt;
 	BezTriple *prevbezt = NULL;
@@ -214,8 +206,11 @@ static void draw_fcurve_vertices_handles(FCurve *fcu, SpaceIpo *sipo, View2D *v2
 	int i;
 	
 	/* get view settings */
-	hsize = UI_GetThemeValuef(TH_HANDLE_VERTEX_SIZE);
-	UI_view2d_getscale(v2d, &xscale, &yscale);
+	hsize = UI_GetThemeValuef(TH_HANDLE_VERTEX_SIZE) * U.pixelsize;
+	UI_view2d_scale_get(v2d, &xscale, &yscale);
+
+	/* Compensate OGL scale sued for unit mapping, so circle will be circle, not ellipse */
+	yscale *= units_scale;
 	
 	/* set handle color */
 	if (sel) UI_ThemeColor(TH_HANDLE_VERTEX_SELECT);
@@ -234,7 +229,7 @@ static void draw_fcurve_vertices_handles(FCurve *fcu, SpaceIpo *sipo, View2D *v2
 		 * Also, need to take into account whether the keyframe was selected
 		 * if a Graph Editor option to only show handles of selected keys is on.
 		 */
-		if (!sel_handle_only || BEZSELECTED(bezt) ) {
+		if (!sel_handle_only || BEZSELECTED(bezt)) {
 			if ((!prevbezt && (bezt->ipo == BEZT_IPO_BEZ)) || (prevbezt && (prevbezt->ipo == BEZT_IPO_BEZ))) {
 				if ((bezt->f1 & SELECT) == sel) /* && v2d->cur.xmin < bezt->vec[0][0] < v2d->cur.xmax)*/
 					draw_fcurve_handle_control(bezt->vec[0][0], bezt->vec[0][1], xscale, yscale, hsize);
@@ -262,7 +257,7 @@ static void set_fcurve_vertex_color(FCurve *fcu, short sel)
 		/* Curve's points ARE BEING edited */
 		if (sel) UI_ThemeColorShadeAlpha(TH_VERTEX_SELECT, 0, alphaOffset); 
 		else UI_ThemeColorShadeAlpha(TH_VERTEX, 0, alphaOffset);
-	} 
+	}
 	else {
 		/* Curve's points CANNOT BE edited */
 		if (sel) UI_ThemeColorShadeAlpha(TH_TEXT_HI, 0, alphaOffset);
@@ -271,7 +266,7 @@ static void set_fcurve_vertex_color(FCurve *fcu, short sel)
 }
 
 
-static void draw_fcurve_vertices(SpaceIpo *sipo, ARegion *ar, FCurve *fcu, short do_handles, short sel_handle_only)
+static void draw_fcurve_vertices(SpaceIpo *sipo, ARegion *ar, FCurve *fcu, short do_handles, short sel_handle_only, float units_scale)
 {
 	View2D *v2d = &ar->v2d;
 	
@@ -287,10 +282,10 @@ static void draw_fcurve_vertices(SpaceIpo *sipo, ARegion *ar, FCurve *fcu, short
 	/* draw the two handles first (if they're shown, the curve doesn't have just a single keyframe, and the curve is being edited) */
 	if (do_handles) {
 		set_fcurve_vertex_color(fcu, 0);
-		draw_fcurve_vertices_handles(fcu, sipo, v2d, 0, sel_handle_only);
+		draw_fcurve_vertices_handles(fcu, sipo, v2d, 0, sel_handle_only, units_scale);
 		
 		set_fcurve_vertex_color(fcu, 1);
-		draw_fcurve_vertices_handles(fcu, sipo, v2d, 1, sel_handle_only);
+		draw_fcurve_vertices_handles(fcu, sipo, v2d, 1, sel_handle_only, units_scale);
 	}
 		
 	/* draw keyframes over the handles */
@@ -305,7 +300,7 @@ static void draw_fcurve_vertices(SpaceIpo *sipo, ARegion *ar, FCurve *fcu, short
 
 /* Handles ---------------- */
 
-static int draw_fcurve_handles_check(SpaceIpo *sipo, FCurve *fcu)
+static bool draw_fcurve_handles_check(SpaceIpo *sipo, FCurve *fcu)
 {
 	/* don't draw handle lines if handles are not to be shown */
 	if (    (sipo->flag & SIPO_NOHANDLES) || /* handles shouldn't be shown anywhere */
@@ -318,7 +313,7 @@ static int draw_fcurve_handles_check(SpaceIpo *sipo, FCurve *fcu)
 	        )
 	{
 		return 0;
-	} 
+	}
 	else {
 		return 1;
 	}
@@ -342,7 +337,7 @@ static void draw_fcurve_handles(SpaceIpo *sipo, FCurve *fcu)
 	for (sel = 0; sel < 2; sel++) {
 		BezTriple *bezt = fcu->bezt, *prevbezt = NULL;
 		int basecol = (sel) ? TH_HANDLE_SEL_FREE : TH_HANDLE_FREE;
-		float *fp;
+		const float *fp;
 		unsigned char col[4];
 		
 		for (b = 0; b < fcu->totvert; b++, prevbezt = bezt, bezt++) {
@@ -404,7 +399,7 @@ static void draw_fcurve_handles(SpaceIpo *sipo, FCurve *fcu)
 		}
 	}
 	
-	glEnd(); // GL_LINES 
+	glEnd();  /* GL_LINES */
 }
 
 /* Samples ---------------- */
@@ -428,7 +423,7 @@ static void draw_fcurve_sample_control(float x, float y, float xscale, float ysc
 			
 		glVertex2f(-0.7f, +0.7f);
 		glVertex2f(+0.7f, -0.7f);
-		glEnd(); // GL_LINES
+		glEnd();  /* GL_LINES */
 		
 		glEndList();
 	}
@@ -453,7 +448,7 @@ static void draw_fcurve_samples(SpaceIpo *sipo, ARegion *ar, FCurve *fcu)
 	
 	/* get view settings */
 	hsize = UI_GetThemeValuef(TH_VERTEX_SIZE);
-	UI_view2d_getscale(&ar->v2d, &xscale, &yscale);
+	UI_view2d_scale_get(&ar->v2d, &xscale, &yscale);
 	
 	/* set vertex color */
 	if (fcu->flag & (FCURVE_ACTIVE | FCURVE_SELECTED)) UI_ThemeColor(TH_TEXT_HI);
@@ -482,11 +477,14 @@ static void draw_fcurve_samples(SpaceIpo *sipo, ARegion *ar, FCurve *fcu)
 /* helper func - just draw the F-Curve by sampling the visible region (for drawing curves with modifiers) */
 static void draw_fcurve_curve(bAnimContext *ac, ID *id, FCurve *fcu, View2D *v2d, View2DGrid *grid)
 {
+	SpaceIpo *sipo = (SpaceIpo *)ac->sl;
 	ChannelDriver *driver;
-	float samplefreq, ctime;
+	float samplefreq;
 	float stime, etime;
 	float unitFac;
 	float dx, dy;
+	short mapping_flag = ANIM_get_normalization_flags(ac);
+	int i, n;
 
 	/* when opening a blend file on a different sized screen or while dragging the toolbar this can happen
 	 * best just bail out in this case */
@@ -500,14 +498,14 @@ static void draw_fcurve_curve(bAnimContext *ac, ID *id, FCurve *fcu, View2D *v2d
 	fcu->driver = NULL;
 	
 	/* compute unit correction factor */
-	unitFac = ANIM_unit_mapping_get_factor(ac->scene, id, fcu, 0);
+	unitFac = ANIM_unit_mapping_get_factor(ac->scene, id, fcu, mapping_flag);
 	
 	/* Note about sampling frequency:
 	 *  Ideally, this is chosen such that we have 1-2 pixels = 1 segment
 	 *	which means that our curves can be as smooth as possible. However,
 	 *  this does mean that curves may not be fully accurate (i.e. if they have
 	 *  sudden spikes which happen at the sampling point, we may have problems).
-	 *  Also, this may introduce lower performance on less densely detailed curves,'
+	 *  Also, this may introduce lower performance on less densely detailed curves,
 	 *	though it is impossible to predict this from the modifiers!
 	 *
 	 *	If the automatically determined sampling frequency is likely to cause an infinite
@@ -515,9 +513,27 @@ static void draw_fcurve_curve(bAnimContext *ac, ID *id, FCurve *fcu, View2D *v2d
 	 *  chosen here is just the coarsest value which still looks reasonable...
 	 */
 	/* grid->dx represents the number of 'frames' between gridlines, but we divide by U.v2d_min_gridsize to get pixels-steps */
-	// TODO: perhaps we should have 1.0 frames as upper limit so that curves don't get too distorted?
-	samplefreq = dx / U.v2d_min_gridsize;
-	if (samplefreq < 0.00001f) samplefreq = 0.00001f;
+	/* TODO: perhaps we should have 1.0 frames as upper limit so that curves don't get too distorted? */
+	samplefreq = dx / (U.v2d_min_gridsize * U.pixelsize);
+	
+	if (sipo->flag & SIPO_BEAUTYDRAW_OFF) {
+		/* Low Precision = coarse lower-bound clamping
+		 * 
+		 * Although the "Beauty Draw" flag was originally for AA'd
+		 * line drawing, the sampling rate here has a much greater
+		 * impact on performance (e.g. for T40372)!
+		 *
+		 * This one still amounts to 10 sample-frames for each 1-frame interval
+		 * which should be quite a decent approximation in many situations.
+		 */
+		if (samplefreq < 0.1f)
+			samplefreq = 0.1f;
+	}
+	else {
+		/* "Higher Precision" but slower - especially on larger windows (e.g. T40372) */
+		if (samplefreq < 0.00001f)
+			samplefreq = 0.00001f;
+	}
 	
 	
 	/* the start/end times are simply the horizontal extents of the 'cur' rect */
@@ -531,8 +547,11 @@ static void draw_fcurve_curve(bAnimContext *ac, ID *id, FCurve *fcu, View2D *v2d
 	 */
 	glBegin(GL_LINE_STRIP);
 	
-	for (ctime = stime; ctime <= etime; ctime += samplefreq)
+	n = (etime - stime) / samplefreq + 0.5f;
+	for (i = 0; i <= n; i++) {
+		float ctime = stime + i * samplefreq;
 		glVertex2f(ctime, evaluate_fcurve(fcu, ctime) * unitFac);
+	}
 	
 	glEnd();
 	
@@ -547,11 +566,15 @@ static void draw_fcurve_curve_samples(bAnimContext *ac, ID *id, FCurve *fcu, Vie
 	FPoint *fpt = prevfpt + 1;
 	float fac, v[2];
 	int b = fcu->totvert - 1;
-	
-	glBegin(GL_LINE_STRIP);
-	
+	float unit_scale;
+	short mapping_flag = ANIM_get_normalization_flags(ac);
+
 	/* apply unit mapping */
-	ANIM_unit_mapping_apply_fcurve(ac->scene, id, fcu, 0);
+	glPushMatrix();
+	unit_scale = ANIM_unit_mapping_get_factor(ac->scene, id, fcu, mapping_flag);
+	glScalef(1.0f, unit_scale, 1.0f);
+
+	glBegin(GL_LINE_STRIP);
 	
 	/* extrapolate to left? - left-side of view comes before first keyframe? */
 	if (prevfpt->vec[0] > v2d->cur.xmin) {
@@ -561,9 +584,9 @@ static void draw_fcurve_curve_samples(bAnimContext *ac, ID *id, FCurve *fcu, Vie
 		if ((fcu->extend == FCURVE_EXTRAPOLATE_CONSTANT) || (fcu->flag & FCURVE_INT_VALUES) || (fcu->totvert == 1)) {
 			/* just extend across the first keyframe's value */
 			v[1] = prevfpt->vec[1];
-		} 
+		}
 		else {
-			/* extrapolate linear dosnt use the handle, use the next points center instead */
+			/* extrapolate linear doesn't use the handle, use the next points center instead */
 			fac = (prevfpt->vec[0] - fpt->vec[0]) / (prevfpt->vec[0] - v[0]);
 			if (fac) fac = 1.0f / fac;
 			v[1] = prevfpt->vec[1] - fac * (prevfpt->vec[1] - fpt->vec[1]);
@@ -599,9 +622,9 @@ static void draw_fcurve_curve_samples(bAnimContext *ac, ID *id, FCurve *fcu, Vie
 		if ((fcu->extend == FCURVE_EXTRAPOLATE_CONSTANT) || (fcu->flag & FCURVE_INT_VALUES) || (fcu->totvert == 1)) {
 			/* based on last keyframe's value */
 			v[1] = prevfpt->vec[1];
-		} 
+		}
 		else {
-			/* extrapolate linear dosnt use the handle, use the previous points center instead */
+			/* extrapolate linear doesn't use the handle, use the previous points center instead */
 			fpt = prevfpt - 1;
 			fac = (prevfpt->vec[0] - fpt->vec[0]) / (prevfpt->vec[0] - v[0]);
 			if (fac) fac = 1.0f / fac;
@@ -611,13 +634,28 @@ static void draw_fcurve_curve_samples(bAnimContext *ac, ID *id, FCurve *fcu, Vie
 		glVertex2fv(v);
 	}
 	
-	/* unapply unit mapping */
-	ANIM_unit_mapping_apply_fcurve(ac->scene, id, fcu, ANIM_UNITCONV_RESTORE);
-	
 	glEnd();
+	glPopMatrix();
 }
 
-/* helper func - draw one repeat of an F-Curve */
+/* helper func - check if the F-Curve only contains easily drawable segments 
+ * (i.e. no easing equation interpolations) 
+ */
+static bool fcurve_can_use_simple_bezt_drawing(FCurve *fcu)
+{
+	BezTriple *bezt;
+	int i;
+	
+	for (i = 0, bezt = fcu->bezt; i < fcu->totvert; i++, bezt++) {
+		if (ELEM(bezt->ipo, BEZT_IPO_CONST, BEZT_IPO_LIN, BEZT_IPO_BEZ) == false) {
+			return false;
+		}
+	}
+	
+	return true;
+}
+
+/* helper func - draw one repeat of an F-Curve (using Bezier curve approximations) */
 static void draw_fcurve_curve_bezts(bAnimContext *ac, ID *id, FCurve *fcu, View2D *v2d)
 {
 	BezTriple *prevbezt = fcu->bezt;
@@ -627,11 +665,15 @@ static void draw_fcurve_curve_bezts(bAnimContext *ac, ID *id, FCurve *fcu, View2
 	float fac = 0.0f;
 	int b = fcu->totvert - 1;
 	int resol;
-	
-	glBegin(GL_LINE_STRIP);
+	float unit_scale;
+	short mapping_flag = ANIM_get_normalization_flags(ac);
 	
 	/* apply unit mapping */
-	ANIM_unit_mapping_apply_fcurve(ac->scene, id, fcu, 0);
+	glPushMatrix();
+	unit_scale = ANIM_unit_mapping_get_factor(ac->scene, id, fcu, mapping_flag);
+	glScalef(1.0f, unit_scale, 1.0f);
+	
+	glBegin(GL_LINE_STRIP);
 	
 	/* extrapolate to left? */
 	if (prevbezt->vec[1][0] > v2d->cur.xmin) {
@@ -642,13 +684,13 @@ static void draw_fcurve_curve_bezts(bAnimContext *ac, ID *id, FCurve *fcu, View2
 		if ((fcu->extend == FCURVE_EXTRAPOLATE_CONSTANT) || (prevbezt->ipo == BEZT_IPO_CONST) || (fcu->totvert == 1)) {
 			/* just extend across the first keyframe's value */
 			v1[1] = prevbezt->vec[1][1];
-		} 
+		}
 		else if (prevbezt->ipo == BEZT_IPO_LIN) {
 			/* extrapolate linear dosnt use the handle, use the next points center instead */
 			fac = (prevbezt->vec[1][0] - bezt->vec[1][0]) / (prevbezt->vec[1][0] - v1[0]);
 			if (fac) fac = 1.0f / fac;
 			v1[1] = prevbezt->vec[1][1] - fac * (prevbezt->vec[1][1] - bezt->vec[1][1]);
-		} 
+		}
 		else {
 			/* based on angle of handle 1 (relative to keyframe) */
 			fac = (prevbezt->vec[0][0] - prevbezt->vec[1][0]) / (prevbezt->vec[1][0] - v1[0]);
@@ -667,7 +709,7 @@ static void draw_fcurve_curve_bezts(bAnimContext *ac, ID *id, FCurve *fcu, View2
 	}
 	
 	/* draw curve between first and last keyframe (if there are enough to do so) */
-	// TODO: optimize this to not have to calc stuff out of view too?
+	/* TODO: optimize this to not have to calc stuff out of view too? */
 	while (b--) {
 		if (prevbezt->ipo == BEZT_IPO_CONST) {
 			/* Constant-Interpolation: draw segment between previous keyframe and next, but holding same value */
@@ -685,17 +727,19 @@ static void draw_fcurve_curve_bezts(bAnimContext *ac, ID *id, FCurve *fcu, View2
 			v1[1] = prevbezt->vec[1][1];
 			glVertex2fv(v1);
 		}
-		else {
+		else if (prevbezt->ipo == BEZT_IPO_BEZ) {
 			/* Bezier-Interpolation: draw curve as series of segments between keyframes 
 			 *	- resol determines number of points to sample in between keyframes
 			 */
 			
 			/* resol depends on distance between points (not just horizontal) OR is a fixed high res */
-			// TODO: view scale should factor into this someday too...
-			if (fcu->driver) 
+			/* TODO: view scale should factor into this someday too... */
+			if (fcu->driver) {
 				resol = 32;
-			else 
+			}
+			else {
 				resol = (int)(5.0f * len_v2v2(bezt->vec[1], prevbezt->vec[1]));
+			}
 			
 			if (resol < 2) {
 				/* only draw one */
@@ -705,14 +749,14 @@ static void draw_fcurve_curve_bezts(bAnimContext *ac, ID *id, FCurve *fcu, View2
 			}
 			else {
 				/* clamp resolution to max of 32 */
-				// NOTE: higher values will crash
+				/* NOTE: higher values will crash */
 				if (resol > 32) resol = 32;
 				
 				v1[0] = prevbezt->vec[1][0];
 				v1[1] = prevbezt->vec[1][1];
 				v2[0] = prevbezt->vec[2][0];
 				v2[1] = prevbezt->vec[2][1];
-
+				
 				v3[0] = bezt->vec[0][0];
 				v3[1] = bezt->vec[0][1];
 				v4[0] = bezt->vec[1][0];
@@ -748,14 +792,14 @@ static void draw_fcurve_curve_bezts(bAnimContext *ac, ID *id, FCurve *fcu, View2
 		if ((fcu->extend == FCURVE_EXTRAPOLATE_CONSTANT) || (fcu->flag & FCURVE_INT_VALUES) || (prevbezt->ipo == BEZT_IPO_CONST) || (fcu->totvert == 1)) {
 			/* based on last keyframe's value */
 			v1[1] = prevbezt->vec[1][1];
-		} 
+		}
 		else if (prevbezt->ipo == BEZT_IPO_LIN) {
 			/* extrapolate linear dosnt use the handle, use the previous points center instead */
 			bezt = prevbezt - 1;
 			fac = (prevbezt->vec[1][0] - bezt->vec[1][0]) / (prevbezt->vec[1][0] - v1[0]);
 			if (fac) fac = 1.0f / fac;
 			v1[1] = prevbezt->vec[1][1] - fac * (prevbezt->vec[1][1] - bezt->vec[1][1]);
-		} 
+		}
 		else {
 			/* based on angle of handle 1 (relative to keyframe) */
 			fac = (prevbezt->vec[2][0] - prevbezt->vec[1][0]) / (prevbezt->vec[1][0] - v1[0]);
@@ -766,11 +810,115 @@ static void draw_fcurve_curve_bezts(bAnimContext *ac, ID *id, FCurve *fcu, View2
 		glVertex2fv(v1);
 	}
 	
-	/* unapply unit mapping */
-	ANIM_unit_mapping_apply_fcurve(ac->scene, id, fcu, ANIM_UNITCONV_RESTORE);
-	
 	glEnd();
-} 
+	glPopMatrix();
+}
+
+/* Debugging -------------------------------- */
+
+/* Draw indicators which show the value calculated from the driver, 
+ * and how this is mapped to the value that comes out of it. This
+ * is handy for helping users better understand how to interpret
+ * the graphs, and also facilitates debugging. 
+ */
+static void graph_draw_driver_debug(bAnimContext *ac, ID *id, FCurve *fcu)
+{
+	ChannelDriver *driver = fcu->driver;
+	View2D *v2d = &ac->ar->v2d;
+	short mapping_flag = ANIM_get_normalization_flags(ac);
+	float unitfac = ANIM_unit_mapping_get_factor(ac->scene, id, fcu, mapping_flag);
+	
+	/* for now, only show when debugging driver... */
+	//if ((driver->flag & DRIVER_FLAG_SHOWDEBUG) == 0)
+	//	return;
+	
+	/* No curve to modify/visualise the result? 
+	 * => We still want to show the 1-1 default... 
+	 */
+	if ((fcu->totvert == 0) && BLI_listbase_is_empty(&fcu->modifiers)) {
+		float t;
+		
+		/* draw with thin dotted lines in style of what curve would have been */
+		glColor3fv(fcu->color);
+		
+		setlinestyle(20);
+		glLineWidth(2.0f);
+		
+		/* draw 1-1 line, stretching just past the screen limits 
+		 * NOTE: we need to scale the y-values to be valid for the units
+		 */
+		glBegin(GL_LINES);
+			t = v2d->cur.xmin;
+			glVertex2f(t, t * unitfac);
+			
+			t = v2d->cur.xmax;
+			glVertex2f(t, t * unitfac); 
+		glEnd();
+		
+		/* cleanup line drawing */
+		setlinestyle(0);
+		glLineWidth(1.0f);
+	}
+	
+	/* draw driver only if actually functional */
+	if ((driver->flag & DRIVER_FLAG_INVALID) == 0) {
+		/* grab "coordinates" for driver outputs */
+		float x = driver->curval;
+		float y = fcu->curval * unitfac;
+		
+		/* only draw indicators if the point is in range*/
+		if (x >= v2d->cur.xmin) {
+			float co[2];
+			
+			/* draw dotted lines leading towards this point from both axes ....... */
+			glColor3f(0.9f, 0.9f, 0.9f);
+			setlinestyle(5);
+			
+			glBegin(GL_LINES);
+				/* x-axis lookup */
+				co[0] = x;
+				
+				if (y >= v2d->cur.ymin) {
+					co[1] = v2d->cur.ymin - 1.0f;
+					glVertex2fv(co);
+					
+					co[1] = y;
+					glVertex2fv(co);
+				}
+				
+				/* y-axis lookup */
+				co[1] = y;
+				
+				co[0] = v2d->cur.xmin - 1.0f;
+				glVertex2fv(co);
+				
+				co[0] = x;
+				glVertex2fv(co);
+			glEnd();
+			
+			setlinestyle(0);
+			
+			/* x marks the spot .................................................... */
+			/* -> outer frame */
+			glColor3f(0.9f, 0.9f, 0.9f);
+			glPointSize(7.0);
+			
+			glBegin(GL_POINTS);
+				glVertex2f(x, y);
+			glEnd();
+			
+			/* inner frame */
+			glColor3f(0.9f, 0.0f, 0.0f);
+			glPointSize(3.0);
+			
+			glBegin(GL_POINTS);
+				glVertex2f(x, y);
+			glEnd();
+			
+			glPointSize(1.0f);
+		}
+	}
+}
 
 /* Public Curve-Drawing API  ---------------- */
 
@@ -846,13 +994,13 @@ void graph_draw_curves(bAnimContext *ac, SpaceIpo *sipo, ARegion *ar, View2DGrid
 		/* 1) draw curve line */
 		{
 			/* set color/drawing style for curve itself */
-			if (((fcu->grp) && (fcu->grp->flag & AGRP_PROTECTED)) || (fcu->flag & FCURVE_PROTECTED)) {
+			if (BKE_fcurve_is_protected(fcu)) {
 				/* protected curves (non editable) are drawn with dotted lines */
 				setlinestyle(2);
 			}
 			if (((fcu->grp) && (fcu->grp->flag & AGRP_MUTED)) || (fcu->flag & FCURVE_MUTED)) {
 				/* muted curves are drawn in a grayish hue */
-				// XXX should we have some variations?
+				/* XXX should we have some variations? */
 				UI_ThemeColorShade(TH_HEADER, 50);
 			}
 			else {
@@ -880,10 +1028,15 @@ void graph_draw_curves(bAnimContext *ac, SpaceIpo *sipo, ARegion *ar, View2DGrid
 			}
 			else if (((fcu->bezt) || (fcu->fpt)) && (fcu->totvert)) {
 				/* just draw curve based on defined data (i.e. no modifiers) */
-				if (fcu->bezt)
-					draw_fcurve_curve_bezts(ac, ale->id, fcu, &ar->v2d);
-				else if (fcu->fpt)
+				if (fcu->bezt) {
+					if (fcurve_can_use_simple_bezt_drawing(fcu))
+						draw_fcurve_curve_bezts(ac, ale->id, fcu, &ar->v2d);
+					else
+						draw_fcurve_curve(ac, ale->id, fcu, &ar->v2d, grid);
+				}
+				else if (fcu->fpt) {
 					draw_fcurve_curve_samples(ac, ale->id, fcu, &ar->v2d);
+				}
 			}
 			
 			/* restore settings */
@@ -909,11 +1062,14 @@ void graph_draw_curves(bAnimContext *ac, SpaceIpo *sipo, ARegion *ar, View2DGrid
 				}
 			}
 			else if (((fcu->bezt) || (fcu->fpt)) && (fcu->totvert)) {
-				/* apply unit mapping */
-				ANIM_unit_mapping_apply_fcurve(ac->scene, ale->id, fcu, 0);
-				
+				short mapping_flag = ANIM_get_normalization_flags(ac);
+				float unit_scale = ANIM_unit_mapping_get_factor(ac->scene, ale->id, fcu, mapping_flag);
+
+				glPushMatrix();
+				glScalef(1.0f, unit_scale, 1.0f);
+
 				if (fcu->bezt) {
-					int do_handles = draw_fcurve_handles_check(sipo, fcu);
+					bool do_handles = draw_fcurve_handles_check(sipo, fcu);
 					
 					if (do_handles) {
 						/* only draw handles/vertices on keyframes */
@@ -922,25 +1078,29 @@ void graph_draw_curves(bAnimContext *ac, SpaceIpo *sipo, ARegion *ar, View2DGrid
 						glDisable(GL_BLEND);
 					}
 					
-					draw_fcurve_vertices(sipo, ar, fcu, do_handles, (sipo->flag & SIPO_SELVHANDLESONLY));
+					draw_fcurve_vertices(sipo, ar, fcu, do_handles, (sipo->flag & SIPO_SELVHANDLESONLY), unit_scale);
 				}
 				else {
 					/* samples: only draw two indicators at either end as indicators */
 					draw_fcurve_samples(sipo, ar, fcu);
 				}
-				
-				/* unapply unit mapping */
-				ANIM_unit_mapping_apply_fcurve(ac->scene, ale->id, fcu, ANIM_UNITCONV_RESTORE);
+
+				glPopMatrix();
 			}
 		}
 		
+		/* 3) draw driver debugging stuff */
+		if ((ac->datatype == ANIMCONT_DRIVERS) && (fcu->flag & FCURVE_ACTIVE)) {
+			graph_draw_driver_debug(ac, ale->id, fcu);
+		}
+		
 		/* undo mapping of keyframes for drawing if scaled F-Curve */
 		if (adt)
 			ANIM_nla_mapping_apply_fcurve(adt, ale->key_data, 1, 0); 
 	}
 	
 	/* free list of curves */
-	BLI_freelistN(&anim_data);
+	ANIM_animdata_freelist(&anim_data);
 }
 
 /* ************************************************************************* */
@@ -971,7 +1131,7 @@ void graph_draw_channel_names(bContext *C, bAnimContext *ac, ARegion *ar)
 	height = (float)((items * ACHANNEL_STEP) + (ACHANNEL_HEIGHT * 2));
 	UI_view2d_totRect_set(v2d, ar->winx, height);
 	
-	/* loop through channels, and set up drawing depending on their type  */	
+	/* loop through channels, and set up drawing depending on their type  */
 	{   /* first pass: just the standard GL-drawing for backdrop + text */
 		y = (float)ACHANNEL_FIRST;
 		
@@ -992,7 +1152,7 @@ void graph_draw_channel_names(bContext *C, bAnimContext *ac, ARegion *ar)
 		}
 	}
 	{   /* second pass: widgets */
-		uiBlock *block = uiBeginBlock(C, ar, __func__, UI_EMBOSS);
+		uiBlock *block = UI_block_begin(C, ar, __func__, UI_EMBOSS);
 		size_t channel_index = 0;
 		
 		y = (float)ACHANNEL_FIRST;
@@ -1018,12 +1178,12 @@ void graph_draw_channel_names(bContext *C, bAnimContext *ac, ARegion *ar)
 			channel_index++;
 		}
 		
-		uiEndBlock(C, block);
-		uiDrawBlock(C, block);
+		UI_block_end(C, block);
+		UI_block_draw(C, block);
 		
 		glDisable(GL_BLEND);
 	}
 	
 	/* free tempolary channels */
-	BLI_freelistN(&anim_data);
+	ANIM_animdata_freelist(&anim_data);
 }
