@@ -30,25 +30,18 @@
 #include <string.h>
 #include <stdio.h>
 
-#include "DNA_meshdata_types.h"
-#include "DNA_object_types.h"
 #include "DNA_node_types.h"
 #include "DNA_scene_types.h"
 
 #include "MEM_guardedalloc.h"
 
 #include "BLI_blenlib.h"
-#include "BLI_math.h"
-#include "BLI_rand.h"
 #include "BLI_utildefines.h"
 
 #include "BLF_translation.h"
 
-#include "BKE_colortools.h"
 #include "BKE_context.h"
-#include "BKE_customdata.h"
 #include "BKE_image.h"
-#include "BKE_mesh.h"
 #include "BKE_node.h"
 #include "BKE_screen.h"
 
@@ -58,7 +51,6 @@
 #include "IMB_imbuf_types.h"
 
 #include "ED_gpencil.h"
-#include "ED_image.h"
 #include "ED_screen.h"
 
 #include "RNA_access.h"
@@ -71,53 +63,59 @@
 
 #include "image_intern.h"
 
+#define B_NOP -1
+
 /* proto */
 
-static void image_info(Scene *scene, ImageUser *iuser, Image *ima, ImBuf *ibuf, char *str)
+static void image_info(Scene *scene, ImageUser *iuser, Image *ima, ImBuf *ibuf, char *str, size_t len)
 {
-	int ofs = 0;
+	size_t ofs = 0;
 
 	str[0] = 0;
-	
-	if (ima == NULL) return;
+	if (ima == NULL)
+		return;
 
 	if (ibuf == NULL) {
-		ofs += sprintf(str, "%s", IFACE_("Can't Load Image"));
+		ofs += BLI_strncpy_rlen(str + ofs, IFACE_("Can't Load Image"), len - ofs);
 	}
 	else {
 		if (ima->source == IMA_SRC_MOVIE) {
-			ofs += sprintf(str, "%s", IFACE_("Movie"));
+			ofs += BLI_strncpy_rlen(str + ofs, IFACE_("Movie"), len - ofs);
 			if (ima->anim)
-				ofs += sprintf(str + ofs, IFACE_("%d frs"), IMB_anim_get_duration(ima->anim, IMB_TC_RECORD_RUN));
+				ofs += BLI_snprintf(str + ofs, len - ofs, IFACE_(" %d frs"),
+				                    IMB_anim_get_duration(ima->anim, IMB_TC_RECORD_RUN));
+		}
+		else {
+			ofs += BLI_strncpy_rlen(str, IFACE_("Image"), len - ofs);
 		}
-		else
-			ofs += sprintf(str, "%s", IFACE_("Image"));
 
-		ofs += sprintf(str + ofs, ": %s %d x %d,", IFACE_("size"), ibuf->x, ibuf->y);
+		ofs += BLI_snprintf(str + ofs, len - ofs, IFACE_(": size %d x %d,"), ibuf->x, ibuf->y);
 
 		if (ibuf->rect_float) {
 			if (ibuf->channels != 4) {
-				ofs += sprintf(str + ofs, "%d %s", ibuf->channels, IFACE_("float channel(s)"));
+				ofs += BLI_snprintf(str + ofs, len - ofs, IFACE_("%d float channel(s)"), ibuf->channels);
 			}
 			else if (ibuf->planes == R_IMF_PLANES_RGBA)
-				ofs += sprintf(str + ofs, "%s", IFACE_(" RGBA float"));
+				ofs += BLI_strncpy_rlen(str + ofs, IFACE_(" RGBA float"), len - ofs);
 			else
-				ofs += sprintf(str + ofs, "%s", IFACE_(" RGB float"));
+				ofs += BLI_strncpy_rlen(str + ofs, IFACE_(" RGB float"), len - ofs);
 		}
 		else {
 			if (ibuf->planes == R_IMF_PLANES_RGBA)
-				ofs += sprintf(str + ofs, "%s", IFACE_(" RGBA byte"));
+				ofs += BLI_strncpy_rlen(str + ofs, IFACE_(" RGBA byte"), len - ofs);
 			else
-				ofs += sprintf(str + ofs, "%s", IFACE_(" RGB byte"));
+				ofs += BLI_strncpy_rlen(str + ofs, IFACE_(" RGB byte"), len - ofs);
 		}
 		if (ibuf->zbuf || ibuf->zbuf_float)
-			ofs += sprintf(str + ofs, "%s", IFACE_(" + Z"));
+			ofs += BLI_strncpy_rlen(str + ofs, IFACE_(" + Z"), len - ofs);
 
 		if (ima->source == IMA_SRC_SEQUENCE) {
-			char *file = BLI_last_slash(ibuf->name);
-			if (file == NULL) file = ibuf->name;
-			else file++;
-			ofs += sprintf(str + ofs, ", %s", file);
+			const char *file = BLI_last_slash(ibuf->name);
+			if (file == NULL)
+				file = ibuf->name;
+			else
+				file++;
+			ofs += BLI_snprintf(str + ofs, len - ofs, ", %s", file);
 		}
 	}
 
@@ -125,10 +123,8 @@ static void image_info(Scene *scene, ImageUser *iuser, Image *ima, ImBuf *ibuf,
 	if (ima->source == IMA_SRC_SEQUENCE) {
 		/* don't use iuser->framenr directly because it may not be updated if auto-refresh is off */
 		const int framenr = BKE_image_user_frame_get(iuser, CFRA, 0, NULL);
-		ofs += sprintf(str + ofs, IFACE_(", Frame: %d"), framenr);
+		ofs += BLI_snprintf(str + ofs, len - ofs, IFACE_(", Frame: %d"), framenr);
 	}
-
-	(void)ofs;
 }
 
 /* gets active viewer user */
@@ -176,7 +172,7 @@ void image_preview_event(int event)
 		
 		ntreeCompositTagGenerators(G.scene->nodetree);
 
-		G.is_break = FALSE;
+		G.is_break = false;
 		G.scene->nodetree->timecursor = set_timecursor;
 		G.scene->nodetree->test_break = blender_test_break;
 		
@@ -191,7 +187,7 @@ void image_preview_event(int event)
 		waitcursor(0);
 		
 		WM_event_add_notifier(C, NC_IMAGE, ima_v);
-	}	
+	}
 }
 
 
@@ -219,7 +215,7 @@ static void preview_cb(ScrArea *sa, struct uiBlock *block)
 	BLI_rcti_translate(disprect, -curarea->winrct.xmin, -curarea->winrct.ymin);
 	
 	calc_image_view(sima, 'p');
-//	printf("winrct %d %d %d %d\n", disprect->xmin, disprect->ymin,disprect->xmax, disprect->ymax);
+//	printf("winrct %d %d %d %d\n", disprect->xmin, disprect->ymin, disprect->xmax, disprect->ymax);
 	/* map to image space coordinates */
 	mval[0] = disprect->xmin; mval[1] = disprect->ymin;
 	areamouseco_to_ipoco(v2d, mval, &dispf.xmin, &dispf.ymin);
@@ -236,11 +232,11 @@ static void preview_cb(ScrArea *sa, struct uiBlock *block)
 	CLAMP(disprect->xmax, 0, winx);
 	CLAMP(disprect->ymin, 0, winy);
 	CLAMP(disprect->ymax, 0, winy);
-//	printf("drawrct %d %d %d %d\n", disprect->xmin, disprect->ymin,disprect->xmax, disprect->ymax);
+//	printf("drawrct %d %d %d %d\n", disprect->xmin, disprect->ymin, disprect->xmax, disprect->ymax);
 
 }
 
-static int is_preview_allowed(ScrArea *cur)
+static bool is_preview_allowed(ScrArea *cur)
 {
 	SpaceImage *sima = cur->spacedata.first;
 	ScrArea *sa;
@@ -272,7 +268,7 @@ static void image_panel_preview(ScrArea *sa, short cntrl)   // IMAGE_HANDLER_PRE
 		return;
 	}
 	
-	block = uiBeginBlock(C, ar, __func__, UI_EMBOSS);
+	block = UI_block_begin(C, ar, __func__, UI_EMBOSS);
 	uiPanelControl(UI_PNL_SOLID | UI_PNL_CLOSE | UI_PNL_SCALE | cntrl);
 	uiSetPanelHandler(IMAGE_HANDLER_PREVIEW);  // for close and esc
 	
@@ -280,7 +276,7 @@ static void image_panel_preview(ScrArea *sa, short cntrl)   // IMAGE_HANDLER_PRE
 	ofsy = -100 + (sa->winy / 2) / sima->blockscale;
 	if (uiNewPanel(C, ar, block, "Preview", "Image", ofsx, ofsy, 300, 200) == 0) return;
 	
-	uiBlockSetDrawExtraFunc(block, preview_cb);
+	UI_but_func_drawextra_set(block, preview_cb);
 	
 }
 #endif
@@ -288,76 +284,160 @@ static void image_panel_preview(ScrArea *sa, short cntrl)   // IMAGE_HANDLER_PRE
 
 /* ********************* callbacks for standard image buttons *************** */
 
-static char *slot_menu(void)
+static void ui_imageuser_slot_menu(bContext *UNUSED(C), uiLayout *layout, void *image_p)
 {
-	char *str;
-	int a, slot;
-	
-	str = MEM_callocN(IMA_MAX_RENDER_SLOT * 32, "menu slots");
-	
-	strcpy(str, IFACE_("Slot %t"));
-	a = strlen(str);
+	uiBlock *block = uiLayoutGetBlock(layout);
+	Image *image = image_p;
+	int slot;
 
-	for (slot = 0; slot < IMA_MAX_RENDER_SLOT; slot++)
-		a += sprintf(str + a, IFACE_("|Slot %d %%x%d"), slot + 1, slot);
-	
-	return str;
+	uiDefBut(block, UI_BTYPE_LABEL, 0, IFACE_("Slot"),
+	         0, 0, UI_UNIT_X * 5, UI_UNIT_Y, NULL, 0.0, 0.0, 0, 0, "");
+	uiItemS(layout);
+
+	slot = IMA_MAX_RENDER_SLOT;
+	while (slot--) {
+		char str[64];
+		if (image->render_slots[slot].name[0] != '\0') {
+			BLI_strncpy(str, image->render_slots[slot].name, sizeof(str));
+		}
+		else {
+			BLI_snprintf(str, sizeof(str), IFACE_("Slot %d"), slot + 1);
+		}
+		uiDefButS(block, UI_BTYPE_BUT_MENU, B_NOP, str, 0, 0,
+		          UI_UNIT_X * 5, UI_UNIT_X, &image->render_slot, (float) slot, 0.0, 0, -1, "");
+	}
 }
 
-/* TODO, curlay should be removed? */
-static char *layer_menu(RenderResult *rr, short *UNUSED(curlay))
+static const char *ui_imageuser_layer_fake_name(RenderResult *rr)
 {
-	RenderLayer *rl;
-	int len = 64 + RE_MAXNAME * BLI_countlist(&rr->layers);
-	short a, nr = 0;
-	char *str = MEM_callocN(len, "menu layers");
-	
-	strcpy(str, IFACE_("Layer %t"));
-	a = strlen(str);
-	
-	/* compo result */
 	if (rr->rectf) {
-		a += sprintf(str + a, "|%s %%x0", IFACE_("Composite"));
-		nr = 1;
+		return IFACE_("Composite");
 	}
 	else if (rr->rect32) {
-		a += sprintf(str + a, "|%s %%x0", IFACE_("Sequence"));
-		nr = 1;
+		return IFACE_("Sequence");
 	}
-	for (rl = rr->layers.first; rl; rl = rl->next, nr++) {
-		a += sprintf(str + a, "|%s %%x%d", rl->name, nr);
+	else {
+		return NULL;
 	}
-	
-	/* no curlay clip here, on render (redraws) the amount of layers can be 1 fir single-layer render */
-	
-	return str;
 }
 
-/* rl==NULL means composite result */
-static char *pass_menu(RenderLayer *rl, short *curpass)
+static void ui_imageuser_layer_menu(bContext *UNUSED(C), uiLayout *layout, void *rnd_pt)
 {
-	RenderPass *rpass;
-	int len = 64 + 32 * (rl ? BLI_countlist(&rl->passes) : 1);
-	short a, nr = 0;
-	char *str = MEM_callocN(len, "menu layers");
-	
-	strcpy(str, IFACE_("Pass %t"));
-	a = strlen(str);
-	
-	/* rendered results don't have a Combined pass */
-	if (rl == NULL || rl->rectf) {
-		a += sprintf(str + a, "|%s %%x0", IFACE_("Combined"));
-		nr = 1;
+	void **rnd_data = rnd_pt;
+	uiBlock *block = uiLayoutGetBlock(layout);
+	Image *image = rnd_data[0];
+	ImageUser *iuser = rnd_data[1];
+	Scene *scene = iuser->scene;
+	RenderResult *rr;
+	RenderLayer *rl;
+	RenderLayer rl_fake = {NULL};
+	const char *fake_name;
+	int nr;
+
+	/* may have been freed since drawing */
+	rr = BKE_image_acquire_renderresult(scene, image);
+	if (UNLIKELY(rr == NULL)) {
+		return;
 	}
-	
-	if (rl)
-		for (rpass = rl->passes.first; rpass; rpass = rpass->next, nr++)
-			a += sprintf(str + a, "|%s %%x%d", IFACE_(rpass->name), nr);
-	
-	if (*curpass >= nr)
-		*curpass = 0;
-	
-	return str;
+
+	UI_block_layout_set_current(block, layout);
+	uiLayoutColumn(layout, false);
+
+	uiDefBut(block, UI_BTYPE_LABEL, 0, IFACE_("Layer"),
+	         0, 0, UI_UNIT_X * 5, UI_UNIT_Y, NULL, 0.0, 0.0, 0, 0, "");
+	uiItemS(layout);
+
+	nr = BLI_listbase_count(&rr->layers) - 1;
+	fake_name = ui_imageuser_layer_fake_name(rr);
+
+	if (fake_name) {
+		BLI_strncpy(rl_fake.name, fake_name, sizeof(rl_fake.name));
+		nr += 1;
+	}
+
+	for (rl = rr->layers.last; rl; rl = rl->prev, nr--) {
+final:
+		uiDefButS(block, UI_BTYPE_BUT_MENU, B_NOP, IFACE_(rl->name), 0, 0,
+		          UI_UNIT_X * 5, UI_UNIT_X, &iuser->layer, (float) nr, 0.0, 0, -1, "");
+	}
+
+	if (fake_name) {
+		fake_name = NULL;
+		rl = &rl_fake;
+		goto final;
+	}
+
+	BLI_assert(nr == -1);
+
+	BKE_image_release_renderresult(scene, image);
+}
+
+static const char *ui_imageuser_pass_fake_name(RenderLayer *rl)
+{
+	if (rl == NULL || rl->rectf) {
+		return IFACE_("Combined");
+	}
+	else {
+		return NULL;
+	}
+}
+
+static void ui_imageuser_pass_menu(bContext *UNUSED(C), uiLayout *layout, void *ptrpair_p)
+{
+	void **rnd_data = ptrpair_p;
+	uiBlock *block = uiLayoutGetBlock(layout);
+	Image *image = rnd_data[0];
+	ImageUser *iuser = rnd_data[1];
+	/* (rpass_index == -1) means composite result */
+	const int rpass_index = GET_INT_FROM_POINTER(rnd_data[2]);
+	Scene *scene = iuser->scene;
+	RenderResult *rr;
+	RenderLayer *rl;
+	RenderPass rpass_fake = {NULL};
+	RenderPass *rpass;
+	const char *fake_name;
+	int nr;
+
+	/* may have been freed since drawing */
+	rr = BKE_image_acquire_renderresult(scene, image);
+	if (UNLIKELY(rr == NULL)) {
+		return;
+	}
+
+	rl = BLI_findlink(&rr->layers, rpass_index);
+
+	UI_block_layout_set_current(block, layout);
+	uiLayoutColumn(layout, false);
+
+	uiDefBut(block, UI_BTYPE_LABEL, 0, IFACE_("Pass"),
+	         0, 0, UI_UNIT_X * 5, UI_UNIT_Y, NULL, 0.0, 0.0, 0, 0, "");
+
+	uiItemS(layout);
+
+	nr = (rl ? BLI_listbase_count(&rl->passes) : 0) - 1;
+	fake_name = ui_imageuser_pass_fake_name(rl);
+
+	if (fake_name) {
+		BLI_strncpy(rpass_fake.name, fake_name, sizeof(rpass_fake.name));
+		nr += 1;
+	}
+
+	/* rendered results don't have a Combined pass */
+	for (rpass = rl ? rl->passes.last : NULL; rpass; rpass = rpass->prev, nr--) {
+final:
+		uiDefButS(block, UI_BTYPE_BUT_MENU, B_NOP, IFACE_(rpass->name), 0, 0,
+		          UI_UNIT_X * 5, UI_UNIT_X, &iuser->pass, (float) nr, 0.0, 0, -1, "");
+	}
+
+	if (fake_name) {
+		fake_name = NULL;
+		rpass = &rpass_fake;
+		goto final;
+	}
+
+	BLI_assert(nr == -1);
+
+	BKE_image_release_renderresult(scene, image);
 }
 
 /* 5 layer button callbacks... */
@@ -372,7 +452,7 @@ static void image_multi_inclay_cb(bContext *C, void *rr_v, void *iuser_v)
 {
 	RenderResult *rr = rr_v;
 	ImageUser *iuser = iuser_v;
-	int tot = BLI_countlist(&rr->layers);
+	int tot = BLI_listbase_count(&rr->layers);
 
 	if (rr->rectf || rr->rect32)
 		tot++;  /* fake compo/sequencer layer */
@@ -400,7 +480,7 @@ static void image_multi_incpass_cb(bContext *C, void *rr_v, void *iuser_v)
 	RenderLayer *rl = BLI_findlink(&rr->layers, iuser->layer);
 
 	if (rl) {
-		int tot = BLI_countlist(&rl->passes);
+		int tot = BLI_listbase_count(&rl->passes);
 
 		if (rr->rectf || rr->rect32)
 			tot++;  /* fake compo/sequencer layer */
@@ -440,78 +520,99 @@ static void image_user_change(bContext *C, void *iuser_v, void *unused)
 }
 #endif
 
-static void uiblock_layer_pass_buttons(uiLayout *layout, RenderResult *rr, ImageUser *iuser, int w, short *render_slot)
+static void uiblock_layer_pass_buttons(uiLayout *layout, Image *image, RenderResult *rr, ImageUser *iuser, int w, short *render_slot)
 {
+	static void *rnd_pt[3];  /* XXX, workaround */
 	uiBlock *block = uiLayoutGetBlock(layout);
 	uiBut *but;
 	RenderLayer *rl = NULL;
-	int wmenu1, wmenu2, wmenu3, layer;
-	char *strp;
+	int wmenu1, wmenu2, wmenu3;
+	const char *fake_name;
+	const char *display_name;
 
-	uiLayoutRow(layout, TRUE);
+	uiLayoutRow(layout, true);
 
 	/* layer menu is 1/3 larger than pass */
 	wmenu1 = (2 * w) / 5;
 	wmenu2 = (3 * w) / 5;
 	wmenu3 = (3 * w) / 6;
 	
+	rnd_pt[0] = image;
+	rnd_pt[1] = iuser;
+	rnd_pt[2] = NULL;
+
 	/* menu buts */
 	if (render_slot) {
-		strp = slot_menu();
-		but = uiDefButS(block, MENU, 0, strp, 0, 0, wmenu1, UI_UNIT_Y, render_slot, 0, 0, 0, 0, TIP_("Select Slot"));
-		uiButSetFunc(but, image_multi_cb, rr, iuser);
-		MEM_freeN(strp);
+		char str[64];
+		if (image->render_slots[*render_slot].name[0] != '\0') {
+			BLI_strncpy(str, image->render_slots[*render_slot].name, sizeof(str));
+		}
+		else {
+			BLI_snprintf(str, sizeof(str), IFACE_("Slot %d"), *render_slot + 1);
+		}
+		but = uiDefMenuBut(block, ui_imageuser_slot_menu, image, str, 0, 0, wmenu1, UI_UNIT_Y, TIP_("Select Slot"));
+		UI_but_func_set(but, image_multi_cb, rr, iuser);
+		UI_but_type_set_menu_from_pulldown(but);
 	}
 
 	if (rr) {
-		strp = layer_menu(rr, &iuser->layer);
-		but = uiDefButS(block, MENU, 0, strp, 0, 0, wmenu2, UI_UNIT_Y, &iuser->layer, 0, 0, 0, 0, TIP_("Select Layer"));
-		uiButSetFunc(but, image_multi_cb, rr, iuser);
-		MEM_freeN(strp);
+		RenderPass *rpass;
+		int rpass_index;
 
-		layer = iuser->layer;
-		if (rr->rectf || rr->rect32)
-			layer--;  /* fake compo/sequencer layer */
-		
-		rl = BLI_findlink(&rr->layers, layer); /* return NULL is meant to be */
-		strp = pass_menu(rl, &iuser->pass);
-		but = uiDefButS(block, MENU, 0, strp, 0, 0, wmenu3, UI_UNIT_Y, &iuser->pass, 0, 0, 0, 0, TIP_("Select Pass"));
-		uiButSetFunc(but, image_multi_cb, rr, iuser);
-		MEM_freeN(strp);
+		/* layer */
+		fake_name = ui_imageuser_layer_fake_name(rr);
+		rpass_index = iuser->layer  - (fake_name ? 1 : 0);
+		rl = BLI_findlink(&rr->layers, rpass_index);
+		rnd_pt[2] = SET_INT_IN_POINTER(rpass_index);
+
+		display_name = rl ? rl->name : (fake_name ? fake_name : "");
+		but = uiDefMenuBut(block, ui_imageuser_layer_menu, rnd_pt, display_name, 0, 0, wmenu2, UI_UNIT_Y, TIP_("Select Layer"));
+		UI_but_func_set(but, image_multi_cb, rr, iuser);
+		UI_but_type_set_menu_from_pulldown(but);
+
+
+		/* pass */
+		fake_name = ui_imageuser_pass_fake_name(rl);
+		rpass = (rl ? BLI_findlink(&rl->passes, iuser->pass  - (fake_name ? 1 : 0)) : NULL);
+
+		display_name = rpass ? rpass->name : (fake_name ? fake_name : "");
+		but = uiDefMenuBut(block, ui_imageuser_pass_menu, rnd_pt, display_name, 0, 0, wmenu3, UI_UNIT_Y, TIP_("Select Pass"));
+		UI_but_func_set(but, image_multi_cb, rr, iuser);
+		UI_but_type_set_menu_from_pulldown(but);
 	}
 }
 
-static void uiblock_layer_pass_arrow_buttons(uiLayout *layout, RenderResult *rr, ImageUser *iuser, short *render_slot)
+static void uiblock_layer_pass_arrow_buttons(uiLayout *layout, Image *image, RenderResult *rr, ImageUser *iuser, short *render_slot)
 {
 	uiBlock *block = uiLayoutGetBlock(layout);
 	uiLayout *row;
 	uiBut *but;
 	const float dpi_fac = UI_DPI_FAC;
 	
-	row = uiLayoutRow(layout, TRUE);
+	row = uiLayoutRow(layout, true);
 
 	if (rr == NULL || iuser == NULL)
 		return;
-	if (rr->layers.first == NULL) {
+	if (BLI_listbase_is_empty(&rr->layers)) {
 		uiItemL(row, IFACE_("No Layers in Render Result"), ICON_NONE);
 		return;
 	}
 
 	/* decrease, increase arrows */
-	but = uiDefIconBut(block, BUT, 0, ICON_TRIA_LEFT,   0, 0, 17, 20, NULL, 0, 0, 0, 0, TIP_("Previous Layer"));
-	uiButSetFunc(but, image_multi_declay_cb, rr, iuser);
-	but = uiDefIconBut(block, BUT, 0, ICON_TRIA_RIGHT,  0, 0, 18, 20, NULL, 0, 0, 0, 0, TIP_("Next Layer"));
-	uiButSetFunc(but, image_multi_inclay_cb, rr, iuser);
+	but = uiDefIconBut(block, UI_BTYPE_BUT, 0, ICON_TRIA_LEFT,   0, 0, 0.85f * UI_UNIT_X, UI_UNIT_Y, NULL, 0, 0, 0, 0, TIP_("Previous Layer"));
+	UI_but_func_set(but, image_multi_declay_cb, rr, iuser);
+	but = uiDefIconBut(block, UI_BTYPE_BUT, 0, ICON_TRIA_RIGHT,  0, 0, 0.90f * UI_UNIT_X, UI_UNIT_Y, NULL, 0, 0, 0, 0, TIP_("Next Layer"));
+	UI_but_func_set(but, image_multi_inclay_cb, rr, iuser);
 
-	uiblock_layer_pass_buttons(row, rr, iuser, 230 * dpi_fac, render_slot);
+	uiblock_layer_pass_buttons(row, image, rr, iuser, 230 * dpi_fac, render_slot);
 
 	/* decrease, increase arrows */
-	but = uiDefIconBut(block, BUT, 0, ICON_TRIA_LEFT,   0, 0, 17, 20, NULL, 0, 0, 0, 0, TIP_("Previous Pass"));
-	uiButSetFunc(but, image_multi_decpass_cb, rr, iuser);
-	but = uiDefIconBut(block, BUT, 0, ICON_TRIA_RIGHT,  0, 0, 18, 20, NULL, 0, 0, 0, 0, TIP_("Next Pass"));
-	uiButSetFunc(but, image_multi_incpass_cb, rr, iuser);
+	but = uiDefIconBut(block, UI_BTYPE_BUT, 0, ICON_TRIA_LEFT,   0, 0, 0.85f * UI_UNIT_X, UI_UNIT_Y, NULL, 0, 0, 0, 0, TIP_("Previous Pass"));
+	UI_but_func_set(but, image_multi_decpass_cb, rr, iuser);
+	but = uiDefIconBut(block, UI_BTYPE_BUT, 0, ICON_TRIA_RIGHT,  0, 0, 0.90f * UI_UNIT_X, UI_UNIT_Y, NULL, 0, 0, 0, 0, TIP_("Next Pass"));
+	UI_but_func_set(but, image_multi_incpass_cb, rr, iuser);
 
-	uiBlockEndAlign(block);
+	UI_block_align_end(block);
 }
 
 // XXX HACK!
@@ -539,16 +640,17 @@ static void rna_update_cb(bContext *C, void *arg_cb, void *UNUSED(arg))
 
 void uiTemplateImage(uiLayout *layout, bContext *C, PointerRNA *ptr, const char *propname, PointerRNA *userptr, int compact)
 {
+#define MAX_INFO_LEN  128
+
 	PropertyRNA *prop;
 	PointerRNA imaptr;
 	RNAUpdateCb *cb;
 	Image *ima;
 	ImageUser *iuser;
-	ImBuf *ibuf;
 	Scene *scene = CTX_data_scene(C);
 	uiLayout *row, *split, *col;
 	uiBlock *block;
-	char str[128];
+	char str[MAX_INFO_LEN];
 
 	void *lock;
 
@@ -588,12 +690,12 @@ void uiTemplateImage(uiLayout *layout, bContext *C, PointerRNA *ptr, const char
 		uiTemplateID(layout, C, ptr, propname, "IMAGE_OT_new", "IMAGE_OT_open", NULL);
 
 	if (ima) {
-		uiBlockSetNFunc(block, rna_update_cb, MEM_dupallocN(cb), NULL);
+		UI_block_funcN_set(block, rna_update_cb, MEM_dupallocN(cb), NULL);
 
 		if (ima->source == IMA_SRC_VIEWER) {
-			ibuf = BKE_image_acquire_ibuf(ima, iuser, &lock);
-			image_info(scene, iuser, ima, ibuf, str);
-			BKE_image_release_ibuf(ima, lock);
+			ImBuf *ibuf = BKE_image_acquire_ibuf(ima, iuser, &lock);
+			image_info(scene, iuser, ima, ibuf, str, MAX_INFO_LEN);
+			BKE_image_release_ibuf(ima, ibuf, lock);
 
 			uiItemL(layout, ima->id.name + 2, ICON_NONE);
 			uiItemL(layout, str, ICON_NONE);
@@ -603,18 +705,18 @@ void uiTemplateImage(uiLayout *layout, bContext *C, PointerRNA *ptr, const char
 #if 0
 				iuser = ntree_get_active_iuser(scene->nodetree);
 				if (iuser) {
-					uiBlockBeginAlign(block);
-					uiDefIconTextBut(block, BUT, B_SIMA_RECORD, ICON_REC, "Record", 10, 120, 100, 20, 0, 0, 0, 0, 0, "");
-					uiDefIconTextBut(block, BUT, B_SIMA_PLAY, ICON_PLAY, "Play",    110, 120, 100, 20, 0, 0, 0, 0, 0, "");
-					but = uiDefBut(block, BUT, B_NOP, "Free Cache", 210, 120, 100, 20, 0, 0, 0, 0, 0, "");
-					uiButSetFunc(but, image_freecache_cb, ima, NULL);
+					UI_block_align_begin(block);
+					uiDefIconTextBut(block, UI_BTYPE_BUT, B_SIMA_RECORD, ICON_REC, "Record", 10, 120, 100, 20, 0, 0, 0, 0, 0, "");
+					uiDefIconTextBut(block, UI_BTYPE_BUT, B_SIMA_PLAY, ICON_PLAY, "Play",    110, 120, 100, 20, 0, 0, 0, 0, 0, "");
+					but = uiDefBut(block, UI_BTYPE_BUT, B_NOP, "Free Cache", 210, 120, 100, 20, 0, 0, 0, 0, 0, "");
+					UI_but_func_set(but, image_freecache_cb, ima, NULL);
 					
 					if (iuser->frames)
 						BLI_snprintf(str, sizeof(str), "(%d) Frames:", iuser->framenr);
 					else strcpy(str, "Frames:");
-					uiBlockBeginAlign(block);
-					uiDefButI(block, NUM, imagechanged, str,        10, 90, 150, 20, &iuser->frames, 0.0, MAXFRAMEF, 0, 0, "Number of images of a movie to use");
-					uiDefButI(block, NUM, imagechanged, "StartFr:", 160, 90, 150, 20, &iuser->sfra, 1.0, MAXFRAMEF, 0, 0, "Global starting frame of the movie");
+					UI_block_align_begin(block);
+					uiDefButI(block, UI_BTYPE_NUM, imagechanged, str,        10, 90, 150, 20, &iuser->frames, 0.0, MAXFRAMEF, 0, 0, "Number of images of a movie to use");
+					uiDefButI(block, UI_BTYPE_NUM, imagechanged, "StartFr:", 160, 90, 150, 20, &iuser->sfra, 1.0, MAXFRAMEF, 0, 0, "Global starting frame of the movie");
 				}
 #endif
 			}
@@ -624,7 +726,7 @@ void uiTemplateImage(uiLayout *layout, bContext *C, PointerRNA *ptr, const char
 
 				/* use BKE_image_acquire_renderresult  so we get the correct slot in the menu */
 				rr = BKE_image_acquire_renderresult(scene, ima);
-				uiblock_layer_pass_arrow_buttons(layout, rr, iuser, &ima->render_slot);
+				uiblock_layer_pass_arrow_buttons(layout, ima, rr, iuser, &ima->render_slot);
 				BKE_image_release_renderresult(scene, ima);
 			}
 		}
@@ -632,13 +734,13 @@ void uiTemplateImage(uiLayout *layout, bContext *C, PointerRNA *ptr, const char
 			uiItemR(layout, &imaptr, "source", 0, NULL, ICON_NONE);
 
 			if (ima->source != IMA_SRC_GENERATED) {
-				row = uiLayoutRow(layout, TRUE);
+				row = uiLayoutRow(layout, true);
 				if (ima->packedfile)
 					uiItemO(row, "", ICON_PACKAGE, "image.unpack");
 				else
 					uiItemO(row, "", ICON_UGLYPACKAGE, "image.pack");
 				
-				row = uiLayoutRow(row, FALSE);
+				row = uiLayoutRow(row, true);
 				uiLayoutSetEnabled(row, ima->packedfile == NULL);
 				uiItemR(row, &imaptr, "filepath", 0, "", ICON_NONE);
 				uiItemO(row, "", ICON_FILE_REFRESH, "image.reload");
@@ -657,84 +759,105 @@ void uiTemplateImage(uiLayout *layout, bContext *C, PointerRNA *ptr, const char
 
 			/* multilayer? */
 			if (ima->type == IMA_TYPE_MULTILAYER && ima->rr) {
-				uiblock_layer_pass_arrow_buttons(layout, ima->rr, iuser, NULL);
+				uiblock_layer_pass_arrow_buttons(layout, ima, ima->rr, iuser, NULL);
 			}
 			else if (ima->source != IMA_SRC_GENERATED) {
 				if (compact == 0) {
-					ibuf = BKE_image_acquire_ibuf(ima, iuser, &lock);
-					image_info(scene, iuser, ima, ibuf, str);
-					BKE_image_release_ibuf(ima, lock);
+					ImBuf *ibuf = BKE_image_acquire_ibuf(ima, iuser, &lock);
+					image_info(scene, iuser, ima, ibuf, str, MAX_INFO_LEN);
+					BKE_image_release_ibuf(ima, ibuf, lock);
 					uiItemL(layout, str, ICON_NONE);
 				}
 			}
 
-			col = uiLayoutColumn(layout, FALSE);
+			col = uiLayoutColumn(layout, false);
 			uiTemplateColorspaceSettings(col, &imaptr, "colorspace_settings");
-			uiItemR(col, &imaptr, "view_as_render", 0, NULL, ICON_NONE);
+			uiItemR(col, &imaptr, "use_view_as_render", 0, NULL, ICON_NONE);
 
 			if (ima->source != IMA_SRC_GENERATED) {
 				if (compact == 0) { /* background image view doesnt need these */
+					ImBuf *ibuf = BKE_image_acquire_ibuf(ima, iuser, NULL);
+					bool has_alpha = true;
+
+					if (ibuf) {
+						int imtype = BKE_ftype_to_imtype(ibuf->ftype);
+						char valid_channels = BKE_imtype_valid_channels(imtype, false);
+
+						has_alpha = (valid_channels & IMA_CHAN_FLAG_ALPHA) != 0;
+
+						BKE_image_release_ibuf(ima, ibuf, NULL);
+					}
+
+					if (has_alpha) {
+						col = uiLayoutColumn(layout, false);
+						uiItemR(col, &imaptr, "use_alpha", 0, NULL, ICON_NONE);
+						uiItemR(col, &imaptr, "alpha_mode", 0, IFACE_("Alpha"), ICON_NONE);
+					}
+
 					uiItemS(layout);
 
-					split = uiLayoutSplit(layout, 0.0f, FALSE);
+					split = uiLayoutSplit(layout, 0.0f, false);
 
-					col = uiLayoutColumn(split, FALSE);
+					col = uiLayoutColumn(split, false);
 					/* XXX Why only display fields_per_frame only for video image types?
 					 *     And why allow fields for non-video image types at all??? */
-					if (ELEM(ima->source, IMA_SRC_MOVIE, IMA_SRC_SEQUENCE)) {
-						uiLayout *subsplit = uiLayoutSplit(col, 0.0f, FALSE);
-						uiLayout *subcol = uiLayoutColumn(subsplit, FALSE);
+					if (BKE_image_is_animated(ima)) {
+						uiLayout *subsplit = uiLayoutSplit(col, 0.0f, false);
+						uiLayout *subcol = uiLayoutColumn(subsplit, false);
 						uiItemR(subcol, &imaptr, "use_fields", 0, NULL, ICON_NONE);
-						subcol = uiLayoutColumn(subsplit, FALSE);
+						subcol = uiLayoutColumn(subsplit, false);
 						uiLayoutSetActive(subcol, RNA_boolean_get(&imaptr, "use_fields"));
 						uiItemR(subcol, userptr, "fields_per_frame", 0, IFACE_("Fields"), ICON_NONE);
 					}
 					else
 						uiItemR(col, &imaptr, "use_fields", 0, NULL, ICON_NONE);
-					row = uiLayoutRow(col, FALSE);
+					row = uiLayoutRow(col, false);
 					uiLayoutSetActive(row, RNA_boolean_get(&imaptr, "use_fields"));
 					uiItemR(row, &imaptr, "field_order", UI_ITEM_R_EXPAND, NULL, ICON_NONE);
-					
-					row = uiLayoutRow(layout, FALSE);
-					uiItemR(row, &imaptr, "use_premultiply", 0, NULL, ICON_NONE);
-					uiItemR(row, &imaptr, "use_color_unpremultiply", 0, NULL, ICON_NONE);
 				}
 			}
 
-			if (ELEM(ima->source, IMA_SRC_MOVIE, IMA_SRC_SEQUENCE)) {
+			if (BKE_image_is_animated(ima)) {
 				uiItemS(layout);
 
-				split = uiLayoutSplit(layout, 0.0f, FALSE);
+				split = uiLayoutSplit(layout, 0.0f, false);
 
-				col = uiLayoutColumn(split, FALSE);
+				col = uiLayoutColumn(split, false);
 
 				BLI_snprintf(str, sizeof(str), IFACE_("(%d) Frames"), iuser->framenr);
 				uiItemR(col, userptr, "frame_duration", 0, str, ICON_NONE);
 				uiItemR(col, userptr, "frame_start", 0, IFACE_("Start"), ICON_NONE);
 				uiItemR(col, userptr, "frame_offset", 0, NULL, ICON_NONE);
 
-				col = uiLayoutColumn(split, FALSE);
+				col = uiLayoutColumn(split, false);
 				uiItemO(col, NULL, ICON_NONE, "IMAGE_OT_match_movie_length");
 				uiItemR(col, userptr, "use_auto_refresh", 0, NULL, ICON_NONE);
 				uiItemR(col, userptr, "use_cyclic", 0, NULL, ICON_NONE);
 			}
 			else if (ima->source == IMA_SRC_GENERATED) {
-				split = uiLayoutSplit(layout, 0.0f, FALSE);
+				split = uiLayoutSplit(layout, 0.0f, false);
 
-				col = uiLayoutColumn(split, TRUE);
+				col = uiLayoutColumn(split, true);
 				uiItemR(col, &imaptr, "generated_width", 0, "X", ICON_NONE);
 				uiItemR(col, &imaptr, "generated_height", 0, "Y", ICON_NONE);
+				
 				uiItemR(col, &imaptr, "use_generated_float", 0, NULL, ICON_NONE);
 
 				uiItemR(split, &imaptr, "generated_type", UI_ITEM_R_EXPAND, NULL, ICON_NONE);
+
+				if (ima->gen_type == IMA_GENTYPE_BLANK) {
+					uiItemR(layout, &imaptr, "generated_color", 0, NULL, ICON_NONE);
+				}
 			}
 
 		}
 
-		uiBlockSetNFunc(block, NULL, NULL, NULL);
+		UI_block_funcN_set(block, NULL, NULL, NULL);
 	}
 
 	MEM_freeN(cb);
+
+#undef MAX_INFO_LEN
 }
 
 void uiTemplateImageSettings(uiLayout *layout, PointerRNA *imfptr, int color_management)
@@ -745,29 +868,32 @@ void uiTemplateImageSettings(uiLayout *layout, PointerRNA *imfptr, int color_man
 	PropertyRNA *prop;
 	const int depth_ok = BKE_imtype_valid_depths(imf->imtype);
 	/* some settings depend on this being a scene thats rendered */
-	const short is_render_out = (id && GS(id->name) == ID_SCE);
+	const bool is_render_out = (id && GS(id->name) == ID_SCE);
 
 	uiLayout *col, *row, *split, *sub;
-	int show_preview = FALSE;
+	bool show_preview = false;
 
-	col = uiLayoutColumn(layout, FALSE);
+	col = uiLayoutColumn(layout, false);
 
-	split = uiLayoutSplit(col, 0.5f, FALSE);
+	split = uiLayoutSplit(col, 0.5f, false);
 	
 	uiItemR(split, imfptr, "file_format", 0, "", ICON_NONE);
-	sub = uiLayoutRow(split, FALSE);
+	sub = uiLayoutRow(split, false);
 	uiItemR(sub, imfptr, "color_mode", UI_ITEM_R_EXPAND, IFACE_("Color"), ICON_NONE);
 
 	/* only display depth setting if multiple depths can be used */
-	if ((ELEM6(depth_ok,
-	           R_IMF_CHAN_DEPTH_1,
-	           R_IMF_CHAN_DEPTH_8,
-	           R_IMF_CHAN_DEPTH_12,
-	           R_IMF_CHAN_DEPTH_16,
-	           R_IMF_CHAN_DEPTH_24,
-	           R_IMF_CHAN_DEPTH_32)) == 0)
+	if ((ELEM(depth_ok,
+	          R_IMF_CHAN_DEPTH_1,
+	          R_IMF_CHAN_DEPTH_8,
+	          R_IMF_CHAN_DEPTH_10,
+	          R_IMF_CHAN_DEPTH_12,
+	          R_IMF_CHAN_DEPTH_16,
+	          R_IMF_CHAN_DEPTH_24,
+	          R_IMF_CHAN_DEPTH_32)) == 0)
 	{
-		row = uiLayoutRow(col, FALSE);
+		row = uiLayoutRow(col, false);
+
+		uiItemL(row, IFACE_("Color Depth:"), ICON_NONE);
 		uiItemR(row, imfptr, "color_depth", UI_ITEM_R_EXPAND, NULL, ICON_NONE);
 	}
 
@@ -783,27 +909,33 @@ void uiTemplateImageSettings(uiLayout *layout, PointerRNA *imfptr, int color_man
 		uiItemR(col, imfptr, "exr_codec", 0, NULL, ICON_NONE);
 	}
 	
-	row = uiLayoutRow(col, FALSE);
+	row = uiLayoutRow(col, false);
 	if (BKE_imtype_supports_zbuf(imf->imtype)) {
 		uiItemR(row, imfptr, "use_zbuffer", 0, NULL, ICON_NONE);
 	}
 
 	if (is_render_out && (imf->imtype == R_IMF_IMTYPE_OPENEXR)) {
-		show_preview = TRUE;
+		show_preview = true;
 		uiItemR(row, imfptr, "use_preview", 0, NULL, ICON_NONE);
 	}
 
 	if (imf->imtype == R_IMF_IMTYPE_JP2) {
-		row = uiLayoutRow(col, FALSE);
+		uiItemR(col, imfptr, "jpeg2k_codec", 0, NULL, ICON_NONE);
+
+		row = uiLayoutRow(col, false);
 		uiItemR(row, imfptr, "use_jpeg2k_cinema_preset", 0, NULL, ICON_NONE);
 		uiItemR(row, imfptr, "use_jpeg2k_cinema_48", 0, NULL, ICON_NONE);
 		
 		uiItemR(col, imfptr, "use_jpeg2k_ycc", 0, NULL, ICON_NONE);
 	}
 
+	if (imf->imtype == R_IMF_IMTYPE_DPX) {
+		uiItemR(col, imfptr, "use_cineon_log", 0, NULL, ICON_NONE);
+	}
+
 	if (imf->imtype == R_IMF_IMTYPE_CINEON) {
 #if 1
-		uiItemL(col, IFACE_("Hard coded Non-Linear, Gamma: 1.0"), ICON_NONE);
+		uiItemL(col, IFACE_("Hard coded Non-Linear, Gamma:1.7"), ICON_NONE);
 #else
 		uiItemR(col, imfptr, "use_cineon_log", 0, NULL, ICON_NONE);
 		uiItemR(col, imfptr, "cineon_black", 0, NULL, ICON_NONE);
@@ -820,7 +952,7 @@ void uiTemplateImageSettings(uiLayout *layout, PointerRNA *imfptr, int color_man
 		prop = RNA_struct_find_property(imfptr, "display_settings");
 		display_settings_ptr = RNA_property_pointer_get(imfptr, prop);
 
-		col = uiLayoutColumn(layout, FALSE);
+		col = uiLayoutColumn(layout, false);
 		uiItemL(col, IFACE_("Color Management"), ICON_NONE);
 
 		uiItemR(col, &display_settings_ptr, "display_device", 0, NULL, ICON_NONE);
@@ -840,23 +972,17 @@ void uiTemplateImageLayers(uiLayout *layout, bContext *C, Image *ima, ImageUser
 
 		/* use BKE_image_acquire_renderresult  so we get the correct slot in the menu */
 		rr = BKE_image_acquire_renderresult(scene, ima);
-		uiblock_layer_pass_buttons(layout, rr, iuser, 160 * dpi_fac, (ima->type == IMA_TYPE_R_RESULT) ? &ima->render_slot : NULL);
+		uiblock_layer_pass_buttons(layout, ima, rr, iuser, 160 * dpi_fac, (ima->type == IMA_TYPE_R_RESULT) ? &ima->render_slot : NULL);
 		BKE_image_release_renderresult(scene, ima);
 	}
 }
 
-void image_buttons_register(ARegionType *art)
+void image_buttons_register(ARegionType *UNUSED(art))
 {
-	PanelType *pt;
-
-	pt = MEM_callocN(sizeof(PanelType), "spacetype image panel gpencil");
-	strcpy(pt->idname, "IMAGE_PT_gpencil");
-	strcpy(pt->label, "Grease Pencil");
-	pt->draw = gpencil_panel_standard;
-	BLI_addtail(&art->paneltypes, pt);
+	
 }
 
-static int image_properties(bContext *C, wmOperator *UNUSED(op))
+static int image_properties_toggle_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	ScrArea *sa = CTX_wm_area(C);
 	ARegion *ar = image_has_buttons_region(sa);
@@ -873,17 +999,17 @@ void IMAGE_OT_properties(wmOperatorType *ot)
 	ot->idname = "IMAGE_OT_properties";
 	ot->description = "Toggle display properties panel";
 	
-	ot->exec = image_properties;
+	ot->exec = image_properties_toggle_exec;
 	ot->poll = ED_operator_image_active;
 	
 	/* flags */
 	ot->flag = 0;
 }
 
-static int image_scopes(bContext *C, wmOperator *UNUSED(op))
+static int image_scopes_toggle_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	ScrArea *sa = CTX_wm_area(C);
-	ARegion *ar = image_has_scope_region(sa);
+	ARegion *ar = image_has_tools_region(sa);
 	
 	if (ar)
 		ED_region_toggle_hidden(C, ar);
@@ -891,13 +1017,13 @@ static int image_scopes(bContext *C, wmOperator *UNUSED(op))
 	return OPERATOR_FINISHED;
 }
 
-void IMAGE_OT_scopes(wmOperatorType *ot)
+void IMAGE_OT_toolshelf(wmOperatorType *ot)
 {
-	ot->name = "Scopes";
-	ot->idname = "IMAGE_OT_scopes";
-	ot->description = "Toggle display scopes panel";
-	
-	ot->exec = image_scopes;
+	ot->name = "Tool Shelf";
+	ot->idname = "IMAGE_OT_toolshelf";
+	ot->description = "Toggles tool shelf display";
+
+	ot->exec = image_scopes_toggle_exec;
 	ot->poll = ED_operator_image_active;
 	
 	/* flags */
