@@ -40,6 +40,7 @@
 #include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_brush_types.h"
+#include "DNA_mask_types.h"
 
 #include "PIL_time.h"
 
@@ -52,6 +53,7 @@
 #include "IMB_imbuf.h"
 #include "IMB_imbuf_types.h"
 #include "IMB_colormanagement.h"
+#include "IMB_moviecache.h"
 
 #include "BKE_context.h"
 #include "BKE_global.h"
@@ -65,49 +67,88 @@
 
 #include "ED_gpencil.h"
 #include "ED_image.h"
+#include "ED_mask.h"
 #include "ED_screen.h"
 
 #include "UI_interface.h"
 #include "UI_resources.h"
 #include "UI_view2d.h"
 
-#include "WM_api.h"
-#include "WM_types.h"
-
 #include "RE_pipeline.h"
+#include "RE_engine.h"
 
 #include "image_intern.h"
 
-static void draw_render_info(Scene *scene, Image *ima, ARegion *ar)
+static void draw_render_info(Scene *scene, Image *ima, ARegion *ar, float zoomx, float zoomy)
 {
 	RenderResult *rr;
-	
+	Render *re = RE_GetRender(scene->id.name);
+	RenderData *rd = RE_engine_get_render_data(re);
+
 	rr = BKE_image_acquire_renderresult(scene, ima);
 
 	if (rr && rr->text) {
-		ED_region_info_draw(ar, rr->text, 1, 0.25);
+		float fill_color[4] = {0.0f, 0.0f, 0.0f, 0.25f};
+		ED_region_info_draw(ar, rr->text, 1, fill_color);
 	}
 
 	BKE_image_release_renderresult(scene, ima);
+
+	if (re) {
+		int total_tiles;
+		rcti *tiles;
+
+		RE_engine_get_current_tiles(re, &total_tiles, &tiles);
+
+		if (total_tiles) {
+			int i, x, y;
+			rcti *tile;
+
+			/* find window pixel coordinates of origin */
+			UI_view2d_view_to_region(&ar->v2d, 0.0f, 0.0f, &x, &y);
+
+			glPushMatrix();
+			glTranslatef(x, y, 0.0f);
+			glScalef(zoomx, zoomy, 1.0f);
+
+			if (rd->mode & R_BORDER) {
+				glTranslatef((int)(-rd->border.xmin * rd->xsch * rd->size / 100.0f),
+				             (int)(-rd->border.ymin * rd->ysch * rd->size / 100.0f),
+				             0.0f);
+			}
+
+			UI_ThemeColor(TH_FACE_SELECT);
+
+			for (i = 0, tile = tiles; i < total_tiles; i++, tile++) {
+				glaDrawBorderCorners(tile, zoomx, zoomy);
+			}
+
+			MEM_freeN(tiles);
+
+			glPopMatrix();
+		}
+	}
 }
 
 /* used by node view too */
-void ED_image_draw_info(Scene *scene, ARegion *ar, int color_manage, int use_default_view, int channels, int x, int y,
-                        const unsigned char cp[4], const float fp[4], int *zp, float *zpf)
+void ED_image_draw_info(Scene *scene, ARegion *ar, bool color_manage, bool use_default_view, int channels, int x, int y,
+                        const unsigned char cp[4], const float fp[4], const float linearcol[4], int *zp, float *zpf)
 {
+	rcti color_rect;
 	char str[256];
-	float dx = 6;
+	int dx = 6;
+	const int dy = 0.3f * UI_UNIT_Y;
 	/* text colors */
 	/* XXX colored text not allowed in Blender UI */
-	#if 0
+#if 0
 	unsigned char red[3] = {255, 50, 50};
 	unsigned char green[3] = {0, 255, 0};
 	unsigned char blue[3] = {100, 100, 255};
-	#else
+#else
 	unsigned char red[3] = {255, 255, 255};
 	unsigned char green[3] = {255, 255, 255};
 	unsigned char blue[3] = {255, 255, 255};
-	#endif
+#endif
 	float hue = 0, sat = 0, val = 0, lum = 0, u = 0, v = 0;
 	float col[4], finalcol[4];
 
@@ -116,66 +157,65 @@ void ED_image_draw_info(Scene *scene, ARegion *ar, int color_manage, int use_def
 
 	/* noisy, high contrast make impossible to read if lower alpha is used. */
 	glColor4ub(0, 0, 0, 190);
-	glRecti(0.0, 0.0, BLI_rcti_size_x(&ar->winrct) + 1, 20);
+	glRecti(0.0, 0.0, BLI_rcti_size_x(&ar->winrct) + 1, UI_UNIT_Y);
 	glDisable(GL_BLEND);
 
-	BLF_size(blf_mono_font, 11, 72);
+	BLF_size(blf_mono_font, 11 * U.pixelsize, U.dpi);
 
 	glColor3ub(255, 255, 255);
 	BLI_snprintf(str, sizeof(str), "X:%-4d  Y:%-4d |", x, y);
-	// UI_DrawString(6, 6, str); // works ok but fixed width is nicer.
-	BLF_position(blf_mono_font, dx, 6, 0);
+	BLF_position(blf_mono_font, dx, dy, 0);
 	BLF_draw_ascii(blf_mono_font, str, sizeof(str));
-	dx += BLF_width(blf_mono_font, str);
+	dx += BLF_width(blf_mono_font, str, sizeof(str));
 
 	if (zp) {
 		glColor3ub(255, 255, 255);
 		BLI_snprintf(str, sizeof(str), " Z:%-.4f |", 0.5f + 0.5f * (((float)*zp) / (float)0x7fffffff));
-		BLF_position(blf_mono_font, dx, 6, 0);
+		BLF_position(blf_mono_font, dx, dy, 0);
 		BLF_draw_ascii(blf_mono_font, str, sizeof(str));
-		dx += BLF_width(blf_mono_font, str);
+		dx += BLF_width(blf_mono_font, str, sizeof(str));
 	}
 	if (zpf) {
 		glColor3ub(255, 255, 255);
 		BLI_snprintf(str, sizeof(str), " Z:%-.3f |", *zpf);
-		BLF_position(blf_mono_font, dx, 6, 0);
+		BLF_position(blf_mono_font, dx, dy, 0);
 		BLF_draw_ascii(blf_mono_font, str, sizeof(str));
-		dx += BLF_width(blf_mono_font, str);
+		dx += BLF_width(blf_mono_font, str, sizeof(str));
 	}
 
 	if (channels >= 3) {
 		glColor3ubv(red);
 		if (fp)
-			BLI_snprintf(str, sizeof(str), "  R:%-.4f", fp[0]);
+			BLI_snprintf(str, sizeof(str), "  R:%-.5f", fp[0]);
 		else if (cp)
 			BLI_snprintf(str, sizeof(str), "  R:%-3d", cp[0]);
 		else
 			BLI_snprintf(str, sizeof(str), "  R:-");
-		BLF_position(blf_mono_font, dx, 6, 0);
+		BLF_position(blf_mono_font, dx, dy, 0);
 		BLF_draw_ascii(blf_mono_font, str, sizeof(str));
-		dx += BLF_width(blf_mono_font, str);
+		dx += BLF_width(blf_mono_font, str, sizeof(str));
 		
 		glColor3ubv(green);
 		if (fp)
-			BLI_snprintf(str, sizeof(str), "  G:%-.4f", fp[1]);
+			BLI_snprintf(str, sizeof(str), "  G:%-.5f", fp[1]);
 		else if (cp)
 			BLI_snprintf(str, sizeof(str), "  G:%-3d", cp[1]);
 		else
 			BLI_snprintf(str, sizeof(str), "  G:-");
-		BLF_position(blf_mono_font, dx, 6, 0);
+		BLF_position(blf_mono_font, dx, dy, 0);
 		BLF_draw_ascii(blf_mono_font, str, sizeof(str));
-		dx += BLF_width(blf_mono_font, str);
+		dx += BLF_width(blf_mono_font, str, sizeof(str));
 		
 		glColor3ubv(blue);
 		if (fp)
-			BLI_snprintf(str, sizeof(str), "  B:%-.4f", fp[2]);
+			BLI_snprintf(str, sizeof(str), "  B:%-.5f", fp[2]);
 		else if (cp)
 			BLI_snprintf(str, sizeof(str), "  B:%-3d", cp[2]);
 		else
 			BLI_snprintf(str, sizeof(str), "  B:-");
-		BLF_position(blf_mono_font, dx, 6, 0);
+		BLF_position(blf_mono_font, dx, dy, 0);
 		BLF_draw_ascii(blf_mono_font, str, sizeof(str));
-		dx += BLF_width(blf_mono_font, str);
+		dx += BLF_width(blf_mono_font, str, sizeof(str));
 		
 		if (channels == 4) {
 			glColor3ub(255, 255, 255);
@@ -185,23 +225,29 @@ void ED_image_draw_info(Scene *scene, ARegion *ar, int color_manage, int use_def
 				BLI_snprintf(str, sizeof(str), "  A:%-3d", cp[3]);
 			else
 				BLI_snprintf(str, sizeof(str), "- ");
-			BLF_position(blf_mono_font, dx, 6, 0);
+			BLF_position(blf_mono_font, dx, dy, 0);
 			BLF_draw_ascii(blf_mono_font, str, sizeof(str));
-			dx += BLF_width(blf_mono_font, str);
+			dx += BLF_width(blf_mono_font, str, sizeof(str));
 		}
 
-		if (color_manage && channels == 4) {
-			float pixel[4];
+		if (color_manage) {
+			float rgba[4];
+
+			copy_v3_v3(rgba, linearcol);
+			if (channels == 3)
+				rgba[3] = 1.0f;
+			else
+				rgba[3] = linearcol[3];
 
 			if (use_default_view)
-				IMB_colormanagement_pixel_to_display_space_v4(pixel, fp,  NULL, &scene->display_settings);
+				IMB_colormanagement_pixel_to_display_space_v4(rgba, rgba,  NULL, &scene->display_settings);
 			else
-				IMB_colormanagement_pixel_to_display_space_v4(pixel, fp,  &scene->view_settings, &scene->display_settings);
+				IMB_colormanagement_pixel_to_display_space_v4(rgba, rgba,  &scene->view_settings, &scene->display_settings);
 
-			BLI_snprintf(str, sizeof(str), "  |  CM  R:%-.4f  G:%-.4f  B:%-.4f", pixel[0], pixel[1], pixel[2]);
-			BLF_position(blf_mono_font, dx, 6, 0);
+			BLI_snprintf(str, sizeof(str), "  |  CM  R:%-.4f  G:%-.4f  B:%-.4f", rgba[0], rgba[1], rgba[2]);
+			BLF_position(blf_mono_font, dx, dy, 0);
 			BLF_draw_ascii(blf_mono_font, str, sizeof(str));
-			dx += BLF_width(blf_mono_font, str);
+			dx += BLF_width(blf_mono_font, str, sizeof(str));
 		}
 	}
 	
@@ -219,26 +265,11 @@ void ED_image_draw_info(Scene *scene, ARegion *ar, int color_manage, int use_def
 		col[3] = 1.0f;
 	}
 	else if (channels == 3) {
-		if (fp) {
-			copy_v3_v3(col, fp);
-		}
-		else if (cp) {
-			rgb_uchar_to_float(col, cp);
-		}
-		else {
-			zero_v3(col);
-		}
+		copy_v3_v3(col, linearcol);
 		col[3] = 1.0f;
 	}
 	else if (channels == 4) {
-		if (fp)
-			copy_v4_v4(col, fp);
-		else if (cp) {
-			rgba_uchar_to_float(col, cp);
-		}
-		else {
-			zero_v4(col);
-		}
+		copy_v4_v4(col, linearcol);
 	}
 	else {
 		BLI_assert(0);
@@ -256,25 +287,49 @@ void ED_image_draw_info(Scene *scene, ARegion *ar, int color_manage, int use_def
 	}
 
 	glDisable(GL_BLEND);
-	glColor3fv(finalcol);
-	dx += 5;
-	glBegin(GL_QUADS);
-	glVertex2f(dx, 3);
-	glVertex2f(dx, 17);
-	glVertex2f(dx + 30, 17);
-	glVertex2f(dx + 30, 3);
-	glEnd();
+	dx += 0.25f * UI_UNIT_X;
+
+	BLI_rcti_init(&color_rect, dx, dx + (1.5f * UI_UNIT_X), 0.15f * UI_UNIT_Y, 0.85f * UI_UNIT_Y);
+
+	if (channels == 4) {
+		rcti color_rect_half;
+		int color_quater_x, color_quater_y;
+
+		color_rect_half = color_rect;
+		color_rect_half.xmax = BLI_rcti_cent_x(&color_rect);
+		glRecti(color_rect.xmin, color_rect.ymin, color_rect.xmax, color_rect.ymax);
+
+		color_rect_half = color_rect;
+		color_rect_half.xmin = BLI_rcti_cent_x(&color_rect);
+
+		color_quater_x = BLI_rcti_cent_x(&color_rect_half);
+		color_quater_y = BLI_rcti_cent_y(&color_rect_half);
+
+		glColor4ub(UI_ALPHA_CHECKER_DARK, UI_ALPHA_CHECKER_DARK, UI_ALPHA_CHECKER_DARK, 255);
+		glRecti(color_rect_half.xmin, color_rect_half.ymin, color_rect_half.xmax, color_rect_half.ymax);
+
+		glColor4ub(UI_ALPHA_CHECKER_LIGHT, UI_ALPHA_CHECKER_LIGHT, UI_ALPHA_CHECKER_LIGHT, 255);
+		glRecti(color_quater_x, color_quater_y, color_rect_half.xmax, color_rect_half.ymax);
+		glRecti(color_rect_half.xmin, color_rect_half.ymin, color_quater_x, color_quater_y);
+
+		glEnable(GL_BLEND);
+		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+		glColor4f(UNPACK3(finalcol), fp ? fp[3] : (cp[3] / 255.0f));
+		glRecti(color_rect.xmin, color_rect.ymin, color_rect.xmax, color_rect.ymax);
+		glDisable(GL_BLEND);
+	}
+	else {
+		glColor3fv(finalcol);
+		glRecti(color_rect.xmin, color_rect.ymin, color_rect.xmax, color_rect.ymax);
+	}
 
 	/* draw outline */
 	glColor3ub(128, 128, 128);
-	glBegin(GL_LINE_LOOP);
-	glVertex2f(dx, 3);
-	glVertex2f(dx, 17);
-	glVertex2f(dx + 30, 17);
-	glVertex2f(dx + 30, 3);
-	glEnd();
+	glPolygonMode(GL_FRONT_AND_BACK, GL_LINE);
+	glRecti(color_rect.xmin, color_rect.ymin, color_rect.xmax, color_rect.ymax);
+	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL);
 
-	dx += 35;
+	dx += 1.75f * UI_UNIT_X;
 
 	glColor3ub(255, 255, 255);
 	if (channels == 1) {
@@ -288,44 +343,38 @@ void ED_image_draw_info(Scene *scene, ARegion *ar, int color_manage, int use_def
 		}
 		
 		BLI_snprintf(str, sizeof(str), "V:%-.4f", val);
-		BLF_position(blf_mono_font, dx, 6, 0);
+		BLF_position(blf_mono_font, dx, dy, 0);
 		BLF_draw_ascii(blf_mono_font, str, sizeof(str));
-		dx += BLF_width(blf_mono_font, str);
+		dx += BLF_width(blf_mono_font, str, sizeof(str));
 
 		BLI_snprintf(str, sizeof(str), "   L:%-.4f", lum);
-		BLF_position(blf_mono_font, dx, 6, 0);
+		BLF_position(blf_mono_font, dx, dy, 0);
 		BLF_draw_ascii(blf_mono_font, str, sizeof(str));
-		dx += BLF_width(blf_mono_font, str);
+		dx += BLF_width(blf_mono_font, str, sizeof(str));
 	}
 	else if (channels >= 3) {
-		if (fp) {
-			rgb_to_hsv(fp[0], fp[1], fp[2], &hue, &sat, &val);
-			rgb_to_yuv(fp[0], fp[1], fp[2], &lum, &u, &v);
-		}
-		else if (cp) {
-			rgb_to_hsv((float)cp[0] / 255.0f, (float)cp[1] / 255.0f, (float)cp[2] / 255.0f, &hue, &sat, &val);
-			rgb_to_yuv((float)cp[0] / 255.0f, (float)cp[1] / 255.0f, (float)cp[2] / 255.0f, &lum, &u, &v);
-		}
+		rgb_to_hsv(finalcol[0], finalcol[1], finalcol[2], &hue, &sat, &val);
+		rgb_to_yuv(finalcol[0], finalcol[1], finalcol[2], &lum, &u, &v);
 
 		BLI_snprintf(str, sizeof(str), "H:%-.4f", hue);
-		BLF_position(blf_mono_font, dx, 6, 0);
+		BLF_position(blf_mono_font, dx, dy, 0);
 		BLF_draw_ascii(blf_mono_font, str, sizeof(str));
-		dx += BLF_width(blf_mono_font, str);
+		dx += BLF_width(blf_mono_font, str, sizeof(str));
 
 		BLI_snprintf(str, sizeof(str), "  S:%-.4f", sat);
-		BLF_position(blf_mono_font, dx, 6, 0);
+		BLF_position(blf_mono_font, dx, dy, 0);
 		BLF_draw_ascii(blf_mono_font, str, sizeof(str));
-		dx += BLF_width(blf_mono_font, str);
+		dx += BLF_width(blf_mono_font, str, sizeof(str));
 
 		BLI_snprintf(str, sizeof(str), "  V:%-.4f", val);
-		BLF_position(blf_mono_font, dx, 6, 0);
+		BLF_position(blf_mono_font, dx, dy, 0);
 		BLF_draw_ascii(blf_mono_font, str, sizeof(str));
-		dx += BLF_width(blf_mono_font, str);
+		dx += BLF_width(blf_mono_font, str, sizeof(str));
 
 		BLI_snprintf(str, sizeof(str), "   L:%-.4f", lum);
-		BLF_position(blf_mono_font, dx, 6, 0);
+		BLF_position(blf_mono_font, dx, dy, 0);
 		BLF_draw_ascii(blf_mono_font, str, sizeof(str));
-		dx += BLF_width(blf_mono_font, str);
+		dx += BLF_width(blf_mono_font, str, sizeof(str));
 	}
 
 	(void)dx;
@@ -356,11 +405,11 @@ static void sima_draw_alpha_pixelsf(float x1, float y1, int rectx, int recty, fl
 	MEM_freeN(trectf);
 	/* ogl trick below is slower... (on ATI 9600) */
 //	glColorMask(1, 0, 0, 0);
-//	glaDrawPixelsSafe(x1, y1, rectx, recty, rectx, GL_RGBA, GL_FLOAT, rectf+3);
+//	glaDrawPixelsSafe(x1, y1, rectx, recty, rectx, GL_RGBA, GL_FLOAT, rectf + 3);
 //	glColorMask(0, 1, 0, 0);
-//	glaDrawPixelsSafe(x1, y1, rectx, recty, rectx, GL_RGBA, GL_FLOAT, rectf+2);
+//	glaDrawPixelsSafe(x1, y1, rectx, recty, rectx, GL_RGBA, GL_FLOAT, rectf + 2);
 //	glColorMask(0, 0, 1, 0);
-//	glaDrawPixelsSafe(x1, y1, rectx, recty, rectx, GL_RGBA, GL_FLOAT, rectf+1);
+//	glaDrawPixelsSafe(x1, y1, rectx, recty, rectx, GL_RGBA, GL_FLOAT, rectf + 1);
 //	glColorMask(1, 1, 1, 1);
 }
 
@@ -423,8 +472,10 @@ static void draw_image_buffer(const bContext *C, SpaceImage *sima, ARegion *ar,
 	/* set zoom */
 	glPixelZoom(zoomx, zoomy);
 
+	glaDefine2DArea(&ar->winrct);
+	
 	/* find window pixel coordinates of origin */
-	UI_view2d_to_region_no_clip(&ar->v2d, fx, fy, &x, &y);
+	UI_view2d_view_to_region(&ar->v2d, fx, fy, &x, &y);
 
 	/* this part is generic image display */
 	if (sima->flag & SI_SHOW_ALPHA) {
@@ -442,26 +493,14 @@ static void draw_image_buffer(const bContext *C, SpaceImage *sima, ARegion *ar,
 			sima_draw_zbuffloat_pixels(scene, x, y, ibuf->x, ibuf->y, ibuf->rect_float);
 	}
 	else {
-		unsigned char *display_buffer;
-		void *cache_handle;
-
 		if (sima->flag & SI_USE_ALPHA) {
-			fdrawcheckerboard(x, y, x + ibuf->x * zoomx, y + ibuf->y * zoomy);
-
 			glEnable(GL_BLEND);
 			glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+
+			fdrawcheckerboard(x, y, x + ibuf->x * zoomx, y + ibuf->y * zoomy);
 		}
 
-		display_buffer = IMB_display_buffer_acquire_ctx(C, ibuf, &cache_handle);
-
-		if (display_buffer)
-			glaDrawPixelsSafe(x, y, ibuf->x, ibuf->y, ibuf->x, GL_RGBA, GL_UNSIGNED_BYTE, display_buffer);
-#if 0
-		else
-			glaDrawPixelsSafe(x, y, ibuf->x, ibuf->y, ibuf->x, GL_RGBA, GL_FLOAT, ibuf->rect_float);
-#endif
-
-		IMB_display_buffer_release(cache_handle);
+		glaDrawImBuf_glsl_ctx(C, ibuf, x, y, GL_NEAREST);
 
 		if (sima->flag & SI_USE_ALPHA)
 			glDisable(GL_BLEND);
@@ -518,8 +557,8 @@ static void draw_image_buffer_tiled(SpaceImage *sima, ARegion *ar, Scene *scene,
 		sima->curtile = ima->xrep * ima->yrep - 1;
 	
 	/* retrieve part of image buffer */
-	dx = ibuf->x / ima->xrep;
-	dy = ibuf->y / ima->yrep;
+	dx = max_ii(ibuf->x / ima->xrep, 1);
+	dy = max_ii(ibuf->y / ima->yrep, 1);
 	sx = (sima->curtile % ima->xrep) * dx;
 	sy = (sima->curtile / ima->xrep) * dy;
 	rect = get_part_from_buffer((unsigned int *)display_buffer, ibuf->x, sx, sy, sx + dx, sy + dy);
@@ -527,7 +566,7 @@ static void draw_image_buffer_tiled(SpaceImage *sima, ARegion *ar, Scene *scene,
 	/* draw repeated */
 	for (sy = 0; sy + dy <= ibuf->y; sy += dy) {
 		for (sx = 0; sx + dx <= ibuf->x; sx += dx) {
-			UI_view2d_to_region_no_clip(&ar->v2d, fx + (float)sx / (float)ibuf->x, fy + (float)sy / (float)ibuf->y, &x, &y);
+			UI_view2d_view_to_region(&ar->v2d, fx + (float)sx / (float)ibuf->x, fy + (float)sy / (float)ibuf->y, &x, &y);
 
 			glaDrawPixelsSafe(x, y, dx, dy, dx, GL_RGBA, GL_UNSIGNED_BYTE, rect);
 		}
@@ -569,19 +608,19 @@ static void draw_image_buffer_repeated(const bContext *C, SpaceImage *sima, AReg
 /* draw uv edit */
 
 /* draw grease pencil */
-void draw_image_grease_pencil(bContext *C, short onlyv2d)
+void draw_image_grease_pencil(bContext *C, bool onlyv2d)
 {
 	/* draw in View2D space? */
 	if (onlyv2d) {
 		/* draw grease-pencil ('image' strokes) */
-		draw_gpencil_2dimage(C);
+		ED_gpencil_draw_2dimage(C);
 	}
 	else {
 		/* assume that UI_view2d_restore(C) has been called... */
-		//SpaceImage *sima= (SpaceImage *)CTX_wm_space_data(C);
+		//SpaceImage *sima = (SpaceImage *)CTX_wm_space_data(C);
 		
 		/* draw grease-pencil ('screen' strokes) */
-		draw_gpencil_view2d(C, 0);
+		ED_gpencil_draw_view2d(C, 0);
 	}
 }
 
@@ -642,7 +681,7 @@ static void draw_image_view_tool(Scene *scene)
 
 static unsigned char *get_alpha_clone_image(const bContext *C, Scene *scene, int *width, int *height)
 {
-	Brush *brush = paint_brush(&scene->toolsettings->imapaint.paint);
+	Brush *brush = BKE_paint_brush(&scene->toolsettings->imapaint.paint);
 	ImBuf *ibuf;
 	unsigned int size, alpha;
 	unsigned char *display_buffer;
@@ -652,7 +691,7 @@ static unsigned char *get_alpha_clone_image(const bContext *C, Scene *scene, int
 	if (!brush || !brush->clone.image)
 		return NULL;
 	
-	ibuf = BKE_image_get_ibuf(brush->clone.image, NULL);
+	ibuf = BKE_image_acquire_ibuf(brush->clone.image, NULL, NULL);
 
 	if (!ibuf)
 		return NULL;
@@ -660,6 +699,7 @@ static unsigned char *get_alpha_clone_image(const bContext *C, Scene *scene, int
 	display_buffer = IMB_display_buffer_acquire_ctx(C, ibuf, &cache_handle);
 
 	if (!display_buffer) {
+		BKE_image_release_ibuf(brush->clone.image, ibuf, NULL);
 		IMB_display_buffer_release(cache_handle);
 
 		return NULL;
@@ -669,8 +709,10 @@ static unsigned char *get_alpha_clone_image(const bContext *C, Scene *scene, int
 
 	IMB_display_buffer_release(cache_handle);
 
-	if (!rect)
+	if (!rect) {
+		BKE_image_release_ibuf(brush->clone.image, ibuf, NULL);
 		return NULL;
+	}
 
 	*width = ibuf->x;
 	*height = ibuf->y;
@@ -684,6 +726,8 @@ static unsigned char *get_alpha_clone_image(const bContext *C, Scene *scene, int
 		cp += 4;
 	}
 
+	BKE_image_release_ibuf(brush->clone.image, ibuf, NULL);
+
 	return rect;
 }
 
@@ -693,7 +737,7 @@ static void draw_image_paint_helpers(const bContext *C, ARegion *ar, Scene *scen
 	int x, y, w, h;
 	unsigned char *clonerect;
 
-	brush = paint_brush(&scene->toolsettings->imapaint.paint);
+	brush = BKE_paint_brush(&scene->toolsettings->imapaint.paint);
 
 	if (brush && (brush->imagepaint_tool == PAINT_TOOL_CLONE)) {
 		/* this is not very efficient, but glDrawPixels doesn't allow
@@ -701,7 +745,7 @@ static void draw_image_paint_helpers(const bContext *C, ARegion *ar, Scene *scen
 		clonerect = get_alpha_clone_image(C, scene, &w, &h);
 
 		if (clonerect) {
-			UI_view2d_to_region_no_clip(&ar->v2d, brush->clone.offset[0], brush->clone.offset[1], &x, &y);
+			UI_view2d_view_to_region(&ar->v2d, brush->clone.offset[0], brush->clone.offset[1], &x, &y);
 
 			glPixelZoom(zoomx, zoomy);
 
@@ -726,7 +770,7 @@ void draw_image_main(const bContext *C, ARegion *ar)
 	Image *ima;
 	ImBuf *ibuf;
 	float zoomx, zoomy;
-	int show_viewer, show_render;
+	bool show_viewer, show_render, show_paint;
 	void *lock;
 
 	/* XXX can we do this in refresh? */
@@ -753,11 +797,12 @@ void draw_image_main(const bContext *C, ARegion *ar)
 	ima = ED_space_image(sima);
 	ED_space_image_get_zoom(sima, ar, &zoomx, &zoomy);
 
-	show_viewer = (ima && ima->source == IMA_SRC_VIEWER);
-	show_render = (show_viewer && ima->type == IMA_TYPE_R_RESULT);
+	show_viewer = (ima && ima->source == IMA_SRC_VIEWER) != 0;
+	show_render = (show_viewer && ima->type == IMA_TYPE_R_RESULT) != 0;
+	show_paint = (ima && (sima->mode == SI_MODE_PAINT) && (show_viewer == false) && (show_render == false));
 
 	if (show_viewer) {
-		/* use locked draw for drawing viewer image buffer since the conpositor
+		/* use locked draw for drawing viewer image buffer since the compositor
 		 * is running in separated thread and compositor could free this buffers.
 		 * other images are not modifying in such a way so they does not require
 		 * lock (sergey)
@@ -778,10 +823,9 @@ void draw_image_main(const bContext *C, ARegion *ar)
 		draw_image_buffer(C, sima, ar, scene, ibuf, 0.0f, 0.0f, zoomx, zoomy);
 
 	/* paint helpers */
-	if (sima->mode == SI_MODE_PAINT)
+	if (show_paint)
 		draw_image_paint_helpers(C, ar, scene, zoomx, zoomy);
 
-
 	/* XXX integrate this code */
 #if 0
 	if (ibuf) {
@@ -791,7 +835,7 @@ void draw_image_main(const bContext *C, ARegion *ar)
 			xoffs = scene->r.disprect.xmin;
 			yoffs = scene->r.disprect.ymin;
 			glColor3ub(0, 0, 0);
-			calc_image_view(sima, 'f');	
+			calc_image_view(sima, 'f');
 			myortho2(G.v2d->cur.xmin, G.v2d->cur.xmax, G.v2d->cur.ymin, G.v2d->cur.ymax);
 			glRectf(0.0f, 0.0f, 1.0f, 1.0f);
 			glLoadIdentity();
@@ -799,7 +843,7 @@ void draw_image_main(const bContext *C, ARegion *ar)
 	}
 #endif
 
-	ED_space_image_release_buffer(sima, lock);
+	ED_space_image_release_buffer(sima, ibuf, lock);
 
 	if (show_viewer) {
 		BLI_unlock_thread(LOCK_DRAW_IMAGE);
@@ -807,5 +851,66 @@ void draw_image_main(const bContext *C, ARegion *ar)
 
 	/* render info */
 	if (ima && show_render)
-		draw_render_info(scene, ima, ar);
+		draw_render_info(sima->iuser.scene, ima, ar, zoomx, zoomy);
+}
+
+bool ED_space_image_show_cache(SpaceImage *sima)
+{
+	Image *image = ED_space_image(sima);
+	Mask *mask = NULL;
+	if (sima->mode == SI_MODE_MASK) {
+		mask = ED_space_image_get_mask(sima);
+	}
+	if (image == NULL && mask == NULL) {
+		return false;
+	}
+	if (mask == NULL) {
+		return ELEM(image->source, IMA_SRC_SEQUENCE, IMA_SRC_MOVIE);
+	}
+	return true;
+}
+
+void draw_image_cache(const bContext *C, ARegion *ar)
+{
+	SpaceImage *sima = CTX_wm_space_image(C);
+	Scene *scene = CTX_data_scene(C);
+	Image *image = ED_space_image(sima);
+	float x, cfra = CFRA, sfra = SFRA, efra = EFRA, framelen = ar->winx / (efra - sfra + 1);
+	Mask *mask = NULL;
+
+	if (!ED_space_image_show_cache(sima)) {
+		return;
+	}
+
+	if (sima->mode == SI_MODE_MASK) {
+		mask = ED_space_image_get_mask(sima);
+	}
+
+	glEnable(GL_BLEND);
+	glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+
+	/* Draw cache background. */
+	ED_region_cache_draw_background(ar);
+
+	/* Draw cached segments. */
+	if (image != NULL && image->cache != NULL && ELEM(image->source, IMA_SRC_SEQUENCE, IMA_SRC_MOVIE)) {
+		int num_segments = 0;
+		int *points = NULL;
+
+		IMB_moviecache_get_cache_segments(image->cache, IMB_PROXY_NONE, 0, &num_segments, &points);
+		ED_region_cache_draw_cached_segments(ar, num_segments, points, sfra + sima->iuser.offset, efra + sima->iuser.offset);
+	}
+
+	glDisable(GL_BLEND);
+
+	/* Draw current frame. */
+	x = (cfra - sfra) / (efra - sfra + 1) * ar->winx;
+
+	UI_ThemeColor(TH_CFRAME);
+	glRecti(x, 0, x + ceilf(framelen), 8 * UI_DPI_FAC);
+	ED_region_cache_draw_curfra_label(cfra, x, 8.0f * UI_DPI_FAC);
+
+	if (mask != NULL) {
+		ED_mask_draw_frames(mask, ar, cfra, sfra, efra);
+	}
 }
