@@ -39,8 +39,6 @@
 #include "DNA_scene_types.h"
 
 #include "BLI_blenlib.h"
-#include "BLI_math.h"
-#include "BLI_rand.h"
 #include "BLI_utildefines.h"
 
 #include "BKE_animsys.h"
@@ -48,9 +46,11 @@
 #include "BKE_context.h"
 #include "BKE_global.h"
 #include "BKE_screen.h"
+#include "BKE_report.h"
 
 #include "ED_anim_api.h"
 #include "ED_keyframes_edit.h"
+#include "ED_object.h"
 #include "ED_screen.h"
 
 #include "RNA_access.h"
@@ -74,7 +74,7 @@
  *	--> Most channels are now selection only...
  */
 
-static int mouse_nla_channels(bAnimContext *ac, float x, int channel_index, short selectmode)
+static int mouse_nla_channels(bContext *C, bAnimContext *ac, float x, int channel_index, short selectmode)
 {
 	ListBase anim_data = {NULL, NULL};
 	bAnimListElem *ale;
@@ -95,7 +95,7 @@ static int mouse_nla_channels(bAnimContext *ac, float x, int channel_index, shor
 		if (G.debug & G_DEBUG)
 			printf("Error: animation channel (index = %d) not found in mouse_anim_channels()\n", channel_index);
 		
-		BLI_freelistN(&anim_data);
+		ANIM_animdata_freelist(&anim_data);
 		return 0;
 	}
 	
@@ -119,8 +119,8 @@ static int mouse_nla_channels(bAnimContext *ac, float x, int channel_index, shor
 			}
 			
 			notifierFlags |= (ND_ANIMCHAN | NA_SELECTED);
+			break;
 		}
-		break;
 		case ANIMTYPE_OBJECT:
 		{
 			bDopeSheet *ads = (bDopeSheet *)ac->data;
@@ -140,7 +140,7 @@ static int mouse_nla_channels(bAnimContext *ac, float x, int channel_index, shor
 				}
 				else {
 					Base *b;
-
+					
 					/* deselect all */
 					/* TODO: should this deselect all other types of channels too? */
 					for (b = sce->base.first; b; b = b->next) {
@@ -155,15 +155,17 @@ static int mouse_nla_channels(bAnimContext *ac, float x, int channel_index, shor
 					if (adt) adt->flag |= ADT_UI_SELECTED;
 				}
 				
+				/* change active object - regardless of whether it is now selected [T37883] */
+				ED_base_object_activate(C, base); /* adds notifier */
+				
 				if ((adt) && (adt->flag & ADT_UI_SELECTED))
 					adt->flag |= ADT_UI_ACTIVE;
 				
 				/* notifiers - channel was selected */
 				notifierFlags |= (ND_ANIMCHAN | NA_SELECTED);
 			}
+			break;
 		}
-		break;
-			
 		case ANIMTYPE_FILLACTD: /* Action Expander */
 		case ANIMTYPE_DSMAT:    /* Datablock AnimData Expanders */
 		case ANIMTYPE_DSLAM:
@@ -178,7 +180,9 @@ static int mouse_nla_channels(bAnimContext *ac, float x, int channel_index, shor
 		case ANIMTYPE_DSMESH:
 		case ANIMTYPE_DSTEX:
 		case ANIMTYPE_DSLAT:
+		case ANIMTYPE_DSLINESTYLE:
 		case ANIMTYPE_DSSPK:
+		case ANIMTYPE_DSGPENCIL:
 		{
 			/* sanity checking... */
 			if (ale->adt) {
@@ -199,9 +203,8 @@ static int mouse_nla_channels(bAnimContext *ac, float x, int channel_index, shor
 			}
 			
 			notifierFlags |= (ND_ANIMCHAN | NA_SELECTED);
-		}	
-		break;
-			
+			break;
+		}
 		case ANIMTYPE_NLATRACK:
 		{
 			NlaTrack *nlt = (NlaTrack *)ale->data;
@@ -259,12 +262,13 @@ static int mouse_nla_channels(bAnimContext *ac, float x, int channel_index, shor
 				/* notifier flags - channel was selected */
 				notifierFlags |= (ND_ANIMCHAN | NA_SELECTED);
 			}
+			break;
 		}
-		break;
 		case ANIMTYPE_NLAACTION:
 		{
 			AnimData *adt = BKE_animdata_from_id(ale->id);
 			
+			/* button area... */
 			if (x >= (v2d->cur.xmax - NLACHANNEL_BUTTON_WIDTH)) {
 				if (nlaedit_is_tweakmode_on(ac) == 0) {
 					/* 'push-down' action - only usable when not in TweakMode */
@@ -280,16 +284,50 @@ static int mouse_nla_channels(bAnimContext *ac, float x, int channel_index, shor
 				/* changes to NLA-Action occurred */
 				notifierFlags |= ND_NLA_ACTCHANGE;
 			}
+			/* OR rest of name... */
+			else {
+				/* NOTE: rest of NLA-Action name doubles for operating on the AnimData block 
+				 * - this is useful when there's no clear divider, and makes more sense in
+				 *   the case of users trying to use this to change actions
+				 * - in tweakmode, clicking here gets us out of tweakmode, as changing selection
+				 *   while in tweakmode is really evil!
+				 */
+				if (nlaedit_is_tweakmode_on(ac)) {
+					/* exit tweakmode immediately */
+					nlaedit_disable_tweakmode(ac);
+					
+					/* changes to NLA-Action occurred */
+					notifierFlags |= ND_NLA_ACTCHANGE;
+				}
+				else {
+					/* select/deselect */
+					if (selectmode == SELECT_INVERT) {
+						/* inverse selection status of this AnimData block only */
+						adt->flag ^= ADT_UI_SELECTED;
+					}
+					else {
+						/* select AnimData block by itself */
+						ANIM_deselect_anim_channels(ac, ac->data, ac->datatype, 0, ACHANNEL_SETFLAG_CLEAR);
+						adt->flag |= ADT_UI_SELECTED;
+					}
+					
+					/* set active? */
+					if (adt->flag & ADT_UI_SELECTED)
+						adt->flag |= ADT_UI_ACTIVE;
+					
+					notifierFlags |= (ND_ANIMCHAN | NA_SELECTED);
+				}
+			}
+			break;
 		}
-		break;
-			
 		default:
 			if (G.debug & G_DEBUG)
 				printf("Error: Invalid channel type in mouse_nla_channels()\n");
+			break;
 	}
 	
 	/* free channels */
-	BLI_freelistN(&anim_data);
+	ANIM_animdata_freelist(&anim_data);
 	
 	/* return the notifier-flags set */
 	return notifierFlags;
@@ -298,7 +336,7 @@ static int mouse_nla_channels(bAnimContext *ac, float x, int channel_index, shor
 /* ------------------- */
 
 /* handle clicking */
-static int nlachannels_mouseclick_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int nlachannels_mouseclick_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	bAnimContext ac;
 	SpaceNla *snla;
@@ -333,7 +371,7 @@ static int nlachannels_mouseclick_invoke(bContext *C, wmOperator *op, wmEvent *e
 	UI_view2d_listview_view_to_cell(v2d, NLACHANNEL_NAMEWIDTH, NLACHANNEL_STEP(snla), 0, (float)NLACHANNEL_HEIGHT_HALF(snla), x, y, NULL, &channel_index);
 	
 	/* handle mouse-click in the relevant channel then */
-	notifierFlags = mouse_nla_channels(&ac, x, channel_index, selectmode);
+	notifierFlags = mouse_nla_channels(C, &ac, x, channel_index, selectmode);
 	
 	/* set notifier that things have changed */
 	WM_event_add_notifier(C, NC_ANIMATION | notifierFlags, NULL);
@@ -355,7 +393,7 @@ void NLA_OT_channels_click(wmOperatorType *ot)
 	ot->poll = ED_operator_nla_active;
 	
 	/* flags */
-	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
+	ot->flag = OPTYPE_UNDO;
 	
 	/* props */
 	prop = RNA_def_boolean(ot->srna, "extend", 0, "Extend Select", ""); // SHIFTKEY
@@ -365,27 +403,122 @@ void NLA_OT_channels_click(wmOperatorType *ot)
 /* *********************************************** */
 /* Special Operators */
 
-/* ******************** Add Tracks Operator ***************************** */
-/* Add NLA Tracks to the same AnimData block as a selected track, or above the selected tracks */
+/* ******************** Action Push Down ******************************** */
 
-static int nlaedit_add_tracks_exec(bContext *C, wmOperator *op)
+static int nlachannels_pushdown_exec(bContext *C, wmOperator *op)
 {
 	bAnimContext ac;
-	
-	ListBase anim_data = {NULL, NULL};
-	bAnimListElem *ale;
-	int filter;
-	
-	AnimData *lastAdt = NULL;
-	short above_sel = RNA_boolean_get(op->ptr, "above_selected");
+	AnimData *adt = NULL;
+	int channel_index = RNA_int_get(op->ptr, "channel_index");
 	
 	/* get editor data */
 	if (ANIM_animdata_get_context(C, &ac) == 0)
 		return OPERATOR_CANCELLED;
 		
-	/* get a list of the AnimData blocks being shown in the NLA */
-	filter = (ANIMFILTER_DATA_VISIBLE | ANIMFILTER_LIST_VISIBLE | ANIMFILTER_SEL);
-	ANIM_animdata_filter(&ac, &anim_data, filter, ac.data, ac.datatype);
+	/* get anim-channel to use (or more specifically, the animdata block behind it) */
+	if (channel_index == -1) {
+		PointerRNA adt_ptr = {{NULL}};
+		
+		/* active animdata block */
+		if (nla_panel_context(C, &adt_ptr, NULL, NULL) == 0 || (adt_ptr.data == NULL)) {
+			BKE_report(op->reports, RPT_ERROR, "No active AnimData block to use "
+			           "(select a datablock expander first or set the appropriate flags on an AnimData block)");
+			return OPERATOR_CANCELLED;
+		}
+		else {
+			adt = adt_ptr.data;
+		}
+	}
+	else {
+		/* indexed channel */
+		ListBase anim_data = {NULL, NULL};
+		bAnimListElem *ale;
+		int filter;
+		
+		/* filter channels */
+		filter = (ANIMFILTER_DATA_VISIBLE | ANIMFILTER_LIST_VISIBLE | ANIMFILTER_LIST_CHANNELS);
+		ANIM_animdata_filter(&ac, &anim_data, filter, ac.data, ac.datatype);
+		
+		/* get channel from index */
+		ale = BLI_findlink(&anim_data, channel_index);
+		if (ale == NULL) {
+			BKE_reportf(op->reports, RPT_ERROR, "No animation channel found at index %d", channel_index);
+			ANIM_animdata_freelist(&anim_data);
+			return OPERATOR_CANCELLED;
+		}
+		else if (ale->type != ANIMTYPE_NLAACTION) {
+			BKE_reportf(op->reports, RPT_ERROR, "Animation channel at index %d is not a NLA 'Active Action' channel", channel_index);
+			ANIM_animdata_freelist(&anim_data);
+			return OPERATOR_CANCELLED;
+		}
+		
+		/* grab AnimData from the channel */
+		adt = ale->adt;
+		
+		/* we don't need anything here anymore, so free it all */
+		ANIM_animdata_freelist(&anim_data);
+	}
+	
+	/* double-check that we are free to push down here... */
+	if (adt == NULL) {
+		BKE_report(op->reports, RPT_WARNING, "Internal Error - AnimData block is not valid");
+		return OPERATOR_CANCELLED;
+	}
+	else if (nlaedit_is_tweakmode_on(&ac)) {
+		BKE_report(op->reports, RPT_WARNING,
+		           "Cannot push down actions while tweaking a strip's action, exit tweak mode first");
+		return OPERATOR_CANCELLED;
+	}
+	else if (adt->action == NULL) {
+		BKE_report(op->reports, RPT_WARNING, "No active action to push down");
+		return OPERATOR_CANCELLED;
+	}
+	else {
+		/* 'push-down' action - only usable when not in TweakMode */
+		BKE_nla_action_pushdown(adt);
+	}
+	
+	/* set notifier that things have changed */
+	WM_event_add_notifier(C, NC_ANIMATION | ND_NLA_ACTCHANGE, NULL);
+	return OPERATOR_FINISHED;
+}
+
+void NLA_OT_action_pushdown(wmOperatorType *ot)
+{
+	/* identifiers */
+	ot->name = "Push Down Action";
+	ot->idname = "NLA_OT_action_pushdown";
+	ot->description = "Push action down onto the top of the NLA stack as a new strip";
+	
+	/* callbacks */
+	ot->exec = nlachannels_pushdown_exec;
+	ot->poll = nlaop_poll_tweakmode_off;
+	
+	/* flags */
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
+	
+	/* properties */
+	ot->prop = RNA_def_int(ot->srna, "channel_index", -1, -1, INT_MAX, "Channel Index",
+	                       "Index of NLA action channel to perform pushdown operation on",
+	                       0, INT_MAX);
+	RNA_def_property_flag(ot->prop, PROP_SKIP_SAVE);
+}
+
+/* ******************** Add Tracks Operator ***************************** */
+/* Add NLA Tracks to the same AnimData block as a selected track, or above the selected tracks */
+
+/* helper - add NLA Tracks alongside existing ones */
+bool nlaedit_add_tracks_existing(bAnimContext *ac, bool above_sel)
+{
+	ListBase anim_data = {NULL, NULL};
+	bAnimListElem *ale;
+	int filter;
+	AnimData *lastAdt = NULL;
+	bool added = false;
+	
+	/* get a list of the (selected) NLA Tracks being shown in the NLA */
+	filter = (ANIMFILTER_DATA_VISIBLE | ANIMFILTER_LIST_VISIBLE | ANIMFILTER_SEL | ANIMFILTER_NODUPLIS);
+	ANIM_animdata_filter(ac, &anim_data, filter, ac->data, ac->datatype);
 	
 	/* add tracks... */
 	for (ale = anim_data.first; ale; ale = ale->next) {
@@ -399,29 +532,94 @@ static int nlaedit_add_tracks_exec(bContext *C, wmOperator *op)
 			if (above_sel) {
 				/* just add a new one above this one */
 				add_nlatrack(adt, nlt);
+				added = true;
 			}
 			else if ((lastAdt == NULL) || (adt != lastAdt)) {
 				/* add one track to the top of the owning AnimData's stack, then don't add anymore to this stack */
 				add_nlatrack(adt, NULL);
 				lastAdt = adt;
+				added = true;
 			}
 		}
 	}
 	
 	/* free temp data */
-	BLI_freelistN(&anim_data);
+	ANIM_animdata_freelist(&anim_data);
 	
-	/* set notifier that things have changed */
-	WM_event_add_notifier(C, NC_ANIMATION | ND_NLA | NA_EDITED, NULL);
+	return added;
+}
+
+/* helper - add NLA Tracks to empty (and selected) AnimData blocks */
+bool nlaedit_add_tracks_empty(bAnimContext *ac)
+{
+	ListBase anim_data = {NULL, NULL};
+	bAnimListElem *ale;
+	int filter;
+	bool added = false;
 	
-	/* done */
-	return OPERATOR_FINISHED;
+	/* get a list of the selected AnimData blocks in the NLA */
+	filter = (ANIMFILTER_DATA_VISIBLE | ANIMFILTER_LIST_VISIBLE | ANIMFILTER_ANIMDATA | ANIMFILTER_SEL | ANIMFILTER_NODUPLIS);
+	ANIM_animdata_filter(ac, &anim_data, filter, ac->data, ac->datatype);
+	
+	/* check if selected AnimData blocks are empty, and add tracks if so... */
+	for (ale = anim_data.first; ale; ale = ale->next) {
+		AnimData *adt = ale->adt;
+		
+		/* sanity check */
+		BLI_assert(adt->flag & ADT_UI_SELECTED);
+		
+		/* ensure it is empty */
+		if (BLI_listbase_is_empty(&adt->nla_tracks)) {
+			/* add new track to this AnimData block then */
+			add_nlatrack(adt, NULL);
+			added = true;
+		}
+	}
+	
+	/* cleanup */
+	ANIM_animdata_freelist(&anim_data);
+	
+	return added;
+}
+
+/* ----- */
+
+static int nlaedit_add_tracks_exec(bContext *C, wmOperator *op)
+{
+	bAnimContext ac;
+	bool above_sel = RNA_boolean_get(op->ptr, "above_selected");
+	bool op_done = false;
+	
+	/* get editor data */
+	if (ANIM_animdata_get_context(C, &ac) == 0)
+		return OPERATOR_CANCELLED;
+		
+	/* perform adding in two passes - existing first so that we don't double up for empty */
+	op_done |= nlaedit_add_tracks_existing(&ac, above_sel);
+	op_done |= nlaedit_add_tracks_empty(&ac);
+	
+	/* done? */
+	if (op_done) {
+		/* set notifier that things have changed */
+		WM_event_add_notifier(C, NC_ANIMATION | ND_NLA | NA_EDITED, NULL);
+		
+		/* done */
+		return OPERATOR_FINISHED;
+	}
+	else {
+		/* failed to add any tracks */
+		BKE_report(op->reports, RPT_WARNING,
+		           "Select an existing NLA Track or an empty action line first");
+		
+		/* not done */
+		return OPERATOR_CANCELLED;
+	}
 }
 
 void NLA_OT_tracks_add(wmOperatorType *ot)
 {
 	/* identifiers */
-	ot->name = "Add Track(s)";
+	ot->name = "Add Tracks";
 	ot->idname = "NLA_OT_tracks_add";
 	ot->description = "Add NLA-Tracks above/after the selected tracks";
 	
@@ -473,7 +671,7 @@ static int nlaedit_delete_tracks_exec(bContext *C, wmOperator *UNUSED(op))
 	}
 	
 	/* free temp data */
-	BLI_freelistN(&anim_data);
+	ANIM_animdata_freelist(&anim_data);
 	
 	/* set notifier that things have changed */
 	WM_event_add_notifier(C, NC_ANIMATION | ND_NLA | NA_EDITED, NULL);
@@ -482,11 +680,11 @@ static int nlaedit_delete_tracks_exec(bContext *C, wmOperator *UNUSED(op))
 	return OPERATOR_FINISHED;
 }
 
-void NLA_OT_delete_tracks(wmOperatorType *ot)
+void NLA_OT_tracks_delete(wmOperatorType *ot)
 {
 	/* identifiers */
 	ot->name = "Delete Tracks";
-	ot->idname = "NLA_OT_delete_tracks";
+	ot->idname = "NLA_OT_tracks_delete";
 	ot->description = "Delete selected NLA-Tracks and the strips they contain";
 	
 	/* api callbacks */
@@ -498,3 +696,59 @@ void NLA_OT_delete_tracks(wmOperatorType *ot)
 }
 
 /* *********************************************** */
+/* AnimData Related Operators */
+
+/* ******************** Include Objects Operator ***************************** */
+/* Include selected objects in NLA Editor, by giving them AnimData blocks 
+ * NOTE: This doesn't help for non-object AnimData, where we do not have any effective
+ *       selection mechanism in place. Unfortunately, this means that non-object AnimData
+ *       once again becomes a second-class citizen here. However, at least for the most 
+ *       common use case, we now have a nice shortcut again.
+ */
+
+static int nlaedit_objects_add_exec(bContext *C, wmOperator *UNUSED(op))
+{
+	bAnimContext ac;
+	SpaceNla *snla;
+	
+	/* get editor data */
+	if (ANIM_animdata_get_context(C, &ac) == 0)
+		return OPERATOR_CANCELLED;
+	
+	/* ensure that filters are set so that the effect will be immediately visible */
+	snla = (SpaceNla *)ac.sl;
+	if (snla && snla->ads) {
+		snla->ads->filterflag &= ~ADS_FILTER_NLA_NOACT;
+	}
+	
+	/* operate on selected objects... */	
+	CTX_DATA_BEGIN (C, Object *, ob, selected_objects)
+	{
+		/* ensure that object has AnimData... that's all */
+		BKE_id_add_animdata(&ob->id);
+	}
+	CTX_DATA_END;
+	
+	/* set notifier that things have changed */
+	WM_event_add_notifier(C, NC_ANIMATION | ND_NLA | NA_EDITED, NULL);
+	
+	/* done */
+	return OPERATOR_FINISHED;
+}
+
+void NLA_OT_selected_objects_add(wmOperatorType *ot)
+{
+	/* identifiers */
+	ot->name = "Include Selected Objects";
+	ot->idname = "NLA_OT_selected_objects_add";
+	ot->description = "Make selected objects appear in NLA Editor by adding Animation Data";
+	
+	/* api callbacks */
+	ot->exec = nlaedit_objects_add_exec;
+	ot->poll = nlaop_poll_tweakmode_off;
+	
+	/* flags */
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
+}
+
+/* *********************************************** */
