@@ -57,6 +57,7 @@
 
 /* ******************* node space & buttons ************** */
 
+#if 0
 /* poll for active nodetree */
 static int active_nodetree_poll(const bContext *C, PanelType *UNUSED(pt))
 {
@@ -64,73 +65,7 @@ static int active_nodetree_poll(const bContext *C, PanelType *UNUSED(pt))
 	
 	return (snode && snode->nodetree);
 }
-
-/* poll callback for active node */
-static int active_node_poll(const bContext *C, PanelType *UNUSED(pt))
-{
-	SpaceNode *snode = CTX_wm_space_node(C);
-	
-	return (snode && snode->edittree && nodeGetActive(snode->edittree));
-}
-
-/* active node */
-static void active_node_panel(const bContext *C, Panel *pa)
-{
-	SpaceNode *snode = CTX_wm_space_node(C);
-	bNodeTree *ntree = (snode) ? snode->edittree : NULL;
-	bNode *node = (ntree) ? nodeGetActive(ntree) : NULL; // xxx... for editing group nodes
-	uiLayout *layout, *row, *col, *sub;
-	PointerRNA ptr, opptr;
-	
-	/* verify pointers, and create RNA pointer for the node */
-	if (ELEM(NULL, ntree, node))
-		return;
-	//if (node->id) /* for group nodes */
-	//	RNA_pointer_create(node->id, &RNA_Node, node, &ptr);
-	//else
-	RNA_pointer_create(&ntree->id, &RNA_Node, node, &ptr);
-	
-	layout = uiLayoutColumn(pa->layout, FALSE);
-	uiLayoutSetContextPointer(layout, "node", &ptr);
-	
-	/* draw this node's name, etc. */
-	uiItemR(layout, &ptr, "label", 0, NULL, ICON_NODE);
-	uiItemS(layout);
-	uiItemR(layout, &ptr, "name", 0, NULL, ICON_NODE);
-	uiItemS(layout);
-	
-	uiItemO(layout, NULL, 0, "NODE_OT_hide_socket_toggle");
-	uiItemS(layout);
-	uiItemS(layout);
-
-	row = uiLayoutRow(layout, FALSE);
-	
-	col = uiLayoutColumn(row, TRUE);
-	uiItemM(col, (bContext *)C, "NODE_MT_node_color_presets", NULL, 0);
-	uiItemR(col, &ptr, "use_custom_color", UI_ITEM_R_ICON_ONLY, NULL, ICON_NONE);
-	sub = uiLayoutRow(col, FALSE);
-	if (!(node->flag & NODE_CUSTOM_COLOR))
-		uiLayoutSetEnabled(sub, FALSE);
-	uiItemR(sub, &ptr, "color", 0, "", ICON_NONE);
-	
-	col = uiLayoutColumn(row, TRUE);
-	uiItemO(col, "", ICON_ZOOMIN, "node.node_color_preset_add");
-	opptr = uiItemFullO(col, "node.node_color_preset_add", "", ICON_ZOOMOUT, NULL, WM_OP_INVOKE_DEFAULT, UI_ITEM_O_RETURN_PROPS);
-	RNA_boolean_set(&opptr, "remove_active", 1);
-	uiItemM(col, (bContext *)C, "NODE_MT_node_color_specials", "", ICON_DOWNARROW_HLT);
-	
-	/* draw this node's settings */
-	if (node->typeinfo && node->typeinfo->uifuncbut) {
-		uiItemS(layout);
-		uiItemS(layout);
-		node->typeinfo->uifuncbut(layout, (bContext *)C, &ptr);
-	}
-	else if (node->typeinfo && node->typeinfo->uifunc) {
-		uiItemS(layout);
-		uiItemS(layout);
-		node->typeinfo->uifunc(layout, (bContext *)C, &ptr);
-	}
-}
+#endif
 
 static int node_sockets_poll(const bContext *C, PanelType *UNUSED(pt))
 {
@@ -154,42 +89,119 @@ static void node_sockets_panel(const bContext *C, Panel *pa)
 	for (sock = node->inputs.first; sock; sock = sock->next) {
 		BLI_snprintf(name, sizeof(name), "%s:", sock->name);
 
-		split = uiLayoutSplit(layout, 0.35f, FALSE);
+		split = uiLayoutSplit(layout, 0.35f, false);
 		uiItemL(split, name, ICON_NONE);
 		uiTemplateNodeLink(split, ntree, node, sock);
 	}
 }
 
+static int node_tree_interface_poll(const bContext *C, PanelType *UNUSED(pt))
+{
+	SpaceNode *snode = CTX_wm_space_node(C);
+	
+	return (snode && snode->edittree && (snode->edittree->inputs.first || snode->edittree->outputs.first));
+}
+
+static bool node_tree_find_active_socket(bNodeTree *ntree, bNodeSocket **r_sock, int *r_in_out)
+{
+	bNodeSocket *sock;
+	for (sock = ntree->inputs.first; sock; sock = sock->next) {
+		if (sock->flag & SELECT) {
+			*r_sock = sock;
+			*r_in_out = SOCK_IN;
+			return true;
+		}
+	}
+	for (sock = ntree->outputs.first; sock; sock = sock->next) {
+		if (sock->flag & SELECT) {
+			*r_sock = sock;
+			*r_in_out = SOCK_OUT;
+			return true;
+		}
+	}
+	
+	*r_sock = NULL;
+	*r_in_out = 0;
+	return false;
+}
+
+static void node_tree_interface_panel(const bContext *C, Panel *pa)
+{
+	SpaceNode *snode = CTX_wm_space_node(C);
+	bNodeTree *ntree = (snode) ? snode->edittree : NULL;
+	bNodeSocket *sock;
+	int in_out;
+	uiLayout *layout = pa->layout, *row, *split, *col;
+	PointerRNA ptr, sockptr, opptr;
+
+	if (!ntree)
+		return;
+	
+	RNA_id_pointer_create((ID *)ntree, &ptr);
+	
+	node_tree_find_active_socket(ntree, &sock, &in_out);
+	RNA_pointer_create((ID *)ntree, &RNA_NodeSocketInterface, sock, &sockptr);
+	
+	row = uiLayoutRow(layout, false);
+	
+	split = uiLayoutRow(row, true);
+	col = uiLayoutColumn(split, true);
+	uiItemL(col, IFACE_("Inputs:"), ICON_NONE);
+	uiTemplateList(col, (bContext *)C, "NODE_UL_interface_sockets", "inputs", &ptr, "inputs", &ptr, "active_input",
+	               0, 0, 0, 0);
+	opptr = uiItemFullO(col, "NODE_OT_tree_socket_add", "", ICON_PLUS, NULL, WM_OP_EXEC_DEFAULT, UI_ITEM_O_RETURN_PROPS);
+	RNA_enum_set(&opptr, "in_out", SOCK_IN);
+	
+	col = uiLayoutColumn(split, true);
+	uiItemL(col, IFACE_("Outputs:"), ICON_NONE);
+	uiTemplateList(col, (bContext *)C, "NODE_UL_interface_sockets", "outputs", &ptr, "outputs", &ptr, "active_output",
+	               0, 0, 0, 0);
+	opptr = uiItemFullO(col, "NODE_OT_tree_socket_add", "", ICON_PLUS, NULL, WM_OP_EXEC_DEFAULT, UI_ITEM_O_RETURN_PROPS);
+	RNA_enum_set(&opptr, "in_out", SOCK_OUT);
+	
+	col = uiLayoutColumn(row, true);
+	opptr = uiItemFullO(col, "NODE_OT_tree_socket_move", "", ICON_TRIA_UP, NULL, WM_OP_EXEC_DEFAULT, UI_ITEM_O_RETURN_PROPS);
+	RNA_enum_set(&opptr, "direction", 1);
+	opptr = uiItemFullO(col, "NODE_OT_tree_socket_move", "", ICON_TRIA_DOWN, NULL, WM_OP_EXEC_DEFAULT, UI_ITEM_O_RETURN_PROPS);
+	RNA_enum_set(&opptr, "direction", 2);
+	
+	if (sock) {
+		row = uiLayoutRow(layout, true);
+		uiItemR(row, &sockptr, "name", 0, NULL, ICON_NONE);
+		uiItemO(row, "", ICON_X, "NODE_OT_tree_socket_remove");
+		
+		if (sock->typeinfo->interface_draw) {
+			uiItemS(layout);
+			sock->typeinfo->interface_draw((bContext *)C, layout, &sockptr);
+		}
+	}
+}
+
 /* ******************* node buttons registration ************** */
 
 void node_buttons_register(ARegionType *art)
 {
 	PanelType *pt;
 	
-	pt = MEM_callocN(sizeof(PanelType), "spacetype node panel active node");
-	strcpy(pt->idname, "NODE_PT_item");
-	strcpy(pt->label, IFACE_("Active Node"));
-	pt->draw = active_node_panel;
-	pt->poll = active_node_poll;
-	BLI_addtail(&art->paneltypes, pt);
-
 	pt = MEM_callocN(sizeof(PanelType), "spacetype node panel node sockets");
 	strcpy(pt->idname, "NODE_PT_sockets");
-	strcpy(pt->label, "Sockets");
+	strcpy(pt->label, N_("Sockets"));
+	strcpy(pt->translation_context, BLF_I18NCONTEXT_DEFAULT_BPYRNA);
 	pt->draw = node_sockets_panel;
 	pt->poll = node_sockets_poll;
 	pt->flag |= PNL_DEFAULT_CLOSED;
 	BLI_addtail(&art->paneltypes, pt);
-	
-	pt = MEM_callocN(sizeof(PanelType), "spacetype node panel gpencil");
-	strcpy(pt->idname, "NODE_PT_gpencil");
-	strcpy(pt->label, "Grease Pencil");
-	pt->draw = gpencil_panel_standard;
-	pt->poll = active_nodetree_poll;
+
+	pt = MEM_callocN(sizeof(PanelType), "spacetype node panel tree interface");
+	strcpy(pt->idname, "NODE_PT_node_tree_interface");
+	strcpy(pt->label, N_("Interface"));
+	strcpy(pt->translation_context, BLF_I18NCONTEXT_DEFAULT_BPYRNA);
+	pt->draw = node_tree_interface_panel;
+	pt->poll = node_tree_interface_poll;
 	BLI_addtail(&art->paneltypes, pt);
 }
 
-static int node_properties(bContext *C, wmOperator *UNUSED(op))
+static int node_properties_toggle_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	ScrArea *sa = CTX_wm_area(C);
 	ARegion *ar = node_has_buttons_region(sa);
@@ -213,7 +225,7 @@ void NODE_OT_properties(wmOperatorType *ot)
 	ot->description = "Toggles the properties panel display";
 	ot->idname = "NODE_OT_properties";
 	
-	ot->exec = node_properties;
+	ot->exec = node_properties_toggle_exec;
 	ot->poll = node_properties_poll;
 	
 	/* flags */
