@@ -29,16 +29,17 @@
  *  \ingroup spnode
  */
 
+#include <ctype.h>
+
 #include "MEM_guardedalloc.h"
 
 #include "DNA_node_types.h"
-#include "DNA_object_types.h"
 
 #include "BLI_math.h"
 #include "BLI_blenlib.h"
 
 #include "BKE_context.h"
-#include "BKE_main.h"
+#include "BKE_global.h"
 #include "BKE_node.h"
 
 #include "ED_node.h"  /* own include */
@@ -53,6 +54,8 @@
 
 #include "UI_view2d.h"
 
+#include "BLF_translation.h"
+
 #include "node_intern.h"  /* own include */
 
 /* ****************** Add *********************** */
@@ -63,12 +66,12 @@ typedef struct bNodeListItem {
 	struct bNode *node;
 } bNodeListItem;
 
-static int sort_nodes_locx(void *a, void *b)
+static int sort_nodes_locx(const void *a, const void *b)
 {
-	bNodeListItem *nli1 = (bNodeListItem *)a;
-	bNodeListItem *nli2 = (bNodeListItem *)b;
-	bNode *node1 = nli1->node;
-	bNode *node2 = nli2->node;
+	const bNodeListItem *nli1 = a;
+	const bNodeListItem *nli2 = b;
+	const bNode *node1 = nli1->node;
+	const bNode *node2 = nli2->node;
 
 	if (node1->locx > node2->locx)
 		return 1;
@@ -76,7 +79,7 @@ static int sort_nodes_locx(void *a, void *b)
 		return 0;
 }
 
-static int socket_is_available(bNodeTree *UNUSED(ntree), bNodeSocket *sock, int allow_used)
+static bool socket_is_available(bNodeTree *UNUSED(ntree), bNodeSocket *sock, const bool allow_used)
 {
 	if (nodeSocketIsHidden(sock))
 		return 0;
@@ -87,7 +90,7 @@ static int socket_is_available(bNodeTree *UNUSED(ntree), bNodeSocket *sock, int
 	return 1;
 }
 
-static bNodeSocket *best_socket_output(bNodeTree *ntree, bNode *node, bNodeSocket *sock_target, int allow_multiple)
+static bNodeSocket *best_socket_output(bNodeTree *ntree, bNode *node, bNodeSocket *sock_target, const bool allow_multiple)
 {
 	bNodeSocket *sock;
 
@@ -107,7 +110,7 @@ static bNodeSocket *best_socket_output(bNodeTree *ntree, bNode *node, bNodeSocke
 
 		/* check for same types */
 		if (sock->type == sock_target->type) {
-			if (strcmp(sock->name, sock_target->name) == 0)
+			if (STREQ(sock->name, sock_target->name))
 				return sock;
 		}
 	}
@@ -136,7 +139,7 @@ static bNodeSocket *best_socket_input(bNodeTree *ntree, bNode *node, int num, in
 	int a = 0;
 
 	for (sock = node->inputs.first; sock; sock = sock->next) {
-		maxtype = MAX2(sock->type, maxtype);
+		maxtype = max_ii(sock->type, maxtype);
 	}
 
 	/* find sockets of higher 'types' first (i.e. image) */
@@ -172,7 +175,7 @@ static int snode_autoconnect_input(SpaceNode *snode, bNode *node_fr, bNodeSocket
 
 	link = nodeAddLink(ntree, node_fr, sock_fr, node_to, sock_to);
 	/* validate the new link */
-	ntreeUpdateTree(ntree);
+	ntreeUpdateTree(G.main, ntree);
 	if (!(link->flag & NODE_LINK_VALID)) {
 		nodeRemLink(ntree, link);
 		return 0;
@@ -182,7 +185,7 @@ static int snode_autoconnect_input(SpaceNode *snode, bNode *node_fr, bNodeSocket
 	return 1;
 }
 
-static void snode_autoconnect(SpaceNode *snode, int allow_multiple, int replace)
+static void snode_autoconnect(SpaceNode *snode, const bool allow_multiple, const bool replace)
 {
 	bNodeTree *ntree = snode->edittree;
 	ListBase *nodelist = MEM_callocN(sizeof(ListBase), "items_list");
@@ -199,12 +202,12 @@ static void snode_autoconnect(SpaceNode *snode, int allow_multiple, int replace)
 	}
 
 	/* sort nodes left to right */
-	BLI_sortlist(nodelist, sort_nodes_locx);
+	BLI_listbase_sort(nodelist, sort_nodes_locx);
 
 	for (nli = nodelist->first; nli; nli = nli->next) {
 		bNode *node_fr, *node_to;
 		bNodeSocket *sock_fr, *sock_to;
-		int has_selected_inputs = 0;
+		bool has_selected_inputs = false;
 
 		if (nli->next == NULL) break;
 
@@ -232,7 +235,7 @@ static void snode_autoconnect(SpaceNode *snode, int allow_multiple, int replace)
 
 		if (!has_selected_inputs) {
 			/* no selected inputs, connect by finding suitable match */
-			int num_inputs = BLI_countlist(&node_to->inputs);
+			int num_inputs = BLI_listbase_count(&node_to->inputs);
 
 			for (i = 0; i < num_inputs; i++) {
 
@@ -255,7 +258,7 @@ static void snode_autoconnect(SpaceNode *snode, int allow_multiple, int replace)
 	}
 
 	if (numlinks > 0) {
-		ntreeUpdateTree(ntree);
+		ntreeUpdateTree(G.main, ntree);
 	}
 
 	BLI_freelistN(nodelist);
@@ -272,7 +275,7 @@ static int node_link_viewer(const bContext *C, bNode *tonode)
 	bNodeSocket *sock;
 
 	/* context check */
-	if (tonode == NULL || tonode->outputs.first == NULL)
+	if (tonode == NULL || BLI_listbase_is_empty(&tonode->outputs))
 		return OPERATOR_CANCELLED;
 	if (ELEM(tonode->type, CMP_NODE_VIEWER, CMP_NODE_SPLITVIEWER))
 		return OPERATOR_CANCELLED;
@@ -323,13 +326,8 @@ static int node_link_viewer(const bContext *C, bNode *tonode)
 	if (sock) {
 		/* add a new viewer if none exists yet */
 		if (!node) {
-			Main *bmain = CTX_data_main(C);
-			Scene *scene = CTX_data_scene(C);
-			bNodeTemplate ntemp;
-
-			ntemp.type = CMP_NODE_VIEWER;
 			/* XXX location is a quick hack, just place it next to the linked socket */
-			node = node_add_node(snode, bmain, scene, &ntemp, sock->locx + 100, sock->locy);
+			node = node_add_node(C, NULL, CMP_NODE_VIEWER, sock->locx + 100, sock->locy);
 			if (!node)
 				return OPERATOR_CANCELLED;
 
@@ -351,7 +349,7 @@ static int node_link_viewer(const bContext *C, bNode *tonode)
 			/* make sure the dependency sorting is updated */
 			snode->edittree->update |= NTREE_UPDATE_LINKS;
 		}
-		ntreeUpdateTree(snode->edittree);
+		ntreeUpdateTree(CTX_data_main(C), snode->edittree);
 		snode_update(snode, node);
 	}
 
@@ -359,17 +357,17 @@ static int node_link_viewer(const bContext *C, bNode *tonode)
 }
 
 
-static int node_active_link_viewer(bContext *C, wmOperator *UNUSED(op))
+static int node_active_link_viewer_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	SpaceNode *snode = CTX_wm_space_node(C);
 	bNode *node;
 
-	node = editnode_get_active(snode->edittree);
+	node = nodeGetActive(snode->edittree);
 
 	if (!node)
 		return OPERATOR_CANCELLED;
 
-	ED_preview_kill_jobs(C);
+	ED_preview_kill_jobs(CTX_wm_manager(C), CTX_data_main(C));
 
 	if (node_link_viewer(C, node) == OPERATOR_CANCELLED)
 		return OPERATOR_CANCELLED;
@@ -388,8 +386,8 @@ void NODE_OT_link_viewer(wmOperatorType *ot)
 	ot->idname = "NODE_OT_link_viewer";
 
 	/* api callbacks */
-	ot->exec = node_active_link_viewer;
-	ot->poll = composite_node_active;
+	ot->exec = node_active_link_viewer_exec;
+	ot->poll = composite_node_editable;
 
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -398,230 +396,256 @@ void NODE_OT_link_viewer(wmOperatorType *ot)
 
 /* *************************** add link op ******************** */
 
-static void node_remove_extra_links(SpaceNode *snode, bNodeSocket *tsock, bNodeLink *link)
+static void node_link_update_header(bContext *C, bNodeLinkDrag *UNUSED(nldrag))
 {
-	bNodeLink *tlink;
-	bNodeSocket *sock;
+#define HEADER_LENGTH 256
+	char header[HEADER_LENGTH];
 
-	if (tsock && nodeCountSocketLinks(snode->edittree, link->tosock) > tsock->limit) {
+	BLI_strncpy(header, IFACE_("LMB: drag node link, RMB: cancel"), HEADER_LENGTH);
+	ED_area_headerprint(CTX_wm_area(C), header);
+#undef HEADER_LENGTH
+}
 
-		for (tlink = snode->edittree->links.first; tlink; tlink = tlink->next) {
-			if (link != tlink && tlink->tosock == link->tosock)
-				break;
+/* update link_count fields to avoid repeated link counting */
+static int node_count_links(bNodeTree *ntree, bNodeSocket *sock)
+{
+	bNodeLink *link;
+	int count = 0;
+	for (link = ntree->links.first; link; link = link->next) {
+		if (link->fromsock == sock)
+			++count;
+		if (link->tosock == sock)
+			++count;
+	}
+	return count;
+}
+
+/* test if two sockets are interchangeable
+ * XXX this could be made into a tree-type callback for flexibility
+ */
+static bool node_link_socket_match(bNodeSocket *a, bNodeSocket *b)
+{
+	/* tests if alphabetic prefix matches
+	 * this allows for imperfect matches, such as numeric suffixes,
+	 * like Color1/Color2
+	 */
+	int prefix_len = 0;
+	char *ca = a->name, *cb = b->name;
+	for (; *ca != '\0' && *cb != '\0'; ++ca, ++cb) {
+		/* end of common prefix? */
+		if (*ca != *cb) {
+			/* prefix delimited by non-alphabetic char */
+			if (isalpha(*ca) || isalpha(*cb))
+				return false;
+			break;
 		}
-		if (tlink) {
-			/* try to move the existing link to the next available socket */
-			if (tlink->tonode) {
-				/* is there a free input socket with the target type? */
-				for (sock = tlink->tonode->inputs.first; sock; sock = sock->next) {
-					if (sock->type == tlink->tosock->type)
-						if (nodeCountSocketLinks(snode->edittree, sock) < sock->limit)
-							break;
-				}
-				if (sock) {
-					tlink->tosock = sock;
-					sock->flag &= ~SOCK_HIDDEN;
-				}
-				else {
-					nodeRemLink(snode->edittree, tlink);
-				}
-			}
-			else
-				nodeRemLink(snode->edittree, tlink);
+		++prefix_len;
+	}
+	return prefix_len > 0;
+}
 
-			snode->edittree->update |= NTREE_UPDATE_LINKS;
+/* find an eligible socket for linking */
+static bNodeSocket *node_find_linkable_socket(bNodeTree *ntree, bNode *node, bNodeSocket *cur, bool use_swap)
+{
+	int cur_link_count = node_count_links(ntree, cur);
+	if (cur_link_count <= cur->limit) {
+		/* current socket is fine, use it */
+		return cur;
+	}
+	else if (use_swap) {
+		/* link swapping: try to find a free slot with a matching name */
+		
+		bNodeSocket *first = cur->in_out == SOCK_IN ? node->inputs.first : node->outputs.first;
+		bNodeSocket *sock;
+		
+		sock = cur->next ? cur->next : first; /* wrap around the list end */
+		while (sock != cur) {
+			if (!nodeSocketIsHidden(sock) && node_link_socket_match(sock, cur)) {
+				int link_count = node_count_links(ntree, sock);
+				/* take +1 into account since we would add a new link */
+				if (link_count + 1 <= sock->limit)
+					return sock; /* found a valid free socket we can swap to */
+			}
+			
+			sock = sock->next ? sock->next : first; /* wrap around the list end */
+		}
+	}
+	return NULL;
+}
+
+static void node_remove_extra_links(SpaceNode *snode, bNodeLink *link, bool use_swap)
+{
+	bNodeTree *ntree = snode->edittree;
+	bNodeSocket *from = link->fromsock, *to = link->tosock;
+	bNodeLink *tlink, *tlink_next;
+	
+	for (tlink = ntree->links.first; tlink; tlink = tlink_next) {
+		tlink_next = tlink->next;
+		if (tlink == link)
+			continue;
+		
+		if (tlink && tlink->fromsock == from) {
+			bNodeSocket *new_from = node_find_linkable_socket(ntree, tlink->fromnode, from, use_swap);
+			if (new_from && new_from != from) {
+				/* redirect existing link */
+				tlink->fromsock = new_from;
+			}
+			else if (!new_from) {
+				/* no possible replacement, remove tlink */
+				nodeRemLink(ntree, tlink);
+				tlink = NULL;
+			}
+		}
+		
+		if (tlink && tlink->tosock == to) {
+			bNodeSocket *new_to = node_find_linkable_socket(ntree, tlink->tonode, to, use_swap);
+			if (new_to && new_to != to) {
+				/* redirect existing link */
+				tlink->tosock = new_to;
+			}
+			else if (!new_to) {
+				/* no possible replacement, remove tlink */
+				nodeRemLink(ntree, tlink);
+				tlink = NULL;
+			}
 		}
 	}
 }
 
-static int outside_group_rect(SpaceNode *snode)
+static void node_link_exit(bContext *C, wmOperator *op, bool apply_links)
 {
-	bNode *gnode = node_tree_get_editgroup(snode->nodetree);
-	if (gnode) {
-		return (snode->cursor[0] <  gnode->totr.xmin ||
-		        snode->cursor[0] >= gnode->totr.xmax ||
-		        snode->cursor[1] <  gnode->totr.ymin ||
-		        snode->cursor[1] >= gnode->totr.ymax);
+	SpaceNode *snode = CTX_wm_space_node(C);
+	bNodeTree *ntree = snode->edittree;
+	bNodeLinkDrag *nldrag = op->customdata;
+	LinkData *linkdata;
+	
+	for (linkdata = nldrag->links.first; linkdata; linkdata = linkdata->next) {
+		bNodeLink *link = linkdata->data;
+		
+		if (apply_links && link->tosock && link->fromsock) {
+			/* add link to the node tree */
+			BLI_addtail(&ntree->links, link);
+			
+			ntree->update |= NTREE_UPDATE_LINKS;
+			
+			/* tag tonode for update */
+			link->tonode->update |= NODE_UPDATE;
+			
+			/* we might need to remove a link */
+			node_remove_extra_links(snode, link, true);
+		}
+		else
+			nodeRemLink(ntree, link);
+	}
+	
+	ntreeUpdateTree(CTX_data_main(C), ntree);
+	snode_notify(C, snode);
+	snode_dag_update(C, snode);
+	
+	BLI_remlink(&snode->linkdrag, nldrag);
+	/* links->data pointers are either held by the tree or freed already */
+	BLI_freelistN(&nldrag->links);
+	MEM_freeN(nldrag);
+}
+
+static void node_link_find_socket(bContext *C, wmOperator *op, float cursor[2])
+{
+	SpaceNode *snode = CTX_wm_space_node(C);
+	bNodeLinkDrag *nldrag = op->customdata;
+	bNode *tnode;
+	bNodeSocket *tsock = NULL;
+	LinkData *linkdata;
+
+	if (nldrag->in_out == SOCK_OUT) {
+		if (node_find_indicated_socket(snode, &tnode, &tsock, cursor, SOCK_IN)) {
+			for (linkdata = nldrag->links.first; linkdata; linkdata = linkdata->next) {
+				bNodeLink *link = linkdata->data;
+				
+				/* skip if this is already the target socket */
+				if (link->tosock == tsock)
+					continue;
+				/* skip if socket is on the same node as the fromsock */
+				if (tnode && link->fromnode == tnode)
+					continue;
+				
+				/* attach links to the socket */
+				link->tonode = tnode;
+				link->tosock = tsock;
+			}
+		}
+		else {
+			for (linkdata = nldrag->links.first; linkdata; linkdata = linkdata->next) {
+				bNodeLink *link = linkdata->data;
+				
+				link->tonode = NULL;
+				link->tosock = NULL;
+			}
+		}
+	}
+	else {
+		if (node_find_indicated_socket(snode, &tnode, &tsock, cursor, SOCK_OUT)) {
+			for (linkdata = nldrag->links.first; linkdata; linkdata = linkdata->next) {
+				bNodeLink *link = linkdata->data;
+				
+				/* skip if this is already the target socket */
+				if (link->fromsock == tsock)
+					continue;
+				/* skip if socket is on the same node as the fromsock */
+				if (tnode && link->tonode == tnode)
+					continue;
+				
+				/* attach links to the socket */
+				link->fromnode = tnode;
+				link->fromsock = tsock;
+			}
+		}
+		else {
+			for (linkdata = nldrag->links.first; linkdata; linkdata = linkdata->next) {
+				bNodeLink *link = linkdata->data;
+				
+				link->fromnode = NULL;
+				link->fromsock = NULL;
+			}
+		}
 	}
-	return 0;
 }
 
 /* loop that adds a nodelink, called by function below  */
 /* in_out = starting socket */
-static int node_link_modal(bContext *C, wmOperator *op, wmEvent *event)
+static int node_link_modal(bContext *C, wmOperator *op, const wmEvent *event)
 {
-	SpaceNode *snode = CTX_wm_space_node(C);
-	ARegion *ar = CTX_wm_region(C);
 	bNodeLinkDrag *nldrag = op->customdata;
-	bNodeTree *ntree = snode->edittree;
-	bNode *tnode;
-	bNodeSocket *tsock = NULL;
-	bNodeLink *link;
-	LinkData *linkdata;
-	int in_out;
-
-	in_out = nldrag->in_out;
-
+	ARegion *ar = CTX_wm_region(C);
+	float cursor[2];
+	
 	UI_view2d_region_to_view(&ar->v2d, event->mval[0], event->mval[1],
-	                         &snode->cursor[0], &snode->cursor[1]);
-
+	                         &cursor[0], &cursor[1]);
+	
 	switch (event->type) {
 		case MOUSEMOVE:
-
-			if (in_out == SOCK_OUT) {
-				if (node_find_indicated_socket(snode, &tnode, &tsock, SOCK_IN)) {
-					for (linkdata = nldrag->links.first; linkdata; linkdata = linkdata->next) {
-						link = linkdata->data;
-
-						/* skip if this is already the target socket */
-						if (link->tosock == tsock)
-							continue;
-						/* skip if socket is on the same node as the fromsock */
-						if (tnode && link->fromnode == tnode)
-							continue;
-
-						/* attach links to the socket */
-						link->tonode = tnode;
-						link->tosock = tsock;
-						/* add it to the node tree temporarily */
-						if (BLI_findindex(&ntree->links, link) < 0)
-							BLI_addtail(&ntree->links, link);
-
-						ntree->update |= NTREE_UPDATE_LINKS;
-					}
-					ntreeUpdateTree(ntree);
-				}
-				else {
-					int do_update = FALSE;
-					for (linkdata = nldrag->links.first; linkdata; linkdata = linkdata->next) {
-						link = linkdata->data;
-
-						if (link->tonode || link->tosock) {
-							BLI_remlink(&ntree->links, link);
-							link->prev = link->next = NULL;
-							link->tonode = NULL;
-							link->tosock = NULL;
-
-							ntree->update |= NTREE_UPDATE_LINKS;
-							do_update = TRUE;
-						}
-					}
-					if (do_update) {
-						ntreeUpdateTree(ntree);
-					}
-				}
-			}
-			else {
-				if (node_find_indicated_socket(snode, &tnode, &tsock, SOCK_OUT)) {
-					for (linkdata = nldrag->links.first; linkdata; linkdata = linkdata->next) {
-						link = linkdata->data;
-
-						/* skip if this is already the target socket */
-						if (link->fromsock == tsock)
-							continue;
-						/* skip if socket is on the same node as the fromsock */
-						if (tnode && link->tonode == tnode)
-							continue;
-
-						/* attach links to the socket */
-						link->fromnode = tnode;
-						link->fromsock = tsock;
-						/* add it to the node tree temporarily */
-						if (BLI_findindex(&ntree->links, link) < 0)
-							BLI_addtail(&ntree->links, link);
-
-						ntree->update |= NTREE_UPDATE_LINKS;
-					}
-					ntreeUpdateTree(ntree);
-				}
-				else {
-					int do_update = FALSE;
-					for (linkdata = nldrag->links.first; linkdata; linkdata = linkdata->next) {
-						link = linkdata->data;
-
-						if (link->fromnode || link->fromsock) {
-							BLI_remlink(&ntree->links, link);
-							link->prev = link->next = NULL;
-							link->fromnode = NULL;
-							link->fromsock = NULL;
-
-							ntree->update |= NTREE_UPDATE_LINKS;
-							do_update = TRUE;
-						}
-					}
-					if (do_update) {
-						ntreeUpdateTree(ntree);
-					}
-				}
-			}
-
+			node_link_find_socket(C, op, cursor);
+			
+			node_link_update_header(C, nldrag);
 			ED_region_tag_redraw(ar);
 			break;
-
+			
 		case LEFTMOUSE:
 		case RIGHTMOUSE:
 		case MIDDLEMOUSE:
 		{
-			for (linkdata = nldrag->links.first; linkdata; linkdata = linkdata->next) {
-				link = linkdata->data;
-
-				if (link->tosock && link->fromsock) {
-					/* send changed events for original tonode and new */
-					if (link->tonode)
-						snode_update(snode, link->tonode);
-
-					/* we might need to remove a link */
-					if (in_out == SOCK_OUT)
-						node_remove_extra_links(snode, link->tosock, link);
-
-					/* when linking to group outputs, update the socket type */
-					/* XXX this should all be part of a generic update system */
-					if (!link->tonode) {
-						if (link->tosock->type != link->fromsock->type)
-							nodeSocketSetType(link->tosock, link->fromsock->type);
-					}
-				}
-				else if (outside_group_rect(snode) && (link->tonode || link->fromnode)) {
-					/* automatically add new group socket */
-					if (link->tonode && link->tosock) {
-						link->fromsock = node_group_expose_socket(ntree, link->tosock, SOCK_IN);
-						link->fromnode = NULL;
-						if (BLI_findindex(&ntree->links, link) < 0)
-							BLI_addtail(&ntree->links, link);
-
-						ntree->update |= NTREE_UPDATE_GROUP_IN | NTREE_UPDATE_LINKS;
-					}
-					else if (link->fromnode && link->fromsock) {
-						link->tosock = node_group_expose_socket(ntree, link->fromsock, SOCK_OUT);
-						link->tonode = NULL;
-						if (BLI_findindex(&ntree->links, link) < 0)
-							BLI_addtail(&ntree->links, link);
-
-						ntree->update |= NTREE_UPDATE_GROUP_OUT | NTREE_UPDATE_LINKS;
-					}
-				}
-				else
-					nodeRemLink(ntree, link);
-			}
-
-			ntreeUpdateTree(ntree);
-			snode_notify(C, snode);
-			snode_dag_update(C, snode);
-
-			BLI_remlink(&snode->linkdrag, nldrag);
-			/* links->data pointers are either held by the tree or freed already */
-			BLI_freelistN(&nldrag->links);
-			MEM_freeN(nldrag);
-
+			node_link_exit(C, op, true);
+			
+			ED_area_headerprint(CTX_wm_area(C), NULL);
+			ED_region_tag_redraw(ar);
 			return OPERATOR_FINISHED;
 		}
 	}
-
+	
 	return OPERATOR_RUNNING_MODAL;
 }
 
 /* return 1 when socket clicked */
-static bNodeLinkDrag *node_link_init(SpaceNode *snode, int detach)
+static bNodeLinkDrag *node_link_init(SpaceNode *snode, float cursor[2], bool detach)
 {
 	bNode *node;
 	bNodeSocket *sock;
@@ -631,7 +655,7 @@ static bNodeLinkDrag *node_link_init(SpaceNode *snode, int detach)
 	int num_links;
 
 	/* output indicated? */
-	if (node_find_indicated_socket(snode, &node, &sock, SOCK_OUT)) {
+	if (node_find_indicated_socket(snode, &node, &sock, cursor, SOCK_OUT)) {
 		nldrag = MEM_callocN(sizeof(bNodeLinkDrag), "drag link op customdata");
 
 		num_links = nodeCountSocketLinks(snode->edittree, sock);
@@ -646,6 +670,8 @@ static bNodeLinkDrag *node_link_init(SpaceNode *snode, int detach)
 					linkdata->data = oplink = MEM_callocN(sizeof(bNodeLink), "drag link op link");
 					*oplink = *link;
 					oplink->next = oplink->prev = NULL;
+					oplink->flag |= NODE_LINK_VALID;
+					
 					BLI_addtail(&nldrag->links, linkdata);
 					nodeRemLink(snode->edittree, link);
 				}
@@ -659,11 +685,13 @@ static bNodeLinkDrag *node_link_init(SpaceNode *snode, int detach)
 			linkdata->data = oplink = MEM_callocN(sizeof(bNodeLink), "drag link op link");
 			oplink->fromnode = node;
 			oplink->fromsock = sock;
+			oplink->flag |= NODE_LINK_VALID;
+			
 			BLI_addtail(&nldrag->links, linkdata);
 		}
 	}
 	/* or an input? */
-	else if (node_find_indicated_socket(snode, &node, &sock, SOCK_IN)) {
+	else if (node_find_indicated_socket(snode, &node, &sock, cursor, SOCK_IN)) {
 		nldrag = MEM_callocN(sizeof(bNodeLinkDrag), "drag link op customdata");
 
 		num_links = nodeCountSocketLinks(snode->edittree, sock);
@@ -678,9 +706,11 @@ static bNodeLinkDrag *node_link_init(SpaceNode *snode, int detach)
 					linkdata->data = oplink = MEM_callocN(sizeof(bNodeLink), "drag link op link");
 					*oplink = *link;
 					oplink->next = oplink->prev = NULL;
+					oplink->flag |= NODE_LINK_VALID;
+					
 					BLI_addtail(&nldrag->links, linkdata);
 					nodeRemLink(snode->edittree, link);
-
+					
 					/* send changed event to original link->tonode */
 					if (node)
 						snode_update(snode, node);
@@ -695,26 +725,30 @@ static bNodeLinkDrag *node_link_init(SpaceNode *snode, int detach)
 			linkdata->data = oplink = MEM_callocN(sizeof(bNodeLink), "drag link op link");
 			oplink->tonode = node;
 			oplink->tosock = sock;
+			oplink->flag |= NODE_LINK_VALID;
+			
 			BLI_addtail(&nldrag->links, linkdata);
 		}
 	}
-
+	
 	return nldrag;
 }
 
-static int node_link_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int node_link_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	SpaceNode *snode = CTX_wm_space_node(C);
 	ARegion *ar = CTX_wm_region(C);
 	bNodeLinkDrag *nldrag;
-	int detach = RNA_boolean_get(op->ptr, "detach");
+	float cursor[2];
+	
+	bool detach = RNA_boolean_get(op->ptr, "detach");
 
 	UI_view2d_region_to_view(&ar->v2d, event->mval[0], event->mval[1],
-	                         &snode->cursor[0], &snode->cursor[1]);
+	                         &cursor[0], &cursor[1]);
 
-	ED_preview_kill_jobs(C);
+	ED_preview_kill_jobs(CTX_wm_manager(C), CTX_data_main(C));
 
-	nldrag = node_link_init(snode, detach);
+	nldrag = node_link_init(snode, cursor, detach);
 
 	if (nldrag) {
 		op->customdata = nldrag;
@@ -729,7 +763,7 @@ static int node_link_invoke(bContext *C, wmOperator *op, wmEvent *event)
 		return OPERATOR_CANCELLED | OPERATOR_PASS_THROUGH;
 }
 
-static int node_link_cancel(bContext *C, wmOperator *op)
+static void node_link_cancel(bContext *C, wmOperator *op)
 {
 	SpaceNode *snode = CTX_wm_space_node(C);
 	bNodeLinkDrag *nldrag = op->customdata;
@@ -738,8 +772,6 @@ static int node_link_cancel(bContext *C, wmOperator *op)
 
 	BLI_freelistN(&nldrag->links);
 	MEM_freeN(nldrag);
-
-	return OPERATOR_CANCELLED;
 }
 
 void NODE_OT_link(wmOperatorType *ot)
@@ -753,13 +785,13 @@ void NODE_OT_link(wmOperatorType *ot)
 	ot->invoke = node_link_invoke;
 	ot->modal = node_link_modal;
 //	ot->exec = node_link_exec;
-	ot->poll = ED_operator_node_active;
+	ot->poll = ED_operator_node_editable;
 	ot->cancel = node_link_cancel;
 
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO | OPTYPE_BLOCKING;
 
-	RNA_def_boolean(ot->srna, "detach", FALSE, "Detach", "Detach and redirect existing links");
+	RNA_def_boolean(ot->srna, "detach", false, "Detach", "Detach and redirect existing links");
 }
 
 /* ********************** Make Link operator ***************** */
@@ -768,9 +800,9 @@ void NODE_OT_link(wmOperatorType *ot)
 static int node_make_link_exec(bContext *C, wmOperator *op)
 {
 	SpaceNode *snode = CTX_wm_space_node(C);
-	int replace = RNA_boolean_get(op->ptr, "replace");
+	const bool replace = RNA_boolean_get(op->ptr, "replace");
 
-	ED_preview_kill_jobs(C);
+	ED_preview_kill_jobs(CTX_wm_manager(C), CTX_data_main(C));
 
 	snode_autoconnect(snode, 1, replace);
 
@@ -778,7 +810,7 @@ static int node_make_link_exec(bContext *C, wmOperator *op)
 	node_deselect_all_input_sockets(snode, 0);
 	node_deselect_all_output_sockets(snode, 0);
 
-	ntreeUpdateTree(snode->edittree);
+	ntreeUpdateTree(CTX_data_main(C), snode->edittree);
 	snode_notify(C, snode);
 	snode_dag_update(C, snode);
 
@@ -794,7 +826,7 @@ void NODE_OT_link_make(wmOperatorType *ot)
 
 	/* callbacks */
 	ot->exec = node_make_link_exec;
-	ot->poll = ED_operator_node_active; // XXX we need a special poll which checks that there are selected input/output sockets
+	ot->poll = ED_operator_node_editable; // XXX we need a special poll which checks that there are selected input/output sockets
 
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -825,7 +857,7 @@ static int cut_links_exec(bContext *C, wmOperator *op)
 	float mcoords[256][2];
 	int i = 0;
 
-	RNA_BEGIN(op->ptr, itemptr, "path")
+	RNA_BEGIN (op->ptr, itemptr, "path")
 	{
 		float loc[2];
 
@@ -838,17 +870,22 @@ static int cut_links_exec(bContext *C, wmOperator *op)
 	RNA_END;
 
 	if (i > 1) {
-		int found = FALSE;
+		bool found = false;
 		bNodeLink *link, *next;
-
+		
+		ED_preview_kill_jobs(CTX_wm_manager(C), CTX_data_main(C));
+		
 		for (link = snode->edittree->links.first; link; link = next) {
 			next = link->next;
+			if (nodeLinkIsHidden(link))
+				continue;
 
 			if (cut_links_intersect(link, mcoords, i)) {
 
-				if (found == FALSE) {
-					ED_preview_kill_jobs(C);
-					found = TRUE;
+				if (found == false) {
+					/* TODO(sergey): Why did we kill jobs twice? */
+					ED_preview_kill_jobs(CTX_wm_manager(C), CTX_data_main(C));
+					found = true;
 				}
 
 				snode_update(snode, link->tonode);
@@ -857,7 +894,7 @@ static int cut_links_exec(bContext *C, wmOperator *op)
 		}
 
 		if (found) {
-			ntreeUpdateTree(snode->edittree);
+			ntreeUpdateTree(CTX_data_main(C), snode->edittree);
 			snode_notify(C, snode);
 			snode_dag_update(C, snode);
 
@@ -876,7 +913,7 @@ void NODE_OT_links_cut(wmOperatorType *ot)
 {
 	PropertyRNA *prop;
 
-	ot->name = "Cut links";
+	ot->name = "Cut Links";
 	ot->idname = "NODE_OT_links_cut";
 	ot->description = "Use the mouse to cut (remove) some links";
 
@@ -885,7 +922,7 @@ void NODE_OT_links_cut(wmOperatorType *ot)
 	ot->exec = cut_links_exec;
 	ot->cancel = WM_gesture_lines_cancel;
 
-	ot->poll = ED_operator_node_active;
+	ot->poll = ED_operator_node_editable;
 
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -904,7 +941,7 @@ static int detach_links_exec(bContext *C, wmOperator *UNUSED(op))
 	bNodeTree *ntree = snode->edittree;
 	bNode *node;
 
-	ED_preview_kill_jobs(C);
+	ED_preview_kill_jobs(CTX_wm_manager(C), CTX_data_main(C));
 
 	for (node = ntree->nodes.first; node; node = node->next) {
 		if (node->flag & SELECT) {
@@ -912,7 +949,7 @@ static int detach_links_exec(bContext *C, wmOperator *UNUSED(op))
 		}
 	}
 
-	ntreeUpdateTree(ntree);
+	ntreeUpdateTree(CTX_data_main(C), ntree);
 
 	snode_notify(C, snode);
 	snode_dag_update(C, snode);
@@ -927,41 +964,13 @@ void NODE_OT_links_detach(wmOperatorType *ot)
 	ot->description = "Remove all links to selected nodes, and try to connect neighbor nodes together";
 
 	ot->exec = detach_links_exec;
-	ot->poll = ED_operator_node_active;
+	ot->poll = ED_operator_node_editable;
 
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 }
 
 
-/* ****************** Show Cyclic Dependencies Operator  ******************* */
-
-static int node_show_cycles_exec(bContext *C, wmOperator *UNUSED(op))
-{
-	SpaceNode *snode = CTX_wm_space_node(C);
-
-	/* this is just a wrapper around this call... */
-	ntreeUpdateTree(snode->nodetree);
-	snode_notify(C, snode);
-
-	return OPERATOR_FINISHED;
-}
-
-void NODE_OT_show_cyclic_dependencies(wmOperatorType *ot)
-{
-	/* identifiers */
-	ot->name = "Show Cyclic Dependencies";
-	ot->description = "Sort the nodes and show the cyclic dependencies between the nodes";
-	ot->idname = "NODE_OT_show_cyclic_dependencies";
-
-	/* callbacks */
-	ot->exec = node_show_cycles_exec;
-	ot->poll = ED_operator_node_active;
-
-	/* flags */
-	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
-}
-
 /* ****************** Set Parent ******************* */
 
 static int node_parent_set_exec(bContext *C, wmOperator *UNUSED(op))
@@ -996,41 +1005,7 @@ void NODE_OT_parent_set(wmOperatorType *ot)
 
 	/* api callbacks */
 	ot->exec = node_parent_set_exec;
-	ot->poll = ED_operator_node_active;
-
-	/* flags */
-	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
-}
-
-/* ****************** Clear Parent ******************* */
-
-static int node_parent_clear_exec(bContext *C, wmOperator *UNUSED(op))
-{
-	SpaceNode *snode = CTX_wm_space_node(C);
-	bNodeTree *ntree = snode->edittree;
-	bNode *node;
-
-	for (node = ntree->nodes.first; node; node = node->next) {
-		if (node->flag & NODE_SELECT) {
-			nodeDetachNode(node);
-		}
-	}
-
-	WM_event_add_notifier(C, NC_NODE | ND_DISPLAY, NULL);
-
-	return OPERATOR_FINISHED;
-}
-
-void NODE_OT_parent_clear(wmOperatorType *ot)
-{
-	/* identifiers */
-	ot->name = "Clear Parent";
-	ot->description = "Detach selected nodes";
-	ot->idname = "NODE_OT_parent_clear";
-
-	/* api callbacks */
-	ot->exec = node_parent_clear_exec;
-	ot->poll = ED_operator_node_active;
+	ot->poll = ED_operator_node_editable;
 
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -1073,11 +1048,8 @@ static void node_join_attach_recursive(bNode *node, bNode *frame)
 static int node_join_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	SpaceNode *snode = CTX_wm_space_node(C);
-	Main *bmain = CTX_data_main(C);
-	Scene *scene = CTX_data_scene(C);
 	bNodeTree *ntree = snode->edittree;
 	bNode *node, *frame;
-	bNodeTemplate ntemp;
 
 	/* XXX save selection: node_add_node call below sets the new frame as single active+selected node */
 	for (node = ntree->nodes.first; node; node = node->next) {
@@ -1087,10 +1059,7 @@ static int node_join_exec(bContext *C, wmOperator *UNUSED(op))
 			node->flag &= ~NODE_TEST;
 	}
 
-	ntemp.main = bmain;
-	ntemp.scene = scene;
-	ntemp.type = NODE_FRAME;
-	frame = node_add_node(snode, bmain, scene, &ntemp, 0.0f, 0.0f);
+	frame = node_add_node(C, NULL, NODE_FRAME, 0.0f, 0.0f);
 
 	/* reset tags */
 	for (node = ntree->nodes.first; node; node = node->next)
@@ -1122,7 +1091,7 @@ void NODE_OT_join(wmOperatorType *ot)
 
 	/* api callbacks */
 	ot->exec = node_join_exec;
-	ot->poll = ED_operator_node_active;
+	ot->poll = ED_operator_node_editable;
 
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -1130,18 +1099,23 @@ void NODE_OT_join(wmOperatorType *ot)
 
 /* ****************** Attach ******************* */
 
-static int node_attach_exec(bContext *C, wmOperator *UNUSED(op))
+static int node_attach_invoke(bContext *C, wmOperator *UNUSED(op), const wmEvent *event)
 {
+	ARegion *ar = CTX_wm_region(C);
 	SpaceNode *snode = CTX_wm_space_node(C);
 	bNodeTree *ntree = snode->edittree;
 	bNode *frame;
+	float cursor[2];
+	
+	/* convert mouse coordinates to v2d space */
+	UI_view2d_region_to_view(&ar->v2d, event->mval[0], event->mval[1], &cursor[0], &cursor[1]);
 
 	/* check nodes front to back */
 	for (frame = ntree->nodes.last; frame; frame = frame->prev) {
 		/* skip selected, those are the nodes we want to attach */
 		if ((frame->type != NODE_FRAME) || (frame->flag & NODE_SELECT))
 			continue;
-		if (BLI_rctf_isect_pt(&frame->totr, snode->cursor[0], snode->cursor[1]))
+		if (BLI_rctf_isect_pt(&frame->totr, cursor[0], cursor[1]))
 			break;
 	}
 	if (frame) {
@@ -1150,7 +1124,7 @@ static int node_attach_exec(bContext *C, wmOperator *UNUSED(op))
 			if (node->flag & NODE_SELECT) {
 				if (node->parent == NULL) {
 					/* disallow moving a parent into its child */
-					if (nodeAttachNodeCheck(frame, node) == FALSE) {
+					if (nodeAttachNodeCheck(frame, node) == false) {
 						/* attach all unparented nodes */
 						nodeAttachNode(node, frame);
 					}
@@ -1165,7 +1139,7 @@ static int node_attach_exec(bContext *C, wmOperator *UNUSED(op))
 
 					if (parent) {
 						/* disallow moving a parent into its child */
-						if (nodeAttachNodeCheck(frame, node) == FALSE) {
+						if (nodeAttachNodeCheck(frame, node) == false) {
 							nodeDetachNode(node);
 							nodeAttachNode(node, frame);
 						}
@@ -1181,16 +1155,6 @@ static int node_attach_exec(bContext *C, wmOperator *UNUSED(op))
 	return OPERATOR_FINISHED;
 }
 
-static int node_attach_invoke(bContext *C, wmOperator *op, wmEvent *event)
-{
-	ARegion *ar = CTX_wm_region(C);
-	SpaceNode *snode = CTX_wm_space_node(C);
-
-	/* convert mouse coordinates to v2d space */
-	UI_view2d_region_to_view(&ar->v2d, event->mval[0], event->mval[1], &snode->cursor[0], &snode->cursor[1]);
-
-	return node_attach_exec(C, op);
-}
 
 void NODE_OT_attach(wmOperatorType *ot)
 {
@@ -1200,9 +1164,9 @@ void NODE_OT_attach(wmOperatorType *ot)
 	ot->idname = "NODE_OT_attach";
 
 	/* api callbacks */
-	ot->exec = node_attach_exec;
+	
 	ot->invoke = node_attach_invoke;
-	ot->poll = ED_operator_node_active;
+	ot->poll = ED_operator_node_editable;
 
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -1271,7 +1235,7 @@ void NODE_OT_detach(wmOperatorType *ot)
 
 	/* api callbacks */
 	ot->exec = node_detach_exec;
-	ot->poll = ED_operator_node_active;
+	ot->poll = ED_operator_node_editable;
 
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -1281,49 +1245,62 @@ void NODE_OT_detach(wmOperatorType *ot)
 
 
 /* prevent duplicate testing code below */
-static SpaceNode *ed_node_link_conditions(ScrArea *sa, bNode **select)
+static bool ed_node_link_conditions(ScrArea *sa, bool test, SpaceNode **r_snode, bNode **r_select)
 {
 	SpaceNode *snode = sa ? sa->spacedata.first : NULL;
-	bNode *node;
+	bNode *node, *select = NULL;
 	bNodeLink *link;
 
-	/* no unlucky accidents */
-	if (sa == NULL || sa->spacetype != SPACE_NODE) return NULL;
+	*r_snode = snode;
+	*r_select = NULL;
 
-	*select = NULL;
+	/* no unlucky accidents */
+	if (sa == NULL || sa->spacetype != SPACE_NODE)
+		return false;
+
+	if (!test) {
+		/* no need to look for a node */
+		return true;
+	}
 
 	for (node = snode->edittree->nodes.first; node; node = node->next) {
 		if (node->flag & SELECT) {
-			if (*select)
+			if (select)
 				break;
 			else
-				*select = node;
+				select = node;
 		}
 	}
 	/* only one selected */
-	if (node || *select == NULL) return NULL;
+	if (node || select == NULL)
+		return false;
 
 	/* correct node */
-	if ((*select)->inputs.first == NULL || (*select)->outputs.first == NULL) return NULL;
+	if (BLI_listbase_is_empty(&select->inputs) || BLI_listbase_is_empty(&select->outputs))
+		return false;
 
 	/* test node for links */
 	for (link = snode->edittree->links.first; link; link = link->next) {
-		if (link->tonode == *select || link->fromnode == *select)
-			return NULL;
+		if (nodeLinkIsHidden(link))
+			continue;
+		
+		if (link->tonode == select || link->fromnode == select)
+			return false;
 	}
 
-	return snode;
+	*r_select = select;
+	return true;
 }
 
 /* test == 0, clear all intersect flags */
 void ED_node_link_intersect_test(ScrArea *sa, int test)
 {
 	bNode *select;
-	SpaceNode *snode = ed_node_link_conditions(sa, &select);
+	SpaceNode *snode;
 	bNodeLink *link, *selink = NULL;
 	float mcoords[6][2];
 
-	if (snode == NULL) return;
+	if (!ed_node_link_conditions(sa, test, &snode, &select)) return;
 
 	/* clear flags */
 	for (link = snode->edittree->links.first; link; link = link->next)
@@ -1348,7 +1325,9 @@ void ED_node_link_intersect_test(ScrArea *sa, int test)
 	/* we only tag a single link for intersect now */
 	/* idea; use header dist when more? */
 	for (link = snode->edittree->links.first; link; link = link->next) {
-
+		if (nodeLinkIsHidden(link))
+			continue;
+		
 		if (cut_links_intersect(link, mcoords, 5)) { /* intersect code wants edges */
 			if (selink)
 				break;
@@ -1368,7 +1347,7 @@ static bNodeSocket *socket_best_match(ListBase *sockets)
 
 	/* find type range */
 	for (sock = sockets->first; sock; sock = sock->next)
-		maxtype = MAX2(sock->type, maxtype);
+		maxtype = max_ii(sock->type, maxtype);
 
 	/* try all types, starting from 'highest' (i.e. colors, vectors, values) */
 	for (type = maxtype; type >= 0; --type) {
@@ -1396,11 +1375,11 @@ static bNodeSocket *socket_best_match(ListBase *sockets)
 void ED_node_link_insert(ScrArea *sa)
 {
 	bNode *node, *select;
-	SpaceNode *snode = ed_node_link_conditions(sa, &select);
+	SpaceNode *snode;
 	bNodeLink *link;
 	bNodeSocket *sockto;
 
-	if (snode == NULL) return;
+	if (!ed_node_link_conditions(sa, true, &snode, &select)) return;
 
 	/* get the link */
 	for (link = snode->edittree->links.first; link; link = link->next)
@@ -1408,16 +1387,23 @@ void ED_node_link_insert(ScrArea *sa)
 			break;
 
 	if (link) {
-		node = link->tonode;
-		sockto = link->tosock;
-
-		link->tonode = select;
-		link->tosock = socket_best_match(&select->inputs);
-		link->flag &= ~NODE_LINKFLAG_HILITE;
-
-		nodeAddLink(snode->edittree, select, socket_best_match(&select->outputs), node, sockto);
-		ntreeUpdateTree(snode->edittree);   /* needed for pointers */
-		snode_update(snode, select);
-		ED_node_changed_update(snode->id, select);
+		bNodeSocket *best_input = socket_best_match(&select->inputs);
+		bNodeSocket *best_output = socket_best_match(&select->outputs);
+		
+		if (best_input && best_output) {
+			node = link->tonode;
+			sockto = link->tosock;
+			
+			link->tonode = select;
+			link->tosock = best_input;
+			node_remove_extra_links(snode, link, false);
+			link->flag &= ~NODE_LINKFLAG_HILITE;
+			
+			nodeAddLink(snode->edittree, select, best_output, node, sockto);
+			
+			ntreeUpdateTree(G.main, snode->edittree);   /* needed for pointers */
+			snode_update(snode, select);
+			ED_node_tag_update_id(snode->id);
+		}
 	}
 }
