@@ -81,7 +81,7 @@ static int outliner_open_back(SpaceOops *soops, TreeElement *te)
 	
 	for (te = te->parent; te; te = te->parent) {
 		tselem = TREESTORE(te);
-		if (tselem->flag & TSE_CLOSED) { 
+		if (tselem->flag & TSE_CLOSED) {
 			tselem->flag &= ~TSE_CLOSED;
 			retval = 1;
 		}
@@ -113,12 +113,43 @@ static void outliner_open_reveal(SpaceOops *soops, ListBase *lb, TreeElement *te
 }
 #endif
 
+static TreeElement *outliner_dropzone_element(const SpaceOops *soops, TreeElement *te, const float fmval[2], const int children)
+{
+	if ((fmval[1] > te->ys) && (fmval[1] < (te->ys + UI_UNIT_Y))) {
+		/* name and first icon */
+		if ((fmval[0] > te->xs + UI_UNIT_X) && (fmval[0] < te->xend))
+			return te;
+	}
+	/* Not it.  Let's look at its children. */
+	if (children && (TREESTORE(te)->flag & TSE_CLOSED) == 0 && (te->subtree.first)) {
+		for (te = te->subtree.first; te; te = te->next) {
+			TreeElement *te_valid = outliner_dropzone_element(soops, te, fmval, children);
+			if (te_valid)
+				return te_valid;
+		}
+	}
+	return NULL;
+}
+
+/* Used for drag and drop parenting */
+TreeElement *outliner_dropzone_find(const SpaceOops *soops, const float fmval[2], const int children)
+{
+	TreeElement *te;
+
+	for (te = soops->tree.first; te; te = te->next) {
+		TreeElement *te_valid = outliner_dropzone_element(soops, te, fmval, children);
+		if (te_valid)
+			return te_valid;
+	}
+	return NULL;
+}
+
 /* ************************************************************** */
 /* Click Activated */
 
 /* Toggle Open/Closed ------------------------------------------- */
 
-static int do_outliner_item_openclose(bContext *C, SpaceOops *soops, TreeElement *te, int all, const float mval[2])
+static int do_outliner_item_openclose(bContext *C, SpaceOops *soops, TreeElement *te, const bool all, const float mval[2])
 {
 	
 	if (mval[1] > te->ys && mval[1] < te->ys + UI_UNIT_Y) {
@@ -146,15 +177,15 @@ static int do_outliner_item_openclose(bContext *C, SpaceOops *soops, TreeElement
 }
 
 /* event can enterkey, then it opens/closes */
-static int outliner_item_openclose(bContext *C, wmOperator *op, wmEvent *event)
+static int outliner_item_openclose(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	ARegion *ar = CTX_wm_region(C);
 	SpaceOops *soops = CTX_wm_space_outliner(C);
 	TreeElement *te;
 	float fmval[2];
-	int all = RNA_boolean_get(op->ptr, "all");
+	const bool all = RNA_boolean_get(op->ptr, "all");
 	
-	UI_view2d_region_to_view(&ar->v2d, event->mval[0], event->mval[1], fmval, fmval + 1);
+	UI_view2d_region_to_view(&ar->v2d, event->mval[0], event->mval[1], &fmval[0], &fmval[1]);
 	
 	for (te = soops->tree.first; te; te = te->next) {
 		if (do_outliner_item_openclose(C, soops, te, all, fmval)) 
@@ -183,24 +214,24 @@ void OUTLINER_OT_item_openclose(wmOperatorType *ot)
 
 static void do_item_rename(ARegion *ar, TreeElement *te, TreeStoreElem *tselem, ReportList *reports)
 {
-	/* can't rename rna datablocks entries */
-	if (ELEM3(tselem->type, TSE_RNA_STRUCT, TSE_RNA_PROPERTY, TSE_RNA_ARRAY_ELEM)) {
+	/* can't rename rna datablocks entries or listbases */
+	if (ELEM(tselem->type, TSE_RNA_STRUCT, TSE_RNA_PROPERTY, TSE_RNA_ARRAY_ELEM, TSE_ID_BASE)) {
 		/* do nothing */;
 	}
-	else if (ELEM10(tselem->type, TSE_ANIM_DATA, TSE_NLA, TSE_DEFGROUP_BASE, TSE_CONSTRAINT_BASE, TSE_MODIFIER_BASE,
-	                TSE_SCRIPT_BASE, TSE_POSE_BASE, TSE_POSEGRP_BASE, TSE_R_LAYER_BASE, TSE_R_PASS))
+	else if (ELEM(tselem->type, TSE_ANIM_DATA, TSE_NLA, TSE_DEFGROUP_BASE, TSE_CONSTRAINT_BASE, TSE_MODIFIER_BASE,
+	              TSE_DRIVER_BASE, TSE_POSE_BASE, TSE_POSEGRP_BASE, TSE_R_LAYER_BASE, TSE_R_PASS))
 	{
 		BKE_report(reports, RPT_WARNING, "Cannot edit builtin name");
 	}
-	else if (ELEM3(tselem->type, TSE_SEQUENCE, TSE_SEQ_STRIP, TSE_SEQUENCE_DUP)) {
+	else if (ELEM(tselem->type, TSE_SEQUENCE, TSE_SEQ_STRIP, TSE_SEQUENCE_DUP)) {
 		BKE_report(reports, RPT_WARNING, "Cannot edit sequence name");
 	}
 	else if (tselem->id->lib) {
-		// XXX						error_libdata();
-	} 
+		BKE_report(reports, RPT_WARNING, "Cannot edit external libdata");
+	}
 	else if (te->idcode == ID_LI && te->parent) {
 		BKE_report(reports, RPT_WARNING, "Cannot edit the path of an indirectly linked library");
-	} 
+	}
 	else {
 		tselem->flag |= TSE_TEXTBUT;
 		ED_region_tag_redraw(ar);
@@ -222,12 +253,12 @@ static int do_outliner_item_rename(bContext *C, ARegion *ar, SpaceOops *soops, T
 	if (mval[1] > te->ys && mval[1] < te->ys + UI_UNIT_Y) {
 		TreeStoreElem *tselem = TREESTORE(te);
 		
-		/* name and first icon */
-		if (mval[0] > te->xs + UI_UNIT_X && mval[0] < te->xend) {
-			
+		/* click on name */
+		if (mval[0] > te->xs + UI_UNIT_X * 2 && mval[0] < te->xend) {
 			do_item_rename(ar, te, tselem, reports);
+			return 1;
 		}
-		return 1;
+		return 0;
 	}
 	
 	for (te = te->subtree.first; te; te = te->next) {
@@ -236,20 +267,24 @@ static int do_outliner_item_rename(bContext *C, ARegion *ar, SpaceOops *soops, T
 	return 0;
 }
 
-static int outliner_item_rename(bContext *C, wmOperator *UNUSED(op), wmEvent *event)
+static int outliner_item_rename(bContext *C, wmOperator *UNUSED(op), const wmEvent *event)
 {
 	ARegion *ar = CTX_wm_region(C);
 	SpaceOops *soops = CTX_wm_space_outliner(C);
 	TreeElement *te;
 	float fmval[2];
+	bool changed = false;
 	
-	UI_view2d_region_to_view(&ar->v2d, event->mval[0], event->mval[1], fmval, fmval + 1);
+	UI_view2d_region_to_view(&ar->v2d, event->mval[0], event->mval[1], &fmval[0], &fmval[1]);
 	
 	for (te = soops->tree.first; te; te = te->next) {
-		if (do_outliner_item_rename(C, ar, soops, te, fmval)) break;
+		if (do_outliner_item_rename(C, ar, soops, te, fmval)) {
+			changed = true;
+			break;
+		}
 	}
 	
-	return OPERATOR_FINISHED;
+	return changed ? OPERATOR_FINISHED : OPERATOR_PASS_THROUGH;
 }
 
 
@@ -272,7 +307,7 @@ void OUTLINER_OT_item_rename(wmOperatorType *ot)
 
 /* Apply Settings ------------------------------- */
 
-static int outliner_count_levels(SpaceOops *soops, ListBase *lb, int curlevel)
+static int outliner_count_levels(SpaceOops *soops, ListBase *lb, const int curlevel)
 {
 	TreeElement *te;
 	int level = curlevel, lev;
@@ -285,7 +320,7 @@ static int outliner_count_levels(SpaceOops *soops, ListBase *lb, int curlevel)
 	return level;
 }
 
-int outliner_has_one_flag(SpaceOops *soops, ListBase *lb, short flag, short curlevel)
+int outliner_has_one_flag(SpaceOops *soops, ListBase *lb, short flag, const int curlevel)
 {
 	TreeElement *te;
 	TreeStoreElem *tselem;
@@ -368,12 +403,14 @@ void group_toggle_visibility_cb(bContext *UNUSED(C), Scene *scene, TreeElement *
 
 static int outliner_toggle_visibility_exec(bContext *C, wmOperator *UNUSED(op))
 {
+	Main *bmain = CTX_data_main(C);
 	SpaceOops *soops = CTX_wm_space_outliner(C);
 	Scene *scene = CTX_data_scene(C);
 	ARegion *ar = CTX_wm_region(C);
 	
 	outliner_do_object_operation(C, scene, soops, &soops->tree, object_toggle_visibility_cb);
 	
+	DAG_id_type_tag(bmain, ID_OB);
 	WM_event_add_notifier(C, NC_SCENE | ND_OB_VISIBLE, scene);
 	ED_region_tag_redraw(ar);
 	
@@ -464,11 +501,13 @@ void group_toggle_renderability_cb(bContext *UNUSED(C), Scene *scene, TreeElemen
 
 static int outliner_toggle_renderability_exec(bContext *C, wmOperator *UNUSED(op))
 {
+	Main *bmain = CTX_data_main(C);
 	SpaceOops *soops = CTX_wm_space_outliner(C);
 	Scene *scene = CTX_data_scene(C);
 	
 	outliner_do_object_operation(C, scene, soops, &soops->tree, object_toggle_renderability_cb);
 	
+	DAG_id_type_tag(bmain, ID_OB);
 	WM_event_add_notifier(C, NC_SCENE | ND_OB_RENDER, scene);
 	
 	return OPERATOR_FINISHED;
@@ -579,7 +618,7 @@ static int outliner_show_active_exec(bContext *C, wmOperator *UNUSED(op))
 	te = outliner_find_id(so, &so->tree, (ID *)OBACT);
 	if (te) {
 		/* make te->ys center of view */
-		ytop = (int)(te->ys + BLI_rcti_size_y(&v2d->mask) / 2);
+		ytop = te->ys + BLI_rcti_size_y(&v2d->mask) / 2;
 		if (ytop > 0) ytop = 0;
 		
 		v2d->cur.ymax = (float)ytop;
@@ -726,7 +765,7 @@ static void outliner_find_panel(Scene *UNUSED(scene), ARegion *ar, SpaceOops *so
 	
 	/* determine which type of search to do */
 	if (again && last_find) {
-		/* no popup panel - previous + user wanted to search for next after previous */		
+		/* no popup panel - previous + user wanted to search for next after previous */
 		BLI_strncpy(name, soops->search_string, sizeof(name));
 		flags = soops->search_flags;
 		
@@ -742,7 +781,7 @@ static void outliner_find_panel(Scene *UNUSED(scene), ARegion *ar, SpaceOops *so
 		/* pop up panel - no previous, or user didn't want search after previous */
 		name[0] = '\0';
 // XXX		if (sbutton(name, 0, sizeof(name)-1, "Find: ") && name[0]) {
-//			te= outliner_find_name(soops, &soops->tree, name, flags, NULL, &prevFound);
+//			te = outliner_find_name(soops, &soops->tree, name, flags, NULL, &prevFound);
 //		}
 //		else return; /* XXX RETURN! XXX */
 	}
@@ -782,7 +821,7 @@ static void outliner_find_panel(Scene *UNUSED(scene), ARegion *ar, SpaceOops *so
 	}
 	else {
 		/* no tree-element found */
-		BKE_report(reports, RPT_WARNING, "Not found: %s", name);
+		BKE_reportf(reports, RPT_WARNING, "Not found: %s", name);
 	}
 }
 #endif
@@ -813,7 +852,7 @@ static int outliner_one_level_exec(bContext *C, wmOperator *op)
 {
 	SpaceOops *soops = CTX_wm_space_outliner(C);
 	ARegion *ar = CTX_wm_region(C);
-	int add = RNA_boolean_get(op->ptr, "open");
+	const bool add = RNA_boolean_get(op->ptr, "open");
 	int level;
 	
 	level = outliner_has_one_flag(soops, &soops->tree, TSE_CLOSED, 1);
@@ -832,6 +871,8 @@ static int outliner_one_level_exec(bContext *C, wmOperator *op)
 
 void OUTLINER_OT_show_one_level(wmOperatorType *ot)
 {
+	PropertyRNA *prop;
+
 	/* identifiers */
 	ot->name = "Show/Hide One Level";
 	ot->idname = "OUTLINER_OT_show_one_level";
@@ -844,7 +885,8 @@ void OUTLINER_OT_show_one_level(wmOperatorType *ot)
 	/* no undo or registry, UI option */
 	
 	/* properties */
-	RNA_def_boolean(ot->srna, "open", 1, "Open", "Expand all entries one level deep");
+	prop = RNA_def_boolean(ot->srna, "open", 1, "Open", "Expand all entries one level deep");
+	RNA_def_property_flag(prop, PROP_SKIP_SAVE);
 }
 
 /* Show Hierarchy ----------------------------------------------- */
@@ -883,9 +925,13 @@ static void tree_element_show_hierarchy(Scene *scene, SpaceOops *soops, ListBase
 				else tselem->flag |= TSE_CLOSED;
 			}
 		}
-		else tselem->flag |= TSE_CLOSED;
-		
-		if (TSELEM_OPEN(tselem, soops)) tree_element_show_hierarchy(scene, soops, &te->subtree);
+		else {
+			tselem->flag |= TSE_CLOSED;
+		}
+
+		if (TSELEM_OPEN(tselem, soops)) {
+			tree_element_show_hierarchy(scene, soops, &te->subtree);
+		}
 	}
 }
 
@@ -938,7 +984,7 @@ static int ed_operator_outliner_datablocks_active(bContext *C)
  * NOTE: the caller must zero-out all values of the pointers that it passes here first, as
  * this function does not do that yet 
  */
-static void tree_element_to_path(SpaceOops *soops, TreeElement *te, TreeStoreElem *tselem, 
+static void tree_element_to_path(TreeElement *te, TreeStoreElem *tselem,
                                  ID **id, char **path, int *array_index, short *flag, short *UNUSED(groupmode))
 {
 	ListBase hierarchy = {NULL, NULL};
@@ -994,7 +1040,7 @@ static void tree_element_to_path(SpaceOops *soops, TreeElement *te, TreeStoreEle
 					char buf[128], *name;
 					
 					temnext = (TreeElement *)(ld->next->data);
-					/* tsenext= TREESTORE(temnext); */ /* UNUSED */
+					/* tsenext = TREESTORE(temnext); */ /* UNUSED */
 					
 					nextptr = &temnext->rnaptr;
 					name = RNA_struct_name_get_alloc(nextptr, buf, sizeof(buf), NULL);
@@ -1056,7 +1102,7 @@ static void tree_element_to_path(SpaceOops *soops, TreeElement *te, TreeStoreEle
 			/* item is part of an array, so must set the array_index */
 			*array_index = te->index;
 		}
-		else if (RNA_property_array_length(ptr, prop)) {
+		else if (RNA_property_array_check(prop)) {
 			/* entire array was selected, so keyframe all */
 			*flag |= KSP_FLAG_WHOLE_ARRAY;
 		}
@@ -1106,7 +1152,7 @@ static void do_outliner_drivers_editop(SpaceOops *soops, ListBase *tree, ReportL
 			    RNA_property_animateable(&te->rnaptr, te->directdata))
 			{
 				/* get id + path + index info from the selected element */
-				tree_element_to_path(soops, te, tselem, 
+				tree_element_to_path(te, tselem,
 				                     &id, &path, &array_index, &flag, &groupmode);
 			}
 			
@@ -1135,14 +1181,14 @@ static void do_outliner_drivers_editop(SpaceOops *soops, ListBase *tree, ReportL
 						{
 							/* add a new driver with the information obtained (only if valid) */
 							ANIM_add_driver(reports, id, path, array_index, dflags, DRIVER_TYPE_PYTHON);
+							break;
 						}
-						break;
 						case DRIVERS_EDITMODE_REMOVE:
 						{
 							/* remove driver matching the information obtained (only if valid) */
 							ANIM_remove_driver(reports, id, path, array_index, dflags);
+							break;
 						}
-						break;
 					}
 				}
 				
@@ -1259,7 +1305,7 @@ static KeyingSet *verify_active_keyingset(Scene *scene, short add)
 	// XXX the default settings have yet to evolve
 	if ((add) && (ks == NULL)) {
 		ks = BKE_keyingset_add(&scene->keyingsets, NULL, NULL, KEYINGSET_ABSOLUTE, 0);
-		scene->active_keyingset = BLI_countlist(&scene->keyingsets);
+		scene->active_keyingset = BLI_listbase_count(&scene->keyingsets);
 	}
 	
 	return ks;
@@ -1287,7 +1333,7 @@ static void do_outliner_keyingset_editop(SpaceOops *soops, KeyingSet *ks, ListBa
 			    RNA_property_animateable(&te->rnaptr, te->directdata))
 			{
 				/* get id + path + index info from the selected element */
-				tree_element_to_path(soops, te, tselem, 
+				tree_element_to_path(te, tselem,
 				                     &id, &path, &array_index, &flag, &groupmode);
 			}
 			
@@ -1301,9 +1347,9 @@ static void do_outliner_keyingset_editop(SpaceOops *soops, KeyingSet *ks, ListBa
 						/* TODO: what do we do with group name?
 						 * for now, we don't supply one, and just let this use the KeyingSet name */
 						BKE_keyingset_add_path(ks, id, NULL, path, array_index, flag, groupmode);
-						ks->active_path = BLI_countlist(&ks->paths);
+						ks->active_path = BLI_listbase_count(&ks->paths);
+						break;
 					}
-					break;
 					case KEYINGSET_EDITMODE_REMOVE:
 					{
 						/* find the relevant path, then remove it from the KeyingSet */
@@ -1315,8 +1361,8 @@ static void do_outliner_keyingset_editop(SpaceOops *soops, KeyingSet *ks, ListBa
 
 							ks->active_path = 0;
 						}
+						break;
 					}
-					break;
 				}
 				
 				/* free path, since it had to be generated */
@@ -1340,7 +1386,7 @@ static int outliner_keyingset_additems_exec(bContext *C, wmOperator *op)
 	
 	/* check for invalid states */
 	if (ks == NULL) {
-		BKE_report(op->reports, RPT_ERROR, "Operation requires an Active Keying Set");
+		BKE_report(op->reports, RPT_ERROR, "Operation requires an active keying set");
 		return OPERATOR_CANCELLED;
 	}
 	if (soutliner == NULL)
@@ -1423,47 +1469,21 @@ static int parent_drop_exec(bContext *C, wmOperator *op)
 	RNA_string_get(op->ptr, "child", childname);
 	ob = (Object *)BKE_libblock_find_name(ID_OB, childname);
 
-	ED_object_parent_set(op->reports, bmain, scene, ob, par, partype, FALSE, FALSE);
+	if (ob->id.lib) {
+		BKE_report(op->reports, RPT_INFO, "Can't edit library linked object");
+		return OPERATOR_CANCELLED;
+	}
 
-	DAG_scene_sort(bmain, scene);
-	DAG_ids_flush_update(bmain, 0);
+	ED_object_parent_set(op->reports, bmain, scene, ob, par, partype, false, false, NULL);
+
+	DAG_relations_tag_update(bmain);
 	WM_event_add_notifier(C, NC_OBJECT | ND_TRANSFORM, NULL);
 	WM_event_add_notifier(C, NC_OBJECT | ND_PARENT, NULL);
 
 	return OPERATOR_FINISHED;
 }
 
-/* Used for drag and drop parenting */
-TreeElement *outliner_dropzone_parent(bContext *C, wmEvent *event, TreeElement *te, float *fmval)
-{
-	SpaceOops *soops = CTX_wm_space_outliner(C);
-	TreeStoreElem *tselem = TREESTORE(te);
-
-	if ((fmval[1] > te->ys) && (fmval[1] < (te->ys + UI_UNIT_Y))) {
-		/* name and first icon */
-		if ((fmval[0] > te->xs + UI_UNIT_X) && (fmval[0] < te->xend)) {
-			/* always makes active object */
-			if (te->idcode == ID_OB && tselem->type == 0) {
-				return te;
-			}
-			else {
-				return NULL;
-			}
-		}
-	}
-
-	/* Not it.  Let's look at its children. */
-	if ((tselem->flag & TSE_CLOSED) == 0 && (te->subtree.first)) {
-		for (te = te->subtree.first; te; te = te->next) {
-			TreeElement *te_valid;
-			te_valid = outliner_dropzone_parent(C, event, te, fmval);
-			if (te_valid) return te_valid;
-		}
-	}
-	return NULL;
-}
-
-static int parent_drop_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int parent_drop_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	Object *par = NULL;
 	Object *ob = NULL;
@@ -1472,7 +1492,6 @@ static int parent_drop_invoke(bContext *C, wmOperator *op, wmEvent *event)
 	Main *bmain = CTX_data_main(C);
 	Scene *scene = NULL;
 	TreeElement *te = NULL;
-	TreeElement *te_found = NULL;
 	char childname[MAX_ID_NAME];
 	char parname[MAX_ID_NAME];
 	int partype = 0;
@@ -1481,13 +1500,10 @@ static int parent_drop_invoke(bContext *C, wmOperator *op, wmEvent *event)
 	UI_view2d_region_to_view(&ar->v2d, event->mval[0], event->mval[1], &fmval[0], &fmval[1]);
 
 	/* Find object hovered over */
-	for (te = soops->tree.first; te; te = te->next) {
-		te_found = outliner_dropzone_parent(C, event, te, fmval);
-		if (te_found) break;
-	}
+	te = outliner_dropzone_find(soops, fmval, 1);
 
-	if (te_found) {
-		RNA_string_set(op->ptr, "parent", te_found->name);
+	if (te) {
+		RNA_string_set(op->ptr, "parent", te->name);
 		/* Identify parent and child */
 		RNA_string_get(op->ptr, "child", childname);
 		ob = (Object *)BKE_libblock_find_name(ID_OB, childname);
@@ -1501,8 +1517,12 @@ static int parent_drop_invoke(bContext *C, wmOperator *op, wmEvent *event)
 		if (ob == par) {
 			return OPERATOR_CANCELLED;
 		}
+		if (ob->id.lib) {
+			BKE_report(op->reports, RPT_INFO, "Can't edit library linked object");
+			return OPERATOR_CANCELLED;
+		}
 		
-		scene = (Scene *)outliner_search_back(soops, te_found, ID_SCE);
+		scene = (Scene *)outliner_search_back(soops, te, ID_SCE);
 
 		if (scene == NULL) {
 			/* currently outlier organized in a way, that if there's no parent scene
@@ -1514,99 +1534,90 @@ static int parent_drop_invoke(bContext *C, wmOperator *op, wmEvent *event)
 		}
 
 		if ((par->type != OB_ARMATURE) && (par->type != OB_CURVE) && (par->type != OB_LATTICE)) {
-			if (ED_object_parent_set(op->reports, bmain, scene, ob, par, partype, FALSE, FALSE)) {
-				DAG_scene_sort(bmain, scene);
-				DAG_ids_flush_update(bmain, 0);
+			if (ED_object_parent_set(op->reports, bmain, scene, ob, par, partype, false, false, NULL)) {
+				DAG_relations_tag_update(bmain);
 				WM_event_add_notifier(C, NC_OBJECT | ND_TRANSFORM, NULL);
 				WM_event_add_notifier(C, NC_OBJECT | ND_PARENT, NULL);
 			}
 		}
 		else {
 			/* Menu creation */
-			uiPopupMenu *pup = uiPupMenuBegin(C, IFACE_("Set Parent To"), ICON_NONE);
-			uiLayout *layout = uiPupMenuLayout(pup);
+			wmOperatorType *ot = WM_operatortype_find("OUTLINER_OT_parent_drop", false);
+			uiPopupMenu *pup = UI_popup_menu_begin(C, IFACE_("Set Parent To"), ICON_NONE);
+			uiLayout *layout = UI_popup_menu_layout(pup);
 			
 			PointerRNA ptr;
 			
-			WM_operator_properties_create(&ptr, "OUTLINER_OT_parent_drop");
+			WM_operator_properties_create_ptr(&ptr, ot);
 			RNA_string_set(&ptr, "parent", parname);
 			RNA_string_set(&ptr, "child", childname);
 			RNA_enum_set(&ptr, "type", PAR_OBJECT);
 			/* Cannot use uiItemEnumO()... have multiple properties to set. */
-			uiItemFullO(layout, "OUTLINER_OT_parent_drop", IFACE_("Object"),
-			            0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
+			uiItemFullO_ptr(layout, ot, IFACE_("Object"), 0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
 			
 			/* par becomes parent, make the associated menus */
 			if (par->type == OB_ARMATURE) {
-				WM_operator_properties_create(&ptr, "OUTLINER_OT_parent_drop");
+				WM_operator_properties_create_ptr(&ptr, ot);
 				RNA_string_set(&ptr, "parent", parname);
 				RNA_string_set(&ptr, "child", childname);
 				RNA_enum_set(&ptr, "type", PAR_ARMATURE);
-				uiItemFullO(layout, "OUTLINER_OT_parent_drop", IFACE_("Armature Deform"),
-				            0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
+				uiItemFullO_ptr(layout, ot, IFACE_("Armature Deform"), 0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
 				
-				WM_operator_properties_create(&ptr, "OUTLINER_OT_parent_drop");
+				WM_operator_properties_create_ptr(&ptr, ot);
 				RNA_string_set(&ptr, "parent", parname);
 				RNA_string_set(&ptr, "child", childname);
 				RNA_enum_set(&ptr, "type", PAR_ARMATURE_NAME);
-				uiItemFullO(layout, "OUTLINER_OT_parent_drop", IFACE_("   With Empty Groups"),
-				            0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
+				uiItemFullO_ptr(layout, ot, IFACE_("   With Empty Groups"), 0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
 				
-				WM_operator_properties_create(&ptr, "OUTLINER_OT_parent_drop");
+				WM_operator_properties_create_ptr(&ptr, ot);
 				RNA_string_set(&ptr, "parent", parname);
 				RNA_string_set(&ptr, "child", childname);
 				RNA_enum_set(&ptr, "type", PAR_ARMATURE_ENVELOPE);
-				uiItemFullO(layout, "OUTLINER_OT_parent_drop", IFACE_("   With Envelope Weights"),
-				            0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
+				uiItemFullO_ptr(layout, ot, IFACE_("   With Envelope Weights"), 0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
 				
-				WM_operator_properties_create(&ptr, "OUTLINER_OT_parent_drop");
+				WM_operator_properties_create_ptr(&ptr, ot);
 				RNA_string_set(&ptr, "parent", parname);
 				RNA_string_set(&ptr, "child", childname);
 				RNA_enum_set(&ptr, "type", PAR_ARMATURE_AUTO);
-				uiItemFullO(layout, "OUTLINER_OT_parent_drop", IFACE_("   With Automatic Weights"),
-				            0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
+				uiItemFullO_ptr(layout, ot, IFACE_("   With Automatic Weights"), 0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
 				
-				WM_operator_properties_create(&ptr, "OUTLINER_OT_parent_drop");
+				WM_operator_properties_create_ptr(&ptr, ot);
 				RNA_string_set(&ptr, "parent", parname);
 				RNA_string_set(&ptr, "child", childname);
 				RNA_enum_set(&ptr, "type", PAR_BONE);
-				uiItemFullO(layout, "OUTLINER_OT_parent_drop", IFACE_("Bone"),
+				uiItemFullO_ptr(layout, ot, IFACE_("Bone"),
 				            0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
 			}
 			else if (par->type == OB_CURVE) {
-				WM_operator_properties_create(&ptr, "OUTLINER_OT_parent_drop");
+				WM_operator_properties_create_ptr(&ptr, ot);
 				RNA_string_set(&ptr, "parent", parname);
 				RNA_string_set(&ptr, "child", childname);
 				RNA_enum_set(&ptr, "type", PAR_CURVE);
-				uiItemFullO(layout, "OUTLINER_OT_parent_drop", IFACE_("Curve Deform"),
-				            0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
+				uiItemFullO_ptr(layout, ot, IFACE_("Curve Deform"), 0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
 				
-				WM_operator_properties_create(&ptr, "OUTLINER_OT_parent_drop");
+				WM_operator_properties_create_ptr(&ptr, ot);
 				RNA_string_set(&ptr, "parent", parname);
 				RNA_string_set(&ptr, "child", childname);
 				RNA_enum_set(&ptr, "type", PAR_FOLLOW);
-				uiItemFullO(layout, "OUTLINER_OT_parent_drop", IFACE_("Follow Path"),
-				            0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
+				uiItemFullO_ptr(layout, ot, IFACE_("Follow Path"), 0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
 				
-				WM_operator_properties_create(&ptr, "OUTLINER_OT_parent_drop");
+				WM_operator_properties_create_ptr(&ptr, ot);
 				RNA_string_set(&ptr, "parent", parname);
 				RNA_string_set(&ptr, "child", childname);
 				RNA_enum_set(&ptr, "type", PAR_PATH_CONST);
-				uiItemFullO(layout, "OUTLINER_OT_parent_drop", IFACE_("Path Constraint"),
-				            0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
+				uiItemFullO_ptr(layout, ot, IFACE_("Path Constraint"), 0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
 			}
 			else if (par->type == OB_LATTICE) {
-				WM_operator_properties_create(&ptr, "OUTLINER_OT_parent_drop");
+				WM_operator_properties_create_ptr(&ptr, ot);
 				RNA_string_set(&ptr, "parent", parname);
 				RNA_string_set(&ptr, "child", childname);
 				RNA_enum_set(&ptr, "type", PAR_LATTICE);
-				uiItemFullO(layout, "OUTLINER_OT_parent_drop", IFACE_("Lattice Deform"),
-				            0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
+				uiItemFullO_ptr(layout, ot, IFACE_("Lattice Deform"), 0, ptr.data, WM_OP_EXEC_DEFAULT, 0);
 			}
 			
-			uiPupMenuEnd(C, pup);
+			UI_popup_menu_end(C, pup);
 			
-			return OPERATOR_CANCELLED;
+			return OPERATOR_INTERFACE;
 		}
 	}
 	else {
@@ -1630,7 +1641,7 @@ void OUTLINER_OT_parent_drop(wmOperatorType *ot)
 	ot->poll = ED_operator_outliner_active;
 
 	/* flags */
-	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO | OPTYPE_INTERNAL;
 
 	/* properties */
 	RNA_def_string(ot->srna, "child", "Object", MAX_ID_NAME, "Child", "Child Object");
@@ -1638,69 +1649,33 @@ void OUTLINER_OT_parent_drop(wmOperatorType *ot)
 	RNA_def_enum(ot->srna, "type", prop_make_parent_types, 0, "Type", "");
 }
 
-int outliner_dropzone_parent_clear(bContext *C, wmEvent *event, TreeElement *te, float *fmval)
+static int outliner_parenting_poll(bContext *C)
 {
 	SpaceOops *soops = CTX_wm_space_outliner(C);
-	TreeStoreElem *tselem = TREESTORE(te);
 
-	/* Check for row */
-	if ((fmval[1] > te->ys) && (fmval[1] < (te->ys + UI_UNIT_Y))) {
-		/* Ignore drop on scene tree elements */
-		if ((fmval[0] > te->xs + UI_UNIT_X) && (fmval[0] < te->xend)) {
-			if ((te->idcode == ID_SCE) && 
-			    !ELEM3(tselem->type, TSE_R_LAYER_BASE, TSE_R_LAYER, TSE_R_PASS))
-			{
-				return 0;
-			}
-			// Other codes to ignore?
-		}
-		
-		/* Left or right of: (+), first icon, and name */
-		if ((fmval[0] < (te->xs + UI_UNIT_X)) || (fmval[0] > te->xend)) {
-			return 1;
-		}
-		else if (te->idcode != ID_OB || ELEM(tselem->type, TSE_MODIFIER_BASE, TSE_CONSTRAINT_BASE)) {
-			return 1;
-		}
-		
-		return 0;       // ID_OB, but mouse in undefined dropzone.
+	if (soops) {
+		return ELEM(soops->outlinevis, SO_ALL_SCENES, SO_CUR_SCENE, SO_VISIBLE, SO_GROUPS);
 	}
 
-	/* Not this row.  Let's look at its children. */
-	if ((tselem->flag & TSE_CLOSED) == 0 && (te->subtree.first)) {
-		for (te = te->subtree.first; te; te = te->next) {
-			if (outliner_dropzone_parent_clear(C, event, te, fmval)) 
-				return 1;
-		}
-	}
-	return 0;
+	return false;
 }
 
-static int parent_clear_invoke(bContext *C, wmOperator *op, wmEvent *UNUSED(event))
+static int parent_clear_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
 	Main *bmain = CTX_data_main(C);
-	Scene *scene = NULL;
 	Object *ob = NULL;
 	SpaceOops *soops = CTX_wm_space_outliner(C);
-	TreeElement *te;
 	char obname[MAX_ID_NAME];
 
 	RNA_string_get(op->ptr, "dragged_obj", obname);
 	ob = (Object *)BKE_libblock_find_name(ID_OB, obname);
 
 	/* search forwards to find the object */
-	te = outliner_find_id(soops, &soops->tree, (ID *)ob);
-	/* then search backwards to get the scene */
-	scene = (Scene *)outliner_search_back(soops, te, ID_SCE);
-
-	if (scene == NULL) {
-		return OPERATOR_CANCELLED;
-	}
+	outliner_find_id(soops, &soops->tree, (ID *)ob);
 
 	ED_object_parent_clear(ob, RNA_enum_get(op->ptr, "type"));
 
-	DAG_scene_sort(bmain, scene);
-	DAG_ids_flush_update(bmain, 0);
+	DAG_relations_tag_update(bmain);
 	WM_event_add_notifier(C, NC_OBJECT | ND_TRANSFORM, NULL);
 	WM_event_add_notifier(C, NC_OBJECT | ND_PARENT, NULL);
 	return OPERATOR_FINISHED;
@@ -1716,33 +1691,17 @@ void OUTLINER_OT_parent_clear(wmOperatorType *ot)
 	/* api callbacks */
 	ot->invoke = parent_clear_invoke;
 
-	ot->poll = ED_operator_outliner_active;
+	ot->poll = outliner_parenting_poll;
 
 	/* flags */
-	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO | OPTYPE_INTERNAL;
 
 	/* properties */
 	RNA_def_string(ot->srna, "dragged_obj", "Object", MAX_ID_NAME, "Child", "Child Object");
 	RNA_def_enum(ot->srna, "type", prop_clear_parent_types, 0, "Type", "");
 }
 
-TreeElement *outliner_dropzone_scene(bContext *C, wmEvent *UNUSED(event), TreeElement *te, float *fmval)
-{
-	SpaceOops *soops = CTX_wm_space_outliner(C);
-	TreeStoreElem *tselem = TREESTORE(te);
-
-	if ((fmval[1] > te->ys) && (fmval[1] < (te->ys + UI_UNIT_Y))) {
-		/* name and first icon */
-		if ((fmval[0] > te->xs + UI_UNIT_X) && (fmval[0] < te->xend)) {
-			if (te->idcode == ID_SCE && tselem->type == 0) {
-				return te;
-			}
-		}
-	}
-	return NULL;
-}
-
-static int scene_drop_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int scene_drop_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	Scene *scene = NULL;
 	Object *ob = NULL;
@@ -1750,24 +1709,19 @@ static int scene_drop_invoke(bContext *C, wmOperator *op, wmEvent *event)
 	ARegion *ar = CTX_wm_region(C);
 	Main *bmain = CTX_data_main(C);
 	TreeElement *te = NULL;
-	TreeElement *te_found = NULL;
 	char obname[MAX_ID_NAME];
 	float fmval[2];
 
 	UI_view2d_region_to_view(&ar->v2d, event->mval[0], event->mval[1], &fmval[0], &fmval[1]);
 
 	/* Find object hovered over */
-	for (te = soops->tree.first; te; te = te->next) {
-		te_found = outliner_dropzone_scene(C, event, te, fmval);
-		if (te_found)
-			break;
-	}
+	te = outliner_dropzone_find(soops, fmval, 0);
 
-	if (te_found) {
+	if (te) {
 		Base *base;
 
-		RNA_string_set(op->ptr, "scene", te_found->name);
-		scene = (Scene *)BKE_libblock_find_name(ID_SCE, te_found->name);
+		RNA_string_set(op->ptr, "scene", te->name);
+		scene = (Scene *)BKE_libblock_find_name(ID_SCE, te->name);
 
 		RNA_string_get(op->ptr, "object", obname);
 		ob = (Object *)BKE_libblock_find_name(ID_OB, obname);
@@ -1788,8 +1742,7 @@ static int scene_drop_invoke(bContext *C, wmOperator *op, wmEvent *event)
 			ED_base_object_select(base, BA_SELECT);
 		}
 
-		DAG_scene_sort(bmain, scene);
-		DAG_ids_flush_update(bmain, 0);
+		DAG_relations_tag_update(bmain);
 
 		WM_main_add_notifier(NC_SCENE | ND_OB_SELECT, scene);
 
@@ -1812,37 +1765,31 @@ void OUTLINER_OT_scene_drop(wmOperatorType *ot)
 	ot->poll = ED_operator_outliner_active;
 
 	/* flags */
-	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO | OPTYPE_INTERNAL;
 
 	/* properties */
 	RNA_def_string(ot->srna, "object", "Object", MAX_ID_NAME, "Object", "Target Object");
 	RNA_def_string(ot->srna, "scene", "Scene", MAX_ID_NAME, "Scene", "Target Scene");
 }
 
-static int material_drop_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int material_drop_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
-	Main *bmain = CTX_data_main(C);
 	Material *ma = NULL;
 	Object *ob = NULL;
 	SpaceOops *soops = CTX_wm_space_outliner(C);
 	ARegion *ar = CTX_wm_region(C);
 	TreeElement *te = NULL;
-	TreeElement *te_found = NULL;
 	char mat_name[MAX_ID_NAME - 2];
 	float fmval[2];
 
 	UI_view2d_region_to_view(&ar->v2d, event->mval[0], event->mval[1], &fmval[0], &fmval[1]);
 
 	/* Find object hovered over */
-	for (te = soops->tree.first; te; te = te->next) {
-		te_found = outliner_dropzone_parent(C, event, te, fmval);
-		if (te_found)
-			break;
-	}
+	te = outliner_dropzone_find(soops, fmval, 1);
 
-	if (te_found) {
-		RNA_string_set(op->ptr, "object", te_found->name);
-		ob = (Object *)BKE_libblock_find_name(ID_OB, te_found->name);
+	if (te) {
+		RNA_string_set(op->ptr, "object", te->name);
+		ob = (Object *)BKE_libblock_find_name(ID_OB, te->name);
 
 		RNA_string_get(op->ptr, "material", mat_name);
 		ma = (Material *)BKE_libblock_find_name(ID_MA, mat_name);
@@ -1853,9 +1800,8 @@ static int material_drop_invoke(bContext *C, wmOperator *op, wmEvent *event)
 
 		assign_material(ob, ma, ob->totcol + 1, BKE_MAT_ASSIGN_USERPREF);
 
-		DAG_ids_flush_update(bmain, 0);
-		WM_event_add_notifier(C, NC_SPACE | ND_SPACE_VIEW3D, CTX_wm_view3d(C));		
-		WM_event_add_notifier(C, NC_MATERIAL | ND_SHADING, ma);
+		WM_event_add_notifier(C, NC_SPACE | ND_SPACE_VIEW3D, CTX_wm_view3d(C));
+		WM_event_add_notifier(C, NC_MATERIAL | ND_SHADING_LINKS, ma);
 
 		return OPERATOR_FINISHED;
 	}
@@ -1876,7 +1822,7 @@ void OUTLINER_OT_material_drop(wmOperatorType *ot)
 	ot->poll = ED_operator_outliner_active;
 
 	/* flags */
-	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
+	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO | OPTYPE_INTERNAL;
 
 	/* properties */
 	RNA_def_string(ot->srna, "object", "Object", MAX_ID_NAME, "Object", "Target Object");
