@@ -31,8 +31,6 @@
 
 #include <stdlib.h>
 
-#include "MEM_guardedalloc.h"
-
 #include "DNA_armature_types.h"
 #include "DNA_group_types.h"
 #include "DNA_lamp_types.h"
@@ -47,8 +45,10 @@
 
 #include "BKE_context.h"
 #include "BKE_depsgraph.h"
+#include "BKE_object.h"
 #include "BKE_scene.h"
 #include "BKE_sequencer.h"
+#include "BKE_armature.h"
 
 #include "ED_armature.h"
 #include "ED_object.h"
@@ -75,7 +75,7 @@ static int outliner_select(SpaceOops *soops, ListBase *lb, int *index, short *se
 {
 	TreeElement *te;
 	TreeStoreElem *tselem;
-	int change = 0;
+	bool changed = false;
 	
 	for (te = lb->first; te && *index >= 0; te = te->next, (*index)--) {
 		tselem = TREESTORE(te);
@@ -98,7 +98,7 @@ static int outliner_select(SpaceOops *soops, ListBase *lb, int *index, short *se
 				else 
 					tselem->flag &= ~TSE_SELECTED;
 
-				change |= 1;
+				changed |= true;
 			}
 		}
 		else if (TSELEM_OPEN(tselem, soops)) {
@@ -110,37 +110,82 @@ static int outliner_select(SpaceOops *soops, ListBase *lb, int *index, short *se
 			 *  function correctly
 			 */
 			(*index)--;
-			change |= outliner_select(soops, &te->subtree, index, selecting);
+			changed |= outliner_select(soops, &te->subtree, index, selecting);
 			(*index)++;
 		}
 	}
 
-	return change;
+	return changed;
 }
 
 /* ****************************************************** */
 /* Outliner Element Selection/Activation on Click */
 
-static int tree_element_active_renderlayer(bContext *C, TreeElement *te, TreeStoreElem *tselem, int set)
+static eOLDrawState tree_element_active_renderlayer(
+        bContext *C, TreeElement *te, TreeStoreElem *tselem, const eOLSetState set)
 {
 	Scene *sce;
 	
 	/* paranoia check */
 	if (te->idcode != ID_SCE)
-		return 0;
+		return OL_DRAWSEL_NONE;
 	sce = (Scene *)tselem->id;
 	
-	if (set) {
+	if (set != OL_SETSEL_NONE) {
 		sce->r.actlay = tselem->nr;
 		WM_event_add_notifier(C, NC_SCENE | ND_RENDER_OPTIONS, sce);
 	}
 	else {
 		return sce->r.actlay == tselem->nr;
 	}
-	return 0;
+	return OL_DRAWSEL_NONE;
 }
 
-static int  tree_element_set_active_object(bContext *C, Scene *scene, SpaceOops *soops, TreeElement *te, int set)
+/**
+ * Select object tree:
+ * CTRL+LMB: Select/Deselect object and all cildren
+ * CTRL+SHIFT+LMB: Add/Remove object and all children
+ */
+static void do_outliner_object_select_recursive(Scene *scene, Object *ob_parent, bool select)
+{
+	Base *base;
+
+	for (base = FIRSTBASE; base; base = base->next) {
+		Object *ob = base->object;
+		if ((((ob->restrictflag & OB_RESTRICT_VIEW) == 0) && BKE_object_is_child_recursive(ob_parent, ob))) {
+			ED_base_object_select(base, select ? BA_SELECT : BA_DESELECT);
+		}
+	}
+}
+
+static void do_outliner_bone_select_recursive(bArmature *arm, Bone *bone_parent, bool select)
+{
+	Bone *bone;
+	for (bone = bone_parent->childbase.first; bone; bone = bone->next) {
+		if (select && PBONE_SELECTABLE(arm, bone))
+			bone->flag |= BONE_SELECTED;
+		else
+			bone->flag &= ~(BONE_TIPSEL | BONE_SELECTED | BONE_ROOTSEL);
+		do_outliner_bone_select_recursive(arm, bone, select);
+	}
+}
+
+static void do_outliner_ebone_select_recursive(bArmature *arm, EditBone *ebone_parent, bool select)
+{
+	EditBone *ebone;
+	for (ebone = ebone_parent->next; ebone; ebone = ebone->next) {
+		if (ED_armature_ebone_is_child_recursive(ebone_parent, ebone)) {
+			if (select && EBONE_SELECTABLE(arm, ebone))
+				ebone->flag |= BONE_TIPSEL | BONE_SELECTED | BONE_ROOTSEL;
+			else
+				ebone->flag &= ~(BONE_TIPSEL | BONE_SELECTED | BONE_ROOTSEL);
+		}
+	}
+}
+
+static eOLDrawState tree_element_set_active_object(
+        bContext *C, Scene *scene, SpaceOops *soops,
+        TreeElement *te, const eOLSetState set, bool recursive)
 {
 	TreeStoreElem *tselem = TREESTORE(te);
 	Scene *sce;
@@ -148,23 +193,30 @@ static int  tree_element_set_active_object(bContext *C, Scene *scene, SpaceOops
 	Object *ob = NULL;
 	
 	/* if id is not object, we search back */
-	if (te->idcode == ID_OB) ob = (Object *)tselem->id;
+	if (te->idcode == ID_OB) {
+		ob = (Object *)tselem->id;
+	}
 	else {
 		ob = (Object *)outliner_search_back(soops, te, ID_OB);
-		if (ob == OBACT) return 0;
+		if (ob == OBACT) {
+			return OL_DRAWSEL_NONE;
+		}
+	}
+	if (ob == NULL) {
+		return OL_DRAWSEL_NONE;
 	}
-	if (ob == NULL) return 0;
 	
 	sce = (Scene *)outliner_search_back(soops, te, ID_SCE);
 	if (sce && scene != sce) {
 		ED_screen_set_scene(C, CTX_wm_screen(C), sce);
+		scene = sce;
 	}
 	
 	/* find associated base in current scene */
 	base = BKE_scene_base_find(scene, ob);
 
 	if (base) {
-		if (set == 2) {
+		if (set == OL_SETSEL_EXTEND) {
 			/* swap select */
 			if (base->flag & SELECT)
 				ED_base_object_select(base, BA_DESELECT);
@@ -176,6 +228,12 @@ static int  tree_element_set_active_object(bContext *C, Scene *scene, SpaceOops
 			BKE_scene_base_deselect_all(scene);
 			ED_base_object_select(base, BA_SELECT);
 		}
+
+		if (recursive) {
+			/* Recursive select/deselect for Object hierarchies */
+			do_outliner_object_select_recursive(scene, ob, (ob->flag & SELECT) != 0);
+		}
+
 		if (C) {
 			ED_base_object_activate(C, base); /* adds notifier */
 			WM_event_add_notifier(C, NC_SCENE | ND_OB_SELECT, scene);
@@ -183,12 +241,14 @@ static int  tree_element_set_active_object(bContext *C, Scene *scene, SpaceOops
 	}
 	
 	if (ob != scene->obedit)
-		ED_object_exit_editmode(C, EM_FREEDATA | EM_FREEUNDO | EM_WAITCURSOR | EM_DO_UNDO);
+		ED_object_editmode_exit(C, EM_FREEDATA | EM_FREEUNDO | EM_WAITCURSOR | EM_DO_UNDO);
 		
-	return 1;
+	return OL_DRAWSEL_NORMAL;
 }
 
-static int tree_element_active_material(bContext *C, Scene *scene, SpaceOops *soops, TreeElement *te, int set)
+static eOLDrawState tree_element_active_material(
+        bContext *C, Scene *scene, SpaceOops *soops,
+        TreeElement *te, const eOLSetState set)
 {
 	TreeElement *tes;
 	Object *ob;
@@ -196,47 +256,64 @@ static int tree_element_active_material(bContext *C, Scene *scene, SpaceOops *so
 	/* we search for the object parent */
 	ob = (Object *)outliner_search_back(soops, te, ID_OB);
 	// note: ob->matbits can be NULL when a local object points to a library mesh.
-	if (ob == NULL || ob != OBACT || ob->matbits == NULL) return 0;  // just paranoia
+	if (ob == NULL || ob != OBACT || ob->matbits == NULL) {
+		return OL_DRAWSEL_NONE;  /* just paranoia */
+	}
 	
 	/* searching in ob mat array? */
 	tes = te->parent;
 	if (tes->idcode == ID_OB) {
-		if (set) {
+		if (set != OL_SETSEL_NONE) {
 			ob->actcol = te->index + 1;
 			ob->matbits[te->index] = 1;  // make ob material active too
 		}
 		else {
-			if (ob->actcol == te->index + 1)
-				if (ob->matbits[te->index]) return 1;
+			if (ob->actcol == te->index + 1) {
+				if (ob->matbits[te->index]) {
+					return OL_DRAWSEL_NORMAL;
+				}
+			}
 		}
 	}
 	/* or we search for obdata material */
 	else {
-		if (set) {
+		if (set != OL_SETSEL_NONE) {
 			ob->actcol = te->index + 1;
 			ob->matbits[te->index] = 0;  // make obdata material active too
 		}
 		else {
-			if (ob->actcol == te->index + 1)
-				if (ob->matbits[te->index] == 0) return 1;
+			if (ob->actcol == te->index + 1) {
+				if (ob->matbits[te->index] == 0) {
+					return OL_DRAWSEL_NORMAL;
+				}
+			}
 		}
 	}
-	if (set) {
-		WM_event_add_notifier(C, NC_MATERIAL | ND_SHADING, NULL);
+	if (set != OL_SETSEL_NONE) {
+		/* Tagging object for update seems a bit stupid here, but looks like we have to do it
+		 * for render views to update. See T42973.
+		 * Note that RNA material update does it too, see e.g. rna_MaterialSlot_update(). */
+		DAG_id_tag_update((ID *)ob, OB_RECALC_OB);
+		WM_event_add_notifier(C, NC_MATERIAL | ND_SHADING_LINKS, NULL);
 	}
-	return 0;
+	return OL_DRAWSEL_NONE;
 }
 
-static int tree_element_active_texture(bContext *C, Scene *scene, SpaceOops *soops, TreeElement *te, int set)
+static eOLDrawState tree_element_active_texture(
+        bContext *C, Scene *scene, SpaceOops *UNUSED(soops),
+        TreeElement *te, const eOLSetState set)
 {
 	TreeElement *tep;
 	TreeStoreElem /* *tselem,*/ *tselemp;
 	Object *ob = OBACT;
 	SpaceButs *sbuts = NULL;
 	
-	if (ob == NULL) return 0;  // no active object
+	if (ob == NULL) {
+		/* no active object */
+		return OL_DRAWSEL_NONE;
+	}
 	
-	/*tselem= TREESTORE(te);*/ /*UNUSED*/
+	/*tselem = TREESTORE(te);*/ /*UNUSED*/
 	
 	/* find buttons area (note, this is undefined really still, needs recode in blender) */
 	/* XXX removed finding sbuts */
@@ -248,40 +325,44 @@ static int tree_element_active_texture(bContext *C, Scene *scene, SpaceOops *soo
 	if (tep->idcode == ID_WO) {
 		World *wrld = (World *)tselemp->id;
 
-		if (set) {
+		if (set != OL_SETSEL_NONE) {
 			if (sbuts) {
-				// XXX sbuts->tabo= TAB_SHADING_TEX;	// hack from header_buttonswin.c
-				// XXX sbuts->texfrom= 1;
+				// XXX sbuts->tabo = TAB_SHADING_TEX;	// hack from header_buttonswin.c
+				// XXX sbuts->texfrom = 1;
 			}
 // XXX			extern_set_butspace(F6KEY, 0);	// force shading buttons texture
 			wrld->texact = te->index;
 		}
 		else if (tselemp->id == (ID *)(scene->world)) {
-			if (wrld->texact == te->index) return 1;
+			if (wrld->texact == te->index) {
+				return OL_DRAWSEL_NORMAL;
+			}
 		}
 	}
 	else if (tep->idcode == ID_LA) {
 		Lamp *la = (Lamp *)tselemp->id;
-		if (set) {
+		if (set != OL_SETSEL_NONE) {
 			if (sbuts) {
-				// XXX sbuts->tabo= TAB_SHADING_TEX;	// hack from header_buttonswin.c
-				// XXX sbuts->texfrom= 2;
+				// XXX sbuts->tabo = TAB_SHADING_TEX;	// hack from header_buttonswin.c
+				// XXX sbuts->texfrom = 2;
 			}
 // XXX			extern_set_butspace(F6KEY, 0);	// force shading buttons texture
 			la->texact = te->index;
 		}
 		else {
 			if (tselemp->id == ob->data) {
-				if (la->texact == te->index) return 1;
+				if (la->texact == te->index) {
+					return OL_DRAWSEL_NORMAL;
+				}
 			}
 		}
 	}
 	else if (tep->idcode == ID_MA) {
 		Material *ma = (Material *)tselemp->id;
-		if (set) {
+		if (set != OL_SETSEL_NONE) {
 			if (sbuts) {
-				//sbuts->tabo= TAB_SHADING_TEX;	// hack from header_buttonswin.c
-				// XXX sbuts->texfrom= 0;
+				//sbuts->tabo = TAB_SHADING_TEX;	// hack from header_buttonswin.c
+				// XXX sbuts->texfrom = 0;
 			}
 // XXX			extern_set_butspace(F6KEY, 0);	// force shading buttons texture
 			ma->texact = (char)te->index;
@@ -290,44 +371,60 @@ static int tree_element_active_texture(bContext *C, Scene *scene, SpaceOops *soo
 			ob->actcol = tep->index + 1;
 		}
 		else if (tep->flag & TE_ACTIVE) {   // this is active material
-			if (ma->texact == te->index) return 1;
+			if (ma->texact == te->index) {
+				return OL_DRAWSEL_NORMAL;
+			}
 		}
 	}
 	
-	if (set)
+	if (set != OL_SETSEL_NONE) {
 		WM_event_add_notifier(C, NC_TEXTURE, NULL);
+	}
 
-	return 0;
+	/* no active object */
+	return OL_DRAWSEL_NONE;
 }
 
 
-static int tree_element_active_lamp(bContext *UNUSED(C), Scene *scene, SpaceOops *soops, TreeElement *te, int set)
+static eOLDrawState tree_element_active_lamp(
+        bContext *UNUSED(C), Scene *scene, SpaceOops *soops,
+        TreeElement *te, const eOLSetState set)
 {
 	Object *ob;
 	
 	/* we search for the object parent */
 	ob = (Object *)outliner_search_back(soops, te, ID_OB);
-	if (ob == NULL || ob != OBACT) return 0;  // just paranoia
+	if (ob == NULL || ob != OBACT) {
+		/* just paranoia */
+		return OL_DRAWSEL_NONE;
+	}
 	
-	if (set) {
+	if (set != OL_SETSEL_NONE) {
 // XXX		extern_set_butspace(F5KEY, 0);
 	}
-	else return 1;
+	else {
+		return OL_DRAWSEL_NORMAL;
+	}
 	
-	return 0;
+	return OL_DRAWSEL_NONE;
 }
 
-static int tree_element_active_camera(bContext *UNUSED(C), Scene *scene, SpaceOops *soops, TreeElement *te, int set)
+static eOLDrawState tree_element_active_camera(
+        bContext *UNUSED(C), Scene *scene, SpaceOops *soops,
+        TreeElement *te, const eOLSetState set)
 {
 	Object *ob = (Object *)outliner_search_back(soops, te, ID_OB);
 
-	if (set)
-		return 0;
+	if (set != OL_SETSEL_NONE) {
+		return OL_DRAWSEL_NONE;
+	}
 
 	return scene->camera == ob;
 }
 
-static int tree_element_active_world(bContext *C, Scene *scene, SpaceOops *soops, TreeElement *te, int set)
+static eOLDrawState tree_element_active_world(
+        bContext *C, Scene *scene, SpaceOops *UNUSED(soops),
+        TreeElement *te, const eOLSetState set)
 {
 	TreeElement *tep;
 	TreeStoreElem *tselem = NULL;
@@ -336,33 +433,36 @@ static int tree_element_active_world(bContext *C, Scene *scene, SpaceOops *soops
 	tep = te->parent;
 	if (tep) {
 		tselem = TREESTORE(tep);
-		sce = (Scene *)tselem->id;
+		if (tselem->type == 0)
+			sce = (Scene *)tselem->id;
 	}
 	
-	if (set) {  // make new scene active
+	if (set != OL_SETSEL_NONE) {
+		/* make new scene active */
 		if (sce && scene != sce) {
 			ED_screen_set_scene(C, CTX_wm_screen(C), sce);
 		}
 	}
 	
 	if (tep == NULL || tselem->id == (ID *)scene) {
-		if (set) {
+		if (set != OL_SETSEL_NONE) {
 // XXX			extern_set_butspace(F8KEY, 0);
 		}
 		else {
-			return 1;
+			return OL_DRAWSEL_NORMAL;
 		}
 	}
-	return 0;
+	return OL_DRAWSEL_NONE;
 }
 
-static int tree_element_active_defgroup(bContext *C, Scene *scene, TreeElement *te, TreeStoreElem *tselem, int set)
+static eOLDrawState tree_element_active_defgroup(
+        bContext *C, Scene *scene, TreeElement *te, TreeStoreElem *tselem, const eOLSetState set)
 {
 	Object *ob;
 	
 	/* id in tselem is object */
 	ob = (Object *)tselem->id;
-	if (set) {
+	if (set != OL_SETSEL_NONE) {
 		BLI_assert(te->index + 1 >= 0);
 		ob->actdef = te->index + 1;
 
@@ -371,16 +471,19 @@ static int tree_element_active_defgroup(bContext *C, Scene *scene, TreeElement *
 	}
 	else {
 		if (ob == OBACT)
-			if (ob->actdef == te->index + 1) return 1;
+			if (ob->actdef == te->index + 1) {
+				return OL_DRAWSEL_NORMAL;
+			}
 	}
-	return 0;
+	return OL_DRAWSEL_NONE;
 }
 
-static int tree_element_active_posegroup(bContext *C, Scene *scene, TreeElement *te, TreeStoreElem *tselem, int set)
+static eOLDrawState tree_element_active_posegroup(
+        bContext *C, Scene *scene, TreeElement *te, TreeStoreElem *tselem, const eOLSetState set)
 {
 	Object *ob = (Object *)tselem->id;
 	
-	if (set) {
+	if (set != OL_SETSEL_NONE) {
 		if (ob->pose) {
 			ob->pose->active_group = te->index + 1;
 			WM_event_add_notifier(C, NC_OBJECT | ND_DRAW, ob);
@@ -388,64 +491,89 @@ static int tree_element_active_posegroup(bContext *C, Scene *scene, TreeElement
 	}
 	else {
 		if (ob == OBACT && ob->pose) {
-			if (ob->pose->active_group == te->index + 1) return 1;
+			if (ob->pose->active_group == te->index + 1) {
+				return OL_DRAWSEL_NORMAL;
+			}
 		}
 	}
-	return 0;
+	return OL_DRAWSEL_NONE;
 }
 
-static int tree_element_active_posechannel(bContext *C, Scene *scene, TreeElement *te, TreeStoreElem *tselem, int set)
+static eOLDrawState tree_element_active_posechannel(
+        bContext *C, Scene *scene, TreeElement *te, TreeStoreElem *tselem, const eOLSetState set, bool recursive)
 {
 	Object *ob = (Object *)tselem->id;
 	bArmature *arm = ob->data;
 	bPoseChannel *pchan = te->directdata;
 	
-	if (set) {
+	if (set != OL_SETSEL_NONE) {
 		if (!(pchan->bone->flag & BONE_HIDDEN_P)) {
 			
-			if (set == 2) ED_pose_deselectall(ob, 2);  // 2 = clear active tag
-			else ED_pose_deselectall(ob, 0);    // 0 = deselect
-			
-			if (set == 2 && (pchan->bone->flag & BONE_SELECTED)) {
+			if (set != OL_SETSEL_EXTEND) {
+				bPoseChannel *pchannel;
+				/* single select forces all other bones to get unselected */
+				for (pchannel = ob->pose->chanbase.first; pchannel; pchannel = pchannel->next)
+					pchannel->bone->flag &= ~(BONE_TIPSEL | BONE_SELECTED | BONE_ROOTSEL);
+			}
+
+			if ((set == OL_SETSEL_EXTEND) && (pchan->bone->flag & BONE_SELECTED)) {
 				pchan->bone->flag &= ~BONE_SELECTED;
 			}
 			else {
 				pchan->bone->flag |= BONE_SELECTED;
 				arm->act_bone = pchan->bone;
 			}
-			
+
+			if (recursive) {
+				/* Recursive select/deselect */
+				do_outliner_bone_select_recursive(arm, pchan->bone, (pchan->bone->flag & BONE_SELECTED) != 0);
+			}
+
 			WM_event_add_notifier(C, NC_OBJECT | ND_BONE_ACTIVE, ob);
 
 		}
 	}
 	else {
 		if (ob == OBACT && ob->pose) {
-			if (pchan->bone->flag & BONE_SELECTED) return 1;
+			if (pchan->bone->flag & BONE_SELECTED) {
+				return OL_DRAWSEL_NORMAL;
+			}
 		}
 	}
-	return 0;
+	return OL_DRAWSEL_NONE;
 }
 
-static int tree_element_active_bone(bContext *C, Scene *scene, TreeElement *te, TreeStoreElem *tselem, int set)
+static eOLDrawState tree_element_active_bone(
+        bContext *C, Scene *scene, TreeElement *te, TreeStoreElem *tselem, const eOLSetState set, bool recursive)
 {
 	bArmature *arm = (bArmature *)tselem->id;
 	Bone *bone = te->directdata;
 	
-	if (set) {
+	if (set != OL_SETSEL_NONE) {
 		if (!(bone->flag & BONE_HIDDEN_P)) {
 			Object *ob = OBACT;
 			if (ob) {
-				if (set == 2) ED_pose_deselectall(ob, 2);  // 2 is clear active tag
-				else ED_pose_deselectall(ob, 0);
+				if (set != OL_SETSEL_EXTEND) {
+					bPoseChannel *pchannel;
+					/* single select forces all other bones to get unselected */
+					for (pchannel = ob->pose->chanbase.first; pchannel; pchannel = pchannel->next)
+						pchannel->bone->flag &= ~(BONE_TIPSEL | BONE_SELECTED | BONE_ROOTSEL);
+				}
 			}
 			
-			if (set == 2 && (bone->flag & BONE_SELECTED)) {
+			if (set == OL_SETSEL_EXTEND && (bone->flag & BONE_SELECTED)) {
 				bone->flag &= ~BONE_SELECTED;
 			}
 			else {
 				bone->flag |= BONE_SELECTED;
 				arm->act_bone = bone;
 			}
+
+			if (recursive) {
+				/* Recursive select/deselect */
+				do_outliner_bone_select_recursive(arm, bone, (bone->flag & BONE_SELECTED) != 0);
+			}
+
 			
 			WM_event_add_notifier(C, NC_OBJECT | ND_BONE_ACTIVE, ob);
 		}
@@ -454,10 +582,12 @@ static int tree_element_active_bone(bContext *C, Scene *scene, TreeElement *te,
 		Object *ob = OBACT;
 		
 		if (ob && ob->data == arm) {
-			if (bone->flag & BONE_SELECTED) return 1;
+			if (bone->flag & BONE_SELECTED) {
+				return OL_DRAWSEL_NORMAL;
+			}
 		}
 	}
-	return 0;
+	return OL_DRAWSEL_NONE;
 }
 
 
@@ -478,40 +608,51 @@ static void tree_element_active_ebone__sel(bContext *C, Scene *scene, bArmature
 
 	WM_event_add_notifier(C, NC_OBJECT | ND_BONE_ACTIVE, scene->obedit);
 }
-static int tree_element_active_ebone(bContext *C, Scene *scene, TreeElement *te, TreeStoreElem *UNUSED(tselem), int set)
+static eOLDrawState tree_element_active_ebone(
+        bContext *C, Scene *scene, TreeElement *te, TreeStoreElem *UNUSED(tselem), const eOLSetState set, bool recursive)
 {
 	bArmature *arm = scene->obedit->data;
 	EditBone *ebone = te->directdata;
+	eOLDrawState status = OL_DRAWSEL_NONE;
 
-	if (set == 1) {
-		if (!(ebone->flag & BONE_HIDDEN_A)) {
-			ED_armature_deselect_all(scene->obedit, 0); // deselect
-			tree_element_active_ebone__sel(C, scene, arm, ebone, TRUE);
-			return 1;
+	if (set != OL_SETSEL_NONE) {
+		if (set == OL_SETSEL_NORMAL) {
+			if (!(ebone->flag & BONE_HIDDEN_A)) {
+				ED_armature_deselect_all(scene->obedit, 0); // deselect
+				tree_element_active_ebone__sel(C, scene, arm, ebone, true);
+				status = OL_DRAWSEL_NORMAL;
+			}
 		}
-	}
-	else if (set == 2) {
-		if (!(ebone->flag & BONE_HIDDEN_A)) {
-			if (!(ebone->flag & BONE_SELECTED)) {
-				tree_element_active_ebone__sel(C, scene, arm, ebone, TRUE);
-				return 1;
-			}
-			else {
-				/* entirely selected, so de-select */
-				tree_element_active_ebone__sel(C, scene, arm, ebone, FALSE);
-				return 0;
+		else if (set == OL_SETSEL_EXTEND) {
+			if (!(ebone->flag & BONE_HIDDEN_A)) {
+				if (!(ebone->flag & BONE_SELECTED)) {
+					tree_element_active_ebone__sel(C, scene, arm, ebone, true);
+					status = OL_DRAWSEL_NORMAL;
+				}
+				else {
+					/* entirely selected, so de-select */
+					tree_element_active_ebone__sel(C, scene, arm, ebone, false);
+					status = OL_DRAWSEL_NONE;
+				}
 			}
 		}
+
+		if (recursive) {
+			/* Recursive select/deselect */
+			do_outliner_ebone_select_recursive(arm, ebone, (ebone->flag & BONE_SELECTED) != 0);
+		}
 	}
 	else if (ebone->flag & BONE_SELECTED) {
-		return 1;
+		status = OL_DRAWSEL_NORMAL;
 	}
-	return 0;
+
+	return status;
 }
 
-static int tree_element_active_modifier(bContext *C, TreeElement *UNUSED(te), TreeStoreElem *tselem, int set)
+static eOLDrawState tree_element_active_modifier(
+        bContext *C, TreeElement *UNUSED(te), TreeStoreElem *tselem, const eOLSetState set)
 {
-	if (set) {
+	if (set != OL_SETSEL_NONE) {
 		Object *ob = (Object *)tselem->id;
 		
 		WM_event_add_notifier(C, NC_OBJECT | ND_MODIFIER, ob);
@@ -519,12 +660,13 @@ static int tree_element_active_modifier(bContext *C, TreeElement *UNUSED(te), Tr
 // XXX		extern_set_butspace(F9KEY, 0);
 	}
 	
-	return 0;
+	return OL_DRAWSEL_NONE;
 }
 
-static int tree_element_active_psys(bContext *C, Scene *UNUSED(scene), TreeElement *UNUSED(te), TreeStoreElem *tselem, int set)
+static eOLDrawState tree_element_active_psys(
+        bContext *C, Scene *UNUSED(scene), TreeElement *UNUSED(te), TreeStoreElem *tselem, const eOLSetState set)
 {
-	if (set) {
+	if (set != OL_SETSEL_NONE) {
 		Object *ob = (Object *)tselem->id;
 		
 		WM_event_add_notifier(C, NC_OBJECT | ND_PARTICLE | NA_EDITED, ob);
@@ -532,35 +674,39 @@ static int tree_element_active_psys(bContext *C, Scene *UNUSED(scene), TreeEleme
 // XXX		extern_set_butspace(F7KEY, 0);
 	}
 	
-	return 0;
+	return OL_DRAWSEL_NONE;
 }
 
-static int tree_element_active_constraint(bContext *C, TreeElement *UNUSED(te), TreeStoreElem *tselem, int set)
+static int tree_element_active_constraint(
+        bContext *C, TreeElement *UNUSED(te), TreeStoreElem *tselem, const eOLSetState set)
 {
-	if (set) {
+	if (set != OL_SETSEL_NONE) {
 		Object *ob = (Object *)tselem->id;
 		
 		WM_event_add_notifier(C, NC_OBJECT | ND_CONSTRAINT, ob);
 // XXX		extern_set_butspace(F7KEY, 0);
 	}
 	
-	return 0;
+	return OL_DRAWSEL_NONE;
 }
 
-static int tree_element_active_text(bContext *UNUSED(C), Scene *UNUSED(scene), SpaceOops *UNUSED(soops), TreeElement *UNUSED(te), int UNUSED(set))
+static eOLDrawState tree_element_active_text(
+        bContext *UNUSED(C), Scene *UNUSED(scene), SpaceOops *UNUSED(soops),
+        TreeElement *UNUSED(te), int UNUSED(set))
 {
 	// XXX removed
-	return 0;
+	return OL_DRAWSEL_NONE;
 }
 
-static int tree_element_active_pose(bContext *C, Scene *scene, TreeElement *UNUSED(te), TreeStoreElem *tselem, int set)
+static eOLDrawState tree_element_active_pose(
+        bContext *C, Scene *scene, TreeElement *UNUSED(te), TreeStoreElem *tselem, const eOLSetState set)
 {
 	Object *ob = (Object *)tselem->id;
 	Base *base = BKE_scene_base_find(scene, ob);
 	
-	if (set) {
+	if (set != OL_SETSEL_NONE) {
 		if (scene->obedit)
-			ED_object_exit_editmode(C, EM_FREEDATA | EM_FREEUNDO | EM_WAITCURSOR | EM_DO_UNDO);
+			ED_object_editmode_exit(C, EM_FREEDATA | EM_FREEUNDO | EM_WAITCURSOR | EM_DO_UNDO);
 		
 		if (ob->mode & OB_MODE_POSE)
 			ED_armature_exit_posemode(C, base);
@@ -568,25 +714,28 @@ static int tree_element_active_pose(bContext *C, Scene *scene, TreeElement *UNUS
 			ED_armature_enter_posemode(C, base);
 	}
 	else {
-		if (ob->mode & OB_MODE_POSE) return 1;
+		if (ob->mode & OB_MODE_POSE) {
+			return OL_DRAWSEL_NORMAL;
+		}
 	}
-	return 0;
+	return OL_DRAWSEL_NONE;
 }
 
-static int tree_element_active_sequence(bContext *C, Scene *scene, TreeElement *te, TreeStoreElem *UNUSED(tselem), int set)
+static eOLDrawState tree_element_active_sequence(
+        bContext *C, Scene *scene, TreeElement *te, TreeStoreElem *UNUSED(tselem), const eOLSetState set)
 {
 	Sequence *seq = (Sequence *) te->directdata;
-	Editing *ed = BKE_sequencer_editing_get(scene, FALSE);
+	Editing *ed = BKE_sequencer_editing_get(scene, false);
 
-	if (set) {
+	if (set != OL_SETSEL_NONE) {
 		/* only check on setting */
 		if (BLI_findindex(ed->seqbasep, seq) != -1) {
-			if (set == 2) {
+			if (set == OL_SETSEL_EXTEND) {
 				BKE_sequencer_active_set(scene, NULL);
 			}
 			ED_sequencer_deselect_all(scene);
 
-			if (set == 2 && seq->flag & SELECT) {
+			if ((set == OL_SETSEL_EXTEND) && seq->flag & SELECT) {
 				seq->flag &= ~SELECT;
 			}
 			else {
@@ -599,28 +748,29 @@ static int tree_element_active_sequence(bContext *C, Scene *scene, TreeElement *
 	}
 	else {
 		if (ed->act_seq == seq && seq->flag & SELECT) {
-			return 1;
+			return OL_DRAWSEL_NORMAL;
 		}
 	}
-	return(0);
+	return OL_DRAWSEL_NONE;
 }
 
-static int tree_element_active_sequence_dup(Scene *scene, TreeElement *te, TreeStoreElem *UNUSED(tselem), int set)
+static eOLDrawState tree_element_active_sequence_dup(
+        Scene *scene, TreeElement *te, TreeStoreElem *UNUSED(tselem), const eOLSetState set)
 {
 	Sequence *seq, *p;
-	Editing *ed = BKE_sequencer_editing_get(scene, FALSE);
+	Editing *ed = BKE_sequencer_editing_get(scene, false);
 
 	seq = (Sequence *)te->directdata;
-	if (set == 0) {
+	if (set == OL_SETSEL_NONE) {
 		if (seq->flag & SELECT)
-			return(1);
-		return(0);
+			return OL_DRAWSEL_NORMAL;
+		return OL_DRAWSEL_NONE;
 	}
 
 // XXX	select_single_seq(seq, 1);
 	p = ed->seqbasep->first;
 	while (p) {
-		if ((!p->strip) || (!p->strip->stripdata) || (!p->strip->stripdata->name)) {
+		if ((!p->strip) || (!p->strip->stripdata) || (p->strip->stripdata->name[0] == '\0')) {
 			p = p->next;
 			continue;
 		}
@@ -629,32 +779,40 @@ static int tree_element_active_sequence_dup(Scene *scene, TreeElement *te, TreeS
 // XXX			select_single_seq(p, 0);
 		p = p->next;
 	}
-	return(0);
+	return OL_DRAWSEL_NONE;
 }
 
-static int tree_element_active_keymap_item(bContext *UNUSED(C), TreeElement *te, TreeStoreElem *UNUSED(tselem), int set)
+static eOLDrawState tree_element_active_keymap_item(
+        bContext *UNUSED(C), TreeElement *te, TreeStoreElem *UNUSED(tselem), const eOLSetState set)
 {
 	wmKeyMapItem *kmi = te->directdata;
 	
-	if (set == 0) {
-		if (kmi->flag & KMI_INACTIVE) return 0;
-		return 1;
+	if (set == OL_SETSEL_NONE) {
+		if (kmi->flag & KMI_INACTIVE) {
+			return OL_DRAWSEL_NONE;
+		}
+		return OL_DRAWSEL_NORMAL;
 	}
 	else {
 		kmi->flag ^= KMI_INACTIVE;
 	}
-	return 0;
+	return OL_DRAWSEL_NONE;
 }
 
 /* ---------------------------------------------- */
 
 /* generic call for ID data check or make/check active in UI */
-int tree_element_active(bContext *C, Scene *scene, SpaceOops *soops, TreeElement *te, int set)
+eOLDrawState tree_element_active(bContext *C, Scene *scene, SpaceOops *soops, TreeElement *te,
+                                 const eOLSetState set, const bool handle_all_types)
 {
-
 	switch (te->idcode) {
-		/* Note: no ID_OB: objects are handled specially to allow multiple
+		/* Note: ID_OB only if handle_all_type is true, else objects are handled specially to allow multiple
 		 * selection. See do_outliner_item_activate. */
+		case ID_OB:
+			if (handle_all_types) {
+				return tree_element_set_active_object(C, scene, soops, te, set, false);
+			}
+			break;
 		case ID_MA:
 			return tree_element_active_material(C, scene, soops, te, set);
 		case ID_WO:
@@ -668,33 +826,41 @@ int tree_element_active(bContext *C, Scene *scene, SpaceOops *soops, TreeElement
 		case ID_CA:
 			return tree_element_active_camera(C, scene, soops, te, set);
 	}
-	return 0;
+	return OL_DRAWSEL_NONE;
 }
 
-/* generic call for non-id data to make/check active in UI */
-/* Context can be NULL when set==0 */
-int tree_element_type_active(bContext *C, Scene *scene, SpaceOops *soops,
-                             TreeElement *te, TreeStoreElem *tselem, int set)
+/**
+ * Generic call for non-id data to make/check active in UI
+ *
+ * \note Context can be NULL when ``(set == OL_SETSEL_NONE)``
+ */
+eOLDrawState tree_element_type_active(
+        bContext *C, Scene *scene, SpaceOops *soops,
+        TreeElement *te, TreeStoreElem *tselem, const eOLSetState set, bool recursive)
 {
 	switch (tselem->type) {
 		case TSE_DEFGROUP:
 			return tree_element_active_defgroup(C, scene, te, tselem, set);
 		case TSE_BONE:
-			return tree_element_active_bone(C, scene, te, tselem, set);
+			return tree_element_active_bone(C, scene, te, tselem, set, recursive);
 		case TSE_EBONE:
-			return tree_element_active_ebone(C, scene, te, tselem, set);
+			return tree_element_active_ebone(C, scene, te, tselem, set, recursive);
 		case TSE_MODIFIER:
 			return tree_element_active_modifier(C, te, tselem, set);
 		case TSE_LINKED_OB:
-			if (set) tree_element_set_active_object(C, scene, soops, te, set);
-			else if (tselem->id == (ID *)OBACT) return 1;
+			if (set != OL_SETSEL_NONE) {
+				tree_element_set_active_object(C, scene, soops, te, set, false);
+			}
+			else if (tselem->id == (ID *)OBACT) {
+				return OL_DRAWSEL_NORMAL;
+			}
 			break;
 		case TSE_LINKED_PSYS:
 			return tree_element_active_psys(C, scene, te, tselem, set);
 		case TSE_POSE_BASE:
 			return tree_element_active_pose(C, scene, te, tselem, set);
 		case TSE_POSE_CHANNEL:
-			return tree_element_active_posechannel(C, scene, te, tselem, set);
+			return tree_element_active_posechannel(C, scene, te, tselem, set, recursive);
 		case TSE_CONSTRAINT:
 			return tree_element_active_constraint(C, te, tselem, set);
 		case TSE_R_LAYER:
@@ -709,23 +875,23 @@ int tree_element_type_active(bContext *C, Scene *scene, SpaceOops *soops,
 			return tree_element_active_keymap_item(C, te, tselem, set);
 			
 	}
-	return 0;
+	return OL_DRAWSEL_NONE;
 }
 
 /* ================================================ */
 
-static int do_outliner_item_activate(bContext *C, Scene *scene, ARegion *ar, SpaceOops *soops,
-                                     TreeElement *te, int extend, const float mval[2])
+static bool do_outliner_item_activate(bContext *C, Scene *scene, ARegion *ar, SpaceOops *soops,
+                                      TreeElement *te, bool extend, bool recursive, const float mval[2])
 {
 	
 	if (mval[1] > te->ys && mval[1] < te->ys + UI_UNIT_Y) {
 		TreeStoreElem *tselem = TREESTORE(te);
-		int openclose = 0;
+		bool openclose = false;
 		
 		/* open close icon */
 		if ((te->flag & TE_ICONROW) == 0) {               // hidden icon, no open/close
 			if (mval[0] > te->xs && mval[0] < te->xs + UI_UNIT_X)
-				openclose = 1;
+				openclose = true;
 		}
 		
 		if (openclose) {
@@ -740,14 +906,16 @@ static int do_outliner_item_activate(bContext *C, Scene *scene, ARegion *ar, Spa
 				
 			}
 			
-			return 1;
+			return true;
 		}
 		/* name and first icon */
 		else if (mval[0] > te->xs + UI_UNIT_X && mval[0] < te->xend) {
 			
 			/* always makes active object */
 			if (tselem->type != TSE_SEQUENCE && tselem->type != TSE_SEQ_STRIP && tselem->type != TSE_SEQUENCE_DUP)
-				tree_element_set_active_object(C, scene, soops, te, 1 + (extend != 0 && tselem->type == 0));
+				tree_element_set_active_object(C, scene, soops, te,
+				                               (extend && tselem->type == 0) ? OL_SETSEL_EXTEND : OL_SETSEL_NORMAL,
+				                               recursive && tselem->type == 0 );
 			
 			if (tselem->type == 0) { // the lib blocks
 				/* editmode? */
@@ -784,39 +952,43 @@ static int do_outliner_item_activate(bContext *C, Scene *scene, ARegion *ar, Spa
 					
 					WM_event_add_notifier(C, NC_SCENE | ND_OB_SELECT, scene);
 				}
-				else if (ELEM5(te->idcode, ID_ME, ID_CU, ID_MB, ID_LT, ID_AR)) {
+				else if (ELEM(te->idcode, ID_ME, ID_CU, ID_MB, ID_LT, ID_AR)) {
 					WM_operator_name_call(C, "OBJECT_OT_editmode_toggle", WM_OP_INVOKE_REGION_WIN, NULL);
 				}
 				else {  // rest of types
-					tree_element_active(C, scene, soops, te, 1);
+					tree_element_active(C, scene, soops, te, OL_SETSEL_NORMAL, false);
 				}
-				
+
+			}
+			else {
+				tree_element_type_active(C, scene, soops, te, tselem,
+				                         extend ? OL_SETSEL_EXTEND : OL_SETSEL_NORMAL,
+				                         recursive);
 			}
-			else tree_element_type_active(C, scene, soops, te, tselem, 1 + (extend != 0));
 			
-			return 1;
+			return true;
 		}
 	}
 	
 	for (te = te->subtree.first; te; te = te->next) {
-		if (do_outliner_item_activate(C, scene, ar, soops, te, extend, mval)) return 1;
+		if (do_outliner_item_activate(C, scene, ar, soops, te, extend, recursive, mval)) {
+			return true;
+		}
 	}
-	return 0;
+	return false;
 }
 
-/* event can enterkey, then it opens/closes */
-static int outliner_item_activate(bContext *C, wmOperator *op, wmEvent *event)
+int outliner_item_do_activate(bContext *C, int x, int y, bool extend, bool recursive)
 {
 	Scene *scene = CTX_data_scene(C);
 	ARegion *ar = CTX_wm_region(C);
 	SpaceOops *soops = CTX_wm_space_outliner(C);
 	TreeElement *te;
 	float fmval[2];
-	int extend = RNA_boolean_get(op->ptr, "extend");
 
-	UI_view2d_region_to_view(&ar->v2d, event->mval[0], event->mval[1], fmval, fmval + 1);
+	UI_view2d_region_to_view(&ar->v2d, x, y, &fmval[0], &fmval[1]);
 
-	if (!ELEM3(soops->outlinevis, SO_DATABLOCKS, SO_USERDEF, SO_KEYMAP) &&
+	if (!ELEM(soops->outlinevis, SO_DATABLOCKS, SO_USERDEF) &&
 	    !(soops->flag & SO_HIDE_RESTRICTCOLS) &&
 	    (fmval[0] > ar->v2d.cur.xmax - OL_TOG_RESTRICT_VIEWX))
 	{
@@ -824,7 +996,7 @@ static int outliner_item_activate(bContext *C, wmOperator *op, wmEvent *event)
 	}
 
 	for (te = soops->tree.first; te; te = te->next) {
-		if (do_outliner_item_activate(C, scene, ar, soops, te, extend, fmval)) break;
+		if (do_outliner_item_activate(C, scene, ar, soops, te, extend, recursive, fmval)) break;
 	}
 	
 	if (te) {
@@ -854,6 +1026,16 @@ static int outliner_item_activate(bContext *C, wmOperator *op, wmEvent *event)
 	return OPERATOR_FINISHED;
 }
 
+/* event can enterkey, then it opens/closes */
+static int outliner_item_activate(bContext *C, wmOperator *op, const wmEvent *event)
+{
+	bool extend    = RNA_boolean_get(op->ptr, "extend");
+	bool recursive = RNA_boolean_get(op->ptr, "recursive");
+	int x = event->mval[0];
+	int y = event->mval[1];
+	return outliner_item_do_activate(C, x, y, extend, recursive);
+}
+
 void OUTLINER_OT_item_activate(wmOperatorType *ot)
 {
 	ot->name = "Activate Item";
@@ -864,7 +1046,8 @@ void OUTLINER_OT_item_activate(wmOperatorType *ot)
 	
 	ot->poll = ED_operator_outliner_active;
 	
-	RNA_def_boolean(ot->srna, "extend", 1, "Extend", "Extend selection for activation");
+	RNA_def_boolean(ot->srna, "extend", true, "Extend", "Extend selection for activation");
+	RNA_def_boolean(ot->srna, "recursive", false, "Recursive", "Select Objects and their children");
 }
 
 /* ****************************************************** */
@@ -889,7 +1072,6 @@ static void outliner_item_border_select(Scene *scene, SpaceOops *soops, rctf *re
 			outliner_item_border_select(scene, soops, rectf, te, gesture_mode);
 		}
 	}
-	return;
 }
 
 static int outliner_border_select_exec(bContext *C, wmOperator *op)
@@ -898,14 +1080,11 @@ static int outliner_border_select_exec(bContext *C, wmOperator *op)
 	SpaceOops *soops = CTX_wm_space_outliner(C);
 	ARegion *ar = CTX_wm_region(C);
 	TreeElement *te;
-	rcti rect;
 	rctf rectf;
 	int gesture_mode = RNA_int_get(op->ptr, "gesture_mode");
 
-	WM_operator_properties_border_to_rcti(op, &rect);
-
-	UI_view2d_region_to_view(&ar->v2d, rect.xmin, rect.ymin, &rectf.xmin, &rectf.ymin);
-	UI_view2d_region_to_view(&ar->v2d, rect.xmax, rect.ymax, &rectf.xmax, &rectf.ymax);
+	WM_operator_properties_border_to_rctf(op, &rectf);
+	UI_view2d_region_to_view_rctf(&ar->v2d, &rectf, &rectf);
 
 	for (te = soops->tree.first; te; te = te->next) {
 		outliner_item_border_select(scene, soops, &rectf, te, gesture_mode);
@@ -936,7 +1115,7 @@ void OUTLINER_OT_select_border(wmOperatorType *ot)
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 
 	/* rna */
-	WM_operator_properties_gesture_border(ot, FALSE);
+	WM_operator_properties_gesture_border(ot, false);
 }
 
 /* ****************************************************** */
