@@ -27,18 +27,10 @@
  *  \ingroup spseq
  */
 
-
 #include <stdlib.h>
 #include <math.h>
 #include <string.h>
 
-#ifndef WIN32
-#include <unistd.h>
-#else
-#include <io.h>
-#endif
-#include <sys/types.h>
-
 #include "MEM_guardedalloc.h"
 
 #include "BLI_blenlib.h"
@@ -47,15 +39,13 @@
 
 #include "DNA_scene_types.h"
 #include "DNA_mask_types.h"
-#include "DNA_userdef_types.h"
+
 
 #include "BKE_context.h"
 #include "BKE_global.h"
-#include "BKE_library.h"
 #include "BKE_main.h"
 #include "BKE_sequencer.h"
 #include "BKE_movieclip.h"
-#include "BKE_sequencer.h"
 #include "BKE_mask.h"
 #include "BKE_report.h"
 
@@ -70,7 +60,6 @@
 #include "ED_screen.h"
 #include "ED_sequencer.h"
 
-#include "UI_view2d.h"
 
 #include "BKE_sound.h"
 
@@ -116,7 +105,7 @@ static void sequencer_generic_invoke_path__internal(bContext *C, wmOperator *op,
 		Scene *scene = CTX_data_scene(C);
 		Sequence *last_seq = BKE_sequencer_active_get(scene);
 		if (last_seq && last_seq->strip && SEQ_HAS_PATH(last_seq)) {
-			char path[sizeof(last_seq->strip->dir)];
+			char path[FILE_MAX];
 			BLI_strncpy(path, last_seq->strip->dir, sizeof(path));
 			BLI_path_abs(path, G.main->name);
 			RNA_string_set(op->ptr, identifier, path);
@@ -124,23 +113,50 @@ static void sequencer_generic_invoke_path__internal(bContext *C, wmOperator *op,
 	}
 }
 
-static void sequencer_generic_invoke_xy__internal(bContext *C, wmOperator *op, wmEvent *event, int flag)
+static int sequencer_generic_invoke_xy_guess_channel(bContext *C, int type)
 {
-	View2D *v2d = UI_view2d_fromcontext(C);
-	
-	float mval_v2d[2];
-	
-	UI_view2d_region_to_view(v2d, event->mval[0], event->mval[1], &mval_v2d[0], &mval_v2d[1]);
+	Sequence *tgt = NULL;
+	Sequence *seq;
+	Scene *scene = CTX_data_scene(C);
+	Editing *ed = BKE_sequencer_editing_get(scene, true);
+	int cfra = (int) CFRA;
+	int proximity = INT_MAX;
 
-	/* effect strips don't need a channel initialized from the mouse */
-	if (!(flag & SEQPROP_NOCHAN)) {
-		RNA_int_set(op->ptr, "channel", (int)mval_v2d[1] + 0.5f);
+	if (!ed || !ed->seqbasep) {
+		return 1;
 	}
 
-	RNA_int_set(op->ptr, "frame_start", (int)mval_v2d[0]);
+	for (seq = ed->seqbasep->first; seq; seq = seq->next) {
+		if ((type == -1 || seq->type == type) &&
+		    (seq->enddisp < cfra) &&
+		    (cfra - seq->enddisp < proximity))
+		{
+			tgt = seq;
+			proximity = cfra - seq->enddisp;
+		}
+	}
+	
+	if (tgt) {
+		return tgt->machine;
+	}
+	return 1;
+}
+
+static void sequencer_generic_invoke_xy__internal(bContext *C, wmOperator *op, int flag, int type)
+{
+	Scene *scene = CTX_data_scene(C);
+	
+	int cfra = (int) CFRA;
+	
+	/* effect strips don't need a channel initialized from the mouse */
+	if (!(flag & SEQPROP_NOCHAN)) {
+		RNA_int_set(op->ptr, "channel", sequencer_generic_invoke_xy_guess_channel(C, type));
+	}
+
+	RNA_int_set(op->ptr, "frame_start", cfra);
 	
 	if ((flag & SEQPROP_ENDFRAME) && RNA_struct_property_is_set(op->ptr, "frame_end") == 0)
-		RNA_int_set(op->ptr, "frame_end", (int)mval_v2d[0] + 25);  // XXX arbitary but ok for now.
+		RNA_int_set(op->ptr, "frame_end", cfra + 25);  // XXX arbitary but ok for now.
 
 	if (!(flag & SEQPROP_NOPATHS)) {
 		sequencer_generic_invoke_path__internal(C, op, "filepath");
@@ -150,7 +166,8 @@ static void sequencer_generic_invoke_xy__internal(bContext *C, wmOperator *op, w
 
 static void seq_load_operator_info(SeqLoadInfo *seq_load, wmOperator *op)
 {
-	int relative = RNA_struct_find_property(op->ptr, "relative_path") && RNA_boolean_get(op->ptr, "relative_path");
+	PropertyRNA *prop;
+	const bool relative = (prop = RNA_struct_find_property(op->ptr, "relative_path")) && RNA_property_boolean_get(op->ptr, prop);
 	int is_file = -1;
 	memset(seq_load, 0, sizeof(SeqLoadInfo));
 
@@ -160,12 +177,12 @@ static void seq_load_operator_info(SeqLoadInfo *seq_load, wmOperator *op)
 	seq_load->channel =      RNA_int_get(op->ptr, "channel");
 	seq_load->len =          1; // images only, if endframe isn't set!
 
-	if (RNA_struct_find_property(op->ptr, "filepath")) {
-		RNA_string_get(op->ptr, "filepath", seq_load->path); /* full path, file is set by the caller */
+	if ((prop = RNA_struct_find_property(op->ptr, "filepath"))) {
+		RNA_property_string_get(op->ptr, prop, seq_load->path); /* full path, file is set by the caller */
 		is_file = 1;
 	}
-	else if (RNA_struct_find_property(op->ptr, "directory")) {
-		RNA_string_get(op->ptr, "directory", seq_load->path); /* full path, file is set by the caller */
+	else if ((prop = RNA_struct_find_property(op->ptr, "directory"))) {
+		RNA_property_string_get(op->ptr, prop, seq_load->path); /* full path, file is set by the caller */
 		is_file = 0;
 	}
 
@@ -173,17 +190,17 @@ static void seq_load_operator_info(SeqLoadInfo *seq_load, wmOperator *op)
 		BLI_path_rel(seq_load->path, G.main->name);
 
 	
-	if (RNA_struct_find_property(op->ptr, "frame_end")) {
-		seq_load->end_frame = RNA_int_get(op->ptr, "frame_end");
+	if ((prop = RNA_struct_find_property(op->ptr, "frame_end"))) {
+		seq_load->end_frame = RNA_property_int_get(op->ptr, prop);
 	}
 
-	if (RNA_struct_find_property(op->ptr, "replace_sel") && RNA_boolean_get(op->ptr, "replace_sel"))
+	if ((prop = RNA_struct_find_property(op->ptr, "replace_sel")) && RNA_property_boolean_get(op->ptr, prop))
 		seq_load->flag |= SEQ_LOAD_REPLACE_SEL;
 
-	if (RNA_struct_find_property(op->ptr, "cache") && RNA_boolean_get(op->ptr, "cache"))
+	if ((prop = RNA_struct_find_property(op->ptr, "cache")) && RNA_property_boolean_get(op->ptr, prop))
 		seq_load->flag |= SEQ_LOAD_SOUND_CACHE;
 
-	if (RNA_struct_find_property(op->ptr, "sound") && RNA_boolean_get(op->ptr, "sound"))
+	if ((prop = RNA_struct_find_property(op->ptr, "sound")) && RNA_property_boolean_get(op->ptr, prop))
 		seq_load->flag |= SEQ_LOAD_MOVIE_SOUND;
 
 	/* always use this for ops */
@@ -193,17 +210,43 @@ static void seq_load_operator_info(SeqLoadInfo *seq_load, wmOperator *op)
 	if (is_file == 1) {
 		BLI_strncpy(seq_load->name, BLI_path_basename(seq_load->path), sizeof(seq_load->name));
 	}
-	else if (RNA_struct_find_property(op->ptr, "files")) {
+	else if ((prop = RNA_struct_find_property(op->ptr, "files"))) {
 		/* used for image strip */
 		/* best guess, first images name */
-		RNA_BEGIN (op->ptr, itemptr, "files")
+		RNA_PROP_BEGIN (op->ptr, itemptr, prop)
 		{
 			char *name = RNA_string_get_alloc(&itemptr, "name", NULL, 0);
 			BLI_strncpy(seq_load->name, name, sizeof(seq_load->name));
 			MEM_freeN(name);
 			break;
 		}
-		RNA_END;
+		RNA_PROP_END;
+	}
+}
+
+/**
+ * Apply generic operator options.
+ */
+static void sequencer_add_apply_overlap(bContext *C, wmOperator *op, Sequence *seq)
+{
+	Scene *scene = CTX_data_scene(C);
+	Editing *ed = BKE_sequencer_editing_get(scene, false);
+
+	if (RNA_boolean_get(op->ptr, "overlap") == false) {
+		if (BKE_sequence_test_overlap(ed->seqbasep, seq)) {
+			BKE_sequence_base_shuffle(ed->seqbasep, seq, scene);
+		}
+	}
+}
+
+static void sequencer_add_apply_replace_sel(bContext *C, wmOperator *op, Sequence *seq)
+{
+	Scene *scene = CTX_data_scene(C);
+
+	if (RNA_boolean_get(op->ptr, "replace_sel")) {
+		ED_sequencer_deselect_all(scene);
+		BKE_sequencer_active_set(scene, seq);
+		seq->flag |= SELECT;
 	}
 }
 
@@ -211,7 +254,7 @@ static void seq_load_operator_info(SeqLoadInfo *seq_load, wmOperator *op)
 static int sequencer_add_scene_strip_exec(bContext *C, wmOperator *op)
 {
 	Scene *scene = CTX_data_scene(C);
-	Editing *ed = BKE_sequencer_editing_get(scene, TRUE);
+	Editing *ed = BKE_sequencer_editing_get(scene, true);
 	
 	Scene *sce_seq;
 
@@ -249,15 +292,8 @@ static int sequencer_add_scene_strip_exec(bContext *C, wmOperator *op)
 	BKE_sequence_calc_disp(scene, seq);
 	BKE_sequencer_sort(scene);
 	
-	if (RNA_boolean_get(op->ptr, "replace_sel")) {
-		ED_sequencer_deselect_all(scene);
-		BKE_sequencer_active_set(scene, seq);
-		seq->flag |= SELECT;
-	}
-
-	if (RNA_boolean_get(op->ptr, "overlap") == FALSE) {
-		if (BKE_sequence_test_overlap(ed->seqbasep, seq)) BKE_sequence_base_shuffle(ed->seqbasep, seq, scene);
-	}
+	sequencer_add_apply_replace_sel(C, op, seq);
+	sequencer_add_apply_overlap(C, op, seq);
 
 	WM_event_add_notifier(C, NC_SCENE | ND_SEQUENCER, scene);
 	
@@ -265,17 +301,12 @@ static int sequencer_add_scene_strip_exec(bContext *C, wmOperator *op)
 }
 
 
-static int sequencer_add_scene_strip_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int sequencer_add_scene_strip_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
-	if (!ED_operator_sequencer_active(C)) {
-		BKE_report(op->reports, RPT_ERROR, "Sequencer area not active");
-		return OPERATOR_CANCELLED;
-	}
-
 	if (!RNA_struct_property_is_set(op->ptr, "scene"))
 		return WM_enum_search_invoke(C, op, event);
 
-	sequencer_generic_invoke_xy__internal(C, op, event, 0);
+	sequencer_generic_invoke_xy__internal(C, op, 0, SEQ_TYPE_SCENE);
 	return sequencer_add_scene_strip_exec(C, op);
 	// needs a menu
 	// return WM_menu_invoke(C, op, event);
@@ -295,7 +326,7 @@ void SEQUENCER_OT_scene_strip_add(struct wmOperatorType *ot)
 	ot->invoke = sequencer_add_scene_strip_invoke;
 	ot->exec = sequencer_add_scene_strip_exec;
 
-	ot->poll = ED_operator_scene_editable;
+	ot->poll = ED_operator_sequencer_active_editable;
 	
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -303,6 +334,7 @@ void SEQUENCER_OT_scene_strip_add(struct wmOperatorType *ot)
 	sequencer_generic_props__internal(ot, SEQPROP_STARTFRAME);
 	prop = RNA_def_enum(ot->srna, "scene", DummyRNA_NULL_items, 0, "Scene", "");
 	RNA_def_enum_funcs(prop, RNA_scene_itemf);
+	RNA_def_property_flag(prop, PROP_ENUM_NO_TRANSLATE);
 	ot->prop = prop;
 }
 
@@ -310,7 +342,7 @@ void SEQUENCER_OT_scene_strip_add(struct wmOperatorType *ot)
 static int sequencer_add_movieclip_strip_exec(bContext *C, wmOperator *op)
 {
 	Scene *scene = CTX_data_scene(C);
-	Editing *ed = BKE_sequencer_editing_get(scene, TRUE);
+	Editing *ed = BKE_sequencer_editing_get(scene, true);
 	
 	MovieClip *clip;
 
@@ -325,7 +357,7 @@ static int sequencer_add_movieclip_strip_exec(bContext *C, wmOperator *op)
 	clip = BLI_findlink(&CTX_data_main(C)->movieclip, RNA_enum_get(op->ptr, "clip"));
 	
 	if (clip == NULL) {
-		BKE_report(op->reports, RPT_ERROR, "MovieClip not found");
+		BKE_report(op->reports, RPT_ERROR, "Movie clip not found");
 		return OPERATOR_CANCELLED;
 	}
 	
@@ -348,32 +380,20 @@ static int sequencer_add_movieclip_strip_exec(bContext *C, wmOperator *op)
 	BKE_sequence_calc_disp(scene, seq);
 	BKE_sequencer_sort(scene);
 	
-	if (RNA_boolean_get(op->ptr, "replace_sel")) {
-		ED_sequencer_deselect_all(scene);
-		BKE_sequencer_active_set(scene, seq);
-		seq->flag |= SELECT;
-	}
-
-	if (RNA_boolean_get(op->ptr, "overlap") == FALSE) {
-		if (BKE_sequence_test_overlap(ed->seqbasep, seq)) BKE_sequence_base_shuffle(ed->seqbasep, seq, scene);
-	}
+	sequencer_add_apply_replace_sel(C, op, seq);
+	sequencer_add_apply_overlap(C, op, seq);
 
 	WM_event_add_notifier(C, NC_SCENE | ND_SEQUENCER, scene);
 	
 	return OPERATOR_FINISHED;
 }
 
-static int sequencer_add_movieclip_strip_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int sequencer_add_movieclip_strip_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
-	if (!ED_operator_sequencer_active(C)) {
-		BKE_report(op->reports, RPT_ERROR, "Sequencer area not active");
-		return OPERATOR_CANCELLED;
-	}
-
 	if (!RNA_struct_property_is_set(op->ptr, "clip"))
 		return WM_enum_search_invoke(C, op, event);
 
-	sequencer_generic_invoke_xy__internal(C, op, event, 0);
+	sequencer_generic_invoke_xy__internal(C, op, 0, SEQ_TYPE_MOVIECLIP);
 	return sequencer_add_movieclip_strip_exec(C, op);
 	// needs a menu
 	// return WM_menu_invoke(C, op, event);
@@ -392,7 +412,7 @@ void SEQUENCER_OT_movieclip_strip_add(struct wmOperatorType *ot)
 	ot->invoke = sequencer_add_movieclip_strip_invoke;
 	ot->exec = sequencer_add_movieclip_strip_exec;
 
-	ot->poll = ED_operator_scene_editable;
+	ot->poll = ED_operator_sequencer_active_editable;
 
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -400,13 +420,14 @@ void SEQUENCER_OT_movieclip_strip_add(struct wmOperatorType *ot)
 	sequencer_generic_props__internal(ot, SEQPROP_STARTFRAME);
 	prop = RNA_def_enum(ot->srna, "clip", DummyRNA_NULL_items, 0, "Clip", "");
 	RNA_def_enum_funcs(prop, RNA_movieclip_itemf);
+	RNA_def_property_flag(prop, PROP_ENUM_NO_TRANSLATE);
 	ot->prop = prop;
 }
 
 static int sequencer_add_mask_strip_exec(bContext *C, wmOperator *op)
 {
 	Scene *scene = CTX_data_scene(C);
-	Editing *ed = BKE_sequencer_editing_get(scene, TRUE);
+	Editing *ed = BKE_sequencer_editing_get(scene, true);
 
 	Mask *mask;
 
@@ -444,32 +465,20 @@ static int sequencer_add_mask_strip_exec(bContext *C, wmOperator *op)
 	BKE_sequence_calc_disp(scene, seq);
 	BKE_sequencer_sort(scene);
 
-	if (RNA_boolean_get(op->ptr, "replace_sel")) {
-		ED_sequencer_deselect_all(scene);
-		BKE_sequencer_active_set(scene, seq);
-		seq->flag |= SELECT;
-	}
-
-	if (RNA_boolean_get(op->ptr, "overlap") == FALSE) {
-		if (BKE_sequence_test_overlap(ed->seqbasep, seq)) BKE_sequence_base_shuffle(ed->seqbasep, seq, scene);
-	}
+	sequencer_add_apply_replace_sel(C, op, seq);
+	sequencer_add_apply_overlap(C, op, seq);
 
 	WM_event_add_notifier(C, NC_SCENE | ND_SEQUENCER, scene);
 
 	return OPERATOR_FINISHED;
 }
 
-static int sequencer_add_mask_strip_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int sequencer_add_mask_strip_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
-	if (!ED_operator_sequencer_active(C)) {
-		BKE_report(op->reports, RPT_ERROR, "Sequencer area not active");
-		return OPERATOR_CANCELLED;
-	}
-
 	if (!RNA_struct_property_is_set(op->ptr, "mask"))
 		return WM_enum_search_invoke(C, op, event);
 
-	sequencer_generic_invoke_xy__internal(C, op, event, 0);
+	sequencer_generic_invoke_xy__internal(C, op, 0, SEQ_TYPE_MASK);
 	return sequencer_add_mask_strip_exec(C, op);
 	// needs a menu
 	// return WM_menu_invoke(C, op, event);
@@ -489,7 +498,7 @@ void SEQUENCER_OT_mask_strip_add(struct wmOperatorType *ot)
 	ot->invoke = sequencer_add_mask_strip_invoke;
 	ot->exec = sequencer_add_mask_strip_exec;
 
-	ot->poll = ED_operator_scene_editable;
+	ot->poll = ED_operator_sequencer_active_editable;
 
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -497,6 +506,7 @@ void SEQUENCER_OT_mask_strip_add(struct wmOperatorType *ot)
 	sequencer_generic_props__internal(ot, SEQPROP_STARTFRAME);
 	prop = RNA_def_enum(ot->srna, "mask", DummyRNA_NULL_items, 0, "Mask", "");
 	RNA_def_enum_funcs(prop, RNA_mask_itemf);
+	RNA_def_property_flag(prop, PROP_ENUM_NO_TRANSLATE);
 	ot->prop = prop;
 }
 
@@ -504,11 +514,9 @@ void SEQUENCER_OT_mask_strip_add(struct wmOperatorType *ot)
 static int sequencer_add_generic_strip_exec(bContext *C, wmOperator *op, SeqLoadFunc seq_load_func)
 {
 	Scene *scene = CTX_data_scene(C); /* only for sound */
-	Editing *ed = BKE_sequencer_editing_get(scene, TRUE);
+	Editing *ed = BKE_sequencer_editing_get(scene, true);
 	SeqLoadInfo seq_load;
-	Sequence *seq;
 	int tot_files;
-	const short overlap = RNA_boolean_get(op->ptr, "overlap");
 
 	seq_load_operator_info(&seq_load, op);
 
@@ -529,31 +537,39 @@ static int sequencer_add_generic_strip_exec(bContext *C, wmOperator *op, SeqLoad
 
 		RNA_BEGIN (op->ptr, itemptr, "files")
 		{
+			Sequence *seq;
+
 			RNA_string_get(&itemptr, "name", file_only);
 			BLI_join_dirfile(seq_load.path, sizeof(seq_load.path), dir_only, file_only);
 
+			/* Set seq_load.name, else all video/audio files get the same name! ugly! */
+			BLI_strncpy(seq_load.name, file_only, sizeof(seq_load.name));
+
 			seq = seq_load_func(C, ed->seqbasep, &seq_load);
 			if (seq) {
-				if (overlap == FALSE) {
-					if (BKE_sequence_test_overlap(ed->seqbasep, seq)) BKE_sequence_base_shuffle(ed->seqbasep, seq, scene);
+				sequencer_add_apply_overlap(C, op, seq);
+				if (seq_load.seq_sound) {
+					sequencer_add_apply_overlap(C, op, seq_load.seq_sound);
 				}
 			}
 		}
 		RNA_END;
 	}
 	else {
+		Sequence *seq;
+
 		/* single file */
 		seq = seq_load_func(C, ed->seqbasep, &seq_load);
 		if (seq) {
-			if (overlap == FALSE) {
-				if (BKE_sequence_test_overlap(ed->seqbasep, seq))
-					BKE_sequence_base_shuffle(ed->seqbasep, seq, scene);
+			sequencer_add_apply_overlap(C, op, seq);
+			if (seq_load.seq_sound) {
+				sequencer_add_apply_overlap(C, op, seq_load.seq_sound);
 			}
 		}
 	}
 
 	if (seq_load.tot_success == 0) {
-		BKE_reportf(op->reports, RPT_ERROR, "File \"%s\" could not be loaded", seq_load.path);
+		BKE_reportf(op->reports, RPT_ERROR, "File '%s' could not be loaded", seq_load.path);
 		return OPERATOR_CANCELLED;
 	}
 
@@ -572,23 +588,17 @@ static int sequencer_add_movie_strip_exec(bContext *C, wmOperator *op)
 }
 
 
-static int sequencer_add_movie_strip_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int sequencer_add_movie_strip_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
-
-	if (!ED_operator_sequencer_active(C)) {
-		BKE_report(op->reports, RPT_ERROR, "Sequencer area not active");
-		return OPERATOR_CANCELLED;
-	}
-
 	/* This is for drag and drop */
 	if ((RNA_struct_property_is_set(op->ptr, "files") && RNA_collection_length(op->ptr, "files")) ||
 	    RNA_struct_property_is_set(op->ptr, "filepath"))
 	{
-		sequencer_generic_invoke_xy__internal(C, op, event, SEQPROP_NOPATHS);
+		sequencer_generic_invoke_xy__internal(C, op, SEQPROP_NOPATHS, SEQ_TYPE_MOVIE);
 		return sequencer_add_movie_strip_exec(C, op);
 	}
 	
-	sequencer_generic_invoke_xy__internal(C, op, event, 0);
+	sequencer_generic_invoke_xy__internal(C, op, 0, SEQ_TYPE_MOVIE);
 	
 	WM_event_add_fileselect(C, op);
 	return OPERATOR_RUNNING_MODAL;
@@ -609,7 +619,7 @@ void SEQUENCER_OT_movie_strip_add(struct wmOperatorType *ot)
 	ot->invoke = sequencer_add_movie_strip_invoke;
 	ot->exec = sequencer_add_movie_strip_exec;
 
-	ot->poll = ED_operator_scene_editable;
+	ot->poll = ED_operator_sequencer_active_editable;
 	
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -617,7 +627,7 @@ void SEQUENCER_OT_movie_strip_add(struct wmOperatorType *ot)
 	WM_operator_properties_filesel(ot, FOLDERFILE | MOVIEFILE, FILE_SPECIAL, FILE_OPENFILE,
 	                               WM_FILESEL_FILEPATH | WM_FILESEL_RELPATH | WM_FILESEL_FILES, FILE_DEFAULTDISPLAY);
 	sequencer_generic_props__internal(ot, SEQPROP_STARTFRAME);
-	RNA_def_boolean(ot->srna, "sound", TRUE, "Sound", "Load sound with the movie");
+	RNA_def_boolean(ot->srna, "sound", true, "Sound", "Load sound with the movie");
 }
 
 /* add sound operator */
@@ -627,23 +637,17 @@ static int sequencer_add_sound_strip_exec(bContext *C, wmOperator *op)
 	return sequencer_add_generic_strip_exec(C, op, BKE_sequencer_add_sound_strip);
 }
 
-static int sequencer_add_sound_strip_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int sequencer_add_sound_strip_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
-
-	if (!ED_operator_sequencer_active(C)) {
-		BKE_report(op->reports, RPT_ERROR, "Sequencer area not active");
-		return OPERATOR_CANCELLED;
-	}
-	
 	/* This is for drag and drop */
 	if ((RNA_struct_property_is_set(op->ptr, "files") && RNA_collection_length(op->ptr, "files")) ||
 	    RNA_struct_property_is_set(op->ptr, "filepath"))
 	{
-		sequencer_generic_invoke_xy__internal(C, op, event, SEQPROP_NOPATHS);
+		sequencer_generic_invoke_xy__internal(C, op, SEQPROP_NOPATHS, SEQ_TYPE_SOUND_RAM);
 		return sequencer_add_sound_strip_exec(C, op);
 	}
 	
-	sequencer_generic_invoke_xy__internal(C, op, event, 0);
+	sequencer_generic_invoke_xy__internal(C, op, 0, SEQ_TYPE_SOUND_RAM);
 
 	WM_event_add_fileselect(C, op);
 	return OPERATOR_RUNNING_MODAL;
@@ -664,7 +668,7 @@ void SEQUENCER_OT_sound_strip_add(struct wmOperatorType *ot)
 	ot->invoke = sequencer_add_sound_strip_invoke;
 	ot->exec = sequencer_add_sound_strip_exec;
 
-	ot->poll = ED_operator_scene_editable;
+	ot->poll = ED_operator_sequencer_active_editable;
 	
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -672,7 +676,7 @@ void SEQUENCER_OT_sound_strip_add(struct wmOperatorType *ot)
 	WM_operator_properties_filesel(ot, FOLDERFILE | SOUNDFILE, FILE_SPECIAL, FILE_OPENFILE,
 	                               WM_FILESEL_FILEPATH | WM_FILESEL_RELPATH | WM_FILESEL_FILES, FILE_DEFAULTDISPLAY);
 	sequencer_generic_props__internal(ot, SEQPROP_STARTFRAME);
-	RNA_def_boolean(ot->srna, "cache", FALSE, "Cache", "Cache the sound in memory");
+	RNA_def_boolean(ot->srna, "cache", false, "Cache", "Cache the sound in memory");
 }
 
 /* add image operator */
@@ -681,7 +685,7 @@ static int sequencer_add_image_strip_exec(bContext *C, wmOperator *op)
 	/* cant use the generic function for this */
 
 	Scene *scene = CTX_data_scene(C); /* only for sound */
-	Editing *ed = BKE_sequencer_editing_get(scene, TRUE);
+	Editing *ed = BKE_sequencer_editing_get(scene, true);
 	SeqLoadInfo seq_load;
 	Sequence *seq;
 
@@ -719,40 +723,32 @@ static int sequencer_add_image_strip_exec(bContext *C, wmOperator *op)
 			seq->endstill = seq_load.end_frame - seq_load.start_frame;
 		}
 	}
-	
+
+	BKE_sequence_init_colorspace(seq);
+
 	BKE_sequence_calc_disp(scene, seq);
 
 	BKE_sequencer_sort(scene);
 
 	/* last active name */
-	strncpy(ed->act_imagedir, strip->dir, FILE_MAXDIR - 1);
+	BLI_strncpy(ed->act_imagedir, strip->dir, sizeof(ed->act_imagedir));
 
-	if (RNA_boolean_get(op->ptr, "overlap") == FALSE) {
-		if (BKE_sequence_test_overlap(ed->seqbasep, seq))
-			BKE_sequence_base_shuffle(ed->seqbasep, seq, scene);
-	}
+	sequencer_add_apply_overlap(C, op, seq);
 
 	WM_event_add_notifier(C, NC_SCENE | ND_SEQUENCER, scene);
 
 	return OPERATOR_FINISHED;
 }
 
-static int sequencer_add_image_strip_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int sequencer_add_image_strip_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
-
-	if (!ED_operator_sequencer_active(C)) {
-		BKE_report(op->reports, RPT_ERROR, "Sequencer area not active");
-		return OPERATOR_CANCELLED;
-	}
-
-
 	/* drag drop has set the names */
 	if (RNA_struct_property_is_set(op->ptr, "files") && RNA_collection_length(op->ptr, "files")) {
-		sequencer_generic_invoke_xy__internal(C, op, event, SEQPROP_ENDFRAME | SEQPROP_NOPATHS);
+		sequencer_generic_invoke_xy__internal(C, op, SEQPROP_ENDFRAME | SEQPROP_NOPATHS, SEQ_TYPE_IMAGE);
 		return sequencer_add_image_strip_exec(C, op);
 	}
 	
-	sequencer_generic_invoke_xy__internal(C, op, event, SEQPROP_ENDFRAME);
+	sequencer_generic_invoke_xy__internal(C, op, SEQPROP_ENDFRAME, SEQ_TYPE_IMAGE);
 
 	WM_event_add_fileselect(C, op);
 	return OPERATOR_RUNNING_MODAL;
@@ -771,7 +767,7 @@ void SEQUENCER_OT_image_strip_add(struct wmOperatorType *ot)
 	ot->invoke = sequencer_add_image_strip_invoke;
 	ot->exec = sequencer_add_image_strip_exec;
 
-	ot->poll = ED_operator_scene_editable;
+	ot->poll = ED_operator_sequencer_active_editable;
 	
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -786,7 +782,7 @@ void SEQUENCER_OT_image_strip_add(struct wmOperatorType *ot)
 static int sequencer_add_effect_strip_exec(bContext *C, wmOperator *op)
 {
 	Scene *scene = CTX_data_scene(C);
-	Editing *ed = BKE_sequencer_editing_get(scene, TRUE);
+	Editing *ed = BKE_sequencer_editing_get(scene, true);
 
 	Sequence *seq;  /* generic strip vars */
 	Strip *strip;
@@ -857,31 +853,24 @@ static int sequencer_add_effect_strip_exec(bContext *C, wmOperator *op)
 	 * the other strips. */
 	if (!RNA_struct_property_is_set(op->ptr, "channel")) {
 		if (seq->seq1) {
-			int chan = MAX3(seq->seq1 ? seq->seq1->machine : 0,
-			                seq->seq2 ? seq->seq2->machine : 0,
-			                seq->seq3 ? seq->seq3->machine : 0);
+			int chan = max_iii(seq->seq1 ? seq->seq1->machine : 0,
+			                   seq->seq2 ? seq->seq2->machine : 0,
+			                   seq->seq3 ? seq->seq3->machine : 0);
 			if (chan < MAXSEQ)
 				seq->machine = chan;
 		}
 	}
 
-	if (RNA_boolean_get(op->ptr, "overlap") == FALSE) {
-		if (BKE_sequence_test_overlap(ed->seqbasep, seq)) BKE_sequence_base_shuffle(ed->seqbasep, seq, scene);
-	}
+	sequencer_add_apply_replace_sel(C, op, seq);
+	sequencer_add_apply_overlap(C, op, seq);
 
-	BKE_sequencer_update_changed_seq_and_deps(scene, seq, 1, 1); /* runs calc_sequence */
+	BKE_sequencer_update_changed_seq_and_deps(scene, seq, 1, 1); /* runs BKE_sequence_calc */
 
 
 	/* not sure if this is needed with update_changed_seq_and_deps.
 	 * it was NOT called in blender 2.4x, but wont hurt */
 	BKE_sequencer_sort(scene); 
 
-	if (RNA_boolean_get(op->ptr, "replace_sel")) {
-		ED_sequencer_deselect_all(scene);
-		BKE_sequencer_active_set(scene, seq);
-		seq->flag |= SELECT;
-	}
-
 	WM_event_add_notifier(C, NC_SCENE | ND_SEQUENCER, scene);
 
 	return OPERATOR_FINISHED;
@@ -889,17 +878,12 @@ static int sequencer_add_effect_strip_exec(bContext *C, wmOperator *op)
 
 
 /* add color */
-static int sequencer_add_effect_strip_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int sequencer_add_effect_strip_invoke(bContext *C, wmOperator *op, const wmEvent *UNUSED(event))
 {
-	short is_type_set = RNA_struct_property_is_set(op->ptr, "type");
+	bool is_type_set = RNA_struct_property_is_set(op->ptr, "type");
 	int type = -1;
 	int prop_flag = SEQPROP_ENDFRAME;
 
-	if (!ED_operator_sequencer_active(C)) {
-		BKE_report(op->reports, RPT_ERROR, "Sequencer area not active");
-		return OPERATOR_CANCELLED;
-	}
-
 	if (is_type_set) {
 		type = RNA_enum_get(op->ptr, "type");
 
@@ -912,7 +896,7 @@ static int sequencer_add_effect_strip_invoke(bContext *C, wmOperator *op, wmEven
 		}
 	}
 
-	sequencer_generic_invoke_xy__internal(C, op, event, prop_flag);
+	sequencer_generic_invoke_xy__internal(C, op, prop_flag, type);
 
 	return sequencer_add_effect_strip_exec(C, op);
 }
@@ -928,7 +912,7 @@ void SEQUENCER_OT_effect_strip_add(struct wmOperatorType *ot)
 	ot->invoke = sequencer_add_effect_strip_invoke;
 	ot->exec = sequencer_add_effect_strip_exec;
 
-	ot->poll = ED_operator_scene_editable;
+	ot->poll = ED_operator_sequencer_active_editable;
 	
 	/* flags */
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
@@ -937,5 +921,6 @@ void SEQUENCER_OT_effect_strip_add(struct wmOperatorType *ot)
 	                               WM_FILESEL_FILEPATH | WM_FILESEL_RELPATH, FILE_DEFAULTDISPLAY);
 	sequencer_generic_props__internal(ot, SEQPROP_STARTFRAME | SEQPROP_ENDFRAME);
 	RNA_def_enum(ot->srna, "type", sequencer_prop_effect_types, SEQ_TYPE_CROSS, "Type", "Sequencer effect type");
-	RNA_def_float_vector(ot->srna, "color", 3, NULL, 0.0f, 1.0f, "Color", "Initialize the strip with this color (only used when type='COLOR')", 0.0f, 1.0f);
+	RNA_def_float_vector(ot->srna, "color", 3, NULL, 0.0f, 1.0f, "Color",
+	                     "Initialize the strip with this color (only used when type='COLOR')", 0.0f, 1.0f);
 }
