@@ -32,15 +32,7 @@
 #include <math.h>
 #include <string.h>
 
-#ifndef WIN32
-#include <unistd.h>
-#else
-#include <io.h>
-#endif
-#include <sys/types.h>
-
 #include "BLI_blenlib.h"
-#include "BLI_math.h"
 #include "BLI_utildefines.h"
 
 #include "DNA_scene_types.h"
@@ -56,7 +48,6 @@
 
 /* for menu/popup icons etc etc*/
 
-#include "ED_types.h"
 #include "ED_screen.h"
 #include "ED_sequencer.h"
 
@@ -162,9 +153,9 @@ void select_surround_from_last(Scene *scene)
 }
 #endif
 
-void ED_sequencer_select_sequence_single(Scene * scene, Sequence * seq, int deselect_all)
+void ED_sequencer_select_sequence_single(Scene *scene, Sequence *seq, bool deselect_all)
 {
-	Editing *ed = BKE_sequencer_editing_get(scene, FALSE);
+	Editing *ed = BKE_sequencer_editing_get(scene, false);
 	
 	if (deselect_all)
 		ED_sequencer_deselect_all(scene);
@@ -188,7 +179,7 @@ static void select_neighbor_from_last(Scene *scene, int lr)
 {
 	Sequence *seq = BKE_sequencer_active_get(scene);
 	Sequence *neighbor;
-	int change = 0;
+	bool changed = false;
 	if (seq) {
 		neighbor = find_neighboring_sequence(scene, seq, lr, -1);
 		if (neighbor) {
@@ -207,10 +198,10 @@ static void select_neighbor_from_last(Scene *scene, int lr)
 					break;
 			}
 			seq->flag |= SELECT;
-			change = 1;
+			changed = true;
 		}
 	}
-	if (change) {
+	if (changed) {
 	}
 }
 #endif
@@ -221,7 +212,7 @@ static int sequencer_de_select_all_exec(bContext *C, wmOperator *op)
 	int action = RNA_enum_get(op->ptr, "action");
 
 	Scene *scene = CTX_data_scene(C);
-	Editing *ed = BKE_sequencer_editing_get(scene, FALSE);
+	Editing *ed = BKE_sequencer_editing_get(scene, false);
 	Sequence *seq;
 
 	if (action == SEL_TOGGLE) {
@@ -282,7 +273,7 @@ void SEQUENCER_OT_select_all(struct wmOperatorType *ot)
 static int sequencer_select_inverse_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	Scene *scene = CTX_data_scene(C);
-	Editing *ed = BKE_sequencer_editing_get(scene, FALSE);
+	Editing *ed = BKE_sequencer_editing_get(scene, false);
 	Sequence *seq;
 
 	for (seq = ed->seqbasep->first; seq; seq = seq->next) {
@@ -315,15 +306,15 @@ void SEQUENCER_OT_select_inverse(struct wmOperatorType *ot)
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 }
 
-static int sequencer_select_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int sequencer_select_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	View2D *v2d = UI_view2d_fromcontext(C);
 	Scene *scene = CTX_data_scene(C);
-	Editing *ed = BKE_sequencer_editing_get(scene, FALSE);
-	short extend = RNA_boolean_get(op->ptr, "extend");
-	short linked_handle = RNA_boolean_get(op->ptr, "linked_handle");
-	short left_right = RNA_boolean_get(op->ptr, "left_right");
-	short linked_time = RNA_boolean_get(op->ptr, "linked_time");
+	Editing *ed = BKE_sequencer_editing_get(scene, false);
+	const bool extend = RNA_boolean_get(op->ptr, "extend");
+	const bool linked_handle = RNA_boolean_get(op->ptr, "linked_handle");
+	const bool linked_time = RNA_boolean_get(op->ptr, "linked_time");
+	int left_right = RNA_enum_get(op->ptr, "left_right");
 	
 	Sequence *seq, *neighbor, *act_orig;
 	int hand, sel_side;
@@ -337,8 +328,8 @@ static int sequencer_select_invoke(bContext *C, wmOperator *op, wmEvent *event)
 	seq = find_nearest_seq(scene, v2d, &hand, event->mval);
 
 	// XXX - not nice, Ctrl+RMB needs to do left_right only when not over a strip
-	if (seq && linked_time && left_right)
-		left_right = FALSE;
+	if (seq && linked_time && (left_right == SEQ_SELECT_LR_MOUSE))
+		left_right = SEQ_SELECT_LR_NONE;
 
 
 	if (marker) {
@@ -353,26 +344,35 @@ static int sequencer_select_invoke(bContext *C, wmOperator *op, wmEvent *event)
 		}
 		else {
 			/* deselect_markers(0, 0); */ /* XXX, in 2.4x, seq selection used to deselect all, need to re-thnik this for 2.5 */
-			marker->flag |= SELECT;				
+			marker->flag |= SELECT;
 		}
 		
 	}
-	else if (left_right) {
+	else if (left_right != SEQ_SELECT_LR_NONE) {
 		/* use different logic for this */
 		float x;
 		ED_sequencer_deselect_all(scene);
-		UI_view2d_region_to_view(v2d, event->mval[0], event->mval[1], &x, NULL);
-
+		
+		switch (left_right) {
+			case SEQ_SELECT_LR_MOUSE:
+				x = UI_view2d_region_to_view_x(v2d, event->mval[0]);
+				break;
+			case SEQ_SELECT_LR_LEFT:
+			case SEQ_SELECT_LR_RIGHT:
+				x = CFRA;
+				break;
+		}
+		
 		SEQP_BEGIN (ed, seq)
 		{
 			if (x < CFRA) {
-				if (seq->enddisp < CFRA) {
+				if (seq->enddisp <= CFRA) {
 					seq->flag |= SELECT;
 					recurs_sel_seq(seq);
 				}
 			}
 			else {
-				if (seq->startdisp > CFRA) {
+				if (seq->startdisp >= CFRA) {
 					seq->flag |= SELECT;
 					recurs_sel_seq(seq);
 				}
@@ -399,7 +399,7 @@ static int sequencer_select_invoke(bContext *C, wmOperator *op, wmEvent *event)
 		}
 	}
 	else {
-		// seq= find_nearest_seq(scene, v2d, &hand, mval);
+		// seq = find_nearest_seq(scene, v2d, &hand, mval);
 
 		act_orig = ed->act_seq;
 
@@ -414,8 +414,7 @@ static int sequencer_select_invoke(bContext *C, wmOperator *op, wmEvent *event)
 					BLI_strncpy(ed->act_imagedir, seq->strip->dir, FILE_MAXDIR);
 				}
 			}
-			else
-			if (seq->type == SEQ_TYPE_SOUND_RAM) {
+			else if (seq->type == SEQ_TYPE_SOUND_RAM) {
 				if (seq->strip) {
 					BLI_strncpy(ed->act_sounddir, seq->strip->dir, FILE_MAXDIR);
 				}
@@ -532,6 +531,14 @@ static int sequencer_select_invoke(bContext *C, wmOperator *op, wmEvent *event)
 
 void SEQUENCER_OT_select(wmOperatorType *ot)
 {
+	static EnumPropertyItem sequencer_select_left_right_types[] = {
+		{SEQ_SELECT_LR_NONE, "NONE", 0, "None", "Don't do left-right selection"},
+		{SEQ_SELECT_LR_MOUSE, "MOUSE", 0, "Mouse", "Use mouse position for selection"},
+		{SEQ_SELECT_LR_LEFT, "LEFT", 0, "Left", "Select left"},
+		{SEQ_SELECT_LR_RIGHT, "RIGHT", 0, "Right", "Select right"},
+		{0, NULL, 0, NULL, NULL}
+	};
+	
 	/* identifiers */
 	ot->name = "Activate/Select";
 	ot->idname = "SEQUENCER_OT_select";
@@ -548,22 +555,21 @@ void SEQUENCER_OT_select(wmOperatorType *ot)
 	RNA_def_boolean(ot->srna, "extend", 0, "Extend", "Extend the selection");
 	RNA_def_boolean(ot->srna, "linked_handle", 0, "Linked Handle", "Select handles next to the active strip");
 	/* for animation this is an enum but atm having an enum isn't useful for us */
-	RNA_def_boolean(ot->srna, "left_right", 0, "Left/Right", "Select based on the current frame side the cursor is on");
+	RNA_def_enum(ot->srna, "left_right", sequencer_select_left_right_types, 0, "Left/Right", "Select based on the current frame side the cursor is on");
 	RNA_def_boolean(ot->srna, "linked_time", 0, "Linked Time", "Select other strips at the same time");
 }
 
 
-
-
 /* run recursively to select linked */
-static int select_more_less_seq__internal(Scene *scene, int sel, int linked)
+static bool select_more_less_seq__internal(Scene *scene, bool sel, const bool linked)
 {
-	Editing *ed = BKE_sequencer_editing_get(scene, FALSE);
+	Editing *ed = BKE_sequencer_editing_get(scene, false);
 	Sequence *seq, *neighbor;
-	int change = 0;
+	bool changed = false;
 	int isel;
 	
-	if (ed == NULL) return 0;
+	if (ed == NULL)
+		return changed;
 	
 	if (sel) {
 		sel = SELECT;
@@ -590,20 +596,20 @@ static int select_more_less_seq__internal(Scene *scene, int sel, int linked)
 					if (sel) { neighbor->flag |= SELECT; recurs_sel_seq(neighbor); }
 					else neighbor->flag &= ~SELECT;
 					if (linked == 0) neighbor->tmp = (Sequence *)1;
-					change = 1;
+					changed = true;
 				}
 				neighbor = find_neighboring_sequence(scene, seq, SEQ_SIDE_RIGHT, isel);
 				if (neighbor) {
 					if (sel) { neighbor->flag |= SELECT; recurs_sel_seq(neighbor); }
 					else neighbor->flag &= ~SELECT;
 					if (linked == 0) neighbor->tmp = (void *)1;
-					change = 1;
+					changed = true;
 				}
 			}
 		}
 	}
 	
-	return change;
+	return changed;
 }
 
 
@@ -671,12 +677,12 @@ void SEQUENCER_OT_select_less(wmOperatorType *ot)
 
 
 /* select pick linked operator (uses the mouse) */
-static int sequencer_select_linked_pick_invoke(bContext *C, wmOperator *op, wmEvent *event)
+static int sequencer_select_linked_pick_invoke(bContext *C, wmOperator *op, const wmEvent *event)
 {
 	Scene *scene = CTX_data_scene(C);
 	View2D *v2d = UI_view2d_fromcontext(C);
 	
-	short extend = RNA_boolean_get(op->ptr, "extend");
+	bool extend = RNA_boolean_get(op->ptr, "extend");
 	
 	Sequence *mouse_seq;
 	int selected, hand;
@@ -705,7 +711,7 @@ static int sequencer_select_linked_pick_invoke(bContext *C, wmOperator *op, wmEv
 void SEQUENCER_OT_select_linked_pick(wmOperatorType *ot)
 {
 	/* identifiers */
-	ot->name = "Select pick linked";
+	ot->name = "Select Pick Linked";
 	ot->idname = "SEQUENCER_OT_select_linked_pick";
 	ot->description = "Select a chain of linked strips nearest to the mouse pointer";
 	
@@ -725,11 +731,11 @@ void SEQUENCER_OT_select_linked_pick(wmOperatorType *ot)
 static int sequencer_select_linked_exec(bContext *C, wmOperator *UNUSED(op))
 {
 	Scene *scene = CTX_data_scene(C);
-	int selected;
+	bool selected;
 
-	selected = 1;
+	selected = true;
 	while (selected) {
-		selected = select_more_less_seq__internal(scene, 1, 1);
+		selected = select_more_less_seq__internal(scene, true, true);
 	}
 
 	WM_event_add_notifier(C, NC_SCENE | ND_SEQUENCER | NA_SELECTED, scene);
@@ -740,7 +746,7 @@ static int sequencer_select_linked_exec(bContext *C, wmOperator *UNUSED(op))
 void SEQUENCER_OT_select_linked(wmOperatorType *ot)
 {
 	/* identifiers */
-	ot->name = "Select linked";
+	ot->name = "Select Linked";
 	ot->idname = "SEQUENCER_OT_select_linked";
 	ot->description = "Select all strips adjacent to the current selection";
 	
@@ -759,7 +765,7 @@ void SEQUENCER_OT_select_linked(wmOperatorType *ot)
 static int sequencer_select_handles_exec(bContext *C, wmOperator *op)
 {
 	Scene *scene = CTX_data_scene(C);
-	Editing *ed = BKE_sequencer_editing_get(scene, 0);
+	Editing *ed = BKE_sequencer_editing_get(scene, false);
 	Sequence *seq;
 	int sel_side = RNA_enum_get(op->ptr, "side");
 
@@ -809,7 +815,7 @@ void SEQUENCER_OT_select_handles(wmOperatorType *ot)
 static int sequencer_select_active_side_exec(bContext *C, wmOperator *op)
 {
 	Scene *scene = CTX_data_scene(C);
-	Editing *ed = BKE_sequencer_editing_get(scene, 0);
+	Editing *ed = BKE_sequencer_editing_get(scene, false);
 	Sequence *seq_act = BKE_sequencer_active_get(scene);
 
 	if (ed == NULL || seq_act == NULL)
@@ -847,33 +853,25 @@ void SEQUENCER_OT_select_active_side(wmOperatorType *ot)
 static int sequencer_borderselect_exec(bContext *C, wmOperator *op)
 {
 	Scene *scene = CTX_data_scene(C);
-	Editing *ed = BKE_sequencer_editing_get(scene, FALSE);
+	Editing *ed = BKE_sequencer_editing_get(scene, false);
 	View2D *v2d = UI_view2d_fromcontext(C);
 	
 	Sequence *seq;
-	rcti rect;
 	rctf rectf, rq;
-	short selecting = (RNA_int_get(op->ptr, "gesture_mode") == GESTURE_MODAL_SELECT);
-	int extend = RNA_boolean_get(op->ptr, "extend");
-	int mval[2];
+	const bool select = (RNA_int_get(op->ptr, "gesture_mode") == GESTURE_MODAL_SELECT);
+	const bool extend = RNA_boolean_get(op->ptr, "extend");
 
 	if (ed == NULL)
 		return OPERATOR_CANCELLED;
 
-	WM_operator_properties_border_to_rcti(op, &rect);
-	
-	mval[0] = rect.xmin;
-	mval[1] = rect.ymin;
-	UI_view2d_region_to_view(v2d, mval[0], mval[1], &rectf.xmin, &rectf.ymin);
-	mval[0] = rect.xmax;
-	mval[1] = rect.ymax;
-	UI_view2d_region_to_view(v2d, mval[0], mval[1], &rectf.xmax, &rectf.ymax);
+	WM_operator_properties_border_to_rctf(op, &rectf);
+	UI_view2d_region_to_view_rctf(v2d, &rectf, &rectf);
 
 	for (seq = ed->seqbasep->first; seq; seq = seq->next) {
 		seq_rectf(seq, &rq);
 		
 		if (BLI_rctf_isect(&rq, &rectf, NULL)) {
-			if (selecting) seq->flag |= SELECT;
+			if (select) seq->flag |= SELECT;
 			else seq->flag &= ~SEQ_ALLSEL;
 			recurs_sel_seq(seq);
 		}
@@ -895,7 +893,7 @@ void SEQUENCER_OT_select_border(wmOperatorType *ot)
 	/* identifiers */
 	ot->name = "Border Select";
 	ot->idname = "SEQUENCER_OT_select_border";
-	ot->description = "Enable border select mode";
+	ot->description = "Select strips using border selection";
 	
 	/* api callbacks */
 	ot->invoke = WM_border_select_invoke;
@@ -909,7 +907,7 @@ void SEQUENCER_OT_select_border(wmOperatorType *ot)
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 	
 	/* rna */
-	WM_operator_properties_gesture_border(ot, TRUE);
+	WM_operator_properties_gesture_border(ot, true);
 }
 
 /* ****** Selected Grouped ****** */
@@ -929,20 +927,20 @@ static EnumPropertyItem sequencer_prop_select_grouped_types[] = {
 
 #define SEQ_IS_SOUND(_seq) ((_seq->type & SEQ_TYPE_SOUND_RAM) && !(_seq->type & SEQ_TYPE_EFFECT))
 
-#define SEQ_IS_EFFECT(_seq) (_seq->type & SEQ_TYPE_EFFECT)
+#define SEQ_IS_EFFECT(_seq) ((_seq->type & SEQ_TYPE_EFFECT) != 0)
 
-#define SEQ_USE_DATA(_seq) (ELEM3(_seq->type, SEQ_TYPE_SCENE, SEQ_TYPE_MOVIECLIP, SEQ_TYPE_MASK) || SEQ_HAS_PATH(_seq))
+#define SEQ_USE_DATA(_seq) (ELEM(_seq->type, SEQ_TYPE_SCENE, SEQ_TYPE_MOVIECLIP, SEQ_TYPE_MASK) || SEQ_HAS_PATH(_seq))
 
-static short select_grouped_type(Editing *ed, Sequence *actseq)
+static bool select_grouped_type(Editing *ed, Sequence *actseq)
 {
 	Sequence *seq;
-	short changed = FALSE;
+	bool changed = false;
 
 	SEQP_BEGIN (ed, seq)
 	{
 		if (seq->type == actseq->type) {
 			seq->flag |= SELECT;
-			changed = TRUE;
+			changed = true;
 		}
 	}
 	SEQ_END;
@@ -950,17 +948,17 @@ static short select_grouped_type(Editing *ed, Sequence *actseq)
 	return changed;
 }
 
-static short select_grouped_type_basic(Editing *ed, Sequence *actseq)
+static bool select_grouped_type_basic(Editing *ed, Sequence *actseq)
 {
 	Sequence *seq;
-	short changed = FALSE;
-	short is_sound = SEQ_IS_SOUND(actseq);
+	bool changed = false;
+	const bool is_sound = SEQ_IS_SOUND(actseq);
 
 	SEQP_BEGIN (ed, seq)
 	{
 		if (is_sound ? SEQ_IS_SOUND(seq) : !SEQ_IS_SOUND(seq)) {
 			seq->flag |= SELECT;
-			changed = TRUE;
+			changed = true;
 		}
 	}
 	SEQ_END;
@@ -968,17 +966,17 @@ static short select_grouped_type_basic(Editing *ed, Sequence *actseq)
 	return changed;
 }
 
-static short select_grouped_type_effect(Editing *ed, Sequence *actseq)
+static bool select_grouped_type_effect(Editing *ed, Sequence *actseq)
 {
 	Sequence *seq;
-	short changed = FALSE;
-	short is_effect = SEQ_IS_EFFECT(actseq);
+	bool changed = false;
+	const bool is_effect = SEQ_IS_EFFECT(actseq);
 
 	SEQP_BEGIN (ed, seq)
 	{
 		if (is_effect ? SEQ_IS_EFFECT(seq) : !SEQ_IS_EFFECT(seq)) {
 			seq->flag |= SELECT;
-			changed = TRUE;
+			changed = true;
 		}
 	}
 	SEQ_END;
@@ -986,11 +984,11 @@ static short select_grouped_type_effect(Editing *ed, Sequence *actseq)
 	return changed;
 }
 
-static short select_grouped_data(Editing *ed, Sequence *actseq)
+static bool select_grouped_data(Editing *ed, Sequence *actseq)
 {
 	Sequence *seq;
-	short changed = FALSE;
-	char *dir = actseq->strip ? actseq->strip->dir : NULL;
+	bool changed = false;
+	const char *dir = actseq->strip ? actseq->strip->dir : NULL;
 
 	if (!SEQ_USE_DATA(actseq))
 		return changed;
@@ -1000,7 +998,7 @@ static short select_grouped_data(Editing *ed, Sequence *actseq)
 		{
 			if (SEQ_HAS_PATH(seq) && seq->strip && strcmp(seq->strip->dir, dir) == 0) {
 				seq->flag |= SELECT;
-				changed = TRUE;
+				changed = true;
 			}
 		}
 		SEQ_END;
@@ -1011,7 +1009,7 @@ static short select_grouped_data(Editing *ed, Sequence *actseq)
 		{
 			if (seq->type == SEQ_TYPE_SCENE && seq->scene == sce) {
 				seq->flag |= SELECT;
-				changed = TRUE;
+				changed = true;
 			}
 		}
 		SEQ_END;
@@ -1022,7 +1020,7 @@ static short select_grouped_data(Editing *ed, Sequence *actseq)
 		{
 			if (seq->type == SEQ_TYPE_MOVIECLIP && seq->clip == clip) {
 				seq->flag |= SELECT;
-				changed = TRUE;
+				changed = true;
 			}
 		}
 		SEQ_END;
@@ -1033,7 +1031,7 @@ static short select_grouped_data(Editing *ed, Sequence *actseq)
 		{
 			if (seq->type == SEQ_TYPE_MASK && seq->mask == mask) {
 				seq->flag |= SELECT;
-				changed = TRUE;
+				changed = true;
 			}
 		}
 		SEQ_END;
@@ -1042,20 +1040,20 @@ static short select_grouped_data(Editing *ed, Sequence *actseq)
 	return changed;
 }
 
-static short select_grouped_effect(Editing *ed, Sequence *actseq)
+static bool select_grouped_effect(Editing *ed, Sequence *actseq)
 {
 	Sequence *seq;
-	short changed = FALSE;
-	short effects[SEQ_TYPE_EFFECT_MAX + 1];
+	bool changed = false;
+	bool effects[SEQ_TYPE_EFFECT_MAX + 1];
 	int i;
 
 	for (i = 0; i <= SEQ_TYPE_EFFECT_MAX; i++)
-		effects[i] = FALSE;
+		effects[i] = false;
 
 	SEQP_BEGIN (ed, seq)
 	{
-		if (ELEM3(actseq, seq->seq1, seq->seq2, seq->seq3)) {
-			effects[seq->type] = TRUE;
+		if (ELEM(actseq, seq->seq1, seq->seq2, seq->seq3)) {
+			effects[seq->type] = true;
 		}
 	}
 	SEQ_END;
@@ -1066,7 +1064,7 @@ static short select_grouped_effect(Editing *ed, Sequence *actseq)
 			if (seq->seq1) seq->seq1->flag |= SELECT;
 			if (seq->seq2) seq->seq2->flag |= SELECT;
 			if (seq->seq3) seq->seq3->flag |= SELECT;
-			changed = TRUE;
+			changed = true;
 		}
 	}
 	SEQ_END;
@@ -1074,16 +1072,16 @@ static short select_grouped_effect(Editing *ed, Sequence *actseq)
 	return changed;
 }
 
-static short select_grouped_time_overlap(Editing *ed, Sequence *actseq)
+static bool select_grouped_time_overlap(Editing *ed, Sequence *actseq)
 {
 	Sequence *seq;
-	short changed = FALSE;
+	bool changed = false;
 
 	SEQP_BEGIN (ed, seq)
 	{
 		if (!((seq->startdisp >= actseq->enddisp) || (seq->enddisp < actseq->startdisp))) {
 			seq->flag |= SELECT;
-			changed = TRUE;
+			changed = true;
 		}
 	}
 	SEQ_END;
@@ -1091,11 +1089,11 @@ static short select_grouped_time_overlap(Editing *ed, Sequence *actseq)
 	return changed;
 }
 
-static short select_grouped_effect_link(Editing *ed, Sequence *actseq)
+static bool select_grouped_effect_link(Editing *ed, Sequence *actseq)
 {
 	Sequence *seq = NULL;
-	short changed = FALSE;
-	short is_audio = ((actseq->type == SEQ_TYPE_META) || SEQ_IS_SOUND(actseq));
+	bool changed = false;
+	const bool is_audio = ((actseq->type == SEQ_TYPE_META) || SEQ_IS_SOUND(actseq));
 	int startdisp = actseq->startdisp;
 	int enddisp   = actseq->enddisp;
 	int machine   = actseq->machine;
@@ -1107,9 +1105,9 @@ static short select_grouped_effect_link(Editing *ed, Sequence *actseq)
 	}
 	SEQ_END;
 
-	actseq->tmp = SET_INT_IN_POINTER(TRUE);
+	actseq->tmp = SET_INT_IN_POINTER(true);
 
-	for (BKE_sequence_iterator_begin(ed, &iter, TRUE); iter.valid; BKE_sequence_iterator_next(&iter)) {
+	for (BKE_sequence_iterator_begin(ed, &iter, true); iter.valid; BKE_sequence_iterator_next(&iter)) {
 		seq = iter.seq;
 
 		/* Ignore all seqs already selected! */
@@ -1131,20 +1129,20 @@ static short select_grouped_effect_link(Editing *ed, Sequence *actseq)
 			if (enddisp < seq->enddisp) enddisp = seq->enddisp;
 			if (machine < seq->machine) machine = seq->machine;
 
-			seq->tmp = SET_INT_IN_POINTER(TRUE);
+			seq->tmp = SET_INT_IN_POINTER(true);
 
 			seq->flag |= SELECT;
-			changed = TRUE;
+			changed = true;
 
 			/* Unfortunately, we must restart checks from the beginning. */
 			BKE_sequence_iterator_end(&iter);
-			BKE_sequence_iterator_begin(ed, &iter, TRUE);
+			BKE_sequence_iterator_begin(ed, &iter, true);
 		}
 
 		/* Video strips bellow active one, or any strip for audio (order do no matters here!). */
 		else if (seq->machine < machine || is_audio) {
 			seq->flag |= SELECT;
-			changed = TRUE;
+			changed = true;
 		}
 	}
 	BKE_sequence_iterator_end(&iter);
@@ -1159,15 +1157,15 @@ static short select_grouped_effect_link(Editing *ed, Sequence *actseq)
 static int sequencer_select_grouped_exec(bContext *C, wmOperator *op)
 {
 	Scene *scene  = CTX_data_scene(C);
-	Editing *ed   = BKE_sequencer_editing_get(scene, 0);
+	Editing *ed   = BKE_sequencer_editing_get(scene, false);
 	Sequence *seq, *actseq = BKE_sequencer_active_get(scene);
 	int type = RNA_enum_get(op->ptr, "type");
-	short changed = 0, extend;
+	bool changed = false, extend;
 
 	extend = RNA_boolean_get(op->ptr, "extend");
 
 	if (actseq == NULL) {
-		BKE_report(op->reports, RPT_ERROR, "No Active Sequence!");
+		BKE_report(op->reports, RPT_ERROR, "No active sequence!");
 		return OPERATOR_CANCELLED;
 	}
 
@@ -1175,7 +1173,7 @@ static int sequencer_select_grouped_exec(bContext *C, wmOperator *op)
 		SEQP_BEGIN (ed, seq)
 		{
 			seq->flag &= ~SELECT;
-			changed = TRUE;
+			changed = true;
 		}
 		SEQ_END;
 	}
@@ -1212,7 +1210,7 @@ void SEQUENCER_OT_select_grouped(wmOperatorType *ot)
 	ot->flag = OPTYPE_REGISTER | OPTYPE_UNDO;
 	
 	/* properties */
-	RNA_def_boolean(ot->srna, "extend", FALSE, "Extend", "Extend selection instead of deselecting everything first");
+	RNA_def_boolean(ot->srna, "extend", false, "Extend", "Extend selection instead of deselecting everything first");
 	ot->prop = RNA_def_enum(ot->srna, "type", sequencer_prop_select_grouped_types, 0, "Type", "");
 }
 
