@@ -38,7 +38,6 @@
 #include "MEM_guardedalloc.h"
 
 #include "BLI_blenlib.h"
-#include "BLI_math.h"
 #include "BLI_utildefines.h"
 
 #include "BKE_context.h"
@@ -47,13 +46,13 @@
 #include "BKE_global.h"
 
 #include "ED_space_api.h"
-#include "ED_sequencer.h"
 #include "ED_screen.h"
 #include "ED_view3d.h" /* only for sequencer view3d drawing callback */
 
 #include "WM_api.h"
 #include "WM_types.h"
 
+#include "UI_interface.h"
 #include "UI_resources.h"
 #include "UI_view2d.h"
 
@@ -120,6 +119,8 @@ static SpaceLink *sequencer_new(const bContext *C)
 	sseq->chanshown = 0;
 	sseq->view = SEQ_VIEW_SEQUENCE;
 	sseq->mainb = SEQ_DRAW_IMG_IMBUF;
+	sseq->flag = SEQ_SHOW_GPENCIL | SEQ_USE_ALPHA;
+
 	/* header */
 	ar = MEM_callocN(sizeof(ARegion), "header for sequencer");
 	
@@ -231,28 +232,28 @@ static void sequencer_refresh(const bContext *C, ScrArea *sa)
 	SpaceSeq *sseq = (SpaceSeq *)sa->spacedata.first;
 	ARegion *ar_main = sequencer_find_region(sa, RGN_TYPE_WINDOW);
 	ARegion *ar_preview = sequencer_find_region(sa, RGN_TYPE_PREVIEW);
-	int view_changed = 0;
+	bool view_changed = false;
 
 	switch (sseq->view) {
 		case SEQ_VIEW_SEQUENCE:
 			if (ar_main && (ar_main->flag & RGN_FLAG_HIDDEN)) {
 				ar_main->flag &= ~RGN_FLAG_HIDDEN;
 				ar_main->v2d.flag &= ~V2D_IS_INITIALISED;
-				view_changed = 1;
+				view_changed = true;
 			}
 			if (ar_preview && !(ar_preview->flag & RGN_FLAG_HIDDEN)) {
 				ar_preview->flag |= RGN_FLAG_HIDDEN;
 				ar_preview->v2d.flag &= ~V2D_IS_INITIALISED;
 				WM_event_remove_handlers((bContext *)C, &ar_preview->handlers);
-				view_changed = 1;
+				view_changed = true;
 			}
 			if (ar_main && ar_main->alignment != RGN_ALIGN_NONE) {
 				ar_main->alignment = RGN_ALIGN_NONE;
-				view_changed = 1;
+				view_changed = true;
 			}
 			if (ar_preview && ar_preview->alignment != RGN_ALIGN_NONE) {
 				ar_preview->alignment = RGN_ALIGN_NONE;
-				view_changed = 1;
+				view_changed = true;
 			}
 			break;
 		case SEQ_VIEW_PREVIEW:
@@ -260,42 +261,58 @@ static void sequencer_refresh(const bContext *C, ScrArea *sa)
 				ar_main->flag |= RGN_FLAG_HIDDEN;
 				ar_main->v2d.flag &= ~V2D_IS_INITIALISED;
 				WM_event_remove_handlers((bContext *)C, &ar_main->handlers);
-				view_changed = 1;
+				view_changed = true;
 			}
 			if (ar_preview && (ar_preview->flag & RGN_FLAG_HIDDEN)) {
 				ar_preview->flag &= ~RGN_FLAG_HIDDEN;
 				ar_preview->v2d.flag &= ~V2D_IS_INITIALISED;
 				ar_preview->v2d.cur = ar_preview->v2d.tot;
-				view_changed = 1;
+				view_changed = true;
 			}
 			if (ar_main && ar_main->alignment != RGN_ALIGN_NONE) {
 				ar_main->alignment = RGN_ALIGN_NONE;
-				view_changed = 1;
+				view_changed = true;
 			}
 			if (ar_preview && ar_preview->alignment != RGN_ALIGN_NONE) {
 				ar_preview->alignment = RGN_ALIGN_NONE;
-				view_changed = 1;
+				view_changed = true;
 			}
 			break;
 		case SEQ_VIEW_SEQUENCE_PREVIEW:
-			if (ar_main && (ar_main->flag & RGN_FLAG_HIDDEN)) {
-				ar_main->flag &= ~RGN_FLAG_HIDDEN;
-				ar_main->v2d.flag &= ~V2D_IS_INITIALISED;
-				view_changed = 1;
-			}
-			if (ar_preview && (ar_preview->flag & RGN_FLAG_HIDDEN)) {
-				ar_preview->flag &= ~RGN_FLAG_HIDDEN;
-				ar_preview->v2d.flag &= ~V2D_IS_INITIALISED;
-				ar_preview->v2d.cur = ar_preview->v2d.tot;
-				view_changed = 1;
-			}
-			if (ar_main && ar_main->alignment != RGN_ALIGN_NONE) {
-				ar_main->alignment = RGN_ALIGN_NONE;
-				view_changed = 1;
-			}
-			if (ar_preview && ar_preview->alignment != RGN_ALIGN_TOP) {
-				ar_preview->alignment = RGN_ALIGN_TOP;
-				view_changed = 1;
+			if (ar_main && ar_preview) {
+				/* Get available height (without DPI correction). */
+				const float height = (sa->winy - ED_area_headersize()) / UI_DPI_FAC;
+
+				/* We reuse hidden area's size, allows to find same layout as before if we just switch
+				 * between one 'full window' view and the combined one. This gets lost if we switch to both
+				 * 'full window' views before, though... Better than nothing. */
+				if (ar_main->flag & RGN_FLAG_HIDDEN) {
+					ar_main->flag &= ~RGN_FLAG_HIDDEN;
+					ar_main->v2d.flag &= ~V2D_IS_INITIALISED;
+					ar_preview->sizey = (int)(height - ar_main->sizey);
+					view_changed = true;
+				}
+				if (ar_preview->flag & RGN_FLAG_HIDDEN) {
+					ar_preview->flag &= ~RGN_FLAG_HIDDEN;
+					ar_preview->v2d.flag &= ~V2D_IS_INITIALISED;
+					ar_preview->v2d.cur = ar_preview->v2d.tot;
+					ar_main->sizey = (int)(height - ar_preview->sizey);
+					view_changed = true;
+				}
+				if (ar_main->alignment != RGN_ALIGN_NONE) {
+					ar_main->alignment = RGN_ALIGN_NONE;
+					view_changed = true;
+				}
+				if (ar_preview->alignment != RGN_ALIGN_TOP) {
+					ar_preview->alignment = RGN_ALIGN_TOP;
+					view_changed = true;
+				}
+				/* Final check that both preview and main height are reasonable! */
+				if (ar_preview->sizey < 10 || ar_main->sizey < 10 || ar_preview->sizey + ar_main->sizey > height) {
+					ar_preview->sizey = (int)(height * 0.4f + 0.5f);
+					ar_main->sizey = (int)(height - ar_preview->sizey);
+					view_changed = true;
+				}
 			}
 			break;
 	}
@@ -311,12 +328,14 @@ static SpaceLink *sequencer_duplicate(SpaceLink *sl)
 	SpaceSeq *sseqn = MEM_dupallocN(sl);
 	
 	/* clear or remove stuff from old */
-// XXX	sseq->gpd= gpencil_data_duplicate(sseq->gpd);
+// XXX	sseq->gpd = gpencil_data_duplicate(sseq->gpd, false);
+
+	memset(&sseqn->scopes, 0, sizeof(sseqn->scopes));
 
 	return (SpaceLink *)sseqn;
 }
 
-static void sequencer_listener(ScrArea *sa, wmNotifier *wmn)
+static void sequencer_listener(bScreen *UNUSED(sc), ScrArea *sa, wmNotifier *wmn)
 {
 	/* context changes */
 	switch (wmn->category) {
@@ -328,50 +347,21 @@ static void sequencer_listener(ScrArea *sa, wmNotifier *wmn)
 					break;
 			}
 			break;
+		case NC_WINDOW:
 		case NC_SPACE:
 			if (wmn->data == ND_SPACE_SEQUENCER)
 				sequencer_scopes_tag_refresh(sa);
 			break;
+		case NC_GPENCIL:
+			if (wmn->data & ND_GPENCIL_EDITMODE)
+				ED_area_tag_redraw(sa);
+			break;
 	}
 }
 
-/* *********************** sequencer (main) region ************************ */
-/* add handlers, stuff you only do once or on area/region changes */
-static void sequencer_main_area_init(wmWindowManager *wm, ARegion *ar)
-{
-	wmKeyMap *keymap;
-	ListBase *lb;
-	
-	UI_view2d_region_reinit(&ar->v2d, V2D_COMMONVIEW_CUSTOM, ar->winx, ar->winy);
-
-//	keymap = WM_keymap_find(wm->defaultconf, "Mask Editing", 0, 0);
-//	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
-
-	keymap = WM_keymap_find(wm->defaultconf, "SequencerCommon", SPACE_SEQ, 0);
-	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
-	
-	/* own keymap */
-	keymap = WM_keymap_find(wm->defaultconf, "Sequencer", SPACE_SEQ, 0);
-	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
-	
-	/* add drop boxes */
-	lb = WM_dropboxmap_find("Sequencer", SPACE_SEQ, RGN_TYPE_WINDOW);
-	
-	WM_event_add_dropbox_handler(&ar->handlers, lb);
-	
-}
-
-static void sequencer_main_area_draw(const bContext *C, ARegion *ar)
-{
-//	ScrArea *sa= CTX_wm_area(C);
-	
-	/* NLE - strip editing timeline interface */
-	draw_timeline_seq(C, ar);
-}
-
 /* ************* dropboxes ************* */
 
-static int image_drop_poll(bContext *C, wmDrag *drag, wmEvent *event)
+static int image_drop_poll(bContext *C, wmDrag *drag, const wmEvent *event)
 {
 	ARegion *ar = CTX_wm_region(C);
 	Scene *scene = CTX_data_scene(C);
@@ -385,20 +375,20 @@ static int image_drop_poll(bContext *C, wmDrag *drag, wmEvent *event)
 	return 0;
 }
 
-static int movie_drop_poll(bContext *C, wmDrag *drag, wmEvent *event)
+static int movie_drop_poll(bContext *C, wmDrag *drag, const wmEvent *event)
 {
 	ARegion *ar = CTX_wm_region(C);
 	Scene *scene = CTX_data_scene(C);
 	int hand;
 
 	if (drag->type == WM_DRAG_PATH)
-		if (ELEM3(drag->icon, 0, ICON_FILE_MOVIE, ICON_FILE_BLANK)) /* rule might not work? */
+		if (ELEM(drag->icon, 0, ICON_FILE_MOVIE, ICON_FILE_BLANK)) /* rule might not work? */
 			if (find_nearest_seq(scene, &ar->v2d, &hand, event->mval) == NULL)
 				return 1;
 	return 0;
 }
 
-static int sound_drop_poll(bContext *C, wmDrag *drag, wmEvent *event)
+static int sound_drop_poll(bContext *C, wmDrag *drag, const wmEvent *event)
 {
 	ARegion *ar = CTX_wm_region(C);
 	Scene *scene = CTX_data_scene(C);
@@ -435,7 +425,7 @@ static void sequencer_drop_copy(wmDrag *drag, wmDropBox *drop)
 static void sequencer_dropboxes(void)
 {
 	ListBase *lb = WM_dropboxmap_find("Sequencer", SPACE_SEQ, RGN_TYPE_WINDOW);
-	
+
 	WM_dropbox_add(lb, "SEQUENCER_OT_image_strip_add", image_drop_poll, sequencer_drop_copy);
 	WM_dropbox_add(lb, "SEQUENCER_OT_movie_strip_add", movie_drop_poll, sequencer_drop_copy);
 	WM_dropbox_add(lb, "SEQUENCER_OT_sound_strip_add", sound_drop_poll, sequencer_drop_copy);
@@ -452,32 +442,53 @@ static int sequencer_context(const bContext *C, const char *member, bContextData
 	if (CTX_data_dir(member)) {
 		CTX_data_dir_set(result, sequencer_context_dir);
 
-		return TRUE;
+		return true;
 	}
 	else if (CTX_data_equals(member, "edit_mask")) {
 		Mask *mask = BKE_sequencer_mask_get(scene);
 		if (mask) {
 			CTX_data_id_pointer_set(result, &mask->id);
 		}
-		return TRUE;
+		return true;
 	}
 
-	return FALSE;
+	return false;
 }
 
-
+/* *********************** sequencer (main) region ************************ */
 /* add handlers, stuff you only do once or on area/region changes */
-static void sequencer_header_area_init(wmWindowManager *UNUSED(wm), ARegion *ar)
+static void sequencer_main_area_init(wmWindowManager *wm, ARegion *ar)
 {
-	ED_region_header_init(ar);
+	wmKeyMap *keymap;
+	ListBase *lb;
+
+	UI_view2d_region_reinit(&ar->v2d, V2D_COMMONVIEW_CUSTOM, ar->winx, ar->winy);
+
+#if 0
+	keymap = WM_keymap_find(wm->defaultconf, "Mask Editing", 0, 0);
+	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
+#endif
+
+	keymap = WM_keymap_find(wm->defaultconf, "SequencerCommon", SPACE_SEQ, 0);
+	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
+
+	/* own keymap */
+	keymap = WM_keymap_find(wm->defaultconf, "Sequencer", SPACE_SEQ, 0);
+	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
+
+	/* add drop boxes */
+	lb = WM_dropboxmap_find("Sequencer", SPACE_SEQ, RGN_TYPE_WINDOW);
+
+	WM_event_add_dropbox_handler(&ar->handlers, lb);
 }
 
-static void sequencer_header_area_draw(const bContext *C, ARegion *ar)
+static void sequencer_main_area_draw(const bContext *C, ARegion *ar)
 {
-	ED_region_header(C, ar);
+	/* NLE - strip editing timeline interface */
+	draw_timeline_seq(C, ar);
 }
 
-static void sequencer_main_area_listener(ARegion *ar, wmNotifier *wmn)
+static void sequencer_main_area_listener(bScreen *UNUSED(sc), ScrArea *UNUSED(sa), ARegion *ar, wmNotifier *wmn)
 {
 	/* context changes */
 	switch (wmn->category) {
@@ -488,6 +499,7 @@ static void sequencer_main_area_listener(ARegion *ar, wmNotifier *wmn)
 				case ND_MARKERS:
 				case ND_RENDER_OPTIONS: /* for FPS and FPS Base */
 				case ND_SEQUENCER:
+				case ND_RENDER_RESULT:
 					ED_region_tag_redraw(ar);
 					break;
 			}
@@ -507,15 +519,29 @@ static void sequencer_main_area_listener(ARegion *ar, wmNotifier *wmn)
 	}
 }
 
+/* *********************** header region ************************ */
+/* add handlers, stuff you only do once or on area/region changes */
+static void sequencer_header_area_init(wmWindowManager *UNUSED(wm), ARegion *ar)
+{
+	ED_region_header_init(ar);
+}
+
+static void sequencer_header_area_draw(const bContext *C, ARegion *ar)
+{
+	ED_region_header(C, ar);
+}
+
 /* *********************** preview region ************************ */
 static void sequencer_preview_area_init(wmWindowManager *wm, ARegion *ar)
 {
 	wmKeyMap *keymap;
 
 	UI_view2d_region_reinit(&ar->v2d, V2D_COMMONVIEW_CUSTOM, ar->winx, ar->winy);
-	
-//	keymap = WM_keymap_find(wm->defaultconf, "Mask Editing", 0, 0);
-//	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
+
+#if 0
+	keymap = WM_keymap_find(wm->defaultconf, "Mask Editing", 0, 0);
+	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
+#endif
 
 	keymap = WM_keymap_find(wm->defaultconf, "SequencerCommon", SPACE_SEQ, 0);
 	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
@@ -530,13 +556,14 @@ static void sequencer_preview_area_draw(const bContext *C, ARegion *ar)
 	ScrArea *sa = CTX_wm_area(C);
 	SpaceSeq *sseq = sa->spacedata.first;
 	Scene *scene = CTX_data_scene(C);
+	wmWindowManager *wm = CTX_wm_manager(C);
 	int show_split = scene->ed && scene->ed->over_flag & SEQ_EDIT_OVERLAY_SHOW && sseq->mainb == SEQ_DRAW_IMG_IMBUF;
-	
+
 	/* XXX temp fix for wrong setting in sseq->mainb */
 	if (sseq->mainb == SEQ_DRAW_SEQUENCE) sseq->mainb = SEQ_DRAW_IMG_IMBUF;
 
 	if (!show_split || sseq->overlay_type != SEQ_DRAW_OVERLAY_REFERENCE)
-		draw_image_seq(C, scene, ar, sseq, scene->r.cfra, 0, FALSE);
+		draw_image_seq(C, scene, ar, sseq, scene->r.cfra, 0, false, false);
 
 	if (show_split && sseq->overlay_type != SEQ_DRAW_OVERLAY_CURRENT) {
 		int over_cfra;
@@ -547,17 +574,22 @@ static void sequencer_preview_area_draw(const bContext *C, ARegion *ar)
 			over_cfra = scene->r.cfra + scene->ed->over_ofs;
 
 		if (over_cfra != scene->r.cfra || sseq->overlay_type != SEQ_DRAW_OVERLAY_RECT)
-			draw_image_seq(C, scene, ar, sseq, scene->r.cfra, over_cfra - scene->r.cfra, TRUE);
+			draw_image_seq(C, scene, ar, sseq, scene->r.cfra, over_cfra - scene->r.cfra, true, false);
 	}
 
+	if ((U.uiflag & USER_SHOW_FPS) && ED_screen_animation_playing(wm)) {
+		rcti rect;
+		ED_region_visible_rect(ar, &rect);
+		ED_scene_draw_fps(scene, &rect);
+	}
 }
 
-static void sequencer_preview_area_listener(ARegion *ar, wmNotifier *wmn)
+static void sequencer_preview_area_listener(bScreen *UNUSED(sc), ScrArea *UNUSED(sa), ARegion *ar, wmNotifier *wmn)
 {
 	/* context changes */
 	switch (wmn->category) {
 		case NC_GPENCIL:
-			if (wmn->action == NA_EDITED) {
+			if (ELEM(wmn->action, NA_EDITED, NA_SELECTED)) {
 				ED_region_tag_redraw(ar);
 			}
 			break;
@@ -582,7 +614,6 @@ static void sequencer_preview_area_listener(ARegion *ar, wmNotifier *wmn)
 					break;
 			}
 			break;
-
 		case NC_MASK:
 			if (wmn->action == NA_EDITED) {
 				ED_region_tag_redraw(ar);
@@ -596,9 +627,12 @@ static void sequencer_preview_area_listener(ARegion *ar, wmNotifier *wmn)
 /* add handlers, stuff you only do once or on area/region changes */
 static void sequencer_buttons_area_init(wmWindowManager *wm, ARegion *ar)
 {
-	
+	wmKeyMap *keymap;
+
+	keymap = WM_keymap_find(wm->defaultconf, "SequencerCommon", SPACE_SEQ, 0);
+	WM_event_add_keymap_handler_bb(&ar->handlers, keymap, &ar->v2d.mask, &ar->winrct);
+
 	ED_region_panels_init(wm, ar);
-	
 }
 
 static void sequencer_buttons_area_draw(const bContext *C, ARegion *ar)
@@ -606,12 +640,12 @@ static void sequencer_buttons_area_draw(const bContext *C, ARegion *ar)
 	ED_region_panels(C, ar, 1, NULL, -1);
 }
 
-static void sequencer_buttons_area_listener(ARegion *ar, wmNotifier *wmn)
+static void sequencer_buttons_area_listener(bScreen *UNUSED(sc), ScrArea *UNUSED(sa), ARegion *ar, wmNotifier *wmn)
 {
 	/* context changes */
 	switch (wmn->category) {
 		case NC_GPENCIL:
-			if (wmn->data == ND_DATA) {
+			if (ELEM(wmn->action, NA_EDITED, NA_SELECTED)) {
 				ED_region_tag_redraw(ar);
 			}
 			break;
@@ -640,10 +674,10 @@ void ED_spacetype_sequencer(void)
 {
 	SpaceType *st = MEM_callocN(sizeof(SpaceType), "spacetype sequencer");
 	ARegionType *art;
-	
+
 	st->spaceid = SPACE_SEQ;
 	strncpy(st->name, "Sequencer", BKE_ST_MAXNAME);
-	
+
 	st->new = sequencer_new;
 	st->free = sequencer_free;
 	st->init = sequencer_init;
@@ -668,13 +702,12 @@ void ED_spacetype_sequencer(void)
 	/* preview */
 	art = MEM_callocN(sizeof(ARegionType), "spacetype sequencer region");
 	art->regionid = RGN_TYPE_PREVIEW;
-	art->prefsizey = 240; // XXX
 	art->init = sequencer_preview_area_init;
 	art->draw = sequencer_preview_area_draw;
 	art->listener = sequencer_preview_area_listener;
 	art->keymapflag = ED_KEYMAP_VIEW2D | ED_KEYMAP_FRAMES | ED_KEYMAP_GPENCIL;
 	BLI_addhead(&st->regiontypes, art);
-	
+
 	/* regions: listview/buttons */
 	art = MEM_callocN(sizeof(ARegionType), "spacetype sequencer region");
 	art->regionid = RGN_TYPE_UI;
@@ -684,7 +717,7 @@ void ED_spacetype_sequencer(void)
 	art->init = sequencer_buttons_area_init;
 	art->draw = sequencer_buttons_area_draw;
 	BLI_addhead(&st->regiontypes, art);
-	
+
 	sequencer_buttons_register(art);
 
 	/* regions: header */
@@ -692,13 +725,13 @@ void ED_spacetype_sequencer(void)
 	art->regionid = RGN_TYPE_HEADER;
 	art->prefsizey = HEADERY;
 	art->keymapflag = ED_KEYMAP_UI | ED_KEYMAP_VIEW2D | ED_KEYMAP_FRAMES | ED_KEYMAP_HEADER;
-	
+
 	art->init = sequencer_header_area_init;
 	art->draw = sequencer_header_area_draw;
 	art->listener = sequencer_main_area_listener;
-	
+
 	BLI_addhead(&st->regiontypes, art);
-	
+
 	BKE_spacetype_register(st);
 
 	/* set the sequencer callback when not in background mode */
