@@ -50,15 +50,15 @@ void draw_text_main(struct SpaceText *st, struct ARegion *ar);
 void text_update_line_edited(struct TextLine *line);
 void text_update_edited(struct Text *text);
 void text_update_character_width(struct SpaceText *st);
-void text_scroll_to_cursor(struct SpaceText *st, struct ScrArea *sa);
+void text_scroll_to_cursor(struct SpaceText *st, struct ARegion *ar, bool center);
+void text_scroll_to_cursor__area(struct SpaceText *st, struct ScrArea *sa, bool center);
 void text_update_cursor_moved(struct bContext *C);
 
-	/* TXT_OFFSET used to be 35 when the scrollbar was on the left... */
-#define TXT_OFFSET 15
-#define TXT_SCROLL_WIDTH 20
-#define TXT_SCROLL_SPACE 2
-
-#define TEXTXLOC		(st->cwidth * st->linenrs_tot)
+#define TXT_OFFSET			((int)(0.5f * U.widget_unit))
+#define TXT_SCROLL_WIDTH	U.widget_unit
+#define TXT_SCROLL_SPACE	((int)(0.1f * U.widget_unit))
+#define TXT_LINE_SPACING	((int)(0.3f * st->lheight_dpi)) /* space between lines */
+#define TEXTXLOC			(st->cwidth * st->linenrs_tot)
 
 #define SUGG_LIST_SIZE	7
 #define SUGG_LIST_WIDTH	20
@@ -68,21 +68,6 @@ void text_update_cursor_moved(struct bContext *C);
 #define TOOL_SUGG_LIST	0x01
 #define TOOL_DOCUMENT	0x02
 
-#define TMARK_GRP_CUSTOM	0x00010000	/* Lower 2 bytes used for Python groups */
-#define TMARK_GRP_FINDALL	0x00020000
-
-typedef struct FlattenString {
-	char fixedbuf[256];
-	int fixedaccum[256];
-
-	char *buf;
-	int *accum;
-	int pos, len;
-} FlattenString;
-
-int flatten_string(struct SpaceText *st, FlattenString *fs, const char *in);
-void flatten_string_free(FlattenString *fs);
-
 int wrap_width(struct SpaceText *st, struct ARegion *ar);
 void wrap_offset(struct SpaceText *st, struct ARegion *ar, struct TextLine *linein, int cursin, int *offl, int *offc);
 void wrap_offset_in_line(struct SpaceText *st, struct ARegion *ar, struct TextLine *linep, int cursin, int *offl, int *offc);
@@ -91,8 +76,6 @@ int text_get_char_pos(struct SpaceText *st, const char *line, int cur);
 void text_drawcache_tag_update(struct SpaceText *st, int full);
 void text_free_caches(struct SpaceText *st);
 
-int text_file_modified(struct Text *text);
-
 int text_do_suggest_select(struct SpaceText *st, struct ARegion *ar);
 void text_pop_suggest_list(void);
 
@@ -129,10 +112,6 @@ void TEXT_OT_indent(struct wmOperatorType *ot);
 void TEXT_OT_line_break(struct wmOperatorType *ot);
 void TEXT_OT_insert(struct wmOperatorType *ot);
 
-void TEXT_OT_markers_clear(struct wmOperatorType *ot);
-void TEXT_OT_next_marker(struct wmOperatorType *ot);
-void TEXT_OT_previous_marker(struct wmOperatorType *ot);
-
 void TEXT_OT_select_line(struct wmOperatorType *ot);
 void TEXT_OT_select_all(struct wmOperatorType *ot);
 void TEXT_OT_select_word(struct wmOperatorType *ot);
@@ -153,16 +132,24 @@ void TEXT_OT_line_number(struct wmOperatorType *ot);
 
 void TEXT_OT_properties(struct wmOperatorType *ot);
 
+/* find = find indicated text */
 void TEXT_OT_find(struct wmOperatorType *ot);
 void TEXT_OT_find_set_selected(struct wmOperatorType *ot);
 void TEXT_OT_replace(struct wmOperatorType *ot);
 void TEXT_OT_replace_set_selected(struct wmOperatorType *ot);
-void TEXT_OT_mark_all(struct wmOperatorType *ot);
+
+/* text_find = open properties, activate search button */
+void TEXT_OT_start_find(struct wmOperatorType *ot);
 
 void TEXT_OT_to_3d_object(struct wmOperatorType *ot);
 
 void TEXT_OT_resolve_conflict(struct wmOperatorType *ot);
 
+int text_space_edit_poll(struct bContext *C);
+
+/* text_autocomplete.c */
+void TEXT_OT_autocomplete(struct wmOperatorType *ot);
+
 /* space_text.c */
 extern const char *text_context_dir[]; /* doc access */
 
