@@ -32,6 +32,7 @@
 #include <string.h>
 #include <stdio.h>
 
+#include "DNA_gpencil_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 
@@ -42,7 +43,6 @@
 #include "BLI_utildefines.h"
 
 #include "BKE_context.h"
-#include "BKE_global.h"
 #include "BKE_screen.h"
 #include "BKE_pointcache.h"
 
@@ -311,6 +311,9 @@ static void time_draw_idblock_keyframes(View2D *v2d, ID *id, short onlysel)
 		case ID_OB:
 			ob_to_keylist(&ads, (Object *)id, &keys, NULL);
 			break;
+		case ID_GD:
+			gpencil_to_keylist(&ads, (bGPdata *)id, &keys);
+			break;
 	}
 		
 	/* build linked-list for searching */
@@ -336,18 +339,25 @@ static void time_draw_idblock_keyframes(View2D *v2d, ID *id, short onlysel)
 }
 
 /* draw keyframe lines for timeline */
-static void time_draw_keyframes(const bContext *C, SpaceTime *stime, ARegion *ar)
+static void time_draw_keyframes(const bContext *C, ARegion *ar)
 {
 	Scene *scene = CTX_data_scene(C);
 	Object *ob = CTX_data_active_object(C);
+	bGPdata *gpd = CTX_data_gpencil_data(C);
 	View2D *v2d = &ar->v2d;
-	short onlysel = (stime->flag & TIME_ONLYACTSEL);
+	bool onlysel = ((scene->flag & SCE_KEYS_NO_SELONLY) == 0);
+	
+	/* draw grease pencil keyframes (if available) */
+	if (gpd) {
+		glColor3ub(0xB5, 0xE6, 0x1D);
+		time_draw_idblock_keyframes(v2d, (ID *)gpd, onlysel);
+	}
 	
 	/* draw scene keyframes first 
 	 *	- don't try to do this when only drawing active/selected data keyframes,
 	 *	  since this can become quite slow
 	 */
-	if (scene && onlysel == 0) {
+	if (onlysel == 0) {
 		/* set draw color */
 		glColor3ub(0xDD, 0xA7, 0x00);
 		time_draw_idblock_keyframes(v2d, (ID *)scene, onlysel);
@@ -365,7 +375,7 @@ static void time_draw_keyframes(const bContext *C, SpaceTime *stime, ARegion *ar
 		time_draw_idblock_keyframes(v2d, (ID *)ob, onlysel);
 	}
 	else {
-		short active_done = FALSE;
+		bool active_done = false;
 		
 		/* draw keyframes from all selected objects */
 		CTX_DATA_BEGIN (C, Object *, obsel, selected_objects)
@@ -375,7 +385,7 @@ static void time_draw_keyframes(const bContext *C, SpaceTime *stime, ARegion *ar
 			
 			/* if this object is the active one, set flag so that we don't draw again */
 			if (obsel == ob)
-				active_done = TRUE;
+				active_done = true;
 		}
 		CTX_DATA_END;
 		
@@ -406,6 +416,7 @@ static void time_listener(bScreen *UNUSED(sc), ScrArea *sa, wmNotifier *wmn)
 		case NC_OBJECT:
 		{
 			switch (wmn->data) {
+				case ND_BONE_SELECT:
 				case ND_BONE_ACTIVE:
 				case ND_POINTCACHE:
 				case ND_MODIFIER:
@@ -516,11 +527,11 @@ static void time_main_area_draw(const bContext *C, ARegion *ar)
 	UI_view2d_view_ortho(v2d);
 	
 	/* keyframes */
-	time_draw_keyframes(C, stime, ar);
+	time_draw_keyframes(C, ar);
 	
 	/* markers */
 	UI_view2d_view_orthoSpecial(ar, v2d, 1);
-	draw_markers_time(C, 0);
+	ED_markers_draw(C, 0);
 	
 	/* caches */
 	time_draw_cache(stime, obact, scene);
