@@ -34,7 +34,6 @@
 #include <string.h>
 #include <math.h>
 
-
 #include "DNA_anim_types.h"
 #include "DNA_armature_types.h"
 #include "DNA_constraint_types.h"
@@ -62,12 +61,12 @@
 #include "ED_armature.h"
 #include "ED_keyframes_draw.h"
 
-#include "BLF_api.h"
 
 #include "UI_resources.h"
 
 #include "view3d_intern.h"
 
+#include "GPU_select.h"
 
 /* *************** Armature Drawing - Coloring API ***************************** */
 
@@ -94,13 +93,13 @@ static void set_pchan_colorset(Object *ob, bPoseChannel *pchan)
 	short color_index = 0;
 	
 	/* sanity check */
-	if (ELEM4(NULL, ob, arm, pose, pchan)) {
+	if (ELEM(NULL, ob, arm, pose, pchan)) {
 		bcolor = NULL;
 		return;
 	}
 	
 	/* only try to set custom color if enabled for armature */
-	if (arm->flag & ARM_COL_CUSTOM) {	
+	if (arm->flag & ARM_COL_CUSTOM) {
 		/* currently, a bone can only use a custom color set if it's group (if it has one),
 		 * has been set to use one
 		 */
@@ -144,7 +143,7 @@ static void cp_shade_color3ub(unsigned char cp[3], const int offset)
 }
 
 /* This function sets the gl-color for coloring a certain bone (based on bcolor) */
-static short set_pchan_glColor(short colCode, int boneflag, short constflag)
+static bool set_pchan_glColor(short colCode, int boneflag, short constflag)
 {
 	switch (colCode) {
 		case PCHAN_COLOR_NORMAL:
@@ -184,10 +183,8 @@ static short set_pchan_glColor(short colCode, int boneflag, short constflag)
 				}
 			}
 	
-			return 1;
+			return true;
 		}
-		break;
-		
 		case PCHAN_COLOR_SOLID:
 		{
 			if (bcolor) {
@@ -196,10 +193,8 @@ static short set_pchan_glColor(short colCode, int boneflag, short constflag)
 			else
 				UI_ThemeColor(TH_BONE_SOLID);
 			
-			return 1;
+			return true;
 		}
-		break;
-		
 		case PCHAN_COLOR_CONSTS:
 		{
 			if ((bcolor == NULL) || (bcolor->flag & TH_WIRECOLOR_CONSTCOLS)) {
@@ -208,13 +203,10 @@ static short set_pchan_glColor(short colCode, int boneflag, short constflag)
 				else if (constflag & PCHAN_HAS_SPLINEIK) glColor4ub(200, 255, 0, 80);
 				else if (constflag & PCHAN_HAS_CONST) glColor4ub(0, 255, 120, 80);
 			
-				return 1;
+				return true;
 			}
-			else
-				return 0;
+			return false;
 		}
-		break;
-
 		case PCHAN_COLOR_SPHEREBONE_BASE:
 		{
 			if (bcolor) {
@@ -238,9 +230,8 @@ static short set_pchan_glColor(short colCode, int boneflag, short constflag)
 				else UI_ThemeColor(TH_BONE_SOLID);
 			}
 			
-			return 1;
+			return true;
 		}
-		break;
 		case PCHAN_COLOR_SPHEREBONE_END:
 		{
 			if (bcolor) {
@@ -266,9 +257,8 @@ static short set_pchan_glColor(short colCode, int boneflag, short constflag)
 				else if (boneflag & BONE_SELECTED) UI_ThemeColorShade(TH_BONE_POSE, -30);
 				else UI_ThemeColorShade(TH_BONE_SOLID, -30);
 			}
+			break;
 		}
-		break;
-		
 		case PCHAN_COLOR_LINEBONE:
 		{
 			/* inner part in background color or constraint */
@@ -281,19 +271,18 @@ static short set_pchan_glColor(short colCode, int boneflag, short constflag)
 			}
 			else {
 				if (bcolor) {
-					char *cp = bcolor->solid;
+					const char *cp = bcolor->solid;
 					glColor4ub(cp[0], cp[1], cp[2], 204);
 				}
 				else
 					UI_ThemeColorShade(TH_BACK, -30);
 			}
 		
-			return 1;
+			return true;
 		}
-		break;
 	}
 	
-	return 0;
+	return false;
 }
 
 static void set_ebone_glColor(const unsigned int boneflag)
@@ -302,20 +291,20 @@ static void set_ebone_glColor(const unsigned int boneflag)
 		UI_ThemeColor(TH_EDGE_SELECT);
 	}
 	else if (boneflag & BONE_DRAW_ACTIVE) {
-		UI_ThemeColorBlend(TH_WIRE, TH_EDGE_SELECT, 0.15f); /* unselected active */
+		UI_ThemeColorBlend(TH_WIRE_EDIT, TH_EDGE_SELECT, 0.15f); /* unselected active */
 	}
 	else if (boneflag & BONE_SELECTED) {
 		UI_ThemeColorShade(TH_EDGE_SELECT, -20);
 	}
 	else {
-		UI_ThemeColor(TH_WIRE);
+		UI_ThemeColor(TH_WIRE_EDIT);
 	}
 }
 
 /* *************** Armature drawing, helper calls for parts ******************* */
 
 /* half the cube, in Y */
-static float cube[8][3] = {
+static const float cube[8][3] = {
 	{-1.0,  0.0, -1.0},
 	{-1.0,  0.0,  1.0},
 	{-1.0,  1.0,  1.0},
@@ -451,7 +440,7 @@ static void draw_bonevert_solid(void)
 	glCallList(displist);
 }
 
-static float bone_octahedral_verts[6][3] = {
+static const float bone_octahedral_verts[6][3] = {
 	{ 0.0f, 0.0f,  0.0f},
 	{ 0.1f, 0.1f,  0.1f},
 	{ 0.1f, 0.1f, -0.1f},
@@ -460,10 +449,10 @@ static float bone_octahedral_verts[6][3] = {
 	{ 0.0f, 1.0f,  0.0f}
 };
 
-static unsigned int bone_octahedral_wire_sides[8] = {0, 1, 5, 3, 0, 4, 5, 2};
-static unsigned int bone_octahedral_wire_square[8] = {1, 2, 3, 4, 1};
+static const unsigned int bone_octahedral_wire_sides[8] = {0, 1, 5, 3, 0, 4, 5, 2};
+static const unsigned int bone_octahedral_wire_square[8] = {1, 2, 3, 4, 1};
 
-static unsigned int bone_octahedral_solid_tris[8][3] = {
+static const unsigned int bone_octahedral_solid_tris[8][3] = {
 	{2, 1, 0}, /* bottom */
 	{3, 2, 0},
 	{4, 3, 0},
@@ -476,11 +465,11 @@ static unsigned int bone_octahedral_solid_tris[8][3] = {
 };
 
 /* aligned with bone_octahedral_solid_tris */
-static float bone_octahedral_solid_normals[8][3] = {
-	{ 0.70710683f, -0.70710683f,  0.00000000f},
-	{-0.00000000f, -0.70710683f, -0.70710683f},
-	{-0.70710683f, -0.70710683f,  0.00000000f},
-	{ 0.00000000f, -0.70710683f,  0.70710683f},
+static const float bone_octahedral_solid_normals[8][3] = {
+	{ M_SQRT1_2,   -M_SQRT1_2,    0.00000000f},
+	{-0.00000000f, -M_SQRT1_2,   -M_SQRT1_2},
+	{-M_SQRT1_2,   -M_SQRT1_2,    0.00000000f},
+	{ 0.00000000f, -M_SQRT1_2,    M_SQRT1_2},
 	{ 0.99388373f,  0.11043154f, -0.00000000f},
 	{ 0.00000000f,  0.11043154f, -0.99388373f},
 	{-0.99388373f,  0.11043154f,  0.00000000f},
@@ -562,7 +551,7 @@ static void draw_bone_points(const short dt, int armflag, unsigned int boneflag,
 	/*	Draw root point if we are not connected */
 	if ((boneflag & BONE_CONNECTED) == 0) {
 		if (id != -1)
-			glLoadName(id | BONESEL_ROOT);
+			GPU_select_load_id(id | BONESEL_ROOT);
 		
 		if (dt <= OB_WIRE) {
 			if (armflag & ARM_EDITMODE) {
@@ -585,7 +574,7 @@ static void draw_bone_points(const short dt, int armflag, unsigned int boneflag,
 	
 	/*	Draw tip point */
 	if (id != -1)
-		glLoadName(id | BONESEL_TIP);
+		GPU_select_load_id(id | BONESEL_TIP);
 	
 	if (dt <= OB_WIRE) {
 		if (armflag & ARM_EDITMODE) {
@@ -610,7 +599,7 @@ static void draw_bone_points(const short dt, int armflag, unsigned int boneflag,
 }
 
 /* 16 values of sin function (still same result!) */
-static float si[16] = {
+static const float si[16] = {
 	0.00000000f,
 	0.20129852f, 0.39435585f,
 	0.57126821f, 0.72479278f,
@@ -622,7 +611,7 @@ static float si[16] = {
 	0.10116832f
 };
 /* 16 values of cos function (still same result!) */
-static float co[16] = {
+static const float co[16] = {
 	1.00000000f,
 	0.97952994f, 0.91895781f,
 	0.82076344f, 0.68896691f,
@@ -637,7 +626,7 @@ static float co[16] = {
 
 
 /* smat, imat = mat & imat to draw screenaligned */
-static void draw_sphere_bone_dist(float smat[][4], float imat[][4], bPoseChannel *pchan, EditBone *ebone)
+static void draw_sphere_bone_dist(float smat[4][4], float imat[4][4], bPoseChannel *pchan, EditBone *ebone)
 {
 	float head, tail, dist /*, length*/;
 	float *headvec, *tailvec, dirvec[3];
@@ -647,7 +636,7 @@ static void draw_sphere_bone_dist(float smat[][4], float imat[][4], bPoseChannel
 		/* this routine doesn't call get_matrix_editbone() that calculates it */
 		ebone->length = len_v3v3(ebone->head, ebone->tail);
 
-		/*length= ebone->length;*/ /*UNUSED*/
+		/*length = ebone->length;*/ /*UNUSED*/
 		tail = ebone->rad_tail;
 		dist = ebone->dist;
 		if (ebone->parent && (ebone->flag & BONE_CONNECTED))
@@ -658,7 +647,7 @@ static void draw_sphere_bone_dist(float smat[][4], float imat[][4], bPoseChannel
 		tailvec = ebone->tail;
 	}
 	else {
-		/*length= pchan->bone->length;*/ /*UNUSED*/
+		/*length = pchan->bone->length;*/ /*UNUSED*/
 		tail = pchan->bone->rad_tail;
 		dist = pchan->bone->dist;
 		if (pchan->parent && (pchan->bone->flag & BONE_CONNECTED))
@@ -755,7 +744,7 @@ static void draw_sphere_bone_dist(float smat[][4], float imat[][4], bPoseChannel
 
 
 /* smat, imat = mat & imat to draw screenaligned */
-static void draw_sphere_bone_wire(float smat[][4], float imat[][4],
+static void draw_sphere_bone_wire(float smat[4][4], float imat[4][4],
                                   int armflag, int boneflag, short constflag, unsigned int id,
                                   bPoseChannel *pchan, EditBone *ebone)
 {
@@ -767,7 +756,7 @@ static void draw_sphere_bone_wire(float smat[][4], float imat[][4],
 		/* this routine doesn't call get_matrix_editbone() that calculates it */
 		ebone->length = len_v3v3(ebone->head, ebone->tail);
 		
-		/*length= ebone->length;*/ /*UNUSED*/
+		/*length = ebone->length;*/ /*UNUSED*/
 		tail = ebone->rad_tail;
 		if (ebone->parent && (boneflag & BONE_CONNECTED))
 			head = ebone->parent->rad_tail;
@@ -777,7 +766,7 @@ static void draw_sphere_bone_wire(float smat[][4], float imat[][4],
 		tailvec = ebone->tail;
 	}
 	else {
-		/*length= pchan->bone->length;*/ /*UNUSED*/
+		/*length = pchan->bone->length;*/ /*UNUSED*/
 		tail = pchan->bone->rad_tail;
 		if ((pchan->parent) && (boneflag & BONE_CONNECTED))
 			head = pchan->parent->bone->rad_tail;
@@ -798,7 +787,7 @@ static void draw_sphere_bone_wire(float smat[][4], float imat[][4],
 	/*	Draw root point if we are not connected */
 	if ((boneflag & BONE_CONNECTED) == 0) {
 		if (id != -1)
-			glLoadName(id | BONESEL_ROOT);
+			GPU_select_load_id(id | BONESEL_ROOT);
 		
 		drawcircball(GL_LINE_LOOP, headvec, head, imat);
 	}
@@ -810,14 +799,14 @@ static void draw_sphere_bone_wire(float smat[][4], float imat[][4],
 	}
 	
 	if (id != -1)
-		glLoadName(id | BONESEL_TIP);
+		GPU_select_load_id(id | BONESEL_TIP);
 	
 	drawcircball(GL_LINE_LOOP, tailvec, tail, imat);
 	
 	/* base */
 	if (armflag & ARM_EDITMODE) {
 		if (boneflag & BONE_SELECTED) UI_ThemeColor(TH_SELECT);
-		else UI_ThemeColor(TH_WIRE);
+		else UI_ThemeColor(TH_WIRE_EDIT);
 	}
 	
 	sub_v3_v3v3(dirvec, tailvec, headvec);
@@ -841,7 +830,7 @@ static void draw_sphere_bone_wire(float smat[][4], float imat[][4],
 		cross_v3_v3v3(norvect, vec, imat[2]);
 		
 		if (id != -1)
-			glLoadName(id | BONESEL_BONE);
+			GPU_select_load_id(id | BONESEL_BONE);
 		
 		glBegin(GL_LINES);
 
@@ -918,7 +907,7 @@ static void draw_sphere_bone(const short dt, int armflag, int boneflag, short co
 	/*	Draw root point if we are not connected */
 	if ((boneflag & BONE_CONNECTED) == 0) {
 		if (id != -1)
-			glLoadName(id | BONESEL_ROOT);
+			GPU_select_load_id(id | BONESEL_ROOT);
 		gluSphere(qobj, head, 16, 10);
 	}
 	
@@ -929,7 +918,7 @@ static void draw_sphere_bone(const short dt, int armflag, int boneflag, short co
 	}
 
 	if (id != -1)
-		glLoadName(id | BONESEL_TIP);
+		GPU_select_load_id(id | BONESEL_TIP);
 	
 	glTranslatef(0.0f, 0.0f, length);
 	gluSphere(qobj, tail, 16, 10);
@@ -950,7 +939,7 @@ static void draw_sphere_bone(const short dt, int armflag, int boneflag, short co
 	
 	if (length > (head + tail)) {
 		if (id != -1)
-			glLoadName(id | BONESEL_BONE);
+			GPU_select_load_id(id | BONESEL_BONE);
 		
 		glEnable(GL_POLYGON_OFFSET_FILL);
 		glPolygonOffset(-1.0f, -1.0f);
@@ -969,7 +958,7 @@ static void draw_sphere_bone(const short dt, int armflag, int boneflag, short co
 		glTranslatef(0.0f, 0.0f, head);
 		gluSphere(qobj, fac1 * head + (1.0f - fac1) * tail, 16, 10);
 	}
-	else {		
+	else {
 		/* 1 sphere in center */
 		glTranslatef(0.0f, 0.0f, (head + length - tail) / 2.0f);
 		gluSphere(qobj, fac1 * head + (1.0f - fac1) * tail, 16, 10);
@@ -1014,13 +1003,13 @@ static void draw_line_bone(int armflag, int boneflag, short constflag, unsigned
 		if (armflag & ARM_POSEMODE)
 			set_pchan_glColor(PCHAN_COLOR_NORMAL, boneflag, constflag);
 		else if (armflag & ARM_EDITMODE) {
-			UI_ThemeColor(TH_WIRE);
+			UI_ThemeColor(TH_WIRE_EDIT);
 		}
 		
 		/*	Draw root point if we are not connected */
 		if ((boneflag & BONE_CONNECTED) == 0) {
 			if (G.f & G_PICKSEL) {  /* no bitmap in selection mode, crashes 3d cards... */
-				glLoadName(id | BONESEL_ROOT);
+				GPU_select_load_id(id | BONESEL_ROOT);
 				glBegin(GL_POINTS);
 				glVertex3f(0.0f, 0.0f, 0.0f);
 				glEnd();
@@ -1032,7 +1021,7 @@ static void draw_line_bone(int armflag, int boneflag, short constflag, unsigned
 		}
 		
 		if (id != -1)
-			glLoadName((GLuint) id | BONESEL_BONE);
+			GPU_select_load_id((GLuint) id | BONESEL_BONE);
 		
 		glBegin(GL_LINES);
 		glVertex3f(0.0f, 0.0f, 0.0f);
@@ -1040,9 +1029,9 @@ static void draw_line_bone(int armflag, int boneflag, short constflag, unsigned
 		glEnd();
 		
 		/* tip */
-		if (G.f & G_PICKSEL) {	
+		if (G.f & G_PICKSEL) {
 			/* no bitmap in selection mode, crashes 3d cards... */
-			glLoadName(id | BONESEL_TIP);
+			GPU_select_load_id(id | BONESEL_TIP);
 			glBegin(GL_POINTS);
 			glVertex3f(0.0f, 1.0f, 0.0f);
 			glEnd();
@@ -1054,7 +1043,7 @@ static void draw_line_bone(int armflag, int boneflag, short constflag, unsigned
 		
 		/* further we send no names */
 		if (id != -1)
-			glLoadName(id & 0xFFFF);  /* object tag, for bordersel optim */
+			GPU_select_load_id(id & 0xFFFF);  /* object tag, for bordersel optim */
 		
 		if (armflag & ARM_POSEMODE)
 			set_pchan_glColor(PCHAN_COLOR_LINEBONE, boneflag, constflag);
@@ -1109,12 +1098,14 @@ static void draw_b_bone_boxes(const short dt, bPoseChannel *pchan, float xwidth,
 	
 	if ((segments > 1) && (pchan)) {
 		float dlen = length / (float)segments;
-		Mat4 *bbone = b_bone_spline_setup(pchan, 0);
+		Mat4 bbone[MAX_BBONE_SUBDIV];
 		int a;
-		
-		for (a = 0; a < segments; a++, bbone++) {
+
+		b_bone_spline_setup(pchan, 0, bbone);
+
+		for (a = 0; a < segments; a++) {
 			glPushMatrix();
-			glMultMatrixf(bbone->mat);
+			glMultMatrixf(bbone[a].mat);
 			if (dt == OB_SOLID) drawsolidcube_size(xwidth, dlen, zwidth);
 			else drawcube_size(xwidth, dlen, zwidth);
 			glPopMatrix();
@@ -1170,7 +1161,7 @@ static void draw_b_bone(const short dt, int armflag, int boneflag, short constfl
 	}
 	
 	if (id != -1) {
-		glLoadName((GLuint) id | BONESEL_BONE);
+		GPU_select_load_id((GLuint) id | BONESEL_BONE);
 	}
 	
 	/* set up solid drawing */
@@ -1189,7 +1180,7 @@ static void draw_b_bone(const short dt, int armflag, int boneflag, short constfl
 		glDisable(GL_COLOR_MATERIAL);
 		glDisable(GL_LIGHTING);
 	}
-	else {	
+	else {
 		/* wire */
 		if (armflag & ARM_POSEMODE) {
 			if (constflag) {
@@ -1205,9 +1196,9 @@ static void draw_b_bone(const short dt, int armflag, int boneflag, short constfl
 				/* restore colors */
 				set_pchan_glColor(PCHAN_COLOR_NORMAL, boneflag, constflag);
 			}
-		}		
+		}
 		
-		draw_b_bone_boxes(OB_WIRE, pchan, xwidth, length, zwidth);		
+		draw_b_bone_boxes(OB_WIRE, pchan, xwidth, length, zwidth);
 	}
 }
 
@@ -1245,6 +1236,7 @@ static void draw_wire_bone_segments(bPoseChannel *pchan, Mat4 *bbones, float len
 static void draw_wire_bone(const short dt, int armflag, int boneflag, short constflag, unsigned int id,
                            bPoseChannel *pchan, EditBone *ebone)
 {
+	Mat4 bbones_array[MAX_BBONE_SUBDIV];
 	Mat4 *bbones = NULL;
 	int segments = 0;
 	float length;
@@ -1253,8 +1245,10 @@ static void draw_wire_bone(const short dt, int armflag, int boneflag, short cons
 		segments = pchan->bone->segments;
 		length = pchan->bone->length;
 		
-		if (segments > 1)
-			bbones = b_bone_spline_setup(pchan, 0);
+		if (segments > 1) {
+			b_bone_spline_setup(pchan, 0, bbones_array);
+			bbones = bbones_array;
+		}
 	}
 	else 
 		length = ebone->length;
@@ -1272,13 +1266,13 @@ static void draw_wire_bone(const short dt, int armflag, int boneflag, short cons
 	/* this chunk not in object mode */
 	if (armflag & (ARM_EDITMODE | ARM_POSEMODE)) {
 		if (id != -1)
-			glLoadName((GLuint) id | BONESEL_BONE);
+			GPU_select_load_id((GLuint) id | BONESEL_BONE);
 		
 		draw_wire_bone_segments(pchan, bbones, length, segments);
 		
 		/* further we send no names */
 		if (id != -1)
-			glLoadName(id & 0xFFFF);    /* object tag, for bordersel optim */
+			GPU_select_load_id(id & 0xFFFF);    /* object tag, for bordersel optim */
 	}
 	
 	/* colors for modes */
@@ -1321,7 +1315,7 @@ static void draw_bone(const short dt, int armflag, int boneflag, short constflag
 	
 	/* now draw the bone itself */
 	if (id != -1) {
-		glLoadName((GLuint) id | BONESEL_BONE);
+		GPU_select_load_id((GLuint) id | BONESEL_BONE);
 	}
 	
 	/* wire? */
@@ -1333,7 +1327,7 @@ static void draw_bone(const short dt, int armflag, int boneflag, short constflag
 		else if (armflag & ARM_POSEMODE) {
 			if (constflag) {
 				/* draw constraint colors */
-				if (set_pchan_glColor(PCHAN_COLOR_CONSTS, boneflag, constflag)) {	
+				if (set_pchan_glColor(PCHAN_COLOR_CONSTS, boneflag, constflag)) {
 					glEnable(GL_BLEND);
 					
 					draw_bone_solid_octahedral();
@@ -1344,10 +1338,10 @@ static void draw_bone(const short dt, int armflag, int boneflag, short constflag
 				/* restore colors */
 				set_pchan_glColor(PCHAN_COLOR_NORMAL, boneflag, constflag);
 			}
-		}		
+		}
 		draw_bone_octahedral();
 	}
-	else {	
+	else {
 		/* solid */
 		if (armflag & ARM_POSEMODE)
 			set_pchan_glColor(PCHAN_COLOR_SOLID, boneflag, constflag);
@@ -1376,7 +1370,7 @@ static void draw_custom_bone(Scene *scene, View3D *v3d, RegionView3D *rv3d, Obje
 	}
 	
 	if (id != -1) {
-		glLoadName((GLuint) id | BONESEL_BONE);
+		GPU_select_load_id((GLuint) id | BONESEL_BONE);
 	}
 	
 	draw_object_instance(scene, v3d, rv3d, ob, dt, armflag & ARM_POSEMODE);
@@ -1426,8 +1420,8 @@ static void pchan_draw_IK_root_lines(bPoseChannel *pchan, short only_temp)
 				
 				glEnd();
 				setlinestyle(0);
+				break;
 			}
-			break;
 			case CONSTRAINT_TYPE_SPLINEIK: 
 			{
 				bSplineIKConstraint *data = (bSplineIKConstraint *)con->data;
@@ -1451,8 +1445,8 @@ static void pchan_draw_IK_root_lines(bPoseChannel *pchan, short only_temp)
 
 				glEnd();
 				setlinestyle(0);
+				break;
 			}
-			break;
 		}
 	}
 }
@@ -1462,7 +1456,7 @@ static void bgl_sphere_project(float ax, float az)
 	float dir[3], sine, q3;
 
 	sine = 1.0f - ax * ax - az * az;
-	q3 = (sine < 0.0f) ? 0.0f : (float)(2.0 * sqrt(sine));
+	q3 = (sine < 0.0f) ? 0.0f : (2.0f * sqrtf(sine));
 
 	dir[0] = -az * q3;
 	dir[1] = 1.0f - 2.0f * sine;
@@ -1473,7 +1467,7 @@ static void bgl_sphere_project(float ax, float az)
 
 static void draw_dof_ellipse(float ax, float az)
 {
-	static float staticSine[16] = {
+	const float staticSine[16] = {
 		0.0f, 0.104528463268f, 0.207911690818f, 0.309016994375f,
 		0.406736643076f, 0.5f, 0.587785252292f, 0.669130606359f,
 		0.743144825477f, 0.809016994375f, 0.866025403784f,
@@ -1495,7 +1489,7 @@ static void draw_dof_ellipse(float ax, float az)
 		z = staticSine[i];
 		
 		px = 0.0f;
-		for (j = 1; j < n - i + 1; j++) {
+		for (j = 1; j <= (n - i); j++) {
 			x = staticSine[j];
 			
 			if (j == n - i) {
@@ -1544,7 +1538,7 @@ static void draw_pose_dofs(Object *ob)
 			if (bone->flag & BONE_SELECTED) {
 				if (bone->layer & arm->layer) {
 					if (pchan->ikflag & (BONE_IK_XLIMIT | BONE_IK_ZLIMIT)) {
-						if (ED_pose_channel_in_IK_chain(ob, pchan)) {
+						if (BKE_pose_channel_in_IK_chain(ob, pchan)) {
 							float corner[4][3], posetrans[3], mat[4][4];
 							float phi = 0.0f, theta = 0.0f, scale;
 							int a, i;
@@ -1573,8 +1567,8 @@ static void draw_pose_dofs(Object *ob)
 									
 									for (i = 0; i < 3; i++) {
 										/* *0.5f here comes from M_PI/360.0f when rotations were still in degrees */
-										amin[i] = (float)sin(pchan->limitmin[i] * 0.5f);
-										amax[i] = (float)sin(pchan->limitmax[i] * 0.5f);
+										amin[i] = sinf(pchan->limitmin[i] * 0.5f);
+										amax[i] = sinf(pchan->limitmax[i] * 0.5f);
 									}
 									
 									glScalef(1.0f, -1.0f, 1.0f);
@@ -1605,8 +1599,8 @@ static void draw_pose_dofs(Object *ob)
 									phi = fac * (pchan->limitmax[2] - pchan->limitmin[2]);
 									
 									i = (a == -16) ? 0 : 1;
-									corner[i][0] = (float)sin(phi);
-									corner[i][1] = (float)cos(phi);
+									corner[i][0] = sinf(phi);
+									corner[i][1] = cosf(phi);
 									corner[i][2] = 0.0f;
 									glVertex3fv(corner[i]);
 								}
@@ -1629,8 +1623,8 @@ static void draw_pose_dofs(Object *ob)
 									
 									i = (a == -16) ? 2 : 3;
 									corner[i][0] = 0.0f;
-									corner[i][1] = (float)sin(phi);
-									corner[i][2] = (float)cos(phi);
+									corner[i][1] = sinf(phi);
+									corner[i][2] = cosf(phi);
 									glVertex3fv(corner[i]);
 								}
 								glEnd();
@@ -1648,7 +1642,7 @@ static void draw_pose_dofs(Object *ob)
 	}
 }
 
-static void bone_matrix_translate_y(float mat[][4], float y)
+static void bone_matrix_translate_y(float mat[4][4], float y)
 {
 	float trans[3];
 
@@ -1660,7 +1654,7 @@ static void bone_matrix_translate_y(float mat[][4], float y)
 /* assumes object is Armature with pose */
 static void draw_pose_bones(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
                             const short dt, const unsigned char ob_wire_col[4],
-                            const short do_const_color, const short is_outline)
+                            const bool do_const_color, const bool is_outline)
 {
 	RegionView3D *rv3d = ar->regiondata;
 	Object *ob = base->object;
@@ -1670,8 +1664,10 @@ static void draw_pose_bones(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
 	GLfloat tmp;
 	float smat[4][4], imat[4][4], bmat[4][4];
 	int index = -1;
-	short do_dashed = 3, draw_wire = FALSE;
+	short do_dashed = 3;
+	bool draw_wire = false;
 	int flag;
+	bool is_cull_enabled;
 	
 	/* being set below */
 	arm->layer_used = 0;
@@ -1719,9 +1715,15 @@ static void draw_pose_bones(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
 	}
 	
 	/* little speedup, also make sure transparent only draws once */
-	glCullFace(GL_BACK); 
-	glEnable(GL_CULL_FACE);
-	
+	glCullFace(GL_BACK);
+	if (v3d->flag2 & V3D_BACKFACE_CULLING) {
+		glEnable(GL_CULL_FACE);
+		is_cull_enabled = true;
+	}
+	else {
+		is_cull_enabled = false;
+	}
+
 	/* if solid we draw that first, with selection codes, but without names, axes etc */
 	if (dt > OB_WIRE) {
 		if (arm->flag & ARM_POSEMODE) 
@@ -1736,12 +1738,12 @@ static void draw_pose_bones(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
 			    ((G.f & G_PICKSEL) == 0 || (bone->flag & BONE_UNSELECTABLE) == 0))
 			{
 				if (bone->layer & arm->layer) {
-					int use_custom = (pchan->custom) && !(arm->flag & ARM_NO_CUSTOM);
+					const bool use_custom = (pchan->custom) && !(arm->flag & ARM_NO_CUSTOM);
 					glPushMatrix();
 					
 					if (use_custom && pchan->custom_tx) {
 						glMultMatrixf(pchan->custom_tx->pose_mat);
-					} 
+					}
 					else {
 						glMultMatrixf(pchan->pose_mat);
 					}
@@ -1767,29 +1769,41 @@ static void draw_pose_bones(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
 					if (use_custom) {
 						/* if drawwire, don't try to draw in solid */
 						if (pchan->bone->flag & BONE_DRAWWIRE) {
-							draw_wire = 1;
+							draw_wire = true;
 						}
 						else {
+							if (is_cull_enabled && (v3d->flag2 & V3D_BACKFACE_CULLING) == 0) {
+								is_cull_enabled = false;
+								glDisable(GL_CULL_FACE);
+							}
+
 							draw_custom_bone(scene, v3d, rv3d, pchan->custom,
 							                 OB_SOLID, arm->flag, flag, index, bone->length);
 						}
 					}
-					else if (arm->drawtype == ARM_LINE) {
-						/* nothing in solid */
-					}
-					else if (arm->drawtype == ARM_WIRE) {
-						/* nothing in solid */
-					}
-					else if (arm->drawtype == ARM_ENVELOPE) {
-						draw_sphere_bone(OB_SOLID, arm->flag, flag, 0, index, pchan, NULL);
-					}
-					else if (arm->drawtype == ARM_B_BONE) {
-						draw_b_bone(OB_SOLID, arm->flag, flag, 0, index, pchan, NULL);
-					}
 					else {
-						draw_bone(OB_SOLID, arm->flag, flag, 0, index, bone->length);
+						if (is_cull_enabled == false) {
+							is_cull_enabled = true;
+							glEnable(GL_CULL_FACE);
+						}
+
+						if (arm->drawtype == ARM_LINE) {
+							/* nothing in solid */
+						}
+						else if (arm->drawtype == ARM_WIRE) {
+							/* nothing in solid */
+						}
+						else if (arm->drawtype == ARM_ENVELOPE) {
+							draw_sphere_bone(OB_SOLID, arm->flag, flag, 0, index, pchan, NULL);
+						}
+						else if (arm->drawtype == ARM_B_BONE) {
+							draw_b_bone(OB_SOLID, arm->flag, flag, 0, index, pchan, NULL);
+						}
+						else {
+							draw_bone(OB_SOLID, arm->flag, flag, 0, index, bone->length);
+						}
 					}
-						
+
 					glPopMatrix();
 				}
 			}
@@ -1798,19 +1812,19 @@ static void draw_pose_bones(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
 				index += 0x10000;  /* pose bones count in higher 2 bytes only */
 		}
 		
-		/* very very confusing... but in object mode, solid draw, we cannot do glLoadName yet,
+		/* very very confusing... but in object mode, solid draw, we cannot do GPU_select_load_id yet,
 		 * stick bones and/or wire custom-shapes are drawn in next loop 
 		 */
-		if (ELEM(arm->drawtype, ARM_LINE, ARM_WIRE) == 0 && (draw_wire == 0)) {
+		if (ELEM(arm->drawtype, ARM_LINE, ARM_WIRE) == 0 && (draw_wire == false) && index != -1) {
 			/* object tag, for bordersel optim */
-			glLoadName(index & 0xFFFF);	
+			GPU_select_load_id(index & 0xFFFF);
 			index = -1;
 		}
 	}
 	
 	/* draw custom bone shapes as wireframes */
 	if (!(arm->flag & ARM_NO_CUSTOM) &&
-	    ((draw_wire) || (dt <= OB_WIRE)) )
+	    (draw_wire || (dt <= OB_WIRE)) )
 	{
 		if (arm->flag & ARM_POSEMODE)
 			index = base->selcol;
@@ -1830,7 +1844,7 @@ static void draw_pose_bones(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
 							
 							if (pchan->custom_tx) {
 								glMultMatrixf(pchan->custom_tx->pose_mat);
-							} 
+							}
 							else {
 								glMultMatrixf(pchan->pose_mat);
 							}
@@ -1867,9 +1881,9 @@ static void draw_pose_bones(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
 				index += 0x10000;  /* pose bones count in higher 2 bytes only */
 		}
 		/* stick or wire bones have not been drawn yet so don't clear object selection in this case */
-		if (ELEM(arm->drawtype, ARM_LINE, ARM_WIRE) == 0 && draw_wire) {
+		if (ELEM(arm->drawtype, ARM_LINE, ARM_WIRE) == 0 && draw_wire && index != -1) {
 			/* object tag, for bordersel optim */
-			glLoadName(index & 0xFFFF);	
+			GPU_select_load_id(index & 0xFFFF);
 			index = -1;
 		}
 	}
@@ -1883,14 +1897,19 @@ static void draw_pose_bones(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
 		}
 		/* if solid && posemode, we draw again with polygonoffset */
 		else if ((dt > OB_WIRE) && (arm->flag & ARM_POSEMODE)) {
-			bglPolygonOffset(rv3d->dist, 1.0);
+			ED_view3d_polygon_offset(rv3d, 1.0);
 		}
 		else {
 			/* and we use selection indices if not done yet */
 			if (arm->flag & ARM_POSEMODE) 
 				index = base->selcol;
 		}
-		
+
+		if (is_cull_enabled == false) {
+			is_cull_enabled = true;
+			glEnable(GL_CULL_FACE);
+		}
+
 		for (pchan = ob->pose->chanbase.first; pchan; pchan = pchan->next) {
 			bone = pchan->bone;
 			arm->layer_used |= bone->layer;
@@ -1907,7 +1926,7 @@ static void draw_pose_bones(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
 						 */
 						if ((do_dashed & 2) && ((bone->flag & BONE_CONNECTED) == 0)) {
 							if (arm->flag & ARM_POSEMODE) {
-								glLoadName(index & 0xFFFF);  /* object tag, for bordersel optim */
+								GPU_select_load_id(index & 0xFFFF);  /* object tag, for bordersel optim */
 								UI_ThemeColor(TH_WIRE);
 							}
 							setlinestyle(3);
@@ -1927,7 +1946,7 @@ static void draw_pose_bones(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
 									if (constflag & PCHAN_HAS_TARGET) glColor3ub(200, 120, 0);
 									else glColor3ub(200, 200, 50);  /* add theme! */
 
-									glLoadName(index & 0xFFFF);
+									GPU_select_load_id(index & 0xFFFF);
 									pchan_draw_IK_root_lines(pchan, !(do_dashed & 2));
 								}
 							}
@@ -1935,10 +1954,10 @@ static void draw_pose_bones(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
 								if (bone->flag & BONE_SELECTED) {
 									glColor3ub(150, 200, 50);  /* add theme! */
 									
-									glLoadName(index & 0xFFFF);
+									GPU_select_load_id(index & 0xFFFF);
 									pchan_draw_IK_root_lines(pchan, !(do_dashed & 2));
 								}
-							}	
+							}
 						}
 					}
 					
@@ -1991,15 +2010,17 @@ static void draw_pose_bones(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
 		}
 		/* restore things */
 		if (!ELEM(arm->drawtype, ARM_WIRE, ARM_LINE) && (dt > OB_WIRE) && (arm->flag & ARM_POSEMODE))
-			bglPolygonOffset(rv3d->dist, 0.0);
-	}	
+			ED_view3d_polygon_offset(rv3d, 0.0);
+	}
 	
 	/* restore */
-	glDisable(GL_CULL_FACE);
+	if (is_cull_enabled) {
+		glDisable(GL_CULL_FACE);
+	}
 	
 	/* draw DoFs */
 	if (arm->flag & ARM_POSEMODE) {
-		if (((base->flag & OB_FROMDUPLI) == 0)) {
+		if (((base->flag & OB_FROMDUPLI) == 0) && ((v3d->flag & V3D_HIDE_HELPLINES) == 0)) {
 			draw_pose_dofs(ob);
 		}
 	}
@@ -2044,8 +2065,8 @@ static void draw_pose_bones(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
 						/*  Draw names of bone  */
 						if (arm->flag & ARM_DRAWNAMES) {
 							mid_v3_v3v3(vec, pchan->pose_head, pchan->pose_tail);
-							view3d_cached_text_draw_add(vec, pchan->name, 10, 0, col);
-						}	
+							view3d_cached_text_draw_add(vec, pchan->name, strlen(pchan->name), 10, 0, col);
+						}
 						
 						/*	Draw additional axes on the bone tail  */
 						if ((arm->flag & ARM_DRAWAXES) && (arm->flag & ARM_POSEMODE)) {
@@ -2066,23 +2087,17 @@ static void draw_pose_bones(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
 			if (v3d->zbuf) glEnable(GL_DEPTH_TEST);
 		}
 	}
+
+	if (index != -1) {
+		GPU_select_load_id(-1);
+	}
 }
 
 /* in editmode, we don't store the bone matrix... */
-static void get_matrix_editbone(EditBone *eBone, float bmat[][4])
+static void get_matrix_editbone(EditBone *ebone, float bmat[4][4])
 {
-	float delta[3];
-	float mat[3][3];
-	
-	/* Compose the parent transforms (i.e. their translations) */
-	sub_v3_v3v3(delta, eBone->tail, eBone->head);	
-	
-	eBone->length = (float)sqrt(delta[0] * delta[0] + delta[1] * delta[1] + delta[2] * delta[2]);
-	
-	vec_roll_to_mat3(delta, eBone->roll, mat);
-	copy_m4_m3(bmat, mat);
-
-	add_v3_v3(bmat[3], eBone->head);
+	ebone->length = len_v3v3(ebone->tail, ebone->head);
+	ED_armature_ebone_to_mat4(ebone, bmat);
 }
 
 static void draw_ebones(View3D *v3d, ARegion *ar, Object *ob, const short dt)
@@ -2096,7 +2111,9 @@ static void draw_ebones(View3D *v3d, ARegion *ar, Object *ob, const short dt)
 	
 	/* being set in code below */
 	arm->layer_used = 0;
-	
+
+	ED_view3d_check_mats_rv3d(rv3d);
+
 	/* envelope (deform distance) */
 	if (arm->drawtype == ARM_ENVELOPE) {
 		/* precalc inverse matrix for drawing screen aligned */
@@ -2161,13 +2178,13 @@ static void draw_ebones(View3D *v3d, ARegion *ar, Object *ob, const short dt)
 	
 	/* if wire over solid, set offset */
 	index = -1;
-	glLoadName(-1);
+	GPU_select_load_id(-1);
 	if (ELEM(arm->drawtype, ARM_LINE, ARM_WIRE)) {
 		if (G.f & G_PICKSEL)
 			index = 0;
 	}
 	else if (dt > OB_WIRE) 
-		bglPolygonOffset(rv3d->dist, 1.0f);
+		ED_view3d_polygon_offset(rv3d, 1.0);
 	else if (arm->flag & ARM_EDITMODE) 
 		index = 0;  /* do selection codes */
 	
@@ -2209,8 +2226,8 @@ static void draw_ebones(View3D *v3d, ARegion *ar, Object *ob, const short dt)
 				
 				/* offset to parent */
 				if (eBone->parent) {
-					UI_ThemeColor(TH_WIRE);
-					glLoadName(-1);  /* -1 here is OK! */
+					UI_ThemeColor(TH_WIRE_EDIT);
+					GPU_select_load_id(-1);  /* -1 here is OK! */
 					setlinestyle(3);
 					
 					glBegin(GL_LINES);
@@ -2227,14 +2244,14 @@ static void draw_ebones(View3D *v3d, ARegion *ar, Object *ob, const short dt)
 	
 	/* restore */
 	if (index != -1) {
-		glLoadName(-1);
+		GPU_select_load_id(-1);
 	}
 
 	if (ELEM(arm->drawtype, ARM_LINE, ARM_WIRE)) {
 		/* pass */
 	}
 	else if (dt > OB_WIRE) {
-		bglPolygonOffset(rv3d->dist, 0.0f);
+		ED_view3d_polygon_offset(rv3d, 0.0);
 	}
 	
 	/* finally names and axes */
@@ -2257,8 +2274,8 @@ static void draw_ebones(View3D *v3d, ARegion *ar, Object *ob, const short dt)
 						if (arm->flag & ARM_DRAWNAMES) {
 							mid_v3_v3v3(vec, eBone->head, eBone->tail);
 							glRasterPos3fv(vec);
-							view3d_cached_text_draw_add(vec, eBone->name, 10, 0, col);
-						}					
+							view3d_cached_text_draw_add(vec, eBone->name, strlen(eBone->name), 10, 0, col);
+						}
 						/*	Draw additional axes */
 						if (arm->flag & ARM_DRAWAXES) {
 							glPushMatrix();
@@ -2378,13 +2395,13 @@ static void draw_ghost_poses_range(Scene *scene, View3D *v3d, ARegion *ar, Base
 	if (v3d->zbuf) glDisable(GL_DEPTH_TEST);
 	
 	/* draw from first frame of range to last */
-	for (CFRA = (int)start; CFRA < end; CFRA += (int)stepsize) {
+	for (CFRA = (int)start; CFRA <= end; CFRA += (int)stepsize) {
 		colfac = (end - (float)CFRA) / range;
-		UI_ThemeColorShadeAlpha(TH_WIRE, 0, -128 - (int)(120.0 * sqrt(colfac)));
+		UI_ThemeColorShadeAlpha(TH_WIRE, 0, -128 - (int)(120.0f * sqrtf(colfac)));
 		
 		BKE_animsys_evaluate_animdata(scene, &ob->id, adt, (float)CFRA, ADT_RECALC_ALL);
 		BKE_pose_where_is(scene, ob);
-		draw_pose_bones(scene, v3d, ar, base, OB_WIRE, NULL, TRUE, FALSE);
+		draw_pose_bones(scene, v3d, ar, base, OB_WIRE, NULL, true, false);
 	}
 	glDisable(GL_BLEND);
 	if (v3d->zbuf) glEnable(GL_DEPTH_TEST);
@@ -2460,13 +2477,13 @@ static void draw_ghost_poses_keys(Scene *scene, View3D *v3d, ARegion *ar, Base *
 	/* draw from first frame of range to last */
 	for (ak = keys.first, i = 0; ak; ak = ak->next, i++) {
 		colfac = i / range;
-		UI_ThemeColorShadeAlpha(TH_WIRE, 0, -128 - (int)(120.0 * sqrt(colfac)));
+		UI_ThemeColorShadeAlpha(TH_WIRE, 0, -128 - (int)(120.0f * sqrtf(colfac)));
 		
 		CFRA = (int)ak->cfra;
 		
 		BKE_animsys_evaluate_animdata(scene, &ob->id, adt, (float)CFRA, ADT_RECALC_ALL);
 		BKE_pose_where_is(scene, ob);
-		draw_pose_bones(scene, v3d, ar, base, OB_WIRE, NULL, TRUE, FALSE);
+		draw_pose_bones(scene, v3d, ar, base, OB_WIRE, NULL, true, false);
 	}
 	glDisable(GL_BLEND);
 	if (v3d->zbuf) glEnable(GL_DEPTH_TEST);
@@ -2530,7 +2547,7 @@ static void draw_ghost_poses(Scene *scene, View3D *v3d, ARegion *ar, Base *base)
 	for (cur = stepsize; cur < range; cur += stepsize) {
 		ctime = cur - (float)fmod(cfrao, stepsize);  /* ensures consistent stepping */
 		colfac = ctime / range;
-		UI_ThemeColorShadeAlpha(TH_WIRE, 0, -128 - (int)(120.0 * sqrt(colfac)));
+		UI_ThemeColorShadeAlpha(TH_WIRE, 0, -128 - (int)(120.0f * sqrtf(colfac)));
 		
 		/* only within action range */
 		if (actframe + ctime >= start && actframe + ctime <= end) {
@@ -2539,13 +2556,13 @@ static void draw_ghost_poses(Scene *scene, View3D *v3d, ARegion *ar, Base *base)
 			if (CFRA != cfrao) {
 				BKE_animsys_evaluate_animdata(scene, &ob->id, adt, (float)CFRA, ADT_RECALC_ALL);
 				BKE_pose_where_is(scene, ob);
-				draw_pose_bones(scene, v3d, ar, base, OB_WIRE, NULL, TRUE, FALSE);
+				draw_pose_bones(scene, v3d, ar, base, OB_WIRE, NULL, true, false);
 			}
 		}
 		
 		ctime = cur + (float)fmod((float)cfrao, stepsize) - stepsize + 1.0f;   /* ensures consistent stepping */
 		colfac = ctime / range;
-		UI_ThemeColorShadeAlpha(TH_WIRE, 0, -128 - (int)(120.0 * sqrt(colfac)));
+		UI_ThemeColorShadeAlpha(TH_WIRE, 0, -128 - (int)(120.0f * sqrtf(colfac)));
 		
 		/* only within action range */
 		if ((actframe - ctime >= start) && (actframe - ctime <= end)) {
@@ -2554,7 +2571,7 @@ static void draw_ghost_poses(Scene *scene, View3D *v3d, ARegion *ar, Base *base)
 			if (CFRA != cfrao) {
 				BKE_animsys_evaluate_animdata(scene, &ob->id, adt, (float)CFRA, ADT_RECALC_ALL);
 				BKE_pose_where_is(scene, ob);
-				draw_pose_bones(scene, v3d, ar, base, OB_WIRE, NULL, TRUE, FALSE);
+				draw_pose_bones(scene, v3d, ar, base, OB_WIRE, NULL, true, false);
 			}
 		}
 	}
@@ -2579,16 +2596,16 @@ static void draw_ghost_poses(Scene *scene, View3D *v3d, ARegion *ar, Base *base)
 
 /* called from drawobject.c, return 1 if nothing was drawn
  * (ob_wire_col == NULL) when drawing ghost */
-int draw_armature(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
-                  const short dt, const short dflag, const unsigned char ob_wire_col[4],
-                  const short is_outline)
+bool draw_armature(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
+                   const short dt, const short dflag, const unsigned char ob_wire_col[4],
+                   const bool is_outline)
 {
 	Object *ob = base->object;
 	bArmature *arm = ob->data;
-	int retval = 0;
+	bool retval = false;
 
 	if (v3d->flag2 & V3D_RENDER_OVERRIDE)
-		return 1;
+		return true;
 	
 	if (dt > OB_WIRE && !ELEM(arm->drawtype, ARM_LINE, ARM_WIRE)) {
 		/* we use color for solid lighting */
@@ -2643,7 +2660,7 @@ int draw_armature(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
 						}
 						draw_pose_paths(scene, v3d, ar, ob);
 					}
-				}	
+				}
 			}
 			draw_pose_bones(scene, v3d, ar, base, dt, ob_wire_col, (dflag & DRAW_CONSTCOLOR), is_outline);
 			arm->flag &= ~ARM_POSEMODE; 
@@ -2652,7 +2669,7 @@ int draw_armature(Scene *scene, View3D *v3d, ARegion *ar, Base *base,
 				UI_ThemeColor(TH_WIRE);  /* restore, for extra draw stuff */
 		}
 		else {
-			retval = 1;
+			retval = true;
 		}
 	}
 	/* restore */
