@@ -34,10 +34,11 @@
 
 #include "MEM_guardedalloc.h"
 
-#include "BLO_sys_types.h" /* for intptr_t support */
+#include "BLI_sys_types.h" /* for intptr_t support */
 
 #include "DNA_anim_types.h"
 #include "DNA_armature_types.h"
+#include "DNA_brush_types.h"
 #include "DNA_lattice_types.h"
 #include "DNA_screen_types.h"
 #include "DNA_sequence_types.h"
@@ -45,35 +46,38 @@
 #include "DNA_scene_types.h"
 #include "DNA_object_types.h"
 #include "DNA_mesh_types.h"
-#include "DNA_meshdata_types.h"
 #include "DNA_view3d_types.h"
 #include "DNA_modifier_types.h"
 #include "DNA_movieclip_types.h"
 #include "DNA_mask_types.h"
+#include "DNA_meta_types.h"
 
 #include "BLI_math.h"
 #include "BLI_blenlib.h"
 #include "BLI_rand.h"
 #include "BLI_utildefines.h"
 
+#include "BLF_translation.h"
+
 #include "RNA_access.h"
 
 #include "BIF_gl.h"
 #include "BIF_glutil.h"
 
+#include "BIK_api.h"
+
 #include "BKE_animsys.h"
 #include "BKE_action.h"
 #include "BKE_armature.h"
 #include "BKE_curve.h"
 #include "BKE_depsgraph.h"
-#include "BKE_displist.h"
 #include "BKE_fcurve.h"
 #include "BKE_lattice.h"
-#include "BKE_mesh.h"
 #include "BKE_nla.h"
 #include "BKE_context.h"
+#include "BKE_paint.h"
 #include "BKE_sequencer.h"
-#include "BKE_tessmesh.h"
+#include "BKE_editmesh.h"
 #include "BKE_tracking.h"
 #include "BKE_mask.h"
 
@@ -96,6 +100,7 @@
 #include "WM_api.h"
 
 #include "UI_resources.h"
+#include "UI_view2d.h"
 
 #include "transform.h"
 
@@ -104,21 +109,7 @@
 void getViewVector(TransInfo *t, float coord[3], float vec[3])
 {
 	if (t->persp != RV3D_ORTHO) {
-		float p1[4], p2[4];
-		
-		copy_v3_v3(p1, coord);
-		p1[3] = 1.0f;
-		copy_v3_v3(p2, p1);
-		p2[3] = 1.0f;
-		mul_m4_v4(t->viewmat, p2);
-		
-		p2[0] = 2.0f * p2[0];
-		p2[1] = 2.0f * p2[1];
-		p2[2] = 2.0f * p2[2];
-		
-		mul_m4_v4(t->viewinv, p2);
-		
-		sub_v3_v3v3(vec, p1, p2);
+		sub_v3_v3v3(vec, coord, t->viewinv[3]);
 	}
 	else {
 		copy_v3_v3(vec, t->viewinv[2]);
@@ -128,6 +119,7 @@ void getViewVector(TransInfo *t, float coord[3], float vec[3])
 
 /* ************************** GENERICS **************************** */
 
+
 static void clipMirrorModifier(TransInfo *t, Object *ob)
 {
 	ModifierData *md = ob->modifiers.first;
@@ -161,7 +153,7 @@ static void clipMirrorModifier(TransInfo *t, Object *ob)
 						float obinv[4][4];
 						
 						invert_m4_m4(obinv, mmd->mirror_ob->obmat);
-						mult_m4_m4m4(mtx, obinv, ob->obmat);
+						mul_m4_m4m4(mtx, obinv, ob->obmat);
 						invert_m4_m4(imtx, mtx);
 					}
 					
@@ -259,7 +251,7 @@ static void animrecord_check_state(Scene *scene, ID *id, wmTimer *animtimer)
 	ScreenAnimData *sad = (animtimer) ? animtimer->customdata : NULL;
 	
 	/* sanity checks */
-	if (ELEM3(NULL, scene, id, sad))
+	if (ELEM(NULL, scene, id, sad))
 		return;
 	
 	/* check if we need a new strip if:
@@ -301,7 +293,7 @@ static void animrecord_check_state(Scene *scene, ID *id, wmTimer *animtimer)
 	}
 }
 
-static int fcu_test_selected(FCurve *fcu)
+static bool fcu_test_selected(FCurve *fcu)
 {
 	BezTriple *bezt = fcu->bezt;
 	unsigned int i;
@@ -360,7 +352,7 @@ static void recalcData_actedit(TransInfo *t)
 		}
 		
 		/* now free temp channels */
-		BLI_freelistN(&anim_data);
+		ANIM_animdata_freelist(&anim_data);
 	}
 }
 /* helper for recalcData() - for Graph Editor transforms */
@@ -375,8 +367,7 @@ static void recalcData_graphedit(TransInfo *t)
 	
 	bAnimListElem *ale;
 	int dosort = 0;
-	
-	
+
 	/* initialize relevant anim-context 'context' data from TransInfo data */
 	/* NOTE: sync this with the code in ANIM_animdata_get_context() */
 	scene = ac.scene = t->scene;
@@ -400,14 +391,10 @@ static void recalcData_graphedit(TransInfo *t)
 	for (ale = anim_data.first; ale; ale = ale->next) {
 		FCurve *fcu = (FCurve *)ale->key_data;
 		
-		/* ignore unselected fcurves */
+		/* ignore FC-Curves without any selected verts */
 		if (!fcu_test_selected(fcu))
 			continue;
-		
-		// fixme: only do this for selected verts...
-		ANIM_unit_mapping_apply_fcurve(ac.scene, ale->id, ale->key_data, ANIM_UNITCONV_ONLYSEL | ANIM_UNITCONV_SELVERTS | ANIM_UNITCONV_RESTORE);
-		
-		
+
 		/* watch it: if the time is wrong: do not correct handles yet */
 		if (test_time_fcurve(fcu))
 			dosort++;
@@ -425,7 +412,7 @@ static void recalcData_graphedit(TransInfo *t)
 	if (dosort) remake_graph_transdata(t, &anim_data);
 	
 	/* now free temp channels */
-	BLI_freelistN(&anim_data);
+	ANIM_animdata_freelist(&anim_data);
 }
 
 /* helper for recalcData() - for NLA Editor transforms */
@@ -497,7 +484,7 @@ static void recalcData_nla(TransInfo *t)
 			pExceeded = ((strip->prev) && (strip->prev->type != NLASTRIP_TYPE_TRANSITION) && (tdn->h1[0] < strip->prev->end));
 			nExceeded = ((strip->next) && (strip->next->type != NLASTRIP_TYPE_TRANSITION) && (tdn->h2[0] > strip->next->start));
 			
-			if ((pExceeded && nExceeded) || (iter == 4) ) {
+			if ((pExceeded && nExceeded) || (iter == 4)) {
 				/* both endpoints exceeded (or iteration ping-pong'd meaning that we need a compromise)
 				 *	- simply crop strip to fit within the bounds of the strips bounding it
 				 *	- if there were no neighbors, clear the transforms (make it default to the strip's current values)
@@ -529,32 +516,62 @@ static void recalcData_nla(TransInfo *t)
 				break;
 		}
 		
-		/* handle auto-snapping */
-		switch (snla->autosnap) {
-			case SACTSNAP_FRAME: /* snap to nearest frame/time  */
-				if (snla->flag & SNLA_DRAWTIME) {
-					tdn->h1[0] = (float)(floor(((double)tdn->h1[0] / secf) + 0.5) * secf);
-					tdn->h2[0] = (float)(floor(((double)tdn->h2[0] / secf) + 0.5) * secf);
-				}
-				else {
+		/* handle auto-snapping
+		 * NOTE: only do this when transform is still running, or we can't restore
+		 */
+		if (t->state != TRANS_CANCEL) {
+			switch (snla->autosnap) {
+				case SACTSNAP_FRAME: /* snap to nearest frame */
+				case SACTSNAP_STEP: /* frame step - this is basically the same, since we don't have any remapping going on */
+				{
 					tdn->h1[0] = floorf(tdn->h1[0] + 0.5f);
 					tdn->h2[0] = floorf(tdn->h2[0] + 0.5f);
+					break;
 				}
-				break;
-			
-			case SACTSNAP_MARKER: /* snap to nearest marker */
-				tdn->h1[0] = (float)ED_markers_find_nearest_marker_time(&t->scene->markers, tdn->h1[0]);
-				tdn->h2[0] = (float)ED_markers_find_nearest_marker_time(&t->scene->markers, tdn->h2[0]);
-				break;
+				
+				case SACTSNAP_SECOND: /* snap to nearest second */
+				case SACTSNAP_TSTEP: /* second step - this is basically the same, since we don't have any remapping going on */
+				{
+					/* This case behaves differently from the rest, since lengths of strips
+					 * may not be multiples of a second. If we just naively resize adjust
+					 * the handles, things may not work correctly. Instead, we only snap
+					 * the first handle, and move the other to fit.
+					 *
+					 * FIXME: we do run into problems here when user attempts to negatively
+					 *        scale the strip, as it then just compresses down and refuses
+					 *        to expand out the other end.
+					 */
+					float h1_new = (float)(floor(((double)tdn->h1[0] / secf) + 0.5) * secf);
+					float delta  = h1_new - tdn->h1[0];
+					
+					tdn->h1[0] = h1_new;
+					tdn->h2[0] += delta;
+					break;
+				}
+				
+				case SACTSNAP_MARKER: /* snap to nearest marker */
+				{
+					tdn->h1[0] = (float)ED_markers_find_nearest_marker_time(&t->scene->markers, tdn->h1[0]);
+					tdn->h2[0] = (float)ED_markers_find_nearest_marker_time(&t->scene->markers, tdn->h2[0]);
+					break;
+				}
+			}
 		}
 		
-		/* use RNA to write the values... */
-		// TODO: do we need to write in 2 passes to make sure that no truncation goes on?
+		/* Use RNA to write the values to ensure that constraints on these are obeyed
+		 * (e.g. for transition strips, the values are taken from the neighbours)
+		 * 
+		 * NOTE: we write these twice to avoid truncation errors which can arise when
+		 * moving the strips a large distance using numeric input [#33852] 
+		 */
 		RNA_pointer_create(NULL, &RNA_NlaStrip, strip, &strip_ptr);
 		
 		RNA_float_set(&strip_ptr, "frame_start", tdn->h1[0]);
 		RNA_float_set(&strip_ptr, "frame_end", tdn->h2[0]);
 		
+		RNA_float_set(&strip_ptr, "frame_start", tdn->h1[0]);
+		RNA_float_set(&strip_ptr, "frame_end", tdn->h2[0]);
+		
 		/* flush transforms to child strips (since this should be a meta) */
 		BKE_nlameta_flush_transforms(strip);
 		
@@ -625,6 +642,9 @@ static void recalcData_image(TransInfo *t)
 	if (t->options & CTX_MASK) {
 		recalcData_mask_common(t);
 	}
+	else if (t->options & CTX_PAINT_CURVE) {
+		flushTransPaintCurve(t);
+	}
 	else if (t->obedit && t->obedit->type == OB_MESH) {
 		SpaceImage *sima = t->sa->spacedata.first;
 		
@@ -682,8 +702,8 @@ static void recalcData_spaceclip(TransInfo *t)
 	}
 }
 
-/* helper for recalcData() - for 3d-view transforms */
-static void recalcData_view3d(TransInfo *t)
+/* helper for recalcData() - for object transforms, typically in the 3D view */
+static void recalcData_objects(TransInfo *t)
 {
 	Base *base = t->scene->basact;
 	
@@ -705,7 +725,7 @@ static void recalcData_view3d(TransInfo *t)
 					BKE_nurb_handles_calc(nu); /* Cant do testhandlesNurb here, it messes up the h1 and h2 flags */
 					nu = nu->next;
 				}
-			} 
+			}
 			else {
 				/* Normal updating */
 				while (nu) {
@@ -727,7 +747,7 @@ static void recalcData_view3d(TransInfo *t)
 			if (la->editlatt->latt->flag & LT_OUTSIDE) outside_lattice(la->editlatt->latt);
 		}
 		else if (t->obedit->type == OB_MESH) {
-			BMEditMesh *em = BMEdit_FromObject(t->obedit);
+			BMEditMesh *em = BKE_editmesh_from_object(t->obedit);
 			/* mirror modifier clipping? */
 			if (t->state != TRANS_CANCEL) {
 				/* apply clipping after so we never project past the clip plane [#25423] */
@@ -740,12 +760,12 @@ static void recalcData_view3d(TransInfo *t)
 			DAG_id_tag_update(t->obedit->data, 0);  /* sets recalc flags */
 			
 			EDBM_mesh_normals_update(em);
-			BMEdit_RecalcTessellation(em);
+			BKE_editmesh_tessface_calc(em);
 		}
 		else if (t->obedit->type == OB_ARMATURE) { /* no recalc flag, does pose */
 			bArmature *arm = t->obedit->data;
 			ListBase *edbo = arm->edbo;
-			EditBone *ebo;
+			EditBone *ebo, *ebo_parent;
 			TransData *td = t->data;
 			int i;
 			
@@ -755,17 +775,18 @@ static void recalcData_view3d(TransInfo *t)
 			
 			/* Ensure all bones are correctly adjusted */
 			for (ebo = edbo->first; ebo; ebo = ebo->next) {
+				ebo_parent = (ebo->flag & BONE_CONNECTED) ? ebo->parent : NULL;
 				
-				if ((ebo->flag & BONE_CONNECTED) && ebo->parent) {
+				if (ebo_parent) {
 					/* If this bone has a parent tip that has been moved */
-					if (ebo->parent->flag & BONE_TIPSEL) {
-						copy_v3_v3(ebo->head, ebo->parent->tail);
-						if (t->mode == TFM_BONE_ENVELOPE) ebo->rad_head = ebo->parent->rad_tail;
+					if (ebo_parent->flag & BONE_TIPSEL) {
+						copy_v3_v3(ebo->head, ebo_parent->tail);
+						if (t->mode == TFM_BONE_ENVELOPE) ebo->rad_head = ebo_parent->rad_tail;
 					}
 					/* If this bone has a parent tip that has NOT been moved */
 					else {
-						copy_v3_v3(ebo->parent->tail, ebo->head);
-						if (t->mode == TFM_BONE_ENVELOPE) ebo->parent->rad_tail = ebo->rad_head;
+						copy_v3_v3(ebo_parent->tail, ebo->head);
+						if (t->mode == TFM_BONE_ENVELOPE) ebo_parent->rad_tail = ebo->rad_head;
 					}
 				}
 				
@@ -789,35 +810,42 @@ static void recalcData_view3d(TransInfo *t)
 				}
 			}
 			
-			
-			if (t->mode != TFM_BONE_ROLL) {
+			if (!ELEM(t->mode, TFM_BONE_ROLL, TFM_BONE_ENVELOPE, TFM_BONESIZE)) {
 				/* fix roll */
 				for (i = 0; i < t->total; i++, td++) {
 					if (td->extra) {
 						float vec[3], up_axis[3];
 						float qrot[4];
+						float roll;
 						
 						ebo = td->extra;
-						copy_v3_v3(up_axis, td->axismtx[2]);
-						
-						if (t->mode != TFM_ROTATION) {
+
+						if (t->state == TRANS_CANCEL) {
+							/* restore roll */
+							ebo->roll = td->ival;
+						}
+						else {
+							copy_v3_v3(up_axis, td->axismtx[2]);
+
 							sub_v3_v3v3(vec, ebo->tail, ebo->head);
 							normalize_v3(vec);
 							rotation_between_vecs_to_quat(qrot, td->axismtx[1], vec);
 							mul_qt_v3(qrot, up_axis);
+
+							/* roll has a tendency to flip in certain orientations - [#34283], [#33974] */
+							roll = ED_rollBoneToVector(ebo, up_axis, false);
+							ebo->roll = angle_compat_rad(roll, td->ival);
 						}
-						else {
-							mul_m3_v3(t->mat, up_axis);
-						}
-						
-						ebo->roll = ED_rollBoneToVector(ebo, up_axis, FALSE);
 					}
 				}
 			}
 			
-			if (arm->flag & ARM_MIRROR_EDIT)
-				transform_armature_mirror_update(t->obedit);
-			
+			if (arm->flag & ARM_MIRROR_EDIT) {
+				if (t->state != TRANS_CANCEL)
+					transform_armature_mirror_update(t->obedit);
+				else
+					restoreBones(t);
+			}
 		}
 		else {
 			if (t->state != TRANS_CANCEL) {
@@ -847,6 +875,8 @@ static void recalcData_view3d(TransInfo *t)
 		/* old optimize trick... this enforces to bypass the depgraph */
 		if (!(arm->flag & ARM_DELAYDEFORM)) {
 			DAG_id_tag_update(&ob->id, OB_RECALC_DATA);  /* sets recalc flags */
+			/* transformation of pose may affect IK tree, make sure it is rebuilt */
+			BIK_clear_data(ob->pose);
 		}
 		else
 			BKE_pose_where_is(t->scene, ob);
@@ -888,6 +918,9 @@ static void recalcData_view3d(TransInfo *t)
 			 * otherwise proxies don't function correctly
 			 */
 			DAG_id_tag_update(&ob->id, OB_RECALC_OB);
+
+			if (t->flag & T_TEXTURE)
+				DAG_id_tag_update(&ob->id, OB_RECALC_DATA);
 		}
 	}
 }
@@ -918,33 +951,42 @@ static void recalcData_sequencer(TransInfo *t)
 /* called for updating while transform acts, once per redraw */
 void recalcData(TransInfo *t)
 {
-	if (t->spacetype == SPACE_NODE) {
-		flushTransNodes(t);
+	/* if tests must match createTransData for correct updates */
+	if (t->options & CTX_TEXTURE) {
+		recalcData_objects(t);
 	}
-	else if (t->spacetype == SPACE_SEQ) {
-		recalcData_sequencer(t);
+	else if (t->options & CTX_EDGE) {
+		recalcData_objects(t);
 	}
-	else if (t->spacetype == SPACE_ACTION) {
-		recalcData_actedit(t);
+	else if (t->options & CTX_PAINT_CURVE) {
+		flushTransPaintCurve(t);
 	}
-	else if (t->spacetype == SPACE_IPO) {
-		recalcData_graphedit(t);
-	}
-	else if (t->spacetype == SPACE_NLA) {
-		recalcData_nla(t);
+	else if (t->options & CTX_GPENCIL_STROKES) {
+		/* pass? */
 	}
 	else if (t->spacetype == SPACE_IMAGE) {
 		recalcData_image(t);
 	}
-	else if (t->spacetype == SPACE_VIEW3D) {
-		recalcData_view3d(t);
+	else if (t->spacetype == SPACE_ACTION) {
+		recalcData_actedit(t);
+	}
+	else if (t->spacetype == SPACE_NLA) {
+		recalcData_nla(t);
+	}
+	else if (t->spacetype == SPACE_SEQ) {
+		recalcData_sequencer(t);
+	}
+	else if (t->spacetype == SPACE_IPO) {
+		recalcData_graphedit(t);
+	}
+	else if (t->spacetype == SPACE_NODE) {
+		flushTransNodes(t);
 	}
 	else if (t->spacetype == SPACE_CLIP) {
 		recalcData_spaceclip(t);
 	}
-
-	if (t->options & CTX_MASK) {
-
+	else {
+		recalcData_objects(t);
 	}
 }
 
@@ -986,32 +1028,65 @@ void drawLine(TransInfo *t, const float center[3], const float dir[3], char axis
 	}
 }
 
+/**
+ * Free data before switching to another mode.
+ */
+void resetTransModal(TransInfo *t)
+{
+	if (t->mode == TFM_EDGE_SLIDE) {
+		freeEdgeSlideVerts(t);
+	}
+	else if (t->mode == TFM_VERT_SLIDE) {
+		freeVertSlideVerts(t);
+	}
+}
+
 void resetTransRestrictions(TransInfo *t)
 {
 	t->flag &= ~T_ALL_RESTRICTIONS;
 }
 
-/* the *op can be NULL */
-int initTransInfo(bContext *C, TransInfo *t, wmOperator *op, wmEvent *event)
+static int initTransInfo_edit_pet_to_flag(const int proportional)
+{
+	switch (proportional) {
+		case PROP_EDIT_ON:
+			return T_PROP_EDIT;
+		case PROP_EDIT_CONNECTED:
+			return T_PROP_EDIT | T_PROP_CONNECTED;
+		case PROP_EDIT_PROJECTED:
+			return T_PROP_EDIT | T_PROP_PROJECTED;
+		default:
+			return 0;
+	}
+}
+
+/**
+ * Setup internal data, mouse, vectors
+ *
+ * \note \a op and \a event can be NULL
+ */
+void initTransInfo(bContext *C, TransInfo *t, wmOperator *op, const wmEvent *event)
 {
 	Scene *sce = CTX_data_scene(C);
 	ToolSettings *ts = CTX_data_tool_settings(C);
 	ARegion *ar = CTX_wm_region(C);
 	ScrArea *sa = CTX_wm_area(C);
 	Object *obedit = CTX_data_edit_object(C);
-	
-	/* moving: is shown in drawobject() (transform color) */
-//  TRANSFORM_FIX_ME
-//	if (obedit || (t->flag & T_POSE) ) G.moving= G_TRANSFORM_EDIT;
-//	else if (G.f & G_PARTICLEEDIT) G.moving= G_TRANSFORM_PARTICLE;
-//	else G.moving= G_TRANSFORM_OBJ;
+	Object *ob = CTX_data_active_object(C);
+	PropertyRNA *prop;
 	
 	t->scene = sce;
 	t->sa = sa;
 	t->ar = ar;
 	t->obedit = obedit;
 	t->settings = ts;
-	
+	t->reports = op ? op->reports : NULL;
+
+	if (obedit) {
+		copy_m3_m4(t->obedit_mat, obedit->obmat);
+		normalize_m3(t->obedit_mat);
+	}
+
 	t->data = NULL;
 	t->ext = NULL;
 	
@@ -1019,7 +1094,7 @@ int initTransInfo(bContext *C, TransInfo *t, wmOperator *op, wmEvent *event)
 	
 	t->flag = 0;
 	
-	t->redraw = 1; /* redraw first time */
+	t->redraw = TREDRAW_HARD;  /* redraw first time */
 	
 	if (event) {
 		copy_v2_v2_int(t->imval, event->mval);
@@ -1058,6 +1133,13 @@ int initTransInfo(bContext *C, TransInfo *t, wmOperator *op, wmEvent *event)
 		t->options |= CTX_EDGE;
 	}
 
+	t->remove_on_cancel = false;
+
+	if (op && (prop = RNA_struct_find_property(op->ptr, "remove_on_cancel")) && RNA_property_is_set(op->ptr, prop)) {
+		if (RNA_property_boolean_get(op->ptr, prop)) {
+			t->remove_on_cancel = true;
+		}
+	}
 
 	/* Assign the space type, some exceptions for running in different mode */
 	if (sa == NULL) {
@@ -1091,10 +1173,15 @@ int initTransInfo(bContext *C, TransInfo *t, wmOperator *op, wmEvent *event)
 		if (v3d->flag & V3D_ALIGN) t->flag |= T_V3D_ALIGN;
 		t->around = v3d->around;
 		
-		if (op && (RNA_struct_find_property(op->ptr, "constraint_orientation") &&
-		           RNA_struct_property_is_set(op->ptr, "constraint_orientation")))
+		/* bend always uses the cursor */
+		if (t->mode == TFM_BEND) {
+			t->around = V3D_CURSOR;
+		}
+
+		if (op && ((prop = RNA_struct_find_property(op->ptr, "constraint_orientation")) &&
+		           RNA_property_is_set(op->ptr, prop)))
 		{
-			t->current_orientation = RNA_enum_get(op->ptr, "constraint_orientation");
+			t->current_orientation = RNA_property_enum_get(op->ptr, prop);
 
 			if (t->current_orientation >= V3D_MANIP_CUSTOM + BIF_countTransformOrientation(C)) {
 				t->current_orientation = V3D_MANIP_GLOBAL;
@@ -1105,16 +1192,27 @@ int initTransInfo(bContext *C, TransInfo *t, wmOperator *op, wmEvent *event)
 		}
 
 		/* exceptional case */
-		if (t->around == V3D_LOCAL && (t->settings->selectmode & SCE_SELECT_FACE)) {
-			if (ELEM3(t->mode, TFM_ROTATION, TFM_RESIZE, TFM_TRACKBALL)) {
-				t->options |= CTX_NO_PET;
+		if (t->around == V3D_LOCAL) {
+			if (ELEM(t->mode, TFM_ROTATION, TFM_RESIZE, TFM_TRACKBALL)) {
+				const bool use_island = transdata_check_local_islands(t, t->around);
+
+				if (!use_island) {
+					t->options |= CTX_NO_PET;
+				}
+			}
+		}
+
+		if (ob && ob->mode & OB_MODE_ALL_PAINT) {
+			Paint *p = BKE_paint_get_active_from_context(C);
+			if (p && p->brush && (p->brush->flag & BRUSH_CURVE)) {
+				t->options |= CTX_PAINT_CURVE;
 			}
 		}
 
 		/* initialize UV transform from */
-		if (op && RNA_struct_find_property(op->ptr, "correct_uv")) {
-			if (RNA_struct_property_is_set(op->ptr, "correct_uv")) {
-				if (RNA_boolean_get(op->ptr, "correct_uv")) {
+		if (op && ((prop = RNA_struct_find_property(op->ptr, "correct_uv")))) {
+			if (RNA_property_is_set(op->ptr, prop)) {
+				if (RNA_property_boolean_get(op->ptr, prop)) {
 					t->settings->uvcalc_flag |= UVCALC_TRANSFORM_CORRECT;
 				}
 				else {
@@ -1122,7 +1220,7 @@ int initTransInfo(bContext *C, TransInfo *t, wmOperator *op, wmEvent *event)
 				}
 			}
 			else {
-				RNA_boolean_set(op->ptr, "correct_uv", t->settings->uvcalc_flag & UVCALC_TRANSFORM_CORRECT);
+				RNA_property_boolean_set(op->ptr, prop, t->settings->uvcalc_flag & UVCALC_TRANSFORM_CORRECT);
 			}
 		}
 
@@ -1139,9 +1237,13 @@ int initTransInfo(bContext *C, TransInfo *t, wmOperator *op, wmEvent *event)
 		else if (sima->mode == SI_MODE_MASK) {
 			t->options |= CTX_MASK;
 		}
-		else {
-			BLI_assert(0);
+		else if (sima->mode == SI_MODE_PAINT) {
+			Paint *p = &sce->toolsettings->imapaint.paint;
+			if (p->brush && (p->brush->flag & BRUSH_CURVE)) {
+				t->options |= CTX_PAINT_CURVE;
+			}
 		}
+		/* image not in uv edit, nor in mask mode, can happen for some tools */
 	}
 	else if (t->spacetype == SPACE_NODE) {
 		// XXX for now, get View2D from the active region
@@ -1175,8 +1277,10 @@ int initTransInfo(bContext *C, TransInfo *t, wmOperator *op, wmEvent *event)
 		t->around = V3D_CENTER;
 	}
 	
-	if (op && RNA_struct_property_is_set(op->ptr, "release_confirm")) {
-		if (RNA_boolean_get(op->ptr, "release_confirm")) {
+	if (op && ((prop = RNA_struct_find_property(op->ptr, "release_confirm")) &&
+	           RNA_property_is_set(op->ptr, prop)))
+	{
+		if (RNA_property_boolean_get(op->ptr, prop)) {
 			t->flag |= T_RELEASE_CONFIRM;
 		}
 	}
@@ -1186,10 +1290,10 @@ int initTransInfo(bContext *C, TransInfo *t, wmOperator *op, wmEvent *event)
 		}
 	}
 
-	if (op && (RNA_struct_find_property(op->ptr, "mirror") &&
-	           RNA_struct_property_is_set(op->ptr, "mirror")))
+	if (op && ((prop = RNA_struct_find_property(op->ptr, "mirror")) &&
+	           RNA_property_is_set(op->ptr, prop)))
 	{
-		if (RNA_boolean_get(op->ptr, "mirror")) {
+		if (RNA_property_boolean_get(op->ptr, prop)) {
 			t->flag |= T_MIRROR;
 			t->mirror = 1;
 		}
@@ -1203,26 +1307,19 @@ int initTransInfo(bContext *C, TransInfo *t, wmOperator *op, wmEvent *event)
 	}
 	
 	/* setting PET flag only if property exist in operator. Otherwise, assume it's not supported */
-	if (op && RNA_struct_find_property(op->ptr, "proportional")) {
-		if (RNA_struct_property_is_set(op->ptr, "proportional")) {
-			switch (RNA_enum_get(op->ptr, "proportional")) {
-				case PROP_EDIT_CONNECTED:
-					t->flag |= T_PROP_CONNECTED;
-				case PROP_EDIT_ON:
-					t->flag |= T_PROP_EDIT;
-					break;
-			}
+	if (op && (prop = RNA_struct_find_property(op->ptr, "proportional"))) {
+		if (RNA_property_is_set(op->ptr, prop)) {
+			t->flag |= initTransInfo_edit_pet_to_flag(RNA_property_enum_get(op->ptr, prop));
 		}
 		else {
 			/* use settings from scene only if modal */
 			if (t->flag & T_MODAL) {
 				if ((t->options & CTX_NO_PET) == 0) {
-					if (t->obedit && ts->proportional != PROP_EDIT_OFF) {
-						t->flag |= T_PROP_EDIT;
-
-						if (ts->proportional == PROP_EDIT_CONNECTED) {
-							t->flag |= T_PROP_CONNECTED;
-						}
+					if (t->obedit) {
+						t->flag |= initTransInfo_edit_pet_to_flag(ts->proportional);
+					}
+					else if (t->options & CTX_GPENCIL_STROKES) {
+						t->flag |= initTransInfo_edit_pet_to_flag(ts->proportional);
 					}
 					else if (t->options & CTX_MASK) {
 						if (ts->proportional_mask) {
@@ -1240,10 +1337,10 @@ int initTransInfo(bContext *C, TransInfo *t, wmOperator *op, wmEvent *event)
 			}
 		}
 		
-		if (op && (RNA_struct_find_property(op->ptr, "proportional_size") &&
-		           RNA_struct_property_is_set(op->ptr, "proportional_size")))
+		if (op && ((prop = RNA_struct_find_property(op->ptr, "proportional_size")) &&
+		           RNA_property_is_set(op->ptr, prop)))
 		{
-			t->prop_size = RNA_float_get(op->ptr, "proportional_size");
+			t->prop_size = RNA_property_float_get(op->ptr, prop);
 		}
 		else {
 			t->prop_size = ts->proportional_size;
@@ -1252,14 +1349,14 @@ int initTransInfo(bContext *C, TransInfo *t, wmOperator *op, wmEvent *event)
 		
 		/* TRANSFORM_FIX_ME rna restrictions */
 		if (t->prop_size <= 0.00001f) {
-			printf("Proportional size (%f) under 0.00001, reseting to 1!\n", t->prop_size);
+			printf("Proportional size (%f) under 0.00001, resetting to 1!\n", t->prop_size);
 			t->prop_size = 1.0f;
 		}
 		
-		if (op && (RNA_struct_find_property(op->ptr, "proportional_edit_falloff") &&
-		           RNA_struct_property_is_set(op->ptr, "proportional_edit_falloff")))
+		if (op && ((prop = RNA_struct_find_property(op->ptr, "proportional_edit_falloff")) &&
+		           RNA_property_is_set(op->ptr, prop)))
 		{
-			t->prop_mode = RNA_enum_get(op->ptr, "proportional_edit_falloff");
+			t->prop_mode = RNA_property_enum_get(op->ptr, prop);
 		}
 		else {
 			t->prop_mode = ts->prop_mode;
@@ -1270,14 +1367,14 @@ int initTransInfo(bContext *C, TransInfo *t, wmOperator *op, wmEvent *event)
 	}
 	
 	// Mirror is not supported with PET, turn it off.
+#if 0
 	if (t->flag & T_PROP_EDIT) {
 		t->flag &= ~T_MIRROR;
 	}
+#endif
 
 	setTransformViewMatrices(t);
 	initNumInput(&t->num);
-	
-	return 1;
 }
 
 /* Here I would suggest only TransInfo related issues, like free data & reset vars. Not redraws */
@@ -1330,7 +1427,7 @@ void postTrans(bContext *C, TransInfo *t)
 	}
 	
 	if (t->spacetype == SPACE_IMAGE) {
-		if (t->options & CTX_MASK) {
+		if (t->options & (CTX_MASK | CTX_PAINT_CURVE)) {
 			/* pass */
 		}
 		else {
@@ -1436,42 +1533,46 @@ void calculateCenter2D(TransInfo *t)
 		
 		copy_v3_v3(vec, t->center);
 		mul_m4_v3(ob->obmat, vec);
-		projectIntView(t, vec, t->center2d);
+		projectFloatView(t, vec, t->center2d);
 	}
 	else {
-		projectIntView(t, t->center, t->center2d);
+		projectFloatView(t, t->center, t->center2d);
 	}
 }
 
-void calculateCenterCursor(TransInfo *t)
+void calculateCenterCursor(TransInfo *t, float r_center[3])
 {
-	float *cursor;
+	const float *cursor;
 	
-	cursor = give_cursor(t->scene, t->view);
-	copy_v3_v3(t->center, cursor);
+	cursor = ED_view3d_cursor3d_get(t->scene, t->view);
+	copy_v3_v3(r_center, cursor);
 	
 	/* If edit or pose mode, move cursor in local space */
 	if (t->flag & (T_EDIT | T_POSE)) {
 		Object *ob = t->obedit ? t->obedit : t->poseobj;
 		float mat[3][3], imat[3][3];
 		
-		sub_v3_v3v3(t->center, t->center, ob->obmat[3]);
+		sub_v3_v3v3(r_center, r_center, ob->obmat[3]);
 		copy_m3_m4(mat, ob->obmat);
 		invert_m3_m3(imat, mat);
-		mul_m3_v3(imat, t->center);
+		mul_m3_v3(imat, r_center);
+	}
+	else if (t->options & CTX_PAINT_CURVE) {
+		if (ED_view3d_project_float_global(t->ar, cursor, r_center, V3D_PROJ_TEST_NOP) != V3D_PROJ_RET_OK) {
+			r_center[0] = t->ar->winx / 2.0f;
+			r_center[1] = t->ar->winy / 2.0f;
+		}
+		r_center[2] = 0.0f;
 	}
-	
-	calculateCenter2D(t);
 }
 
-void calculateCenterCursor2D(TransInfo *t)
+void calculateCenterCursor2D(TransInfo *t, float r_center[2])
 {
 	float aspx = 1.0, aspy = 1.0;
-	float *cursor = NULL;
+	const float *cursor = NULL;
 	
 	if (t->spacetype == SPACE_IMAGE) {
 		SpaceImage *sima = (SpaceImage *)t->sa->spacedata.first;
-		/* only space supported right now but may change */
 		if (t->options & CTX_MASK) {
 			ED_space_image_get_aspect(sima, &aspx, &aspy);
 		}
@@ -1480,43 +1581,60 @@ void calculateCenterCursor2D(TransInfo *t)
 		}
 		cursor = sima->cursor;
 	}
+	else if (t->spacetype == SPACE_CLIP) {
+		SpaceClip *space_clip = (SpaceClip *) t->sa->spacedata.first;
+		if (t->options & CTX_MOVIECLIP) {
+			ED_space_clip_get_aspect_dimension_aware(space_clip, &aspx, &aspy);
+		}
+		else {
+			ED_space_clip_get_aspect(space_clip, &aspx, &aspy);
+		}
+		cursor = space_clip->cursor;
+	}
 	
 	if (cursor) {
 		if (t->options & CTX_MASK) {
 			float co[2];
-			float frame_size[2];
-			SpaceImage *sima = (SpaceImage *)t->sa->spacedata.first;
-			ED_space_image_get_size_fl(sima, frame_size);
 
-			BKE_mask_coord_from_frame(co, cursor, frame_size);
+			if (t->spacetype == SPACE_IMAGE) {
+				SpaceImage *sima = (SpaceImage *)t->sa->spacedata.first;
+				BKE_mask_coord_from_image(sima->image, &sima->iuser, co, cursor);
+			}
+			else if (t->spacetype == SPACE_CLIP) {
+				SpaceClip *space_clip = (SpaceClip *) t->sa->spacedata.first;
+				BKE_mask_coord_from_movieclip(space_clip->clip, &space_clip->user, co, cursor);
+			}
+			else {
+				BLI_assert(!"Shall not happen");
+			}
 
-			ED_space_image_get_aspect(sima, &aspx, &aspy);
-
-			t->center[0] = co[0] * aspx;
-			t->center[1] = co[1] * aspy;
+			r_center[0] = co[0] * aspx;
+			r_center[1] = co[1] * aspy;
+		}
+		else if (t->options & CTX_PAINT_CURVE) {
+			if (t->spacetype == SPACE_IMAGE) {
+				r_center[0] = UI_view2d_view_to_region_x(&t->ar->v2d, cursor[0]);
+				r_center[1] = UI_view2d_view_to_region_y(&t->ar->v2d, cursor[1]);
+			}
 		}
 		else {
-			t->center[0] = cursor[0] * aspx;
-			t->center[1] = cursor[1] * aspy;
+			r_center[0] = cursor[0] * aspx;
+			r_center[1] = cursor[1] * aspy;
 		}
 	}
-	
-	calculateCenter2D(t);
 }
 
-static void calculateCenterCursorGraph2D(TransInfo *t)
+void calculateCenterCursorGraph2D(TransInfo *t, float r_center[2])
 {
 	SpaceIpo *sipo = (SpaceIpo *)t->sa->spacedata.first;
 	Scene *scene = t->scene;
 	
 	/* cursor is combination of current frame, and graph-editor cursor value */
-	t->center[0] = (float)(scene->r.cfra);
-	t->center[1] = sipo->cursorVal;
-	
-	calculateCenter2D(t);
+	r_center[0] = (float)(scene->r.cfra);
+	r_center[1] = sipo->cursorVal;
 }
 
-void calculateCenterMedian(TransInfo *t)
+void calculateCenterMedian(TransInfo *t, float r_center[3])
 {
 	float partial[3] = {0.0f, 0.0f, 0.0f};
 	int total = 0;
@@ -1529,22 +1647,14 @@ void calculateCenterMedian(TransInfo *t)
 				total++;
 			}
 		}
-		else {
-			/*
-			 * All the selected elements are at the head of the array
-			 * which means we can stop when it finds unselected data
-			 */
-			break;
-		}
 	}
-	if (i)
-		mul_v3_fl(partial, 1.0f / total);
-	copy_v3_v3(t->center, partial);
-	
-	calculateCenter2D(t);
+	if (total) {
+		mul_v3_fl(partial, 1.0f / (float)total);
+	}
+	copy_v3_v3(r_center, partial);
 }
 
-void calculateCenterBound(TransInfo *t)
+void calculateCenterBound(TransInfo *t, float r_center[3])
 {
 	float max[3];
 	float min[3];
@@ -1555,88 +1665,151 @@ void calculateCenterBound(TransInfo *t)
 				if (!(t->data[i].flag & TD_NOCENTER))
 					minmax_v3v3_v3(min, max, t->data[i].center);
 			}
-			else {
-				/*
-				 * All the selected elements are at the head of the array
-				 * which means we can stop when it finds unselected data
-				 */
-				break;
-			}
 		}
 		else {
 			copy_v3_v3(max, t->data[i].center);
 			copy_v3_v3(min, t->data[i].center);
 		}
 	}
-	add_v3_v3v3(t->center, min, max);
-	mul_v3_fl(t->center, 0.5);
-	
-	calculateCenter2D(t);
+	mid_v3_v3v3(r_center, min, max);
 }
 
+/**
+ * \param select_only only get active center from data being transformed.
+ */
+bool calculateCenterActive(TransInfo *t, bool select_only, float r_center[3])
+{
+	bool ok = false;
+
+	if (t->obedit) {
+		switch (t->obedit->type) {
+			case OB_MESH:
+			{
+				BMEditSelection ese;
+				BMEditMesh *em = BKE_editmesh_from_object(t->obedit);
+
+				if (BM_select_history_active_get(em->bm, &ese)) {
+					BM_editselection_center(&ese, r_center);
+					ok = true;
+				}
+				break;
+			}
+			case OB_ARMATURE:
+			{
+				bArmature *arm = t->obedit->data;
+				EditBone *ebo = arm->act_edbone;
+
+				if (ebo && (!select_only || (ebo->flag & (BONE_SELECTED | BONE_ROOTSEL)))) {
+					copy_v3_v3(r_center, ebo->head);
+					ok = true;
+				}
+
+				break;
+			}
+			case OB_CURVE:
+			case OB_SURF:
+			{
+				float center[3];
+				Curve *cu = (Curve *)t->obedit->data;
+
+				if (ED_curve_active_center(cu, center)) {
+					copy_v3_v3(r_center, center);
+					ok = true;
+				}
+				break;
+			}
+			case OB_MBALL:
+			{
+				MetaBall *mb = (MetaBall *)t->obedit->data;
+				MetaElem *ml_act = mb->lastelem;
+
+				if (ml_act && (!select_only || (ml_act->flag & SELECT))) {
+					copy_v3_v3(r_center, &ml_act->x);
+					ok = true;
+				}
+				break;
+			}
+			case OB_LATTICE:
+			{
+				BPoint *actbp = BKE_lattice_active_point_get(t->obedit->data);
+
+				if (actbp) {
+					copy_v3_v3(r_center, actbp->vec);
+					ok = true;
+				}
+				break;
+			}
+		}
+	}
+	else if (t->flag & T_POSE) {
+		Scene *scene = t->scene;
+		Object *ob = OBACT;
+		if (ob) {
+			bPoseChannel *pchan = BKE_pose_channel_active(ob);
+			if (pchan && (!select_only || (pchan->bone->flag & BONE_SELECTED))) {
+				copy_v3_v3(r_center, pchan->pose_head);
+				ok = true;
+			}
+		}
+	}
+	else if (t->options & CTX_PAINT_CURVE) {
+		Paint *p = BKE_paint_get_active(t->scene);
+		Brush *br = p->brush;
+		PaintCurve *pc = br->paint_curve;
+		copy_v3_v3(r_center, pc->points[pc->add_index - 1].bez.vec[1]);
+		r_center[2] = 0.0f;
+		ok = true;
+	}
+	else {
+		/* object mode */
+		Scene *scene = t->scene;
+		Object *ob = OBACT;
+		if (ob && (!select_only || (ob->flag & SELECT))) {
+			copy_v3_v3(r_center, ob->obmat[3]);
+			ok = true;
+		}
+	}
+
+	return ok;
+}
+
+
 void calculateCenter(TransInfo *t)
 {
 	switch (t->around) {
 		case V3D_CENTER:
-			calculateCenterBound(t);
+			calculateCenterBound(t, t->center);
 			break;
 		case V3D_CENTROID:
-			calculateCenterMedian(t);
+			calculateCenterMedian(t, t->center);
 			break;
 		case V3D_CURSOR:
-			if (t->spacetype == SPACE_IMAGE)
-				calculateCenterCursor2D(t);
+			if (ELEM(t->spacetype, SPACE_IMAGE, SPACE_CLIP))
+				calculateCenterCursor2D(t, t->center);
 			else if (t->spacetype == SPACE_IPO)
-				calculateCenterCursorGraph2D(t);
+				calculateCenterCursorGraph2D(t, t->center);
 			else
-				calculateCenterCursor(t);
+				calculateCenterCursor(t, t->center);
 			break;
 		case V3D_LOCAL:
 			/* Individual element center uses median center for helpline and such */
-			calculateCenterMedian(t);
+			calculateCenterMedian(t, t->center);
 			break;
 		case V3D_ACTIVE:
 		{
-			/* set median, and if if if... do object center */
-		
-			/* EDIT MODE ACTIVE EDITMODE ELEMENT */
-
-			if (t->obedit) {
-				if (t->obedit && t->obedit->type == OB_MESH) {
-					BMEditSelection ese;
-					BMEditMesh *em = BMEdit_FromObject(t->obedit);
-
-					if (BM_select_history_active_get(em->bm, &ese)) {
-						BM_editselection_center(&ese, t->center);
-						calculateCenter2D(t);
-						break;
-					}
-				}
-				else if (ELEM(t->obedit->type, OB_CURVE, OB_SURF)) {
-					float center[3];
-					Curve *cu = (Curve *)t->obedit->data;
-
-					if (ED_curve_actSelection(cu, center)) {
-						copy_v3_v3(t->center, center);
-						calculateCenter2D(t);
-						break;
-					}
-				}
-			} /* END EDIT MODE ACTIVE ELEMENT */
-
-			calculateCenterMedian(t);
-			if ((t->flag & (T_EDIT | T_POSE)) == 0) {
-				Scene *scene = t->scene;
-				Object *ob = OBACT;
-				if (ob) {
-					copy_v3_v3(t->center, ob->obmat[3]);
-					projectIntView(t, t->center, t->center2d);
-				}
+			if (calculateCenterActive(t, false, t->center)) {
+				/* pass */
 			}
-		
+			else {
+				/* fallback */
+				calculateCenterMedian(t, t->center);
+			}
+			break;
 		}
 	}
-	
+
+	calculateCenter2D(t);
+
 	/* setting constraint center */
 	copy_v3_v3(t->con.center, t->center);
 	if (t->flag & (T_EDIT | T_POSE)) {
@@ -1647,11 +1820,8 @@ void calculateCenter(TransInfo *t)
 	/* for panning from cameraview */
 	if (t->flag & T_OBJECT) {
 		if (t->spacetype == SPACE_VIEW3D && t->ar && t->ar->regiontype == RGN_TYPE_WINDOW) {
-			View3D *v3d = t->view;
-			Scene *scene = t->scene;
-			RegionView3D *rv3d = t->ar->regiondata;
 			
-			if (v3d->camera == OBACT && rv3d->persp == RV3D_CAMOB) {
+			if (t->flag & T_CAMERA) {
 				float axis[3];
 				/* persinv is nasty, use viewinv instead, always right */
 				copy_v3_v3(axis, t->viewinv[2]);
@@ -1662,9 +1832,9 @@ void calculateCenter(TransInfo *t)
 				axis[1] = t->center[1] - 6.0f * axis[1];
 				axis[2] = t->center[2] - 6.0f * axis[2];
 				
-				projectIntView(t, axis, t->center2d);
+				projectFloatView(t, axis, t->center2d);
 				
-				/* rotate only needs correct 2d center, grab needs initgrabz() value */
+				/* rotate only needs correct 2d center, grab needs ED_view3d_calc_zfac() value */
 				if (t->mode == TFM_TRANSLATION) {
 					copy_v3_v3(t->center, axis);
 					copy_v3_v3(t->con.center, t->center);
@@ -1674,17 +1844,27 @@ void calculateCenter(TransInfo *t)
 	}
 	
 	if (t->spacetype == SPACE_VIEW3D) {
-		/* initgrabz() defines a factor for perspective depth correction, used in window_to_3d_delta() */
+		/* ED_view3d_calc_zfac() defines a factor for perspective depth correction, used in ED_view3d_win_to_delta() */
+		float vec[3];
 		if (t->flag & (T_EDIT | T_POSE)) {
 			Object *ob = t->obedit ? t->obedit : t->poseobj;
-			float vec[3];
-			
-			copy_v3_v3(vec, t->center);
-			mul_m4_v3(ob->obmat, vec);
-			initgrabz(t->ar->regiondata, vec[0], vec[1], vec[2]);
+			mul_v3_m4v3(vec, ob->obmat, t->center);
 		}
 		else {
-			initgrabz(t->ar->regiondata, t->center[0], t->center[1], t->center[2]);
+			copy_v3_v3(vec, t->center);
+		}
+
+		/* zfac is only used convertViewVec only in cases operator was invoked in RGN_TYPE_WINDOW
+		 * and never used in other cases.
+		 *
+		 * We need special case here as well, since ED_view3d_calc_zfac will crahs when called
+		 * for a region different from RGN_TYPE_WINDOW.
+		 */
+		if (t->ar->regiontype == RGN_TYPE_WINDOW) {
+			t->zfac = ED_view3d_calc_zfac(t->ar->regiondata, vec, NULL);
+		}
+		else {
+			t->zfac = 0.0f;
 		}
 	}
 }
@@ -1712,8 +1892,9 @@ void calculatePropRatio(TransInfo *t)
 				/*
 				 * The elements are sorted according to their dist member in the array,
 				 * that means we can stop when it finds one element outside of the propsize.
+				 * do not set 'td->flag |= TD_NOACTION', the prop circle is being changed.
 				 */
-				td->flag |= TD_NOACTION;
+				
 				td->factor = 0.0f;
 				restoreElement(td);
 			}
@@ -1742,7 +1923,7 @@ void calculatePropRatio(TransInfo *t)
 						td->factor = 3.0f * dist * dist - 2.0f * dist * dist * dist;
 						break;
 					case PROP_ROOT:
-						td->factor = (float)sqrt(dist);
+						td->factor = sqrtf(dist);
 						break;
 					case PROP_LIN:
 						td->factor = dist;
@@ -1751,41 +1932,42 @@ void calculatePropRatio(TransInfo *t)
 						td->factor = 1.0f;
 						break;
 					case PROP_SPHERE:
-						td->factor = (float)sqrt(2 * dist - dist * dist);
+						td->factor = sqrtf(2 * dist - dist * dist);
 						break;
 					case PROP_RANDOM:
-						BLI_srand(BLI_rand()); /* random seed */
 						td->factor = BLI_frand() * dist;
 						break;
 					default:
 						td->factor = 1;
+						break;
 				}
 			}
 		}
 		switch (t->prop_mode) {
 			case PROP_SHARP:
-				strcpy(t->proptext, "(Sharp)");
+				strcpy(t->proptext, IFACE_("(Sharp)"));
 				break;
 			case PROP_SMOOTH:
-				strcpy(t->proptext, "(Smooth)");
+				strcpy(t->proptext, IFACE_("(Smooth)"));
 				break;
 			case PROP_ROOT:
-				strcpy(t->proptext, "(Root)");
+				strcpy(t->proptext, IFACE_("(Root)"));
 				break;
 			case PROP_LIN:
-				strcpy(t->proptext, "(Linear)");
+				strcpy(t->proptext, IFACE_("(Linear)"));
 				break;
 			case PROP_CONST:
-				strcpy(t->proptext, "(Constant)");
+				strcpy(t->proptext, IFACE_("(Constant)"));
 				break;
 			case PROP_SPHERE:
-				strcpy(t->proptext, "(Sphere)");
+				strcpy(t->proptext, IFACE_("(Sphere)"));
 				break;
 			case PROP_RANDOM:
-				strcpy(t->proptext, "(Random)");
+				strcpy(t->proptext, IFACE_("(Random)"));
 				break;
 			default:
 				t->proptext[0] = '\0';
+				break;
 		}
 	}
 	else {
