@@ -35,7 +35,7 @@
 
 #include "DNA_listBase.h"
 #include "GPU_material.h"
-#include "GL/glew.h"
+#include "GPU_glew.h"
 
 struct ListBase;
 struct GPUShader;
@@ -43,6 +43,7 @@ struct GPUOutput;
 struct GPUNode;
 struct GPUVertexAttribs;
 struct GPUFrameBuffer;
+struct PreviewImage;
 
 #define MAX_FUNCTION_NAME	64
 #define MAX_PARAMETER		32
@@ -69,6 +70,7 @@ GPUFunction *GPU_lookup_function(const char *name);
 typedef enum GPUDataSource {
 	GPU_SOURCE_VEC_UNIFORM,
 	GPU_SOURCE_BUILTIN,
+	GPU_SOURCE_OPENGL_BUILTIN,
 	GPU_SOURCE_TEX_PIXEL,
 	GPU_SOURCE_TEX,
 	GPU_SOURCE_ATTRIB
@@ -91,7 +93,7 @@ struct GPUNodeLink {
 	const char *attribname;
 
 	int image;
-	int imagencd;
+	int image_isdata;
 
 	int texture;
 	int texturesize;
@@ -99,14 +101,15 @@ struct GPUNodeLink {
 	void *ptr1, *ptr2;
 
 	int dynamic;
-	int dynamictype;	
+	int dynamictype;
 
 	int type;
 	int users;
 
-	GPUTexture *dynamictex;
+	struct GPUTexture *dynamictex;
 
 	GPUBuiltin builtin;
+	GPUOpenGLBuiltin oglbuiltin;
 
 	struct GPUOutput *output;
 };
@@ -138,11 +141,12 @@ typedef struct GPUInput {
 
 	struct Image *ima;		/* image */
 	struct ImageUser *iuser;/* image user */
-	int imagencd;			/* image does not contain color data */
+	struct PreviewImage *prv;	/* preview images & icons */
+	int image_isdata;		/* image does not contain color data */
 	float *dynamicvec;		/* vector data in case it is dynamic */
 	int dynamictype;		/* origin of the dynamic uniform (GPUDynamicType) */
 	void *dynamicdata;		/* data source of the dynamic uniform */
-	GPUTexture *tex;		/* input texture, only set at runtime */
+	struct GPUTexture *tex;	/* input texture, only set at runtime */
 	int shaderloc;			/* id from opengl */
 	char shadername[32];	/* name in shader */
 
@@ -153,6 +157,7 @@ typedef struct GPUInput {
 	char attribname[32];	/* attribute name */
 	int attribfirst;		/* this is the first one that is bound */
 	GPUBuiltin builtin;		/* builtin uniform */
+	GPUOpenGLBuiltin oglbuiltin; /* opengl built in varying */
 } GPUInput;
 
 struct GPUPass {
@@ -170,7 +175,7 @@ struct GPUPass {
 typedef struct GPUPass GPUPass;
 
 GPUPass *GPU_generate_pass(ListBase *nodes, struct GPUNodeLink *outlink,
-	struct GPUVertexAttribs *attribs, int *builtin, const char *name);
+	struct GPUVertexAttribs *attribs, int *builtin, int type, const char *name);
 
 struct GPUShader *GPU_pass_shader(GPUPass *pass);
 
@@ -180,8 +185,8 @@ void GPU_pass_unbind(GPUPass *pass);
 
 void GPU_pass_free(GPUPass *pass);
 
-void GPU_codegen_init(void);
-void GPU_codegen_exit(void);
+void gpu_codegen_init(void);
+void gpu_codegen_exit(void);
 
 /* Material calls */
 
