@@ -27,25 +27,29 @@
 
 /** \file blender/gpu/intern/gpu_extensions.c
  *  \ingroup gpu
+ *
+ * Wrap OpenGL features such as textures, shaders and GLSL
+ * with checks for drivers and GPU support.
  */
 
 
-#include "GL/glew.h"
+#include "GPU_glew.h"
 
 #include "DNA_image_types.h"
 
 #include "MEM_guardedalloc.h"
 
-#include "BKE_global.h"
-
-
 #include "BLI_blenlib.h"
 #include "BLI_utildefines.h"
 #include "BLI_math_base.h"
 
+#include "BKE_global.h"
+
 #include "GPU_draw.h"
 #include "GPU_extensions.h"
-#include "gpu_codegen.h"
+#include "GPU_simple_shader.h"
+
+#include "intern/gpu_extensions_private.h"
 
 #include <stdlib.h>
 #include <stdio.h>
@@ -55,6 +59,8 @@
 #  include "BLI_winstuff.h"
 #endif
 
+#define MAX_DEFINE_LENGTH 72
+
 /* Extensions support */
 
 /* extensions used:
@@ -79,18 +85,33 @@ typedef struct GPUShaders {
 } GPUShaders;
 
 static struct GPUGlobal {
+	GLint maxtexsize;
 	GLint maxtextures;
 	GLuint currentfb;
 	int glslsupport;
 	int extdisabled;
 	int colordepth;
 	int npotdisabled; /* ATI 3xx-5xx (and more) chipsets support NPoT partially (== not enough) */
+	int dlistsdisabled; /* Legacy ATI driver does not support display lists well */
 	GPUDeviceType device;
 	GPUOSType os;
 	GPUDriverType driver;
 	GPUShaders shaders;
+	GPUTexture *invalid_tex_1D; /* texture used in place of invalid textures (not loaded correctly, missing) */
+	GPUTexture *invalid_tex_2D;
+	GPUTexture *invalid_tex_3D;
 } GG = {1, 0};
 
+/* Number of maximum output slots. We support 4 outputs for now (usually we wouldn't need more to preserve fill rate) */
+#define GPU_FB_MAX_SLOTS 4
+
+struct GPUFrameBuffer {
+	GLuint object;
+	GPUTexture *colortex[GPU_FB_MAX_SLOTS];
+	GPUTexture *depthtex;
+};
+
+
 /* GPU Types */
 
 int GPU_type_matches(GPUDeviceType device, GPUOSType os, GPUDriverType driver)
@@ -100,30 +121,28 @@ int GPU_type_matches(GPUDeviceType device, GPUOSType os, GPUDriverType driver)
 
 /* GPU Extensions */
 
-static int gpu_extensions_init = 0;
-
 void GPU_extensions_disable(void)
 {
 	GG.extdisabled = 1;
 }
 
-void GPU_extensions_init(void)
+int GPU_max_texture_size(void)
+{
+	return GG.maxtexsize;
+}
+
+void gpu_extensions_init(void)
 {
 	GLint r, g, b;
 	const char *vendor, *renderer;
 
-	/* can't avoid calling this multiple times, see wm_window_add_ghostwindow */
-	if (gpu_extensions_init) return;
-	gpu_extensions_init= 1;
-
-	glewInit();
-	GPU_codegen_init();
-
 	/* glewIsSupported("GL_VERSION_2_0") */
 
 	if (GLEW_ARB_multitexture)
 		glGetIntegerv(GL_MAX_TEXTURE_IMAGE_UNITS_ARB, &GG.maxtextures);
 
+	glGetIntegerv(GL_MAX_TEXTURE_SIZE, &GG.maxtexsize);
+
 	GG.glslsupport = 1;
 	if (!GLEW_ARB_multitexture) GG.glslsupport = 0;
 	if (!GLEW_ARB_vertex_shader) GG.glslsupport = 0;
@@ -134,8 +153,8 @@ void GPU_extensions_init(void)
 	glGetIntegerv(GL_BLUE_BITS, &b);
 	GG.colordepth = r+g+b; /* assumes same depth for RGB */
 
-	vendor = (const char*)glGetString(GL_VENDOR);
-	renderer = (const char*)glGetString(GL_RENDERER);
+	vendor = (const char *)glGetString(GL_VENDOR);
+	renderer = (const char *)glGetString(GL_RENDERER);
 
 	if (strstr(vendor, "ATI")) {
 		GG.device = GPU_DEVICE_ATI;
@@ -182,6 +201,9 @@ void GPU_extensions_init(void)
 		 * Incomplete list http://dri.freedesktop.org/wiki/ATIRadeon
 		 * New IDs from MESA's src/gallium/drivers/r300/r300_screen.c
 		 */
+		/* This list is close enough to those using the legacy driver which
+		 * has a bug with display lists and glVertexAttrib 
+		 */
 		if (strstr(renderer, "R3") || strstr(renderer, "RV3") ||
 		    strstr(renderer, "R4") || strstr(renderer, "RV4") ||
 		    strstr(renderer, "RS4") || strstr(renderer, "RC4") ||
@@ -192,6 +214,7 @@ void GPU_extensions_init(void)
 		    strstr(renderer, "RADEON 9"))
 		{
 			GG.npotdisabled = 1;
+			GG.dlistsdisabled = 1;
 		}
 	}
 
@@ -206,12 +229,16 @@ void GPU_extensions_init(void)
 #else
 	GG.os = GPU_OS_UNIX;
 #endif
+
+
+	GPU_invalid_tex_init();
+	GPU_simple_shaders_init();
 }
 
-void GPU_extensions_exit(void)
+void gpu_extensions_exit(void)
 {
-	gpu_extensions_init = 0;
-	GPU_codegen_exit();
+	GPU_simple_shaders_exit();
+	GPU_invalid_tex_free();
 }
 
 int GPU_glsl_support(void)
@@ -227,6 +254,11 @@ int GPU_non_power_of_two_support(void)
 	return GLEW_ARB_texture_non_power_of_two;
 }
 
+int GPU_display_list_support(void)
+{
+	return !GG.dlistsdisabled;
+}
+
 int GPU_color_depth(void)
 {
 	return GG.colordepth;
@@ -236,7 +268,7 @@ int GPU_print_error(const char *str)
 {
 	GLenum errCode;
 
-	if (G.debug & G_DEBUG) {
+	if ((G.debug & G_DEBUG)) {
 		if ((errCode = glGetError()) != GL_NO_ERROR) {
 			fprintf(stderr, "%s opengl error: %s\n", str, gluErrorString(errCode));
 			return 1;
@@ -253,6 +285,9 @@ static void GPU_print_framebuffer_error(GLenum status, char err_out[256])
 	switch (status) {
 		case GL_FRAMEBUFFER_COMPLETE_EXT:
 			break;
+		case GL_INVALID_OPERATION:
+			err= "Invalid operation";
+			break;
 		case GL_FRAMEBUFFER_INCOMPLETE_ATTACHMENT_EXT:
 			err= "Incomplete attachment";
 			break;
@@ -297,13 +332,14 @@ struct GPUTexture {
 	int fromblender;		/* we got the texture from Blender */
 
 	GPUFrameBuffer *fb;		/* GPUFramebuffer this texture is attached to */
+	int fb_attachment;		/* slot the texture is attached to */
 	int depth;				/* is a depth texture? */
 };
 
 static unsigned char *GPU_texture_convert_pixels(int length, float *fpixels)
 {
 	unsigned char *pixels, *p;
-	float *fp;
+	const float *fp;
 	int a, len;
 
 	len = 4*length;
@@ -344,6 +380,7 @@ static GPUTexture *GPU_texture_create_nD(int w, int h, int n, float *fpixels, in
 	tex->refcount = 1;
 	tex->target = (n == 1)? GL_TEXTURE_1D: GL_TEXTURE_2D;
 	tex->depth = depth;
+	tex->fb_attachment = -1;
 
 	glGenTextures(1, &tex->bindcode);
 
@@ -387,7 +424,7 @@ static GPUTexture *GPU_texture_create_nD(int w, int h, int n, float *fpixels, in
 
 		if (fpixels) {
 			glTexSubImage1D(tex->target, 0, 0, w, format, type,
-				pixels? pixels: fpixels);
+				pixels ? pixels : fpixels);
 
 			if (tex->w > w)
 				GPU_glTexSubImageEmpty(tex->target, format, w, 0,
@@ -396,11 +433,11 @@ static GPUTexture *GPU_texture_create_nD(int w, int h, int n, float *fpixels, in
 	}
 	else {
 		glTexImage2D(tex->target, 0, internalformat, tex->w, tex->h, 0,
-			format, type, NULL);
+		             format, type, NULL);
 
 		if (fpixels) {
 			glTexSubImage2D(tex->target, 0, 0, 0, w, h,
-				format, type, pixels? pixels: fpixels);
+				format, type, pixels ? pixels : fpixels);
 
 			if (tex->w > w)
 				GPU_glTexSubImageEmpty(tex->target, format, w, 0, tex->w-w, tex->h);
@@ -442,7 +479,7 @@ static GPUTexture *GPU_texture_create_nD(int w, int h, int n, float *fpixels, in
 }
 
 
-GPUTexture *GPU_texture_create_3D(int w, int h, int depth, float *fpixels)
+GPUTexture *GPU_texture_create_3D(int w, int h, int depth, int channels, float *fpixels)
 {
 	GPUTexture *tex;
 	GLenum type, format, internalformat;
@@ -480,9 +517,15 @@ GPUTexture *GPU_texture_create_3D(int w, int h, int depth, float *fpixels)
 
 	GPU_print_error("3D glBindTexture");
 
-	type = GL_FLOAT; // GL_UNSIGNED_BYTE
-	format = GL_RED;
-	internalformat = GL_INTENSITY;
+	type = GL_FLOAT;
+	if (channels == 4) {
+		format = GL_RGBA;
+		internalformat = GL_RGBA;
+	}
+	else {
+		format = GL_RED;
+		internalformat = GL_INTENSITY;
+	}
 
 	//if (fpixels)
 	//	pixels = GPU_texture_convert_pixels(w*h*depth, fpixels);
@@ -522,7 +565,7 @@ GPUTexture *GPU_texture_create_3D(int w, int h, int depth, float *fpixels)
 	return tex;
 }
 
-GPUTexture *GPU_texture_from_blender(Image *ima, ImageUser *iuser, int ncd, double time, int mipmap)
+GPUTexture *GPU_texture_from_blender(Image *ima, ImageUser *iuser, bool is_data, double time, int mipmap)
 {
 	GPUTexture *tex;
 	GLint w, h, border, lastbindcode, bindcode;
@@ -530,7 +573,8 @@ GPUTexture *GPU_texture_from_blender(Image *ima, ImageUser *iuser, int ncd, doub
 	glGetIntegerv(GL_TEXTURE_BINDING_2D, &lastbindcode);
 
 	GPU_update_image_time(ima, time);
-	bindcode = GPU_verify_image(ima, iuser, 0, 0, mipmap, ncd);
+	/* this binds a texture, so that's why to restore it with lastbindcode */
+	bindcode = GPU_verify_image(ima, iuser, 0, 0, mipmap, is_data);
 
 	if (ima->gputexture) {
 		ima->gputexture->bindcode = bindcode;
@@ -538,11 +582,6 @@ GPUTexture *GPU_texture_from_blender(Image *ima, ImageUser *iuser, int ncd, doub
 		return ima->gputexture;
 	}
 
-	if (!bindcode) {
-		glBindTexture(GL_TEXTURE_2D, lastbindcode);
-		return NULL;
-	}
-
 	tex = MEM_callocN(sizeof(GPUTexture), "GPUTexture");
 	tex->bindcode = bindcode;
 	tex->number = -1;
@@ -553,7 +592,7 @@ GPUTexture *GPU_texture_from_blender(Image *ima, ImageUser *iuser, int ncd, doub
 	ima->gputexture= tex;
 
 	if (!glIsTexture(tex->bindcode)) {
-		GPU_print_error("Blender Texture");
+		GPU_print_error("Blender Texture Not Loaded");
 	}
 	else {
 		glBindTexture(GL_TEXTURE_2D, tex->bindcode);
@@ -570,6 +609,53 @@ GPUTexture *GPU_texture_from_blender(Image *ima, ImageUser *iuser, int ncd, doub
 	return tex;
 }
 
+GPUTexture *GPU_texture_from_preview(PreviewImage *prv, int mipmap)
+{
+	GPUTexture *tex = prv->gputexture[0];
+	GLint w, h, lastbindcode;
+	GLuint bindcode = 0;
+	
+	glGetIntegerv(GL_TEXTURE_BINDING_2D, &lastbindcode);
+	
+	if (tex)
+		bindcode = tex->bindcode;
+	
+	/* this binds a texture, so that's why to restore it */
+	if (bindcode == 0) {
+		GPU_create_gl_tex(&bindcode, prv->rect[0], NULL, prv->w[0], prv->h[0], mipmap, 0, NULL);
+	}
+	if (tex) {
+		tex->bindcode = bindcode;
+		glBindTexture(GL_TEXTURE_2D, lastbindcode);
+		return tex;
+	}
+
+	tex = MEM_callocN(sizeof(GPUTexture), "GPUTexture");
+	tex->bindcode = bindcode;
+	tex->number = -1;
+	tex->refcount = 1;
+	tex->target = GL_TEXTURE_2D;
+	
+	prv->gputexture[0]= tex;
+	
+	if (!glIsTexture(tex->bindcode)) {
+		GPU_print_error("Blender Texture Not Loaded");
+	}
+	else {
+		glBindTexture(GL_TEXTURE_2D, tex->bindcode);
+		glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_WIDTH, &w);
+		glGetTexLevelParameteriv(GL_TEXTURE_2D, 0, GL_TEXTURE_HEIGHT, &h);
+		
+		tex->w = w;
+		tex->h = h;
+	}
+	
+	glBindTexture(GL_TEXTURE_2D, lastbindcode);
+	
+	return tex;
+
+}
+
 GPUTexture *GPU_texture_create_1D(int w, float *fpixels, char err_out[256])
 {
 	GPUTexture *tex = GPU_texture_create_nD(w, 1, 1, fpixels, 0, err_out);
@@ -611,7 +697,7 @@ GPUTexture *GPU_texture_create_vsm_shadow_map(int size, char err_out[256])
 		/* Now we tweak some of the settings */
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
 		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
-		glTexImage2D(GL_TEXTURE_2D, 0, GL_RG32F, size, size, 0, GL_RG, GL_FLOAT, 0);
+		glTexImage2D(GL_TEXTURE_2D, 0, GL_RG32F, size, size, 0, GL_RG, GL_FLOAT, NULL);
 
 		GPU_texture_unbind(tex);
 	}
@@ -619,6 +705,40 @@ GPUTexture *GPU_texture_create_vsm_shadow_map(int size, char err_out[256])
 	return tex;
 }
 
+void GPU_invalid_tex_init(void)
+{
+	float color[4] = {1.0f, 0.0f, 1.0f, 1.0};
+	GG.invalid_tex_1D = GPU_texture_create_1D(1, color, NULL);
+	GG.invalid_tex_2D = GPU_texture_create_2D(1, 1, color, NULL);
+	GG.invalid_tex_3D = GPU_texture_create_3D(1, 1, 1, 4, color);
+}
+
+void GPU_invalid_tex_bind(int mode)
+{
+	switch (mode) {
+		case GL_TEXTURE_1D:
+			glBindTexture(GL_TEXTURE_1D, GG.invalid_tex_1D->bindcode);
+			break;
+		case GL_TEXTURE_2D:
+			glBindTexture(GL_TEXTURE_2D, GG.invalid_tex_2D->bindcode);
+			break;
+		case GL_TEXTURE_3D:
+			glBindTexture(GL_TEXTURE_3D, GG.invalid_tex_3D->bindcode);
+			break;
+	}
+}
+
+void GPU_invalid_tex_free(void)
+{
+	if (GG.invalid_tex_1D)
+		GPU_texture_free(GG.invalid_tex_1D);
+	if (GG.invalid_tex_2D)
+		GPU_texture_free(GG.invalid_tex_2D);
+	if (GG.invalid_tex_3D)
+		GPU_texture_free(GG.invalid_tex_3D);
+}
+
+
 void GPU_texture_bind(GPUTexture *tex, int number)
 {
 	GLenum arbnumber;
@@ -628,14 +748,24 @@ void GPU_texture_bind(GPUTexture *tex, int number)
 		return;
 	}
 
-	if (number == -1)
+	if ((G.debug & G_DEBUG)) {
+		if (tex->fb && tex->fb->object == GG.currentfb) {
+			fprintf(stderr, "Feedback loop warning!: Attempting to bind texture attached to current framebuffer!\n");
+		}
+	}
+
+	if (number < 0)
 		return;
 
 	GPU_print_error("Pre Texture Bind");
 
 	arbnumber = (GLenum)((GLuint)GL_TEXTURE0_ARB + number);
 	if (number != 0) glActiveTextureARB(arbnumber);
-	glBindTexture(tex->target, tex->bindcode);
+	if (tex->bindcode != 0) {
+		glBindTexture(tex->target, tex->bindcode);
+	}
+	else
+		GPU_invalid_tex_bind(tex->target);
 	glEnable(tex->target);
 	if (number != 0) glActiveTextureARB(GL_TEXTURE0_ARB);
 
@@ -678,7 +808,7 @@ void GPU_texture_free(GPUTexture *tex)
 	
 	if (tex->refcount == 0) {
 		if (tex->fb)
-			GPU_framebuffer_texture_detach(tex->fb, tex);
+			GPU_framebuffer_texture_detach(tex);
 		if (tex->bindcode && !tex->fromblender)
 			glDeleteTextures(1, &tex->bindcode);
 
@@ -718,12 +848,6 @@ GPUFrameBuffer *GPU_texture_framebuffer(GPUTexture *tex)
 
 /* GPUFrameBuffer */
 
-struct GPUFrameBuffer {
-	GLuint object;
-	GPUTexture *colortex;
-	GPUTexture *depthtex;
-};
-
 GPUFrameBuffer *GPU_framebuffer_create(void)
 {
 	GPUFrameBuffer *fb;
@@ -741,61 +865,76 @@ GPUFrameBuffer *GPU_framebuffer_create(void)
 		return NULL;
 	}
 
+	/* make sure no read buffer is enabled, so completeness check will not fail. We set those at binding time */
+	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fb->object);
+	glReadBuffer(GL_NONE);
+	glDrawBuffer(GL_NONE);
+	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, 0);
+	
 	return fb;
 }
 
-int GPU_framebuffer_texture_attach(GPUFrameBuffer *fb, GPUTexture *tex, char err_out[256])
+int GPU_framebuffer_texture_attach(GPUFrameBuffer *fb, GPUTexture *tex, int slot, char err_out[256])
 {
-	GLenum status;
 	GLenum attachment;
+	GLenum error;
+
+	if (slot >= GPU_FB_MAX_SLOTS) {
+		fprintf(stderr, "Attaching to index %d framebuffer slot unsupported in blender use at most %d\n", slot, GPU_FB_MAX_SLOTS);
+		return 0;
+	}
+
+	if ((G.debug & G_DEBUG)) {
+		if (tex->number != -1) {
+			fprintf(stderr, "Feedback loop warning!: Attempting to attach texture to framebuffer while still bound to texture unit for drawing!");
+		}
+	}
 
 	if (tex->depth)
 		attachment = GL_DEPTH_ATTACHMENT_EXT;
 	else
-		attachment = GL_COLOR_ATTACHMENT0_EXT;
+		attachment = GL_COLOR_ATTACHMENT0_EXT + slot;
 
 	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fb->object);
 	GG.currentfb = fb->object;
 
+	/* Clean glError buffer. */
+	while (glGetError() != GL_NO_ERROR) {}
+
 	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, attachment, 
 		tex->target, tex->bindcode, 0);
 
-	if (tex->depth) {
-		glDrawBuffer(GL_NONE);
-		glReadBuffer(GL_NONE);
-	}
-	else {
-		glDrawBuffer(GL_COLOR_ATTACHMENT0_EXT);
-		glReadBuffer(GL_COLOR_ATTACHMENT0_EXT);
-	}
+	error = glGetError();
 
-	status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
-
-	if (status != GL_FRAMEBUFFER_COMPLETE_EXT) {
+	if (error == GL_INVALID_OPERATION) {
 		GPU_framebuffer_restore();
-		GPU_print_framebuffer_error(status, err_out);
+		GPU_print_framebuffer_error(error, err_out);
 		return 0;
 	}
 
 	if (tex->depth)
 		fb->depthtex = tex;
 	else
-		fb->colortex = tex;
+		fb->colortex[slot] = tex;
 
 	tex->fb= fb;
+	tex->fb_attachment = slot;
 
 	return 1;
 }
 
-void GPU_framebuffer_texture_detach(GPUFrameBuffer *fb, GPUTexture *tex)
+void GPU_framebuffer_texture_detach(GPUTexture *tex)
 {
 	GLenum attachment;
+	GPUFrameBuffer *fb;
 
 	if (!tex->fb)
 		return;
 
-	if (GG.currentfb != tex->fb->object) {
-		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, tex->fb->object);
+	fb = tex->fb;
+
+	if (GG.currentfb != fb->object) {
+		glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fb->object);
 		GG.currentfb = tex->fb->object;
 	}
 
@@ -804,38 +943,81 @@ void GPU_framebuffer_texture_detach(GPUFrameBuffer *fb, GPUTexture *tex)
 		attachment = GL_DEPTH_ATTACHMENT_EXT;
 	}
 	else {
-		fb->colortex = NULL;
-		attachment = GL_COLOR_ATTACHMENT0_EXT;
+		BLI_assert(fb->colortex[tex->fb_attachment] == tex);
+		fb->colortex[tex->fb_attachment] = NULL;
+		attachment = GL_COLOR_ATTACHMENT0_EXT + tex->fb_attachment;
 	}
 
 	glFramebufferTexture2DEXT(GL_FRAMEBUFFER_EXT, attachment,
 		tex->target, 0, 0);
 
 	tex->fb = NULL;
+	tex->fb_attachment = -1;
 }
 
-void GPU_framebuffer_texture_bind(GPUFrameBuffer *UNUSED(fb), GPUTexture *tex, int w, int h)
+void GPU_texture_bind_as_framebuffer(GPUTexture *tex)
 {
+	if (!tex->fb) {
+		fprintf(stderr, "Error, texture not bound to framebuffer!");
+		return;
+	}
+
 	/* push attributes */
-	glPushAttrib(GL_ENABLE_BIT);
-	glPushAttrib(GL_VIEWPORT_BIT);
+	glPushAttrib(GL_ENABLE_BIT | GL_VIEWPORT_BIT);
 	glDisable(GL_SCISSOR_TEST);
 
 	/* bind framebuffer */
 	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, tex->fb->object);
 
+	if (tex->depth) {
+		glDrawBuffer(GL_NONE);
+		glReadBuffer(GL_NONE);
+	}
+	else {
+		/* last bound prevails here, better allow explicit control here too */
+		glDrawBuffer(GL_COLOR_ATTACHMENT0_EXT + tex->fb_attachment);
+		glReadBuffer(GL_COLOR_ATTACHMENT0_EXT + tex->fb_attachment);
+	}
+	
 	/* push matrices and set default viewport and matrix */
-	glViewport(0, 0, w, h);
+	glViewport(0, 0, tex->w, tex->h);
 	GG.currentfb = tex->fb->object;
 
 	glMatrixMode(GL_PROJECTION);
 	glPushMatrix();
-	glLoadIdentity();
 	glMatrixMode(GL_MODELVIEW);
 	glPushMatrix();
-	glLoadIdentity();
 }
 
+void GPU_framebuffer_slot_bind(GPUFrameBuffer *fb, int slot)
+{
+	if (!fb->colortex[slot]) {
+		fprintf(stderr, "Error, framebuffer slot empty!");
+		return;
+	}
+
+	/* push attributes */
+	glPushAttrib(GL_ENABLE_BIT | GL_VIEWPORT_BIT);
+	glDisable(GL_SCISSOR_TEST);
+
+	/* bind framebuffer */
+	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fb->object);
+
+	/* last bound prevails here, better allow explicit control here too */
+	glDrawBuffer(GL_COLOR_ATTACHMENT0_EXT + slot);
+	glReadBuffer(GL_COLOR_ATTACHMENT0_EXT + slot);
+
+	/* push matrices and set default viewport and matrix */
+	glViewport(0, 0, fb->colortex[slot]->w, fb->colortex[slot]->h);
+	GG.currentfb = fb->object;
+
+	glMatrixMode(GL_PROJECTION);
+	glPushMatrix();
+	glMatrixMode(GL_MODELVIEW);
+	glPushMatrix();
+}
+
+
 void GPU_framebuffer_texture_unbind(GPUFrameBuffer *UNUSED(fb), GPUTexture *UNUSED(tex))
 {
 	/* restore matrix */
@@ -846,16 +1028,42 @@ void GPU_framebuffer_texture_unbind(GPUFrameBuffer *UNUSED(fb), GPUTexture *UNUS
 
 	/* restore attributes */
 	glPopAttrib();
-	glPopAttrib();
-	glEnable(GL_SCISSOR_TEST);
 }
 
+
+bool GPU_framebuffer_check_valid(GPUFrameBuffer *fb, char err_out[256])
+{
+	GLenum status;
+	
+	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fb->object);
+	GG.currentfb = fb->object;
+	
+	/* Clean glError buffer. */
+	while (glGetError() != GL_NO_ERROR) {}
+	
+	status = glCheckFramebufferStatusEXT(GL_FRAMEBUFFER_EXT);
+	
+	if (status != GL_FRAMEBUFFER_COMPLETE_EXT) {
+		GPU_framebuffer_restore();
+		GPU_print_framebuffer_error(status, err_out);
+		return false;
+	}
+	
+	return true;
+}
+
+
 void GPU_framebuffer_free(GPUFrameBuffer *fb)
 {
+	int i;
 	if (fb->depthtex)
-		GPU_framebuffer_texture_detach(fb, fb->depthtex);
-	if (fb->colortex)
-		GPU_framebuffer_texture_detach(fb, fb->colortex);
+		GPU_framebuffer_texture_detach(fb->depthtex);
+
+	for (i = 0; i < GPU_FB_MAX_SLOTS; i++) {
+		if (fb->colortex[i]) {
+			GPU_framebuffer_texture_detach(fb->colortex[i]);
+		}
+	}
 
 	if (fb->object) {
 		glDeleteFramebuffersEXT(1, &fb->object);
@@ -896,9 +1104,13 @@ void GPU_framebuffer_blur(GPUFrameBuffer *fb, GPUTexture *tex, GPUFrameBuffer *b
 	/* We do the bind ourselves rather than using GPU_framebuffer_texture_bind() to avoid
 	 * pushing unnecessary matrices onto the OpenGL stack. */
 	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, blurfb->object);
+	glDrawBuffer(GL_COLOR_ATTACHMENT0);
+	
+	/* avoid warnings from texture binding */
+	GG.currentfb = blurfb->object;
 
 	GPU_shader_bind(blur_shader);
-	GPU_shader_uniform_vector(blur_shader, scale_uniform, 2, 1, (float*)scaleh);
+	GPU_shader_uniform_vector(blur_shader, scale_uniform, 2, 1, (float *)scaleh);
 	GPU_shader_uniform_texture(blur_shader, texture_source_uniform, tex);
 	glViewport(0, 0, GPU_texture_opengl_width(blurtex), GPU_texture_opengl_height(blurtex));
 
@@ -910,6 +1122,8 @@ void GPU_framebuffer_blur(GPUFrameBuffer *fb, GPUTexture *tex, GPUFrameBuffer *b
 	glMatrixMode(GL_PROJECTION);
 	glLoadIdentity();
 
+	glDisable(GL_DEPTH_TEST);
+
 	GPU_texture_bind(tex, 0);
 
 	/* Drawing quad */
@@ -923,8 +1137,12 @@ void GPU_framebuffer_blur(GPUFrameBuffer *fb, GPUTexture *tex, GPUFrameBuffer *b
 	/* Blurring vertically */
 
 	glBindFramebufferEXT(GL_FRAMEBUFFER_EXT, fb->object);
+	glDrawBuffer(GL_COLOR_ATTACHMENT0);
+	
+	GG.currentfb = fb->object;
+	
 	glViewport(0, 0, GPU_texture_opengl_width(tex), GPU_texture_opengl_height(tex));
-	GPU_shader_uniform_vector(blur_shader, scale_uniform, 2, 1, (float*)scalev);
+	GPU_shader_uniform_vector(blur_shader, scale_uniform, 2, 1, (float *)scalev);
 	GPU_shader_uniform_texture(blur_shader, texture_source_uniform, blurtex);
 	GPU_texture_bind(blurtex, 0);
 
@@ -935,7 +1153,7 @@ void GPU_framebuffer_blur(GPUFrameBuffer *fb, GPUTexture *tex, GPUFrameBuffer *b
 	glTexCoord2d(0, 1); glVertex2f(1, -1);
 	glEnd();
 
-	GPU_shader_unbind(blur_shader);
+	GPU_shader_unbind();
 }
 
 /* GPUOffScreen */
@@ -944,10 +1162,6 @@ struct GPUOffScreen {
 	GPUFrameBuffer *fb;
 	GPUTexture *color;
 	GPUTexture *depth;
-
-	/* requested width/height, may be smaller than actual texture size due
-	 * to missing non-power of two support, so we compensate for that */
-	int w, h;
 };
 
 GPUOffScreen *GPU_offscreen_create(int width, int height, char err_out[256])
@@ -955,8 +1169,6 @@ GPUOffScreen *GPU_offscreen_create(int width, int height, char err_out[256])
 	GPUOffScreen *ofs;
 
 	ofs= MEM_callocN(sizeof(GPUOffScreen), "GPUOffScreen");
-	ofs->w= width;
-	ofs->h= height;
 
 	ofs->fb = GPU_framebuffer_create();
 	if (!ofs->fb) {
@@ -970,7 +1182,7 @@ GPUOffScreen *GPU_offscreen_create(int width, int height, char err_out[256])
 		return NULL;
 	}
 
-	if (!GPU_framebuffer_texture_attach(ofs->fb, ofs->depth, err_out)) {
+	if (!GPU_framebuffer_texture_attach(ofs->fb, ofs->depth, 0, err_out)) {
 		GPU_offscreen_free(ofs);
 		return NULL;
 	}
@@ -981,10 +1193,16 @@ GPUOffScreen *GPU_offscreen_create(int width, int height, char err_out[256])
 		return NULL;
 	}
 
-	if (!GPU_framebuffer_texture_attach(ofs->fb, ofs->color, err_out)) {
+	if (!GPU_framebuffer_texture_attach(ofs->fb, ofs->color, 0, err_out)) {
 		GPU_offscreen_free(ofs);
 		return NULL;
 	}
+	
+	/* check validity at the very end! */
+	if (!GPU_framebuffer_check_valid(ofs->fb, err_out)) {
+		GPU_offscreen_free(ofs);
+		return NULL;		
+	}
 
 	GPU_framebuffer_restore();
 
@@ -1006,7 +1224,7 @@ void GPU_offscreen_free(GPUOffScreen *ofs)
 void GPU_offscreen_bind(GPUOffScreen *ofs)
 {
 	glDisable(GL_SCISSOR_TEST);
-	GPU_framebuffer_texture_bind(ofs->fb, ofs->color, ofs->w, ofs->h);
+	GPU_texture_bind_as_framebuffer(ofs->color);
 }
 
 void GPU_offscreen_unbind(GPUOffScreen *ofs)
@@ -1018,7 +1236,17 @@ void GPU_offscreen_unbind(GPUOffScreen *ofs)
 
 void GPU_offscreen_read_pixels(GPUOffScreen *ofs, int type, void *pixels)
 {
-	glReadPixels(0, 0, ofs->w, ofs->h, GL_RGBA, type, pixels);
+	glReadPixels(0, 0, ofs->color->w, ofs->color->h, GL_RGBA, type, pixels);
+}
+
+int GPU_offscreen_width(GPUOffScreen *ofs)
+{
+	return ofs->color->w;
+}
+
+int GPU_offscreen_height(GPUOffScreen *ofs)
+{
+	return ofs->color->h;
 }
 
 /* GPUShader */
@@ -1031,36 +1259,81 @@ struct GPUShader {
 	int totattrib;			/* total number of attributes */
 };
 
-static void shader_print_errors(const char *task, char *log, const char *code)
+static void shader_print_errors(const char *task, char *log, const char **code, int totcode)
 {
-	const char *c, *pos, *end = code + strlen(code);
-	int line = 1;
+	int i;
 
 	fprintf(stderr, "GPUShader: %s error:\n", task);
 
-	if (G.debug & G_DEBUG) {
-		c = code;
-		while ((c < end) && (pos = strchr(c, '\n'))) {
-			fprintf(stderr, "%2d  ", line);
-			fwrite(c, (pos+1)-c, 1, stderr);
-			c = pos+1;
-			line++;
+	for (i = 0; i < totcode; i++) {
+		const char *c, *pos, *end = code[i] + strlen(code[i]);
+		int line = 1;
+
+		if ((G.debug & G_DEBUG)) {
+			fprintf(stderr, "===== shader string %d ====\n", i + 1);
+
+			c = code[i];
+			while ((c < end) && (pos = strchr(c, '\n'))) {
+				fprintf(stderr, "%2d  ", line);
+				fwrite(c, (pos+1)-c, 1, stderr);
+				c = pos+1;
+				line++;
+			}
+			
+			fprintf(stderr, "%s", c);
 		}
-
-		fprintf(stderr, "%s", c);
 	}
-
+	
 	fprintf(stderr, "%s\n", log);
 }
 
-GPUShader *GPU_shader_create(const char *vertexcode, const char *fragcode, /*GPUShader *lib,*/ const char *libcode)
+static const char *gpu_shader_version(void)
+{
+	/* turn on glsl 1.30 for bicubic bump mapping and ATI clipping support */
+	if (GLEW_VERSION_3_0 &&
+	    (GPU_bicubic_bump_support() || GPU_type_matches(GPU_DEVICE_ATI, GPU_OS_ANY, GPU_DRIVER_ANY)))
+	{
+		return "#version 130\n";
+	}
+
+	return "";
+}
+
+
+static const char *gpu_shader_standard_extensions(void)
+{
+	/* need this extensions for high quality bump mapping */
+	if (GPU_bicubic_bump_support())
+		return "#extension GL_ARB_texture_query_lod: enable\n";
+
+	return "";
+}
+
+static void gpu_shader_standard_defines(char defines[MAX_DEFINE_LENGTH])
+{
+	/* some useful defines to detect GPU type */
+	if (GPU_type_matches(GPU_DEVICE_ATI, GPU_OS_ANY, GPU_DRIVER_ANY)) {
+		strcat(defines, "#define GPU_ATI\n");
+		if (GLEW_VERSION_3_0)
+			strcat(defines, "#define CLIP_WORKAROUND\n");
+	}
+	else if (GPU_type_matches(GPU_DEVICE_NVIDIA, GPU_OS_ANY, GPU_DRIVER_ANY))
+		strcat(defines, "#define GPU_NVIDIA\n");
+	else if (GPU_type_matches(GPU_DEVICE_INTEL, GPU_OS_ANY, GPU_DRIVER_ANY))
+		strcat(defines, "#define GPU_INTEL\n");
+
+	if (GPU_bicubic_bump_support())
+		strcat(defines, "#define BUMP_BICUBIC\n");
+	return;
+}
+
+GPUShader *GPU_shader_create(const char *vertexcode, const char *fragcode, const char *libcode, const char *defines)
 {
 	GLint status;
 	GLcharARB log[5000];
-	const char *fragsource[2];
 	GLsizei length = 0;
-	GLint count;
 	GPUShader *shader;
+	char standard_defines[MAX_DEFINE_LENGTH] = "";
 
 	if (!GLEW_ARB_vertex_shader || !GLEW_ARB_fragment_shader)
 		return NULL;
@@ -1082,16 +1355,29 @@ GPUShader *GPU_shader_create(const char *vertexcode, const char *fragcode, /*GPU
 		return NULL;
 	}
 
+	gpu_shader_standard_defines(standard_defines);
+
 	if (vertexcode) {
+		const char *source[5];
+		/* custom limit, may be too small, beware */
+		int num_source = 0;
+
+		source[num_source++] = gpu_shader_version();
+		source[num_source++] = gpu_shader_standard_extensions();
+		source[num_source++] = standard_defines;
+
+		if (defines) source[num_source++] = defines;
+		if (vertexcode) source[num_source++] = vertexcode;
+
 		glAttachObjectARB(shader->object, shader->vertex);
-		glShaderSourceARB(shader->vertex, 1, (const char**)&vertexcode, NULL);
+		glShaderSourceARB(shader->vertex, num_source, source, NULL);
 
 		glCompileShaderARB(shader->vertex);
 		glGetObjectParameterivARB(shader->vertex, GL_OBJECT_COMPILE_STATUS_ARB, &status);
 
 		if (!status) {
 			glGetInfoLogARB(shader->vertex, sizeof(log), &length, log);
-			shader_print_errors("compile", log, vertexcode);
+			shader_print_errors("compile", log, source, num_source);
 
 			GPU_shader_free(shader);
 			return NULL;
@@ -1099,19 +1385,26 @@ GPUShader *GPU_shader_create(const char *vertexcode, const char *fragcode, /*GPU
 	}
 
 	if (fragcode) {
-		count = 0;
-		if (libcode) fragsource[count++] = libcode;
-		if (fragcode) fragsource[count++] = fragcode;
+		const char *source[6];
+		int num_source = 0;
+
+		source[num_source++] = gpu_shader_version();
+		source[num_source++] = gpu_shader_standard_extensions();
+		source[num_source++] = standard_defines;
+
+		if (defines) source[num_source++] = defines;
+		if (libcode) source[num_source++] = libcode;
+		if (fragcode) source[num_source++] = fragcode;
 
 		glAttachObjectARB(shader->object, shader->fragment);
-		glShaderSourceARB(shader->fragment, count, fragsource, NULL);
+		glShaderSourceARB(shader->fragment, num_source, source, NULL);
 
 		glCompileShaderARB(shader->fragment);
 		glGetObjectParameterivARB(shader->fragment, GL_OBJECT_COMPILE_STATUS_ARB, &status);
 
 		if (!status) {
 			glGetInfoLogARB(shader->fragment, sizeof(log), &length, log);
-			shader_print_errors("compile", log, fragcode);
+			shader_print_errors("compile", log, source, num_source);
 
 			GPU_shader_free(shader);
 			return NULL;
@@ -1127,9 +1420,9 @@ GPUShader *GPU_shader_create(const char *vertexcode, const char *fragcode, /*GPU
 	glGetObjectParameterivARB(shader->object, GL_OBJECT_LINK_STATUS_ARB, &status);
 	if (!status) {
 		glGetInfoLogARB(shader->object, sizeof(log), &length, log);
-		if (fragcode) shader_print_errors("linking", log, fragcode);
-		else if (vertexcode) shader_print_errors("linking", log, vertexcode);
-		else if (libcode) shader_print_errors("linking", log, libcode);
+		if (fragcode) shader_print_errors("linking", log, &fragcode, 1);
+		else if (vertexcode) shader_print_errors("linking", log, &vertexcode, 1);
+		else if (libcode) shader_print_errors("linking", log, &libcode, 1);
 
 		GPU_shader_free(shader);
 		return NULL;
@@ -1183,7 +1476,7 @@ void GPU_shader_bind(GPUShader *shader)
 	GPU_print_error("Post Shader Bind");
 }
 
-void GPU_shader_unbind(GPUShader *UNUSED(shader))
+void GPU_shader_unbind(void)
 {
 	GPU_print_error("Pre Shader Unbind");
 	glUseProgramObjectARB(0);
@@ -1225,6 +1518,16 @@ void GPU_shader_uniform_vector(GPUShader *UNUSED(shader), int location, int leng
 	GPU_print_error("Post Uniform Vector");
 }
 
+void GPU_shader_uniform_int(GPUShader *UNUSED(shader), int location, int value)
+{
+	if (location == -1)
+		return;
+
+	GPU_print_error("Pre Uniform Int");
+	glUniform1iARB(location, value);
+	GPU_print_error("Post Uniform Int");
+}
+
 void GPU_shader_uniform_texture(GPUShader *UNUSED(shader), int location, GPUTexture *tex)
 {
 	GLenum arbnumber;
@@ -1245,7 +1548,10 @@ void GPU_shader_uniform_texture(GPUShader *UNUSED(shader), int location, GPUText
 	arbnumber = (GLenum)((GLuint)GL_TEXTURE0_ARB + tex->number);
 
 	if (tex->number != 0) glActiveTextureARB(arbnumber);
-	glBindTexture(tex->target, tex->bindcode);
+	if (tex->bindcode != 0)
+		glBindTexture(tex->target, tex->bindcode);
+	else
+		GPU_invalid_tex_bind(tex->target);
 	glUniform1iARB(location, tex->number);
 	glEnable(tex->target);
 	if (tex->number != 0) glActiveTextureARB(GL_TEXTURE0_ARB);
@@ -1273,12 +1579,12 @@ GPUShader *GPU_shader_get_builtin_shader(GPUBuiltinShader shader)
 	switch (shader) {
 		case GPU_SHADER_VSM_STORE:
 			if (!GG.shaders.vsm_store)
-				GG.shaders.vsm_store = GPU_shader_create(datatoc_gpu_shader_vsm_store_vert_glsl, datatoc_gpu_shader_vsm_store_frag_glsl, NULL);
+				GG.shaders.vsm_store = GPU_shader_create(datatoc_gpu_shader_vsm_store_vert_glsl, datatoc_gpu_shader_vsm_store_frag_glsl, NULL, NULL);
 			retval = GG.shaders.vsm_store;
 			break;
 		case GPU_SHADER_SEP_GAUSSIAN_BLUR:
 			if (!GG.shaders.sep_gaussian_blur)
-				GG.shaders.sep_gaussian_blur = GPU_shader_create(datatoc_gpu_shader_sep_gaussian_blur_vert_glsl, datatoc_gpu_shader_sep_gaussian_blur_frag_glsl, NULL);
+				GG.shaders.sep_gaussian_blur = GPU_shader_create(datatoc_gpu_shader_sep_gaussian_blur_vert_glsl, datatoc_gpu_shader_sep_gaussian_blur_frag_glsl, NULL, NULL);
 			retval = GG.shaders.sep_gaussian_blur;
 			break;
 	}
@@ -1289,7 +1595,7 @@ GPUShader *GPU_shader_get_builtin_shader(GPUBuiltinShader shader)
 	return retval;
 }
 
-void GPU_shader_free_builtin_shaders()
+void GPU_shader_free_builtin_shaders(void)
 {
 	if (GG.shaders.vsm_store) {
 		MEM_freeN(GG.shaders.vsm_store);
