@@ -64,19 +64,13 @@
 #include <io.h>
 #endif
 
+#include "BLI_utildefines.h"
 #include "BLI_string.h"
 #include "BLI_path_util.h"
-#include "BLI_utildefines.h"
-#include "BLI_math_base.h"
 
 #include "MEM_guardedalloc.h"
 
-#include "DNA_userdef_types.h"
-
 #include "BKE_global.h"
-#include "BKE_depsgraph.h"
-
-#include "imbuf.h"
 
 #ifdef WITH_AVI
 #  include "AVI_avi.h"
@@ -91,7 +85,6 @@
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"
 
-#include "IMB_allocimbuf.h"
 #include "IMB_anim.h"
 #include "IMB_indexer.h"
 
@@ -147,7 +140,8 @@ static void free_anim_movie(struct anim *UNUSED(anim))
 static int an_stringdec(const char *string, char *head, char *tail, unsigned short *numlen)
 {
 	unsigned short len, nume, nums = 0;
-	short i, found = FALSE;
+	short i;
+	bool found = false;
 
 	len = strlen(string);
 	nume = len;
@@ -161,7 +155,7 @@ static int an_stringdec(const char *string, char *head, char *tail, unsigned sho
 			else {
 				nume = i;
 				nums = i;
-				found = TRUE;
+				found = true;
 			}
 		}
 		else {
@@ -178,7 +172,7 @@ static int an_stringdec(const char *string, char *head, char *tail, unsigned sho
 	tail[0] = '\0';
 	strcpy(head, string);
 	*numlen = 0;
-	return TRUE;
+	return true;
 }
 
 
@@ -429,10 +423,9 @@ static ImBuf *avi_fetchibuf(struct anim *anim, int position)
 			}
 		}
 	}
-	else {
-#else
-	if (1) {
+	else
 #endif
+	{
 		ibuf = IMB_allocImBuf(anim->x, anim->y, 24, IB_rect);
 
 		tmp = AVI_read_frame(anim->avi, AVI_FORMAT_RGB32, position,
@@ -467,6 +460,7 @@ static int startffmpeg(struct anim *anim)
 	AVCodec *pCodec;
 	AVFormatContext *pFormatCtx = NULL;
 	AVCodecContext *pCodecCtx;
+	AVRational frame_rate;
 	int frs_num;
 	double frs_den;
 	int streamcount;
@@ -478,7 +472,7 @@ static int startffmpeg(struct anim *anim)
 	const int *inv_table;
 #endif
 
-	if (anim == 0) return(-1);
+	if (anim == NULL) return(-1);
 
 	streamcount = anim->streamindex;
 
@@ -486,8 +480,8 @@ static int startffmpeg(struct anim *anim)
 		return -1;
 	}
 
-	if (av_find_stream_info(pFormatCtx) < 0) {
-		av_close_input_file(pFormatCtx);
+	if (avformat_find_stream_info(pFormatCtx, NULL) < 0) {
+		avformat_close_input(&pFormatCtx);
 		return -1;
 	}
 
@@ -508,7 +502,7 @@ static int startffmpeg(struct anim *anim)
 		}
 
 	if (videoStream == -1) {
-		av_close_input_file(pFormatCtx);
+		avformat_close_input(&pFormatCtx);
 		return -1;
 	}
 
@@ -517,23 +511,24 @@ static int startffmpeg(struct anim *anim)
 	/* Find the decoder for the video stream */
 	pCodec = avcodec_find_decoder(pCodecCtx->codec_id);
 	if (pCodec == NULL) {
-		av_close_input_file(pFormatCtx);
+		avformat_close_input(&pFormatCtx);
 		return -1;
 	}
 
 	pCodecCtx->workaround_bugs = 1;
 
-	if (avcodec_open(pCodecCtx, pCodec) < 0) {
-		av_close_input_file(pFormatCtx);
+	if (avcodec_open2(pCodecCtx, pCodec, NULL) < 0) {
+		avformat_close_input(&pFormatCtx);
 		return -1;
 	}
 
+	frame_rate = av_get_r_frame_rate_compat(pFormatCtx->streams[videoStream]);
 	anim->duration = ceil(pFormatCtx->duration *
-	                      av_q2d(pFormatCtx->streams[videoStream]->r_frame_rate) /
+	                      av_q2d(frame_rate) /
 	                      AV_TIME_BASE);
 
-	frs_num = pFormatCtx->streams[videoStream]->r_frame_rate.num;
-	frs_den = pFormatCtx->streams[videoStream]->r_frame_rate.den;
+	frs_num = frame_rate.num;
+	frs_den = frame_rate.den;
 
 	frs_den *= AV_TIME_BASE;
 
@@ -548,7 +543,13 @@ static int startffmpeg(struct anim *anim)
 	anim->params = 0;
 
 	anim->x = pCodecCtx->width;
-	anim->y = pCodecCtx->height;
+	anim->y = av_get_cropped_height_from_codec(pCodecCtx);
+
+	anim->pFormatCtx = pFormatCtx;
+	anim->pCodecCtx = pCodecCtx;
+	anim->pCodec = pCodec;
+	anim->videoStream = videoStream;
+
 	anim->interlacing = 0;
 	anim->orientation = 0;
 	anim->framesize = anim->x * anim->y * 4;
@@ -559,13 +560,8 @@ static int startffmpeg(struct anim *anim)
 	anim->next_pts = -1;
 	anim->next_packet.stream_index = -1;
 
-	anim->pFormatCtx = pFormatCtx;
-	anim->pCodecCtx = pCodecCtx;
-	anim->pCodec = pCodec;
-	anim->videoStream = videoStream;
-
 	anim->pFrame = avcodec_alloc_frame();
-	anim->pFrameComplete = FALSE;
+	anim->pFrameComplete = false;
 	anim->pFrameDeinterlaced = avcodec_alloc_frame();
 	anim->pFrameRGB = avcodec_alloc_frame();
 
@@ -575,7 +571,7 @@ static int startffmpeg(struct anim *anim)
 		fprintf(stderr,
 		        "ffmpeg has changed alloc scheme ... ARGHHH!\n");
 		avcodec_close(anim->pCodecCtx);
-		av_close_input_file(anim->pFormatCtx);
+		avformat_close_input(&anim->pFormatCtx);
 		av_free(anim->pFrameRGB);
 		av_free(anim->pFrameDeinterlaced);
 		av_free(anim->pFrame);
@@ -587,9 +583,12 @@ static int startffmpeg(struct anim *anim)
 		avpicture_fill((AVPicture *) anim->pFrameDeinterlaced,
 		               MEM_callocN(avpicture_get_size(
 		                               anim->pCodecCtx->pix_fmt,
-		                               anim->x, anim->y),
+		                               anim->pCodecCtx->width,
+		                               anim->pCodecCtx->height),
 		                           "ffmpeg deinterlace"),
-		               anim->pCodecCtx->pix_fmt, anim->x, anim->y);
+		               anim->pCodecCtx->pix_fmt, 
+		               anim->pCodecCtx->width,
+		               anim->pCodecCtx->height);
 	}
 
 	if (pCodecCtx->has_b_frames) {
@@ -600,20 +599,20 @@ static int startffmpeg(struct anim *anim)
 	}
 	
 	anim->img_convert_ctx = sws_getContext(
-	        anim->pCodecCtx->width,
-	        anim->pCodecCtx->height,
+	        anim->x,
+	        anim->y,
 	        anim->pCodecCtx->pix_fmt,
-	        anim->pCodecCtx->width,
-	        anim->pCodecCtx->height,
+	        anim->x,
+	        anim->y,
 	        PIX_FMT_RGBA,
-	        SWS_FAST_BILINEAR | SWS_PRINT_INFO,
+	        SWS_FAST_BILINEAR | SWS_PRINT_INFO | SWS_FULL_CHR_H_INT,
 	        NULL, NULL, NULL);
 		
 	if (!anim->img_convert_ctx) {
 		fprintf(stderr,
 		        "Can't transform color space??? Bailing out...\n");
 		avcodec_close(anim->pCodecCtx);
-		av_close_input_file(anim->pFormatCtx);
+		avformat_close_input(&anim->pFormatCtx);
 		av_free(anim->pFrameRGB);
 		av_free(anim->pFrameDeinterlaced);
 		av_free(anim->pFrame);
@@ -632,11 +631,11 @@ static int startffmpeg(struct anim *anim)
 		if (sws_setColorspaceDetails(anim->img_convert_ctx, (int *)inv_table, srcRange,
 		                             table, dstRange, brightness, contrast, saturation))
 		{
-			printf("Warning: Could not set libswscale colorspace details.\n");
+			fprintf(stderr, "Warning: Could not set libswscale colorspace details.\n");
 		}
 	}
 	else {
-		printf("Warning: Could not set libswscale colorspace details.\n");
+		fprintf(stderr, "Warning: Could not set libswscale colorspace details.\n");
 	}
 #endif
 		
@@ -685,7 +684,7 @@ static void ffmpeg_postprocess(struct anim *anim)
 		        anim->pCodecCtx->width,
 		        anim->pCodecCtx->height) < 0)
 		{
-			filter_y = TRUE;
+			filter_y = true;
 		}
 		else {
 			input = anim->pFrameDeinterlaced;
@@ -709,7 +708,7 @@ static void ffmpeg_postprocess(struct anim *anim)
 		          (const uint8_t *const *)input->data,
 		          input->linesize,
 		          0,
-		          anim->pCodecCtx->height,
+		          anim->y,
 		          dst2,
 		          dstStride2);
 		
@@ -754,7 +753,7 @@ static void ffmpeg_postprocess(struct anim *anim)
 		          (const uint8_t *const *)input->data,
 		          input->linesize,
 		          0,
-		          anim->pCodecCtx->height,
+		          anim->y,
 		          dst2,
 		          dstStride2);
 	}
@@ -821,6 +820,14 @@ static int ffmpeg_decode_video_frame(struct anim *anim)
 	}
 	
 	if (rval == AVERROR_EOF) {
+		/* this sets size and data fields to zero,
+		 * which is necessary to decode the remaining data
+		 * in the decoder engine after EOF. It also prevents a memory
+		 * leak, since av_read_frame spills out a full size packet even
+		 * on EOF... (and: it's save to call on NULL packets) */
+
+		av_free_packet(&anim->next_packet);
+
 		anim->next_packet.size = 0;
 		anim->next_packet.data = 0;
 
@@ -923,32 +930,33 @@ static int ffmpeg_seek_by_byte(AVFormatContext *pFormatCtx)
 	const char **p;
 
 	if (pFormatCtx->iformat->flags & AVFMT_TS_DISCONT) {
-		return TRUE;
+		return true;
 	}
 
 	p = byte_seek_list;
 
 	while (*p) {
 		if (match_format(*p++, pFormatCtx)) {
-			return TRUE;
+			return true;
 		}
 	}
 
-	return FALSE;
+	return false;
 }
 
 static ImBuf *ffmpeg_fetchibuf(struct anim *anim, int position,
-                               IMB_Timecode_Type tc) {
+                               IMB_Timecode_Type tc)
+{
 	int64_t pts_to_search = 0;
 	double frame_rate;
 	double pts_time_base;
 	long long st_time; 
 	struct anim_index *tc_index = 0;
 	AVStream *v_st;
-	int new_frame_index = 0; /* To quite gcc barking... */
-	int old_frame_index = 0; /* To quite gcc barking... */
+	int new_frame_index = 0; /* To quiet gcc barking... */
+	int old_frame_index = 0; /* To quiet gcc barking... */
 
-	if (anim == 0) return (0);
+	if (anim == NULL) return (0);
 
 	av_log(anim->pFormatCtx, AV_LOG_DEBUG, "FETCH: pos=%d\n", position);
 
@@ -958,7 +966,7 @@ static ImBuf *ffmpeg_fetchibuf(struct anim *anim, int position,
 
 	v_st = anim->pFormatCtx->streams[anim->videoStream];
 
-	frame_rate = av_q2d(v_st->r_frame_rate);
+	frame_rate = av_q2d(av_get_r_frame_rate_compat(v_st));
 
 	st_time = anim->pFormatCtx->start_time;
 	pts_time_base = av_q2d(v_st->time_base);
@@ -1033,7 +1041,7 @@ static ImBuf *ffmpeg_fetchibuf(struct anim *anim, int position,
 			av_log(anim->pFormatCtx, AV_LOG_DEBUG, 
 			       "TC INDEX seek pos = %lld\n", pos);
 			av_log(anim->pFormatCtx, AV_LOG_DEBUG, 
-			       "TC INDEX seek dts = %lld\n", dts);
+			       "TC INDEX seek dts = %llu\n", dts);
 
 			if (ffmpeg_seek_by_byte(anim->pFormatCtx)) {
 				av_log(anim->pFormatCtx, AV_LOG_DEBUG, 
@@ -1130,7 +1138,7 @@ static void free_anim_ffmpeg(struct anim *anim)
 
 	if (anim->pCodecCtx) {
 		avcodec_close(anim->pCodecCtx);
-		av_close_input_file(anim->pFormatCtx);
+		avformat_close_input(&anim->pFormatCtx);
 		av_free(anim->pFrameRGB);
 		av_free(anim->pFrame);
 
@@ -1233,7 +1241,7 @@ static ImBuf *anim_getnew(struct anim *anim)
 
 
 	if (anim->curtype != 0) return (NULL);
-	anim->curtype = imb_get_anim_type(anim->name);	
+	anim->curtype = imb_get_anim_type(anim->name);
 
 	switch (anim->curtype) {
 		case ANIM_SEQUENCE:
@@ -1295,7 +1303,8 @@ struct ImBuf *IMB_anim_previewframe(struct anim *anim)
 
 struct ImBuf *IMB_anim_absolute(struct anim *anim, int position,
                                 IMB_Timecode_Type tc,
-                                IMB_Proxy_Size preview_size) {
+                                IMB_Proxy_Size preview_size)
+{
 	struct ImBuf *ibuf = NULL;
 	char head[256], tail[256];
 	unsigned short digits;
@@ -1410,15 +1419,15 @@ int IMB_anim_get_duration(struct anim *anim, IMB_Timecode_Type tc)
 	return IMB_indexer_get_duration(idx);
 }
 
-int IMB_anim_get_fps(struct anim *anim,
+bool IMB_anim_get_fps(struct anim *anim,
                      short *frs_sec, float *frs_sec_base)
 {
 	if (anim->frs_sec) {
 		*frs_sec = anim->frs_sec;
 		*frs_sec_base = anim->frs_sec_base;
-		return TRUE;
+		return true;
 	}
-	return FALSE;
+	return false;
 }
 
 void IMB_anim_set_preseek(struct anim *anim, int preseek)
