@@ -29,13 +29,13 @@
  *  \ingroup imbuf
  */
 
+#include "BLI_utildefines.h"
 #include "BLI_fileops.h"
 
 #include "imbuf.h"
 
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"
-#include "IMB_allocimbuf.h"
 #include "IMB_filetype.h"
 
 #include "IMB_colormanagement.h"
@@ -60,6 +60,7 @@ typedef struct BMPINFOHEADER {
 	unsigned int    biClrImportant;
 } BMPINFOHEADER;
 
+#if 0
 typedef struct BMPHEADER {
 	unsigned short biType;
 	unsigned int biSize;
@@ -67,6 +68,7 @@ typedef struct BMPHEADER {
 	unsigned short biRes2;
 	unsigned int biOffBits;
 } BMPHEADER;
+#endif
 
 #define BMP_FILEHEADER_SIZE 14
 
@@ -97,11 +99,11 @@ static int checkbmp(unsigned char *mem)
 		memcpy(&bmi, mem, sizeof(bmi));
 
 		u = LITTLE_LONG(bmi.biSize);
-		/* we only support uncompressed 24 or 32 bits images for now */
+		/* we only support uncompressed images for now. */
 		if (u >= sizeof(BMPINFOHEADER)) {
-			if ((bmi.biCompression == 0) && (bmi.biClrUsed == 0)) {
+			if (bmi.biCompression == 0) {
 				u = LITTLE_SHORT(bmi.biBitCount);
-				if (u >= 16) {
+				if (u >= 8) {
 					ret_val = 1;
 				}
 			}
@@ -122,9 +124,10 @@ struct ImBuf *imb_bmp_decode(unsigned char *mem, size_t size, int flags, char co
 {
 	struct ImBuf *ibuf = NULL;
 	BMPINFOHEADER bmi;
-	int x, y, depth, skip, i;
+	int x, y, depth, ibuf_depth, skip, i;
 	unsigned char *bmp, *rect;
 	unsigned short col;
+	double xppm, yppm;
 	
 	(void)size; /* unused */
 
@@ -144,6 +147,15 @@ struct ImBuf *imb_bmp_decode(unsigned char *mem, size_t size, int flags, char co
 	x = LITTLE_LONG(bmi.biWidth);
 	y = LITTLE_LONG(bmi.biHeight);
 	depth = LITTLE_SHORT(bmi.biBitCount);
+	xppm = LITTLE_LONG(bmi.biXPelsPerMeter);
+	yppm = LITTLE_LONG(bmi.biYPelsPerMeter);
+
+	if (depth <= 8) {
+		ibuf_depth = 24;
+	}
+	else {
+		ibuf_depth = depth;
+	}
 
 #if 0
 	printf("skip: %d, x: %d y: %d, depth: %d (%x)\n", skip, x, y,
@@ -153,14 +165,33 @@ struct ImBuf *imb_bmp_decode(unsigned char *mem, size_t size, int flags, char co
 #endif
 
 	if (flags & IB_test) {
-		ibuf = IMB_allocImBuf(x, y, depth, 0);
+		ibuf = IMB_allocImBuf(x, y, ibuf_depth, 0);
 	}
 	else {
-		ibuf = IMB_allocImBuf(x, y, depth, IB_rect);
+		ibuf = IMB_allocImBuf(x, y, ibuf_depth, IB_rect);
 		bmp = mem + skip;
 		rect = (unsigned char *) ibuf->rect;
 
-		if (depth == 16) {
+		if (depth == 8) {
+			const int x_pad = (4 - (x % 4)) % 4;
+			const char (*palette)[4] = (void *)bmp;
+			bmp += bmi.biClrUsed * 4;
+			for (i = y; i > 0; i--) {
+				int j;
+				for (j = x; j > 0; j--) {
+					const char *pcol = palette[bmp[0]];
+					rect[0] = pcol[0];
+					rect[1] = pcol[1];
+					rect[2] = pcol[2];
+
+					rect[3] = 255;
+					rect += 4; bmp += 1;
+				}
+				/* rows are padded to multiples of 4 */
+				bmp += x_pad;
+			}
+		}
+		else if (depth == 16) {
 			for (i = x * y; i > 0; i--) {
 				col = bmp[0] + (bmp[1] << 8);
 				rect[0] = ((col >> 10) & 0x1f) << 3;
@@ -173,6 +204,7 @@ struct ImBuf *imb_bmp_decode(unsigned char *mem, size_t size, int flags, char co
 
 		}
 		else if (depth == 24) {
+			const int x_pad = x % 4;
 			for (i = y; i > 0; i--) {
 				int j;
 				for (j = x; j > 0; j--) {
@@ -184,7 +216,7 @@ struct ImBuf *imb_bmp_decode(unsigned char *mem, size_t size, int flags, char co
 					rect += 4; bmp += 3;
 				}
 				/* for 24-bit images, rows are padded to multiples of 4 */
-				bmp += x % 4;	
+				bmp += x_pad;
 			}
 		}
 		else if (depth == 32) {
@@ -199,6 +231,8 @@ struct ImBuf *imb_bmp_decode(unsigned char *mem, size_t size, int flags, char co
 	}
 
 	if (ibuf) {
+		ibuf->ppm[0] = xppm;
+		ibuf->ppm[1] = yppm;
 		ibuf->ftype = BMP;
 	}
 	
@@ -238,7 +272,7 @@ int imb_savebmp(struct ImBuf *ibuf, const char *name, int flags)
 	if (!ofile) return 0;
 
 	putShortLSB(19778, ofile); /* "BM" */
-	putIntLSB(0, ofile); /* This can be 0 for BI_RGB bitmaps */
+	putIntLSB(bytesize + BMP_FILEHEADER_SIZE + sizeof(infoheader), ofile);  /* Total file size */
 	putShortLSB(0, ofile); /* Res1 */
 	putShortLSB(0, ofile); /* Res2 */
 	putIntLSB(BMP_FILEHEADER_SIZE + sizeof(infoheader), ofile);
@@ -249,9 +283,9 @@ int imb_savebmp(struct ImBuf *ibuf, const char *name, int flags)
 	putShortLSB(1, ofile);
 	putShortLSB(24, ofile);
 	putIntLSB(0, ofile);
-	putIntLSB(bytesize + BMP_FILEHEADER_SIZE + sizeof(infoheader), ofile);
-	putIntLSB(0, ofile);
-	putIntLSB(0, ofile);
+	putIntLSB(bytesize, ofile);
+	putIntLSB((int)(ibuf->ppm[0] + 0.5), ofile);
+	putIntLSB((int)(ibuf->ppm[1] + 0.5), ofile);
 	putIntLSB(0, ofile);
 	putIntLSB(0, ofile);
 
