@@ -31,19 +31,17 @@
  *  \ingroup imbuf
  */
 
-#include "BLI_rand.h"
 #include "BLI_math.h"
 #include "BLI_utildefines.h"
 
 #include "imbuf.h"
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"
-#include "IMB_allocimbuf.h"
+#include "IMB_filter.h"
 
 #include "IMB_colormanagement.h"
 #include "IMB_colormanagement_intern.h"
 
-#include "BLI_threads.h"
 
 #include "MEM_guardedalloc.h"
 
@@ -62,7 +60,7 @@ void IMB_de_interlace(ImBuf *ibuf)
 		tbuf1 = IMB_allocImBuf(ibuf->x, ibuf->y / 2, 32, IB_rect);
 		tbuf2 = IMB_allocImBuf(ibuf->x, ibuf->y / 2, 32, IB_rect);
 		
-		ibuf->x *= 2;	
+		ibuf->x *= 2;
 		IMB_rectcpy(tbuf1, ibuf, 0, 0, 0, 0, ibuf->x, ibuf->y);
 		IMB_rectcpy(tbuf2, ibuf, 0, 0, tbuf2->x, 0, ibuf->x, ibuf->y);
 	
@@ -106,63 +104,33 @@ void IMB_interlace(ImBuf *ibuf)
 /************************* Floyd-Steinberg dithering *************************/
 
 typedef struct DitherContext {
-	int *error_buf, *e;
-	int v[4], v0[4], v1[4];
-	float f;
+	float dither;
 } DitherContext;
 
-static DitherContext *create_dither_context(int w, float factor)
+static DitherContext *create_dither_context(float dither)
 {
 	DitherContext *di;
-	int i;
-	
-	di = MEM_callocN(sizeof(DitherContext), "dithering context");
-	di->f = factor / 16.0f;
-	di->error_buf = MEM_callocN(4 * (w + 1) * sizeof(int), "dithering error");
-	di->e = di->error_buf;
 
-	for (i = 0; i < 4; ++i)
-		di->v[i] = di->v0[i] = di->v1[i] = 1024.0f * (BLI_frand() - 0.5f);
+	di = MEM_mallocN(sizeof(DitherContext), "dithering context");
+	di->dither = dither;
 
 	return di;
 }
 
 static void clear_dither_context(DitherContext *di)
 {
-	MEM_freeN(di->error_buf);
 	MEM_freeN(di);
 }
 
-static void dither_finish_row(DitherContext *di)
+MINLINE float dither_random_value(float s, float t)
 {
-	int i;
+	static float vec[2] = {12.9898f, 78.233f};
+	float st[2];
+	float value;
+	copy_v2_fl2(st, s, t);
 
-	for (i = 0; i < 4; i++)
-		di->v[i] = di->v0[i] = di->v1[i] = 0;
-
-	di->e = di->error_buf;
-}
-
-MINLINE unsigned char dither_value(unsigned short v_in, DitherContext *di, int i)
-{
-	int dv, d2;
-	unsigned char v_out;
-
-	di->v[i] = v_in + (2 * di->v[i] + di->e[4]) * di->f;
-	CLAMP(di->v[i], 0, 0xFF00);
-	v_out = USHORTTOUCHAR(di->v[i]);
-	di->v[i] -= v_out << 8;
-	dv = di->v[i];
-	d2 = di->v[i] << 1;
-	di->v[i] += d2;
-	*(di->e++) = di->v[i] + di->v0[i];
-	di->v[i] += d2;
-
-	di->v0[i] = di->v[i] + di->v1[i];
-	di->v1[i] = dv;
-	di->v[i] += d2;
-
-	return v_out;
+	value = sinf(dot_v2v2(st, vec)) * 43758.5453f;
+	return value - floorf(value);
 }
 
 /************************* Generic Buffer Conversion *************************/
@@ -175,37 +143,55 @@ MINLINE void ushort_to_byte_v4(uchar b[4], const unsigned short us[4])
 	b[3] = USHORTTOUCHAR(us[3]);
 }
 
-MINLINE void ushort_to_byte_dither_v4(uchar b[4], const unsigned short us[4], DitherContext *di)
+MINLINE unsigned char ftochar(float value)
 {
-	b[0] = dither_value(us[0], di, 0);
-	b[1] = dither_value(us[1], di, 1);
-	b[2] = dither_value(us[2], di, 2);
-	b[3] = dither_value(us[3], di, 3);
+	return FTOCHAR(value);
 }
 
-MINLINE void float_to_byte_dither_v4(uchar b[4], const float f[4], DitherContext *di)
+MINLINE void ushort_to_byte_dither_v4(uchar b[4], const unsigned short us[4], DitherContext *di, float s, float t)
 {
-	unsigned short us[4] = {FTOUSHORT(f[0]), FTOUSHORT(f[1]), FTOUSHORT(f[2]), FTOUSHORT(f[3])};
-	ushort_to_byte_dither_v4(b, us, di);
+#define USHORTTOFLOAT(val) ((float)val / 65535.0f)
+	float dither_value = dither_random_value(s, t) * 0.005f * di->dither;
+
+	b[0] = ftochar(dither_value + USHORTTOFLOAT(us[0]));
+	b[1] = ftochar(dither_value + USHORTTOFLOAT(us[1]));
+	b[2] = ftochar(dither_value + USHORTTOFLOAT(us[2]));
+	b[3] = USHORTTOUCHAR(us[3]);
+
+#undef USHORTTOFLOAT
+}
+
+MINLINE void float_to_byte_dither_v4(uchar b[4], const float f[4], DitherContext *di, float s, float t)
+{
+	float dither_value = dither_random_value(s, t) * 0.005f * di->dither;
+
+	b[0] = ftochar(dither_value + f[0]);
+	b[1] = ftochar(dither_value + f[1]);
+	b[2] = ftochar(dither_value + f[2]);
+	b[3] = FTOCHAR(f[3]);
 }
 
 /* float to byte pixels, output 4-channel RGBA */
 void IMB_buffer_byte_from_float(uchar *rect_to, const float *rect_from,
-                                int channels_from, float dither, int profile_to, int profile_from, int predivide,
+                                int channels_from, float dither, int profile_to, int profile_from, bool predivide,
                                 int width, int height, int stride_to, int stride_from)
 {
 	float tmp[4];
 	int x, y;
-	DitherContext *di;
+	DitherContext *di = NULL;
+	float inv_width = 1.0f / width,
+	      inv_height = 1.0f / height;
 
 	/* we need valid profiles */
 	BLI_assert(profile_to != IB_PROFILE_NONE);
 	BLI_assert(profile_from != IB_PROFILE_NONE);
 
 	if (dither)
-		di = create_dither_context(width, dither);
+		di = create_dither_context(dither);
 
 	for (y = 0; y < height; y++) {
+		float t = y * inv_height;
+
 		if (channels_from == 1) {
 			/* single channel input */
 			const float *from = rect_from + stride_from * y;
@@ -249,10 +235,24 @@ void IMB_buffer_byte_from_float(uchar *rect_to, const float *rect_from,
 			uchar *to = rect_to + stride_to * y * 4;
 
 			if (profile_to == profile_from) {
+				float straight[4];
+
 				/* no color space conversion */
-				if (dither) {
+				if (dither && predivide) {
+					for (x = 0; x < width; x++, from += 4, to += 4) {
+						premul_to_straight_v4_v4(straight, from);
+						float_to_byte_dither_v4(to, straight, di, (float) x * inv_width, t);
+					}
+				}
+				else if (dither) {
 					for (x = 0; x < width; x++, from += 4, to += 4)
-						float_to_byte_dither_v4(to, from, di);
+						float_to_byte_dither_v4(to, from, di, (float) x * inv_width, t);
+				}
+				else if (predivide) {
+					for (x = 0; x < width; x++, from += 4, to += 4) {
+						premul_to_straight_v4_v4(straight, from);
+						rgba_float_to_uchar(to, straight);
+					}
 				}
 				else {
 					for (x = 0; x < width; x++, from += 4, to += 4)
@@ -262,22 +262,25 @@ void IMB_buffer_byte_from_float(uchar *rect_to, const float *rect_from,
 			else if (profile_to == IB_PROFILE_SRGB) {
 				/* convert from linear to sRGB */
 				unsigned short us[4];
+				float straight[4];
 
 				if (dither && predivide) {
 					for (x = 0; x < width; x++, from += 4, to += 4) {
-						linearrgb_to_srgb_ushort4_predivide(us, from);
-						ushort_to_byte_dither_v4(to, us, di);
+						premul_to_straight_v4_v4(straight, from);
+						linearrgb_to_srgb_ushort4(us, from);
+						ushort_to_byte_dither_v4(to, us, di, (float) x * inv_width, t);
 					}
 				}
 				else if (dither) {
 					for (x = 0; x < width; x++, from += 4, to += 4) {
 						linearrgb_to_srgb_ushort4(us, from);
-						ushort_to_byte_dither_v4(to, us, di);
+						ushort_to_byte_dither_v4(to, us, di, (float) x * inv_width, t);
 					}
 				}
 				else if (predivide) {
 					for (x = 0; x < width; x++, from += 4, to += 4) {
-						linearrgb_to_srgb_ushort4_predivide(us, from);
+						premul_to_straight_v4_v4(straight, from);
+						linearrgb_to_srgb_ushort4(us, from);
 						ushort_to_byte_v4(to, us);
 					}
 				}
@@ -293,13 +296,13 @@ void IMB_buffer_byte_from_float(uchar *rect_to, const float *rect_from,
 				if (dither && predivide) {
 					for (x = 0; x < width; x++, from += 4, to += 4) {
 						srgb_to_linearrgb_predivide_v4(tmp, from);
-						float_to_byte_dither_v4(to, tmp, di);
+						float_to_byte_dither_v4(to, tmp, di, (float) x * inv_width, t);
 					}
 				}
 				else if (dither) {
 					for (x = 0; x < width; x++, from += 4, to += 4) {
 						srgb_to_linearrgb_v4(tmp, from);
-						float_to_byte_dither_v4(to, tmp, di);
+						float_to_byte_dither_v4(to, tmp, di, (float) x * inv_width, t);
 					}
 				}
 				else if (predivide) {
@@ -316,9 +319,84 @@ void IMB_buffer_byte_from_float(uchar *rect_to, const float *rect_from,
 				}
 			}
 		}
+	}
 
-		if (dither)
-			dither_finish_row(di);
+	if (dither)
+		clear_dither_context(di);
+}
+
+
+/* float to byte pixels, output 4-channel RGBA */
+void IMB_buffer_byte_from_float_mask(uchar *rect_to, const float *rect_from,
+                                int channels_from, float dither, bool predivide,
+                                int width, int height, int stride_to, int stride_from, char *mask)
+{
+	int x, y;
+	DitherContext *di = NULL;
+	float inv_width = 1.0f / width,
+	inv_height = 1.0f / height;
+
+	if (dither)
+		di = create_dither_context(dither);
+
+	for (y = 0; y < height; y++) {
+		float t = y * inv_height;
+
+		if (channels_from == 1) {
+			/* single channel input */
+			const float *from = rect_from + stride_from * y;
+			uchar *to = rect_to + stride_to * y * 4;
+
+			for (x = 0; x < width; x++, from++, to += 4)
+				if (*mask++ == FILTER_MASK_USED)
+					to[0] = to[1] = to[2] = to[3] = FTOCHAR(from[0]);
+		}
+		else if (channels_from == 3) {
+			/* RGB input */
+			const float *from = rect_from + stride_from * y * 3;
+			uchar *to = rect_to + stride_to * y * 4;
+
+			for (x = 0; x < width; x++, from += 3, to += 4) {
+				if (*mask++ == FILTER_MASK_USED) {
+					rgb_float_to_uchar(to, from);
+					to[3] = 255;
+				}
+			}
+		}
+		else if (channels_from == 4) {
+			/* RGBA input */
+			const float *from = rect_from + stride_from * y * 4;
+			uchar *to = rect_to + stride_to * y * 4;
+
+			float straight[4];
+
+			if (dither && predivide) {
+				for (x = 0; x < width; x++, from += 4, to += 4) {
+					if (*mask++ == FILTER_MASK_USED) {
+						premul_to_straight_v4_v4(straight, from);
+						float_to_byte_dither_v4(to, straight, di, (float) x * inv_width, t);
+					}
+				}
+			}
+			else if (dither) {
+				for (x = 0; x < width; x++, from += 4, to += 4)
+					if (*mask++ == FILTER_MASK_USED)
+						float_to_byte_dither_v4(to, from, di, (float) x * inv_width, t);
+			}
+			else if (predivide) {
+				for (x = 0; x < width; x++, from += 4, to += 4) {
+					if (*mask++ == FILTER_MASK_USED) {
+						premul_to_straight_v4_v4(straight, from);
+						rgba_float_to_uchar(to, straight);
+					}
+				}
+			}
+			else {
+				for (x = 0; x < width; x++, from += 4, to += 4)
+					if (*mask++ == FILTER_MASK_USED)
+						rgba_float_to_uchar(to, from);
+			}
+		}
 	}
 
 	if (dither)
@@ -327,7 +405,7 @@ void IMB_buffer_byte_from_float(uchar *rect_to, const float *rect_from,
 
 /* byte to float pixels, input and output 4-channel RGBA  */
 void IMB_buffer_float_from_byte(float *rect_to, const uchar *rect_from,
-                                int profile_to, int profile_from, int predivide,
+                                int profile_to, int profile_from, bool predivide,
                                 int width, int height, int stride_to, int stride_from)
 {
 	float tmp[4];
@@ -380,7 +458,7 @@ void IMB_buffer_float_from_byte(float *rect_to, const uchar *rect_from,
 
 /* float to float pixels, output 4-channel RGBA */
 void IMB_buffer_float_from_float(float *rect_to, const float *rect_from,
-                                 int channels_from, int profile_to, int profile_from, int predivide,
+                                 int channels_from, int profile_to, int profile_from, bool predivide,
                                  int width, int height, int stride_to, int stride_from)
 {
 	int x, y;
@@ -464,9 +542,53 @@ void IMB_buffer_float_from_float(float *rect_to, const float *rect_from,
 	}
 }
 
+/* float to float pixels, output 4-channel RGBA */
+void IMB_buffer_float_from_float_mask(float *rect_to, const float *rect_from, int channels_from,
+                                      int width, int height, int stride_to, int stride_from, char *mask)
+{
+	int x, y;
+
+	if (channels_from == 1) {
+		/* single channel input */
+		for (y = 0; y < height; y++) {
+			const float *from = rect_from + stride_from * y;
+			float *to = rect_to + stride_to * y * 4;
+
+			for (x = 0; x < width; x++, from++, to += 4)
+				if (*mask++ == FILTER_MASK_USED)
+					to[0] = to[1] = to[2] = to[3] = from[0];
+		}
+	}
+	else if (channels_from == 3) {
+		/* RGB input */
+		for (y = 0; y < height; y++) {
+			const float *from = rect_from + stride_from * y * 3;
+			float *to = rect_to + stride_to * y * 4;
+
+			for (x = 0; x < width; x++, from += 3, to += 4) {
+				if (*mask++ == FILTER_MASK_USED) {
+					copy_v3_v3(to, from);
+					to[3] = 1.0f;
+				}
+			}
+		}
+	}
+	else if (channels_from == 4) {
+		/* RGBA input */
+		for (y = 0; y < height; y++) {
+			const float *from = rect_from + stride_from * y * 4;
+			float *to = rect_to + stride_to * y * 4;
+
+			for (x = 0; x < width; x++, from += 4, to += 4)
+				if (*mask++ == FILTER_MASK_USED)
+					copy_v4_v4(to, from);
+		}
+	}
+}
+
 /* byte to byte pixels, input and output 4-channel RGBA */
 void IMB_buffer_byte_from_byte(uchar *rect_to, const uchar *rect_from,
-                               int profile_to, int profile_from, int predivide,
+                               int profile_to, int profile_from, bool predivide,
                                int width, int height, int stride_to, int stride_from)
 {
 	float tmp[4];
@@ -526,7 +648,6 @@ void IMB_buffer_byte_from_byte(uchar *rect_to, const uchar *rect_from,
 
 void IMB_rect_from_float(ImBuf *ibuf)
 {
-	int predivide = (ibuf->flags & IB_cm_predivide);
 	float *buffer;
 	const char *from_colorspace;
 
@@ -548,11 +669,14 @@ void IMB_rect_from_float(ImBuf *ibuf)
 	buffer = MEM_dupallocN(ibuf->rect_float);
 
 	/* first make float buffer in byte space */
-	IMB_colormanagement_transform(buffer, ibuf->x, ibuf->y, ibuf->channels, from_colorspace, ibuf->rect_colorspace->name, predivide);
+	IMB_colormanagement_transform(buffer, ibuf->x, ibuf->y, ibuf->channels, from_colorspace, ibuf->rect_colorspace->name, true);
+
+	/* convert from float's premul alpha to byte's straight alpha */
+	IMB_unpremultiply_rect_float(buffer, ibuf->channels, ibuf->x, ibuf->y);
 
 	/* convert float to byte */
 	IMB_buffer_byte_from_float((unsigned char *) ibuf->rect, buffer, ibuf->channels, ibuf->dither, IB_PROFILE_SRGB, IB_PROFILE_SRGB,
-	                           FALSE, ibuf->x, ibuf->y, ibuf->x, ibuf->x);
+	                           false, ibuf->x, ibuf->y, ibuf->x, ibuf->x);
 
 	MEM_freeN(buffer);
 
@@ -561,11 +685,10 @@ void IMB_rect_from_float(ImBuf *ibuf)
 }
 
 /* converts from linear float to sRGB byte for part of the texture, buffer will hold the changed part */
-void IMB_partial_rect_from_float(ImBuf *ibuf, float *buffer, int x, int y, int w, int h)
+void IMB_partial_rect_from_float(ImBuf *ibuf, float *buffer, int x, int y, int w, int h, bool is_data)
 {
-	float *rect_float;
+	const float *rect_float;
 	uchar *rect_byte;
-	int predivide = (ibuf->flags & IB_cm_predivide);
 	int profile_from = IB_PROFILE_LINEAR_RGB;
 
 	/* verify we have a float buffer */
@@ -580,14 +703,28 @@ void IMB_partial_rect_from_float(ImBuf *ibuf, float *buffer, int x, int y, int w
 	rect_float = ibuf->rect_float + (x + y * ibuf->x) * ibuf->channels;
 	rect_byte = (uchar *)ibuf->rect + (x + y * ibuf->x) * 4;
 
-	IMB_buffer_float_from_float(buffer, rect_float,
-	                            ibuf->channels, IB_PROFILE_SRGB, profile_from, predivide,
-	                            w, h, w, ibuf->x);
+	if (is_data) {
+		/* exception for non-color data, just copy float */
+		IMB_buffer_float_from_float(buffer, rect_float,
+		                            ibuf->channels, IB_PROFILE_LINEAR_RGB, IB_PROFILE_LINEAR_RGB, 0,
+		                            w, h, w, ibuf->x);
 
-	/* XXX: need to convert to image buffer's rect space */
-	IMB_buffer_byte_from_float(rect_byte, buffer,
-	                           4, ibuf->dither, IB_PROFILE_SRGB, IB_PROFILE_SRGB, 0,
-	                           w, h, ibuf->x, w);
+		/* and do color space conversion to byte */
+		IMB_buffer_byte_from_float(rect_byte, rect_float,
+		                           4, ibuf->dither, IB_PROFILE_SRGB, profile_from, true,
+		                           w, h, ibuf->x, w);
+	}
+	else {
+		IMB_buffer_float_from_float(buffer, rect_float,
+		                            ibuf->channels, IB_PROFILE_SRGB, profile_from, true,
+		                            w, h, w, ibuf->x);
+
+		IMB_buffer_float_unpremultiply(buffer, w, h);
+		/* XXX: need to convert to image buffer's rect space */
+		IMB_buffer_byte_from_float(rect_byte, buffer,
+		                           4, ibuf->dither, IB_PROFILE_SRGB, IB_PROFILE_SRGB, 0,
+		                           w, h, ibuf->x, w);
+	}
 
 	/* ensure user flag is reset */
 	ibuf->userflags &= ~IB_RECT_INVALID;
@@ -595,94 +732,45 @@ void IMB_partial_rect_from_float(ImBuf *ibuf, float *buffer, int x, int y, int w
 
 void IMB_float_from_rect(ImBuf *ibuf)
 {
-	int predivide = (ibuf->flags & IB_cm_predivide);
+	float *rect_float;
 
 	/* verify if we byte and float buffers */
 	if (ibuf->rect == NULL)
 		return;
 
-	/* lock the color management thread
-	 * need this because allocated but not filled float buffer will confuse
-	 * display transform which lead to black areas across the frame
+	/* allocate float buffer outside of image buffer,
+	 * so work-in-progress color space conversion doesn't
+	 * interfere with other parts of blender
 	 */
-	BLI_lock_thread(LOCK_COLORMANAGE);
+	rect_float = ibuf->rect_float;
+	if (rect_float == NULL) {
+		int size;
 
-	if (ibuf->rect_float == NULL) {
-		if (imb_addrectfloatImBuf(ibuf) == 0) {
-			BLI_unlock_thread(LOCK_COLORMANAGE);
+		size = ibuf->x * ibuf->y;
+		size = size * 4 * sizeof(float);
+		ibuf->channels = 4;
 
+		rect_float = MEM_mapallocN(size, "IMB_float_from_rect");
+
+		if (rect_float == NULL)
 			return;
-		}
 	}
 
 	/* first, create float buffer in non-linear space */
-	IMB_buffer_float_from_byte(ibuf->rect_float, (unsigned char *) ibuf->rect, IB_PROFILE_SRGB, IB_PROFILE_SRGB,
-	                           FALSE, ibuf->x, ibuf->y, ibuf->x, ibuf->x);
+	IMB_buffer_float_from_byte(rect_float, (unsigned char *) ibuf->rect, IB_PROFILE_SRGB, IB_PROFILE_SRGB,
+	                           false, ibuf->x, ibuf->y, ibuf->x, ibuf->x);
 
 	/* then make float be in linear space */
-	IMB_colormanagement_colorspace_to_scene_linear(ibuf->rect_float, ibuf->x, ibuf->y, ibuf->channels,
-	                                               ibuf->rect_colorspace, predivide);
+	IMB_colormanagement_colorspace_to_scene_linear(rect_float, ibuf->x, ibuf->y, ibuf->channels,
+	                                               ibuf->rect_colorspace, false);
 
-	BLI_unlock_thread(LOCK_COLORMANAGE);
-}
+	/* byte buffer is straight alpha, float should always be premul */
+	IMB_premultiply_rect_float(rect_float, ibuf->channels, ibuf->x, ibuf->y);
 
-/* no profile conversion */
-void IMB_float_from_rect_simple(ImBuf *ibuf)
-{
-	int predivide = (ibuf->flags & IB_cm_predivide);
-
-	if (ibuf->rect_float == NULL)
-		imb_addrectfloatImBuf(ibuf);
-
-	IMB_buffer_float_from_byte(ibuf->rect_float, (uchar *)ibuf->rect,
-	                           IB_PROFILE_SRGB, IB_PROFILE_SRGB, predivide,
-	                           ibuf->x, ibuf->y, ibuf->x, ibuf->x);
-}
-
-/* use when you need to get a buffer with a certain profile
- * if the return  */
-
-/* OCIO_TODO: used only by Cineon/DPX exporter which is still broken, so can not guarantee
- *            this function is working properly
- */
-float *IMB_float_profile_ensure(ImBuf *ibuf, int profile, int *alloc)
-{
-	int predivide = (ibuf->flags & IB_cm_predivide);
-	int profile_from = IB_PROFILE_LINEAR_RGB;
-	int profile_to;
-
-	/* determine profile */
-	if (profile == IB_PROFILE_NONE)
-		profile_to = IB_PROFILE_LINEAR_RGB;
-	else
-		profile_to = IB_PROFILE_SRGB;
-	
-	if (profile_from == profile_to) {
-		/* simple case, just allocate the buffer and return */
-		*alloc = 0;
-
-		if (ibuf->rect_float == NULL)
-			IMB_float_from_rect(ibuf);
-
-		return ibuf->rect_float;
-	}
-	else {
-		/* conversion is needed, first check */
-		float *fbuf = MEM_mallocN(ibuf->x * ibuf->y * sizeof(float) * 4, "IMB_float_profile_ensure");
-		*alloc = 1;
-
-		if (ibuf->rect_float == NULL) {
-			IMB_buffer_float_from_byte(fbuf, (uchar *)ibuf->rect,
-			                           profile_to, profile_from, predivide,
-			                           ibuf->x, ibuf->y, ibuf->x, ibuf->x);
-		}
-		else {
-			IMB_buffer_float_from_float(fbuf, ibuf->rect_float,
-			                            4, profile_to, profile_from, predivide,
-			                            ibuf->x, ibuf->y, ibuf->x, ibuf->x);
-		}
-
-		return fbuf;
+	if (ibuf->rect_float == NULL) {
+		ibuf->rect_float = rect_float;
+		ibuf->mall |= IB_rectfloat;
+		ibuf->flags |= IB_rectfloat;
 	}
 }
 
@@ -691,13 +779,13 @@ float *IMB_float_profile_ensure(ImBuf *ibuf, int profile, int *alloc)
 /* no profile conversion */
 void IMB_color_to_bw(ImBuf *ibuf)
 {
-	float *rctf = ibuf->rect_float;
+	float *rct_fl = ibuf->rect_float;
 	uchar *rct = (uchar *)ibuf->rect;
 	int i;
 
-	if (rctf) {
-		for (i = ibuf->x * ibuf->y; i > 0; i--, rctf += 4)
-			rctf[0] = rctf[1] = rctf[2] = rgb_to_grayscale(rctf);
+	if (rct_fl) {
+		for (i = ibuf->x * ibuf->y; i > 0; i--, rct_fl += 4)
+			rct_fl[0] = rct_fl[1] = rct_fl[2] = rgb_to_grayscale(rct_fl);
 	}
 
 	if (rct) {
@@ -710,7 +798,27 @@ void IMB_buffer_float_clamp(float *buf, int width, int height)
 {
 	int i, total = width * height * 4;
 	for (i = 0; i < total; i++) {
-		buf[i] = minf(1.0, buf[i]);
+		buf[i] = min_ff(1.0, buf[i]);
+	}
+}
+
+void IMB_buffer_float_unpremultiply(float *buf, int width, int height)
+{
+	int total = width * height;
+	float *fp = buf;
+	while (total--) {
+		premul_to_straight_v4(fp);
+		fp += 4;
+	}
+}
+
+void IMB_buffer_float_premultiply(float *buf, int width, int height)
+{
+	int total = width * height;
+	float *fp = buf;
+	while (total--) {
+		straight_to_premul_v4(fp);
+		fp += 4;
 	}
 }
 
@@ -720,7 +828,7 @@ void IMB_saturation(ImBuf *ibuf, float sat)
 {
 	int i;
 	unsigned char *rct = (unsigned char *)ibuf->rect;
-	float *rctf = ibuf->rect_float;
+	float *rct_fl = ibuf->rect_float;
 	float hsv[3];
 
 	if (rct) {
@@ -733,10 +841,10 @@ void IMB_saturation(ImBuf *ibuf, float sat)
 		}
 	}
 
-	if (rctf) {
-		for (i = ibuf->x * ibuf->y; i > 0; i--, rctf += 4) {
-			rgb_to_hsv_v(rctf, hsv);
-			hsv_to_rgb(hsv[0], hsv[1] * sat, hsv[2], rctf, rctf + 1, rctf + 2);
+	if (rct_fl) {
+		for (i = ibuf->x * ibuf->y; i > 0; i--, rct_fl += 4) {
+			rgb_to_hsv_v(rct_fl, hsv);
+			hsv_to_rgb(hsv[0], hsv[1] * sat, hsv[2], rct_fl, rct_fl + 1, rct_fl + 2);
 		}
 	}
 }
