@@ -22,6 +22,10 @@
  * ***** END GPL LICENSE BLOCK *****
  */
 
+/** \file blender/imbuf/intern/indexer.c
+ *  \ingroup imbuf
+ */
+
 #include <stdlib.h>
 
 #include "MEM_guardedalloc.h"
@@ -30,16 +34,13 @@
 #include "BLI_endian_switch.h"
 #include "BLI_path_util.h"
 #include "BLI_string.h"
-#include "BLI_path_util.h"
 #include "BLI_fileops.h"
-#include "BLI_math_base.h"
 
 #include "IMB_indexer.h"
 #include "IMB_anim.h"
 #include "imbuf.h"
 
 #include "MEM_guardedalloc.h"
-#include "DNA_userdef_types.h"
 #include "BKE_global.h"
 
 #ifdef WITH_AVI
@@ -51,12 +52,12 @@
 #endif
 
 
-static char magic[] = "BlenMIdx";
-static char temp_ext[] = "_part";
+static const char magic[] = "BlenMIdx";
+static const char temp_ext[] = "_part";
 
-static int proxy_sizes[] = { IMB_PROXY_25, IMB_PROXY_50, IMB_PROXY_75,
+static const int proxy_sizes[] = { IMB_PROXY_25, IMB_PROXY_50, IMB_PROXY_75,
 	                         IMB_PROXY_100 };
-static float proxy_fac[] = { 0.25, 0.50, 0.75, 1.00 };
+static const float proxy_fac[] = { 0.25, 0.50, 0.75, 1.00 };
 
 #ifdef WITH_FFMPEG
 static int tc_types[] = {IMB_TC_RECORD_RUN,
@@ -332,8 +333,7 @@ int IMB_proxy_size_to_array_index(IMB_Proxy_Size pr_size)
 			return 3;
 		default:
 			return 0;
-	};
-	return 0;
+	}
 }
 
 int IMB_timecode_to_array_index(IMB_Timecode_Type tc)
@@ -352,8 +352,7 @@ int IMB_timecode_to_array_index(IMB_Timecode_Type tc)
 			return 3;
 		default:
 			return 0;
-	};
-	return 0;
+	}
 }
 
 
@@ -361,22 +360,21 @@ int IMB_timecode_to_array_index(IMB_Timecode_Type tc)
  * - rebuild helper functions
  * ---------------------------------------------------------------------- */
 
-static void get_index_dir(struct anim *anim, char *index_dir)
+static void get_index_dir(struct anim *anim, char *index_dir, size_t index_dir_len)
 {
 	if (!anim->index_dir[0]) {
 		char fname[FILE_MAXFILE];
-		BLI_strncpy(index_dir, anim->name, FILE_MAXDIR);
-		BLI_splitdirstring(index_dir, fname);
-		BLI_join_dirfile(index_dir, FILE_MAXDIR, index_dir, "BL_proxy");
-		BLI_join_dirfile(index_dir, FILE_MAXDIR, index_dir, fname);
+		BLI_split_dirfile(anim->name, index_dir, fname, index_dir_len, sizeof(fname));
+		BLI_path_append(index_dir, index_dir_len, "BL_proxy");
+		BLI_path_append(index_dir, index_dir_len, fname);
 	}
 	else {
-		BLI_strncpy(index_dir, anim->index_dir, FILE_MAXDIR);
+		BLI_strncpy(index_dir, anim->index_dir, index_dir_len);
 	}
 }
 
 static void get_proxy_filename(struct anim *anim, IMB_Proxy_Size preview_size,
-                               char *fname, int temp)
+                               char *fname, bool temp)
 {
 	char index_dir[FILE_MAXDIR];
 	int i = IMB_proxy_size_to_array_index(preview_size);
@@ -396,7 +394,7 @@ static void get_proxy_filename(struct anim *anim, IMB_Proxy_Size preview_size,
 	BLI_snprintf(proxy_temp_name, sizeof(proxy_temp_name), "proxy_%d%s_part.avi",
 	             (int) (proxy_fac[i] * 100), stream_suffix);
 
-	get_index_dir(anim, index_dir);
+	get_index_dir(anim, index_dir, sizeof(index_dir));
 
 	BLI_join_dirfile(fname, FILE_MAXFILE + FILE_MAXDIR, index_dir, 
 	                 temp ? proxy_temp_name : proxy_name);
@@ -425,7 +423,7 @@ static void get_tc_filename(struct anim *anim, IMB_Timecode_Type tc,
 	
 	BLI_snprintf(index_name, 256, index_names[i], stream_suffix);
 
-	get_index_dir(anim, index_dir);
+	get_index_dir(anim, index_dir, sizeof(index_dir));
 	
 	BLI_join_dirfile(fname, FILE_MAXFILE + FILE_MAXDIR, 
 	                 index_dir, index_name);
@@ -453,8 +451,6 @@ struct proxy_output_ctx {
 	AVCodec *codec;
 	struct SwsContext *sws_ctx;
 	AVFrame *frame;
-	uint8_t *video_buffer;
-	int video_buffersize;
 	int cfra;
 	int proxy_size;
 	int orig_height;
@@ -486,20 +482,22 @@ static struct proxy_output_ctx *alloc_proxy_output_ffmpeg(
 	rv->proxy_size = proxy_size;
 	rv->anim = anim;
 
-	get_proxy_filename(rv->anim, rv->proxy_size, fname, TRUE);
+	get_proxy_filename(rv->anim, rv->proxy_size, fname, true);
 	BLI_make_existing_file(fname);
 
 	rv->of = avformat_alloc_context();
 	rv->of->oformat = av_guess_format("avi", NULL, NULL);
 	
-	BLI_snprintf(rv->of->filename, sizeof(rv->of->filename), "%s", fname);
+	BLI_strncpy(rv->of->filename, fname, sizeof(rv->of->filename));
 
 	fprintf(stderr, "Starting work on proxy: %s\n", rv->of->filename);
 
-	rv->st = av_new_stream(rv->of, 0);
+	rv->st = avformat_new_stream(rv->of, NULL);
+	rv->st->id = 0;
+
 	rv->c = rv->st->codec;
 	rv->c->codec_type = AVMEDIA_TYPE_VIDEO;
-	rv->c->codec_id = CODEC_ID_MJPEG;
+	rv->c->codec_id = AV_CODEC_ID_MJPEG;
 	rv->c->width = width;
 	rv->c->height = height;
 
@@ -531,8 +529,8 @@ static struct proxy_output_ctx *alloc_proxy_output_ffmpeg(
 	/* there's no  way to set JPEG quality in the same way as in AVI JPEG and image sequence,
 	 * but this seems to be giving expected quality result */
 	ffmpeg_quality = (int)(1.0f + 30.0f * (1.0f - (float)quality / 100.0f) + 0.5f);
-	av_set_int(rv->c, "qmin", ffmpeg_quality);
-	av_set_int(rv->c, "qmax", ffmpeg_quality);
+	av_opt_set_int(rv->c, "qmin", ffmpeg_quality, 0);
+	av_opt_set_int(rv->c, "qmax", ffmpeg_quality, 0);
 
 	if (rv->of->flags & AVFMT_GLOBALHEADER) {
 		rv->c->flags |= CODEC_FLAG_GLOBAL_HEADER;
@@ -545,13 +543,9 @@ static struct proxy_output_ctx *alloc_proxy_output_ffmpeg(
 		return 0;
 	}
 
-	avcodec_open(rv->c, rv->codec);
+	avcodec_open2(rv->c, rv->codec, NULL);
 
-	rv->video_buffersize = 2000000;
-	rv->video_buffer = (uint8_t *)MEM_mallocN(
-	        rv->video_buffersize, "FFMPEG video buffer");
-
-	rv->orig_height = st->codec->height;
+	rv->orig_height = av_get_cropped_height_from_codec(st->codec);
 
 	if (st->codec->width != width || st->codec->height != height ||
 	    st->codec->pix_fmt != rv->c->pix_fmt)
@@ -566,7 +560,7 @@ static struct proxy_output_ctx *alloc_proxy_output_ffmpeg(
 
 		rv->sws_ctx = sws_getContext(
 		        st->codec->width,
-		        st->codec->height,
+		        rv->orig_height,
 		        st->codec->pix_fmt,
 		        width, height,
 		        rv->c->pix_fmt,
@@ -587,7 +581,10 @@ static struct proxy_output_ctx *alloc_proxy_output_ffmpeg(
 static int add_to_proxy_output_ffmpeg(
         struct proxy_output_ctx *ctx, AVFrame *frame)
 {
-	int outsize = 0;
+	AVPacket packet = { 0 };
+	int ret, got_output;
+
+	av_init_packet(&packet);
 
 	if (!ctx) {
 		return 0;
@@ -608,31 +605,26 @@ static int add_to_proxy_output_ffmpeg(
 		frame->pts = ctx->cfra++;
 	}
 
-	outsize = avcodec_encode_video(
-	        ctx->c, ctx->video_buffer, ctx->video_buffersize,
-	        frame);
-
-	if (outsize < 0) {
+	ret = avcodec_encode_video2(ctx->c, &packet, frame, &got_output);
+	if (ret < 0) {
 		fprintf(stderr, "Error encoding proxy frame %d for '%s'\n", 
 		        ctx->cfra - 1, ctx->of->filename);
 		return 0;
 	}
 
-	if (outsize != 0) {
-		AVPacket packet;
-		av_init_packet(&packet);
-
-		if (ctx->c->coded_frame->pts != AV_NOPTS_VALUE) {
-			packet.pts = av_rescale_q(ctx->c->coded_frame->pts,
+	if (got_output) {
+		if (packet.pts != AV_NOPTS_VALUE) {
+			packet.pts = av_rescale_q(packet.pts,
+			                          ctx->c->time_base,
+			                          ctx->st->time_base);
+		}
+		if (packet.dts != AV_NOPTS_VALUE) {
+			packet.dts = av_rescale_q(packet.dts,
 			                          ctx->c->time_base,
 			                          ctx->st->time_base);
 		}
-		if (ctx->c->coded_frame->key_frame)
-			packet.flags |= AV_PKT_FLAG_KEY;
 
 		packet.stream_index = ctx->st->index;
-		packet.data = ctx->video_buffer;
-		packet.size = outsize;
 
 		if (av_interleaved_write_frame(ctx->of, &packet) != 0) {
 			fprintf(stderr, "Error writing proxy frame %d "
@@ -651,7 +643,6 @@ static int add_to_proxy_output_ffmpeg(
 static void free_proxy_output_ffmpeg(struct proxy_output_ctx *ctx,
                                      int rollback)
 {
-	int i;
 	char fname[FILE_MAX];
 	char fname_tmp[FILE_MAX];
 
@@ -669,20 +660,12 @@ static void free_proxy_output_ffmpeg(struct proxy_output_ctx *ctx,
 	
 	avcodec_close(ctx->c);
 	
-	for (i = 0; i < ctx->of->nb_streams; i++) {
-		if (&ctx->of->streams[i]) {
-			av_freep(&ctx->of->streams[i]);
-		}
-	}
-
 	if (ctx->of->oformat) {
 		if (!(ctx->of->oformat->flags & AVFMT_NOFILE)) {
 			avio_close(ctx->of->pb);
 		}
 	}
-	av_free(ctx->of);
-
-	MEM_freeN(ctx->video_buffer);
+	avformat_free_context(ctx->of);
 
 	if (ctx->sws_ctx) {
 		sws_freeContext(ctx->sws_ctx);
@@ -692,14 +675,14 @@ static void free_proxy_output_ffmpeg(struct proxy_output_ctx *ctx,
 	}
 
 	get_proxy_filename(ctx->anim, ctx->proxy_size, 
-	                   fname_tmp, TRUE);
+	                   fname_tmp, true);
 
 	if (rollback) {
 		unlink(fname_tmp);
 	}
 	else {
 		get_proxy_filename(ctx->anim, ctx->proxy_size, 
-		                   fname, FALSE);
+		                   fname, false);
 		unlink(fname);
 		BLI_rename(fname_tmp, fname);
 	}
@@ -758,8 +741,8 @@ static IndexBuildContext *index_ffmpeg_create_context(struct anim *anim, IMB_Tim
 		return NULL;
 	}
 
-	if (av_find_stream_info(context->iFormatCtx) < 0) {
-		av_close_input_file(context->iFormatCtx);
+	if (avformat_find_stream_info(context->iFormatCtx, NULL) < 0) {
+		avformat_close_input(&context->iFormatCtx);
 		MEM_freeN(context);
 		return NULL;
 	}
@@ -779,7 +762,7 @@ static IndexBuildContext *index_ffmpeg_create_context(struct anim *anim, IMB_Tim
 		}
 
 	if (context->videoStream == -1) {
-		av_close_input_file(context->iFormatCtx);
+		avformat_close_input(&context->iFormatCtx);
 		MEM_freeN(context);
 		return NULL;
 	}
@@ -790,15 +773,15 @@ static IndexBuildContext *index_ffmpeg_create_context(struct anim *anim, IMB_Tim
 	context->iCodec = avcodec_find_decoder(context->iCodecCtx->codec_id);
 
 	if (context->iCodec == NULL) {
-		av_close_input_file(context->iFormatCtx);
+		avformat_close_input(&context->iFormatCtx);
 		MEM_freeN(context);
 		return NULL;
 	}
 
 	context->iCodecCtx->workaround_bugs = 1;
 
-	if (avcodec_open(context->iCodecCtx, context->iCodec) < 0) {
-		av_close_input_file(context->iFormatCtx);
+	if (avcodec_open2(context->iCodecCtx, context->iCodec, NULL) < 0) {
+		avformat_close_input(&context->iFormatCtx);
 		MEM_freeN(context);
 		return NULL;
 	}
@@ -808,7 +791,8 @@ static IndexBuildContext *index_ffmpeg_create_context(struct anim *anim, IMB_Tim
 			context->proxy_ctx[i] = alloc_proxy_output_ffmpeg(
 			        anim, context->iStream, proxy_sizes[i],
 			        context->iCodecCtx->width * proxy_fac[i],
-			        context->iCodecCtx->height * proxy_fac[i],
+			        av_get_cropped_height_from_codec(
+			        context->iCodecCtx) * proxy_fac[i],
 			        quality);
 			if (!context->proxy_ctx[i]) {
 				proxy_sizes_in_use &= ~proxy_sizes[i];
@@ -848,6 +832,9 @@ static void index_rebuild_ffmpeg_finish(FFmpegIndexBuilderContext *context, int
 		}
 	}
 
+	avcodec_close(context->iCodecCtx);
+	avformat_close_input(&context->iFormatCtx);
+
 	MEM_freeN(context);
 }
 
@@ -867,12 +854,12 @@ static void index_rebuild_ffmpeg_proc_decoded_frame(
 
 	if (!context->start_pts_set) {
 		context->start_pts = pts;
-		context->start_pts_set = TRUE;
+		context->start_pts_set = true;
 	}
 
-	context->frameno = floor((pts - context->start_pts) 
-				 * context->pts_time_base 
-				 * context->frame_rate + 0.5f);
+	context->frameno = floor((pts - context->start_pts) *
+	                         context->pts_time_base  *
+	                         context->frame_rate + 0.5);
 
 	/* decoding starts *always* on I-Frames,
 	 * so: P-Frames won't work, even if all the
@@ -912,11 +899,13 @@ static int index_rebuild_ffmpeg(FFmpegIndexBuilderContext *context,
 	AVPacket next_packet;
 	uint64_t stream_size;
 
+	memset(&next_packet, 0, sizeof(AVPacket));
+
 	in_frame = avcodec_alloc_frame();
 
 	stream_size = avio_size(context->iFormatCtx->pb);
 
-	context->frame_rate = av_q2d(context->iStream->r_frame_rate);
+	context->frame_rate = av_q2d(av_get_r_frame_rate_compat(context->iStream));
 	context->pts_time_base = av_q2d(context->iStream->time_base);
 
 	while (av_read_frame(context->iFormatCtx, &next_packet) >= 0) {
@@ -926,7 +915,7 @@ static int index_rebuild_ffmpeg(FFmpegIndexBuilderContext *context,
 
 		if (*progress != next_progress) {
 			*progress = next_progress;
-			*do_update = TRUE;
+			*do_update = true;
 		}
 
 		if (*stop) {
@@ -959,12 +948,13 @@ static int index_rebuild_ffmpeg(FFmpegIndexBuilderContext *context,
 	 * according to ffmpeg docs using 0-size packets.
 	 *
 	 * At least, if we haven't already stopped... */
+
+	/* this creates the 0-size packet and prevents a memory leak. */
+	av_free_packet(&next_packet);
+
 	if (!*stop) {
 		int frame_finished;
 
-		next_packet.size = 0;
-		next_packet.data = 0;
-
 		do {
 			frame_finished = 0;
 
@@ -1064,7 +1054,7 @@ static IndexBuildContext *index_fallback_create_context(struct anim *anim, IMB_T
 		if (context->proxy_sizes_in_use & proxy_sizes[i]) {
 			char fname[FILE_MAX];
 
-			get_proxy_filename(anim, proxy_sizes[i], fname, TRUE);
+			get_proxy_filename(anim, proxy_sizes[i], fname, true);
 			BLI_make_existing_file(fname);
 
 			context->proxy_ctx[i] = alloc_proxy_output_avi(anim, fname,
@@ -1087,8 +1077,8 @@ static void index_rebuild_fallback_finish(FallbackIndexBuilderContext *context,
 			AVI_close_compress(context->proxy_ctx[i]);
 			MEM_freeN(context->proxy_ctx[i]);
 
-			get_proxy_filename(anim, proxy_sizes[i], fname_tmp, TRUE);
-			get_proxy_filename(anim, proxy_sizes[i], fname, FALSE);
+			get_proxy_filename(anim, proxy_sizes[i], fname_tmp, true);
+			get_proxy_filename(anim, proxy_sizes[i], fname, false);
 
 			if (stop) {
 				unlink(fname_tmp);
@@ -1115,7 +1105,7 @@ static void index_rebuild_fallback(FallbackIndexBuilderContext *context,
 
 		if (*progress != next_progress) {
 			*progress = next_progress;
-			*do_update = TRUE;
+			*do_update = true;
 		}
 		
 		if (*stop) {
@@ -1180,7 +1170,7 @@ IndexBuildContext *IMB_anim_index_rebuild_context(struct anim *anim, IMB_Timecod
 
 	return context;
 
-	(void)tcs_in_use, (void)proxy_sizes_in_use, (void)quality;
+	UNUSED_VARS(tcs_in_use, proxy_sizes_in_use, quality);
 }
 
 void IMB_anim_index_rebuild(struct IndexBuildContext *context,
@@ -1199,7 +1189,7 @@ void IMB_anim_index_rebuild(struct IndexBuildContext *context,
 #endif
 	}
 
-	(void)stop, (void)do_update, (void)progress;
+	UNUSED_VARS(stop, do_update, progress);
 }
 
 void IMB_anim_index_rebuild_finish(IndexBuildContext *context, short stop)
@@ -1217,8 +1207,8 @@ void IMB_anim_index_rebuild_finish(IndexBuildContext *context, short stop)
 #endif
 	}
 
-	(void)stop;
-	(void)proxy_sizes;  /* static defined at top of the file */
+	/* static defined at top of the file */
+	UNUSED_VARS(stop, proxy_sizes);
 }
 
 
@@ -1269,10 +1259,10 @@ struct anim *IMB_anim_open_proxy(
 		return NULL;
 	}
 
-	get_proxy_filename(anim, preview_size, fname, FALSE);
+	get_proxy_filename(anim, preview_size, fname, false);
 
-	/* proxies are generated in default color space */
-	anim->proxy_anim[i] = IMB_open_anim(fname, 0, 0, NULL);
+	/* proxies are generated in the same color space as animation itself */
+	anim->proxy_anim[i] = IMB_open_anim(fname, 0, 0, anim->colorspace);
 	
 	anim->proxies_tried |= preview_size;
 
