@@ -27,6 +27,10 @@
 
 /** \file DNA_action_types.h
  *  \ingroup DNA
+ *
+ * Define actions data-block for the animation system.
+ * A collection of animation curves and drivers to be assigned to data-blocks
+ * or sequenced in the non-linear-editor (NLA).
  */
 
 #ifndef __DNA_ACTION_TYPES_H__
@@ -155,7 +159,9 @@ typedef enum eMotionPaths_ViewFlag {
 	/* show keyframe/frame numbers */
 	MOTIONPATH_VIEW_KFNOS       = (1 << 2),
 	/* find keyframes in whole action (instead of just in matching group name) */
-	MOTIONPATH_VIEW_KFACT       = (1 << 3)
+	MOTIONPATH_VIEW_KFACT       = (1 << 3),
+	/* draw lines on path */
+	MOTIONPATH_VIEW_LINES       = (1 << 4)
 } eMotionPath_ViewFlag;
 
 /* bAnimVizSettings->path_bakeflag */
@@ -244,6 +250,7 @@ typedef enum ePchan_Flag {
 	POSE_ROT        =   (1 << 1),
 	POSE_SIZE       =   (1 << 2),
 	/* old IK/cache stuff... */
+#if 0
 	POSE_IK_MAT     =   (1 << 3),
 	POSE_UNUSED2    =   (1 << 4),
 	POSE_UNUSED3    =   (1 << 5),
@@ -251,6 +258,7 @@ typedef enum ePchan_Flag {
 	POSE_UNUSED5    =   (1 << 7),
 	/* has Standard IK */
 	POSE_HAS_IK     =   (1 << 8),
+#endif
 	/* IK/Pose solving*/
 	POSE_CHAIN      =   (1 << 9),
 	POSE_DONE       =   (1 << 10),
@@ -259,8 +267,10 @@ typedef enum ePchan_Flag {
 	POSE_STRIDE     =   (1 << 12),
 	/* standard IK solving */
 	POSE_IKTREE     =   (1 << 13),
+#if 0
 	/* has Spline IK */
 	POSE_HAS_IKS    =   (1 << 14),
+#endif
 	/* spline IK solving */
 	POSE_IKSPLINE   =   (1 << 15)
 } ePchan_Flag;
@@ -372,7 +382,7 @@ typedef enum ePose_Flags {
 
 /* bPose->iksolver and bPose->ikparam->iksolver */
 typedef enum ePose_IKSolverType {
-	IKSOLVER_LEGACY = 0,
+	IKSOLVER_STANDARD = 0,
 	IKSOLVER_ITASC = 1
 } ePose_IKSolverType;
 
@@ -535,7 +545,7 @@ typedef enum eDopeSheet_FilterFlag {
 	ADS_FILTER_ONLYNLA          = (1 << 2),   /* for 'NLA' editor - only include NLA data from AnimData */
 	ADS_FILTER_SELEDIT          = (1 << 3),   /* for Graph Editor - used to indicate whether to include a filtering flag or not */
 
-	/* general filtering 2 */
+	/* general filtering */
 	ADS_FILTER_SUMMARY          = (1 << 4),   /* for 'DopeSheet' Editors - include 'summary' line */
 	ADS_FILTER_ONLYOBGROUP      = (1 << 5),   /* only the objects in the specified object group get used */
 
@@ -556,6 +566,10 @@ typedef enum eDopeSheet_FilterFlag {
 	ADS_FILTER_NONTREE          = (1 << 19),
 	ADS_FILTER_NOTEX            = (1 << 20),
 	ADS_FILTER_NOSPK            = (1 << 21),
+	ADS_FILTER_NOLINESTYLE      = (1 << 22),
+	ADS_FILTER_NOMODIFIERS      = (1 << 23),
+	ADS_FILTER_NOGPENCIL        = (1 << 24),
+	/* NOTE: all new datablock filters will have to go in filterflag2 (see below) */
 
 	/* NLA-specific filters */
 	ADS_FILTER_NLA_NOACT        = (1 << 25),  /* if the AnimData block has no NLA data, don't include to just show Action-line */
@@ -563,15 +577,18 @@ typedef enum eDopeSheet_FilterFlag {
 	/* general filtering 3 */
 	ADS_FILTER_INCL_HIDDEN      = (1 << 26),  /* include 'hidden' channels too (i.e. those from hidden Objects/Bones) */
 	ADS_FILTER_BY_FCU_NAME      = (1 << 27),  /* for F-Curves, filter by the displayed name (i.e. to isolate all Location curves only) */
+	ADS_FILTER_ONLY_ERRORS		= (1 << 28),  /* show only F-Curves which are disabled/have errors - for debugging drivers */
 	
 	/* combination filters (some only used at runtime) */
-	ADS_FILTER_NOOBDATA = (ADS_FILTER_NOCAM | ADS_FILTER_NOMAT | ADS_FILTER_NOLAM | ADS_FILTER_NOCUR | ADS_FILTER_NOPART | ADS_FILTER_NOARM | ADS_FILTER_NOSPK)
+	ADS_FILTER_NOOBDATA = (ADS_FILTER_NOCAM | ADS_FILTER_NOMAT | ADS_FILTER_NOLAM | ADS_FILTER_NOCUR | ADS_FILTER_NOPART | ADS_FILTER_NOARM | ADS_FILTER_NOSPK | ADS_FILTER_NOMODIFIERS)
 } eDopeSheet_FilterFlag;	
 
 /* DopeSheet general flags */
 typedef enum eDopeSheet_Flag {
 	ADS_FLAG_SUMMARY_COLLAPSED  = (1 << 0),   /* when summary is shown, it is collapsed, so all other channels get hidden */
 	ADS_FLAG_SHOW_DBFILTERS     = (1 << 1)    /* show filters for datablocks */
+	
+	/* NOTE: datablock filter flags continued (1 << 10) onwards... */
 } eDopeSheet_Flag;
 
 
@@ -646,7 +663,11 @@ typedef enum eAnimEdit_AutoSnap {
 	/* snap to actual frames/seconds (nla-action time) */
 	SACTSNAP_FRAME = 2,
 	/* snap to nearest marker */
-	SACTSNAP_MARKER = 3
+	SACTSNAP_MARKER = 3,
+	/* snap to actual seconds (nla-action time) */
+	SACTSNAP_SECOND = 4,
+	/* snap to 1.0 second increments */
+	SACTSNAP_TSTEP = 5
 } eAnimEdit_AutoSnap;
 
 
@@ -688,4 +709,4 @@ typedef enum ACHAN_FLAG {
 	ACHAN_MOVED     = (1 << 31)
 } ACHAN_FLAG; 
 
-#endif
+#endif  /* __DNA_ACTION_TYPES_H__ */
