@@ -27,6 +27,8 @@
 
 /** \file DNA_actuator_types.h
  *  \ingroup DNA
+ *
+ * #bActuator type is specifically for use by Object logic-bricks in the game-engine.
  */
 
 #ifndef __DNA_ACTUATOR_TYPES_H__
@@ -46,21 +48,21 @@ typedef struct bAddObjectActuator {
 	struct Object *ob;
 } bAddObjectActuator;
 
-typedef struct bActionActuator {								
-	struct bAction *act;	/* Pointer to action */				
-	short	type, flag;		/* Playback type */  // not in use
-	float	sta, end;		/* Start & End frames */			
-	char	name[64];		/* For property-driven playback, MAX_NAME */	
-	char	frameProp[64];	/* Set this property to the actions current frame, MAX_NAME */
-	short	blendin;		/* Number of frames of blending */
-	short	priority;		/* Execution priority */
-	short	layer;			/* Animation layer */
-	short	end_reset;	/* Ending the actuator (negative pulse) wont reset the the action to its starting frame */
-	short	strideaxis;		/* Displacement axis */
-	short	pad;
-	float	stridelength;	/* Displacement incurred by cycle */ // not in use
-	float	layer_weight;	/* How much of the previous layer to use for blending. (<0 = disable, 0 = add mode) */
-} bActionActuator;												
+typedef struct bActionActuator {
+	struct bAction *act;    /* Pointer to action */
+	short   type, flag;     /* Playback type */  // not in use
+	float   sta, end;       /* Start & End frames */
+	char    name[64];       /* For property-driven playback, MAX_NAME */
+	char    frameProp[64];  /* Set this property to the actions current frame, MAX_NAME */
+	short   blendin;        /* Number of frames of blending */
+	short   priority;       /* Execution priority */
+	short   layer;          /* Animation layer */
+	short   end_reset;      /* Ending the actuator (negative pulse) wont reset the action to its starting frame */
+	short   strideaxis;     /* Displacement axis */
+	short   blend_mode;		/* Layer blending mode */
+	float   stridelength;   /* Displacement incurred by cycle */ // not in use
+	float   layer_weight;   /* How much of the previous layer to use for blending. (<0 = disable, 0 = add mode) */
+} bActionActuator;
 
 typedef struct Sound3D {
 	float min_gain;
@@ -95,6 +97,8 @@ typedef struct bEditObjectActuator {
 	float mass;
 	short localflag; /* flag for the lin & ang. vel: apply locally   */
 	short dyn_operation;
+	short upflag, trackflag; /* flag for up axis and track axis */
+	int pad;
 } bEditObjectActuator;
 
 typedef struct bSceneActuator {
@@ -243,6 +247,18 @@ typedef struct bSteeringActuator {
 	struct Object *navmesh;
 } bSteeringActuator;
 
+typedef struct bMouseActuator {
+	short type; /* 0=Visibility, 1=Look */
+	short flag;
+
+	int object_axis[2];
+	float threshold[2];
+	float sensitivity[2];
+	float limit_x[2];
+	float limit_y[2];
+} bMouseActuator;
+
+
 typedef struct bActuator {
 	struct bActuator *next, *prev, *mynew;
 	short type;
@@ -261,8 +277,8 @@ typedef struct bActuator {
 	/**
 	 * For ipo's and props: to find out which object the actuator
 	 * belongs to */
-	struct Object *ob;		
-	
+	struct Object *ob;
+
 } bActuator;
 
 /* objectactuator->flag */
@@ -277,10 +293,13 @@ typedef struct bActuator {
 #define ACT_ANG_VEL_LOCAL		32
 //#define ACT_ADD_LIN_VEL_LOCAL	64
 #define ACT_ADD_LIN_VEL			64
+#define ACT_ADD_CHAR_LOC		128
+#define ACT_CHAR_JUMP			256
 
 /* objectactuator->type */
-#define ACT_OBJECT_NORMAL	0
-#define ACT_OBJECT_SERVO	1
+#define ACT_OBJECT_NORMAL		0
+#define ACT_OBJECT_SERVO		1
+#define ACT_OBJECT_CHARACTER	2
 
 /* actuator->type */
 #define ACT_OBJECT		0
@@ -309,6 +328,7 @@ typedef struct bActuator {
 #define ACT_STATE		22
 #define ACT_ARMATURE	23
 #define ACT_STEERING    24
+#define ACT_MOUSE		25
 
 /* actuator flag */
 #define ACT_SHOW		1
@@ -317,6 +337,7 @@ typedef struct bActuator {
 #define ACT_LINKED		8	
 #define ACT_VISIBLE		16	
 #define ACT_PIN			32
+#define ACT_DEACTIVATE  64
 
 /* link codes */
 #define LINK_SENSOR		0
@@ -336,6 +357,10 @@ typedef struct bActuator {
 #define ACT_ACTION_FROM_PROP	6
 #define ACT_ACTION_MOTION		7
 
+/* actionactuator->blend_mode */
+#define ACT_ACTION_BLEND		0
+#define ACT_ACTION_ADD			1
+
 /* ipoactuator->type */
 #define ACT_IPO_PLAY		0
 #define ACT_IPO_PINGPONG	1
@@ -372,6 +397,7 @@ typedef struct bActuator {
 #define ACT_PROP_ADD		1
 #define ACT_PROP_COPY		2
 #define ACT_PROP_TOGGLE		3
+#define ACT_PROP_LEVEL		4
 
 /* constraint flag */
 #define ACT_CONST_NONE		0
@@ -416,6 +442,19 @@ typedef struct bActuator {
 /* editObjectActuator->flag */
 #define ACT_TRACK_3D			1
 
+/* editObjectActuator->upflag */
+#define ACT_TRACK_UP_X			0
+#define ACT_TRACK_UP_Y			1
+#define ACT_TRACK_UP_Z			2
+
+/* editObjectActuator->trackflag */
+#define ACT_TRACK_TRAXIS_X			0
+#define ACT_TRACK_TRAXIS_Y			1
+#define ACT_TRACK_TRAXIS_Z			2
+#define ACT_TRACK_TRAXIS_NEGX		3
+#define ACT_TRACK_TRAXIS_NEGY		4
+#define ACT_TRACK_TRAXIS_NEGZ		5
+
 /* editObjectActuator->flag for replace mesh actuator */
 #define ACT_EDOB_REPLACE_MESH_NOGFX		2 /* use for replace mesh actuator */
 #define ACT_EDOB_REPLACE_MESH_PHYS		4
@@ -532,6 +571,22 @@ typedef struct bActuator {
 #define ACT_STEERING_AUTOMATICFACING   4
 #define ACT_STEERING_NORMALUP  8
 
-#endif
+/* mouseactuator->type */
+#define ACT_MOUSE_VISIBILITY	0
+#define ACT_MOUSE_LOOK			1
 
+/* mouseactuator->flag */
+#define ACT_MOUSE_VISIBLE	(1 << 0)
+#define ACT_MOUSE_USE_AXIS_X	(1 << 1)
+#define ACT_MOUSE_USE_AXIS_Y	(1 << 2)
+#define ACT_MOUSE_RESET_X	(1 << 3)
+#define ACT_MOUSE_RESET_Y	(1 << 4)
+#define ACT_MOUSE_LOCAL_X	(1 << 5)
+#define ACT_MOUSE_LOCAL_Y	(1 << 6)
 
+/* mouseactuator->object_axis */
+#define ACT_MOUSE_OBJECT_AXIS_X	0
+#define ACT_MOUSE_OBJECT_AXIS_Y	1
+#define ACT_MOUSE_OBJECT_AXIS_Z	2
+
+#endif  /* __DNA_ACTUATOR_TYPES_H__ */
