@@ -44,11 +44,19 @@ struct SpaceNode;
 struct bNodeLink;
 struct bNodeType;
 struct bNodeTreeExec;
+struct bNodePreview;
+struct bNodeInstanceHash;
 struct AnimData;
 struct bGPdata;
 struct uiBlock;
 struct Image;
 
+/* In writefile.c: write deprecated DNA data,
+ * to ensure forward compatibility in 2.6x versions.
+ * Will be removed eventually.
+ */
+#define USE_NODE_COMPAT_CUSTOMNODES
+
 #define NODE_MAXSTR 64
 
 typedef struct bNodeStack {
@@ -79,13 +87,22 @@ typedef struct bNodeStack {
 typedef struct bNodeSocket {
 	struct bNodeSocket *next, *prev, *new_sock;
 	
+	IDProperty *prop;			/* user-defined properties */
+	
+	char identifier[64];		/* unique identifier for mapping */
+	
 	char name[64];	/* MAX_NAME */
 	
+	/* XXX deprecated, only used for the Image and OutputFile nodes,
+	 * should be removed at some point.
+	 */
 	void *storage;				/* custom storage */
 	
 	short type, flag;
 	short limit;				/* max. number of links */
-	short pad1;
+	short in_out;				/* input/output type */
+	struct bNodeSocketType *typeinfo;	/* runtime type information */
+	char idname[64];			/* runtime type identifier */
 	
 	float locx, locy;
 	
@@ -95,14 +112,18 @@ typedef struct bNodeSocket {
 	short stack_index;			/* local stack index */
 	/* XXX deprecated, kept for forward compatibility */
 	short stack_type  DNA_DEPRECATED;
-	int resizemode;				/* compositor resize mode of the socket */
+	int pad;
+	
 	void *cache;				/* cached data from execution */
 	
-	/* internal data to retrieve relations and groups */
-	int own_index;				/* group socket identifiers, to find matching pairs after reading files */
+	/* internal data to retrieve relations and groups
+	 * DEPRECATED, now uses the generic identifier string instead
+	 */
+	int own_index  DNA_DEPRECATED;	/* group socket identifiers, to find matching pairs after reading files */
 	/* XXX deprecated, only used for restoring old group node links */
 	int to_index  DNA_DEPRECATED;
-	struct bNodeSocket *groupsock;
+	/* XXX deprecated, still forward compatible since verification restores pointer from matching own_index. */
+	struct bNodeSocket *groupsock  DNA_DEPRECATED;
 	
 	struct bNodeLink *link;		/* a link pointer, set in ntreeUpdateTree */
 
@@ -111,51 +132,49 @@ typedef struct bNodeSocket {
 } bNodeSocket;
 
 /* sock->type */
-#define SOCK_FLOAT			0
-#define SOCK_VECTOR			1
-#define SOCK_RGBA			2
-#define SOCK_SHADER			3
-#define SOCK_BOOLEAN		4
-#define SOCK_MESH			5
-#define SOCK_INT			6
-#define NUM_SOCKET_TYPES	7	/* must be last! */
+typedef enum eNodeSocketDatatype {
+	SOCK_CUSTOM			= -1,	/* socket has no integer type */
+	SOCK_FLOAT			= 0,
+	SOCK_VECTOR			= 1,
+	SOCK_RGBA			= 2,
+	SOCK_SHADER			= 3,
+	SOCK_BOOLEAN		= 4,
+	__SOCK_MESH			= 5,	/* deprecated */
+	SOCK_INT			= 6,
+	SOCK_STRING			= 7
+} eNodeSocketDatatype;
 
 /* socket side (input/output) */
-#define SOCK_IN		1
-#define SOCK_OUT	2
+typedef enum eNodeSocketInOut {
+	SOCK_IN = 1,
+	SOCK_OUT = 2
+} eNodeSocketInOut;
 
 /* sock->flag, first bit is select */
-	/* hidden is user defined, to hide unused */
-#define SOCK_HIDDEN				2
-	/* for quick check if socket is linked */
-#define SOCK_IN_USE				4	/* XXX deprecated */
-	/* unavailable is for dynamic sockets */
-#define SOCK_UNAVAIL			8
-	/* dynamic socket (can be modified by user) */
-#define SOCK_DYNAMIC			16
-	/* group socket should not be exposed */
-#define SOCK_INTERNAL			32
-	/* socket collapsed in UI */
-#define SOCK_COLLAPSED			64
-	/* hide socket value, if it gets auto default */
-#define SOCK_HIDE_VALUE			128
-	/* socket hidden automatically, to distinguish from manually hidden */
-	/* DEPRECATED, only kept here to avoid reusing the flag */
-#define SOCK_AUTO_HIDDEN__DEPRECATED	256
-
-typedef struct bNodePreview {
-	unsigned char *rect;
-	short xsize, ysize;
-	int pad;
-} bNodePreview;
+typedef enum eNodeSocketFlag {
+	SOCK_HIDDEN = 2,					/* hidden is user defined, to hide unused */
+	SOCK_IN_USE = 4,					/* for quick check if socket is linked */
+	SOCK_UNAVAIL = 8,					/* unavailable is for dynamic sockets */
+	// SOCK_DYNAMIC = 16,				/* DEPRECATED  dynamic socket (can be modified by user) */
+	// SOCK_INTERNAL = 32,				/* DEPRECATED  group socket should not be exposed */
+	SOCK_COLLAPSED = 64,				/* socket collapsed in UI */
+	SOCK_HIDE_VALUE = 128,				/* hide socket value, if it gets auto default */
+	SOCK_AUTO_HIDDEN__DEPRECATED = 256,	/* socket hidden automatically, to distinguish from manually hidden */
+	SOCK_NO_INTERNAL_LINK = 512
+} eNodeSocketFlag;
 
 /* limit data in bNode to what we want to see saved? */
 typedef struct bNode {
 	struct bNode *next, *prev, *new_node;
+
+	IDProperty *prop;		/* user-defined properties */
+	
+	struct bNodeType *typeinfo;	/* runtime type information */
+	char idname[64];			/* runtime type identifier */
 	
 	char name[64];	/* MAX_NAME */
 	int flag;
-	short type, pad2;
+	short type, pad;
 	short done, level;		/* both for dependency and sorting */
 	short lasty, menunr;	/* lasty: check preview render status, menunr: browse ID blocks */
 	short stack_index;		/* for groupnode, offset in global caller stack */
@@ -167,6 +186,7 @@ typedef struct bNode {
 	struct ID *id;			/* optional link to libdata */
 	void *storage;			/* custom data, must be struct, for storage in file */
 	struct bNode *original;	/* the original node in the tree (for localized tree) */
+	ListBase internal_links; /* list of cached internal links (input to output), for muted nodes and operators */
 	
 	float locx, locy;		/* root offset for drawing (parent space) */
 	float width, height;	/* node custom width and height */
@@ -184,10 +204,17 @@ typedef struct bNode {
 	rctf totr;				/* entire boundbox (worldspace) */
 	rctf butr;				/* optional buttons area */
 	rctf prvr;				/* optional preview area */
-	bNodePreview *preview;	/* optional preview image */
+	/* XXX TODO
+	 * Node totr size depends on the prvr size, which in turn is determined from preview size.
+	 * In earlier versions bNodePreview was stored directly in nodes, but since now there can be
+	 * multiple instances using different preview images it is possible that required node size varies between instances.
+	 * preview_xsize, preview_ysize defines a common reserved size for preview rect for now,
+	 * could be replaced by more accurate node instance drawing, but that requires removing totr from DNA
+	 * and replacing all uses with per-instance data.
+	 */
+	short preview_xsize, preview_ysize;	/* reserved size of the preview rect */
+	int pad2;
 	struct uiBlock *block;	/* runtime during drawing */
-	
-	struct bNodeType *typeinfo;	/* lookup of callbacks and defaults */
 } bNode;
 
 /* node->flag */
@@ -198,12 +225,12 @@ typedef struct bNode {
 #define NODE_ACTIVE			16
 #define NODE_ACTIVE_ID		32
 #define NODE_DO_OUTPUT		64
-#define NODE_GROUP_EDIT		128
+#define __NODE_GROUP_EDIT	128		/* DEPRECATED */
 	/* free test flag, undefined */
 #define NODE_TEST			256
 	/* node is disabled */
 #define NODE_MUTED			512
-#define NODE_CUSTOM_NAME	1024	/* deprecated! */
+// #define NODE_CUSTOM_NAME	1024	/* deprecated! */
 	/* group node types: use const outputs by default */
 #define NODE_CONST_OUTPUT	(1<<11)
 	/* node is always behind others */
@@ -219,6 +246,17 @@ typedef struct bNode {
 #define NODE_ACTIVE_TEXTURE	(1<<14)
 	/* use a custom color for the node */
 #define NODE_CUSTOM_COLOR	(1<<15)
+	/* Node has been initialized
+	 * This flag indicates the node->typeinfo->init function has been called.
+	 * In case of undefined type at creation time this can be delayed until
+	 * until the node type is registered.
+	 */
+#define NODE_INIT			(1<<16)
+
+	/* do recalc of output, used to skip recalculation of unwanted
+	 * composite out nodes when editing tree
+	 */
+#define NODE_DO_OUTPUT_RECALC	(1<<17)
 
 /* node->update */
 /* XXX NODE_UPDATE is a generic update flag. More fine-grained updates
@@ -227,6 +265,36 @@ typedef struct bNode {
 #define NODE_UPDATE			0xFFFF	/* generic update flag (includes all others) */
 #define NODE_UPDATE_ID		1		/* associated id data block has changed */
 
+/* Unique hash key for identifying node instances
+ * Defined as a struct because DNA does not support other typedefs.
+ */
+typedef struct bNodeInstanceKey
+{
+	unsigned int value;
+} bNodeInstanceKey;
+
+/* Base struct for entries in node instance hash.
+ * WARNING: pointers are cast to this struct internally,
+ * it must be first member in hash entry structs!
+ */
+typedef struct bNodeInstanceHashEntry {
+	bNodeInstanceKey key;
+	
+	/* tags for cleaning the cache */
+	short tag;
+	short pad;
+} bNodeInstanceHashEntry;
+
+
+typedef struct bNodePreview {
+	bNodeInstanceHashEntry hash_entry;	/* must be first */
+	
+	unsigned char *rect;
+	short xsize, ysize;
+	int pad;
+} bNodePreview;
+
+
 typedef struct bNodeLink {
 	struct bNodeLink *next, *prev;
 	
@@ -240,6 +308,7 @@ typedef struct bNodeLink {
 /* link->flag */
 #define NODE_LINKFLAG_HILITE	1		/* link has been successfully validated */
 #define NODE_LINK_VALID			2
+#define NODE_LINK_TEST			4		/* free test flag, undefined */
 
 /* tree->edit_quality/tree->render_quality */
 #define NTREE_QUALITY_HIGH    0
@@ -260,7 +329,13 @@ typedef struct bNodeTree {
 	ID id;
 	struct AnimData *adt;		/* animation data (must be immediately after id for utilities to use it) */ 
 	
+	struct bNodeTreeType *typeinfo;	/* runtime type information */
+	char idname[64];				/* runtime type identifier */
+	
+	struct StructRNA *interface_type;	/* runtime RNA type of the group interface */
+	
 	struct bGPdata *gpd;		/* grease pencil data */
+	float view_center[2];		/* node tree stores own offset for consistent editor view */
 	
 	ListBase nodes, links;
 	
@@ -269,14 +344,33 @@ typedef struct bNodeTree {
 									 * will increase this counter */
 	int flag;
 	int update;						/* update flags */
+	short is_updating;				/* flag to prevent reentrant update calls */
+	short done;						/* generic temporary flag for recursion check (DFS/BFS) */
+	int pad2;
 	
-	int nodetype;					/* specific node type this tree is used for */
+	int nodetype DNA_DEPRECATED;	/* specific node type this tree is used for */
 
 	short edit_quality;				/* Quality setting when editing */
 	short render_quality;				/* Quality setting when rendering */
 	int chunksize;					/* tile size for compositor engine */
 	
-	ListBase inputs, outputs;		/* external sockets for group nodes */
+	rctf viewer_border;
+	
+	/* Lists of bNodeSocket to hold default values and own_index.
+	 * Warning! Don't make links to these sockets, input/output nodes are used for that.
+	 * These sockets are used only for generating external interfaces.
+	 */
+	ListBase inputs, outputs;
+	
+	/* Node preview hash table
+	 * Only available in base node trees (e.g. scene->node_tree)
+	 */
+	struct bNodeInstanceHash *previews;
+	/* Defines the node tree instance to use for the "active" context,
+	 * in case multiple different editors are used and make context ambiguous.
+	 */
+	bNodeInstanceKey active_viewer_key;
+	int pad;
 	
 	/* execution data */
 	/* XXX It would be preferable to completely move this data out of the underlying node tree,
@@ -291,38 +385,49 @@ typedef struct bNodeTree {
 	void (*progress)(void *, float progress);
 	void (*stats_draw)(void *, char *str);
 	int (*test_break)(void *);
-	void *tbh, *prh, *sdh;
+	void (*update_draw)(void *);
+	void *tbh, *prh, *sdh, *udh;
 	
 } bNodeTree;
 
 /* ntree->type, index */
+#define NTREE_CUSTOM		-1		/* for dynamically registered custom types */
 #define NTREE_SHADER		0
 #define NTREE_COMPOSIT		1
 #define NTREE_TEXTURE		2
-#define NUM_NTREE_TYPES		3
 
 /* ntree->init, flag */
 #define NTREE_TYPE_INIT		1
 
 /* ntree->flag */
-#define NTREE_DS_EXPAND		1	/* for animation editors */
-#define NTREE_COM_OPENCL	2	/* use opencl */
-#define NTREE_TWO_PASS		4	/* two pass */
+#define NTREE_DS_EXPAND				1	/* for animation editors */
+#define NTREE_COM_OPENCL			2	/* use opencl */
+#define NTREE_TWO_PASS				4	/* two pass */
+#define NTREE_COM_GROUPNODE_BUFFER	8	/* use groupnode buffers */
+#define NTREE_VIEWER_BORDER			16	/* use a border for viewer nodes */
+#define NTREE_IS_LOCALIZED			32	/* tree is localized copy, free when deleting node groups */
+
 /* XXX not nice, but needed as a temporary flags
  * for group updates after library linking.
  */
-#define NTREE_DO_VERSIONS_GROUP_EXPOSE	1024
+#define NTREE_DO_VERSIONS_GROUP_EXPOSE_2_56_2	1024	/* changes from r35033 */
+#define NTREE_DO_VERSIONS_CUSTOMNODES_GROUP		2048	/* custom_nodes branch: remove links to node tree sockets */
+#define NTREE_DO_VERSIONS_CUSTOMNODES_GROUP_CREATE_INTERFACE	4096	/* custom_nodes branch: create group input/output nodes */
 
 /* ntree->update */
-#define NTREE_UPDATE			0xFFFF	/* generic update flag (includes all others) */
-#define NTREE_UPDATE_LINKS		1		/* links have been added or removed */
-#define NTREE_UPDATE_NODES		2		/* nodes or sockets have been added or removed */
-#define NTREE_UPDATE_GROUP_IN	16		/* group inputs have changed */
-#define NTREE_UPDATE_GROUP_OUT	32		/* group outputs have changed */
-#define NTREE_UPDATE_GROUP		48		/* group has changed (generic flag including all other group flags) */
+typedef enum eNodeTreeUpdate {
+	NTREE_UPDATE            = 0xFFFF,	/* generic update flag (includes all others) */
+	NTREE_UPDATE_LINKS      = 1,		/* links have been added or removed */
+	NTREE_UPDATE_NODES      = 2,		/* nodes or sockets have been added or removed */
+	NTREE_UPDATE_GROUP_IN   = 16,		/* group inputs have changed */
+	NTREE_UPDATE_GROUP_OUT  = 32,		/* group outputs have changed */
+	NTREE_UPDATE_GROUP      = 48		/* group has changed (generic flag including all other group flags) */
+} eNodeTreeUpdate;
 
 
-/* socket value structs for input buttons */
+/* socket value structs for input buttons
+ * DEPRECATED now using ID properties
+ */
 
 typedef struct bNodeSocketValueInt {
 	int subtype;				/* RNA subtype */
@@ -351,6 +456,11 @@ typedef struct bNodeSocketValueRGBA {
 	float value[4];
 } bNodeSocketValueRGBA;
 
+typedef struct bNodeSocketValueString {
+	int subtype;
+	int pad;
+	char value[1024];	/* 1024 = FILEMAX */
+} bNodeSocketValueString;
 
 /* data structs, for node->storage */
 enum {
@@ -570,19 +680,15 @@ typedef struct NodeLensDist {
 } NodeLensDist;
 
 typedef struct NodeColorBalance {
-	/* for processing */
+	/* ASC CDL parameters */
 	float slope[3];
 	float offset[3];
 	float power[3];
 	
-	/* for ui representation */
+	/* LGG parameters */
 	float lift[3];
 	float gamma[3];
 	float gain[3];
-
-	/* temp storage for inverted lift */
-	float lift_lgg[3];
-	float gamma_inv[3];
 } NodeColorBalance;
 
 typedef struct NodeColorspill {
@@ -607,8 +713,10 @@ typedef struct NodeTexBase {
 
 typedef struct NodeTexSky {
 	NodeTexBase base;
+	int sky_model;
 	float sun_direction[3];
 	float turbidity;
+	float ground_albedo;
 } NodeTexSky;
 
 typedef struct NodeTexImage {
@@ -617,7 +725,7 @@ typedef struct NodeTexImage {
 	int color_space;
 	int projection;
 	float projection_blend;
-	int pad;
+	int interpolation;
 } NodeTexImage;
 
 typedef struct NodeTexChecker {
@@ -675,6 +783,12 @@ typedef struct NodeShaderAttribute {
 	char name[64];
 } NodeShaderAttribute;
 
+typedef struct NodeShaderVectTransform {
+	int type;
+	int convert_from, convert_to;
+	int pad;
+} NodeShaderVectTransform;
+
 /* TEX_output */
 typedef struct TexNodeOutput {
 	char name[64];
@@ -702,10 +816,63 @@ typedef struct NodeTrackPosData {
 	char track_name[64];
 } NodeTrackPosData;
 
+typedef struct NodeTranslateData {
+	char wrap_axis;
+	char relative;
+	char pad[6];
+} NodeTranslateData;
+
+typedef struct NodePlaneTrackDeformData {
+	char tracking_object[64];
+	char plane_track_name[64];
+} NodePlaneTrackDeformData;
+
+typedef struct NodeShaderScript {
+	int mode;
+	int flag;
+
+	char filepath[1024]; /* 1024 = FILE_MAX */
+
+	char bytecode_hash[64];
+	char *bytecode;
+} NodeShaderScript;
+
+typedef struct NodeShaderTangent {
+	int direction_type;
+	int axis;
+	char uv_map[64];
+} NodeShaderTangent;
+
+typedef struct NodeShaderNormalMap {
+	int space;
+	char uv_map[64];
+} NodeShaderNormalMap;
+
+typedef struct NodeShaderUVMap {
+	char uv_map[64];
+} NodeShaderUVMap;
+
+typedef struct NodeSunBeams {
+	float source[2];
+
+	float ray_length;
+} NodeSunBeams;
+
+/* script node mode */
+#define NODE_SCRIPT_INTERNAL		0
+#define NODE_SCRIPT_EXTERNAL		1
+
+/* script node flag */
+#define NODE_SCRIPT_AUTO_UPDATE		1
+
+
 /* frame node flags */
 #define NODE_FRAME_SHRINK		1	/* keep the bounding box minimal */
 #define NODE_FRAME_RESIZEABLE	2	/* test flag, if frame can be resized by user */
 
+/* proxy node flags */
+#define NODE_PROXY_AUTOTYPE			1	/* automatically change output type based on link */
+
 /* comp channel matte */
 #define CMP_NODE_CHANNEL_MATTE_CS_RGB	1
 #define CMP_NODE_CHANNEL_MATTE_CS_HSV	2
@@ -713,9 +880,27 @@ typedef struct NodeTrackPosData {
 #define CMP_NODE_CHANNEL_MATTE_CS_YCC	4
 
 /* glossy distributions */
-#define SHD_GLOSSY_BECKMANN	0
-#define SHD_GLOSSY_SHARP	1
-#define SHD_GLOSSY_GGX		2
+#define SHD_GLOSSY_BECKMANN				0
+#define SHD_GLOSSY_SHARP				1
+#define SHD_GLOSSY_GGX					2
+#define SHD_GLOSSY_ASHIKHMIN_SHIRLEY	3
+
+/* vector transform */
+#define SHD_VECT_TRANSFORM_TYPE_VECTOR	0
+#define SHD_VECT_TRANSFORM_TYPE_POINT	1
+#define SHD_VECT_TRANSFORM_TYPE_NORMAL	2
+
+#define SHD_VECT_TRANSFORM_SPACE_WORLD	0
+#define SHD_VECT_TRANSFORM_SPACE_OBJECT	1
+#define SHD_VECT_TRANSFORM_SPACE_CAMERA	2
+
+/* toon modes */
+#define SHD_TOON_DIFFUSE	0
+#define SHD_TOON_GLOSSY		1
+
+/* hair components */
+#define SHD_HAIR_REFLECTION		0
+#define SHD_HAIR_TRANSMISSION		1
 
 /* blend texture */
 #define SHD_BLEND_LINEAR			0
@@ -762,9 +947,9 @@ typedef struct NodeTrackPosData {
 #define SHD_WAVE_BANDS		0
 #define SHD_WAVE_RINGS		1
 
-#define SHD_WAVE_SINE	0
-#define SHD_WAVE_SAW	1
-#define SHD_WAVE_TRI	2
+/* sky texture */
+#define SHD_SKY_OLD		0
+#define SHD_SKY_NEW		1
 
 /* image/environment texture */
 #define SHD_COLORSPACE_NONE		0
@@ -778,11 +963,119 @@ typedef struct NodeTrackPosData {
 #define SHD_PROJ_FLAT				0
 #define SHD_PROJ_BOX				1
 
+/* image texture interpolation */
+#define SHD_INTERP_LINEAR		0
+#define SHD_INTERP_CLOSEST		1
+#define SHD_INTERP_CUBIC			2
+#define SHD_INTERP_SMART			3
+
+/* tangent */
+#define SHD_TANGENT_RADIAL			0
+#define SHD_TANGENT_UVMAP			1
+
+/* tangent */
+#define SHD_TANGENT_AXIS_X			0
+#define SHD_TANGENT_AXIS_Y			1
+#define SHD_TANGENT_AXIS_Z			2
+
+/* normal map space */
+#define SHD_NORMAL_MAP_TANGENT			0
+#define SHD_NORMAL_MAP_OBJECT			1
+#define SHD_NORMAL_MAP_WORLD			2
+#define SHD_NORMAL_MAP_BLENDER_OBJECT	3
+#define SHD_NORMAL_MAP_BLENDER_WORLD	4
+
+/* math node clamp */
+#define SHD_MATH_CLAMP		1
+
+/* Math node operation/ */
+enum {
+	NODE_MATH_ADD     = 0,
+	NODE_MATH_SUB     = 1,
+	NODE_MATH_MUL     = 2,
+	NODE_MATH_DIVIDE  = 3,
+	NODE_MATH_SIN     = 4,
+	NODE_MATH_COS     = 5,
+	NODE_MATH_TAN     = 6,
+	NODE_MATH_ASIN    = 7,
+	NODE_MATH_ACOS    = 8,
+	NODE_MATH_ATAN    = 9,
+	NODE_MATH_POW     = 10,
+	NODE_MATH_LOG     = 11,
+	NODE_MATH_MIN     = 12,
+	NODE_MATH_MAX     = 13,
+	NODE_MATH_ROUND   = 14,
+	NODE_MATH_LESS    = 15,
+	NODE_MATH_GREATER = 16,
+	NODE_MATH_MOD     = 17,
+	NODE_MATH_ABS     = 18,
+};
+
+/* mix rgb node flags */
+#define SHD_MIXRGB_USE_ALPHA	1
+#define SHD_MIXRGB_CLAMP	2
+
+/* subsurface */
+enum {
+	SHD_SUBSURFACE_COMPATIBLE		= 0, // Deprecated
+	SHD_SUBSURFACE_CUBIC			= 1,
+	SHD_SUBSURFACE_GAUSSIAN			= 2,
+};
+
+#if (DNA_DEPRECATED_GCC_POISON == 1)
+#pragma GCC poison SHD_SUBSURFACE_COMPATIBLE
+#endif
+
 /* blur node */
 #define CMP_NODE_BLUR_ASPECT_NONE		0
 #define CMP_NODE_BLUR_ASPECT_Y			1
 #define CMP_NODE_BLUR_ASPECT_X			2
 
+/* wrapping */
+#define CMP_NODE_WRAP_NONE		0
+#define CMP_NODE_WRAP_X			1
+#define CMP_NODE_WRAP_Y			2
+#define CMP_NODE_WRAP_XY		3
+
 #define CMP_NODE_MASK_MBLUR_SAMPLES_MAX 64
 
+/* geometry output socket defines */
+#define GEOM_OUT_GLOB	0
+#define GEOM_OUT_LOCAL	1
+#define GEOM_OUT_VIEW	2
+#define GEOM_OUT_ORCO	3
+#define GEOM_OUT_UV		4
+#define GEOM_OUT_NORMAL	5
+#define GEOM_OUT_VCOL	6
+#define GEOM_OUT_VCOL_ALPHA	7
+#define GEOM_OUT_FRONTBACK	8
+
+/* material input socket defines */
+#define MAT_IN_COLOR	0
+#define MAT_IN_SPEC		1
+#define MAT_IN_REFL		2
+#define MAT_IN_NORMAL	3
+#define MAT_IN_MIR		4
+#define MAT_IN_AMB		5
+#define MAT_IN_EMIT	6
+#define MAT_IN_SPECTRA	7
+#define MAT_IN_RAY_MIRROR	8
+#define MAT_IN_ALPHA	9
+#define MAT_IN_TRANSLUCENCY	10
+#define NUM_MAT_IN		11	/* for array size */
+
+/* material output socket defines */
+#define MAT_OUT_COLOR		0
+#define MAT_OUT_ALPHA		1
+#define MAT_OUT_NORMAL	2
+#define MAT_OUT_DIFFUSE	3
+#define MAT_OUT_SPEC		4
+#define MAT_OUT_AO		5
+
+/* image */
+#define CMP_NODE_IMAGE_USE_STRAIGHT_OUTPUT	1
+
+/* viewer and cmposite output */
+#define CMP_NODE_OUTPUT_IGNORE_ALPHA		1
+
 #endif
