@@ -116,6 +116,9 @@ typedef struct ParticleData {
 
 	float size;				/* size and multiplier so that we can update size when ever */
 
+	float sphdensity;		/* density of sph particle */
+	int pad;
+
 	int hair_index;
 	short flag;
 	short alive;			/* the life state of a particle */
@@ -130,6 +133,8 @@ typedef struct SPHFluidSettings {
 	float stiffness_k, stiffness_knear, rest_density;
 	float buoyancy;
 	int flag, spring_frames;
+	short solver;
+	short pad[3];
 } SPHFluidSettings;
 
 /* fluid->flag */
@@ -141,6 +146,10 @@ typedef struct SPHFluidSettings {
 #define SPH_FAC_VISCOSITY			32
 #define SPH_FAC_REST_LENGTH			64
 
+/* fluid->solver (numerical ID field, not bitfield) */
+#define SPH_SOLVER_DDR					0
+#define SPH_SOLVER_CLASSICAL			1
+
 typedef struct ParticleSettings {
 	ID id;
 	struct AnimData *adt;
@@ -164,7 +173,8 @@ typedef struct ParticleSettings {
 	/* adaptive path rendering */
 	short adapt_angle, adapt_pix;
 
-	short disp, omat, interpolation, rotfrom, integrator;
+	short disp, omat, interpolation, integrator;
+	short rotfrom DNA_DEPRECATED;
 	short kink, kink_axis;
 
 	/* billboards */
@@ -233,6 +243,11 @@ typedef struct ParticleSettings {
 	struct Ipo *ipo  DNA_DEPRECATED;  /* old animation system, deprecated for 2.5 */
 	struct PartDeflect *pd;
 	struct PartDeflect *pd2;
+
+	/* modified dm support */
+	short use_modifier_stack;
+	short pad[3];
+
 } ParticleSettings;
 
 typedef struct ParticleSystem {
@@ -257,7 +272,9 @@ typedef struct ParticleSystem {
 	struct DerivedMesh *hair_in_dm, *hair_out_dm;	/* input/output for cloth simulation */
 
 	struct Object *target_ob;
-	struct Object *lattice;
+
+	struct LatticeDeformData *lattice_deform_data;		/* run-time only lattice deformation data */
+
 	struct Object *parent;					/* particles from global space -> parent space */
 
 	struct ListBase targets;				/* used for keyed and boid physics */
@@ -276,7 +293,7 @@ typedef struct ParticleSystem {
 	short vgroup[12], vg_neg, rt3;			/* vertex groups, 0==disable, 1==starting index */
 
 	/* temporary storage during render */
-	void *renderdata;
+	struct ParticleRenderData *renderdata;
 
 	/* point cache */
 	struct PointCache *pointcache;
@@ -287,13 +304,11 @@ typedef struct ParticleSystem {
 	ParticleSpring *fluid_springs;
 	int tot_fluidsprings, alloc_fluidsprings;
 
-	struct KDTree *tree;								/* used for interactions with self and other systems */
-	struct BVHTree *bvhtree;								/* used for interactions with self and other systems */
+	struct KDTree *tree;					/* used for interactions with self and other systems */
+	struct BVHTree *bvhtree;				/* used for interactions with self and other systems */
 
 	struct ParticleDrawData *pdd;
 
-	float *frand;							/* array of 1024 random floats for fast lookups */
-
 	float dt_frac;							/* current time step, as a fraction of a frame */
 	float _pad;								/* spare capacity */
 } ParticleSystem;
@@ -350,11 +365,6 @@ typedef struct ParticleSystem {
 
 #define PART_SELF_EFFECT	(1<<22)
 
-/* part->rotfrom */
-#define PART_ROT_KEYS		0	/* interpolate directly from keys */
-#define PART_ROT_ZINCR		1	/* same as zdir but done incrementally from previous position */
-#define PART_ROT_IINCR		2	/* same as idir but done incrementally from previous position */
-
 /* part->from */
 #define PART_FROM_VERT		0
 #define PART_FROM_FACE		1
@@ -463,6 +473,7 @@ typedef struct ParticleSystem {
 #define PART_ROT_OB_X		6
 #define PART_ROT_OB_Y		7
 #define PART_ROT_OB_Z		8
+#define PART_ROT_NOR_TAN	9
 
 /* part->avemode */
 #define PART_AVE_VELOCITY	1
@@ -496,7 +507,7 @@ typedef struct ParticleSystem {
 #define PSYS_GLOBAL_HAIR	2
 #define PSYS_HAIR_DYNAMICS	4
 #define	PSYS_KEYED_TIMING	8
-#define PSYS_ENABLED		16	/* deprecated */
+//#define PSYS_ENABLED		16	/* deprecated */
 #define PSYS_HAIR_UPDATED	32  /* signal for updating hair particle mode */
 #define PSYS_DRAWING		64
 #define PSYS_USE_IMAT		128
@@ -551,7 +562,7 @@ typedef struct ParticleSystem {
 
 /* mapto */
 /* init */
-#define PAMAP_INIT		15
+#define PAMAP_INIT		(PAMAP_TIME | PAMAP_LIFE | PAMAP_DENS | PAMAP_SIZE)
 #define PAMAP_TIME		(1<<0)	/* emission time */
 #define PAMAP_LIFE		(1<<1)	/* life time */
 #define PAMAP_DENS		(1<<2)	/* density */
@@ -559,12 +570,12 @@ typedef struct ParticleSystem {
 /* reset */
 #define PAMAP_IVEL		(1<<5)	/* initial velocity */
 /* physics */
-#define PAMAP_PHYSICS	3136
+#define PAMAP_PHYSICS	(PAMAP_FIELD | PAMAP_GRAVITY | PAMAP_DAMP)
 #define PAMAP_FIELD		(1<<6)	/* force fields */
 #define PAMAP_GRAVITY	(1<<10)
 #define PAMAP_DAMP		(1<<11)
 /* children */
-#define PAMAP_CHILD		912
+#define PAMAP_CHILD		(PAMAP_CLUMP | PAMAP_KINK | PAMAP_ROUGH | PAMAP_LENGTH)
 #define PAMAP_CLUMP		(1<<7)
 #define PAMAP_KINK		(1<<8)
 #define PAMAP_ROUGH		(1<<9)
