@@ -45,6 +45,7 @@ extern "C" {
 #include "DNA_vec_types.h"
 #include "DNA_listBase.h"
 #include "DNA_ID.h"
+#include "DNA_freestyle_types.h"
 
 struct Object;
 struct Brush;
@@ -59,6 +60,7 @@ struct Editing;
 struct SceneStats;
 struct bGPdata;
 struct MovieClip;
+struct ColorSpace;
 
 /* ************************************************************* */
 /* Scene Data */
@@ -186,7 +188,9 @@ typedef struct SceneRenderLayer {
 	int pass_xor;
 
 	int samples;
-	int pad;
+	float pass_alpha_threshold;
+	
+	struct FreestyleConfig freestyleConfig;
 } SceneRenderLayer;
 
 /* srl->layflag */
@@ -196,7 +200,8 @@ typedef struct SceneRenderLayer {
 #define SCE_LAY_EDGE	8
 #define SCE_LAY_SKY		16
 #define SCE_LAY_STRAND	32
-	/* flags between 32 and 0x8000 are set to 1 already, for future options */
+#define SCE_LAY_FRS		64
+	/* flags between 128 and 0x8000 are set to 1 already, for future options */
 
 #define SCE_LAY_ALL_Z		0x8000
 #define SCE_LAY_XOR			0x10000
@@ -205,34 +210,40 @@ typedef struct SceneRenderLayer {
 #define SCE_LAY_NEG_ZMASK	0x80000
 
 /* srl->passflag */
-#define SCE_PASS_COMBINED			(1<<0)
-#define SCE_PASS_Z					(1<<1)
-#define SCE_PASS_RGBA				(1<<2)
-#define SCE_PASS_DIFFUSE			(1<<3)
-#define SCE_PASS_SPEC				(1<<4)
-#define SCE_PASS_SHADOW				(1<<5)
-#define SCE_PASS_AO					(1<<6)
-#define SCE_PASS_REFLECT			(1<<7)
-#define SCE_PASS_NORMAL				(1<<8)
-#define SCE_PASS_VECTOR				(1<<9)
-#define SCE_PASS_REFRACT			(1<<10)
-#define SCE_PASS_INDEXOB			(1<<11)
-#define SCE_PASS_UV					(1<<12)
-#define SCE_PASS_INDIRECT			(1<<13)
-#define SCE_PASS_MIST				(1<<14)
-#define SCE_PASS_RAYHITS			(1<<15)
-#define SCE_PASS_EMIT				(1<<16)
-#define SCE_PASS_ENVIRONMENT		(1<<17)
-#define SCE_PASS_INDEXMA			(1<<18)
-#define SCE_PASS_DIFFUSE_DIRECT		(1<<19)
-#define SCE_PASS_DIFFUSE_INDIRECT	(1<<20)
-#define SCE_PASS_DIFFUSE_COLOR		(1<<21)
-#define SCE_PASS_GLOSSY_DIRECT		(1<<22)
-#define SCE_PASS_GLOSSY_INDIRECT	(1<<23)
-#define SCE_PASS_GLOSSY_COLOR		(1<<24)
-#define SCE_PASS_TRANSM_DIRECT		(1<<25)
-#define SCE_PASS_TRANSM_INDIRECT	(1<<26)
-#define SCE_PASS_TRANSM_COLOR		(1<<27)
+typedef enum ScenePassType {
+	SCE_PASS_COMBINED                 = (1 << 0),
+	SCE_PASS_Z                        = (1 << 1),
+	SCE_PASS_RGBA                     = (1 << 2),
+	SCE_PASS_DIFFUSE                  = (1 << 3),
+	SCE_PASS_SPEC                     = (1 << 4),
+	SCE_PASS_SHADOW                   = (1 << 5),
+	SCE_PASS_AO                       = (1 << 6),
+	SCE_PASS_REFLECT                  = (1 << 7),
+	SCE_PASS_NORMAL                   = (1 << 8),
+	SCE_PASS_VECTOR                   = (1 << 9),
+	SCE_PASS_REFRACT                  = (1 << 10),
+	SCE_PASS_INDEXOB                  = (1 << 11),
+	SCE_PASS_UV                       = (1 << 12),
+	SCE_PASS_INDIRECT                 = (1 << 13),
+	SCE_PASS_MIST                     = (1 << 14),
+	SCE_PASS_RAYHITS                  = (1 << 15),
+	SCE_PASS_EMIT                     = (1 << 16),
+	SCE_PASS_ENVIRONMENT              = (1 << 17),
+	SCE_PASS_INDEXMA                  = (1 << 18),
+	SCE_PASS_DIFFUSE_DIRECT           = (1 << 19),
+	SCE_PASS_DIFFUSE_INDIRECT         = (1 << 20),
+	SCE_PASS_DIFFUSE_COLOR            = (1 << 21),
+	SCE_PASS_GLOSSY_DIRECT            = (1 << 22),
+	SCE_PASS_GLOSSY_INDIRECT          = (1 << 23),
+	SCE_PASS_GLOSSY_COLOR             = (1 << 24),
+	SCE_PASS_TRANSM_DIRECT            = (1 << 25),
+	SCE_PASS_TRANSM_INDIRECT          = (1 << 26),
+	SCE_PASS_TRANSM_COLOR             = (1 << 27),
+	SCE_PASS_SUBSURFACE_DIRECT        = (1 << 28),
+	SCE_PASS_SUBSURFACE_INDIRECT      = (1 << 29),
+	SCE_PASS_SUBSURFACE_COLOR         = (1 << 30),
+	SCE_PASS_DEBUG                    = (1 << 31),  /* This is a virtual pass. */
+} ScenePassType;
 
 /* note, srl->passflag is treestore element 'nr' in outliner, short still... */
 
@@ -271,8 +282,9 @@ typedef struct ImageFormatData {
 
 	/* Jpeg2000 */
 	char  jp2_flag;
+	char jp2_codec;
 
-	char pad[7];
+	char pad[6];
 
 	/* color management */
 	ColorManagedViewSettings view_settings;
@@ -308,6 +320,7 @@ typedef struct ImageFormatData {
 #define R_IMF_IMTYPE_H264           31
 #define R_IMF_IMTYPE_XVID           32
 #define R_IMF_IMTYPE_THEORA         33
+#define R_IMF_IMTYPE_PSD            34
 
 #define R_IMF_IMTYPE_INVALID        255
 
@@ -318,10 +331,11 @@ typedef struct ImageFormatData {
 /* return values from BKE_imtype_valid_depths, note this is depts per channel */
 #define R_IMF_CHAN_DEPTH_1  (1<<0) /* 1bits  (unused) */
 #define R_IMF_CHAN_DEPTH_8  (1<<1) /* 8bits  (default) */
-#define R_IMF_CHAN_DEPTH_12 (1<<2) /* 12bits (uncommon, jp2 supports) */
-#define R_IMF_CHAN_DEPTH_16 (1<<3) /* 16bits (tiff, halff float exr) */
-#define R_IMF_CHAN_DEPTH_24 (1<<4) /* 24bits (unused) */
-#define R_IMF_CHAN_DEPTH_32 (1<<5) /* 32bits (full float exr) */
+#define R_IMF_CHAN_DEPTH_10 (1<<2) /* 10bits (uncommon, Cineon/DPX support) */
+#define R_IMF_CHAN_DEPTH_12 (1<<3) /* 12bits (uncommon, jp2/DPX support) */
+#define R_IMF_CHAN_DEPTH_16 (1<<4) /* 16bits (tiff, halff float exr) */
+#define R_IMF_CHAN_DEPTH_24 (1<<5) /* 24bits (unused) */
+#define R_IMF_CHAN_DEPTH_32 (1<<6) /* 32bits (full float exr) */
 
 /* ImageFormatData.planes */
 #define R_IMF_PLANES_RGB   24
@@ -340,9 +354,49 @@ typedef struct ImageFormatData {
 #define R_IMF_JP2_FLAG_CINE_PRESET  (1<<1)  /* was R_JPEG2K_CINE_PRESET */
 #define R_IMF_JP2_FLAG_CINE_48      (1<<2)  /* was R_JPEG2K_CINE_48FPS */
 
+/* ImageFormatData.jp2_codec */
+#define R_IMF_JP2_CODEC_JP2  0
+#define R_IMF_JP2_CODEC_J2K  1
+
 /* ImageFormatData.cineon_flag */
 #define R_IMF_CINEON_FLAG_LOG (1<<0)  /* was R_CINEON_LOG */
 
+typedef struct BakeData {
+	struct ImageFormatData im_format;
+
+	char filepath[1024]; /* FILE_MAX */
+
+	short width, height;
+	short margin, flag;
+
+	float cage_extrusion;
+	float pad2;
+
+	char normal_swizzle[3];
+	char normal_space;
+
+	char save_mode;
+	char pad[3];
+
+	char cage[64];  /* MAX_NAME */
+} BakeData;
+
+/* (char) normal_swizzle */
+typedef enum BakeNormalSwizzle {
+	R_BAKE_POSX = 0,
+	R_BAKE_POSY = 1,
+	R_BAKE_POSZ = 2,
+	R_BAKE_NEGX = 3,
+	R_BAKE_NEGY = 4,
+	R_BAKE_NEGZ = 5,
+} BakeNormalSwizzle;
+
+/* (char) save_mode */
+typedef enum BakeSaveMode {
+	R_BAKE_SAVE_INTERNAL = 0,
+	R_BAKE_SAVE_EXTERNAL = 1,
+} BakeSaveMode;
+
 /* *************************************************************** */
 /* Render Data */
 
@@ -382,24 +436,35 @@ typedef struct RenderData {
 
 	short filtertype;	/* filter is box, tent, gauss, mitch, etc */
 
-	short size, maximsize;	/* size in %, max in Kb */
+	short size; /* size in % */
+	
+	short maximsize DNA_DEPRECATED; /* max in Kb */
+
+	short pad6;
+
 	/* from buttons: */
 	/**
 	 * The desired number of pixels in the x direction
 	 */
-	short xsch;
+	int xsch;
 	/**
 	 * The desired number of pixels in the y direction
 	 */
-	short ysch;
+	int ysch;
+
 	/**
 	 * The number of part to use in the x direction
 	 */
-	short xparts;
+	short xparts DNA_DEPRECATED;
 	/**
 	 * The number of part to use in the y direction
 	 */
-	short yparts;
+	short yparts DNA_DEPRECATED;
+
+	/**
+	 * render tile dimensions
+	 */
+	int tilex, tiley;
 
 	short planes  DNA_DEPRECATED, imtype  DNA_DEPRECATED, subimtype  DNA_DEPRECATED, quality  DNA_DEPRECATED; /*deprecated!*/
 	
@@ -407,6 +472,8 @@ typedef struct RenderData {
 	 * Render to image editor, fullscreen or to new window.
 	 */
 	short displaymode;
+	char use_lock_interface;
+	char pad7;
 
 	/**
 	 * Flags for render settings. Use bit-masking to access the settings.
@@ -475,7 +542,7 @@ typedef struct RenderData {
 	int color_mgt_flag;
 	
 	/** post-production settings. deprecated, but here for upwards compat (initialized to 1) */
-	float postgamma, posthue, postsat;	 
+	float postgamma, posthue, postsat;
 	
 	 /* Dither noise intensity */
 	float dither_intensity;
@@ -483,7 +550,9 @@ typedef struct RenderData {
 	/* Bake Render options */
 	short bake_osa, bake_filter, bake_mode, bake_flag;
 	short bake_normal_space, bake_quad_split;
-	float bake_maxdist, bake_biasdist, bake_pad;
+	float bake_maxdist, bake_biasdist;
+	short bake_samples, bake_pad;
+	float bake_user_scale, bake_pad1;
 
 	/* path to render output */
 	char pic[1024]; /* 1024 = FILE_MAX */
@@ -527,8 +596,18 @@ typedef struct RenderData {
 	float pad2;
 	struct Text *dometext  DNA_DEPRECATED;	//  XXX deprecated since 2.5
 
+	/* Freestyle line thickness options */
+	int line_thickness_mode;
+	float unit_line_thickness; /* in pixels */
+
 	/* render engine */
 	char engine[32];
+
+	/* Cycles baking */
+	struct BakeData bake;
+
+	int preview_start_resolution;
+	int pad;
 } RenderData;
 
 /* *************************************************************** */
@@ -625,9 +704,11 @@ typedef struct GameData {
 	short mode, matmode;
 	short occlusionRes;		/* resolution of occlusion Z buffer in pixel */
 	short physicsEngine;
-	short exitkey, pad;
+	short exitkey;
+	short vsync; /* Controls vsync: off, on, or adaptive (if supported) */
 	short ticrate, maxlogicstep, physubstep, maxphystep;
-	short obstacleSimulation, pad1;
+	short obstacleSimulation;
+	short raster_storage;
 	float levelHeight;
 	float deactivationtime, lineardeactthreshold, angulardeactthreshold, pad2;
 } GameData;
@@ -644,6 +725,7 @@ typedef struct GameData {
 #define STEREO_SIDEBYSIDE	6
 #define STEREO_VINTERLACE	7
 //#define STEREO_DOME		8
+#define STEREO_3DTVTOPBOTTOM 9
 
 /* physicsEngine */
 #define WOPHY_NONE		0
@@ -654,6 +736,17 @@ typedef struct GameData {
 #define OBSTSIMULATION_TOI_rays		1
 #define OBSTSIMULATION_TOI_cells	2
 
+/* Raster storage */
+#define RAS_STORE_AUTO		0
+#define RAS_STORE_IMMEDIATE	1
+#define RAS_STORE_VA		2
+#define RAS_STORE_VBO		3
+
+/* vsync */
+#define VSYNC_ON	0
+#define VSYNC_OFF	1
+#define VSYNC_ADAPTIVE	2
+
 /* GameData.flag */
 #define GAME_RESTRICT_ANIM_UPDATES			(1 << 0)
 #define GAME_ENABLE_ALL_FRAMES				(1 << 1)
@@ -672,6 +765,7 @@ typedef struct GameData {
 #define GAME_SHOW_MOUSE						(1 << 14)
 #define GAME_GLSL_NO_COLOR_MANAGEMENT		(1 << 15)
 #define GAME_SHOW_OBSTACLE_SIMULATION		(1 << 16)
+#define GAME_NO_MATERIAL_CACHING			(1 << 17)
 /* Note: GameData.flag is now an int (max 32 flags). A short could only take 16 flags */
 
 /* GameData.playerflag */
@@ -679,9 +773,15 @@ typedef struct GameData {
 #define GAME_PLAYER_DESKTOP_RESOLUTION		(1 << 1)
 
 /* GameData.matmode */
-#define GAME_MAT_TEXFACE	0
-#define GAME_MAT_MULTITEX	1
-#define GAME_MAT_GLSL		2
+enum {
+	GAME_MAT_TEXFACE    = 0, /* deprecated */
+	GAME_MAT_MULTITEX   = 1,
+	GAME_MAT_GLSL       = 2,
+};
+
+#if (DNA_DEPRECATED_GCC_POISON == 1)
+#pragma GCC poison GAME_MAT_TEXFACE
+#endif
 
 /* UV Paint */
 #define UV_SCULPT_LOCK_BORDERS				1
@@ -712,7 +812,8 @@ typedef struct TimeMarker {
 /* Paint Tool Base */
 typedef struct Paint {
 	struct Brush *brush;
-	
+	struct Palette *palette;
+
 	/* WM Paint cursor */
 	void *paint_cursor;
 	unsigned char paint_cursor_col[4];
@@ -724,7 +825,8 @@ typedef struct Paint {
 	 * smooth the stroke */
 	int num_input_samples;
 	
-	int pad;
+	/* flags used for symmetry */
+	int symmetry_flags;
 } Paint;
 
 /* ------------------------------------------- */
@@ -734,15 +836,20 @@ typedef struct Paint {
 typedef struct ImagePaintSettings {
 	Paint paint;
 
-	short flag, pad;
+	short flag, missing_data;
 	
 	/* for projection painting only */
 	short seam_bleed, normal_angle;
 	short screen_grab_size[2]; /* capture size for re-projection */
 
-	int pad1;
+	int mode;                  /* mode used for texture painting */
 
-	void *paintcursor;			/* wm handle */
+	void *paintcursor;		   /* wm handle */
+	struct Image *stencil;     /* workaround until we support true layer masks */
+	struct Image *clone;       /* clone layer for image mode for projective texture painting */
+	struct Image *canvas;      /* canvas when the explicit system is used for painting */
+	float stencil_col[3];
+	float pad1;
 } ImagePaintSettings;
 
 /* ------------------------------------------- */
@@ -792,23 +899,20 @@ typedef struct Sculpt {
 	//char tablet_size, tablet_strength; XXX not used?
 	int radial_symm[3];
 
-	// all this below is used to communicate with the cursor drawing routine
+	/* Maximum edge length for dynamic topology sculpting (in pixels) */
+	float detail_size;
 
-	/* record movement of mouse so that rake can start at an intuitive angle */
-	float last_x, last_y;
-	float last_angle;
+	/* Direction used for SCULPT_OT_symmetrize operator */
+	int symmetrize_direction;
 
-	int draw_anchored;
-	int   anchored_size;
-	float anchored_location[3];
-	float anchored_initial_mouse[2];
+	/* gravity factor for sculpting */
+	float gravity_factor;
 
-	int draw_pressure;
-	float pressure_value;
+	/* scale for constant detail size */
+	float constant_detail;
 
-	float special_rotation;
-
-	int pad;
+	struct Object *gravity_object;
+	void *pad2;
 } Sculpt;
 
 typedef struct UvSculpt {
@@ -832,7 +936,7 @@ typedef struct VPaint {
 /* VPaint.flag */
 enum {
 	// VP_COLINDEX  = (1 << 0),  /* only paint onto active material*/  /* deprecated since before 2.49 */
-	VP_AREA         = (1 << 1),
+	// VP_AREA      = (1 << 1),  /* deprecated since 2.70 */
 	VP_NORMALS      = (1 << 3),
 	VP_SPRAY        = (1 << 4),
 	// VP_MIRROR_X  = (1 << 5),  /* deprecated in 2.5x use (me->editflag & ME_EDIT_MIRROR_X) */
@@ -869,26 +973,94 @@ typedef struct UnifiedPaintSettings {
 	/* unified brush weight, [0, 1] */
 	float weight;
 
+	/* unified brush color */
+	float rgb[3];
+	/* unified brush secondary color */
+	float secondary_rgb[3];
+
 	/* user preferences for sculpt and paint */
 	int flag;
-	int pad;
+
+	/* rake rotation */
+
+	/* record movement of mouse so that rake can start at an intuitive angle */
+	float last_rake[2];
+
+	float brush_rotation;
+
+	/*********************************************************************************
+	 *  all data below are used to communicate with cursor drawing and tex sampling  *
+	 *********************************************************************************/
+	int draw_anchored;
+	int anchored_size;
+
+	char draw_inverted;
+	char pad3[7];
+
+	float overlap_factor; /* normalization factor due to accumulated value of curve along spacing.
+	                       * Calculated when brush spacing changes to dampen strength of stroke
+	                       * if space attenuation is used*/
+	float anchored_initial_mouse[2];
+
+	/* check is there an ongoing stroke right now */
+	int stroke_active;
+
+	/* drawing pressure */
+	float size_pressure_value;
+
+	/* position of mouse, used to sample the texture */
+	float tex_mouse[2];
+
+	/* position of mouse, used to sample the mask texture */
+	float mask_tex_mouse[2];
+
+	/* ColorSpace cache to avoid locking up during sampling */
+	int do_linear_conversion;
+	struct ColorSpace *colorspace;
+
+	/* radius of brush, premultiplied with pressure.
+	 * In case of anchored brushes contains the anchored radius */
+	float pixel_radius;
+	int pad4;
 } UnifiedPaintSettings;
 
 typedef enum {
-	UNIFIED_PAINT_SIZE  = (1<<0),
-	UNIFIED_PAINT_ALPHA = (1<<1),
-	UNIFIED_PAINT_WEIGHT = (1<<5),
+	UNIFIED_PAINT_SIZE  = (1 << 0),
+	UNIFIED_PAINT_ALPHA = (1 << 1),
+	UNIFIED_PAINT_WEIGHT = (1 << 5),
+	UNIFIED_PAINT_COLOR = (1 << 6),
 
-	/* only used if unified size is enabled, mirros the brush flags
+	/* only used if unified size is enabled, mirrors the brush flags
 	 * BRUSH_LOCK_SIZE and BRUSH_SIZE_PRESSURE */
-	UNIFIED_PAINT_BRUSH_LOCK_SIZE = (1<<2),
-	UNIFIED_PAINT_BRUSH_SIZE_PRESSURE   = (1<<3),
+	UNIFIED_PAINT_BRUSH_LOCK_SIZE = (1 << 2),
+	UNIFIED_PAINT_BRUSH_SIZE_PRESSURE   = (1 << 3),
 
 	/* only used if unified alpha is enabled, mirrors the brush flag
 	 * BRUSH_ALPHA_PRESSURE */
-	UNIFIED_PAINT_BRUSH_ALPHA_PRESSURE  = (1<<4)
+	UNIFIED_PAINT_BRUSH_ALPHA_PRESSURE  = (1 << 4)
 } UnifiedPaintSettingsFlags;
 
+typedef struct MeshStatVis {
+	char type;
+	char _pad1[2];
+
+	/* overhang */
+	char  overhang_axis;
+	float overhang_min, overhang_max;
+
+	/* thickness */
+	float thickness_min, thickness_max;
+	char thickness_samples;
+	char _pad2[3];
+
+	/* distort */
+	float distort_min, distort_max;
+
+	/* sharp */
+	float sharp_min, sharp_max;
+} MeshStatVis;
+
+
 /* *************************************************************** */
 /* Tool Settings */
 
@@ -902,17 +1074,6 @@ typedef struct ToolSettings {
 	 * paint */
 	float vgroup_weight;
 
-	/* Subdivide Settings */
-	short cornertype;
-	short pad1;
-	/*Triangle to Quad conversion threshold*/
-	float jointrilimit;
-	/* Editmode Tools */
-	float degr; 
-	short step;
-	short turn; 
-	
-	float extr_offs; 	/* extrude offset */
 	float doublimit;	/* remove doubles limit */
 	float normalsize;	/* size of normals */
 	short automerge;
@@ -920,30 +1081,22 @@ typedef struct ToolSettings {
 	/* Selection Mode for Mesh */
 	short selectmode;
 
-	/* Primitive Settings */
-	/* UV Sphere */
-	short segments;
-	short rings;
-	
-	/* Cylinder - Tube - Circle */
-	short vertices;
-
 	/* UV Calculation */
-	short unwrapper;
-	float uvcalc_radius;
-	float uvcalc_cubesize;
+	char unwrapper;
+	char uvcalc_flag;
+	char uv_flag;
+	char uv_selectmode;
+
 	float uvcalc_margin;
-	short uvcalc_mapdir;
-	short uvcalc_mapalign;
-	short uvcalc_flag;
-	short uv_flag, uv_selectmode;
-	short uv_subsurf_level;
-	
-	/* Grease Pencil */
-	short gpencil_flags;
-	
+
 	/* Auto-IK */
-	short autoik_chainlen;
+	short autoik_chainlen;  /* runtime only */
+
+	/* Grease Pencil */
+	char gpencil_flags;		/* flags/options for how the tool works */
+	char gpencil_src;		/* for main 3D view Grease Pencil, where data comes from */
+
+	char pad[4];
 
 	/* Image Paint (8 byttse aligned please!) */
 	struct ImagePaintSettings imapaint;
@@ -956,16 +1109,13 @@ typedef struct ToolSettings {
 
 	/* Select Group Threshold */
 	float select_thresh;
-	
-	/* Graph Editor */
-	float clean_thresh;
 
 	/* Auto-Keying Mode */
 	short autokey_mode, autokey_flag;	/* defines in DNA_userdef_types.h */
 
 	/* Multires */
 	char multires_subdiv_type;
-	char pad2[5];
+	char pad3[1];
 
 	/* Skeleton generation */
 	short skgen_resolution;
@@ -1001,15 +1151,16 @@ typedef struct ToolSettings {
 
 	/* Transform */
 	char snap_mode, snap_node_mode;
-	char pad3;
+	char snap_uv_mode;
 	short snap_flag, snap_target;
 	short proportional, prop_mode;
 	char proportional_objects; /* proportional edit, object mode */
 	char proportional_mask; /* proportional edit, object mode */
-	char pad4[2];
 
 	char auto_normalize; /*auto normalizing mode in wpaint*/
 	char multipaint; /* paint multiple bones in wpaint */
+	char weightuser;
+	char vgroupsubset; /* subset selection filter in wpaint */
 
 	/* UV painting */
 	int use_uv_sculpt;
@@ -1025,6 +1176,8 @@ typedef struct ToolSettings {
 
 	/* Unified Paint Settings */
 	struct UnifiedPaintSettings unified_paint_settings;
+
+	struct MeshStatVis statvis;
 } ToolSettings;
 
 /* *************************************************************** */
@@ -1084,9 +1237,10 @@ typedef struct Scene {
 	
 	short flag;								/* various settings */
 	
-	short use_nodes;
+	char use_nodes;
+	char pad[1];
 	
-	struct bNodeTree *nodetree;	
+	struct bNodeTree *nodetree;
 	
 	struct Editing *ed;								/* sequence editor data is allocated here */
 	
@@ -1110,12 +1264,9 @@ typedef struct Scene {
 	
 	/* none of the dependency graph  vars is mean to be saved */
 	struct  DagForest *theDag;
-	short dagisvalid, dagflags;
+	short dagflags;
 	short recalc;				/* recalc = counterpart of ob->recalc */
 
-	short pad6;
-	int pad5;
-
 	/* User-Defined KeyingSets */
 	int active_keyingset;			/* index of the active KeyingSet. first KeyingSet has index 1, 'none' active is 0, 'add new' is -1 */
 	ListBase keyingsets;			/* KeyingSets for this scene */
@@ -1143,14 +1294,19 @@ typedef struct Scene {
 	ColorManagedViewSettings view_settings;
 	ColorManagedDisplaySettings display_settings;
 	ColorManagedColorspaceSettings sequencer_colorspace_settings;
+	
+	/* RigidBody simulation world+settings */
+	struct RigidBodyWorld *rigidbody_world;
 } Scene;
 
-
 /* **************** RENDERDATA ********************* */
 
 /* flag */
 	/* use preview range */
 #define SCER_PRV_RANGE	(1<<0)
+#define SCER_LOCK_FRAME_SELECTION	(1<<1)
+	/* timeline/keyframe jumping - only selected items (on by default) */
+#define SCE_KEYS_NO_SELONLY	(1<<2)
 
 /* mode (int now) */
 #define R_OSA			0x0001
@@ -1179,15 +1335,18 @@ typedef struct Scene {
 		/* Use the same flag for autothreads */
 #define R_FIXED_THREADS		0x80000 
 
-#define R_SPEED			0x100000
-#define R_SSS			0x200000
-#define R_NO_OVERWRITE	0x400000 /* skip existing files */
-#define R_TOUCH			0x800000 /* touch files before rendering */
-#define R_SIMPLIFY		0x1000000
+#define R_SPEED				0x100000
+#define R_SSS				0x200000
+#define R_NO_OVERWRITE		0x400000  /* skip existing files */
+#define R_TOUCH				0x800000  /* touch files before rendering */
+#define R_SIMPLIFY			0x1000000
+#define R_EDGE_FRS			0x2000000 /* R_EDGE reserved for Freestyle */
+#define R_PERSISTENT_DATA	0x4000000 /* keep data around for re-render */
 
 /* seq_flag */
 #define R_SEQ_GL_PREV 1
-#define R_SEQ_GL_REND 2
+// #define R_SEQ_GL_REND 2  // UNUSED, opengl render has its own operator now.
+#define R_SEQ_SOLID_TEX 4
 
 /* displaymode */
 
@@ -1210,7 +1369,7 @@ typedef struct Scene {
 /* raytrace structure */
 #define R_RAYSTRUCTURE_AUTO				0
 #define R_RAYSTRUCTURE_OCTREE			1
-#define R_RAYSTRUCTURE_BLIBVH			2
+#define R_RAYSTRUCTURE_BLIBVH			2	/* removed */
 #define R_RAYSTRUCTURE_VBVH				3
 #define R_RAYSTRUCTURE_SIMD_SVBVH		4	/* needs SIMD */
 #define R_RAYSTRUCTURE_SIMD_QBVH		5	/* needs SIMD */
@@ -1224,7 +1383,7 @@ typedef struct Scene {
 #define R_BG_RENDER			0x0002
 		/* passepartout is camera option now, keep this for backward compatibility */
 #define R_PASSEPARTOUT		0x0004
-#define R_PREVIEWBUTS		0x0008
+#define R_BUTS_PREVIEW		0x0008
 #define R_EXTENSION			0x0010
 #define R_MATNODE_PREVIEW	0x0020
 #define R_DOCOMP			0x0040
@@ -1240,6 +1399,8 @@ typedef struct Scene {
 /* #define R_DEPRECATED		0x10000 */
 /* #define R_RECURS_PROTECTION	0x20000 */
 #define R_TEXNODE_PREVIEW	0x40000
+#define R_VIEWPORT_PREVIEW	0x80000
+#define R_EXR_CACHE_FILE	0x100000
 
 /* r->stamp */
 #define R_STAMP_TIME 	0x0001
@@ -1261,24 +1422,39 @@ typedef struct Scene {
 /* alphamode */
 #define R_ADDSKY		0
 #define R_ALPHAPREMUL	1
-#define R_ALPHAKEY		2
+/*#define R_ALPHAKEY		2*/ /* deprecated, shouldn't be used */
 
 /* color_mgt_flag */
-#define R_COLOR_MANAGEMENT              (1 << 0)  /* deprecated, should only be used in versioning code only */
-#define R_COLOR_MANAGEMENT_PREDIVIDE    (1 << 1)
+enum {
+	R_COLOR_MANAGEMENT              = (1 << 0),  /* deprecated, should only be used in versioning code only */
+	/*R_COLOR_MANAGEMENT_PREDIVIDE    = (1 << 1)*/  /* deprecated, shouldn't be used */
+};
+
+#if 0  /* TODO */
+#if (DNA_DEPRECATED_GCC_POISON == 1)
+#pragma GCC poison R_COLOR_MANAGEMENT
+#endif
+#endif
 
 /* subimtype, flag options for imtype */
-#define R_OPENEXR_HALF    1                                      /*deprecated*/
-#define R_OPENEXR_ZBUF    2                                      /*deprecated*/
-#define R_PREVIEW_JPG    4                                       /*deprecated*/
-#define R_CINEON_LOG     8                                       /*deprecated*/
-#define R_TIFF_16BIT    16                                       /*deprecated*/
+enum {
+	R_OPENEXR_HALF	= 1,  /*deprecated*/
+	R_OPENEXR_ZBUF	= 2,  /*deprecated*/
+	R_PREVIEW_JPG	= 4,  /*deprecated*/
+	R_CINEON_LOG	= 8,  /*deprecated*/
+	R_TIFF_16BIT	= 16, /*deprecated*/
 
-#define R_JPEG2K_12BIT    32 /* Jpeg2000 */                      /*deprecated*/
-#define R_JPEG2K_16BIT    64                                     /*deprecated*/
-#define R_JPEG2K_YCC    128 /* when disabled use RGB */          /*deprecated*/
-#define R_JPEG2K_CINE_PRESET    256                              /*deprecated*/
-#define R_JPEG2K_CINE_48FPS        512                           /*deprecated*/
+	R_JPEG2K_12BIT			=     32,  /* Jpeg2000 */                    /*deprecated*/
+	R_JPEG2K_16BIT			=     64,                                    /*deprecated*/
+	R_JPEG2K_YCC			=     128,  /* when disabled use RGB */      /*deprecated*/
+	R_JPEG2K_CINE_PRESET	=     256,                                   /*deprecated*/
+	R_JPEG2K_CINE_48FPS		=     512,                                   /*deprecated*/
+};
+
+#if (DNA_DEPRECATED_GCC_POISON == 1)
+#pragma GCC poison R_OPENEXR_HALF R_OPENEXR_ZBUF R_PREVIEW_JPG R_CINEON_LOG R_TIFF_16BIT
+#pragma GCC poison R_JPEG2K_12BIT R_JPEG2K_16BIT R_JPEG2K_YCC R_JPEG2K_CINE_PRESET R_JPEG2K_CINE_48FPS
+#endif
 
 /* bake_mode: same as RE_BAKE_xxx defines */
 /* bake_flag: */
@@ -1288,6 +1464,11 @@ typedef struct Scene {
 #define R_BAKE_NORMALIZE	8
 #define R_BAKE_MULTIRES		16
 #define R_BAKE_LORES_MESH	32
+#define R_BAKE_VCOL			64
+#define R_BAKE_USERSCALE	128
+#define R_BAKE_CAGE			256
+#define R_BAKE_SPLIT_MAT	512
+#define R_BAKE_AUTO_NAME	1024
 
 /* bake_normal_space */
 #define R_BAKE_SPACE_CAMERA	 0
@@ -1298,8 +1479,16 @@ typedef struct Scene {
 /* simplify_flag */
 #define R_SIMPLE_NO_TRIANGULATE		1
 
+/* line_thickness_mode */
+#define R_LINE_THICKNESS_ABSOLUTE 1
+#define R_LINE_THICKNESS_RELATIVE 2
+
 /* sequencer seq_prev_type seq_rend_type */
 
+/* scene->r.engine (scene.c) */
+extern const char *RE_engine_id_BLENDER_RENDER;
+extern const char *RE_engine_id_BLENDER_GAME;
+extern const char *RE_engine_id_CYCLES;
 
 /* **************** SCENE ********************* */
 
@@ -1318,46 +1507,49 @@ typedef struct Scene {
 #define TESTBASE(v3d, base)  (                                                \
 	((base)->flag & SELECT) &&                                                \
 	((base)->lay & v3d->lay) &&                                               \
-	(((base)->object->restrictflag & OB_RESTRICT_VIEW)==0)  )
+	(((base)->object->restrictflag & OB_RESTRICT_VIEW) == 0))
 #define TESTBASELIB(v3d, base)  (                                             \
 	((base)->flag & SELECT) &&                                                \
 	((base)->lay & v3d->lay) &&                                               \
-	((base)->object->id.lib==NULL) &&                                         \
-	(((base)->object->restrictflag & OB_RESTRICT_VIEW)==0)  )
+	((base)->object->id.lib == NULL) &&                                       \
+	(((base)->object->restrictflag & OB_RESTRICT_VIEW) == 0))
 #define TESTBASELIB_BGMODE(v3d, scene, base)  (                               \
 	((base)->flag & SELECT) &&                                                \
 	((base)->lay & (v3d ? v3d->lay : scene->lay)) &&                          \
-	((base)->object->id.lib==NULL) &&                                         \
-	(((base)->object->restrictflag & OB_RESTRICT_VIEW)==0)  )
+	((base)->object->id.lib == NULL) &&                                       \
+	(((base)->object->restrictflag & OB_RESTRICT_VIEW) == 0))
 #define BASE_EDITABLE_BGMODE(v3d, scene, base)  (                             \
 	((base)->lay & (v3d ? v3d->lay : scene->lay)) &&                          \
-	((base)->object->id.lib==NULL) &&                                         \
-	(((base)->object->restrictflag & OB_RESTRICT_VIEW)==0))
+	((base)->object->id.lib == NULL) &&                                       \
+	(((base)->object->restrictflag & OB_RESTRICT_VIEW) == 0))
 #define BASE_SELECTABLE(v3d, base)  (                                         \
 	(base->lay & v3d->lay) &&                                                 \
-	(base->object->restrictflag & (OB_RESTRICT_SELECT|OB_RESTRICT_VIEW))==0  )
+	(base->object->restrictflag & (OB_RESTRICT_SELECT | OB_RESTRICT_VIEW)) == 0)
 #define BASE_VISIBLE(v3d, base)  (                                            \
 	(base->lay & v3d->lay) &&                                                 \
-	(base->object->restrictflag & OB_RESTRICT_VIEW)==0  )
+	(base->object->restrictflag & OB_RESTRICT_VIEW) == 0)
+#define BASE_VISIBLE_BGMODE(v3d, scene, base)  (                              \
+	(base->lay & (v3d ? v3d->lay : scene->lay)) &&                            \
+	(base->object->restrictflag & OB_RESTRICT_VIEW) == 0)
 
 #define FIRSTBASE		scene->base.first
 #define LASTBASE		scene->base.last
 #define BASACT			(scene->basact)
-#define OBACT			(BASACT? BASACT->object: NULL)
+#define OBACT			(BASACT ? BASACT->object: NULL)
 
 #define V3D_CAMERA_LOCAL(v3d) ((!(v3d)->scenelock && (v3d)->camera) ? (v3d)->camera : NULL)
 #define V3D_CAMERA_SCENE(scene, v3d) ((!(v3d)->scenelock && (v3d)->camera) ? (v3d)->camera : (scene)->camera)
 
-#define	CFRA			(scene->r.cfra)
-#define SUBFRA			(scene->r.subframe)
-#define	SFRA			(scene->r.sfra)
-#define	EFRA			(scene->r.efra)
-#define PRVRANGEON		(scene->r.flag & SCER_PRV_RANGE)
-#define PSFRA			((PRVRANGEON)? (scene->r.psfra): (scene->r.sfra))
-#define PEFRA			((PRVRANGEON)? (scene->r.pefra): (scene->r.efra))
-#define FRA2TIME(a)           ((((double) scene->r.frs_sec_base) * (double)(a)) / (double)scene->r.frs_sec)
-#define TIME2FRA(a)           ((((double) scene->r.frs_sec) * (double)(a)) / (double)scene->r.frs_sec_base)
-#define FPS                     (((double) scene->r.frs_sec) / (double)scene->r.frs_sec_base)
+#define CFRA            (scene->r.cfra)
+#define SUBFRA          (scene->r.subframe)
+#define SFRA            (scene->r.sfra)
+#define EFRA            (scene->r.efra)
+#define PRVRANGEON      (scene->r.flag & SCER_PRV_RANGE)
+#define PSFRA           ((PRVRANGEON) ? (scene->r.psfra) : (scene->r.sfra))
+#define PEFRA           ((PRVRANGEON) ? (scene->r.pefra) : (scene->r.efra))
+#define FRA2TIME(a)     ((((double) scene->r.frs_sec_base) * (double)(a)) / (double)scene->r.frs_sec)
+#define TIME2FRA(a)     ((((double) scene->r.frs_sec) * (double)(a)) / (double)scene->r.frs_sec_base)
+#define FPS              (((double) scene->r.frs_sec) / (double)scene->r.frs_sec_base)
 
 /* base->flag is in DNA_object_types.h */
 
@@ -1381,12 +1573,20 @@ typedef struct Scene {
 #define SCE_SNAP_MODE_NODE_X	5
 #define SCE_SNAP_MODE_NODE_Y	6
 #define SCE_SNAP_MODE_NODE_XY	7
+#define SCE_SNAP_MODE_GRID		8
 
 /* toolsettings->selectmode */
 #define SCE_SELECT_VERTEX	1 /* for mesh */
 #define SCE_SELECT_EDGE		2
 #define SCE_SELECT_FACE		4
 
+/* toolsettings->statvis->type */
+#define SCE_STATVIS_OVERHANG	0
+#define SCE_STATVIS_THICKNESS	1
+#define SCE_STATVIS_INTERSECT	2
+#define SCE_STATVIS_DISTORT		3
+#define SCE_STATVIS_SHARP		4
+
 /* toolsettings->particle.selectmode for particles */
 #define SCE_SELECT_PATH		1
 #define SCE_SELECT_POINT	2
@@ -1408,7 +1608,33 @@ typedef struct Scene {
 /* toolsettings->proportional */
 #define PROP_EDIT_OFF			0
 #define PROP_EDIT_ON			1
-#define PROP_EDIT_CONNECTED	2
+#define PROP_EDIT_CONNECTED		2
+#define PROP_EDIT_PROJECTED		3
+
+/* toolsettings->weightuser */
+enum {
+	OB_DRAW_GROUPUSER_NONE      = 0,
+	OB_DRAW_GROUPUSER_ACTIVE    = 1,
+	OB_DRAW_GROUPUSER_ALL       = 2
+};
+
+/* toolsettings->vgroupsubset */
+/* object_vgroup.c */
+typedef enum eVGroupSelect {
+	WT_VGROUP_ALL = 0,
+	WT_VGROUP_ACTIVE = 1,
+	WT_VGROUP_BONE_SELECT = 2,
+	WT_VGROUP_BONE_DEFORM = 3,
+	WT_VGROUP_BONE_DEFORM_OFF = 4
+} eVGroupSelect;
+
+#define WT_VGROUP_MASK_ALL \
+	((1 << WT_VGROUP_ACTIVE) | \
+	 (1 << WT_VGROUP_BONE_SELECT) | \
+	 (1 << WT_VGROUP_BONE_DEFORM) | \
+	 (1 << WT_VGROUP_BONE_DEFORM_OFF) | \
+	 (1 << WT_VGROUP_ALL))
+
 
 /* sce->flag */
 #define SCE_DS_SELECTED			(1<<0)
@@ -1417,8 +1643,8 @@ typedef struct Scene {
 #define SCE_FRAME_DROP			(1<<3)
 
 
-	/* return flag BKE_scene_base_iter_next function */
-#define F_ERROR			-1
+	/* return flag BKE_scene_base_iter_next functions */
+/* #define F_ERROR			-1 */  /* UNUSED */
 #define F_START			0
 #define F_SCENE			1
 #define F_DUPLI			3
@@ -1429,38 +1655,82 @@ typedef struct Scene {
 #define AUDIO_SCRUB		          (1<<2)
 #define AUDIO_VOLUME_ANIMATED     (1<<3)
 
-#define FFMPEG_MULTIPLEX_AUDIO  1 /* deprecated, you can choose none as audiocodec now */
-#define FFMPEG_AUTOSPLIT_OUTPUT 2
-#define FFMPEG_LOSSLESS_OUTPUT  4
+enum {
+	FFMPEG_MULTIPLEX_AUDIO  = 1,  /* deprecated, you can choose none as audiocodec now */
+	FFMPEG_AUTOSPLIT_OUTPUT = 2,
+	FFMPEG_LOSSLESS_OUTPUT  = 4,
+};
+
+#if (DNA_DEPRECATED_GCC_POISON == 1)
+#pragma GCC poison FFMPEG_MULTIPLEX_AUDIO
+#endif
+
 
 /* Paint.flags */
 typedef enum {
-	PAINT_SHOW_BRUSH = (1<<0),
-	PAINT_FAST_NAVIGATE = (1<<1),
-	PAINT_SHOW_BRUSH_ON_SURFACE = (1<<2),
+	PAINT_SHOW_BRUSH = (1 << 0),
+	PAINT_FAST_NAVIGATE = (1 << 1),
+	PAINT_SHOW_BRUSH_ON_SURFACE = (1 << 2)
 } PaintFlags;
 
+/* Paint.symmetry_flags
+ * (for now just a duplicate of sculpt symmetry flags) */
+typedef enum SymmetryFlags {
+	PAINT_SYMM_X = (1 << 0),
+	PAINT_SYMM_Y = (1 << 1),
+	PAINT_SYMM_Z = (1 << 2),
+	PAINT_SYMMETRY_FEATHER = (1 << 3)
+} SymmetryFlags;
+
+#define PAINT_SYMM_AXIS_ALL (PAINT_SYMM_X | PAINT_SYMM_Y | PAINT_SYMM_Z)
+
 /* Sculpt.flags */
 /* These can eventually be moved to paint flags? */
 typedef enum SculptFlags {
-	SCULPT_SYMM_X = (1<<0),
-	SCULPT_SYMM_Y = (1<<1),
-	SCULPT_SYMM_Z = (1<<2),
-	SCULPT_LOCK_X = (1<<3),
-	SCULPT_LOCK_Y = (1<<4),
-	SCULPT_LOCK_Z = (1<<5),
-	SCULPT_SYMMETRY_FEATHER = (1<<6),
-	SCULPT_USE_OPENMP = (1<<7),
-	SCULPT_ONLY_DEFORM = (1<<8),
+	/* deprecated, part of paint struct symmetry_flags now */
+	SCULPT_SYMM_X = (1 << 0),
+	SCULPT_SYMM_Y = (1 << 1),
+	SCULPT_SYMM_Z = (1 << 2),
+
+	SCULPT_LOCK_X = (1 << 3),
+	SCULPT_LOCK_Y = (1 << 4),
+	SCULPT_LOCK_Z = (1 << 5),
+	/* deprecated, part of paint struct symmetry_flags now */
+	SCULPT_SYMMETRY_FEATHER = (1 << 6),
+
+	SCULPT_USE_OPENMP = (1 << 7),
+	SCULPT_ONLY_DEFORM = (1 << 8),
+	SCULPT_SHOW_DIFFUSE = (1 << 9),
+
+	/* If set, the mesh will be drawn with smooth-shading in
+	 * dynamic-topology mode */
+	SCULPT_DYNTOPO_SMOOTH_SHADING = (1 << 10),
+
+	/* If set, dynamic-topology brushes will subdivide short edges */
+	SCULPT_DYNTOPO_SUBDIVIDE = (1 << 12),
+	/* If set, dynamic-topology brushes will collapse short edges */
+	SCULPT_DYNTOPO_COLLAPSE = (1 << 11),
+
+	/* If set, dynamic-topology detail size will be constant in object space */
+	SCULPT_DYNTOPO_DETAIL_CONSTANT = (1 << 13)
 } SculptFlags;
 
+typedef enum ImagePaintMode {
+	IMAGEPAINT_MODE_MATERIAL, /* detect texture paint slots from the material */
+	IMAGEPAINT_MODE_IMAGE,    /* select texture paint image directly */
+} ImagePaintMode;
+
+#if (DNA_DEPRECATED_GCC_POISON == 1)
+#pragma GCC poison SCULPT_SYMM_X SCULPT_SYMM_Y SCULPT_SYMM_Z SCULPT_SYMMETRY_FEATHER
+#endif
+
+
 /* ImagePaintSettings.flag */
 #define IMAGEPAINT_DRAWING				1
 // #define IMAGEPAINT_DRAW_TOOL			2 // deprecated
 // #define IMAGEPAINT_DRAW_TOOL_DRAWING	4 // deprecated
 
 /* projection painting only */
-#define IMAGEPAINT_PROJECT_DISABLE		8	/* Non projection 3D painting */
 #define IMAGEPAINT_PROJECT_XRAY			16
 #define IMAGEPAINT_PROJECT_BACKFACE		32
 #define IMAGEPAINT_PROJECT_FLAT			64
@@ -1468,6 +1738,11 @@ typedef enum SculptFlags {
 #define IMAGEPAINT_PROJECT_LAYER_STENCIL	256
 #define IMAGEPAINT_PROJECT_LAYER_STENCIL_INV	512
 
+#define IMAGEPAINT_MISSING_UVS       (1 << 0)
+#define IMAGEPAINT_MISSING_MATERIAL  (1 << 1)
+#define IMAGEPAINT_MISSING_TEX       (1 << 2)
+#define IMAGEPAINT_MISSING_STENCIL   (1 << 3)
+
 /* toolsettings->uvcalc_flag */
 #define UVCALC_FILLHOLES			1
 #define UVCALC_NO_ASPECT_CORRECT	2	/* would call this UVCALC_ASPECT_CORRECT, except it should be default with old file */
@@ -1490,10 +1765,17 @@ typedef enum SculptFlags {
 #define EDGE_MODE_TAG_SHARP				2
 #define EDGE_MODE_TAG_CREASE			3
 #define EDGE_MODE_TAG_BEVEL				4
+#define EDGE_MODE_TAG_FREESTYLE			5
 
 /* toolsettings->gpencil_flags */
 #define GP_TOOL_FLAG_PAINTSESSIONS_ON	(1<<0)
 
+/* toolsettings->gpencil_src */
+typedef enum eGPencil_Source_3D {
+	GP_TOOL_SOURCE_SCENE    = 0,
+	GP_TOOL_SOURCE_OBJECT   = 1
+} eGPencil_Source_3d;
+
 /* toolsettings->particle flag */
 #define PE_KEEP_LENGTHS			1
 #define PE_LOCK_FIRST			2
@@ -1588,4 +1870,4 @@ typedef enum SculptFlags {
 }
 #endif
 
-#endif
+#endif  /* __DNA_SCENE_TYPES_H__ */
