@@ -28,6 +28,14 @@
  *  \ingroup DNA
  *  \since mar-2001
  *  \author nzc
+ *
+ * Structs for use by the 'Sequencer' (Video Editor)
+ *
+ * Note on terminology
+ * - #Sequence: video/effect/audio data you can select and manipulate in the sequencer.
+ * - #Sequence.machine: Strange name for the channel.
+ * - #Strip: The data referenced by the #Sequence
+ * - Meta Strip (SEQ_TYPE_META): Support for nesting Sequences.
  */
 
 #ifndef __DNA_SEQUENCE_TYPES_H__
@@ -100,6 +108,9 @@ typedef struct Strip {
 	StripCrop *crop;
 	StripTransform *transform;
 	StripColorBalance *color_balance DNA_DEPRECATED;
+
+	/* color management */
+	ColorManagedColorspaceSettings colorspace_settings;
 } Strip;
 
 /**
@@ -120,10 +131,11 @@ typedef struct Sequence {
 
 	int flag, type; /*flags bitmap (see below) and the type of sequence*/
 	int len; /* the length of the contents of this strip - before handles are applied */
-	int start, startofs, endofs;
-	int startstill, endstill;
+	int start; /* start frame of contents of strip in absolute frame coordinates. For metastrips start of first strip startdisp */
+	int startofs, endofs; /* frames after the first frame where display starts, frames before the last frame where display ends */
+	int startstill, endstill; /* frames that use the first frame before data begins, frames that use the last frame after data ends */
 	int machine, depth; /*machine - the strip channel, depth - the depth in the sequence when dealing with metastrips */
-	int startdisp, enddisp; /*starting and ending points in the sequence*/
+	int startdisp, enddisp; /* starting and ending points of the strip in the sequence*/
 	float sat;
 	float mul, handsize;
 
@@ -169,7 +181,10 @@ typedef struct Sequence {
 	float blend_opacity;
 
 	/* is sfra needed anymore? - it looks like its only used in one place */
-	int sfra, pad;  /* starting frame according to the timeline of the scene. */
+	int sfra;  /* starting frame according to the timeline of the scene. */
+
+	char alpha_mode;
+	char pad[3];
 
 	/* modifiers */
 	ListBase modifiers;
@@ -179,6 +194,8 @@ typedef struct MetaStack {
 	struct MetaStack *next, *prev;
 	ListBase *oldbasep;
 	Sequence *parseq;
+	/* the startdisp/enddisp when entering the meta */
+	int disp_range[2];
 } MetaStack;
 
 typedef struct Editing {
@@ -235,6 +252,11 @@ typedef struct SpeedControlVars {
 	int lastValidFrame;
 } SpeedControlVars;
 
+typedef struct GaussianBlurVars {
+	float size_x;
+	float size_y;
+} GaussianBlurVars;
+
 /* ***************** Sequence modifiers ****************** */
 
 typedef struct SequenceModifierData {
@@ -275,6 +297,10 @@ typedef struct BrightContrastModifierData {
 	float contrast;
 } BrightContrastModifierData;
 
+typedef struct SequencerMaskModifierData {
+	SequenceModifierData modifier;
+} SequencerMaskModifierData;
+
 /* ***************** Scopes ****************** */
 
 typedef struct SequencerScopes {
@@ -300,44 +326,53 @@ typedef struct SequencerScopes {
 
 /* SpeedControlVars->flags */
 #define SEQ_SPEED_INTEGRATE      1
-#define SEQ_SPEED_BLEND          2
+/* #define SEQ_SPEED_BLEND          2 */ /* DEPRECATED */
 #define SEQ_SPEED_COMPRESS_IPO_Y 4
 
 /* ***************** SEQUENCE ****************** */
 #define SEQ_NAME_MAXSTR         64
 
 /* seq->flag */
-#define SEQ_LEFTSEL                 (1 << 1)
-#define SEQ_RIGHTSEL                (1 << 2)
-#define SEQ_OVERLAP                 (1 << 3)
-#define SEQ_FILTERY                 (1 << 4)
-#define SEQ_MUTE                    (1 << 5)
-#define SEQ_MAKE_PREMUL             (1 << 6)
-#define SEQ_REVERSE_FRAMES          (1 << 7)
-#define SEQ_IPO_FRAME_LOCKED        (1 << 8)
-#define SEQ_EFFECT_NOT_LOADED       (1 << 9)
-#define SEQ_FLAG_DELETE             (1 << 10)
-#define SEQ_FLIPX                   (1 << 11)
-#define SEQ_FLIPY                   (1 << 12)
-#define SEQ_MAKE_FLOAT              (1 << 13)
-#define SEQ_LOCK                    (1 << 14)
-#define SEQ_USE_PROXY               (1 << 15)
-#define SEQ_USE_TRANSFORM           (1 << 16)
-#define SEQ_USE_CROP                (1 << 17)
-/* #define SEQ_USE_COLOR_BALANCE       (1 << 18) */ /* DEPRECATED */
-#define SEQ_USE_PROXY_CUSTOM_DIR    (1 << 19)
+enum {
+	SEQ_LEFTSEL                 = (1 << 1),
+	SEQ_RIGHTSEL                = (1 << 2),
+	SEQ_OVERLAP                 = (1 << 3),
+	SEQ_FILTERY                 = (1 << 4),
+	SEQ_MUTE                    = (1 << 5),
+	SEQ_MAKE_PREMUL             = (1 << 6), /* deprecated, used for compatibility code only */
+	SEQ_REVERSE_FRAMES          = (1 << 7),
+	SEQ_IPO_FRAME_LOCKED        = (1 << 8),
+	SEQ_EFFECT_NOT_LOADED       = (1 << 9),
+	SEQ_FLAG_DELETE             = (1 << 10),
+	SEQ_FLIPX                   = (1 << 11),
+	SEQ_FLIPY                   = (1 << 12),
+	SEQ_MAKE_FLOAT              = (1 << 13),
+	SEQ_LOCK                    = (1 << 14),
+	SEQ_USE_PROXY               = (1 << 15),
+	SEQ_USE_TRANSFORM           = (1 << 16),
+	SEQ_USE_CROP                = (1 << 17),
+	/* SEQ_USE_COLOR_BALANCE       = (1 << 18), */ /* DEPRECATED */
+	SEQ_USE_PROXY_CUSTOM_DIR    = (1 << 19),
 
-#define SEQ_USE_PROXY_CUSTOM_FILE   (1 << 21)
-#define SEQ_USE_EFFECT_DEFAULT_FADE (1 << 22)
-#define SEQ_USE_LINEAR_MODIFIERS    (1 << 23)
+	SEQ_USE_PROXY_CUSTOM_FILE   = (1 << 21),
+	SEQ_USE_EFFECT_DEFAULT_FADE = (1 << 22),
+	SEQ_USE_LINEAR_MODIFIERS    = (1 << 23),
 
-// flags for whether those properties are animated or not
-#define SEQ_AUDIO_VOLUME_ANIMATED   (1 << 24)
-#define SEQ_AUDIO_PITCH_ANIMATED    (1 << 25)
-#define SEQ_AUDIO_PAN_ANIMATED      (1 << 26)
-#define SEQ_AUDIO_DRAW_WAVEFORM     (1 << 27)
+	/* flags for whether those properties are animated or not */
+	SEQ_AUDIO_VOLUME_ANIMATED   = (1 << 24),
+	SEQ_AUDIO_PITCH_ANIMATED    = (1 << 25),
+	SEQ_AUDIO_PAN_ANIMATED      = (1 << 26),
+	SEQ_AUDIO_DRAW_WAVEFORM     = (1 << 27),
+	
+	/* don't include Grease Pencil in OpenGL previews of Scene strips */
+	SEQ_SCENE_NO_GPENCIL        = (1 << 28),
 
-#define SEQ_INVALID_EFFECT          (1 << 31)
+	SEQ_INVALID_EFFECT          = (1 << 31),
+};
+
+#if (DNA_DEPRECATED_GCC_POISON == 1)
+#pragma GCC poison SEQ_MAKE_PREMUL
+#endif
 
 /* convenience define for all selection flags */
 #define SEQ_ALLSEL  (SELECT + SEQ_LEFTSEL + SEQ_RIGHTSEL)
@@ -363,6 +398,12 @@ typedef struct SequencerScopes {
 #define SEQ_PROXY_TC_RECORD_RUN_NO_GAPS         8
 #define SEQ_PROXY_TC_ALL                        15
 
+/* seq->alpha_mode */
+enum {
+	SEQ_ALPHA_STRAIGHT = 0,
+	SEQ_ALPHA_PREMUL   = 1
+};
+
 /* seq->type WATCH IT: SEQ_TYPE_EFFECT BIT is used to determine if this is an effect strip!!! */
 enum {
 	SEQ_TYPE_IMAGE       = 0,
@@ -391,7 +432,8 @@ enum {
 	SEQ_TYPE_SPEED       = 29,
 	SEQ_TYPE_MULTICAM    = 30,
 	SEQ_TYPE_ADJUSTMENT  = 31,
-	SEQ_TYPE_EFFECT_MAX  = 31
+	SEQ_TYPE_GAUSSIAN_BLUR = 40,
+	SEQ_TYPE_EFFECT_MAX  = 40
 };
 
 #define SEQ_MOVIECLIP_RENDER_UNDISTORTED (1 << 0)
@@ -404,7 +446,7 @@ enum {
  */
 
 
-#define SEQ_HAS_PATH(_seq) (ELEM4((_seq)->type, SEQ_TYPE_MOVIE, SEQ_TYPE_IMAGE, SEQ_TYPE_SOUND_RAM, SEQ_TYPE_SOUND_HD))
+#define SEQ_HAS_PATH(_seq) (ELEM((_seq)->type, SEQ_TYPE_MOVIE, SEQ_TYPE_IMAGE, SEQ_TYPE_SOUND_RAM, SEQ_TYPE_SOUND_HD))
 
 /* modifiers */
 
@@ -414,6 +456,7 @@ enum {
 	seqModifierType_Curves         = 2,
 	seqModifierType_HueCorrect     = 3,
 	seqModifierType_BrightContrast = 4,
+	seqModifierType_Mask           = 5,
 
 	NUM_SEQUENCE_MODIFIER_TYPES
 };
@@ -429,4 +472,4 @@ enum {
 	SEQUENCE_MASK_INPUT_ID      = 1
 };
 
-#endif
+#endif  /* __DNA_SEQUENCE_TYPES_H__ */
