@@ -66,7 +66,7 @@ typedef struct uiFont {
 	short blf_id;		/* from blfont lib */
 	short uifont_id;	/* own id */
 	short r_to_l;		/* fonts that read from left to right */
-	short pad;
+	short hinting;
 } uiFont;
 
 /* this state defines appearance of text */
@@ -143,26 +143,44 @@ typedef struct uiWidgetStateColors {
 
 typedef struct uiPanelColors {
 	char header[4];
+	char back[4];
 	short show_header;
-	short pad;
+	short show_back;
+	int pad;
 } uiPanelColors;
 
+typedef struct uiGradientColors {
+	char gradient[4];
+	char high_gradient[4];
+	int show_grad;
+	int pad2;
+} uiGradientColors;
+
 typedef struct ThemeUI {
 	/* Interface Elements (buttons, menus, icons) */
 	uiWidgetColors wcol_regular, wcol_tool, wcol_text;
 	uiWidgetColors wcol_radio, wcol_option, wcol_toggle;
 	uiWidgetColors wcol_num, wcol_numslider;
 	uiWidgetColors wcol_menu, wcol_pulldown, wcol_menu_back, wcol_menu_item, wcol_tooltip;
-	uiWidgetColors wcol_box, wcol_scroll, wcol_progress, wcol_list_item;
+	uiWidgetColors wcol_box, wcol_scroll, wcol_progress, wcol_list_item, wcol_pie_menu;
 	
 	uiWidgetStateColors wcol_state;
 
-	uiPanelColors panel;
+	uiPanelColors panel; /* depricated, but we keep it for do_versions (2.66.1) */
 
+	char widget_emboss[4];
+
+	/* fac: 0 - 1 for blend factor, width in pixels */
+	float menu_shadow_fac;
+	short menu_shadow_width;
+	
+	short pad[3];
+	
 	char iconfile[256];	// FILE_MAXFILE length
 	float icon_alpha;
 
-	float pad;
+	/* Axis Colors */
+	char xaxis[4], yaxis[4], zaxis[4];
 } ThemeUI;
 
 /* try to put them all in one, if needed a special struct can be created as well
@@ -171,54 +189,69 @@ typedef struct ThemeUI {
 typedef struct ThemeSpace {
 	/* main window colors */
 	char back[4];
-	char title[4];
+	char title[4]; 	/* panel title */
 	char text[4];
 	char text_hi[4];
 	
 	/* header colors */
-	char header[4];
-	char header_title[4];
+	char header[4];			/* region background */
+	char header_title[4];	/* unused */
 	char header_text[4];
 	char header_text_hi[4];
 
+	/* region tabs */
+	char tab_active[4];
+	char tab_inactive[4];
+	char tab_back[4];
+	char tab_outline[4];
+
 	/* button/tool regions */
-	char button[4];
-	char button_title[4];
-	char button_text[4];	
+	char button[4];			/* region background */
+	char button_title[4];	/* panel title */
+	char button_text[4];
 	char button_text_hi[4];
 	
 	/* listview regions */
-	char list[4];
-	char list_title[4];
+	char list[4];			/* region background */
+	char list_title[4]; 	/* panel title */
 	char list_text[4];
 	char list_text_hi[4];
 	
 	/* float panel */
-	char panel[4];
-	char panel_title[4];	
-	char panel_text[4];	
-	char panel_text_hi[4];
+/*	char panel[4];			unused */
+/*	char panel_title[4];	unused */
+/*	char panel_text[4];		unused */
+/*	char panel_text_hi[4];	unused */
 	
+	/* note, cannot use name 'panel' because of DNA mapping old files */
+	uiPanelColors panelcolors;
+
+	uiGradientColors gradients;
+
 	char shade1[4];
 	char shade2[4];
 	
 	char hilite[4];
 	char grid[4]; 
 	
-	char wire[4], select[4];
-	char lamp[4], speaker[4], empty[4], camera[4], pad[8];
+	char view_overlay[4];
+
+	char wire[4], wire_edit[4], select[4];
+	char lamp[4], speaker[4], empty[4], camera[4];
 	char active[4], group[4], group_active[4], transform[4];
-	char vertex[4], vertex_select[4];
+	char vertex[4], vertex_select[4], vertex_unreferenced[4];
 	char edge[4], edge_select[4];
 	char edge_seam[4], edge_sharp[4], edge_facesel[4], edge_crease[4];
 	char face[4], face_select[4];	/* solid faces */
 	char face_dot[4];				/*  selected color */
-	char extra_edge_len[4], extra_face_angle[4], extra_face_area[4], pad3[4];
+	char extra_edge_len[4], extra_edge_angle[4], extra_face_angle[4], extra_face_area[4];
 	char normal[4];
 	char vertex_normal[4];
+	char loop_normal[4];
 	char bone_solid[4], bone_pose[4], bone_pose_active[4];
 	char strip[4], strip_select[4];
 	char cframe[4];
+	char freestyle_edge_mark[4], freestyle_face_mark[4];
 	
 	char nurb_uline[4], nurb_vline[4];
 	char act_spline[4], nurb_sel_uline[4], nurb_sel_vline[4], lastsel_point[4];
@@ -227,22 +260,34 @@ typedef struct ThemeSpace {
 	char handle_sel_free[4], handle_sel_auto[4], handle_sel_vect[4], handle_sel_align[4], handle_sel_auto_clamped[4];
 	
 	char ds_channel[4], ds_subchannel[4]; /* dopesheet */
+	char keytype_keyframe[4], keytype_extreme[4], keytype_breakdown[4], keytype_jitter[4]; /* keytypes */
+	char keytype_keyframe_select[4], keytype_extreme_select[4], keytype_breakdown_select[4], keytype_jitter_select[4]; /* keytypes */
+	char keyborder[4], keyborder_select[4];
 	
 	char console_output[4], console_input[4], console_info[4], console_error[4];
-	char console_cursor[4];
+	char console_cursor[4], console_select[4];
 	
 	char vertex_size, outline_width, facedot_size;
 	char noodle_curving;
 
-	char syntaxl[4], syntaxn[4], syntaxb[4]; /* syntax for textwindow and nodes */
-	char syntaxv[4], syntaxc[4];
+	/* syntax for textwindow and nodes */
+	char syntaxl[4], syntaxs[4]; // in nodespace used for backdrop matte 
+	char syntaxb[4], syntaxn[4]; // in nodespace used for color input
+	char syntaxv[4], syntaxc[4]; // in nodespace used for converter group
+	char syntaxd[4], syntaxr[4]; // in nodespace used for distort 
+
+	char nodeclass_output[4], nodeclass_filter[4];
+	char nodeclass_vector[4], nodeclass_texture[4];
+	char nodeclass_shader[4], nodeclass_script[4];
+	char nodeclass_pattern[4], nodeclass_layout[4];
 	
 	char movie[4], movieclip[4], mask[4], image[4], scene[4], audio[4];		/* for sequence editor */
-	char effect[4], hpad0[4], transition[4], meta[4];
+	char effect[4], transition[4], meta[4];
 	char editmesh_active[4]; 
 
 	char handle_vertex[4];
 	char handle_vertex_select[4];
+	char pad2[4];
 	
 	char handle_vertex_size;
 	
@@ -260,6 +305,9 @@ typedef struct ThemeSpace {
 	char preview_stitch_unstitchable[4];
 	char preview_stitch_active[4];
 	
+	char uv_shadow[4]; /* two uses, for uvs with modifier applied on mesh and uvs during painting */
+	char uv_others[4]; /* uvs of other objects */
+
 	char match[4];				/* outliner - filter match */
 	char selected_highlight[4];	/* outliner - selected item */
 
@@ -275,6 +323,16 @@ typedef struct ThemeSpace {
 	char nla_transition[4], nla_transition_sel[4]; /* NLA "Transition" strips */
 	char nla_meta[4], nla_meta_sel[4];             /* NLA "Meta" strips */
 	char nla_sound[4], nla_sound_sel[4];           /* NLA "Sound" strips */
+
+	/* info */
+	char info_selected[4], info_selected_text[4];
+	char info_error[4], info_error_text[4];
+	char info_warning[4], info_warning_text[4];
+	char info_info[4], info_info_text[4];
+	char info_debug[4], info_debug_text[4];
+
+	char paint_curve_pivot[4];
+	char paint_curve_handle[4];
 } ThemeSpace;
 
 
@@ -302,7 +360,8 @@ typedef struct bTheme {
 	ThemeUI tui;
 	
 	/* Individual Spacetypes */
-	ThemeSpace tbuts;	
+	/* note: ensure UI_THEMESPACE_END is updated when adding */
+	ThemeSpace tbuts;
 	ThemeSpace tv3d;
 	ThemeSpace tfile;
 	ThemeSpace tipo;
@@ -316,7 +375,7 @@ typedef struct bTheme {
 	ThemeSpace ttime;
 	ThemeSpace tnode;
 	ThemeSpace tlogic;
-	ThemeSpace tuserpref;	
+	ThemeSpace tuserpref;
 	ThemeSpace tconsole;
 	ThemeSpace tclip;
 	
@@ -327,23 +386,49 @@ typedef struct bTheme {
 	int active_theme_area, pad;
 } bTheme;
 
+#define UI_THEMESPACE_START(btheme)  (CHECK_TYPE_INLINE(btheme, bTheme *),  &((btheme)->tbuts))
+#define UI_THEMESPACE_END(btheme)    (CHECK_TYPE_INLINE(btheme, bTheme *), (&((btheme)->tclip) + 1))
+
 /* for the moment only the name. may want to store options with this later */
 typedef struct bAddon {
 	struct bAddon *next, *prev;
 	char module[64];
+	IDProperty *prop;  /* User-Defined Properties on this  Addon (for storing preferences) */
 } bAddon;
 
+typedef struct bPathCompare {
+	struct bPathCompare *next, *prev;
+	char path[768];  /* FILE_MAXDIR */
+	char flag, pad[7];
+} bPathCompare;
+
 typedef struct SolidLight {
 	int flag, pad;
 	float col[4], spec[4], vec[4];
 } SolidLight;
 
+typedef struct WalkNavigation {
+	float mouse_speed;  /* speed factor for look around */
+	float walk_speed;
+	float walk_speed_factor;
+	float view_height;
+	float jump_height;
+	float teleport_time;  /* duration to use for teleporting */
+	short flag;
+	short pad[3];
+} WalkNavigation;
+
 typedef struct UserDef {
+	/* UserDef has separate do-version handling, and can be read from other files */
+	int versionfile, subversionfile;
+	
 	int flag, dupflag;
 	int savetime;
 	char tempdir[768];	/* FILE_MAXDIR length */
 	char fontdir[768];
 	char renderdir[1024]; /* FILE_MAX length */
+	/* EXR cache path */
+	char render_cachedir[768];  /* 768 = FILE_MAXDIR */
 	char textudir[768];
 	char pythondir[768];
 	char sounddir[768];
@@ -358,9 +443,10 @@ typedef struct UserDef {
 	short versions;
 	short dbl_click_time;
 	
-	int gameflags;
-	int wheellinescroll;
-	int uiflag, language;
+	short gameflags;
+	short wheellinescroll;
+	int uiflag, uiflag2;
+	int language;
 	short userpref, viewzoom;
 	
 	int mixbufsize;
@@ -381,6 +467,7 @@ typedef struct UserDef {
 	struct ListBase keymaps  DNA_DEPRECATED; /* deprecated in favor of user_keymaps */
 	struct ListBase user_keymaps;
 	struct ListBase addons;
+	struct ListBase autoexec_paths;
 	char keyconfigstr[64];
 	
 	short undosteps;
@@ -405,21 +492,26 @@ typedef struct UserDef {
 	short glreslimit;
 	short curssize;
 	short color_picker_type;
-	short ipo_new;			/* interpolation mode for newly added F-Curves */
-	short keyhandles_new;	/* handle types for newly added keyframes */
+	char  ipo_new;			/* interpolation mode for newly added F-Curves */
+	char  keyhandles_new;	/* handle types for newly added keyframes */
+	char  gpu_select_method;
+	char  pad1;
 
 	short scrcastfps;		/* frame rate for screencast to be played back */
 	short scrcastwait;		/* milliseconds between screencast snapshots */
 	
-	short widget_unit;		/* defaults to 20 for 72 DPI setting */
+	short widget_unit;		/* private, defaults to 20 for 72 DPI setting */
 	short anisotropic_filter;
 	short use_16bit_textures, use_gpu_mipmap;
 
 	float ndof_sensitivity;	/* overall sensitivity of 3D mouse */
 	float ndof_orbit_sensitivity;
-	float pad4;
 	int ndof_flag;			/* flags for 3D mouse */
 
+	short ogl_multisamples;	/* amount of samples for OpenGL FSA, if zero no FSA */
+
+	short image_draw_method; /* Method to be used to draw the images (AUTO, GLSL, Textures or DrawPixels) */
+	
 	float glalphaclip;
 	
 	short autokey_mode;		/* autokeying mode */
@@ -430,17 +522,32 @@ typedef struct UserDef {
 	struct ColorBand coba_weight;	/* from texture.h */
 
 	float sculpt_paint_overlay_col[3];
+	float gpencil_new_layer_col[4]; /* default color for newly created Grease Pencil layers */
 
 	short tweak_threshold;
-	short pad3;
+	char navigation_mode, pad;
 
 	char author[80];	/* author name for file formats supporting it */
 
+	char font_path_ui[1024];
+
 	int compute_device_type;
 	int compute_device_id;
 	
 	float fcu_inactive_alpha;	/* opacity of inactive F-Curves in F-Curve Editor */
-	float pad;
+	float pixelsize;			/* private, set by GHOST, to multiply DPI with */
+	int virtual_pixel;			/* virtual pixelsize mode */
+
+	short pie_interaction_type;     /* if keeping a pie menu spawn button pressed after this time, it turns into
+	                             * a drag/release pie menu */
+	short pie_initial_timeout;  /* direction in the pie menu will always be calculated from the initial position
+	                             * within this time limit */
+	short pie_animation_timeout;
+	short pie_menu_confirm;
+	short pie_menu_radius;        /* pie menu radius */
+	short pie_menu_threshold;     /* pie menu distance from center before a direction is set */
+
+	struct WalkNavigation walk_navigation;
 } UserDef;
 
 extern UserDef U; /* from blenkernel blender.c */
@@ -488,7 +595,12 @@ typedef enum eUserPref_Flag {
 	USER_TXT_TABSTOSPACES_DISABLE	= (1 << 25),
 	USER_TOOLTIPS_PYTHON    = (1 << 26),
 } eUserPref_Flag;
-	
+
+/* flag */
+typedef enum ePathCompare_Flag {
+	USER_PATHCMP_GLOB		= (1 << 0),
+} ePathCompare_Flag;
+
 /* helper macro for checking frame clamping */
 #define FRAMENUMBER_MIN_CLAMP(cfra)  {                                        \
 	if ((U.flag & USER_NONEGFRAMES) && (cfra < 0))                            \
@@ -502,6 +614,18 @@ typedef enum eViewZoom_Style {
 	USER_ZOOM_DOLLY			= 2
 } eViewZoom_Style;
 
+/* navigation_mode */
+typedef enum eViewNavigation_Method {
+	VIEW_NAVIGATION_WALK = 0,
+	VIEW_NAVIGATION_FLY  = 1,
+} eViewNavigation_Method;
+
+/* flag */
+typedef enum eWalkNavigation_Flag {
+	USER_WALK_GRAVITY			= (1 << 0),
+	USER_WALK_MOUSE_REVERSE		= (1 << 1),
+} eWalkNavigation_Flag;
+
 /* uiflag */
 typedef enum eUserpref_UI_Flag {
 	/* flags 0 and 1 were old flags (for autokeying) that aren't used anymore */
@@ -532,9 +656,17 @@ typedef enum eUserpref_UI_Flag {
 	USER_SPLASH_DISABLE		= (1 << 27),
 	USER_HIDE_RECENT		= (1 << 28),
 	USER_SHOW_THUMBNAILS	= (1 << 29),
-	USER_QUIT_PROMPT		= (1 << 30)
+	USER_QUIT_PROMPT		= (1 << 30),
+	USER_HIDE_SYSTEM_BOOKMARKS = (1 << 31)
 } eUserpref_UI_Flag;
 
+/* uiflag2 */
+typedef enum eUserpref_UI_Flag2 {
+	USER_KEEP_SESSION		= (1 << 0),
+	USER_REGION_OVERLAP		= (1 << 1),
+	USER_TRACKPAD_NATURAL	= (1 << 2)
+} eUserpref_UI_Flag2;
+	
 /* Auto-Keying mode */
 typedef enum eAutokey_Mode {
 	/* AUTOKEY_ON is a bitflag */
@@ -565,12 +697,13 @@ typedef enum eAutokey_Flag {
 typedef enum eUserpref_Translation_Flags {
 	USER_TR_TOOLTIPS		= (1 << 0),
 	USER_TR_IFACE			= (1 << 1),
-/*	USER_TR_MENUS			= (1 << 2)	deprecated */
-/*	USER_TR_FILESELECT	= (1 << 3)	deprecated */
-/*	USER_TR_TEXTEDIT		= (1 << 4)	deprecated */
+/*	USER_TR_MENUS			= (1 << 2),  deprecated */
+/*	USER_TR_FILESELECT		= (1 << 3),  deprecated */
+/*	USER_TR_TEXTEDIT		= (1 << 4),  deprecated */
 	USER_DOTRANSLATE		= (1 << 5),
 	USER_USETEXTUREFONT		= (1 << 6),
-/*	CONVERT_TO_UTF8			= (1 << 7)	deprecated */
+/*	CONVERT_TO_UTF8			= (1 << 7),  deprecated */
+	USER_TR_NEWDATANAME		= (1 << 8),
 } eUserpref_Translation_Flags;
 
 /* dupflag */
@@ -596,9 +729,16 @@ typedef enum eOpenGL_RenderingOptions {
 	                                     /* backwards compatibilty in do_versions! */
 	USER_DISABLE_MIPMAP		= (1 << 2),
 	USER_DISABLE_VBO		= (1 << 3),
-	USER_DISABLE_AA			= (1 << 4),
+	/* USER_DISABLE_AA			= (1 << 4), */ /* DEPRECATED */
 } eOpenGL_RenderingOptions;
 
+/* selection method for opengl gpu_select_method */
+typedef enum eOpenGL_SelectOptions {
+	USER_SELECT_AUTO = 0,
+	USER_SELECT_USE_OCCLUSION_QUERY = 1,
+	USER_SELECT_USE_SELECT_RENDERMODE = 2
+} eOpenGL_SelectOptions;
+
 /* wm draw method */
 typedef enum eWM_DrawMethod {
 	USER_DRAW_TRIPLE		= 0,
@@ -623,10 +763,11 @@ typedef enum eGP_UserdefSettings {
 
 /* color picker types */
 typedef enum eColorPicker_Types {
-	USER_CP_CIRCLE		= 0,
+	USER_CP_CIRCLE_HSV	= 0,
 	USER_CP_SQUARE_SV	= 1,
 	USER_CP_SQUARE_HS	= 2,
 	USER_CP_SQUARE_HV	= 3,
+	USER_CP_CIRCLE_HSL	= 4,
 } eColorPicker_Types;
 
 /* timecode display styles */
@@ -672,28 +813,27 @@ typedef enum eNdof_Flag {
 	NDOF_SHOULD_ZOOM    = (1 << 4),
 	NDOF_SHOULD_ROTATE  = (1 << 5),
 
-	/* orbit navigation modes
-	 * only two options, so it's sort of a hybrid bool/enum
-	 * if ((U.ndof_flag & NDOF_ORBIT_MODE) == NDOF_OM_OBJECT)... */
+	/* orbit navigation modes */
 
-	// NDOF_ORBIT_MODE     = (1 << 6),
-	// #define NDOF_OM_TARGETCAMERA 0
-	// #define NDOF_OM_OBJECT      NDOF_ORBIT_MODE
+	/* exposed as Orbit|Explore in the UI */
+	NDOF_MODE_ORBIT      = (1 << 6),
 
 	/* actually... users probably don't care about what the mode
 	 * is called, just that it feels right */
 	/* zoom is up/down if this flag is set (otherwise forward/backward) */
-	NDOF_ZOOM_UPDOWN        = (1 << 7),
+	NDOF_PAN_YZ_SWAP_AXIS   = (1 << 7),
 	NDOF_ZOOM_INVERT        = (1 << 8),
-	NDOF_ROTATE_INVERT_AXIS = (1 << 9),
-	NDOF_TILT_INVERT_AXIS   = (1 << 10),
-	NDOF_ROLL_INVERT_AXIS   = (1 << 11),
+	NDOF_ROTX_INVERT_AXIS   = (1 << 9),
+	NDOF_ROTY_INVERT_AXIS   = (1 << 10),
+	NDOF_ROTZ_INVERT_AXIS   = (1 << 11),
 	NDOF_PANX_INVERT_AXIS   = (1 << 12),
 	NDOF_PANY_INVERT_AXIS   = (1 << 13),
 	NDOF_PANZ_INVERT_AXIS   = (1 << 14),
 	NDOF_TURNTABLE          = (1 << 15),
 } eNdof_Flag;
 
+#define NDOF_PIXELS_PER_SECOND 600.0f
+
 /* compute_device_type */
 typedef enum eCompute_Device_Type {
 	USER_COMPUTE_DEVICE_NONE	= 0,
@@ -701,6 +841,27 @@ typedef enum eCompute_Device_Type {
 	USER_COMPUTE_DEVICE_CUDA	= 2,
 } eCompute_Device_Type;
 
+	
+typedef enum eMultiSample_Type {
+	USER_MULTISAMPLE_NONE	= 0,
+	USER_MULTISAMPLE_2	= 2,
+	USER_MULTISAMPLE_4	= 4,
+	USER_MULTISAMPLE_8	= 8,
+	USER_MULTISAMPLE_16	= 16,
+} eMultiSample_Type;
+	
+typedef enum eImageDrawMethod {
+	/* IMAGE_DRAW_METHOD_AUTO = 0, */ /* Currently unused */
+	IMAGE_DRAW_METHOD_GLSL = 1,
+	IMAGE_DRAW_METHOD_2DTEXTURE = 2,
+	IMAGE_DRAW_METHOD_DRAWPIXELS = 3,
+} eImageDrawMethod;
+
+typedef enum eUserpref_VirtualPixel {
+	VIRTUAL_PIXEL_NATIVE = 0,
+	VIRTUAL_PIXEL_DOUBLE = 1,
+} eUserpref_VirtualPixel;
+
 #ifdef __cplusplus
 }
 #endif
