@@ -41,7 +41,6 @@ struct Base;
 struct BoundBox;
 struct MovieClip;
 struct MovieClipUser;
-struct RenderInfo;
 struct RenderEngine;
 struct bGPdata;
 struct SmoothView3DStore;
@@ -101,10 +100,7 @@ typedef struct RegionView3D {
 	float clip_local[6][4]; /* clip in object space, means we can test for clipping in editmode without first going into worldspace */
 	struct BoundBox *clipbb;
 
-	struct bGPdata *gpd;		/* Grease-Pencil Data (annotation layers) */
-
 	struct RegionView3D *localvd; /* allocated backup of its self while in localview */
-	struct RenderInfo *ri;
 	struct RenderEngine *render_engine;
 	struct ViewDepths *depths;
 	void *gpuoffscreen;
@@ -125,7 +121,7 @@ typedef struct RegionView3D {
 								 * also matches -viewinv[3][0:3] in ortho mode.*/
 	float camzoom;				/* viewport zoom on the camera frame, see BKE_screen_view3d_zoom_to_fac */
 	char is_persp;				/* check if persp/ortho view, since 'persp' cant be used for this since
-								 * it can have cameras assigned as well. (only set in setwinmatrixview3d) */
+								 * it can have cameras assigned as well. (only set in view3d_winmatrix_set) */
 	char persp;
 	char view;
 	char viewlock;
@@ -142,7 +138,7 @@ typedef struct RegionView3D {
 	short lpersp, lview; /* lpersp can never be set to 'RV3D_CAMOB' */
 
 	float gridview;
-	float twangle[3];
+	float tw_idot[3];  /* manipulator runtime: (1 - dot) product with view vector (used to check view alignment) */
 
 
 	/* active rotation from NDOF or elsewhere */
@@ -205,7 +201,9 @@ typedef struct View3D {
 	char gridflag;
 
 	/* transform widget info */
-	char twtype, twmode, twflag, pad2[2];
+	char twtype, twmode, twflag;
+	
+	short flag3;
 	
 	/* afterdraw, for xray & transparent */
 	struct ListBase afterdraw_transp;
@@ -218,7 +216,7 @@ typedef struct View3D {
 
 	void *properties_storage;		/* Nkey panel stores stuff here (runtime only!) */
 	struct Material *defmaterial;	/* used by matcap now */
-	
+
 	/* XXX deprecated? */
 	struct bGPdata *gpd  DNA_DEPRECATED;		/* Grease-Pencil Data (annotation layers) */
 
@@ -247,6 +245,7 @@ typedef struct View3D {
 #define RV3D_CLIPPING				4
 #define RV3D_NAVIGATING				8
 #define RV3D_GPULIGHT_UPDATE		16
+#define RV3D_IS_GAME_ENGINE			32  /* runtime flag, used to check if LoD's should be used */
 
 /* RegionView3d->viewlock */
 #define RV3D_LOCKED			(1 << 0)
@@ -270,22 +269,25 @@ typedef struct View3D {
 	((view >= RV3D_VIEW_FRONT) && (view <= RV3D_VIEW_BOTTOM))
 
 /* View3d->flag2 (short) */
-#define V3D_RENDER_OVERRIDE		4
-#define V3D_SOLID_TEX			8
-#define V3D_SHOW_GPENCIL		16
-#define V3D_LOCK_CAMERA			32
-#define V3D_RENDER_SHADOW		64		/* This is a runtime only flag that's used to tell draw_mesh_object() that we're doing a shadow pass instead of a regular draw */
-#define V3D_SHOW_RECONSTRUCTION	128
-#define V3D_SHOW_CAMERAPATH		256
-#define V3D_SHOW_BUNDLENAME		512
-#define V3D_BACKFACE_CULLING	1024
-#define V3D_RENDER_BORDER		2048
-#define V3D_SOLID_MATCAP		4096	/* user flag */
-#define V3D_SHOW_SOLID_MATCAP	8192	/* runtime flag */
-#define V3D_OCCLUDE_WIRE		16384
-#define V3D_SHADELESS_TEX		32768
+#define V3D_RENDER_OVERRIDE		(1 << 2)
+#define V3D_SOLID_TEX			(1 << 3)
+#define V3D_SHOW_GPENCIL		(1 << 4)
+#define V3D_LOCK_CAMERA			(1 << 5)
+#define V3D_RENDER_SHADOW		(1 << 6)		/* This is a runtime only flag that's used to tell draw_mesh_object() that we're doing a shadow pass instead of a regular draw */
+#define V3D_SHOW_RECONSTRUCTION	(1 << 7)
+#define V3D_SHOW_CAMERAPATH		(1 << 8)
+#define V3D_SHOW_BUNDLENAME		(1 << 9)
+#define V3D_BACKFACE_CULLING	(1 << 10)
+#define V3D_RENDER_BORDER		(1 << 11)
+#define V3D_SOLID_MATCAP		(1 << 12)	/* user flag */
+#define V3D_SHOW_SOLID_MATCAP	(1 << 13)	/* runtime flag */
+#define V3D_OCCLUDE_WIRE		(1 << 14)
+#define V3D_SHADELESS_TEX		(1 << 15)
 
 
+/* View3d->flag3 (short) */
+#define V3D_SHOW_WORLD			(1 << 0)
+
 /* View3D->around */
 #define V3D_CENTER		 0
 #define V3D_CENTROID	 3
