@@ -24,37 +24,43 @@
  *  \ingroup RNA
  */
 
-
 #include <stdlib.h>
 
-#include "RNA_define.h"
-#include "RNA_enum_types.h"
-
-#include "rna_internal.h"
-
 #include "DNA_anim_types.h"
 #include "DNA_action_types.h"
 #include "DNA_scene_types.h"
 
 #include "MEM_guardedalloc.h"
 
+#include "BLI_utildefines.h"
+
 #include "BKE_action.h"
 
+#include "RNA_access.h"
+#include "RNA_define.h"
+#include "RNA_enum_types.h"
+
+#include "rna_internal.h"
+
 #include "WM_types.h"
 
 
 #ifdef RNA_RUNTIME
 
-#include "ED_keyframing.h"
+#include "BLI_math_base.h"
+
 #include "BKE_fcurve.h"
 
+#include "ED_keyframing.h"
+
+
 static void rna_ActionGroup_channels_next(CollectionPropertyIterator *iter)
 {
-	ListBaseIterator *internal = iter->internal;
+	ListBaseIterator *internal = &iter->internal.listbase;
 	FCurve *fcu = (FCurve *)internal->link;
 	bActionGroup *grp = fcu->grp;
 	
-	/* only continue if the next F-Curve (if existant) belongs in the same group */
+	/* only continue if the next F-Curve (if existent) belongs in the same group */
 	if ((fcu->next) && (fcu->next->grp == grp))
 		internal->link = (Link *)fcu->next;
 	else
@@ -68,29 +74,30 @@ static bActionGroup *rna_Action_groups_new(bAction *act, const char name[])
 	return action_groups_add_new(act, name);
 }
 
-static void rna_Action_groups_remove(bAction *act, ReportList *reports, bActionGroup *agrp)
+static void rna_Action_groups_remove(bAction *act, ReportList *reports, PointerRNA *agrp_ptr)
 {
+	bActionGroup *agrp = agrp_ptr->data;
 	FCurve *fcu, *fcn;
 	
 	/* try to remove the F-Curve from the action */
-	if (!BLI_remlink_safe(&act->groups, agrp)) {
-		BKE_reportf(reports, RPT_ERROR, "ActionGroup '%s' not found in action '%s'", agrp->name, act->id.name + 2);
+	if (BLI_remlink_safe(&act->groups, agrp) == false) {
+		BKE_reportf(reports, RPT_ERROR, "Action group '%s' not found in action '%s'", agrp->name, act->id.name + 2);
 		return;
 	}
 
 	/* move every one one of the group's F-Curves out into the Action again */
 	for (fcu = agrp->channels.first; (fcu) && (fcu->grp == agrp); fcu = fcn) {
 		fcn = fcu->next;
-		
+
 		/* remove from group */
 		action_groups_remove_channel(act, fcu);
-		
+
 		/* tack onto the end */
 		BLI_addtail(&act->curves, fcu);
 	}
-	
-	/* XXX, invalidates PyObject */
+
 	MEM_freeN(agrp);
+	RNA_POINTER_INVALIDATE(agrp_ptr);
 }
 
 static FCurve *rna_Action_fcurve_new(bAction *act, ReportList *reports, const char *data_path,
@@ -112,17 +119,19 @@ static FCurve *rna_Action_fcurve_new(bAction *act, ReportList *reports, const ch
 	return verify_fcurve(act, group, NULL, data_path, index, 1);
 }
 
-static void rna_Action_fcurve_remove(bAction *act, ReportList *reports, FCurve *fcu)
+static void rna_Action_fcurve_remove(bAction *act, ReportList *reports, PointerRNA *fcu_ptr)
 {
+	FCurve *fcu = fcu_ptr->data;
 	if (fcu->grp) {
 		if (BLI_findindex(&act->groups, fcu->grp) == -1) {
-			BKE_reportf(reports, RPT_ERROR, "F-Curve's ActionGroup '%s' not found in action '%s'",
+			BKE_reportf(reports, RPT_ERROR, "F-Curve's action group '%s' not found in action '%s'",
 			            fcu->grp->name, act->id.name + 2);
 			return;
 		}
 		
 		action_groups_remove_channel(act, fcu);
 		free_fcurve(fcu);
+		RNA_POINTER_INVALIDATE(fcu_ptr);
 	}
 	else {
 		if (BLI_findindex(&act->curves, fcu) == -1) {
@@ -132,6 +141,7 @@ static void rna_Action_fcurve_remove(bAction *act, ReportList *reports, FCurve *
 		
 		BLI_remlink(&act->curves, fcu);
 		free_fcurve(fcu);
+		RNA_POINTER_INVALIDATE(fcu_ptr);
 	}
 }
 
@@ -145,15 +155,16 @@ static TimeMarker *rna_Action_pose_markers_new(bAction *act, const char name[])
 	return marker;
 }
 
-static void rna_Action_pose_markers_remove(bAction *act, ReportList *reports, TimeMarker *marker)
+static void rna_Action_pose_markers_remove(bAction *act, ReportList *reports, PointerRNA *marker_ptr)
 {
+	TimeMarker *marker = marker_ptr->data;
 	if (!BLI_remlink_safe(&act->markers, marker)) {
-		BKE_reportf(reports, RPT_ERROR, "TimelineMarker '%s' not found in Action '%s'", marker->name, act->id.name + 2);
+		BKE_reportf(reports, RPT_ERROR, "Timeline marker '%s' not found in action '%s'", marker->name, act->id.name + 2);
 		return;
 	}
 
-	/* XXX, invalidates PyObject */
 	MEM_freeN(marker);
+	RNA_POINTER_INVALIDATE(marker_ptr);
 }
 
 static PointerRNA rna_Action_active_pose_marker_get(PointerRNA *ptr)
@@ -180,13 +191,13 @@ static void rna_Action_active_pose_marker_index_set(PointerRNA *ptr, int value)
 	act->active_marker = value + 1;
 }
 
-static void rna_Action_active_pose_marker_index_range(PointerRNA *ptr, int *min, int *max, int *softmin, int *softmax)
+static void rna_Action_active_pose_marker_index_range(PointerRNA *ptr, int *min, int *max,
+                                                      int *UNUSED(softmin), int *UNUSED(softmax))
 {
 	bAction *act = (bAction *)ptr->data;
 
 	*min = 0;
-	*max = BLI_countlist(&act->markers) - 1;
-	*max = MAX2(0, *max);
+	*max = max_ii(0, BLI_listbase_count(&act->markers) - 1);
 }
 
 
@@ -194,7 +205,7 @@ static void rna_Action_active_pose_marker_index_range(PointerRNA *ptr, int *min,
 static void rna_Action_frame_range_get(PointerRNA *ptr, float *values)
 {   /* don't include modifiers because they too easily can have very large
 	 * ranges: MINAFRAMEF to MAXFRAMEF. */
-	calc_action_range(ptr->id.data, values, values + 1, FALSE);
+	calc_action_range(ptr->id.data, values, values + 1, false);
 }
 
 
@@ -256,22 +267,21 @@ static void rna_def_dopesheet(BlenderRNA *brna)
 
 	srna = RNA_def_struct(brna, "DopeSheet", NULL);
 	RNA_def_struct_sdna(srna, "bDopeSheet");
-	RNA_def_struct_ui_text(srna, "DopeSheet", "Settings for filtering the channels shown in Animation Editors");
+	RNA_def_struct_ui_text(srna, "Dope Sheet", "Settings for filtering the channels shown in animation editors");
 	
 	/* Source of DopeSheet data */
 	/* XXX: make this obsolete? */
 	prop = RNA_def_property(srna, "source", PROP_POINTER, PROP_NONE);
 	RNA_def_property_struct_type(prop, "ID");
 	RNA_def_property_ui_text(prop, "Source",
-	                         "ID-Block representing source data, currently ID_SCE (for Dopesheet), "
-	                         "and ID_SC (for Grease Pencil)");
+	                         "ID-Block representing source data, usually ID_SCE (i.e. Scene)");
 	
 	/* Show datablock filters */
 	prop = RNA_def_property(srna, "show_datablock_filters", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_sdna(prop, NULL, "flag", ADS_FLAG_SHOW_DBFILTERS);
 	RNA_def_property_ui_text(prop, "Show Datablock Filters",
 	                         "Show options for whether channels related to certain types of data are included");
-	RNA_def_property_ui_icon(prop, ICON_DISCLOSURE_TRI_RIGHT, -1);
+	RNA_def_property_ui_icon(prop, ICON_DISCLOSURE_TRI_RIGHT, 1);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN, NULL);
 	
 	/* General Filtering Settings */
@@ -283,22 +293,29 @@ static void rna_def_dopesheet(BlenderRNA *brna)
 	
 	prop = RNA_def_property(srna, "show_hidden", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_sdna(prop, NULL, "filterflag", ADS_FILTER_INCL_HIDDEN);
-	RNA_def_property_ui_text(prop, "Display Hidden", "Include channels from objects/bone that aren't visible");
+	RNA_def_property_ui_text(prop, "Display Hidden", "Include channels from objects/bone that are not visible");
 	RNA_def_property_ui_icon(prop, ICON_GHOST_ENABLED, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
+	/* Debug Filtering Settings */
+	prop = RNA_def_property(srna, "show_only_errors", PROP_BOOLEAN, PROP_NONE);
+	RNA_def_property_boolean_sdna(prop, NULL, "filterflag", ADS_FILTER_ONLY_ERRORS);
+	RNA_def_property_ui_text(prop, "Show Errors", "Only include F-Curves and drivers that are disabled or have errors");
+	RNA_def_property_ui_icon(prop, ICON_HELP, 0); /* XXX: this doesn't quite fit */
+	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
+	
 	/* Object Group Filtering Settings */
 	prop = RNA_def_property(srna, "show_only_group_objects", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_sdna(prop, NULL, "filterflag", ADS_FILTER_ONLYOBGROUP);
 	RNA_def_property_ui_text(prop, "Only Objects in Group",
-	                         "Only include channels from Objects in the specified Group");
+	                         "Only include channels from objects in the specified group");
 	RNA_def_property_ui_icon(prop, ICON_GROUP, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	prop = RNA_def_property(srna, "filter_group", PROP_POINTER, PROP_NONE);
 	RNA_def_property_pointer_sdna(prop, NULL, "filter_grp");
 	RNA_def_property_flag(prop, PROP_EDITABLE);
-	RNA_def_property_ui_text(prop, "Filtering Group", "Group that included Object should be a member of");
+	RNA_def_property_ui_text(prop, "Filtering Group", "Group that included object should be a member of");
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	/* FCurve Display Name Search Settings */
@@ -318,21 +335,21 @@ static void rna_def_dopesheet(BlenderRNA *brna)
 	prop = RNA_def_property(srna, "show_missing_nla", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NLA_NOACT);
 	RNA_def_property_ui_text(prop, "Include Missing NLA",
-	                         "Include Animation Data blocks with no NLA data (NLA Editor only)");
+	                         "Include animation data blocks with no NLA data (NLA editor only)");
 	RNA_def_property_ui_icon(prop, ICON_ACTION, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	/* Summary Settings (DopeSheet editors only) */
 	prop = RNA_def_property(srna, "show_summary", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_sdna(prop, NULL, "filterflag", ADS_FILTER_SUMMARY);
-	RNA_def_property_ui_text(prop, "Display Summary", "Display an additional 'summary' line (DopeSheet Editors only)");
+	RNA_def_property_ui_text(prop, "Display Summary", "Display an additional 'summary' line (Dope Sheet editors only)");
 	RNA_def_property_ui_icon(prop, ICON_BORDERMOVE, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	prop = RNA_def_property(srna, "show_expanded_summary", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "flag", ADS_FLAG_SUMMARY_COLLAPSED);
 	RNA_def_property_ui_text(prop, "Collapse Summary",
-	                         "Collapse summary when shown, so all other channels get hidden (DopeSheet Editors Only)");
+	                         "Collapse summary when shown, so all other channels get hidden (Dope Sheet editors only)");
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	
@@ -340,99 +357,117 @@ static void rna_def_dopesheet(BlenderRNA *brna)
 	prop = RNA_def_property(srna, "show_transforms", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NOOBJ);
 	RNA_def_property_ui_text(prop, "Display Transforms",
-	                         "Include visualization of Object-level Animation data (mostly Transforms)");
+	                         "Include visualization of object-level animation data (mostly transforms)");
 	RNA_def_property_ui_icon(prop, ICON_MANIPUL, 0); /* XXX? */
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	prop = RNA_def_property(srna, "show_shapekeys", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NOSHAPEKEYS);
-	RNA_def_property_ui_text(prop, "Display Shapekeys", "Include visualization of ShapeKey related Animation data");
+	RNA_def_property_ui_text(prop, "Display Shapekeys", "Include visualization of shape key related animation data");
 	RNA_def_property_ui_icon(prop, ICON_SHAPEKEY_DATA, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
+	prop = RNA_def_property(srna, "show_modifiers", PROP_BOOLEAN, PROP_NONE);
+	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NOMODIFIERS);
+	RNA_def_property_ui_text(prop, "Display Modifier Data", "Include visualization of animation data related to datablocks linked to modifiers");
+	RNA_def_property_ui_icon(prop, ICON_MODIFIER, 0);
+	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
+	
 	prop = RNA_def_property(srna, "show_meshes", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NOMESH);
-	RNA_def_property_ui_text(prop, "Display Meshes", "Include visualization of Mesh related Animation data");
+	RNA_def_property_ui_text(prop, "Display Meshes", "Include visualization of mesh related animation data");
 	RNA_def_property_ui_icon(prop, ICON_MESH_DATA, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	prop = RNA_def_property(srna, "show_lattices", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NOLAT);
-	RNA_def_property_ui_text(prop, "Display Lattices", "Include visualization of Lattice related Animation data");
+	RNA_def_property_ui_text(prop, "Display Lattices", "Include visualization of lattice related animation data");
 	RNA_def_property_ui_icon(prop, ICON_LATTICE_DATA, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	prop = RNA_def_property(srna, "show_cameras", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NOCAM);
-	RNA_def_property_ui_text(prop, "Display Camera", "Include visualization of Camera related Animation data");
+	RNA_def_property_ui_text(prop, "Display Camera", "Include visualization of camera related animation data");
 	RNA_def_property_ui_icon(prop, ICON_CAMERA_DATA, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	prop = RNA_def_property(srna, "show_materials", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NOMAT);
-	RNA_def_property_ui_text(prop, "Display Material", "Include visualization of Material related Animation data");
+	RNA_def_property_ui_text(prop, "Display Material", "Include visualization of material related animation data");
 	RNA_def_property_ui_icon(prop, ICON_MATERIAL_DATA, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	prop = RNA_def_property(srna, "show_lamps", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NOLAM);
-	RNA_def_property_ui_text(prop, "Display Lamp", "Include visualization of Lamp related Animation data");
+	RNA_def_property_ui_text(prop, "Display Lamp", "Include visualization of lamp related animation data");
 	RNA_def_property_ui_icon(prop, ICON_LAMP_DATA, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
+	prop = RNA_def_property(srna, "show_linestyles", PROP_BOOLEAN, PROP_NONE);
+	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NOLINESTYLE);
+	RNA_def_property_ui_text(prop, "Display Line Style", "Include visualization of Line Style related Animation data");
+	RNA_def_property_ui_icon(prop, ICON_LINE_DATA, 0);
+	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
+	
 	prop = RNA_def_property(srna, "show_textures", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NOTEX);
-	RNA_def_property_ui_text(prop, "Display Texture", "Include visualization of Texture related Animation data");
+	RNA_def_property_ui_text(prop, "Display Texture", "Include visualization of texture related animation data");
 	RNA_def_property_ui_icon(prop, ICON_TEXTURE_DATA, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	prop = RNA_def_property(srna, "show_curves", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NOCUR);
-	RNA_def_property_ui_text(prop, "Display Curve", "Include visualization of Curve related Animation data");
+	RNA_def_property_ui_text(prop, "Display Curve", "Include visualization of curve related animation data");
 	RNA_def_property_ui_icon(prop, ICON_CURVE_DATA, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	prop = RNA_def_property(srna, "show_worlds", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NOWOR);
-	RNA_def_property_ui_text(prop, "Display World", "Include visualization of World related Animation data");
+	RNA_def_property_ui_text(prop, "Display World", "Include visualization of world related animation data");
 	RNA_def_property_ui_icon(prop, ICON_WORLD_DATA, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	prop = RNA_def_property(srna, "show_scenes", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NOSCE);
-	RNA_def_property_ui_text(prop, "Display Scene", "Include visualization of Scene related Animation data");
+	RNA_def_property_ui_text(prop, "Display Scene", "Include visualization of scene related animation data");
 	RNA_def_property_ui_icon(prop, ICON_SCENE_DATA, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	prop = RNA_def_property(srna, "show_particles", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NOPART);
-	RNA_def_property_ui_text(prop, "Display Particle", "Include visualization of Particle related Animation data");
+	RNA_def_property_ui_text(prop, "Display Particle", "Include visualization of particle related animation data");
 	RNA_def_property_ui_icon(prop, ICON_PARTICLE_DATA, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	prop = RNA_def_property(srna, "show_metaballs", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NOMBA);
-	RNA_def_property_ui_text(prop, "Display Metaball", "Include visualization of Metaball related Animation data");
+	RNA_def_property_ui_text(prop, "Display Metaball", "Include visualization of metaball related animation data");
 	RNA_def_property_ui_icon(prop, ICON_META_DATA, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	prop = RNA_def_property(srna, "show_armatures", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NOARM);
-	RNA_def_property_ui_text(prop, "Display Armature", "Include visualization of Armature related Animation data");
+	RNA_def_property_ui_text(prop, "Display Armature", "Include visualization of armature related animation data");
 	RNA_def_property_ui_icon(prop, ICON_ARMATURE_DATA, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	prop = RNA_def_property(srna, "show_nodes", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NONTREE);
-	RNA_def_property_ui_text(prop, "Display Node", "Include visualization of Node related Animation data");
+	RNA_def_property_ui_text(prop, "Display Node", "Include visualization of node related animation data");
 	RNA_def_property_ui_icon(prop, ICON_NODETREE, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 
 	prop = RNA_def_property(srna, "show_speakers", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NOSPK);
-	RNA_def_property_ui_text(prop, "Display Speaker", "Include visualization of Speaker related Animation data");
+	RNA_def_property_ui_text(prop, "Display Speaker", "Include visualization of speaker related animation data");
 	RNA_def_property_ui_icon(prop, ICON_SPEAKER, 0);
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
+	
+	prop = RNA_def_property(srna, "show_gpencil", PROP_BOOLEAN, PROP_NONE);
+	RNA_def_property_boolean_negative_sdna(prop, NULL, "filterflag", ADS_FILTER_NOGPENCIL);
+	RNA_def_property_ui_text(prop, "Display Grease Pencil", "Include visualization of Grease Pencil related animation data and frames");
+	RNA_def_property_ui_icon(prop, ICON_GREASEPENCIL, 0);
+	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 }
 
 static void rna_def_action_group(BlenderRNA *brna)
@@ -462,22 +497,22 @@ static void rna_def_action_group(BlenderRNA *brna)
 	prop = RNA_def_property(srna, "channels", PROP_COLLECTION, PROP_NONE);
 	RNA_def_property_collection_sdna(prop, NULL, "channels", NULL);
 	RNA_def_property_struct_type(prop, "FCurve");
-	RNA_def_property_collection_funcs(prop, 0, "rna_ActionGroup_channels_next", NULL, NULL, NULL, NULL, NULL, NULL);
+	RNA_def_property_collection_funcs(prop, NULL, "rna_ActionGroup_channels_next", NULL, NULL, NULL, NULL, NULL, NULL);
 	RNA_def_property_ui_text(prop, "Channels", "F-Curves in this group");
 	
 	prop = RNA_def_property(srna, "select", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_sdna(prop, NULL, "flag", AGRP_SELECTED);
-	RNA_def_property_ui_text(prop, "Select", "Action Group is selected");
+	RNA_def_property_ui_text(prop, "Select", "Action group is selected");
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_SELECTED, NULL);
 	
 	prop = RNA_def_property(srna, "lock", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_sdna(prop, NULL, "flag", AGRP_PROTECTED);
-	RNA_def_property_ui_text(prop, "Lock", "Action Group is locked");
+	RNA_def_property_ui_text(prop, "Lock", "Action group is locked");
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	prop = RNA_def_property(srna, "show_expanded", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_sdna(prop, NULL, "flag", AGRP_EXPANDED);
-	RNA_def_property_ui_text(prop, "Expanded", "Action Group is expanded");
+	RNA_def_property_ui_text(prop, "Expanded", "Action group is expanded");
 	RNA_def_property_update(prop, NC_ANIMATION | ND_ANIMCHAN | NA_EDITED, NULL);
 	
 	/* color set */
@@ -510,7 +545,8 @@ static void rna_def_action_groups(BlenderRNA *brna, PropertyRNA *cprop)
 	RNA_def_function_ui_description(func, "Remove action group");
 	RNA_def_function_flag(func, FUNC_USE_REPORTS);
 	parm = RNA_def_pointer(func, "action_group", "ActionGroup", "", "Action group to remove");
-	RNA_def_property_flag(parm, PROP_REQUIRED | PROP_NEVER_NULL);
+	RNA_def_property_flag(parm, PROP_REQUIRED | PROP_NEVER_NULL | PROP_RNAPTR);
+	RNA_def_property_clear_flag(parm, PROP_THICK_WRAP);
 }
 
 static void rna_def_action_fcurves(BlenderRNA *brna, PropertyRNA *cprop)
@@ -528,10 +564,10 @@ static void rna_def_action_fcurves(BlenderRNA *brna, PropertyRNA *cprop)
 	func = RNA_def_function(srna, "new", "rna_Action_fcurve_new");
 	RNA_def_function_ui_description(func, "Add a keyframe to the F-Curve");
 	RNA_def_function_flag(func, FUNC_USE_REPORTS);
-	parm = RNA_def_string(func, "data_path", "", 0, "Data Path", "F-Curve data path to use");
+	parm = RNA_def_string(func, "data_path", NULL, 0, "Data Path", "F-Curve data path to use");
 	RNA_def_property_flag(parm, PROP_REQUIRED);
 	RNA_def_int(func, "index", 0, 0, INT_MAX, "Index", "Array index", 0, INT_MAX);
-	RNA_def_string(func, "action_group", "", 0, "Action Group", "Acton group to add this F-Curve into");
+	RNA_def_string(func, "action_group", NULL, 0, "Action Group", "Acton group to add this F-Curve into");
 
 	parm = RNA_def_pointer(func, "fcurve", "FCurve", "", "Newly created F-Curve");
 	RNA_def_function_return(func, parm);
@@ -541,7 +577,8 @@ static void rna_def_action_fcurves(BlenderRNA *brna, PropertyRNA *cprop)
 	RNA_def_function_ui_description(func, "Remove action group");
 	RNA_def_function_flag(func, FUNC_USE_REPORTS);
 	parm = RNA_def_pointer(func, "fcurve", "FCurve", "", "F-Curve to remove");
-	RNA_def_property_flag(parm, PROP_REQUIRED | PROP_NEVER_NULL);
+	RNA_def_property_flag(parm, PROP_REQUIRED | PROP_NEVER_NULL | PROP_RNAPTR);
+	RNA_def_property_clear_flag(parm, PROP_THICK_WRAP);
 }
 
 static void rna_def_action_pose_markers(BlenderRNA *brna, PropertyRNA *cprop)
@@ -559,7 +596,7 @@ static void rna_def_action_pose_markers(BlenderRNA *brna, PropertyRNA *cprop)
 
 	func = RNA_def_function(srna, "new", "rna_Action_pose_markers_new");
 	RNA_def_function_ui_description(func, "Add a pose marker to the action");
-	parm = RNA_def_string(func, "name", "Marker", 0, "", "New name for the marker (not unique)");
+	parm = RNA_def_string(func, "name", "Marker", 0, NULL, "New name for the marker (not unique)");
 	RNA_def_property_flag(parm, PROP_REQUIRED);
 
 	parm = RNA_def_pointer(func, "marker", "TimelineMarker", "", "Newly created marker");
@@ -569,16 +606,17 @@ static void rna_def_action_pose_markers(BlenderRNA *brna, PropertyRNA *cprop)
 	RNA_def_function_ui_description(func, "Remove a timeline marker");
 	RNA_def_function_flag(func, FUNC_USE_REPORTS);
 	parm = RNA_def_pointer(func, "marker", "TimelineMarker", "", "Timeline marker to remove");
-	RNA_def_property_flag(parm, PROP_REQUIRED | PROP_NEVER_NULL);
+	RNA_def_property_flag(parm, PROP_REQUIRED | PROP_NEVER_NULL | PROP_RNAPTR);
+	RNA_def_property_clear_flag(parm, PROP_THICK_WRAP);
 	
 	prop = RNA_def_property(srna, "active", PROP_POINTER, PROP_NONE);
 	RNA_def_property_struct_type(prop, "TimelineMarker");
 	RNA_def_property_flag(prop, PROP_EDITABLE);
 	RNA_def_property_pointer_funcs(prop, "rna_Action_active_pose_marker_get",
 	                               "rna_Action_active_pose_marker_set", NULL, NULL);
-	RNA_def_property_ui_text(prop, "Active Pose Marker", "Active pose marker for this Action");
+	RNA_def_property_ui_text(prop, "Active Pose Marker", "Active pose marker for this action");
 	
-	prop = RNA_def_property(srna, "active_index", PROP_INT, PROP_NONE);
+	prop = RNA_def_property(srna, "active_index", PROP_INT, PROP_UNSIGNED);
 	RNA_def_property_int_sdna(prop, NULL, "active_marker");
 	RNA_def_property_int_funcs(prop, "rna_Action_active_pose_marker_index_get",
 	                           "rna_Action_active_pose_marker_index_set", "rna_Action_active_pose_marker_index_range");
@@ -599,7 +637,7 @@ static void rna_def_action(BlenderRNA *brna)
 	prop = RNA_def_property(srna, "fcurves", PROP_COLLECTION, PROP_NONE);
 	RNA_def_property_collection_sdna(prop, NULL, "curves", NULL);
 	RNA_def_property_struct_type(prop, "FCurve");
-	RNA_def_property_ui_text(prop, "F-Curves", "The individual F-Curves that make up the Action");
+	RNA_def_property_ui_text(prop, "F-Curves", "The individual F-Curves that make up the action");
 	rna_def_action_fcurves(brna, prop);
 	
 	prop = RNA_def_property(srna, "groups", PROP_COLLECTION, PROP_NONE);
@@ -611,7 +649,7 @@ static void rna_def_action(BlenderRNA *brna)
 	prop = RNA_def_property(srna, "pose_markers", PROP_COLLECTION, PROP_NONE);
 	RNA_def_property_collection_sdna(prop, NULL, "markers", NULL);
 	RNA_def_property_struct_type(prop, "TimelineMarker");
-	RNA_def_property_ui_text(prop, "Pose Markers", "Markers specific to this Action, for labeling poses");
+	RNA_def_property_ui_text(prop, "Pose Markers", "Markers specific to this action, for labeling poses");
 	rna_def_action_pose_markers(brna, prop);
 	
 	/* properties */
@@ -626,8 +664,8 @@ static void rna_def_action(BlenderRNA *brna)
 	RNA_def_property_enum_sdna(prop, NULL, "idroot");
 	RNA_def_property_enum_items(prop, id_type_items);
 	RNA_def_property_ui_text(prop, "ID Root Type",
-	                         "Type of ID-block that action can be used on - "
-	                         "DO NOT CHANGE UNLESS YOU KNOW WHAT YOU'RE DOING");
+	                         "Type of ID block that action can be used on - "
+	                         "DO NOT CHANGE UNLESS YOU KNOW WHAT YOU ARE DOING");
 	
 	/* API calls */
 	RNA_api_action(srna);
