@@ -33,30 +33,38 @@
 #include <string.h>
 #include <ctype.h>
 
+#include "BLI_utildefines.h"
 #include "MEM_guardedalloc.h"
 
 #include "DNA_genfile.h"
 #include "DNA_sdna_types.h"
 
-#include "BLI_utildefines.h"
 #include "BLI_listbase.h"
 #include "BLI_ghash.h"
 
+#include "BLF_translation.h"
+
 #include "RNA_define.h"
 
 #include "rna_internal.h"
 
+
+#ifdef DEBUG
+#  define ASSERT_SOFT_HARD_LIMITS \
+	if (softmin < hardmin || softmax > hardmax) { \
+		fprintf(stderr, "Error with soft/hard limits: %s.%s\n", CONTAINER_RNA_ID(cont), identifier); \
+		BLI_assert(!"invalid soft/hard limits"); \
+	} (void)0
+#else
+#  define  ASSERT_SOFT_HARD_LIMITS (void)0
+#endif
+
 /* Global used during defining */
 
-BlenderDefRNA DefRNA = {NULL, {NULL, NULL}, {NULL, NULL}, NULL, 0, 0, 0, 1};
+BlenderDefRNA DefRNA = {NULL, {NULL, NULL}, {NULL, NULL}, NULL, 0, 0, 0, 1, 1};
 
 /* Duplicated code since we can't link in blenkernel or blenlib */
 
-#ifndef MIN2
-#define MIN2(x, y) ((x) < (y) ? (x) : (y))
-#define MAX2(x, y) ((x) > (y) ? (x) : (y))
-#endif
-
 /* pedantic check for '.', do this since its a hassle for translators */
 #ifndef NDEBUG
 #  define DESCR_CHECK(description, id1, id2)                                  \
@@ -316,7 +324,7 @@ static int rna_member_cmp(const char *name, const char *oname)
 static int rna_find_sdna_member(SDNA *sdna, const char *structname, const char *membername, DNAStructMember *smember)
 {
 	const char *dnaname;
-	short *sp;
+	const short *sp;
 	int a, b, structnr, totmember, cmp;
 
 	structnr = DNA_struct_find_nr(sdna, structname);
@@ -339,7 +347,7 @@ static int rna_find_sdna_member(SDNA *sdna, const char *structname, const char *
 			if (strstr(membername, "["))
 				smember->arraylength = 0;
 			else
-				smember->arraylength = DNA_elem_array_size(smember->name, strlen(smember->name));
+				smember->arraylength = DNA_elem_array_size(smember->name);
 
 			smember->pointerlevel = 0;
 			for (b = 0; dnaname[b] == '*'; b++)
@@ -374,11 +382,13 @@ static int rna_find_sdna_member(SDNA *sdna, const char *structname, const char *
 	return 0;
 }
 
-static int rna_validate_identifier(const char *identifier, char *error, int property)
+static int rna_validate_identifier(const char *identifier, char *error, bool property)
 {
 	int a = 0;
 	
-	/*  list from http://docs.python.org/py3k/reference/lexical_analysis.html#keywords */
+	/* list is from...
+	 * ", ".join(['"%s"' % kw for kw in  __import__("keyword").kwlist if kw not in {"False", "None", "True"}])
+	 */
 	static const char *kwlist[] = {
 		/* "False", "None", "True", */
 		"and", "as", "assert", "break",
@@ -442,6 +452,78 @@ static int rna_validate_identifier(const char *identifier, char *error, int prop
 	return 1;
 }
 
+void RNA_identifier_sanitize(char *identifier, int property)
+{
+	int a = 0;
+	
+	/*  list from http://docs.python.org/py3k/reference/lexical_analysis.html#keywords */
+	static const char *kwlist[] = {
+		/* "False", "None", "True", */
+		"and", "as", "assert", "break",
+		"class", "continue", "def", "del", "elif", "else", "except",
+		"finally", "for", "from", "global", "if", "import", "in",
+		"is", "lambda", "nonlocal", "not", "or", "pass", "raise",
+		"return", "try", "while", "with", "yield", NULL
+	};
+	
+	
+	if (!isalpha(identifier[0])) {
+		/* first character failed isalpha() check */
+		identifier[0] = '_';
+	}
+	
+	for (a = 0; identifier[a]; a++) {
+		if (DefRNA.preprocess && property) {
+			if (isalpha(identifier[a]) && isupper(identifier[a])) {
+				/* property names must contain lower case characters only */
+				identifier[a] = tolower(identifier[a]);
+			}
+		}
+		
+		if (identifier[a] == '_') {
+			continue;
+		}
+
+		if (identifier[a] == ' ') {
+			/* spaces are not okay in identifier names */
+			identifier[a] = '_';
+		}
+
+		if (isalnum(identifier[a]) == 0) {
+			/* one of the characters failed an isalnum() check and is not an underscore */
+			identifier[a] = '_';
+		}
+	}
+	
+	for (a = 0; kwlist[a]; a++) {
+		if (strcmp(identifier, kwlist[a]) == 0) {
+			/* this keyword is reserved by python.
+			 * just replace the last character by '_' to keep it readable.
+			 */
+			identifier[strlen(identifier) - 1] = '_';
+			break;
+		}
+	}
+
+	if (property) {
+		static const char *kwlist_prop[] = {
+			/* not keywords but reserved all the same because py uses */
+			"keys", "values", "items", "get",
+			NULL
+		};
+
+		for (a = 0; kwlist_prop[a]; a++) {
+			if (strcmp(identifier, kwlist_prop[a]) == 0) {
+				/* this keyword is reserved by python.
+				 * just replace the last character by '_' to keep it readable.
+				 */
+				identifier[strlen(identifier) - 1] = '_';
+				break;
+			}
+		}
+	}
+}
+
 /* Blender Data Definition */
 
 BlenderRNA *RNA_create(void)
@@ -450,7 +532,7 @@ BlenderRNA *RNA_create(void)
 
 	brna = MEM_callocN(sizeof(BlenderRNA), "BlenderRNA");
 
-	DefRNA.sdna = DNA_sdna_from_data(DNAstr,  DNAlen, 0);
+	DefRNA.sdna = DNA_sdna_from_data(DNAstr,  DNAlen, false);
 	DefRNA.structs.first = DefRNA.structs.last = NULL;
 	DefRNA.error = 0;
 	DefRNA.preprocess = 1;
@@ -486,11 +568,18 @@ void RNA_define_free(BlenderRNA *UNUSED(brna))
 	DefRNA.error = 0;
 }
 
-void RNA_define_verify_sdna(int verify)
+void RNA_define_verify_sdna(bool verify)
 {
 	DefRNA.verify = verify;
 }
 
+#ifndef RNA_RUNTIME
+void RNA_define_animate_sdna(bool animate)
+{
+	DefRNA.animate = animate;
+}
+#endif
+
 void RNA_struct_free_extension(StructRNA *srna, ExtensionRNA *ext)
 {
 #ifdef RNA_RUNTIME
@@ -550,7 +639,7 @@ void RNA_struct_free(BlenderRNA *brna, StructRNA *srna)
 	if (srna->flag & STRUCT_RUNTIME)
 		rna_freelinkN(&brna->structs, srna);
 #else
-	(void)brna, (void)srna;
+	UNUSED_VARS(brna, srna);
 #endif
 }
 
@@ -608,33 +697,20 @@ static StructDefRNA *rna_find_def_struct(StructRNA *srna)
 }
 
 /* Struct Definition */
-
-StructRNA *RNA_def_struct(BlenderRNA *brna, const char *identifier, const char *from)
+StructRNA *RNA_def_struct_ptr(BlenderRNA *brna, const char *identifier, StructRNA *srnafrom)
 {
-	StructRNA *srna, *srnafrom = NULL;
+	StructRNA *srna;
 	StructDefRNA *ds = NULL, *dsfrom = NULL;
 	PropertyRNA *prop;
-	
+
 	if (DefRNA.preprocess) {
 		char error[512];
 
-		if (rna_validate_identifier(identifier, error, 0) == 0) {
+		if (rna_validate_identifier(identifier, error, false) == 0) {
 			fprintf(stderr, "%s: struct identifier \"%s\" error - %s\n", __func__, identifier, error);
 			DefRNA.error = 1;
 		}
 	}
-	
-	if (from) {
-		/* find struct to derive from */
-		for (srnafrom = brna->structs.first; srnafrom; srnafrom = srnafrom->cont.next)
-			if (strcmp(srnafrom->identifier, from) == 0)
-				break;
-
-		if (!srnafrom) {
-			fprintf(stderr, "%s: struct %s not found to define %s.\n", __func__, from, identifier);
-			DefRNA.error = 1;
-		}
-	}
 
 	srna = MEM_callocN(sizeof(StructRNA), "StructRNA");
 	DefRNA.laststruct = srna;
@@ -655,10 +731,12 @@ StructRNA *RNA_def_struct(BlenderRNA *brna, const char *identifier, const char *
 		else
 			srna->base = srnafrom;
 	}
-	
+
 	srna->identifier = identifier;
 	srna->name = identifier; /* may be overwritten later RNA_def_struct_ui_text */
 	srna->description = "";
+	/* may be overwritten later RNA_def_struct_translation_context */
+	srna->translation_context = BLF_I18NCONTEXT_DEFAULT_BPYRNA;
 	srna->flag |= STRUCT_UNDO;
 	if (!srnafrom)
 		srna->icon = ICON_DOT;
@@ -726,6 +804,28 @@ StructRNA *RNA_def_struct(BlenderRNA *brna, const char *identifier, const char *
 	return srna;
 }
 
+StructRNA *RNA_def_struct(BlenderRNA *brna, const char *identifier, const char *from)
+{
+	StructRNA *srnafrom = NULL;
+
+	/* only use RNA_def_struct() while pre-processing, otherwise use RNA_def_struct_ptr() */
+	BLI_assert(DefRNA.preprocess);
+
+	if (from) {
+		/* find struct to derive from */
+		for (srnafrom = brna->structs.first; srnafrom; srnafrom = srnafrom->cont.next)
+			if (strcmp(srnafrom->identifier, from) == 0)
+				break;
+
+		if (!srnafrom) {
+			fprintf(stderr, "%s: struct %s not found to define %s.\n", __func__, from, identifier);
+			DefRNA.error = 1;
+		}
+	}
+
+	return RNA_def_struct_ptr(brna, identifier, srnafrom);
+}
+
 void RNA_def_struct_sdna(StructRNA *srna, const char *structname)
 {
 	StructDefRNA *ds;
@@ -737,13 +837,17 @@ void RNA_def_struct_sdna(StructRNA *srna, const char *structname)
 
 	ds = rna_find_def_struct(srna);
 
-	if (!DNA_struct_find_nr(DefRNA.sdna, structname)) {
+	/* there are far too many structs which initialize without valid DNA struct names,
+	 * this can't be checked without adding an option to disable (tested this and it means changes all over - Campbell) */
+#if 0
+	if (DNA_struct_find_nr(DefRNA.sdna, structname) == -1) {
 		if (!DefRNA.silent) {
 			fprintf(stderr, "%s: %s not found.\n", __func__, structname);
 			DefRNA.error = 1;
 		}
 		return;
 	}
+#endif
 
 	ds->dnaname = structname;
 }
@@ -764,7 +868,7 @@ void RNA_def_struct_sdna_from(StructRNA *srna, const char *structname, const cha
 		return;
 	}
 
-	if (!DNA_struct_find_nr(DefRNA.sdna, structname)) {
+	if (DNA_struct_find_nr(DefRNA.sdna, structname) == -1) {
 		if (!DefRNA.silent) {
 			fprintf(stderr, "%s: %s not found.\n", __func__, structname);
 			DefRNA.error = 1;
@@ -880,14 +984,14 @@ void RNA_def_struct_ui_icon(StructRNA *srna, int icon)
 
 void RNA_def_struct_translation_context(StructRNA *srna, const char *context)
 {
-	srna->translation_context = context;
+	srna->translation_context = context ? context : BLF_I18NCONTEXT_DEFAULT_BPYRNA;
 }
 
 /* Property Definition */
 
 PropertyRNA *RNA_def_property(StructOrFunctionRNA *cont_, const char *identifier, int type, int subtype)
 {
-	/*StructRNA *srna= DefRNA.laststruct;*/ /* invalid for python defined props */
+	/*StructRNA *srna = DefRNA.laststruct;*/ /* invalid for python defined props */
 	ContainerRNA *cont = cont_;
 	ContainerDefRNA *dcont;
 	PropertyDefRNA *dprop = NULL;
@@ -896,7 +1000,7 @@ PropertyRNA *RNA_def_property(StructOrFunctionRNA *cont_, const char *identifier
 	if (DefRNA.preprocess) {
 		char error[512];
 		
-		if (rna_validate_identifier(identifier, error, 1) == 0) {
+		if (rna_validate_identifier(identifier, error, true) == 0) {
 			fprintf(stderr, "%s: property identifier \"%s.%s\" - %s\n", __func__,
 			        CONTAINER_RNA_ID(cont), identifier, error);
 			DefRNA.error = 1;
@@ -913,6 +1017,16 @@ PropertyRNA *RNA_def_property(StructOrFunctionRNA *cont_, const char *identifier
 		dprop = MEM_callocN(sizeof(PropertyDefRNA), "PropertyDefRNA");
 		rna_addtail(&dcont->properties, dprop);
 	}
+	else {
+#ifdef DEBUG
+		char error[512];
+		if (rna_validate_identifier(identifier, error, true) == 0) {
+			fprintf(stderr, "%s: runtime property identifier \"%s.%s\" - %s\n", __func__,
+			        CONTAINER_RNA_ID(cont), identifier, error);
+			DefRNA.error = 1;
+		}
+#endif
+	}
 
 	prop = MEM_callocN(rna_property_type_sizeof(type), "PropertyRNA");
 
@@ -930,6 +1044,14 @@ PropertyRNA *RNA_def_property(StructOrFunctionRNA *cont_, const char *identifier
 		{
 			IntPropertyRNA *iprop = (IntPropertyRNA *)prop;
 
+#ifndef RNA_RUNTIME
+			if (subtype == PROP_DISTANCE) {
+				fprintf(stderr, "%s: subtype does not apply to 'PROP_INT' \"%s.%s\"\n", __func__,
+				        CONTAINER_RNA_ID(cont), identifier);
+				DefRNA.error = 1;
+			}
+#endif
+
 			iprop->hardmin = (subtype == PROP_UNSIGNED) ? 0 : INT_MIN;
 			iprop->hardmax = INT_MAX;
 
@@ -946,7 +1068,7 @@ PropertyRNA *RNA_def_property(StructOrFunctionRNA *cont_, const char *identifier
 			fprop->hardmax = FLT_MAX;
 
 			if (ELEM(subtype, PROP_COLOR, PROP_COLOR_GAMMA)) {
-				fprop->softmin = 0.0f;
+				fprop->softmin = fprop->hardmin = 0.0f;
 				fprop->softmax = 1.0f;
 			}
 			else if (subtype == PROP_FACTOR) {
@@ -968,8 +1090,10 @@ PropertyRNA *RNA_def_property(StructOrFunctionRNA *cont_, const char *identifier
 			sprop->defaultvalue = "";
 			break;
 		}
-		case PROP_ENUM:
 		case PROP_POINTER:
+			prop->flag |= PROP_THICK_WRAP;  /* needed for default behavior when PROP_RNAPTR is set */
+			break;
+		case PROP_ENUM:
 		case PROP_COLLECTION:
 			break;
 		default:
@@ -989,14 +1113,22 @@ PropertyRNA *RNA_def_property(StructOrFunctionRNA *cont_, const char *identifier
 	prop->subtype = subtype;
 	prop->name = identifier;
 	prop->description = "";
+	prop->translation_context = BLF_I18NCONTEXT_DEFAULT_BPYRNA;
 	/* a priori not raw editable */
 	prop->rawtype = -1;
 
 	if (type != PROP_COLLECTION && type != PROP_POINTER) {
 		prop->flag = PROP_EDITABLE;
 	
-		if (type != PROP_STRING)
+		if (type != PROP_STRING) {
+#ifdef RNA_RUNTIME
 			prop->flag |= PROP_ANIMATABLE;
+#else
+			if (DefRNA.animate) {
+				prop->flag |= PROP_ANIMATABLE;
+			}
+#endif
+		}
 	}
 
 	if (type == PROP_STRING) {
@@ -1119,6 +1251,11 @@ void RNA_def_property_array(PropertyRNA *prop, int length)
 	}
 }
 
+/* common args for length */
+const int rna_matrix_dimsize_3x3[] = {3, 3};
+const int rna_matrix_dimsize_4x4[] = {4, 4};
+const int rna_matrix_dimsize_4x2[] = {4, 2};
+
 void RNA_def_property_multi_array(PropertyRNA *prop, int dimension, const int length[])
 {
 	StructRNA *srna = DefRNA.laststruct;
@@ -1167,17 +1304,46 @@ void RNA_def_property_ui_text(PropertyRNA *prop, const char *name, const char *d
 	prop->description = description;
 }
 
-void RNA_def_property_ui_icon(PropertyRNA *prop, int icon, int consecutive)
+void RNA_def_property_ui_icon(PropertyRNA *prop, int icon, bool consecutive)
 {
 	prop->icon = icon;
 	if (consecutive)
 		prop->flag |= PROP_ICONS_CONSECUTIVE;
 }
 
+/**
+ * The values hare are a little confusing:
+ *
+ * \param step For floats this is (step / 100), why /100? - nobody knows.
+ * for int's, whole values are used.
+ *
+ * \param precision The number of zeros to show
+ * (as a whole number - common range is 1 - 6), see PRECISION_FLOAT_MAX
+ */
 void RNA_def_property_ui_range(PropertyRNA *prop, double min, double max, double step, int precision)
 {
 	StructRNA *srna = DefRNA.laststruct;
 
+#ifdef DEBUG
+	if (min > max) {
+		fprintf(stderr, "%s: \"%s.%s\", min > max.\n",
+		        __func__, srna->identifier, prop->identifier);
+		DefRNA.error = 1;
+	}
+
+	if (step < 0 || step > 100) {
+		fprintf(stderr, "%s: \"%s.%s\", step outside range.\n",
+		        __func__, srna->identifier, prop->identifier);
+		DefRNA.error = 1;
+	}
+
+	if (precision < -1 || precision > 10) {
+		fprintf(stderr, "%s: \"%s.%s\", step outside range.\n",
+		        __func__, srna->identifier, prop->identifier);
+		DefRNA.error = 1;
+	}
+#endif
+
 	switch (prop->type) {
 		case PROP_INT:
 		{
@@ -1194,6 +1360,21 @@ void RNA_def_property_ui_range(PropertyRNA *prop, double min, double max, double
 			fprop->softmax = (float)max;
 			fprop->step = (float)step;
 			fprop->precision = (int)precision;
+#if 0 /* handy but annoying */
+			if (DefRNA.preprocess) {
+				/* check we're not over PRECISION_FLOAT_MAX */
+				if (fprop->precision > 6) {
+					fprintf(stderr, "%s: \"%s.%s\", precision value over maximum.\n",
+					        __func__, srna->identifier, prop->identifier);
+					DefRNA.error = 1;
+				}
+				else if (fprop->precision < 1) {
+					fprintf(stderr, "%s: \"%s.%s\", precision value under minimum.\n",
+					        __func__, srna->identifier, prop->identifier);
+					DefRNA.error = 1;
+				}
+			}
+#endif
 			break;
 		}
 		default:
@@ -1208,6 +1389,14 @@ void RNA_def_property_range(PropertyRNA *prop, double min, double max)
 {
 	StructRNA *srna = DefRNA.laststruct;
 
+#ifdef DEBUG
+	if (min > max) {
+		fprintf(stderr, "%s: \"%s.%s\", min > max.\n",
+		        __func__, srna->identifier, prop->identifier);
+		DefRNA.error = 1;
+	}
+#endif
+
 	switch (prop->type) {
 		case PROP_INT:
 		{
@@ -1469,6 +1658,20 @@ void RNA_def_property_string_default(PropertyRNA *prop, const char *value)
 		case PROP_STRING:
 		{
 			StringPropertyRNA *sprop = (StringPropertyRNA *)prop;
+
+			if (value == NULL) {
+				fprintf(stderr, "%s: \"%s.%s\", NULL string passed (dont call in this case).\n", __func__, srna->identifier, prop->identifier);
+				DefRNA.error = 1;
+				break;
+			}
+
+			if (!value[0]) {
+				fprintf(stderr, "%s: \"%s.%s\", empty string passed (dont call in this case).\n", __func__, srna->identifier, prop->identifier);
+				DefRNA.error = 1;
+				// BLI_assert(0);
+				break;
+			}
+
 			sprop->defaultvalue = value;
 			break;
 		}
@@ -1902,7 +2105,7 @@ void RNA_def_property_collection_sdna(PropertyRNA *prop, const char *structname,
 
 void RNA_def_property_translation_context(PropertyRNA *prop, const char *context)
 {
-	prop->translation_context = context;
+	prop->translation_context = context ? context : BLF_I18NCONTEXT_DEFAULT_BPYRNA;
 }
 
 /* Functions */
@@ -1938,9 +2141,9 @@ void RNA_def_property_update(PropertyRNA *prop, int noteflag, const char *func)
 	prop->update = (UpdateFunc)func;
 }
 
-void RNA_def_property_update_runtime(PropertyRNA *prop, void *func)
+void RNA_def_property_update_runtime(PropertyRNA *prop, const void *func)
 {
-	prop->update = func;
+	prop->update = (void *)func;
 }
 
 void RNA_def_property_dynamic_array_funcs(PropertyRNA *prop, const char *getlength)
@@ -1990,6 +2193,38 @@ void RNA_def_property_boolean_funcs(PropertyRNA *prop, const char *get, const ch
 	}
 }
 
+void RNA_def_property_boolean_funcs_runtime(PropertyRNA *prop, BooleanPropertyGetFunc getfunc, BooleanPropertySetFunc setfunc)
+{
+	BoolPropertyRNA *bprop = (BoolPropertyRNA *)prop;
+
+	if (getfunc) bprop->get_ex = getfunc;
+	if (setfunc) bprop->set_ex = setfunc;
+
+	if (getfunc || setfunc) {
+		/* don't save in id properties */
+		prop->flag &= ~PROP_IDPROPERTY;
+
+		if (!setfunc)
+			prop->flag &= ~PROP_EDITABLE;
+	}
+}
+
+void RNA_def_property_boolean_array_funcs_runtime(PropertyRNA *prop, BooleanArrayPropertyGetFunc getfunc, BooleanArrayPropertySetFunc setfunc)
+{
+	BoolPropertyRNA *bprop = (BoolPropertyRNA *)prop;
+
+	if (getfunc) bprop->getarray_ex = getfunc;
+	if (setfunc) bprop->setarray_ex = setfunc;
+
+	if (getfunc || setfunc) {
+		/* don't save in id properties */
+		prop->flag &= ~PROP_IDPROPERTY;
+
+		if (!setfunc)
+			prop->flag &= ~PROP_EDITABLE;
+	}
+}
+
 void RNA_def_property_int_funcs(PropertyRNA *prop, const char *get, const char *set, const char *range)
 {
 	StructRNA *srna = DefRNA.laststruct;
@@ -2022,6 +2257,40 @@ void RNA_def_property_int_funcs(PropertyRNA *prop, const char *get, const char *
 	}
 }
 
+void RNA_def_property_int_funcs_runtime(PropertyRNA *prop, IntPropertyGetFunc getfunc, IntPropertySetFunc setfunc, IntPropertyRangeFunc rangefunc)
+{
+	IntPropertyRNA *iprop = (IntPropertyRNA *)prop;
+
+	if (getfunc) iprop->get_ex = getfunc;
+	if (setfunc) iprop->set_ex = setfunc;
+	if (rangefunc) iprop->range_ex = rangefunc;
+
+	if (getfunc || setfunc) {
+		/* don't save in id properties */
+		prop->flag &= ~PROP_IDPROPERTY;
+
+		if (!setfunc)
+			prop->flag &= ~PROP_EDITABLE;
+	}
+}
+
+void RNA_def_property_int_array_funcs_runtime(PropertyRNA *prop, IntArrayPropertyGetFunc getfunc, IntArrayPropertySetFunc setfunc, IntPropertyRangeFunc rangefunc)
+{
+	IntPropertyRNA *iprop = (IntPropertyRNA *)prop;
+
+	if (getfunc) iprop->getarray_ex = getfunc;
+	if (setfunc) iprop->setarray_ex = setfunc;
+	if (rangefunc) iprop->range_ex = rangefunc;
+
+	if (getfunc || setfunc) {
+		/* don't save in id properties */
+		prop->flag &= ~PROP_IDPROPERTY;
+
+		if (!setfunc)
+			prop->flag &= ~PROP_EDITABLE;
+	}
+}
+
 void RNA_def_property_float_funcs(PropertyRNA *prop, const char *get, const char *set, const char *range)
 {
 	StructRNA *srna = DefRNA.laststruct;
@@ -2054,6 +2323,40 @@ void RNA_def_property_float_funcs(PropertyRNA *prop, const char *get, const char
 	}
 }
 
+void RNA_def_property_float_funcs_runtime(PropertyRNA *prop, FloatPropertyGetFunc getfunc, FloatPropertySetFunc setfunc, FloatPropertyRangeFunc rangefunc)
+{
+	FloatPropertyRNA *fprop = (FloatPropertyRNA *)prop;
+
+	if (getfunc) fprop->get_ex = getfunc;
+	if (setfunc) fprop->set_ex = setfunc;
+	if (rangefunc) fprop->range_ex = rangefunc;
+
+	if (getfunc || setfunc) {
+		/* don't save in id properties */
+		prop->flag &= ~PROP_IDPROPERTY;
+
+		if (!setfunc)
+			prop->flag &= ~PROP_EDITABLE;
+	}
+}
+
+void RNA_def_property_float_array_funcs_runtime(PropertyRNA *prop, FloatArrayPropertyGetFunc getfunc, FloatArrayPropertySetFunc setfunc, FloatPropertyRangeFunc rangefunc)
+{
+	FloatPropertyRNA *fprop = (FloatPropertyRNA *)prop;
+
+	if (getfunc) fprop->getarray_ex = getfunc;
+	if (setfunc) fprop->setarray_ex = setfunc;
+	if (rangefunc) fprop->range_ex = rangefunc;
+
+	if (getfunc || setfunc) {
+		/* don't save in id properties */
+		prop->flag &= ~PROP_IDPROPERTY;
+
+		if (!setfunc)
+			prop->flag &= ~PROP_EDITABLE;
+	}
+}
+
 void RNA_def_property_enum_funcs(PropertyRNA *prop, const char *get, const char *set, const char *item)
 {
 	StructRNA *srna = DefRNA.laststruct;
@@ -2080,6 +2383,29 @@ void RNA_def_property_enum_funcs(PropertyRNA *prop, const char *get, const char
 	}
 }
 
+void RNA_def_property_enum_funcs_runtime(PropertyRNA *prop, EnumPropertyGetFunc getfunc, EnumPropertySetFunc setfunc, EnumPropertyItemFunc itemfunc)
+{
+	EnumPropertyRNA *eprop = (EnumPropertyRNA *)prop;
+
+	if (getfunc) eprop->get_ex = getfunc;
+	if (setfunc) eprop->set_ex = setfunc;
+	if (itemfunc) eprop->itemf = itemfunc;
+
+	if (getfunc || setfunc) {
+		/* don't save in id properties */
+		prop->flag &= ~PROP_IDPROPERTY;
+
+		if (!setfunc)
+			prop->flag &= ~PROP_EDITABLE;
+	}
+}
+
+void RNA_def_property_enum_py_data(PropertyRNA *prop, void *py_data)
+{
+	EnumPropertyRNA *eprop = (EnumPropertyRNA *)prop;
+	eprop->py_data = py_data;
+}
+
 void RNA_def_property_string_funcs(PropertyRNA *prop, const char *get, const char *length, const char *set)
 {
 	StructRNA *srna = DefRNA.laststruct;
@@ -2106,6 +2432,23 @@ void RNA_def_property_string_funcs(PropertyRNA *prop, const char *get, const cha
 	}
 }
 
+void RNA_def_property_string_funcs_runtime(PropertyRNA *prop, StringPropertyGetFunc getfunc, StringPropertyLengthFunc lengthfunc, StringPropertySetFunc setfunc)
+{
+	StringPropertyRNA *sprop = (StringPropertyRNA *)prop;
+
+	if (getfunc) sprop->get_ex = getfunc;
+	if (lengthfunc) sprop->length_ex = lengthfunc;
+	if (setfunc) sprop->set_ex = setfunc;
+
+	if (getfunc || setfunc) {
+		/* don't save in id properties */
+		prop->flag &= ~PROP_IDPROPERTY;
+
+		if (!setfunc)
+			prop->flag &= ~PROP_EDITABLE;
+	}
+}
+
 void RNA_def_property_pointer_funcs(PropertyRNA *prop, const char *get, const char *set,
                                     const char *typef, const char *poll)
 {
@@ -2248,12 +2591,15 @@ PropertyRNA *RNA_def_boolean_vector(StructOrFunctionRNA *cont_, const char *iden
 	return prop;
 }
 
-PropertyRNA *RNA_def_int(StructOrFunctionRNA *cont_, const char *identifier, int default_value, int hardmin,
-                         int hardmax, const char *ui_name, const char *ui_description, int softmin, int softmax)
+PropertyRNA *RNA_def_int(StructOrFunctionRNA *cont_, const char *identifier, int default_value,
+                         int hardmin, int hardmax, const char *ui_name, const char *ui_description,
+                         int softmin, int softmax)
 {
 	ContainerRNA *cont = cont_;
 	PropertyRNA *prop;
 	
+	ASSERT_SOFT_HARD_LIMITS;
+
 	prop = RNA_def_property(cont, identifier, PROP_INT, PROP_NONE);
 	RNA_def_property_int_default(prop, default_value);
 	if (hardmin != hardmax) RNA_def_property_range(prop, hardmin, hardmax);
@@ -2270,6 +2616,8 @@ PropertyRNA *RNA_def_int_vector(StructOrFunctionRNA *cont_, const char *identifi
 	ContainerRNA *cont = cont_;
 	PropertyRNA *prop;
 	
+	ASSERT_SOFT_HARD_LIMITS;
+
 	prop = RNA_def_property(cont, identifier, PROP_INT, PROP_XYZ); /* XXX */
 	if (len != 0) RNA_def_property_array(prop, len);
 	if (default_value) RNA_def_property_int_array_default(prop, default_value);
@@ -2287,6 +2635,8 @@ PropertyRNA *RNA_def_int_array(StructOrFunctionRNA *cont_, const char *identifie
 	ContainerRNA *cont = cont_;
 	PropertyRNA *prop;
 	
+	ASSERT_SOFT_HARD_LIMITS;
+
 	prop = RNA_def_property(cont, identifier, PROP_INT, PROP_NONE);
 	if (len != 0) RNA_def_property_array(prop, len);
 	if (default_value) RNA_def_property_int_array_default(prop, default_value);
@@ -2302,7 +2652,9 @@ PropertyRNA *RNA_def_string(StructOrFunctionRNA *cont_, const char *identifier,
 {
 	ContainerRNA *cont = cont_;
 	PropertyRNA *prop;
-	
+
+	BLI_assert(default_value == NULL || default_value[0]);
+
 	prop = RNA_def_property(cont, identifier, PROP_STRING, PROP_NONE);
 	if (maxlen != 0) RNA_def_property_string_maxlength(prop, maxlen);
 	if (default_value) RNA_def_property_string_default(prop, default_value);
@@ -2316,7 +2668,9 @@ PropertyRNA *RNA_def_string_file_path(StructOrFunctionRNA *cont_, const char *id
 {
 	ContainerRNA *cont = cont_;
 	PropertyRNA *prop;
-	
+
+	BLI_assert(default_value == NULL || default_value[0]);
+
 	prop = RNA_def_property(cont, identifier, PROP_STRING, PROP_FILEPATH);
 	if (maxlen != 0) RNA_def_property_string_maxlength(prop, maxlen);
 	if (default_value) RNA_def_property_string_default(prop, default_value);
@@ -2330,7 +2684,9 @@ PropertyRNA *RNA_def_string_dir_path(StructOrFunctionRNA *cont_, const char *ide
 {
 	ContainerRNA *cont = cont_;
 	PropertyRNA *prop;
-	
+
+	BLI_assert(default_value == NULL || default_value[0]);
+
 	prop = RNA_def_property(cont, identifier, PROP_STRING, PROP_DIRPATH);
 	if (maxlen != 0) RNA_def_property_string_maxlength(prop, maxlen);
 	if (default_value) RNA_def_property_string_default(prop, default_value);
@@ -2344,7 +2700,9 @@ PropertyRNA *RNA_def_string_file_name(StructOrFunctionRNA *cont_, const char *id
 {
 	ContainerRNA *cont = cont_;
 	PropertyRNA *prop;
-	
+
+	BLI_assert(default_value == NULL || default_value[0]);
+
 	prop = RNA_def_property(cont, identifier, PROP_STRING, PROP_FILENAME);
 	if (maxlen != 0) RNA_def_property_string_maxlength(prop, maxlen);
 	if (default_value) RNA_def_property_string_default(prop, default_value);
@@ -2353,20 +2711,6 @@ PropertyRNA *RNA_def_string_file_name(StructOrFunctionRNA *cont_, const char *id
 	return prop;
 }
 
-PropertyRNA *RNA_def_string_translate(StructOrFunctionRNA *cont_, const char *identifier, const char *default_value,
-                                      int maxlen, const char *ui_name, const char *ui_description)
-{
-	ContainerRNA *cont = cont_;
-	PropertyRNA *prop;
-
-	prop = RNA_def_property(cont, identifier, PROP_STRING, PROP_TRANSLATE);
-	if (maxlen != 0) RNA_def_property_string_maxlength(prop, maxlen);
-	if (default_value) RNA_def_property_string_default(prop, default_value);
-	RNA_def_property_ui_text(prop, ui_name, ui_description);
-
-	return prop;
-}
-
 PropertyRNA *RNA_def_enum(StructOrFunctionRNA *cont_, const char *identifier, const EnumPropertyItem *items,
                           int default_value, const char *ui_name, const char *ui_description)
 {
@@ -2413,12 +2757,6 @@ void RNA_def_enum_funcs(PropertyRNA *prop, EnumPropertyItemFunc itemfunc)
 	eprop->itemf = itemfunc;
 }
 
-void RNA_def_enum_py_data(PropertyRNA *prop, void *py_data)
-{
-	EnumPropertyRNA *eprop = (EnumPropertyRNA *)prop;
-	eprop->py_data = py_data;
-}
-
 PropertyRNA *RNA_def_float(StructOrFunctionRNA *cont_, const char *identifier, float default_value,
                            float hardmin, float hardmax, const char *ui_name, const char *ui_description,
                            float softmin, float softmax)
@@ -2426,6 +2764,8 @@ PropertyRNA *RNA_def_float(StructOrFunctionRNA *cont_, const char *identifier, f
 	ContainerRNA *cont = cont_;
 	PropertyRNA *prop;
 	
+	ASSERT_SOFT_HARD_LIMITS;
+
 	prop = RNA_def_property(cont, identifier, PROP_FLOAT, PROP_NONE);
 	RNA_def_property_float_default(prop, default_value);
 	if (hardmin != hardmax) RNA_def_property_range(prop, hardmin, hardmax);
@@ -2442,6 +2782,8 @@ PropertyRNA *RNA_def_float_vector(StructOrFunctionRNA *cont_, const char *identi
 	ContainerRNA *cont = cont_;
 	PropertyRNA *prop;
 	
+	ASSERT_SOFT_HARD_LIMITS;
+
 	prop = RNA_def_property(cont, identifier, PROP_FLOAT, PROP_XYZ);
 	if (len != 0) RNA_def_property_array(prop, len);
 	if (default_value) RNA_def_property_float_array_default(prop, default_value);
@@ -2472,6 +2814,8 @@ PropertyRNA *RNA_def_float_color(StructOrFunctionRNA *cont_, const char *identif
 	ContainerRNA *cont = cont_;
 	PropertyRNA *prop;
 	
+	ASSERT_SOFT_HARD_LIMITS;
+
 	prop = RNA_def_property(cont, identifier, PROP_FLOAT, PROP_COLOR);
 	if (len != 0) RNA_def_property_array(prop, len);
 	if (default_value) RNA_def_property_float_array_default(prop, default_value);
@@ -2489,10 +2833,9 @@ PropertyRNA *RNA_def_float_matrix(StructOrFunctionRNA *cont_, const char *identi
 {
 	ContainerRNA *cont = cont_;
 	PropertyRNA *prop;
-	int length[2];
+	const int length[2] = {rows, columns};
 
-	length[0] = rows;
-	length[1] = columns;
+	ASSERT_SOFT_HARD_LIMITS;
 
 	prop = RNA_def_property(cont, identifier, PROP_FLOAT, PROP_MATRIX);
 	RNA_def_property_multi_array(prop, 2, length);
@@ -2510,7 +2853,9 @@ PropertyRNA *RNA_def_float_rotation(StructOrFunctionRNA *cont_, const char *iden
 {
 	ContainerRNA *cont = cont_;
 	PropertyRNA *prop;
-	
+
+	ASSERT_SOFT_HARD_LIMITS;
+
 	prop = RNA_def_property(cont, identifier, PROP_FLOAT, (len != 0) ? PROP_EULER : PROP_ANGLE);
 	if (len != 0) {
 		RNA_def_property_array(prop, len);
@@ -2534,6 +2879,8 @@ PropertyRNA *RNA_def_float_array(StructOrFunctionRNA *cont_, const char *identif
 	ContainerRNA *cont = cont_;
 	PropertyRNA *prop;
 	
+	ASSERT_SOFT_HARD_LIMITS;
+
 	prop = RNA_def_property(cont, identifier, PROP_FLOAT, PROP_NONE);
 	if (len != 0) RNA_def_property_array(prop, len);
 	if (default_value) RNA_def_property_float_array_default(prop, default_value);
@@ -2551,6 +2898,8 @@ PropertyRNA *RNA_def_float_percentage(StructOrFunctionRNA *cont_, const char *id
 	ContainerRNA *cont = cont_;
 	PropertyRNA *prop;
 	
+	ASSERT_SOFT_HARD_LIMITS;
+
 	prop = RNA_def_property(cont, identifier, PROP_FLOAT, PROP_PERCENTAGE);
 	RNA_def_property_float_default(prop, default_value);
 	if (hardmin != hardmax) RNA_def_property_range(prop, hardmin, hardmax);
@@ -2567,6 +2916,8 @@ PropertyRNA *RNA_def_float_factor(StructOrFunctionRNA *cont_, const char *identi
 	ContainerRNA *cont = cont_;
 	PropertyRNA *prop;
 	
+	ASSERT_SOFT_HARD_LIMITS;
+
 	prop = RNA_def_property(cont, identifier, PROP_FLOAT, PROP_FACTOR);
 	RNA_def_property_float_default(prop, default_value);
 	if (hardmin != hardmax) RNA_def_property_range(prop, hardmin, hardmax);
@@ -2639,7 +2990,7 @@ static FunctionRNA *rna_def_function(StructRNA *srna, const char *identifier)
 	if (DefRNA.preprocess) {
 		char error[512];
 
-		if (rna_validate_identifier(identifier, error, 0) == 0) {
+		if (rna_validate_identifier(identifier, error, false) == 0) {
 			fprintf(stderr, "%s: function identifier \"%s\" - %s\n", __func__, identifier, error);
 			DefRNA.error = 1;
 		}
@@ -2717,6 +3068,7 @@ void RNA_def_function_return(FunctionRNA *func, PropertyRNA *ret)
 		return;
 	}
 
+	BLI_assert(func->c_ret == NULL);
 	func->c_ret = ret;
 
 	RNA_def_function_output(func, ret);
@@ -2740,13 +3092,13 @@ void RNA_def_function_ui_description(FunctionRNA *func, const char *description)
 int rna_parameter_size(PropertyRNA *parm)
 {
 	PropertyType ptype = parm->type;
-	int len = parm->totarraylength; /* only supports fixed length at the moment */
+	int len = parm->totarraylength;
+
+	/* XXX in other parts is mentioned that strings can be dynamic as well */
+	if (parm->flag & PROP_DYNAMIC)
+		return sizeof(ParameterDynAlloc);
 
 	if (len > 0) {
-		/* XXX in other parts is mentioned that strings can be dynamic as well */
-		if (parm->flag & PROP_DYNAMIC)
-			return sizeof(void *);
-
 		switch (ptype) {
 			case PROP_BOOLEAN:
 			case PROP_INT:
@@ -2777,14 +3129,26 @@ int rna_parameter_size(PropertyRNA *parm)
 			{
 #ifdef RNA_RUNTIME
 				if (parm->flag & PROP_RNAPTR)
-					return sizeof(PointerRNA);
+					if (parm->flag & PROP_THICK_WRAP) {
+						return sizeof(PointerRNA);
+					}
+					else {
+						return sizeof(PointerRNA *);
+					}
 				else
 					return sizeof(void *);
 #else
-				if (parm->flag & PROP_RNAPTR)
-					return sizeof(PointerRNA);
-				else
+				if (parm->flag & PROP_RNAPTR) {
+					if (parm->flag & PROP_THICK_WRAP) {
+						return sizeof(PointerRNA);
+					}
+					else {
+						return sizeof(PointerRNA *);
+					}
+				}
+				else {
 					return sizeof(void *);
+				}
 #endif
 			}
 			case PROP_COLLECTION:
@@ -2795,18 +3159,6 @@ int rna_parameter_size(PropertyRNA *parm)
 	return sizeof(void *);
 }
 
-/* this function returns the size of the memory allocated for the parameter,
- * useful for instance for memory alignment or for storing additional information */
-int rna_parameter_size_alloc(PropertyRNA *parm)
-{
-	int size = rna_parameter_size(parm);
-
-	if (parm->flag & PROP_DYNAMIC)
-		size += sizeof(((ParameterDynAlloc *)NULL)->array_tot);
-
-	return size;
-}
-
 /* Dynamic Enums */
 
 void RNA_enum_item_add(EnumPropertyItem **items, int *totitem, const EnumPropertyItem *item)
@@ -2908,9 +3260,8 @@ void RNA_def_property_duplicate_pointers(StructOrFunctionRNA *cont_, PropertyRNA
 	 * in the first place */
 	if (prop->identifier) {
 		if (cont->prophash) {
-			BLI_ghash_remove(cont->prophash, (void *)prop->identifier, NULL, NULL);
 			prop->identifier = BLI_strdup(prop->identifier);
-			BLI_ghash_insert(cont->prophash, (void *)prop->identifier, prop);
+			BLI_ghash_reinsert(cont->prophash, (void *)prop->identifier, prop, NULL, NULL);
 		}
 		else {
 			prop->identifier = BLI_strdup(prop->identifier);
