@@ -34,13 +34,16 @@
 #include <string.h>
 #include <time.h>
 
-#include "RNA_define.h"
-#include "RNA_enum_types.h"
-
 #include "DNA_packedFile_types.h"
 
+#include "BLI_utildefines.h"
+#include "BLI_path_util.h"
+
 #include "BIF_gl.h"
 
+#include "RNA_define.h"
+#include "RNA_enum_types.h"
+
 #include "rna_internal.h"  /* own include */
 
 #ifdef RNA_RUNTIME
@@ -71,7 +74,7 @@ static void rna_Image_save_render(Image *image, bContext *C, ReportList *reports
 	}
 
 	if (scene) {
-		ImageUser iuser;
+		ImageUser iuser = {0};
 		void *lock;
 
 		iuser.scene = scene;
@@ -80,43 +83,43 @@ static void rna_Image_save_render(Image *image, bContext *C, ReportList *reports
 		ibuf = BKE_image_acquire_ibuf(image, &iuser, &lock);
 
 		if (ibuf == NULL) {
-			BKE_reportf(reports, RPT_ERROR, "Couldn't acquire buffer from image");
+			BKE_report(reports, RPT_ERROR, "Could not acquire buffer from image");
 		}
 		else {
 			ImBuf *write_ibuf;
 
-			write_ibuf = IMB_colormanagement_imbuf_for_write(ibuf, TRUE, TRUE, &scene->view_settings,
+			write_ibuf = IMB_colormanagement_imbuf_for_write(ibuf, true, true, &scene->view_settings,
 			                                                 &scene->display_settings, &scene->r.im_format);
 
 			write_ibuf->planes = scene->r.im_format.planes;
 			write_ibuf->dither = scene->r.dither_intensity;
 
 			if (!BKE_imbuf_write(write_ibuf, path, &scene->r.im_format)) {
-				BKE_reportf(reports, RPT_ERROR, "Couldn't write image: %s", path);
+				BKE_reportf(reports, RPT_ERROR, "Could not write image '%s'", path);
 			}
 
 			if (write_ibuf != ibuf)
 				IMB_freeImBuf(write_ibuf);
 		}
 
-		BKE_image_release_ibuf(image, lock);
+		BKE_image_release_ibuf(image, ibuf, lock);
 	}
 	else {
-		BKE_reportf(reports, RPT_ERROR, "Scene not in context, couldn't get save parameters");
+		BKE_report(reports, RPT_ERROR, "Scene not in context, could not get save parameters");
 	}
 }
 
-static void rna_Image_save(Image *image, ReportList *reports)
+static void rna_Image_save(Image *image, bContext *C, ReportList *reports)
 {
-	ImBuf *ibuf = BKE_image_get_ibuf(image, NULL);
+	ImBuf *ibuf = BKE_image_acquire_ibuf(image, NULL, NULL);
 	if (ibuf) {
 		char filename[FILE_MAX];
 		BLI_strncpy(filename, image->name, sizeof(filename));
-		BLI_path_abs(filename, G.main->name);
+		BLI_path_abs(filename, ID_BLEND_PATH(G.main, &image->id));
 
 		if (image->packedfile) {
 			if (writePackedFile(reports, image->name, image->packedfile, 0) != RET_OK) {
-				BKE_reportf(reports, RPT_ERROR, "Image \"%s\" could saved packed file to \"%s\"", image->id.name + 2, image->name);
+				BKE_reportf(reports, RPT_ERROR, "Image '%s' could not save packed file to '%s'", image->id.name + 2, image->name);
 			}
 		}
 		else if (IMB_saveiff(ibuf, filename, ibuf->flags)) {
@@ -130,20 +133,23 @@ static void rna_Image_save(Image *image, ReportList *reports)
 			ibuf->userflags &= ~IB_BITMAPDIRTY;
 		}
 		else {
-			BKE_reportf(reports, RPT_ERROR, "Image \"%s\" could not be saved to \"%s\"", image->id.name + 2, image->name);
+			BKE_reportf(reports, RPT_ERROR, "Image '%s' could not be saved to '%s'", image->id.name + 2, image->name);
 		}
 	}
 	else {
-		BKE_reportf(reports, RPT_ERROR, "Image \"%s\" does not have any image data", image->id.name + 2);
+		BKE_reportf(reports, RPT_ERROR, "Image '%s' does not have any image data", image->id.name + 2);
 	}
+
+	BKE_image_release_ibuf(image, ibuf, NULL);
+	WM_event_add_notifier(C, NC_IMAGE | NA_EDITED, image);
 }
 
-static void rna_Image_pack(Image *image, ReportList *reports, int as_png)
+static void rna_Image_pack(Image *image, bContext *C, ReportList *reports, int as_png)
 {
-	ImBuf *ibuf = BKE_image_get_ibuf(image, NULL);
+	ImBuf *ibuf = BKE_image_acquire_ibuf(image, NULL, NULL);
 
 	if (!as_png && (ibuf && (ibuf->userflags & IB_BITMAPDIRTY))) {
-		BKE_reportf(reports, RPT_ERROR, "Can't pack edited image from disk, only as internal PNG");
+		BKE_report(reports, RPT_ERROR, "Cannot pack edited image from disk, only as internal PNG");
 	}
 	else {
 		if (as_png) {
@@ -153,6 +159,9 @@ static void rna_Image_pack(Image *image, ReportList *reports, int as_png)
 			image->packedfile = newPackedFile(reports, image->name, ID_BLEND_PATH(G.main, &image->id));
 		}
 	}
+
+	BKE_image_release_ibuf(image, ibuf, NULL);
+	WM_event_add_notifier(C, NC_IMAGE | NA_EDITED, image);
 }
 
 static void rna_Image_unpack(Image *image, ReportList *reports, int method)
@@ -160,7 +169,7 @@ static void rna_Image_unpack(Image *image, ReportList *reports, int method)
 	if (!image->packedfile) {
 		BKE_report(reports, RPT_ERROR, "Image not packed");
 	}
-	else if (image->source == IMA_SRC_SEQUENCE || image->source == IMA_SRC_MOVIE) {
+	else if (BKE_image_is_animated(image)) {
 		BKE_report(reports, RPT_ERROR, "Unpacking movies or image sequences not supported");
 		return;
 	}
@@ -177,36 +186,46 @@ static void rna_Image_reload(Image *image)
 
 static void rna_Image_update(Image *image, ReportList *reports)
 {
-	ImBuf *ibuf = BKE_image_get_ibuf(image, NULL);
+	ImBuf *ibuf = BKE_image_acquire_ibuf(image, NULL, NULL);
 
 	if (ibuf == NULL) {
-		BKE_reportf(reports, RPT_ERROR, "Image \"%s\" does not have any image data", image->id.name + 2);
+		BKE_reportf(reports, RPT_ERROR, "Image '%s' does not have any image data", image->id.name + 2);
 		return;
 	}
 
-	IMB_rect_from_float(ibuf);
+	if (ibuf->rect)
+		IMB_rect_from_float(ibuf);
+
+	ibuf->userflags |= IB_DISPLAY_BUFFER_INVALID;
+
+	BKE_image_release_ibuf(image, ibuf, NULL);
 }
 
 static void rna_Image_scale(Image *image, ReportList *reports, int width, int height)
 {
 	if (!BKE_image_scale(image, width, height)) {
-		BKE_reportf(reports, RPT_ERROR, "Image \"%s\" does not have any image data", image->id.name + 2);
+		BKE_reportf(reports, RPT_ERROR, "Image '%s' does not have any image data", image->id.name + 2);
 	}
 }
 
-static int rna_Image_gl_load(Image *image, ReportList *reports, int filter, int mag)
+static int rna_Image_gl_load(Image *image, ReportList *reports, int frame, int filter, int mag)
 {
 	ImBuf *ibuf;
 	unsigned int *bind = &image->bindcode;
 	int error = GL_NO_ERROR;
+	ImageUser iuser = {NULL};
+	void *lock;
 
 	if (*bind)
 		return error;
+	iuser.framenr = frame;
+	iuser.ok = true;
 
-	ibuf = BKE_image_get_ibuf(image, NULL);
+	ibuf = BKE_image_acquire_ibuf(image, &iuser, &lock);
 
 	if (ibuf == NULL || ibuf->rect == NULL) {
-		BKE_reportf(reports, RPT_ERROR, "Image \"%s\" does not have any image data", image->id.name + 2);
+		BKE_reportf(reports, RPT_ERROR, "Image '%s' does not have any image data", image->id.name + 2);
+		BKE_image_release_ibuf(image, ibuf, NULL);
 		return (int)GL_INVALID_OPERATION;
 	}
 
@@ -235,10 +254,12 @@ static int rna_Image_gl_load(Image *image, ReportList *reports, int filter, int
 		image->bindcode = 0;
 	}
 
+	BKE_image_release_ibuf(image, ibuf, NULL);
+
 	return error;
 }
 
-static int rna_Image_gl_touch(Image *image, ReportList *reports, int filter, int mag)
+static int rna_Image_gl_touch(Image *image, ReportList *reports, int frame, int filter, int mag)
 {
 	unsigned int *bind = &image->bindcode;
 	int error = GL_NO_ERROR;
@@ -246,7 +267,7 @@ static int rna_Image_gl_touch(Image *image, ReportList *reports, int filter, int
 	BKE_image_tag_time(image);
 
 	if (*bind == 0)
-		error = rna_Image_gl_load(image, reports, filter, mag);
+		error = rna_Image_gl_load(image, reports, frame, filter, mag);
 
 	return error;
 }
@@ -259,6 +280,11 @@ static void rna_Image_gl_free(Image *image)
 	image->flag &= ~IMA_NOCOLLECT;
 }
 
+static void rna_Image_filepath_from_user(Image *image, ImageUser *image_user, char *filepath)
+{
+	BKE_image_user_file_path(image_user, image, filepath);
+}
+
 #else
 
 void RNA_api_image(StructRNA *srna)
@@ -269,17 +295,17 @@ void RNA_api_image(StructRNA *srna)
 	func = RNA_def_function(srna, "save_render", "rna_Image_save_render");
 	RNA_def_function_ui_description(func, "Save image to a specific path using a scenes render settings");
 	RNA_def_function_flag(func, FUNC_USE_CONTEXT | FUNC_USE_REPORTS);
-	parm = RNA_def_string_file_path(func, "filepath", "", 0, "", "Save path");
+	parm = RNA_def_string_file_path(func, "filepath", NULL, 0, "", "Save path");
 	RNA_def_property_flag(parm, PROP_REQUIRED);
 	RNA_def_pointer(func, "scene", "Scene", "", "Scene to take image parameters from");
 
 	func = RNA_def_function(srna, "save", "rna_Image_save");
 	RNA_def_function_ui_description(func, "Save image to its source path");
-	RNA_def_function_flag(func, FUNC_USE_REPORTS);
+	RNA_def_function_flag(func, FUNC_USE_CONTEXT | FUNC_USE_REPORTS);
 
 	func = RNA_def_function(srna, "pack", "rna_Image_pack");
 	RNA_def_function_ui_description(func, "Pack an image as embedded data into the .blend file");
-	RNA_def_function_flag(func, FUNC_USE_REPORTS);
+	RNA_def_function_flag(func, FUNC_USE_CONTEXT | FUNC_USE_REPORTS);
 	RNA_def_boolean(func, "as_png", 0, "as_png", "Pack the image as PNG (needed for generated/dirty images)");
 
 	func = RNA_def_function(srna, "unpack", "rna_Image_unpack");
@@ -297,18 +323,20 @@ void RNA_api_image(StructRNA *srna)
 	func = RNA_def_function(srna, "scale", "rna_Image_scale");
 	RNA_def_function_ui_description(func, "Scale the image in pixels");
 	RNA_def_function_flag(func, FUNC_USE_REPORTS);
-	parm = RNA_def_int(func, "width", 0, 1, 10000, "", "Width", 1, 10000);
+	parm = RNA_def_int(func, "width", 1, 1, 10000, "", "Width", 1, 10000);
 	RNA_def_property_flag(parm, PROP_REQUIRED);
-	parm = RNA_def_int(func, "height", 0, 1, 10000, "", "Height", 1, 10000);
+	parm = RNA_def_int(func, "height", 1, 1, 10000, "", "Height", 1, 10000);
 	RNA_def_property_flag(parm, PROP_REQUIRED);
 
 	func = RNA_def_function(srna, "gl_touch", "rna_Image_gl_touch");
 	RNA_def_function_ui_description(func, "Delay the image from being cleaned from the cache due inactivity");
 	RNA_def_function_flag(func, FUNC_USE_REPORTS);
+	RNA_def_int(func, "frame", 0, 0, INT_MAX, "Frame",
+	            "Frame of image sequence or movie", 0, INT_MAX);
 	RNA_def_int(func, "filter", GL_LINEAR_MIPMAP_NEAREST, -INT_MAX, INT_MAX, "Filter",
-	            "The texture minifying function to use if the image wan't loaded", -INT_MAX, INT_MAX);
+	            "The texture minifying function to use if the image wasn't loaded", -INT_MAX, INT_MAX);
 	RNA_def_int(func, "mag", GL_LINEAR, -INT_MAX, INT_MAX, "Magnification",
-	            "The texture magnification function to use if the image wan't loaded", -INT_MAX, INT_MAX);
+	            "The texture magnification function to use if the image wasn't loaded", -INT_MAX, INT_MAX);
 	/* return value */
 	parm = RNA_def_int(func, "error", 0, -INT_MAX, INT_MAX, "Error", "OpenGL error value", -INT_MAX, INT_MAX);
 	RNA_def_function_return(func, parm);
@@ -316,10 +344,13 @@ void RNA_api_image(StructRNA *srna)
 	func = RNA_def_function(srna, "gl_load", "rna_Image_gl_load");
 	RNA_def_function_ui_description(func, "Load the image into OpenGL graphics memory");
 	RNA_def_function_flag(func, FUNC_USE_REPORTS);
+	RNA_def_int(func, "frame", 0, 0, INT_MAX, "Frame",
+	            "Frame of image sequence or movie", 0, INT_MAX);
 	RNA_def_int(func, "filter", GL_LINEAR_MIPMAP_NEAREST, -INT_MAX, INT_MAX, "Filter",
 	            "The texture minifying function", -INT_MAX, INT_MAX);
 	RNA_def_int(func, "mag", GL_LINEAR, -INT_MAX, INT_MAX, "Magnification",
 	            "The texture magnification function", -INT_MAX, INT_MAX);
+
 	/* return value */
 	parm = RNA_def_int(func, "error", 0, -INT_MAX, INT_MAX, "Error", "OpenGL error value", -INT_MAX, INT_MAX);
 	RNA_def_function_return(func, parm);
@@ -327,6 +358,15 @@ void RNA_api_image(StructRNA *srna)
 	func = RNA_def_function(srna, "gl_free", "rna_Image_gl_free");
 	RNA_def_function_ui_description(func, "Free the image from OpenGL graphics memory");
 
+	/* path to an frame specified by image user */
+	func = RNA_def_function(srna, "filepath_from_user", "rna_Image_filepath_from_user");
+	RNA_def_function_ui_description(func, "Return the absolute path to the filepath of an image frame specified by the image user");
+	RNA_def_pointer(func, "image_user", "ImageUser", "", "Image user of the image to get filepath for");
+	parm = RNA_def_string_file_path(func, "filepath", NULL, FILE_MAX, "File Path",
+	                                "The resulting filepath from the image and it's user");
+	RNA_def_property_flag(parm, PROP_THICK_WRAP);  /* needed for string return value */
+	RNA_def_function_output(func, parm);
+
 	/* TODO, pack/unpack, maybe should be generic functions? */
 }
 
