@@ -42,6 +42,7 @@
 
 #include "MEM_guardedalloc.h"
 
+#include "BKE_context.h"
 #include "BKE_depsgraph.h"
 #include "BKE_main.h"
 #include "BKE_texture.h"
@@ -55,10 +56,6 @@ static PointerRNA rna_World_lighting_get(PointerRNA *ptr)
 	return rna_pointer_inherit_refine(ptr, &RNA_WorldLighting, ptr->id.data);
 }
 
-static PointerRNA rna_World_stars_get(PointerRNA *ptr)
-{
-	return rna_pointer_inherit_refine(ptr, &RNA_WorldStarsSettings, ptr->id.data);
-}
 
 static PointerRNA rna_World_mist_get(PointerRNA *ptr)
 {
@@ -92,7 +89,7 @@ static void rna_World_update(Main *UNUSED(bmain), Scene *UNUSED(scene), PointerR
 	World *wo = ptr->id.data;
 
 	DAG_id_tag_update(&wo->id, 0);
-	WM_main_add_notifier(NC_WORLD, wo);
+	WM_main_add_notifier(NC_WORLD | ND_WORLD, wo);
 }
 
 static void rna_World_draw_update(Main *UNUSED(bmain), Scene *UNUSED(scene), PointerRNA *ptr)
@@ -113,22 +110,14 @@ static void rna_World_draw_mist_update(Main *UNUSED(bmain), Scene *UNUSED(scene)
 	WM_main_add_notifier(NC_OBJECT | ND_DRAW, NULL);
 }
 
-static void rna_World_stars_update(Main *UNUSED(bmain), Scene *UNUSED(scene), PointerRNA *ptr)
-{
-	World *wo = ptr->id.data;
-
-	DAG_id_tag_update(&wo->id, 0);
-	WM_main_add_notifier(NC_WORLD | ND_WORLD_STARS, wo);
-}
-
-static void rna_World_use_nodes_update(Main *bmain, Scene *scene, PointerRNA *ptr)
+static void rna_World_use_nodes_update(bContext *C, PointerRNA *ptr)
 {
 	World *wrld = (World *)ptr->data;
 
 	if (wrld->use_nodes && wrld->nodetree == NULL)
-		ED_node_shader_default(scene, &wrld->id);
+		ED_node_shader_default(C, &wrld->id);
 	
-	rna_World_update(bmain, scene, ptr);
+	rna_World_update(CTX_data_main(C), CTX_data_scene(C), ptr);
 }
 
 #else
@@ -449,63 +438,11 @@ static void rna_def_world_mist(BlenderRNA *brna)
 	RNA_def_property_update(prop, 0, "rna_World_update");
 }
 
-static void rna_def_world_stars(BlenderRNA *brna)
-{
-	StructRNA *srna;
-	PropertyRNA *prop;
-
-	srna = RNA_def_struct(brna, "WorldStarsSettings", NULL);
-	RNA_def_struct_sdna(srna, "World");
-	RNA_def_struct_nested(brna, srna, "World");
-	RNA_def_struct_ui_text(srna, "World Stars", "Stars settings for a World data-block");
-
-	prop = RNA_def_property(srna, "use_stars", PROP_BOOLEAN, PROP_NONE);
-	RNA_def_property_boolean_sdna(prop, NULL, "mode", WO_STARS);
-	RNA_def_property_ui_text(prop, "Use Stars", "Enable starfield generation");
-	RNA_def_property_update(prop, 0, "rna_World_stars_update");
-
-	prop = RNA_def_property(srna, "size", PROP_FLOAT, PROP_NONE);
-	RNA_def_property_float_sdna(prop, NULL, "starsize");
-	RNA_def_property_range(prop, 0, 10);
-	RNA_def_property_ui_text(prop, "Size", "Average screen dimension of stars");
-	RNA_def_property_update(prop, 0, "rna_World_draw_update"); /* use normal update since this isn't visualized */
-
-	prop = RNA_def_property(srna, "distance_min", PROP_FLOAT, PROP_DISTANCE);
-	RNA_def_property_float_sdna(prop, NULL, "starmindist");
-	RNA_def_property_range(prop, 0, 1000);
-	RNA_def_property_ui_text(prop, "Minimum Distance", "Minimum distance to the camera for stars");
-	RNA_def_property_update(prop, 0, "rna_World_stars_update");
-
-	prop = RNA_def_property(srna, "average_separation", PROP_FLOAT, PROP_NONE);
-	RNA_def_property_float_sdna(prop, NULL, "stardist");
-	RNA_def_property_range(prop, 2, 1000);
-	RNA_def_property_ui_text(prop, "Average Separation", "Average distance between any two stars");
-	RNA_def_property_update(prop, 0, "rna_World_stars_update");
-
-	prop = RNA_def_property(srna, "color_random", PROP_FLOAT, PROP_NONE);
-	RNA_def_property_float_sdna(prop, NULL, "starcolnoise");
-	RNA_def_property_range(prop, 0, 1);
-	RNA_def_property_ui_text(prop, "Color Randomization", "Randomize star colors");
-	RNA_def_property_update(prop, 0, "rna_World_stars_update");
-}
-
 void RNA_def_world(BlenderRNA *brna)
 {
 	StructRNA *srna;
 	PropertyRNA *prop;
 
-#if 0
-	static EnumPropertyItem physics_engine_items[] = {
-		{WOPHY_NONE, "NONE", 0, "None", ""},
-		/*{WOPHY_ENJI, "ENJI", 0, "Enji", ""}, */
-		/*{WOPHY_SUMO, "SUMO", 0, "Sumo (Deprecated)", ""}, */
-		/*{WOPHY_DYNAMO, "DYNAMO", 0, "Dynamo", ""}, */
-		/*{WOPHY_ODE, "ODE", 0, "ODE", ""}, */
-		{WOPHY_BULLET, "BULLET", 0, "Bullet", ""},
-		{0, NULL, 0, NULL, NULL}
-	};
-#endif
-
 	srna = RNA_def_struct(brna, "World", "ID");
 	RNA_def_struct_ui_text(srna, "World",
 	                       "World datablock describing the environment and ambient lighting of a scene");
@@ -513,7 +450,8 @@ void RNA_def_world(BlenderRNA *brna)
 
 	rna_def_animdata_common(srna);
 	rna_def_mtex_common(brna, srna, "rna_World_mtex_begin", "rna_World_active_texture_get",
-	                    "rna_World_active_texture_set", NULL, "WorldTextureSlot", "WorldTextureSlots", "rna_World_update");
+	                    "rna_World_active_texture_set", NULL, "WorldTextureSlot", "WorldTextureSlots",
+	                    "rna_World_update", "rna_World_update");
 
 	/* colors */
 	prop = RNA_def_property(srna, "horizon_color", PROP_FLOAT, PROP_COLOR);
@@ -529,7 +467,7 @@ void RNA_def_world(BlenderRNA *brna)
 	RNA_def_property_float_sdna(prop, NULL, "zenr");
 	RNA_def_property_array(prop, 3);
 	RNA_def_property_ui_text(prop, "Zenith Color", "Color at the zenith");
-	RNA_def_property_update(prop, 0, "rna_World_update");
+	RNA_def_property_update(prop, NC_WORLD | ND_WORLD_DRAW, "rna_World_update");
 
 	prop = RNA_def_property(srna, "ambient_color", PROP_FLOAT, PROP_COLOR);
 	RNA_def_property_float_sdna(prop, NULL, "ambr");
@@ -554,17 +492,17 @@ void RNA_def_world(BlenderRNA *brna)
 	prop = RNA_def_property(srna, "use_sky_blend", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_sdna(prop, NULL, "skytype", WO_SKYBLEND);
 	RNA_def_property_ui_text(prop, "Blend Sky", "Render background with natural progression from horizon to zenith");
-	RNA_def_property_update(prop, 0, "rna_World_update");
+	RNA_def_property_update(prop, NC_WORLD | ND_WORLD_DRAW, "rna_World_update");
 
 	prop = RNA_def_property(srna, "use_sky_paper", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_sdna(prop, NULL, "skytype", WO_SKYPAPER);
 	RNA_def_property_ui_text(prop, "Paper Sky", "Flatten blend or texture coordinates");
-	RNA_def_property_update(prop, 0, "rna_World_update");
+	RNA_def_property_update(prop, NC_WORLD | ND_WORLD_DRAW, "rna_World_update");
 
 	prop = RNA_def_property(srna, "use_sky_real", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_sdna(prop, NULL, "skytype", WO_SKYREAL);
 	RNA_def_property_ui_text(prop, "Real Sky", "Render background with a real horizon, relative to the camera angle");
-	RNA_def_property_update(prop, 0, "rna_World_update");
+	RNA_def_property_update(prop, NC_WORLD | ND_WORLD_DRAW, "rna_World_update");
 
 	/* nested structs */
 	prop = RNA_def_property(srna, "light_settings", PROP_POINTER, PROP_NONE);
@@ -579,12 +517,6 @@ void RNA_def_world(BlenderRNA *brna)
 	RNA_def_property_pointer_funcs(prop, "rna_World_mist_get", NULL, NULL, NULL);
 	RNA_def_property_ui_text(prop, "Mist", "World mist settings");
 
-	prop = RNA_def_property(srna, "star_settings", PROP_POINTER, PROP_NONE);
-	RNA_def_property_flag(prop, PROP_NEVER_NULL);
-	RNA_def_property_struct_type(prop, "WorldStarsSettings");
-	RNA_def_property_pointer_funcs(prop, "rna_World_stars_get", NULL, NULL, NULL);
-	RNA_def_property_ui_text(prop, "Stars", "World stars settings");
-
 	/* nodes */
 	prop = RNA_def_property(srna, "node_tree", PROP_POINTER, PROP_NONE);
 	RNA_def_property_pointer_sdna(prop, NULL, "nodetree");
@@ -593,12 +525,12 @@ void RNA_def_world(BlenderRNA *brna)
 	prop = RNA_def_property(srna, "use_nodes", PROP_BOOLEAN, PROP_NONE);
 	RNA_def_property_boolean_sdna(prop, NULL, "use_nodes", 1);
 	RNA_def_property_clear_flag(prop, PROP_ANIMATABLE);
+	RNA_def_property_flag(prop, PROP_CONTEXT_UPDATE);
 	RNA_def_property_ui_text(prop, "Use Nodes", "Use shader nodes to render the world");
 	RNA_def_property_update(prop, 0, "rna_World_use_nodes_update");
 
 	rna_def_lighting(brna);
 	rna_def_world_mist(brna);
-	rna_def_world_stars(brna);
 	rna_def_world_mtex(brna);
 }
 
