@@ -37,25 +37,23 @@
 #include "DNA_scene_types.h"
 #include "DNA_object_types.h"
 
+#include "BLI_utildefines.h"
 #include "BLI_kdtree.h"
 #include "BLI_rand.h"
 #include "BLI_math.h"
 #include "BLI_edgehash.h"
-#include "BLI_utildefines.h"
 
 #include "BKE_cdderivedmesh.h"
 #include "BKE_deform.h"
 #include "BKE_lattice.h"
 #include "BKE_mesh.h"
 #include "BKE_modifier.h"
-#include "BKE_object.h"
 #include "BKE_particle.h"
 #include "BKE_scene.h"
 
 
 #include "MEM_guardedalloc.h"
 
-#include "MOD_util.h"
 
 static void initData(ModifierData *md)
 {
@@ -72,17 +70,18 @@ static void freeData(ModifierData *md)
 }
 static void copyData(ModifierData *md, ModifierData *target)
 {
+#if 0
 	ExplodeModifierData *emd = (ExplodeModifierData *) md;
+#endif
 	ExplodeModifierData *temd = (ExplodeModifierData *) target;
 
+	modifier_copyData_generic(md, target);
+
 	temd->facepa = NULL;
-	temd->flag = emd->flag;
-	temd->protect = emd->protect;
-	temd->vgroup = emd->vgroup;
 }
-static int dependsOnTime(ModifierData *UNUSED(md)) 
+static bool dependsOnTime(ModifierData *UNUSED(md))
 {
-	return 1;
+	return true;
 }
 static CustomDataMask requiredDataMask(Object *UNUSED(ob), ModifierData *md)
 {
@@ -104,6 +103,7 @@ static void createFacepa(ExplodeModifierData *emd,
 	MVert *mvert = NULL;
 	ParticleData *pa;
 	KDTree *tree;
+	RNG *rng;
 	float center[3], co[3];
 	int *facepa = NULL, *vertpa = NULL, totvert = 0, totface = 0, totpart = 0;
 	int i, p, v1, v2, v3, v4 = 0;
@@ -114,7 +114,7 @@ static void createFacepa(ExplodeModifierData *emd,
 	totvert = dm->getNumVerts(dm);
 	totpart = psmd->psys->totpart;
 
-	BLI_srandom(psys->seed);
+	rng = BLI_rng_new_srandom(psys->seed);
 
 	if (emd->facepa)
 		MEM_freeN(emd->facepa);
@@ -136,7 +136,7 @@ static void createFacepa(ExplodeModifierData *emd,
 		if (dvert) {
 			const int defgrp_index = emd->vgroup - 1;
 			for (i = 0; i < totvert; i++, dvert++) {
-				float val = BLI_frand();
+				float val = BLI_rng_get_float(rng);
 				val = (1.0f - emd->protect) * val + emd->protect * 0.5f;
 				if (val < defvert_find_weight(dvert, defgrp_index))
 					vertpa[i] = -1;
@@ -148,7 +148,7 @@ static void createFacepa(ExplodeModifierData *emd,
 	tree = BLI_kdtree_new(totpart);
 	for (p = 0, pa = psys->particles; p < totpart; p++, pa++) {
 		psys_particle_on_emitter(psmd, psys->part->from, pa->num, pa->num_dmcache, pa->fuv, pa->foffset, co, NULL, NULL, NULL, NULL, NULL);
-		BLI_kdtree_insert(tree, p, co, NULL);
+		BLI_kdtree_insert(tree, p, co);
 	}
 	BLI_kdtree_balance(tree);
 
@@ -161,9 +161,9 @@ static void createFacepa(ExplodeModifierData *emd,
 			mul_v3_fl(center, 0.25);
 		}
 		else
-			mul_v3_fl(center, 0.3333f);
+			mul_v3_fl(center, 1.0f / 3.0f);
 
-		p = BLI_kdtree_find_nearest(tree, center, NULL, NULL);
+		p = BLI_kdtree_find_nearest(tree, center, NULL);
 
 		v1 = vertpa[fa->v1];
 		v2 = vertpa[fa->v2];
@@ -182,6 +182,8 @@ static void createFacepa(ExplodeModifierData *emd,
 
 	if (vertpa) MEM_freeN(vertpa);
 	BLI_kdtree_free(tree);
+
+	BLI_rng_free(rng);
 }
 
 static int edgecut_get(EdgeHash *edgehash, unsigned int v1, unsigned int v2)
@@ -205,15 +207,16 @@ static MFace *get_dface(DerivedMesh *dm, DerivedMesh *split, int cur, int i, MFa
 	return df;
 }
 
-#define SET_VERTS(a, b, c, d)  {        \
-        v[0] = mf->v##a; uv[0] = a - 1; \
-        v[1] = mf->v##b; uv[1] = b - 1; \
-        v[2] = mf->v##c; uv[2] = c - 1; \
-        v[3] = mf->v##d; uv[3] = d - 1; \
-    } (void)0
+#define SET_VERTS(a, b, c, d)           \
+	{                                   \
+		v[0] = mf->v##a; uv[0] = a - 1; \
+		v[1] = mf->v##b; uv[1] = b - 1; \
+		v[2] = mf->v##c; uv[2] = c - 1; \
+		v[3] = mf->v##d; uv[3] = d - 1; \
+	} (void)0
 
 #define GET_ES(v1, v2) edgecut_get(eh, v1, v2)
-#define INT_UV(uvf, c0, c1) interp_v2_v2v2(uvf, mf->uv[c0], mf->uv[c1], 0.5f)
+#define INT_UV(uvf, c0, c1) mid_v2_v2v2(uvf, mf->uv[c0], mf->uv[c1])
 
 static void remap_faces_3_6_9_12(DerivedMesh *dm, DerivedMesh *split, MFace *mf, int *facepa, int *vertpa, int i, EdgeHash *eh, int cur, int v1, int v2, int v3, int v4)
 {
@@ -564,7 +567,7 @@ static DerivedMesh *cutEdges(ExplodeModifierData *emd, DerivedMesh *dm)
 	int numlayer;
 	unsigned int ed_v1, ed_v2;
 
-	edgehash = BLI_edgehash_new();
+	edgehash = BLI_edgehash_new(__func__);
 
 	/* recreate vertpa from facepa calculation */
 	for (i = 0, mf = mface; i < totface; i++, mf++) {
@@ -582,12 +585,12 @@ static DerivedMesh *cutEdges(ExplodeModifierData *emd, DerivedMesh *dm)
 		v3 = vertpa[mf->v3];
 
 		if (v1 != v2) {
-			BLI_edgehash_insert(edgehash, mf->v1, mf->v2, NULL);
+			BLI_edgehash_reinsert(edgehash, mf->v1, mf->v2, NULL);
 			(*fs) |= 1;
 		}
 
 		if (v2 != v3) {
-			BLI_edgehash_insert(edgehash, mf->v2, mf->v3, NULL);
+			BLI_edgehash_reinsert(edgehash, mf->v2, mf->v3, NULL);
 			(*fs) |= 2;
 		}
 
@@ -595,24 +598,24 @@ static DerivedMesh *cutEdges(ExplodeModifierData *emd, DerivedMesh *dm)
 			v4 = vertpa[mf->v4];
 
 			if (v3 != v4) {
-				BLI_edgehash_insert(edgehash, mf->v3, mf->v4, NULL);
+				BLI_edgehash_reinsert(edgehash, mf->v3, mf->v4, NULL);
 				(*fs) |= 4;
 			}
 
 			if (v1 != v4) {
-				BLI_edgehash_insert(edgehash, mf->v1, mf->v4, NULL);
+				BLI_edgehash_reinsert(edgehash, mf->v1, mf->v4, NULL);
 				(*fs) |= 8;
 			}
 
 			/* mark center vertex as a fake edge split */
 			if (*fs == 15)
-				BLI_edgehash_insert(edgehash, mf->v1, mf->v3, NULL);
+				BLI_edgehash_reinsert(edgehash, mf->v1, mf->v3, NULL);
 		}
 		else {
 			(*fs) |= 16; /* mark face as tri */
 
 			if (v1 != v3) {
-				BLI_edgehash_insert(edgehash, mf->v1, mf->v3, NULL);
+				BLI_edgehash_reinsert(edgehash, mf->v1, mf->v3, NULL);
 				(*fs) |= 4;
 			}
 		}
@@ -669,8 +672,7 @@ static DerivedMesh *cutEdges(ExplodeModifierData *emd, DerivedMesh *dm)
 
 		mv = CDDM_get_vert(splitdm, ed_v1);
 
-		add_v3_v3(dupve->co, mv->co);
-		mul_v3_fl(dupve->co, 0.5f);
+		mid_v3_v3v3(dupve->co, dupve->co, mv->co);
 	}
 	BLI_edgehashIterator_free(ehi);
 
@@ -797,7 +799,7 @@ static DerivedMesh *explodeMesh(ExplodeModifierData *emd,
 	float rot[4];
 	float cfra;
 	/* float timestep; */
-	int *facepa = emd->facepa;
+	const int *facepa = emd->facepa;
 	int totdup = 0, totvert = 0, totface = 0, totpart = 0, delface = 0;
 	int i, v, u;
 	unsigned int ed_v1, ed_v2, mindex = 0;
@@ -813,12 +815,12 @@ static DerivedMesh *explodeMesh(ExplodeModifierData *emd,
 	sim.psys = psmd->psys;
 	sim.psmd = psmd;
 
-	/* timestep= psys_get_timestep(&sim); */
+	/* timestep = psys_get_timestep(&sim); */
 
 	cfra = BKE_scene_frame_get(scene);
 
 	/* hash table for vertice <-> particle relations */
-	vertpahash = BLI_edgehash_new();
+	vertpahash = BLI_edgehash_new(__func__);
 
 	for (i = 0; i < totface; i++) {
 		if (facepa[i] != totpart) {
@@ -843,11 +845,11 @@ static DerivedMesh *explodeMesh(ExplodeModifierData *emd,
 		mf = &mface[i];
 
 		/* set face vertices to exist in particle group */
-		BLI_edgehash_insert(vertpahash, mf->v1, mindex, NULL);
-		BLI_edgehash_insert(vertpahash, mf->v2, mindex, NULL);
-		BLI_edgehash_insert(vertpahash, mf->v3, mindex, NULL);
+		BLI_edgehash_reinsert(vertpahash, mf->v1, mindex, NULL);
+		BLI_edgehash_reinsert(vertpahash, mf->v2, mindex, NULL);
+		BLI_edgehash_reinsert(vertpahash, mf->v3, mindex, NULL);
 		if (mf->v4)
-			BLI_edgehash_insert(vertpahash, mf->v4, mindex, NULL);
+			BLI_edgehash_reinsert(vertpahash, mf->v4, mindex, NULL);
 	}
 
 	/* make new vertice indexes & count total vertices after duplication */
@@ -861,12 +863,12 @@ static DerivedMesh *explodeMesh(ExplodeModifierData *emd,
 	/* the final duplicated vertices */
 	explode = CDDM_from_template(dm, totdup, 0, totface - delface, 0, 0);
 	mtface = CustomData_get_layer_named(&explode->faceData, CD_MTFACE, emd->uvname);
-	/*dupvert= CDDM_get_verts(explode);*/
+	/*dupvert = CDDM_get_verts(explode);*/
 
 	/* getting back to object space */
 	invert_m4_m4(imat, ob->obmat);
 
-	psmd->psys->lattice = psys_get_lattice(&sim);
+	psmd->psys->lattice_deform_data = psys_create_lattice_deform_data(&sim);
 
 	/* duplicate & displace vertices */
 	ehi = BLI_edgehashIterator_new(vertpahash);
@@ -968,11 +970,11 @@ static DerivedMesh *explodeMesh(ExplodeModifierData *emd,
 	/* finalization */
 	CDDM_calc_edges_tessface(explode);
 	CDDM_tessfaces_to_faces(explode);
-	CDDM_calc_normals(explode);
+	explode->dirty |= DM_DIRTY_NORMALS;
 
-	if (psmd->psys->lattice) {
-		end_latt_deform(psmd->psys->lattice);
-		psmd->psys->lattice = NULL;
+	if (psmd->psys->lattice_deform_data) {
+		end_latt_deform(psmd->psys->lattice_deform_data);
+		psmd->psys->lattice_deform_data = NULL;
 	}
 
 	return explode;
@@ -1039,26 +1041,26 @@ static DerivedMesh *applyModifier(ModifierData *md, Object *ob,
 
 
 ModifierTypeInfo modifierType_Explode = {
-	/* name */ "Explode",
-	/* structName */ "ExplodeModifierData",
-	/* structSize */ sizeof(ExplodeModifierData),
-	/* type */ eModifierTypeType_Constructive,
-	/* flags */ eModifierTypeFlag_AcceptsMesh,
-	/* copyData */ copyData,
-	/* deformVerts */ NULL,
-	/* deformMatrices */ NULL,
-	/* deformVertsEM */ NULL,
-	/* deformMatricesEM */ NULL,
-	/* applyModifier */ applyModifier,
-	/* applyModifierEM */ NULL,
-	/* initData */ initData,
-	/* requiredDataMask */ requiredDataMask,
-	/* freeData */ freeData,
-	/* isDisabled */ NULL,
-	/* updateDepgraph */ NULL,
-	/* dependsOnTime */ dependsOnTime,
-	/* dependsOnNormals */ NULL,
+	/* name */              "Explode",
+	/* structName */        "ExplodeModifierData",
+	/* structSize */        sizeof(ExplodeModifierData),
+	/* type */              eModifierTypeType_Constructive,
+	/* flags */             eModifierTypeFlag_AcceptsMesh,
+	/* copyData */          copyData,
+	/* deformVerts */       NULL,
+	/* deformMatrices */    NULL,
+	/* deformVertsEM */     NULL,
+	/* deformMatricesEM */  NULL,
+	/* applyModifier */     applyModifier,
+	/* applyModifierEM */   NULL,
+	/* initData */          initData,
+	/* requiredDataMask */  requiredDataMask,
+	/* freeData */          freeData,
+	/* isDisabled */        NULL,
+	/* updateDepgraph */    NULL,
+	/* dependsOnTime */     dependsOnTime,
+	/* dependsOnNormals */  NULL,
 	/* foreachObjectLink */ NULL,
-	/* foreachIDLink */ NULL,
-	/* foreachTexLink */ NULL,
+	/* foreachIDLink */     NULL,
+	/* foreachTexLink */    NULL,
 };
