@@ -48,8 +48,6 @@
 #include "BKE_particle.h"
 #include "BKE_pointcache.h"
 
-#include "MOD_util.h"
-
 #include "depsgraph_private.h"
 
 
@@ -66,21 +64,50 @@ static void initData(ModifierData *md)
 }
 static void copyData(ModifierData *md, ModifierData *target)
 {
+#if 0
 	ParticleInstanceModifierData *pimd = (ParticleInstanceModifierData *) md;
 	ParticleInstanceModifierData *tpimd = (ParticleInstanceModifierData *) target;
-
-	tpimd->ob = pimd->ob;
-	tpimd->psys = pimd->psys;
-	tpimd->flag = pimd->flag;
-	tpimd->axis = pimd->axis;
-	tpimd->position = pimd->position;
-	tpimd->random_position = pimd->random_position;
+#endif
+	modifier_copyData_generic(md, target);
 }
 
-static int dependsOnTime(ModifierData *UNUSED(md))
+static bool isDisabled(ModifierData *md, int useRenderParams)
 {
-	return 0;
+	ParticleInstanceModifierData *pimd = (ParticleInstanceModifierData *)md;
+	ParticleSystem *psys;
+	ModifierData *ob_md;
+	
+	if (!pimd->ob)
+		return true;
+	
+	psys = BLI_findlink(&pimd->ob->particlesystem, pimd->psys - 1);
+	if (psys == NULL)
+		return true;
+	
+	/* If the psys modifier is disabled we cannot use its data.
+	 * First look up the psys modifier from the object, then check if it is enabled.
+	 */
+	for (ob_md = pimd->ob->modifiers.first; ob_md; ob_md = ob_md->next) {
+		if (ob_md->type == eModifierType_ParticleSystem) {
+			ParticleSystemModifierData *psmd = (ParticleSystemModifierData *)ob_md;
+			if (psmd->psys == psys) {
+				int required_mode;
+				
+				if (useRenderParams) required_mode = eModifierMode_Render;
+				else required_mode = eModifierMode_Realtime;
+				
+				if (!modifier_isEnabled(md->scene, ob_md, required_mode))
+					return true;
+				
+				break;
+			}
+		}
+	}
+	
+	return false;
 }
+
+
 static void updateDepgraph(ModifierData *md, DagForest *forest,
                            struct Scene *UNUSED(scene),
                            Object *UNUSED(ob),
@@ -171,7 +198,9 @@ static DerivedMesh *applyModifier(ModifierData *md, Object *ob,
 		if (psys == NULL || psys->totpart == 0)
 			return derivedData;
 	}
-	else return derivedData;
+	else {
+		return derivedData;
+	}
 
 	if (pimd->flag & eParticleInstanceFlag_Parents)
 		totpart += psys->totpart;
@@ -226,14 +255,14 @@ static DerivedMesh *applyModifier(ModifierData *md, Object *ob,
 		maxloop += totloop;
 	}
 
-	psys->lattice = psys_get_lattice(&sim);
+	psys->lattice_deform_data = psys_create_lattice_deform_data(&sim);
 
 	if (psys->flag & (PSYS_HAIR_DONE | PSYS_KEYED) || psys->pointcache->flag & PTCACHE_BAKED) {
-		float min_r[3], max_r[3];
-		INIT_MINMAX(min_r, max_r);
-		dm->getMinMax(dm, min_r, max_r);
-		min_co = min_r[track];
-		max_co = max_r[track];
+		float min[3], max[3];
+		INIT_MINMAX(min, max);
+		dm->getMinMax(dm, min, max);
+		min_co = min[track];
+		max_co = max[track];
 	}
 
 	result = CDDM_from_template(dm, maxvert, 0, 0, maxloop, maxpoly);
@@ -269,12 +298,11 @@ static DerivedMesh *applyModifier(ModifierData *md, Object *ob,
 
 			/* get particle state */
 			if ((psys->flag & (PSYS_HAIR_DONE | PSYS_KEYED) || psys->pointcache->flag & PTCACHE_BAKED) &&
-				(pimd->flag & eParticleInstanceFlag_Path))
+			    (pimd->flag & eParticleInstanceFlag_Path))
 			{
 				float ran = 0.0f;
 				if (pimd->random_position != 0.0f) {
-					BLI_srandom(psys->seed + p);
-					ran = pimd->random_position * BLI_frand();
+					ran = pimd->random_position * BLI_hash_frand(psys->seed + p);
 				}
 
 				if (pimd->flag & eParticleInstanceFlag_KeepShape) {
@@ -344,51 +372,43 @@ static DerivedMesh *applyModifier(ModifierData *md, Object *ob,
 
 	CDDM_calc_edges(result);
 
-	if (psys->lattice) {
-		end_latt_deform(psys->lattice);
-		psys->lattice = NULL;
+	if (psys->lattice_deform_data) {
+		end_latt_deform(psys->lattice_deform_data);
+		psys->lattice_deform_data = NULL;
 	}
 
 	if (size)
 		MEM_freeN(size);
 
-	CDDM_calc_normals(result);
+	result->dirty |= DM_DIRTY_NORMALS;
 
 	return result;
 }
-static DerivedMesh *applyModifierEM(ModifierData *md, Object *ob,
-                                    struct BMEditMesh *UNUSED(editData),
-                                    DerivedMesh *derivedData)
-{
-	return applyModifier(md, ob, derivedData, MOD_APPLY_USECACHE);
-}
-
-
 ModifierTypeInfo modifierType_ParticleInstance = {
-	/* name */ "ParticleInstance",
-	/* structName */ "ParticleInstanceModifierData",
-	/* structSize */ sizeof(ParticleInstanceModifierData),
-	/* type */ eModifierTypeType_Constructive,
-	/* flags */ eModifierTypeFlag_AcceptsMesh |
-	eModifierTypeFlag_SupportsMapping |
-	eModifierTypeFlag_SupportsEditmode |
-	eModifierTypeFlag_EnableInEditmode,
+	/* name */              "ParticleInstance",
+	/* structName */        "ParticleInstanceModifierData",
+	/* structSize */        sizeof(ParticleInstanceModifierData),
+	/* type */              eModifierTypeType_Constructive,
+	/* flags */             eModifierTypeFlag_AcceptsMesh |
+	                        eModifierTypeFlag_SupportsMapping |
+	                        eModifierTypeFlag_SupportsEditmode |
+	                        eModifierTypeFlag_EnableInEditmode,
 
-	/* copyData */ copyData,
-	/* deformVerts */ NULL,
-	/* deformMatrices */ NULL,
-	/* deformVertsEM */ NULL,
-	/* deformMatricesEM */ NULL,
-	/* applyModifier */ applyModifier,
-	/* applyModifierEM */ applyModifierEM,
-	/* initData */ initData,
-	/* requiredDataMask */ NULL,
-	/* freeData */ NULL,
-	/* isDisabled */ NULL,
-	/* updateDepgraph */ updateDepgraph,
-	/* dependsOnTime */ dependsOnTime,
-	/* dependsOnNormals */ NULL,
+	/* copyData */          copyData,
+	/* deformVerts */       NULL,
+	/* deformMatrices */    NULL,
+	/* deformVertsEM */     NULL,
+	/* deformMatricesEM */  NULL,
+	/* applyModifier */     applyModifier,
+	/* applyModifierEM */   NULL,
+	/* initData */          initData,
+	/* requiredDataMask */  NULL,
+	/* freeData */          NULL,
+	/* isDisabled */        isDisabled,
+	/* updateDepgraph */    updateDepgraph,
+	/* dependsOnTime */     NULL,
+	/* dependsOnNormals */  NULL,
 	/* foreachObjectLink */ foreachObjectLink,
-	/* foreachIDLink */ NULL,
-	/* foreachTexLink */ NULL,
+	/* foreachIDLink */     NULL,
+	/* foreachTexLink */    NULL,
 };
