@@ -59,7 +59,6 @@
 
 #include "MEM_guardedalloc.h"
 
-#include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_object_types.h"
 #include "DNA_modifier_types.h"
@@ -67,19 +66,18 @@
 #include "BLI_utildefines.h"
 #include "BLI_array.h"
 #include "BLI_heap.h"
-#include "BLI_listbase.h"
 #include "BLI_math.h"
 #include "BLI_stack.h"
-#include "BLI_string.h"
+#include "BLI_bitmap.h"
 
 #include "BKE_cdderivedmesh.h"
 #include "BKE_deform.h"
 #include "BKE_DerivedMesh.h"
 #include "BKE_mesh.h"
+#include "BKE_mesh_mapping.h"
 #include "BKE_modifier.h"
-#include "BKE_tessmesh.h"
 
-#include "MOD_util.h"
+#include "bmesh.h"
 
 typedef struct {
 	float mat[3][3];
@@ -109,9 +107,9 @@ typedef struct Frame {
 	} merge[4];
 
 	/* For hull frames, whether each vertex is detached or not */
-	int inside_hull[4];
+	bool inside_hull[4];
 	/* Whether any part of the frame (corner or edge) is detached */
-	int detached;
+	bool detached;
 } Frame;
 
 #define MAX_SKIN_NODE_FRAMES 2
@@ -139,10 +137,11 @@ static void add_poly(SkinOutput *so,
 
 /***************************** Convex Hull ****************************/
 
-static int is_quad_symmetric(BMVert *quad[4],
-                             const SkinModifierData *smd)
+static bool is_quad_symmetric(BMVert *quad[4],
+                              const SkinModifierData *smd)
 {
 	const float threshold = 0.0001f;
+	const float threshold_squared = threshold * threshold;
 	int axis;
 
 	for (axis = 0; axis < 3; axis++) {
@@ -152,16 +151,16 @@ static int is_quad_symmetric(BMVert *quad[4],
 			copy_v3_v3(a, quad[0]->co);
 			a[axis] = -a[axis];
 
-			if (len_v3v3(a, quad[1]->co) < threshold) {
+			if (len_squared_v3v3(a, quad[1]->co) < threshold_squared) {
 				copy_v3_v3(a, quad[2]->co);
 				a[axis] = -a[axis];
-				if (len_v3v3(a, quad[3]->co) < threshold)
+				if (len_squared_v3v3(a, quad[3]->co) < threshold_squared)
 					return 1;
 			}
-			else if (len_v3v3(a, quad[3]->co) < threshold) {
+			else if (len_squared_v3v3(a, quad[3]->co) < threshold_squared) {
 				copy_v3_v3(a, quad[2]->co);
 				a[axis] = -a[axis];
-				if (len_v3v3(a, quad[1]->co) < threshold)
+				if (len_squared_v3v3(a, quad[1]->co) < threshold_squared)
 					return 1;
 			}
 		}
@@ -171,34 +170,35 @@ static int is_quad_symmetric(BMVert *quad[4],
 }
 
 /* Returns true if the quad crosses the plane of symmetry, false otherwise */
-static int quad_crosses_symmetry_plane(BMVert *quad[4],
-                                       const SkinModifierData *smd)
+static bool quad_crosses_symmetry_plane(BMVert *quad[4],
+                                        const SkinModifierData *smd)
 {
 	int axis;
 
 	for (axis = 0; axis < 3; axis++) {
 		if (smd->symmetry_axes & (1 << axis)) {
-			int i, left = 0, right = 0;
+			bool left = false, right = false;
+			int i;
 
 			for (i = 0; i < 4; i++) {
-				if (quad[i]->co[axis] < 0)
-					left = 1;
-				else if (quad[i]->co[axis] > 0)
-					right = 1;
+				if (quad[i]->co[axis] < 0.0f)
+					left = true;
+				else if (quad[i]->co[axis] > 0.0f)
+					right = true;
 
 				if (left && right)
-					return TRUE;
+					return true;
 			}
 		}
 	}
 
-	return FALSE;
+	return false;
 }
 
 /* Returns true if the frame is filled by precisely two faces (and
  * outputs those faces to fill_faces), otherwise returns false. */
-static int skin_frame_find_contained_faces(const Frame *frame,
-                                           BMFace *fill_faces[2])
+static bool skin_frame_find_contained_faces(const Frame *frame,
+                                            BMFace *fill_faces[2])
 {
 	BMEdge *diag;
 
@@ -210,12 +210,13 @@ static int skin_frame_find_contained_faces(const Frame *frame,
 	if (diag)
 		return BM_edge_face_pair(diag, &fill_faces[0], &fill_faces[1]);
 	else
-		return FALSE;
+		return false;
 }
 
-/* Returns TRUE if hull is successfully built, FALSE otherwise */
-static int build_hull(SkinOutput *so, Frame **frames, int totframe)
+/* Returns true if hull is successfully built, false otherwise */
+static bool build_hull(SkinOutput *so, Frame **frames, int totframe)
 {
+#ifdef WITH_BULLET
 	BMesh *bm = so->bm;
 	BMOperator op;
 	BMIter iter;
@@ -225,7 +226,7 @@ static int build_hull(SkinOutput *so, Frame **frames, int totframe)
 	BMEdge *e;
 	int i, j;
 
-	BM_mesh_elem_hflag_disable_all(bm, BM_VERT, BM_ELEM_TAG, FALSE);
+	BM_mesh_elem_hflag_disable_all(bm, BM_VERT, BM_ELEM_TAG, false);
 
 	for (i = 0; i < totframe; i++) {
 		for (j = 0; j < 4; j++) {
@@ -235,7 +236,7 @@ static int build_hull(SkinOutput *so, Frame **frames, int totframe)
 
 	/* Deselect all faces so that only new hull output faces are
 	 * selected after the operator is run */
-	BM_mesh_elem_hflag_disable_all(bm, BM_ALL, BM_ELEM_SELECT, 0);
+	BM_mesh_elem_hflag_disable_all(bm, BM_ALL_NOLOOP, BM_ELEM_SELECT, false);
 
 	BMO_op_initf(bm, &op, (BMO_FLAG_DEFAULTS & ~BMO_FLAG_RESPECT_HIDE),
 	             "convex_hull input=%hv", BM_ELEM_TAG);
@@ -243,26 +244,26 @@ static int build_hull(SkinOutput *so, Frame **frames, int totframe)
 
 	if (BMO_error_occurred(bm)) {
 		BMO_op_finish(bm, &op);
-		return FALSE;
+		return false;
 	}
 
 	/* Apply face attributes to hull output */
-	BMO_ITER (f, &oiter, bm, &op, "geomout", BM_FACE) {
+	BMO_ITER (f, &oiter, op.slots_out, "geom.out", BM_FACE) {
 		if (so->smd->flag & MOD_SKIN_SMOOTH_SHADING)
 			BM_elem_flag_enable(f, BM_ELEM_SMOOTH);
 		f->mat_nr = so->mat_nr;
 	}
 
 	/* Mark interior frames */
-	BMO_ITER (v, &oiter, bm, &op, "interior_geom", BM_VERT) {
+	BMO_ITER (v, &oiter, op.slots_out, "geom_interior.out", BM_VERT) {
 		for (i = 0; i < totframe; i++) {
 			Frame *frame = frames[i];
 			
 			if (!frame->detached) {
 				for (j = 0; j < 4; j++) {
 					if (frame->verts[j] == v) {
-						frame->inside_hull[j] = TRUE;
-						frame->detached = TRUE;
+						frame->inside_hull[j] = true;
+						frame->detached = true;
 						break;
 					}
 				}
@@ -280,13 +281,13 @@ static int build_hull(SkinOutput *so, Frame **frames, int totframe)
 		     !BM_edge_exists(frame->verts[2], frame->verts[3]) ||
 		     !BM_edge_exists(frame->verts[3], frame->verts[0])))
 		{
-			frame->detached = TRUE;
+			frame->detached = true;
 		}
 	}
 	
 	/* Remove triangles that would fill the original frames -- skip if
 	 * frame is partially detached */
-	BM_mesh_elem_hflag_disable_all(bm, BM_ALL, BM_ELEM_TAG, FALSE);
+	BM_mesh_elem_hflag_disable_all(bm, BM_ALL_NOLOOP, BM_ELEM_TAG, false);
 	for (i = 0; i < totframe; i++) {
 		Frame *frame = frames[i];
 		if (!frame->detached) {
@@ -301,17 +302,17 @@ static int build_hull(SkinOutput *so, Frame **frames, int totframe)
 				BM_elem_flag_enable(fill_faces[1], BM_ELEM_TAG);
 			}
 			else
-				frame->detached = TRUE;
+				frame->detached = true;
 		}
 	}
 
 	/* Check if removing triangles above will create wire triangles,
 	 * mark them too */
-	BMO_ITER (e, &oiter, bm, &op, "geomout", BM_EDGE) {
-		int is_wire = TRUE;
+	BMO_ITER (e, &oiter, op.slots_out, "geom.out", BM_EDGE) {
+		bool is_wire = true;
 		BM_ITER_ELEM (f, &iter, e, BM_FACES_OF_EDGE) {
 			if (!BM_elem_flag_test(f, BM_ELEM_TAG)) {
-				is_wire = FALSE;
+				is_wire = false;
 				break;
 			}
 		}
@@ -321,11 +322,13 @@ static int build_hull(SkinOutput *so, Frame **frames, int totframe)
 
 	BMO_op_finish(bm, &op);
 
-	BMO_op_callf(bm, BMO_FLAG_DEFAULTS,
-	             "delete geom=%hef context=%i",
-	             BM_ELEM_TAG, DEL_ONLYTAGGED);
+	BM_mesh_delete_hflag_tagged(bm, BM_ELEM_TAG, BM_EDGE | BM_FACE);
 
-	return TRUE;
+	return true;
+#else
+	UNUSED_VARS(so, frames, totframe, skin_frame_find_contained_faces);
+	return false;
+#endif
 }
 
 /* Returns the average frame side length (frames are rectangular, so
@@ -355,7 +358,7 @@ static void merge_frame_corners(Frame **frames, int totframe)
 				BLI_assert(frames[i] != frames[k]);
 
 				side_b = frame_len(frames[k]);
-				thresh = minf(side_a, side_b) / 2.0f;
+				thresh = min_ff(side_a, side_b) / 2.0f;
 
 				/* Compare with each corner of all other frames... */
 				for (l = 0; l < 4; l++) {
@@ -409,7 +412,7 @@ static Frame **collect_hull_frames(int v, SkinNode *frames,
 	int nbr, i;
 
 	(*tothullframe) = emap[v].count;
-	hull_frames = MEM_callocN(sizeof(Frame * *) * (*tothullframe),
+	hull_frames = MEM_callocN(sizeof(Frame *) * (*tothullframe),
 	                          "hull_from_frames.hull_frames");
 	i = 0;
 	for (nbr = 0; nbr < emap[v].count; nbr++) {
@@ -535,6 +538,7 @@ static int connection_node_mat(float mat[3][3], int v, const MeshElemMap *emap,
 	/* Get axis and angle to rotate frame by */
 	angle = angle_normalized_v3v3(ine[0], oute[0]) / 2.0f;
 	cross_v3_v3v3(axis, ine[0], oute[0]);
+	normalize_v3(axis);
 
 	/* Build frame matrix (don't care about X axis here) */
 	copy_v3_v3(mat[0], ine[0]);
@@ -641,9 +645,9 @@ typedef struct {
 	int e;
 } EdgeStackElem;
 
-static void build_emats_stack(BLI_Stack *stack, int *visited_e, EMat *emat,
-							  const MeshElemMap *emap, const MEdge *medge,
-							  const MVertSkin *vs, const MVert *mvert)
+static void build_emats_stack(BLI_Stack *stack, BLI_bitmap *visited_e, EMat *emat,
+                              const MeshElemMap *emap, const MEdge *medge,
+                              const MVertSkin *vs, const MVert *mvert)
 {
 	EdgeStackElem stack_elem;
 	float axis[3], angle;
@@ -654,11 +658,11 @@ static void build_emats_stack(BLI_Stack *stack, int *visited_e, EMat *emat,
 	e = stack_elem.e;
 
 	/* Skip if edge already visited */
-	if (visited_e[e])
+	if (BLI_BITMAP_TEST(visited_e, e))
 		return;
 
 	/* Mark edge as visited */
-	visited_e[e] = TRUE;
+	BLI_BITMAP_ENABLE(visited_e, e);
 	
 	/* Process edge */
 
@@ -671,7 +675,7 @@ static void build_emats_stack(BLI_Stack *stack, int *visited_e, EMat *emat,
 	/* If parent is a branch node, start a new edge chain */
 	if (parent_is_branch) {
 		calc_edge_mat(emat[e].mat, mvert[parent_v].co,
-					  mvert[v].co);
+		              mvert[v].co);
 	}
 	else {
 		/* Build edge matrix guided by parent matrix */
@@ -687,7 +691,7 @@ static void build_emats_stack(BLI_Stack *stack, int *visited_e, EMat *emat,
 	/* Add neighbors to stack */
 	for (i = 0; i < emap[v].count; i++) {
 		/* Add neighbors to stack */
-		memcpy(stack_elem.mat, emat[e].mat, sizeof(float) * 3 * 3);
+		copy_m3_m3(stack_elem.mat, emat[e].mat);
 		stack_elem.e = emap[v].indices[i];
 		stack_elem.parent_v = v;
 		BLI_stack_push(stack, &stack_elem);
@@ -695,20 +699,21 @@ static void build_emats_stack(BLI_Stack *stack, int *visited_e, EMat *emat,
 }
 
 static EMat *build_edge_mats(const MVertSkin *vs,
-							 const MVert *mvert,
-							 int totvert,
+                             const MVert *mvert,
+                             int totvert,
                              const MEdge *medge,
-							 const MeshElemMap *emap,
-							 int totedge)
+                             const MeshElemMap *emap,
+                             int totedge)
 {
 	BLI_Stack *stack;
 	EMat *emat;
 	EdgeStackElem stack_elem;
-	int *visited_e, i, v;
+	BLI_bitmap *visited_e;
+	int i, v;
 
 	stack = BLI_stack_new(sizeof(stack_elem), "build_edge_mats.stack");
 
-	visited_e = MEM_callocN(sizeof(int) * totedge, "build_edge_mats.visited_e");
+	visited_e = BLI_BITMAP_NEW(totedge, "build_edge_mats.visited_e");
 	emat = MEM_callocN(sizeof(EMat) * totedge, "build_edge_mats.emat");
 
 	/* Edge matrices are built from the root nodes, add all roots with
@@ -730,7 +735,7 @@ static EMat *build_edge_mats(const MVertSkin *vs,
 		}
 	}
 
-	while (!BLI_stack_empty(stack)) {
+	while (!BLI_stack_is_empty(stack)) {
 		build_emats_stack(stack, visited_e, emat, emap, medge, vs, mvert);
 	}
 
@@ -749,12 +754,15 @@ static EMat *build_edge_mats(const MVertSkin *vs,
  * having any special cases for dealing with sharing a frame between
  * two hulls.) */
 static int calc_edge_subdivisions(const MVert *mvert, const MVertSkin *nodes,
-                                  const MEdge *e, int *degree)
+                                  const MEdge *e, const int *degree)
 {
+	/* prevent memory errors [#38003] */
+#define NUM_SUBDIVISIONS_MAX 128
+
 	const MVertSkin *evs[2] = {&nodes[e->v1], &nodes[e->v2]};
-	float edge_len, avg[2];
-	int v1_branch = degree[e->v1] > 2;
-	int v2_branch = degree[e->v2] > 2;
+	float avg_radius;
+	const bool v1_branch = degree[e->v1] > 2;
+	const bool v2_branch = degree[e->v2] > 2;
 	int num_subdivisions;
 
 	/* If either end is a branch node marked 'loose', don't subdivide
@@ -768,11 +776,23 @@ static int calc_edge_subdivisions(const MVert *mvert, const MVertSkin *nodes,
 			return 0;
 	}
 
-	edge_len = len_v3v3(mvert[e->v1].co, mvert[e->v2].co);
+	avg_radius = half_v2(evs[0]->radius) + half_v2(evs[1]->radius);
 
-	avg[0] = half_v2(evs[0]->radius);
-	avg[1] = half_v2(evs[1]->radius);
-	num_subdivisions = (int)((float)edge_len / (avg[0] + avg[1]));
+	if (avg_radius != 0.0f) {
+		/* possible (but unlikely) that we overflow INT_MAX */
+		float num_subdivisions_fl;
+		const float edge_len = len_v3v3(mvert[e->v1].co, mvert[e->v2].co);
+		num_subdivisions_fl = (edge_len / avg_radius);
+		if (num_subdivisions_fl < NUM_SUBDIVISIONS_MAX) {
+			num_subdivisions = (int)num_subdivisions_fl;
+		}
+		else {
+			num_subdivisions = NUM_SUBDIVISIONS_MAX;
+		}
+	}
+	else {
+		num_subdivisions = 0;
+	}
 
 	/* If both ends are branch nodes, two intermediate nodes are
 	 * required */
@@ -780,6 +800,8 @@ static int calc_edge_subdivisions(const MVert *mvert, const MVertSkin *nodes,
 		num_subdivisions = 2;
 
 	return num_subdivisions;
+
+#undef NUM_SUBDIVISIONS_MAX
 }
 
 /* Take a DerivedMesh and subdivide its edges to keep skin nodes
@@ -866,7 +888,7 @@ static DerivedMesh *subdivide_base(DerivedMesh *orig)
 				if (vg) {
 					vg->def_nr = dv1->dw[j].def_nr;
 					vg->w1 = dv1->dw[j].weight;
-					vg->w1 = dv2->dw[k].weight;
+					vg->w2 = dv2->dw[k].weight;
 				}
 			}
 		}
@@ -930,7 +952,6 @@ static void add_poly(SkinOutput *so,
                      BMVert *v4)
 {
 	BMVert *verts[4] = {v1, v2, v3, v4};
-	BMEdge *edges[4];
 	BMFace *f;
 	
 	BLI_assert(v1 != v2 && v1 != v3 && v1 != v4);
@@ -938,18 +959,7 @@ static void add_poly(SkinOutput *so,
 	BLI_assert(v3 != v4);
 	BLI_assert(v1 && v2 && v3);
 
-	edges[0] = BM_edge_create(so->bm, v1, v2, NULL, TRUE);
-	edges[1] = BM_edge_create(so->bm, v2, v3, NULL, TRUE);
-	if (v4) {
-		edges[2] = BM_edge_create(so->bm, v3, v4, NULL, TRUE);
-		edges[3] = BM_edge_create(so->bm, v4, v1, NULL, TRUE);
-	}
-	else {
-		edges[2] = BM_edge_create(so->bm, v3, v1, NULL, TRUE);
-		edges[3] = NULL;
-	}
-
-	f = BM_face_create(so->bm, verts, edges, v4 ? 4 : 3, TRUE);
+	f = BM_face_create_verts(so->bm, verts, v4 ? 4 : 3, NULL, BM_CREATE_NO_DOUBLE, true);
 	if (so->smd->flag & MOD_SKIN_SMOOTH_SHADING)
 		BM_elem_flag_enable(f, BM_ELEM_SMOOTH);
 	f->mat_nr = so->mat_nr;
@@ -957,12 +967,12 @@ static void add_poly(SkinOutput *so,
 
 static void connect_frames(SkinOutput *so,
                            BMVert *frame1[4],
-                           BMVert *frame2[4])
+BMVert *frame2[4])
 {
 	BMVert *q[4][4] = {{frame2[0], frame2[1], frame1[1], frame1[0]},
-					   {frame2[1], frame2[2], frame1[2], frame1[1]},
-					   {frame2[2], frame2[3], frame1[3], frame1[2]},
-					   {frame2[3], frame2[0], frame1[0], frame1[3]}};
+	                   {frame2[1], frame2[2], frame1[2], frame1[1]},
+	                   {frame2[2], frame2[3], frame1[3], frame1[2]},
+	                   {frame2[3], frame2[0], frame1[0], frame1[3]}};
 	float p[3], no[3];
 	int i, swap;
 
@@ -993,7 +1003,7 @@ static void output_frames(BMesh *bm,
 		f = &sn->frames[i];
 		for (j = 0; j < 4; j++) {
 			if (!f->merge[j].frame) {
-				BMVert *v = f->verts[j] = BM_vert_create(bm, f->co[j], NULL);
+				BMVert *v = f->verts[j] = BM_vert_create(bm, f->co[j], NULL, BM_CREATE_NOP);
 
 				if (input_dvert) {
 					MDeformVert *dv;
@@ -1029,20 +1039,20 @@ static int isect_ray_poly(const float ray_start[3],
 	BMVert *v, *v_first = NULL, *v_prev = NULL;
 	BMIter iter;
 	float best_dist = FLT_MAX;
-	int hit = 0;
+	bool hit = false;
 	
 	BM_ITER_ELEM (v, &iter, f, BM_VERTS_OF_FACE) {
 		if (!v_first)
 			v_first = v;
 		else if (v_prev != v_first) {
 			float dist;
-			int curhit;
+			bool curhit;
 			
 			curhit = isect_ray_tri_v3(ray_start, ray_dir,
 			                          v_first->co, v_prev->co, v->co,
 			                          &dist, NULL);
 			if (curhit && dist < best_dist) {
-				hit = TRUE;
+				hit = true;
 				best_dist = dist;
 			}
 		}
@@ -1077,32 +1087,36 @@ static BMFace *collapse_face_corners(BMesh *bm, BMFace *f, int n,
 		BMOperator op;
 		BMIter iter;
 		int i;
+		BMOpSlot *slot_targetmap;
 
 		shortest_edge = BM_face_find_shortest_loop(f)->e;
 		BMO_op_initf(bm, &op, (BMO_FLAG_DEFAULTS & ~BMO_FLAG_RESPECT_HIDE), "weld_verts");
 
+		slot_targetmap = BMO_slot_get(op.slots_in, "targetmap");
+
 		/* Note: could probably calculate merges in one go to be
 		 * faster */
 
 		v_safe = shortest_edge->v1;
 		v_merge = shortest_edge->v2;
 		mid_v3_v3v3(v_safe->co, v_safe->co, v_merge->co);
-		BMO_slot_map_ptr_insert(bm, &op, "targetmap", v_merge, v_safe);
+		BMO_slot_map_elem_insert(&op, slot_targetmap, v_merge, v_safe);
 		BMO_op_exec(bm, &op);
 		BMO_op_finish(bm, &op);
 
 		/* Find the new face */
 		f = NULL;
 		BM_ITER_ELEM (vf, &iter, v_safe, BM_FACES_OF_VERT) {
-			int wrong_face = FALSE;
+			bool wrong_face = false;
 			
 			for (i = 0; i < orig_len; i++) {
-				if (orig_verts[i] == v_merge)
+				if (orig_verts[i] == v_merge) {
 					orig_verts[i] = NULL;
+				}
 				else if (orig_verts[i] &&
 				         !BM_vert_in_face(vf, orig_verts[i]))
 				{
-					wrong_face = TRUE;
+					wrong_face = true;
 					break;
 				}
 			}
@@ -1213,11 +1227,12 @@ static void skin_fix_hole_no_good_verts(BMesh *bm, Frame *frame, BMFace *split_f
 	BMOIter oiter;
 	BMOperator op;
 	int i, best_order[4];
+	BMOpSlot *slot_targetmap;
 
 	BLI_assert(split_face->len >= 3);
 
 	/* Extrude the split face */
-	BM_mesh_elem_hflag_disable_all(bm, BM_FACE, BM_ELEM_TAG, FALSE);
+	BM_mesh_elem_hflag_disable_all(bm, BM_FACE, BM_ELEM_TAG, false);
 	BM_elem_flag_enable(split_face, BM_ELEM_TAG);
 	BMO_op_initf(bm, &op, (BMO_FLAG_DEFAULTS & ~BMO_FLAG_RESPECT_HIDE),
 	             "extrude_discrete_faces faces=%hf", BM_ELEM_TAG);
@@ -1226,7 +1241,7 @@ static void skin_fix_hole_no_good_verts(BMesh *bm, Frame *frame, BMFace *split_f
 	/* Update split face (should only be one new face created
 	 * during extrusion) */
 	split_face = NULL;
-	BMO_ITER (f, &oiter, bm, &op, "faceout", BM_FACE) {
+	BMO_ITER (f, &oiter, op.slots_out, "faces.out", BM_FACE) {
 		BLI_assert(!split_face);
 		split_face = f;
 	}
@@ -1240,12 +1255,12 @@ static void skin_fix_hole_no_good_verts(BMesh *bm, Frame *frame, BMFace *split_f
 		 * face is a triangle */
 		longest_edge = BM_face_find_longest_loop(split_face)->e;
 		
-		BM_mesh_elem_hflag_disable_all(bm, BM_EDGE, BM_ELEM_TAG, FALSE);
+		BM_mesh_elem_hflag_disable_all(bm, BM_EDGE, BM_ELEM_TAG, false);
 		BM_elem_flag_enable(longest_edge, BM_ELEM_TAG);
 
 		BMO_op_callf(bm, BMO_FLAG_DEFAULTS,
-		             "subdivide_edges edges=%he numcuts=%i quadcornertype=%i",
-		             BM_ELEM_TAG, 1, SUBD_STRAIGHT_CUT);
+		             "subdivide_edges edges=%he cuts=%i quad_corner_type=%i",
+		             BM_ELEM_TAG, 1, SUBD_CORNER_STRAIGHT_CUT);
 	}
 	else if (split_face->len > 4) {
 		/* Maintain a dynamic vert array containing the split_face's
@@ -1271,16 +1286,17 @@ static void skin_fix_hole_no_good_verts(BMesh *bm, Frame *frame, BMFace *split_f
 		return;
 
 	/* Get split face's verts */
-	BM_iter_as_array(bm, BM_VERTS_OF_FACE, split_face, (void **)verts, 4);
+	// BM_iter_as_array(bm, BM_VERTS_OF_FACE, split_face, (void **)verts, 4);
+	BM_face_as_array_vert_quad(split_face, verts);
 	skin_choose_quad_bridge_order(verts, frame->verts, best_order);
 
 	/* Delete split face and merge */
 	BM_face_kill(bm, split_face);
 	BMO_op_init(bm, &op, (BMO_FLAG_DEFAULTS & ~BMO_FLAG_RESPECT_HIDE),
 	            "weld_verts");
+	slot_targetmap = BMO_slot_get(op.slots_in, "targetmap");
 	for (i = 0; i < 4; i++) {
-		BMO_slot_map_ptr_insert(bm, &op, "targetmap",
-		                        verts[i], frame->verts[best_order[i]]);
+		BMO_slot_map_elem_insert(&op, slot_targetmap, verts[i], frame->verts[best_order[i]]);
 	}
 	BMO_op_exec(bm, &op);
 	BMO_op_finish(bm, &op);
@@ -1302,21 +1318,26 @@ static void skin_hole_detach_partially_attached_frame(BMesh *bm, Frame *frame)
 	/* Detach everything */
 	for (i = 0; i < totattached; i++) {
 		BMVert **av = &frame->verts[attached[i]];
-		(*av) = BM_vert_create(bm, (*av)->co, *av);
+		(*av) = BM_vert_create(bm, (*av)->co, *av, BM_CREATE_NOP);
 	}
 }
 
 
-static void quad_from_tris(BMesh *bm, BMEdge *e, BMFace *adj[2], BMVert *ndx[4])
+static void quad_from_tris(BMEdge *e, BMFace *adj[2], BMVert *ndx[4])
 {
 	BMVert *tri[2][3];
 	BMVert *opp = NULL;
 	int i, j;
 
 	BLI_assert(adj[0]->len == 3 && adj[1]->len == 3);
-	
+
+#if 0
 	BM_iter_as_array(bm, BM_VERTS_OF_FACE, adj[0], (void **)tri[0], 3);
 	BM_iter_as_array(bm, BM_VERTS_OF_FACE, adj[1], (void **)tri[1], 3);
+#else
+	BM_face_as_array_vert_tri(adj[0], tri[0]);
+	BM_face_as_array_vert_tri(adj[1], tri[1]);
+#endif
 
 	/* Find what the second tri has that the first doesn't */
 	for (i = 0; i < 3; i++) {
@@ -1347,29 +1368,11 @@ static void add_quad_from_tris(SkinOutput *so, BMEdge *e, BMFace *adj[2])
 {
 	BMVert *quad[4];
 
-	quad_from_tris(so->bm, e, adj, quad);
+	quad_from_tris(e, adj, quad);
 
 	add_poly(so, quad[0], quad[1], quad[2], quad[3]);
 }
 
-/* Returns the number of faces that are adjacent to both f1 and f2 */
-static int BM_face_share_face_count(BMFace *f1, BMFace *f2)
-{
-	BMIter iter1, iter2;
-	BMEdge *e;
-	BMFace *f;
-	int count = 0;
-
-	BM_ITER_ELEM (e, &iter1, f1, BM_EDGES_OF_FACE) {
-		BM_ITER_ELEM (f, &iter2, e, BM_FACES_OF_EDGE) {
-			if (f != f1 && f != f2 && BM_face_share_edge_count(f, f2))
-				count++;
-		}
-	}
-
-	return count;
-}
-
 static void hull_merge_triangles(SkinOutput *so, const SkinModifierData *smd)
 {
 	BMIter iter;
@@ -1379,7 +1382,7 @@ static void hull_merge_triangles(SkinOutput *so, const SkinModifierData *smd)
 
 	heap = BLI_heap_new();
 
-	BM_mesh_elem_hflag_disable_all(so->bm, BM_FACE, BM_ELEM_TAG, FALSE);
+	BM_mesh_elem_hflag_disable_all(so->bm, BM_FACE, BM_ELEM_TAG, false);
 
 	/* Build heap */
 	BM_ITER_MESH (e, &iter, so->bm, BM_EDGES_OF_MESH) {
@@ -1392,7 +1395,7 @@ static void hull_merge_triangles(SkinOutput *so, const SkinModifierData *smd)
 
 				/* Construct quad using the two triangles adjacent to
 				 * the edge */
-				quad_from_tris(so->bm, e, adj, quad);
+				quad_from_tris(e, adj, quad);
 
 				/* Calculate a score for the quad, higher score for
 				 * triangles being closer to coplanar */
@@ -1422,7 +1425,7 @@ static void hull_merge_triangles(SkinOutput *so, const SkinModifierData *smd)
 		}
 	}
 
-	while (!BLI_heap_empty(heap)) {
+	while (!BLI_heap_is_empty(heap)) {
 		BMFace *adj[2];
 
 		e = BLI_heap_popmin(heap);
@@ -1432,7 +1435,7 @@ static void hull_merge_triangles(SkinOutput *so, const SkinModifierData *smd)
 			 * share a border with another face, output as a quad */
 			if (!BM_elem_flag_test(adj[0], BM_ELEM_TAG) &&
 			    !BM_elem_flag_test(adj[1], BM_ELEM_TAG) &&
-			    !BM_face_share_face_count(adj[0], adj[1]))
+			    !BM_face_share_face_check(adj[0], adj[1]))
 			{
 				add_quad_from_tris(so, e, adj);
 				BM_elem_flag_enable(adj[0], BM_ELEM_TAG);
@@ -1442,11 +1445,10 @@ static void hull_merge_triangles(SkinOutput *so, const SkinModifierData *smd)
 		}
 	}
 
-	BMO_op_callf(so->bm, BMO_FLAG_DEFAULTS,
-	             "delete geom=%hef context=%i",
-	             BM_ELEM_TAG, DEL_ONLYTAGGED);
-
 	BLI_heap_free(heap, NULL);
+
+	BM_mesh_delete_hflag_tagged(so->bm, BM_ELEM_TAG, BM_EDGE | BM_FACE);
+
 }
 
 static void skin_merge_close_frame_verts(SkinNode *skin_nodes, int totvert,
@@ -1544,23 +1546,23 @@ static void skin_output_end_nodes(SkinOutput *so, SkinNode *skin_nodes,
 		if (sn->flag & CAP_START) {
 			if (sn->flag & ROOT) {
 				add_poly(so,
-						 sn->frames[0].verts[0],
-						 sn->frames[0].verts[1],
-						 sn->frames[0].verts[2],
-						 sn->frames[0].verts[3]);
+				         sn->frames[0].verts[0],
+				         sn->frames[0].verts[1],
+				         sn->frames[0].verts[2],
+				         sn->frames[0].verts[3]);
 			}
 			else {
 				add_poly(so,
-						 sn->frames[0].verts[3],
-						 sn->frames[0].verts[2],
-						 sn->frames[0].verts[1],
-						 sn->frames[0].verts[0]);
+				         sn->frames[0].verts[3],
+				         sn->frames[0].verts[2],
+				         sn->frames[0].verts[1],
+				         sn->frames[0].verts[0]);
 			}
 		}
 		if (sn->flag & CAP_END) {
 			add_poly(so,
 			         sn->frames[1].verts[0],
-			         sn->frames[1].verts[1],
+			        sn->frames[1].verts[1],
 			         sn->frames[1].verts[2],
 			         sn->frames[1].verts[3]);
 		}
@@ -1614,7 +1616,7 @@ static void skin_smooth_hulls(BMesh *bm, SkinNode *skin_nodes,
 		return;
 
 	/* Mark all frame vertices */
-	BM_mesh_elem_hflag_disable_all(bm, BM_VERT, BM_ELEM_TAG, FALSE);
+	BM_mesh_elem_hflag_disable_all(bm, BM_VERT, BM_ELEM_TAG, false);
 	for (i = 0; i < totvert; i++) {
 		for (j = 0; j < skin_nodes[i].totframe; j++) {
 			Frame *frame = &skin_nodes[i].frames[j];
@@ -1664,12 +1666,13 @@ static void skin_smooth_hulls(BMesh *bm, SkinNode *skin_nodes,
 	BM_data_layer_free_n(bm, &bm->vdata, CD_SHAPEKEY, skey);
 }
 
-/* Returns TRUE if all hulls are successfully built, FALSE otherwise */
-static int skin_output_branch_hulls(SkinOutput *so, SkinNode *skin_nodes,
-                                    int totvert, const MeshElemMap *emap,
-                                    const MEdge *medge)
+/* Returns true if all hulls are successfully built, false otherwise */
+static bool skin_output_branch_hulls(SkinOutput *so, SkinNode *skin_nodes,
+                                     int totvert, const MeshElemMap *emap,
+                                     const MEdge *medge)
 {
-	int result = TRUE, v;
+	bool result = true;
+	int v;
 	
 	for (v = 0; v < totvert; v++) {
 		SkinNode *sn = &skin_nodes[v];
@@ -1683,7 +1686,7 @@ static int skin_output_branch_hulls(SkinOutput *so, SkinNode *skin_nodes,
 			                                  emap, medge,
 			                                  &tothullframe);
 			if (!build_hull(so, hull_frames, tothullframe))
-				result = FALSE;
+				result = false;
 
 			MEM_freeN(hull_frames);
 		}
@@ -1705,6 +1708,11 @@ static BMesh *build_skin(SkinNode *skin_nodes,
 	so.bm = BM_mesh_create(&bm_mesh_allocsize_default);
 	so.mat_nr = 0;
 	
+	/* BMESH_TODO: bumping up the stack level (see MOD_array.c) */
+	BM_mesh_elem_toolflags_ensure(so.bm);
+	BMO_push(so.bm, NULL);
+	bmesh_edit_begin(so.bm, 0);
+
 	if (input_dvert)
 		BM_data_layer_add(so.bm, &so.bm->vdata, CD_MDEFORMVERT);
 
@@ -1752,13 +1760,12 @@ static BMesh *build_skin(SkinNode *skin_nodes,
 
 static void skin_set_orig_indices(DerivedMesh *dm)
 {
-	int *orig, totpoly, i;
+	int *orig, totpoly;
 
 	totpoly = dm->getNumPolys(dm);
 	orig = CustomData_add_layer(&dm->polyData, CD_ORIGINDEX,
-	                            CD_CALLOC, 0, totpoly);
-	for (i = 0; i < totpoly; i++)
-		orig[i] = ORIGINDEX_NONE;
+	                            CD_CALLOC, NULL, totpoly);
+	fill_vn_i(orig, totpoly, ORIGINDEX_NONE);
 }
 
 /*
@@ -1770,7 +1777,6 @@ static void skin_set_orig_indices(DerivedMesh *dm)
 static DerivedMesh *base_skin(DerivedMesh *origdm,
                               SkinModifierData *smd)
 {
-	BMEditMesh fake_em;
 	DerivedMesh *result;
 	MVertSkin *nodes;
 	BMesh *bm;
@@ -1791,7 +1797,7 @@ static DerivedMesh *base_skin(DerivedMesh *origdm,
 	totvert = origdm->getNumVerts(origdm);
 	totedge = origdm->getNumEdges(origdm);
 
-	create_vert_edge_map(&emap, &emapmem, medge, totvert, totedge);
+	BKE_mesh_vert_edge_map_create(&emap, &emapmem, medge, totvert, totedge);
 
 	emat = build_edge_mats(nodes, mvert, totvert, medge, emap, totedge);
 	skin_nodes = build_frames(mvert, totvert, nodes, emap, emat);
@@ -1807,12 +1813,11 @@ static DerivedMesh *base_skin(DerivedMesh *origdm,
 	if (!bm)
 		return NULL;
 	
-	fake_em.bm = bm;
-	result = CDDM_from_BMEditMesh(&fake_em, NULL, FALSE, FALSE);
+	result = CDDM_from_bmesh(bm, false);
 	BM_mesh_free(bm);
 
 	CDDM_calc_edges(result);
-	CDDM_calc_normals(result);
+	result->dirty |= DM_DIRTY_NORMALS;
 
 	skin_set_orig_indices(result);
 
@@ -1853,22 +1858,11 @@ static void initData(ModifierData *md)
 
 static void copyData(ModifierData *md, ModifierData *target)
 {
+#if 0
 	SkinModifierData *smd = (SkinModifierData *) md;
 	SkinModifierData *tsmd = (SkinModifierData *) target;
-
-	*tsmd = *smd;
-}
-
-static DerivedMesh *applyModifierEM(ModifierData *md,
-                                    Object *UNUSED(ob),
-                                    BMEditMesh *UNUSED(em),
-                                    DerivedMesh *dm)
-{
-	DerivedMesh *result;
-
-	if (!(result = final_skin((SkinModifierData *)md, dm)))
-		return dm;
-	return result;
+#endif
+	modifier_copyData_generic(md, target);
 }
 
 static DerivedMesh *applyModifier(ModifierData *md,
@@ -1902,7 +1896,7 @@ ModifierTypeInfo modifierType_Skin = {
 	/* deformVertsEM */     NULL,
 	/* deformMatricesEM */  NULL,
 	/* applyModifier */     applyModifier,
-	/* applyModifierEM */   applyModifierEM,
+	/* applyModifierEM */   NULL,
 	/* initData */          initData,
 	/* requiredDataMask */  requiredDataMask,
 	/* freeData */          NULL,
