@@ -25,15 +25,15 @@
  *  \ingroup pygen
  */
 
-
 #include <Python.h>
 
-#include "idprop_py_api.h"
 #include "MEM_guardedalloc.h"
 
-#include "BLI_string.h"
 #include "BLI_utildefines.h"
 
+#include "idprop_py_api.h"
+
+
 #include "BKE_idprop.h"
 
 
@@ -43,43 +43,39 @@
 #include "py_capi_utils.h"
 #endif
 
-extern PyTypeObject BPy_IDArray_Type;
-extern PyTypeObject BPy_IDGroup_Iter_Type;
-extern PyTypeObject BPy_IDGroup_Type;
-
 /*********************** ID Property Main Wrapper Stuff ***************/
 
 /* ----------------------------------------------------------------------------
  * static conversion functions to avoid duplicate code, no type checking.
  */
 
-static PyObject *idprop_py_from_idp_string(IDProperty *prop)
+static PyObject *idprop_py_from_idp_string(const IDProperty *prop)
 {
 	if (prop->subtype == IDP_STRING_SUB_BYTE) {
-		return PyBytes_FromStringAndSize(IDP_Array(prop), prop->len);
+		return PyBytes_FromStringAndSize(IDP_String(prop), prop->len);
 	}
 	else {
 #ifdef USE_STRING_COERCE
 		return PyC_UnicodeFromByteAndSize(IDP_Array(prop), prop->len - 1);
 #else
-		return PyUnicode_FromStringAndSize(IDP_Array(prop), prop->len - 1);
+		return PyUnicode_FromStringAndSize(IDP_String(prop), prop->len - 1);
 #endif
 	}
 }
 
-static PyObject *idprop_py_from_idp_int(IDProperty *prop)
+static PyObject *idprop_py_from_idp_int(const IDProperty *prop)
 {
-	return PyLong_FromLong((long)prop->data.val);
+	return PyLong_FromLong((long)IDP_Int(prop));
 }
 
-static PyObject *idprop_py_from_idp_float(IDProperty *prop)
+static PyObject *idprop_py_from_idp_float(const IDProperty *prop)
 {
-	return PyFloat_FromDouble((double)(*(float *)(&prop->data.val)));
+	return PyFloat_FromDouble((double)IDP_Float(prop));
 }
 
-static PyObject *idprop_py_from_idp_double(IDProperty *prop)
+static PyObject *idprop_py_from_idp_double(const IDProperty *prop)
 {
-	return PyFloat_FromDouble((*(double *)(&prop->data.val)));
+	return PyFloat_FromDouble(IDP_Double(prop));
 }
 
 static PyObject *idprop_py_from_idp_group(ID *id, IDProperty *prop, IDProperty *parent)
@@ -193,7 +189,7 @@ static int BPy_IDGroup_SetData(BPy_IDProperty *self, IDProperty *prop, PyObject
 				PyErr_SetString(PyExc_TypeError, "expected an int type");
 				return -1;
 			}
-			prop->data.val = ivalue;
+			IDP_Int(prop) = ivalue;
 			break;
 		}
 		case IDP_FLOAT:
@@ -203,7 +199,7 @@ static int BPy_IDGroup_SetData(BPy_IDProperty *self, IDProperty *prop, PyObject
 				PyErr_SetString(PyExc_TypeError, "expected a float");
 				return -1;
 			}
-			*(float *)&self->prop->data.val = fvalue;
+			IDP_Float(self->prop) = fvalue;
 			break;
 		}
 		case IDP_DOUBLE:
@@ -213,7 +209,7 @@ static int BPy_IDGroup_SetData(BPy_IDProperty *self, IDProperty *prop, PyObject
 				PyErr_SetString(PyExc_TypeError, "expected a float");
 				return -1;
 			}
-			*(double *)&self->prop->data.val = dvalue;
+			IDP_Double(self->prop) = dvalue;
 			break;
 		}
 		default:
@@ -253,7 +249,7 @@ static int BPy_IDGroup_SetName(BPy_IDProperty *self, PyObject *value, void *UNUS
 #if 0
 static PyObject *BPy_IDGroup_GetType(BPy_IDProperty *self)
 {
-	return PyLong_FromSsize_t(self->prop->type);
+	return PyLong_FromLong(self->prop->type);
 }
 #endif
 
@@ -300,60 +296,70 @@ static PyObject *BPy_IDGroup_Map_GetItem(BPy_IDProperty *self, PyObject *item)
 }
 
 /* returns NULL on success, error string on failure */
-static int idp_sequence_type(PyObject *seq)
+static char idp_sequence_type(PyObject *seq_fast)
 {
 	PyObject *item;
-	int type = IDP_INT;
+	char type = IDP_INT;
 
-	Py_ssize_t i, len = PySequence_Size(seq);
+	Py_ssize_t i, len = PySequence_Fast_GET_SIZE(seq_fast);
 	for (i = 0; i < len; i++) {
-		item = PySequence_GetItem(seq, i);
+		item = PySequence_Fast_GET_ITEM(seq_fast, i);
 		if (PyFloat_Check(item)) {
 			if (type == IDP_IDPARRAY) { /* mixed dict/int */
-				Py_DECREF(item);
 				return -1;
 			}
 			type = IDP_DOUBLE;
 		}
 		else if (PyLong_Check(item)) {
 			if (type == IDP_IDPARRAY) { /* mixed dict/int */
-				Py_DECREF(item);
 				return -1;
 			}
 		}
 		else if (PyMapping_Check(item)) {
 			if (i != 0 && (type != IDP_IDPARRAY)) { /* mixed dict/int */
-				Py_DECREF(item);
 				return -1;
 			}
 			type = IDP_IDPARRAY;
 		}
 		else {
-			Py_XDECREF(item);
 			return -1;
 		}
-
-		Py_DECREF(item);
 	}
 
 	return type;
 }
 
-/* note: group can be a pointer array or a group.
- * assume we already checked key is a string. */
-const char *BPy_IDProperty_Map_ValidateAndCreate(PyObject *name_obj, IDProperty *group, PyObject *ob)
+/**
+ * \note group can be a pointer array or a group.
+ * assume we already checked key is a string.
+ *
+ * \return success.
+ */
+bool BPy_IDProperty_Map_ValidateAndCreate(PyObject *name_obj, IDProperty *group, PyObject *ob)
 {
 	IDProperty *prop = NULL;
 	IDPropertyTemplate val = {0};
 
-	const char *name = "";
+	const char *name;
 
 	if (name_obj) {
 		Py_ssize_t name_size;
 		name = _PyUnicode_AsStringAndSize(name_obj, &name_size);
-		if (name_size > MAX_IDPROP_NAME) {
-			return "the length of IDProperty names is limited to 63 characters";
+
+		if (name == NULL) {
+			PyErr_Format(PyExc_KeyError,
+			             "invalid id-property key, expected a string, not a %.200s",
+			             Py_TYPE(name_obj)->tp_name);
+			return false;
 		}
+
+		if (name_size > MAX_IDPROP_NAME) {
+			PyErr_SetString(PyExc_KeyError, "the length of IDProperty names is limited to 63 characters");
+			return false;
+		}
+	}
+	else {
+		name = "";
 	}
 
 	if (PyFloat_Check(ob)) {
@@ -361,7 +367,10 @@ const char *BPy_IDProperty_Map_ValidateAndCreate(PyObject *name_obj, IDProperty
 		prop = IDP_New(IDP_DOUBLE, &val, name);
 	}
 	else if (PyLong_Check(ob)) {
-		val.i = (int) PyLong_AsSsize_t(ob);
+		val.i = _PyLong_AsInt(ob);
+		if (val.i == -1 && PyErr_Occurred()) {
+			return false;
+		}
 		prop = IDP_New(IDP_INT, &val, name);
 	}
 	else if (PyUnicode_Check(ob)) {
@@ -386,48 +395,77 @@ const char *BPy_IDProperty_Map_ValidateAndCreate(PyObject *name_obj, IDProperty
 		//prop->subtype = IDP_STRING_SUB_BYTE;
 	}
 	else if (PySequence_Check(ob)) {
+		PyObject *ob_seq_fast = PySequence_Fast(ob, "py -> idprop");
 		PyObject *item;
 		int i;
 
-		if ((val.array.type = idp_sequence_type(ob)) == -1)
-			return "only floats, ints and dicts are allowed in ID property arrays";
+		if (ob_seq_fast == NULL) {
+			return false;
+		}
+
+		if ((val.array.type = idp_sequence_type(ob_seq_fast)) == (char)-1) {
+			Py_DECREF(ob_seq_fast);
+			PyErr_SetString(PyExc_TypeError, "only floats, ints and dicts are allowed in ID property arrays");
+			return false;
+		}
 
 		/* validate sequence and derive type.
 		 * we assume IDP_INT unless we hit a float
 		 * number; then we assume it's */
 
-		val.array.len = PySequence_Size(ob);
+		val.array.len = PySequence_Fast_GET_SIZE(ob_seq_fast);
 
 		switch (val.array.type) {
 			case IDP_DOUBLE:
+			{
+				double *prop_data;
+
 				prop = IDP_New(IDP_ARRAY, &val, name);
+				prop_data = IDP_Array(prop);
 				for (i = 0; i < val.array.len; i++) {
-					item = PySequence_GetItem(ob, i);
-					((double *)IDP_Array(prop))[i] = (float)PyFloat_AsDouble(item);
-					Py_DECREF(item);
+					item = PySequence_Fast_GET_ITEM(ob_seq_fast, i);
+					if (((prop_data[i] = PyFloat_AsDouble(item)) == -1.0) && PyErr_Occurred()) {
+						Py_DECREF(ob_seq_fast);
+						return false;
+					}
 				}
 				break;
+			}
 			case IDP_INT:
+			{
+				int *prop_data;
 				prop = IDP_New(IDP_ARRAY, &val, name);
+				prop_data = IDP_Array(prop);
 				for (i = 0; i < val.array.len; i++) {
-					item = PySequence_GetItem(ob, i);
-					((int *)IDP_Array(prop))[i] = (int)PyLong_AsSsize_t(item);
-					Py_DECREF(item);
+					item = PySequence_Fast_GET_ITEM(ob_seq_fast, i);
+					if (((prop_data[i] = _PyLong_AsInt(item)) == -1) && PyErr_Occurred()) {
+						Py_DECREF(ob_seq_fast);
+						return false;
+					}
 				}
 				break;
+			}
 			case IDP_IDPARRAY:
+			{
 				prop = IDP_NewIDPArray(name);
 				for (i = 0; i < val.array.len; i++) {
-					const char *error;
-					item = PySequence_GetItem(ob, i);
-					error = BPy_IDProperty_Map_ValidateAndCreate(NULL, prop, item);
-					Py_DECREF(item);
+					item = PySequence_Fast_GET_ITEM(ob_seq_fast, i);
 
-					if (error)
-						return error;
+					if (BPy_IDProperty_Map_ValidateAndCreate(NULL, prop, item) == false) {
+						Py_DECREF(ob_seq_fast);
+						return false;
+					}
 				}
 				break;
+			}
+			default:
+				/* should never happen */
+				Py_DECREF(ob_seq_fast);
+				PyErr_SetString(PyExc_RuntimeError, "internal error with idp array.type");
+				return false;
 		}
+
+		Py_DECREF(ob_seq_fast);
 	}
 	else if (PyMapping_Check(ob)) {
 		PyObject *keys, *vals, *key, *pval;
@@ -443,23 +481,15 @@ const char *BPy_IDProperty_Map_ValidateAndCreate(PyObject *name_obj, IDProperty
 		for (i = 0; i < len; i++) {
 			key = PySequence_GetItem(keys, i);
 			pval = PySequence_GetItem(vals, i);
-			if (!PyUnicode_Check(key)) {
+			if (BPy_IDProperty_Map_ValidateAndCreate(key, prop, pval) == false) {
 				IDP_FreeProperty(prop);
 				MEM_freeN(prop);
 				Py_XDECREF(keys);
 				Py_XDECREF(vals);
 				Py_XDECREF(key);
 				Py_XDECREF(pval);
-				return "invalid element in subgroup dict template!";
-			}
-			if (BPy_IDProperty_Map_ValidateAndCreate(key, prop, pval)) {
-				IDP_FreeProperty(prop);
-				MEM_freeN(prop);
-				Py_XDECREF(keys);
-				Py_XDECREF(vals);
-				Py_XDECREF(key);
-				Py_XDECREF(pval);
-				return "invalid element in subgroup dict template!";
+				/* error is already set */
+				return false;
 			}
 			Py_XDECREF(key);
 			Py_XDECREF(pval);
@@ -467,18 +497,42 @@ const char *BPy_IDProperty_Map_ValidateAndCreate(PyObject *name_obj, IDProperty
 		Py_XDECREF(keys);
 		Py_XDECREF(vals);
 	}
-	else return "invalid property value";
+	else {
+		PyErr_Format(PyExc_TypeError,
+		             "invalid id-property type %.200s not supported",
+		             Py_TYPE(ob)->tp_name);
+		return false;
+	}
 
 	if (group->type == IDP_IDPARRAY) {
 		IDP_AppendArray(group, prop);
-		// IDP_FreeProperty(item); // IDP_AppendArray does a shallow copy (memcpy), only free memory
+		// IDP_FreeProperty(item);  /* IDP_AppendArray does a shallow copy (memcpy), only free memory */
 		MEM_freeN(prop);
 	}
 	else {
-		IDP_ReplaceInGroup(group, prop);
+		IDProperty *prop_exist;
+
+		/* avoid freeing when types match in case they are referenced by the UI, see: T37073
+		 * obviously this isn't a complete solution, but helps for common cases. */
+		prop_exist = IDP_GetPropertyFromGroup(group, prop->name);
+		if ((prop_exist != NULL) &&
+		    (prop_exist->type == prop->type) &&
+		    (prop_exist->subtype == prop->subtype))
+		{
+			/* Preserve prev/next links!!! See T42593. */
+			prop->prev = prop_exist->prev;
+			prop->next = prop_exist->next;
+
+			IDP_FreeProperty(prop_exist);
+			*prop_exist = *prop;
+			MEM_freeN(prop);
+		}
+		else {
+			IDP_ReplaceInGroup_ex(group, prop, prop_exist);
+		}
 	}
 
-	return NULL;
+	return true;
 }
 
 int BPy_Wrap_SetMapItem(IDProperty *prop, PyObject *key, PyObject *val)
@@ -489,11 +543,19 @@ int BPy_Wrap_SetMapItem(IDProperty *prop, PyObject *key, PyObject *val)
 	}
 
 	if (val == NULL) { /* del idprop[key] */
-		IDProperty *pkey = IDP_GetPropertyFromGroup(prop, _PyUnicode_AsString(key));
+		IDProperty *pkey;
+		const char *name = _PyUnicode_AsString(key);
+
+		if (name == NULL) {
+			PyErr_Format(PyExc_KeyError,
+			             "expected a string, not %.200s",
+			             Py_TYPE(key)->tp_name);
+			return -1;
+		}
+
+		pkey = IDP_GetPropertyFromGroup(prop, name);
 		if (pkey) {
-			IDP_RemFromGroup(prop, pkey);
-			IDP_FreeProperty(pkey);
-			MEM_freeN(pkey);
+			IDP_FreeFromGroup(prop, pkey);
 			return 0;
 		}
 		else {
@@ -502,16 +564,10 @@ int BPy_Wrap_SetMapItem(IDProperty *prop, PyObject *key, PyObject *val)
 		}
 	}
 	else {
-		const char *err;
+		bool ok;
 
-		if (!PyUnicode_Check(key)) {
-			PyErr_SetString(PyExc_TypeError, "only strings are allowed as subgroup keys");
-			return -1;
-		}
-
-		err = BPy_IDProperty_Map_ValidateAndCreate(key, prop, val);
-		if (err) {
-			PyErr_SetString(PyExc_KeyError, err);
+		ok = BPy_IDProperty_Map_ValidateAndCreate(key, prop, val);
+		if (ok == false) {
 			return -1;
 		}
 
@@ -561,7 +617,7 @@ static PyObject *BPy_IDGroup_MapDataToPy(IDProperty *prop)
 			switch (prop->subtype) {
 				case IDP_FLOAT:
 				{
-					float *array = (float *)IDP_Array(prop);
+					const float *array = (float *)IDP_Array(prop);
 					for (i = 0; i < prop->len; i++) {
 						PyList_SET_ITEM(seq, i, PyFloat_FromDouble(array[i]));
 					}
@@ -569,7 +625,7 @@ static PyObject *BPy_IDGroup_MapDataToPy(IDProperty *prop)
 				}
 				case IDP_DOUBLE:
 				{
-					double *array = (double *)IDP_Array(prop);
+					const double *array = (double *)IDP_Array(prop);
 					for (i = 0; i < prop->len; i++) {
 						PyList_SET_ITEM(seq, i, PyFloat_FromDouble(array[i]));
 					}
@@ -577,7 +633,7 @@ static PyObject *BPy_IDGroup_MapDataToPy(IDProperty *prop)
 				}
 				case IDP_INT:
 				{
-					int *array = (int *)IDP_Array(prop);
+					const int *array = (int *)IDP_Array(prop);
 					for (i = 0; i < prop->len; i++) {
 						PyList_SET_ITEM(seq, i, PyLong_FromLong(array[i]));
 					}
@@ -665,7 +721,7 @@ static PyObject *BPy_IDGroup_Pop(BPy_IDProperty *self, PyObject *value)
 			return NULL;
 		}
 
-		IDP_RemFromGroup(self->prop, idprop);
+		IDP_RemoveFromGroup(self->prop, idprop);
 		return pyform;
 	}
 
@@ -802,17 +858,28 @@ static PyObject *BPy_IDGroup_Update(BPy_IDProperty *self, PyObject *value)
 	PyObject *pkey, *pval;
 	Py_ssize_t i = 0;
 
-	if (!PyDict_Check(value)) {
+	if (BPy_IDGroup_Check(value)) {
+		BPy_IDProperty *other = (BPy_IDProperty *)value;
+		if (UNLIKELY(self->prop == other->prop)) {
+			Py_RETURN_NONE;
+		}
+
+		/* XXX, possible one is inside the other */
+		IDP_MergeGroup(self->prop, other->prop, true);
+	}
+	else if (PyDict_Check(value)) {
+		while (PyDict_Next(value, &i, &pkey, &pval)) {
+			BPy_IDGroup_Map_SetItem(self, pkey, pval);
+			if (PyErr_Occurred()) return NULL;
+		}
+	}
+	else {
 		PyErr_Format(PyExc_TypeError,
-		             "expected a dict not a %.200s",
+		             "expected a dict or an IDPropertyGroup type, not a %.200s",
 		             Py_TYPE(value)->tp_name);
 		return NULL;
 	}
 
-	while (PyDict_Next(value, &i, &pkey, &pval)) {
-		BPy_IDGroup_Map_SetItem(self, pkey, pval);
-		if (PyErr_Occurred()) return NULL;
-	}
 
 	Py_RETURN_NONE;
 }
@@ -822,12 +889,17 @@ static PyObject *BPy_IDGroup_to_dict(BPy_IDProperty *self)
 	return BPy_IDGroup_MapDataToPy(self->prop);
 }
 
+static PyObject *BPy_IDGroup_clear(BPy_IDProperty *self)
+{
+	IDP_ClearProperty(self->prop);
+	Py_RETURN_NONE;
+}
 
 /* Matches python dict.get(key, [default]) */
 static PyObject *BPy_IDGroup_Get(BPy_IDProperty *self, PyObject *args)
 {
 	IDProperty *idprop;
-	char *key;
+	const char *key;
 	PyObject *def = Py_None;
 
 	if (!PyArg_ParseTuple(args, "s|O:get", &key, &def))
@@ -861,6 +933,8 @@ static struct PyMethodDef BPy_IDGroup_methods[] = {
 	 "idprop.get(k[,d]) -> idprop[k] if k in idprop, else d.  d defaults to None"},
 	{"to_dict", (PyCFunction)BPy_IDGroup_to_dict, METH_NOARGS,
 	 "return a purely python version of the group"},
+	{"clear", (PyCFunction)BPy_IDGroup_clear, METH_NOARGS,
+	 "clear all members from this group"},
 	{NULL, NULL, 0, NULL}
 };
 
@@ -946,22 +1020,22 @@ PyTypeObject BPy_IDGroup_Type = {
 
 /********Array Wrapper********/
 
-static PyTypeObject *idp_array_py_type(BPy_IDArray *self, short *is_double)
+static PyTypeObject *idp_array_py_type(BPy_IDArray *self, bool *r_is_double)
 {
 	switch (self->prop->subtype) {
 		case IDP_FLOAT:
-			*is_double = 0;
+			*r_is_double = false;
 			return &PyFloat_Type;
 		case IDP_DOUBLE:
-			*is_double = 1;
+			*r_is_double = true;
 			return &PyFloat_Type;
 		case IDP_INT:
-			*is_double = 0;
+			*r_is_double = false;
 			return &PyLong_Type;
+		default:
+			*r_is_double = false;
+			return NULL;
 	}
-
-	*is_double = 0;
-	return NULL;
 }
 
 static PyObject *BPy_IDArray_repr(BPy_IDArray *self)
@@ -1031,10 +1105,6 @@ static PyObject *BPy_IDArray_GetItem(BPy_IDArray *self, int index)
 
 static int BPy_IDArray_SetItem(BPy_IDArray *self, int index, PyObject *value)
 {
-	int i;
-	float f;
-	double d;
-
 	if (index < 0 || index >= self->prop->len) {
 		PyErr_SetString(PyExc_RuntimeError, "index out of range!");
 		return -1;
@@ -1042,30 +1112,33 @@ static int BPy_IDArray_SetItem(BPy_IDArray *self, int index, PyObject *value)
 
 	switch (self->prop->subtype) {
 		case IDP_FLOAT:
-			f = (float)PyFloat_AsDouble(value);
+		{
+			const float f = (float)PyFloat_AsDouble(value);
 			if (f == -1 && PyErr_Occurred()) {
-				PyErr_SetString(PyExc_TypeError, "expected a float");
 				return -1;
 			}
 			((float *)IDP_Array(self->prop))[index] = f;
 			break;
+		}
 		case IDP_DOUBLE:
-			d = PyFloat_AsDouble(value);
+		{
+			const double d = PyFloat_AsDouble(value);
 			if (d == -1 && PyErr_Occurred()) {
-				PyErr_SetString(PyExc_TypeError, "expected a float");
 				return -1;
 			}
 			((double *)IDP_Array(self->prop))[index] = d;
 			break;
+		}
 		case IDP_INT:
-			i = PyLong_AsSsize_t(value);
+		{
+			const int i = _PyLong_AsInt(value);
 			if (i == -1 && PyErr_Occurred()) {
-				PyErr_SetString(PyExc_TypeError, "expected an int type");
 				return -1;
 			}
 
 			((int *)IDP_Array(self->prop))[index] = i;
 			break;
+		}
 	}
 	return 0;
 }
@@ -1103,7 +1176,7 @@ static PyObject *BPy_IDArray_slice(BPy_IDArray *self, int begin, int end)
 	switch (prop->subtype) {
 		case IDP_FLOAT:
 		{
-			float *array = (float *)IDP_Array(prop);
+			const float *array = (float *)IDP_Array(prop);
 			for (count = begin; count < end; count++) {
 				PyTuple_SET_ITEM(tuple, count - begin, PyFloat_FromDouble(array[count]));
 			}
@@ -1111,7 +1184,7 @@ static PyObject *BPy_IDArray_slice(BPy_IDArray *self, int begin, int end)
 		}
 		case IDP_DOUBLE:
 		{
-			double *array = (double *)IDP_Array(prop);
+			const double *array = (double *)IDP_Array(prop);
 			for (count = begin; count < end; count++) {
 				PyTuple_SET_ITEM(tuple, count - begin, PyFloat_FromDouble(array[count]));
 			}
@@ -1119,7 +1192,7 @@ static PyObject *BPy_IDArray_slice(BPy_IDArray *self, int begin, int end)
 		}
 		case IDP_INT:
 		{
-			int *array = (int *)IDP_Array(prop);
+			const int *array = (int *)IDP_Array(prop);
 			for (count = begin; count < end; count++) {
 				PyTuple_SET_ITEM(tuple, count - begin, PyLong_FromLong(array[count]));
 			}
@@ -1133,7 +1206,7 @@ static PyObject *BPy_IDArray_slice(BPy_IDArray *self, int begin, int end)
 static int BPy_IDArray_ass_slice(BPy_IDArray *self, int begin, int end, PyObject *seq)
 {
 	IDProperty *prop = self->prop;
-	short is_double = 0;
+	bool is_double;
 	const PyTypeObject *py_type = idp_array_py_type(self, &is_double);
 	const size_t elem_size = is_double ? sizeof(double) : sizeof(float);
 	size_t alloc_len;
@@ -1173,7 +1246,7 @@ static PyObject *BPy_IDArray_subscript(BPy_IDArray *self, PyObject *item)
 	else if (PySlice_Check(item)) {
 		Py_ssize_t start, stop, step, slicelength;
 
-		if (PySlice_GetIndicesEx((void *)item, self->prop->len, &start, &stop, &step, &slicelength) < 0)
+		if (PySlice_GetIndicesEx(item, self->prop->len, &start, &stop, &step, &slicelength) < 0)
 			return NULL;
 
 		if (slicelength <= 0) {
@@ -1208,7 +1281,7 @@ static int BPy_IDArray_ass_subscript(BPy_IDArray *self, PyObject *item, PyObject
 	else if (PySlice_Check(item)) {
 		Py_ssize_t start, stop, step, slicelength;
 
-		if (PySlice_GetIndicesEx((void *)item, self->prop->len, &start, &stop, &step, &slicelength) < 0)
+		if (PySlice_GetIndicesEx(item, self->prop->len, &start, &stop, &step, &slicelength) < 0)
 			return -1;
 
 		if (step == 1)
@@ -1342,7 +1415,7 @@ static PyObject *BPy_Group_Iter_Next(BPy_IDGroup_Iter *self)
 		}
 	}
 	else {
-		PyErr_SetString(PyExc_StopIteration, "iterator at end");
+		PyErr_SetNone(PyExc_StopIteration);
 		return NULL;
 	}
 }
@@ -1416,15 +1489,15 @@ void IDProp_Init_Types(void)
 /* --- */
 
 static struct PyModuleDef IDProp_types_module_def = {
-    PyModuleDef_HEAD_INIT,
-    "idprop.types",  /* m_name */
-    NULL,  /* m_doc */
-    0,  /* m_size */
-    NULL,  /* m_methods */
-    NULL,  /* m_reload */
-    NULL,  /* m_traverse */
-    NULL,  /* m_clear */
-    NULL,  /* m_free */
+	PyModuleDef_HEAD_INIT,
+	"idprop.types",  /* m_name */
+	NULL,  /* m_doc */
+	0,  /* m_size */
+	NULL,  /* m_methods */
+	NULL,  /* m_reload */
+	NULL,  /* m_traverse */
+	NULL,  /* m_clear */
+	NULL,  /* m_free */
 };
 
 static PyObject *BPyInit_idprop_types(void)
@@ -1433,6 +1506,8 @@ static PyObject *BPyInit_idprop_types(void)
 
 	submodule = PyModule_Create(&IDProp_types_module_def);
 
+	IDProp_Init_Types();
+
 #define MODULE_TYPE_ADD(s, t) \
 	PyModule_AddObject(s, t.tp_name, (PyObject *)&t); Py_INCREF((PyObject *)&t)
 
@@ -1476,10 +1551,47 @@ PyObject *BPyInit_idprop(void)
 
 	mod = PyModule_Create(&IDProp_module_def);
 
-	/* bmesh.types */
+	/* idprop.types */
 	PyModule_AddObject(mod, "types", (submodule = BPyInit_idprop_types()));
 	PyDict_SetItemString(sys_modules, PyModule_GetName(submodule), submodule);
 	Py_INCREF(submodule);
 
 	return mod;
 }
+
+
+#ifndef NDEBUG
+/* -------------------------------------------------------------------- */
+/* debug only function */
+
+void IDP_spit(IDProperty *prop)
+{
+	if (prop) {
+		PyGILState_STATE gilstate;
+		bool use_gil = true; /* !PyC_IsInterpreterActive(); */
+		PyObject *ret_dict;
+		PyObject *ret_str;
+
+		if (use_gil) {
+			gilstate = PyGILState_Ensure();
+		}
+
+		/* to_dict() */
+		ret_dict = BPy_IDGroup_MapDataToPy(prop);
+		ret_str = PyObject_Repr(ret_dict);
+		Py_DECREF(ret_dict);
+
+		printf("IDProperty(%p): %s\n", prop, _PyUnicode_AsString(ret_str));
+
+		Py_DECREF(ret_str);
+
+		if (use_gil) {
+			PyGILState_Release(gilstate);
+		}
+	}
+	else {
+		printf("IDProperty: <NIL>\n");
+	}
+}
+
+#endif
