@@ -33,18 +33,21 @@
 #include <Python.h>
 #include <frameobject.h>
 
+#include "BLI_utildefines.h"  /* for bool */
+
 #include "py_capi_utils.h"
 
 /* only for BLI_strncpy_wchar_from_utf8, should replace with py funcs but too late in release now */
 #include "BLI_string_utf8.h"
 
-#ifdef _WIN32 /* BLI_setenv */
-#include "BLI_path_util.h"
+#ifdef _WIN32
+#include "BLI_path_util.h"  /* BLI_setenv() */
+#include "BLI_math_base.h"  /* finite() */
 #endif
 
 /* array utility function */
 int PyC_AsArray(void *array, PyObject *value, const Py_ssize_t length,
-                const PyTypeObject *type, const short is_double, const char *error_prefix)
+                const PyTypeObject *type, const bool is_double, const char *error_prefix)
 {
 	PyObject *value_fast;
 	Py_ssize_t value_len;
@@ -83,13 +86,13 @@ int PyC_AsArray(void *array, PyObject *value, const Py_ssize_t length,
 		/* could use is_double for 'long int' but no use now */
 		int *array_int = array;
 		for (i = 0; i < length; i++) {
-			array_int[i] = PyLong_AsSsize_t(PySequence_Fast_GET_ITEM(value_fast, i));
+			array_int[i] = PyLong_AsLong(PySequence_Fast_GET_ITEM(value_fast, i));
 		}
 	}
 	else if (type == &PyBool_Type) {
 		int *array_bool = array;
 		for (i = 0; i < length; i++) {
-			array_bool[i] = (PyLong_AsSsize_t(PySequence_Fast_GET_ITEM(value_fast, i)) != 0);
+			array_bool[i] = (PyLong_AsLong(PySequence_Fast_GET_ITEM(value_fast, i)) != 0);
 		}
 	}
 	else {
@@ -112,6 +115,68 @@ int PyC_AsArray(void *array, PyObject *value, const Py_ssize_t length,
 	return 0;
 }
 
+/* array utility function */
+PyObject *PyC_FromArray(const void *array, int length, const PyTypeObject *type,
+                        const bool is_double, const char *error_prefix)
+{
+	PyObject *tuple;
+	int i;
+
+	tuple = PyTuple_New(length);
+
+	/* for each type */
+	if (type == &PyFloat_Type) {
+		if (is_double) {
+			const double *array_double = array;
+			for (i = 0; i < length; ++i) {
+				PyTuple_SET_ITEM(tuple, i, PyFloat_FromDouble(array_double[i]));
+			}
+		}
+		else {
+			const float *array_float = array;
+			for (i = 0; i < length; ++i) {
+				PyTuple_SET_ITEM(tuple, i, PyFloat_FromDouble(array_float[i]));
+			}
+		}
+	}
+	else if (type == &PyLong_Type) {
+		/* could use is_double for 'long int' but no use now */
+		const int *array_int = array;
+		for (i = 0; i < length; ++i) {
+			PyTuple_SET_ITEM(tuple, i, PyLong_FromLong(array_int[i]));
+		}
+	}
+	else if (type == &PyBool_Type) {
+		const int *array_bool = array;
+		for (i = 0; i < length; ++i) {
+			PyTuple_SET_ITEM(tuple, i, PyBool_FromLong(array_bool[i]));
+		}
+	}
+	else {
+		Py_DECREF(tuple);
+		PyErr_Format(PyExc_TypeError,
+		             "%s: internal error %s is invalid",
+		             error_prefix, type->tp_name);
+		return NULL;
+	}
+
+	return tuple;
+}
+
+/**
+ * Caller needs to ensure tuple is uninitialized.
+ * Handy for filling a typle with None for eg.
+ */
+void PyC_Tuple_Fill(PyObject *tuple, PyObject *value)
+{
+	unsigned int tot = PyTuple_GET_SIZE(tuple);
+	unsigned int i;
+
+	for (i = 0; i < tot; i++) {
+		PyTuple_SET_ITEM(tuple, i, value);
+		Py_INCREF(value);
+	}
+}
 
 /* for debugging */
 void PyC_ObSpit(const char *name, PyObject *var)
@@ -141,7 +206,7 @@ void PyC_LineSpit(void)
 	int lineno;
 
 	/* Note, allow calling from outside python (RNA) */
-	if (!PYC_INTERPRETER_ACTIVE) {
+	if (!PyC_IsInterpreterActive()) {
 		fprintf(stderr, "python line lookup failed, interpreter inactive\n");
 		return;
 	}
@@ -155,7 +220,7 @@ void PyC_LineSpit(void)
 void PyC_StackSpit(void)
 {
 	/* Note, allow calling from outside python (RNA) */
-	if (!PYC_INTERPRETER_ACTIVE) {
+	if (!PyC_IsInterpreterActive()) {
 		fprintf(stderr, "python line lookup failed, interpreter inactive\n");
 		return;
 	}
@@ -208,7 +273,7 @@ void PyC_FileAndNum(const char **filename, int *lineno)
 
 void PyC_FileAndNum_Safe(const char **filename, int *lineno)
 {
-	if (!PYC_INTERPRETER_ACTIVE) {
+	if (!PyC_IsInterpreterActive()) {
 		return;
 	}
 
@@ -220,7 +285,7 @@ PyObject *PyC_Object_GetAttrStringArgs(PyObject *o, Py_ssize_t n, ...)
 {
 	Py_ssize_t i;
 	PyObject *item = o;
-	char *attr;
+	const char *attr;
 	
 	va_list vargs;
 
@@ -241,6 +306,23 @@ PyObject *PyC_Object_GetAttrStringArgs(PyObject *o, Py_ssize_t n, ...)
 	return item;
 }
 
+PyObject *PyC_FrozenSetFromStrings(const char **strings)
+{
+	const char **str;
+	PyObject *ret;
+
+	ret = PyFrozenSet_New(NULL);
+
+	for (str = strings; *str; str++) {
+		PyObject *py_str = PyUnicode_FromString(*str);
+		PySet_Add(ret, py_str);
+		Py_DECREF(py_str);
+	}
+
+	return ret;
+}
+
+
 /* similar to PyErr_Format(),
  *
  * implementation - we cant actually preprend the existing exception,
@@ -337,7 +419,7 @@ PyObject *PyC_ExceptionBuffer(void)
 	if (!(string_io_mod = PyImport_ImportModule("io"))) {
 		goto error_cleanup;
 	}
-	else if (!(string_io = PyObject_CallMethod(string_io_mod, (char *)"StringIO", NULL))) {
+	else if (!(string_io = PyObject_CallMethod(string_io_mod, "StringIO", NULL))) {
 		goto error_cleanup;
 	}
 	else if (!(string_io_getvalue = PyObject_GetAttrString(string_io, "getvalue"))) {
@@ -462,8 +544,9 @@ PyObject *PyC_DefaultNameSpace(const char *filename)
 	Py_DECREF(mod_main); /* sys.modules owns now */
 	PyModule_AddStringConstant(mod_main, "__name__", "__main__");
 	if (filename) {
-		/* __file__ mainly for nice UI'ness */
-		PyModule_AddObject(mod_main, "__file__", PyUnicode_DecodeFSDefault(filename));
+		/* __file__ mainly for nice UI'ness
+		 * note: this wont map to a real file when executing text-blocks and buttons. */
+		PyModule_AddObject(mod_main, "__file__", PyC_UnicodeFromByte(filename));
 	}
 	PyModule_AddObject(mod_main, "__builtins__", interp->builtins);
 	Py_INCREF(interp->builtins); /* AddObject steals a reference */
@@ -485,7 +568,7 @@ void PyC_MainModule_Restore(PyObject *main_mod)
 	Py_XDECREF(main_mod);
 }
 
-/* must be called before Py_Initialize, expects output of BLI_get_folder(BLENDER_PYTHON, NULL) */
+/* must be called before Py_Initialize, expects output of BKE_appdir_folder_id(BLENDER_PYTHON, NULL) */
 void PyC_SetHomePath(const char *py_path_bundle)
 {
 	if (py_path_bundle == NULL) {
@@ -504,7 +587,7 @@ void PyC_SetHomePath(const char *py_path_bundle)
 	 * but current Python lib (release 3.1.1) doesn't handle these correctly */
 	if (strchr(py_path_bundle, ':'))
 		printf("Warning : Blender application is located in a path containing : or / chars\
-			   \nThis may make python import function fail\n");
+		       \nThis may make python import function fail\n");
 #endif
 
 
@@ -524,14 +607,18 @@ void PyC_SetHomePath(const char *py_path_bundle)
 		/* cant use this, on linux gives bug: #23018, TODO: try LANG="en_US.UTF-8" /usr/bin/blender, suggested 22008 */
 		/* mbstowcs(py_path_bundle_wchar, py_path_bundle, FILE_MAXDIR); */
 
-		BLI_strncpy_wchar_from_utf8(py_path_bundle_wchar, py_path_bundle,
-		                            sizeof(py_path_bundle_wchar) / sizeof(wchar_t));
+		BLI_strncpy_wchar_from_utf8(py_path_bundle_wchar, py_path_bundle, ARRAY_SIZE(py_path_bundle_wchar));
 
 		Py_SetPythonHome(py_path_bundle_wchar);
 		// printf("found python (wchar_t) '%ls'\n", py_path_bundle_wchar);
 	}
 }
 
+bool PyC_IsInterpreterActive(void)
+{
+	return (((PyThreadState *)_Py_atomic_load_relaxed(&_PyThreadState_Current)) != NULL);
+}
+
 /* Would be nice if python had this built in
  * See: http://wiki.blender.org/index.php/Dev:Doc/Tools/Debugging/PyFromC
  */
@@ -542,7 +629,7 @@ void PyC_RunQuicky(const char *filepath, int n, ...)
 	if (fp) {
 		PyGILState_STATE gilstate = PyGILState_Ensure();
 
-		va_list vargs;	
+		va_list vargs;
 
 		int *sizes = PyMem_MALLOC(sizeof(int) * (n / 2));
 		int i;
@@ -561,15 +648,15 @@ void PyC_RunQuicky(const char *filepath, int n, ...)
 
 		va_start(vargs, n);
 		for (i = 0; i * 2 < n; i++) {
-			char *format = va_arg(vargs, char *);
+			const char *format = va_arg(vargs, char *);
 			void *ptr = va_arg(vargs, void *);
 
-			ret = PyObject_CallFunction(calcsize, (char *)"s", format);
+			ret = PyObject_CallFunction(calcsize, "s", format);
 
 			if (ret) {
-				sizes[i] = PyLong_AsSsize_t(ret);
+				sizes[i] = PyLong_AsLong(ret);
 				Py_DECREF(ret);
-				ret = PyObject_CallFunction(unpack, (char *)"sy#", format, (char *)ptr, sizes[i]);
+				ret = PyObject_CallFunction(unpack, "sy#", format, (char *)ptr, sizes[i]);
 			}
 
 			if (ret == NULL) {
@@ -618,7 +705,7 @@ void PyC_RunQuicky(const char *filepath, int n, ...)
 				/* now get the values back */
 				va_start(vargs, n);
 				for (i = 0; i * 2 < n; i++) {
-					char *format = va_arg(vargs, char *);
+					const char *format = va_arg(vargs, char *);
 					void *ptr = va_arg(vargs, void *);
 					
 					PyObject *item;
@@ -676,6 +763,9 @@ void PyC_RunQuicky(const char *filepath, int n, ...)
 
 		PyGILState_Release(gilstate);
 	}
+	else {
+		fprintf(stderr, "%s: '%s' missing\n", __func__, filepath);
+	}
 }
 
 /* generic function to avoid depending on RNA */
@@ -684,7 +774,7 @@ void *PyC_RNA_AsPointer(PyObject *value, const char *type_name)
 	PyObject *as_pointer;
 	PyObject *pointer;
 
-	if (!strcmp(Py_TYPE(value)->tp_name, type_name) &&
+	if (STREQ(Py_TYPE(value)->tp_name, type_name) &&
 	    (as_pointer = PyObject_GetAttrString(value, "as_pointer")) != NULL &&
 	    PyCallable_Check(as_pointer))
 	{
@@ -737,7 +827,7 @@ char *PyC_FlagSet_AsString(PyC_FlagSet *item)
 int PyC_FlagSet_ValueFromID_int(PyC_FlagSet *item, const char *identifier, int *value)
 {
 	for ( ; item->identifier; item++) {
-		if (strcmp(item->identifier, identifier) == 0) {
+		if (STREQ(item->identifier, identifier)) {
 			*value = item->value;
 			return 1;
 		}
@@ -815,3 +905,73 @@ PyObject *PyC_FlagSet_FromBitfield(PyC_FlagSet *items, int flag)
 
 	return ret;
 }
+
+
+/**
+ * \return -1 on error, else 0
+ *
+ * \note it is caller's responsibility to acquire & release GIL!
+ */
+int PyC_RunString_AsNumber(const char *expr, double *value, const char *filename)
+{
+	PyObject *py_dict, *mod, *retval;
+	int error_ret = 0;
+	PyObject *main_mod = NULL;
+
+	PyC_MainModule_Backup(&main_mod);
+
+	py_dict = PyC_DefaultNameSpace(filename);
+
+	mod = PyImport_ImportModule("math");
+	if (mod) {
+		PyDict_Merge(py_dict, PyModule_GetDict(mod), 0); /* 0 - don't overwrite existing values */
+		Py_DECREF(mod);
+	}
+	else { /* highly unlikely but possibly */
+		PyErr_Print();
+		PyErr_Clear();
+	}
+
+	retval = PyRun_String(expr, Py_eval_input, py_dict, py_dict);
+
+	if (retval == NULL) {
+		error_ret = -1;
+	}
+	else {
+		double val;
+
+		if (PyTuple_Check(retval)) {
+			/* Users my have typed in 10km, 2m
+			 * add up all values */
+			int i;
+			val = 0.0;
+
+			for (i = 0; i < PyTuple_GET_SIZE(retval); i++) {
+				const double val_item = PyFloat_AsDouble(PyTuple_GET_ITEM(retval, i));
+				if (val_item == -1 && PyErr_Occurred()) {
+					val = -1;
+					break;
+				}
+				val += val_item;
+			}
+		}
+		else {
+			val = PyFloat_AsDouble(retval);
+		}
+		Py_DECREF(retval);
+
+		if (val == -1 && PyErr_Occurred()) {
+			error_ret = -1;
+		}
+		else if (!finite(val)) {
+			*value = 0.0;
+		}
+		else {
+			*value = val;
+		}
+	}
+
+	PyC_MainModule_Restore(main_mod);
+
+	return error_ret;
+}
