@@ -29,12 +29,18 @@
 
 #include <Python.h>
 
-#include "bpy_util.h"
+#include "BLI_utildefines.h"
 #include "BLI_dynstr.h"
+
+#include "bpy_util.h"
+
 #include "MEM_guardedalloc.h"
+
 #include "BKE_report.h"
 #include "BKE_context.h"
 
+#include "BLF_translation.h"
+
 #include "../generic/py_capi_utils.h"
 
 static bContext *__py_context = NULL;
@@ -57,13 +63,13 @@ char *BPy_enum_as_string(EnumPropertyItem *item)
 	return cstring;
 }
 
-short BPy_reports_to_error(ReportList *reports, PyObject *exception, const short clear)
+short BPy_reports_to_error(ReportList *reports, PyObject *exception, const bool clear)
 {
 	char *report_str;
 
 	report_str = BKE_reports_string(reports, RPT_ERROR);
 
-	if (clear) {
+	if (clear == true) {
 		BKE_reports_clear(reports);
 	}
 
@@ -79,8 +85,8 @@ short BPy_reports_to_error(ReportList *reports, PyObject *exception, const short
 short BPy_errors_to_report(ReportList *reports)
 {
 	PyObject *pystring;
-	PyObject *pystring_format = NULL; // workaround, see below
-	char *cstring;
+	PyObject *pystring_format = NULL;  /* workaround, see below */
+	const char *cstring;
 
 	const char *filename;
 	int lineno;
@@ -98,7 +104,7 @@ short BPy_errors_to_report(ReportList *reports)
 	pystring = PyC_ExceptionBuffer();
 	
 	if (pystring == NULL) {
-		BKE_report(reports, RPT_ERROR, "unknown py-exception, couldn't convert");
+		BKE_report(reports, RPT_ERROR, "Unknown py-exception, could not convert");
 		return 0;
 	}
 	
@@ -108,17 +114,18 @@ short BPy_errors_to_report(ReportList *reports)
 	
 	cstring = _PyUnicode_AsString(pystring);
 
-#if 0 // ARG!. workaround for a bug in blenders use of vsnprintf
-	BKE_reportf(reports, RPT_ERROR, "%s\nlocation:%s:%d\n", cstring, filename, lineno);
+#if 0 /* ARG!. workaround for a bug in blenders use of vsnprintf */
+	BKE_reportf(reports, RPT_ERROR, "%s\nlocation: %s:%d\n", cstring, filename, lineno);
 #else
-	pystring_format = PyUnicode_FromFormat("%s\nlocation:%s:%d\n", cstring, filename, lineno);
+	pystring_format = PyUnicode_FromFormat(TIP_("%s\nlocation: %s:%d\n"), cstring, filename, lineno);
 	cstring = _PyUnicode_AsString(pystring_format);
 	BKE_report(reports, RPT_ERROR, cstring);
 #endif
-	
-	fprintf(stderr, "%s\nlocation:%s:%d\n", cstring, filename, lineno); // not exactly needed. just for testing
-	
+
+	/* not exactly needed. just for testing */
+	fprintf(stderr, TIP_("%s\nlocation: %s:%d\n"), cstring, filename, lineno);
+
 	Py_DECREF(pystring);
-	Py_DECREF(pystring_format); // workaround
+	Py_DECREF(pystring_format);  /* workaround */
 	return 1;
 }
