@@ -15,11 +15,6 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
- * All rights reserved.
- *
- * This is a new part of Blender.
- *
  * Contributor(s): Joseph Gilbert, Campbell Barton
  *
  * ***** END GPL LICENSE BLOCK *****
@@ -32,6 +27,7 @@
 
 #include <Python.h>
 
+#include "mathutils.h"
 #include "mathutils_geometry.h"
 
 /* Used for PolyFill */
@@ -39,6 +35,7 @@
 #  include "MEM_guardedalloc.h"
 #  include "BLI_blenlib.h"
 #  include "BLI_boxpack2d.h"
+#  include "BLI_convexhull2d.h"
 #  include "BKE_displist.h"
 #  include "BKE_curve.h"
 #endif
@@ -51,7 +48,7 @@ PyDoc_STRVAR(M_Geometry_doc,
 "The Blender geometry module"
 );
 
-//---------------------------------INTERSECTION FUNCTIONS--------------------
+/* ---------------------------------INTERSECTION FUNCTIONS-------------------- */
 
 PyDoc_STRVAR(M_Geometry_intersect_ray_tri_doc,
 ".. function:: intersect_ray_tri(v1, v2, v3, ray, orig, clip=True)\n"
@@ -152,6 +149,11 @@ static PyObject *M_Geometry_intersect_ray_tri(PyObject *UNUSED(self), PyObject *
 	/* calculate t, ray intersects triangle */
 	t = dot_v3v3(e2, qvec) * inv_det;
 
+	/* ray hit behind */
+	if (t < 0.0f) {
+		Py_RETURN_NONE;
+	}
+
 	mul_v3_fl(dir, t);
 	add_v3_v3v3(pvec, orig, dir);
 
@@ -232,6 +234,11 @@ static PyObject *M_Geometry_intersect_line_line(PyObject *UNUSED(self), PyObject
 		}
 
 		result = isect_line_line_v3(v1, v2, v3, v4, i1, i2);
+		/* The return-code isnt exposed,
+		 * this way we can check know how close the lines are. */
+		if (result == 1) {
+			closest_to_line_v3(i2, i1, v3, v4);
+		}
 
 		if (result == 0) {
 			/* colinear */
@@ -251,10 +258,86 @@ static PyObject *M_Geometry_intersect_line_line(PyObject *UNUSED(self), PyObject
 	}
 }
 
+/* Line-Line intersection using algorithm from mathworld.wolfram.com */
 
+PyDoc_STRVAR(M_Geometry_intersect_sphere_sphere_2d_doc,
+".. function:: intersect_sphere_sphere_2d(p_a, radius_a, p_b, radius_b)\n"
+"\n"
+"   Returns 2 points on between intersecting circles.\n"
+"\n"
+"   :arg p_a: Center of the first circle\n"
+"   :type p_a: :class:`mathutils.Vector`\n"
+"   :arg radius_a: Radius of the first circle\n"
+"   :type radius_a: float\n"
+"   :arg p_b: Center of the second circle\n"
+"   :type p_b: :class:`mathutils.Vector`\n"
+"   :arg radius_b: Radius of the second circle\n"
+"   :type radius_b: float\n"
+"   :rtype: tuple of :class:`mathutils.Vector`'s or None when there is no intersection\n"
+);
+static PyObject *M_Geometry_intersect_sphere_sphere_2d(PyObject *UNUSED(self), PyObject *args)
+{
+	PyObject *ret;
+	VectorObject *vec_a, *vec_b;
+	const float *v_a, *v_b;
+	float rad_a, rad_b;
+	float v_ab[2];
+	float dist;
 
+	if (!PyArg_ParseTuple(args, "O!fO!f:intersect_sphere_sphere_2d",
+	                      &vector_Type, &vec_a, &rad_a,
+	                      &vector_Type, &vec_b, &rad_b))
+	{
+		return NULL;
+	}
+
+	if (BaseMath_ReadCallback(vec_a) == -1 ||
+	    BaseMath_ReadCallback(vec_b) == -1)
+	{
+		return NULL;
+	}
+
+	ret = PyTuple_New(2);
+
+	v_a = vec_a->vec;
+	v_b = vec_b->vec;
+
+	sub_v2_v2v2(v_ab, v_b, v_a);
+	dist = len_v2(v_ab);
+
+	if (/* out of range */
+	    (dist > rad_a + rad_b) ||
+	    /* fully-contained in the other */
+	    (dist < fabsf(rad_a - rad_b)) ||
+	    /* co-incident */
+	    (dist < FLT_EPSILON))
+	{
+		/* out of range */
+		PyTuple_SET_ITEM(ret, 0,  Py_None); Py_INCREF(Py_None);
+		PyTuple_SET_ITEM(ret, 1,  Py_None); Py_INCREF(Py_None);
+	}
+	else {
+		const float dist_delta = ((rad_a * rad_a) - (rad_b * rad_b) + (dist * dist)) / (2.0f * dist);
+		const float h = powf(fabsf((rad_a * rad_a) - (dist_delta * dist_delta)), 0.5f);
+		float i_cent[2];
+		float i1[2], i2[2];
+
+		i_cent[0] = v_a[0] + ((v_ab[0] * dist_delta) / dist);
+		i_cent[1] = v_a[1] + ((v_ab[1] * dist_delta) / dist);
+
+		i1[0] = i_cent[0] + h * v_ab[1] / dist;
+		i1[1] = i_cent[1] - h * v_ab[0] / dist;
+
+		i2[0] = i_cent[0] - h * v_ab[1] / dist;
+		i2[1] = i_cent[1] + h * v_ab[0] / dist;
+
+		PyTuple_SET_ITEM(ret, 0, Vector_CreatePyObject(i1, 2, Py_NEW, NULL));
+		PyTuple_SET_ITEM(ret, 1, Vector_CreatePyObject(i2, 2, Py_NEW, NULL));
+	}
+
+	return ret;
+}
 
-//----------------------------geometry.normal() -------------------
 PyDoc_STRVAR(M_Geometry_normal_doc,
 ".. function:: normal(v1, v2, v3, v4=None)\n"
 "\n"
@@ -338,7 +421,7 @@ static PyObject *M_Geometry_normal(PyObject *UNUSED(self), PyObject *args)
 	return Vector_CreatePyObject(n, 3, Py_NEW, NULL);
 }
 
-//--------------------------------- AREA FUNCTIONS--------------------
+/* --------------------------------- AREA FUNCTIONS-------------------- */
 
 PyDoc_STRVAR(M_Geometry_area_tri_doc,
 ".. function:: area_tri(v1, v2, v3)\n"
@@ -391,6 +474,51 @@ static PyObject *M_Geometry_area_tri(PyObject *UNUSED(self), PyObject *args)
 	}
 }
 
+PyDoc_STRVAR(M_Geometry_volume_tetrahedron_doc,
+".. function:: volume_tetrahedron(v1, v2, v3, v4)\n"
+"\n"
+"   Return the volume formed by a tetrahedron (points can be in any order).\n"
+"\n"
+"   :arg v1: Point1\n"
+"   :type v1: :class:`mathutils.Vector`\n"
+"   :arg v2: Point2\n"
+"   :type v2: :class:`mathutils.Vector`\n"
+"   :arg v3: Point3\n"
+"   :type v3: :class:`mathutils.Vector`\n"
+"   :arg v4: Point4\n"
+"   :type v4: :class:`mathutils.Vector`\n"
+"   :rtype: float\n"
+);
+static PyObject *M_Geometry_volume_tetrahedron(PyObject *UNUSED(self), PyObject *args)
+{
+	VectorObject *vec1, *vec2, *vec3, *vec4;
+
+	if (!PyArg_ParseTuple(args, "O!O!O!O!:volume_tetrahedron",
+	                      &vector_Type, &vec1,
+	                      &vector_Type, &vec2,
+	                      &vector_Type, &vec3,
+	                      &vector_Type, &vec4))
+	{
+		return NULL;
+	}
+
+	if (vec1->size < 3 || vec2->size < 3 || vec3->size < 3 || vec4->size < 3) {
+		PyErr_SetString(PyExc_ValueError,
+		                "geometry.volume_tetrahedron(...): "
+		                " can't use 2D Vectors");
+		return NULL;
+	}
+
+	if (BaseMath_ReadCallback(vec1) == -1 ||
+	    BaseMath_ReadCallback(vec2) == -1 ||
+	    BaseMath_ReadCallback(vec3) == -1 ||
+	    BaseMath_ReadCallback(vec4) == -1)
+	{
+		return NULL;
+	}
+
+	return PyFloat_FromDouble(volume_tetrahedron_v3(vec1->vec, vec2->vec, vec3->vec, vec4->vec));
+}
 
 PyDoc_STRVAR(M_Geometry_intersect_line_line_2d_doc,
 ".. function:: intersect_line_line_2d(lineA_p1, lineA_p2, lineB_p1, lineB_p2)\n"
@@ -452,16 +580,15 @@ PyDoc_STRVAR(M_Geometry_intersect_line_plane_doc,
 "   :type plane_co: :class:`mathutils.Vector`\n"
 "   :arg plane_no: The direction the plane is facing\n"
 "   :type plane_no: :class:`mathutils.Vector`\n"
-"   :arg no_flip: Always return an intersection on the directon defined bt line_a -> line_b\n"
-"   :type no_flip: :boolean\n"
 "   :return: The point of intersection or None when not found\n"
 "   :rtype: :class:`mathutils.Vector` or None\n"
 );
 static PyObject *M_Geometry_intersect_line_plane(PyObject *UNUSED(self), PyObject *args)
 {
 	VectorObject *line_a, *line_b, *plane_co, *plane_no;
-	int no_flip = 0;
 	float isect[3];
+	int no_flip = false;
+
 	if (!PyArg_ParseTuple(args, "O!O!O!O!|i:intersect_line_plane",
 	                      &vector_Type, &line_a,
 	                      &vector_Type, &line_b,
@@ -480,14 +607,15 @@ static PyObject *M_Geometry_intersect_line_plane(PyObject *UNUSED(self), PyObjec
 		return NULL;
 	}
 
-	if (ELEM4(2, line_a->size, line_b->size, plane_co->size, plane_no->size)) {
+	if (ELEM(2, line_a->size, line_b->size, plane_co->size, plane_no->size)) {
 		PyErr_SetString(PyExc_ValueError,
 		                "geometry.intersect_line_plane(...): "
 		                " can't use 2D Vectors");
 		return NULL;
 	}
 
-	if (isect_line_plane_v3(isect, line_a->vec, line_b->vec, plane_co->vec, plane_no->vec, no_flip) == 1) {
+	/* TODO: implements no_flip */
+	if (isect_line_plane_v3(isect, line_a->vec, line_b->vec, plane_co->vec, plane_no->vec) == 1) {
 		return Vector_CreatePyObject(isect, 3, Py_NEW, NULL);
 	}
 	else {
@@ -509,17 +637,17 @@ PyDoc_STRVAR(M_Geometry_intersect_plane_plane_doc,
 "   :arg plane_b_no: Normal of the second plane\n"
 "   :type plane_b_no: :class:`mathutils.Vector`\n"
 "   :return: The line of the intersection represented as a point and a vector\n"
-"   :rtype: tuple pair of :class:`mathutils.Vector`\n"
+"   :rtype: tuple pair of :class:`mathutils.Vector` or None if the intersection can't be calculated\n"
 );
 static PyObject *M_Geometry_intersect_plane_plane(PyObject *UNUSED(self), PyObject *args)
 {
-	PyObject *ret;
+	PyObject *ret, *ret_co, *ret_no;
 	VectorObject *plane_a_co, *plane_a_no, *plane_b_co, *plane_b_no;
 
 	float isect_co[3];
 	float isect_no[3];
 
-	if (!PyArg_ParseTuple(args, "O!O!O!O!|i:intersect_plane_plane",
+	if (!PyArg_ParseTuple(args, "O!O!O!O!:intersect_plane_plane",
 	                      &vector_Type, &plane_a_co,
 	                      &vector_Type, &plane_a_no,
 	                      &vector_Type, &plane_b_co,
@@ -536,34 +664,45 @@ static PyObject *M_Geometry_intersect_plane_plane(PyObject *UNUSED(self), PyObje
 		return NULL;
 	}
 
-	if (ELEM4(2, plane_a_co->size, plane_a_no->size, plane_b_co->size, plane_b_no->size)) {
+	if (ELEM(2, plane_a_co->size, plane_a_no->size, plane_b_co->size, plane_b_no->size)) {
 		PyErr_SetString(PyExc_ValueError,
 		                "geometry.intersect_plane_plane(...): "
 		                " can't use 2D Vectors");
 		return NULL;
 	}
 
-	isect_plane_plane_v3(isect_co, isect_no,
-	                     plane_a_co->vec, plane_a_no->vec,
-	                     plane_b_co->vec, plane_b_no->vec);
+	if (isect_plane_plane_v3(isect_co, isect_no,
+	                         plane_a_co->vec, plane_a_no->vec,
+	                         plane_b_co->vec, plane_b_no->vec))
+	{
+		normalize_v3(isect_no);
 
-	normalize_v3(isect_no);
+		ret_co = Vector_CreatePyObject(isect_co, 3, Py_NEW, NULL);
+		ret_no = Vector_CreatePyObject(isect_no, 3, Py_NEW, NULL);
+	}
+	else {
+		ret_co = Py_None;
+		ret_no = Py_None;
+
+		Py_INCREF(ret_co);
+		Py_INCREF(ret_no);
+	}
 
 	ret = PyTuple_New(2);
-	PyTuple_SET_ITEM(ret, 0, Vector_CreatePyObject(isect_co, 3, Py_NEW, NULL));
-	PyTuple_SET_ITEM(ret, 1, Vector_CreatePyObject(isect_no, 3, Py_NEW, NULL));
+	PyTuple_SET_ITEM(ret, 0, ret_co);
+	PyTuple_SET_ITEM(ret, 1, ret_no);
 	return ret;
 }
 
 PyDoc_STRVAR(M_Geometry_intersect_line_sphere_doc,
 ".. function:: intersect_line_sphere(line_a, line_b, sphere_co, sphere_radius, clip=True)\n"
 "\n"
-"   Takes a lines (as 2 vectors), a sphere as a point and a radius and\n"
+"   Takes a line (as 2 points) and a sphere (as a point and a radius) and\n"
 "   returns the intersection\n"
 "\n"
-"   :arg line_a: First point of the first line\n"
+"   :arg line_a: First point of the line\n"
 "   :type line_a: :class:`mathutils.Vector`\n"
-"   :arg line_b: Second point of the first line\n"
+"   :arg line_b: Second point of the line\n"
 "   :type line_b: :class:`mathutils.Vector`\n"
 "   :arg sphere_co: The center of the sphere\n"
 "   :type sphere_co: :class:`mathutils.Vector`\n"
@@ -576,7 +715,7 @@ static PyObject *M_Geometry_intersect_line_sphere(PyObject *UNUSED(self), PyObje
 {
 	VectorObject *line_a, *line_b, *sphere_co;
 	float sphere_radius;
-	int clip = TRUE;
+	int clip = true;
 
 	float isect_a[3];
 	float isect_b[3];
@@ -597,31 +736,32 @@ static PyObject *M_Geometry_intersect_line_sphere(PyObject *UNUSED(self), PyObje
 		return NULL;
 	}
 
-	if (ELEM3(2, line_a->size, line_b->size, sphere_co->size)) {
+	if (ELEM(2, line_a->size, line_b->size, sphere_co->size)) {
 		PyErr_SetString(PyExc_ValueError,
 		                "geometry.intersect_line_sphere(...): "
 		                " can't use 2D Vectors");
 		return NULL;
 	}
 	else {
-		short use_a = TRUE;
-		short use_b = TRUE;
+		bool use_a = true;
+		bool use_b = true;
 		float lambda;
 
 		PyObject *ret = PyTuple_New(2);
 
 		switch (isect_line_sphere_v3(line_a->vec, line_b->vec, sphere_co->vec, sphere_radius, isect_a, isect_b)) {
 			case 1:
-				if (!(!clip || (((lambda = line_point_factor_v3(isect_a, line_a->vec, line_b->vec)) >= 0.0f) && (lambda <= 1.0f)))) use_a = FALSE;
-				use_b = FALSE;
+				if (!(!clip || (((lambda = line_point_factor_v3(isect_a, line_a->vec, line_b->vec)) >= 0.0f) && (lambda <= 1.0f)))) use_a = false;
+				use_b = false;
 				break;
 			case 2:
-				if (!(!clip || (((lambda = line_point_factor_v3(isect_a, line_a->vec, line_b->vec)) >= 0.0f) && (lambda <= 1.0f)))) use_a = FALSE;
-				if (!(!clip || (((lambda = line_point_factor_v3(isect_b, line_a->vec, line_b->vec)) >= 0.0f) && (lambda <= 1.0f)))) use_b = FALSE;
+				if (!(!clip || (((lambda = line_point_factor_v3(isect_a, line_a->vec, line_b->vec)) >= 0.0f) && (lambda <= 1.0f)))) use_a = false;
+				if (!(!clip || (((lambda = line_point_factor_v3(isect_b, line_a->vec, line_b->vec)) >= 0.0f) && (lambda <= 1.0f)))) use_b = false;
 				break;
 			default:
-				use_a = FALSE;
-				use_b = FALSE;
+				use_a = false;
+				use_b = false;
+				break;
 		}
 
 		if (use_a) { PyTuple_SET_ITEM(ret, 0,  Vector_CreatePyObject(isect_a, 3, Py_NEW, NULL)); }
@@ -638,12 +778,12 @@ static PyObject *M_Geometry_intersect_line_sphere(PyObject *UNUSED(self), PyObje
 PyDoc_STRVAR(M_Geometry_intersect_line_sphere_2d_doc,
 ".. function:: intersect_line_sphere_2d(line_a, line_b, sphere_co, sphere_radius, clip=True)\n"
 "\n"
-"   Takes a lines (as 2 vectors), a sphere as a point and a radius and\n"
+"   Takes a line (as 2 points) and a sphere (as a point and a radius) and\n"
 "   returns the intersection\n"
 "\n"
-"   :arg line_a: First point of the first line\n"
+"   :arg line_a: First point of the line\n"
 "   :type line_a: :class:`mathutils.Vector`\n"
-"   :arg line_b: Second point of the first line\n"
+"   :arg line_b: Second point of the line\n"
 "   :type line_b: :class:`mathutils.Vector`\n"
 "   :arg sphere_co: The center of the sphere\n"
 "   :type sphere_co: :class:`mathutils.Vector`\n"
@@ -656,10 +796,10 @@ static PyObject *M_Geometry_intersect_line_sphere_2d(PyObject *UNUSED(self), PyO
 {
 	VectorObject *line_a, *line_b, *sphere_co;
 	float sphere_radius;
-	int clip = TRUE;
+	int clip = true;
 
-	float isect_a[3];
-	float isect_b[3];
+	float isect_a[2];
+	float isect_b[2];
 
 	if (!PyArg_ParseTuple(args, "O!O!O!f|i:intersect_line_sphere_2d",
 	                      &vector_Type, &line_a,
@@ -677,24 +817,25 @@ static PyObject *M_Geometry_intersect_line_sphere_2d(PyObject *UNUSED(self), PyO
 		return NULL;
 	}
 	else {
-		short use_a = TRUE;
-		short use_b = TRUE;
+		bool use_a = true;
+		bool use_b = true;
 		float lambda;
 
 		PyObject *ret = PyTuple_New(2);
 
 		switch (isect_line_sphere_v2(line_a->vec, line_b->vec, sphere_co->vec, sphere_radius, isect_a, isect_b)) {
 			case 1:
-				if (!(!clip || (((lambda = line_point_factor_v2(isect_a, line_a->vec, line_b->vec)) >= 0.0f) && (lambda <= 1.0f)))) use_a = FALSE;
-				use_b = FALSE;
+				if (!(!clip || (((lambda = line_point_factor_v2(isect_a, line_a->vec, line_b->vec)) >= 0.0f) && (lambda <= 1.0f)))) use_a = false;
+				use_b = false;
 				break;
 			case 2:
-				if (!(!clip || (((lambda = line_point_factor_v2(isect_a, line_a->vec, line_b->vec)) >= 0.0f) && (lambda <= 1.0f)))) use_a = FALSE;
-				if (!(!clip || (((lambda = line_point_factor_v2(isect_b, line_a->vec, line_b->vec)) >= 0.0f) && (lambda <= 1.0f)))) use_b = FALSE;
+				if (!(!clip || (((lambda = line_point_factor_v2(isect_a, line_a->vec, line_b->vec)) >= 0.0f) && (lambda <= 1.0f)))) use_a = false;
+				if (!(!clip || (((lambda = line_point_factor_v2(isect_b, line_a->vec, line_b->vec)) >= 0.0f) && (lambda <= 1.0f)))) use_b = false;
 				break;
 			default:
-				use_a = FALSE;
-				use_b = FALSE;
+				use_a = false;
+				use_b = false;
+				break;
 		}
 
 		if (use_a) { PyTuple_SET_ITEM(ret, 0,  Vector_CreatePyObject(isect_a, 2, Py_NEW, NULL)); }
@@ -762,13 +903,69 @@ static PyObject *M_Geometry_intersect_point_line(PyObject *UNUSED(self), PyObjec
 	return ret;
 }
 
+PyDoc_STRVAR(M_Geometry_intersect_point_tri_doc,
+".. function:: intersect_point_tri(pt, tri_p1, tri_p2, tri_p3)\n"
+"\n"
+"   Takes 4 vectors: one is the point and the next 3 define the triangle.\n"
+"\n"
+"   :arg pt: Point\n"
+"   :type pt: :class:`mathutils.Vector`\n"
+"   :arg tri_p1: First point of the triangle\n"
+"   :type tri_p1: :class:`mathutils.Vector`\n"
+"   :arg tri_p2: Second point of the triangle\n"
+"   :type tri_p2: :class:`mathutils.Vector`\n"
+"   :arg tri_p3: Third point of the triangle\n"
+"   :type tri_p3: :class:`mathutils.Vector`\n"
+"   :return: Point on the triangles plane or None if its outside the triangle\n"
+"   :rtype: :class:`mathutils.Vector` or None\n"
+);
+static PyObject *M_Geometry_intersect_point_tri(PyObject *UNUSED(self), PyObject *args)
+{
+	VectorObject *pt_vec, *tri_p1, *tri_p2, *tri_p3;
+	float vi[3];
+
+	if (!PyArg_ParseTuple(args, "O!O!O!O!:intersect_point_tri",
+	                      &vector_Type, &pt_vec,
+	                      &vector_Type, &tri_p1,
+	                      &vector_Type, &tri_p2,
+	                      &vector_Type, &tri_p3))
+	{
+		return NULL;
+	}
+
+	if (BaseMath_ReadCallback(pt_vec) == -1 ||
+	    BaseMath_ReadCallback(tri_p1) == -1 ||
+	    BaseMath_ReadCallback(tri_p2) == -1 ||
+	    BaseMath_ReadCallback(tri_p3) == -1)
+	{
+		return NULL;
+	}
+
+	if (pt_vec->size < 3 ||
+	    tri_p1->size < 3 ||
+	    tri_p2->size < 3 ||
+	    tri_p3->size < 3)
+	{
+		PyErr_SetString(PyExc_ValueError,
+		                "One of more of the vector arguments wasn't a 3D vector");
+		return NULL;
+	}
+
+	if (isect_point_tri_v3(pt_vec->vec, tri_p1->vec, tri_p2->vec, tri_p3->vec, vi)) {
+		return Vector_CreatePyObject(vi, 3, Py_NEW, NULL);
+	}
+	else {
+		Py_RETURN_NONE;
+	}
+}
+
 PyDoc_STRVAR(M_Geometry_intersect_point_tri_2d_doc,
 ".. function:: intersect_point_tri_2d(pt, tri_p1, tri_p2, tri_p3)\n"
 "\n"
 "   Takes 4 vectors (using only the x and y coordinates): one is the point and the next 3 define the triangle. Returns 1 if the point is within the triangle, otherwise 0.\n"
 "\n"
 "   :arg pt: Point\n"
-"   :type v1: :class:`mathutils.Vector`\n"
+"   :type pt: :class:`mathutils.Vector`\n"
 "   :arg tri_p1: First point of the triangle\n"
 "   :type tri_p1: :class:`mathutils.Vector`\n"
 "   :arg tri_p2: Second point of the triangle\n"
@@ -854,32 +1051,43 @@ PyDoc_STRVAR(M_Geometry_distance_point_to_plane_doc,
 "\n"
 "   :arg pt: Point\n"
 "   :type pt: :class:`mathutils.Vector`\n"
-"   :arg plane_co: First point of the quad\n"
+"   :arg plane_co: A point on the plane\n"
 "   :type plane_co: :class:`mathutils.Vector`\n"
-"   :arg plane_no: Second point of the quad\n"
+"   :arg plane_no: The direction the plane is facing\n"
 "   :type plane_no: :class:`mathutils.Vector`\n"
 "   :rtype: float\n"
 );
 static PyObject *M_Geometry_distance_point_to_plane(PyObject *UNUSED(self), PyObject *args)
 {
-	VectorObject *pt, *plene_co, *plane_no;
+	VectorObject *pt, *plane_co, *plane_no;
+	float plane[4];
 
 	if (!PyArg_ParseTuple(args, "O!O!O!:distance_point_to_plane",
 	                      &vector_Type, &pt,
-	                      &vector_Type, &plene_co,
+	                      &vector_Type, &plane_co,
 	                      &vector_Type, &plane_no))
 	{
 		return NULL;
 	}
 
+	if (pt->size != 3 ||
+	    plane_co->size != 3 ||
+	    plane_no->size != 3)
+	{
+		PyErr_SetString(PyExc_ValueError,
+		                "One of more of the vector arguments wasn't a 3D vector");
+		return NULL;
+	}
+
 	if (BaseMath_ReadCallback(pt) == -1 ||
-	    BaseMath_ReadCallback(plene_co) == -1 ||
+	    BaseMath_ReadCallback(plane_co) == -1 ||
 	    BaseMath_ReadCallback(plane_no) == -1)
 	{
 		return NULL;
 	}
 
-	return PyFloat_FromDouble(dist_to_plane_v3(pt->vec, plene_co->vec, plane_no->vec));
+	plane_from_point_normal_v3(plane, plane_co->vec, plane_no->vec);
+	return PyFloat_FromDouble(dist_signed_to_plane_v3(pt->vec, plane));
 }
 
 PyDoc_STRVAR(M_Geometry_barycentric_transform_doc,
@@ -911,7 +1119,7 @@ static PyObject *M_Geometry_barycentric_transform(PyObject *UNUSED(self), PyObje
 	VectorObject *vec_t1_src, *vec_t2_src, *vec_t3_src;
 	float vec[3];
 
-	if (!PyArg_ParseTuple(args, "O!O!O!O!O!O!O!:barycentric_transform",
+	if (!PyArg_ParseTuple(args, "O!O!O!O!O!O!O!::barycentric_transform",
 	                      &vector_Type, &vec_pt,
 	                      &vector_Type, &vec_t1_src,
 	                      &vector_Type, &vec_t2_src,
@@ -936,9 +1144,21 @@ static PyObject *M_Geometry_barycentric_transform(PyObject *UNUSED(self), PyObje
 		return NULL;
 	}
 
-	barycentric_transform(vec, vec_pt->vec,
-	                      vec_t1_tar->vec, vec_t2_tar->vec, vec_t3_tar->vec,
-	                      vec_t1_src->vec, vec_t2_src->vec, vec_t3_src->vec);
+	if (BaseMath_ReadCallback(vec_pt) == -1 ||
+	    BaseMath_ReadCallback(vec_t1_src) == -1 ||
+	    BaseMath_ReadCallback(vec_t2_src) == -1 ||
+	    BaseMath_ReadCallback(vec_t3_src) == -1 ||
+	    BaseMath_ReadCallback(vec_t1_tar) == -1 ||
+	    BaseMath_ReadCallback(vec_t2_tar) == -1 ||
+	    BaseMath_ReadCallback(vec_t3_tar) == -1)
+	{
+		return NULL;
+	}
+
+	transform_point_by_tri_v3(
+	        vec, vec_pt->vec,
+	        vec_t1_tar->vec, vec_t2_tar->vec, vec_t3_tar->vec,
+	        vec_t1_src->vec, vec_t2_src->vec, vec_t3_src->vec);
 
 	return Vector_CreatePyObject(vec, 3, Py_NEW, NULL);
 }
@@ -950,7 +1170,7 @@ PyDoc_STRVAR(M_Geometry_points_in_planes_doc,
 "\n"
 "   :arg planes: List of planes (4D vectors).\n"
 "   :type planes: list of :class:`mathutils.Vector`\n"
-"   :return: two lists, once containing the vertices inside the planes, another containing the plane indicies used\n"
+"   :return: two lists, once containing the vertices inside the planes, another containing the plane indices used\n"
 "   :rtype: pair of lists\n"
 );
 /* note: this function could be optimized by some spatial structure */
@@ -977,11 +1197,13 @@ static PyObject *M_Geometry_points_in_planes(PyObject *UNUSED(self), PyObject *a
 
 		float n1n2[3], n2n3[3], n3n1[3];
 		float potentialVertex[3];
-		char *planes_used = MEM_callocN(sizeof(char) * len, __func__);
+		char *planes_used = PyMem_Malloc(sizeof(char) * len);
 
 		/* python */
 		PyObject *py_verts = PyList_New(0);
-		PyObject *py_plene_index = PyList_New(0);
+		PyObject *py_plane_index = PyList_New(0);
+
+		memset(planes_used, 0, sizeof(char) * len);
 
 		for (i = 0; i < len; i++) {
 			const float *N1 = planes[i];
@@ -1015,7 +1237,7 @@ static PyObject *M_Geometry_points_in_planes(PyObject *UNUSED(self), PyObject *a
 										PyList_Append(py_verts, item);
 										Py_DECREF(item);
 
-										planes_used[i] = planes_used[j] = planes_used[k] = TRUE;
+										planes_used[i] = planes_used[j] = planes_used[k] = true;
 									}
 								}
 							}
@@ -1031,16 +1253,16 @@ static PyObject *M_Geometry_points_in_planes(PyObject *UNUSED(self), PyObject *a
 		for (i = 0; i < len; i++) {
 			if (planes_used[i]) {
 				PyObject *item = PyLong_FromLong(i);
-				PyList_Append(py_plene_index, item);
+				PyList_Append(py_plane_index, item);
 				Py_DECREF(item);
 			}
 		}
-		MEM_freeN(planes_used);
+		PyMem_Free(planes_used);
 
 		{
 			PyObject *ret = PyTuple_New(2);
 			PyTuple_SET_ITEM(ret, 0, py_verts);
-			PyTuple_SET_ITEM(ret, 1, py_plene_index);
+			PyTuple_SET_ITEM(ret, 1, py_plane_index);
 			return ret;
 		}
 	}
@@ -1104,7 +1326,7 @@ static PyObject *M_Geometry_interpolate_bezier(PyObject *UNUSED(self), PyObject
 		return NULL;
 	}
 
-	dims = MAX4(vec_k1->size, vec_h1->size, vec_h2->size, vec_k2->size);
+	dims = max_iiii(vec_k1->size, vec_h1->size, vec_h2->size, vec_k2->size);
 
 	for (i = 0; i < vec_k1->size; i++) k1[i] = vec_k1->vec[i];
 	for (i = 0; i < vec_h1->size; i++) h1[i] = vec_h1->vec[i];
@@ -1220,7 +1442,8 @@ static PyObject *M_Geometry_tessellate_polygon(PyObject *UNUSED(self), PyObject
 	}
 	else if (totpoints) {
 		/* now make the list to return */
-		BKE_displist_fill(&dispbase, &dispbase, 0);
+		/* TODO, add normal arg */
+		BKE_displist_fill(&dispbase, &dispbase, NULL, false);
 
 		/* The faces are stored in a new DisplayList
 		 * thats added to the head of the listbase */
@@ -1352,14 +1575,95 @@ static PyObject *M_Geometry_box_pack_2d(PyObject *UNUSED(self), PyObject *boxlis
 		}
 
 		/* Non Python function */
-		BLI_box_pack_2D(boxarray, len, &tot_width, &tot_height);
+		BLI_box_pack_2d(boxarray, len, &tot_width, &tot_height);
 
 		boxPack_ToPyObject(boxlist, &boxarray);
 	}
 
 	ret = PyTuple_New(2);
 	PyTuple_SET_ITEM(ret, 0, PyFloat_FromDouble(tot_width));
-	PyTuple_SET_ITEM(ret, 1, PyFloat_FromDouble(tot_width));
+	PyTuple_SET_ITEM(ret, 1, PyFloat_FromDouble(tot_height));
+	return ret;
+}
+
+PyDoc_STRVAR(M_Geometry_box_fit_2d_doc,
+".. function:: box_fit_2d(points)\n"
+"\n"
+"   Returns an angle that best fits the points to an axis aligned rectangle\n"
+"\n"
+"   :arg points: list of 2d points.\n"
+"   :type points: list\n"
+"   :return: angle\n"
+"   :rtype: float\n"
+);
+static PyObject *M_Geometry_box_fit_2d(PyObject *UNUSED(self), PyObject *pointlist)
+{
+	float (*points)[2];
+	Py_ssize_t len;
+
+	float angle = 0.0f;
+
+	len = mathutils_array_parse_alloc_v(((float **)&points), 2, pointlist, "box_fit_2d");
+	if (len == -1) {
+		return NULL;
+	}
+
+	if (len) {
+		/* Non Python function */
+		angle = BLI_convexhull_aabb_fit_points_2d((const float (*)[2])points, len);
+
+		PyMem_Free(points);
+	}
+
+
+	return PyFloat_FromDouble(angle);
+}
+
+PyDoc_STRVAR(M_Geometry_convex_hull_2d_doc,
+".. function:: convex_hull_2d(points)\n"
+"\n"
+"   Returns a list of indices into the list given\n"
+"\n"
+"   :arg points: list of 2d points.\n"
+"   :type points: list\n"
+"   :return: a list of indices\n"
+"   :rtype: list of ints\n"
+);
+static PyObject *M_Geometry_convex_hull_2d(PyObject *UNUSED(self), PyObject *pointlist)
+{
+	float (*points)[2];
+	Py_ssize_t len;
+
+	PyObject *ret;
+
+	len = mathutils_array_parse_alloc_v(((float **)&points), 2, pointlist, "convex_hull_2d");
+	if (len == -1) {
+		return NULL;
+	}
+
+	if (len) {
+		int *index_map;
+		Py_ssize_t len_ret, i;
+
+		index_map  = MEM_mallocN(sizeof(*index_map) * len * 2, __func__);
+
+		/* Non Python function */
+		len_ret = BLI_convexhull_2d((const float (*)[2])points, len, index_map);
+
+		ret = PyList_New(len_ret);
+		for (i = 0; i < len_ret; i++) {
+			PyList_SET_ITEM(ret, i, PyLong_FromLong(index_map[i]));
+		}
+
+		MEM_freeN(index_map);
+
+		PyMem_Free(points);
+	}
+	else {
+		ret = PyList_New(0);
+	}
+
+
 	return ret;
 }
 
@@ -1369,6 +1673,7 @@ static PyObject *M_Geometry_box_pack_2d(PyObject *UNUSED(self), PyObject *boxlis
 static PyMethodDef M_Geometry_methods[] = {
 	{"intersect_ray_tri", (PyCFunction) M_Geometry_intersect_ray_tri, METH_VARARGS, M_Geometry_intersect_ray_tri_doc},
 	{"intersect_point_line", (PyCFunction) M_Geometry_intersect_point_line, METH_VARARGS, M_Geometry_intersect_point_line_doc},
+	{"intersect_point_tri", (PyCFunction) M_Geometry_intersect_point_tri, METH_VARARGS, M_Geometry_intersect_point_tri_doc},
 	{"intersect_point_tri_2d", (PyCFunction) M_Geometry_intersect_point_tri_2d, METH_VARARGS, M_Geometry_intersect_point_tri_2d_doc},
 	{"intersect_point_quad_2d", (PyCFunction) M_Geometry_intersect_point_quad_2d, METH_VARARGS, M_Geometry_intersect_point_quad_2d_doc},
 	{"intersect_line_line", (PyCFunction) M_Geometry_intersect_line_line, METH_VARARGS, M_Geometry_intersect_line_line_doc},
@@ -1378,13 +1683,17 @@ static PyMethodDef M_Geometry_methods[] = {
 	{"intersect_line_sphere", (PyCFunction) M_Geometry_intersect_line_sphere, METH_VARARGS, M_Geometry_intersect_line_sphere_doc},
 	{"intersect_line_sphere_2d", (PyCFunction) M_Geometry_intersect_line_sphere_2d, METH_VARARGS, M_Geometry_intersect_line_sphere_2d_doc},
 	{"distance_point_to_plane", (PyCFunction) M_Geometry_distance_point_to_plane, METH_VARARGS, M_Geometry_distance_point_to_plane_doc},
+	{"intersect_sphere_sphere_2d", (PyCFunction) M_Geometry_intersect_sphere_sphere_2d, METH_VARARGS, M_Geometry_intersect_sphere_sphere_2d_doc},
 	{"area_tri", (PyCFunction) M_Geometry_area_tri, METH_VARARGS, M_Geometry_area_tri_doc},
+	{"volume_tetrahedron", (PyCFunction) M_Geometry_volume_tetrahedron, METH_VARARGS, M_Geometry_volume_tetrahedron_doc},
 	{"normal", (PyCFunction) M_Geometry_normal, METH_VARARGS, M_Geometry_normal_doc},
 	{"barycentric_transform", (PyCFunction) M_Geometry_barycentric_transform, METH_VARARGS, M_Geometry_barycentric_transform_doc},
 	{"points_in_planes", (PyCFunction) M_Geometry_points_in_planes, METH_VARARGS, M_Geometry_points_in_planes_doc},
 #ifndef MATH_STANDALONE
 	{"interpolate_bezier", (PyCFunction) M_Geometry_interpolate_bezier, METH_VARARGS, M_Geometry_interpolate_bezier_doc},
 	{"tessellate_polygon", (PyCFunction) M_Geometry_tessellate_polygon, METH_O, M_Geometry_tessellate_polygon_doc},
+	{"convex_hull_2d", (PyCFunction) M_Geometry_convex_hull_2d, METH_O, M_Geometry_convex_hull_2d_doc},
+	{"box_fit_2d", (PyCFunction) M_Geometry_box_fit_2d, METH_O, M_Geometry_box_fit_2d_doc},
 	{"box_pack_2d", (PyCFunction) M_Geometry_box_pack_2d, METH_O, M_Geometry_box_pack_2d_doc},
 #endif
 	{NULL, NULL, 0, NULL}
