@@ -15,11 +15,6 @@
  * along with this program; if not, write to the Free Software Foundation,
  * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
- * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
- * All rights reserved.
- *
- * This is a new part of Blender.
- *
  * Contributor(s): eeshlo, Campbell Barton
  *
  * ***** END GPL LICENSE BLOCK *****
@@ -39,14 +34,10 @@
 
 #include <Python.h>
 
-#include "structseq.h"
-
-#include "BLI_blenlib.h"
 #include "BLI_math.h"
+#include "BLI_noise.h"
 #include "BLI_utildefines.h"
 
-#include "MEM_guardedalloc.h"
-
 #include "DNA_texture_types.h"
 
 #include "mathutils.h"
@@ -357,6 +348,7 @@ PyDoc_STRVAR(M_Noise_seed_set_doc,
 "   Sets the random seed used for random_unit_vector, random_vector and random.\n"
 "\n"
 "   :arg seed: Seed used for the random generator.\n"
+"      When seed is zero, the current time will be used instead.\n"
 "   :type seed: Int\n"
 );
 static PyObject *M_Noise_seed_set(PyObject *UNUSED(self), PyObject *args)
@@ -796,7 +788,7 @@ static PyObject *M_Noise_cell_vector(PyObject *UNUSED(self), PyObject *args)
 		return NULL;
 
 	cellNoiseV(vec[0], vec[1], vec[2], r_vec);
-	return Vector_CreatePyObject(NULL, 3, Py_NEW, NULL);
+	return Vector_CreatePyObject(r_vec, 3, Py_NEW, NULL);
 }
 
 static PyMethodDef M_Noise_methods[] = {
@@ -869,16 +861,16 @@ PyMODINIT_FUNC PyInit_mathutils_noise_types(void)
 {
 	PyObject *submodule = PyModule_Create(&M_NoiseTypes_module_def);
 
-	PyModule_AddIntConstant(submodule, (char *)"BLENDER", TEX_BLENDER);
-	PyModule_AddIntConstant(submodule, (char *)"STDPERLIN", TEX_STDPERLIN);
-	PyModule_AddIntConstant(submodule, (char *)"NEWPERLIN", TEX_NEWPERLIN);
-	PyModule_AddIntConstant(submodule, (char *)"VORONOI_F1", TEX_VORONOI_F1);
-	PyModule_AddIntConstant(submodule, (char *)"VORONOI_F2", TEX_VORONOI_F2);
-	PyModule_AddIntConstant(submodule, (char *)"VORONOI_F3", TEX_VORONOI_F3);
-	PyModule_AddIntConstant(submodule, (char *)"VORONOI_F4", TEX_VORONOI_F4);
-	PyModule_AddIntConstant(submodule, (char *)"VORONOI_F2F1", TEX_VORONOI_F2F1);
-	PyModule_AddIntConstant(submodule, (char *)"VORONOI_CRACKLE", TEX_VORONOI_CRACKLE);
-	PyModule_AddIntConstant(submodule, (char *)"CELLNOISE", TEX_CELLNOISE);
+	PyModule_AddIntConstant(submodule, "BLENDER", TEX_BLENDER);
+	PyModule_AddIntConstant(submodule, "STDPERLIN", TEX_STDPERLIN);
+	PyModule_AddIntConstant(submodule, "NEWPERLIN", TEX_NEWPERLIN);
+	PyModule_AddIntConstant(submodule, "VORONOI_F1", TEX_VORONOI_F1);
+	PyModule_AddIntConstant(submodule, "VORONOI_F2", TEX_VORONOI_F2);
+	PyModule_AddIntConstant(submodule, "VORONOI_F3", TEX_VORONOI_F3);
+	PyModule_AddIntConstant(submodule, "VORONOI_F4", TEX_VORONOI_F4);
+	PyModule_AddIntConstant(submodule, "VORONOI_F2F1", TEX_VORONOI_F2F1);
+	PyModule_AddIntConstant(submodule, "VORONOI_CRACKLE", TEX_VORONOI_CRACKLE);
+	PyModule_AddIntConstant(submodule, "CELLNOISE", TEX_CELLNOISE);
 
 	return submodule;
 }
@@ -899,13 +891,13 @@ PyMODINIT_FUNC PyInit_mathutils_noise_metrics(void)
 {
 	PyObject *submodule = PyModule_Create(&M_NoiseMetrics_module_def);
 
-	PyModule_AddIntConstant(submodule, (char *)"DISTANCE", TEX_DISTANCE);
-	PyModule_AddIntConstant(submodule, (char *)"DISTANCE_SQUARED", TEX_DISTANCE_SQUARED);
-	PyModule_AddIntConstant(submodule, (char *)"MANHATTAN", TEX_MANHATTAN);
-	PyModule_AddIntConstant(submodule, (char *)"CHEBYCHEV", TEX_CHEBYCHEV);
-	PyModule_AddIntConstant(submodule, (char *)"MINKOVSKY_HALF", TEX_MINKOVSKY_HALF);
-	PyModule_AddIntConstant(submodule, (char *)"MINKOVSKY_FOUR", TEX_MINKOVSKY_FOUR);
-	PyModule_AddIntConstant(submodule, (char *)"MINKOVSKY", TEX_MINKOVSKY);
+	PyModule_AddIntConstant(submodule, "DISTANCE", TEX_DISTANCE);
+	PyModule_AddIntConstant(submodule, "DISTANCE_SQUARED", TEX_DISTANCE_SQUARED);
+	PyModule_AddIntConstant(submodule, "MANHATTAN", TEX_MANHATTAN);
+	PyModule_AddIntConstant(submodule, "CHEBYCHEV", TEX_CHEBYCHEV);
+	PyModule_AddIntConstant(submodule, "MINKOVSKY_HALF", TEX_MINKOVSKY_HALF);
+	PyModule_AddIntConstant(submodule, "MINKOVSKY_FOUR", TEX_MINKOVSKY_FOUR);
+	PyModule_AddIntConstant(submodule, "MINKOVSKY", TEX_MINKOVSKY);
 
 	return submodule;
 }
