@@ -31,40 +31,34 @@
 #include <string.h>
 
 /* external modules: */
-#include "MEM_guardedalloc.h"
 
 #include "BLI_math.h"
 #include "BLI_blenlib.h"
 #include "BLI_threads.h"
 #include "BLI_utildefines.h"
 
+#include "BLF_translation.h"
+
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"        /* for rectcpy */
 
 #include "DNA_group_types.h"
 #include "DNA_image_types.h"
+#include "DNA_lamp_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 #include "DNA_texture_types.h"
 
-#include "BKE_library.h"
 #include "BKE_main.h"
 #include "BKE_image.h"   /* BKE_imbuf_write */
 #include "BKE_texture.h"
 
-
-
-
 /* this module */
 #include "render_types.h"
-#include "renderpipeline.h"
 #include "envmap.h"
-#include "rendercore.h" 
 #include "renderdatabase.h" 
 #include "texture.h"
 #include "zbuf.h"
-#include "initrender.h"
-
 
 /* ------------------------------------------------------------------------- */
 
@@ -76,7 +70,7 @@ static void envmap_split_ima(EnvMap *env, ImBuf *ibuf)
 	BLI_lock_thread(LOCK_IMAGE);
 	if (env->cube[1] == NULL) {
 
-		BKE_free_envmapdata(env);	
+		BKE_free_envmapdata(env);
 	
 		dx = ibuf->y;
 		dx /= 2;
@@ -120,7 +114,7 @@ static void envmap_split_ima(EnvMap *env, ImBuf *ibuf)
 				IMB_float_from_rect(env->cube[1]);
 			}
 		}
-	}	
+	}
 	BLI_unlock_thread(LOCK_IMAGE);
 }
 
@@ -146,25 +140,31 @@ static Render *envmap_render_copy(Render *re, EnvMap *env)
 	/* set up renderdata */
 	envre->r = re->r;
 	envre->r.mode &= ~(R_BORDER | R_PANORAMA | R_ORTHO | R_MBLUR);
-	envre->r.layers.first = envre->r.layers.last = NULL;
+	BLI_listbase_clear(&envre->r.layers);
 	envre->r.filtertype = 0;
-	envre->r.xparts = envre->r.yparts = 2;
+	envre->r.tilex = envre->r.xsch / 2;
+	envre->r.tiley = envre->r.ysch / 2;
 	envre->r.size = 100;
 	envre->r.yasp = envre->r.xasp = 1;
 	
 	RE_InitState(envre, NULL, &envre->r, NULL, cuberes, cuberes, NULL);
+	envre->main = re->main;
 	envre->scene = re->scene;    /* unsure about this... */
+	envre->scene_color_manage = re->scene_color_manage;
 	envre->lay = re->lay;
 
 	/* view stuff in env render */
 	viewscale = (env->type == ENV_PLANE) ? env->viewscale : 1.0f;
 	RE_SetEnvmapCamera(envre, env->object, viewscale, env->clipsta, env->clipend);
+	copy_m4_m4(envre->viewmat_orig, re->viewmat_orig);
 	
 	/* callbacks */
-	envre->display_draw = re->display_draw;
-	envre->ddh = re->ddh;
+	envre->display_update = re->display_update;
+	envre->duh = re->duh;
 	envre->test_break = re->test_break;
 	envre->tbh = re->tbh;
+	envre->current_scene_update = re->current_scene_update;
+	envre->suh = re->suh;
 	
 	/* and for the evil stuff; copy the database... */
 	envre->totvlak = re->totvlak;
@@ -195,11 +195,11 @@ static void envmap_free_render_copy(Render *envre)
 	envre->totlamp = 0;
 	envre->totinstance = 0;
 	envre->sortedhalos = NULL;
-	envre->lights.first = envre->lights.last = NULL;
-	envre->objecttable.first = envre->objecttable.last = NULL;
-	envre->customdata_names.first = envre->customdata_names.last = NULL;
+	BLI_listbase_clear(&envre->lights);
+	BLI_listbase_clear(&envre->objecttable);
+	BLI_listbase_clear(&envre->customdata_names);
 	envre->raytree = NULL;
-	envre->instancetable.first = envre->instancetable.last = NULL;
+	BLI_listbase_clear(&envre->instancetable);
 	envre->objectinstance = NULL;
 	envre->qmcsamplers = NULL;
 	
@@ -208,14 +208,14 @@ static void envmap_free_render_copy(Render *envre)
 
 /* ------------------------------------------------------------------------- */
 
-static void envmap_transmatrix(float mat[][4], int part)
+static void envmap_transmatrix(float mat[4][4], int part)
 {
 	float tmat[4][4], eul[3], rotmat[4][4];
 	
 	eul[0] = eul[1] = eul[2] = 0.0;
 	
 	if (part == 0) {          /* neg z */
-		;
+		/* pass */
 	}
 	else if (part == 1) { /* pos z */
 		eul[0] = M_PI;
@@ -238,39 +238,57 @@ static void envmap_transmatrix(float mat[][4], int part)
 	
 	copy_m4_m4(tmat, mat);
 	eul_to_mat4(rotmat, eul);
-	mul_serie_m4(mat, tmat, rotmat,
-	             NULL, NULL, NULL,
-	             NULL, NULL, NULL);
+	mul_m4_m4m4(mat, tmat, rotmat);
+}
+/* ------------------------------------------------------------------------- */
+
+static void env_set_imats(Render *re)
+{
+	Base *base;
+	float mat[4][4];
+	
+	base = re->scene->base.first;
+	while (base) {
+		mul_m4_m4m4(mat, re->viewmat, base->object->obmat);
+		invert_m4_m4(base->object->imat, mat);
+		
+		base = base->next;
+	}
+	
 }
 
 /* ------------------------------------------------------------------------- */
 
-static void env_rotate_scene(Render *re, float mat[][4], int mode)
+void env_rotate_scene(Render *re, float mat[4][4], int do_rotate)
 {
 	GroupObject *go;
 	ObjectRen *obr;
 	ObjectInstanceRen *obi;
 	LampRen *lar = NULL;
 	HaloRen *har = NULL;
-	float imat[3][3], pmat[4][4], smat[4][4], tmat[4][4], cmat[3][3], tmpmat[4][4];
+	float imat[3][3], mat_inverse[4][4], smat[4][4], tmat[4][4], cmat[3][3], tmpmat[4][4];
 	int a;
 	
-	if (mode == 0) {
+	if (do_rotate == 0) {
 		invert_m4_m4(tmat, mat);
 		copy_m3_m4(imat, tmat);
+		
+		copy_m4_m4(mat_inverse, mat);
 	}
 	else {
 		copy_m4_m4(tmat, mat);
 		copy_m3_m4(imat, mat);
+		
+		invert_m4_m4(mat_inverse, tmat);
 	}
 
 	for (obi = re->instancetable.first; obi; obi = obi->next) {
 		/* append or set matrix depending on dupli */
 		if (obi->flag & R_DUPLI_TRANSFORMED) {
 			copy_m4_m4(tmpmat, obi->mat);
-			mult_m4_m4m4(obi->mat, tmat, tmpmat);
+			mul_m4_m4m4(obi->mat, tmat, tmpmat);
 		}
-		else if (mode == 1)
+		else if (do_rotate == 1)
 			copy_m4_m4(obi->mat, tmat);
 		else
 			unit_m4(obi->mat);
@@ -280,10 +298,12 @@ static void env_rotate_scene(Render *re, float mat[][4], int mode)
 		transpose_m3(obi->nmat);
 
 		/* indicate the renderer has to use transform matrices */
-		if (mode == 0)
+		if (do_rotate == 0)
 			obi->flag &= ~R_ENV_TRANSFORMED;
-		else
+		else {
 			obi->flag |= R_ENV_TRANSFORMED;
+			copy_m4_m4(obi->imat, mat_inverse);
+		}
 	}
 	
 
@@ -294,36 +314,62 @@ static void env_rotate_scene(Render *re, float mat[][4], int mode)
 		
 			mul_m4_v3(tmat, har->co);
 		}
+
+		/* imat_ren is needed for correct texture coordinates */
+		mul_m4_m4m4(obr->ob->imat_ren, re->viewmat, obr->ob->obmat);
+		invert_m4(obr->ob->imat_ren);
 	}
 	
 	for (go = re->lights.first; go; go = go->next) {
 		lar = go->lampren;
 		
-		/* removed here some horrible code of someone in NaN who tried to fix
-		 * prototypes... just solved by introducing a correct cmat[3][3] instead
-		 * of using smat. this works, check square spots in reflections  (ton) */
-		copy_m3_m3(cmat, lar->imat); 
-		mul_m3_m3m3(lar->imat, cmat, imat); 
-
-		mul_m3_v3(imat, lar->vec);
-		mul_m4_v3(tmat, lar->co);
-
-		lar->sh_invcampos[0] = -lar->co[0];
-		lar->sh_invcampos[1] = -lar->co[1];
-		lar->sh_invcampos[2] = -lar->co[2];
-		mul_m3_v3(lar->imat, lar->sh_invcampos);
-		lar->sh_invcampos[2] *= lar->sh_zfac;
+		/* copy from add_render_lamp */
+		if (do_rotate == 1)
+			mul_m4_m4m4(tmpmat, re->viewmat, lar->lampmat);
+		else
+			mul_m4_m4m4(tmpmat, re->viewmat_orig, lar->lampmat);
+		invert_m4_m4(go->ob->imat, tmpmat);
 		
-		if (lar->shb) {
-			if (mode == 1) {
-				invert_m4_m4(pmat, mat);
-				mult_m4_m4m4(smat, lar->shb->viewmat, pmat);
-				mult_m4_m4m4(lar->shb->persmat, lar->shb->winmat, smat);
+		copy_m3_m4(lar->mat, tmpmat);
+		
+		copy_m3_m4(lar->imat, go->ob->imat);
+
+		lar->vec[0]= -tmpmat[2][0];
+		lar->vec[1]= -tmpmat[2][1];
+		lar->vec[2]= -tmpmat[2][2];
+		normalize_v3(lar->vec);
+		lar->co[0]= tmpmat[3][0];
+		lar->co[1]= tmpmat[3][1];
+		lar->co[2]= tmpmat[3][2];
+
+		if (lar->type == LA_AREA) {
+			area_lamp_vectors(lar);
+		}
+		else if (lar->type == LA_SPOT) {
+			normalize_v3(lar->imat[0]);
+			normalize_v3(lar->imat[1]);
+			normalize_v3(lar->imat[2]);
+		
+			lar->sh_invcampos[0] = -lar->co[0];
+			lar->sh_invcampos[1] = -lar->co[1];
+			lar->sh_invcampos[2] = -lar->co[2];
+			mul_m3_v3(lar->imat, lar->sh_invcampos);
+			lar->sh_invcampos[2] *= lar->sh_zfac;
+		
+			if (lar->shb) {
+				if (do_rotate == 1) {
+					mul_m4_m4m4(smat, lar->shb->viewmat, mat_inverse);
+					mul_m4_m4m4(lar->shb->persmat, lar->shb->winmat, smat);
+				}
+				else mul_m4_m4m4(lar->shb->persmat, lar->shb->winmat, lar->shb->viewmat);
 			}
-			else mult_m4_m4m4(lar->shb->persmat, lar->shb->winmat, lar->shb->viewmat);
 		}
 	}
 	
+	if (do_rotate) {
+		init_render_world(re);
+		env_set_imats(re);
+	}
 }
 
 /* ------------------------------------------------------------------------- */
@@ -391,23 +437,6 @@ static void env_showobjects(Render *re)
 
 /* ------------------------------------------------------------------------- */
 
-static void env_set_imats(Render *re)
-{
-	Base *base;
-	float mat[4][4];
-	
-	base = re->scene->base.first;
-	while (base) {
-		mult_m4_m4m4(mat, re->viewmat, base->object->obmat);
-		invert_m4_m4(base->object->imat, mat);
-		
-		base = base->next;
-	}
-
-}	
-
-/* ------------------------------------------------------------------------- */
-
 static void render_envmap(Render *re, EnvMap *env)
 {
 	/* only the cubemap and planar map is implemented */
@@ -427,7 +456,7 @@ static void render_envmap(Render *re, EnvMap *env)
 	normalize_m4(orthmat);
 	
 	/* need imat later for texture imat */
-	mult_m4_m4m4(mat, re->viewmat, orthmat);
+	mul_m4_m4m4(mat, re->viewmat, orthmat);
 	invert_m4_m4(tmat, mat);
 	copy_m3_m4(env->obimat, tmat);
 
@@ -446,15 +475,13 @@ static void render_envmap(Render *re, EnvMap *env)
 		copy_m4_m4(envre->viewinv, tmat);
 		
 		/* we have to correct for the already rotated vertexcoords */
-		mult_m4_m4m4(tmat, envre->viewmat, oldviewinv);
+		mul_m4_m4m4(tmat, envre->viewmat, oldviewinv);
 		invert_m4_m4(env->imat, tmat);
 		
 		env_rotate_scene(envre, tmat, 1);
-		init_render_world(envre);
 		project_renderdata(envre, projectverto, 0, 0, 1);
 		env_layerflags(envre, env->notlay);
 		env_hideobject(envre, env->object);
-		env_set_imats(envre);
 				
 		if (re->test_break(re->tbh) == 0) {
 			RE_TileProcessor(envre);
@@ -502,7 +529,8 @@ static void render_envmap(Render *re, EnvMap *env)
 void make_envmaps(Render *re)
 {
 	Tex *tex;
-	int do_init = FALSE, depth = 0, trace;
+	bool do_init = false;
+	int depth = 0, trace;
 	
 	if (!(re->r.mode & R_ENVMAP)) return;
 	
@@ -510,7 +538,7 @@ void make_envmaps(Render *re)
 	trace = (re->r.mode & R_RAYTRACE);
 	re->r.mode &= ~R_RAYTRACE;
 
-	re->i.infostr = "Creating Environment maps";
+	re->i.infostr = IFACE_("Creating Environment maps");
 	re->stats_draw(re->sdh, &re->i);
 	
 	/* 5 = hardcoded max recursion level */
@@ -530,7 +558,7 @@ void make_envmaps(Render *re)
 							normalize_m4(orthmat);
 							
 							/* need imat later for texture imat */
-							mult_m4_m4m4(mat, re->viewmat, orthmat);
+							mul_m4_m4m4(mat, re->viewmat, orthmat);
 							invert_m4_m4(tmat, mat);
 							copy_m3_m4(env->obimat, tmat);
 						}
@@ -556,7 +584,7 @@ void make_envmaps(Render *re)
 								if (env->ok == 0 && depth == 0) env->recalc = 1;
 								
 								if (env->ok == 0) {
-									do_init = TRUE;
+									do_init = true;
 									render_envmap(re, env);
 									
 									if (depth == env->depth) env->recalc = 0;
@@ -575,7 +603,7 @@ void make_envmaps(Render *re)
 		re->display_init(re->dih, re->result);
 		re->display_clear(re->dch, re->result);
 		// re->flag |= R_REDRAW_PRV;
-	}	
+	}
 	/* restore */
 	re->r.mode |= trace;
 
@@ -585,53 +613,53 @@ void make_envmaps(Render *re)
 
 static int envcube_isect(EnvMap *env, const float vec[3], float answ[2])
 {
-	float labda;
+	float lambda;
 	int face;
 	
 	if (env->type == ENV_PLANE) {
 		face = 1;
 		
-		labda = 1.0f / vec[2];
-		answ[0] = env->viewscale * labda * vec[0];
-		answ[1] = -env->viewscale * labda * vec[1];
+		lambda = 1.0f / vec[2];
+		answ[0] = env->viewscale * lambda * vec[0];
+		answ[1] = -env->viewscale * lambda * vec[1];
 	}
 	else {
 		/* which face */
 		if (vec[2] <= -fabsf(vec[0]) && vec[2] <= -fabsf(vec[1]) ) {
 			face = 0;
-			labda = -1.0f / vec[2];
-			answ[0] = labda * vec[0];
-			answ[1] = labda * vec[1];
+			lambda = -1.0f / vec[2];
+			answ[0] = lambda * vec[0];
+			answ[1] = lambda * vec[1];
 		}
 		else if (vec[2] >= fabsf(vec[0]) && vec[2] >= fabsf(vec[1])) {
 			face = 1;
-			labda = 1.0f / vec[2];
-			answ[0] = labda * vec[0];
-			answ[1] = -labda * vec[1];
+			lambda = 1.0f / vec[2];
+			answ[0] = lambda * vec[0];
+			answ[1] = -lambda * vec[1];
 		}
 		else if (vec[1] >= fabsf(vec[0])) {
 			face = 2;
-			labda = 1.0f / vec[1];
-			answ[0] = labda * vec[0];
-			answ[1] = labda * vec[2];
+			lambda = 1.0f / vec[1];
+			answ[0] = lambda * vec[0];
+			answ[1] = lambda * vec[2];
 		}
 		else if (vec[0] <= -fabsf(vec[1])) {
 			face = 3;
-			labda = -1.0f / vec[0];
-			answ[0] = labda * vec[1];
-			answ[1] = labda * vec[2];
+			lambda = -1.0f / vec[0];
+			answ[0] = lambda * vec[1];
+			answ[1] = lambda * vec[2];
 		}
 		else if (vec[1] <= -fabsf(vec[0])) {
 			face = 4;
-			labda = -1.0f / vec[1];
-			answ[0] = -labda * vec[0];
-			answ[1] = labda * vec[2];
+			lambda = -1.0f / vec[1];
+			answ[0] = -lambda * vec[0];
+			answ[1] = lambda * vec[2];
 		}
 		else {
 			face = 5;
-			labda = 1.0f / vec[0];
-			answ[0] = -labda * vec[1];
-			answ[1] = labda * vec[2];
+			lambda = 1.0f / vec[0];
+			answ[0] = -lambda * vec[1];
+			answ[1] = lambda * vec[2];
 		}
 	}
 	
@@ -642,31 +670,31 @@ static int envcube_isect(EnvMap *env, const float vec[3], float answ[2])
 
 /* ------------------------------------------------------------------------- */
 
-static void set_dxtdyt(float *dxts, float *dyts, float *dxt, float *dyt, int face)
+static void set_dxtdyt(float r_dxt[3], float r_dyt[3], const float dxt[3], const float dyt[3], int face)
 {
 	if (face == 2 || face == 4) {
-		dxts[0] = dxt[0];
-		dyts[0] = dyt[0];
-		dxts[1] = dxt[2];
-		dyts[1] = dyt[2];
+		r_dxt[0] = dxt[0];
+		r_dyt[0] = dyt[0];
+		r_dxt[1] = dxt[2];
+		r_dyt[1] = dyt[2];
 	}
 	else if (face == 3 || face == 5) {
-		dxts[0] = dxt[1];
-		dxts[1] = dxt[2];
-		dyts[0] = dyt[1];
-		dyts[1] = dyt[2];
+		r_dxt[0] = dxt[1];
+		r_dxt[1] = dxt[2];
+		r_dyt[0] = dyt[1];
+		r_dyt[1] = dyt[2];
 	}
 	else {
-		dxts[0] = dxt[0];
-		dyts[0] = dyt[0];
-		dxts[1] = dxt[1];
-		dyts[1] = dyt[1];
+		r_dxt[0] = dxt[0];
+		r_dyt[0] = dyt[0];
+		r_dxt[1] = dxt[1];
+		r_dyt[1] = dyt[1];
 	}
 }
 
 /* ------------------------------------------------------------------------- */
 
-int envmaptex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexResult *texres)
+int envmaptex(Tex *tex, const float texvec[3], float dxt[3], float dyt[3], int osatex, TexResult *texres, struct ImagePool *pool)
 {
 	extern Render R;                /* only in this call */
 	/* texvec should be the already reflected normal */
@@ -685,11 +713,16 @@ int envmaptex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexRe
 		env->ima = tex->ima;
 		if (env->ima && env->ima->ok) {
 			if (env->cube[1] == NULL) {
-				ImBuf *ibuf_ima = BKE_image_get_ibuf(env->ima, NULL);
+				ImBuf *ibuf_ima = BKE_image_pool_acquire_ibuf(env->ima, NULL, pool);
 				if (ibuf_ima)
 					envmap_split_ima(env, ibuf_ima);
 				else
 					env->ok = 0;
+
+				if (env->type == ENV_PLANE)
+					tex->extend = TEX_EXTEND;
+
+				BKE_image_pool_release_ibuf(env->ima, ibuf_ima, pool);
 			}
 		}
 	}
@@ -717,7 +750,7 @@ int envmaptex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexRe
 			mul_mat3_m4_v3(R.viewinv, dyt);
 		}
 		set_dxtdyt(dxts, dyts, dxt, dyt, face);
-		imagewraposa(tex, NULL, ibuf, sco, dxts, dyts, texres);
+		imagewraposa(tex, NULL, ibuf, sco, dxts, dyts, texres, pool);
 		
 		/* edges? */
 		
@@ -734,7 +767,7 @@ int envmaptex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexRe
 			if (face != face1) {
 				ibuf = env->cube[face1];
 				set_dxtdyt(dxts, dyts, dxt, dyt, face1);
-				imagewraposa(tex, NULL, ibuf, sco, dxts, dyts, &texr1);
+				imagewraposa(tex, NULL, ibuf, sco, dxts, dyts, &texr1, pool);
 			}
 			else texr1.tr = texr1.tg = texr1.tb = texr1.ta = 0.0;
 			
@@ -747,7 +780,7 @@ int envmaptex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexRe
 			if (face != face1) {
 				ibuf = env->cube[face1];
 				set_dxtdyt(dxts, dyts, dxt, dyt, face1);
-				imagewraposa(tex, NULL, ibuf, sco, dxts, dyts, &texr2);
+				imagewraposa(tex, NULL, ibuf, sco, dxts, dyts, &texr2, pool);
 			}
 			else texr2.tr = texr2.tg = texr2.tb = texr2.ta = 0.0;
 			
@@ -763,7 +796,7 @@ int envmaptex(Tex *tex, float *texvec, float *dxt, float *dyt, int osatex, TexRe
 		}
 	}
 	else {
-		imagewrap(tex, NULL, ibuf, sco, texres);
+		imagewrap(tex, NULL, ibuf, sco, texres, pool);
 	}
 	
 	return 1;
