@@ -36,34 +36,17 @@
 
 #include "MEM_guardedalloc.h"
 
-#include "PIL_time.h"
-
 #include "BLI_math.h"
 #include "BLI_blenlib.h"
 #include "BLI_jitter.h"
 #include "BLI_utildefines.h"
 
 #include "DNA_camera_types.h"
-#include "DNA_group_types.h"
 #include "DNA_image_types.h"
-#include "DNA_lamp_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
 
-
 #include "BKE_camera.h"
-#include "BKE_global.h"
-#include "BKE_material.h"
-#include "BKE_object.h"
-#include "BKE_image.h"
-#include "BKE_ipo.h"
-#include "BKE_key.h"
-#include "BKE_action.h"
-#include "BKE_writeavi.h"
-#include "BKE_scene.h"
-
-#include "IMB_imbuf_types.h"
-#include "IMB_imbuf.h"
 
 #ifdef WITH_QUICKTIME
 #include "quicktime_export.h"
@@ -73,10 +56,6 @@
 #include "renderpipeline.h"
 #include "render_types.h"
 
-#include "rendercore.h"
-#include "pixelshading.h"
-#include "zbuf.h"
-
 /* Own includes */
 #include "initrender.h"
 
@@ -92,10 +71,10 @@ static void init_render_jit(Render *re)
 	
 	if (lastjit != re->r.osa || last_mblur_jit != re->r.mblur_samples) {
 		memset(jit, 0, sizeof(jit));
-		BLI_jitter_init(jit[0], re->r.osa);
+		BLI_jitter_init(jit, re->r.osa);
 		
 		memset(mblur_jit, 0, sizeof(mblur_jit));
-		BLI_jitter_init(mblur_jit[0], re->r.mblur_samples);
+		BLI_jitter_init(mblur_jit, re->r.mblur_samples);
 	}
 	
 	lastjit = re->r.osa;
@@ -175,8 +154,11 @@ float RE_filter_value(int type, float x)
 			return 1.0f - x;
 			
 		case R_FILTER_GAUSS:
-			x *= gaussfac;
-			return (1.0f / expf(x * x) - 1.0f / expf(gaussfac * gaussfac * 2.25f));
+		{
+			const float two_gaussfac2 = 2.0f * gaussfac * gaussfac;
+			x *= 3.0f * gaussfac;
+			return 1.0f / sqrtf(M_PI * two_gaussfac2) * expf(-x*x / two_gaussfac2);
+		}
 			
 		case R_FILTER_MITCH:
 			return filt_mitchell(x * gaussfac);
@@ -201,7 +183,7 @@ static float calc_weight(Render *re, float *weight, int i, int j)
 	for (a = 0; a < re->osa; a++) {
 		x = re->jit[a][0] + i;
 		y = re->jit[a][1] + j;
-		dist = sqrt(x * x + y * y);
+		dist = sqrtf(x * x + y * y);
 
 		weight[a] = 0.0;
 
@@ -268,7 +250,7 @@ void make_sample_tables(Render *re)
 	SampleTables *st;
 	float flweight[32];
 	float weight[32], totw, val, *fpx1, *fpx2, *fpy1, *fpy2, *m3, *m4;
-	int i, j, a;
+	int i, j, a, centmasksize;
 
 	/* optimization tables, only once */
 	if (firsttime) {
@@ -303,7 +285,8 @@ void make_sample_tables(Render *re)
 		if (a & 128) st->cmask[a]++;
 	}
 	
-	st->centmask = MEM_mallocN((1 << re->osa), "Initfilt3");
+	centmasksize = (1 << re->osa);
+	st->centmask = MEM_mallocN(centmasksize, "Initfilt3");
 	
 	for (a = 0; a < 16; a++) {
 		st->centLut[a] = -0.45f + ((float)a) / 16.0f;
@@ -425,7 +408,7 @@ void make_sample_tables(Render *re)
 		}
 	}
 
-	for (a = (1 << re->osa) - 1; a > 0; a--) {
+	for (a = centmasksize - 1; a > 0; a--) {
 		val = st->cmask[a & 255] + st->cmask[a >> 8];
 		i = 8 + (15.9f * (fpy1[a & 255] + fpy2[a >> 8]) / val);
 		CLAMP(i, 0, 15);
@@ -515,7 +498,7 @@ void RE_SetPixelSize(Render *re, float pixsize)
 	re->viewdy = re->ycor * pixsize;
 }
 
-void RE_GetCameraWindow(struct Render *re, struct Object *camera, int frame, float mat[][4])
+void RE_GetCameraWindow(struct Render *re, struct Object *camera, int frame, float mat[4][4])
 {
 	re->r.cfra = frame;
 	RE_SetCamera(re, camera);
@@ -525,7 +508,7 @@ void RE_GetCameraWindow(struct Render *re, struct Object *camera, int frame, flo
 /* ~~~~~~~~~~~~~~~~ part (tile) calculus ~~~~~~~~~~~~~~~~~~~~~~ */
 
 
-void freeparts(Render *re)
+void RE_parts_free(Render *re)
 {
 	RenderPart *part = re->parts.first;
 	
@@ -537,17 +520,24 @@ void freeparts(Render *re)
 	BLI_freelistN(&re->parts);
 }
 
-void initparts(Render *re, int do_crop)
+void RE_parts_clamp(Render *re)
+{
+	/* part size */
+	re->partx = max_ii(1, min_ii(re->r.tilex, re->rectx));
+	re->party = max_ii(1, min_ii(re->r.tiley, re->recty));
+}
+
+void RE_parts_init(Render *re, bool do_crop)
 {
 	int nr, xd, yd, partx, party, xparts, yparts;
 	int xminb, xmaxb, yminb, ymaxb;
 	
-	freeparts(re);
+	RE_parts_free(re);
 	
 	/* this is render info for caller, is not reset when parts are freed! */
 	re->i.totpart = 0;
 	re->i.curpart = 0;
-	re->i.partsdone = FALSE;
+	re->i.partsdone = 0;
 	
 	/* just for readable code.. */
 	xminb = re->disprect.xmin;
@@ -555,31 +545,13 @@ void initparts(Render *re, int do_crop)
 	xmaxb = re->disprect.xmax;
 	ymaxb = re->disprect.ymax;
 	
-	xparts = re->r.xparts;
-	yparts = re->r.yparts;
-	
-	/* mininum part size, but for exr tile saving it was checked already */
-	if (!(re->r.scemode & (R_EXR_TILE_FILE | R_FULL_SAMPLE))) {
-		if (re->r.mode & R_PANORAMA) {
-			if (ceil(re->rectx / (float)xparts) < 8)
-				xparts = 1 + re->rectx / 8;
-		}
-		else
-		if (ceil(re->rectx / (float)xparts) < 64)
-			xparts = 1 + re->rectx / 64;
-		
-		if (ceil(re->recty / (float)yparts) < 64)
-			yparts = 1 + re->recty / 64;
-	}
-	
-	/* part size */
-	partx = ceil(re->rectx / (float)xparts);
-	party = ceil(re->recty / (float)yparts);
-	
-	re->xparts = xparts;
-	re->yparts = yparts;
-	re->partx = partx;
-	re->party = party;
+	RE_parts_clamp(re);
+
+	partx = re->partx;
+	party = re->party;
+	/* part count */
+	xparts = (re->rectx + partx - 1) / partx;
+	yparts = (re->recty + party - 1) / party;
 	
 	/* calculate rotation factor of 1 pixel */
 	if (re->r.mode & R_PANORAMA)
