@@ -36,14 +36,15 @@
 
 #include "BLI_math.h"
 #include "BLI_blenlib.h"
+#include "BLI_noise.h"
 #include "BLI_kdopbvh.h"
 #include "BLI_utildefines.h"
 
+#include "BLF_translation.h"
+
 #include "BKE_DerivedMesh.h"
-#include "BKE_global.h"
 #include "BKE_lattice.h"
 #include "BKE_main.h"
-#include "BKE_object.h"
 #include "BKE_particle.h"
 #include "BKE_scene.h"
 #include "BKE_texture.h"
@@ -54,7 +55,6 @@
 #include "DNA_particle_types.h"
 
 #include "render_types.h"
-#include "renderdatabase.h"
 #include "texture.h"
 #include "pointdensity.h"
 
@@ -104,6 +104,7 @@ static void pointdensity_cache_psys(Render *re, PointDensity *pd, Object *ob, Pa
 {
 	DerivedMesh* dm;
 	ParticleKey state;
+	ParticleCacheKey *cache;
 	ParticleSimulationData sim= {NULL};
 	ParticleData *pa=NULL;
 	float cfra = BKE_scene_frame_get(re->scene);
@@ -116,7 +117,7 @@ static void pointdensity_cache_psys(Render *re, PointDensity *pd, Object *ob, Pa
 	/* init everything */
 	if (!psys || !ob || !pd) return;
 
-	mult_m4_m4m4(obview, ob->obmat, re->viewinv);
+	mul_m4_m4m4(obview, ob->obmat, re->viewinv);
 	
 	/* Just to create a valid rendering context for particles */
 	psys_render_set(ob, psys, re->viewmat, re->winmat, re->winx, re->winy, 0);
@@ -136,7 +137,7 @@ static void pointdensity_cache_psys(Render *re, PointDensity *pd, Object *ob, Pa
 	invert_m4_m4(ob->imat, ob->obmat);
 	
 	total_particles = psys->totpart+psys->totchild;
-	psys->lattice=psys_get_lattice(&sim);
+	psys->lattice_deform_data = psys_create_lattice_deform_data(&sim);
 	
 	pd->point_tree = BLI_bvhtree_new(total_particles, 0.0, 4, 6);
 	alloc_point_data(pd, total_particles, data_used);
@@ -150,52 +151,70 @@ static void pointdensity_cache_psys(Render *re, PointDensity *pd, Object *ob, Pa
 
 	for (i=0, pa=psys->particles; i < total_particles; i++, pa++) {
 
-		state.time = cfra;
-		if (psys_get_particle_state(&sim, i, &state, 0)) {
-			
-			copy_v3_v3(partco, state.co);
-			
-			if (pd->psys_cache_space == TEX_PD_OBJECTSPACE)
-				mul_m4_v3(ob->imat, partco);
-			else if (pd->psys_cache_space == TEX_PD_OBJECTLOC) {
-				sub_v3_v3(partco, ob->loc);
-			}
-			else {
-				/* TEX_PD_WORLDSPACE */
-			}
-			
-			BLI_bvhtree_insert(pd->point_tree, i, partco, 1);
-			
-			if (data_used & POINT_DATA_VEL) {
-				pd->point_data[i*3 + 0] = state.vel[0];
-				pd->point_data[i*3 + 1] = state.vel[1];
-				pd->point_data[i*3 + 2] = state.vel[2];
-			} 
+		if (psys->part->type == PART_HAIR) {
+			/* hair particles */
+			if (i < psys->totpart && psys->pathcache)
+				cache = psys->pathcache[i];
+			else if (i >= psys->totpart && psys->childcache)
+				cache = psys->childcache[i - psys->totpart];
+			else
+				continue;
+
+			cache += cache->steps; /* use endpoint */
+
+			copy_v3_v3(state.co, cache->co);
+			zero_v3(state.vel);
+			state.time = 0.0f;
+		}
+		else {
+			/* emitter particles */
+			state.time = cfra;
+
+			if (!psys_get_particle_state(&sim, i, &state, 0))
+				continue;
+
 			if (data_used & POINT_DATA_LIFE) {
-				float pa_time;
-				
 				if (i < psys->totpart) {
-					pa_time = (cfra - pa->time)/pa->lifetime;
+					state.time = (cfra - pa->time)/pa->lifetime;
 				}
 				else {
 					ChildParticle *cpa= (psys->child + i) - psys->totpart;
 					float pa_birthtime, pa_dietime;
 					
-					pa_time = psys_get_child_time(psys, cpa, cfra, &pa_birthtime, &pa_dietime);
+					state.time = psys_get_child_time(psys, cpa, cfra, &pa_birthtime, &pa_dietime);
 				}
-				
-				pd->point_data[offset + i] = pa_time;
-				
 			}
 		}
+
+		copy_v3_v3(partco, state.co);
+		
+		if (pd->psys_cache_space == TEX_PD_OBJECTSPACE)
+			mul_m4_v3(ob->imat, partco);
+		else if (pd->psys_cache_space == TEX_PD_OBJECTLOC) {
+			sub_v3_v3(partco, ob->loc);
+		}
+		else {
+			/* TEX_PD_WORLDSPACE */
+		}
+		
+		BLI_bvhtree_insert(pd->point_tree, i, partco, 1);
+		
+		if (data_used & POINT_DATA_VEL) {
+			pd->point_data[i*3 + 0] = state.vel[0];
+			pd->point_data[i*3 + 1] = state.vel[1];
+			pd->point_data[i*3 + 2] = state.vel[2];
+		}
+		if (data_used & POINT_DATA_LIFE) {
+			pd->point_data[offset + i] = state.time;
+		}
 	}
 	
 	BLI_bvhtree_balance(pd->point_tree);
 	dm->release(dm);
 	
-	if (psys->lattice) {
-		end_latt_deform(psys->lattice);
-		psys->lattice=0;
+	if (psys->lattice_deform_data) {
+		end_latt_deform(psys->lattice_deform_data);
+		psys->lattice_deform_data = NULL;
 	}
 	
 	psys_render_restore(ob, psys);
@@ -295,10 +314,10 @@ void make_pointdensities(Render *re)
 {
 	Tex *tex;
 	
-	if (re->scene->r.scemode & R_PREVIEWBUTS)
+	if (re->scene->r.scemode & R_BUTS_PREVIEW)
 		return;
 	
-	re->i.infostr= "Caching Point Densities";
+	re->i.infostr = IFACE_("Caching Point Densities");
 	re->stats_draw(re->sdh, &re->i);
 
 	for (tex= re->main->tex.first; tex; tex= tex->id.next) {
@@ -307,7 +326,7 @@ void make_pointdensities(Render *re)
 		}
 	}
 	
-	re->i.infostr= NULL;
+	re->i.infostr = NULL;
 	re->stats_draw(re->sdh, &re->i);
 }
 
@@ -315,7 +334,7 @@ void free_pointdensities(Render *re)
 {
 	Tex *tex;
 	
-	if (re->scene->r.scemode & R_PREVIEWBUTS)
+	if (re->scene->r.scemode & R_BUTS_PREVIEW)
 		return;
 	
 	for (tex= re->main->tex.first; tex; tex= tex->id.next) {
@@ -328,7 +347,7 @@ void free_pointdensities(Render *re)
 typedef struct PointDensityRangeData {
 	float *density;
 	float squared_radius;
-	float *point_data;
+	const float *point_data;
 	float *vec;
 	float softness;
 	short falloff_type;
@@ -364,7 +383,7 @@ static void accum_density(void *userdata, int index, float squared_dist)
 	else if (pdr->falloff_type == TEX_PD_FALLOFF_CONSTANT)
 		density = pdr->squared_radius;
 	else if (pdr->falloff_type == TEX_PD_FALLOFF_ROOT)
-		density = sqrt(dist);
+		density = sqrtf(dist);
 	else if (pdr->falloff_type == TEX_PD_FALLOFF_PARTICLE_AGE) {
 		if (pdr->point_data_used & POINT_DATA_LIFE)
 			density = dist*MIN2(pdr->point_data[pdr->offset + index], 1.0f);
@@ -405,7 +424,7 @@ static void init_pointdensityrangedata(PointDensity *pd, PointDensityRangeData *
 }
 
 
-int pointdensitytex(Tex *tex, float *texvec, TexResult *texres)
+int pointdensitytex(Tex *tex, const float texvec[3], TexResult *texres)
 {
 	int retval = TEX_INT;
 	PointDensity *pd = tex->pd;
@@ -418,7 +437,7 @@ int pointdensitytex(Tex *tex, float *texvec, TexResult *texres)
 	
 	texres->tin = 0.0f;
 	
-	if ((!pd) || (!pd->point_tree))		
+	if ((!pd) || (!pd->point_tree))
 		return 0;
 		
 	init_pointdensityrangedata(pd, &pdr, &density, vec, &age, 
@@ -446,7 +465,7 @@ int pointdensitytex(Tex *tex, float *texvec, TexResult *texres)
 			turb = BLI_gTurbulence(pd->noise_size, texvec[0]+age, texvec[1]+age, texvec[2]+age, pd->noise_depth, 0, pd->noise_basis);
 		}
 		else if (pd->noise_influence == TEX_PD_NOISE_TIME) {
-			time = R.cfra / (float)R.r.efra;
+			time = R.r.cfra / (float)R.r.efra;
 			turb = BLI_gTurbulence(pd->noise_size, texvec[0]+time, texvec[1]+time, texvec[2]+time, pd->noise_depth, 0, pd->noise_basis);
 			//turb = BLI_turbulence(pd->noise_size, texvec[0]+time, texvec[1]+time, texvec[2]+time, pd->noise_depth);
 		}
@@ -481,7 +500,7 @@ int pointdensitytex(Tex *tex, float *texvec, TexResult *texres)
 		case TEX_PD_COLOR_PARTAGE:
 			if (pd->coba) {
 				if (do_colorband(pd->coba, age, col)) {
-					texres->talpha = TRUE;
+					texres->talpha = true;
 					copy_v3_v3(&texres->tr, col);
 					texres->tin *= col[3];
 					texres->ta = texres->tin;
@@ -494,7 +513,7 @@ int pointdensitytex(Tex *tex, float *texvec, TexResult *texres)
 			
 			if (pd->coba) {
 				if (do_colorband(pd->coba, speed, col)) {
-					texres->talpha = TRUE;
+					texres->talpha = true;
 					copy_v3_v3(&texres->tr, col);
 					texres->tin *= col[3];
 					texres->ta = texres->tin;
@@ -503,7 +522,7 @@ int pointdensitytex(Tex *tex, float *texvec, TexResult *texres)
 			break;
 		}
 		case TEX_PD_COLOR_PARTVEL:
-			texres->talpha = TRUE;
+			texres->talpha = true;
 			mul_v3_fl(vec, pd->speed_scale);
 			copy_v3_v3(&texres->tr, vec);
 			texres->ta = texres->tin;
