@@ -35,9 +35,10 @@
 
 #include "MEM_guardedalloc.h"
 
+#include "BKE_appdir.h"
 #include "BLI_utildefines.h"
-#include "BLI_fileops.h"
 #include "BLI_listbase.h"
+#include "BLI_hash_md5.h"
 #include "BLI_path_util.h"
 #include "BLI_rect.h"
 #include "BLI_string.h"
@@ -47,7 +48,9 @@
 #include "BKE_global.h"
 #include "BKE_main.h"
 #include "BKE_report.h"
-#include "BKE_freestyle.h"
+#ifdef WITH_CYCLES_DEBUG
+#  include "BKE_scene.h"
+#endif
 
 #include "IMB_imbuf.h"
 #include "IMB_imbuf_types.h"
@@ -91,6 +94,8 @@ void render_result_free(RenderResult *res)
 		MEM_freeN(res->rectf);
 	if (res->text)
 		MEM_freeN(res->text);
+	if (res->error)
+		MEM_freeN(res->error);
 	
 	MEM_freeN(res);
 }
@@ -402,7 +407,7 @@ static int passtype_from_name(const char *str)
 
 /********************************** New **************************************/
 
-static void render_layer_add_pass(RenderResult *rr, RenderLayer *rl, int channels, int passtype)
+static RenderPass *render_layer_add_pass(RenderResult *rr, RenderLayer *rl, int channels, int passtype)
 {
 	const char *typestr = get_pass_name(passtype, 0);
 	RenderPass *rpass = MEM_callocN(sizeof(RenderPass), typestr);
@@ -438,8 +443,34 @@ static void render_layer_add_pass(RenderResult *rr, RenderLayer *rl, int channel
 				rect[x] = 10e10;
 		}
 	}
+	return rpass;
 }
 
+#ifdef WITH_CYCLES_DEBUG
+static const char *debug_pass_type_name_get(int debug_type)
+{
+	switch (debug_type) {
+		case RENDER_PASS_DEBUG_BVH_TRAVERSAL_STEPS:
+			return "BVH Traversal Steps";
+	}
+	return "Unknown";
+}
+
+static RenderPass *render_layer_add_debug_pass(RenderResult *rr,
+                                               RenderLayer *rl,
+                                               int channels,
+                                               int pass_type,
+                                               int debug_type)
+{
+	RenderPass *rpass = render_layer_add_pass(rr, rl, channels, pass_type);
+	rpass->debug_type = debug_type;
+	BLI_strncpy(rpass->name,
+	            debug_pass_type_name_get(debug_type),
+	            sizeof(rpass->name));
+	return rpass;
+}
+#endif
+
 /* called by main render as well for parts */
 /* will read info from Render *re to define layers */
 /* called in threads */
@@ -471,7 +502,7 @@ RenderResult *render_result_new(Render *re, rcti *partrct, int crop, int savebuf
 	rr->tilerect.ymax = partrct->ymax - re->disprect.ymin;
 	
 	if (savebuffers) {
-		rr->do_exr_tile = TRUE;
+		rr->do_exr_tile = true;
 	}
 
 	/* check renderdata for amount of layers */
@@ -481,10 +512,14 @@ RenderResult *render_result_new(Render *re, rcti *partrct, int crop, int savebuf
 			if (strcmp(srl->name, layername) != 0)
 				continue;
 
-		if ((re->r.scemode & R_SINGLE_LAYER) && nr != re->r.actlay)
-			continue;
-		if (srl->layflag & SCE_LAY_DISABLE)
-			continue;
+		if (re->r.scemode & R_SINGLE_LAYER) {
+			if (nr != re->r.actlay)
+				continue;
+		}
+		else {
+			if (srl->layflag & SCE_LAY_DISABLE)
+				continue;
+		}
 		
 		rl = MEM_callocN(sizeof(RenderLayer), "new render layer");
 		BLI_addtail(&rr->layers, rl);
@@ -574,6 +609,13 @@ RenderResult *render_result_new(Render *re, rcti *partrct, int crop, int savebuf
 			render_layer_add_pass(rr, rl, 3, SCE_PASS_SUBSURFACE_INDIRECT);
 		if (srl->passflag  & SCE_PASS_SUBSURFACE_COLOR)
 			render_layer_add_pass(rr, rl, 3, SCE_PASS_SUBSURFACE_COLOR);
+
+#ifdef WITH_CYCLES_DEBUG
+		if(BKE_scene_use_new_shading_nodes(re->scene)) {
+			render_layer_add_debug_pass(rr, rl, 1, SCE_PASS_DEBUG,
+			        RENDER_PASS_DEBUG_BVH_TRAVERSAL_STEPS);
+		}
+#endif
 	}
 	/* sss, previewrender and envmap don't do layers, so we make a default one */
 	if (BLI_listbase_is_empty(&rr->layers) && !(layername && layername[0])) {
@@ -585,6 +627,8 @@ RenderResult *render_result_new(Render *re, rcti *partrct, int crop, int savebuf
 
 		/* duplicate code... */
 		if (rr->do_exr_tile) {
+			rl->display_buffer = MEM_mapallocN(rectx * recty * sizeof(unsigned int), "Combined display space rgba");
+
 			rl->exrhandle = IMB_exr_get_handle();
 
 			IMB_exr_add_channel(rl->exrhandle, rl->name, "Combined.R", 0, 0, NULL);
@@ -664,7 +708,7 @@ static void ml_addpass_cb(void *UNUSED(base), void *lay, const char *str, float
 }
 
 /* from imbuf, if a handle was returned we convert this to render result */
-RenderResult *render_result_new_from_exr(void *exrhandle, const char *colorspace, int predivide, int rectx, int recty)
+RenderResult *render_result_new_from_exr(void *exrhandle, const char *colorspace, bool predivide, int rectx, int recty)
 {
 	RenderResult *rr = MEM_callocN(sizeof(RenderResult), __func__);
 	RenderLayer *rl;
@@ -771,12 +815,12 @@ static char *make_pass_name(RenderPass *rpass, int chan)
 
 /* filename already made absolute */
 /* called from within UI, saves both rendered result as a file-read result */
-int RE_WriteRenderResult(ReportList *reports, RenderResult *rr, const char *filename, int compress)
+bool RE_WriteRenderResult(ReportList *reports, RenderResult *rr, const char *filename, int compress)
 {
 	RenderLayer *rl;
 	RenderPass *rpass;
 	void *exrhandle = IMB_exr_get_handle();
-	int success;
+	bool success;
 
 	BLI_make_existing_file(filename);
 	
@@ -819,12 +863,12 @@ int RE_WriteRenderResult(ReportList *reports, RenderResult *rr, const char *file
 	/* when the filename has no permissions, this can fail */
 	if (IMB_exr_begin_write(exrhandle, filename, rr->rectx, rr->recty, compress)) {
 		IMB_exr_write_channels(exrhandle);
-		success = TRUE;
+		success = true;
 	}
 	else {
 		/* TODO, get the error from openexr's exception */
 		BKE_report(reports, RPT_ERROR, "Error writing render result (see console)");
-		success = FALSE;
+		success = false;
 	}
 	IMB_exr_close(exrhandle);
 
@@ -1001,13 +1045,13 @@ void render_result_exr_file_end(Render *re)
 			rl->exrhandle = NULL;
 		}
 
-		rr->do_exr_tile = FALSE;
+		rr->do_exr_tile = false;
 	}
 	
 	render_result_free_list(&re->fullresult, re->result);
 	re->result = NULL;
 
-	render_result_exr_file_read(re, 0);
+	render_result_exr_file_read_sample(re, 0);
 }
 
 /* save part into exr file */
@@ -1023,33 +1067,33 @@ void render_result_exr_file_path(Scene *scene, const char *layname, int sample,
 	char name[FILE_MAXFILE + MAX_ID_NAME + MAX_ID_NAME + 100], fi[FILE_MAXFILE];
 	
 	BLI_split_file_part(G.main->name, fi, sizeof(fi));
-	if (sample == 0)
+	if (sample == 0) {
 		BLI_snprintf(name, sizeof(name), "%s_%s_%s.exr", fi, scene->id.name + 2, layname);
-	else
+	}
+	else {
 		BLI_snprintf(name, sizeof(name), "%s_%s_%s%d.exr", fi, scene->id.name + 2, layname, sample);
+	}
 
-	BLI_make_file_string("/", filepath, BLI_temporary_dir(), name);
+	BLI_make_file_string("/", filepath, BKE_tempdir_session(), name);
 }
 
-/* only for temp buffer files, makes exact copy of render result */
-int render_result_exr_file_read(Render *re, int sample)
+/* only for temp buffer, makes exact copy of render result */
+int render_result_exr_file_read_sample(Render *re, int sample)
 {
 	RenderLayer *rl;
-	char str[FILE_MAX];
-	int success = TRUE;
+	char str[FILE_MAXFILE + MAX_ID_NAME + MAX_ID_NAME + 100] = "";
+	bool success = true;
 
 	RE_FreeRenderResult(re->result);
 	re->result = render_result_new(re, &re->disprect, 0, RR_USE_MEM, RR_ALL_LAYERS);
 
 	for (rl = re->result->layers.first; rl; rl = rl->next) {
-
 		render_result_exr_file_path(re->scene, rl->name, sample, str);
 		printf("read exr tmp file: %s\n", str);
 
 		if (!render_result_exr_file_read_path(re->result, rl, str)) {
 			printf("cannot read: %s\n", str);
-			success = FALSE;
-
+			success = false;
 		}
 	}
 
@@ -1108,6 +1152,65 @@ int render_result_exr_file_read_path(RenderResult *rr, RenderLayer *rl_single, c
 	return 1;
 }
 
+static void render_result_exr_file_cache_path(Scene *sce, const char *root, char *r_path)
+{
+	char filename_full[FILE_MAX + MAX_ID_NAME + 100], filename[FILE_MAXFILE], dirname[FILE_MAXDIR];
+	char path_digest[16] = {0};
+	char path_hexdigest[33];
+
+	/* If root is relative, use either current .blend file dir, or temp one if not saved. */
+	if (G.main->name[0]) {
+		BLI_split_dirfile(G.main->name, dirname, filename, sizeof(dirname), sizeof(filename));
+		BLI_replace_extension(filename, sizeof(filename), "");  /* strip '.blend' */
+		BLI_hash_md5_buffer(G.main->name, strlen(G.main->name), path_digest);
+	}
+	else {
+		BLI_strncpy(dirname, BKE_tempdir_base(), sizeof(dirname));
+		BLI_strncpy(filename, "UNSAVED", sizeof(filename));
+	}
+	BLI_hash_md5_to_hexdigest(path_digest, path_hexdigest);
+
+	/* Default to *non-volatile* tmp dir. */
+	if (*root == '\0') {
+		root = BKE_tempdir_base();
+	}
+
+	BLI_snprintf(filename_full, sizeof(filename_full), "cached_RR_%s_%s_%s.exr",
+	             filename, sce->id.name + 2, path_hexdigest);
+	BLI_make_file_string(dirname, r_path, root, filename_full);
+}
+
+void render_result_exr_file_cache_write(Render *re)
+{
+	RenderResult *rr = re->result;
+	char str[FILE_MAXFILE + FILE_MAXFILE + MAX_ID_NAME + 100];
+	char *root = U.render_cachedir;
+
+	render_result_exr_file_cache_path(re->scene, root, str);
+	printf("Caching exr file, %dx%d, %s\n", rr->rectx, rr->recty, str);
+	RE_WriteRenderResult(NULL, rr, str, 0);
+}
+
+/* For cache, makes exact copy of render result */
+bool render_result_exr_file_cache_read(Render *re)
+{
+	char str[FILE_MAXFILE + MAX_ID_NAME + MAX_ID_NAME + 100] = "";
+	char *root = U.render_cachedir;
+
+	RE_FreeRenderResult(re->result);
+	re->result = render_result_new(re, &re->disprect, 0, RR_USE_MEM, RR_ALL_LAYERS);
+
+	/* First try cache. */
+	render_result_exr_file_cache_path(re->scene, root, str);
+
+	printf("read exr cache file: %s\n", str);
+	if (!render_result_exr_file_read_path(re->result, NULL, str)) {
+		printf("cannot read: %s\n", str);
+		return false;
+	}
+	return true;
+}
+
 /*************************** Combined Pixel Rect *****************************/
 
 ImBuf *render_result_rect_to_ibuf(RenderResult *rr, RenderData *rd)
