@@ -47,9 +47,7 @@
 
 /* local include */
 #include "occlusion.h"
-#include "renderpipeline.h"
 #include "render_types.h"
-#include "pixelblending.h"
 #include "rendercore.h"
 #include "shadbuf.h"
 #include "sss.h"
@@ -57,6 +55,11 @@
 
 #include "shading.h" /* own include */
 
+/* could enable at some point but for now there are far too many conversions */
+#ifdef __GNUC__
+#  pragma GCC diagnostic ignored "-Wdouble-promotion"
+#endif
+
 /* ~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ */
 /* defined in pipeline.c, is hardcopy of active dynamic allocated Render */
 /* only to be used here in this file, it's for speed */
@@ -66,7 +69,7 @@ extern struct Render R;
 ListBase *get_lights(ShadeInput *shi)
 {
 	
-	if (R.r.scemode & R_PREVIEWBUTS)
+	if (R.r.scemode & R_BUTS_PREVIEW)
 		return &R.lights;
 	if (shi->light_override)
 		return &shi->light_override->gobject;
@@ -114,7 +117,7 @@ static void fogcolor(const float colf[3], float *rco, float *view)
 		addAlphaOverFloat(colf, hor);
 		
 		sub_v3_v3(vec, dview);
-	}	
+	}
 }
 #endif
 
@@ -139,7 +142,7 @@ float mistfactor(float zcor, float const co[3])
 				/* pass */
 			}
 			else {
-				fac = sqrt(fac);
+				fac = sqrtf(fac);
 			}
 		}
 		else {
@@ -168,7 +171,7 @@ float mistfactor(float zcor, float const co[3])
 		}
 	}
 
-	return (1.0f-fac)* (1.0f-R.wrld.misi);	
+	return (1.0f-fac)* (1.0f-R.wrld.misi);
 }
 
 static void spothalo(struct LampRen *lar, ShadeInput *shi, float *intens)
@@ -176,7 +179,8 @@ static void spothalo(struct LampRen *lar, ShadeInput *shi, float *intens)
 	double a, b, c, disc, nray[3], npos[3];
 	double t0, t1 = 0.0f, t2= 0.0f, t3;
 	float p1[3], p2[3], ladist, maxz = 0.0f, maxy = 0.0f, haint;
-	int snijp, do_clip = TRUE, use_yco = FALSE;
+	int cuts;
+	bool do_clip = true, use_yco = false;
 
 	*intens= 0.0f;
 	haint= lar->haint;
@@ -212,7 +216,7 @@ static void spothalo(struct LampRen *lar, ShadeInput *shi, float *intens)
 
 	/* rotate maxz */
 	if (shi->co[2]==0.0f) {
-		do_clip = FALSE;  /* for when halo at sky */
+		do_clip = false;  /* for when halo at sky */
 	}
 	else {
 		p1[0]= shi->co[0]-lar->co[0];
@@ -223,12 +227,12 @@ static void spothalo(struct LampRen *lar, ShadeInput *shi, float *intens)
 		maxz*= lar->sh_zfac;
 		maxy= lar->imat[0][1]*p1[0]+lar->imat[1][1]*p1[1]+lar->imat[2][1]*p1[2];
 
-		if (fabsf(nray[2]) < FLT_EPSILON) {
-			use_yco = TRUE;
+		if (fabs(nray[2]) < FLT_EPSILON) {
+			use_yco = true;
 		}
 	}
 	
-	/* scale z to make sure volume is normalized */	
+	/* scale z to make sure volume is normalized */
 	nray[2] *= (double)lar->sh_zfac;
 	/* nray does not need normalization */
 	
@@ -239,7 +243,7 @@ static void spothalo(struct LampRen *lar, ShadeInput *shi, float *intens)
 	b = nray[0] * npos[0] + nray[1] * npos[1] - nray[2]*npos[2];
 	c = npos[0] * npos[0] + npos[1] * npos[1] - npos[2]*npos[2];
 
-	snijp= 0;
+	cuts= 0;
 	if (fabs(a) < DBL_EPSILON) {
 		/*
 		 * Only one intersection point...
@@ -251,16 +255,16 @@ static void spothalo(struct LampRen *lar, ShadeInput *shi, float *intens)
 		
 		if (disc==0.0) {
 			t1=t2= (-b)/ a;
-			snijp= 2;
+			cuts= 2;
 		}
 		else if (disc > 0.0) {
 			disc = sqrt(disc);
 			t1 = (-b + disc) / a;
 			t2 = (-b - disc) / a;
-			snijp= 2;
+			cuts= 2;
 		}
 	}
-	if (snijp==2) {
+	if (cuts==2) {
 		int ok1=0, ok2=0;
 
 		/* sort */
@@ -280,7 +284,7 @@ static void spothalo(struct LampRen *lar, ShadeInput *shi, float *intens)
 		if (ok1==0 && ok2==0) return;
 		
 		/* intersction point with -ladist, the bottom of the cone */
-		if (use_yco == FALSE) {
+		if (use_yco == false) {
 			t3= ((double)(-ladist)-npos[2])/nray[2];
 				
 			/* de we have to replace one of the intersection points? */
@@ -314,8 +318,8 @@ static void spothalo(struct LampRen *lar, ShadeInput *shi, float *intens)
 		
 		/* calculate t0: is the maximum visible z (when halo is intersected by face) */ 
 		if (do_clip) {
-			if (use_yco == FALSE) t0 = (maxz - npos[2]) / nray[2];
-			else t0 = (maxy - npos[1]) / nray[1];
+			if (use_yco == false) t0 = ((double)maxz - npos[2]) / nray[2];
+			else                  t0 = ((double)maxy - npos[1]) / nray[1];
 
 			if (t0 < t1) return;
 			if (t0 < t2) t2= t0;
@@ -332,9 +336,9 @@ static void spothalo(struct LampRen *lar, ShadeInput *shi, float *intens)
 			
 		/* now we have 2 points, make three lengths with it */
 		
-		a= sqrt(p1[0]*p1[0]+p1[1]*p1[1]+p1[2]*p1[2]);
-		b= sqrt(p2[0]*p2[0]+p2[1]*p2[1]+p2[2]*p2[2]);
-		c= len_v3v3(p1, p2);
+		a = len_v3(p1);
+		b = len_v3(p2);
+		c = len_v3v3(p1, p2);
 		
 		a/= ladist;
 		a= sqrt(a);
@@ -468,9 +472,10 @@ static float area_lamp_energy(float (*area)[3], const float co[3], const float v
 
 	/* cross product */
 #define CROSS(dest, a, b) \
-	{ dest[0]= a[1] * b[2] - a[2] * b[1]; \
-	  dest[1]= a[2] * b[0] - a[0] * b[2]; \
-	  dest[2]= a[0] * b[1] - a[1] * b[0]; \
+	{ \
+		dest[0]= a[1] * b[2] - a[2] * b[1]; \
+		dest[1]= a[2] * b[0] - a[0] * b[2]; \
+		dest[2]= a[0] * b[1] - a[1] * b[0]; \
 	} (void)0
 
 	CROSS(cross[0], vec[0], vec[1]);
@@ -547,7 +552,7 @@ static float spec(float inp, int hard)
 	
 	b1= inp*inp;
 	/* avoid FPE */
-	if (b1<0.01f) b1= 0.01f;	
+	if (b1<0.01f) b1= 0.01f;
 	
 	if ((hard & 1)==0)  inp= 1.0f;
 	if (hard & 2)  inp*= b1;
@@ -560,7 +565,7 @@ static float spec(float inp, int hard)
 	b1*= b1;
 
 	/* avoid FPE */
-	if (b1<0.001f) b1= 0.0f;	
+	if (b1<0.001f) b1= 0.0f;
 
 	if (hard & 32) inp*= b1;
 	b1*= b1;
@@ -568,7 +573,7 @@ static float spec(float inp, int hard)
 	b1*= b1;
 	if (hard & 128) inp*=b1;
 
-	if (b1<0.001f) b1= 0.0f;	
+	if (b1<0.001f) b1= 0.0f;
 
 	if (hard & 256) {
 		b1*= b1;
@@ -633,7 +638,7 @@ static float Blinn_Spec(const float n[3], const float l[3], const float v[3], fl
 	
 	/* conversion from 'hardness' (1-255) to 'spec_power' (50 maps at 0.1) */
 	if (spec_power<100.0f)
-		spec_power= sqrt(1.0f/spec_power);
+		spec_power = sqrtf(1.0f / spec_power);
 	else spec_power= 10.0f/spec_power;
 	
 	h[0]= v[0]+l[0];
@@ -724,7 +729,7 @@ static float WardIso_Spec(const float n[3], const float l[3], const float v[3],
 	if (tangent) nl = sasqrt(1.0f - nl*nl);
 	if (nl<=0.0f) nl = 0.001f;
 
-	angle = tan(saacos(nh));
+	angle = tanf(saacos(nh));
 	alpha = MAX2(rms, 0.001f);
 
 	i= nl * (1.0f/(4.0f*(float)M_PI*alpha*alpha)) * (expf( -(angle*angle)/(alpha*alpha))/(sqrtf(nv*nl)));
@@ -739,7 +744,7 @@ static float Toon_Diff(const float n[3], const float l[3], const float UNUSED(v[
 
 	rslt = n[0]*l[0] + n[1]*l[1] + n[2]*l[2];
 
-	ang = saacos( (double)(rslt) );
+	ang = saacos(rslt);
 
 	if ( ang < size ) rslt = 1.0f;
 	else if ( ang >= (size + smooth) || smooth == 0.0f ) rslt = 0.0f;
@@ -826,7 +831,7 @@ static float Minnaert_Diff(float nl, const float n[3], const float v[3], float d
 		nv = 0.0f;
 
 	if (darkness <= 1.0f)
-		i = nl * pow(maxf(nv * nl, 0.1f), (darkness - 1.0f) ); /*The Real model*/
+		i = nl * pow(max_ff(nv * nl, 0.1f), (darkness - 1.0f) ); /*The Real model*/
 	else
 		i = nl * pow( (1.001f - nv), (darkness  - 1.0f) ); /*Nvidia model*/
 
@@ -918,6 +923,21 @@ void shade_color(ShadeInput *shi, ShadeResult *shr)
 	shr->diff[1]= shi->g;
 	shr->diff[2]= shi->b;
 	shr->alpha= shi->alpha;
+
+	/* modulate by the object color */
+	if ((ma->shade_flag & MA_OBCOLOR) && shi->obr->ob) {
+		float obcol[4];
+
+		copy_v4_v4(obcol, shi->obr->ob->col);
+		CLAMP(obcol[3], 0.0f, 1.0f);
+
+		shr->diff[0] *= obcol[0];
+		shr->diff[1] *= obcol[1];
+		shr->diff[2] *= obcol[2];
+		if (shi->mode & MA_TRANSP) shr->alpha *= obcol[3];
+	}
+
+	copy_v3_v3(shr->diffshad, shr->diff);
 }
 
 /* ramp for at end of shade */
@@ -1172,8 +1192,8 @@ float lamp_get_visibility(LampRen *lar, const float co[3], float lv[3], float *d
 		float visifac= 1.0f, t;
 		
 		sub_v3_v3v3(lv, co, lar->co);
-		*dist= sqrtf(dot_v3v3(lv, lv));
-		t= 1.0f/dist[0];
+		*dist = len_v3(lv);
+		t = 1.0f / (*dist);
 		mul_v3_fl(lv, t);
 		
 		/* area type has no quad or sphere option */
@@ -1232,10 +1252,10 @@ float lamp_get_visibility(LampRen *lar, const float co[3], float lv[3], float *d
 							copy_v3_v3(lvrot, lv);
 							mul_m3_v3(lar->imat, lvrot);
 							
-							x = maxf(fabsf(lvrot[0]/lvrot[2]), fabsf(lvrot[1]/lvrot[2]));
+							x = max_ff(fabsf(lvrot[0]/lvrot[2]), fabsf(lvrot[1]/lvrot[2]));
 							/* 1.0f/(sqrt(1+x*x)) is equivalent to cos(atan(x)) */
 							
-							inpr= 1.0f/(sqrt(1.0f+x*x));
+							inpr = 1.0f / (sqrtf(1.0f + x * x));
 						}
 						else inpr= 0.0f;
 					}
@@ -1316,7 +1336,7 @@ static void shade_one_light(LampRen *lar, ShadeInput *shi, ShadeResult *shr, int
 	if (lar->mode & LA_TEXTURE)	do_lamp_tex(lar, lv, shi, lacol, LA_TEXTURE);
 	if (lar->mode & LA_SHAD_TEX)	do_lamp_tex(lar, lv, shi, lashdw, LA_SHAD_TEX);
 
-		/* tangent case; calculate fake face normal, aligned with lampvector */	
+		/* tangent case; calculate fake face normal, aligned with lampvector */
 		/* note, vnor==vn is used as tangent trigger for buffer shadow */
 	if (vlr->flag & R_TANGENT) {
 		float cross[3], nstrand[3], blend;
@@ -1362,7 +1382,7 @@ static void shade_one_light(LampRen *lar, ShadeInput *shi, ShadeResult *shr, int
 	/* inp = dotproduct, is = shader result, i = lamp energy (with shadow), i_noshad = i without shadow */
 	inp= dot_v3v3(vn, lv);
 
-	/* phong threshold to prevent backfacing faces having artefacts on ray shadow (terminator problem) */
+	/* phong threshold to prevent backfacing faces having artifacts on ray shadow (terminator problem) */
 	/* this complex construction screams for a nicer implementation! (ton) */
 	if (R.r.mode & R_SHADOW) {
 		if (ma->mode & MA_SHADOW) {
@@ -1444,9 +1464,11 @@ static void shade_one_light(LampRen *lar, ShadeInput *shi, ShadeResult *shr, int
 						shr->shad[1] -= shadfac[3]*shi->g*(1.0f-lashdw[1]);
 						shr->shad[2] -= shadfac[3]*shi->b*(1.0f-lashdw[2]);
 						
-						shr->spec[0] -= shadfac[3]*shi->specr*(1.0f-lashdw[0]);
-						shr->spec[1] -= shadfac[3]*shi->specg*(1.0f-lashdw[1]);
-						shr->spec[2] -= shadfac[3]*shi->specb*(1.0f-lashdw[2]);
+						if (!(lar->mode & LA_NO_SPEC)) {
+							shr->spec[0] -= shadfac[3]*shi->specr*(1.0f-lashdw[0]);
+							shr->spec[1] -= shadfac[3]*shi->specg*(1.0f-lashdw[1]);
+							shr->spec[2] -= shadfac[3]*shi->specb*(1.0f-lashdw[2]);
+						}
 						
 						return;
 					}
@@ -1470,11 +1492,14 @@ static void shade_one_light(LampRen *lar, ShadeInput *shi, ShadeResult *shr, int
 				add_to_diffuse(shr->shad, shi, is, lashdw[0]*(i_noshad-i)*lacol[0], lashdw[1]*(i_noshad-i)*lacol[1], lashdw[2]*(i_noshad-i)*lacol[2]);
 			}
 			if (i_noshad>0.0f) {
-				if (passflag & (SCE_PASS_DIFFUSE|SCE_PASS_SHADOW)) {
+				if (passflag & (SCE_PASS_DIFFUSE|SCE_PASS_SHADOW) ||
+				    ((passflag & SCE_PASS_COMBINED) && !(shi->combinedflag & SCE_PASS_SHADOW)))
+				{
 					add_to_diffuse(shr->diff, shi, is, i_noshad*lacol[0], i_noshad*lacol[1], i_noshad*lacol[2]);
 				}
-				else
+				else {
 					copy_v3_v3(shr->diff, shr->shad);
+				}
 			}
 		}
 		
@@ -1624,7 +1649,8 @@ static void shade_lamp_loop_only_shadow(ShadeInput *shi, ShadeResult *shr)
 				else { /* shadowonly_flag == MA_SO_SHADED */
 					/* Use shaded value */
 					accum = 1.0f - shaded;
-			}}
+				}
+			}
 
 			shr->alpha= (shi->alpha)*(accum);
 			if (shr->alpha<0.0f) shr->alpha=0.0f;
@@ -1636,7 +1662,7 @@ static void shade_lamp_loop_only_shadow(ShadeInput *shi, ShadeResult *shr)
 		}
 	}
 	
-	/* quite disputable this...  also note it doesn't mirror-raytrace */	
+	/* quite disputable this...  also note it doesn't mirror-raytrace */
 	if ((R.wrld.mode & (WO_AMB_OCC|WO_ENV_LIGHT)) && shi->amb!=0.0f) {
 		float f;
 		
@@ -1682,9 +1708,19 @@ static void wrld_exposure_correct(float diff[3])
 
 void shade_lamp_loop(ShadeInput *shi, ShadeResult *shr)
 {
+	/* Passes which might need to know material color.
+	 *
+	 * It seems to be faster to just calculate material color
+	 * even if the pass doesn't really need it than trying to
+	 * figure out whether color is really needed or not.
+	 */
+	const int color_passes =
+		SCE_PASS_COMBINED | SCE_PASS_RGBA | SCE_PASS_DIFFUSE | SCE_PASS_SPEC |
+		SCE_PASS_REFLECT | SCE_PASS_NORMAL | SCE_PASS_REFRACT | SCE_PASS_EMIT | SCE_PASS_SHADOW;
+
 	Material *ma= shi->mat;
 	int passflag= shi->passflag;
-	
+
 	memset(shr, 0, sizeof(ShadeResult));
 	
 	if (!(shi->mode & MA_TRANSP)) shi->alpha = 1.0f;
@@ -1699,7 +1735,7 @@ void shade_lamp_loop(ShadeInput *shi, ShadeResult *shr)
 	shi->refcol[0]= shi->refcol[1]= shi->refcol[2]= shi->refcol[3]= 0.0f;
 	
 	/* material color itself */
-	if (passflag & (SCE_PASS_COMBINED|SCE_PASS_RGBA)) {
+	if (passflag & color_passes) {
 		if (ma->mode & (MA_FACETEXTURE)) {
 			shi->r= shi->vcol[0];
 			shi->g= shi->vcol[1];
@@ -1707,6 +1743,14 @@ void shade_lamp_loop(ShadeInput *shi, ShadeResult *shr)
 			if (ma->mode & (MA_FACETEXTURE_ALPHA))
 				shi->alpha= shi->vcol[3];
 		}
+#ifdef WITH_FREESTYLE
+		else if (ma->vcol_alpha) {
+			shi->r= shi->vcol[0];
+			shi->g= shi->vcol[1];
+			shi->b= shi->vcol[2];
+			shi->alpha= shi->vcol[3];
+		}
+#endif
 		else if (ma->mode & (MA_VERTEXCOLP)) {
 			float neg_alpha = 1.0f - shi->vcol[3];
 			shi->r= shi->r*neg_alpha + shi->vcol[0]*shi->vcol[3];
@@ -1729,15 +1773,15 @@ void shade_lamp_loop(ShadeInput *shi, ShadeResult *shr)
 				shr->col[0]= shr->col[1]= shr->col[2]= shr->col[3]= 1.0f;
 			}
 			else {
-				shi->r= pow(shi->r, ma->sss_texfac);
-				shi->g= pow(shi->g, ma->sss_texfac);
-				shi->b= pow(shi->b, ma->sss_texfac);
-				shi->alpha= pow(shi->alpha, ma->sss_texfac);
+				shi->r= pow(max_ff(shi->r, 0.0f), ma->sss_texfac);
+				shi->g= pow(max_ff(shi->g, 0.0f), ma->sss_texfac);
+				shi->b= pow(max_ff(shi->b, 0.0f), ma->sss_texfac);
+				shi->alpha= pow(max_ff(shi->alpha, 0.0f), ma->sss_texfac);
 				
-				shr->col[0]= pow(shr->col[0], ma->sss_texfac);
-				shr->col[1]= pow(shr->col[1], ma->sss_texfac);
-				shr->col[2]= pow(shr->col[2], ma->sss_texfac);
-				shr->col[3]= pow(shr->col[3], ma->sss_texfac);
+				shr->col[0]= pow(max_ff(shr->col[0], 0.0f), ma->sss_texfac);
+				shr->col[1]= pow(max_ff(shr->col[1], 0.0f), ma->sss_texfac);
+				shr->col[2]= pow(max_ff(shr->col[2], 0.0f), ma->sss_texfac);
+				shr->col[3]= pow(max_ff(shr->col[3], 0.0f), ma->sss_texfac);
 			}
 		}
 	}
@@ -1762,18 +1806,20 @@ void shade_lamp_loop(ShadeInput *shi, ShadeResult *shr)
 	}
 	
 	/* AO pass */
-	if (R.wrld.mode & (WO_AMB_OCC|WO_ENV_LIGHT|WO_INDIRECT_LIGHT)) {
-		if (((passflag & SCE_PASS_COMBINED) && (shi->combinedflag & (SCE_PASS_AO|SCE_PASS_ENVIRONMENT|SCE_PASS_INDIRECT))) ||
-		    (passflag & (SCE_PASS_AO|SCE_PASS_ENVIRONMENT|SCE_PASS_INDIRECT)))
-		{
-			if (R.r.mode & R_SHADOW) {
-				/* AO was calculated for scanline already */
-				if (shi->depth || shi->volume_depth)
-					ambient_occlusion(shi);
-				copy_v3_v3(shr->ao, shi->ao);
-				copy_v3_v3(shr->env, shi->env); /* XXX multiply */
-				copy_v3_v3(shr->indirect, shi->indirect); /* XXX multiply */
-			}
+	if (((passflag & SCE_PASS_COMBINED) && (shi->combinedflag & (SCE_PASS_AO|SCE_PASS_ENVIRONMENT|SCE_PASS_INDIRECT))) ||
+	    (passflag & (SCE_PASS_AO|SCE_PASS_ENVIRONMENT|SCE_PASS_INDIRECT))) {
+		if ((R.wrld.mode & (WO_AMB_OCC|WO_ENV_LIGHT|WO_INDIRECT_LIGHT)) && (R.r.mode & R_SHADOW)) {
+			/* AO was calculated for scanline already */
+			if (shi->depth || shi->volume_depth)
+				ambient_occlusion(shi);
+			copy_v3_v3(shr->ao, shi->ao);
+			copy_v3_v3(shr->env, shi->env); /* XXX multiply */
+			copy_v3_v3(shr->indirect, shi->indirect); /* XXX multiply */
+		}
+		else {
+			shr->ao[0]= shr->ao[1]= shr->ao[2]= 1.0f;
+			zero_v3(shr->env);
+			zero_v3(shr->indirect);
 		}
 	}
 	
@@ -1827,9 +1873,9 @@ void shade_lamp_loop(ShadeInput *shi, ShadeResult *shr)
 				else {
 					copy_v3_v3(col, shr->col);
 					mul_v3_fl(col, invalpha);
-					col[0]= pow(col[0], 1.0f-texfac);
-					col[1]= pow(col[1], 1.0f-texfac);
-					col[2]= pow(col[2], 1.0f-texfac);
+					col[0]= pow(max_ff(col[0], 0.0f), 1.0f-texfac);
+					col[1]= pow(max_ff(col[1], 0.0f), 1.0f-texfac);
+					col[2]= pow(max_ff(col[2], 0.0f), 1.0f-texfac);
 				}
 
 				shr->diff[0]= sss[0]*col[0];
@@ -1844,10 +1890,12 @@ void shade_lamp_loop(ShadeInput *shi, ShadeResult *shr)
 			}
 		}
 		
-		if (shi->combinedflag & SCE_PASS_SHADOW)	
-			copy_v3_v3(shr->combined, shr->shad); 	/* note, no ';' ! */
+		if (shi->combinedflag & SCE_PASS_SHADOW)
+			copy_v3_v3(shr->diffshad, shr->shad);
 		else
-			copy_v3_v3(shr->combined, shr->diff);
+			copy_v3_v3(shr->diffshad, shr->diff);
+
+		copy_v3_v3(shr->combined, shr->diffshad);
 			
 		/* calculate shadow pass, we use a multiplication mask */
 		/* if diff = 0,0,0 it doesn't matter what the shadow pass is, so leave it as is */
@@ -1878,7 +1926,7 @@ void shade_lamp_loop(ShadeInput *shi, ShadeResult *shr)
 		/* note: shi->mode! */
 		if (shi->mode & MA_TRANSP && (shi->mode & (MA_ZTRANSP|MA_RAYTRANSP))) {
 			if (shi->spectra!=0.0f) {
-				float t = MAX3(shr->spec[0], shr->spec[1], shr->spec[2]);
+				float t = max_fff(shr->spec[0], shr->spec[1], shr->spec[2]);
 				t *= shi->spectra;
 				if (t>1.0f) t= 1.0f;
 				shi->alpha= (1.0f-t)*shi->alpha+t;
@@ -1953,3 +2001,94 @@ void shade_lamp_loop(ShadeInput *shi, ShadeResult *shr)
 	shr->combined[3]= shr->alpha;
 }
 
+/* used for "Lamp Data" shader node */
+static float lamp_get_data_internal(ShadeInput *shi, GroupObject *go, float col[4], float lv[3], float *dist, float shadow[4])
+{
+	LampRen *lar = go->lampren;
+	float visifac, inp;
+
+	if (!lar || lar->type == LA_YF_PHOTON
+	    || ((lar->mode & LA_LAYER) && (lar->lay & shi->obi->lay) == 0)
+	    || (lar->lay & shi->lay) == 0)
+		return 0.0f;
+
+	if (lar->mode & LA_TEXTURE)
+		do_lamp_tex(lar, lv, shi, col, LA_TEXTURE);
+
+	visifac = lamp_get_visibility(lar, shi->co, lv, dist);
+
+	if (visifac == 0.0f
+	    || lar->type == LA_HEMI
+	    || (lar->type != LA_SPOT && !(lar->mode & LA_SHAD_RAY))
+	    || (R.r.scemode & R_BUTS_PREVIEW))
+		return visifac;
+
+	inp = dot_v3v3(shi->vn, lv);
+
+	if (inp > 0.0f) {
+		float shadfac[4];
+
+		shadow[0] = lar->shdwr;
+		shadow[1] = lar->shdwg;
+		shadow[2] = lar->shdwb;
+
+		if (lar->mode & LA_SHAD_TEX)
+			do_lamp_tex(lar, lv, shi, shadow, LA_SHAD_TEX);
+
+		lamp_get_shadow(lar, shi, inp, shadfac, shi->depth);
+
+		shadow[0] = 1.0f - ((1.0f - shadfac[0] * shadfac[3]) * (1.0f - shadow[0]));
+		shadow[1] = 1.0f - ((1.0f - shadfac[1] * shadfac[3]) * (1.0f - shadow[1]));
+		shadow[2] = 1.0f - ((1.0f - shadfac[2] * shadfac[3]) * (1.0f - shadow[2]));
+	}
+
+	return visifac;
+}
+
+float RE_lamp_get_data(ShadeInput *shi, Object *lamp_obj, float col[4], float lv[3], float *dist, float shadow[4])
+{
+	col[0] = col[1] = col[2] = 0.0f;
+	col[3] = 1.0f;
+	copy_v3_v3(lv, shi->vn);
+	*dist = 1.0f;
+	shadow[0] = shadow[1] = shadow[2] = shadow[3] = 1.0f;
+
+	if (lamp_obj->type == OB_LAMP) {
+		GroupObject *go;
+		Lamp *lamp = (Lamp *)lamp_obj->data;
+
+		col[0] = lamp->r * lamp->energy;
+		col[1] = lamp->g * lamp->energy;
+		col[2] = lamp->b * lamp->energy;
+
+		if (R.r.scemode & R_BUTS_PREVIEW) {
+			for (go = R.lights.first; go; go = go->next) {
+				/* "Lamp.002" is main key light of material preview */
+				if (strcmp(go->ob->id.name + 2, "Lamp.002") == 0)
+					return lamp_get_data_internal(shi, go, col, lv, dist, shadow);
+			}
+			return 0.0f;
+		}
+
+		if (shi->light_override) {
+			for (go = shi->light_override->gobject.first; go; go = go->next) {
+				if (go->ob == lamp_obj)
+					return lamp_get_data_internal(shi, go, col, lv, dist, shadow);
+			}
+		}
+
+		if (shi->mat && shi->mat->group) {
+			for (go = shi->mat->group->gobject.first; go; go = go->next) {
+				if (go->ob == lamp_obj)
+					return lamp_get_data_internal(shi, go, col, lv, dist, shadow);
+			}
+		}
+
+		for (go = R.lights.first; go; go = go->next) {
+			if (go->ob == lamp_obj)
+				return lamp_get_data_internal(shi, go, col, lv, dist, shadow);
+		}
+	}
+
+	return 0.0f;
+}
