@@ -55,25 +55,19 @@
 #include "BLI_ghash.h"
 #include "BLI_memarena.h"
 
-#include "PIL_time.h"
+#include "BLF_translation.h"
+
 
 #include "DNA_material_types.h"
 
-#include "BKE_colortools.h"
 #include "BKE_global.h"
 #include "BKE_main.h"
-#include "BKE_material.h"
-#include "BKE_node.h"
 #include "BKE_scene.h"
 
 
 /* this module */
 #include "render_types.h"
-#include "rendercore.h"
-#include "renderdatabase.h" 
-#include "shading.h"
 #include "sss.h"
-#include "zbuf.h"
 
 /* Generic Multiple Scattering API */
 
@@ -166,7 +160,7 @@ static float f_Rd(float alpha_, float A, float ro)
 {
 	float sq;
 
-	sq= sqrt(3.0f*(1.0f - alpha_));
+	sq = sqrtf(3.0f * (1.0f - alpha_));
 	return (alpha_/2.0f)*(1.0f + expf((-4.0f/3.0f)*A*sq))*expf(-sq) - ro;
 }
 
@@ -213,8 +207,8 @@ static float Rd_rsquare(ScatterSettings *ss, float rr)
 {
 	float sr, sv, Rdr, Rdv;
 
-	sr= sqrt(rr + ss->zr*ss->zr);
-	sv= sqrt(rr + ss->zv*ss->zv);
+	sr = sqrtf(rr + ss->zr * ss->zr);
+	sv = sqrtf(rr + ss->zv * ss->zv);
 
 	Rdr= ss->zr*(1.0f + ss->sigma*sr)*expf(-ss->sigma*sr)/(sr*sr*sr);
 	Rdv= ss->zv*(1.0f + ss->sigma*sv)*expf(-ss->sigma*sv)/(sv*sv*sv);
@@ -242,7 +236,7 @@ static void approximate_Rd_rgb(ScatterSettings **ss, float rr, float *rd)
 		/* pass */
 	}
 	else if (rr > RD_TABLE_RANGE) {
-		rr= sqrt(rr);
+		rr = sqrtf(rr);
 		indexf= rr*(RD_TABLE_SIZE/RD_TABLE_RANGE_2);
 		index= (int)indexf;
 		idxf= (float)index;
@@ -287,7 +281,7 @@ static void build_Rd_table(ScatterSettings *ss)
 		r= i*(RD_TABLE_RANGE/RD_TABLE_SIZE);
 		/*if (r < ss->invsigma_t_*ss->invsigma_t_)
 			r= ss->invsigma_t_*ss->invsigma_t_;*/
-		ss->tableRd[i]= Rd(ss, sqrt(r));
+		ss->tableRd[i]= Rd(ss, sqrtf(r));
 
 		r= i*(RD_TABLE_RANGE_2/RD_TABLE_SIZE);
 		/*if (r < ss->invsigma_t_)
@@ -307,7 +301,7 @@ ScatterSettings *scatter_settings_new(float refl, float radius, float ior, float
 	ss->Fdr= -1.440f/ior*ior + 0.710f/ior + 0.668f + 0.0636f*ior;
 	ss->A= (1.0f + ss->Fdr)/(1.0f - ss->Fdr);
 	ss->ld= radius;
-	ss->ro= minf(refl, 0.999f);
+	ss->ro= min_ff(refl, 0.99f);
 	ss->color= ss->ro*reflfac + (1.0f-reflfac);
 
 	ss->alpha_= compute_reduced_albedo(ss);
@@ -381,7 +375,7 @@ static void add_radiance(ScatterTree *tree, float *frontrad, float *backrad, flo
 	}
 }
 
-static void traverse_octree(ScatterTree *tree, ScatterNode *node, float *co, int self, ScatterResult *result)
+static void traverse_octree(ScatterTree *tree, ScatterNode *node, const float co[3], int self, ScatterResult *result)
 {
 	float sub[3], dist;
 	int i, index = 0;
@@ -432,7 +426,7 @@ static void traverse_octree(ScatterTree *tree, ScatterNode *node, float *co, int
 	}
 }
 
-static void compute_radiance(ScatterTree *tree, float *co, float *rad)
+static void compute_radiance(ScatterTree *tree, const float co[3], float *rad)
 {
 	ScatterResult result;
 	float rdsum[3], backrad[3], backrdsum[3];
@@ -855,7 +849,7 @@ static void sss_create_tree_mat(Render *re, Material *mat)
 	RenderResult *rr;
 	ListBase points;
 	float (*co)[3] = NULL, (*color)[3] = NULL, *area = NULL;
-	int totpoint = 0, osa, osaflag, partsdone;
+	int totpoint = 0, osa, osaflag, frsflag, partsdone;
 
 	if (re->test_break(re->tbh))
 		return;
@@ -870,22 +864,23 @@ static void sss_create_tree_mat(Render *re, Material *mat)
 	rr= re->result;
 	osa= re->osa;
 	osaflag= re->r.mode & R_OSA;
+	frsflag= re->r.mode & R_EDGE_FRS;
 	partsdone= re->i.partsdone;
 
 	re->osa= 0;
-	re->r.mode &= ~R_OSA;
+	re->r.mode &= ~(R_OSA | R_EDGE_FRS);
 	re->sss_points= &points;
 	re->sss_mat= mat;
-	re->i.partsdone = FALSE;
+	re->i.partsdone = 0;
 
-	if (!(re->r.scemode & R_PREVIEWBUTS))
+	if (!(re->r.scemode & (R_BUTS_PREVIEW|R_VIEWPORT_PREVIEW)))
 		re->result= NULL;
 	BLI_rw_mutex_unlock(&re->resultmutex);
 
 	RE_TileProcessor(re);
 	
 	BLI_rw_mutex_lock(&re->resultmutex, THREAD_LOCK_WRITE);
-	if (!(re->r.scemode & R_PREVIEWBUTS)) {
+	if (!(re->r.scemode & (R_BUTS_PREVIEW|R_VIEWPORT_PREVIEW))) {
 		RE_FreeRenderResult(re->result);
 		re->result= rr;
 	}
@@ -896,6 +891,7 @@ static void sss_create_tree_mat(Render *re, Material *mat)
 	re->sss_points= NULL;
 	re->osa= osa;
 	if (osaflag) re->r.mode |= R_OSA;
+	if (frsflag) re->r.mode |= R_EDGE_FRS;
 
 	/* no points? no tree */
 	if (!points.first)
@@ -930,12 +926,12 @@ static void sss_create_tree_mat(Render *re, Material *mat)
 	if (!re->test_break(re->tbh)) {
 		SSSData *sss= MEM_callocN(sizeof(*sss), "SSSData");
 		float ior= mat->sss_ior, cfac= mat->sss_colfac;
-		float *radius= mat->sss_radius;
+		const float *radius = mat->sss_radius;
 		float fw= mat->sss_front, bw= mat->sss_back;
 		float error = mat->sss_error;
 
 		error= get_render_aosss_error(&re->r, error);
-		if ((re->r.scemode & R_PREVIEWBUTS) && error < 0.5f)
+		if ((re->r.scemode & (R_BUTS_PREVIEW|R_VIEWPORT_PREVIEW)) && error < 0.5f)
 			error= 0.5f;
 		
 		sss->ss[0]= scatter_settings_new(mat->sss_col[0], radius[0], ior, cfac, fw, bw);
@@ -991,24 +987,45 @@ static void sss_free_tree(SSSData *sss)
 void make_sss_tree(Render *re)
 {
 	Material *mat;
+	bool infostr_set = false;
+	const char *prevstr = NULL;
+
+	free_sss(re);
 	
 	re->sss_hash= BLI_ghash_ptr_new("make_sss_tree gh");
 
-	re->i.infostr= "SSS preprocessing";
 	re->stats_draw(re->sdh, &re->i);
 	
-	for (mat= re->main->mat.first; mat; mat= mat->id.next)
-		if (mat->id.us && (mat->flag & MA_IS_USED) && (mat->sss_flag & MA_DIFF_SSS))
+	for (mat= re->main->mat.first; mat; mat= mat->id.next) {
+		if (mat->id.us && (mat->flag & MA_IS_USED) && (mat->sss_flag & MA_DIFF_SSS)) {
+			if (!infostr_set) {
+				prevstr = re->i.infostr;
+				re->i.infostr = IFACE_("SSS preprocessing");
+				infostr_set = true;
+			}
+
 			sss_create_tree_mat(re, mat);
+		}
+	}
 	
 	/* XXX preview exception */
 	/* localizing preview render data is not fun for node trees :( */
 	if (re->main!=G.main) {
-		for (mat= G.main->mat.first; mat; mat= mat->id.next)
-			if (mat->id.us && (mat->flag & MA_IS_USED) && (mat->sss_flag & MA_DIFF_SSS))
+		for (mat= G.main->mat.first; mat; mat= mat->id.next) {
+			if (mat->id.us && (mat->flag & MA_IS_USED) && (mat->sss_flag & MA_DIFF_SSS)) {
+				if (!infostr_set) {
+					prevstr = re->i.infostr;
+					re->i.infostr = IFACE_("SSS preprocessing");
+					infostr_set = true;
+				}
+
 				sss_create_tree_mat(re, mat);
+			}
+		}
 	}
 	
+	if (infostr_set)
+		re->i.infostr = prevstr;
 }
 
 void free_sss(Render *re)
@@ -1016,7 +1033,7 @@ void free_sss(Render *re)
 	if (re->sss_hash) {
 		GHashIterator *it= BLI_ghashIterator_new(re->sss_hash);
 
-		while (!BLI_ghashIterator_isDone(it)) {
+		while (!BLI_ghashIterator_done(it)) {
 			sss_free_tree(BLI_ghashIterator_getValue(it));
 			BLI_ghashIterator_step(it);
 		}
