@@ -128,11 +128,11 @@ struct ImBuf;
 #define OPTYPE_UNDO			2	/* do undo push after after */
 #define OPTYPE_BLOCKING		4	/* let blender grab all input from the WM (X11) */
 #define OPTYPE_MACRO		8
-#define OPTYPE_GRAB_POINTER	16	/* */
+#define OPTYPE_GRAB_POINTER	16	/* grabs the cursor and optionally enables continuous cursor wrapping */
 #define OPTYPE_PRESET		32	/* show preset menu */
 #define OPTYPE_INTERNAL		64	/* some operators are mainly for internal use
 								 * and don't make sense to be accessed from the
-								 * search menu, even if poll() returns TRUE.
+								 * search menu, even if poll() returns true.
 								 * currently only used for the search toolbox */
 #define OPTYPE_LOCK_BYPASS		128	/* Allow operator to run when interface is locked */
 
@@ -169,7 +169,7 @@ enum {
 #define KM_OSKEY2	128
 
 /* KM_MOD_ flags for wmKeyMapItem and wmEvent.alt/shift/oskey/ctrl  */
-/* note that KM_ANY and FALSE are used with these defines too */
+/* note that KM_ANY and false are used with these defines too */
 #define KM_MOD_FIRST  1
 #define KM_MOD_SECOND 2
 
@@ -190,9 +190,6 @@ enum {
 #define WM_UI_HANDLER_CONTINUE	0
 #define WM_UI_HANDLER_BREAK		1
 
-typedef int (*wmUIHandlerFunc)(struct bContext *C, const struct wmEvent *event, void *userdata);
-typedef void (*wmUIHandlerRemoveFunc)(struct bContext *C, void *userdata);
-
 /* ************** Notifiers ****************** */
 
 typedef struct wmNotifier {
@@ -252,6 +249,7 @@ typedef struct wmNotifier {
 #define ND_DATACHANGED		(3<<16)
 #define ND_HISTORY			(4<<16)
 #define ND_JOB				(5<<16)
+#define ND_UNDO				(6<<16)
 
 	/* NC_SCREEN screen */
 #define ND_SCREENBROWSE		(1<<16)
@@ -326,6 +324,9 @@ typedef struct wmNotifier {
 #define ND_NLA_ACTCHANGE	(74<<16)
 #define ND_FCURVES_ORDER	(75<<16)
 
+	/* NC_GPENCIL */
+#define ND_GPENCIL_EDITMODE	(85<<16)
+
 	/* NC_GEOM Geometry */
 	/* Mesh, Curve, MetaBall, Armature, .. */
 #define ND_SELECT			(90<<16)
@@ -556,9 +557,7 @@ typedef struct wmOperatorType {
 	/* pointer to modal keymap, do not free! */
 	struct wmKeyMap *modalkeymap;
 
-	/* only used for operators defined with python
-	 * use to store pointers to python functions */
-	void *pyop_data;
+	/* python needs the operator type as well */
 	int (*pyop_poll)(struct bContext *, struct wmOperatorType *ot) ATTR_WARN_UNUSED_RESULT;
 
 	/* RNA integration */
@@ -569,6 +568,24 @@ typedef struct wmOperatorType {
 
 } wmOperatorType;
 
+#ifdef WITH_INPUT_IME
+/* *********** Input Method Editor (IME) *********** */
+
+/* similar to GHOST_TEventImeData */
+typedef struct wmIMEData {
+	size_t result_len, composite_len;
+
+	char *str_result;           /* utf8 encoding */
+	char *str_composite;        /* utf8 encoding */
+
+	int cursor_pos;             /* cursor position in the IME composition. */
+	int sel_start;              /* beginning of the selection */
+	int sel_end;                /* end of the selection */
+
+	bool is_ime_composing;
+} wmIMEData;
+#endif
+
 /* **************** Paint Cursor ******************* */
 
 typedef void (*wmPaintCursorDraw)(struct bContext *C, int, int, void *customdata);
@@ -600,6 +617,12 @@ typedef struct wmReport {
 #define WM_DRAG_PATH	2
 #define WM_DRAG_NAME	3
 #define WM_DRAG_VALUE	4
+#define WM_DRAG_COLOR	5
+
+typedef enum wmDragFlags {
+	WM_DRAG_NOP         = 0,
+	WM_DRAG_FREE_DATA   = 1,
+} wmDragFlags;
 
 /* note: structs need not exported? */
 
@@ -616,6 +639,7 @@ typedef struct wmDrag {
 	int sx, sy;
 	
 	char opname[200]; /* if set, draws operator name*/
+	unsigned int flags;
 } wmDrag;
 
 /* dropboxes are like keymaps, part of the screen/area/region definition */
