@@ -25,6 +25,8 @@
 
 /** \file blender/windowmanager/intern/wm_files.c
  *  \ingroup wm
+ *
+ * User level access for blend file read/write, file-history and userprefs.
  */
 
 
@@ -44,10 +46,7 @@
 #  endif
 #  include <shlobj.h>  /* for SHGetSpecialFolderPath, has to be done before BLI_winstuff
                         * because 'near' is disabled through BLI_windstuff */
-#  include <process.h> /* getpid */
 #  include "BLI_winstuff.h"
-#else
-#  include <unistd.h> /* getpid */
 #endif
 
 #include "MEM_guardedalloc.h"
@@ -58,11 +57,11 @@
 #include "BLI_utildefines.h"
 #include "BLI_threads.h"
 #include "BLI_callbacks.h"
+#include "BLI_system.h"
+#include BLI_SYSTEM_PID_H
 
 #include "BLF_translation.h"
 
-#include "DNA_anim_types.h"
-#include "DNA_ipo_types.h" // XXX old animation system
 #include "DNA_object_types.h"
 #include "DNA_space_types.h"
 #include "DNA_userdef_types.h"
@@ -70,21 +69,18 @@
 #include "DNA_screen_types.h"
 #include "DNA_windowmanager_types.h"
 
+#include "BKE_appdir.h"
+#include "BKE_utildefines.h"
+#include "BKE_autoexec.h"
 #include "BKE_blender.h"
 #include "BKE_context.h"
 #include "BKE_depsgraph.h"
-#include "BKE_DerivedMesh.h"
-#include "BKE_font.h"
 #include "BKE_global.h"
-#include "BKE_library.h"
 #include "BKE_main.h"
-#include "BKE_multires.h"
 #include "BKE_packedFile.h"
 #include "BKE_report.h"
 #include "BKE_sound.h"
 #include "BKE_screen.h"
-#include "BKE_texture.h"
-
 
 #include "BLO_readfile.h"
 #include "BLO_writefile.h"
@@ -96,18 +92,16 @@
 #include "IMB_thumbs.h"
 
 #include "ED_datafiles.h"
-#include "ED_object.h"
+#include "ED_fileselect.h"
 #include "ED_screen.h"
-#include "ED_sculpt.h"
 #include "ED_view3d.h"
 #include "ED_util.h"
 
-#include "RE_pipeline.h" /* only to report missing engine */
-
 #include "GHOST_C-api.h"
 #include "GHOST_Path-api.h"
 
 #include "UI_interface.h"
+#include "UI_view2d.h"
 
 #include "GPU_draw.h"
 
@@ -136,7 +130,7 @@ static void wm_window_match_init(bContext *C, ListBase *wmlist)
 	wmWindow *win, *active_win;
 	
 	*wmlist = G.main->wm;
-	G.main->wm.first = G.main->wm.last = NULL;
+	BLI_listbase_clear(&G.main->wm);
 	
 	active_win = CTX_wm_window(C);
 
@@ -144,7 +138,7 @@ static void wm_window_match_init(bContext *C, ListBase *wmlist)
 	/* code copied from wm_init_exit.c */
 	for (wm = wmlist->first; wm; wm = wm->id.next) {
 		
-		WM_jobs_stop_all(wm);
+		WM_jobs_kill_all(wm);
 		
 		for (win = wm->windows.first; win; win = win->next) {
 		
@@ -176,6 +170,28 @@ static void wm_window_match_init(bContext *C, ListBase *wmlist)
 #endif
 }
 
+static void wm_window_substitute_old(wmWindowManager *wm, wmWindow *oldwin, wmWindow *win)
+{
+	win->ghostwin = oldwin->ghostwin;
+	win->active = oldwin->active;
+	if (win->active)
+		wm->winactive = win;
+
+	if (!G.background) /* file loading in background mode still calls this */
+		GHOST_SetWindowUserData(win->ghostwin, win);    /* pointer back */
+
+	oldwin->ghostwin = NULL;
+
+	win->eventstate = oldwin->eventstate;
+	oldwin->eventstate = NULL;
+
+	/* ensure proper screen rescaling */
+	win->sizex = oldwin->sizex;
+	win->sizey = oldwin->sizey;
+	win->posx = oldwin->posx;
+	win->posy = oldwin->posy;
+}
+
 /* match old WM with new, 4 cases:
  * 1- no current wm, no read wm: make new default
  * 2- no current wm, but read wm: that's OK, do nothing
@@ -189,7 +205,7 @@ static void wm_window_match_do(bContext *C, ListBase *oldwmlist)
 	wmWindow *oldwin, *win;
 	
 	/* cases 1 and 2 */
-	if (oldwmlist->first == NULL) {
+	if (BLI_listbase_is_empty(oldwmlist)) {
 		if (G.main->wm.first) {
 			/* nothing todo */
 		}
@@ -201,7 +217,7 @@ static void wm_window_match_do(bContext *C, ListBase *oldwmlist)
 		/* cases 3 and 4 */
 		
 		/* we've read file without wm..., keep current one entirely alive */
-		if (G.main->wm.first == NULL) {
+		if (BLI_listbase_is_empty(&G.main->wm)) {
 			bScreen *screen = NULL;
 
 			/* when loading without UI, no matching needed */
@@ -229,6 +245,8 @@ static void wm_window_match_do(bContext *C, ListBase *oldwmlist)
 			ED_screens_initialize(G.main->wm.first);
 		}
 		else {
+			bool has_match = false;
+
 			/* what if old was 3, and loaded 1? */
 			/* this code could move to setup_appdata */
 			oldwm = oldwmlist->first;
@@ -240,7 +258,7 @@ static void wm_window_match_do(bContext *C, ListBase *oldwmlist)
 			wm->defaultconf = oldwm->defaultconf;
 			wm->userconf = oldwm->userconf;
 
-			oldwm->keyconfigs.first = oldwm->keyconfigs.last = NULL;
+			BLI_listbase_clear(&oldwm->keyconfigs);
 			oldwm->addonconf = NULL;
 			oldwm->defaultconf = NULL;
 			oldwm->userconf = NULL;
@@ -248,58 +266,61 @@ static void wm_window_match_do(bContext *C, ListBase *oldwmlist)
 			/* ensure making new keymaps and set space types */
 			wm->initialized = 0;
 			wm->winactive = NULL;
-			
+
 			/* only first wm in list has ghostwins */
 			for (win = wm->windows.first; win; win = win->next) {
 				for (oldwin = oldwm->windows.first; oldwin; oldwin = oldwin->next) {
-					
+
 					if (oldwin->winid == win->winid) {
-						win->ghostwin = oldwin->ghostwin;
-						win->active = oldwin->active;
-						if (win->active)
-							wm->winactive = win;
+						has_match = true;
 
-						if (!G.background) /* file loading in background mode still calls this */
-							GHOST_SetWindowUserData(win->ghostwin, win);    /* pointer back */
-
-						oldwin->ghostwin = NULL;
-						
-						win->eventstate = oldwin->eventstate;
-						oldwin->eventstate = NULL;
-						
-						/* ensure proper screen rescaling */
-						win->sizex = oldwin->sizex;
-						win->sizey = oldwin->sizey;
-						win->posx = oldwin->posx;
-						win->posy = oldwin->posy;
+						wm_window_substitute_old(wm, oldwin, win);
 					}
 				}
 			}
+
+			/* make sure at least one window is kept open so we don't lose the context, check T42303 */
+			if (!has_match) {
+				oldwin = oldwm->windows.first;
+				win = wm->windows.first;
+
+				wm_window_substitute_old(wm, oldwin, win);
+			}
+
 			wm_close_and_free_all(C, oldwmlist);
 		}
 	}
 }
 
 /* in case UserDef was read, we re-initialize all, and do versioning */
-static void wm_init_userdef(bContext *C)
+static void wm_init_userdef(bContext *C, const bool from_memory)
 {
+	Main *bmain = CTX_data_main(C);
+
+	/* versioning is here */
 	UI_init_userdef();
+	
 	MEM_CacheLimiter_set_maximum(((size_t)U.memcachelimit) * 1024 * 1024);
-	sound_init(CTX_data_main(C));
+	sound_init(bmain);
 
 	/* needed so loading a file from the command line respects user-pref [#26156] */
-	if (U.flag & USER_FILENOUI) G.fileflags |= G_FILE_NO_UI;
-	else G.fileflags &= ~G_FILE_NO_UI;
+	BKE_BIT_TEST_SET(G.fileflags, U.flag & USER_FILENOUI, G_FILE_NO_UI);
 
 	/* set the python auto-execute setting from user prefs */
 	/* enabled by default, unless explicitly enabled in the command line which overrides */
 	if ((G.f & G_SCRIPT_OVERRIDE_PREF) == 0) {
-		if ((U.flag & USER_SCRIPT_AUTOEXEC_DISABLE) == 0) G.f |=  G_SCRIPT_AUTOEXEC;
-		else G.f &= ~G_SCRIPT_AUTOEXEC;
+		BKE_BIT_TEST_SET(G.f, (U.flag & USER_SCRIPT_AUTOEXEC_DISABLE) == 0, G_SCRIPT_AUTOEXEC);
+	}
+
+	/* avoid re-saving for every small change to our prefs, allow overrides */
+	if (from_memory) {
+		BLO_update_defaults_userpref_blend();
 	}
 
 	/* update tempdir from user preferences */
-	BLI_init_temporary_dir(U.tempdir);
+	BKE_tempdir_init(U.tempdir);
+
+	BKE_userdef_state();
 }
 
 
@@ -323,7 +344,7 @@ static int wm_read_exotic(Scene *UNUSED(scene), const char *name)
 	/* make sure we're not trying to read a directory.... */
 
 	len = strlen(name);
-	if (ELEM(name[len - 1], '/', '\\')) {
+	if (len > 0 && ELEM(name[len - 1], '/', '\\')) {
 		retval = BKE_READ_EXOTIC_FAIL_PATH;
 	}
 	else {
@@ -339,7 +360,7 @@ static int wm_read_exotic(Scene *UNUSED(scene), const char *name)
 			}
 			else {
 #if 0           /* historic stuff - no longer used */
-				WM_cursor_wait(TRUE);
+				WM_cursor_wait(true);
 
 				if (is_foo_format(name)) {
 					read_foo(name);
@@ -351,7 +372,7 @@ static int wm_read_exotic(Scene *UNUSED(scene), const char *name)
 					retval = BKE_READ_EXOTIC_FAIL_FORMAT;
 				}
 #if 0
-				WM_cursor_wait(FALSE);
+				WM_cursor_wait(false);
 #endif
 			}
 		}
@@ -360,8 +381,24 @@ static int wm_read_exotic(Scene *UNUSED(scene), const char *name)
 	return retval;
 }
 
-void WM_file_read(bContext *C, const char *filepath, ReportList *reports)
+void WM_file_autoexec_init(const char *filepath)
 {
+	if (G.f & G_SCRIPT_OVERRIDE_PREF) {
+		return;
+	}
+
+	if (G.f & G_SCRIPT_AUTOEXEC) {
+		char path[FILE_MAX];
+		BLI_split_dir_part(filepath, path, sizeof(path));
+		if (BKE_autoexec_match(path)) {
+			G.f &= ~G_SCRIPT_AUTOEXEC;
+		}
+	}
+}
+
+bool WM_file_read(bContext *C, const char *filepath, ReportList *reports)
+{
+	bool success = false;
 	int retval;
 
 	/* so we can get the error message */
@@ -371,6 +408,8 @@ void WM_file_read(bContext *C, const char *filepath, ReportList *reports)
 
 	BLI_callback_exec(CTX_data_main(C), NULL, BLI_CB_EVT_LOAD_PRE);
 
+	UI_view2d_zoom_cache_reset();
+
 	/* first try to append data from exotic file formats... */
 	/* it throws error box when file doesn't exist and returns -1 */
 	/* note; it should set some error message somewhere... (ton) */
@@ -382,14 +421,23 @@ void WM_file_read(bContext *C, const char *filepath, ReportList *reports)
 		ListBase wmbase;
 
 		/* assume automated tasks with background, don't write recent file list */
-		const int do_history = (G.background == FALSE) && (CTX_wm_manager(C)->op_undo_depth == 0);
+		const bool do_history = (G.background == false) && (CTX_wm_manager(C)->op_undo_depth == 0);
 
 		/* put aside screens to match with persistent windows later */
 		/* also exit screens and editors */
 		wm_window_match_init(C, &wmbase); 
 		
+		/* confusing this global... */
+		G.relbase_valid = 1;
 		retval = BKE_read_file(C, filepath, reports);
-		G.save_over = 1;
+		/* when loading startup.blend's, we can be left with a blank path */
+		if (G.main->name[0]) {
+			G.save_over = 1;
+		}
+		else {
+			G.save_over = 0;
+			G.relbase_valid = 0;
+		}
 
 		/* this flag is initialized by the operator but overwritten on read.
 		 * need to re-enable it here else drivers + registered scripts wont work. */
@@ -404,11 +452,10 @@ void WM_file_read(bContext *C, const char *filepath, ReportList *reports)
 
 		if (retval == BKE_READ_FILE_OK_USERPREFS) {
 			/* in case a userdef is read from regular .blend */
-			wm_init_userdef(C);
+			wm_init_userdef(C, false);
 		}
 		
 		if (retval != BKE_READ_FILE_FAIL) {
-			G.relbase_valid = 1;
 			if (do_history) {
 				write_history();
 			}
@@ -421,16 +468,17 @@ void WM_file_read(bContext *C, const char *filepath, ReportList *reports)
 		CTX_wm_window_set(C, CTX_wm_manager(C)->windows.first);
 
 		ED_editors_init(C);
-		DAG_on_visible_update(CTX_data_main(C), TRUE);
+		DAG_on_visible_update(CTX_data_main(C), true);
 
 #ifdef WITH_PYTHON
 		/* run any texts that were loaded in and flagged as modules */
-		BPY_driver_reset();
-		BPY_app_handlers_reset(FALSE);
-		BPY_modules_load_user(C);
+		BPY_python_reset(C);
 #endif
 
+		WM_operatortype_last_properties_clear_all();
+
 		/* important to do before NULL'ing the context */
+		BLI_callback_exec(CTX_data_main(C), NULL, BLI_CB_EVT_VERSION_UPDATE);
 		BLI_callback_exec(CTX_data_main(C), NULL, BLI_CB_EVT_LOAD_POST);
 
 		if (!G.background) {
@@ -450,9 +498,8 @@ void WM_file_read(bContext *C, const char *filepath, ReportList *reports)
 				if (sce->r.engine[0] &&
 				    BLI_findstring(&R_engines, sce->r.engine, offsetof(RenderEngineType, idname)) == NULL)
 				{
-					BKE_reportf(reports, RPT_WARNING,
-					            "Engine not available: '%s' for scene: %s, "
-					            "an addon may need to be installed or enabled",
+					BKE_reportf(reports, RPT_ERROR, "Engine '%s' not available for scene '%s' "
+					            "(an addon may need to be installed or enabled)",
 					            sce->r.engine, sce->id.name + 2);
 				}
 			}
@@ -461,83 +508,135 @@ void WM_file_read(bContext *C, const char *filepath, ReportList *reports)
 
 		BKE_reset_undo();
 		BKE_write_undo(C, "original");  /* save current state */
+
+		success = true;
 	}
 	else if (retval == BKE_READ_EXOTIC_OK_OTHER)
 		BKE_write_undo(C, "Import file");
 	else if (retval == BKE_READ_EXOTIC_FAIL_OPEN) {
-		BKE_reportf(reports, RPT_ERROR, IFACE_("Can't read file: \"%s\", %s."), filepath,
-		            errno ? strerror(errno) : IFACE_("Unable to open the file"));
+		BKE_reportf(reports, RPT_ERROR, "Cannot read file '%s': %s", filepath,
+		            errno ? strerror(errno) : TIP_("unable to open the file"));
 	}
 	else if (retval == BKE_READ_EXOTIC_FAIL_FORMAT) {
-		BKE_reportf(reports, RPT_ERROR, IFACE_("File format is not supported in file: \"%s\"."), filepath);
+		BKE_reportf(reports, RPT_ERROR, "File format is not supported in file '%s'", filepath);
 	}
 	else if (retval == BKE_READ_EXOTIC_FAIL_PATH) {
-		BKE_reportf(reports, RPT_ERROR, IFACE_("File path invalid: \"%s\"."), filepath);
+		BKE_reportf(reports, RPT_ERROR, "File path '%s' invalid", filepath);
 	}
 	else {
-		BKE_reportf(reports, RPT_ERROR, IFACE_("Unknown error loading: \"%s\"."), filepath);
+		BKE_reportf(reports, RPT_ERROR, "Unknown error loading '%s'", filepath);
 		BLI_assert(!"invalid 'retval'");
 	}
 
 	WM_cursor_wait(0);
 
+	return success;
+
 }
 
 
 /* called on startup,  (context entirely filled with NULLs) */
 /* or called for 'New File' */
-/* op can be NULL */
-int WM_homefile_read(bContext *C, ReportList *UNUSED(reports), short from_memory)
+/* both startup.blend and userpref.blend are checked */
+/* the optional paramater custom_file points to an alterntive startup page */
+/* custom_file can be NULL */
+int wm_homefile_read(bContext *C, ReportList *reports, bool from_memory, const char *custom_file)
 {
 	ListBase wmbase;
-	char tstr[FILE_MAX];
+	char startstr[FILE_MAX];
+	char prefstr[FILE_MAX];
 	int success = 0;
 
+	/* Indicates whether user preferences were really load from memory.
+	 *
+	 * This is used for versioning code, and for this we can not rely on from_memory
+	 * passed via argument. This is because there might be configuration folder
+	 * exists but it might not have userpref.blend and in this case we fallback to
+	 * reading home file from memory.
+	 *
+	 * And in this case versioning code is to be run.
+	 */
+	bool read_userdef_from_memory = true;
+
+	/* options exclude eachother */
+	BLI_assert((from_memory && custom_file) == 0);
+
 	BLI_callback_exec(CTX_data_main(C), NULL, BLI_CB_EVT_LOAD_PRE);
 
+	UI_view2d_zoom_cache_reset();
+
 	G.relbase_valid = 0;
 	if (!from_memory) {
-		char *cfgdir = BLI_get_folder(BLENDER_USER_CONFIG, NULL);
-		if (cfgdir) {
-			BLI_make_file_string(G.main->name, tstr, cfgdir, BLENDER_STARTUP_FILE);
+		const char * const cfgdir = BKE_appdir_folder_id(BLENDER_USER_CONFIG, NULL);
+		if (custom_file) {
+			BLI_strncpy(startstr, custom_file, FILE_MAX);
+
+			if (cfgdir) {
+				BLI_make_file_string(G.main->name, prefstr, cfgdir, BLENDER_USERPREF_FILE);
+			}
+			else {
+				prefstr[0] = '\0';
+			}
+		}
+		else if (cfgdir) {
+			BLI_make_file_string(G.main->name, startstr, cfgdir, BLENDER_STARTUP_FILE);
+			BLI_make_file_string(G.main->name, prefstr, cfgdir, BLENDER_USERPREF_FILE);
 		}
 		else {
-			tstr[0] = '\0';
+			startstr[0] = '\0';
+			prefstr[0] = '\0';
 			from_memory = 1;
 		}
 	}
 	
-	/* prevent loading no UI */
-	G.fileflags &= ~G_FILE_NO_UI;
-	
 	/* put aside screens to match with persistent windows later */
-	wm_window_match_init(C, &wmbase); 
+	wm_window_match_init(C, &wmbase);
 	
-	if (!from_memory && BLI_exists(tstr)) {
-		success = (BKE_read_file(C, tstr, NULL) != BKE_READ_FILE_FAIL);
-		
-		if (U.themes.first == NULL) {
-			printf("\nError: No valid "STRINGIFY (BLENDER_STARTUP_FILE)", fall back to built-in default.\n\n");
+	if (!from_memory) {
+		if (BLI_access(startstr, R_OK) == 0) {
+			success = (BKE_read_file(C, startstr, NULL) != BKE_READ_FILE_FAIL);
+		}
+		if (BLI_listbase_is_empty(&U.themes)) {
+			if (G.debug & G_DEBUG)
+				printf("\nNote: No (valid) '%s' found, fall back to built-in default.\n\n", startstr);
 			success = 0;
 		}
 	}
-	if (success == 0) {
-		success = BKE_read_file_from_memory(C, datatoc_startup_blend, datatoc_startup_blend_size, NULL);
-		if (wmbase.first == NULL) wm_clear_default_size(C);
 
-#ifdef WITH_PYTHON_SECURITY /* not default */
+	if (success == 0 && custom_file && reports) {
+		BKE_reportf(reports, RPT_ERROR, "Could not read '%s'", custom_file);
+		/*We can not return from here because wm is already reset*/
+	}
+
+	if (success == 0) {
+		success = BKE_read_file_from_memory(C, datatoc_startup_blend, datatoc_startup_blend_size, NULL, true);
+		if (BLI_listbase_is_empty(&wmbase)) {
+			wm_clear_default_size(C);
+		}
+		BKE_tempdir_init(U.tempdir);
+
+#ifdef WITH_PYTHON_SECURITY
 		/* use alternative setting for security nuts
 		 * otherwise we'd need to patch the binary blob - startup.blend.c */
 		U.flag |= USER_SCRIPT_AUTOEXEC_DISABLE;
 #endif
 	}
 	
+	/* check new prefs only after startup.blend was finished */
+	if (!from_memory && BLI_exists(prefstr)) {
+		int done = BKE_read_file_userdef(prefstr, NULL);
+		if (done != BKE_READ_FILE_FAIL) {
+			read_userdef_from_memory = false;
+			printf("Read new prefs: %s\n", prefstr);
+		}
+	}
+	
 	/* prevent buggy files that had G_FILE_RELATIVE_REMAP written out by mistake. Screws up autosaves otherwise
 	 * can remove this eventually, only in a 2.53 and older, now its not written */
 	G.fileflags &= ~G_FILE_RELATIVE_REMAP;
 	
 	/* check userdef before open window, keymaps etc */
-	wm_init_userdef(C);
+	wm_init_userdef(C, read_userdef_from_memory);
 	
 	/* match the read WM with current WM */
 	wm_window_match_do(C, &wmbase); 
@@ -559,20 +658,21 @@ int WM_homefile_read(bContext *C, ReportList *UNUSED(reports), short from_memory
 	BKE_write_undo(C, "original");  /* save current state */
 
 	ED_editors_init(C);
-	DAG_on_visible_update(CTX_data_main(C), TRUE);
+	DAG_on_visible_update(CTX_data_main(C), true);
 
 #ifdef WITH_PYTHON
 	if (CTX_py_init_get(C)) {
 		/* sync addons, these may have changed from the defaults */
 		BPY_string_exec(C, "__import__('addon_utils').reset_all()");
 
-		BPY_driver_reset();
-		BPY_app_handlers_reset(FALSE);
-		BPY_modules_load_user(C);
+		BPY_python_reset(C);
 	}
 #endif
 
+	WM_operatortype_last_properties_clear_all();
+
 	/* important to do before NULL'ing the context */
+	BLI_callback_exec(CTX_data_main(C), NULL, BLI_CB_EVT_VERSION_UPDATE);
 	BLI_callback_exec(CTX_data_main(C), NULL, BLI_CB_EVT_LOAD_POST);
 
 	WM_event_add_notifier(C, NC_WM | ND_FILEREAD, NULL);
@@ -582,23 +682,51 @@ int WM_homefile_read(bContext *C, ReportList *UNUSED(reports), short from_memory
 		CTX_wm_window_set(C, NULL); /* exits queues */
 	}
 
-	return TRUE;
+	return true;
 }
 
-int WM_homefile_read_exec(bContext *C, wmOperator *op)
+int wm_history_read_exec(bContext *UNUSED(C), wmOperator *UNUSED(op))
 {
-	int from_memory = strcmp(op->type->idname, "WM_OT_read_factory_settings") == 0;
-	return WM_homefile_read(C, op->reports, from_memory) ? OPERATOR_FINISHED : OPERATOR_CANCELLED;
+	ED_file_read_bookmarks();
+	wm_read_history();
+	return OPERATOR_FINISHED;
 }
 
-void WM_read_history(void)
+int wm_homefile_read_exec(bContext *C, wmOperator *op)
+{
+	const bool from_memory = (STREQ(op->type->idname, "WM_OT_read_factory_settings"));
+	char filepath_buf[FILE_MAX];
+	const char *filepath = NULL;
+
+	if (!from_memory) {
+		PropertyRNA *prop = RNA_struct_find_property(op->ptr, "filepath");
+
+		/* This can be used when loading of a start-up file should only change
+		 * the scene content but keep the blender UI as it is. */
+		wm_open_init_load_ui(op, true);
+		BKE_BIT_TEST_SET(G.fileflags, !RNA_boolean_get(op->ptr, "load_ui"), G_FILE_NO_UI);
+
+		if (RNA_property_is_set(op->ptr, prop)) {
+			RNA_property_string_get(op->ptr, prop, filepath_buf);
+			filepath = filepath_buf;
+			if (BLI_access(filepath, R_OK)) {
+				BKE_reportf(op->reports, RPT_ERROR, "Can't read alternative start-up file: '%s'", filepath);
+				return OPERATOR_CANCELLED;
+			}
+		}
+	}
+
+	return wm_homefile_read(C, op->reports, from_memory, filepath) ? OPERATOR_FINISHED : OPERATOR_CANCELLED;
+}
+
+void wm_read_history(void)
 {
 	char name[FILE_MAX];
 	LinkNode *l, *lines;
 	struct RecentFile *recent;
-	char *line;
+	const char *line;
 	int num;
-	char *cfgdir = BLI_get_folder(BLENDER_USER_CONFIG, NULL);
+	const char * const cfgdir = BKE_appdir_folder_id(BLENDER_USER_CONFIG, NULL);
 
 	if (!cfgdir) return;
 
@@ -606,7 +734,7 @@ void WM_read_history(void)
 
 	lines = BLI_file_read_as_lines(name);
 
-	G.recent_files.first = G.recent_files.last = NULL;
+	BLI_listbase_clear(&G.recent_files);
 
 	/* read list of recent opened files from recent-files.txt to memory */
 	for (l = lines, num = 0; l && (num < U.recent_files); l = l->next) {
@@ -620,19 +748,22 @@ void WM_read_history(void)
 	}
 	
 	BLI_file_free_lines(lines);
-
 }
 
 static void write_history(void)
 {
 	struct RecentFile *recent, *next_recent;
 	char name[FILE_MAX];
-	char *user_config_dir;
+	const char *user_config_dir;
 	FILE *fp;
 	int i;
 
+	/* no write history for recovered startup files */
+	if (G.main->name[0] == 0)
+		return;
+	
 	/* will be NULL in background mode */
-	user_config_dir = BLI_get_folder_create(BLENDER_USER_CONFIG, NULL);
+	user_config_dir = BKE_appdir_folder_id_create(BLENDER_USER_CONFIG, NULL);
 	if (!user_config_dir)
 		return;
 
@@ -709,14 +840,14 @@ static ImBuf *blend_file_thumb(Scene *scene, bScreen *screen, int **thumb_pt)
 	if (scene->camera) {
 		ibuf = ED_view3d_draw_offscreen_imbuf_simple(scene, scene->camera,
 		                                             BLEN_THUMB_SIZE * 2, BLEN_THUMB_SIZE * 2,
-		                                             IB_rect, OB_SOLID, FALSE, FALSE, err_out);
+		                                             IB_rect, OB_SOLID, false, false, false, R_ADDSKY, err_out);
 	}
 	else {
 		ibuf = ED_view3d_draw_offscreen_imbuf(scene, v3d, ar, BLEN_THUMB_SIZE * 2, BLEN_THUMB_SIZE * 2,
-		                                      IB_rect, FALSE, FALSE, err_out);
+		                                      IB_rect, false, R_ADDSKY, err_out);
 	}
 
-	if (ibuf) {		
+	if (ibuf) {
 		float aspect = (scene->r.xsch * scene->r.xasp) / (scene->r.ysch * scene->r.yasp);
 
 		/* dirty oversampling */
@@ -746,7 +877,7 @@ static ImBuf *blend_file_thumb(Scene *scene, bScreen *screen, int **thumb_pt)
 }
 
 /* easy access from gdb */
-int write_crash_blend(void)
+bool write_crash_blend(void)
 {
 	char path[FILE_MAX];
 	int fileflags = G.fileflags & ~(G_FILE_HISTORY); /* don't do file history on crash file */
@@ -763,16 +894,17 @@ int write_crash_blend(void)
 	}
 }
 
-int WM_file_write(bContext *C, const char *target, int fileflags, ReportList *reports, int copy)
+/**
+ * \see #wm_homefile_write_exec wraps #BLO_write_file in a similar way.
+ */
+int wm_file_write(bContext *C, const char *filepath, int fileflags, ReportList *reports)
 {
 	Library *li;
 	int len;
-	char filepath[FILE_MAX];
-
 	int *thumb = NULL;
 	ImBuf *ibuf_thumb = NULL;
 
-	len = strlen(target);
+	len = strlen(filepath);
 	
 	if (len == 0) {
 		BKE_report(reports, RPT_ERROR, "Path is empty, cannot save");
@@ -783,15 +915,21 @@ int WM_file_write(bContext *C, const char *target, int fileflags, ReportList *re
 		BKE_report(reports, RPT_ERROR, "Path too long, cannot save");
 		return -1;
 	}
+	
+	/* Check if file write permission is ok */
+	if (BLI_exists(filepath) && !BLI_file_is_writable(filepath)) {
+		BKE_reportf(reports, RPT_ERROR, "Cannot save blend file, path '%s' is not writable", filepath);
+		return -1;
+	}
  
-	BLI_strncpy(filepath, target, FILE_MAX);
-	BLI_replace_extension(filepath, FILE_MAX, ".blend");
-	/* don't use 'target' anymore */
+	/* note: used to replace the file extension (to ensure '.blend'),
+	 * no need to now because the operator ensures,
+	 * its handy for scripts to save to a predefined name without blender editing it */
 	
 	/* send the OnSave event */
 	for (li = G.main->library.first; li; li = li->id.next) {
 		if (BLI_path_cmp(li->filepath, filepath) == 0) {
-			BKE_reportf(reports, RPT_ERROR, "Can't overwrite used library '%.240s'", filepath);
+			BKE_reportf(reports, RPT_ERROR, "Cannot overwrite used library '%.240s'", filepath);
 			return -1;
 		}
 	}
@@ -809,28 +947,33 @@ int WM_file_write(bContext *C, const char *target, int fileflags, ReportList *re
 	if (G.fileflags & G_AUTOPACK) {
 		packAll(G.main, reports);
 	}
-	
-	ED_object_exit_editmode(C, EM_DO_UNDO);
-	ED_sculpt_force_update(C);
 
 	/* don't forget not to return without! */
 	WM_cursor_wait(1);
 	
+	ED_editors_flush_edits(C, false);
+
 	fileflags |= G_FILE_HISTORY; /* write file history */
 
+	/* first time saving */
+	/* XXX temp solution to solve bug, real fix coming (ton) */
+	if ((G.main->name[0] == '\0') && !(fileflags & G_FILE_SAVE_COPY)) {
+		BLI_strncpy(G.main->name, filepath, sizeof(G.main->name));
+	}
+
+	/* XXX temp solution to solve bug, real fix coming (ton) */
+	G.main->recovered = 0;
+	
 	if (BLO_write_file(CTX_data_main(C), filepath, fileflags, reports, thumb)) {
-		if (!copy) {
+		if (!(fileflags & G_FILE_SAVE_COPY)) {
 			G.relbase_valid = 1;
 			BLI_strncpy(G.main->name, filepath, sizeof(G.main->name));  /* is guaranteed current file */
 	
 			G.save_over = 1; /* disable untitled.blend convention */
 		}
 
-		if (fileflags & G_FILE_COMPRESS) G.fileflags |= G_FILE_COMPRESS;
-		else G.fileflags &= ~G_FILE_COMPRESS;
-		
-		if (fileflags & G_FILE_AUTOPLAY) G.fileflags |= G_FILE_AUTOPLAY;
-		else G.fileflags &= ~G_FILE_AUTOPLAY;
+		BKE_BIT_TEST_SET(G.fileflags, fileflags & G_FILE_COMPRESS, G_FILE_COMPRESS);
+		BKE_BIT_TEST_SET(G.fileflags, fileflags & G_FILE_AUTOPLAY, G_FILE_AUTOPLAY);
 
 		/* prevent background mode scripts from clobbering history */
 		if (!G.background) {
@@ -861,28 +1004,34 @@ int WM_file_write(bContext *C, const char *target, int fileflags, ReportList *re
 	return 0;
 }
 
-/* operator entry */
-int WM_homefile_write_exec(bContext *C, wmOperator *op)
+/**
+ * \see #wm_file_write wraps #BLO_write_file in a similar way.
+ */
+int wm_homefile_write_exec(bContext *C, wmOperator *op)
 {
 	wmWindowManager *wm = CTX_wm_manager(C);
 	wmWindow *win = CTX_wm_window(C);
 	char filepath[FILE_MAX];
 	int fileflags;
 
+	BLI_callback_exec(G.main, NULL, BLI_CB_EVT_SAVE_PRE);
+
 	/* check current window and close it if temp */
-	if (win->screen->temp)
+	if (win && win->screen->temp)
 		wm_window_close(C, wm, win);
 	
 	/* update keymaps in user preferences */
 	WM_keyconfig_update(wm);
 	
-	BLI_make_file_string("/", filepath, BLI_get_folder_create(BLENDER_USER_CONFIG, NULL), BLENDER_STARTUP_FILE);
+	BLI_make_file_string("/", filepath, BKE_appdir_folder_id_create(BLENDER_USER_CONFIG, NULL), BLENDER_STARTUP_FILE);
 	printf("trying to save homefile at %s ", filepath);
 	
+	ED_editors_flush_edits(C, false);
+
 	/*  force save as regular blend file */
 	fileflags = G.fileflags & ~(G_FILE_COMPRESS | G_FILE_AUTOPLAY | G_FILE_LOCK | G_FILE_SIGN | G_FILE_HISTORY);
 
-	if (BLO_write_file(CTX_data_main(C), filepath, fileflags, op->reports, NULL) == 0) {
+	if (BLO_write_file(CTX_data_main(C), filepath, fileflags | G_FILE_USERPREFS, op->reports, NULL) == 0) {
 		printf("fail\n");
 		return OPERATOR_CANCELLED;
 	}
@@ -891,6 +1040,30 @@ int WM_homefile_write_exec(bContext *C, wmOperator *op)
 
 	G.save_over = 0;
 
+	BLI_callback_exec(G.main, NULL, BLI_CB_EVT_SAVE_POST);
+
+	return OPERATOR_FINISHED;
+}
+
+/* Only save the prefs block. operator entry */
+int wm_userpref_write_exec(bContext *C, wmOperator *op)
+{
+	wmWindowManager *wm = CTX_wm_manager(C);
+	char filepath[FILE_MAX];
+	
+	/* update keymaps in user preferences */
+	WM_keyconfig_update(wm);
+	
+	BLI_make_file_string("/", filepath, BKE_appdir_folder_id_create(BLENDER_USER_CONFIG, NULL), BLENDER_USERPREF_FILE);
+	printf("trying to save userpref at %s ", filepath);
+	
+	if (BKE_write_file_userdef(filepath, op->reports) == 0) {
+		printf("fail\n");
+		return OPERATOR_CANCELLED;
+	}
+	
+	printf("ok\n");
+	
 	return OPERATOR_FINISHED;
 }
 
@@ -898,12 +1071,20 @@ int WM_homefile_write_exec(bContext *C, wmOperator *op)
 
 void wm_autosave_location(char *filepath)
 {
-	char pidstr[32];
+	const int pid = abs(getpid());
+	char path[1024];
 #ifdef WIN32
-	char *savedir;
+	const char *savedir;
 #endif
 
-	BLI_snprintf(pidstr, sizeof(pidstr), "%d.blend", abs(getpid()));
+	if (G.main && G.relbase_valid) {
+		const char *basename = BLI_path_basename(G.main->name);
+		int len = strlen(basename) - 6;
+		BLI_snprintf(path, sizeof(path), "%.*s-%d.blend", len, basename, pid);
+	}
+	else {
+		BLI_snprintf(path, sizeof(path), "%d.blend", pid);
+	}
 
 #ifdef WIN32
 	/* XXX Need to investigate how to handle default location of '/tmp/'
@@ -914,14 +1095,14 @@ void wm_autosave_location(char *filepath)
 	 * BLI_make_file_string will create string that has it most likely on C:\
 	 * through get_default_root().
 	 * If there is no C:\tmp autosave fails. */
-	if (!BLI_exists(BLI_temporary_dir())) {
-		savedir = BLI_get_folder_create(BLENDER_USER_AUTOSAVE, NULL);
-		BLI_make_file_string("/", filepath, savedir, pidstr);
+	if (!BLI_exists(BKE_tempdir_base())) {
+		savedir = BKE_appdir_folder_id_create(BLENDER_USER_AUTOSAVE, NULL);
+		BLI_make_file_string("/", filepath, savedir, path);
 		return;
 	}
 #endif
 
-	BLI_make_file_string("/", filepath, BLI_temporary_dir(), pidstr);
+	BLI_make_file_string("/", filepath, BKE_tempdir_base(), path);
 }
 
 void WM_autosave_init(wmWindowManager *wm)
@@ -937,9 +1118,7 @@ void wm_autosave_timer(const bContext *C, wmWindowManager *wm, wmTimer *UNUSED(w
 	wmWindow *win;
 	wmEventHandler *handler;
 	char filepath[FILE_MAX];
-	int fileflags;
-	Scene *scene = CTX_data_scene(C);
-
+	
 	WM_event_remove_timer(wm, NULL, wm->autosavetimer);
 
 	/* if a modal operator is running, don't autosave, but try again in 10 seconds */
@@ -952,21 +1131,21 @@ void wm_autosave_timer(const bContext *C, wmWindowManager *wm, wmTimer *UNUSED(w
 		}
 	}
 
-	if (scene) {
-		Object *ob = OBACT;
-
-		if (ob && ob->mode & OB_MODE_SCULPT)
-			multires_force_update(ob);
-	}
+	ED_editors_flush_edits(C, false);
 
 	wm_autosave_location(filepath);
 
-	/*  force save as regular blend file */
-	fileflags = G.fileflags & ~(G_FILE_COMPRESS | G_FILE_AUTOPLAY | G_FILE_LOCK | G_FILE_SIGN | G_FILE_HISTORY);
-
-	/* no error reporting to console */
-	BLO_write_file(CTX_data_main(C), filepath, fileflags, NULL, NULL);
+	if (U.uiflag & USER_GLOBALUNDO) {
+		/* fast save of last undobuffer, now with UI */
+		BKE_undo_save_file(filepath);
+	}
+	else {
+		/*  save as regular blend file */
+		int fileflags = G.fileflags & ~(G_FILE_COMPRESS | G_FILE_AUTOPLAY | G_FILE_LOCK | G_FILE_SIGN | G_FILE_HISTORY);
 
+		/* no error reporting to console */
+		BLO_write_file(CTX_data_main(C), filepath, fileflags, NULL, NULL);
+	}
 	/* do timer after file write, just in case file write takes a long time */
 	wm->autosavetimer = WM_event_add_timer(wm, NULL, TIMERAUTOSAVE, U.savetime * 60.0);
 }
@@ -987,10 +1166,10 @@ void wm_autosave_delete(void)
 
 	if (BLI_exists(filename)) {
 		char str[FILE_MAX];
-		BLI_make_file_string("/", str, BLI_temporary_dir(), "quit.blend");
+		BLI_make_file_string("/", str, BKE_tempdir_base(), BLENDER_QUIT_FILE);
 
 		/* if global undo; remove tempsave, otherwise rename */
-		if (U.uiflag & USER_GLOBALUNDO) BLI_delete(filename, 0, 0);
+		if (U.uiflag & USER_GLOBALUNDO) BLI_delete(filename, false, false);
 		else BLI_rename(filename, str);
 	}
 }
@@ -1003,3 +1182,39 @@ void wm_autosave_read(bContext *C, ReportList *reports)
 	WM_file_read(C, filename, reports);
 }
 
+
+/** \name Initialize WM_OT_open_xxx properties
+ *
+ * Check if load_ui was set by the caller.
+ * Fall back to user preference when file flags not specified.
+ *
+ * \{ */
+
+void wm_open_init_load_ui(wmOperator *op, bool use_prefs)
+{
+	PropertyRNA *prop = RNA_struct_find_property(op->ptr, "load_ui");
+	if (!RNA_property_is_set(op->ptr, prop)) {
+		bool value = use_prefs ?
+		             ((U.flag & USER_FILENOUI) == 0) :
+		             ((G.fileflags & G_FILE_NO_UI) == 0);
+
+		RNA_property_boolean_set(op->ptr, prop, value);
+	}
+}
+
+void wm_open_init_use_scripts(wmOperator *op, bool use_prefs)
+{
+	PropertyRNA *prop = RNA_struct_find_property(op->ptr, "use_scripts");
+	if (!RNA_property_is_set(op->ptr, prop)) {
+		/* use G_SCRIPT_AUTOEXEC rather than the userpref because this means if
+		 * the flag has been disabled from the command line, then opening
+		 * from the menu wont enable this setting. */
+		bool value = use_prefs ?
+		             ((U.flag & USER_SCRIPT_AUTOEXEC_DISABLE) == 0) :
+		             ((G.f & G_SCRIPT_AUTOEXEC) != 0);
+
+		RNA_property_boolean_set(op->ptr, prop, value);
+	}
+}
+
+/** \} */
