@@ -13,7 +13,7 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software Foundation,
- * Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
  *
  * The Original Code is Copyright (C) 2001-2002 by NaN Holding BV.
  * All rights reserved.
@@ -49,18 +49,17 @@
 
 #include "PIL_time.h"
 
+#include "BLI_utildefines.h"
 #include "BLI_fileops.h"
 #include "BLI_listbase.h"
 #include "BLI_path_util.h"
-#include "BLI_rect.h"
 #include "BLI_string.h"
-#include "BLI_utildefines.h"
 
 #include "IMB_imbuf_types.h"
 #include "IMB_imbuf.h"
 
-#include "BKE_blender.h"
-#include "BKE_global.h"
+#include "BKE_depsgraph.h"
+#include "BKE_image.h"
 
 #include "BIF_gl.h"
 #include "BIF_glutil.h"
@@ -70,24 +69,37 @@
 #include "GHOST_C-api.h"
 #include "BLF_api.h"
 
-#include "wm_event_types.h"
-
 #include "WM_api.h"  /* only for WM_main_playanim */
 
+struct PlayState;
+static void playanim_window_zoom(struct PlayState *ps, const float zoom_offset);
+
 typedef struct PlayState {
 
+	/* window and viewport size */
+	int win_x, win_y;
+	
+	/* current zoom level */
+	float zoom;
+
 	/* playback state */
 	short direction;
-	short next;
-	short once;
-	short turbo;
-	short pingpong;
-	short noskip;
-	short sstep;
-	short pause;
-	short wait2;
-	short stopped;
-	short go;
+	short next_frame;
+
+	bool  once;
+	bool  turbo;
+	bool  pingpong;
+	bool  noskip;
+	bool  sstep;
+	bool  wait2;
+	bool  stopped;
+	bool  go;
+	/* waiting for images to load */
+	bool  loading;
+	/* x/y image flip */
+	bool draw_flip[2];
+	
+	int fstep;
 
 	/* current picture */
 	struct PlayAnimPict *picture;
@@ -98,6 +110,9 @@ typedef struct PlayState {
 
 	/* saves passing args */
 	struct ImBuf *curframe_ibuf;
+	
+	/* restarts player for file drop */
+	char dropped_file[FILE_MAX];
 } PlayState;
 
 /* for debugging */
@@ -130,6 +145,7 @@ typedef enum eWS_Qual {
 	WS_QUAL_ALT     = (WS_QUAL_LALT | WS_QUAL_RALT),
 	WS_QUAL_LCTRL   = (1 << 4),
 	WS_QUAL_RCTRL   = (1 << 5),
+	WS_QUAL_CTRL    = (WS_QUAL_LCTRL | WS_QUAL_RCTRL),
 	WS_QUAL_LMOUSE  = (1 << 16),
 	WS_QUAL_MMOUSE  = (1 << 17),
 	WS_QUAL_RMOUSE  = (1 << 18),
@@ -152,6 +168,15 @@ static void playanim_window_get_size(int *width_r, int *height_r)
 	GHOST_DisposeRectangle(bounds);
 }
 
+static void playanim_gl_matrix(void)
+{
+	/* unified matrix, note it affects offset for drawing */
+	glMatrixMode(GL_PROJECTION);
+	glLoadIdentity();
+	glOrtho(0.0f, 1.0f, 0.0f, 1.0f, -1.0f, 1.0f);
+	glMatrixMode(GL_MODELVIEW);
+}
+
 /* implementation */
 static void playanim_event_qual_update(void)
 {
@@ -177,34 +202,19 @@ static void playanim_event_qual_update(void)
 
 	/* Alt */
 	GHOST_GetModifierKeyState(g_WS.ghost_system, GHOST_kModifierKeyLeftAlt, &val);
-	if (val) g_WS.qual |=  WS_QUAL_LCTRL;
-	else     g_WS.qual &= ~WS_QUAL_LCTRL;
+	if (val) g_WS.qual |=  WS_QUAL_LALT;
+	else     g_WS.qual &= ~WS_QUAL_LALT;
 
 	GHOST_GetModifierKeyState(g_WS.ghost_system, GHOST_kModifierKeyRightAlt, &val);
-	if (val) g_WS.qual |=  WS_QUAL_RCTRL;
-	else     g_WS.qual &= ~WS_QUAL_RCTRL;
-
-	/* LMB */
-	GHOST_GetButtonState(g_WS.ghost_system, GHOST_kButtonMaskLeft, &val);
-	if (val) g_WS.qual |=  WS_QUAL_LMOUSE;
-	else     g_WS.qual &= ~WS_QUAL_LMOUSE;
-
-	/* MMB */
-	GHOST_GetButtonState(g_WS.ghost_system, GHOST_kButtonMaskMiddle, &val);
-	if (val) g_WS.qual |=  WS_QUAL_MMOUSE;
-	else     g_WS.qual &= ~WS_QUAL_MMOUSE;
-
-	/* RMB */
-	GHOST_GetButtonState(g_WS.ghost_system, GHOST_kButtonMaskRight, &val);
-	if (val) g_WS.qual |=  WS_QUAL_RMOUSE;
-	else     g_WS.qual &= ~WS_QUAL_RMOUSE;
+	if (val) g_WS.qual |=  WS_QUAL_RALT;
+	else     g_WS.qual &= ~WS_QUAL_RALT;
 }
 
 typedef struct PlayAnimPict {
 	struct PlayAnimPict *next, *prev;
 	char *mem;
 	int size;
-	char *name;
+	const char *name;
 	struct ImBuf *ibuf;
 	struct anim *anim;
 	int frame;
@@ -212,11 +222,24 @@ typedef struct PlayAnimPict {
 } PlayAnimPict;
 
 static struct ListBase picsbase = {NULL, NULL};
-static int fromdisk = FALSE;
-static int fstep = 1;
-static float zoomx = 1.0, zoomy = 1.0;
+static bool fromdisk = false;
 static double ptottime = 0.0, swaptime = 0.04;
 
+static PlayAnimPict *playanim_step(PlayAnimPict *playanim, int step)
+{
+	if (step > 0) {
+		while (step-- && playanim) {
+			playanim = playanim->next;
+		}
+	}
+	else if (step < 0) {
+		while (step++ && playanim) {
+			playanim = playanim->prev;
+		}
+	}
+	return playanim;
+}
+
 static int pupdate_time(void)
 {
 	static double ltime;
@@ -229,11 +252,13 @@ static int pupdate_time(void)
 	return (ptottime < 0);
 }
 
-static void playanim_toscreen(PlayAnimPict *picture, struct ImBuf *ibuf, int fontid)
+static void playanim_toscreen(PlayState *ps, PlayAnimPict *picture, struct ImBuf *ibuf, int fontid, int fstep)
 {
+	float offs_x, offs_y;
+	float span_x, span_y;
 
 	if (ibuf == NULL) {
-		printf("no ibuf !\n");
+		printf("%s: no ibuf for picture '%s'\n", __func__, picture ? picture->name : "<NIL>");
 		return;
 	}
 	if (ibuf->rect == NULL && ibuf->rect_float) {
@@ -245,10 +270,39 @@ static void playanim_toscreen(PlayAnimPict *picture, struct ImBuf *ibuf, int fon
 
 	GHOST_ActivateWindowDrawingContext(g_WS.ghost_window);
 
-	glRasterPos2f(0.0f, 0.0f);
+	/* size within window */
+	span_x = (ps->zoom * ibuf->x) / (float)ps->win_x;
+	span_y = (ps->zoom * ibuf->y) / (float)ps->win_y;
+
+	/* offset within window */
+	offs_x = 0.5f * (1.0f - span_x);
+	offs_y = 0.5f * (1.0f - span_y);
+
+	CLAMP(offs_x, 0.0f, 1.0f);
+	CLAMP(offs_y, 0.0f, 1.0f);
+	glRasterPos2f(offs_x, offs_y);
+
+	glClearColor(0.1, 0.1, 0.1, 0.0);
+	glClear(GL_COLOR_BUFFER_BIT);
+	
+	/* checkerboard for case alpha */
+	if (ibuf->planes == 32) {
+		glEnable(GL_BLEND);
+		glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
+
+		fdrawcheckerboard(offs_x, offs_y, offs_x + span_x, offs_y + span_y);
+	}
+
+	glRasterPos2f(offs_x + (ps->draw_flip[0] ? span_x : 0.0f),
+	              offs_y + (ps->draw_flip[1] ? span_y : 0.0f));
+
+	glPixelZoom(ps->zoom * (ps->draw_flip[0] ? -1.0f : 1.0f),
+	            ps->zoom * (ps->draw_flip[1] ? -1.0f : 1.0f));
 
 	glDrawPixels(ibuf->x, ibuf->y, GL_RGBA, GL_UNSIGNED_BYTE, ibuf->rect);
 
+	glDisable(GL_BLEND);
+	
 	pupdate_time();
 
 	if (picture && (g_WS.qual & (WS_QUAL_SHIFT | WS_QUAL_LMOUSE)) && (fontid != -1)) {
@@ -271,7 +325,7 @@ static void playanim_toscreen(PlayAnimPict *picture, struct ImBuf *ibuf, int fon
 	GHOST_SwapWindowBuffers(g_WS.ghost_window);
 }
 
-static void build_pict_list(char *first, int totframes, int fstep, int fontid)
+static void build_pict_list_ex(PlayState *ps, const char *first, int totframes, int fstep, int fontid)
 {
 	char *mem, filepath[FILE_MAX];
 //	short val;
@@ -287,7 +341,7 @@ static void build_pict_list(char *first, int totframes, int fstep, int fontid)
 			int pic;
 			ibuf = IMB_anim_absolute(anim, 0, IMB_TC_NONE, IMB_PROXY_NONE);
 			if (ibuf) {
-				playanim_toscreen(NULL, ibuf, fontid);
+				playanim_toscreen(ps, NULL, ibuf, fontid, fstep);
 				IMB_freeImBuf(ibuf);
 			}
 
@@ -325,10 +379,11 @@ static void build_pict_list(char *first, int totframes, int fstep, int fontid)
 		 */
 
 		while (IMB_ispic(filepath) && totframes) {
+			bool hasevent;
 			size_t size;
 			int file;
 
-			file = open(filepath, O_BINARY | O_RDONLY, 0);
+			file = BLI_open(filepath, O_BINARY | O_RDONLY, 0);
 			if (file < 0) {
 				/* print errno? */
 				return;
@@ -351,7 +406,7 @@ static void build_pict_list(char *first, int totframes, int fstep, int fontid)
 			picture->size = size;
 			picture->IB_flags = IB_rect;
 
-			if (fromdisk == FALSE) {
+			if (fromdisk == false) {
 				mem = (char *)MEM_mallocN(size, "build pic list");
 				if (mem == NULL) {
 					printf("Couldn't get memory\n");
@@ -390,7 +445,7 @@ static void build_pict_list(char *first, int totframes, int fstep, int fontid)
 					ibuf = IMB_loadiffname(picture->name, picture->IB_flags, NULL);
 				}
 				if (ibuf) {
-					playanim_toscreen(picture, ibuf, fontid);
+					playanim_toscreen(ps, picture, ibuf, fontid, fstep);
 					IMB_freeImBuf(ibuf);
 				}
 				pupdate_time();
@@ -399,21 +454,28 @@ static void build_pict_list(char *first, int totframes, int fstep, int fontid)
 
 			BLI_newname(filepath, +fstep);
 
-#if 0 // XXX25
-			while (qtest()) {
-				switch (qreadN(&val)) {
-					case ESCKEY:
-						if (val) return;
-						break;
+			while ((hasevent = GHOST_ProcessEvents(g_WS.ghost_system, 0))) {
+				if (hasevent) {
+					GHOST_DispatchEvents(g_WS.ghost_system);
+				}
+				if (ps->loading == false) {
+					return;
 				}
 			}
-#endif
+
 			totframes--;
 		}
 	}
 	return;
 }
 
+static void build_pict_list(PlayState *ps, const char *first, int totframes, int fstep, int fontid)
+{
+	ps->loading = true;
+	build_pict_list_ex(ps, first, totframes, fstep, fontid);
+	ps->loading = false;
+}
+
 static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr ps_void)
 {
 	PlayState *ps = (PlayState *)ps_void;
@@ -427,8 +489,39 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr ps_void)
 	/* convert ghost event into value keyboard or mouse */
 	val = ELEM(type, GHOST_kEventKeyDown, GHOST_kEventButtonDown);
 
+
+	/* first check if we're busy loading files */
+	if (ps->loading) {
+		switch (type) {
+			case GHOST_kEventKeyDown:
+			case GHOST_kEventKeyUp:
+			{
+				GHOST_TEventKeyData *key_data;
+
+				key_data = (GHOST_TEventKeyData *)GHOST_GetEventData(evt);
+				switch (key_data->key) {
+					case GHOST_kKeyEsc:
+						ps->loading = false;
+						break;
+					default:
+						break;
+				}
+				break;
+			}
+			default:
+				break;
+		}
+		return 1;
+	}
+
+
 	if (ps->wait2 && ps->stopped) {
-		ps->stopped = FALSE;
+		ps->stopped = false;
+	}
+
+	if (ps->wait2) {
+		pupdate_time();
+		ptottime = 0;
 	}
 
 	switch (type) {
@@ -445,83 +538,100 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr ps_void)
 				case GHOST_kKeyP:
 					if (val) ps->pingpong = !ps->pingpong;
 					break;
+				case GHOST_kKeyF:
+				{
+					if (val) {
+						int axis = (g_WS.qual & WS_QUAL_SHIFT) ? 1 : 0;
+						ps->draw_flip[axis] = !ps->draw_flip[axis];
+					}
+					break;
+				}
+				case GHOST_kKey1:
 				case GHOST_kKeyNumpad1:
-					if (val) swaptime = fstep / 60.0;
+					if (val) swaptime = ps->fstep / 60.0;
 					break;
+				case GHOST_kKey2:
 				case GHOST_kKeyNumpad2:
-					if (val) swaptime = fstep / 50.0;
+					if (val) swaptime = ps->fstep / 50.0;
 					break;
+				case GHOST_kKey3:
 				case GHOST_kKeyNumpad3:
-					if (val) swaptime = fstep / 30.0;
+					if (val) swaptime = ps->fstep / 30.0;
 					break;
+				case GHOST_kKey4:
 				case GHOST_kKeyNumpad4:
 					if (g_WS.qual & WS_QUAL_SHIFT)
-						swaptime = fstep / 24.0;
+						swaptime = ps->fstep / 24.0;
 					else
-						swaptime = fstep / 25.0;
+						swaptime = ps->fstep / 25.0;
 					break;
+				case GHOST_kKey5:
 				case GHOST_kKeyNumpad5:
-					if (val) swaptime = fstep / 20.0;
+					if (val) swaptime = ps->fstep / 20.0;
 					break;
+				case GHOST_kKey6:
 				case GHOST_kKeyNumpad6:
-					if (val) swaptime = fstep / 15.0;
+					if (val) swaptime = ps->fstep / 15.0;
 					break;
+				case GHOST_kKey7:
 				case GHOST_kKeyNumpad7:
-					if (val) swaptime = fstep / 12.0;
+					if (val) swaptime = ps->fstep / 12.0;
 					break;
+				case GHOST_kKey8:
 				case GHOST_kKeyNumpad8:
-					if (val) swaptime = fstep / 10.0;
+					if (val) swaptime = ps->fstep / 10.0;
 					break;
+				case GHOST_kKey9:
 				case GHOST_kKeyNumpad9:
-					if (val) swaptime = fstep / 6.0;
+					if (val) swaptime = ps->fstep / 6.0;
 					break;
 				case GHOST_kKeyLeftArrow:
 					if (val) {
-						ps->sstep = TRUE;
-						ps->wait2 = FALSE;
+						ps->sstep = true;
+						ps->wait2 = false;
 						if (g_WS.qual & WS_QUAL_SHIFT) {
 							ps->picture = picsbase.first;
-							ps->next = 0;
+							ps->next_frame = 0;
 						}
 						else {
-							ps->next = -1;
+							ps->next_frame = -1;
 						}
 					}
 					break;
 				case GHOST_kKeyDownArrow:
 					if (val) {
-						ps->wait2 = FALSE;
+						ps->wait2 = false;
 						if (g_WS.qual & WS_QUAL_SHIFT) {
-							ps->next = ps->direction = -1;
+							ps->next_frame = ps->direction = -1;
 						}
 						else {
-							ps->next = -10;
-							ps->sstep = TRUE;
+							ps->next_frame = -10;
+							ps->sstep = true;
 						}
 					}
 					break;
 				case GHOST_kKeyRightArrow:
 					if (val) {
-						ps->sstep = TRUE;
-						ps->wait2 = FALSE;
+						ps->sstep = true;
+						ps->wait2 = false;
 						if (g_WS.qual & WS_QUAL_SHIFT) {
 							ps->picture = picsbase.last;
-							ps->next = 0;
+							ps->next_frame = 0;
 						}
 						else {
-							ps->next = 1;
+							ps->next_frame = 1;
 						}
 					}
 					break;
 				case GHOST_kKeyUpArrow:
 					if (val) {
-						ps->wait2 = FALSE;
+						ps->wait2 = false;
 						if (g_WS.qual & WS_QUAL_SHIFT) {
-							ps->next = ps->direction = 1;
+							ps->next_frame = ps->direction = 1;
 						}
 						else {
-							ps->next = 10;
-							ps->sstep = TRUE;
+							ps->next_frame = 10;
+							ps->sstep = true;
 						}
 					}
 					break;
@@ -531,95 +641,115 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr ps_void)
 					if (val) {
 						if (g_WS.qual & WS_QUAL_SHIFT) {
 							if (ps->curframe_ibuf)
-								printf(" Name: %s | Speed: %.2f frames/s\n", ps->curframe_ibuf->name, fstep / swaptime);
+								printf(" Name: %s | Speed: %.2f frames/s\n",
+								       ps->curframe_ibuf->name, ps->fstep / swaptime);
 						}
 						else {
-							swaptime = fstep / 5.0;
-						}
-					}
-					break;
-				case GHOST_kKeyEqual:
-					if (val) {
-						if (g_WS.qual & WS_QUAL_SHIFT) {
-							ps->pause++;
-							printf("pause:%d\n", ps->pause);
-						}
-						else {
-							swaptime /= 1.1;
-						}
-					}
-					break;
-				case GHOST_kKeyMinus:
-					if (val) {
-						if (g_WS.qual & WS_QUAL_SHIFT) {
-							ps->pause--;
-							printf("pause:%d\n", ps->pause);
-						}
-						else {
-							swaptime *= 1.1;
+							swaptime = ps->fstep / 5.0;
 						}
 					}
 					break;
+				case GHOST_kKey0:
 				case GHOST_kKeyNumpad0:
 					if (val) {
 						if (ps->once) {
-							ps->once = ps->wait2 = FALSE;
+							ps->once = ps->wait2 = false;
 						}
 						else {
 							ps->picture = NULL;
-							ps->once = TRUE;
-							ps->wait2 = FALSE;
+							ps->once = true;
+							ps->wait2 = false;
 						}
 					}
 					break;
 				case GHOST_kKeyEnter:
 				case GHOST_kKeyNumpadEnter:
 					if (val) {
-						ps->wait2 = ps->sstep = FALSE;
+						ps->wait2 = ps->sstep = false;
 					}
 					break;
+				case GHOST_kKeyPeriod:
 				case GHOST_kKeyNumpadPeriod:
 					if (val) {
-						if (ps->sstep) ps->wait2 = FALSE;
+						if (ps->sstep) {
+							ps->wait2 = false;
+						}
 						else {
-							ps->sstep = TRUE;
+							ps->sstep = true;
 							ps->wait2 = !ps->wait2;
 						}
 					}
 					break;
+				case GHOST_kKeyEqual:
 				case GHOST_kKeyNumpadPlus:
+				{
 					if (val == 0) break;
-					zoomx += 2.0;
-					zoomy += 2.0;
-					/* no break??? - is this intentional? - campbell XXX25 */
+					if (g_WS.qual & WS_QUAL_CTRL) {
+						playanim_window_zoom(ps, 1.0f);
+					}
+					else {
+						swaptime /= 1.1;
+					}
+					break;
+				}
+				case GHOST_kKeyMinus:
 				case GHOST_kKeyNumpadMinus:
 				{
-					int sizex, sizey;
-					/* int ofsx, ofsy; */ /* UNUSED */
-
 					if (val == 0) break;
-					if (zoomx > 1.0) zoomx -= 1.0;
-					if (zoomy > 1.0) zoomy -= 1.0;
-					// playanim_window_get_position(&ofsx, &ofsy);
-					playanim_window_get_size(&sizex, &sizey);
-					/* ofsx += sizex / 2; */ /* UNUSED */
-					/* ofsy += sizey / 2; */ /* UNUSED */
-					sizex = zoomx * ps->ibufx;
-					sizey = zoomy * ps->ibufy;
-					/* ofsx -= sizex / 2; */ /* UNUSED */
-					/* ofsy -= sizey / 2; */ /* UNUSED */
-					// window_set_position(g_WS.ghost_window,sizex,sizey);
-					GHOST_SetClientSize(g_WS.ghost_window, sizex, sizey);
+					if (g_WS.qual & WS_QUAL_CTRL) {
+						playanim_window_zoom(ps, -1.0f);
+					}
+					else {
+						swaptime *= 1.1;
+					}
 					break;
 				}
 				case GHOST_kKeyEsc:
-					ps->go = FALSE;
+					ps->go = false;
 					break;
 				default:
 					break;
 			}
 			break;
 		}
+		case GHOST_kEventButtonDown:
+		case GHOST_kEventButtonUp:
+		{
+			GHOST_TEventButtonData *bd = GHOST_GetEventData(evt);
+			int cx, cy, sizex, sizey, inside_window;
+			
+			GHOST_GetCursorPosition(g_WS.ghost_system, &cx, &cy);
+			GHOST_ScreenToClient(g_WS.ghost_window, cx, cy, &cx, &cy);
+			playanim_window_get_size(&sizex, &sizey);
+
+			inside_window = (cx >= 0 && cx < sizex && cy >= 0 && cy <= sizey);
+			
+			if (bd->button == GHOST_kButtonMaskLeft) {
+				if (type == GHOST_kEventButtonDown) {
+					if (inside_window)
+						g_WS.qual |= WS_QUAL_LMOUSE;
+				}
+				else
+					g_WS.qual &= ~WS_QUAL_LMOUSE;
+			}
+			else if (bd->button == GHOST_kButtonMaskMiddle) {
+				if (type == GHOST_kEventButtonDown) {
+					if (inside_window)
+						g_WS.qual |= WS_QUAL_MMOUSE;
+				}
+				else
+					g_WS.qual &= ~WS_QUAL_MMOUSE;
+			}
+			else if (bd->button == GHOST_kButtonMaskRight) {
+				if (type == GHOST_kEventButtonDown) {
+					if (inside_window)
+						g_WS.qual |= WS_QUAL_RMOUSE;
+				}
+				else
+					g_WS.qual &= ~WS_QUAL_RMOUSE;
+			}
+			break;
+		}
 		case GHOST_kEventCursorMove:
 		{
 			if (g_WS.qual & WS_QUAL_LMOUSE) {
@@ -645,44 +775,67 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr ps_void)
 					if (ps->picture->next == NULL) break;
 					ps->picture = ps->picture->next;
 				}
-				ps->sstep = TRUE;
-				ps->wait2 = FALSE;
-				ps->next = 0;
+				ps->sstep = true;
+				ps->wait2 = false;
+				ps->next_frame = 0;
 			}
 			break;
 		}
+		case GHOST_kEventWindowActivate:
+		case GHOST_kEventWindowDeactivate:
+		{
+			g_WS.qual &= ~WS_QUAL_MOUSE;
+			break;
+		}
 		case GHOST_kEventWindowSize:
 		case GHOST_kEventWindowMove:
 		{
-			int sizex, sizey;
-
-			playanim_window_get_size(&sizex, &sizey);
+			float zoomx, zoomy;
+			
+			playanim_window_get_size(&ps->win_x, &ps->win_y);
 			GHOST_ActivateWindowDrawingContext(g_WS.ghost_window);
 
-			glViewport(0, 0, sizex, sizey);
-			glScissor(0, 0, sizex, sizey);
+			zoomx = (float) ps->win_x / ps->ibufx;
+			zoomy = (float) ps->win_y / ps->ibufy;
+			
+			/* zoom always show entire image */
+			ps->zoom = MIN2(zoomx, zoomy);
+			
+			/* zoom steps of 2 for speed */
+			ps->zoom = floor(ps->zoom + 0.5f);
+			if (ps->zoom < 1.0f) ps->zoom = 1.0f;
+			
+			glViewport(0, 0, ps->win_x, ps->win_y);
+			glScissor(0, 0, ps->win_x, ps->win_y);
+			
+			playanim_gl_matrix();
 
-			zoomx = (float) sizex / ps->ibufx;
-			zoomy = (float) sizey / ps->ibufy;
-			zoomx = floor(zoomx + 0.5);
-			zoomy = floor(zoomy + 0.5);
-			if (zoomx < 1.0) zoomx = 1.0;
-			if (zoomy < 1.0) zoomy = 1.0;
-
-			sizex = zoomx * ps->ibufx;
-			sizey = zoomy * ps->ibufy;
-
-			glPixelZoom(zoomx, zoomy);
-			glEnable(GL_DITHER);
 			ptottime = 0.0;
-			playanim_toscreen(ps->picture, ps->curframe_ibuf, ps->fontid);
+			playanim_toscreen(ps, ps->picture, ps->curframe_ibuf, ps->fontid, ps->fstep);
 
 			break;
 		}
 		case GHOST_kEventQuit:
 		case GHOST_kEventWindowClose:
 		{
-			ps->go = FALSE;
+			ps->go = false;
+			break;
+		}
+		case GHOST_kEventDraggingDropDone:
+		{
+			GHOST_TEventDragnDropData *ddd = GHOST_GetEventData(evt);
+			
+			if (ddd->dataType == GHOST_kDragnDropTypeFilenames) {
+				GHOST_TStringArray *stra = ddd->data;
+				int a;
+				
+				for (a = 0; a < stra->count; a++) {
+					BLI_strncpy(ps->dropped_file, (char *)stra->strings[a], sizeof(ps->dropped_file));
+					ps->go = false;
+					printf("drop file %s\n", stra->strings[a]);
+					break; /* only one drop element supported now */
+				}
+			}
 			break;
 		}
 		default:
@@ -693,35 +846,48 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr ps_void)
 	return 1;
 }
 
-static void playanim_window_open(const char *title, int posx, int posy, int sizex, int sizey, int start_maximized)
+static void playanim_window_open(const char *title, int posx, int posy, int sizex, int sizey)
 {
-	GHOST_TWindowState inital_state;
 	GHOST_TUns32 scr_w, scr_h;
 
 	GHOST_GetMainDisplayDimensions(g_WS.ghost_system, &scr_w, &scr_h);
 
 	posy = (scr_h - posy - sizey);
 
-	if (start_maximized == G_WINDOWSTATE_FULLSCREEN)
-		inital_state = start_maximized ? GHOST_kWindowStateFullScreen : GHOST_kWindowStateNormal;
-	else
-		inital_state = start_maximized ? GHOST_kWindowStateMaximized : GHOST_kWindowStateNormal;
-
 	g_WS.ghost_window = GHOST_CreateWindow(g_WS.ghost_system,
 	                                       title,
 	                                       posx, posy, sizex, sizey,
-	                                       inital_state,
+	                                       /* could optionally start fullscreen */
+	                                       GHOST_kWindowStateNormal,
 	                                       GHOST_kDrawingContextTypeOpenGL,
-	                                       FALSE /* no stereo */, FALSE);
+	                                       false /* no stereo */, false);
 }
 
+static void playanim_window_zoom(PlayState *ps, const float zoom_offset)
+{
+	int sizex, sizey;
+	/* int ofsx, ofsy; */ /* UNUSED */
 
-void WM_main_playanim(int argc, const char **argv)
+	if (ps->zoom + zoom_offset > 0.0f) ps->zoom += zoom_offset;
+
+	// playanim_window_get_position(&ofsx, &ofsy);
+	playanim_window_get_size(&sizex, &sizey);
+	/* ofsx += sizex / 2; */ /* UNUSED */
+	/* ofsy += sizey / 2; */ /* UNUSED */
+	sizex = ps->zoom * ps->ibufx;
+	sizey = ps->zoom * ps->ibufy;
+	/* ofsx -= sizex / 2; */ /* UNUSED */
+	/* ofsy -= sizey / 2; */ /* UNUSED */
+	// window_set_position(g_WS.ghost_window, sizex, sizey);
+	GHOST_SetClientSize(g_WS.ghost_window, sizex, sizey);
+}
+
+/* return path for restart */
+static char *wm_main_playanim_intern(int argc, const char **argv)
 {
 	struct ImBuf *ibuf = NULL;
-	char filepath[FILE_MAX];
+	static char filepath[FILE_MAX];	/* abused to return dropped file path */
 	GHOST_TUns32 maxwinx, maxwiny;
-	/* short c233 = FALSE, yuvx = FALSE; */ /* UNUSED */
 	int i;
 	/* This was done to disambiguate the name for use under c++. */
 	struct anim *anim = NULL;
@@ -729,23 +895,29 @@ void WM_main_playanim(int argc, const char **argv)
 	int sfra = -1;
 	int efra = -1;
 	int totblock;
-
+	
 	PlayState ps = {0};
 
-	/* ps.doubleb   = TRUE;*/ /* UNUSED */
-	ps.go        = TRUE;
-	ps.direction = TRUE;
-	ps.next      = TRUE;
-	ps.once      = FALSE;
-	ps.turbo     = FALSE;
-	ps.pingpong  = FALSE;
-	ps.noskip    = FALSE;
-	ps.sstep     = FALSE;
-	ps.pause     = FALSE;
-	ps.wait2     = FALSE;
-	ps.stopped   = FALSE;
+	/* ps.doubleb   = true;*/ /* UNUSED */
+	ps.go        = true;
+	ps.direction = true;
+	ps.next_frame = 1;
+	ps.once      = false;
+	ps.turbo     = false;
+	ps.pingpong  = false;
+	ps.noskip    = false;
+	ps.sstep     = false;
+	ps.wait2     = false;
+	ps.stopped   = false;
+	ps.loading   = false;
 	ps.picture   = NULL;
-	/* resetmap = FALSE */
+	ps.dropped_file[0] = 0;
+	ps.zoom      = 1.0f;
+	/* resetmap = false */
+	ps.draw_flip[0] = false;
+	ps.draw_flip[1] = false;
+
+	ps.fstep     = 1;
 
 	ps.fontid = -1;
 
@@ -753,7 +925,7 @@ void WM_main_playanim(int argc, const char **argv)
 		if (argv[1][0] == '-') {
 			switch (argv[1][1]) {
 				case 'm':
-					fromdisk = TRUE;
+					fromdisk = true;
 					break;
 				case 'p':
 					if (argc > 3) {
@@ -784,18 +956,21 @@ void WM_main_playanim(int argc, const char **argv)
 					}
 					break;
 				case 's':
-					sfra = MIN2(MAXFRAME, MAX2(1, atoi(argv[2]) ));
+					sfra = atoi(argv[2]);
+					CLAMP(sfra, 1, MAXFRAME);
 					argc--;
 					argv++;
 					break;
 				case 'e':
-					efra = MIN2(MAXFRAME, MAX2(1, atoi(argv[2]) ));
+					efra = atoi(argv[2]);
+					CLAMP(efra, 1, MAXFRAME);
 					argc--;
 					argv++;
 					break;
 				case 'j':
-					fstep = MIN2(MAXFRAME, MAX2(1, atoi(argv[2])));
-					swaptime *= fstep;
+					ps.fstep = atoi(argv[2]);
+					CLAMP(ps.fstep, 1, MAXFRAME);
+					swaptime *= ps.fstep;
 					argc--;
 					argv++;
 					break;
@@ -829,6 +1004,7 @@ void WM_main_playanim(int argc, const char **argv)
 		}
 	}
 	else if (!IMB_ispic(filepath)) {
+		printf("%s: '%s' not an image file\n", __func__, filepath);
 		exit(1);
 	}
 
@@ -838,34 +1014,26 @@ void WM_main_playanim(int argc, const char **argv)
 	}
 
 	if (ibuf == NULL) {
-		printf("couldn't open %s\n", filepath);
+		printf("%s: '%s' couldn't open\n", __func__, filepath);
 		exit(1);
 	}
 
 #if 0 //XXX25
-	#if !defined(WIN32) && !defined(__APPLE__)
+#if !defined(WIN32) && !defined(__APPLE__)
 	if (fork()) exit(0);
-	#endif
+#endif
 #endif //XXX25
 
-	/* XXX, fixme zr */
 	{
-//		extern void add_to_mainqueue(wmWindow *win, void *user_data, short evt, short val, char ascii);
 
 		GHOST_EventConsumerHandle consumer = GHOST_CreateEventConsumer(ghost_event_proc, &ps);
 
 		g_WS.ghost_system = GHOST_CreateSystem();
 		GHOST_AddEventConsumer(g_WS.ghost_system, consumer);
 
+		playanim_window_open("Blender:Anim", start_x, start_y, ibuf->x, ibuf->y);
 
-
-		playanim_window_open("Blender:Anim", start_x, start_y, ibuf->x, ibuf->y, 0);
-//XXX25		window_set_handler(g_WS.ghost_window, add_to_mainqueue, NULL);
-
-		glMatrixMode(GL_PROJECTION);
-		glLoadIdentity();
-		glOrtho(0.0f, 1.0f, 0.0f, 1.0f, -1.0f, 1.0f);
-		glMatrixMode(GL_MODELVIEW);
+		playanim_gl_matrix();
 	}
 
 	GHOST_GetMainDisplayDimensions(g_WS.ghost_system, &maxwinx, &maxwiny);
@@ -879,11 +1047,15 @@ void WM_main_playanim(int argc, const char **argv)
 
 	ps.ibufx = ibuf->x;
 	ps.ibufy = ibuf->y;
+	
+	ps.win_x = ps.ibufx;
+	ps.win_y = ps.ibufy;
 
 	if (maxwinx % ibuf->x) maxwinx = ibuf->x * (1 + (maxwinx / ibuf->x));
 	if (maxwiny % ibuf->y) maxwiny = ibuf->y * (1 + (maxwiny / ibuf->y));
 
-	glClearColor(0.0, 0.0, 0.0, 0.0);
+	
+	glClearColor(0.1, 0.1, 0.1, 0.0);
 	glClear(GL_COLOR_BUFFER_BIT);
 
 	GHOST_SwapWindowBuffers(g_WS.ghost_window);
@@ -894,11 +1066,11 @@ void WM_main_playanim(int argc, const char **argv)
 		efra = MAXFRAME;
 	}
 
-	build_pict_list(filepath, (efra - sfra) + 1, fstep, ps.fontid);
+	build_pict_list(&ps, filepath, (efra - sfra) + 1, ps.fstep, ps.fontid);
 
 	for (i = 2; i < argc; i++) {
 		BLI_strncpy(filepath, argv[i], sizeof(filepath));
-		build_pict_list(filepath, (efra - sfra) + 1, fstep, ps.fontid);
+		build_pict_list(&ps, filepath, (efra - sfra) + 1, ps.fstep, ps.fontid);
 	}
 
 	IMB_freeImBuf(ibuf);
@@ -923,7 +1095,7 @@ void WM_main_playanim(int argc, const char **argv)
 
 		if (ps.picture == NULL) {
 			printf("couldn't find pictures\n");
-			ps.go = FALSE;
+			ps.go = false;
 		}
 		if (ps.pingpong) {
 			if (ps.direction == 1) {
@@ -936,6 +1108,7 @@ void WM_main_playanim(int argc, const char **argv)
 		if (ptottime > 0.0) ptottime = 0.0;
 
 		while (ps.picture) {
+			int hasevent;
 #ifndef USE_IMB_CACHE
 			if (ibuf != NULL && ibuf->ftype == 0) IMB_freeImBuf(ibuf);
 #endif
@@ -970,8 +1143,8 @@ void WM_main_playanim(int argc, const char **argv)
 
 				while (pupdate_time()) PIL_sleep_ms(1);
 				ptottime -= swaptime;
-				playanim_toscreen(ps.picture, ibuf, ps.fontid);
-			} /* else deleten */
+				playanim_toscreen(&ps, ps.picture, ibuf, ps.fontid, ps.fstep);
+			} /* else delete */
 			else {
 				printf("error: can't play this image type\n");
 				exit(0);
@@ -979,55 +1152,54 @@ void WM_main_playanim(int argc, const char **argv)
 
 			if (ps.once) {
 				if (ps.picture->next == NULL) {
-					ps.wait2 = TRUE;
+					ps.wait2 = true;
 				}
 				else if (ps.picture->prev == NULL) {
-					ps.wait2 = TRUE;
+					ps.wait2 = true;
 				}
 			}
 
-			ps.next = ps.direction;
+			ps.next_frame = ps.direction;
 
 
-			{
-				int hasevent = GHOST_ProcessEvents(g_WS.ghost_system, 0);
+			while ((hasevent = GHOST_ProcessEvents(g_WS.ghost_system, 0)) || ps.wait2) {
 				if (hasevent) {
 					GHOST_DispatchEvents(g_WS.ghost_system);
 				}
-			}
-
-			/* XXX25 - we should not have to do this, but it makes scrubbing functional! */
-			if (g_WS.qual & WS_QUAL_LMOUSE) {
-				ps.next = 0;
-			}
-			else {
-				ps.sstep = 0;
+				/* Note, this still draws for mousemoves on pause */
+				if (ps.wait2) {
+					if (hasevent) {
+						if (ibuf) {
+							while (pupdate_time()) PIL_sleep_ms(1);
+							ptottime -= swaptime;
+							playanim_toscreen(&ps, ps.picture, ibuf, ps.fontid, ps.fstep);
+						}
+					}
+				}
+				if (ps.go == false) {
+					break;
+				}
 			}
 
 			ps.wait2 = ps.sstep;
 
-			if (ps.wait2 == 0 && ps.stopped == 0) {
-				ps.stopped = TRUE;
+			if (ps.wait2 == false && ps.stopped == false) {
+				ps.stopped = true;
 			}
 
 			pupdate_time();
 
-			if (ps.picture && ps.next) {
+			if (ps.picture && ps.next_frame) {
 				/* always at least set one step */
 				while (ps.picture) {
-					if (ps.next < 0) {
-						ps.picture = ps.picture->prev;
-					}
-					else {
-						ps.picture = ps.picture->next;
-					}
+					ps.picture = playanim_step(ps.picture, ps.next_frame);
 
 					if (ps.once && ps.picture != NULL) {
 						if (ps.picture->next == NULL) {
-							ps.wait2 = TRUE;
+							ps.wait2 = true;
 						}
 						else if (ps.picture->prev == NULL) {
-							ps.wait2 = TRUE;
+							ps.wait2 = true;
 						}
 					}
 
@@ -1035,15 +1207,10 @@ void WM_main_playanim(int argc, const char **argv)
 					ptottime -= swaptime;
 				}
 				if (ps.picture == NULL && ps.sstep) {
-					if (ps.next < 0) {
-						ps.picture = picsbase.last;
-					}
-					else if (ps.next > 0) {
-						ps.picture = picsbase.first;
-					}
+					ps.picture = playanim_step(ps.picture, ps.next_frame);
 				}
 			}
-			if (ps.go == FALSE) {
+			if (ps.go == false) {
 				break;
 			}
 		}
@@ -1078,11 +1245,21 @@ void WM_main_playanim(int argc, const char **argv)
 #else
 	/* we still miss freeing a lot!,
 	 * but many areas could skip initialization too for anim play */
-	IMB_exit();
+	
 	BLF_exit();
 #endif
 	GHOST_DisposeWindow(g_WS.ghost_system, g_WS.ghost_window);
 
+	/* early exit, IMB and BKE should be exited only in end */
+	if (ps.dropped_file[0]) {
+		BLI_strncpy(filepath, ps.dropped_file, sizeof(filepath));
+		return filepath;
+	}
+	
+	IMB_exit();
+	BKE_images_exit();
+	DAG_exit();
+
 	totblock = MEM_get_memory_blocks_in_use();
 	if (totblock != 0) {
 		/* prints many bAKey, bArgument's which are tricky to fix */
@@ -1091,4 +1268,25 @@ void WM_main_playanim(int argc, const char **argv)
 		MEM_printmemlist();
 #endif
 	}
+	
+	return NULL;
+}
+
+
+void WM_main_playanim(int argc, const char **argv)
+{
+	bool looping = true;
+
+	while (looping) {
+		const char *filepath = wm_main_playanim_intern(argc, argv);
+
+		if (filepath) {	/* use simple args */
+			argv[1] = "-a";
+			argv[2] = filepath;
+			argc = 3;
+		}
+		else {
+			looping = false;
+		}
+	}
 }
