@@ -26,9 +26,10 @@
 
 /** \file blender/windowmanager/intern/wm_window.c
  *  \ingroup wm
+ *
+ * Window management, wrap GHOST.
  */
 
-
 #include <math.h>
 #include <stdlib.h>
 #include <stdio.h>
@@ -37,12 +38,12 @@
 #include "DNA_listBase.h"	
 #include "DNA_screen_types.h"
 #include "DNA_windowmanager_types.h"
-#include "RNA_access.h"
 
 #include "MEM_guardedalloc.h"
 
 #include "GHOST_C-api.h"
 
+#include "BLI_math.h"
 #include "BLI_blenlib.h"
 #include "BLI_utildefines.h"
 
@@ -55,7 +56,7 @@
 #include "BKE_main.h"
 
 
-#include "BIF_gl.h"
+#include "RNA_access.h"
 
 #include "WM_api.h"
 #include "WM_types.h"
@@ -72,9 +73,16 @@
 
 #include "GPU_draw.h"
 #include "GPU_extensions.h"
+#include "GPU_init_exit.h"
+#include "GPU_glew.h"
 
 #include "UI_interface.h"
 
+/* for assert */
+#ifndef NDEBUG
+#  include "BLI_threads.h"
+#endif
+
 /* the global to talk to ghost */
 static GHOST_SystemHandle g_system = NULL;
 
@@ -91,7 +99,9 @@ static struct WMInitStruct {
 
 	int windowstate;
 	WinOverrideFlag override_flag;
-} wm_init_state = {0, 0, 0, 0, GHOST_kWindowStateNormal, 0};
+	
+	bool native_pixels;
+} wm_init_state = {0, 0, 0, 0, GHOST_kWindowStateNormal, 0, true};
 
 /* ******** win open & close ************ */
 
@@ -107,6 +117,17 @@ void wm_get_screensize(int *width_r, int *height_r)
 	*height_r = uiheight;
 }
 
+/* size of all screens (desktop), useful since the mouse is bound by this */
+void wm_get_desktopsize(int *width_r, int *height_r)
+{
+	unsigned int uiwidth;
+	unsigned int uiheight;
+
+	GHOST_GetAllDisplayDimensions(g_system, &uiwidth, &uiheight);
+	*width_r = uiwidth;
+	*height_r = uiheight;
+}
+
 /* keeps offset and size within monitor bounds */
 /* XXX solve dual screen... */
 static void wm_window_check_position(rcti *rect)
@@ -115,10 +136,6 @@ static void wm_window_check_position(rcti *rect)
 	
 	wm_get_screensize(&width, &height);
 	
-#if defined(__APPLE__) && !defined(GHOST_COCOA)
-	height -= 70;
-#endif
-	
 	if (rect->xmin < 0) {
 		rect->xmax -= rect->xmin;
 		rect->xmin  = 0;
@@ -164,7 +181,7 @@ void wm_window_free(bContext *C, wmWindowManager *wm, wmWindow *win)
 
 		if (CTX_wm_window(C) == win)
 			CTX_wm_window_set(C, NULL);
-	}	
+	}
 
 	/* always set drawable and active to NULL,
 	 * prevents non-drawable state of main windows (bugs #22967 and #25071, possibly #22477 too) */
@@ -238,10 +255,10 @@ wmWindow *wm_window_copy(bContext *C, wmWindow *winorig)
 	BLI_strncpy(win->screenname, win->screen->id.name + 2, sizeof(win->screenname));
 	win->screen->winid = win->winid;
 
-	win->screen->do_refresh = TRUE;
-	win->screen->do_draw = TRUE;
+	win->screen->do_refresh = true;
+	win->screen->do_draw = true;
 
-	win->drawmethod = -1;
+	win->drawmethod = U.wmdrawmethod;
 	win->drawdata = NULL;
 	
 	return win;
@@ -251,51 +268,50 @@ wmWindow *wm_window_copy(bContext *C, wmWindow *winorig)
 void wm_window_close(bContext *C, wmWindowManager *wm, wmWindow *win)
 {
 	wmWindow *tmpwin;
-	bScreen *screen = win->screen;
+	bool do_exit = false;
+	
+	/* first check if we have to quit (there are non-temp remaining windows) */
+	for (tmpwin = wm->windows.first; tmpwin; tmpwin = tmpwin->next) {
+		if (tmpwin == win)
+			continue;
+		if (tmpwin->screen->temp == 0)
+			break;
+	}
+
+	if (tmpwin == NULL)
+		do_exit = 1;
 	
-	/* first check if we have any non-temp remaining windows */
 	if ((U.uiflag & USER_QUIT_PROMPT) && !wm->file_saved) {
-		if (wm->windows.first) {
-			for (tmpwin = wm->windows.first; tmpwin; tmpwin = tmpwin->next) {
-				if (tmpwin == win)
-					continue;
-				if (tmpwin->screen->temp == 0)
-					break;
-			}
-			if (tmpwin == NULL) {
-				if (!GHOST_confirmQuit(win->ghostwin))
-					return;
-			}
+		if (do_exit) {
+			if (!GHOST_confirmQuit(win->ghostwin))
+				return;
 		}
 	}
 
-	BLI_remlink(&wm->windows, win);
-	
-	wm_draw_window_clear(win);
-	CTX_wm_window_set(C, win);  /* needed by handlers */
-	WM_event_remove_handlers(C, &win->handlers);
-	WM_event_remove_handlers(C, &win->modalhandlers);
-	ED_screen_exit(C, win, win->screen); 
-	
-	wm_window_free(C, wm, win);
-	
-	/* if temp screen, delete it after window free (it stops jobs that can access it) */
-	if (screen->temp) {
-		Main *bmain = CTX_data_main(C);
-		BKE_libblock_free(&bmain->screen, screen);
-	}
-	
-	/* check remaining windows */
-	if (wm->windows.first) {
-		for (win = wm->windows.first; win; win = win->next)
-			if (win->screen->temp == 0)
-				break;
-		/* in this case we close all */
-		if (win == NULL)
-			WM_exit(C);
-	}
-	else
+	/* let WM_exit do all freeing, for correct quit.blend save */
+	if (do_exit) {
 		WM_exit(C);
+	}
+	else {
+		bScreen *screen = win->screen;
+		
+		BLI_remlink(&wm->windows, win);
+		
+		wm_draw_window_clear(win);
+		
+		CTX_wm_window_set(C, win);  /* needed by handlers */
+		WM_event_remove_handlers(C, &win->handlers);
+		WM_event_remove_handlers(C, &win->modalhandlers);
+		ED_screen_exit(C, win, win->screen); 
+		
+		wm_window_free(C, wm, win);
+	
+		/* if temp screen, delete it after window free (it stops jobs that can access it) */
+		if (screen->temp) {
+			Main *bmain = CTX_data_main(C);
+			BKE_libblock_free(bmain, screen);
+		}
+	}		
 }
 
 void wm_window_title(wmWindowManager *wm, wmWindow *win)
@@ -304,12 +320,12 @@ void wm_window_title(wmWindowManager *wm, wmWindow *win)
 		/* nothing to do for 'temp' windows,
 		 * because WM_window_open_temp always sets window title  */
 	}
-	else {
-		
+	else if (win->ghostwin) {
 		/* this is set to 1 if you don't have startup.blend open */
 		if (G.save_over && G.main->name[0]) {
-			char str[sizeof(G.main->name) + 12];
-			BLI_snprintf(str, sizeof(str), "Blender%s [%s]", wm->file_saved ? "" : "*", G.main->name);
+			char str[sizeof(G.main->name) + 24];
+			BLI_snprintf(str, sizeof(str), "Blender%s [%s%s]", wm->file_saved ? "" : "*", G.main->name,
+			             G.main->recovered ? " (Recovered)" : "");
 			GHOST_SetTitle(win->ghostwin, str);
 		}
 		else
@@ -320,60 +336,92 @@ void wm_window_title(wmWindowManager *wm, wmWindow *win)
 		 * in case of OS application terminate request (e.g. OS Shortcut Alt+F4, Cmd+Q, (...), or session end) */
 		GHOST_SetWindowModifiedState(win->ghostwin, (GHOST_TUns8) !wm->file_saved);
 		
-#if defined(__APPLE__) && !defined(GHOST_COCOA)
-		if (wm->file_saved)
-			GHOST_SetWindowState(win->ghostwin, GHOST_kWindowStateUnModified);
-		else
-			GHOST_SetWindowState(win->ghostwin, GHOST_kWindowStateModified);
-#endif
+	}
+}
+
+float wm_window_pixelsize(wmWindow *win)
+{
+	float pixelsize = GHOST_GetNativePixelSize(win->ghostwin);
+	
+	switch (U.virtual_pixel) {
+		default:
+		case VIRTUAL_PIXEL_NATIVE:
+			return pixelsize;
+		case VIRTUAL_PIXEL_DOUBLE:
+			return 2.0f * pixelsize;
 	}
 }
 
 /* belongs to below */
-static void wm_window_add_ghostwindow(const char *title, wmWindow *win)
+static void wm_window_add_ghostwindow(wmWindowManager *wm, const char *title, wmWindow *win)
 {
 	GHOST_WindowHandle ghostwin;
+	static int multisamples = -1;
 	int scr_w, scr_h, posy;
 	
+	/* force setting multisamples only once, it requires restart - and you cannot 
+	 * mix it, either all windows have it, or none (tested in OSX opengl) */
+	if (multisamples == -1)
+		multisamples = U.ogl_multisamples;
+	
 	wm_get_screensize(&scr_w, &scr_h);
 	posy = (scr_h - win->posy - win->sizey);
 	
-	/* Disable AA for now, as GL_SELECT (used for border, lasso, ... select)
-	 * doesn't work well when AA is initialized, even if not used. */
 	ghostwin = GHOST_CreateWindow(g_system, title,
 	                              win->posx, posy, win->sizex, win->sizey,
 	                              (GHOST_TWindowState)win->windowstate,
 	                              GHOST_kDrawingContextTypeOpenGL,
 	                              0 /* no stereo */,
-	                              0 /* no AA */);
+	                              multisamples /* AA */);
 	
 	if (ghostwin) {
-		/* needed so we can detect the graphics card below */
-		GPU_extensions_init();
+		GHOST_RectangleHandle bounds;
 		
-		/* set the state*/
-		GHOST_SetWindowState(ghostwin, (GHOST_TWindowState)win->windowstate);
+		/* the new window has already been made drawable upon creation */
+		wm->windrawable = win;
 
+		/* needed so we can detect the graphics card below */
+		GPU_init();
+		
 		win->ghostwin = ghostwin;
 		GHOST_SetWindowUserData(ghostwin, win); /* pointer back */
 		
 		if (win->eventstate == NULL)
 			win->eventstate = MEM_callocN(sizeof(wmEvent), "window event state");
 		
+#ifdef __APPLE__
+		/* set the state here, else OSX would not recognize changed screen resolution */
+		/* we agreed to not set any fullscreen or iconized state on startup */
+		GHOST_SetWindowState(ghostwin, GHOST_kWindowStateNormal);
+#endif
+		/* store actual window size in blender window */
+		bounds = GHOST_GetClientBounds(win->ghostwin);
+		win->sizex = GHOST_GetWidthRectangle(bounds);
+		win->sizey = GHOST_GetHeightRectangle(bounds);
+		GHOST_DisposeRectangle(bounds);
+		
+#ifndef __APPLE__
+		/* set the state here, so minimized state comes up correct on windows */
+		GHOST_SetWindowState(ghostwin, (GHOST_TWindowState)win->windowstate);
+#endif
 		/* until screens get drawn, make it nice gray */
 		glClearColor(0.55, 0.55, 0.55, 0.0);
 		/* Crash on OSS ATI: bugs.launchpad.net/ubuntu/+source/mesa/+bug/656100 */
 		if (!GPU_type_matches(GPU_DEVICE_ATI, GPU_OS_UNIX, GPU_DRIVER_OPENSOURCE)) {
 			glClear(GL_COLOR_BUFFER_BIT);
 		}
-
+		
+		/* displays with larger native pixels, like Macbook. Used to scale dpi with */
+		/* needed here, because it's used before it reads userdef */
+		U.pixelsize = wm_window_pixelsize(win);
+		BKE_userdef_state();
+		
 		wm_window_swap_buffers(win);
 		
 		//GHOST_SetWindowState(ghostwin, GHOST_kWindowStateModified);
 		
 		/* standard state vars for window */
 		glEnable(GL_SCISSOR_TEST);
-		
 		GPU_state_init();
 	}
 }
@@ -393,17 +441,21 @@ void wm_window_add_ghostwindows(wmWindowManager *wm)
 	if (wm_init_state.size_x == 0) {
 		wm_get_screensize(&wm_init_state.size_x, &wm_init_state.size_y);
 		
-#if defined(__APPLE__) && !defined(GHOST_COCOA)
-		/* Cocoa provides functions to get correct max window size */
-		{
-			extern void wm_set_apple_prefsize(int, int);    /* wm_apple.c */
-			
-			wm_set_apple_prefsize(wm_init_state.size_x, wm_init_state.size_y);
-		}
-#else
-		wm_init_state.start_x = 0;
-		wm_init_state.start_y = 0;
-		
+	/* note!, this isnt quite correct, active screen maybe offset 1000s if PX,
+	 * we'd need a wm_get_screensize like function that gives offset,
+	 * in practice the window manager will likely move to the correct monitor */
+	wm_init_state.start_x = 0;
+	wm_init_state.start_y = 0;
+
+#ifdef WITH_X11 /* X11 */
+		/* X11, start maximized but use default sane size */
+		wm_init_state.size_x = min_ii(wm_init_state.size_x, WM_WIN_INIT_SIZE_X);
+		wm_init_state.size_y = min_ii(wm_init_state.size_y, WM_WIN_INIT_SIZE_Y);
+		/* pad */
+		wm_init_state.start_x = WM_WIN_INIT_PAD;
+		wm_init_state.start_y = WM_WIN_INIT_PAD;
+		wm_init_state.size_x -= WM_WIN_INIT_PAD * 2;
+		wm_init_state.size_y -= WM_WIN_INIT_PAD * 2;
 #endif
 	}
 	
@@ -414,6 +466,8 @@ void wm_window_add_ghostwindows(wmWindowManager *wm)
 				win->posy = wm_init_state.start_y;
 				win->sizex = wm_init_state.size_x;
 				win->sizey = wm_init_state.size_y;
+
+				win->windowstate = GHOST_kWindowStateNormal;
 				wm_init_state.override_flag &= ~WIN_OVERRIDE_GEOM;
 			}
 
@@ -422,7 +476,7 @@ void wm_window_add_ghostwindows(wmWindowManager *wm)
 				wm_init_state.override_flag &= ~WIN_OVERRIDE_WINSTATE;
 			}
 
-			wm_window_add_ghostwindow("Blender", win);
+			wm_window_add_ghostwindow(wm, "Blender", win);
 		}
 		/* happens after fileread */
 		if (win->eventstate == NULL)
@@ -450,7 +504,7 @@ void wm_window_add_ghostwindows(wmWindowManager *wm)
 /* new window, no screen yet, but we open ghostwindow for it */
 /* also gets the window level handlers */
 /* area-rip calls this */
-wmWindow *WM_window_open(bContext *C, rcti *rect)
+wmWindow *WM_window_open(bContext *C, const rcti *rect)
 {
 	wmWindow *win = wm_window_new(C);
 	
@@ -459,7 +513,7 @@ wmWindow *WM_window_open(bContext *C, rcti *rect)
 	win->sizex = BLI_rcti_size_x(rect);
 	win->sizey = BLI_rcti_size_y(rect);
 
-	win->drawmethod = -1;
+	win->drawmethod = U.wmdrawmethod;
 	win->drawdata = NULL;
 	
 	WM_check(C);
@@ -475,6 +529,7 @@ void WM_window_open_temp(bContext *C, rcti *position, int type)
 {
 	wmWindow *win;
 	ScrArea *sa;
+	Scene *scene = CTX_data_scene(C);
 	
 	/* changes rect to fit within desktop */
 	wm_window_check_position(position);
@@ -500,9 +555,16 @@ void WM_window_open_temp(bContext *C, rcti *position, int type)
 		wm_window_raise(win);
 	}
 	
-	/* add new screen? */
-	if (win->screen == NULL)
-		win->screen = ED_screen_add(win, CTX_data_scene(C), "temp");
+	if (win->screen == NULL) {
+		/* add new screen */
+		win->screen = ED_screen_add(win, scene, "temp");
+	}
+	else {
+		/* switch scene for rendering */
+		if (win->screen->scene != scene)
+			ED_screen_set_scene(C, win->screen, scene);
+	}
+
 	win->screen->temp = 1; 
 	
 	/* make window active, and validate/resize */
@@ -521,6 +583,7 @@ void WM_window_open_temp(bContext *C, rcti *position, int type)
 	}
 	
 	ED_screen_set(C, win->screen);
+	ED_screen_refresh(CTX_wm_manager(C), win); /* test scale */
 	
 	if (sa->spacetype == SPACE_IMAGE)
 		GHOST_SetTitle(win->ghostwin, IFACE_("Blender Render"));
@@ -569,6 +632,24 @@ int wm_window_fullscreen_toggle_exec(bContext *C, wmOperator *UNUSED(op))
 
 /* ************ events *************** */
 
+static void wm_convert_cursor_position(wmWindow *win, int *x, int *y)
+{
+	float fac = GHOST_GetNativePixelSize(win->ghostwin);
+	
+	GHOST_ScreenToClient(win->ghostwin, *x, *y, x, y);
+	*x *= fac;
+	
+	*y = (win->sizey - 1) - *y;
+	*y *= fac;
+}
+
+
+void wm_get_cursor_position(wmWindow *win, int *x, int *y)
+{
+	GHOST_GetCursorPosition(g_system, x, y);
+	wm_convert_cursor_position(win, x, y);
+}
+
 typedef enum {
 	SHIFT    = 's',
 	CONTROL  = 'c',
@@ -608,10 +689,8 @@ static int query_qual(modifierKeyType qual)
 	return val;
 }
 
-void wm_window_make_drawable(bContext *C, wmWindow *win) 
+void wm_window_make_drawable(wmWindowManager *wm, wmWindow *win) 
 {
-	wmWindowManager *wm = CTX_wm_manager(C);
-
 	if (win != wm->windrawable && win->ghostwin) {
 //		win->lmbut = 0;	/* keeps hanging when mousepressed while other window opened */
 		
@@ -620,10 +699,15 @@ void wm_window_make_drawable(bContext *C, wmWindow *win)
 			printf("%s: set drawable %d\n", __func__, win->winid);
 		}
 		GHOST_ActivateWindowDrawingContext(win->ghostwin);
+		
+		/* this can change per window */
+		U.pixelsize = wm_window_pixelsize(win);
+		BKE_userdef_state();
 	}
 }
 
 /* called by ghost, here we handle events for windows themselves or send to event system */
+/* mouse coordinate converversion happens here */
 static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr C_void_ptr)
 {
 	bContext *C = C_void_ptr;
@@ -639,6 +723,11 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr C_void_ptr
 		GHOST_TEventDataPtr data = GHOST_GetEventData(evt);
 		wmWindow *win;
 		
+		/* Ghost now can call this function for life resizes, but it should return if WM didn't initialize yet.
+		 * Can happen on file read (especially full size window)  */
+		if ((wm->initialized & WM_INIT_WINDOW) == 0) {
+			return 1;
+		}
 		if (!ghostwin) {
 			/* XXX - should be checked, why are we getting an event here, and */
 			/* what is it? */
@@ -648,7 +737,7 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr C_void_ptr
 		else if (!GHOST_ValidWindow(g_system, ghostwin)) {
 			/* XXX - should be checked, why are we getting an event here, and */
 			/* what is it? */
-			puts("<!> event has invalid window");			
+			puts("<!> event has invalid window");
 			return 1;
 		}
 		else {
@@ -659,50 +748,113 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr C_void_ptr
 			case GHOST_kEventWindowDeactivate:
 				wm_event_add_ghostevent(wm, win, type, time, data);
 				win->active = 0; /* XXX */
+				
+				/* clear modifiers for inactive windows */
+				win->eventstate->alt = 0;
+				win->eventstate->ctrl = 0;
+				win->eventstate->shift = 0;
+				win->eventstate->oskey = 0;
+				win->eventstate->keymodifier = 0;
+
 				break;
 			case GHOST_kEventWindowActivate: 
 			{
 				GHOST_TEventKeyData kdata;
 				wmEvent event;
-				int cx, cy, wx, wy;
-				
+				int wx, wy;
+				const int keymodifier = ((query_qual(SHIFT)     ? KM_SHIFT : 0) |
+				                         (query_qual(CONTROL)   ? KM_CTRL  : 0) |
+				                         (query_qual(ALT)       ? KM_ALT   : 0) |
+				                         (query_qual(OS)        ? KM_OSKEY : 0));
+
+				/* Win23/GHOST modifier bug, see T40317 */
+#ifndef WIN32
+//#  define USE_WIN_ACTIVATE
+#endif
+
 				wm->winactive = win; /* no context change! c->wm->windrawable is drawable, or for area queues */
 				
 				win->active = 1;
 //				window_handle(win, INPUTCHANGE, win->active);
 				
 				/* bad ghost support for modifier keys... so on activate we set the modifiers again */
+
+				/* TODO: This is not correct since a modifier may be held when a window is activated...
+				 * better solve this at ghost level. attempted fix r54450 but it caused bug [#34255]
+				 *
+				 * For now don't send GHOST_kEventKeyDown events, just set the 'eventstate'.
+				 */
 				kdata.ascii = '\0';
 				kdata.utf8_buf[0] = '\0';
-				if (win->eventstate->shift && !query_qual(SHIFT)) {
-					kdata.key = GHOST_kKeyLeftShift;
-					wm_event_add_ghostevent(wm, win, GHOST_kEventKeyUp, time, &kdata);
+
+				if (win->eventstate->shift) {
+					if ((keymodifier & KM_SHIFT) == 0) {
+						kdata.key = GHOST_kKeyLeftShift;
+						wm_event_add_ghostevent(wm, win, GHOST_kEventKeyUp, time, &kdata);
+					}
 				}
-				if (win->eventstate->ctrl && !query_qual(CONTROL)) {
-					kdata.key = GHOST_kKeyLeftControl;
-					wm_event_add_ghostevent(wm, win, GHOST_kEventKeyUp, time, &kdata);
+#ifdef USE_WIN_ACTIVATE
+				else {
+					if (keymodifier & KM_SHIFT) {
+						win->eventstate->shift = KM_MOD_FIRST;
+					}
 				}
-				if (win->eventstate->alt && !query_qual(ALT)) {
-					kdata.key = GHOST_kKeyLeftAlt;
-					wm_event_add_ghostevent(wm, win, GHOST_kEventKeyUp, time, &kdata);
+#endif
+				if (win->eventstate->ctrl) {
+					if ((keymodifier & KM_CTRL) == 0) {
+						kdata.key = GHOST_kKeyLeftControl;
+						wm_event_add_ghostevent(wm, win, GHOST_kEventKeyUp, time, &kdata);
+					}
 				}
-				if (win->eventstate->oskey && !query_qual(OS)) {
-					kdata.key = GHOST_kKeyOS;
-					wm_event_add_ghostevent(wm, win, GHOST_kEventKeyUp, time, &kdata);
+#ifdef USE_WIN_ACTIVATE
+				else {
+					if (keymodifier & KM_CTRL) {
+						win->eventstate->ctrl = KM_MOD_FIRST;
+					}
 				}
+#endif
+				if (win->eventstate->alt) {
+					if ((keymodifier & KM_ALT) == 0) {
+						kdata.key = GHOST_kKeyLeftAlt;
+						wm_event_add_ghostevent(wm, win, GHOST_kEventKeyUp, time, &kdata);
+					}
+				}
+#ifdef USE_WIN_ACTIVATE
+				else {
+					if (keymodifier & KM_ALT) {
+						win->eventstate->alt = KM_MOD_FIRST;
+					}
+				}
+#endif
+				if (win->eventstate->oskey) {
+					if ((keymodifier & KM_OSKEY) == 0) {
+						kdata.key = GHOST_kKeyOS;
+						wm_event_add_ghostevent(wm, win, GHOST_kEventKeyUp, time, &kdata);
+					}
+				}
+#ifdef USE_WIN_ACTIVATE
+				else {
+					if (keymodifier & KM_OSKEY) {
+						win->eventstate->oskey = KM_MOD_FIRST;
+					}
+				}
+#endif
+
+#undef USE_WIN_ACTIVATE
+
+
 				/* keymodifier zero, it hangs on hotkeys that open windows otherwise */
 				win->eventstate->keymodifier = 0;
 				
 				/* entering window, update mouse pos. but no event */
-				GHOST_GetCursorPosition(g_system, &wx, &wy);
-				
-				GHOST_ScreenToClient(win->ghostwin, wx, wy, &cx, &cy);
-				win->eventstate->x = cx;
-				win->eventstate->y = (win->sizey - 1) - cy;
+				wm_get_cursor_position(win,  &wx, &wy);
+
+				win->eventstate->x = wx;
+				win->eventstate->y = wy;
 				
 				win->addmousemove = 1;   /* enables highlighted buttons */
 				
-				wm_window_make_drawable(C, win);
+				wm_window_make_drawable(wm, win);
 
 				/* window might be focused by mouse click in configuration of window manager
 				 * when focus is not following mouse
@@ -713,7 +865,7 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr C_void_ptr
 				 * currently it seems to be common practice to generate new event for, but probably
 				 * we'll need utility function for this? (sergey)
 				 */
-				event = *(win->eventstate);
+				wm_event_init_from_window(win, &event);
 				event.type = MOUSEMOVE;
 				event.prevx = event.x;
 				event.prevy = event.y;
@@ -733,7 +885,7 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr C_void_ptr
 					printf("%s: ghost redraw %d\n", __func__, win->winid);
 				}
 				
-				wm_window_make_drawable(C, win);
+				wm_window_make_drawable(wm, win);
 				WM_event_add_notifier(C, NC_WINDOW, NULL);
 
 				break;
@@ -745,6 +897,11 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr C_void_ptr
 				state = GHOST_GetWindowState(win->ghostwin);
 				win->windowstate = state;
 
+				/* stop screencast if resize */
+				if (type == GHOST_kEventWindowSize) {
+					WM_jobs_stop(wm, win->screen, NULL);
+				}
+				
 				/* win32: gives undefined window size when minimized */
 				if (state != GHOST_kWindowStateMinimized) {
 					GHOST_RectangleHandle client_rect;
@@ -756,7 +913,7 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr C_void_ptr
 					
 					GHOST_DisposeRectangle(client_rect);
 					
-					wm_get_screensize(&scr_w, &scr_h);
+					wm_get_desktopsize(&scr_w, &scr_h);
 					sizex = r - l;
 					sizey = b - t;
 					posx = l;
@@ -809,10 +966,17 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr C_void_ptr
 							}
 						}
 					
-						wm_window_make_drawable(C, win);
+						wm_window_make_drawable(wm, win);
 						wm_draw_window_clear(win);
 						WM_event_add_notifier(C, NC_SCREEN | NA_EDITED, NULL);
 						WM_event_add_notifier(C, NC_WINDOW | NA_EDITED, NULL);
+						
+#if defined(__APPLE__) || defined(WIN32)
+						/* OSX and Win32 don't return to the mainloop while resize */
+						wm_event_do_handlers(C);
+						wm_event_do_notifiers(C);
+						wm_draw_update(C);
+#endif
 					}
 				}
 				break;
@@ -822,17 +986,18 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr C_void_ptr
 			{
 				PointerRNA props_ptr;
 				wmWindow *oldWindow;
-				char *path = GHOST_GetEventData(evt);
+				const char *path = GHOST_GetEventData(evt);
 				
 				if (path) {
+					wmOperatorType *ot = WM_operatortype_find("WM_OT_open_mainfile", false);
 					/* operator needs a valid window in context, ensures
 					 * it is correctly set */
 					oldWindow = CTX_wm_window(C);
 					CTX_wm_window_set(C, win);
 					
-					WM_operator_properties_create(&props_ptr, "WM_OT_open_mainfile");
+					WM_operator_properties_create_ptr(&props_ptr, ot);
 					RNA_string_set(&props_ptr, "filepath", path);
-					WM_operator_name_call(C, "WM_OT_open_mainfile", WM_OP_EXEC_DEFAULT, &props_ptr);
+					WM_operator_name_call_ptr(C, ot, WM_OP_EXEC_DEFAULT, &props_ptr);
 					WM_operator_properties_free(&props_ptr);
 					
 					CTX_wm_window_set(C, oldWindow);
@@ -843,16 +1008,14 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr C_void_ptr
 			{
 				wmEvent event;
 				GHOST_TEventDragnDropData *ddd = GHOST_GetEventData(evt);
-				int cx, cy, wx, wy;
+				int wx, wy;
 				
 				/* entering window, update mouse pos */
-				GHOST_GetCursorPosition(g_system, &wx, &wy);
+				wm_get_cursor_position(win, &wx, &wy);
+				win->eventstate->x = wx;
+				win->eventstate->y = wy;
 				
-				GHOST_ScreenToClient(win->ghostwin, wx, wy, &cx, &cy);
-				win->eventstate->x = cx;
-				win->eventstate->y = (win->sizey - 1) - cy;
-				
-				event = *(win->eventstate);  /* copy last state, like mouse coords */
+				wm_event_init_from_window(win, &event);  /* copy last state, like mouse coords */
 				
 				/* activate region */
 				event.type = MOUSEMOVE;
@@ -868,7 +1031,7 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr C_void_ptr
 				/* make blender drop event with custom data pointing to wm drags */
 				event.type = EVT_DROP;
 				event.val = KM_RELEASE;
-				event.custom = EVT_DATA_LISTBASE;
+				event.custom = EVT_DATA_DRAGDROP;
 				event.customdata = &wm->drags;
 				event.customdatafree = 1;
 				
@@ -887,17 +1050,39 @@ static int ghost_event_proc(GHOST_EventHandle evt, GHOST_TUserDataPtr C_void_ptr
 						/* try to get icon type from extension */
 						icon = ED_file_extension_icon((char *)stra->strings[a]);
 						
-						WM_event_start_drag(C, icon, WM_DRAG_PATH, stra->strings[a], 0.0);
+						WM_event_start_drag(C, icon, WM_DRAG_PATH, stra->strings[a], 0.0, WM_DRAG_NOP);
 						/* void poin should point to string, it makes a copy */
 						break; /* only one drop element supported now */
 					}
 				}
 				
-				
-				
 				break;
 			}
-			
+			case GHOST_kEventNativeResolutionChange:
+				// printf("change, pixel size %f\n", GHOST_GetNativePixelSize(win->ghostwin));
+				
+				U.pixelsize = wm_window_pixelsize(win);
+				BKE_userdef_state();
+				WM_event_add_notifier(C, NC_SCREEN | NA_EDITED, NULL);
+				WM_event_add_notifier(C, NC_WINDOW | NA_EDITED, NULL);
+
+				break;
+			case GHOST_kEventTrackpad:
+			{
+				GHOST_TEventTrackpadData *pd = data;
+				
+				wm_convert_cursor_position(win, &pd->x, &pd->y);
+				wm_event_add_ghostevent(wm, win, type, time, data);
+				break;
+			}
+			case GHOST_kEventCursorMove:
+			{
+				GHOST_TEventCursorData *cd = data;
+				
+				wm_convert_cursor_position(win, &cd->x, &cd->y);
+				wm_event_add_ghostevent(wm, win, type, time, data);
+				break;
+			}
 			default:
 				wm_event_add_ghostevent(wm, win, type, time, data);
 				break;
@@ -938,9 +1123,12 @@ static int wm_window_timer(const bContext *C)
 				else if (wt->event_type == TIMERAUTOSAVE)
 					wm_autosave_timer(C, wm, wt);
 				else if (win) {
-					wmEvent event = *(win->eventstate);
+					wmEvent event;
+					wm_event_init_from_window(win, &event);
 					
 					event.type = wt->event_type;
+					event.val = 0;
+					event.keymodifier = 0;
 					event.custom = EVT_DATA_TIMER;
 					event.customdata = wt;
 					wm_event_add(win, &event);
@@ -955,8 +1143,12 @@ static int wm_window_timer(const bContext *C)
 
 void wm_window_process_events(const bContext *C) 
 {
-	int hasevent = GHOST_ProcessEvents(g_system, 0); /* 0 is no wait */
-	
+	int hasevent;
+
+	BLI_assert(BLI_thread_is_main());
+
+	hasevent = GHOST_ProcessEvents(g_system, 0); /* 0 is no wait */
+
 	if (hasevent)
 		GHOST_DispatchEvents(g_system);
 	
@@ -978,7 +1170,9 @@ void wm_window_testbreak(void)
 {
 	static double ltime = 0;
 	double curtime = PIL_check_seconds_timer();
-	
+
+	BLI_assert(BLI_thread_is_main());
+
 	/* only check for breaks every 50 milliseconds
 	 * if we get called more often.
 	 */
@@ -1001,7 +1195,11 @@ void wm_ghost_init(bContext *C)
 		
 		g_system = GHOST_CreateSystem();
 		GHOST_AddEventConsumer(g_system, consumer);
-	}	
+		
+		if (wm_init_state.native_pixels) {
+			GHOST_UseNativePixels();
+		}
+	}
 }
 
 void wm_ghost_exit(void)
@@ -1015,7 +1213,7 @@ void wm_ghost_exit(void)
 /* **************** timer ********************** */
 
 /* to (de)activate running timers temporary */
-void WM_event_timer_sleep(wmWindowManager *wm, wmWindow *UNUSED(win), wmTimer *timer, int dosleep)
+void WM_event_timer_sleep(wmWindowManager *wm, wmWindow *UNUSED(win), wmTimer *timer, bool do_sleep)
 {
 	wmTimer *wt;
 	
@@ -1024,7 +1222,7 @@ void WM_event_timer_sleep(wmWindowManager *wm, wmWindow *UNUSED(win), wmTimer *t
 			break;
 
 	if (wt)
-		wt->sleep = dosleep;
+		wt->sleep = do_sleep;
 }
 
 wmTimer *WM_event_add_timer(wmWindowManager *wm, wmWindow *win, int event_type, double timestep)
@@ -1052,6 +1250,8 @@ void WM_event_remove_timer(wmWindowManager *wm, wmWindow *UNUSED(win), wmTimer *
 		if (wt == timer)
 			break;
 	if (wt) {
+		wmWindow *win;
+		
 		if (wm->reports.reporttimer == wt)
 			wm->reports.reporttimer = NULL;
 		
@@ -1059,42 +1259,94 @@ void WM_event_remove_timer(wmWindowManager *wm, wmWindow *UNUSED(win), wmTimer *
 		if (wt->customdata)
 			MEM_freeN(wt->customdata);
 		MEM_freeN(wt);
+		
+		/* there might be events in queue with this timer as customdata */
+		for (win = wm->windows.first; win; win = win->next) {
+			wmEvent *event;
+			for (event = win->queue.first; event; event = event->next) {
+				if (event->customdata == wt) {
+					event->customdata = NULL;
+					event->type = EVENT_NONE;	/* timer users customdata, dont want NULL == NULL */
+				}
+			}
+		}
 	}
 }
 
 /* ******************* clipboard **************** */
 
-char *WM_clipboard_text_get(int selection)
+static char *wm_clipboard_text_get_ex(bool selection, int *r_len,
+                                      bool firstline)
 {
 	char *p, *p2, *buf, *newbuf;
 
-	if (G.background)
+	if (G.background) {
+		*r_len = 0;
 		return NULL;
+	}
 
 	buf = (char *)GHOST_getClipboard(selection);
-	if (!buf)
+	if (!buf) {
+		*r_len = 0;
 		return NULL;
+	}
 	
 	/* always convert from \r\n to \n */
-	newbuf = MEM_callocN(strlen(buf) + 1, "WM_clipboard_text_get");
+	p2 = newbuf = MEM_mallocN(strlen(buf) + 1, __func__);
 
-	for (p = buf, p2 = newbuf; *p; p++) {
-		if (*p != '\r')
-			*(p2++) = *p;
+	if (firstline) {
+		/* will return an over-alloc'ed value in the case there are newlines */
+		for (p = buf; *p; p++) {
+			if ((*p != '\n') && (*p != '\r')) {
+				*(p2++) = *p;
+			}
+			else {
+				break;
+			}
+		}
 	}
+	else {
+		for (p = buf; *p; p++) {
+			if (*p != '\r') {
+				*(p2++) = *p;
+			}
+		}
+	}
+
 	*p2 = '\0';
 
 	free(buf); /* ghost uses regular malloc */
 	
+	*r_len = (p2 - newbuf);
+
 	return newbuf;
 }
 
-void WM_clipboard_text_set(char *buf, int selection)
+/**
+ * Return text from the clipboard.
+ *
+ * \note Caller needs to check for valid utf8 if this is a requirement.
+ */
+char *WM_clipboard_text_get(bool selection, int *r_len)
+{
+	return wm_clipboard_text_get_ex(selection, r_len, false);
+}
+
+/**
+ * Convenience function for pasting to areas of Blender which don't support newlines.
+ */
+char *WM_clipboard_text_get_firstline(bool selection, int *r_len)
+{
+	return wm_clipboard_text_get_ex(selection, r_len, true);
+}
+
+void WM_clipboard_text_set(const char *buf, bool selection)
 {
 	if (!G.background) {
 #ifdef _WIN32
 		/* do conversion from \n to \r\n on Windows */
-		char *p, *p2, *newbuf;
+		const char *p;
+		char *p2, *newbuf;
 		int newlen = 0;
 		
 		for (p = buf; *p; p++) {
@@ -1110,7 +1362,9 @@ void WM_clipboard_text_set(char *buf, int selection)
 			if (*p == '\n') {
 				*(p2++) = '\r'; *p2 = '\n';
 			}
-			else *p2 = *p;
+			else {
+				*p2 = *p;
+			}
 		}
 		*p2 = '\0';
 	
@@ -1142,23 +1396,6 @@ void wm_window_get_position(wmWindow *win, int *posx_r, int *posy_r)
 	*posy_r = win->posy;
 }
 
-void wm_window_get_size(wmWindow *win, int *width_r, int *height_r) 
-{
-	*width_r = win->sizex;
-	*height_r = win->sizey;
-}
-
-/* exceptional case: - splash is called before events are processed
- * this means we don't actually know the window size so get this from GHOST */
-void wm_window_get_size_ghost(wmWindow *win, int *width_r, int *height_r)
-{
-	GHOST_RectangleHandle bounds = GHOST_GetClientBounds(win->ghostwin);
-	*width_r = GHOST_GetWidthRectangle(bounds);
-	*height_r = GHOST_GetHeightRectangle(bounds);
-	
-	GHOST_DisposeRectangle(bounds);
-}
-
 void wm_window_set_size(wmWindow *win, int width, int height) 
 {
 	GHOST_SetClientSize(win->ghostwin, width, height);
@@ -1186,13 +1423,17 @@ void wm_window_swap_buffers(wmWindow *win)
 #endif
 }
 
-void wm_get_cursor_position(wmWindow *win, int *x, int *y)
+void wm_window_set_swap_interval (wmWindow *win, int interval)
 {
-	GHOST_GetCursorPosition(g_system, x, y);
-	GHOST_ScreenToClient(win->ghostwin, *x, *y, x, y);
-	*y = (win->sizey - 1) - *y;
+	GHOST_SetSwapInterval(win->ghostwin, interval);
 }
 
+bool wm_window_get_swap_interval(wmWindow *win, int *intervalOut)
+{
+	return GHOST_GetSwapInterval(win->ghostwin, intervalOut);
+}
+
+
 /* ******************* exported api ***************** */
 
 
@@ -1201,8 +1442,8 @@ void WM_init_state_size_set(int stax, int stay, int sizx, int sizy)
 {
 	wm_init_state.start_x = stax; /* left hand pos */
 	wm_init_state.start_y = stay; /* bottom pos */
-	wm_init_state.size_x = sizx;
-	wm_init_state.size_y = sizy;
+	wm_init_state.size_x = sizx < 640 ? 640 : sizx;
+	wm_init_state.size_y = sizy < 480 ? 480 : sizy;
 	wm_init_state.override_flag |= WIN_OVERRIDE_GEOM;
 }
 
@@ -1219,12 +1460,20 @@ void WM_init_state_normal_set(void)
 	wm_init_state.override_flag |= WIN_OVERRIDE_WINSTATE;
 }
 
+void WM_init_native_pixels(bool do_it)
+{
+	wm_init_state.native_pixels = do_it;
+}
+
 /* This function requires access to the GHOST_SystemHandle (g_system) */
 void WM_cursor_warp(wmWindow *win, int x, int y)
 {
 	if (win && win->ghostwin) {
+		float f = GHOST_GetNativePixelSize(win->ghostwin);
 		int oldx = x, oldy = y;
 
+		x = x / f;
+		y = y / f;
 		y = win->sizey - y - 1;
 
 		GHOST_ClientToScreen(win->ghostwin, x, y, &x, &y);
@@ -1232,6 +1481,64 @@ void WM_cursor_warp(wmWindow *win, int x, int y)
 
 		win->eventstate->prevx = oldx;
 		win->eventstate->prevy = oldy;
+
+		win->eventstate->x = oldx;
+		win->eventstate->y = oldy;
 	}
 }
 
+/**
+ * Get the cursor pressure, in most cases you'll want to use wmTabletData from the event
+ */
+float WM_cursor_pressure(const struct wmWindow *win)
+{
+	const GHOST_TabletData *td = GHOST_GetTabletData(win->ghostwin);
+	/* if there's tablet data from an active tablet device then add it */
+	if ((td != NULL) && td->Active != GHOST_kTabletModeNone) {
+		return td->Pressure;
+	}
+	else {
+		return -1.0f;
+	}
+}
+
+/* support for native pixel size */
+/* mac retina opens window in size X, but it has up to 2 x more pixels */
+int WM_window_pixels_x(wmWindow *win)
+{
+	float f = GHOST_GetNativePixelSize(win->ghostwin);
+	
+	return (int)(f * (float)win->sizex);
+}
+
+int WM_window_pixels_y(wmWindow *win)
+{
+	float f = GHOST_GetNativePixelSize(win->ghostwin);
+	
+	return (int)(f * (float)win->sizey);
+	
+}
+
+bool WM_window_is_fullscreen(wmWindow *win)
+{
+	return win->windowstate == GHOST_kWindowStateFullScreen;
+}
+
+
+#ifdef WITH_INPUT_IME
+/* note: keep in mind wm_window_IME_begin is also used to reposition the IME window */
+void wm_window_IME_begin(wmWindow *win, int x, int y, int w, int h, bool complete)
+{
+	BLI_assert(win);
+
+	GHOST_BeginIME(win->ghostwin, x, win->sizey - y, w, h, complete);
+}
+
+void wm_window_IME_end(wmWindow *win)
+{
+	BLI_assert(win && win->ime_data);
+
+	GHOST_EndIME(win->ghostwin);
+	win->ime_data = NULL;
+}
+#endif  /* WITH_INPUT_IME */
