@@ -29,14 +29,36 @@
  *  \ingroup blroutines
  */
 
+#include "glew-mx.h"
+
+#include "MEM_guardedalloc.h"
 
 #include "KX_BlenderCanvas.h"
+
+#include "DNA_image_types.h"
+#include "DNA_scene_types.h"
 #include "DNA_screen_types.h"
-#include <stdio.h>
+#include "DNA_windowmanager_types.h"
+
+#include "BKE_image.h"
+#include "BKE_global.h"
+#include "BKE_main.h"
+
+#include "BLI_path_util.h"
+#include "BLI_string.h"
+
 #include <assert.h>
 
+extern "C" {
+#include "IMB_imbuf.h"
+#include "IMB_imbuf_types.h"
+#include "WM_api.h"
+#include "wm_cursors.h"
+#include "wm_window.h"
+}
 
-KX_BlenderCanvas::KX_BlenderCanvas(struct wmWindow *win, RAS_Rect &rect, struct ARegion *ar) :
+KX_BlenderCanvas::KX_BlenderCanvas(wmWindowManager *wm, wmWindow *win, RAS_Rect &rect, struct ARegion *ar) :
+m_wm(wm),
 m_win(win),
 m_frame_rect(rect)
 {
@@ -46,7 +68,7 @@ m_frame_rect(rect)
 	m_area_left = ar->winrct.xmin;
 	m_area_top = ar->winrct.ymax;
 
-	glGetIntegerv(GL_VIEWPORT, (GLint*)m_viewport);
+	glGetIntegerv(GL_VIEWPORT, (GLint *)m_viewport);
 }
 
 KX_BlenderCanvas::~KX_BlenderCanvas()
@@ -61,7 +83,17 @@ void KX_BlenderCanvas::Init()
 
 void KX_BlenderCanvas::SwapBuffers()
 {
-	BL_SwapBuffers(m_win);
+	wm_window_swap_buffers(m_win);
+}
+
+void KX_BlenderCanvas::SetSwapInterval(int interval)
+{
+	wm_window_set_swap_interval(m_win, interval);
+}
+
+bool KX_BlenderCanvas::GetSwapInterval(int &intervalOut)
+{
+	return wm_window_get_swap_interval(m_win, &intervalOut);
 }
 
 void KX_BlenderCanvas::ResizeWindow(int width, int height)
@@ -69,20 +101,40 @@ void KX_BlenderCanvas::ResizeWindow(int width, int height)
 	// Not implemented for the embedded player
 }
 
+void KX_BlenderCanvas::SetFullScreen(bool enable)
+{
+	// Not implemented for the embedded player
+}
+
+bool KX_BlenderCanvas::GetFullScreen()
+{
+	// Not implemented for the embedded player
+	return false;
+}
+
+bool KX_BlenderCanvas::BeginDraw()
+{
+	// in case of multi-window we need to ensure we are drawing to the correct
+	// window always, because it may change in window event handling
+	wm_window_make_drawable(m_wm, m_win);
+	return true;
+}
+
+
+void KX_BlenderCanvas::EndDraw()
+{
+	// nothing needs to be done here
+}
+
 void KX_BlenderCanvas::BeginFrame()
 {
 	glEnable(GL_DEPTH_TEST);
 	glDepthFunc(GL_LEQUAL);
-
 }
 
 
 void KX_BlenderCanvas::EndFrame()
 {
-		// this is needed, else blender distorts a lot
-	glPopAttrib();
-	glPushAttrib(GL_ALL_ATTRIB_BITS);
-		
 	glDisable(GL_FOG);
 }
 
@@ -119,14 +171,14 @@ int KX_BlenderCanvas::GetHeight(
 
 int KX_BlenderCanvas::GetMouseX(int x)
 {
-	float left = GetWindowArea().GetLeft();
-	return float(x - (left - m_area_left));
+	int left = GetWindowArea().GetLeft();
+	return x - (left - m_area_left);
 }
 
 int KX_BlenderCanvas::GetMouseY(int y)
 {
-	float top = GetWindowArea().GetTop();
-	return float(y - (m_area_top - top));
+	int top = GetWindowArea().GetTop();
+	return y - (m_area_top - top);
 }
 
 float KX_BlenderCanvas::GetMouseNormalizedX(int x)
@@ -154,11 +206,11 @@ SetViewPort(
 	int x1, int y1,
 	int x2, int y2
 ) {
-	/*	x1 and y1 are the min pixel coordinate (e.g. 0)
-		x2 and y2 are the max pixel coordinate
-		the width,height is calculated including both pixels
-		therefore: max - min + 1
-	*/
+	/* x1 and y1 are the min pixel coordinate (e.g. 0)
+	 * x2 and y2 are the max pixel coordinate
+	 * the width,height is calculated including both pixels
+	 * therefore: max - min + 1
+	 */
 	int vp_width = (x2 - x1) + 1;
 	int vp_height = (y2 - y1) + 1;
 	int minx = m_frame_rect.GetLeft();
@@ -178,6 +230,18 @@ SetViewPort(
 	glScissor(minx + x1, miny + y1, vp_width, vp_height);
 }
 
+	void
+KX_BlenderCanvas::
+UpdateViewPort(
+	int x1, int y1,
+	int x2, int y2
+) {
+	m_viewport[0] = x1;
+	m_viewport[1] = y1;
+	m_viewport[2] = x2;
+	m_viewport[3] = y2;
+}
+
 	const int*
 KX_BlenderCanvas::
 GetViewPort() {
@@ -203,17 +267,17 @@ void KX_BlenderCanvas::SetMouseState(RAS_MouseState mousestate)
 	{
 	case MOUSE_INVISIBLE:
 		{
-			BL_HideMouse(m_win);
+			WM_cursor_set(m_win, CURSOR_NONE);
 			break;
 		}
 	case MOUSE_WAIT:
 		{
-			BL_WaitMouse(m_win);
+			WM_cursor_set(m_win, CURSOR_WAIT);
 			break;
 		}
 	case MOUSE_NORMAL:
 		{
-			BL_NormalMouse(m_win);
+			WM_cursor_set(m_win, CURSOR_STD);
 			break;
 		}
 	default:
@@ -231,18 +295,71 @@ void KX_BlenderCanvas::SetMousePosition(int x,int y)
 	int winY = m_frame_rect.GetBottom();
 	int winH = m_frame_rect.GetHeight();
 	
-	BL_warp_pointer(m_win, winX + x, winY + (winH-y));
+	WM_cursor_warp(m_win, winX + x, winY + (winH-y));
 }
 
 
+/* get shot from frontbuffer sort of a copy from screendump.c */
+static unsigned int *screenshot(ScrArea *curarea, int *dumpsx, int *dumpsy)
+{
+	int x=0, y=0;
+	unsigned int *dumprect= NULL;
 
-void KX_BlenderCanvas::MakeScreenShot(const char* filename)
+	x= curarea->totrct.xmin;
+	y= curarea->totrct.ymin;
+	*dumpsx= curarea->totrct.xmax-x;
+	*dumpsy= curarea->totrct.ymax-y;
+
+	if (*dumpsx && *dumpsy) {
+
+		dumprect= (unsigned int *)MEM_mallocN(sizeof(int) * (*dumpsx) * (*dumpsy), "dumprect");
+		glReadBuffer(GL_FRONT);
+		glReadPixels(x, y, *dumpsx, *dumpsy, GL_RGBA, GL_UNSIGNED_BYTE, dumprect);
+		glFinish();
+		glReadBuffer(GL_BACK);
+	}
+
+	return dumprect;
+}
+
+void KX_BlenderCanvas::MakeScreenShot(const char *filename)
 {
 	ScrArea area_dummy= {0};
+	bScreen *screen = m_win->screen;
+	unsigned int *dumprect;
+	int dumpsx, dumpsy;
+
 	area_dummy.totrct.xmin = m_frame_rect.GetLeft();
 	area_dummy.totrct.xmax = m_frame_rect.GetRight();
 	area_dummy.totrct.ymin = m_frame_rect.GetBottom();
 	area_dummy.totrct.ymax = m_frame_rect.GetTop();
 
-	BL_MakeScreenShot(&area_dummy, filename);
+	dumprect = screenshot(&area_dummy, &dumpsx, &dumpsy);
+
+	if (dumprect) {
+		/* initialize image file format data */
+		Scene *scene = (screen)? screen->scene: NULL;
+		ImageFormatData im_format;
+
+		if (scene)
+			im_format = scene->r.im_format;
+		else
+			BKE_imformat_defaults(&im_format);
+
+		/* create file path */
+		char path[FILE_MAX];
+		BLI_strncpy(path, filename, sizeof(path));
+		BLI_path_abs(path, G.main->name);
+		BKE_add_image_extension_from_type(path, im_format.imtype);
+
+		/* create and save imbuf */
+		ImBuf *ibuf = IMB_allocImBuf(dumpsx, dumpsy, 24, 0);
+		ibuf->rect = dumprect;
+
+		BKE_imbuf_write_as(ibuf, path, &im_format, false);
+
+		ibuf->rect = NULL;
+		IMB_freeImBuf(ibuf);
+		MEM_freeN(dumprect);
+	}
 }
