@@ -34,21 +34,18 @@
 
 #ifdef WIN32
 #include <windows.h>
-#endif 
-
-#include "GL/glew.h"
+#endif
 
 #include "RAS_ICanvas.h"
 #include "RAS_Rect.h"
 
-#include "KX_BlenderGL.h"
-
 #ifdef WITH_CXX_GUARDEDALLOC
 #include "MEM_guardedalloc.h"
 #endif
 
 struct ARegion;
 struct wmWindow;
+struct wmWindowManager;
 
 /**
  * 2D Blender device context abstraction. 
@@ -58,8 +55,9 @@ struct wmWindow;
 class KX_BlenderCanvas : public RAS_ICanvas
 {
 private:
-	/** Rect that defines the area used for rendering,
-	    relative to the context */
+	/**
+	 * Rect that defines the area used for rendering,
+	 * relative to the context */
 	RAS_Rect m_displayarea;
 	int m_viewport[4];
 
@@ -68,7 +66,7 @@ public:
 	 * 
 	 * \param area The Blender ARegion to run the game within.
 	 */
-	KX_BlenderCanvas(struct wmWindow* win, class RAS_Rect &rect, struct ARegion* ar);
+	KX_BlenderCanvas(struct wmWindowManager *wm, struct wmWindow* win, RAS_Rect &rect, struct ARegion* ar);
 	~KX_BlenderCanvas();
 
 		void 
@@ -78,12 +76,31 @@ public:
 		void 
 	SwapBuffers(
 	);
+
+		void
+	SetSwapInterval(
+		int interval
+	);
+
+		bool
+	GetSwapInterval(
+		int &intervalOut
+	);
+
 		void 
 	ResizeWindow(
 		int width,
 		int height
 	);
 
+		void
+	SetFullScreen(
+		bool enable
+	);
+
+		bool
+	GetFullScreen();
+
 		void
 	BeginFrame(
 	);
@@ -151,6 +168,13 @@ public:
 		int x1, int y1,
 		int x2, int y2
 	);
+
+		void
+	UpdateViewPort(
+		int x1, int y1,
+		int x2, int y2
+	);
+
 		const int*
 	GetViewPort();
 
@@ -170,30 +194,22 @@ public:
 		const char* filename
 	);
 	
-	/**
-	 * Nothing needs be done for BlenderCanvas
-	 * Begin/End Draw, as the game engine GL context
-	 * is always current/active.
-	 */
-
 		bool 
 	BeginDraw(
-	) {
-			return true;
-	};
+	);
 
 		void 
 	EndDraw(
-	) {
-	};
+	);
 
 private:
 	/** Blender area the game engine is running within */
+	struct wmWindowManager *m_wm;
 	struct wmWindow* m_win;
 	RAS_Rect	m_frame_rect;
 	RAS_Rect 	m_area_rect;
-	short		m_area_left;
-	short		m_area_top;
+	int			m_area_left;
+	int			m_area_top;
 
 
 #ifdef WITH_CXX_GUARDEDALLOC
