@@ -32,6 +32,7 @@
 
 #include "BL_ArmatureObject.h"
 #include "BL_ActionActuator.h"
+#include "BL_Action.h"
 #include "KX_BlenderSceneConverter.h"
 #include "MEM_guardedalloc.h"
 #include "BLI_blenlib.h"
@@ -41,6 +42,13 @@
 #include "BIK_api.h"
 #include "BKE_action.h"
 #include "BKE_armature.h"
+#include "BKE_object.h"
+#include "BKE_library.h"
+#include "BKE_global.h"
+
+extern "C" {
+#include "BKE_animsys.h"
+}
 
 #include "BKE_constraint.h"
 #include "CTR_Map.h"
@@ -50,8 +58,8 @@
 #include "DNA_armature_types.h"
 #include "DNA_object_types.h"
 #include "DNA_scene_types.h"
-#include "DNA_nla_types.h"
 #include "DNA_constraint_types.h"
+#include "RNA_access.h"
 #include "KX_PythonSeq.h"
 #include "KX_PythonInit.h"
 #include "KX_KetsjiEngine.h"
@@ -69,7 +77,7 @@
  * When it is about to evaluate the pose, set the KX object position in the obmat of the corresponding
  * Blender objects and restore after the evaluation.
  */
-void game_copy_pose(bPose **dst, bPose *src, int copy_constraint)
+static void game_copy_pose(bPose **dst, bPose *src, int copy_constraint)
 {
 	bPose *out;
 	bPoseChannel *pchan, *outpchan;
@@ -84,7 +92,7 @@ void game_copy_pose(bPose **dst, bPose *src, int copy_constraint)
 		return;
 	}
 	else if (*dst==src) {
-		printf("BKE_pose_copy_data source and target are the same\n");
+		printf("game_copy_pose source and target are the same\n");
 		*dst=NULL;
 		return;
 	}
@@ -93,30 +101,35 @@ void game_copy_pose(bPose **dst, bPose *src, int copy_constraint)
 	out->chanhash = NULL;
 	out->agroups.first= out->agroups.last= NULL;
 	out->ikdata = NULL;
-	out->ikparam = MEM_dupallocN(out->ikparam);
+	out->ikparam = MEM_dupallocN(src->ikparam);
 	out->flag |= POSE_GAME_ENGINE;
 	BLI_duplicatelist(&out->chanbase, &src->chanbase);
 
 	/* remap pointers */
 	ghash= BLI_ghash_new(BLI_ghashutil_ptrhash, BLI_ghashutil_ptrcmp, "game_copy_pose gh");
 
-	pchan= (bPoseChannel*)src->chanbase.first;
-	outpchan= (bPoseChannel*)out->chanbase.first;
+	pchan= (bPoseChannel *)src->chanbase.first;
+	outpchan= (bPoseChannel *)out->chanbase.first;
 	for (; pchan; pchan=pchan->next, outpchan=outpchan->next)
 		BLI_ghash_insert(ghash, pchan, outpchan);
 
-	for (pchan=(bPoseChannel*)out->chanbase.first; pchan; pchan=(bPoseChannel*)pchan->next) {
-		pchan->parent= (bPoseChannel*)BLI_ghash_lookup(ghash, pchan->parent);
-		pchan->child= (bPoseChannel*)BLI_ghash_lookup(ghash, pchan->child);
+	for (pchan = (bPoseChannel *)out->chanbase.first; pchan; pchan = pchan->next) {
+		pchan->parent= (bPoseChannel *)BLI_ghash_lookup(ghash, pchan->parent);
+		pchan->child= (bPoseChannel *)BLI_ghash_lookup(ghash, pchan->child);
 
 		if (copy_constraint) {
 			ListBase listb;
 			// copy all constraint for backward compatibility
-			copy_constraints(&listb, &pchan->constraints, FALSE);  // copy_constraints NULLs listb, no need to make extern for this operation.
+			// BKE_constraints_copy NULLs listb, no need to make extern for this operation.
+			BKE_constraints_copy(&listb, &pchan->constraints, false);
 			pchan->constraints= listb;
-		} else {
-			pchan->constraints.first = NULL;
-			pchan->constraints.last = NULL;
+		}
+		else {
+			BLI_listbase_clear(&pchan->constraints);
+		}
+
+		if (pchan->custom) {
+			id_us_plus(&pchan->custom->id);
 		}
 
 		// fails to link, props are not used in the BGE yet.
@@ -136,29 +149,26 @@ void game_copy_pose(bPose **dst, bPose *src, int copy_constraint)
 
 
 /* Only allowed for Poses with identical channels */
-void game_blend_poses(bPose *dst, bPose *src, float srcweight/*, short mode*/)
+static void game_blend_poses(bPose *dst, bPose *src, float srcweight, short mode)
 {
-	short mode= ACTSTRIPMODE_BLEND;
-	
 	bPoseChannel *dchan;
 	const bPoseChannel *schan;
 	bConstraint *dcon, *scon;
 	float dstweight;
 	int i;
 
-	switch (mode) {
-	case ACTSTRIPMODE_BLEND:
-		dstweight = 1.0F - srcweight;
-		break;
-	case ACTSTRIPMODE_ADD:
-		dstweight = 1.0F;
-		break;
-	default :
-		dstweight = 1.0F;
+	if (mode == BL_Action::ACT_BLEND_BLEND)
+	{
+		dstweight = 1.0f - srcweight;
+	} else if (mode == BL_Action::ACT_BLEND_ADD)
+	{
+		dstweight = 1.0f;
+	} else {
+		dstweight = 1.0f;
 	}
 	
-	schan= (bPoseChannel*)src->chanbase.first;
-	for (dchan = (bPoseChannel*)dst->chanbase.first; dchan; dchan=(bPoseChannel*)dchan->next, schan= (bPoseChannel*)schan->next) {
+	schan= (bPoseChannel *)src->chanbase.first;
+	for (dchan = (bPoseChannel *)dst->chanbase.first; dchan; dchan=(bPoseChannel *)dchan->next, schan= (bPoseChannel *)schan->next) {
 		// always blend on all channels since we don't know which one has been set
 		/* quat interpolation done separate */
 		if (schan->rotmode == ROT_MODE_QUAT) {
@@ -166,7 +176,7 @@ void game_blend_poses(bPose *dst, bPose *src, float srcweight/*, short mode*/)
 			
 			copy_qt_qt(dquat, dchan->quat);
 			copy_qt_qt(squat, schan->quat);
-			if (mode==ACTSTRIPMODE_BLEND)
+			if (mode==BL_Action::ACT_BLEND_BLEND)
 				interp_qt_qtqt(dchan->quat, dquat, squat, srcweight);
 			else {
 				mul_fac_qt_fl(squat, srcweight);
@@ -186,7 +196,10 @@ void game_blend_poses(bPose *dst, bPose *src, float srcweight/*, short mode*/)
 			if (schan->rotmode)
 				dchan->eul[i] = (dchan->eul[i]*dstweight) + (schan->eul[i]*srcweight);
 		}
-		for (dcon= (bConstraint*)dchan->constraints.first, scon= (bConstraint*)schan->constraints.first; dcon && scon; dcon= (bConstraint*)dcon->next, scon= (bConstraint*)scon->next) {
+		for (dcon= (bConstraint *)dchan->constraints.first, scon= (bConstraint *)schan->constraints.first;
+		     dcon && scon;
+		     dcon = dcon->next, scon = scon->next)
+		{
 			/* no 'add' option for constraint blending */
 			dcon->enforce= dcon->enforce*(1.0f-srcweight) + scon->enforce*srcweight;
 		}
@@ -196,23 +209,6 @@ void game_blend_poses(bPose *dst, bPose *src, float srcweight/*, short mode*/)
 	dst->ctime= src->ctime;
 }
 
-void game_free_pose(bPose *pose)
-{
-	if (pose) {
-		/* free pose-channels and constraints */
-		BKE_pose_channels_free(pose);
-		
-		/* free IK solver state */
-		BIK_clear_data(pose);
-
-		/* free IK solver param */
-		if (pose->ikparam)
-			MEM_freeN(pose->ikparam);
-
-		MEM_freeN(pose);
-	}
-}
-
 BL_ArmatureObject::BL_ArmatureObject(
 				void* sgReplicationInfo, 
 				SG_Callbacks callbacks, 
@@ -223,26 +219,20 @@ BL_ArmatureObject::BL_ArmatureObject(
 :	KX_GameObject(sgReplicationInfo,callbacks),
 	m_controlledConstraints(),
 	m_poseChannels(),
-	m_objArma(armature),
-	m_framePose(NULL),
 	m_scene(scene), // maybe remove later. needed for BKE_pose_where_is
 	m_lastframe(0.0),
 	m_timestep(0.040),
-	m_activeAct(NULL),
-	m_activePriority(999),
 	m_vert_deform_type(vert_deform_type),
 	m_constraintNumber(0),
 	m_channelNumber(0),
 	m_lastapplyframe(0.0)
 {
-	m_armature = (bArmature *)armature->data;
-
-	/* we make a copy of blender object's pose, and then always swap it with
-	 * the original pose before calling into blender functions, to deal with
-	 * replica's or other objects using the same blender object */
-	m_pose = NULL;
-	game_copy_pose(&m_pose, m_objArma->pose, 1);
-	// store the original armature object matrix
+	m_origObjArma = armature; // Keep a copy of the original armature so we can fix drivers later
+	m_objArma = BKE_object_copy(armature);
+	m_objArma->data = BKE_armature_copy((bArmature *)armature->data);
+	m_pose = m_objArma->pose;
+    // need this to get iTaSC working ok in the BGE
+    m_pose->flag |= POSE_GAME_ENGINE;
 	memcpy(m_obmat, m_objArma->obmat, sizeof(m_obmat));
 }
 
@@ -256,10 +246,9 @@ BL_ArmatureObject::~BL_ArmatureObject()
 	while ((channel = static_cast<BL_ArmatureChannel*>(m_poseChannels.Remove())) != NULL) {
 		delete channel;
 	}
-	if (m_pose)
-		game_free_pose(m_pose);
-	if (m_framePose)
-		game_free_pose(m_framePose);
+
+	if (m_objArma)
+		BKE_libblock_free(G.main, m_objArma);
 }
 
 
@@ -282,8 +271,8 @@ void BL_ArmatureObject::LoadConstraints(KX_BlenderSceneConverter* converter)
 	KX_GameObject* gamesubtarget;
 
 	// and locate the constraint
-	for (pchan = (bPoseChannel*)m_pose->chanbase.first; pchan; pchan=(bPoseChannel*)pchan->next) {
-		for (pcon = (bConstraint*)pchan->constraints.first; pcon; pcon=(bConstraint*)pcon->next) {
+	for (pchan = (bPoseChannel *)m_pose->chanbase.first; pchan; pchan = pchan->next) {
+		for (pcon = (bConstraint *)pchan->constraints.first; pcon; pcon = pcon->next) {
 			if (pcon->flag & CONSTRAINT_DISABLE)
 				continue;
 			// which constraint should we support?
@@ -301,7 +290,7 @@ void BL_ArmatureObject::LoadConstraints(KX_BlenderSceneConverter* converter)
 			case CONSTRAINT_TYPE_TRANSFORM:
 			case CONSTRAINT_TYPE_DISTLIMIT:
 			case CONSTRAINT_TYPE_TRANSLIKE:
-				cti = constraint_get_typeinfo(pcon);
+				cti = BKE_constraint_typeinfo_get(pcon);
 				gametarget = gamesubtarget = NULL;
 				if (cti && cti->get_constraint_targets) {
 					ListBase listb = { NULL, NULL };
@@ -315,7 +304,7 @@ void BL_ArmatureObject::LoadConstraints(KX_BlenderSceneConverter* converter)
 						}
 						if (target->next != NULL) {
 							// secondary target
-							target = (bConstraintTarget*)target->next;
+							target = target->next;
 							if (target->tar && target->tar != m_objArma) {
 								// only track external object
 								blendtarget = target->tar;
@@ -332,6 +321,10 @@ void BL_ArmatureObject::LoadConstraints(KX_BlenderSceneConverter* converter)
 			}
 		}
 	}
+
+	// If we have constraints, make sure we get treated as an "animated" object
+	if (m_constraintNumber > 0)
+		GetActionManager();
 }
 
 BL_ArmatureConstraint* BL_ArmatureObject::GetConstraint(const char* posechannel, const char* constraintname)
@@ -372,7 +365,7 @@ void BL_ArmatureObject::LoadChannels()
 		BL_ArmatureChannel* proxy;
 	
 		m_channelNumber = 0;
-		for (pchan = (bPoseChannel*)m_pose->chanbase.first; pchan; pchan=(bPoseChannel*)pchan->next) {
+		for (pchan = (bPoseChannel *)m_pose->chanbase.first; pchan; pchan=(bPoseChannel *)pchan->next) {
 			proxy = new BL_ArmatureChannel(this, pchan);
 			m_poseChannels.AddBack(proxy);
 			m_channelNumber++;
@@ -425,12 +418,12 @@ CValue* BL_ArmatureObject::GetReplica()
 
 void BL_ArmatureObject::ProcessReplica()
 {
-	bPose *pose= m_pose;
 	KX_GameObject::ProcessReplica();
 
-	m_pose = NULL;
-	m_framePose = NULL;
-	game_copy_pose(&m_pose, pose, 1);
+	bArmature* tmp = (bArmature*)m_objArma->data;
+	m_objArma = BKE_object_copy(m_objArma);
+	m_objArma->data = BKE_armature_copy(tmp);
+	m_pose = m_objArma->pose;
 }
 
 void BL_ArmatureObject::ReParentLogic()
@@ -500,48 +493,32 @@ void BL_ArmatureObject::SetPose(bPose *pose)
 	m_lastapplyframe = -1.0;
 }
 
-bool BL_ArmatureObject::SetActiveAction(BL_ActionActuator *act, short priority, double curtime)
+void BL_ArmatureObject::SetPoseByAction(bAction *action, float localtime)
+{
+	Object *arm = GetArmatureObject();
+
+	PointerRNA ptrrna;
+	RNA_id_pointer_create(&arm->id, &ptrrna);
+
+	animsys_evaluate_action(&ptrrna, action, NULL, localtime);
+}
+
+void BL_ArmatureObject::BlendInPose(bPose *blend_pose, float weight, short mode)
+{
+	game_blend_poses(m_pose, blend_pose, weight, mode);
+}
+
+bool BL_ArmatureObject::UpdateTimestep(double curtime)
 {
 	if (curtime != m_lastframe) {
-		m_activePriority = 9999;
 		// compute the timestep for the underlying IK algorithm
 		m_timestep = curtime-m_lastframe;
 		m_lastframe= curtime;
-		m_activeAct = NULL;
-		// remember the pose at the start of the frame
-		GetPose(&m_framePose);
 	}
 
-	if (act) 
-	{
-		if (priority<=m_activePriority)
-		{
-			if (priority<m_activePriority) {
-				// this action overwrites the previous ones, start from initial pose to cancel their effects
-				SetPose(m_framePose);
-				if (m_activeAct && (m_activeAct!=act))
-					/* Reset the blend timer since this new action cancels the old one */
-					m_activeAct->SetBlendTime(0.0);
-			}
-			m_activeAct = act;
-			m_activePriority = priority;
-			m_lastframe = curtime;
-		
-			return true;
-		}
-		else {
-			act->SetBlendTime(0.0);
-			return false;
-		}
-	}
 	return false;
 }
 
-BL_ActionActuator * BL_ArmatureObject::GetActiveAction()
-{
-	return m_activeAct;
-}
-
 void BL_ArmatureObject::GetPose(bPose **pose)
 {
 	/* If the caller supplies a null pose, create a new one. */
@@ -564,22 +541,6 @@ void BL_ArmatureObject::GetPose(bPose **pose)
 	}
 }
 
-void BL_ArmatureObject::GetMRDPose(bPose **pose)
-{
-	/* If the caller supplies a null pose, create a new one. */
-	/* Otherwise, copy the armature's pose channels into the caller-supplied pose */
-
-	if (!*pose)
-		game_copy_pose(pose, m_pose, 0);
-	else
-		extract_pose_from_pose(*pose, m_pose);
-}
-
-short BL_ArmatureObject::GetActivePriority()
-{
-	return m_activePriority;
-}
-
 double BL_ArmatureObject::GetLastFrame()
 {
 	return m_lastframe;
@@ -636,7 +597,6 @@ PyTypeObject BL_ArmatureObject::Type = {
 };
 
 PyMethodDef BL_ArmatureObject::Methods[] = {
-
 	KX_PYMETHODTABLE_NOARGS(BL_ArmatureObject, update),
 	{NULL,NULL} //Sentinel
 };
@@ -666,7 +626,7 @@ KX_PYMETHODDEF_DOC_NOARGS(BL_ArmatureObject, update,
 						  "This is automatically done if a KX_ArmatureActuator with mode run is active\n"
 						  "or if an action is playing. This function is useful in other cases.\n")
 {
-	SetActiveAction(NULL, 0, KX_GetActiveEngine()->GetFrameTime());
+	UpdateTimestep(KX_GetActiveEngine()->GetFrameTime());
 	Py_RETURN_NONE;
 }
 
