@@ -39,6 +39,12 @@
 #include "SCA_Joystick.h"
 #include "SCA_JoystickPrivate.h"
 
+#include "BLI_path_util.h"
+
+#ifdef WITH_SDL
+#  define SDL_CHECK(x) ((x) != (void *)0)
+#endif
+
 SCA_Joystick::SCA_Joystick(short int index)
 	:
 	m_joyindex(index),
@@ -51,11 +57,11 @@ SCA_Joystick::SCA_Joystick(short int index)
 	m_istrig_button(0),
 	m_istrig_hat(0)
 {
-	for (int i=0; i<JOYAXIS_MAX; i++)
-		m_axis_array[i]= 0;
+	for (int i=0; i < JOYAXIS_MAX; i++)
+		m_axis_array[i] = 0;
 	
-	for (int i=0; i<JOYHAT_MAX; i++)
-		m_hat_array[i]= 0;
+	for (int i=0; i < JOYHAT_MAX; i++)
+		m_hat_array[i] = 0;
 	
 #ifdef WITH_SDL
 	m_private = new PrivateData();
@@ -80,23 +86,42 @@ SCA_Joystick *SCA_Joystick::GetInstance( short int joyindex )
 #ifndef WITH_SDL
 	return NULL;
 #else  /* WITH_SDL */
+	if (!SDL_CHECK(SDL_InitSubSystem)) {
+		return NULL;
+	}
 	if (joyindex < 0 || joyindex >= JOYINDEX_MAX) {
-		ECHO("Error-invalid joystick index: " << joyindex);
+		JOYSTICK_ECHO("Error-invalid joystick index: " << joyindex);
 		return NULL;
 	}
 
 	if (m_refCount == 0) 
 	{
 		int i;
-		// The video subsystem is required for joystick input to work. However,
-		// when GHOST is running under SDL, video is initialized elsewhere.
-		// Do this once only.
+
+		/* The video subsystem is required for joystick input to work. However,
+		 * when GHOST is running under SDL, video is initialized elsewhere. We
+		 * also need to set the videodriver to dummy, and do it here to avoid
+		 * interfering with addons that may use SDL too.
+		 *
+		 * We also init SDL once only. */
 #  ifdef WITH_GHOST_SDL
-		if (SDL_InitSubSystem(SDL_INIT_JOYSTICK) == -1 ) {
+		int success = (SDL_InitSubSystem(SDL_INIT_JOYSTICK) != -1 );
 #  else
-		if (SDL_InitSubSystem(SDL_INIT_JOYSTICK | SDL_INIT_VIDEO) == -1 ) {
+		/* set and restore environment variable */
+		char *videodriver = getenv("SDL_VIDEODRIVER");
+		BLI_setenv("SDL_VIDEODRIVER", "dummy");
+
+#if SDL_VERSION_ATLEAST(2, 0, 0)
+		int success = (SDL_InitSubSystem(SDL_INIT_JOYSTICK) != -1 );
+#else
+		int success = (SDL_InitSubSystem(SDL_INIT_JOYSTICK | SDL_INIT_VIDEO) != -1 );
+#endif
+
+		BLI_setenv("SDL_VIDEODRIVER", videodriver);
 #  endif
-			ECHO("Error-Initializing-SDL: " << SDL_GetError());
+
+		if (!success) {
+			JOYSTICK_ECHO("Error-Initializing-SDL: " << SDL_GetError());
 			return NULL;
 		}
 		
@@ -121,23 +146,25 @@ void SCA_Joystick::ReleaseInstance()
 	if (--m_refCount == 0)
 	{
 #ifdef WITH_SDL
-		int i;
-		for (i=0; i<JOYINDEX_MAX; i++) {
+		if (!SDL_CHECK(SDL_QuitSubSystem)) {
+			return;
+		}
+		for (int i=0; i<JOYINDEX_MAX; i++) {
 			if (m_instance[i]) {
 				m_instance[i]->DestroyJoystickDevice();
 				delete m_instance[i];
 			}
-			m_instance[i]= NULL;
+			m_instance[i] = NULL;
 		}
 
-		// The video subsystem is required for joystick input to work. However,
-		// when GHOST is running under SDL, video is freed elsewhere.
-		// Do this once only.
-#  ifdef WITH_GHOST_SDL
+		/* The video subsystem is required for joystick input to work. However,
+		 * when GHOST is running under SDL, video is freed elsewhere.
+		 * Do this once only. */
+#if SDL_VERSION_ATLEAST(2, 0, 0)
 		SDL_QuitSubSystem(SDL_INIT_JOYSTICK);
-#  else
+#else
 		SDL_QuitSubSystem(SDL_INIT_JOYSTICK | SDL_INIT_VIDEO);
-#  endif
+#endif
 #endif /* WITH_SDL */
 	}
 }
@@ -177,6 +204,9 @@ bool SCA_Joystick::aAxisIsPositive(int axis_single)
 bool SCA_Joystick::aAnyButtonPressIsPositive(void)
 {
 #ifdef WITH_SDL
+	if (!SDL_CHECK(SDL_JoystickGetButton)) {
+		return false;
+	}
 	/* this is needed for the "all events" option
 	 * so we know if there are no buttons pressed */
 	for (int i=0; i<m_buttonmax; i++)
@@ -192,7 +222,7 @@ bool SCA_Joystick::aButtonPressIsPositive(int button)
 	return false;
 #else
 	bool result;
-	SDL_JoystickGetButton(m_private->m_joystick, button)? result = true:result = false;
+	result = SDL_CHECK(SDL_JoystickGetButton) && SDL_JoystickGetButton(m_private->m_joystick, button);
 	return result;
 #endif
 }
@@ -204,7 +234,7 @@ bool SCA_Joystick::aButtonReleaseIsPositive(int button)
 	return false;
 #else
 	bool result;
-	SDL_JoystickGetButton(m_private->m_joystick, button)? result = false : result = true;
+	result = !(SDL_CHECK(SDL_JoystickGetButton) && SDL_JoystickGetButton(m_private->m_joystick, button));
 	return result;
 #endif
 }
@@ -239,12 +269,12 @@ bool SCA_Joystick::CreateJoystickDevice(void)
 	m_axismax = m_buttonmax = m_hatmax = 0;
 	return false;
 #else /* WITH_SDL */
-	if (m_isinit == false) {
+	if (m_isinit == false && SDL_CHECK(SDL_JoystickOpen)) {
 		if (m_joyindex>=m_joynum) {
-			// don't print a message, because this is done anyway
-			//ECHO("Joystick-Error: " << SDL_NumJoysticks() << " avaiable joystick(s)");
+			/* don't print a message, because this is done anyway */
+			//JOYSTICK_ECHO("Joystick-Error: " << SDL_NumJoysticks() << " avaiable joystick(s)");
 			
-			// Need this so python args can return empty lists
+			/* Need this so python args can return empty lists */
 			m_axismax = m_buttonmax = m_hatmax = 0;
 			return false;
 		}
@@ -253,20 +283,20 @@ bool SCA_Joystick::CreateJoystickDevice(void)
 		SDL_JoystickEventState(SDL_ENABLE);
 		m_isinit = true;
 		
-		ECHO("Joystick " << m_joyindex << " initialized");
+		JOYSTICK_ECHO("Joystick " << m_joyindex << " initialized");
 		
 		/* must run after being initialized */
-		m_axismax =		SDL_JoystickNumAxes(m_private->m_joystick);
-		m_buttonmax =	SDL_JoystickNumButtons(m_private->m_joystick);
-		m_hatmax =		SDL_JoystickNumHats(m_private->m_joystick);
+		m_axismax     = SDL_JoystickNumAxes(m_private->m_joystick);
+		m_buttonmax   = SDL_JoystickNumButtons(m_private->m_joystick);
+		m_hatmax      = SDL_JoystickNumHats(m_private->m_joystick);
+
+		if      (m_axismax > JOYAXIS_MAX) m_axismax = JOYAXIS_MAX;  /* very unlikely */
+		else if (m_axismax < 0)           m_axismax = 0;
 		
-		if (m_axismax > JOYAXIS_MAX) m_axismax= JOYAXIS_MAX;		/* very unlikely */
-		else if (m_axismax < 0) m_axismax = 0;
+		if      (m_hatmax > JOYHAT_MAX) m_hatmax = JOYHAT_MAX;  /* very unlikely */
+		else if (m_hatmax < 0)          m_hatmax = 0;
 		
-		if (m_hatmax > JOYHAT_MAX) m_hatmax= JOYHAT_MAX;			/* very unlikely */
-		else if (m_hatmax<0) m_hatmax= 0;
-		
-		if (m_buttonmax<0) m_buttonmax= 0;
+		if (m_buttonmax < 0) m_buttonmax = 0;
 		
 	}
 	return true;
@@ -278,8 +308,13 @@ void SCA_Joystick::DestroyJoystickDevice(void)
 {
 #ifdef WITH_SDL
 	if (m_isinit) {
-		if (SDL_JoystickOpened(m_joyindex)) {
-			ECHO("Closing-joystick " << m_joyindex);
+#if SDL_VERSION_ATLEAST(2, 0, 0)
+		if (SDL_CHECK(SDL_JoystickGetAttached) && SDL_JoystickGetAttached(m_private->m_joystick))
+#else
+		if (SDL_CHECK(SDL_JoystickOpened) && SDL_JoystickOpened(m_joyindex))
+#endif
+		{
+			JOYSTICK_ECHO("Closing-joystick " << m_joyindex);
 			SDL_JoystickClose(m_private->m_joystick);
 		}
 		m_isinit = false;
@@ -290,8 +325,16 @@ void SCA_Joystick::DestroyJoystickDevice(void)
 int SCA_Joystick::Connected(void)
 {
 #ifdef WITH_SDL
-	if (m_isinit && SDL_JoystickOpened(m_joyindex))
+	if (m_isinit &&
+#if SDL_VERSION_ATLEAST(2, 0, 0)
+	    SDL_CHECK(SDL_JoystickGetAttached) && SDL_JoystickGetAttached(m_private->m_joystick)
+#else
+	    SDL_CHECK(SDL_JoystickOpened) && SDL_JoystickOpened(m_joyindex)
+#endif
+		)
+	{
 		return 1;
+	}
 #endif
 	return 0;
 }
@@ -307,17 +350,33 @@ int SCA_Joystick::pGetAxis(int axisnum, int udlr)
 int SCA_Joystick::pAxisTest(int axisnum)
 {
 #ifdef WITH_SDL
-	short i1= m_axis_array[(axisnum*2)];
-	short i2= m_axis_array[(axisnum*2)+1];
+	/* Use ints instead of shorts here to avoid problems when we get -32768.
+	 * When we take the negative of that later, we should get 32768, which is greater
+	 * than what a short can hold. In other words, abs(MIN_SHORT) > MAX_SHRT. */
+	int i1 = m_axis_array[(axisnum * 2)];
+	int i2 = m_axis_array[(axisnum * 2) + 1];
 	
-	/* long winded way to do
-	 *   return maxf(absf(i1), absf(i2))
-	 * avoid abs from math.h */
+	/* long winded way to do:
+	 * return max_ff(absf(i1), absf(i2))
+	 * ...avoid abs from math.h */
 	if (i1 < 0) i1 = -i1;
 	if (i2 < 0) i2 = -i2;
 	if (i1 <i2) return i2;
-	else		return i1;
+	else        return i1;
 #else /* WITH_SDL */
 	return 0;
 #endif /* WITH_SDL */
 }
+
+const char *SCA_Joystick::GetName()
+{
+#ifdef WITH_SDL
+#if SDL_VERSION_ATLEAST(2, 0, 0)
+	return (SDL_CHECK(SDL_JoystickName)) ? SDL_JoystickName(m_private->m_joystick) : "";
+#else
+	return (SDL_CHECK(SDL_JoystickName)) ? SDL_JoystickName(m_joyindex) : "";
+#endif
+#else /* WITH_SDL */
+	return "";
+#endif /* WITH_SDL */
+}
