@@ -32,12 +32,13 @@
 
 
 #ifdef WIN32
-	#pragma warning (disable:4786) // suppress stl-MSVC debug info warning
-	#include <windows.h>
+#  pragma warning (disable:4786) // suppress stl-MSVC debug info warning
+#  include <windows.h>
 #endif
 
-#include "GL/glew.h"
+#include "glew-mx.h"
 #include "GPU_extensions.h"
+#include "GPU_init_exit.h"
 
 #include "GPG_Application.h"
 #include "BL_BlenderDataConversion.h"
@@ -76,7 +77,6 @@ extern "C"
 #include "SCA_IActuator.h"
 #include "RAS_MeshObject.h"
 #include "RAS_OpenGLRasterizer.h"
-#include "RAS_VAOpenGLRasterizer.h"
 #include "RAS_ListRasterizer.h"
 #include "RAS_GLExtensionManager.h"
 #include "KX_PythonInit.h"
@@ -87,7 +87,6 @@ extern "C"
 #include "NG_LoopBackNetworkDeviceInterface.h"
 
 #include "GPC_MouseDevice.h"
-#include "GPC_RenderTools.h"
 #include "GPG_Canvas.h" 
 #include "GPG_KeyboardDevice.h"
 #include "GPG_System.h"
@@ -100,6 +99,12 @@ extern "C"
 #include "GHOST_IWindow.h"
 #include "GHOST_Rect.h"
 
+#ifdef WITH_AUDASPACE
+#  include "AUD_C-API.h"
+#  include "AUD_I3DDevice.h"
+#  include "AUD_IDevice.h"
+#endif
+
 static void frameTimerProc(GHOST_ITimerTask* task, GHOST_TUns64 time);
 
 static GHOST_ISystem* fSystem = 0;
@@ -121,8 +126,7 @@ GPG_Application::GPG_Application(GHOST_ISystem* system)
 	  m_kxsystem(0), 
 	  m_keyboard(0), 
 	  m_mouse(0), 
-	  m_canvas(0), 
-	  m_rendertools(0), 
+	  m_canvas(0),
 	  m_rasterizer(0), 
 	  m_sceneconverter(0),
 	  m_networkdevice(0),
@@ -184,7 +188,7 @@ static POINT scr_save_mouse_pos;
 
 static LRESULT CALLBACK screenSaverWindowProc(HWND hwnd, UINT uMsg, WPARAM wParam, LPARAM lParam)
 {
-	BOOL close = FALSE;
+	BOOL close = false;
 	switch (uMsg)
 	{
 		case WM_MOUSEMOVE:
@@ -196,7 +200,7 @@ static LRESULT CALLBACK screenSaverWindowProc(HWND hwnd, UINT uMsg, WPARAM wPara
 			if (abs(dx) > SCR_SAVE_MOUSE_MOVE_THRESHOLD
 			        || abs(dy) > SCR_SAVE_MOUSE_MOVE_THRESHOLD)
 			{
-				close = TRUE;
+				close = true;
 			}
 			scr_save_mouse_pos = pt;
 			break;
@@ -205,7 +209,7 @@ static LRESULT CALLBACK screenSaverWindowProc(HWND hwnd, UINT uMsg, WPARAM wPara
 		case WM_MBUTTONDOWN: 
 		case WM_RBUTTONDOWN: 
 		case WM_KEYDOWN:
-			close = TRUE;
+			close = true;
 	}
 	if (close)
 		PostMessage(hwnd,WM_CLOSE,0,0);
@@ -215,11 +219,11 @@ static LRESULT CALLBACK screenSaverWindowProc(HWND hwnd, UINT uMsg, WPARAM wPara
 BOOL CALLBACK findGhostWindowHWNDProc(HWND hwnd, LPARAM lParam)
 {
 	GHOST_IWindow *p = (GHOST_IWindow*) GetWindowLongPtr(hwnd, GWLP_USERDATA);
-	BOOL ret = TRUE;
+	BOOL ret = true;
 	if (p == ghost_window_to_find)
 	{
 		found_ghost_window_hwnd = hwnd;
-		ret = FALSE;
+		ret = false;
 	}
 	return ret;
 }
@@ -261,14 +265,14 @@ bool GPG_Application::startScreenSaverPreview(
 		}
 
 		SetParent(ghost_hwnd, parentWindow);
-		LONG style = GetWindowLong(ghost_hwnd, GWL_STYLE);
-		LONG exstyle = GetWindowLong(ghost_hwnd, GWL_EXSTYLE);
+		LONG_PTR style = GetWindowLongPtr(ghost_hwnd, GWL_STYLE);
+		LONG_PTR exstyle = GetWindowLongPtr(ghost_hwnd, GWL_EXSTYLE);
 
 		RECT adjrc = { 0, 0, windowWidth, windowHeight };
-		AdjustWindowRectEx(&adjrc, style, FALSE, exstyle);
+		AdjustWindowRectEx(&adjrc, style, false, exstyle);
 
 		style = (style & (~(WS_POPUP|WS_OVERLAPPEDWINDOW|WS_OVERLAPPED|WS_CAPTION|WS_SYSMENU|WS_THICKFRAME|WS_MINIMIZEBOX|WS_MAXIMIZEBOX|WS_TILEDWINDOW ))) | WS_CHILD;
-		SetWindowLong(ghost_hwnd, GWL_STYLE, style);
+		SetWindowLongPtr(ghost_hwnd, GWL_STYLE, style);
 		SetWindowPos(ghost_hwnd, NULL, adjrc.left, adjrc.top, 0, 0, SWP_NOZORDER|SWP_NOSIZE|SWP_NOACTIVATE);
 
 		/* Check the size of the client rectangle of the window and resize the window
@@ -309,20 +313,21 @@ bool GPG_Application::startScreenSaverFullScreen(
 
 #endif
 
-bool GPG_Application::startWindow(STR_String& title,
-	int windowLeft,
-	int windowTop,
-	int windowWidth,
-	int windowHeight,
-	const bool stereoVisual,
-	const int stereoMode,
-	const GHOST_TUns16 samples)
+bool GPG_Application::startWindow(
+        STR_String& title,
+        int windowLeft,
+        int windowTop,
+        int windowWidth,
+        int windowHeight,
+        const bool stereoVisual,
+        const int stereoMode,
+        const GHOST_TUns16 samples)
 {
 	bool success;
 	// Create the main window
 	//STR_String title ("Blender Player - GHOST");
 	m_mainWindow = fSystem->createWindow(title, windowLeft, windowTop, windowWidth, windowHeight, GHOST_kWindowStateNormal,
-		GHOST_kDrawingContextTypeOpenGL, stereoVisual, samples);
+	                                     GHOST_kDrawingContextTypeOpenGL, stereoVisual, false, samples);
 	if (!m_mainWindow) {
 		printf("error: could not create main window\n");
 		exit(-1);
@@ -341,16 +346,18 @@ bool GPG_Application::startWindow(STR_String& title,
 	return success;
 }
 
-bool GPG_Application::startEmbeddedWindow(STR_String& title,
-	const GHOST_TEmbedderWindowID parentWindow, 
-	const bool stereoVisual, 
-	const int stereoMode,
-	const GHOST_TUns16 samples) {
+bool GPG_Application::startEmbeddedWindow(
+        STR_String& title,
+        const GHOST_TEmbedderWindowID parentWindow,
+        const bool stereoVisual,
+        const int stereoMode,
+        const GHOST_TUns16 samples)
+{
 	GHOST_TWindowState state = GHOST_kWindowStateNormal;
 	if (parentWindow != 0)
 		state = GHOST_kWindowStateEmbedded;
 	m_mainWindow = fSystem->createWindow(title, 0, 0, 0, 0, state,
-		GHOST_kDrawingContextTypeOpenGL, stereoVisual, samples, parentWindow);
+	                                     GHOST_kDrawingContextTypeOpenGL, stereoVisual, false, samples, parentWindow);
 
 	if (!m_mainWindow) {
 		printf("error: could not create main window\n");
@@ -367,13 +374,13 @@ bool GPG_Application::startEmbeddedWindow(STR_String& title,
 
 
 bool GPG_Application::startFullScreen(
-		int width,
-		int height,
-		int bpp,int frequency,
-		const bool stereoVisual,
-		const int stereoMode,
-		const GHOST_TUns16 samples,
-		bool useDesktop)
+        int width,
+        int height,
+        int bpp,int frequency,
+        const bool stereoVisual,
+        const int stereoMode,
+        const GHOST_TUns16 samples,
+        bool useDesktop)
 {
 	bool success;
 	GHOST_TUns32 sysWidth=0, sysHeight=0;
@@ -387,6 +394,7 @@ bool GPG_Application::startFullScreen(
 
 	fSystem->beginFullScreen(setting, &m_mainWindow, stereoVisual, samples);
 	m_mainWindow->setCursorVisibility(false);
+	/* note that X11 ignores this (it uses a window internally for fullscreen) */
 	m_mainWindow->setState(GHOST_kWindowStateFullScreen);
 
 	success = initEngine(m_mainWindow, stereoMode);
@@ -500,6 +508,7 @@ bool GPG_Application::processEvent(GHOST_IEvent* event)
 				GHOST_Rect bnds;
 				window->getClientBounds(bnds);
 				m_canvas->Resize(bnds.getWidth(), bnds.getHeight());
+				m_ketsjiengine->Resize();
 			}
 			}
 			break;
@@ -537,7 +546,7 @@ bool GPG_Application::initEngine(GHOST_IWindow* window, const int stereoMode)
 {
 	if (!m_engineInitialized)
 	{
-		GPU_extensions_init();
+		GPU_init();
 		bgl::InitExtensions(true);
 
 		// get and set the preferences
@@ -550,14 +559,13 @@ bool GPG_Application::initEngine(GHOST_IWindow* window, const int stereoMode)
 		GameData *gm= &m_startScene->gm;
 		bool properties	= (SYS_GetCommandLineInt(syshandle, "show_properties", 0) != 0);
 		bool profile = (SYS_GetCommandLineInt(syshandle, "show_profile", 0) != 0);
-		bool fixedFr = (gm->flag & GAME_ENABLE_ALL_FRAMES);
 
 		bool showPhysics = (gm->flag & GAME_SHOW_PHYSICS);
 		SYS_WriteCommandLineInt(syshandle, "show_physics", showPhysics);
 
-		bool fixed_framerate= (SYS_GetCommandLineInt(syshandle, "fixed_framerate", fixedFr) != 0);
+		bool fixed_framerate= (SYS_GetCommandLineInt(syshandle, "fixedtime", (gm->flag & GAME_ENABLE_ALL_FRAMES)) != 0);
 		bool frameRate = (SYS_GetCommandLineInt(syshandle, "show_framerate", 0) != 0);
-		bool useLists = (SYS_GetCommandLineInt(syshandle, "displaylists", gm->flag & GAME_DISPLAY_LISTS) != 0);
+		bool useLists = (SYS_GetCommandLineInt(syshandle, "displaylists", gm->flag & GAME_DISPLAY_LISTS) != 0) && GPU_display_list_support();
 		bool nodepwarnings = (SYS_GetCommandLineInt(syshandle, "ignore_deprecation_warnings", 1) != 0);
 		bool restrictAnimFPS = gm->flag & GAME_RESTRICT_ANIM_UPDATES;
 
@@ -573,25 +581,22 @@ bool GPG_Application::initEngine(GHOST_IWindow* window, const int stereoMode)
 		m_canvas = new GPG_Canvas(window);
 		if (!m_canvas)
 			return false;
-				
+
+		if (gm->vsync == VSYNC_ADAPTIVE)
+			m_canvas->SetSwapInterval(-1);
+		else
+			m_canvas->SetSwapInterval((gm->vsync == VSYNC_ON) ? 1 : 0);
+
 		m_canvas->Init();
 		if (gm->flag & GAME_SHOW_MOUSE)
 			m_canvas->SetMouseState(RAS_ICanvas::MOUSE_NORMAL);
-
-		m_rendertools = new GPC_RenderTools();
-		if (!m_rendertools)
-			goto initFailed;
 		
-		if (useLists) {
-			if (GLEW_VERSION_1_1)
-				m_rasterizer = new RAS_ListRasterizer(m_canvas, true);
-			else
-				m_rasterizer = new RAS_ListRasterizer(m_canvas);
-		}
-		else if (GLEW_VERSION_1_1)
-			m_rasterizer = new RAS_VAOpenGLRasterizer(m_canvas);
+		//Don't use displaylists with VBOs
+		//If auto starts using VBOs, make sure to check for that here
+		if (useLists && gm->raster_storage != RAS_STORE_VBO)
+			m_rasterizer = new RAS_ListRasterizer(m_canvas, false, gm->raster_storage);
 		else
-			m_rasterizer = new RAS_OpenGLRasterizer(m_canvas);
+			m_rasterizer = new RAS_OpenGLRasterizer(m_canvas, gm->raster_storage);
 
 		/* Stereo parameters - Eye Separation from the UI - stereomode from the command-line/UI */
 		m_rasterizer->SetStereoMode((RAS_IRasterizer::StereoMode) stereoMode);
@@ -629,11 +634,8 @@ bool GPG_Application::initEngine(GHOST_IWindow* window, const int stereoMode)
 		m_ketsjiengine->SetMouseDevice(m_mouse);
 		m_ketsjiengine->SetNetworkDevice(m_networkdevice);
 		m_ketsjiengine->SetCanvas(m_canvas);
-		m_ketsjiengine->SetRenderTools(m_rendertools);
 		m_ketsjiengine->SetRasterizer(m_rasterizer);
 
-		m_ketsjiengine->SetTimingDisplay(frameRate, false, false);
-
 		KX_KetsjiEngine::SetExitKey(ConvertKeyCode(gm->exitkey));
 #ifdef WITH_PYTHON
 		CValue::SetDeprecationWarnings(nodepwarnings);
@@ -659,10 +661,8 @@ initFailed:
 	delete m_mouse;
 	delete m_keyboard;
 	delete m_rasterizer;
-	delete m_rendertools;
 	delete m_canvas;
 	m_canvas = NULL;
-	m_rendertools = NULL;
 	m_rasterizer = NULL;
 	m_keyboard = NULL;
 	m_mouse = NULL;
@@ -704,10 +704,12 @@ bool GPG_Application::startEngine(void)
 
 		//	if (always_use_expand_framing)
 		//		sceneconverter->SetAlwaysUseExpandFraming(true);
-		if (m_blendermat && (m_globalSettings->matmode != GAME_MAT_TEXFACE))
+		if (m_blendermat)
 			m_sceneconverter->SetMaterials(true);
 		if (m_blenderglslmat && (m_globalSettings->matmode == GAME_MAT_GLSL))
 			m_sceneconverter->SetGLSLMaterials(true);
+		if (m_startScene->gm.flag & GAME_NO_MATERIAL_CACHING)
+			m_sceneconverter->SetCacheMaterials(false);
 
 		KX_Scene* startscene = new KX_Scene(m_keyboard,
 			m_mouse,
@@ -726,6 +728,15 @@ bool GPG_Application::startEngine(void)
 		if (m_startScene->gm.stereoflag == STEREO_DOME)
 			m_ketsjiengine->InitDome(m_startScene->gm.dome.res, m_startScene->gm.dome.mode, m_startScene->gm.dome.angle, m_startScene->gm.dome.resbuf, m_startScene->gm.dome.tilt, m_startScene->gm.dome.warptext);
 
+		// initialize 3D Audio Settings
+		AUD_I3DDevice* dev = AUD_get3DDevice();
+		if (dev)
+		{
+			dev->setSpeedOfSound(m_startScene->audio.speed_of_sound);
+			dev->setDopplerFactor(m_startScene->audio.doppler_factor);
+			dev->setDistanceModel(AUD_DistanceModel(m_startScene->audio.distance_model));
+		}
+
 #ifdef WITH_PYTHON
 		// Set the GameLogic.globalDict from marshal'd data, so we can
 		// load new blend files and keep data in GameLogic.globalDict
@@ -733,7 +744,7 @@ bool GPG_Application::startEngine(void)
 #endif
 		m_sceneconverter->ConvertScene(
 			startscene,
-			m_rendertools,
+			m_rasterizer,
 			m_canvas);
 		m_ketsjiengine->AddScene(startscene);
 		
@@ -773,9 +784,6 @@ void GPG_Application::stopEngine()
 	}
 
 	m_pyGlobalDictString_Length = saveGamePythonConfig(&m_pyGlobalDictString);
-	
-	// when exiting the mainloop
-	exitGamePythonScripting();
 #endif
 	
 	m_ketsjiengine->StopEngine();
@@ -789,6 +797,7 @@ void GPG_Application::stopEngine()
 		m_system->removeTimer(m_frameTimer);
 		m_frameTimer = 0;
 	}
+
 	m_engineRunning = false;
 }
 
@@ -853,18 +862,18 @@ void GPG_Application::exitEngine()
 		delete m_rasterizer;
 		m_rasterizer = 0;
 	}
-	if (m_rendertools)
-	{
-		delete m_rendertools;
-		m_rendertools = 0;
-	}
 	if (m_canvas)
 	{
 		delete m_canvas;
 		m_canvas = 0;
 	}
 
-	GPU_extensions_exit();
+	GPU_exit();
+
+#ifdef WITH_PYTHON
+	// Call this after we're sure nothing needs Python anymore (e.g., destructors)
+	exitGamePlayerPythonScripting();
+#endif
 
 	m_exitRequested = 0;
 	m_engineInitialized = false;
@@ -943,11 +952,12 @@ bool GPG_Application::handleKey(GHOST_IEvent* event, bool isDown)
 	{
 		GHOST_TEventDataPtr eventData = ((GHOST_IEvent*)event)->getData();
 		GHOST_TEventKeyData* keyData = static_cast<GHOST_TEventKeyData*>(eventData);
+		unsigned int unicode = keyData->utf8_buf[0] ? BLI_str_utf8_as_unicode(keyData->utf8_buf) : keyData->ascii;
 
 		if (m_keyboard->ToNative(keyData->key) == KX_KetsjiEngine::GetExitKey() && !m_keyboard->m_hookesc && !m_isEmbedded) {
 			m_exitRequested = KX_EXIT_REQUEST_OUTSIDE;
 		}
-		m_keyboard->ConvertEvent(keyData->key, isDown);
+		m_keyboard->ConvertEvent(keyData->key, isDown, unicode);
 		handled = true;
 	}
 	return handled;
