@@ -49,6 +49,7 @@
 //#include "SCA_RandomEventManager.h"
 //#include "KX_RayEventManager.h"
 #include "SCA_2DFilterActuator.h"
+#include "SCA_PythonController.h"
 #include "KX_TouchEventManager.h"
 #include "SCA_KeyboardManager.h"
 #include "SCA_MouseManager.h"
@@ -83,6 +84,7 @@
 #include "NG_NetworkScene.h"
 #include "PHY_IPhysicsEnvironment.h"
 #include "PHY_IGraphicController.h"
+#include "PHY_IPhysicsController.h"
 #include "KX_BlenderSceneConverter.h"
 #include "KX_MotionState.h"
 
@@ -93,15 +95,14 @@
 
 #ifdef WITH_BULLET
 #include "KX_SoftBodyDeformer.h"
-#include "KX_ConvertPhysicsObject.h"
-#include "CcdPhysicsEnvironment.h"
-#include "CcdPhysicsController.h"
 #endif
 
 #include "KX_Light.h"
 
 #include <stdio.h>
 
+#include "BLI_task.h"
+
 static void *KX_SceneReplicationFunc(SG_IObject* node,void* gameobj,void* scene)
 {
 	KX_GameObject* replica = ((KX_Scene*)scene)->AddNodeReplicaObject(node,(KX_GameObject*)gameobj);
@@ -562,7 +563,6 @@ KX_GameObject* KX_Scene::AddNodeReplicaObject(class SG_IObject* node, class CVal
 		newobj->SetGraphicController(newctrl);
 	}
 
-#ifdef WITH_BULLET
 	// replicate physics controller
 	if (orgobj->GetPhysicsController())
 	{
@@ -575,11 +575,7 @@ KX_GameObject* KX_Scene::AddNodeReplicaObject(class SG_IObject* node, class CVal
 		newctrl->SetNewClientInfo(newobj->getClientInfo());
 		newobj->SetPhysicsController(newctrl, newobj->IsDynamic());
 		newctrl->PostProcessReplica(motionstate, parentctrl);
-
-		if (parent)
-			parent->Release();
 	}
-#endif
 
 	return newobj;
 }
@@ -599,7 +595,9 @@ KX_GameObject* KX_Scene::AddNodeReplicaObject(class SG_IObject* node, class CVal
 void KX_Scene::ReplicateLogic(KX_GameObject* newobj)
 {
 	/* add properties to debug list, for added objects and DupliGroups */
-	AddObjectDebugProperties(newobj);
+	if (KX_GetActiveEngine()->GetAutoAddDebugProperties()) {
+		AddObjectDebugProperties(newobj);
+	}
 	// also relink the controller to sensors/actuators
 	SCA_ControllerList& controllers = newobj->GetControllers();
 	//SCA_SensorList&     sensors     = newobj->GetSensors();
@@ -759,8 +757,6 @@ void KX_Scene::DupliGroupRecurse(CValue* obj, int level)
 		KX_GameObject *parent = gameobj->GetParent();
 		if (parent != NULL)
 		{
-			parent->Release(); // GetParent() increased the refcount
-
 			// this object is not a top parent. Either it is the child of another
 			// object in the group and it will be added automatically when the parent
 			// is added. Or it is the child of an object outside the group and the group
@@ -837,7 +833,7 @@ void KX_Scene::DupliGroupRecurse(CValue* obj, int level)
 		if ((*git)->GetGameObjectType()==SCA_IObject::OBJ_LIGHT)
 		{
 			KX_LightObject* lightobj = static_cast<KX_LightObject*>(*git);
-			lightobj->GetLightData()->m_layer = groupobj->GetLayer();
+			lightobj->SetLayer(groupobj->GetLayer());
 		}
 	}
 
@@ -947,7 +943,7 @@ SCA_IObject* KX_Scene::AddReplicaObject(class CValue* originalobject,
 		if ((*git)->GetGameObjectType()==SCA_IObject::OBJ_LIGHT)
 		{
 			KX_LightObject* lightobj = static_cast<KX_LightObject*>(*git);
-			lightobj->GetLightData()->m_layer = parentobj->GetLayer();
+			lightobj->SetLayer(parentobj->GetLayer());
 		}
 	}
 
@@ -1011,7 +1007,7 @@ int KX_Scene::NewRemoveObject(class CValue* gameobj)
 	int ret;
 	KX_GameObject* newobj = (KX_GameObject*) gameobj;
 
-	/* remove property to debug list */
+	/* remove property from debug list */
 	RemoveObjectDebugProperties(newobj);
 
 	/* Invalidate the python reference, since the object may exist in script lists
@@ -1044,6 +1040,7 @@ int KX_Scene::NewRemoveObject(class CValue* gameobj)
 		 !(itc==controllers.end());itc++)
 	{
 		m_logicmgr->RemoveController(*itc);
+		(*itc)->ReParent(NULL);
 	}
 
 	SCA_ActuatorList& actuators = newobj->GetActuators();
@@ -1172,8 +1169,6 @@ void KX_Scene::ReplaceMesh(class CValue* obj,void* meshobj, bool use_gfx, bool u
 #ifdef WITH_BULLET
 			bool bHasSoftBody = (!parentobj && (blendobj->gameflag & OB_SOFT_BODY));
 #endif
-			bool releaseParent = true;
-
 			
 			if (oldblendobj==NULL) {
 				if (bHasModifier || bHasShapeKey || bHasDvert || bHasArmature) {
@@ -1193,10 +1188,9 @@ void KX_Scene::ReplaceMesh(class CValue* obj,void* meshobj, bool use_gfx, bool u
 						oldblendobj, blendobj,
 						mesh,
 						true,
-						static_cast<BL_ArmatureObject*>( parentobj )
+						static_cast<BL_ArmatureObject*>( parentobj->AddRef() )
 					);
-					releaseParent= false;
-					modifierDeformer->LoadShapeDrivers(blendobj->parent);
+					modifierDeformer->LoadShapeDrivers(parentobj);
 				}
 				else
 				{
@@ -1221,10 +1215,9 @@ void KX_Scene::ReplaceMesh(class CValue* obj,void* meshobj, bool use_gfx, bool u
 						mesh,
 						true,
 						true,
-						static_cast<BL_ArmatureObject*>( parentobj )
+						static_cast<BL_ArmatureObject*>( parentobj->AddRef() )
 					);
-					releaseParent= false;
-					shapeDeformer->LoadShapeDrivers(blendobj->parent);
+					shapeDeformer->LoadShapeDrivers(parentobj);
 				}
 				else
 				{
@@ -1247,9 +1240,8 @@ void KX_Scene::ReplaceMesh(class CValue* obj,void* meshobj, bool use_gfx, bool u
 					mesh,
 					true,
 					true,
-					static_cast<BL_ArmatureObject*>( parentobj )
+					static_cast<BL_ArmatureObject*>( parentobj->AddRef() )
 				);
-				releaseParent= false;
 				newobj->SetDeformer(skinDeformer);
 			}
 			else if (bHasDvert)
@@ -1266,21 +1258,16 @@ void KX_Scene::ReplaceMesh(class CValue* obj,void* meshobj, bool use_gfx, bool u
 				newobj->SetDeformer(softdeformer);
 			}
 #endif
-
-			// release parent reference if its not being used 
-			if ( releaseParent && parentobj)
-				parentobj->Release();
 		}
 	}
 
 	gameobj->AddMeshUser();
 	}
 
-#ifdef WITH_BULLET
 	if (use_phys) { /* update the new assigned mesh with the physics mesh */
-		KX_ReInstanceBulletShapeFromMesh(gameobj, NULL, use_gfx?NULL:mesh);
+		if (gameobj->GetPhysicsController())
+			gameobj->GetPhysicsController()->ReinstancePhysicsShape(NULL, use_gfx?NULL:mesh);
 	}
-#endif
 }
 
 /* Font Object routines */
@@ -1364,17 +1351,6 @@ void KX_Scene::SetCameraOnTop(KX_Camera* cam)
 	}
 }
 
-
-void KX_Scene::UpdateMeshTransformations()
-{
-	// do this incrementally in the future
-	for (int i = 0; i < m_objectlist->GetCount(); i++)
-	{
-		KX_GameObject* gameobj = (KX_GameObject*)m_objectlist->GetValue(i);
-		gameobj->GetOpenGLMatrix();
-	}
-}
-
 void KX_Scene::MarkVisible(SG_Tree *node, RAS_IRasterizer* rasty, KX_Camera* cam, int layer)
 {
 	int intersect = KX_Camera::INTERSECT;
@@ -1526,6 +1502,15 @@ void KX_Scene::CalculateVisibleMeshes(RAS_IRasterizer* rasty,KX_Camera* cam, int
 	bool dbvt_culling = false;
 	if (m_dbvt_culling) 
 	{
+		/* Reset KX_GameObject m_bCulled to true before doing culling
+		 * since DBVT culling will only set it to false.
+		 * This is similar to what RAS_BucketManager does for RAS_MeshSlot culling.
+		 */
+		for (int i = 0; i < m_objectlist->GetCount(); i++) {
+			KX_GameObject *gameobj = static_cast<KX_GameObject*>(m_objectlist->GetValue(i));
+			gameobj->SetCulled(true);
+		}
+
 		// test culling through Bullet
 		MT_Vector4 planes[6];
 		// get the clip planes
@@ -1596,51 +1581,84 @@ void KX_Scene::AddAnimatedObject(CValue* gameobj)
 	m_animatedlist->Add(gameobj);
 }
 
-void KX_Scene::UpdateAnimations(double curtime)
+static void update_anim_thread_func(TaskPool *pool, void *taskdata, int UNUSED(threadid))
 {
-	KX_GameObject *gameobj;
+	KX_GameObject *gameobj, *child, *parent;
+	CListValue *children;
 	bool needs_update;
+	double curtime = *(double*)BLI_task_pool_userdata(pool);
 
-	for (int i=0; i<m_animatedlist->GetCount(); ++i) {
-		gameobj = (KX_GameObject*)m_animatedlist->GetValue(i);
+	gameobj = (KX_GameObject*)taskdata;
 
-		// Non-armature updates are fast enough, so just update them
-		needs_update = gameobj->GetGameObjectType() != SCA_IObject::OBJ_ARMATURE;
+	// Non-armature updates are fast enough, so just update them
+	needs_update = gameobj->GetGameObjectType() != SCA_IObject::OBJ_ARMATURE;
 
-		if (!needs_update) {
-			// If we got here, we're looking to update an armature, so check its children meshes
-			// to see if we need to bother with a more expensive pose update
-			CListValue *children = gameobj->GetChildren();
-			KX_GameObject *child;
+	if (!needs_update) {
+		// If we got here, we're looking to update an armature, so check its children meshes
+		// to see if we need to bother with a more expensive pose update
+		children = gameobj->GetChildren();
 
-			bool has_mesh = false, has_non_mesh = false;
+		bool has_mesh = false, has_non_mesh = false;
 
-			// Check for meshes that haven't been culled
-			for (int j=0; j<children->GetCount(); ++j) {
-				child = (KX_GameObject*)children->GetValue(j);
+		// Check for meshes that haven't been culled
+		for (int j=0; j<children->GetCount(); ++j) {
+			child = (KX_GameObject*)children->GetValue(j);
 
-				if (!child->GetCulled()) {
-					needs_update = true;
-					break;
-				}
-
-				if (child->GetMeshCount() == 0)
-					has_non_mesh = true;
-				else
-					has_mesh = true;
+			if (!child->GetCulled()) {
+				needs_update = true;
+				break;
 			}
 
-			// If we didn't find a non-culled mesh, check to see
-			// if we even have any meshes, and update if this
-			// armature has only non-mesh children.
-			if (!needs_update && !has_mesh && has_non_mesh)
-				needs_update = true;
-
-			children->Release();
+			if (child->GetMeshCount() == 0)
+				has_non_mesh = true;
+			else
+				has_mesh = true;
 		}
 
-		if (needs_update)
-			gameobj->UpdateActionManager(curtime);
+		// If we didn't find a non-culled mesh, check to see
+		// if we even have any meshes, and update if this
+		// armature has only non-mesh children.
+		if (!needs_update && !has_mesh && has_non_mesh)
+			needs_update = true;
+
+		children->Release();
+	}
+
+	if (needs_update) {
+		gameobj->UpdateActionManager(curtime);
+		children = gameobj->GetChildren();
+		parent = gameobj->GetParent();
+
+		// Only do deformers here if they are not parented to an armature, otherwise the armature will
+		// handle updating its children
+		if (gameobj->GetDeformer() && (!parent || (parent && parent->GetGameObjectType() != SCA_IObject::OBJ_ARMATURE)))
+			gameobj->GetDeformer()->Update();
+
+		for (int j=0; j<children->GetCount(); ++j) {
+			child = (KX_GameObject*)children->GetValue(j);
+
+			if (child->GetDeformer()) {
+				child->GetDeformer()->Update();
+			}
+		}
+
+		children->Release();
+	}
+}
+
+void KX_Scene::UpdateAnimations(double curtime)
+{
+	TaskPool *pool = BLI_task_pool_create(KX_GetActiveEngine()->GetTaskScheduler(), &curtime);
+
+	for (int i=0; i<m_animatedlist->GetCount(); ++i) {
+		BLI_task_pool_push(pool, update_anim_thread_func, m_animatedlist->GetValue(i), false, TASK_PRIORITY_LOW);
+	}
+
+	BLI_task_pool_work_and_wait(pool);
+	BLI_task_pool_free(pool);
+
+	for (int i=0; i<m_animatedlist->GetCount(); ++i) {
+		((KX_GameObject*)m_animatedlist->GetValue(i))->UpdateActionIPOs();
 	}
 }
 
@@ -1721,7 +1739,7 @@ void KX_Scene::RenderFonts()
 {
 	list<KX_FontObject*>::iterator it = m_fonts.begin();
 	while (it != m_fonts.end()) {
-		(*it)->DrawText();
+		(*it)->DrawFontText();
 		++it;
 	}
 }
@@ -1848,7 +1866,7 @@ short KX_Scene::GetAnimationFPS()
 	return m_blenderScene->r.frs_sec;
 }
 
-static void MergeScene_LogicBrick(SCA_ILogicBrick* brick, KX_Scene *to)
+static void MergeScene_LogicBrick(SCA_ILogicBrick* brick, KX_Scene *from, KX_Scene *to)
 {
 	SCA_LogicManager *logicmgr= to->GetLogicManager();
 
@@ -1856,12 +1874,13 @@ static void MergeScene_LogicBrick(SCA_ILogicBrick* brick, KX_Scene *to)
 	brick->Replace_NetworkScene(to->GetNetworkScene());
 
 	/* near sensors have physics controllers */
-#ifdef WITH_BULLET
 	KX_TouchSensor *touch_sensor = dynamic_cast<class KX_TouchSensor *>(brick);
 	if (touch_sensor) {
+		KX_TouchEventManager *tmgr = (KX_TouchEventManager*)from->GetLogicManager()->FindEventManager(SCA_EventManager::TOUCH_EVENTMGR);
+		touch_sensor->UnregisterSumo(tmgr);
 		touch_sensor->GetPhysicsController()->SetPhysicsEnvironment(to->GetPhysicsEnvironment());
+		touch_sensor->RegisterSumo(tmgr);
 	}
-#endif
 
 	// If we end up replacing a KX_TouchEventManager, we need to make sure
 	// physics controllers are properly in place. In other words, do this
@@ -1875,11 +1894,20 @@ static void MergeScene_LogicBrick(SCA_ILogicBrick* brick, KX_Scene *to)
 	if (filter_actuator) {
 		filter_actuator->SetScene(to);
 	}
-}
 
-#ifdef WITH_BULLET
-#include "CcdGraphicController.h" // XXX  ctrl->SetPhysicsEnvironment(to->GetPhysicsEnvironment());
+#ifdef WITH_PYTHON
+	// Python must be called from the main thread unless we want to deal
+	// with GIL issues. So, this is delayed until here in case of async
+	// libload (originally in KX_ConvertControllers)
+	SCA_PythonController *pyctrl = dynamic_cast<SCA_PythonController*>(brick);
+	if (pyctrl) {
+		pyctrl->SetNamespace(KX_GetActiveEngine()->GetPyNamespace());
+
+		if (pyctrl->m_mode==SCA_PythonController::SCA_PYEXEC_SCRIPT)
+			pyctrl->Compile();
+	}
 #endif
+}
 
 static void MergeScene_GameObject(KX_GameObject* gameobj, KX_Scene *to, KX_Scene *from)
 {
@@ -1889,7 +1917,7 @@ static void MergeScene_GameObject(KX_GameObject* gameobj, KX_Scene *to, KX_Scene
 
 		for (ita = actuators.begin(); !(ita==actuators.end()); ++ita)
 		{
-			MergeScene_LogicBrick(*ita, to);
+			MergeScene_LogicBrick(*ita, from, to);
 		}
 	}
 
@@ -1900,7 +1928,7 @@ static void MergeScene_GameObject(KX_GameObject* gameobj, KX_Scene *to, KX_Scene
 
 		for (its = sensors.begin(); !(its==sensors.end()); ++its)
 		{
-			MergeScene_LogicBrick(*its, to);
+			MergeScene_LogicBrick(*its, from, to);
 		}
 	}
 
@@ -1911,17 +1939,17 @@ static void MergeScene_GameObject(KX_GameObject* gameobj, KX_Scene *to, KX_Scene
 		for (itc = controllers.begin(); !(itc==controllers.end()); ++itc)
 		{
 			SCA_IController *cont= *itc;
-			MergeScene_LogicBrick(cont, to);
+			MergeScene_LogicBrick(cont, from, to);
 
 			vector<SCA_ISensor*> linkedsensors = cont->GetLinkedSensors();
 			vector<SCA_IActuator*> linkedactuators = cont->GetLinkedActuators();
 
 			for (vector<SCA_IActuator*>::iterator ita = linkedactuators.begin();!(ita==linkedactuators.end());++ita) {
-				MergeScene_LogicBrick(*ita, to);
+				MergeScene_LogicBrick(*ita, from, to);
 			}
 
 			for (vector<SCA_ISensor*>::iterator its = linkedsensors.begin();!(its==linkedsensors.end());++its) {
-				MergeScene_LogicBrick(*its, to);
+				MergeScene_LogicBrick(*its, from, to);
 			}
 		}
 	}
@@ -1933,12 +1961,10 @@ static void MergeScene_GameObject(KX_GameObject* gameobj, KX_Scene *to, KX_Scene
 		ctrl->SetPhysicsEnvironment(to->GetPhysicsEnvironment());
 	}
 
-#ifdef WITH_BULLET
 	ctrl = gameobj->GetPhysicsController();
 	if (ctrl) {
 		ctrl->SetPhysicsEnvironment(to->GetPhysicsEnvironment());
 	}
-#endif
 
 	/* SG_Node can hold a scene reference */
 	SG_Node *sg= gameobj->GetSGNode();
@@ -1969,9 +1995,8 @@ static void MergeScene_GameObject(KX_GameObject* gameobj, KX_Scene *to, KX_Scene
 
 bool KX_Scene::MergeScene(KX_Scene *other)
 {
-#ifdef WITH_BULLET
-	CcdPhysicsEnvironment *env=			dynamic_cast<CcdPhysicsEnvironment *>(this->GetPhysicsEnvironment());
-	CcdPhysicsEnvironment *env_other=	dynamic_cast<CcdPhysicsEnvironment *>(other->GetPhysicsEnvironment());
+	PHY_IPhysicsEnvironment *env = this->GetPhysicsEnvironment();
+	PHY_IPhysicsEnvironment *env_other = other->GetPhysicsEnvironment();
 
 	if ((env==NULL) != (env_other==NULL)) /* TODO - even when both scenes have NONE physics, the other is loaded with bullet enabled, ??? */
 	{
@@ -1979,7 +2004,6 @@ bool KX_Scene::MergeScene(KX_Scene *other)
 		printf("\tsource %d, terget %d\n", (int)(env!=NULL), (int)(env_other!=NULL));
 		return false;
 	}
-#endif // WITH_BULLET
 
 	if (GetSceneConverter() != other->GetSceneConverter()) {
 		printf("KX_Scene::MergeScene: converters differ, aborting\n");
@@ -1995,7 +2019,11 @@ bool KX_Scene::MergeScene(KX_Scene *other)
 	{
 		KX_GameObject* gameobj = (KX_GameObject*)other->GetObjectList()->GetValue(i);
 		MergeScene_GameObject(gameobj, this, other);
-		AddObjectDebugProperties(gameobj); // add properties to debug list for LibLoad objects
+
+		/* add properties to debug list for LibLoad objects */
+		if (KX_GetActiveEngine()->GetAutoAddDebugProperties()) {
+			AddObjectDebugProperties(gameobj);
+		}
 
 		gameobj->UpdateBuckets(false); /* only for active objects */
 	}
@@ -2021,10 +2049,8 @@ bool KX_Scene::MergeScene(KX_Scene *other)
 	GetLightList()->MergeList(other->GetLightList());
 	other->GetLightList()->ReleaseAndRemoveAll();
 
-#ifdef WITH_BULLET
-	if (env) /* bullet scene? - dummy scenes don't need touching */
+	if (env)
 		env->MergeEnvironment(env_other);
-#endif
 
 	/* move materials across, assume they both use the same scene-converters
 	 * Do this after lights are merged so materials can use the lights in shaders
@@ -2470,16 +2496,18 @@ KX_PYMETHODDEF_DOC(KX_Scene, restart,
 
 KX_PYMETHODDEF_DOC(KX_Scene, replace,
 				   "replace(newScene)\n"
-				   "Replaces this scene with another one.\n")
+                   "Replaces this scene with another one.\n"
+                   "Return True if the new scene exists and scheduled for replacement, False otherwise.\n")
 {
 	char* name;
 	
 	if (!PyArg_ParseTuple(args, "s:replace", &name))
 		return NULL;
 	
-	KX_GetActiveEngine()->ReplaceScene(m_sceneName, name);
+    if (KX_GetActiveEngine()->ReplaceScene(m_sceneName, name))
+        Py_RETURN_TRUE;
 	
-	Py_RETURN_NONE;
+    Py_RETURN_FALSE;
 }
 
 KX_PYMETHODDEF_DOC(KX_Scene, suspend,
