@@ -1,8 +1,27 @@
+/*
+ * ***** BEGIN GPL LICENSE BLOCK *****
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *
+ * ***** END GPL LICENSE BLOCK *****
+ */
+
 /** \file gameengine/Ketsji/KX_VehicleWrapper.cpp
  *  \ingroup ketsji
  */
 
-
 #include "PyObjectPlus.h"
 
 #include "KX_VehicleWrapper.h"
@@ -59,19 +78,12 @@ PyObject *KX_VehicleWrapper::PyAddWheel(PyObject *args)
 			PyVecTo(pylistPos,attachPos);
 			PyVecTo(pylistDir,attachDir);
 			PyVecTo(pylistAxleDir,attachAxle);
-			PHY__Vector3 aPos,aDir,aAxle;
-			aPos[0] = attachPos[0];
-			aPos[1] = attachPos[1];
-			aPos[2] = attachPos[2];
-			aDir[0] = attachDir[0];
-			aDir[1] = attachDir[1];
-			aDir[2] = attachDir[2];
-			aAxle[0] = -attachAxle[0];//someone reverse some conventions inside Bullet (axle winding)
-			aAxle[1] = -attachAxle[1];
-			aAxle[2] = -attachAxle[2];
+
+			//someone reverse some conventions inside Bullet (axle winding)
+			attachAxle = -attachAxle;
 			
 			printf("attempt for addWheel: suspensionRestLength%f wheelRadius %f, hasSteering:%d\n",suspensionRestLength,wheelRadius,hasSteering);
-			m_vehicle->AddWheel(motionState,aPos,aDir,aAxle,suspensionRestLength,wheelRadius,hasSteering);
+			m_vehicle->AddWheel(motionState,attachPos,attachDir,attachAxle,suspensionRestLength,wheelRadius,hasSteering);
 		}
 		
 	} else {
@@ -126,13 +138,13 @@ PyObject *KX_VehicleWrapper::PyGetWheelOrientationQuaternion(PyObject *args)
 
 PyObject *KX_VehicleWrapper::PyGetNumWheels(PyObject *args)
 {
-	return PyLong_FromSsize_t(m_vehicle->GetNumWheels());
+	return PyLong_FromLong(m_vehicle->GetNumWheels());
 }
 
 
 PyObject *KX_VehicleWrapper::PyGetConstraintId(PyObject *args)
 {
-	return PyLong_FromSsize_t(m_vehicle->GetUserConstraintId());
+	return PyLong_FromLong(m_vehicle->GetUserConstraintId());
 }
 
 
@@ -263,7 +275,7 @@ PyObject *KX_VehicleWrapper::PySetSteeringValue(PyObject *args)
 
 PyObject *KX_VehicleWrapper::PyGetConstraintType(PyObject *args)
 {
-	return PyLong_FromSsize_t(m_vehicle->GetUserConstraintType());
+	return PyLong_FromLong(m_vehicle->GetUserConstraintType());
 }
 
 
