@@ -35,24 +35,36 @@
  
 #include "RAS_OpenGLRasterizer.h"
 
-#include "GL/glew.h"
+#include "glew-mx.h"
 
+#include "RAS_ICanvas.h"
 #include "RAS_Rect.h"
 #include "RAS_TexVert.h"
 #include "RAS_MeshObject.h"
+#include "RAS_Polygon.h"
+#include "RAS_ILightObject.h"
 #include "MT_CmMatrix4x4.h"
-#include "RAS_IRenderTools.h" // rendering text
+
+#include "RAS_OpenGLLight.h"
+
+#include "RAS_StorageIM.h"
+#include "RAS_StorageVA.h"
+#include "RAS_StorageVBO.h"
 
 #include "GPU_draw.h"
 #include "GPU_material.h"
-#include "GPU_extensions.h"
 
-#include "DNA_image_types.h"
-#include "DNA_meshdata_types.h"
-#include "DNA_material_types.h"
-#include "DNA_scene_types.h"
+extern "C"{
+	#include "BLF_api.h"
+}
 
-#include "BKE_DerivedMesh.h"
+
+// XXX Clean these up <<<
+#include "Value.h"
+#include "KX_Scene.h"
+#include "KX_RayCast.h"
+#include "KX_GameObject.h"
+// >>>
 
 #ifndef M_PI
 #define M_PI		3.14159265358979323846
@@ -71,7 +83,7 @@ static GLuint right_eye_vinterlace_mask[32];
  */
 static GLuint hinterlace_mask[33];
 
-RAS_OpenGLRasterizer::RAS_OpenGLRasterizer(RAS_ICanvas* canvas)
+RAS_OpenGLRasterizer::RAS_OpenGLRasterizer(RAS_ICanvas* canvas, int storage)
 	:RAS_IRasterizer(canvas),
 	m_2DCanvas(canvas),
 	m_fogenabled(false),
@@ -86,11 +98,16 @@ RAS_OpenGLRasterizer::RAS_OpenGLRasterizer(RAS_ICanvas* canvas)
 	m_noOfScanlines(32),
 	m_motionblur(0),
 	m_motionblurvalue(-1.0),
+	m_usingoverrideshader(false),
+    m_clientobject(NULL),
+    m_auxilaryClientInfo(NULL),
+    m_drawingmode(KX_TEXTURED),
 	m_texco_num(0),
 	m_attrib_num(0),
 	//m_last_alphablend(GPU_BLEND_SOLID),
 	m_last_frontface(true),
-	m_materialCachingInfo(0)
+	m_materialCachingInfo(0),
+	m_storage_type(storage)
 {
 	m_viewmatrix.setIdentity();
 	m_viewinvmatrix.setIdentity();
@@ -104,6 +121,28 @@ RAS_OpenGLRasterizer::RAS_OpenGLRasterizer(RAS_ICanvas* canvas)
 	hinterlace_mask[32] = 0;
 
 	m_prevafvalue = GPU_get_anisotropic();
+
+	if (m_storage_type == RAS_VBO /*|| m_storage_type == RAS_AUTO_STORAGE && GLEW_ARB_vertex_buffer_object*/)
+	{
+		m_storage = new RAS_StorageVBO(&m_texco_num, m_texco, &m_attrib_num, m_attrib, m_attrib_layer);
+		m_failsafe_storage = new RAS_StorageIM(&m_texco_num, m_texco, &m_attrib_num, m_attrib, m_attrib_layer);
+		m_storage_type = RAS_VBO;
+	}
+	else if ((m_storage_type == RAS_VA) || (m_storage_type == RAS_AUTO_STORAGE && GLEW_VERSION_1_1))
+	{
+		m_storage = new RAS_StorageVA(&m_texco_num, m_texco, &m_attrib_num, m_attrib, m_attrib_layer);
+		m_failsafe_storage = new RAS_StorageIM(&m_texco_num, m_texco, &m_attrib_num, m_attrib, m_attrib_layer);
+		m_storage_type = RAS_VA;
+	}
+	else
+	{
+		m_storage = m_failsafe_storage = new RAS_StorageIM(&m_texco_num, m_texco, &m_attrib_num, m_attrib, m_attrib_layer);
+		m_storage_type = RAS_IMMEDIATE;
+	}
+
+	glGetIntegerv(GL_MAX_LIGHTS, (GLint *) &m_numgllights);
+	if (m_numgllights < 8)
+		m_numgllights = 8;
 }
 
 
@@ -112,10 +151,16 @@ RAS_OpenGLRasterizer::~RAS_OpenGLRasterizer()
 {
 	// Restore the previous AF value
 	GPU_set_anisotropic(m_prevafvalue);
+	if (m_failsafe_storage && m_failsafe_storage != m_storage)
+		delete m_failsafe_storage;
+
+	if (m_storage)
+		delete m_storage;
 }
 
 bool RAS_OpenGLRasterizer::Init()
 {
+	bool storage_init;
 	GPU_state_init();
 
 
@@ -143,7 +188,9 @@ bool RAS_OpenGLRasterizer::Init()
 
 	glShadeModel(GL_SMOOTH);
 
-	return true;
+	storage_init = m_storage->Init();
+
+	return true && storage_init;
 }
 
 
@@ -239,10 +286,10 @@ void RAS_OpenGLRasterizer::DisplayFog()
 		glFogf(GL_FOG_DENSITY, 0.1f);
 		glFogf(GL_FOG_START, m_fogstart);
 		glFogf(GL_FOG_END, m_fogstart + m_fogdist);
-		params[0]= m_fogr;
-		params[1]= m_fogg;
-		params[2]= m_fogb;
-		params[3]= 0.0;
+		params[0] = m_fogr;
+		params[1] = m_fogg;
+		params[2] = m_fogb;
+		params[3] = 0.0;
 		glFogfv(GL_FOG_COLOR, params); 
 		glEnable(GL_FOG);
 	} 
@@ -264,6 +311,8 @@ bool RAS_OpenGLRasterizer::SetMaterial(const RAS_IPolyMaterial& mat)
 void RAS_OpenGLRasterizer::Exit()
 {
 
+	m_storage->Exit();
+
 	glEnable(GL_CULL_FACE);
 	glEnable(GL_DEPTH_TEST);
 	glClearDepth(1.0); 
@@ -283,21 +332,20 @@ void RAS_OpenGLRasterizer::Exit()
 	EndFrame();
 }
 
-bool RAS_OpenGLRasterizer::BeginFrame(int drawingmode, double time)
+bool RAS_OpenGLRasterizer::BeginFrame(double time)
 {
 	m_time = time;
-	m_drawingmode = drawingmode;
 
 	// Blender camera routine destroys the settings
 	if (m_drawingmode < KX_SOLID)
 	{
-		glDisable (GL_CULL_FACE);
-		glDisable (GL_DEPTH_TEST);
+		glDisable(GL_CULL_FACE);
+		glDisable(GL_DEPTH_TEST);
 	}
 	else
 	{
 		glEnable(GL_DEPTH_TEST);
-		glEnable (GL_CULL_FACE);
+		glEnable(GL_CULL_FACE);
 	}
 
 	glDisable(GL_BLEND);
@@ -313,6 +361,13 @@ bool RAS_OpenGLRasterizer::BeginFrame(int drawingmode, double time)
 	glEnable(GL_MULTISAMPLE_ARB);
 
 	m_2DCanvas->BeginFrame();
+
+	// Render Tools
+	m_clientobject = NULL;
+	m_lastlightlayer = -1;
+	m_lastauxinfo = NULL;
+	m_lastlighting = true; /* force disable in DisableOpenGLLights() */
+	DisableOpenGLLights();
 	
 	return true;
 }
@@ -325,6 +380,11 @@ void RAS_OpenGLRasterizer::SetDrawingMode(int drawingmode)
 
 	if (m_drawingmode == KX_WIREFRAME)
 		glDisable(GL_CULL_FACE);
+
+	m_storage->SetDrawingMode(drawingmode);
+	if (m_failsafe_storage && m_failsafe_storage != m_storage) {
+		m_failsafe_storage->SetDrawingMode(drawingmode);
+	}
 }
 
 int RAS_OpenGLRasterizer::GetDrawingMode()
@@ -447,8 +507,7 @@ void RAS_OpenGLRasterizer::SetRenderArea()
 	switch (m_stereomode)
 	{
 		case RAS_STEREO_ABOVEBELOW:
-			switch(m_curreye)
-			{
+			switch (m_curreye) {
 				case RAS_STEREO_LEFTEYE:
 					// upper half of window
 					area.SetLeft(0);
@@ -469,6 +528,28 @@ void RAS_OpenGLRasterizer::SetRenderArea()
 					break;
 			}
 			break;
+		case RAS_STEREO_3DTVTOPBOTTOM:
+			switch (m_curreye) {
+				case RAS_STEREO_LEFTEYE:
+					// upper half of window
+					area.SetLeft(0);
+					area.SetBottom(m_2DCanvas->GetHeight() -
+						m_2DCanvas->GetHeight() / 2);
+	
+					area.SetRight(m_2DCanvas->GetWidth());
+					area.SetTop(m_2DCanvas->GetHeight());
+					m_2DCanvas->SetDisplayArea(&area);
+					break;
+				case RAS_STEREO_RIGHTEYE:
+					// lower half of window
+					area.SetLeft(0);
+					area.SetBottom(0);
+					area.SetRight(m_2DCanvas->GetWidth());
+					area.SetTop(m_2DCanvas->GetHeight() / 2);
+					m_2DCanvas->SetDisplayArea(&area);
+					break;
+			}
+			break;
 		case RAS_STEREO_SIDEBYSIDE:
 			switch (m_curreye)
 			{
@@ -534,11 +615,11 @@ void RAS_OpenGLRasterizer::SetEye(const StereoEye eye)
 			break;
 		case RAS_STEREO_ANAGLYPH:
 			if (m_curreye == RAS_STEREO_LEFTEYE) {
-				glColorMask(GL_FALSE, GL_TRUE, GL_TRUE, GL_FALSE);
+				glColorMask(GL_TRUE, GL_FALSE, GL_FALSE, GL_FALSE);
 			}
 			else {
 				//glAccum(GL_LOAD, 1.0);
-				glColorMask(GL_TRUE, GL_FALSE, GL_FALSE, GL_FALSE);
+				glColorMask(GL_FALSE, GL_TRUE, GL_TRUE, GL_FALSE);
 				ClearDepthBuffer();
 			}
 			break;
@@ -609,13 +690,14 @@ const MT_Matrix4x4& RAS_OpenGLRasterizer::GetViewInvMatrix() const
 }
 
 void RAS_OpenGLRasterizer::IndexPrimitives_3DText(RAS_MeshSlot& ms,
-									class RAS_IPolyMaterial* polymat,
-									class RAS_IRenderTools* rendertools)
+									class RAS_IPolyMaterial* polymat)
 { 
 	bool obcolor = ms.m_bObjectColor;
 	MT_Vector4& rgba = ms.m_RGBAcolor;
 	RAS_MeshSlot::iterator it;
 
+	const STR_String& mytext = ((CValue*)m_clientobject)->GetPropertyText("Text");
+
 	// handle object color
 	if (obcolor) {
 		glDisableClientState(GL_COLOR_ARRAY);
@@ -664,11 +746,10 @@ void RAS_OpenGLRasterizer::IndexPrimitives_3DText(RAS_MeshSlot& ms,
 				glattrib = -1;
 				if (GLEW_ARB_vertex_program)
 					for (unit=0; unit<m_attrib_num; unit++)
-						if (m_attrib[unit] == RAS_TEXCO_UV1)
+						if (m_attrib[unit] == RAS_TEXCO_UV)
 							glattrib = unit;
-				
-				rendertools->RenderText(polymat->GetDrawingMode(), polymat,
-					v[0], v[1], v[2], (numvert == 4)? v[3]: NULL, glattrib);
+
+				GPU_render_text(polymat->GetMTFace(), polymat->GetDrawingMode(), mytext, mytext.Length(), polymat->GetMCol(), v[0], v[1], v[2], v[3], glattrib);
 
 				ClearCachingInfo();
 			}
@@ -699,264 +780,29 @@ void RAS_OpenGLRasterizer::SetTexCoord(TexCoGen coords, int unit)
 		m_texco[unit] = coords;
 }
 
-void RAS_OpenGLRasterizer::SetAttrib(TexCoGen coords, int unit)
+void RAS_OpenGLRasterizer::SetAttrib(TexCoGen coords, int unit, int layer)
 {
 	// this changes from material to material
-	if (unit < RAS_MAX_ATTRIB)
+	if (unit < RAS_MAX_ATTRIB) {
 		m_attrib[unit] = coords;
-}
-
-void RAS_OpenGLRasterizer::TexCoord(const RAS_TexVert &tv)
-{
-	int unit;
-
-	if (GLEW_ARB_multitexture) {
-		for (unit=0; unit<m_texco_num; unit++) {
-			if (tv.getFlag() & RAS_TexVert::SECOND_UV && (int)tv.getUnit() == unit) {
-				glMultiTexCoord2fvARB(GL_TEXTURE0_ARB+unit, tv.getUV2());
-				continue;
-			}
-			switch(m_texco[unit]) {
-			case RAS_TEXCO_ORCO:
-			case RAS_TEXCO_GLOB:
-				glMultiTexCoord3fvARB(GL_TEXTURE0_ARB+unit, tv.getXYZ());
-				break;
-			case RAS_TEXCO_UV1:
-				glMultiTexCoord2fvARB(GL_TEXTURE0_ARB+unit, tv.getUV1());
-				break;
-			case RAS_TEXCO_NORM:
-				glMultiTexCoord3fvARB(GL_TEXTURE0_ARB+unit, tv.getNormal());
-				break;
-			case RAS_TEXTANGENT:
-				glMultiTexCoord4fvARB(GL_TEXTURE0_ARB+unit, tv.getTangent());
-				break;
-			case RAS_TEXCO_UV2:
-				glMultiTexCoord2fvARB(GL_TEXTURE0_ARB+unit, tv.getUV2());
-				break;
-			default:
-				break;
-			}
-		}
+		m_attrib_layer[unit] = layer;
 	}
-
-	if (GLEW_ARB_vertex_program) {
-		for (unit=0; unit<m_attrib_num; unit++) {
-			switch(m_attrib[unit]) {
-			case RAS_TEXCO_ORCO:
-			case RAS_TEXCO_GLOB:
-				glVertexAttrib3fvARB(unit, tv.getXYZ());
-				break;
-			case RAS_TEXCO_UV1:
-				glVertexAttrib2fvARB(unit, tv.getUV1());
-				break;
-			case RAS_TEXCO_NORM:
-				glVertexAttrib3fvARB(unit, tv.getNormal());
-				break;
-			case RAS_TEXTANGENT:
-				glVertexAttrib4fvARB(unit, tv.getTangent());
-				break;
-			case RAS_TEXCO_UV2:
-				glVertexAttrib2fvARB(unit, tv.getUV2());
-				break;
-			case RAS_TEXCO_VCOL:
-				glVertexAttrib4ubvARB(unit, tv.getRGBA());
-				break;
-			default:
-				break;
-			}
-		}
-	}
-
 }
 
 void RAS_OpenGLRasterizer::IndexPrimitives(RAS_MeshSlot& ms)
 {
-	IndexPrimitivesInternal(ms, false);
+	if (ms.m_pDerivedMesh)
+		m_failsafe_storage->IndexPrimitives(ms);
+	else
+		m_storage->IndexPrimitives(ms);
 }
 
 void RAS_OpenGLRasterizer::IndexPrimitivesMulti(RAS_MeshSlot& ms)
 {
-	IndexPrimitivesInternal(ms, true);
-}
-
-static bool current_wireframe;
-static RAS_MaterialBucket *current_bucket;
-static RAS_IPolyMaterial *current_polymat;
-static RAS_MeshSlot *current_ms;
-static RAS_MeshObject *current_mesh;
-static int current_blmat_nr;
-static GPUVertexAttribs current_gpu_attribs;
-static Image *current_image;
-static int CheckMaterialDM(int matnr, void *attribs)
-{
-	// only draw the current material
-	if (matnr != current_blmat_nr)
-		return 0;
-	GPUVertexAttribs *gattribs = (GPUVertexAttribs *)attribs;
-	if (gattribs)
-		memcpy(gattribs, &current_gpu_attribs, sizeof(GPUVertexAttribs));
-	return 1;
-}
-
-/*
-static int CheckTexfaceDM(void *mcol, int index)
-{
-
-	// index is the original face index, retrieve the polygon
-	RAS_Polygon* polygon = (index >= 0 && index < current_mesh->NumPolygons()) ?
-		current_mesh->GetPolygon(index) : NULL;
-	if (polygon && polygon->GetMaterial() == current_bucket) {
-		// must handle color.
-		if (current_wireframe)
-			return 2;
-		if (current_ms->m_bObjectColor) {
-			MT_Vector4& rgba = current_ms->m_RGBAcolor;
-			glColor4d(rgba[0], rgba[1], rgba[2], rgba[3]);
-			// don't use mcol
-			return 2;
-		}
-		if (!mcol) {
-			// we have to set the color from the material
-			unsigned char rgba[4];
-			current_polymat->GetMaterialRGBAColor(rgba);
-			glColor4ubv((const GLubyte *)rgba);
-			return 2;
-		}
-		return 1;
-	}
-	return 0;
-}
-*/
-
-static DMDrawOption CheckTexDM(MTFace *tface, int has_mcol, int matnr)
-{
-
-	// index is the original face index, retrieve the polygon
-	if (matnr == current_blmat_nr &&
-		(tface == NULL || tface->tpage == current_image)) {
-		// must handle color.
-		if (current_wireframe)
-			return DM_DRAW_OPTION_NO_MCOL;
-		if (current_ms->m_bObjectColor) {
-			MT_Vector4& rgba = current_ms->m_RGBAcolor;
-			glColor4d(rgba[0], rgba[1], rgba[2], rgba[3]);
-			// don't use mcol
-			return DM_DRAW_OPTION_NO_MCOL;
-		}
-		if (!has_mcol) {
-			// we have to set the color from the material
-			unsigned char rgba[4];
-			current_polymat->GetMaterialRGBAColor(rgba);
-			glColor4ubv((const GLubyte *)rgba);
-			return DM_DRAW_OPTION_NO_MCOL;
-		}
-		return DM_DRAW_OPTION_NORMAL;
-	}
-	return DM_DRAW_OPTION_SKIP;
-}
-
-void RAS_OpenGLRasterizer::IndexPrimitivesInternal(RAS_MeshSlot& ms, bool multi)
-{ 
-	bool obcolor = ms.m_bObjectColor;
-	bool wireframe = m_drawingmode <= KX_WIREFRAME;
-	MT_Vector4& rgba = ms.m_RGBAcolor;
-	RAS_MeshSlot::iterator it;
-
-	if (ms.m_pDerivedMesh) {
-		// mesh data is in derived mesh, 
-		current_bucket = ms.m_bucket;
-		current_polymat = current_bucket->GetPolyMaterial();
-		current_ms = &ms;
-		current_mesh = ms.m_mesh;
-		current_wireframe = wireframe;
-		// MCol *mcol = (MCol*)ms.m_pDerivedMesh->getFaceDataArray(ms.m_pDerivedMesh, CD_MCOL); /* UNUSED */
-
-		// handle two-side
-		if (current_polymat->GetDrawingMode() & RAS_IRasterizer::KX_BACKCULL)
-			this->SetCullFace(true);
-		else
-			this->SetCullFace(false);
-
-		if (current_polymat->GetFlag() & RAS_BLENDERGLSL) {
-			// GetMaterialIndex return the original mface material index, 
-			// increment by 1 to match what derived mesh is doing
-			current_blmat_nr = current_polymat->GetMaterialIndex()+1;
-			// For GLSL we need to retrieve the GPU material attribute
-			Material* blmat = current_polymat->GetBlenderMaterial();
-			Scene* blscene = current_polymat->GetBlenderScene();
-			if (!wireframe && blscene && blmat)
-				GPU_material_vertex_attributes(GPU_material_from_blender(blscene, blmat), &current_gpu_attribs);
-			else
-				memset(&current_gpu_attribs, 0, sizeof(current_gpu_attribs));
-			// DM draw can mess up blending mode, restore at the end
-			int current_blend_mode = GPU_get_material_alpha_blend();
-			ms.m_pDerivedMesh->drawFacesGLSL(ms.m_pDerivedMesh, CheckMaterialDM);
-			GPU_set_material_alpha_blend(current_blend_mode);
-		}
-		else {
-			//ms.m_pDerivedMesh->drawMappedFacesTex(ms.m_pDerivedMesh, CheckTexfaceDM, mcol);
-			current_blmat_nr = current_polymat->GetMaterialIndex();
-			current_image = current_polymat->GetBlenderImage();
-			ms.m_pDerivedMesh->drawFacesTex(ms.m_pDerivedMesh, CheckTexDM, NULL, NULL);
-		}
-		return;
-	}
-	// iterate over display arrays, each containing an index + vertex array
-	for (ms.begin(it); !ms.end(it); ms.next(it)) {
-		RAS_TexVert *vertex;
-		size_t i, j, numvert;
-		
-		numvert = it.array->m_type;
-
-		if (it.array->m_type == RAS_DisplayArray::LINE) {
-			// line drawing
-			glBegin(GL_LINES);
-
-			for (i=0; i<it.totindex; i+=2)
-			{
-				vertex = &it.vertex[it.index[i]];
-				glVertex3fv(vertex->getXYZ());
-
-				vertex = &it.vertex[it.index[i+1]];
-				glVertex3fv(vertex->getXYZ());
-			}
-
-			glEnd();
-		}
-		else {
-			// triangle and quad drawing
-			if (it.array->m_type == RAS_DisplayArray::TRIANGLE)
-				glBegin(GL_TRIANGLES);
-			else
-				glBegin(GL_QUADS);
-
-			for (i=0; i<it.totindex; i+=numvert)
-			{
-				if (obcolor)
-					glColor4d(rgba[0], rgba[1], rgba[2], rgba[3]);
-
-				for (j=0; j<numvert; j++) {
-					vertex = &it.vertex[it.index[i+j]];
-
-					if (!wireframe) {
-						if (!obcolor)
-							glColor4ubv((const GLubyte *)(vertex->getRGBA()));
-
-						glNormal3fv(vertex->getNormal());
-
-						if (multi)
-							TexCoord(*vertex);
-						else
-							glTexCoord2fv(vertex->getUV1());
-					}
-
-					glVertex3fv(vertex->getXYZ());
-				}
-			}
-
-			glEnd();
-		}
-	}
+	if (ms.m_pDerivedMesh)
+		m_failsafe_storage->IndexPrimitivesMulti(ms);
+	else
+		m_storage->IndexPrimitivesMulti(ms);
 }
 
 void RAS_OpenGLRasterizer::SetProjectionMatrix(MT_CmMatrix4x4 &mat)
@@ -1007,8 +853,7 @@ MT_Matrix4x4 RAS_OpenGLRasterizer::GetFrustumMatrix(
 
 			near_div_focallength = frustnear / m_focallength;
 			offset = 0.5f * m_eyeseparation * near_div_focallength;
-			switch(m_curreye)
-			{
+			switch (m_curreye) {
 				case RAS_STEREO_LEFTEYE:
 						left += offset;
 						right += offset;
@@ -1019,6 +864,12 @@ MT_Matrix4x4 RAS_OpenGLRasterizer::GetFrustumMatrix(
 						break;
 			}
 			// leave bottom and top untouched
+			if (m_stereomode == RAS_STEREO_3DTVTOPBOTTOM) {
+				// restore the vertical frustrum because the 3DTV will 
+				// expande the top and bottom part to the full size of the screen
+				bottom *= 2.0f;
+				top *= 2.0f;
+			}
 	}
 	
 	glMatrixMode(GL_PROJECTION);
@@ -1078,8 +929,7 @@ void RAS_OpenGLRasterizer::SetViewMatrix(const MT_Matrix4x4 &mat,
 		// vector between eyes
 		eyeline = viewDir.cross(viewupVec);
 
-		switch(m_curreye)
-		{
+		switch (m_curreye) {
 			case RAS_STEREO_LEFTEYE:
 				{
 				// translate to left by half the eye distance
@@ -1206,7 +1056,11 @@ void RAS_OpenGLRasterizer::DisableMotionBlur()
 
 void RAS_OpenGLRasterizer::SetAlphaBlend(int alphablend)
 {
-	GPU_set_material_alpha_blend(alphablend);
+	/* Variance shadow maps don't handle alpha well, best to not allow it for now  */
+	if (m_drawingmode == KX_SHADOW && m_usingoverrideshader)
+		GPU_set_material_alpha_blend(GPU_BLEND_SOLID);
+	else
+		GPU_set_material_alpha_blend(alphablend);
 /*
 	if (alphablend == m_last_alphablend)
 		return;
@@ -1259,3 +1113,482 @@ short RAS_OpenGLRasterizer::GetAnisotropicFiltering()
 {
 	return (short)GPU_get_anisotropic();
 }
+
+void RAS_OpenGLRasterizer::SetMipmapping(MipmapOption val)
+{
+	if (val == RAS_IRasterizer::RAS_MIPMAP_LINEAR)
+	{
+		GPU_set_linear_mipmap(1);
+		GPU_set_mipmap(1);
+	}
+	else if (val == RAS_IRasterizer::RAS_MIPMAP_NEAREST)
+	{
+		GPU_set_linear_mipmap(0);
+		GPU_set_mipmap(1);
+	}
+	else
+	{
+		GPU_set_linear_mipmap(0);
+		GPU_set_mipmap(0);
+	}
+}
+
+RAS_IRasterizer::MipmapOption RAS_OpenGLRasterizer::GetMipmapping()
+{
+	if (GPU_get_linear_mipmap())
+		return RAS_IRasterizer::RAS_MIPMAP_LINEAR;
+	else if (GPU_get_mipmap())
+		return RAS_IRasterizer::RAS_MIPMAP_NEAREST;
+	else
+		return RAS_IRasterizer::RAS_MIPMAP_NONE;
+}
+
+void RAS_OpenGLRasterizer::SetUsingOverrideShader(bool val)
+{
+	m_usingoverrideshader = val;
+}
+
+bool RAS_OpenGLRasterizer::GetUsingOverrideShader()
+{
+	return m_usingoverrideshader;
+}
+
+/**
+ * Render Tools
+ */
+
+/* ProcessLighting performs lighting on objects. the layer is a bitfield that
+ * contains layer information. There are 20 'official' layers in blender. A
+ * light is applied on an object only when they are in the same layer. OpenGL
+ * has a maximum of 8 lights (simultaneous), so 20 * 8 lights are possible in
+ * a scene. */
+
+void RAS_OpenGLRasterizer::ProcessLighting(bool uselights, const MT_Transform& viewmat)
+{
+	bool enable = false;
+	int layer= -1;
+
+	/* find the layer */
+	if (uselights) {
+		if (m_clientobject)
+			layer = static_cast<KX_GameObject*>(m_clientobject)->GetLayer();
+	}
+
+	/* avoid state switching */
+	if (m_lastlightlayer == layer && m_lastauxinfo == m_auxilaryClientInfo)
+		return;
+
+	m_lastlightlayer = layer;
+	m_lastauxinfo = m_auxilaryClientInfo;
+
+	/* enable/disable lights as needed */
+	if (layer >= 0) {
+		//enable = ApplyLights(layer, viewmat);
+		// taken from blender source, incompatibility between Blender Object / GameObject
+		KX_Scene* kxscene = (KX_Scene*)m_auxilaryClientInfo;
+		float glviewmat[16];
+		unsigned int count;
+		std::vector<RAS_OpenGLLight*>::iterator lit = m_lights.begin();
+
+		for (count=0; count<m_numgllights; count++)
+			glDisable((GLenum)(GL_LIGHT0+count));
+
+		viewmat.getValue(glviewmat);
+
+		glPushMatrix();
+		glLoadMatrixf(glviewmat);
+		for (lit = m_lights.begin(), count = 0; !(lit==m_lights.end()) && count < m_numgllights; ++lit)
+		{
+			RAS_OpenGLLight* light = (*lit);
+
+			if (light->ApplyFixedFunctionLighting(kxscene, layer, count))
+				count++;
+		}
+		glPopMatrix();
+
+		enable = count > 0;
+	}
+
+	if (enable)
+		EnableOpenGLLights();
+	else
+		DisableOpenGLLights();
+}
+
+void RAS_OpenGLRasterizer::EnableOpenGLLights()
+{
+	if (m_lastlighting == true)
+		return;
+
+	glEnable(GL_LIGHTING);
+	glEnable(GL_COLOR_MATERIAL);
+
+	glColorMaterial(GL_FRONT_AND_BACK, GL_AMBIENT_AND_DIFFUSE);
+	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);
+	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, (GetCameraOrtho())? GL_FALSE: GL_TRUE);
+	if (GLEW_EXT_separate_specular_color || GLEW_VERSION_1_2)
+		glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR);
+
+	m_lastlighting = true;
+}
+
+void RAS_OpenGLRasterizer::DisableOpenGLLights()
+{
+	if (m_lastlighting == false)
+		return;
+
+	glDisable(GL_LIGHTING);
+	glDisable(GL_COLOR_MATERIAL);
+
+	m_lastlighting = false;
+}
+
+RAS_ILightObject *RAS_OpenGLRasterizer::CreateLight()
+{
+	return new RAS_OpenGLLight(this);
+}
+
+void RAS_OpenGLRasterizer::AddLight(RAS_ILightObject* lightobject)
+{
+	RAS_OpenGLLight* gllight = dynamic_cast<RAS_OpenGLLight*>(lightobject);
+	assert(gllight);
+	m_lights.push_back(gllight);
+}
+
+void RAS_OpenGLRasterizer::RemoveLight(RAS_ILightObject* lightobject)
+{
+	RAS_OpenGLLight* gllight = dynamic_cast<RAS_OpenGLLight*>(lightobject);
+	assert(gllight);
+
+	std::vector<RAS_OpenGLLight*>::iterator lit =
+		std::find(m_lights.begin(),m_lights.end(),gllight);
+
+	if (!(lit==m_lights.end()))
+		m_lights.erase(lit);
+}
+
+bool RAS_OpenGLRasterizer::RayHit(struct KX_ClientObjectInfo *client, KX_RayCast *result, void * const data)
+{
+	double* const oglmatrix = (double* const) data;
+
+	RAS_Polygon* poly = result->m_hitMesh->GetPolygon(result->m_hitPolygon);
+	if (!poly->IsVisible())
+		return false;
+
+	MT_Point3 resultpoint(result->m_hitPoint);
+	MT_Vector3 resultnormal(result->m_hitNormal);
+	MT_Vector3 left(oglmatrix[0],oglmatrix[1],oglmatrix[2]);
+	MT_Vector3 dir = -(left.cross(resultnormal)).safe_normalized();
+	left = (dir.cross(resultnormal)).safe_normalized();
+	// for the up vector, we take the 'resultnormal' returned by the physics
+
+	double maat[16] = {left[0],         left[1],         left[2],         0,
+	                   dir[0],          dir[1],          dir[2],          0,
+	                   resultnormal[0], resultnormal[1], resultnormal[2], 0,
+	                   0,               0,               0,               1};
+
+	glTranslated(resultpoint[0],resultpoint[1],resultpoint[2]);
+	//glMultMatrixd(oglmatrix);
+	glMultMatrixd(maat);
+	return true;
+}
+
+void RAS_OpenGLRasterizer::applyTransform(double* oglmatrix,int objectdrawmode )
+{
+	/* FIXME:
+	blender: intern/moto/include/MT_Vector3.inl:42: MT_Vector3 operator/(const
+	MT_Vector3&, double): Assertion `!MT_fuzzyZero(s)' failed.
+
+	Program received signal SIGABRT, Aborted.
+	[Switching to Thread 16384 (LWP 1519)]
+	0x40477571 in kill () from /lib/libc.so.6
+	(gdb) bt
+	#7  0x08334368 in MT_Vector3::normalized() const ()
+	#8  0x0833e6ec in RAS_OpenGLRasterizer::applyTransform(RAS_IRasterizer*, double*, int) ()
+	*/
+
+	if (objectdrawmode & RAS_IPolyMaterial::BILLBOARD_SCREENALIGNED ||
+		objectdrawmode & RAS_IPolyMaterial::BILLBOARD_AXISALIGNED)
+	{
+		// rotate the billboard/halo
+		//page 360/361 3D Game Engine Design, David Eberly for a discussion
+		// on screen aligned and axis aligned billboards
+		// assumed is that the preprocessor transformed all billboard polygons
+		// so that their normal points into the positive x direction (1.0, 0.0, 0.0)
+		// when new parenting for objects is done, this rotation
+		// will be moved into the object
+
+		MT_Point3 objpos (oglmatrix[12],oglmatrix[13],oglmatrix[14]);
+		MT_Point3 campos = GetCameraPosition();
+		MT_Vector3 dir = (campos - objpos).safe_normalized();
+		MT_Vector3 up(0,0,1.0);
+
+		KX_GameObject* gameobj = (KX_GameObject*)m_clientobject;
+		// get scaling of halo object
+		MT_Vector3  size = gameobj->GetSGNode()->GetWorldScaling();
+
+		bool screenaligned = (objectdrawmode & RAS_IPolyMaterial::BILLBOARD_SCREENALIGNED)!=0;//false; //either screen or axisaligned
+		if (screenaligned)
+		{
+			up = (up - up.dot(dir) * dir).safe_normalized();
+		} else
+		{
+			dir = (dir - up.dot(dir)*up).safe_normalized();
+		}
+
+		MT_Vector3 left = dir.normalized();
+		dir = (up.cross(left)).normalized();
+
+		// we have calculated the row vectors, now we keep
+		// local scaling into account:
+
+		left *= size[0];
+		dir  *= size[1];
+		up   *= size[2];
+
+		double maat[16] = {left[0], left[1], left[2], 0,
+		                   dir[0],  dir[1],  dir[2],  0,
+		                   up[0],   up[1],   up[2],   0,
+		                   0,       0,       0,       1};
+
+		glTranslated(objpos[0],objpos[1],objpos[2]);
+		glMultMatrixd(maat);
+
+	}
+	else {
+		if (objectdrawmode & RAS_IPolyMaterial::SHADOW)
+		{
+			// shadow must be cast to the ground, physics system needed here!
+			MT_Point3 frompoint(oglmatrix[12],oglmatrix[13],oglmatrix[14]);
+			KX_GameObject *gameobj = (KX_GameObject*)m_clientobject;
+			MT_Vector3 direction = MT_Vector3(0,0,-1);
+
+			direction.normalize();
+			direction *= 100000;
+
+			MT_Point3 topoint = frompoint + direction;
+
+			KX_Scene* kxscene = (KX_Scene*) m_auxilaryClientInfo;
+			PHY_IPhysicsEnvironment* physics_environment = kxscene->GetPhysicsEnvironment();
+			PHY_IPhysicsController* physics_controller = gameobj->GetPhysicsController();
+
+			KX_GameObject *parent = gameobj->GetParent();
+			if (!physics_controller && parent)
+				physics_controller = parent->GetPhysicsController();
+
+			KX_RayCast::Callback<RAS_OpenGLRasterizer> callback(this, physics_controller, oglmatrix);
+			if (!KX_RayCast::RayTest(physics_environment, frompoint, topoint, callback))
+			{
+				// couldn't find something to cast the shadow on...
+				glMultMatrixd(oglmatrix);
+			}
+			else
+			{ // we found the "ground", but the cast matrix doesn't take
+			  // scaling in consideration, so we must apply the object scale
+				MT_Vector3  size = gameobj->GetSGNode()->GetLocalScale();
+				glScalef(size[0], size[1], size[2]);
+			}
+		} else
+		{
+
+			// 'normal' object
+			glMultMatrixd(oglmatrix);
+		}
+	}
+}
+
+static void DisableForText()
+{
+	glPolygonMode(GL_FRONT_AND_BACK, GL_FILL); /* needed for texture fonts otherwise they render as wireframe */
+
+	glDisable(GL_BLEND);
+	glDisable(GL_ALPHA_TEST);
+
+	glDisable(GL_LIGHTING);
+	glDisable(GL_COLOR_MATERIAL);
+
+	if (GLEW_ARB_multitexture) {
+		for (int i=0; i<RAS_MAX_TEXCO; i++) {
+			glActiveTextureARB(GL_TEXTURE0_ARB+i);
+
+			if (GLEW_ARB_texture_cube_map)
+				glDisable(GL_TEXTURE_CUBE_MAP_ARB);
+
+			glDisable(GL_TEXTURE_2D);
+		}
+
+		glActiveTextureARB(GL_TEXTURE0_ARB);
+	}
+	else {
+		if (GLEW_ARB_texture_cube_map)
+			glDisable(GL_TEXTURE_CUBE_MAP_ARB);
+
+		glDisable(GL_TEXTURE_2D);
+	}
+}
+
+void RAS_OpenGLRasterizer::RenderBox2D(int xco,
+			int yco,
+			int width,
+			int height,
+			float percentage)
+{
+	/* This is a rather important line :( The gl-mode hasn't been left
+	 * behind quite as neatly as we'd have wanted to. I don't know
+	 * what cause it, though :/ .*/
+	glDisable(GL_DEPTH_TEST);
+
+	glMatrixMode(GL_PROJECTION);
+	glPushMatrix();
+	glLoadIdentity();
+
+	glOrtho(0, width, 0, height, -100, 100);
+
+	glMatrixMode(GL_MODELVIEW);
+	glPushMatrix();
+	glLoadIdentity();
+
+	yco = height - yco;
+	int barsize = 50;
+
+	/* draw in black first*/
+	glColor3ub(0, 0, 0);
+	glBegin(GL_QUADS);
+	glVertex2f(xco + 1 + 1 + barsize * percentage, yco - 1 + 10);
+	glVertex2f(xco + 1, yco - 1 + 10);
+	glVertex2f(xco + 1, yco - 1);
+	glVertex2f(xco + 1 + 1 + barsize * percentage, yco - 1);
+	glEnd();
+
+	glColor3ub(255, 255, 255);
+	glBegin(GL_QUADS);
+	glVertex2f(xco + 1 + barsize * percentage, yco + 10);
+	glVertex2f(xco, yco + 10);
+	glVertex2f(xco, yco);
+	glVertex2f(xco + 1 + barsize * percentage, yco);
+	glEnd();
+
+	glMatrixMode(GL_PROJECTION);
+	glPopMatrix();
+	glMatrixMode(GL_MODELVIEW);
+	glPopMatrix();
+	glEnable(GL_DEPTH_TEST);
+}
+
+void RAS_OpenGLRasterizer::RenderText3D(
+        int fontid, const char *text, int size, int dpi,
+        const float color[4], const double mat[16], float aspect)
+{
+	/* gl prepping */
+	DisableForText();
+
+	/* the actual drawing */
+	glColor4fv(color);
+
+	/* multiply the text matrix by the object matrix */
+	BLF_enable(fontid, BLF_MATRIX|BLF_ASPECT);
+	BLF_matrix(fontid, mat);
+
+	/* aspect is the inverse scale that allows you to increase */
+	/* your resolution without sizing the final text size      */
+	/* the bigger the size, the smaller the aspect	           */
+	BLF_aspect(fontid, aspect, aspect, aspect);
+
+	BLF_size(fontid, size, dpi);
+	BLF_position(fontid, 0, 0, 0);
+	BLF_draw(fontid, text, 65535);
+
+	BLF_disable(fontid, BLF_MATRIX|BLF_ASPECT);
+}
+
+void RAS_OpenGLRasterizer::RenderText2D(
+        RAS_TEXT_RENDER_MODE mode,
+        const char* text,
+        int xco, int yco,
+        int width, int height)
+{
+	/* This is a rather important line :( The gl-mode hasn't been left
+	 * behind quite as neatly as we'd have wanted to. I don't know
+	 * what cause it, though :/ .*/
+	DisableForText();
+	glDisable(GL_DEPTH_TEST);
+
+	glMatrixMode(GL_PROJECTION);
+	glPushMatrix();
+	glLoadIdentity();
+
+	glOrtho(0, width, 0, height, -100, 100);
+
+	glMatrixMode(GL_MODELVIEW);
+	glPushMatrix();
+	glLoadIdentity();
+
+	if (mode == RAS_TEXT_PADDED) {
+		/* draw in black first*/
+		glColor3ub(0, 0, 0);
+		BLF_size(blf_mono_font, 11, 72);
+		BLF_position(blf_mono_font, (float)xco+1, (float)(height-yco-1), 0.0f);
+		BLF_draw(blf_mono_font, text, 65535); /* XXX, use real len */
+	}
+
+	/* the actual drawing */
+	glColor3ub(255, 255, 255);
+	BLF_size(blf_mono_font, 11, 72);
+	BLF_position(blf_mono_font, (float)xco, (float)(height-yco), 0.0f);
+	BLF_draw(blf_mono_font, text, 65535); /* XXX, use real len */
+
+	glMatrixMode(GL_PROJECTION);
+	glPopMatrix();
+	glMatrixMode(GL_MODELVIEW);
+	glPopMatrix();
+	glEnable(GL_DEPTH_TEST);
+}
+
+void RAS_OpenGLRasterizer::PushMatrix()
+{
+	glPushMatrix();
+}
+
+void RAS_OpenGLRasterizer::PopMatrix()
+{
+	glPopMatrix();
+}
+
+void RAS_OpenGLRasterizer::MotionBlur()
+{
+	int state = GetMotionBlurState();
+	float motionblurvalue;
+	if (state)
+	{
+		motionblurvalue = GetMotionBlurValue();
+		if (state==1)
+		{
+			//bugfix:load color buffer into accum buffer for the first time(state=1)
+			glAccum(GL_LOAD, 1.0);
+			SetMotionBlurState(2);
+		}
+		else if (motionblurvalue >= 0.0f && motionblurvalue <= 1.0f) {
+			glAccum(GL_MULT, motionblurvalue);
+			glAccum(GL_ACCUM, 1-motionblurvalue);
+			glAccum(GL_RETURN, 1.0);
+			glFlush();
+		}
+	}
+}
+
+void RAS_OpenGLRasterizer::SetClientObject(void* obj)
+{
+	if (m_clientobject != obj)
+	{
+		bool ccw = (obj == NULL || !((KX_GameObject*)obj)->IsNegativeScaling());
+		SetFrontFace(ccw);
+
+		m_clientobject = obj;
+	}
+}
+
+void RAS_OpenGLRasterizer::SetAuxilaryClientInfo(void* inf)
+{
+	m_auxilaryClientInfo = inf;
+}
+
