@@ -1,24 +1,28 @@
 /*
------------------------------------------------------------------------------
-This source file is part of VideoTexture library
-
-Copyright (c) 2007 The Zdeno Ash Miklas
-
-This program is free software; you can redistribute it and/or modify it under
-the terms of the GNU Lesser General Public License as published by the Free Software
-Foundation; either version 2 of the License, or (at your option) any later
-version.
-
-This program is distributed in the hope that it will be useful, but WITHOUT
-ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
-FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License for more details.
-
-You should have received a copy of the GNU Lesser General Public License along with
-this program; if not, write to the Free Software Foundation, Inc., 59 Temple
-Place - Suite 330, Boston, MA 02111-1307, USA, or go to
-http://www.gnu.org/copyleft/lesser.txt.
------------------------------------------------------------------------------
-*/
+ * ***** BEGIN GPL LICENSE BLOCK *****
+ *
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License
+ * as published by the Free Software Foundation; either version 2
+ * of the License, or (at your option) any later version.
+ *
+ * This program is distributed in the hope that it will be useful,
+ * but WITHOUT ANY WARRANTY; without even the implied warranty of
+ * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+ * GNU General Public License for more details.
+ *
+ * You should have received a copy of the GNU General Public License
+ * along with this program; if not, write to the Free Software  Foundation,
+ * Inc., 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301, USA.
+ *
+ * Copyright (c) 2007 The Zdeno Ash Miklas
+ *
+ * This source file is part of VideoTexture library
+ *
+ * Contributor(s):
+ *
+ * ***** END GPL LICENSE BLOCK *****
+ */
 
 /** \file gameengine/VideoTexture/Texture.cpp
  *  \ingroup bgevideotex
@@ -32,11 +36,11 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include "KX_GameObject.h"
 #include "KX_Light.h"
 #include "RAS_MeshObject.h"
+#include "RAS_ILightObject.h"
 #include "DNA_mesh_types.h"
 #include "DNA_meshdata_types.h"
 #include "DNA_image_types.h"
 #include "IMB_imbuf_types.h"
-#include "KX_PolygonMaterial.h"
 
 #include "MEM_guardedalloc.h"
 
@@ -50,7 +54,7 @@ http://www.gnu.org/copyleft/lesser.txt.
 #include "Exception.h"
 
 #include <memory.h>
-#include "GL/glew.h"
+#include "glew-mx.h"
 
 
 // macro for exception handling and logging
@@ -59,13 +63,13 @@ http://www.gnu.org/copyleft/lesser.txt.
 
 
 // Blender GameObject type
-BlendType<KX_GameObject> gameObjectType ("KX_GameObject");
-BlendType<KX_LightObject> lightObjectType ("KX_LightObject");
+static BlendType<KX_GameObject> gameObjectType ("KX_GameObject");
+static BlendType<KX_LightObject> lightObjectType ("KX_LightObject");
 
 
 // load texture
-void loadTexture (unsigned int texId, unsigned int * texture, short * size,
-				  bool mipmap)
+void loadTexture(unsigned int texId, unsigned int *texture, short *size,
+                 bool mipmap)
 {
 	// load texture for rendering
 	glBindTexture(GL_TEXTURE_2D, texId);
@@ -167,8 +171,8 @@ static PyObject *Texture_new(PyTypeObject *type, PyObject *args, PyObject *kwds)
 
 
 // forward declaration
-PyObject *Texture_close(Texture * self);
-int Texture_setSource (Texture * self, PyObject *value, void *closure);
+PyObject *Texture_close(Texture *self);
+int Texture_setSource(Texture *self, PyObject *value, void *closure);
 
 
 // Texture object deallocation
@@ -187,7 +191,7 @@ static void Texture_dealloc(Texture *self)
 
 
 ExceptionID MaterialNotAvail;
-ExpDesc MaterialNotAvailDesc (MaterialNotAvail, "Texture material is not available");
+ExpDesc MaterialNotAvailDesc(MaterialNotAvail, "Texture material is not available");
 
 // Texture object initialization
 static int Texture_init(Texture *self, PyObject *args, PyObject *kwds)
@@ -225,23 +229,16 @@ static int Texture_init(Texture *self, PyObject *args, PyObject *kwds)
 				{
 					self->m_imgTexture = static_cast<KX_BlenderMaterial*>(mat)->getImage(texID);
 					self->m_useMatTexture = false;
-				} else if (mat->GetFlag() & RAS_BLENDERMAT)
+				} else
 				{
 					// get blender material texture
 					self->m_matTexture = static_cast<KX_BlenderMaterial*>(mat)->getTex(texID);
 					self->m_useMatTexture = true;
 				}
-				else
-				{
-					// get texture pointer from polygon material
-					MTFace * tface = static_cast<KX_PolygonMaterial*>(mat)->GetMTFace();
-					self->m_imgTexture = (Image*)tface->tpage;
-					self->m_useMatTexture = false;
-				}
 			}
 			else if (lamp != NULL)
 			{
-				self->m_imgTexture = lamp->GetTextureImage(texID);
+				self->m_imgTexture = lamp->GetLightData()->GetTextureImage(texID);
 				self->m_useMatTexture = false;
 			}
 
@@ -377,7 +374,7 @@ static PyObject *Texture_refresh(Texture *self, PyObject *args)
 						texture = self->m_scaledImg;
 					}
 					// load texture for rendering
-					loadTexture (self->m_actTex, texture, size, self->m_mipmap);
+					loadTexture(self->m_actTex, texture, size, self->m_mipmap);
 
 					// refresh texture source, if required
 					if (refreshSource) self->m_source->m_image->refresh();
@@ -435,7 +432,7 @@ static PyObject *Texture_getSource(Texture *self, PyObject *value, void *closure
 
 
 // set source object
-int Texture_setSource (Texture * self, PyObject *value, void *closure)
+int Texture_setSource(Texture *self, PyObject *value, void *closure)
 {
 	// check new value
 	if (value == NULL || !pyImageTypes.in(Py_TYPE(value)))
